! Copyright (C) GemTalk Systems 1986-2023.  All Rights Reserved.
! Class extensions for 'AbstractCharacter'

!		Class methods for 'AbstractCharacter'

removeallmethods AbstractCharacter
removeallclassmethods AbstractCharacter

category: 'Non-Printable Characters'
classmethod: AbstractCharacter
backspace

"(Subclass responsibility.)  Returns the back-space character from the
 receiver's character set."

AbstractCharacter subclassResponsibility: #backspace
%

category: 'Non-Printable Characters'
classmethod: AbstractCharacter
cr

"(Subclass responsibility.)  Returns the carriage-return character from the
 receiver's character set."

AbstractCharacter subclassResponsibility: #cr
%

category: 'Non-Printable Characters'
classmethod: AbstractCharacter
esc

"(Subclass responsibility.)  Returns the escape character from the receiver's
 character set."

AbstractCharacter subclassResponsibility: #esc
%

category: 'Instance Creation'
classmethod: AbstractCharacter
fromStream: aStream

"(Subclass responsibility.)  Returns an instance of the receiver's class that is
 the next character in the stream, aStream."

AbstractCharacter subclassResponsibility: #fromStream:
%

category: 'Non-Printable Characters'
classmethod: AbstractCharacter
lf

"(Subclass responsibility.)  Returns the line-feed character from the receiver's
 character set."

AbstractCharacter subclassResponsibility: #lf
%

category: 'Non-Printable Characters'
classmethod: AbstractCharacter
newPage

"(Subclass responsibility.)  Returns the new-page character from the receiver's
 character set."

AbstractCharacter subclassResponsibility: #newPage
%

category: 'Non-Printable Characters'
classmethod: AbstractCharacter
space

"(Subclass responsibility.)  Returns the space character from the receiver's
 character set."

AbstractCharacter subclassResponsibility: #space
%

category: 'Non-Printable Characters'
classmethod: AbstractCharacter
tab

"(Subclass responsibility.)  Returns the tab character from the receiver's
 character set."

AbstractCharacter subclassResponsibility: #tab
%

category: 'Instance Creation'
classmethod: AbstractCharacter
withValue: anInteger

"(Subclass responsibility.)  Returns an instance of the receiver's class that
 has the specified internal value."

AbstractCharacter subclassResponsibility: #withValue:
%

!		Instance methods for 'AbstractCharacter'

category: 'Comparing'
method: AbstractCharacter
<= aCharacter

"Returns true if the receiver is less than or equal to the argument,
 otherwise returns false."

^ (self < aCharacter) or: [self = aCharacter]
%

category: 'Comparing'
method: AbstractCharacter
> aCharacter

"Returns true if the receiver is greater than the argument,
 otherwise returns false."

^ (self <= aCharacter) not
%

category: 'Comparing'
method: AbstractCharacter
>= aCharacter

"Returns true if the receiver is greater than the argument,
 otherwise returns false."

^ (self < aCharacter) not
%

category: 'Converting'
method: AbstractCharacter
asCharacter

"(Subclass responsibility.)  Returns the Character corresponding to the
 receiver."

AbstractCharacter subclassResponsibility: #asCharacter
%

category: 'Deprecated'
method: AbstractCharacter
asciiLessThan: aChar

"Returns true if the ASCII code of the receiver is less than that of
 aCharacter."

self deprecated: 'asciiLessThan: deprecated in 3.2.'.
^ self codePoint < aChar codePoint
%

category: 'Converting'
method: AbstractCharacter
asInteger

"(Subclass responsibility.)  Returns the internal code of the receiver."

AbstractCharacter subclassResponsibility: #asInteger
%

category: 'Converting'
method: AbstractCharacter
asLowercase

"(Subclass responsibility.)  Returns a kind of AbstractCharacter that is the
 lower-case character corresponding to the receiver.  If the receiver
 is lower-case or has no case, this returns the receiver itself."

AbstractCharacter subclassResponsibility: #asLowercase
%

category: 'Converting'
method: AbstractCharacter
asUppercase

"(Subclass responsibility.)  Returns a kind of AbstractCharacter that is the
 upper-case character corresponding to the receiver.  If the receiver
 is upper-case or has no case, this returns the receiver itself."

AbstractCharacter subclassResponsibility: #asUppercase
%

category: 'Converting'
method: AbstractCharacter
digitValue

"(Subclass responsibility.)  Returns a SmallInteger representing the value of
 the receiver, a digit, or returns nil if the receiver is not a digit."

AbstractCharacter subclassResponsibility: #digitValue
%

category: 'Formatting'
method: AbstractCharacter
displayWidth

"(Subclass responsibility.)  Returns the width necessary to display the
 receiver."

AbstractCharacter subclassResponsibility: #displayWidth
%

category: 'Comparing'
method: AbstractCharacter
hash

"Returns a numeric hash key for the receiver."

^self asInteger
%

category: 'Testing'
method: AbstractCharacter
isDigit

"(Subclass responsibility.)  Returns true if the receiver is a digit.  Returns
 false otherwise."

AbstractCharacter subclassResponsibility: #isDigit
%

category: 'Testing'
method: AbstractCharacter
isEquivalent: aCharacter

"(Subclass responsibility.)  Returns true if the receiver is equivalent to
 aCharacter.  The receiver is equivalent to aCharacter if the receiver is the
 same character as the argument regardless of case or internal representation."

AbstractCharacter subclassResponsibility: #isEquivalent:
%

category: 'Testing'
method: AbstractCharacter
isLowercase

"(Subclass responsibility.)  Returns true if the receiver is a lower-case
 character.  Returns false otherwise."

AbstractCharacter subclassResponsibility: #isLowercase
%

category: 'Testing'
method: AbstractCharacter
isUppercase

"(Subclass responsibility.)  Returns true if the receiver is an upper-case
 character.  Returns false otherwise."

AbstractCharacter subclassResponsibility: #isUppercase
%

category: 'New Indexing Comparison'
method: AbstractCharacter
_classSortOrdinal

^ 30
%

category: 'Indexing Support'
method: AbstractCharacter
_idxBasicCanCompareWithClass: aClass
  "Returns true if the receiver may be inserted into a basic BtreeNode whose
   #lastElementClass is <aClass> (see RangeEqualityIndex class>>isBasicClass:)."

  (super _idxBasicCanCompareWithClass: aClass)
    ifTrue: [ ^ true ].
  ^ aClass isSubclassOf: AbstractCharacter
%

! Class extensions for 'AbstractCollisionBucket'

!		Class methods for 'AbstractCollisionBucket'

removeallmethods AbstractCollisionBucket
removeallclassmethods AbstractCollisionBucket

category: 'Instance Creation'
classmethod: AbstractCollisionBucket
new

"Returns an AbstractCollisionBucket with a default capacity of four key/value
 pairs."

^ self new: 4
%

category: 'Instance Creation'
classmethod: AbstractCollisionBucket
new: aSize

"Returns an AbstractCollisionBucket with the specified size."

|result|
result := super new: (aSize + aSize).
result initialize.
^result
%

!		Instance methods for 'AbstractCollisionBucket'

category: 'Comparing'
method: AbstractCollisionBucket
= anObject

  "Reimplemented to compare by identity."

  ^ self == anObject
%

category: 'Adding'
method: AbstractCollisionBucket
add: anAssociation

self shouldNotImplement: #add:  " use at:put:keyValDict_coll: "
%

category: 'Accessing'
method: AbstractCollisionBucket
at: aKey

"Returns the value that corresponds to aKey."

^self at: aKey
    ifAbsent: nil "optimization, errors reported via _reportKeyNotFound"
%

category: 'Accessing'
method: AbstractCollisionBucket
at: aKey ifAbsent: aBlock

"Returns the value that corresponds aKey.  If no such key/value pair exists,
 returns the result of evaluating the zero-argument block aBlock."

  | index |
  index := self searchForKey: aKey.
  index ifNotNil:[ ^ self valueAt: index ] .
  aBlock ifNil:[ ^ self _errorKeyNotFound: aKey ] .
  ^ aBlock value
%

category: 'Accessing'
method: AbstractCollisionBucket
at: aKey otherwise: aValue

"Returns the value that corresponds to aKey by searching the elements in the
 bucket.  If no such key/value pair exists, returns the given alternate value."

  | index |
  index := self searchForKey: aKey.
  index ifNotNil:[ ^ self valueAt: index ].
   ^aValue .
%

category: 'Updating'
method: AbstractCollisionBucket
at: aKey put: aValue

self shouldNotImplement: #at:put:  " use at:put:keyValDict_coll: "
%

category: 'Updating'
method: AbstractCollisionBucket
at: aKey put: aValue keyValDict_coll: aKeyValDict
 "Stores the aKey/aValue pair in the receiver.
  Returns self size if this at:put: added a new key, 0 if this at:put:
  replaced the value of an existing key."
 | emptySlotIdx startTableSize thisKey numElem |

  startTableSize := self tableSize .
  aKey ifNil:[ ^ self _error: #rtErrNilKey ] .
  (numElem := numElements) == 0 ifTrue:[
    emptySlotIdx := 1
  ] ifFalse:[ | idx |
    "search for aKey, or for the first empty slot "
    idx := 1 .
    1 to: startTableSize do:[:n |
      thisKey := self _at: idx . "inline keyAt:"
      thisKey ifNotNil:[
        (self compareKey: aKey with: thisKey) ifTrue:[ "Key found.  Store given value"
          self _at: idx + 1 put: aValue .  "inline at:putValue:"
          aKeyValDict _markDirty .
          ^ 0
        ].
      ] ifNil:[
        emptySlotIdx ifNil:[ emptySlotIdx := idx ].
      ].
      idx := idx + 2 .
    ] .
    "Key not found so add key and value"
    emptySlotIdx ifNil:[ " bucket is full so grow it "
      emptySlotIdx := self _basicSize + 1 .
      self size: emptySlotIdx + 7  .  "accommodate 4 more key,value pairs"
    ] .
  ].
  numElem := numElem + 1.
  numElements := numElem .
  self _at: emptySlotIdx put: aKey. "inline at:putKey:"
  self _at: emptySlotIdx + 1 put: aValue . "inline at:putValue"
  ^ numElem
%

category: 'Updating'
method: AbstractCollisionBucket
at: anIndex putKey: aKey

"Stores the key aKey into the key part of the key/value pair referenced by
 anIndex.  Note that this method overwrites the key value at the given index.
 Returns aKey."

super at: (anIndex + anIndex) - 1 put: aKey
%

category: 'Updating'
method: AbstractCollisionBucket
at: anIndex putValue: aValue

"Stores the value aValue into the value part of the key/value pair referenced
 by atIndex.  Returns aValue."

super at: (anIndex + anIndex) put: aValue
%

category: 'Private'
method: AbstractCollisionBucket
compareKey: key1 with: key2

"Returns true if key1 is equivalent to key2, and false otherwise."

^ key1 = key2
%

category: 'Enumerating'
method: AbstractCollisionBucket
do: aBlock

"For each key/value pair in the receiver, evaluates the one-argument block
 aBlock with the value as the argument.  Returns the receiver."

	self valuesDo: aBlock
%

category: 'Deprecated'
method: AbstractCollisionBucket
doKeys: aBlock

self deprecated: 'doKeys: Obsolete in GemStone/64.  Use the keysDo: method instead.'.
^ self keysDo: aBlock.
%

category: 'Deprecated'
method: AbstractCollisionBucket
doValues: aBlock

self deprecated: 'doValues: Obsolete in GemStone/64.  Use the valuesDo: method instead.'.
^ self valuesDo: aBlock.
%

category: 'Searching'
method: AbstractCollisionBucket
firstPair

"Returns an Association containing the receiver's first key/value pair.
 If the receiver is empty, returns an Association containing nils."

   | aKey |
   numElements == 0 ifFalse: [
      "Search for the first non-nil index"
      1 to: self tableSize do: [ :i |
         (aKey := self keyAt: i) ifNotNil: [
            ^Association new key: aKey value: (self valueAt: i)
         ]
      ]
   ].
   "No first pair was found"
   ^Association newWithKey: nil value: nil
%

category: 'Comparing'
method: AbstractCollisionBucket
hash
  ^ self identityHash
%

category: 'Searching'
method: AbstractCollisionBucket
includesKey: aKey

"Returns true if the receiver contains a key that is equal to 'aKey'.
 Otherwise, returns false."

^(self searchForKey: aKey) ~~ nil
%

category: 'Initializing'
method: AbstractCollisionBucket
initialize

"Initializes the instance variable of the receiver to be an empty
 CollisionBucket."

numElements := 0.
%

category: 'Accessing'
method: AbstractCollisionBucket
keyAt: index

"Returns the key at the specified index."

^super at: (index + index - 1)
%

category: 'Accessing'
method: AbstractCollisionBucket
keyAt: aKey otherwise: aValue

"Returns the key that corresponds to aKey by searching the elements in the
 bucket.  If no such key/value pair exists, returns the given alternate value."

  | index |
  index := self searchForKey: aKey.
  index ifNotNil:[ ^ self keyAt: index ].
  ^ aValue
%

category: 'Enumerating'
method: AbstractCollisionBucket
keysAndValuesDo: aBlock

"For each key/value pair in the receiver, evaluates the two-argument block
 aBlock with the key and value as the arguments.  Returns the receiver."

| aKey |
1 to: self tableSize do: [ :index |
  (aKey := self keyAt: index) ~~ nil
    ifTrue: [
      aBlock value: aKey value: (self valueAt: index)
      ]
  ].
%

category: 'Enumerating'
method: AbstractCollisionBucket
keysDo: aBlock

"For each key/value pair in the receiver, evaluates the one-argument block
 aBlock with the key as the argument.  Returns the receiver."

| aKey |
1 to: self tableSize do: [ :index |
  (aKey := self keyAt: index) ~~ nil ifTrue: [ aBlock value: aKey ]
  ].
%

category: 'Accessing'
method: AbstractCollisionBucket
keyValueDictionary

"Returns nil.  Only IdentityCollisionBuckets have the keyValueDictionary
 instance variable."

^nil
%

category: 'Updating'
method: AbstractCollisionBucket
keyValueDictionary: aDict

"No-op for AbstractCollisionBuckets.  Used only for IdentityCollisionBuckets."
%

category: 'Accessing'
method: AbstractCollisionBucket
numElements

"Returns value of the numElements instance variable.  (The name numElements
 is provided for compatibility with earlier releases.  The instance method
 size is preferred for new code.)"

^numElements
%

category: 'Formatting'
method: AbstractCollisionBucket
printOn: aStream

"Puts a displayable representation of the receiver on the given stream."

"Copy the implementation from Object so we don't inherit it from Collection."

aStream nextPutAll: self asString
%

category: 'Removing'
method: AbstractCollisionBucket
removeKey: aKey ifAbsent: aBlock

"Removes the key/value pair having the key aKey.  If aKey is not found,
 returns the result of evaluating the zero-argument block aBlock."

  | keyIndex aValue |

  keyIndex := self searchForKey: aKey.
  keyIndex ifNotNil:[
      numElements := numElements - 1.
      aValue := self valueAt: keyIndex.
      self at: keyIndex putKey: nil.
      self at: keyIndex putValue: nil.
      ^aValue
  ].
  aBlock ifNil:[ ^ self _errorKeyNotFound: aKey ] .
  ^ aBlock value
%

category: 'Removing'
method: AbstractCollisionBucket
removeKey: aKey otherwise: notFoundValue

"Removes the key/value pair having the key aKey.  If aKey is not found,
 returns the notFoundValue. "

  | keyIndex aValue |

  keyIndex := self searchForKey: aKey.
  keyIndex == nil
    ifFalse: [
      numElements := numElements - 1.
      aValue := self valueAt: keyIndex.
      self at: keyIndex putKey: nil.
      self at: keyIndex putValue: nil.
      ^aValue ]
    ifTrue: [ ^ notFoundValue ]
%

category: 'Searching'
method: AbstractCollisionBucket
searchForKey: argKey

"Returns the index of argKey, or if not found, nil."
   | idx |
   argKey ifNil:[ ^ self _error: #rtErrNilKey ] .
   idx := 1 .
   1 to: self tableSize do: [ :n | | aKey |
      aKey := self _at: idx . "inline keyAt:"
      aKey ifNotNil:[
        (self compareKey: argKey with: aKey) ifTrue:[ ^ n ].
      ].
      idx := idx + 2
   ].
   ^ nil "Key not found"
%

category: 'Accessing'
method: AbstractCollisionBucket
size

"Returns value of the numElements instance variable."

^numElements
%

category: 'Accessing'
method: AbstractCollisionBucket
tableSize

"Returns the number of key/value pairs in the capacity of the receiver."

^self _basicSize // 2
%

category: 'Accessing'
method: AbstractCollisionBucket
valueAt: index

"Returns the value at the specified index."

^super at: (index + index)
%

category: 'Enumerating'
method: AbstractCollisionBucket
valuesDo: aBlock

"For each key/value pair in the receiver, evaluates the one-argument block
 aBlock with the value as the argument.  Returns the receiver."

1 to: self tableSize do: [ :index |
  (self keyAt: index) == nil ifFalse: [ aBlock value: (self valueAt: index) ]
  ].
%

category: 'Error Handling'
method: AbstractCollisionBucket
_errorKeyNotFound: aKey

"No key/value pair with given key, 'aKey', was found."

^ self _error: #rtErrKeyNotFound args: { aKey }.
%

category: 'Private'
method: AbstractCollisionBucket
_removeAll

"Dereferences the receiver from its parent and shrinks the receiver.
 Used while rebuilding a KeyValueDictionary."

numElements := 0.
"Gs64 v3.0, don't send size: 0 "
%

category: 'Removing'
method: AbstractCollisionBucket
_removeKey: aKey

"Removes the key/value pair having the key aKey.  If aKey is not found,
 generates an error."

  | keyIndex |

  keyIndex := self searchForKey: aKey.
  keyIndex == nil
    ifFalse: [
      numElements := numElements - 1.
      self at: keyIndex putKey: nil.
      self at: keyIndex putValue: nil.
    ]
    ifTrue: [ self _errorKeyNotFound: aKey ]
%

category: 'Accessing'
method: AbstractCollisionBucket
_reportKeyNotFound: aKey with: aBlock

"Private."

aBlock == nil ifTrue:[^ self _errorKeyNotFound: aKey ] .
^aBlock value
%

! Class extensions for 'AbstractDictionary'

!		Class methods for 'AbstractDictionary'

removeallmethods AbstractDictionary
removeallclassmethods AbstractDictionary

category: 'Storing and Loading'
classmethod: AbstractDictionary
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into a new instance of the receiver.
 Returns the new instance."

| size inst |
size := passiveObj readSize.
inst := self new.
inst loadFrom: passiveObj size: size.
^inst
%

category: 'Instance Creation'
classmethod: AbstractDictionary
new

"(Subclass responsibility.)  Returns a new instance of AbstractDictionary."

^ self subclassResponsibility: #new

%

category: 'Instance Creation'
classmethod: AbstractDictionary
new: count

"(Subclass responsibility.) Returns a new instance of AbstractDictionary. The
 argument count provides a hint of the number of elements the instance should
 be designed to hold."

^ self subclassResponsibility: #new:
%

category: 'Private'
classmethod: AbstractDictionary
_tableSizeFor: targetSize 
  | limit |
  limit := 2034 - self instSize - 1 .
  (targetSize >= (limit - 100) and:[ targetSize <= limit ]) ifTrue:[
    "create the object with basicSize close to max small object size."
    ^ Integer _selectedPrimeGreaterThan: (limit min: 2028 )
  ].
  ^ Integer _selectedPrimeGreaterThan: targetSize .
%

!		Instance methods for 'AbstractDictionary'

category: 'Comparing'
method: AbstractDictionary
= anAbstractDictionary
"Returns true if all of the following conditions are true:

 1.  The receiver and anAbstractDictionary are of the same class.
 2.  The two dictionaries are of the same size.
 3.  The corresponding keys and values of the receiver and anAbstractDictionary
     are equal."

| notFound |
(self == anAbstractDictionary) ifTrue:[ ^ true ].

(self class == anAbstractDictionary class) ifFalse:[ ^ false ].

(self size == anAbstractDictionary size) ifFalse:[ ^ false ].
notFound := Object new .
self keysAndValuesDo: [ :aKey :aValue | | otherVal|
  otherVal := anAbstractDictionary at: aKey otherwise: notFound .
  (aValue == self or:[ aValue == anAbstractDictionary]) ifTrue:[
    (otherVal == self or:[ otherVal == anAbstractDictionary]) ifFalse:[ ^ false].
  ] ifFalse:[
    aValue = otherVal ifFalse:[ ^ false ]
  ].
].
^ true.
%

category: 'Enumerating'
method: AbstractDictionary
accompaniedBy: anObj keysAndValuesDo: aBlock

"Iteratively evaluates the threee argument block, aBlock,
 using anObj, each key and each value
 of the receiver as the arguments to the block.  Returns the receiver."

^ self subclassResponsibility: #accompaniedBy:keysAndValuesDo: .
%

category: 'Adding'
method: AbstractDictionary
add: anAssociation

"Adds the Association or the key-value pair contained in anAssociation to the
 receiver.  If the receiver already includes an Association/key-value pair
 whose key is equal to that of anAssociation, then this method redefines the
 value portion of that Association/key-value pair. Returns anAssociation."

self at: anAssociation key put: anAssociation value.
^ anAssociation.
%

category: 'Adding'
method: AbstractDictionary
addAll: aCollection

"Adds to the receiver all the Associations or key-value pairs contained in
 aCollection.  aCollection must be a collection of Associations or a dictionary.
 Returns the argument, aCollection."

(aCollection isKindOf: AbstractDictionary) ifTrue: [
  aCollection accompaniedBy: self
    keysAndValuesDo:[ :rcvr :aKey :aVal| rcvr at: aKey put: aVal ].
] ifFalse: [
  aCollection accompaniedBy: self do: [ :me :anAssociation | me add: anAssociation ]
].
^ aCollection.
%

category: 'Printing'
method: AbstractDictionary
asReportString

"Returns a String that lists the key-value pairs, one on each line."

| result lf tab |
result := String new.
tab := Character tab .
lf := Character lf .
self keysAndValuesDo: [ :key :val |
  result add: key printString; add: tab ; add: val printString; add: lf
].
^ result
%

category: 'Enumerating'
method: AbstractDictionary
associationsAsArray

"Returns an Array containing all of the receiver's Associations"

^ self selectAssociationsAsArray: [ :assoc| true ]
%

category: 'Enumerating'
method: AbstractDictionary
associationsDetect: aBlock

"Evaluates aBlock repeatedly, with the Associations of the receiver as the
 argument.  Returns the first Association for which the block evaluates to true
 when the Association is used as the argument to the block.  If none of the
 receiver's Associations evaluates to true, generates an error.  The argument
 aBlock must be a one-argument block."

^ self associationsDetect: aBlock
       ifNone: [^ self _error: #assocErrNoElementsDetected args: { aBlock }]
%

category: 'Enumerating'
method: AbstractDictionary
associationsDetect: aBlock ifNone: exceptionBlock

"Evaluates aBlock repeatedly, with the Associations of the receiver as the
 argument.  Returns the first Association for which aBlock evaluates to true
 when the Association is used as the argument to the block.  If none of the
 receiver's Associations evaluates to true, this method evaluates the argument
 exceptionBlock and returns its value.  The argument aBlock must be a
 one-argument block, and exceptionBlock must be a zero-argument block."

self associationsDo: [ :anAssoc|
  (aBlock value: anAssoc) ifTrue: [ ^anAssoc ].
  ].

^ exceptionBlock value.
%

category: 'Enumerating'
method: AbstractDictionary
associationsDo: aBlock

"Iteratively evaluates the one argument block, aBlock, using each Association
 in the receiver as the argument to the block. If the receiver stores the
 key-value pairs directly, new Associations are created for the key-value pairs
 and used as the arguments to aBlock.  Returns the receiver."

^ self subclassResponsibility: #associationsDo:
%

category: 'Accessing'
method: AbstractDictionary
at: aKey

"Returns the value of the Association with key aKey.  Generates an error if no
 such key exists."

    | sentinel result |
    sentinel := AbsDictSentinel"created in bom.c"  .
    (result := self at: aKey otherwise: sentinel) == sentinel ifTrue:
        [^self _errorKeyNotFound: aKey].
    ^result
%

category: 'Accessing'
method: AbstractDictionary
at: aKey ifAbsent: aBlock

"Returns the value associated with key aKey.  If no such key/value pair or
 Association exists, returns the result of evaluating the zero-argument block
 aBlock."

^ self subclassResponsibility: #at:ifAbsent.
%

category: 'Accessing'
method: AbstractDictionary
at: aKey ifAbsentPut: aBlock

"Returns the value associated with key aKey. If no such key/value pair or
 Association exists, returns the result of evaluating the zero-argument block
 aBlock. In the latter case, the result of  evaluating the block aBlock is
 also stored in the receiver using the key aKey."

| known existing value |

known := 'a'.
existing := self at: aKey otherwise: known.
existing == known
  ifTrue: [
    value := aBlock value.
    self at: aKey put: value.
    ^value
    ].
^existing
%

category: 'Accessing'
method: AbstractDictionary
at: key ifPresent: aBlock
"Lookup the given key in the receiver. If it is present, answer the value of
 evaluating the given one argument block with the value associated with the key.
 Otherwise, answer nil."

| v |
v := self at: key ifAbsent: [^ nil].
^ aBlock value: v
%

category: 'Accessing'
method: AbstractDictionary
at: aKey otherwise: value

"Returns the value that corresponds to aKey.  If no such key/value pair or
 Association exists, returns the given alternate value."

    ^self at: aKey ifAbsent: [value]
%

category: 'Updating'
method: AbstractDictionary
at: aKey put: aValue

"Creates a new Association with the given key and value and adds it to the
 receiver or adds the key-value pair to the receiver depending on the class of
 the receiver.  If the receiver already contains an Association or key-value
 pair with the given key, this method makes aValue the value of that
 Association or key-value pair.  Returns aValue."

^ self subclassResponsibility: #at:put:
%

category: 'Enumerating'
method: AbstractDictionary
collect: aBlock

"Evaluates aBlock with each of the receiver's values as the argument and
 collects the resulting values into the appropriate Dictionary at the
 corresponding key values."

| result |

result := self speciesForCollect new: (self size).
self keysAndValuesDo: [ :aKey :aValue |
  result at: aKey put: (aBlock value: aValue).
  ].

^ result.
%

category: 'Enumerating'
method: AbstractDictionary
collectAssociations: aBlock

"Evaluates aBlock with each of the receiver's Associations (or Associations
 created using the key-value pairs) as the argument and collects the resulting
 values into an Array. Returns the newly created Array."

| anArray index |

anArray := Array new: (self size).
index := 0.
self associationsDo: [ :anAssoc |
  index := index + 1.
  anArray at: index put: (aBlock value: anAssoc).
  ].

^ anArray.
%

category: 'Deprecated'
method: AbstractDictionary
collectValues: aBlock

self deprecated: 'AbstractDictionary>>collectValues: deprecated long before v3.0. Use collect: instead.'.
^ self collect: aBlock.
%

category: 'Enumerating'
method: AbstractDictionary
collectValuesAsArray: aBlock

"Evaluates aBlock with each of the receiver's values as the argument and
 collects the resulting values into an Array. Returns the new Array."

| result |
result := { } .
self valuesDo: [ :aValue |
  result add: (aBlock value: aValue).
  ].
^ result.
%

category: 'Deprecated'
method: AbstractDictionary
detectAssociations: aBlock

self deprecated: 'AbstractDictionary>>detectAssociations: deprecated long before v3.0. Use associationsDetect: instead.'.
^ self associationsDetect: aBlock
%

category: 'Deprecated'
method: AbstractDictionary
detectAssociations: aBlock ifNone: exceptionBlock

self deprecated: 'AbstractDictionary>>detectAssociations:ifNone: deprecated long before v3.0. Use associationsDetect:ifNone: instead.'.
^ self associationsDetect: aBlock ifNone: exceptionBlock
%

category: 'Deprecated'
method: AbstractDictionary
detectValues: aBlock

self deprecated: 'AbstractDictionary>>detectValues: deprecated long before v3.0.  Use keysAndValuesDo: instead.'.
^ self detectValues: aBlock ifNone: [ ^ self errorNoElementDetected: aBlock ].
%

category: 'Deprecated'
method: AbstractDictionary
detectValues: aBlock ifNone: exceptionBlock

self deprecated: 'AbstractDictionary>>detectValues:ifNone: deprecated long before v3.0. Use keysAndValuesDo: instead.'.
self keysAndValuesDo: [ :aKey :aValue |
  (aBlock value: aValue) ifTrue: [ ^ aKey ].
  ].
^ exceptionBlock value.
%

category: 'Enumerating'
method: AbstractDictionary
do: aBlock

"Iteratively evaluates the one argument block, aBlock, using the value part of
 each Association or key-value pair as the argument of the block.  Returns the
 receiver."

^ self subclassResponsibility: #do.
%

category: 'Deprecated'
method: AbstractDictionary
doAssociations: aBlock

self deprecated: 'AbstractDictionary>>doAssociations: deprecated long before v3.0. Use associationsDo: instead.'.
^ self associationsDo: aBlock.
%

category: 'Deprecated'
method: AbstractDictionary
doKeys: aBlock

self deprecated: 'AbstractDictionary>>doKeys: deprecated long before v3.0. Use keysDo: instead.'.
^ self keysDo: aBlock.
%

category: 'Deprecated'
method: AbstractDictionary
doKeysAndValues: aBlock

self deprecated: 'AbstractDictionary>>doKeysAndValues: deprecated long before v3.0. Use keysAndValuesDo: instead.'.
^ self keysAndValuesDo: aBlock.
%

category: 'Deprecated'
method: AbstractDictionary
doValues: aBlock

self deprecated: 'AbstractDictionary>>doValues: deprecated long before v3.0. Use valuesDo: instead.'.
^ self valuesDo: aBlock
%

category: 'Private'
method: AbstractDictionary
errorKeyNotFound: aKey

"No Association or key/value pair with the given key, aKey was found."

^ self _error: #rtErrKeyNotFound args: { aKey }
%

category: 'Private'
method: AbstractDictionary
errorNilKey

"A nil key was provided as an argument."

^ self _error: #rtErrNilKey.
%

category: 'Private'
method: AbstractDictionary
errorNoElementDetected: aBlock

"Private."

^ self _error: #assocErrNoElementsDetected args: { aBlock }
%

category: 'Comparing'
method: AbstractDictionary
hash

"Returns a numeric hash key for the receiver."

| hashValue |

hashValue := 97633 bitXor: (self size).
"For large dictionaries, the hash value is just a function of its size"
(self size > 64) ifTrue: [ ^ hashValue abs ].
self keysDo: [ :aKey |
   "Skip if the key is a dictionary."
   (aKey isKindOf: AbstractDictionary)
     ifFalse: [
       hashValue := hashValue bitXor: aKey hash
       ]
     ].
^ hashValue abs
%

category: 'Hashing'
method: AbstractDictionary
hashFunction: aKey

"The hash function should perform some operation on the value of the
 key (aKey) which returns a value in the range 1..tableSize."

^(aKey hash \\  self tableSize) + 1
%

category: 'Searching'
method: AbstractDictionary
includes: aValue

"Returns true if the receiver contains a value that is equal to aValue.
 Returns false otherwise."

self valuesDo: [ :element | (aValue = element) ifTrue: [ ^true ]].
^ false.
%

category: 'Searching'
method: AbstractDictionary
includesAssociation: anAssociation

"Returns true if anAssociation is equal to one of the Associations of the
 receiver.  Returns false otherwise."

self associationsDo: [ :assoc | (anAssociation = assoc) ifTrue: [ ^true ]].
^false
%

category: 'Searching'
method: AbstractDictionary
includesIdentical: aValue

"Returns true if the receiver contains a value that is identical to aValue.
 Returns false otherwise."

self valuesDo: [ :element | (aValue == element) ifTrue: [ ^true ]].
^ false.
%

category: 'Searching'
method: AbstractDictionary
includesIdenticalAssociation: anAssociation

"Returns true if anAssociation is identical to one of the Associations of the
 receiver.  Returns false otherwise."

self associationsDo: [ :assoc | (anAssociation == assoc) ifTrue: [ ^true ]].
^false
%

category: 'Searching'
method: AbstractDictionary
includesKey: aKey

"Returns true if the receiver contains an Association or a key-value pair whose
 key is equal to aKey.  Returns false otherwise."

| res |
res := true .
self at: aKey ifAbsent:[ res := false ] .
^ res .
%

category: 'Deprecated'
method: AbstractDictionary
includesValue: aValue

self deprecated: 'AbstractDictionary>>includesValue: deprecated long before v3.0. Use includes: instead.'.
^ self includes: aValue.
%

category: 'Accessing'
method: AbstractDictionary
keyAtValue: anObject

"Returns the key of the first value equal to anObject. If no match is found,
 runtime error objErrNotInColl is signaled."

"Note: In some implementations of Smalltalk, nil is returned if a match is
 not found."

^self keyAtValue: anObject
      ifAbsent: [^ self _error: #objErrNotInColl args: { anObject }]
%

category: 'Accessing'
method: AbstractDictionary
keyAtValue: anObject ifAbsent: aBlock

"Returns the key of the first value equal to the given object, anObject.
 If no match is found, evaluates and returns the result of the block aBlock."

^ self subclassResponsibility: #keyAtValue:ifAbsent:
%

category: 'Accessing'
method: AbstractDictionary
keys

"Returns a Set containing the receiver's keys."

| aSet nKeys |
nKeys := self size .
aSet := Set new: nKeys + (nKeys // 4) . "avoid rebuild table"
self keysDo: [ :key | aSet add: key ].
^ aSet
%

category: 'Enumerating'
method: AbstractDictionary
keysAndValuesDo: aBlock

"Iteratively evaluates the two argument block, aBlock, using each key and value
 of the receiver as the argument to the block.  Returns the receiver."

^ self subclassResponsibility: #keysAndValuesDo:.
%

category: 'Enumerating'
method: AbstractDictionary
keysDo: aBlock

"Iteratively evaluates the one argument block, aBlock, using each key of
 the receiver as the argument to the block. Returns the receiver."

^ self subclassResponsibility: #keysDo:.
%

category: 'Storing and Loading'
method: AbstractDictionary
loadFrom: passiveObj size: varyingSize

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into the receiver."

"This method is similar to loadFrom:, but is used for objects whose size
 is not known when they are first instantiated (such as an IdentitySet)."

^self basicLoadFrom: passiveObj size: varyingSize
%

category: 'Instance Migration'
method: AbstractDictionary
migrateIndexable: anotherObject myClass: cls otherClass: othercls

"For dictionaries, we need to reconstruct the indexable component rather
 than just copy it over (in case the new class uses a different structure)."

anotherObject keysAndValuesDo: [ :key :value |
    self at: key put: value ].

%

category: 'Updating'
method: AbstractDictionary
objectSecurityPolicy: anObjectSecurityPolicy

  "Assigns the receiver and all its components to the given security policy.
   Returns the receiver."

  super objectSecurityPolicy: anObjectSecurityPolicy.
  self _nodesObjectSecurityPolicy: anObjectSecurityPolicy.
%

category: 'Searching'
method: AbstractDictionary
occurrencesOf: aValue

"Returns the number of Associations or key-value pairs in the receiver with
 value equal to 'aValue'."

| numOccurrences |

numOccurrences := 0.
self valuesDo: [ :aVal |
  aValue = aVal ifTrue: [ numOccurrences := numOccurrences + 1 ]
  ].

^ numOccurrences
%

category: 'Searching'
method: AbstractDictionary
occurrencesOfIdentical: aValue

"Returns the number of Associations or key-value pairs in the receiver with
 a value that is identical to aValue."

| numOccurrences |

numOccurrences := 0.
self valuesDo: [ :aVal |
  aValue == aVal ifTrue: [ numOccurrences := numOccurrences + 1 ]
  ].

^ numOccurrences
%

category: 'Deprecated'
method: AbstractDictionary
occurrencesOfValue: aValue

self deprecated: 'AbstractDictionary>>occurrencesOfValue: deprecated long before v3.0. Use occurrencesOf: instead.'.
^ self occurrencesOf: aValue.
%

category: 'Json'
method: AbstractDictionary
printJsonOn: aStream

	| delimiter |
	delimiter := ''.
	aStream nextPut: ${.
	self keysAndValuesDo: [:key :value |
		aStream nextPutAll: delimiter.
		key asString printJsonOn: aStream.
		aStream nextPut: $:.
		value printJsonOn: aStream.
		delimiter := ','.
	].
	aStream nextPut: $}.
%

category: 'Formatting'
method: AbstractDictionary
printNonRecursiveRepresentationOn: aStream recursionSet: anIdentitySet
	"Put a displayable representation of the receiver on the given stream
	 while avoiding recursion from object reference loops."

	| count sz myCls |

	myCls := self class .
	aStream nextPutAll: myCls name describeClassName .
	(myCls whichClassIncludesSelector: #associationsDo:) == AbstractDictionary
		ifTrue:[ ^ self "can't safely execute associationsDo: " ].

	aStream nextPutAll: '( ' .
	count := 1 .
	sz := self size .
	self associationsDo: [:anAssoc |
		aStream position > 700 ifTrue:[
			"prevent infinite recursion when printing cyclic structures, and
			limit the size of result when printing large collections."
			aStream nextPutAll: '...)' .
			^ self
			] .
		anAssoc printOn: aStream recursionSet: anIdentitySet.
		count < sz ifTrue:[ aStream nextPutAll: ', ' ].
		count := count + 1.
		].
	aStream nextPut: $).
%

category: 'Formatting'
method: AbstractDictionary
printOn: aStream
	"Put a displayable representation of the receiver on the given stream."

	self printNonRecursiveOn: aStream
%

category: 'Enumerating'
method: AbstractDictionary
reject: aBlock

"Evaluates aBlock with each of the receiver's values as the argument. Stores
 the key-value pairs for which aBlock is false into a dictionary of the same
 class as the receiver, and returns the new dictionary. The argument aBlock
 must be a one-argument block."

| result |

result := self species new: (self size).
self keysAndValuesDo: [ :aKey :aValue |
  (aBlock value: aValue) ifFalse: [result at: aKey put: aValue]
  ].

^ result.
%

category: 'Enumerating'
method: AbstractDictionary
rejectAssociations: aBlock

"Evaluates aBlock with each of the receiver's elements as the argument.  Stores
 the values for which aBlock is false into a collection of the same class as
 the receiver, and returns the new collection.  The argument aBlock must be a
 one-argument block.  Uses associative access when the argument is a
 SelectionBlock."

| newCollection |

newCollection := self species new.
self associationsDo: [ :element |
  (aBlock value: element) ifFalse: [ newCollection add: element ].
  ].

^ newCollection.
%

category: 'Deprecated'
method: AbstractDictionary
rejectValues: aBlock

self deprecated: 'AbstractDictionary>>rejectValues: deprecated long before v3.0. Use reject: instead.'.
^ self reject: aBlock.
%

category: 'Enumerating'
method: AbstractDictionary
rejectValuesAsArray: aBlock

"Evaluates aBlock with each of the receiver's values as the argument and
 returns an Array containing the values for which aBlock evaluates false."

| result |
result := { } .
self valuesDo: [ :aValue |
  (aBlock value: aValue) ifFalse: [result add: aValue].
  ].

^ result.
%

category: 'Removing'
method: AbstractDictionary
remove: anObject

"Disallowed.  Use #removeKey: instead."

self shouldNotImplement: #remove:
%

category: 'Removing'
method: AbstractDictionary
remove: anObject ifAbsent: anExceptionBlock

"Disallowed.  Use #removeKey:ifAbsent: instead."

self shouldNotImplement: #remove:ifAbsent.
%

category: 'Removing'
method: AbstractDictionary
removeAll: aCollection

"Disallowed.  Use #removeAllKeys: instead."

self shouldNotImplement: #removeAll:
%

category: 'Removing'
method: AbstractDictionary
removeAllIdentical: aCollection

"Disallowed."

self shouldNotImplement: #removeAllIdentical:
%

category: 'Removing'
method: AbstractDictionary
removeAllKeys: keys

"Removes all the keys equal to the given keys from the receiver. An error is not
 generated if keys equal to any of the specified keys are not present. Returns
 the collection keys."

^ keys accompaniedBy: self do: [ :me :aKey | me removeKey: aKey ifAbsent: [nil] ].
%

category: 'Removing'
method: AbstractDictionary
removeAllKeys: keys ifAbsent: aBlock

"Removes all the keys equal to the given keys from the receiver and returns the
 collection keys. For any key which is not a valid key of the receiver, aBlock
 is evaluated with the key as the argument."

^ keys accompaniedBy: self do: [:me :aKey | me removeKey: aKey ifAbsent: [aBlock value: aKey] ].
%

category: 'Removing'
method: AbstractDictionary
removeIdentical: anObject

"Disallowed."

self shouldNotImplement: #removeIdentical:
%

category: 'Removing'
method: AbstractDictionary
removeIdentical: anObject ifAbsent: anExceptionBlock

"Disallowed."

self shouldNotImplement: #removeIdentical:ifAbsent.
%

category: 'Removing'
method: AbstractDictionary
removeKey: aKey

"Removes the Association or key-value pair with key equal to aKey from the
 receiver and returns the value portion of that Association or key-value pair
 respectively. If no Association is present with key equal to aKey, reports an
 error."

^ self removeKey: aKey ifAbsent: [self _errorKeyNotFound: aKey ].
%

category: 'Removing'
method: AbstractDictionary
removeKey: aKey ifAbsent: aBlock

"Removes the Association or key-value pair with key equal to aKey from the
 receiver and returns the value associated with the Association or the key-value
 pair.  If no Association or key-value pair is present with key equal to
 aKey, evaluates the zero-argument block aBlock and returns the result of that
 evaluation."

^ self subclassResponsibility: #removeKey:ifAbsent.
%

category: 'Removing'
method: AbstractDictionary
removeKey: aKey otherwise: notFoundValue

"Removes the Association or key-value pair with key equal to aKey from the
 receiver and returns the value associated with the Association or the key-value
 pair.  If no Association or key-value pair is present with key equal to
 aKey, returns notFoundValue .

 KeyValueDictionary has an optimized implementation.
"

^ self removeKey: aKey ifAbsent:[ notFoundValue ]
%

category: 'Deprecated'
method: AbstractDictionary
removeKeys: keys

self deprecated: 'AbstractDictionary>>removeKeys: deprecated long before v3.0. Use removeAllKeys: instead.'.
^ self removeAllKeys: keys
%

category: 'Enumerating'
method: AbstractDictionary
select: aBlock

"Evaluates aBlock with each of the receiver's values as the argument.  Stores
 the values for which aBlock is true into the dictionary of the same class as
 the receiver, and returns the new dictionary.  The argument aBlock must be a
 one-argument block."

|result|

result := self species new.
self keysAndValuesDo: [ :aKey :aValue |
  (aBlock value: aValue) ifTrue: [result at: aKey put: aValue]
  ].

^ result.
%

category: 'Enumerating'
method: AbstractDictionary
selectAssociations: aBlock

"Evaluates aBlock with each of the receiver's elements as the argument.  Stores
 the values for which aBlock is true into a collection of the same class as the
 receiver, and returns the new collection.  The argument aBlock must be a
 one-argument block.  Uses associative access when the argument is a
 SelectionBlock."

| newCollection |

newCollection := self species new.
self associationsDo: [ :element |
  (aBlock value: element) ifTrue: [ newCollection add: element ].
  ].

^ newCollection.
%

category: 'Enumerating'
method: AbstractDictionary
selectAssociationsAsArray: aBlock

"Evaluates aBlock with each of the receiver's associations as the argument.  Stores
 the values for which aBlock is true into a new Array and returns the Array.
 The argument aBlock must be a one-argument block.  "

| newCollection |

newCollection := { }  .
self associationsDo: [ :element |
  (aBlock value: element) ifTrue: [ newCollection add: element ].
  ].

^ newCollection.
%

category: 'Deprecated'
method: AbstractDictionary
selectValues: aBlock

self deprecated: 'AbstractDictionary>>selectValues: deprecated long before v3.0. Use select: instead.'.
^ self select: aBlock.
%

category: 'Deprecated'
method: AbstractDictionary
selectValuesAsArray: aBlock


"Evaluates aBlock with each of the receiver's values as the argument and
 returns an Array containing the values for which aBlock evaluates true."

| result index |
self deprecated: 'AbstractDictionary>>selectValuesAsArray: deprecated long before v3.0.'.
result := Array new: (self size).
index := 1.
self valuesDo: [ :aValue |
  (aBlock value: aValue)
    ifTrue: [result at: index put: aValue].
  index := index + 1.
  ].

^ result.
%

category: 'Accessing'
method: AbstractDictionary
size

"Returns the number of elements (Associations/key-value pairs) contained in the
 receiver."

^ self subclassResponsibility: #size.
%

category: 'Updating'
method: AbstractDictionary
size: newSize

"Disallowed.  You should not change the size of a dictionary explicitly."

self shouldNotImplement: #size: .
%

category: 'Accessing'
method: AbstractDictionary
squeakBasicAt: anIndex
  ^ self _basicAt: anIndex
%

category: 'Updating'
method: AbstractDictionary
squeakBasicAt: anIndex put: aValue

  ^ self _basicAt: anIndex put: aValue
%

category: 'Private'
method: AbstractDictionary
tableSize

"Returns the size of hash table used for storing the entries."

^ self subclassResponsibility: #tableSize.
%

category: 'Accessing'
method: AbstractDictionary
values

"Returns an OrderedCollection containing the receiver's values."

|result|
result:= OrderedCollection new.
self valuesDo: [ :value | result add: value ].
^ result.
%

category: 'Enumerating'
method: AbstractDictionary
valuesDo: aBlock

"Iteratively evaluates the one argument block, aBlock, using each value of
 the receiver as the argument to the block. Returns the receiver."

"Note: This method has the same behavior as #do:."

^ self subclassResponsibility: #valuesDo:.
%

category: 'Private'
method: AbstractDictionary
_asCollectionForSorting

"Creates a new collection that can be easily indexed for sorting."

| result |

result := { } .
self do: [ :aValue | result add: aValue].
^ result.
%

category: 'Accessing'
method: AbstractDictionary
_at: anIndex

"Signals an error if the receiver is not indexable,
 or if anIndex is not a SmallInteger or is out of bounds.

 This method is for internal use in implementation of the kernel classes.

 The methods _at:, _basicAt:, and _basicSize operate on the logically
 indexable contents of the receiver.

 This method must not be overridden."

<primitive: 699>
(anIndex _isInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #at: args: { anIndex }.
self _uncontinuableError
%

category: 'Updating'
method: AbstractDictionary
_at: anIndex put: aValue

"Stores the argument aValue in the indexed variable of the
 receiver indicated by anIndex.  The argument anIndex must not be
 larger than 1 + the size of the receiver, and must not be less than 1.

 Returns aValue.

 Generates an error if anIndex is not a SmallInteger or is out of
 bounds."

<primitive: 745>

(anIndex _isInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
((anIndex < 1) | (anIndex > (self size + 1))) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].

self _primitiveFailed: #at:put: args: { anIndex . aValue }.
self _uncontinuableError
%

category: 'Private'
method: AbstractDictionary
_atZ: zOffset
  "return varying instVar specified by zero-based offset zOffset"
<primitive: 858>
| sz |
zOffset _validateClass: SmallInteger .
sz := self _basicSize .
(zOffset < 0 or:[ zOffset >= sz ]) ifTrue:[
  OffsetError new offset: zOffset maximum: sz - 1 ; signal
].
self _primitiveFailed: #_atZ: args: { zOffset }
%

category: 'Private'
method: AbstractDictionary
_atZ: zOffset put: aValue
  "store aValue into varying instVar specified by zero-based offset zOffset.
   will auto-grow up to 6 instVars past current end"
<primitive: 860>
| max |
zOffset _validateClass: SmallInteger .
max := self _basicSize + 6 .
(zOffset < 0 or:[ zOffset >= max ]) ifTrue:[
  OffsetError new offset: zOffset maximum: max ; signal
].
self _primitiveFailed: #_atZ:put: args: { zOffset . aValue }
%

category: 'Private'
method: AbstractDictionary
_atZ: zOffset putKey: aKey value: aValue
  "store aKey into varying instVar specified by zero-based offset zOffset.
   store aValue into varying instVar specified by zero-based offset zOffset + 1.
   will auto-grow up to 7 instVars past current end"
<primitive: 892>
| max |
zOffset _validateClass: SmallInteger .
max := self _basicSize + 6 .
(zOffset < 0 or:[ zOffset >= max ]) ifTrue:[
  OffsetError new offset: zOffset maximum: max ; signal
].
self _primitiveFailed: #_atZ:putKey:value:
     args: { zOffset . aKey . aValue }
%

category: 'Accessing'
method: AbstractDictionary
_basicAt: anIndex

"Signals an error if the receiver is not indexable,
 or if anIndex is not a SmallInteger or is out of bounds.

 This method is for internal use in implementation of the kernel classes.

 The methods _at:, _basicAt:, and _basicSize operate on the logically
 indexable contents of the receiver.

 This method must not be overridden."

<primitive: 699>
(anIndex _isInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #at: args: { anIndex }.
self _uncontinuableError
%

category: 'Updating'
method: AbstractDictionary
_basicAt: anIndex put: aValue

"Stores the argument aValue in the indexed variable of the
 receiver indicated by anIndex.  The argument anIndex must not be
 larger than 1 + the size of the receiver, and must not be less than 1.

 Returns aValue.

 Generates an error if anIndex is not a SmallInteger or is out of
 bounds."

<primitive: 745>

(anIndex _isInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
((anIndex < 1) | (anIndex > (self size + 1))) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].

self _primitiveFailed: #at:put: args: { anIndex . aValue }
%

category: 'Private'
method: AbstractDictionary
_deepCopyWith: copiedObjDict

| copy myClass |

copy := copiedObjDict at: self otherwise: nil.
copy ~~ nil ifTrue: [ ^ copy ].

myClass := self class .
copy := myClass new: (self size).
copiedObjDict at: self put: copy.

self _deepCopyNamedIvsWith: copiedObjDict to: copy .

self keysAndValuesDo: [ :aKey :aValue |
  copy at: (aKey _deepCopyWith: copiedObjDict)
       put: (aValue _deepCopyWith: copiedObjDict).
  ].

^ copy.
%

category: 'Private'
method: AbstractDictionary
_deferredGciUpdateWith: valueArray

^ self subclassResponsibility: #_deferredGciUpdateWith:
%

category: 'Private'
method: AbstractDictionary
_errorKeyNotFound: aKey

"No Association or key-value pair with given key, aKey, was found."

^ self _error: #rtErrKeyNotFound args: { aKey }
%

category: 'Private'
method: AbstractDictionary
_nodesObjectSecurityPolicy: anObjectSecurityPolicy
  "Assigns receiver's components to the given security policy. "

  self subclassResponsibility: #_nodesObjectSecurityPolicy:
%

category: 'Private'
method: AbstractDictionary
_reportKeyNotFound: aKey with: aBlock

"Returns the value of aBlock if aBlock is not nil.  Otherwise, raises an error."

aBlock == nil ifTrue:[^ self _errorKeyNotFound: aKey ] .
^aBlock value
%

! Class extensions for 'AbstractException'

!		Class methods for 'AbstractException'

removeallmethods AbstractException
removeallclassmethods AbstractException

category: 'ExceptionSet support'
classmethod: AbstractException
, anExceptionSetOrClass

 "Return an ExceptionSet containing the receiver and argument."

  ^ ExceptionSet with: self with: anExceptionSetOrClass
%

category: 'Default handlers'
classmethod: AbstractException
addDefaultHandler: handlerBlock

"Install the one-argument block handlerBlock as a static GsExceptionHandler
 to field exceptions whose classes are subclasses of the receiver.
 The handlerBlock may send  #resume: , #pass, or #outer to the
 instance of Exception .

 If handlerBlock returns normally, or sends #return:
 to the instance of Exception, an Error will be signalled
 because there is no send of on:do:  which installed handlerBlock .

 Returns a new instance of GsExceptionHandler .

 When searching for a handlerBlock to handle a signaled exception, the VM
 uses Behavior>>_subclassOf: semantics . classHistories of the
 class of the signaled exception and of self are ignored.  "

^ self _installStaticException: handlerBlock class: self
       category: nil number: nil subtype: nil
%

category: 'Legacy Handlers'
classmethod: AbstractException
block: aBlock category: aCategory number: num subtype: atype

"This form of creating a Legacy handler is no longer supported.
 You must use category:number:do:  or  installStaticException:category:number: ."

self shouldNotImplement: #block:category:number:subtype: .
self _uncontinuableError
%

category: 'Legacy Handlers'
classmethod: AbstractException
category: aCategory number: aNumber do: handlerBlock

"Install a legacy exception handler in the frame of the sender of
   category:number:do:  .
The handler is represented as an instance of GsExceptionHandler .

aCategory is ignored; it is assumed to be nil or
the SymbolDictionary GemStoneError .
All exceptions except those with instVar gsTrappable==false  are trapped,
subject to the value of aNumber .

handlerBlock must be a 4-argument instance of ExecBloc.

aNumber is an error number of an exception to trap
or nil (to trap all exceptions) .

The subtype of the handler is always nil."

<primitive: 376>
" aCategory ~~ nil ifTrue:[ aCategory _validateClass: SymbolDictionary ] . "
aNumber ~~ nil ifTrue:[ aNumber _validateClass: SmallInteger ] .
handlerBlock _validateClass: ExecBlock .
handlerBlock argumentCount == 4 ifFalse:[
  handlerBlock _error: #rtErrBadBlockArgCount args: { 4 . handlerBlock argumentCount } .
  ] .
^ self _primitiveFailed: #category:number:do:
       args: { aCategory . aNumber . handlerBlock }
%

category: 'Debugging Support'
classmethod: AbstractException
cpuOsKind

"Returns an integer specifying the CPU and OS on which
 this VM is running. The result is one of
   1 - reserved for use within VM
   2 - sparc Solaris
   3 - x86_64 Linux
   4 - PowerPc AIX
   5 - x86_64 Apple Unix
   6 - x86_64 Solaris
   7 - Itanium HPUX
"
<primitive: 345>
self _primitiveFailed: #cpuOsKind
%

category: 'Debugging Support'
classmethod: AbstractException
cpuOsKindString

^ #( 'unknown'
     'sparc-solaris'
     'x86_64-linux'
     'powerpc-AIX'
     'x86_64-darwin'
     'x86_64-solaris'
     'ia64-HPUX'       ) at: self cpuOsKind
%

category: 'Default handlers'
classmethod: AbstractException
defaultHandlers
  "Returns all currently installed ANSI default handlers which
   handle the receiver. "
  | handler res hcls |
  res := { } .
  handler := self _staticExceptions .
  [ handler ~~ nil ] whileTrue:[
    (hcls := handler exceptionClass) ifNotNil:[
      (self _subclassOf: hcls) ifTrue:[ res add: handler ].
    ].
    handler := handler next
  ].
  ^ res
%

category: 'Documentation'
classmethod: AbstractException
errnoTables
 "Returns the class variable ErrnoTables .
  The class variable ErrnoTables is a zero-based invariant Array ,
   to be indexed by the one-based result of  AbstractException(C)>>cpuOsKind .

   Each element of ErrnoTables is an Array of Strings, to be accessed
     by a one-based errno value, translating the errno value to a name.

   On some cpu/OS there are multiple names for some errno values.
   The tables contain only the preferred name, as follows
     Solaris:  EWOULDBLOCK==EAGAIN , EAGAIN is the preferred name .
     Linux: EWOULDBLOCK == EAGAIN, EAGAIN is the preferred name ,
            EDEADLOCK == EDEADLK,  EDEADLK is the preferred name .
     Apple Unix: EWOULDBLOCK==EAGAIN , EAGAIN is the preferred name ,
                 EOPNOTSUPP == ENOTSUP, ENOTSUP is the preferred name .
  "
  ^ ErrnoTables
%

category: 'Documentation'
classmethod: AbstractException
errnoToName: aSmallInteger
 "Return the name for specified errno value, or nil if argument
  is out of range.

  See AbstractException>>errnoTables for discussion of preferred names
"

 | arr |
 arr := ErrnoTables at: self cpuOsKind .
 (aSmallInteger >= 1 and:[aSmallInteger <= arr size]) ifTrue:[
   ^ arr at: aSmallInteger .
 ].
 ^ nil
%

category: 'ExceptionSet support'
classmethod: AbstractException
handles: anException

^ anException isKindOf: self
%

category: 'Debugging Support'
classmethod: AbstractException
installDebugBlock: aBlock

"aBlock must be either nil or a one-argument ExecBlock taking an instance
 of AbstractException as the argument.
 aBlock == nil clears any previously installed block from the VM.
 Each GciLogin includes an automatic   AbstractException installDebugBlock: nil
 for the newly created session.

 aBlock ~~ nil installs the specified block into VM transient state,
 for use by AbstractException>>_debugException:  .
 aBlock will be sent   value: anException   prior to invocation of any
 handler block installed with an on:do:  (or prior to _executeGsHandler:).
 If debugger desires to handle the exception normally, aBlock must return
 normally (i.e. do not return from home, or signal any exception within aBlock).

 aBlock will not be invoked for non-trappable exceptions such as Object>>pause.

 aBlock is only executed if a matching handler was found on the stack.
 If there is no matching handler (i.e. no matching on:do:) then the
 exception is signalled directly to the GCI client.
"

<primitive: 798>
aBlock _validateClass: ExecBlock .
aBlock numArgs ~~ 1 ifTrue:[ ArgumentError signal:'must be a 1-arg block' ].
self _primitiveFailed: #installDebugBlock args: { aBlock }
%

category: 'Static legacy handlers'
classmethod: AbstractException
installStaticException: handlerBlock category: category number: num

"Install the 4-argument handlerBlock as a static GsExceptionHandler
 to field errors of the specified category and number.
 Returns a new instance of GsExceptionHandler "

^ self _installStaticException: handlerBlock class: nil
       category: category number: num subtype: nil
%

category: 'Static legacy handlers'
classmethod: AbstractException
installStaticException: handlerBlock category: aCategory number: aNumber subtype: atype

"Install the 4-argument handlerBlock at the head of the static handlers
 list, not associated with a particular stack frame ,
 to field errors of the specified category, number, and subtype.

 The handler is represented as an instance of GsExceptionHandler,
 and will be a part of the session state.
 The new  GsExceptionHandler is returned ."

^ self _installStaticException: handlerBlock class: nil
      category: aCategory number: aNumber subtype: atype
%

category: 'Instance creation'
classmethod: AbstractException
new
  "Return a newly created object initialized to a standard initial state"

  ^ self _basicNew initialize
%

category: 'Legacy Handlers'
classmethod: AbstractException
removeActivationException: aGsExceptionHandler

"Search the current GemStone Smalltalk call stack for a method or block context
 that specified handler installed, and remove it.  The stack is searched by
 starting with the top method or block context and moving down.  Generates an
 error if aGsExceptionHandler is not installed anywhere in the current GemStone
 Smalltalk call stack or if aGsExceptionHandler is not an instance of
 GsExceptionHandler .

 Does not search the static legacy handlers list."

^ GsExceptionHandler removeActivationException: aGsExceptionHandler

%

category: 'Default handlers'
classmethod: AbstractException
removeAllDefaultHandlers

"Removes all ANSI default handlers and static legacy handlers.
 A faster implementation of
  AbstractException _staticExceptions do:[:h | h remove ].
"
<primitive: 380>
^ self _primitiveFailed: #removeAllDefaultHandlers
%

category: 'Legacy Handlers'
classmethod: AbstractException
removeStaticException: aGsExceptionHandler

"Remove a static exception handler (either an ANSI default handler,
 or a static legacy handler.)
 Returns the removed handler, or nil if not found by identity."

<primitive: 379>
aGsExceptionHandler _validateClass: GsExceptionHandler .
^ self _primitiveFailed: #removeStaticException args: { aGsExceptionHandler }
%

category: 'Instance creation'
classmethod: AbstractException
signal

 "An exception of the type associated with the receiver is signaled."

  ^ self new _signalWith: nil
%

category: 'Instance creation'
classmethod: AbstractException
signal: signalText

 "An exception of the type associated with the receiver is signaled."
  ^ self new signal: signalText.
%

category: 'Instance creation'
classmethod: AbstractException
signalNotTrappable
  ^ self new signalNotTrappable
%

category: 'Instance creation'
classmethod: AbstractException
signalNotTrappable: signalText
  ^ self new details: signalText ;  signalNotTrappable
%

category: 'SUnit'
classmethod: AbstractException
sunitSignalWith: aString
        ^self signal: aString
%

category: 'Private'
classmethod: AbstractException
_installStaticException: handlerBlock
  class: exceptionClass
  category: aCategory number: aNumber subtype: atype

"Install the specified exception block at the head of the static handlers
 list, not associated with a particular stack frame ,
 to field exceptions of the specified class .
 or errors of the specified category, number, and subtype.
 If exceptionClass is not nil   ignores  aCategory,  aNumber, atype  ;
 else assumes aCategory to be GemstoneError .

 The handler is represented as an instance of GsExceptionHandler,
 and will be a part of the session state.
 The new  GsExceptionHandler is returned ."
<primitive: 378>
| nBlockArgs |
handlerBlock _validateClass: ExecBlock .
exceptionClass ifNotNil:[
  nBlockArgs := 1 .
  exceptionClass _validateClass: AbstractException .
] ifNil:[
  nBlockArgs := 4 .
  " aCategory ifNotNil:[ aCategory _validateClass: SymbolDictionary ] . "
  aNumber ifNotNil:[ aNumber _validateClass: SmallInteger ] .
].
handlerBlock argumentCount ~~ nBlockArgs ifTrue: [
  ^ self _error: #rtErrExceptBlockNumArgs args: { nBlockArgs . handlerBlock argumentCount }.
].
^ self _primitiveFailed: #_installStaticException:class:category:number:subtype:
       args: { handlerBlock . exceptionClass . aCategory . aNumber . atype }
%

category: 'Private Legacy'
classmethod: AbstractException
_new: anInteger args: anArray

"return a new exception for the specified error number."

<primitive: 140>
  "If args ok and handler found, this stack frame reused to invoke handler.
   If exception handling succeeds and execution is to resume,
   we return from this method with the resumption value."
anInteger _validateClass: SmallInteger.
anInteger < 1 ifTrue: [
  ^ self _error: #rtErrBadErr args: { anInteger . anArray  }
].
anArray _validateClass: Array.
anArray size > 10 ifTrue: [
 ^ self _error: #rtErrTooManyErrArgs args: { anArray size }
].
self _primitiveFailed: #_new:args: args: { anArray }.
self _uncontinuableError
%

category: 'Private'
classmethod: AbstractException
_staticExceptions

"Returns the head of the static exception list. The list includes
 static legacy handlers, and ANSI default handlers.  Elements of the
 list are instances of GsExceptionHandler."

<primitive: 377>
^ self _primitiveFailed: #_staticExceptions
%

!		Instance methods for 'AbstractException'

category: 'Accessing'
method: AbstractException
addText: aString
  messageText ifNil:[ self asString "build the default messageText" ].
  messageText := (messageText ifNil:[ String new ]), ', ', aString .
%

category: 'Accessing'
method: AbstractException
arg: anObject
  gsArgs ifNotNil:[:a | a _isArray ifTrue:[ a add: anObject ]]
       ifNil:[ gsArgs := { anObject } ]
%

category: 'Accessing'
method: AbstractException
args: anArray
  gsArgs :=  anArray ifNotNil:[:a| a _isArray ifTrue:[ a ]
                                           ifFalse:[ { a } ]]
%

category: 'Compatibility'
method: AbstractException
category
  "Will be deprecated"
  ^ GemStoneError
%

category: 'Handling'
method: AbstractException
defaultAction

"Return an error to the controlling GCI client. Stack is saved
 and available as an argument to the GCI error struct.

 Instance variable of the receiver are enumerated into the GciErrSType.args
 as follows.
 If there are named instVars after AbstractException.args
 the values of those instVars are stored into GciErrSType.args .
 AbstractException.args is then added if space is available.

 If AbstractException.args is the last named instVar and it is an Array
 of size <= GCI_MAX_ERR_ARGS , it is enumerated into GciErrSType.args.
 otherwise the oop of the Array is stored into GciErrSType.args[0] .

 See also ClientForwarderSend >> defaultAction "

<primitive: 33>  "this frame removed from stack before saving stack."
self _uncontinuableError
%

category: 'Accessing'
method: AbstractException
describe
  "used by topaz and VM's error message building"

  ^ self asString
%

category: 'Accessing'
method: AbstractException
description
  "Returns a String."
 | res |
 gsDetails ifNotNil:[ :dt |
   res := [
     | str |
     (str := String withAll: self class name)
         add: ': ' ; add: dt asString  .
     str
   ] onException: Error do:[:ex |
     ex return: nil "ignore"
   ]
 ].
 ^ res ifNil:[ self asString ]
%

category: 'Accessing'
method: AbstractException
details
  ^ gsDetails ifNil:[ messageText ]
%

category: 'Accessing'
method: AbstractException
details: aStringOrArray
  "If arg is a String it will be appended to the fully formed error
   message.

   If arg is an Array and the receiver does not reimplement buildMessageText ,
   the arg will be used as a legacy style error message template."

  gsDetails := aStringOrArray  "store ANSI messageText"
%

category: 'Accessing'
method: AbstractException
genericDescription
  ^  self asString
%

category: 'Compatibility'
method: AbstractException
gsArguments
  "Will be deprecated"
  "Returns an Array"

  ^ self _legacyHandlerArgs
%

category: 'Compatibility'
method: AbstractException
gsCategory
  "Will be deprecated"
  ^ GemStoneError
%

category: 'Compatibility'
method: AbstractException
gsNumber
  "Will be deprecated"
  ^ gsNumber
%

category: 'Instance initialization'
method: AbstractException
initialize

 "subclasses may override but must send   super initialize
  at start of the reimplementation, or else initialize
  the 3 instVars   gsNumber, gsResumable, gsTrappable . "

  gsNumber := ERR_AbstractException.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Handling'
method: AbstractException
isNested
 "When sent within an ANSI handler's handlerBlock ,
  returns true if there is an enclosing handler below the  on:do:
  of the active handler that could handle the receiver.

  Returns false if sent within a legacy handler block.
  "

 <primitive: 700>
 self _uncontinuableError.
%

category: 'Accessing'
method: AbstractException
isResumable
  ^ gsResumable .
%

category: 'Accessing'
method: AbstractException
isTrappable
  ^ gsTrappable ~~ false
%

category: 'Accessing'
method: AbstractException
messageText: aString
  gsDetails := aString   "store ANSI messageText"
%

category: 'Legacy Accessing'
method: AbstractException
next

  "No longer supported.
   Within a legacyHandler block, to get the next legacyHandler after
   the one currently active  use
       anException legacyHandler next
   where anException is the first arg to the legacyHandler block. "

  self shouldNotImplement: #next .
  self _uncontinuableError
%

category: 'Private'
method: AbstractException
number
  ^ gsNumber
%

category: 'Handling'
method: AbstractException
outer
  "When sent within an ANSI handler's handlerBlock ,
   search for an enclosing handler below the active one that will
   handle the receiver.

   If an enclosing handler found,  send #value: to
   the enclosing handler's handlerBlock , with receiver as the argument.
   If the receiver is 'resumable' and the enclosing handler's handlerBlock
   sends #resume: to the receiver,  the resumption value
   will be returned from this message.
   If the receiver is not 'resumable' or if the enclosing handlerBlock
   does not send #resume: , then this message will not return.

   If no such handler found,  returns the result of
   sending #defaultAction to the receiver.

   For exceptions not 'resumable', #outer is equivalent to #pass ."

   ^ self _outer: nil with: nil .
%

category: 'Handling'
method: AbstractException
pass
  "When sent within an ANSI handler's handlerBlock ,
   search for an enclosing handler below the active one that will
   handle the receiver.

   If an enclosing handler found, push a new stack frame and send #value: to
   the enclosing handler's handlerBlock , with receiver as the argument.

   If no such handler found, send #defaultAction to the receiver.
   Control does not return to the currently active handler."

   "Following resume: only happens if the _pass:with: invoked a default handler
    which returned normally. "
   self resume:( self _pass: nil with: nil ) .
   self _uncontinuableError. "should not reach here"
%

category: 'Accessing'
method: AbstractException
reason
  ^ gsReason
%

category: 'Accessing'
method: AbstractException
reason: aString
  "aString must be a Symbol or String
   Only the first 63 bytes of aString are used. "
  | str |
  aString _isSymbol ifTrue:[
    gsReason := aString asString .
  ] ifFalse:[
    (str := aString) _isOneByteString ifFalse:[
       str := aString asString .
       str _isOneByteString ifFalse:[
         str := aString class name  .
         str _isOneByteString ifFalse:[ ^ self "ignore arg" ].
       ].
    ].
    (str size > 64) ifTrue:[
       gsReason := aString copyFrom: 1 to: 63
    ] ifFalse:[
       gsReason := aString
    ].
  ].
%

category: 'Legacy Handlers'
method: AbstractException
remove

"To be sent only within a legacy handler's handlerBlock.

 Remove the receiver's  GsExceptionHandler from
 the GemStone Smalltalk call stack .

 Does not search the static legacy handlers list.
"
  currGsHandler ifNotNil:[ :h | h remove ]
                ifNil:[ self _uncontinuableError ]
%

category: 'Legacy Handlers'
method: AbstractException
resignal: anErrorDict number: anInteger args: anArray

  "When sent within a legacy handler's handlerBlock,
   create a new instance of Exception as specified by the arguments,
   and search for the next handler below this handler.

   If the newly created Exception is handled successfully,
   a normal return of that handler's handlerBlock will
   result in a return from this method.

   anErrorDict is ignored, and may be nil ."

  | ex |
  currGsHandler ifNil:[
     self _uncontinuableError. "illegal in ANSI handler"
  ].
  ex := AbstractException _new: anInteger args: anArray .
  gsResumable ifFalse:[ ex _resumable: false ].
  ^ self _legacyResignal: ex
%

category: 'Handling'
method: AbstractException
resignalAs: replacementException

  "When sent within an ANSI handler's handlerBlock,
   the stack is trimmed back to the point where the receiver was
   orginally signaled, executing any ensure: blocks installed by
   handlerBlock execution .

   Then replacementException is substituted for the originally
   signaled exception and #signal  is sent to it to restart
   exception handling.

   Generates an error if the stack trim would cross the frame
   of a C primitive or user action."

  gsResumable ifFalse:[ replacementException _resumable: false ].
  self _executeEnsuresBelow: 0 .
  self _resignalAs: replacementException  . "does not return"
  self _uncontinuableError.  "should not be here"
%

category: 'Handling'
method: AbstractException
resume

"See  resume:  for documentation ."

 self isResumable ifFalse:[
   "cannot resume from a not-resumable Exception."
   ^ self error:'cannot resume from a not-resumable Exception'
 ].
 ^ self _resume: nil
%

category: 'Handling'
method: AbstractException
resume: resumptionValue

"To be sent from within an ANSI handler's handlerBlock only.

 If the current handler block was invoked by #outer from a previous
 handler block,  #resume will return to that previous handler block,
 with resumptionValue being returned from the send of #outer .
 Otherwise resume execution from the send that signaled the receiver
 with the specified value.

 An Error is signaled if this is send directly or indirectly from
 within a defaultAction method to the receiver of defaultAction."

 self isResumable ifFalse:[
   "cannot resume from a not-resumable Exception."
   ^ self error:'cannot resume from a not-resumable Exception' .
 ].
 ^ self _resume: resumptionValue
%

category: 'Handling'
method: AbstractException
retry
  "When sent within an ANSI handler's handlerBlock ,
   resume execution by sending  #value  to the receiver of the
   #on:do: send which installed the currently active handlerBlock ,
   and using the stack frame of that #on:do: .

   Any ensure: blocks between top of stack and that #on:do: will
   be executed and stack trimmed back before resuming execution.

  Generates an error if sent within a legacy handler's handlerBlock,
  or if stack trim would cross the frame of a C primitive or user action."

^self retryUsing: nil
%

category: 'Handling'
method: AbstractException
retryUsing: alternativeBlock

  "When sent within an ANSI handler's handlerBlock ,
   resume execution in the frame of the #on:do: send which
   installed the currently active handlerBlock.

   Any ensure: blocks between top of stack and that #on:do: will
   be executed and stack trimmed back. If alternativeBlock  is non-nil
   it is substituted for the original receiver of the #on:do: .
   Then #value is sent to the non-nil alternativeBlock, else to the
   original receiver of #on:do: .

   Generates an error if sent within a legacy handler's handlerBlock,
   or if stack trim would cross the frame of a C primitive or user action."

  self _executeEnsuresBelow: 1 .
  self _retryUsing: alternativeBlock
%

category: 'Handling'
method: AbstractException
return

"When sent within an ANSI handler's handlerBlock,
 resume execution by returning nil from the #on:do: send
 which installed the currently active handlerBlock .

 Any ensure: blocks between top of stack and that #on:do:
 will be executed and the stack trimmed back.

 Generates an error if sent within a legacy handler's handlerBlock,
 or if the return would cross the frame of a C primitive or user action.
"
 self _executeEnsuresBelow: 1 .
 self _return: nil .
%

category: 'Handling'
method: AbstractException
return: returnValue

"When sent within an ANSI handler's handlerBlock,
 resume execution by returning returnValue from the #on:do: send
 which installed the currently active handlerBlock .

 Any ensure: blocks between top of stack and that #on:do:
 will be executed and the stack trimmed back.

 If return would cross a C extension , the _executeEnsuresBelow:
 will execute ensure blocks below the C frame, and
 return: will trim frames below the C frame.

 Generates an error if sent within a legacy handler's handlerBlock.
 or if the return would cross the frame of a C primitive or user action.
"
 self _executeEnsuresBelow: 1 .
 self _return: returnValue .
%

category: 'Copying'
method: AbstractException
shallowCopy
  "clears any exception handling information in the copy. Do this
   in shallowCopy  in case a subclass fails to send postCopy.  "
  | res |
  ( res := super shallowCopy ) size: 0 .
  ^ res
%

category: 'Signaling'
method: AbstractException
signal

  "The current stack is searched for an 'exception handler' which matches
  the receiver.  The search proceeds from the top of stack downwards,
  and then checks the static legacy handlers.

  ANSI handlers are installed by using
    ExecBlock>>on:do:    installs an ANSI handler
    ExecBlock>>onException:do:    installs an ANSI handler
  ANSI static handlers may be installed by
    AbstractException(C)>>defaultAction:
  Legacy handlers may be installed by
    AbstractException(C)>>category:number:do:
  Legacy static handlers may be installed by
    AbstractException(C)>>installStaticException:category:number:

  A matching ANSI handler is defined to be one which would return 'true'
  if the message #handles: was sent to its 'exception selector'
  with the 'signaled exception' as the argument.

  If a matching handler is found, its handlerBlock is executed.

    For a legacy handler or ANSI default handler, if the handlerBlock
    returns normally, execution resumes from the point of the signal ,
    with the value returned by the handlerBlock.

    For a handlerBlock installed by on:do: , if the handlerBlock returns normally,
    any ensure: blocks are executed, and execution returns from the send of on:do:
    with the value returned by the handlerBlock .  However if the return from
    the send of on:do: would cross the frame of a C primitive or user action,
    an uncontinuable error is generated.
    Within the handlerBlock, the methods
      outer, pass , resignalAs:, resume, return ,
    can be send to this instance of AbstractException  to alter flow .

  If a matching handler is not found, 'default action' for the receiver
  is performed, as if #defaultAction were sent to the receiver.
  If the receiver is 'resumable' the value returned from the
  #defaultAction method is returned as the value of the #signal message,
  otherwise an error is returned to the GCI client.

  If (System gemConfigurationAt:#GemExceptionSignalCapturesStack) == true ,
  and gsStack==nil when primitive 2022 (AbstractException>>_signalWith:) is invoked,
  then primitive 2022 will fill in gsStack with an Array.
  See AbstractException>>stackReport. "

  ^ self _signalWith: nil
%

category: 'Signaling'
method: AbstractException
signal: signalText

  "The message text of the receiver is set to signalText,
   and the receiver is signaled.  See also  #signal ."

  gsDetails := signalText .  "store ANSI messageText"
  ^ self _signalWith: nil .
%

category: 'Signaling'
method: AbstractException
signalNotTrappable

  "Receiver is signaled and it will not be trappable by an
   exception handler.  An Error will be return to the GCI. "

  gsTrappable := false .
  ^ self _signalWith: nil .
%

category: 'Accessing'
method: AbstractException
stackReport

"Returns a formatted String derived from gsStack instVar, or nil .

 If (System gemConfigurationAt:#GemExceptionSignalCapturesStack) == true ,
 and gsStack==nil when primitive 2022 (AbstractException>>_signalWith:) is invoked,
 then primitive 2022 will fill in gsStack with an Array .
"
| arr nativeStk |
arr := gsStack .
(arr _isArray and:[ (nativeStk := arr atOrNil:1 ) class == Boolean]) ifTrue:[
  | report lf |
  report := String new .  lf := Character lf .
  2 to: arr size by: 3 do:[:j | | meth ip stepPoint rcvr |
    (ip := arr atOrNil: j + 1 ) ifNotNil:[
      meth :=  arr at: j .
      rcvr := arr at: j + 2 .
      ip < 0 ifTrue:[ ip := 0 ].
      report add: (meth _descrForStackPadTo: 0 rcvr: rcvr)  .
      nativeStk ifTrue:[ ip := meth _nativeIpOffsetToPortable: ip asReturn: false].
      stepPoint := meth _stepPointForIp: ip level: 2 useNext: nativeStk .
      report add:' @' ; add: stepPoint asString ;
          add: ' line ';
          add: (meth _lineNumberForStep: stepPoint) asString ;
          add:'  [GsNMethod '; add: meth asOop asString ; add:']';  add: lf .
    ].
  ].
  ^ report
].
^ nil
%

category: 'Legacy Accessing'
method: AbstractException
subtype

   "No longer supported."

   ^ nil
%

category: 'SUnit'
method: AbstractException
sunitExitWith: aValue
        ^self return: aValue
%

category: 'SUnit'
method: AbstractException
sunitSignalWith: aString
        ^self signal: aString
%

category: 'Accessing'
method: AbstractException
tag
  ^ tag ifNil:[ gsDetails ]
%

category: 'Accessing'
method: AbstractException
tag: anObject
  tag := anObject
%

category: 'Private'
method: AbstractException
_debugException: fakeArg

"If a debugger block has been installed with AbstractException(C)>>installDebugBlock:
 execute that block.  Primitive succeeds and returns receiver if no
 debugger block is installed.
 Returns receiver"

<primitive: 799>
"if we get here, a debugger block was found and the argument fakeArg has
 been changed to be that block."

 fakeArg value: self .
 ^ self
%

category: 'Private'
method: AbstractException
_enableEvents

"perform the enable interrupts part of _gsReturnNothingEnableEvents"

<primitive: 141>
self _uncontinuableError
%

category: 'Private'
method: AbstractException
_executeEnsuresBelow: kind
 | ensBlks |
 ensBlks := self _getEnsuresBelow: kind .
 1 to: ensBlks size by: 2 do:[:j|
   self _removeEnsureAtFP: (ensBlks at: j) .
   (ensBlks at: j + 1) value .
 ].
%

category: 'Private'
method: AbstractException
_executeGsHandler: aGsExceptionHandler
  "Execute aGsExceptionHandler's block .  Sent from VM C code only.
   Used for static ANSI handlers and legacy handlers."
  | res err |
  self _debugException: nil .
  currGsHandler := aGsExceptionHandler .
  aGsExceptionHandler _ansiBlock ifNotNil:[ :blk |
    "static ansi handler, normal return from handler disallowed"
    res := blk argumentCount == 0
           ifTrue:[ blk value  ]
           ifFalse:[ blk value: self ].
    (err := UncontinuableError new)
       details:'an ANSI defaultHandler requires explicit resume:'.
    (self isKindOf: UncontinuableError)
       ifTrue:[ err signalNotTrappable  "avoid infinite recursion"]
      ifFalse:[ err signal ].
  ] ifNil:[  "legacy handler , if handler block returns normally
              resume program execution from the point of the signal."
    res := aGsExceptionHandler block value: self value: GemStoneError
		value: gsNumber value: self _legacyHandlerArgs .
  ].
  self isResumable ifFalse:[ | handNum |
    (err := Error new)
       details: 'cannot resume from a not-resumable Exception' .
     handNum := aGsExceptionHandler number .
     (handNum == nil or:[ handNum == err number])
        ifTrue:[ err signalNotTrappable  "avoid infinite recursion" ]
       ifFalse:[ err signal ].
  ].
  currGsHandler := nil .   "clear to allow use of resume:"
  self _resume: res
%

category: 'Private'
method: AbstractException
_executeHandler: aBlock

  "Execute aBlock , which was the second arg of an #on:do: ,
   to handle the receiver.  Sent from VM C code only .
   aBlock must be a block taking zero or one arguments."
  <primitive: 2025> "mark frame with executeHandler_Mark_NIL, always fails"
  | res |
  self _debugException: nil .
  res := aBlock argumentCount == 0
           ifTrue:[ aBlock value  ]
           ifFalse:[ aBlock value: self ].

  "If we get here, then handler block did not send a #resume,
   so continue program execution by returning from the #on:do: send
   which installed the handlerBlock we just executed."

  self return: res .
%

category: 'Private'
method: AbstractException
_executeOuterGsHandler: aGsExceptionHandler
  "Execute aGsExceptionHandler's block .  Sent from image only"
  self _debugException: nil .
  currGsHandler := aGsExceptionHandler .
  aGsExceptionHandler _ansiBlock ifNotNil:[ :blk |
    "static ansi handler, normal return returns to sender of outer "
    ^ blk value: self .
  ] ifNil:[ | res |  "have a legacy handler"
    res := aGsExceptionHandler block value: self value: GemStoneError
                value: gsNumber value: self _legacyHandlerArgs .
    "if legacy handler returns normally, do a resume:"
    self isResumable ifFalse:[ | handNum err |
      (err := Error new)
         details: 'cannot resume from a not-resumable Exception' .
       handNum := aGsExceptionHandler number .
       (handNum == nil or:[ handNum == err number])
          ifTrue:[ err signalNotTrappable  "avoid infinite recursion" ]
         ifFalse:[ err signal ].
    ].
    currGsHandler := nil . "clear to allow use of resume:"
    self _resume: res
  ].
%

category: 'Private'
method: AbstractException
_executeOuterHandler: aBlock
  "Execute aBlock , which was the second arg of an #on:do: ,
   to handle the receiver.   Sent from implementation of #outer only ."

  <primitive: 2025> "mark frame with executeHandler_Mark_NIL, always fails"
  self _debugException: nil .

  ^ aBlock argumentCount == 0
           ifTrue:[ aBlock value  ]
           ifFalse:[ aBlock value: self ].
%

category: 'Private'
method: AbstractException
_getEnsuresBelow: kind

 "kind determines specified frame ,
     kind 0 -->  frame where receiver originally signaled

     kind 1 -->  #on:do: frame for currently active handler,
         and also clears mark_nil from _executeHandler frames
         while searching for the ensure blocks. Stops the
         search at a C FFI or UA  to Smalltalk boundary.

  Returns an Array containing pairs of
    FP offset,  block that was arg to an ensure:
  for all #ensure:  frames  between top of stack and and specified frame

  Does not remove the ensure block from their frames.
  Returns nil if there are none."

 <primitive: 695>
 self _uncontinuableError
%

category: 'Accessing'
method: AbstractException
_gsStack

  " If (System gemConfigurationAt:#GemExceptionSignalCapturesStack) == true ,
   and gsStack==nil when primitive 2022 (AbstractException>>_signalWith:) is invoked,
   then primitive 2022 will fill in gsStack with an Array .
   The Array contains a Boolean (inNativeCode),
   followed by triples of  aGsNMethod , an ipOffset, a receiver
   per the result of GsProcess>>_frameContentsAt: ,
   terminated by the end of the array, or a nil."

  ^ gsStack
%

category: 'Private'
method: AbstractException
_gsStack: anArray
  gsStack := anArray
%

category: 'Private'
method: AbstractException
_handleInCextension

"Private.
 Caller responsible for executing ensure blocks.
 Unwinds Smalltalk stack to the most recent C extension and
 returns to the C code with receiver as the signaled exception."

<primitive: 556>  "primitive fails unless this is Maglev VM"
"should never reach here"
self _primitiveFailed: #_handleInCextension.
self _uncontinuableError .
%

category: 'Private'
method: AbstractException
_handlerActive
  "an exception currently being handled by an on:do: will have
     FP information appended."

  ^ self size ~~ 0
%

category: 'Accessing'
method: AbstractException
_legacyDetails: templateString
  | str theArgs elem |
  theArgs := self _legacyHandlerArgs .
  str := String new .
  1 to: templateString size do:[:j |
    elem := templateString at: j .
    elem _isOneByteString ifTrue:[
      str add: elem
    ] ifFalse:[
      elem _isSmallInteger ifTrue:[
	elem < 0 ifTrue:[ str add: (theArgs atOrNil: 0 - elem) asOop asString ]
	      ifFalse:[ str add: (theArgs atOrNil: elem) describe1K ]
      ]
    ]
  ].
  ^ str
%

category: 'Private'
method: AbstractException
_legacyHandlerArgs
  "Must be reimplemented in subclasses of AbstractException
   which have named instVar beyond 'args' "

  ^ gsArgs ifNil:[ #() ]
%

category: 'Private Legacy'
method: AbstractException
_legacyResignal: newException

"Search for a handler for newException below this handler.
 Current handler should be a legacy handler .

 If the newException is handled successfully,
 a normal return of the handlerBlock found by this resignal will
 result in a return from this method."

<primitive: 2024>  "succeeds or throws error to GCI"
self _uncontinuableError.
%

category: 'Instance initialization'
method: AbstractException
_number: aSmallInt
  gsNumber := aSmallInt
%

category: 'Private'
method: AbstractException
_outer: aHandler with: handlerKind
  "Search for an enclosing handler below the active one that will
   handle the receiver.
   Caller must always pass  nil for aHandler and isAnsi.
   resume:  from within the outer handler will return from this frame."

 <primitive: 697>
 "primitive always fails.
  if handler found, it was substituted in frame for aHandler ,
  and handlerKind was set to a IntHandlerBlockType "

  aHandler ifNotNil:[   "handler found"
    handlerKind == 0 ifTrue: [ "2nd arg to on:do:"
      self return:( self _executeOuterHandler: aHandler ) 
    ] ifFalse:[ 
      "Static ANSI or legacy handler"
      self return:( self _executeOuterGsHandler: aHandler ) 
    ]
  ] ifNil:[ "no enclosing handler found"
    ^ self defaultAction .
  ].
  self _uncontinuableError. "should not reach here"
%

category: 'Private'
method: AbstractException
_pass: aHandler with: handlerKind
  "Search for an enclosing handler below the active one that will
   handle the receiver.
   Caller must always pass  nil for aHandler and isNonStaticAnsi .
   resume:  from within the outer handler will return from this frame."
 <primitive: 697>
 "primitive always fails.
  If handler found, it was substituted in frame for aHandler ,
  and handlerKind was set to a IntHandlerBlockType "

  aHandler ifNotNil:[   "handler found"
    handlerKind == 0 ifTrue: [ "2nd arg to on:do:"
      self return:( self _executeOuterHandler: aHandler ) 
    ] ifFalse:[
     "Static ANSI or legacy handler"
      self return:( self _executeOuterGsHandler: aHandler )
    ]
  ] ifNil:[ "no enclosing handler found"
    self _executeEnsuresBelow: 0 . "ensures down to original signal"
    self _passToDefaultHandler .
    self _uncontinuableError.  "should not be here"
  ].
%

category: 'Private'
method: AbstractException
_passToDefaultHandler

  "Caller has executed ensure: blocks between top of stack and
   point where receiver was originally signalled.
   This primitive will trim stack back to that signal frame and
   restart execution of AbstractException>>signal such that
   the search for a handler will always fail, thus executing
   the defaultAction."

<primitive: 2009>
self _uncontinuableError . "should never reach here"
%

category: 'Private'
method: AbstractException
_removeEnsureAtFP: anOffset

 "Dereference the ensure block in specified frame,
  so it won't be executed more than once.
  Frame changed  from  ensure:[] to  ensure: nil ."

 <primitive: 696>
 self _uncontinuableError
%

category: 'Private'
method: AbstractException
_resignalAs: replacementException

  "Caller has executed ensure: blocks between top of stack and
   point where receiver was originally signalled.
   This primitive will trim stack back to that signal frame.

   Then replacementException is substituted for the receiver
   and execution of the signal method is restarted."

 <primitive: 2023>
 replacementException _validateClass: AbstractException .
  self _uncontinuableError
%

category: 'Instance initialization'
method: AbstractException
_resumable: aBoolean
  gsResumable := aBoolean
%

category: 'Handling'
method: AbstractException
_resume: resumptionValue

"To be sent from within an ANSI handler's handlerBlock only.

 If the current handler block was invoked by #outer from a previous
 handler block,  #resume will return to that previous handler block,
 with resumptionValue being returned from the send of #outer .
 Otherwise resume execution from the send that signaled the receiver
 with the specified value.

 calling methods in image responsible for checking isResumable."

 <primitive: 2021>
 self _uncontinuableError .  "should not be here"
%

category: 'Private'
method: AbstractException
_retryUsing: alternativeBlock

 "When sent within an ANSI handler's handlerBlock ,
  resume execution in the frame of the #on:do: send which
  installed the currently active handlerBlock.

  caller has executed ensure: blocks between top of stack and that #on:do: .
  this primitive will trim stack back to that #on:do: .

  If alternativeBlock  is non-nil , it must be a zero arg
  ExecBlock and it is substituted for the original receiver of the #on:do: .
  Then #value is sent to the non-nil alternativeBlock, else to the
  original receiver of #on:do: .

  Generates an error if sent within a legacy handler's handlerBlock."

  <primitive: 2019>  "does not return"
  self _uncontinuableError
%

category: 'Private'
method: AbstractException
_return: returnValue
 "When sent within an ANSI handler's handlerBlock,
  resume execution by returning returnValue from the #on:do: send
  which installed the currently active handlerBlock .

  Caller has executed ensure: blocks between top of stack and that #on:do: .
  This primitive will trim stack back to that #on:do: .
  Generates an error if sent within a legacy handler's handlerBlock.

  Will return into a C extension exception handling C code, or
  to from the currently active handlerBlock, which ever is closer
  to the top of the stack.
"
<primitive: 2020> "does not return"
self _uncontinuableError
%

category: 'Private'
method: AbstractException
_signalAsync

  "To be sent only by the virtual machine.
   The receiver is an asynchronous Exception, usually a ControlInterrupt.
   Examples are  such as soft-break, (ErrorSymbols at: #rtErrSoftBreak, error 6003)
   or signalAbort (ErrorSymbols at: #rtErrSignalAbort , error 6009) .

   Handling of an asynchronous Exception happens between bytecodes and
   if successful does not alter the stack.  This method is always
   executed for an asynchronous Exception, regardless of whether a
   handler exists. "

  | ok |
  gsTrappable == true ifTrue:[
    gsTrappable := 1  "prevent onSynchronous:do: from handling this".
  ].
  ok := false .
  [
    self _signalWith: nil . "if handler not found  synchronous error thrown to GCI"
    ok := true .  "if handled ok we get to here"
  ] ensure:[
    ok ifFalse:[
      self _enableEvents . "reenable interrupts if resume not successful"
    ]
  ].
  self _gsReturnNothingEnableEvents "special selector optimized by compiler"
%

category: 'Private'
method: AbstractException
_signalFromPrimitive: inCextensionArg

  "Send from within the VM only.
   An exception is being signalled from within a C primitive.
   The signalled exception is not resumable .
   otherwise same behavior as AbstractException>>_signalWith: . "

  <primitive: 2022>
  | res num |
  inCextensionArg ifNotNil:[
    "primitive found a C extension which wants to handle receiver."
    self _executeEnsuresBelow: 1 .  "execute ensures from TOS to C extension"
    self _handleInCextension.  "trims stack and returns to C extension"
  ].
  res := self defaultAction .
    "if you change code from beginning of method to here , may need to
     alter C code at  IntSwiReplaceTos_IPOFFSET"

  ((num := gsNumber) > 6000 and:[   "allow breakpoints to be resumable from Smalltalk."
     num == 6002 or:[ num == 6005 or:[ num == 6006 ]]]) ifTrue:[
       self isResumable ifTrue:[
         ^ res
     ]
  ].
  1 timesRepeat:[ self class ]. "loop to detect/handle termination interrupt"
  self _signalGciError .
  2 timesRepeat:[ self class ]. "loop to detect/handle termination interrupt"
  self _uncontinuableError . "should never reach here"
%

category: 'Private'
method: AbstractException
_signalGcFinalize

  "To be sent only by the virtual machine.
   The receiver is an asynchronous Exception for error 6017 "
[ | chunk chunkSize anEphemeron  |
  chunk := System __sessionStateAt: 2 . "head of finalization queue"
  [ chunk ~~ nil ] whileTrue:[
    chunkSize := chunk size .
    1 to: chunkSize - 1 do:[:n |
      anEphemeron := chunk at: n .
      [ anEphemeron mourn
      ] onException: Error do:[:ex |
        GsFile gciLogServer: ex asString , 'during mourn'
      ].
    ].
    chunk := chunk at: chunkSize . "last element is next chunk"
    System __sessionStateAt: 2 put: chunk. "deref chunk just processed"
  ]
] ensure:[
  self _enableEvents . "reenable interrupts if resume not successful"
].
self _gsReturnNothingEnableEvents "special selector optimized by compiler"
%

category: 'Handling'
method: AbstractException
_signalGciError

"Return an error to the controlling GCI client. Stack is saved
 and available as an argument to the GCI error struct."

<primitive: 33>  "this frame removed from stack before saving stack."
self _uncontinuableError
%

category: 'Private'
method: AbstractException
_signalTerminateProcess
  "sent from within the VM "

  GsProcess _current _serviceTerminationInterrupt .
  self _gsReturnNothingEnableEvents  "special selector optimized by compiler"
%

category: 'Private'
method: AbstractException
_signalTimeout

  "To be sent only by the virtual machine.
   The receiver is an asynchronous Exception, error 6015 "

  | ok |
  ok := false .
  [
    gsNumber == ERR_TIMEOUT_INTERRUPT ifTrue:[
      ProcessorScheduler scheduler _yieldForTimeout .
      ok := true .  "if handled ok we get to here"
    ] ifFalse:[
      self _signalWith: nil .
    ].
  ] ensure:[
    ok ifFalse:[
      self _enableEvents . "reenable interrupts if resume not successful"
    ]
  ].
  self _gsReturnNothingEnableEvents "special selector optimized by compiler"
%

category: 'Private'
method: AbstractException
_signalWith: inCextensionArg
  <primitive: 2022>

   "The current stack is searched for an 'exception handler' per
    AbstractException>>signal .
    If a handler found, new frame pushed
      to execute  _executeHandler:   or   _executeGsHandler:  .
      and the primitive does not return.
    If exception handling succeeds and execution is to resume,
    either the resume: or the return: primitive will do the 'goto'
    and we don't actually return from this frame .

    inCextensionArg must be nil at entry, and may be modified
    by primitive.

    If a Smalltalk handler not found,
    primitive fails so we can defer to a C extension or send defaultAction here."

  | res |
  inCextensionArg ifNotNil:[
    "primitive found a C extension which wants to handle receiver."
    self _executeEnsuresBelow: 1 .  "execute ensures from TOS to C extension"
    self _handleInCextension.  "trims stack and returns to C extension"
  ].
  res := self defaultAction .
    "if you change code from beginning of method to here , may need to
     alter C code at  IntSwiReplaceTos_IPOFFSET"

  1 timesRepeat:[ self class ]. "loop to detect/handle termination interrupt"
  self isResumable ifTrue:[
    ^ res .
  ].
  self _signalGciError .
  self _uncontinuableError . "should never reach here"
%

! Class extensions for 'AbstractFraction'

!		Class methods for 'AbstractFraction'

removeallmethods AbstractFraction
removeallclassmethods AbstractFraction

category: 'Instance Creation'
classmethod: AbstractFraction
fromStream: aStream

"Returns a Fraction from the stream.  The stream must contain two Integers
 separated by a slash.  (There may be blanks around the slash.)  Generates an
 error if the stream contains anything else, or if an attempt is made to read
 beyond the end of the stream."

| ch n d |
self _checkReadStream: aStream forClass: CharacterCollection.
ch := aStream next.
[ ch == $  ] whileTrue:[ ch := aStream next ].
aStream skip: -1.
n := Integer fromStream: aStream.
ch := aStream next.
[ ch == $  ] whileTrue:[ ch := aStream next ].
aStream skip: -1.
(aStream next == $/ ) ifFalse:[ ^ self _errIncorrectFormat: aStream ].
ch := aStream next.
[ ch == $  ] whileTrue:[ ch := aStream next ].
aStream skip: -1 .
d := Integer fromStream: aStream.
^ self numerator: n denominator: d
%

category: 'Instance Creation'
classmethod: AbstractFraction
numerator: n denominator: d
"Returns an instance of Fraction with numerator numInt and denominator
 denomInt.  If that Fraction can be reduced, this method returns the
 corresponding Integer instead.  The result is made invariant.

 If either argument (numerator or denominator) is not an Integer, that
 argument is truncated to the corresponding Integer."

 (self _newSmallFraction: n denom: d reduce: true) ifNotNil:[:res |
    ^ res
 ].
 ^ self _reduce: n denom: d
%

category: 'Private'
classmethod: AbstractFraction
_newFraction: n denominator: d

  "Returns an instance of Fraction , never reduced to a SmallFraction."

  ^ Fraction _basicNew _numerator: n denom: d
%

category: 'Private'
classmethod: AbstractFraction
_newSmallFraction: n denom: d reduce: reduceBoolean

"Returns nil or a reduced instance of SmallFraction ,
 or a Fraction containing reduced SmallInteger values for n and d.
 A SmallFraction is returned if, after moving overall sign to n
 and reducing n and d, n and d satisfy
                0 <  d <= 134217726
   and -536870912 <= n <= 536870911

 Signals an Error if d == 0 ."

<primitive: 996>
reduceBoolean _validateClass: Boolean .
^ self _primitiveFailed: #_newSmallFraction:denom: args: { n . d }
%

category: 'Private'
classmethod: AbstractFraction
_noreduce_numerator: n denominator: d
  (self _newSmallFraction: n denom: d reduce: false) ifNotNil:[:res |
     ^ res
  ].
  ^ Fraction _basicNew _numerator: n denom: d
%

category: 'Private'
classmethod: AbstractFraction
_reduce: numer denom: denom
  | n d gcd |
  n := numer truncated .
  n == 0 ifTrue:[ ^ 0 ].
  d := denom truncated .
  (d == 0) ifTrue: [ ^ n _errorDivideByZero ].
  d < 0 ifTrue:[
    d := 0 - d .
    n := 0 - n .
  ].
  gcd := n gcd: d .
  n := n // gcd.
  d := d // gcd.
  (d == 1) ifTrue:[ ^ n ].
  (self _newSmallFraction: n denom: d reduce: false) ifNotNil:[:res |
     ^ res
  ].
  ^ Fraction _basicNew _numerator: n denom: d
%

!		Instance methods for 'AbstractFraction'

category: 'Arithmetic'
method: AbstractFraction
* aNumber

"Returns the result of multiplying the receiver by aNumber."

(aNumber isKindOf: AbstractFraction) ifTrue: [
  ^ AbstractFraction numerator: (self numerator * aNumber numerator)
           denominator: (self denominator * aNumber denominator)
] ifFalse: [
  (aNumber _isInteger and:[ aNumber = self denominator ]) ifTrue:[
     ^ self numerator
  ].
  ^ self _retry: #* coercing: aNumber
]
%

category: 'Arithmetic'
method: AbstractFraction
+ aFraction

"Returns the sum of the receiver and aFraction."

(aFraction isKindOf: AbstractFraction) ifTrue:[
  | myDenom otherDenom commonDenominator newNumerator
    gcd myDemon_div_gcd |
  (myDenom := self denominator) = (otherDenom := aFraction denominator) ifTrue: [
    | myNumer otherNumer |
    (myNumer := self numerator) = (otherNumer := aFraction numerator) ifTrue:[
      myDenom _bottomBit == 0 ifTrue:[
        ^ AbstractFraction numerator: myNumer denominator: (myDenom bitShift: -1)
      ]
    ].
    ^ AbstractFraction numerator: (myNumer + otherNumer ) denominator: myDenom
  ].
  gcd := myDenom gcd: otherDenom .
  myDemon_div_gcd := myDenom // gcd .
  commonDenominator := myDemon_div_gcd * otherDenom .
  newNumerator := (self numerator * (otherDenom // gcd ))
                 + (aFraction numerator *  myDemon_div_gcd) .
  ^ AbstractFraction numerator: newNumerator denominator: commonDenominator
] ifFalse: [
  ^self _retry: #+ coercing: aFraction
]
%

category: 'Arithmetic'
method: AbstractFraction
- aFraction

"Returns the difference between the receiver and aFraction."
(aFraction isKindOf: AbstractFraction) ifTrue: [
  ^ self + (Fraction _noreduce_numerator:  0 - aFraction numerator
		 denominator: aFraction denominator)
] ifFalse: [
  ^ self _retry: #- coercing: aFraction
]
%

category: 'Arithmetic'
method: AbstractFraction
/ aNumber

"Returns the result of dividing the receiver by aFraction."

aNumber _isInteger ifTrue:[
  aNumber = 0 ifTrue: [ ^ self _errorDivideByZero ].
  ^ AbstractFraction numerator: self numerator denominator: (self denominator * aNumber)
].
(aNumber isKindOf: AbstractFraction) ifTrue:[
  ^ self * aNumber _reciprocal
].
^ self _retry: #/ coercing: aNumber
%

category: 'Comparing'
method: AbstractFraction
< aNumber

"Returns true if the receiver is less than aNumber; returns false otherwise."

aNumber _isNumber ifTrue:[
  (aNumber isKindOf: AbstractFraction) ifTrue: [ | otherNum num |
    (otherNum := aNumber numerator) ifNil:[ ^ false ].
    (num := self numerator) ifNil:[ ^ false ].
    otherNum == 0 ifTrue: [
      ^ num < 0
    ].
    ^ (num * aNumber denominator) < (self denominator * otherNum)
  ].
].
^self _retry: #< coercing: aNumber
%

category: 'Comparing'
method: AbstractFraction
<= aNumber

"Returns true if the receiver is less than or equal to aNumber; returns false
 otherwise."

aNumber _isNumber ifTrue:[
  (aNumber isKindOf: AbstractFraction) ifTrue: [ | otherNum num |
    (otherNum := aNumber numerator) ifNil:[ ^ false ].
    (num := self numerator) ifNil:[ ^ false ].
    otherNum == 0 ifTrue: [
      ^ num <= 0
    ].
    ^ (num * aNumber denominator) <=  (self denominator * otherNum)
  ].
].
^self _retry: #<= coercing: aNumber
%

category: 'Comparing'
method: AbstractFraction
= aNumber

"Returns true if the receiver is equal to aNumber; returns false otherwise."
aNumber _isNumber ifFalse:[ ^ false ].
(aNumber isKindOf: AbstractFraction ) ifTrue: [ | otherNum num |
  (otherNum := aNumber numerator) ifNil:[ ^ false ].
  (num := self numerator) ifNil:[ ^ false ].
  otherNum == 0 ifTrue: [
    ^ num == 0
  ].
  ^ otherNum = num and:[ aNumber denominator = self denominator ]
].
^ self _retry: #= coercing: aNumber
%

category: 'Comparing'
method: AbstractFraction
> aNumber

aNumber _isNumber ifTrue:[
  (aNumber isKindOf: AbstractFraction) ifTrue: [ | otherNum num |
    (otherNum := aNumber numerator) ifNil:[ ^ false ].
    (num := self numerator) ifNil:[ ^ false ].
    otherNum == 0 ifTrue: [
      ^ num > 0
    ].
    ^ (num * aNumber denominator) > (self denominator * otherNum)
  ].
].
^self _retry: #> coercing: aNumber
%

category: 'Comparing'
method: AbstractFraction
>= aNumber

aNumber _isNumber ifTrue:[
  (aNumber isKindOf: AbstractFraction) ifTrue: [ | otherNum num |
    (otherNum := aNumber numerator) ifNil:[ ^ false ].
    (num := self numerator) ifNil:[ ^ false ].
    otherNum == 0 ifTrue: [
      ^ num >= 0
    ].
    ^ (num * aNumber denominator) >= (self denominator * otherNum)
  ].
].
^self _retry: #>= coercing: aNumber
%

category: 'Converting'
method: AbstractFraction
asDecimalFloat

"Returns an instance of DecimalFloat that has the value of the receiver."
| num denom |
(num := self numerator) ifNil:[ ^ PlusSignalingNaN ] .
(denom := self denominator) ifNil:[ ^ PlusSignalingNaN ] .
^ num asDecimalFloat / denom asDecimalFloat
%

category: 'Converting'
method: AbstractFraction
asFixedPoint: scale

^ FixedPoint numerator: self numerator denominator: self denominator scale: scale
%

category: 'Converting'
method: AbstractFraction
asFloat

	"Answer a Float that closely approximates the value of the receiver.
	This implementation will answer the closest floating point number to the receiver.
	In case of a tie, it will use the IEEE 754 round to nearest even mode.
	In case of overflow, it will answer +/- Float infinity.
	If numerator or denominator is nil, anwers NaN."

	| num den n d numeratorSize denominatorSize scalePower scaledValue scaledValueBits significantBits ulp halfULP significanceMask subSignificant scaledFloat |
	(num := self numerator) ifNil: [ ^ PlusSignalingNaN ].
	(den := self denominator) ifNil: [ ^ PlusSignalingNaN ].

	n := num abs.
	d := den.	"denominator is never negative"
	numeratorSize := n highBit.
	denominatorSize := d highBit.	

	"If both numerator and denominator can be represented exactly as floats,
	we can use float division and let it do the correct rounding."
	(numeratorSize <= 53 and: [ denominatorSize <=  53 ])
		ifTrue: [ ^ num asFloat / den asFloat ].	

	"Scale the fraction by a power of two scalePower so as to obtain an integer 
	value with 54 or 55 bits, depending on the values."
	scalePower := numeratorSize - denominatorSize - 54.
	scalePower >= 0
		ifTrue: [ d := d bitShift: scalePower ]
		ifFalse: [ n := n bitShift: scalePower negated ].
	scaledValue := n quo: d.
	scaledValueBits := scaledValue highBit.

	"How many high-order bits of scaledValue will end up in the significand?
	For tiny fractions that underflow to zero, this will be negative."
	significantBits := (scalePower + 1074 + scaledValueBits) min: 53.

	"The 54 or 55-bit integer scaledValue will be rounded to become the 
	significand of the float. Normal floats have a 53-bit significand.
	Subnormal floats have a significand with 0 to 52 significant bits.
	The 0-bit case includes very small fractions that round
	to 0.0 or to Float fmin. A negative number of significantBits will round to 0.0.

	Rounding:
	One or more low-order bits of the scaledValue will not be in the significand of the 
	float, and so are sub-significant. For normal floats, with 53-bit significands,
	there will be one or two sub-significant bits. Subnormal non-zero floats will
	have up to 55 sub-significant bits, and tiny fractions that round to 0.0 may 
	have more than that.
	The lowest-order bit of the significand is a ULP, a unit of least precision.
	If the sub-significant bits more than half a ULP, we round up, if less we round
	down. If equal, we consult the remainder to see whether there are any low-
	order one bits that were rounded away in computation of the scaledValue.
	If there are, we round up, but if there are none, we round either up or down,
	whichever makes the significand even (ULP bit zero).

	We increment the significant bits if rounding up, then clear any sub-significant
	bits to make the conversion to an integral float precise.
	We then scale the integral float to the correct magnitude by
	multiplying it by a power of two. For normal floats, the scale correction
	affects only the exponent of the float; the significand remains the same.
	If the exponent overflows the float becomes an infinity.
	If the exponent underflows the float becomes subnormal."

	ulp := 1 bitShift: scaledValueBits - significantBits.
	halfULP := ulp bitShift: -1.
	significanceMask := ulp - 1. 
	subSignificant := scaledValue bitAnd: significanceMask. "The low-order bits that will not end up in the significand."

	"Round up if needed"
	subSignificant >= halfULP 
		ifTrue: [(subSignificant > halfULP or: [n > (scaledValue * d)])
					ifTrue:  [ scaledValue := scaledValue + ulp ]
					ifFalse: ["Round half to even"
								scaledValue := scaledValue + (scaledValue bitAnd: ulp)]].

	"Clear any subSignificant bits so we only have bits that will go in the significand.
	Otherwise converting scaledValue to a float would round, in some cases incorrectly."
	scaledValue := scaledValue - subSignificant.

	scaledFloat := (self positive
		ifTrue: [ scaledValue asFloat ]
		ifFalse: [ scaledValue asFloat negated ]).
	
	"Scale correction"
	"For very negative exponents, must scale in two steps,
	since (2.0 raisedToInteger: -1075) rounds to 0.0, which would cause premature underflow to zero"
	^ scalePower < -1074 
			ifTrue: [scaledFloat * (2.0 raisedToInteger: scalePower + 1074) * (2.0 raisedToInteger: -1074)]
			ifFalse: [scaledFloat * (2.0 raisedToInteger: scalePower)]
%

category: 'Converting'
method: AbstractFraction
asFraction

"Returns the receiver."

^self
%

category: 'Converting'
method: AbstractFraction
asScaledDecimal: scale

"Returns a ScaledDecimal representation of the receiver."

^ ScaledDecimal numerator: self numerator denominator: self denominator scale: scale.
%

category: 'Formatting'
method: AbstractFraction
asString

"Returns a String of the form 'numerator/denominator'."

| result |
result := self numerator asString .
result add: $/ ; addAll: self denominator asString .
^ result
%

category: 'Testing'
method: AbstractFraction
even

"Returns true if the receiver is an even integer, false otherwise."

 self denominator = 1 ifFalse: [ ^ false ].
 ^ self numerator even

%

category: 'Testing'
method: AbstractFraction
isZero

"Returns true if the receiver is zero."

^ self numerator = 0 .
%

category: 'Arithmetic'
method: AbstractFraction
negated

"Returns a Number that is the negation of the receiver."

^ AbstractFraction _noreduce_numerator:  0 - self numerator
			denominator: self denominator
%

category: 'Testing'
method: AbstractFraction
odd

"Returns true if the receiver is an odd integer, false otherwise."

 self denominator = 1 ifFalse: [ ^ false ].
 ^ self numerator odd
%

category: 'Arithmetic'
method: AbstractFraction
reciprocal

"Returns 1 divided by the receiver.  Generates an error if the receiver is 0.
 Result is reduced."
| numer denom |
((numer := self numerator) == 0 or:[ numer == nil]) ifTrue: [ ^ self _errorDivideByZero].
denom := self denominator .
(numer == 1) ifTrue:[ ^ denom ].
numer < 0 ifTrue:[
  (numer == -1) ifTrue:[ ^ 0 - denom ].
  denom := 0 - denom . numer := 0 - numer .
].
^  AbstractFraction numerator: denom denominator: numer
%

category: 'Truncation and Rounding'
method: AbstractFraction
roundAndCoerceTo: aNumber

"Returns the multiple of aNumber that is nearest in value to the receiver.
 If aNumber is a kind of Integer, ScaledDecimal, or AbstractFraction,
 the result will be an instance of the class of aNumber . "
| r |
aNumber = 0 ifTrue:[ ^ 0 ] .
r := (self / aNumber) rounded * aNumber .
aNumber _isFloat ifTrue:[ ^ r asFloat ].
aNumber _isScaledDecimal ifTrue:[ ^ r ].
^ r asFraction
%

category: 'Truncation and Rounding'
method: AbstractFraction
roundedHalfToEven

"Returns the integer nearest in value to the receiver. If the receiver is
exactly halfway between two integers, return the even one."

| remainder |
remainder := (self numerator abs \\ self denominator).
^ remainder * 2 = self denominator
	ifFalse: [self rounded]
	ifTrue: [| truncated | truncated := self truncated.
			truncated even
				ifTrue: [truncated]
				ifFalse: [truncated + self sign]]
%

category: 'Truncation and Rounding'
method: AbstractFraction
roundTo: aNumber

"Returns the multiple of aNumber that is nearest in value to the receiver."
| r |
aNumber = 0 ifTrue:[ ^ 0 ] .
r := (self / aNumber) rounded * aNumber .
aNumber _isFloat ifTrue:[ ^ r asFloat ].
^ r asFraction
%

category: 'Accessing'
method: AbstractFraction
sign
"Returns 1 if the receiver is greater than zero, -1 if the receiver is less
 than zero, and zero if the receiver is zero."

  ^ self numerator sign
%

category: 'Truncation and Rounding'
method: AbstractFraction
truncateAndCoerceTo: aNumber

"Returns the multiple of aNumber that is closest to the receiver, on
 the same side of the receiver as zero is located.  In particular,
 returns the receiver if the receiver is a multiple of aNumber.

 If aNumber is a kind of Integer, ScaledDecimal, or AbstractFraction,
 the result will be an instance of the class of aNumber ."

| t |
aNumber = 0 ifTrue:[ ^ 0 ].
t := (self quo: aNumber) * aNumber .
aNumber _isFloat ifTrue:[ ^ t asFloat ].
aNumber _isScaledDecimal ifTrue:[ ^ t ].
^ t asFraction
%

category: 'Truncation and Rounding'
method: AbstractFraction
truncated

"Returns the integer that is closest to the receiver, on the same side
 of the receiver as zero is located."

^ self numerator quo: self denominator
%

category: 'Truncation and Rounding'
method: AbstractFraction
truncateTo: aNumber

"Returns the multiple of aNumber that is closest to the receiver, on
 the same side of the receiver as zero is located.  In particular,
 returns the receiver if the receiver is a multiple of aNumber."

| t |
aNumber = 0 ifTrue:[ ^ 0 ].
t := (self quo: aNumber) * aNumber .
aNumber _isFloat ifTrue:[ ^ t asFloat ].
^ t asFraction
%

category: 'Converting'
method: AbstractFraction
_coerce: aNumber

"Reimplemented from Number."
| f |
f := aNumber asFraction .
f _isInteger ifTrue:[
  ^ AbstractFraction _noreduce_numerator: f denominator: 1
].
^ f
%

category: 'Converting'
method: AbstractFraction
_generality

"Reimplemented from Number."

^70
%

category: 'Accessing'
method: AbstractFraction
_getKind

| num denom |
(num := self numerator) ifNil:[ ^ 5 "nan"].
(denom := self denominator) ifNil:[ ^ 5 ].
denom == 0 ifTrue:[ ^ 3 "infinity" ].
num == 0 ifTrue:[ ^ 4 "zero"].
^ 1 "normal"
%

category: 'Indexing Support'
method: AbstractFraction
_isNaN

"Returns whether the receiver is quiet NaN or signaling NaN.
 This method is only to be used by the indexing subsystem."

^ (self numerator == nil) or: [self denominator == nil]
%

category: 'Private'
method: AbstractFraction
_reciprocal

"Returns 1 divided by the receiver.  Generates an error if the receiver is 0.
 Result is not reduced."
| numer denom |
((numer := self numerator) == 0 or:[ numer == nil]) ifTrue: [ ^ self _errorDivideByZero].
denom := self denominator .
numer < 0 ifTrue:[ denom := 0 - denom . numer := 0 - numer  ].

^  AbstractFraction _noreduce_numerator: denom denominator: numer
%

! Class extensions for 'AbstractSession'

!		Instance methods for 'AbstractSession'

removeallmethods AbstractSession
removeallclassmethods AbstractSession

category: 'Commit Processing'
method: AbstractSession
_abort

"(Subclass responsibility.)  Makes the remote session abort any changes to
 permanent state."

AbstractSession subclassResponsibility: #_abort
%

category: 'Commit Processing'
method: AbstractSession
_beginTransaction

"Do nothing for now."

%

category: 'Commit Processing'
method: AbstractSession
_commit: commitMode

"(Subclass responsibility.)  Makes the remote session commit any changes to
 permanent state."

AbstractSession subclassResponsibility: #_commit:
%

category: 'Commit Processing'
method: AbstractSession
_isNonBlocking

"Return true if the synchronized commit algorithm may use non-blocking
 messages and false otherwise."

^ false
%

category: 'Commit Processing'
method: AbstractSession
_nbAbort

"(Subclass responsibility.)  Non-blocking abort."

self _isNonBlocking
  ifTrue: [ AbstractSession subclassResponsibility: #_nbAbort ]
%

category: 'Commit Processing'
method: AbstractSession
_nbAbortResult

"(Subclass responsibility.)  Gets the result of a non-blocking abort."

self _isNonBlocking
  ifTrue: [ AbstractSession subclassResponsibility: #_nbAbortResult ]
%

category: 'Commit Processing'
method: AbstractSession
_nbCancel

"(Subclass responsibility.)  Cancel the execution of a non-blocking call."

self _isNonBlocking
  ifTrue: [ AbstractSession subclassResponsibility: #_nbCancel ]
%

category: 'Commit Processing'
method: AbstractSession
_nbCommit: commitMode

"(Subclass responsibility.)  Non-blocking commit:"

self _isNonBlocking
  ifTrue: [ AbstractSession subclassResponsibility: #_nbCommit: ]
%

category: 'Commit Processing'
method: AbstractSession
_nbCommitResult

"(Subclass responsibility.)  Gets the result of a non-blocking commit."

self _isNonBlocking
  ifTrue: [ AbstractSession subclassResponsibility: #_nbCommitResult ]
%

category: 'Commit Processing'
method: AbstractSession
_nbEnd

"(Subclass responsibility.)  Determine the status of a non-blocking call."

self _isNonBlocking
  ifTrue: [ AbstractSession subclassResponsibility: #_nbEnd ]
%

category: 'Commit Processing'
method: AbstractSession
_nbTimeout

"(Subclass responsibility.)  Returns a SmallInteger, indicating the minimum
 number of seconds that a non-blocking call will be allowed to execute before
 the coordinator considers the non-blocking call to have failed."

AbstractSession subclassResponsibility: #_nbTimeout
%

category: 'Commit Processing'
method: AbstractSession
_nbVoteResult

"(Subclass responsibility.)  Gets the result of a non-blocking vote."

self _isNonBlocking
  ifTrue: [ AbstractSession subclassResponsibility: #_nbVoteResult ]
%

category: 'Commit Processing'
method: AbstractSession
_nbVoteToCommit

"(Subclass responsibility.)  Non-blocking voteToCommit."

self _isNonBlocking
  ifTrue: [ AbstractSession subclassResponsibility: #_nbVoteToCommit ]
%

category: 'Accessing'
method: AbstractSession
_publicName

"(Subclass responsibility.)  Returns a name to use for error messages."

AbstractSession subclassResponsibility: #_publicName
%

category: 'Commit Processing'
method: AbstractSession
_transactionMode: newMode

"Do nothing for now."

%

category: 'Commit Processing'
method: AbstractSession
_voteToCommit

"(Subclass responsibility.)  Returns a SmallInteger indicating the status
 of the vote.  The returned value has the following meaning:

 0: The session was read-only and the vote is affirmative.
 1: The session wrote objects and the vote is affirmative.
 2: The vote is negative; the session cannot commit."

AbstractSession subclassResponsibility: #_voteToCommit
%

! Class extensions for 'AbstractUserProfileSet'

!		Instance methods for 'AbstractUserProfileSet'

removeallmethods AbstractUserProfileSet
removeallclassmethods AbstractUserProfileSet

category: 'Adding'
method: AbstractUserProfileSet
add: aUserProfile

"(Subclass responsibility.)  Adds aUserProfile to the receiver."

AbstractUserProfileSet subclassResponsibility: #add:
%

category: 'Adding'
method: AbstractUserProfileSet
addAll: aCollection

"Reimplemented to maintain KeyValueDictionary on AllUsers."

AbstractUserProfileSet subclassResponsibility: #addAll:
%

category: 'Removing'
method: AbstractUserProfileSet
removeAll: aCollection

"Reimplemented to maintain KeyValueDictionary on AllUsers."

| aColl tmpArr|
aColl := aCollection.
aColl == self ifTrue:[
  aColl == AllUsers ifTrue:[
    self _halt: 'Attempt to remove all elements of AllUsers'  .
    self _uncontinuableError
    ].
  tmpArr := Array withAll: aColl .
  aColl := tmpArr .
  ].
aColl accompaniedBy: self do:[ :me :element | me remove: element ].
^ aCollection
%

category: 'Removing'
method: AbstractUserProfileSet
removeAllPresent: aCollection

"Reimplemented to maintain KeyValueDictionary on AllUsers."

aCollection == self ifTrue:[ ^ self removeAll: aCollection ].

aCollection accompaniedBy: self do:[ :me :element | me removeIfPresent: element ].
^ aCollection
%

category: 'Removing'
method: AbstractUserProfileSet
removeIfPresent: anObject

"Reimplemented to maintain KeyValueDictionary on AllUsers."

^ self remove: anObject ifAbsent: [ nil ]
%

category: 'Accessing'
method: AbstractUserProfileSet
userWithId: aString

"Searches the receiver for a UserProfile whose userId is equal to aString, and
 returns that UserProfile.  Generates an error if no userId is equal to
 aString."

^ self userWithId: aString
         ifAbsent:[ self _error: #objErrNotInColl args: { aString }]
%

category: 'Adding'
method: AbstractUserProfileSet
_add: aUserProfile

AbstractUserProfileSet subclassResponsibility: #_add:
%

category: 'Removing'
method: AbstractUserProfileSet
_removeAll: aCollection errIfAbsent: aBool

"Reimplemented to maintain KeyValueDictionary on AllUsers."

aBool ifTrue:[ self removeAll: aCollection]
     ifFalse:[ self removeAllPresent: aCollection ]
%

! Class extensions for 'Admonition'

!		Instance methods for 'Admonition'

removeallmethods Admonition
removeallclassmethods Admonition

category: 'Instance initialization'
method: Admonition
initialize
  gsNumber := ERR_Admonition.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'AlmostOutOfMemory'

!		Class methods for 'AlmostOutOfMemory'

removeallmethods AlmostOutOfMemory
removeallclassmethods AlmostOutOfMemory

category: 'Control'
classmethod: AlmostOutOfMemory
disable
  "Disables signalling of AlmostOutOfMemory and resets threshold to 90%"
  System _updateSignalErrorStatus: 5 toState: -1 .
  ^ self
%

category: 'Control'
classmethod: AlmostOutOfMemory
enable
  "Enable signaling of AlmostOutOfMemory with the current threshold.
  If no in-memory garbage collection has run since the last delivery
  of an AlmostOutOfMemory exception, then the actual enable will
  be deferred until after in-memory garbage collection .

  You must install a handler for AlmostOutOfMemory to take action
  or execute
    Notification enableSignalling
  to have AlmostOutOfMemory and other Notifications signalled to the
  application to avoid having AlmostOutOfMemory silently ignored."

  System _updateSignalErrorStatus: 5 toState: 0 .
  ^ self
%

category: 'Control'
classmethod: AlmostOutOfMemory
enableAtThreshold: anInteger

  "anInteger represents a percentage and must be between 1 and 125.
  Sets the threshold for signalling AlmostOutOfMemory notification
  and enables the notification.  
  Threshold is also used by AlmostOutOfMemoryError if enabled.
  Returns the receiver."

  self _validateThreshold: anInteger .
  System _updateSignalErrorStatus: 5 toState: anInteger .
  ^ self
%

category: 'Control'
classmethod: AlmostOutOfMemory
enabled
  "Returns true if signaling of AlmostOutOfMemory is enabled.

   See also AlmostOutOfMemory >> enable .
   If the effect of the previous enable was deferred, will return false."
  ^ (System _signalErrorStatus: 5) > 0
%

category: 'Control'
classmethod: AlmostOutOfMemory
threshold
  "Returns the current threshold without changing enabled status
   of the AlmostOutOfMemory notification.
  Threshold is shared with AlmostOutOfMemoryError . "

  ^ System _updateSignalErrorStatus: 7 toState: -1 .
%

category: 'Control'
classmethod: AlmostOutOfMemory
threshold: anInteger
  "anInteger represents a percentage and must be between 1 and 125.
  Sets the threshold for signalling AlmostOutOfMemory notification
  without changing the enabled status .
  Threshold is shared with AlmostOutOfMemoryError .
  Returns the receiver."

  self _validateThreshold: anInteger .
  System _updateSignalErrorStatus: 7 toState: anInteger .
  ^ self
%

category: 'Control'
classmethod: AlmostOutOfMemory
_validateThreshold: anInteger

  "anInteger represents a percentage and must be between 1 and 125."

  anInteger _isSmallInteger ifFalse:[ anInteger _validateClass: SmallInteger ].
  (anInteger < 1 or:[ anInteger > 125]) ifTrue:[
    OutOfRange new name: 'threshold' min: 1 max: 125 actual: anInteger ;
	signal
  ].
%

!		Instance methods for 'AlmostOutOfMemory'

category: 'Instance initialization'
method: AlmostOutOfMemory
initialize
  gsNumber := ERR_AlmostOutOfMemory.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'AlmostOutOfMemoryError'

!		Class methods for 'AlmostOutOfMemoryError'

removeallmethods AlmostOutOfMemoryError
removeallclassmethods AlmostOutOfMemoryError

category: 'Control'
classmethod: AlmostOutOfMemoryError
disable
  "Disables signalling of AlmostOutOfMemoryError and resets threshold to 90%"
  System _updateSignalErrorStatus: 9 toState: -1 .
%

category: 'Control'
classmethod: AlmostOutOfMemoryError
enable
  "Enable signaling of AlmostOutOfMemoryError with the current threshold.
  If no in-memory garbage collection has run since the last delivery
  of an AlmostOutOfMemory or AlmostOutOfMemoryError, then the actual enable will
  be deferred until after in-memory garbage collection ."

  System _updateSignalErrorStatus: 9 toState: 0 .
%

category: 'Control'
classmethod: AlmostOutOfMemoryError
enableAtThreshold: anInteger
  "anInteger represents a percentage and must be between 1 and 125.
  Sets the threshold for signalling AlmostOutOfMemoryError  .
  and enables the signalling of AlmostOutOfMemoryError .
  Threshold is also used by AlmostOutOfMemory (a Notification) if enabled.
  Returns the receiver."

  AlmostOutOfMemory _validateThreshold: anInteger .
  System _updateSignalErrorStatus: 9 toState: anInteger .
%

category: 'Control'
classmethod: AlmostOutOfMemoryError
enabled
  "Returns true if signaling of AlmostOutOfMemoryError is enabled.

   See also AlmostOutOfMemoryError >> enable .
   If the effect of the previous enable was deferred, will return false."
  ^ (System _signalErrorStatus: 9 ) > 0
%

category: 'Control'
classmethod: AlmostOutOfMemoryError
threshold
  "Returns the current threshold without changing enabled status
   of AlmostOutOfMemoryError.
   Threshold is shared with AlmostOutOfMemory (a Notification) ."

  ^ AlmostOutOfMemory threshold
%

category: 'Control'
classmethod: AlmostOutOfMemoryError
threshold: anInteger
  "anInteger represents a percentage and must be between 1 and 125.
  Sets the threshold for signalling AlmostOutOfMemoryError 
  without changing the enabled status .
  Threshold is shared with AlmostOutOfMemory (a Notification) .
  Returns the receiver."

  ^ AlmostOutOfMemory threshold: anInteger .
%

!		Instance methods for 'AlmostOutOfMemoryError'

category: 'Instance initialization'
method: AlmostOutOfMemoryError
initialize
  gsNumber := 2744"ERR_AlmostOutOfMemory".
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'AlmostOutOfStack'

!		Instance methods for 'AlmostOutOfStack'

removeallmethods AlmostOutOfStack
removeallclassmethods AlmostOutOfStack

category: 'Instance initialization'
method: AlmostOutOfStack
initialize
  gsNumber := ERR_AlmostOutOfStack.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'AlmostOutOfStackError'

!		Class methods for 'AlmostOutOfStackError'

removeallmethods AlmostOutOfStackError
removeallclassmethods AlmostOutOfStackError

category: 'Control'
classmethod: AlmostOutOfStackError
disable
  System _updateSignalErrorStatus: 10 toState: false .
%

category: 'Control'
classmethod: AlmostOutOfStackError
enable
  System _updateSignalErrorStatus: 10 toState: true .
%

category: 'Control'
classmethod: AlmostOutOfStackError
enabled
  "returns a Boolean"
  ^ System _signalErrorStatus: 10  
%

!		Instance methods for 'AlmostOutOfStackError'

category: 'Instance initialization'
method: AlmostOutOfStackError
initialize
  gsNumber := ERR_AlmostOutOfStack.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'ArgumentError'

!		Instance methods for 'ArgumentError'

removeallmethods ArgumentError
removeallclassmethods ArgumentError

category: 'Formatting'
method: ArgumentError
buildMessageText
^ self buildMessageText:(
  [ | str |
    gsDetails ifNotNil:[:det |
      (str := String new ) add: det asString .
      object ifNotNil:[:o|
        str add:', '; add: o  asString
      ] ifNil:[
        gsArgs ifNotNil:[:ga | (ga atOrNil: 1) ifNotNil:[:d | str add: d asString]]
      ]
    ].
    str
  ] onException: Error do:[:ex |
    ex return: nil
  ]
)
%

category: 'Instance initialization'
method: ArgumentError
initialize
  gsNumber := ERR_ArgumentError.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Instance initialization'
method: ArgumentError
name: aName
  object := aName
%

! Class extensions for 'ArgumentTypeError'

!		Instance methods for 'ArgumentTypeError'

removeallmethods ArgumentTypeError
removeallclassmethods ArgumentTypeError

category: 'Instance initialization'
method: ArgumentTypeError
args: anArray
  anArray _isArray ifTrue:[ | sz |
    object := anArray atOrNil: 1 .
    expectedClass := anArray atOrNil: 2 .
    actualArg := anArray atOrNil: 3  .
    (sz := anArray size) > 3 ifTrue:[
      gsArgs := anArray copyFrom: 4 to: sz
    ].
  ]
%

category: 'Formatting'
method: ArgumentTypeError
buildMessageText
^ self buildMessageText:(
  [ | str |
    gsDetails ifNotNil:[ :d |  "example use: attempt to modify invariant object"
      str := d asString copy .
      object ifNotNil:[:a | str add: ', ' ; add: a asString ].
    ] ifNil:[ | exp act |
      exp := expectedClass . act := actualArg .
      (exp ~~ nil or:[ act ~~ nil]) ifTrue:[
        str := String new .
        object ifNotNil:[:o | str add: 'for '; add: o asString ; add:'  '].
        exp ifNotNil:[ str add: 'expected a ' ; add: exp name ].
        act ifNotNil:[ str add:', arg is a '; add: act class name ].
      ].
    ].
    str
  ] onException: Error do:[:ex |
    ex return: nil
  ]
)
%

category: 'Instance initialization'
method: ArgumentTypeError
constrainedIv: aName expectedClass: aClass actualArg: anObject
  gsNumber := 2107 .  "legacy error info"
  gsReason := #objErrConstraintViolation .
  object := aName .
  expectedClass := aClass "may be an array of Classes" .
  actualArg := anObject
%

category: 'Instance initialization'
method: ArgumentTypeError
initialize
  gsNumber := ERR_ArgumentTypeError.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Instance initialization'
method: ArgumentTypeError
name: aName expectedClass: aClass actualArg: anObject
  object := aName .
  expectedClass := aClass "may be an array of Classes" .
  actualArg := anObject
%

category: 'Instance initialization'
method: ArgumentTypeError
_legacyHandlerArgs
  | arr |
  arr := { object . expectedClass . actualArg } .
  gsArgs ifNotNil:[:a |  arr addAll: a ].
  ^ arr
%

! Class extensions for 'Array'

!		Class methods for 'Array'

removeallmethods Array
removeallclassmethods Array

category: 'Disallowed'
classmethod: Array
byteSubclass: aString
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
inClassHistory: aClassHistory
description: aDescription
isInvariant: invarBoolean

"Disallowed for Array and its subclasses."
^ self _error: #classErrByteSubclass
%

category: 'Subclass Creation'
classmethod: Array
byteSubclass: aString
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
newVersionOf: oldClass
description: aDescription
options: options

"Disallowed for Array and its subclasses."
^ self _error: #classErrByteSubclass
%

category: 'Instance Creation'
classmethod: Array
with: aValue

"^ (self new: 1) at: 1 put: aValue; yourself"
<primitive: 1099>
^ self _primitiveFailed: #with: args: { aValue }
%

category: 'Instance Creation'
classmethod: Array
with: aValue with: val2

"^ (self new: 2) at: 1 put: aValue; .... yourself"
<primitive: 1099>
^ self _primitiveFailed: #with:with: args: { aValue . val2 }
%

category: 'Instance Creation'
classmethod: Array
with: aValue with: val2 with: val3

"^ (self new: 3) at: 1 put: aValue; .... yourself"
<primitive: 1099>
^ self _primitiveFailed: #with:with: args: { aValue . val2 .val3 }
%

category: 'Instance Creation'
classmethod: Array
with: aValue with: val2 with: val3 with: val4

"^ (self new: 4) at: 1 put: aValue; .... yourself"
<primitive: 1099>
^ self _primitiveFailed: #with:with:with:with; args: { aValue . val2 . val3 . val4 }
%

category: 'Instance Creation'
classmethod: Array
with: aValue with: val2 with: val3 with: val4 with: val5

"^ (self new: 5) at: 1 put: aValue; .... yourself"
<primitive: 1099>
^ self _primitiveFailed: #with:with:with:with:with; args: { aValue . val2 . val3 . val4 . val5 }
%

category: 'Private'
classmethod: Array
_encodingForStringDensity

  "Number of characters encoded per SmallInteger by <primitive: 542>."

^ 7
%

category: 'Private'
classmethod: Array
_insertEncodingForString: aString arraySize: aSmallInteger

  "Encode the given string into SmallIntegers and place them in a new
   instance of Array. Used for user-defined indexing support.

   Note: if there are null characters in the encoded string they will be
      effectively ignored during encoding and the decoded string will
      not match the encoded string."

  <primitive: 542>

  aString  _validateClass: String .
  aSmallInteger _validateClass: SmallInteger .
  ^ self _primitiveFailed: #_insertEncodingForString:arraySize:
         args: { aString . aSmallInteger }

%

category: 'Filein Support'
classmethod: Array
_newKernelByteSubclass: clsName
classVars: classVarArg
poolDictionaries: poolDicts
inDictionary: aDict
options: options
reservedOop: aReservedOop

"Disallowed for Array and its subclasses."

^ self _error: #classErrByteSubclass
%

!		Instance methods for 'Array'

category: 'Adding'
method: Array
add: anObject

"Makes anObject one of the receiver's elements and returns anObject.  The
 new element is actually added as the last element of the receiver.  This
 method is equivalent to #addLast: ."

<primitive: 203>
^ self _primitiveFailed: #add: args: { anObject }
%

category: 'Adding'
method: Array
addAll: aCollection

"Adds all of the elements of aCollection to the receiver and returns
 aCollection."

<primitive: 213>
"primitive handles arg kinds Array, OrderedCollection and IdentityBag."

^ super addAll: aCollection
%

category: 'Adding'
method: Array
addAll: anArray excludingFirst: excludeCount

"If   anArray size > (excludeCount+1)
adds elements of anArray from (excludeCount+1)  to  (anArray size)
to the receiver,  otherwise does nothing.
Returns the receiver .

If excludeCount is less than 0 it is intepreted as 0 ."

<primitive: 231>
anArray _validateClasses: { Array . OrderedCollection }.
excludeCount _validateClass: SmallInteger .
^ self _primitiveFailed: #addAll:excludingFirst:
       args: { anArray . excludeCount }
%

category: 'Adding'
method: Array
addLast: anObject

"Adds anObject as the last element of the receiver and returns anObject."

<primitive: 203>
^ self _primitiveFailed: #addLast: args: { anObject }
%

category: 'Accessing'
method: Array
at: anIndex

"Returns the value of an indexed variable in the receiver.
 The argument anIndex must not be larger than the size of the
 receiver, and must not be less than 1.

 Generates an error if anIndex is not a SmallInteger or is out of
 bounds, or if the receiver is not indexable."

<primitive: 699>
(anIndex _isInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
^ self _primitiveFailed: #at: args: { anIndex }
%

category: 'Private'
method: Array
at: anIndex caseInsensitiveEquals: encodedStringArray

 "Compares without regard to case, in order starting at anIndex, if the contents of the receiver
  are equal to the encoded contents of <encodedStringArray>. The elements of both
  collections must be SmallIntegers. The SmallIntegers represents strings encoded using
  <primitive: 542>."

  ^self at: anIndex compareEqualNoCaseWithEncodedStringArray: encodedStringArray startingAt: 1.
%

category: 'Private'
method: Array
at: anIndex caseSensitiveEquals: encodedStringArray

 "Compares without regard to case, in order starting at anIndex, if the contents of the receiver
  are equal to the encoded contents of <encodedStringArray>. The elements of both
  collections must be SmallIntegers. The SmallIntegers represents strings encoded using
  <primitive: 542>."

  ^self compareStartingAt: anIndex to: encodedStringArray startingAt: 1
%

category: 'Private'
method: Array
at: anIndex compareEqualNoCaseWithEncodedStringArray: encodedStringArray startingAt: stringOffset

 "Compares without regard to case, in order starting at anIndex, if the contents of the receiver
  are equal to the encoded contents of <encodedStringArray>, starting at stringOffset. The elements of both
  collections must be SmallIntegers. The SmallIntegers represents strings encoded using
  <primitive: 542>."

  <primitive: 632 >
  ^self _primitiveFailed: #at:compareEqualNoCaseWithEncodedStringArray:startingAt:
        args: { anIndex .  encodedStringArray . stringOffset }

%

category: 'Private'
method: Array
at: anIndex compareWithEncodedStringArray: encodedStringArray startingAt: stringOffset opCode: anOpCode

 "Compares, in order starting at anIndex, if the contents of the receiver
  are less than/greater than the contents of anArray, starting at stringOffset.
  The elements of both collections must be SmallIntegers. The SmallIntegers
  represents strings encoded using <primitive: 542>.

    op code == 0 means less than
    op code == 1 means greater than"

  <primitive: 629 >
  ^ self _primitiveFailed: #at:compareWithEncodedStringArray:startingAt:opCode:
         args: { anIndex . encodedStringArray . stringOffset . anOpCode }
%

category: 'Private'
method: Array
at: anIndex greaterThanEncodedStringArray: encodedStringArray startingAt: arrayOffset
  "Compares, in order starting at anIndex, if the contents of the receiver are greater
   than the contents of anArray, starting at stringOffset. The elements of both
   collections must be SmallIntegers. The SmallIntegers represent encoded string for
   index. The string was encoded using <primitive: 542>."

  ^self at: anIndex
        compareWithEncodedStringArray: encodedStringArray
        startingAt: arrayOffset
	opCode: 1.
%

category: 'Private'
method: Array
at: anIndex lessThanEncodedStringArray: encodedStringArray startingAt: arrayOffset
   "Compares, in order starting at anIndex, if the contents of the receiver
    are less than the contents of anArray, starting at stringOffset.
    The elements of both collections must be SmallIntegers. The SmallIntegers represent
    encoded string for index. The string was encoded using <primitive: 542>."

  ^self at: anIndex
	compareWithEncodedStringArray: encodedStringArray
	startingAt: arrayOffset
	opCode: 0.
%

category: 'Updating'
method: Array
at: anIndex put: aValue

"Stores the argument aValue in the indexed variable of the
 receiver indicated by anIndex.  The argument anIndex must not be
 larger than 1 + the size of the receiver, and must not be less than 1.

 Returns aValue.

 Generates an error if anIndex is not a SmallInteger or is out of
 bounds."

<primitive: 745>

(anIndex _isInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
((anIndex < 1) | (anIndex > (self size + 1))) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].

^ self _primitiveFailed: #at:put: args: { anIndex . aValue }
%

category: 'Accessing'
method: Array
atOrNil: anIndex

"Return the value   self at: anIndex ,
 or return nil if anIndex is out of range . "

<primitive: 699>
anIndex _isInteger ifTrue:[
  ^ nil
].
self _errorNonIntegerIndex: anIndex .
^ self _primitiveFailed: #atOrNil: args: { anIndex }
%

category: 'Private'
method: Array
compareStartingAt: anIndex to: anArray startingAt: anotherIndex

  "Compare the elements of the receiver starting at <anIndex> to <anArray>
   starting at <anotherIndex>. Note that it is assumed that the objects
   in each slot of the receiver and <anArray> are SmallIntegers encoded
   from strings produced by <primitive: 542>, however, it is not a
   requirement, since each element is compared using identity."

  <primitive: 628 >
  ^ self _primitiveFailed: #compareStartingAt:to:startingAt:
         args: { anIndex . anArray . anotherIndex }
%

category: 'Copying'
method: Array
copyFrom: startIndex to: stopIndex

"Returns a new Array containing the elements of the receiver
 between startIndex and stopIndex, inclusive.  The result is of the same class
 as the receiver.

 If startIndex > stopIndex then an empty collection is returned.
 Otherwise both startIndex and stopIndex must be positive integers not larger than the
 size of the receiver, with startIndex <= stopIndex.
"

<primitive: 817>
(startIndex < 1) ifTrue: [ ^ self _errorIndexOutOfRange: startIndex].

((stopIndex > self size) or: [(stopIndex < 0)])
   ifTrue: [ ^ self _errorIndexOutOfRange: stopIndex].

self _primitiveFailed: #copyFrom:to:
     args: { startIndex . stopIndex }
%

category: 'Copying'
method: Array
copyNotNilFrom: startIndex to: stopIndex
 "Return an Array containing the non-nil elements 
  that are within the specified offsets of the receiver."
 <primitive: 541>
 "Primitive fails for a large object."
 | res |
 startIndex _validateInstanceOf: SmallInteger .
 stopIndex _validateInstanceOf: SmallInteger .
 (startIndex < 1 or:[ startIndex > self size ]) ifTrue:[
   ^ self _errorIndexOutOfRange: startIndex
 ].
 stopIndex < startIndex ifTrue:[ ^ self _errorIndexOutOfRange: stopIndex].
 res := { } .
 startIndex to: stopIndex do:[:j |
   (self at: j) ifNotNil:[:x| res add: x  ] .
 ].
 ^ res
%

category: 'Initializing'
method: Array
fillFrom: index1 resizeTo: newSize with: anObject

"If newSize > 0,
   change size of receiver to newSize and
   store anObject into instVars 1..newSize
 else
   store anObject into instVars 1..(0 - newSize)
"
<primitive: 607>
index1 _validateClass: SmallInteger .
newSize _validateClass: SmallInteger .
(index1 > newSize abs )
  ifTrue:[ index1 _error: #rtErrBadCopyFromTo args: { newSize }].

(index1 < 1) ifTrue:[ self _errorIndexOutOfRange: index1].
self _primitiveFailed: #fillFrom:resizeTo:with:
     args: { index1 . newSize . anObject }
%

category: 'Comparing'
method: Array
hasIdenticalContents: anArray

"Returns true if all of the following conditions are true:

 1.  The receiver and anArray are of the same class.
 2.  The two Arrays are the same size.
 3.  The corresponding elements of the receiver and anArray are identical.

 Returns false otherwise."

<primitive: 612 >

^ self _primitiveFailed: #hasIdenticalContents: args: { anArray }
%

category: 'Searching'
method: Array
includesIdentical: anObject

"Returns true if anObject is identical to one of the elements of the
 receiver.  Returns false otherwise."

<primitive: 59>
self _primitiveFailed: #includesIdentical: args: { anObject }
%

category: 'Searching'
method: Array
includesValue: anObject

"Returns true if anObject is equal to one of the elements of the receiver.
 Returns false otherwise."

^ self includes: anObject
%

category: 'Searching'
method: Array
indexOfIdentical: anObject

"Private.  Returns the index of the first element in the receiver that is
 identical to the argument.  If the receiver does not have any elements that are
 identical to the argument, returns zero."

<primitive: 494>
self _primitiveFailed: #indexOfIdentical: args: { anObject }
%

category: 'Searching'
method: Array
indexOfNotNil: startOffset to: endOffset
"If startOffset <= endOffset, returns the first offset witin in the specified range
 of a non-nil element of the receiver.
 If startOffset > endOffset, returns the last offset within the specified range
 of a non-nil element of the receiver.
 Returns zero if all are nil .
 Intended for use only on Arrays of size <= 2000 .  
 Performance on larger arrays will be slow.
 "
 <primitive: 540>
 "Primitive fails for a large object."
  | sz |
 startOffset _validateInstanceOf: SmallInteger .
 endOffset _validateInstanceOf: SmallInteger .
 sz := self size .
 (startOffset < 1 or:[ startOffset > sz]) ifTrue:[
   ^ self _errorIndexOutOfRange: startOffset
 ].
 (endOffset < 1 or:[ endOffset > sz]) ifTrue:[
   ^ self _errorIndexOutOfRange: startOffset
 ].
 startOffset <= endOffset ifTrue:[
   startOffset to: endOffset do:[:j |
     (self at: j) ifNotNil:[ ^ j ] .
   ].
 ] ifFalse:[
   startOffset _downTo: endOffset do:[:j |
     (self at: j) ifNotNil:[ ^ j ] .
   ].
 ].
 ^ 0
%

category: 'Copying'
method: Array
replaceFrom: startIndex to: stopIndex with: aCollection startingAt: repIndex

"Replaces the elements of the receiver between the indexes startIndex and
 stopIndex inclusive with the elements of aSeqCollection starting at repIndex.
 If aCollection is identical to the receiver, the source and
 destination blocks may overlap.

 aCollection must be a kind of SequenceableCollection or a kind of IdentityBag.

 Returns the receiver."

<primitive: 608>
startIndex _isSmallInteger ifFalse:[ startIndex _validateClass: SmallInteger ].
stopIndex _isSmallInteger ifFalse:[ stopIndex _validateClass: SmallInteger ].
repIndex _isSmallInteger ifFalse:[ repIndex _validateClass: SmallInteger ].

^ super replaceFrom: startIndex to: stopIndex with: aCollection startingAt: repIndex
%

category: 'Adding'
method: Array
_addAllFromNsc: aBag

""

<primitive: 202>
^ self _primitiveFailed: #_addAllFromNsc: args: { aBag }
%

category: 'Decompiling without Sources'
method: Array
_asSource

""

| result word lineLength LF |
self isInvariant ifFalse:[ self _error: #rtErrObjVariant ] .
result := String new .
LF := Character lf .
result addAll: ' #( ' .
lineLength := result size .
1 to: self size do:[:j |
  word := (self at: j) _asSource .
  result addAll: word .
  result add: $  .
  lineLength := lineLength + word size + 1 .
  lineLength > 80 ifTrue:[
    result add: LF ; add: $  .
    lineLength := 1 .
    ].
  ].
result addAll: ') ' .
^ result .
%

category: 'Accessing'
method: Array
_at: anIndex

"Returns the value of an indexed variable in the receiver.
 The argument anIndex must not be larger than the size of the
 receiver, and must not be less than 1.

 Generates an error if anIndex is not a SmallInteger or is out of
 bounds, or if the receiver is not indexable."

<primitive: 699>
(anIndex _isInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
^ self _primitiveFailed: #at: args: { anIndex }
%

category: 'Updating'
method: Array
_at: anIndex put: aValue

"Stores the argument aValue in the indexed variable of the
 receiver indicated by anIndex.  The argument anIndex must not be
 larger than 1 + the size of the receiver, and must not be less than 1.

 Returns aValue.

 Generates an error if anIndex is not a SmallInteger or is out of
 bounds."

<primitive: 745>

(anIndex _isInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
((anIndex < 1) | (anIndex > (self size + 1))) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].

^ self _primitiveFailed: #at:put: args: { anIndex . aValue }
%

category: 'Updating'
method: Array
_basicAt: anIndex put: aValue

"Stores the argument aValue in the indexed variable of the
 receiver indicated by anIndex.  The argument anIndex must not be
 larger than 1 + the size of the receiver, and must not be less than 1.

 Returns aValue.

 Generates an error if anIndex is not a SmallInteger or is out of
 bounds."

<primitive: 745>

(anIndex _isInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
((anIndex < 1) | (anIndex > (self size + 1))) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].

^ self _primitiveFailed: #_basicAt:put: args: { anIndex . aValue }
%

category: 'Private'
method: Array
_decodeKeyAt: keyIdx decoding: numToDecode into: aString
  "Decode <numToDecode> SmallIntegers from the receiver into <aString> as
   encoded by <primitive: 542>.

   Note: if there are null characters in the encoded string they will be
      effectively ignored during encoding and the decoded string will
      not match the encoded string."

  <primitive: 631 >
  aString  _validateClass: String .
  numToDecode _validateClass: SmallInteger .
  keyIdx _validateClass: SmallInteger .
  ^ self _primitiveFailed: #_decodeKeyAt:decoding:into:
         args: { keyIdx . numToDecode . aString }
%

category: 'Removing'
method: Array
_deleteNoShrinkFrom: startIndex to: stopIndex

"The elements of the receiver from startIndex to stopIndex are
 deleted from the receiver.  And the tail of the receiver is filled with
 (stopIndex - startIndex + 1) number of nils.

 The size of the receiver is not changed.
 Does space-efficient tranlogging of the deletion.

 Both startIndex and stopIndex must be positive integers not larger
 than the size of the receiver, with startIndex <= stopIndex."

<primitive: 602 >

(stopIndex < startIndex)
ifTrue:
   [ ^ startIndex _error: #rtErrBadCopyFromTo args: { stopIndex }].
((stopIndex > self size) or: [(stopIndex < 1)])
   ifTrue: [ ^ self _errorIndexOutOfRange: stopIndex].
(startIndex < 1)
   ifTrue: [ ^ self _errorIndexOutOfRange: startIndex].
^ self _primitiveFailed: #_deleteNoShrinkFrom:to:
       args: { startIndex . stopIndex }
%

category: 'Private'
method: Array
_deleteNoShrinkFrom: startIndex to: stopIndex anchorTailSize: tailSize

"Same as _deleteNoShrinkFrom: startIndex to: stopIndex except the last
tailSize objects remain in the tail of the array.  The size of the receiver does not change
and the gap created by the delete operation is filled with nils.

This method can fail for the following reasons:
-receiver participates in an index.
-receiver is a large object.
-the delete operation requested would delete any part of the tail

 The size of the receiver is not changed.

 Both startIndex and stopIndex must be positive integers not larger
 than the size of the receiver, with startIndex <= stopIndex.  "

<primitive: 617 >

(stopIndex < startIndex)
ifTrue:
   [ ^ startIndex _error: #rtErrBadCopyFromTo args: { stopIndex }].
((stopIndex > self size) or: [(stopIndex < 1)])
   ifTrue: [ ^ self _errorIndexOutOfRange: stopIndex].
(startIndex < 1)
   ifTrue: [ ^ self _errorIndexOutOfRange: startIndex].
(stopIndex > (self size - tailSize) )
   ifTrue: [ ^ self _errorIndexOutOfRange: tailSize].

^ self _primitiveFailed: #_deleteNoShrinkFrom:to:anchorTailSize:
       args: { startIndex . stopIndex . tailSize }
%

category: 'Private'
method: Array
_finalize: anObj at: offset

"Finalizes the reference to object anObject by substituting nil."

self at: offset put: nil.
^true
%

category: 'Copying'
method: Array
_insertAt: destOffset
from: anArray
fromStart: startOffset
fromEnd: endOffset
numToMoveDown: numToMove

"Intended for use in manipulating objects which are nodes of B-trees or
 similar structures.  Does not do optimized tranlogging.

 Inserts the portion of anArray from startOffset to endOffset
 into the receiver beginning at destOffset.

 If anArray is nil, then  (endOffset - startOffset + 1) nils are
 inserted into the receiver.

 The indexable instance variables of the receiver from destOffset to
 (destOffset + numToMove) are moved towards the end of the receiver by
 the amount (endOffset - startOffset + 1).
 The receiver must be a small object.

 If the receiver must be grown, it will be grown by 10 times the number
 of OOPs inserted up to a max of 500 oops. The receiver will not be
 grown larger than the max size of a small object. If the number of
 inserted oops alone will cause the object to be larger than the max
 size of a small object, then an error will be generated.

 Generates an error if any of the arguments imply the receiver must be
 grown to be a large object, or if destOffset is beyond the end of
 the receiver."

<primitive: 605 >

| numInserted newSize |

anArray ~~ nil ifTrue:[ anArray _validateClass: Array ].

(destOffset < 1 or:[ destOffset > self _basicSize] )
   ifTrue:[ self _errorIndexOutOfRange: destOffset] .
(startOffset < 1) ifTrue:[ self _errorIndexOutOfRange: startOffset] .
(endOffset < 1)   ifTrue:[ self _errorIndexOutOfRange: endOffset] .
(numToMove < 0)   ifTrue:[ self _errorIndexOutOfRange: numToMove] .
(endOffset < startOffset) ifTrue:[ self _errorIndexOutOfRange: endOffset ].
(anArray ~~ nil and: [startOffset > anArray _basicSize]) ifTrue:[ anArray _errorIndexOutOfRange: startOffset].
(anArray ~~ nil and: [endOffset > anArray _basicSize]) ifTrue:[ anArray _errorIndexOutOfRange: endOffset].

numInserted := endOffset - startOffset + 1 .

newSize := destOffset + numInserted + numToMove .
newSize > 2034 "virtual machine constant OBJ_OOPS_SIZE" ifTrue:[
  self _error: #objErrMaxSize args:{ 2034 "virtual machine constant" . newSize }.
].

self _primitiveFailed: #_insertAt:from:fromStart:fromEnd:numToMoveDown:
     args: { destOffset . anArray . startOffset . endOffset . numToMove }
%

category: 'Copying'
method: Array
_insertAt: destOffset
value: anObject
numToMoveDown: numToMove

"Intended for use in manipulating objects which are nodes of B-trees or
 similar structures.  Does space-efficient tranlogging of the insertion.

 Inserts anObject into receiver at destOffset, with space-efficient tranlogging
 of the insert.

 The indexable instance variables of the receiver from destOffset to
 (destOffset + numToMove) are moved towards the end of the receiver by 1 .
 then does (self at: destOffset put: anObject ).
 The receiver must be a small object;

 Generates an error if any of the arguments imply the receiver must be
 grown to be a large object, or if destOffset is beyond the end of
 the receiver."

<primitive: 1041>
| newSize |
destOffset _validateClass: SmallInteger .
numToMove _validateClass: SmallInteger .
(destOffset < 1)   ifTrue:[ self _errorIndexOutOfRange: destOffset] .
(numToMove < 1)   ifTrue:[ self _errorIndexOutOfRange: destOffset] .
newSize := self namedSize +  destOffset + 1  +  numToMove .
newSize > 2034 "virtual machine constant OBJ_OOPS_SIZE" ifTrue:[
  self _error: #objErrMaxSize args:{ 2034 "virtual machine constant" . newSize }.
].
self _primitiveFailed: #_insertAt:value:numToMoveDown
     args: { destOffset . anObject . numToMove }
%

category: 'Copying'
method: Array
_insertAt: destOffset
value: valOne
value: valTwo
numToMoveDown: numToMove

"Intended for use in manipulating objects which are nodes of B-trees or
 similar structures.  Does space-efficient tranlogging of the insertion.

 Inserts objects into receiver at destOffset, with space-efficient tranlogging
 of the insert.

 The indexable instance variables of the receiver from destOffset to
 (destOffset + numToMove) are moved towards the end of the receiver by 2 .
 then does (self at: destOffset put: valOne ; at: destOffset+1 put: valTwo ).
 The receiver must be a small object;

 Generates an error if any of the arguments imply the receiver must be
 grown to be a large object, or if destOffset is beyond the end of
 the receiver."

<primitive: 1032>
| newSize |
destOffset _validateClass: SmallInteger .
numToMove _validateClass: SmallInteger .
(destOffset < 1)   ifTrue:[ self _errorIndexOutOfRange: destOffset] .
(numToMove < 1)   ifTrue:[ self _errorIndexOutOfRange: destOffset] .
newSize := self namedSize +  destOffset + 2  +  numToMove .
newSize > 2034 "virtual machine constant OBJ_OOPS_SIZE" ifTrue:[
  self _error: #objErrMaxSize args:{ 2034 "virtual machine constant" . newSize }.
].
self _primitiveFailed: #_insertAt:value:value:numToMoveDown
     args: { destOffset . valOne . valTwo . numToMove }
%

category: 'Copying'
method: Array
_insertAt: destOffset
value: valOne
value: valTwo
value: valThree
numToMoveDown: numToMove

"Intended for use in manipulating objects which are nodes of B-trees or
 similar structures.  Does space-efficient tranlogging of the insertion.

 Inserts objects into receiver at destOffset, with space-efficient tranlogging
 of the insert.

 The indexable instance variables of the receiver from destOffset to
 (destOffset + numToMove) are moved towards the end of the receiver by 3 .
 then does (self at: destOffset put: valOne ; at: destOffset+1 put: valTwo ;
                at: destOffset+2 put: valThree )
 The receiver must be a small object;

 Generates an error if any of the arguments imply the receiver must be
 grown to be a large object, or if destOffset is beyond the end of
 the receiver."

<primitive: 1033>
| newSize |
destOffset _validateClass: SmallInteger .
numToMove _validateClass: SmallInteger .
(destOffset < 1)   ifTrue:[ self _errorIndexOutOfRange: destOffset] .
(numToMove < 1)   ifTrue:[ self _errorIndexOutOfRange: destOffset] .
newSize := self namedSize +  destOffset + 3  +  numToMove .
newSize > 2034 "virtual machine constant OBJ_OOPS_SIZE" ifTrue:[
  self _error: #objErrMaxSize args:{ 2034 "virtual machine constant" . newSize }.
].
self _primitiveFailed: #_insertAt:value:value:value:numToMoveDown
     args: { destOffset . valOne . valTwo . valThree . numToMove }
%

category: 'Copying'
method: Array
_insertAt: destOffset
value: valOne
value: valTwo
value: valThree
value: valFour
numToMoveDown: numToMove

"Intended for use in manipulating objects which are nodes of B-trees or
 similar structures.  Does space-efficient tranlogging of the insertion.

 Inserts objects into receiver at destOffset, with space-efficient tranlogging
 of the insert.

 The indexable instance variables of the receiver from destOffset to
 (destOffset + numToMove) are moved towards the end of the receiver by 4 .
 then does (self at: destOffset put: valOne ; at: destOffset+1 put: valTwo ;
         at: destOffset+2 put: valThree ;  at: destOffset+3 put: valFour )
 The receiver must be a small object;

 Generates an error if any of the arguments imply the receiver must be
 grown to be a large object, or if destOffset is beyond the end of
 the receiver."

<primitive: 1034>
| newSize |
destOffset _validateClass: SmallInteger .
numToMove _validateClass: SmallInteger .
(destOffset < 1)   ifTrue:[ self _errorIndexOutOfRange: destOffset] .
(numToMove < 1)   ifTrue:[ self _errorIndexOutOfRange: destOffset] .
newSize := self namedSize +  destOffset + 4  +  numToMove .
newSize > 2034 "virtual machine constant OBJ_OOPS_SIZE" ifTrue:[
  self _error: #objErrMaxSize args:{ 2034 "virtual machine constant" . newSize }.
].
self _primitiveFailed: #_insertAt:value:value:value:value:numToMoveDown
     args: { destOffset . valOne . valTwo . valThree . valFour . numToMove }
%

category: 'Copying'
method: Array
_insertAt: destOffset
value: valOne
value: valTwo
value: valThree
value: valFour
value: valFive
numToMoveDown: numToMove

"Intended for use in manipulating objects which are nodes of B-trees or
 similar structures.  Does space-efficient tranlogging of the insertion.

 Inserts objects into receiver at destOffset, with space-efficient tranlogging
 of the insert.

 The indexable instance variables of the receiver from destOffset to
 (destOffset + numToMove) are moved towards the end of the receiver by 5 .
 then does (self at: destOffset put: valOne ; at: destOffset+1 put: valTwo ;
         at: destOffset+2 put: valThree ;  at: destOffset+3 put: valFour;
         at: destOffset+4 put: valFive  )
 The receiver must be a small object;

 Generates an error if any of the arguments imply the receiver must be
 grown to be a large object, or if destOffset is beyond the end of
 the receiver."

<primitive: 1035>
| newSize |
destOffset _validateClass: SmallInteger .
numToMove _validateClass: SmallInteger .
(destOffset < 1)   ifTrue:[ self _errorIndexOutOfRange: destOffset] .
(numToMove < 1)   ifTrue:[ self _errorIndexOutOfRange: destOffset] .
newSize := self namedSize +  destOffset + 5  +  numToMove .
newSize > 2034 "virtual machine constant OBJ_OOPS_SIZE" ifTrue:[
  self _error: #objErrMaxSize args:{ 2034 "virtual machine constant" . newSize }.
].
self _primitiveFailed: #_insertAt:value:value:value:value:value:numToMoveDown
     args: { destOffset . valOne . valTwo . valThree . valFour . valFive . numToMove }
%

category: 'Copying'
method: Array
_largeInsertAt: destOffset
from: anArray
fromStart: startOffset
fromEnd: endOffset
numToMoveDown: numToMove

"Intended for use in manipulating objects which are nodes of B-trees or
 similar structures. Similar to primitive 605, except that with this
 primitive objects will be grown to large objects if necessary.
 Does not do optimized tranlogging.

 Inserts the portion of anArray from startOffset to endOffset
 into the receiver beginning at destOffset.

 If anArray is nil, then  (endOffset - startOffset + 1) nils are
 inserted into the receiver.

 The indexable instance variables of the receiver from destOffset to
 (destOffset + numToMove) are moved towards the end of the receiver by
 the amount (endOffset - startOffset + 1).
 The receiver may be a small or large object.

 If the receiver must be grown, it will be grown by 10 times the number
 of OOPs inserted up to a max of 500 oops.

 Generates an error if destOffset is beyond the end of the receiver."

<primitive: 552 >

anArray ~~ nil ifTrue:[ anArray _validateClass: Array ].

(destOffset < 1 or:[ destOffset > self _basicSize] )
   ifTrue:[ self _errorIndexOutOfRange: destOffset] .
(startOffset < 1) ifTrue:[ self _errorIndexOutOfRange: startOffset] .
(endOffset < 1)   ifTrue:[ self _errorIndexOutOfRange: endOffset] .
(numToMove < 0)   ifTrue:[ self _errorIndexOutOfRange: numToMove] .
(endOffset < startOffset) ifTrue:[ self _errorIndexOutOfRange: endOffset ].
(anArray ~~ nil and: [startOffset > anArray _basicSize]) ifTrue:[ anArray _errorIndexOutOfRange: startOffset].
(anArray ~~ nil and: [endOffset > anArray _basicSize]) ifTrue:[ anArray _errorIndexOutOfRange: endOffset].

self _primitiveFailed: #_largeInsertAt:from:fromStart:fromEnd:numToMoveDown:
     args: { destOffset . anArray . startOffset . endOffset . numToMove }
%

! Class extensions for 'Association'

!		Class methods for 'Association'

removeallmethods Association
removeallclassmethods Association

category: 'Instance Creation'
classmethod: Association
newWithKey: aKey value: aValue

"Returns a new Association with the argument aKey as its key and with aValue as
 its value."

^ super new key: aKey value: aValue
%

!		Instance methods for 'Association'

category: 'Comparing'
method: Association
< anObject

"Returns true if the key of the receiver collates before anObject.
 Returns false otherwise."

^ key == self
  ifTrue: [ false ]
  ifFalse: [ anObject > self key ]
%

category: 'Comparing'
method: Association
<= anObject

"Returns true if the key of the receiver collates before anObject, or if the
 key of the receiver is equivalent to anObject.  Returns false otherwise."

^ key == self
  ifTrue: [ true ]
  ifFalse: [ anObject >= self key ]
%

category: 'Comparing'
method: Association
= anObject

"Returns true if (a) the receiver and anObject are both kinds of Association,
 (b) the receiver and anObject have equal keys and (c) the receiver and
 anObject have equal values.  Returns false otherwise."

(self == anObject) ifTrue: [^ true].
(anObject isKindOf: Association) ifFalse:[^ false].
^ (key = anObject key) and: [value = anObject value].
%

category: 'Comparing'
method: Association
> anObject

"Returns true if the key of the receiver collates after anObject.
 Returns false otherwise."

^ anObject < self key
%

category: 'Comparing'
method: Association
>= anObject

"Returns true if the key of the receiver collates after anObject, or if the
 key of the receiver is equivalent to anObject.  Returns false otherwise."

^ anObject <= self key
%

category: 'Clustering'
method: Association
clusterDepthFirst

"This method clusters the receiver, its key, and its value in
 depth-first order.  It returns true if the receiver has already
 been clustered during the current transaction, false otherwise."

  self cluster
  ifTrue:
    [ ^ true ]
  ifFalse:
    [ key clusterDepthFirst.
      value clusterDepthFirst.
      ^ false
    ].
%

category: 'Comparing'
method: Association
hash

"Returns an Integer hash code for the receiver."

^ key hash
%

category: 'Testing'
method: Association
isAssociation
  ^ true
%

category: 'Formatting'
method: Association
isInPrintingRecursionSet: anIdentitySet
	"Answer whether we are recursing through the receiver.

	 For associations, there are pathological cases one can create,
	 that we have chosen to portray in their logical representation.
	 Since some dictionaries use associations and others do not,
	 portray the results as if they all do.

	 In addition to the fast identity check, include a slower equality check."

	^(super isInPrintingRecursionSet: anIdentitySet)
		or: [anIdentitySet includesValue: self]
%

category: 'Accessing'
method: Association
key

"Returns the value of the receiver's key."

^key
%

category: 'Updating'
method: Association
key: aKey

"Sets the object aKey as the key of the receiver."

key := aKey
%

category: 'Updating'
method: Association
key: aKey value: aValue

"Sets the object aKey as the key of the receiver, and the object aValue as the
 value of the receiver."

key := aKey.
value := aValue
%

category: 'Formatting'
method: Association
printNonRecursiveRepresentationOn: aStream recursionSet: anIdentitySet
	"Put a displayable representation of the receiver on the given stream
	 while avoiding recursion from object reference loops."

	key printOn: aStream recursionSet: anIdentitySet.
	aStream nextPutAll: '->'.
	value printOn: aStream recursionSet: anIdentitySet
%

category: 'Formatting'
method: Association
printOn: aStream
	"Put a displayable representation of the receiver on the given
	 stream. If the key or the value is identical to the receiver then
	 avoid an infinite recursion. (There is currently no general way to
	 limit these kinds of recursions. See Bug #16259)"

	self printNonRecursiveOn: aStream
%

category: 'Accessing'
method: Association
value

"Returns the value portion of the receiver."

^value
%

category: 'Updating'
method: Association
value: aValue

"Sets the object aValue as the value of the receiver."

value := aValue
%

category: 'Indexing Support'
method: Association
_idxValue

	^self value
%

category: 'Accessing'
method: Association
_value

" Returns the value portion of the receiver.
  Faster than #value  in Gs64 v3.0 since send of #value
  assumes ExecBlock is most likely receiver. "

^value
%

category: 'Updating'
method: Association
_value: aValue

" Updates the value portion of the receiver.
  Faster than #value:  in Gs64 v3.0 since send of #value:
  assumes ExecBlock is most likely receiver. "

value := aValue
%

category: 'Comparing'
method: Association
~= anObject

"Returns true if one or more of the conditions specified in #= method are
 not satisfied.  Returns false otherwise."

^ (self = anObject) == false.
%

! Class extensions for 'Behavior'

!		Class methods for 'Behavior'

removeallmethods Behavior
removeallclassmethods Behavior

category: 'Instance Creation'
classmethod: Behavior
new

"Disallowed.  To create a new Class or metaclass, use
 Class | subclass:instVarNames:... instead."

self shouldNotImplement: #new .
^ nil
%

category: 'Instance Creation'
classmethod: Behavior
new: anInteger

"Disallowed.  To create a new Class or metaclass, use
 Class | subclass:instVarNames:... instead."

self shouldNotImplement: #new: .
^ nil
%

!		Instance methods for 'Behavior'

category: 'Modifying Classes'
method: Behavior
addInstVar: aString

"Adds a new instance variable named aString to the receiver.  The argument
 aString must be a valid GemStone Smalltalk identifier and must be distinct from
 the names of all other instance variables previously defined for the receiver,
 its superclasses, and its subclasses.

 The new instance variable becomes the last named instance variable in the
 receiver, and is inserted at the appropriate position in each of the
 receiver's subclasses, to preserve the rules for inheritance of instance
 variables.  If, for any of the receiver's subclasses, the new instance
 variable is not the last named instance variable, then all instance methods
 for that subclass are recompiled using the symbol list of the current user.
 If an error occurs during recompilation of methods, the new instance
 variable will have been added to the receiver and all of its subclasses, but
 some methods in some subclasses will not have been recompiled.

 To successfully invoke this method, the receiver must meet one of these two
 conditions:

 * The receiver and all of its subclasses must be modifiable.
 * The receiver must disallow subclasses and must have no unnamed instance
   variables."

| newOffset theSymbol |
self _validatePrivilege ifTrue:[
  "3.2, no error check for NSCs"
  (self isBytes) ifTrue:[ ^ self _error: #rtErrInstvarAddToBytes ] .
  theSymbol:= aString asSymbol .
  theSymbol validateIsIdentifier .
  (self _validateNewNamedInstVar: theSymbol ) ifFalse:[ ^ nil ].

  newOffset := self instSize + 1 .  "the offset of the new instance variable"
  self _insertNamedInstVar: theSymbol atOffset: newOffset .
  self _recompileMethodsAfterNewIvOffset: newOffset .
]
%

category: 'Accessing Variables'
method: Behavior
allClassVarNames

"Returns an Array of Symbols, consisting of the names of the
 smalltalk class variables addressable by this class,
 including those inherited from superclasses.  Contrast with classVarNames."

| result currClass |

result:= { } .
currClass:= self.
[ true ] whileTrue:[
  currClass == nil ifTrue:[ ^ result ].
  result insertAll: (currClass classVarNames) at: 1.
  currClass:= currClass superClass .
]
%

category: 'Accessing Variables'
method: Behavior
allInstVarNames

"Returns an Array of Symbols, consisting of the names of all the receiver's
 instance variables, including those inherited from superclasses.  The ordering
 of the names in the Array follows the ordering of the superclass hierarchy;
 that is, instance variable names inherited from Object are listed first, and
 those peculiar to the receiver are last."

^ Array withAll: instVarNames
%

category: 'Modifying Classes'
method: Behavior
allowSubclasses

"Allows creation of subclasses of a class."

format := (format bitOr:32) bitXor: 32 .  "clear bit 32"
%

category: 'Accessing the Method Dictionary'
method: Behavior
allSelectors

^ self allSelectorsForEnvironment: 0
%

category: 'Accessing the Method Dictionary'
method: Behavior
allSelectorsForEnvironment: envId
"Returns an Array of Symbols, consisting of all of the message
 selectors that instances of the receiver can understand, including
 those inherited from superclasses.  For keyword messages, the
 Symbol includes each of the keywords, concatenated together."

 | result currClass |
 result:= IdentitySet new .
 currClass:= self.
 [ currClass == nil ] whileFalse: [
     result _addAll: (currClass selectorsForEnvironment: envId ) forReplay: false .
     currClass:= currClass superclassForEnv: envId .
 ].
 ^ Array withAll: result
%

category: 'Accessing Variables'
method: Behavior
allSharedPools

"Returns an Array of pool dictionaries used by this class and its superclasses.
 Contrast with sharedPools."

| result currClass |

result:= { } .
currClass:= self.
[ true ] whileTrue:[
  currClass == nil ifTrue:[ ^ result ].
  result insertAll: (currClass sharedPools) at: 1.
  currClass:= currClass superClass
].

^ result
%

category: 'Accessing the Class Hierarchy'
method: Behavior
allSuperClasses

"Returns an Array of the environment 0 superclasses of the receiver,
 beginning with the immediate superclass, and excluding the receiver."

| result cls |
result:= { } .
cls := self superClass .
[ true ] whileTrue:[
  cls == nil ifTrue:[ ^ result ].
  result add: cls .
  cls := cls superClass.
]
%

category: 'Enumerating'
method: Behavior
allSuperClassesDo: aBlock

"Evaluates aBlock with each of the receiver's superclasses as
 the argument, beginning with the immediate superclass."

| currClass |

currClass:= self superClass .
[ true ] whileTrue:[
  currClass ifNil:[ ^ self ].
  aBlock value: currClass .
  currClass:= currClass superClass
]
%

category: 'Accessing the Class Hierarchy'
method: Behavior
allSuperClassesForEnv: envId

"Returns an Array of the superclasses of the receiver, in the specified
 environment beginning with the immediate superclass, and excluding the receiver."

| result cls |
result:= { } .
cls := self superclassForEnv: envId .
[ true ] whileTrue:[
  cls == nil ifTrue:[ ^ result ].
  result add: cls .
  cls := cls superclassForEnv: envId .
]
%

category: 'Queries'
method: Behavior
areInstancesSpecial

"Returns whether instances of this class have their state encoded in their
 identities."  "fix 48559"

^ (format bitAnd: 16r3) == 3  "inline _isSpecial"
%

category: 'Formatting'
method: Behavior
asString

"Returns a String that indicates the class of the receiver."
| n |
n := self _name .  "avoid use of squeak Object>>name"
n ifNil:[ ^ '(unNamedClass)' copy ].
n _stringCharSize == 1 ifTrue:[
  n class == Symbol ifTrue:[
    ^ String withAll: n
  ].
  ^ n
].
^ DoubleByteString withAll: n
%

category: 'Formatting'
method: Behavior
asUnicodeString

"Returns a Unicode string that indicates the class of the receiver."
| n |
n := self _name .  "avoid use of squeak Object>>name"
n ifNil:[ ^ Unicode7 withAll: '(unNamedClass)' ].
n _stringCharSize == 1 ifTrue:[
  ^ Unicode7 withAll: n .
].
^ Unicode16 withAll: n  "Unicode representation of DoubleByteSymbol"
%

category: 'Instance Creation'
method: Behavior
basicNew

"Creates a new, uninitialized instance of the receiver."

^self _basicNew
%

category: 'Instance Creation'
method: Behavior
basicNew: anInteger

"Creates a new, uninitialized instance of the receiver with the given
 number of indexed instance variables."

^self _basicNew: anInteger
%

category: 'Indexing Support'
method: Behavior
btreeLeafNodeClass

"Returns the class of BtreeLeafNode to create for an equality index whose last
 object along the path is an instance of the receiver."

| result |
(RangeEqualityIndex isBasicClass: self)
    ifTrue: [ result := self indexManager btreeBasicLeafNodeClass]
    ifFalse: [ result := self indexManager btreeLeafNodeClass ].
^ result
%

category: 'Accessing the Method Dictionary'
method: Behavior
canUnderstand: aSelector

"Returns true if the receiver can respond to the message indicated by
 aSelector, returns false otherwise.  The selector (a String) can be in the
 method dictionary of the receiver or any of the receiver's superclasses."

| aSymbol |
aSymbol := Symbol _existingWithAll: aSelector .
aSymbol ifNil:[ ^ false ].

^ (self whichClassIncludesSelector: aSymbol) ~~ nil
%

category: 'Accessing Categories'
method: Behavior
categoryNames

"Returns an Array of Symbols.  The elements of the Array are the
 receiver's environment 0 category names (excluding names inherited from superclasses)."

 | set |
 set := IdentitySet new .
 self env: 0 categorysDo:[ :categName :selectors | set add: categName ].
 ^ Array withAll: set
%

category: 'Accessing the Method Dictionary'
method: Behavior
categoryOfSelector: aSelector
 ^ self categoryOfSelector: aSelector environmentId: 0
%

category: 'Accessing the Method Dictionary'
method: Behavior
categoryOfSelector: aSelector environmentId: envId

"Returns a Symbol which is the name of the category for the specified
 selector, or nil if the selector was not found in any category."

| aSymbol |
aSymbol := Symbol _existingWithAll: aSelector .
aSymbol ifNil:[ ^ nil ].
self env: envId categorysDo:[ :categName :selectors |
  (selectors includesIdentical: aSymbol) ifTrue:[ ^ categName ]
].
^ nil
%

category: 'Accessing Categories'
method: Behavior
categorysDo: aBlock
  "evaluates aBlock for each category in environment 0.

   aBlock should be a two argument block
   expecting the args  categoryNameSymbol ,   selectorsSet,
   If the package manager is active in the current session,
   aBlock may be invoked more than once for each category name."

  self env: 0 categorysDo: aBlock
%

category: 'Accessing Variables'
method: Behavior
classVarAt: aString

| key |
"Returns the value of the class variable named aString ."

classVars ifNil:[ ^ self _error: #rtErrKeyNotFound args: { aString } ].
key := Symbol _existingWithAll: aString .
key ifNil:[ ^ self _error: #rtErrKeyNotFound args: { aString } ].
^ classVars at: key .
%

category: 'Accessing Variables'
method: Behavior
classVarAt: aString otherwise: defaultValue

| key |
"Returns the value of the class variable named aString ."

classVars ifNil:[ ^ defaultValue ].
key := Symbol _existingWithAll: aString .
key ifNil:[ ^ defaultValue ].
^ classVars at: key otherwise: defaultValue
%

category: 'Accessing Variables'
method: Behavior
classVarNames

"Returns an Array of Symbols naming the class variables defined by this class.
 Inherited class variables are not included; contrast with allClassVarNames."

| cvars |
(cvars := classVars) ifNil:[ ^ { } ].
^ cvars keys asArray
%

category: 'Clustering'
method: Behavior
clusterBehavior
	"This method clusters, in depth-first order, the parts of the receiver required
 for executing GemStone Smalltalk code (the receiver and its method dictionary).
 Returns true if the receiver has already been clustered during the current
 transaction; returns false otherwise.

 Note that methods in the VM's temporary object memory that were
 loaded for execution in this session cannot be clustered by this session.

 It is recommended that when several classes are being clustered in a
 transaction, send clusterBehavior to all classes to be clustered, then send
 clusterDescription."

	self cluster ifTrue: [^true].
	self isMeta
		ifFalse:
			[| arr |
			(arr := instVarNames) ~~ #() ifTrue: [arr cluster]].
	methDicts _isArray ifTrue: [methDicts cluster].
	self persistentMethodDictsDo:
			[:aDict |
			aDict cluster.
			"note GsNmethod>>clusterDepthFirst clusters execution related parts
       to current bucket and other(debug) info info to bucket 5"
			aDict values do: [:meth | meth clusterDepthFirst]].
	self persistentNameSpacesDo: [:aNs | aNs cluster].
	classVars
		ifNotNil:
			[:cvs |
			cvs cluster ifFalse: [cvs associationsDo: [:assoc | assoc cluster]]].
	^false
%

category: 'Clustering'
method: Behavior
clusterBehaviorExceptMethods: aCollectionOfMethodNames

"This method allows you to cluster the receiver more efficiently by omitting
 infrequently-used methods from the clustering.  The methods that you designate
 as aCollectionOfMethodNames will not be clustered with the receiver.  Thus,
 the methods that are frequently used will be packed more densely.  Returns
 true if the receiver has already been clustered during the current
 transaction; returns false otherwise.

 This method works by first clustering all methods named into the max cluster
 bucket, preventing them from showing up in the default cluster bucket.  It
 then uses the standard method to cluster behavior."

| savedBucket otherBucket systm |
systm := System .
savedBucket := systm clusterBucket.
otherBucket := 6 .
systm clusterBucket: otherBucket.
self persistentMethodDictsDo:[ :aDict |
   aDict keysAndValuesDo:[ :aSelector :aMethod |
     (aCollectionOfMethodNames includesValue: aSelector )
       ifTrue: [ aMethod clusterDepthFirst ].
   ].
].
systm clusterBucket: savedBucket.
^ self clusterBehavior
%

category: 'Clustering'
method: Behavior
clusterDescription

"This method clusters, in depth-first order, those instance variables in the
 receiver that are not expected to be accessed heavily during method execution,
 specifically the  poolDictionaries, and categories dictionaries.

 (The receiver itself is not clustered.)  Returns true if the description
 already been clustered during the current transaction; returns
 false otherwise.

 It is recommended that when several classes are being clustered in a
 transaction, send clusterBehavior to all classes to be clustered, then send
 clusterDescription."

| result |
result := false .
self isMeta ifFalse:[
  poolDictionaries ifNotNil:[ poolDictionaries cluster] .
].
result ifFalse:[
  categorys _isArray ifTrue:[
    result := categorys cluster .
    categorys do:[ :aDict | aDict clusterDepthFirst ].
  ] ifFalse:[
    categorys ifNotNil:[ result := categorys clusterDepthFirst ].
  ]
].
^ result
%

category: 'Updating the Method Dictionary'
method: Behavior
compileAccessingMethodsFor: anArrayOfSymbols

"This method create accessing methods for reading and modifying instance and
 other variables in instances of the receiver.  Each element of anArrayOfSymbols
 must be an instance variable, class variable, or pool dictionary variable in
 the receiver.  For each variable 'x' in the Array, two methods are created: 'x'
 reads the variable, in the category 'Accessing', and 'x:newValue' modifies the
 variable, in the category 'Updating'.

 To create class methods to access class variables, class instance variables,
 or pool variables, the message must be sent to the class of the class.

 The new methods have environmentId == 0 .

 Returns the receiver.  Generates an error if any element of anArrayOfSymbols
 is not an instance variable, class variable, or pool variable of the
 receiver."

| allVarNames nvName compileBlk |

self _validatePrivilege ifFalse:[ ^ nil ].
nvName := 'newValue'.
allVarNames := self allInstVarNames collect:[:s | s asString ].
[allVarNames includesValue: nvName ] whileTrue:[ nvName := 'z' , nvName ].
compileBlk := [ :src :cat :varName  |
    [ self compileMethod: src
       dictionaries:  #()
       category: cat
       environmentId: 0
    ] onException: CompileError do:[:ex |
      ^ self _error: #classErrNotAVar args: { varName }
    ]
  ].
anArrayOfSymbols do: [ :var | | getSrc putSrc lf tab |
  lf := Character lf.
  tab := Character tab.
  (getSrc := String new) add: var; add: lf; add: tab; add: '^' ; add: var; add: lf.
  (putSrc := String new) add: var; add: ': ' ; add: nvName; add: lf;
	add: tab; add: var; add: ' := '; add: nvName; add: lf.
  compileBlk value: getSrc value: #Accessing value: var.
  compileBlk value: putSrc value: #Updating  value: var.
]
%

category: 'Accessing the Method Dictionary'
method: Behavior
compiledMethodAt: aSelector
  ^ self compiledMethodAt: aSelector environmentId: 0
%

category: 'Accessing the Method Dictionary'
method: Behavior
compiledMethodAt: aSelector environmentId: envId

"Returns the compiled method associated with the argument aSelector (a String).
 The argument must be a selector in the receiver's method dictionary; if it is
 not, this method generates an error."

| aMeth |
aMeth := self compiledMethodAt: aSelector environmentId: envId otherwise: nil.
aMeth == nil ifTrue:[ self _error: #rtErrKeyNotFound args: { aSelector } ].
^ aMeth
%

category: 'Accessing the Method Dictionary'
method: Behavior
compiledMethodAt: aSelector otherwise: notFoundVal
  ^ self compiledMethodAt: aSelector environmentId: 0 otherwise: notFoundVal
%

category: 'Updating the Method Dictionary'
method: Behavior
compileMethod: sourceString

"If there are no errors, this adds the resulting compiled method to the
 receiver's method dictionary and returns that method,
 otherwise signals a CompileError .
 A CompileWarning may be signaled, after adding the new method
 to a receiver's method dictionary."

^ self compileMethod: sourceString
   dictionaries: GsCurrentSession currentSession symbolList
   category: '(as yet unclassified)'
   environmentId: 0
%

category: 'Updating the Method Dictionary'
method: Behavior
compileMethod: sourceString
dictionaries: aSymbolList
category: aCategoryString

"May be deprecated in the future.
 Returns nil, or an Array of error descriptors"

^ [ self compileMethod: sourceString
       dictionaries: aSymbolList
       category: aCategoryString
       environmentId: 0 .
    nil
  ] onException: CompileError do:[:ex | ex errorDetails ]
%

category: 'Updating the Method Dictionary'
method: Behavior
compileMethod: sourceString
dictionaries: aSymbolList
category: categorySymbol
intoMethodDict: aMethodDict
intoCategories: aCategDict

  ^self
    compileMethod: sourceString
    dictionaries: aSymbolList
    category: categorySymbol
    intoMethodDict: aMethodDict
    intoCategories: aCategDict
    environmentId: 0

%

category: 'Updating the Method Dictionary'
method: Behavior
compileMethod: sourceString
dictionaries: aSymbolList
category: categorySymbol
intoMethodDict: aMethodDict
intoCategories: aCategDict
environmentId: environmentId

"Compiles sourceString as a method for the receiver into the method category
 categorySymbol, using the symbol list aSymbolList. If the compilation succeeds,
 returns the GsNMethod produced by the compilation.
 If the compilation fails, signals a CompileError .
 If the compilation has warnings, signals a CompileWarning, after the GsNMethod
 has been generated and installed in any method dictionaries.

 aMethodDict may be nil, in which case the resulting method is added to the
 receiver's persistent method dictionary and aCategDict is ignored. If
 aMethodDict is not nil it is added to aMethodDict instead of to the receiver's
 method dictionary.  If aMethodDict and aCategDict are both not nil, category
 and selector are added to aCategDict.
 This is used to add methods to per-session dictionaries.

 The caller is responsible for setting the current GsObjectSecurityPolicy to
 match the receiver's GsObjectSecurityPolicy if desired.

 environmentId must be a SmallInteger >= 0 and <= 255 .
 0 denotes the base Smalltalk image.  1 was reserved for use by Ruby .

 You must have code modification privilege to execute this method.
 "

^ self _checkCompileResult:( self _primitiveCompileMethod: sourceString
		symbolList: aSymbolList category: categorySymbol
             oldLitVars: nil intoMethodDict: aMethodDict intoCategories: aCategDict
	     environmentId: environmentId )
        source: sourceString
%

category: 'Updating the Method Dictionary'
method: Behavior
compileMethod: sourceString
dictionaries: aSymbolList
category: categorySymbol
intoMethodDict: aMethodDict
intoCategories: aCategDict
intoPragmas: aPragmasArray
environmentId: environmentId

"pragmasArray is ignored.  use GsNMethod>>pragmas or GsNMethod>>_pragmasArray
 to fetch the pragmas for a method.

 Should eventually be Deprecated."

^ self
  compileMethod: sourceString
  dictionaries: aSymbolList
  category: categorySymbol
  intoMethodDict: aMethodDict
  intoCategories: aCategDict
  environmentId: environmentId
%

category: 'Browser Methods'
method: Behavior
compileMissingAccessingMethods

"Creates accessing and updating methods for all instance variables that do not
 already have such methods.  Sent to a class, creates accessor methods for class
 instance variables that do not have them."

| argName newLine tab allVarNames varNames |

self _validatePrivilege ifTrue:[
  argName := 'newValue' .
  allVarNames := self allInstVarNames collect:[:s | s asString ].
  [allVarNames includesValue: argName] whileTrue: [
    argName := 'z' , argName .
  ].
  newLine := Character lf asString.
  tab := Character tab asString.
  varNames := self instVarNames.
  varNames accompaniedBy: self do: [ :me :var |
    (me includesSelector: var ) ifFalse: [
      me compileMethod: (var , newLine , tab , '^' , var , newLine)
	    dictionaries:  #()
	    category: #Accessing environmentId: 0 .
    ].
    (me includesSelector: var , $: ) ifFalse: [
      me compileMethod: (var , ': ' , argName , newLine , tab , var , ' := ' , argName , newLine)
	    dictionaries:  #()
	    category: #Updating environmentId: 0 .
    ]
  ]
]
%

category: 'Copying'
method: Behavior
copy

"Returns the receiver. Copies of classes and metaclasses are not made."

^ self.
%

category: 'Browsing'
method: Behavior
copyMethodsFrom: sourceClass dictionaries: dicts

"Copies all instance and class methods from the sourceClass.  Returns an Array
 of methods in the source class which failed to compile in this class.  Some of
 them might be class methods.  The Array is empty if no methods failed to
 compile."

| failed srccls targcls envId |
self == sourceClass ifTrue:[
  "because iteration is directly over the source's categories dictionaries"
  ^ self error: 'source of copy must not be self'
].
self _validatePrivilege ifFalse:[ ^ nil ].
failed := { } .

"Copy class and instance methods"
envId := 0 .
1 to: 2 do: [ :i |
  i == 1 ifTrue:[ srccls := sourceClass.  targcls := self ]
        ifFalse:[ srccls := sourceClass class.  targcls := self class ].
  srccls categorysDo:[ :cat :sels |
     sels do: [ :sel | | oldMeth |
        [
          targcls compileMethod: (oldMeth := srccls sourceCodeAt: sel environmentId: envId)
                    dictionaries: dicts category: cat environmentId: envId .
        ] onException: CompileError do:[:ex |
          failed add: oldMeth
        ].
     ].
  ].
].

^failed.
%

category: 'Versions'
method: Behavior
currentVersion
  ^ self
%

category: 'Modifying Classes'
method: Behavior
disallowSubclasses

"Disallows creation of subclasses of a class.  If the receiver is not
 modifiable, this method generates an error.  If the receiver is modifiable and
 already has subclasses, this method generates an error."

self _validatePrivilege ifTrue:[
  self isModifiable ifFalse:[ ^ self validateIsModifiable ].
  self _subclasses size ~~ 0 ifTrue:[ ^ self _error: #rtErrAlreadyHasSubclasses ].
  format := format bitOr: 32
]
%

category: 'Accessing Categories'
method: Behavior
env: environmentId baseCategorysDo: aBlock
  "evaluates aBlock for each method category of receiver
   in specified environment. Returns the receiver.

   aBlock should be a two argument block
   expecting the args  categoryNameSymbol ,   selectorsSet.
   aBlock will be invoked once for each category in the receiver.
   The iteration is directly over the receiver's categories."

  | cats |
  cats := self _baseCategorys: environmentId .
  cats ifNotNil:[ cats keysAndValuesDo: aBlock ].
%

category: 'Accessing Categories'
method: Behavior
env: envId methodsDo: aBlock
  "evaluates aBlock for each method in receiver's method dictionaries
   for specified environment. Returns the receiver.

   aBlock should be a two argument block
   expecting the args  selectorSymbol ,   aGsNMethod ."

  (self persistentMethodDictForEnv: envId ) ifNotNil:[:dict | dict keysAndValuesDo: aBlock ]
%

category: 'Accessing Categories'
method: Behavior
env: envId unifiedCategoriesDo: aBlock
  "evaluates aBlock for each method category of receiver
   in specified environment. Returns the receiver.

   aBlock should be a two argument block
   expecting the args  categoryNameSymbol ,   selectorsSet.
   aBlock will be invoked once for each category in the receiver.
   The iteration is done over a copy of the receiver's categories,
   however the selectorSets are not copied ."

  (self _unifiedCategorys: envId ) keysAndValuesDo: aBlock
%

category: 'Accessing the Class Format'
method: Behavior
firstPublicInstVar

"Returns the index of the first publicly available instance variable storage
 location, whether or not a public instance variable has actually been
 defined."

"Currently, an instance variable is considered to be public if it is to be
 included when passivating the object through PassiveObject.  Being public has
 no relationship to whether or not accessing or updating methods are defined
 for the instance variable."

<primitive: 960>
self _primitiveFailed: #firstPublicInstVar
%

category: 'Accessing the Class Format'
method: Behavior
format

"Returns the value of the format instance variable."

^ format
%

category: 'Accessing the Class Format'
method: Behavior
hasPublicInstVars

"Returns true if the receiver has publicly-visible instance variables."

^ self instSize >= self firstPublicInstVar
%

category: 'Modifying Classes'
method: Behavior
immediateInvariant
	"If the receiver is not invariant, make it invariant and
 conditionally also make the receiver's instVarNames invariant.
 Also clears the subclasses instVar .

 After this method executes,  addInstVar: and removeInstVar: will
 no longer be allowed for the receiver."

	self isInvariant
		ifFalse:
			[self _validatePrivilege ifFalse: [^nil].
			" Gs64 v3.0 and later,
			We no longer recompile methods here, the assumption is that removeInstVar:
			has recompiled methods as needed or has failed and invoked
      		System _disallowCommitClassModFailure."
			self subclassesDisallowed
				ifFalse:
					["only make the instVarNames collection invariant if we
					are not permitting possible dynamic addition of named instance variables."
					instVarNames immediateInvariant].
			"clear the subclasses class instance variable to prevent concurrency problems"
			self _subclasses ~~ nil ifTrue: [self _subclasses: nil].
			super immediateInvariant.	"make self invariant"
			"_codeChangedForEnv: not needed, since no instances can exist yet"
			self _refreshClassCache: false]
%

category: 'Accessing the Class Format'
method: Behavior
implementationFormat

"Returns the three least-significant bits of the receiver's format instance
 variable.  The values of those bits mean the following:

 0   OOP       non-indexable
 1   Byte      non-indexable
 2   NSC       non-indexable
 3   Special   non-indexable
 4   OOP       indexable
 5   Byte      indexable"

^ format bitAnd: 16r7
%

category: 'Accessing Categories'
method: Behavior
includesCategory: aString

"Returns true if aString is equivalent to the name of a category in the
 receiver, false otherwise."

^ self includesCategory: aString environmentId: 0
%

category: 'Accessing Categories'
method: Behavior
includesCategory: aString environmentId: envId

"Returns true if aString is equivalent to the name of a category in the
 receiver, false otherwise."

| aSym |
aSym := Symbol _existingWithAll: aString.
aSym ifNil:[ ^ false ].
self env: envId categorysDo:[ :name :selectors |
   name == aSym ifTrue:[ ^ true ].
].
^ false
%

category: 'Accessing the Method Dictionary'
method: Behavior
includesSelector: aString
  ^ self includesSelector: aString environmentId: 0
%

category: 'Accessing the Class Hierarchy'
method: Behavior
inheritsFrom: aClass

"Returns true if the argument aClass is on the receiver's
 superclass chain; returns false if it isn't."


(aClass isKindOf: Behavior)
ifFalse:
   [self _error: #rtErrBadArgKind args: { Class }].

(self isSubclassOf: aClass)
   ifTrue: [
     self == aClass
       ifTrue: [^false]
       ifFalse: [^true]
   ]
   ifFalse: [^false] .
   self _uncontinuableError " should never get here"
%

category: 'Accessing the Class Format'
method: Behavior
instancesDbTransient

  "Return true if the class format has the DbTransient bit set.
   See also  makeInstancesDbTransient ."

^ (format bitAnd: 16r1000) ~~ 0
%

category: 'Accessing the Class Format'
method: Behavior
instancesInvariant

"Returns true if instances of the receiver may not change value after they have
 been committed to GemStone.  Otherwise, returns false."

^ (format bitAnd: 16r8) ~~ 0
%

category: 'Accessing the Class Format'
method: Behavior
instancesNonPersistent

 "Returns true if instances may not be committed.
  See also makeInstancesNonPersistent ."

^ (format bitAnd: 16r800) ~~ 0
%

category: 'Accessing the Class Format'
method: Behavior
instSize

"Returns the number of named instance variables in the receiver, including all
 inherited instance variables."

^ instVarsInfo bitAnd: Class_numIvs_mask
%

category: 'Updating'
method: Behavior
instVarAt: anIndex put: aValue

anIndex == 7 ifTrue:[
  "Gs64 v3.0 structural update of the methDicts instVar disallowed"
   OffsetError  new _number: 2350; reason: #errNoStructuralUpdate ;
	  details:'methDicts instVar'; object: self ; signal .
   self _uncontinuableError.
].
anIndex == 9 ifTrue:[
  "Gs64 v3.0 structural update of the categories instVar disallowed"
   OffsetError  new _number: 2350; reason: #errNoStructuralUpdate ;
	details:'categorys instVar'; object: self ; signal .
   self _uncontinuableError.
].
^ super instVarAt: anIndex put: aValue
%

category: 'Accessing Variables'
method: Behavior
instVarNames

"Returns an Array of Symbols naming the instance variables defined by the
 receiver, but not including those inherited from superclasses.  Contrast
 with allInstVarNames."

| inheritedInstVars "an Array of the inherited instance variables"
  size              "the size of inherited instance variables"
  myInstVars        "an Array of all of the instance variables"
  supercls     |

(myInstVars := instVarNames) ifNil:[ ^ { } ].
(supercls := self superClass) ifNil: [ ^ myInstVars ].

inheritedInstVars := supercls _instVarNames.
(size := inheritedInstVars size) == myInstVars size ifTrue:[ ^{ } ].

"Assume that each inherited instance variable is added to the end of
 the Array result of allInstVarNames."

^ myInstVars copyFrom: size + 1 to: myInstVars size .
%

category: 'Testing'
method: Behavior
isBehavior

"Returns true if the receiver is a kind of Behavior, and returns false
 otherwise."

  ^true
%

category: 'Accessing the Class Format'
method: Behavior
isBytes

"Returns true if instances of the receiver are byte objects.  Otherwise,
 returns false."

^ (format bitAnd: 16r3) == 1
%

category: 'Accessing the Class Format'
method: Behavior
isBytesOrSpecial

"Returns whether instances of the receiver are byte objects."

^ (format bitAnd: 1) == 1
%

category: 'Accessing the Class Format'
method: Behavior
isIndexable

"Returns true if instances of the receiver have indexed variables.
 Otherwise, returns false."

^ (format bitAnd: 16r4) ~~ 0 "that is, is indexable"
%

category: 'Modifying Classes'
method: Behavior
isModifiable
	"Returns true if the receiver may be modified (that is, if the receiver and its Array of
 instance variable names are both variant, and the receiver has a 'subclasses' class variable).
 Returns false otherwise."

	| ivn |
	self isInvariant ifTrue: [^false].
	((ivn := instVarNames) size ~~ 0 and: [ivn isInvariant]) ifTrue: [^false].
	^self _subclasses ~~ nil
%

category: 'Accessing the Class Format'
method: Behavior
isNonByteVarying

"Returns true if the instances of the receiver are not byte objects and have
 unnamed instance variables; returns false otherwise."

| bits |

bits := format bitAnd: 7.
^bits == 2 or: [bits == 4].
%

category: 'Accessing the Class Format'
method: Behavior
isNsc

"Returns true if instances of the receiver are non-sequenceable
 collections (UnorderedCollections).  Otherwise, returns false."

^ (format bitAnd: 16r3) == 2
%

category: 'Accessing the Class Format'
method: Behavior
isPointers

"Returns true if instances of the receiver are pointer objects.
 Otherwise, returns false."

^ (format bitAnd: 16r3) == 0
%

category: 'Accessing the Class Format'
method: Behavior
isProtected

"Returns true if instances of the receiver may not be accessed structurally
 through GemBuilder for C. "

^ (format bitAnd: 16r80) ~~ 0
%

category: 'Testing Inheritance'
method: Behavior
isSubclassOf: aClassHistory

"Returns true if the receiver is identical to or is a subclass of any class
 in aClassHistory; otherwise, returns false.

 If the aClassHistory argument is actually a class rather than a class history,
 then this method uses the class history of the argument, instead of the class
 itself."

<primitive: 70>
self _primitiveFailed: #isSubclassOf: args: { aClassHistory } .
^ false
%

category: 'Accessing the Class Format'
method: Behavior
isVariable

"Returns true if instances of the receiver have an unnamed part."

self isIndexable ifTrue: [^true].
self isNsc ifTrue: [^true].
^false
%

category: 'Accessing Categories'
method: Behavior
methodsDo: aBlock
  "evaluates aBlock for each method in receiver's method dictionaries
   for environment 0 . Returns the receiver.

   aBlock should be a two argument block
   expecting args  selectorSymbol ,   aGsNMethod ."

  self env: 0 methodsDo: aBlock
%

category: 'Instance Creation'
method: Behavior
migrateNew

"Create a new instance to use in migration.  By default, use #new.
Override in subclasses that can't use #new with #_basicNew. "

^ self new
%

category: 'Private'
method: Behavior
needsRecompileFor30

 "Returns true if the receiver needs to have methods recompiled."

  methDicts _isArray ifTrue:[ ^ false ].
  ^ methDicts valueConstraint ~~ GsNMethod
%

category: 'Private'
method: Behavior
needsRecompileFor33

 "Returns true if the receiver needs to have methods recompiled."

  self needsRecompileFor30 ifTrue:[ ^ true ].
  self env: 0 methodsDo:[ :selector :aMethod |
    aMethod needsRecompile ifTrue:[ ^ true ]
  ].
  ^ false .
%

category: 'Instance Creation'
method: Behavior
new

"Returns an instance of the receiver with no indexed variables."

<primitive: 51>

self _primitiveFailed: #new .
self _uncontinuableError
%

category: 'Instance Creation'
method: Behavior
new: anInteger

"Returns an instance of the receiver with the specified number of indexed
 variables.  Generates an error if the receiver is not indexable or if
 anInteger is not a positive SmallInteger.

 For new byte objects, all indexed variables are set to zero;
 for new pointer objects, all indexed variables are set to nil."

<primitive: 53>

(self isIndexable) ifFalse:[ self _errorNotIndexable. ^ self new ].
(anInteger _isSmallInteger) ifFalse:[
  anInteger _isInteger ifTrue:[
    anInteger _error: #errArgTooLarge args:{ SmallInteger maximumValue }.
    ^ self new
  ].
  anInteger _validateClass: SmallInteger.
  ^ self new
].
(anInteger < 0) ifTrue:[ anInteger _error: #rtErrArgNotPositive. ^ self new].
self _primitiveFailed: #new: args: { anInteger }.
self _uncontinuableError
%

category: 'Accessing Variables'
method: Behavior
offsetOfInstVar: aSymbol

"Returns the integer offset at which the instance variable named aString is
 stored in instances of the receiver.  Returns zero if the instance variable
 is not found."

^ instVarNames indexOfIdentical: aSymbol.
%

category: 'Accessing the Method Dictionary'
method: Behavior
persistentMethodAt: aSelector
  ^ (self persistentMethodAt: aSelector otherwise: nil)
     ifNil:[ self _error: #rtErrKeyNotFound args: { aSelector } ].
%

category: 'Accessing the Method Dictionary'
method: Behavior
persistentMethodAt: aSelector otherwise: notFoundVal
  | sym pmd |
  (sym := Symbol _existingWithAll: aSelector) ifNil:[ ^ notFoundVal ].
  (pmd := self persistentMethodDictForEnv: 0) ifNil:[ ^ notFoundVal ].
  ^ pmd at: sym otherwise: notFoundVal .
%

category: 'Updating'
method: Behavior
persistentMethodDictForEnv: envId
"result will may be nil if no methods exist for specified environmentId."
| mds |
(mds := methDicts) _isArray ifTrue:[
  ^ mds atOrNil: (envId*4 + 1)
].
envId == 0 ifTrue:[ ^ mds ].
^ nil
%

category: 'Updating'
method: Behavior
persistentMethodDictForEnv: envId put: aValue
  "aValue should be a GsMethodDictionary, or nil ,
   caller responsible for _refreshClassCache "

<protected>
| ofs mds |
(mds := methDicts) _isArray ifFalse:[
  envId == 0 ifTrue:[
    methDicts := aValue .
    ^ self
  ].
  mds := { mds }.
  methDicts := mds .
].
ofs := envId*4 + 1 .
mds size < ofs ifTrue:[ mds size: ofs ].
mds at: ofs put: aValue
%

category: 'Enumerating'
method: Behavior
persistentMethodDictsDo: aBlock
| mds |
(mds := methDicts) ifNotNil: [
  mds _isArray ifTrue:[
    1 to: mds size by: 4 do:[:j |  | aDict |
      (aDict := mds at: j) ifNotNil:[ aBlock value: aDict ].
    ].
  ] ifFalse:[
    aBlock value: mds
  ]
]
%

category: 'Enumerating'
method: Behavior
persistentNameSpacesDo: aBlock
| mds |
(mds := methDicts) ~~ nil ifTrue: [
  mds _isArray ifTrue:[
    2 to: mds size by: 4 do:[:j |  | aNs |
      (aNs := mds at: j) ifNotNil:[ aBlock value: aNs ].
    ].
  ]
]
%

category: 'Accessing the Class Format'
method: Behavior
primaryCopy
  "If receiver is a Ruby virtual class, return the class of which it
   is a copy, otherwise return the receiver."

  ^ self "alway self in pure Smalltalk "
%

category: 'Indexing Support'
method: Behavior
rcBtreeLeafNodeClass

"Returns the class of BtreeLeafNode to create for an equality index whose last
 object along the path is an instance of the receiver."

| result |
(RangeEqualityIndex isBasicClass: self)
    ifTrue: [ result := self indexManager rcBtreeBasicLeafNodeClass]
    ifFalse: [ result := self indexManager rcBtreeLeafNodeClass ].
^ result
%

category: 'Repository Conversion'
method: Behavior
recompileAllMethods

"Recompile all environment 0 methods for execution in a Gs64 v3.0 or later system."

| meths md |
meths := { } .
self env: 0 methodsDo:[ :selector :aMethod | meths add: aMethod ].
1 to: meths size do:[:j | (meths at: j) recompile ].
(md := methDicts) class == GsMethodDictionary ifTrue:[
  md valueConstraint == (ObsoleteClasses at: #GsMethod) ifTrue:[
     md size ~~ 0 ifTrue:[ Error signal:'inconsistent methodDict size'].
     md valueConstraint: GsNMethod
  ]
].
%

category: 'Repository Conversion'
method: Behavior
recompileMethodAt: aSelector
  (self compiledMethodAt: aSelector environmentId: 0) recompile
%

category: 'Analysis'
method: Behavior
referencedStrings

"Returns a Set containing all Strings and InvariantStrings referenced by
 the environment 0 methods in this Class and its metaclass."

| set blk |

set := IdentitySet new.
blk := [ :selector :method | method _referencedStringsInto: set ].
self env: 0 methodsDo: blk .
self class env: 0 methodsDo: blk .
^set.
%

category: 'Updating Categories'
method: Behavior
removeCategory: categoryName
  ^ self removeCategory: categoryName environmentId: 0
%

category: 'Modifying Classes'
method: Behavior
removeInstVar: aString
	"Removes the instance variable named aString from the receiver and from all of
 the receiver's subclasses.  The receiver and all of its subclasses must be
 modifiable.

 All instance methods for the receiver and its subclasses are recompiled using
 the symbol list of the current user.  If an error occurs during recompilation
 of methods, the instance variable will have been removed from the receiver and
 from all of its subclasses, but some methods in some subclasses will not have
 been recompiled, and subsquent commit will be disallowed.

 You may not use this method to remove an inherited instance variable."

	| offset aSymbol success |
	self _validatePrivilege ifFalse: [^nil].

	"validate that the instance variable exists"
	aSymbol := Symbol _existingWithAll: aString.
	aSymbol ifNotNil: [offset := self _ivOffsetOf: aSymbol].
	offset ifNil: [^self _error: #classErrNotAVar args: {aString}].
	(self superClass _ivOffsetOf: aSymbol) == nil
		ifFalse: [^self _error: #classErrRemoveInherIv args: {aSymbol}].

	"validate that self and all subclasses are modifiable"
	self validateSubclassesAreModifiable.
	success := false.

	[self _removeInstVarAtOffset: offset.	"remove from self and all subclasses"
	self recompileAllSubclassMethodsInContext: GsCurrentSession currentSession
				symbolList.
	success := true]
			ensure: [success ifFalse: [System _disallowCommitClassModFailure]]
%

category: 'Updating the Method Dictionary'
method: Behavior
removeSelector: aString

  self removeSelector: aString environmentId: 0
%

category: 'Browsing'
method: Behavior
removeSelector: aString environmentId: envId ifAbsent: aBlock
"Removes the method whose selector is aString from the receiver's
 method dictionary.  If the selector is not in the method
 dictionary, returns the result of evaluating the
 zero-argument block aBlock.  Otherwise, returns the receiver."

| aKey meth |

self _validatePrivilege ifFalse:[ ^ nil ].
aKey := Symbol _existingWithAll: aString .
aKey ifNotNil:[
  meth := self compiledMethodAt: aKey environmentId: envId otherwise: nil .
].
meth ifNil:[ ^ aBlock value ].
self removeSelector: aKey environmentId: envId
%

category: 'Browsing'
method: Behavior
removeSelector: aString ifAbsent: aBlock
  ^ self removeSelector: aString environmentId: 0 ifAbsent: aBlock
%

category: 'Accessing Variables'
method: Behavior
scopeHas: aVariableName
ifTrue: aBlock

"If aVariableName (a String) is specified as a variable in the receiver or one
 of its superclasses, this evaluates the zero-argument block aBlock and returns
 the result of evaluating aBlock.  Otherwise, returns false."

| allSharedPools |

(aVariableName isKindOf: String)
ifFalse:
   [ ^ aVariableName _error: #rtErrBadArgKind args: { String }].

( ((self allInstVarNames ) includesValue: aVariableName) or:
 [((self allClassVarNames) includesValue: aVariableName)])
ifTrue:
   [^ aBlock value]
ifFalse: "now check sharedPools"
[
   allSharedPools:= self allSharedPools.
   allSharedPools do: [:poolDict |
                         (poolDict includesKey: aVariableName)
                         ifTrue:
                            [ ^ aBlock value]
                      ]
].
^ false
%

category: 'Accessing the Method Dictionary'
method: Behavior
selectors
^ self selectorsForEnvironment: 0
%

category: 'Accessing Categories'
method: Behavior
selectorsIn: categoryName
  ^ self selectorsIn: categoryName environmentId: 0
%

category: 'Accessing Categories'
method: Behavior
selectorsIn: categoryName environmentId: envId

"Returns an Array of all selectors in the specified category.  If categoryName
 is not in the receiver's method dictionary, generates an error."

| set count catSym |
catSym := Symbol _existingWithAll: categoryName .
catSym ifNil:[ ^ self _categoryNotFound:  categoryName ].
count := 0 .
set := IdentitySet new .
self env: envId categorysDo:[ :categName :selectors |
  categName == catSym ifTrue:[ set addAll: selectors . count := count + 1 ].
].
count == 0 ifTrue:[ ^ self _categoryNotFound:  categoryName ].
^ Array withAll: set
%

category: 'Accessing the Class Format'
method: Behavior
selfCanBeSpecial

"Return true if the class format has the SelfCanBeSpecial bit set.

Any class for which self can be a special object must be created
using the value  #selfCanBeSpecial  as an element of the argument
to the options: keyword of the subclass creation method .
If instances of any of
   Boolean Character SmallDate SmallDateAndTime SmallDouble
   SmallFraction SmallInteger SmallScaledDecimal SmallTime UndefinedObject
will be able to inherit methods from a class, that class
must be created with  #selfCanBeSpecial .

Example, the Pharo class ProtoObject needs #selfCanBeSpecial  .

#selfCanBeSpecial is not inherited from a superclass by subclass
creation 

#selfCanBeSpecial be present in a superclass to be able to specify it for a subclass."

^ (format bitAnd: 16r2000) ~~ 0
%

category: 'Accessing Variables'
method: Behavior
sharedPools

"Returns an Array of pool dictionaries used by this class.  Superclasses
 are not included; contrast with allSharedPools."

| pd |
^ (pd := poolDictionaries) ifNil:[ { } ]
		      ifNotNil:[ Array withAll: pd ]
%

category: 'Accessing Categories'
method: Behavior
sortedCategoryNames

"Returns an Array of Symbols.  The elements of the collection are the
 receiver's category names (excluding names inherited from superclasses)."

 | set |
 set := IdentitySet new .
 self env: 0 categorysDo:[ :categName :selectors | set add: categName ].
 ^ Array withAll: (SortedCollection withAll: set ) .
%

category: 'Accessing Categories'
method: Behavior
sortedSelectorsIn: categoryName
  ^ self sortedSelectorsIn: categoryName environmentId: 0
%

category: 'Accessing Categories'
method: Behavior
sortedSelectorsIn: categoryName environmentId: envId

"Returns an Array of all selectors in the specified category, sorted
 in ascending order."

 ^ Array withAll:( SortedCollection withAll:(
        self selectorsIn: categoryName environmentId: envId) )
%

category: 'Indexing Support'
method: Behavior
sortNodeClass

"Returns the class of SortNode to create for sorting on instances of the
 receiver."

| result |
(RangeEqualityIndex isBasicClass: self)
    ifTrue: [ result := self indexManager sortNodeClass ]
    ifFalse: [ result := SortNode ].
^ result
%

category: 'Accessing the Method Dictionary'
method: Behavior
sourceCodeAt: aSelector

^ self sourceCodeAt: aSelector environmentId: 0
%

category: 'Accessing the Method Dictionary'
method: Behavior
sourceCodeAt: aSelector environmentId: envId

"Returns a String representing the source code for the argument, aSelector.  If
 aSelector (a String) is not a selector in the receiver's method dictionary,
 this generates an error."

^ (self compiledMethodAt: aSelector environmentId: envId) sourceString
%

category: 'Instance Creation'
method: Behavior
squeakBasicNew: anInteger

"Returns with an instance of the receiver.
 If the result is oop format, anInteger specifies the number of varying instVars.
 For an bytes format result, anInteger specifies the number of words
 where (self _bytesPerWord) is the number of bytes per word.

 Signals an error if the Behavior is not indexable or if anInteger is bad."

<primitive: 1082>
| isz |
(self _isInstanceDisallowed) ifTrue: [
   self _error: #objErrCantCreateInstance args:  #()  .
   self _primitiveFailed: #_basicNew: args: { anInteger }.
   self _uncontinuableError
].
(self isIndexable) ifFalse:[self _errorNotIndexable .  ^ self _basicNew ].
anInteger _isSmallInteger ifFalse:[
  anInteger _validateClass: SmallInteger . ^ self _basicNew
].
anInteger < 0 ifTrue:[
  anInteger _error: #rtErrArgNotPositive. ^ self _basicNew
].
(anInteger + (isz := self instSize)) _isSmallInteger ifFalse: [
  anInteger _error: #errArgTooLarge args:{ SmallInteger maximumValue - isz} .
  ^ self _basicNew
].
self _primitiveFailed: #_basicNew: args: { anInteger }.
self _uncontinuableError
%

category: 'Accessing the Class Format'
method: Behavior
subclassesDisallowed

"Returns true if subclasses of the receiver have been disallowed by means of
 Behavior | disallowSubclasses.  Otherwise, returns false."

^ (format bitAnd: 16r20) ~~ 0
%

category: 'Accessing the Class Hierarchy'
method: Behavior
superClass

"Returns the receiver's superclass."

 ^ superClass
%

category: 'Accessing the Class Hierarchy'
method: Behavior
superclass
 "for squeak compatibility"
 ^ self superClass
%

category: 'Accessing the Class Hierarchy'
method: Behavior
superclassForEnv: envId

 "Return receiver's superclass in environment zero .
  Reimplemented in Module to handle envId ~~ 0 ."

  ^ superClass
%

category: 'Updating the Class Hierarchy'
method: Behavior
superclassForEnv: envId put: aCls
  "Update the receiver's persistent superclass for method lookup
   environment specified by envId.
   envId must be a SmallInteger > 0 .
   aClass must be a Behavior.

   It is necessary to execute
     Behavior _clearLookupCaches: envId ; _clearLookupCaches: 0 .
   after executing this method if you want the change in hierarchy
   to take effect immediately for all method lookups.
"
  <primitive: 2001>
  | prot |
  prot := System _protectedMode .
  [ | mds ofs |
    aCls isBehavior ifFalse:[ ^ ArgumentError signal:'expected a Class' ] .
    (self selfCanBeSpecial and:[ aCls selfCanBeSpecial not]) ifTrue:[ "fix 48261"
      ^ ArgumentError signal:'receiver has selfCanBeSpecial and argument does not'.
    ].
    envId > 0 ifFalse:[ ^ ArgumentError signal:'expected envId > 0' ].
    (mds := methDicts) _isArray ifFalse:[
      self persistentMethodDictForEnv: envId put: nil .
      mds := methDicts .
      ].
    ofs := envId*4 + 3 .
    mds size < ofs ifTrue:[ mds size: ofs ].
    mds at: ofs put: aCls
  ] ensure:[
    prot _leaveProtectedMode
  ].
%

category: 'Modifying Classes'
method: Behavior
validateIsModifiable

"Returns the receiver if the receiver and its
 Array of instance variables are modifiable.  Generates an error if the
 receiver cannot be modified (that is, if the receiver
 or its Array of instance variable names is not variant)."

self isModifiable ifFalse:[ ^ self _error: #rtErrClassNotModifiable ]
%

category: 'Modifying Classes'
method: Behavior
validateSubclassesAreModifiable

"Generates an error if the receiver or any of its subclasses cannot be
 modified."

self validateIsModifiable .
self _subclasses do:[:x| x validateSubclassesAreModifiable ].
^ self
%

category: 'Testing Inheritance'
method: Behavior
validateSubclassOf: aClass

"Returns true if receiver is identical to aClass or is a subclass
 of aClass; otherwise, generates an error."

( self isSubclassOf: aClass) ifFalse:[
     ^ self _error: #rtErrNotASubclassOf args:{ aClass } ].
^ true
%

category: 'Accessing the Method Dictionary'
method: Behavior
whichClassIncludesSelector: aString
  ^ self whichClassIncludesSelector: aString environmentId: 0
%

category: 'Modifying Classes'
method: Behavior
_addInvariantClassVar: aSymbol value: aVal

"Adds the class variable with name aSymbol to the receiver's
 class variables dictionary, and make the resulting SymbolAssocation
 invariant."
| assoc dict |
self _validatePrivilege ifTrue:[
  (dict := classVars) ifNil:[
    dict := self _createClassVarsDict .
  ].
  assoc := dict associationAt: aSymbol otherwise: nil .
  assoc ifNil:[
    assoc := SymbolAssociation new key: aSymbol value: aVal .
    assoc objectSecurityPolicy: self objectSecurityPolicy .
    dict addAssociation: assoc .
    assoc immediateInvariant
  ] ifNotNil:[
    assoc _value = aVal ifFalse:[
      assoc _value: aVal .
      assoc immediateInvariant
    ]
  ].
]
%

category: 'Private'
method: Behavior
_allStrippedMethodSelectors

"Returns an Array of two Arrays, signifying that all methods of the receiver
 are to have their source stripped."

  ^ { self selectors . self class selectors }.
%

category: 'Jade browser support'
method: Behavior
_allSuperList

 ^ self _allSuperList: 0
%

category: 'Accessing the Class Hierarchy'
method: Behavior
_allSuperList: envId

"Returns an Array of the superclasses of the receiver, beginning
 with the most remote superclass, and excluding the receiver itself."

  | result currClass |
  result:= { } .
  currClass := self .
  envId ~~ 0 ifTrue:[
    [ true] whileTrue:[
      currClass:= currClass superclassForEnv: envId .
      currClass == nil ifTrue: [^ result].
      result insertObject: currClass at: 1.
    ].
  ] ifFalse:[
    [ true] whileTrue:[
      currClass:= currClass superClass.
      currClass == nil ifTrue: [^ result].
      result insertObject: currClass at: 1.
    ].
  ]
%

category: 'Private'
method: Behavior
_allSuperList: includeRubyVirtual env: envId
  ^ self _allSuperList: envId 
%

category: 'Enumerating'
method: Behavior
_baseCategorys: envId

  "Returns nil or the category dictionary for specified environment"
  | cats |
  (cats := categorys) _isArray ifTrue:[
    cats := cats atOrNil:  envId + 1 .
  ] ifFalse:[
    envId ~~ 0 ifTrue:[ cats := nil ].
  ].
  ^ cats
%

category: 'Accessing Categories'
method: Behavior
_baseCategorysForStore: envId
  | dict carr ofs |
  (dict := self _baseCategorys: envId ) ifNil:[
     dict := GsMethodDictionary new  .
     dict objectSecurityPolicy: self objectSecurityPolicy .
     ofs := envId + 1 .
     (carr := categorys) _isArray ifTrue:[
       (carr size < ofs) ifTrue:[ carr size: ofs ].
        carr at: ofs put: dict .
     ] ifFalse:[
       envId > 0 ifTrue:[
         carr := Array new: ofs .
         carr at: 1 put: categorys .
         carr at: ofs put: dict .
         categorys := carr .
       ] ifFalse:[
         categorys := dict
       ].
     ].
  ].
  ^ dict
%

category: 'Instance Creation'
method: Behavior
_basicNew

"Returns an instance of the receiver, with no indexed
 variables.  Do not override this method; contrast with Behavior | new."

<primitive: 50>

(self _isInstanceDisallowed)
ifTrue:[ self _error: #objErrCantCreateInstance args:  #()  .
         self _uncontinuableError
       ].
self _primitiveFailed: #_basicNew .
self _uncontinuableError
%

category: 'Instance Creation'
method: Behavior
_basicNew: anInteger

"Returns with an instance of the receiver, with the given
 number of fields.  Generates an error if the Behavior is not indexable or if
 anInteger is bad.  Do not override this method; contrast with Behavior | new:."

<primitive: 52>
| isz |
(self _isInstanceDisallowed) ifTrue: [
   self _error: #objErrCantCreateInstance args:  #()  .
   self _primitiveFailed: #_basicNew: args: { anInteger }.
   self _uncontinuableError
].
(self isIndexable) ifFalse:[self _errorNotIndexable .  ^ self _basicNew ].
anInteger _isSmallInteger ifFalse:[
  anInteger _validateClass: SmallInteger . ^ self _basicNew
].
anInteger < 0 ifTrue:[
  anInteger _error: #rtErrArgNotPositive. ^ self _basicNew
].
(anInteger + (isz := self instSize)) _isSmallInteger ifFalse: [
  anInteger _error: #errArgTooLarge args:{ SmallInteger maximumValue - isz} .
  ^ self _basicNew
].
self _primitiveFailed: #_basicNew: args: { anInteger }.
self _uncontinuableError
%

category: 'Browser Methods'
method: Behavior
_categoriesReport

^ self _categoriesReportEnv: 0
%

category: 'Browser Methods'
method: Behavior
_categoriesReportEnv: envId

"Returns an Array containing key-value pairs from the receiver's categories
 for specified environment.
 The key in each key-value pair is the name of a category; the value in each
 key-value pair is a sorted Array of selectors.

 Used by the Topaz 'list categories' command."

| result k sz sortedCats |
sortedCats := SortedCollection new:[ :a :b | a key <= b key ].
self env: envId unifiedCategoriesDo:[ :categName :selectors |
  sortedCats add: ( Association newWithKey: categName value: selectors)
].
result := Array new: (sz := sortedCats size) * 2  .
k := 1 .
1 to: sz do:[:j | | anAssoc |
   anAssoc := sortedCats at: j .
   result at: k put: anAssoc key .
   result at: k + 1 put: (Array withAll:(SortedCollection withAll: anAssoc _value)).
   k := k + 2 .
].
^ result .
%

category: 'Accessing Categories'
method: Behavior
_categoryNotFound: aString
  self _error: #classErrMethCatNotFound args: { aString } .
  ^ nil
%

category: 'Browsing'
method: Behavior
_categoryOfSelector: selector

"Returns the category of the given selector, or 'unknown' if it isn't found."

 | result |
 (result := self categoryOfSelector: selector) == nil ifTrue:[
    ^ 'unknown'
    ].
 ^ result
%

category: 'Private'
method: Behavior
_checkCompileResult: result source: sourceString

  ^self _checkCompileResult: result source: sourceString suppressCompileWarning: false
%

category: 'Private'
method: Behavior
_checkCompileResult: result source: sourceString suppressCompileWarning: suppressCompileWarning
  "process the result Array from _primitiveCompileMethod:...
   Returns a GsNMethod or signals a CompileError or CompileWarning."

  result class == GsNMethod ifTrue:[
    ^ result
  ].
  (result at: 2) ifNotNil:[ :errorArray|
    "Fill in the error message text for each error in the result."
    [ | errDict |
      errDict := GemStoneError at: System myUserProfile nativeLanguage .
      1 to: errorArray size do:[:j | | thisErr |
	thisErr := errorArray at: j .
	(thisErr atOrNil: 3) ifNil:[  | msg |
	  thisErr size < 3 ifTrue:[ thisErr size: 3 ].
	  msg := errDict atOrNil:(thisErr at: 1) .
	  thisErr at: 3 put: ( msg ifNil:[ '(unknown error number)' ]).
	].
      ].
    ] onException: Error do:[:ex | "ignore"].
    ^ (CompileError new) args: { errorArray . sourceString . self } ; signal
  ].
  (result at: 3) ifNotNil:[ :warnStr | | meth |
    (meth := result at: 1) class == GsNMethod ifTrue:[
       suppressCompileWarning ifFalse: [ CompileWarning signal: warnStr method: meth ].
       ^ meth
    ].
  ].
  InternalError signal:'unrecognized result from _primitiveCompileMethod'
%

category: 'Private'
method: Behavior
_classCategory

"Private."

"The classCategory instance variable is defined in Class, so return nil here."

^ nil
%

category: 'Accessing Variables'
method: Behavior
_classVars

"Returns the classVars instance variable."

^ classVars
%

category: 'Private'
method: Behavior
_clearConstraints
 "Used in repository upgrade only.
 As of GemStone 64bit v3.4, constraints are no longer implemented."

 constraints ifNotNil:[ constraints := nil ]
%

category: 'Virtual Machine Control'
method: Behavior
_clearLookupCaches: envId

"Invalidates method lookup caches for all classes.
 Invalidates all send-site caches for the specified environment.

 envId must be a SmallInteger >= 0 and <= 255 .
"
<primitive: 165>
envId _validateClass: SmallInteger .
self _primitiveFailed:#_clearLookupCaches: args: { envId }
%

category: 'Updating the Method Dictionary'
method: Behavior
_codeChangedForEnv: envId
  "set bit in VM's cbCodeChanged word so it can propagate to
   other VMs upon commit.  Code which adds or removes entries
   in persistent method dictionaries must send this method. 
   envId >= 0   methods changed for specified environment .
   envId == -1  sessions methods changed .
  "
<primitive: 854>
envId _validateClass: SmallInteger .
self _primitiveFailed: #_codeChangedForEnv: args: {envId} "envId out of range"
%

category: 'Private'
method: Behavior
_compileMethod: sourceString
symbolList: aSymbolList
"Compile sourceString as an instance method in the receiver without installing
 in any dictionaries.
 Returns a GsNMethod or signals an Error ."

^ self _compileMethod: sourceString symbolList: aSymbolList environmentId: 0
%

category: 'Private'
method: Behavior
_compileMethod: sourceString
symbolList: aSymbolList
environmentId: environmentId
"Compile sourceString as an instance method in the receiver without installing
 in any dictionaries.
 Returns a GsNMethod or signals an Error ."

| res |
res := self _primCompileMethod: sourceString
   symbolList: aSymbolList
   category: nil
   oldLitVars: nil
   intoMethodDict: false
   intoCategories: nil
   environmentId: environmentId .
^ self _checkCompileResult: res source: sourceString
%

category: 'Updating the Method Dictionary'
method: Behavior
_compileMethodTrappingErrors: sourceString
dictionaries: aSymbolList
category: aCategoryString
environmentId: envId

"If there are no errors, add the resulting compiled method to the receiver's
 method dictionary and returns nil.

 If errors occur, returns an Array of pairs.  The first element of each pair is
 the GemStone error number, and the second element is the offset into the
 sourceString where the error occurred.

 This method differs from compileMethod:dictionaries:category:environmentId:
 in that it traps all errors (not just compiler errors).
 Non-compiler errors are reported with a source offset of 0."

  self _validatePrivilege ifFalse:[ ^ nil ].
  ^ [ self compileMethod: sourceString dictionaries: aSymbolList
           category: aCategoryString environmentId: envId  .
      nil
    ] onException: { CompileError , Error } do:
      { [:ex | { ex gsArguments at: 1 }  "a CompileError" ] .
        [:ex | { { ex number . 0 } }  "any other Error" ]
      }

%

category: 'Deprecated'
method: Behavior
_constraintAt: offset
 "As of GemStone 64bit v3.4, constraints are no longer implemented.
 This method is provided for examining classes in repositories upgraded from
 an older version."

  constraints _isArray ifFalse:[ ^ Object].
  ^ (constraints atOrNil: offset) ifNil:[ Object ]
                 ifNotNil:[ :cls | cls currentVersion ]
%

category: 'Deprecated'
method: Behavior
_constraints
	"Returns nil or an Array.

 As of GemStone 64bit v3.4, constraints are no longer implemented.
 This method is provided for examining classes in repositories upgraded from
 an older version."

	self
		deprecated: 'Behavior>>_constraints  deprecated, Constraints are no longer supported'.
	^constraints
%

category: 'Browsing'
method: Behavior
_copyMethodsAndVariablesFrom: sourceClass except: except dictionaries: dicts

"Copies all instance and class methods, pool dictionaries, and references
 to class variables from the given class to ourselves.  Returns an Array of
 methods in the source class which failed to compile.  Some of them
 might be class methods.  The Array will be empty if none failed.

 Method environmentIds are copied from the source methods.

 The except: argument is a list of categories and/or methods that should
 not be copied.  Each exception is two elements in the Array: a Character
 followed by a String or Symbol.
    $C a category of class methods follows
    $c a category of instance methods follows
    $S the selector of a class method follows
    $s the selector of an instance method follows
    $V a list of class variable names follows
    $P a list of pool dictionaries follows"

| failed srccls targcls sel doit pds otherCvs excludeCvs envId |

self _validatePrivilege ifFalse:[ ^ nil ].
self == sourceClass ifTrue:[
  "because iteration is directly over the source's categories dictionaries"
  ^ self error: 'source of copy must not be self'
].
failed := { } .
pds :=  #() .
excludeCvs :=  #() .

except ifNotNil: [ | i |
  i := except indexOf: $P.
  i ~~ 0 ifTrue: [ pds := except at: i+1 ].
  i := except indexOf: $V.
  i ~~ 0 ifTrue: [ excludeCvs := except at: i+1 ].
  ].

"Copy pool dictionaries"
sourceClass _poolDictionaries do: [ :dict | | poolDicts |
  poolDicts := poolDictionaries .
  (poolDicts ~~ nil and:[ poolDicts includesIdentical: dict]) ifFalse: [
    (pds includesIdentical: dict) ifFalse: [
      poolDicts ifNil:[ poolDicts := { } . poolDictionaries := poolDicts ]
          ifNotNil:[ poolDicts isInvariant ifTrue:[
                       poolDicts := Array withAll: poolDicts . poolDictionaries := poolDicts
                     ]].
      poolDicts add: dict
    ].
  ].
].

"Copy class variables"
otherCvs := sourceClass _classVars .
otherCvs ifNotNil:[ | destCvs |
   destCvs := classVars .
   otherCvs associationsDo: [ :assn | | other |
    destCvs ifNotNil:[ other := destCvs associationAt: assn key otherwise: nil ].
    (other == nil or: [other value == nil and: [assn value ~~ nil]]) ifTrue: [
      (excludeCvs includesValue: assn key) ifFalse:[
        destCvs ifNil:[ destCvs := self _createClassVarsDict ].
        destCvs add: assn
      ].
    ].
  ].
].

"Copy class and instance methods"
envId := 0 .  "change for Maglev"
1 to: 2 do: [ :j |
  j == 1 ifTrue:[ srccls := sourceClass.  targcls := self ]
        ifFalse:[ srccls := sourceClass class.  targcls := self class ].
  srccls categorysDo:[ :cat :sels |
      1 to: sels size do: [ :s | | oldMeth |
	sel := sels at: s.
	doit := true.
	1 to: except size by: 2 do: [ :i | | ch exCat |
          ch := except at: i .
          exCat := except at: i + 1 .
	  (((( ch == $C and: [targcls isMeta and: [cat == exCat ]]) or:
	    [ ch == $S and: [targcls isMeta and: [sel == exCat ]]]) or:
	    [ ch == $c and: [targcls isMeta not and: [cat ==  exCat  ]]]) or:
	    [ ch == $s and: [targcls isMeta not and: [sel ==  exCat ]]]) ifTrue: [
	      doit := false
	  ].
	].
	doit ifTrue: [ | methEnvId |
          oldMeth := srccls compiledMethodAt: sel environmentId: envId .
	  methEnvId := oldMeth environmentId .
          methEnvId == envId ifFalse:[ self error:'environmentId mismatch'].
	  ( targcls
	     _compileMethodTrappingErrors: oldMeth sourceString
	     dictionaries: dicts category: cat environmentId: envId ) ifNotNil:[
	    failed add: oldMeth
          ].
        ].
      ].
  ].
].

^failed.
%

category: 'Modifying Classes'
method: Behavior
_createClassVarsDict
  | dict |
  (dict := classVars) ifNil:[
    (dict := SymbolDictionary new ) objectSecurityPolicy: self objectSecurityPolicy .
    self _setClassVars: dict old: nil .
  ].
  ^ dict
%

category: 'Private'
method: Behavior
_deepCopyWith: copiedObjDict

"Private. Used internally to implement deepCopy."

^ self.
%

category: 'Private Methods for Class Modification'
method: Behavior
_disallowGciCreateStore

"Private.

 Sets bit in the format instance variable to cause GemBuilder for C instance
 creation and updates to go through message sends.  Semantics are private to
 GemBuilder for Smalltalk."

self _validatePrivilege ifTrue:[
  format := format bitOr: 16r200 "no update through structural access" .
  self _refreshClassCache: false
]
%

category: 'Private'
method: Behavior
_gciCompileMethod: sourceString dictionaries: aSymbolList category: aCategoryString environmentId: envId
	"used by GCI implementation , when session methods are enabled.
   Returns nil for successful compilation, a warning String,
   or signals a CompileError"

  | warnStr |
  [
    self compileMethod: sourceString dictionaries: aSymbolList
          category: aCategoryString environmentId: envId .
  ] onException: CompileWarning do: [ :ex | "handle CompileWarning"
    warnStr := ex warningString .
    ex resume .
  ].
  ^ warnStr
%

category: 'Stripping Sources'
method: Behavior
_hideSourceCode

"For each environment 0 method defined for this class, hide the source code.  All that
 remains of the source is the method signature and the initial comment if one
 exists."

self _validatePrivilege ifFalse:[ ^ nil ].
self env: 0 methodsDo:[ :selector :method | method _removeAllSourceButFirstComment ]
%

category: 'Private Methods for Class Modification'
method: Behavior
_incrementInstVars: delta

"Increment instVars by 1 to account for adding a named instance variable."
| mask n info_old |
mask := Class_numIvs_mask .
info_old := instVarsInfo .
delta _isSmallInteger ifFalse:[ delta _validateClass: SmallInteger ].
n := (info_old bitAnd: mask) + delta .
n < 0 ifTrue:[ self error:'numInstVars would go negative'].
n > 2030"virtual machine constant GEN_MAX_INSTANCE_VARS" ifTrue:[
   ArgumentError signal:'number of instance variables  would exceed 2030' .
].
instVarsInfo := (info_old bitAnd:( mask bitInvert)) bitOr:( n bitAnd: mask)
%

category: 'Private Methods for Class Modification'
method: Behavior
_insertNamedInstVar: aSymbol atOffset: offset
	"Receiver and all subclasses must have be modifiable.  aSymbol must be unique
 with respect to existing instance variables."
	" add instance variable to self"

	| mySubclasses ivn |
	self _validatePrivilege
		ifTrue:
			[self _incrementInstVars: 1.
			(ivn := instVarNames) == #()
				ifTrue:
					[ivn := {}.
					instVarNames := ivn].
			ivn insertObject: aSymbol at: offset.
			self _refreshClassCache: false.
			mySubclasses := self _subclasses.
			mySubclasses ~~ nil
				ifTrue:
					[mySubclasses do: [:x | x _insertNamedInstVar: aSymbol atOffset: offset]]].
	^self
%

category: 'Accessing the Class Format'
method: Behavior
_instancesNpDbtransient

  ^ (format bitAnd: 16r1800)  "used by _become:fullChecks:  methods"
%

category: 'Accessing Variables'
method: Behavior
_instVarNames

"Returns the receiver's instance variables list.  Contrast with the public
 method #allInstVarNames."

^ instVarNames
%

category: 'Private'
method: Behavior
_instVarsEqual: anArrayOfInstvarNames

  "Return true if the argument matches the instVarNames
   defined by the receiver (excluding inherited instVars), false otherwise."

  ^ (Array withAll: self instVarNames) = (anArrayOfInstvarNames collect:[:n | n asSymbol ])
%

category: 'Private'
method: Behavior
_instVarsInfo
  ^ instVarsInfo
%

category: 'Error Handling'
method: Behavior
_isInstanceDisallowed

^ (InstancesDisallowed includesValue: self) or:[ self isMeta ]
%

category: 'Private'
method: Behavior
_isKernel

"Private.  Returns true if the given class is a GemStone kernel class."

<primitive: 480>
self _primitiveFailed: #_isKernel
%

category: 'Accessing the Class Format'
method: Behavior
_isSpecial

"Returns true if instances of the receiver are special objects.
 Otherwise, returns false."

^ (format bitAnd: 16r3) == 3
%

category: 'Indexing Support'
method: Behavior
_ivOffsetOf: aSymbol

"Searches the instVarNames instance variable of the receiver for an instance
 variable named aSymbol, and returns the offset for that instance variable.
 Returns nil if no instance variable exists with the name aSymbol."

| idx |
idx := instVarNames indexOfIdentical: aSymbol .
^ idx == 0 ifTrue:[ nil ] ifFalse:[ idx ].
%

category: 'Private Methods for Class Modification'
method: Behavior
_makeProtected

"Make the receiver a protected class by setting a bit in its format.  This
 protection disallows structural access through GemBuilder for C."

self _validatePrivilege ifTrue:[
  format := format bitOr: 128
]
%

category: 'Private'
method: Behavior
_makeTraversalByCallback

"Private.

 Make the receiver place its instances in a traversal buffer by
 invoking the clampSpecification's traversal callback method."

self _validatePrivilege ifTrue:[
  format := format bitOr: 16r400. "travByCallback"
  self _refreshClassCache: false .
]
%

category: 'Accessing the Method Dictionary'
method: Behavior
_methodWithSource: aString

"Returns the environment 0 method from the receivers' method dictionary
 whose source string is equal to aString , or nil if not found."

self env: 0 methodsDo:[ :aSelector :aMethod |
   aMethod sourceString = aString ifTrue:[ ^ aMethod ].
].
^ nil
%

category: 'Private'
method: Behavior
_moveMethod: aSelector toCategory: categoryName
  ^ self moveMethod: aSelector toCategory: categoryName environmentId:0
%

category: 'Accessing the Class Format'
method: Behavior
_name
  ^ 'aBehavior'
%

category: 'Deprecated'
method: Behavior
_namedIvConstraintAt: anInteger
  "Returns the constraint at the specified offset, or Object.

 As of GemStone 64bit v3.4, constraints are no longer implemented.
 This method is provided for examining classes in repositories upgraded from
 an older version."

  (anInteger > self instSize ) ifTrue:[ ^ Object].
  ^ self _constraintAt: anInteger
%

category: 'Private'
method: Behavior
_nameForFileout
  "used by topaz"

^ self theNonMetaClass name
%

category: 'Private'
method: Behavior
_noStrippedMethodSelectors

"Returns an Array of two empty Arrays, signifying that no methods are to have
 their source stripped."

  ^ #( #() #() ).
%

category: 'Enumerating'
method: Behavior
_persistentMethodDicts
  "Returns an Array of the persistent method dictionaries for all environments"

 methDicts
   ifNil:[ ^ #() ]
   ifNotNil:[ :mds |
     mds _isArray ifTrue:[ | arr |
       arr := { } .
       1 to: mds size by: 4 do:[:j |
         (mds at: j) ifNotNil:[:aDict | arr add: aDict ]
       ].
       ^ arr
     ] ifFalse:[
      ^ { mds }
     ]
   ].
%

category: 'Browser Methods'
method: Behavior
_poolDictionaries

"Returns the object containing this instance's pool dictionaries."

^  poolDictionaries ifNil:[ #() ]
%

category: 'Private'
method: Behavior
_primCompileMethod: sourceString
symbolList: aSymbolList
category: categorySymbol
oldLitVars: litVarArray
intoMethodDict: aMethodDict
intoCategories: aCategDict
environmentId: environmentId

"Compiles sourceString as a method for the receiver in category categorySymbol,
 using the symbol list aSymbolList.  If the compilation succeeds, the method
 dictionary of the receiver will have been updated.

 Returns the GsNMethod produced by the compilation if the compilation
 succeeded with no warnings or errors, or an Array of of the form
    { (GsNMethod or nil if the compilation has errors) .
       (nil or an Array of error descriptors as described for
        compileMethod:dictionaries:category: ) .
       (nil or a String describing warnings)
     } .

 If litVarArray is not nil, it must be an Array of Symbol,SymbolAssociation pairs
 and this Array will be searched prior to searching aSymbolList to
 resolve literal variables within the method.

 If the compilation succeeds,
   if aMethodDict == false, the method is added to no dictionaries,
      and aCategDict is ignored
   else if aMethodDict ~~ nil, the method is added to per-session
      method dictionaries,
   else aMethodDict == nil, the method is added to receiver's
      persistent method dictionary.

 If aMethodDict is neither nil nor false, and aCategDict is not nil and
 the compilation succeeds, the resulting method is added to aCategDict
 instead of the receiver's categories.

 If the compilation succeeds, the selector of the new method is
 removed from all method lookup caches for the receiver and all subclasses
 thereof,   independent of the value of aMethodDict argument.

 environmentId must be a SmallInteger >= 0 and <= 255 .
 0 denotes the base Smalltalk image.  1 was reserved for use by Ruby .

 You must have code modification privilege to execute this primitive.

 The IR graph produced by the parser (a GsComMethNode) is available as
   (System __sessionStateAt: 19)
 until being overwritten by the next invocation of this primitive.
 See GsNMethod(C)>>generateFromIR: for example code of printing this IR."

<primitive: 228>
sourceString _validateClasses: { String }.
aSymbolList ~~ nil  ifTrue:[ aSymbolList _validateClass: SymbolList ].
categorySymbol _validateClass: Symbol.
litVarArray ~~ nil ifTrue:[ litVarArray _validateClass: Array].
(aMethodDict ~~ nil and:[ aMethodDict ~~ false]) ifTrue:[ aMethodDict _validateClass: GsMethodDictionary ].
aCategDict ~~ nil ifTrue:[ aCategDict _validateClass: GsMethodDictionary ].
environmentId _validateClass: SmallInteger .

^ self _primitiveFailed:
  #_primCompileMethod:symbolList:category:oldLitVars:intoMethodDict:intoCategories:environmentId:
   args: { sourceString .  aSymbolList .  categorySymbol .  litVarArray .
	   aMethodDict .  aCategDict .  environmentId }
%

category: 'Private'
method: Behavior
_primitiveCompileMethod: sourceString
symbolList: aSymbolList
category: categorySymbol
oldLitVars: litVarArray
intoMethodDict: aMethodDict
intoCategories: aCategDict
environmentId: environmentId

  | src cvtAb |
  cvtAb := System _zeroArgPrim: 132 "fast System gemConfigurationAt:#GemConvertArrayBuilder" .
 cvtAb ifTrue:[ | cvtRes |
   sourceString _isOneByteString ifFalse:[
     sourceString _validateClasses: { String }.
   ].
   cvtRes := GsNMethod _convertArrayBuildersIfNeeded: sourceString .
   cvtRes _isArray ifTrue:[
     "error during ArrayBuilder conversion"
     ^ cvtRes "a compile error descriptor"
   ].
   src := cvtRes .
 ] ifFalse:[
   src := sourceString
 ].
 ^ self _primCompileMethod: src
     symbolList: aSymbolList
     category: categorySymbol
     oldLitVars: litVarArray
     intoMethodDict: aMethodDict
     intoCategories: aCategDict
     environmentId: environmentId
%

category: 'Private'
method: Behavior
_recompileAllMethods
  "unconditionally recompile all env 0 methods."
| meths symList |
meths := { } .
symList := GsSession currentSession symbolList .
self env: 0 methodsDo:[ :selector :aMethod | meths add: aMethod ].
1 to: meths size do:[:j |
  (meths at: j) recompileIntoMethodDict: nil intoCategories: nil symbolList: symList
].
%

category: 'Private Methods for Class Modification'
method: Behavior
_recompileMethodsAfterNewIvOffset: newOffset

| mySymList mySubclasses |
self _validatePrivilege ifTrue:[
  mySymList:= GsCurrentSession currentSession symbolList .
  (newOffset < self instSize) ifTrue:[
      self recompileAllMethodsInContext: mySymList ] .
  mySubclasses := self _subclasses .
  mySubclasses ~~ nil ifTrue:[
    mySubclasses do:[:x | x _recompileMethodsAfterNewIvOffset: newOffset ] .
    ].
].
^ self
%

category: 'Private Methods for Class Modification'
method: Behavior
_refreshClassCache: clearLookupCachesBool

"Refreshes in-memory state of the class .
 If clearLookupCachesBool==true, also clears all method lookup caches.
 Must also be sent immediately after any change to the
 format or named instance variables of the receiver."

<primitive: 227>
self _primitiveFailed:#_refreshClassCache: args: { clearLookupCachesBool } .
self _uncontinuableError
%

category: 'Virtual Machine Control'
method: Behavior
_refreshLookupCache: aSelector oldMethod: aMethod env: envId

"This message must be sent whenever a class's method dictionary
 changes, to keep lookup caches current.

 If not nil, aSelector specifieds a selector added or removed from receiver's
 method dictionary(s) for specified envId .

 If the argument aMethod is not nil, any breakpoints in the obsolete
 method aMethod are cleared.
 If aMethod is a SmallInteger, it is assumed to be a Ruby method-hidden
 value , and breakpoint clearing logic is skipped.

 envId must be a SmallInteger >= 0 and <= 255 .
"

<primitive: 374>
aSelector ifNotNil:[ aSelector _validateClass: Symbol ] .
aMethod ifNotNil:[ | gsMethod |
   gsMethod := ObsoleteClasses at: #GsMethod .
   (aMethod isKindOf: gsMethod) ifTrue: [
     ^ self "ignore unconverted method from Gs64 2.x"
   ].
   aMethod _validateClass: gsMethod
].
envId _validateClass: SmallInteger .
self _primitiveFailed: #_refreshLookupCache:oldMethod:env:
     args: { aSelector . aMethod . envId }.
self _uncontinuableError
%

category: 'Updating Categories'
method: Behavior
_removeBaseCategory: categoryName environmentId: envId

"Removes the specified category and all its methods from the receiver's
 method dictionary.  If categoryName is not in the receiver's categorys
 generates an error.  Any breakpoints in removed methods are cleared."
  | cats catName |
  self _validatePrivilege ifFalse:[ ^ nil ].
  catName := Symbol _existingWithAll: categoryName .
  catName ifNil:[ ^ self _error: #rtErrKeyNotFound args: { categoryName } ].
  cats := self _baseCategorys: envId .
  cats ifNotNil:[ | selectors |
    selectors := cats at: catName .
    selectors do:[ :aSelector |
       self _basicRemoveSelector: aSelector environmentId: envId
    ].
    cats removeKey: catName
  ]  ifNil:[
    self _error: #rtErrKeyNotFound args: { categoryName }
  ]
%

category: 'Modifying Classes'
method: Behavior
_removeClassVar: aSymbol

"Removes the class variable with name aSymbol from the receiver's
 class variables dictionary.  Generates an error if aSymbol is not
 the name of a class variable of the receiver."

  self _validatePrivilege ifTrue:[
    classVars ifNil:[ ^ self _errorKeyNotFound: aSymbol ].
    classVars removeKey: aSymbol
            ifAbsent: [classVars _errorKeyNotFound: aSymbol].
  ]
%

category: 'Modifying Classes'
method: Behavior
_removeClassVar: aSymbol ifAbsent: exceptionBlock

"Removes the class variable with name aSymbol from the receiver's
 class variables dictionary.  Executes exceptionBlock if aSymbol is not
 the name of a class variable of the receiver."
  | dict |
  self _validatePrivilege ifTrue:[
    (dict := classVars) ifNil:[ exceptionBlock value ]
       ifNotNil:[ dict removeKey: aSymbol ifAbsent: exceptionBlock ].
  ]
%

category: 'Private Methods for Class Modification'
method: Behavior
_removeInstVarAtOffset: offset
	"Remove named instance variable at specified offset from self and all
 subclasses assuming that all error checks have been done."

	self _validatePrivilege
		ifTrue:
			[self _incrementInstVars: -1.
			instVarNames removeFrom: offset to: offset.
			self _refreshClassCache: false.
			self _subclasses do: [:x | x _removeInstVarAtOffset: offset]]
%

category: 'Reloading Decompiled Methods'
method: Behavior
_resolveClassOrPoolVar: aSymbol

"Searches the receiver's class variables dictionary and pool dictionaries to
 attempt to resolve aSymbol.  Returns the SymbolAssociation for the variable
 with name aSymbol, or nil if aSymbol could not be found."

| assoc |
assoc := self _resolveClassVar: aSymbol .
assoc ~~ nil ifTrue:[ ^ assoc ].

poolDictionaries size ~~ 0 ifTrue:[
  1 to: poolDictionaries size do:[:j | | aDict |
    aDict := poolDictionaries at: j .
    assoc := aDict associationAt: aSymbol otherwise: nil .
    assoc ~~ nil ifTrue:[ ^ assoc ].
    ].
  ].
^ nil
%

category: 'Reloading Decompiled Methods'
method: Behavior
_resolveClassVar: aSymbol

"Searches the receiver's class variables dictionary, to attempt to resolve
 aSymbol.  Returns the SymbolAssociation for the variable with name aSymbol, or
 nil if aSymbol could not be found."

| assoc |

classVars ifNotNil:[ :cvs |
  assoc := cvs associationAt: aSymbol otherwise: nil.
  ].
^ assoc
%

category: 'Reloading Decompiled Methods'
method: Behavior
_resolveLiteralVar: aSymbol

"Attempts to resolve a literal variable with name aSymbol.  To attempt to
 resolve aSymbol, searches the receiver's class variable dictionary and its pool
 dictionaries, then the superclasses' class variables, then the current default
 SymbolList.

 Returns the SymbolAssociation for the variable or nil if aSymbol could not be
 found."

"Implementation here must agree with comgen.c:searchClassOrPool()"

| assoc aClass |

aClass := self .
"search receiver's class variables and pool variables"
assoc := aClass _resolveClassOrPoolVar: aSymbol .
assoc ~~ nil ifTrue:[ ^ assoc ].

"search for a class variable inherited from superclasses"
[ (aClass := aClass superClass) ~~ nil
 ]
whileTrue: [
  assoc := aClass _resolveClassVar: aSymbol .
  assoc ~~ nil ifTrue:[ ^ assoc ].
] .
"search the symbol list"
^ GsSession currentSession resolveSymbol: aSymbol
%

category: 'Browser Methods'
method: Behavior
_selectorPrefixesReport: envId
  "In base Smalltalk image, no ruby prefix logic"
  ^ self _selectorsReport: envId
%

category: 'Accessing Categories'
method: Behavior
_selectorsInBaseCategory: aSymbol

  ^ (self _baseCategorys: 0) at: aSymbol
%

category: 'Browser Methods'
method: Behavior
_selectorsReport: envId
  ^ self _selectorsReport: envId matching: nil
%

category: 'Browser Methods'
method: Behavior
_selectorsReport: envId matching: aString
 ^ self _selectorsReport: envId matching: aString primitivesOnly: false
%

category: 'Browser Methods'
method: Behavior
_selectorsReport: envId matching: aString primitivesOnly: primsBoolean

^ self _selectorsReport: envId matching: aString primitivesOnly: primsBoolean
       includeDeprecated: true
%

category: 'Browser Methods'
method: Behavior
_selectorsReport: envId matching: aString primitivesOnly: primsBoolean includeDeprecated: inclDeprecBool
 "Used by topaz (and GBS?).
  Result is a sorted SequenceableCollection  of Symbols, plus an optional string 'Omitted .. deprecated methods'.
  aString if not nil restricts the result to only include those selectors containing
  the case-insensitive substring aString .
  primsBoolean if true further restricts the result to only include only
  selectors of methods which are primitives."
 | list res deprecSet numDeprecated |
 numDeprecated := 0 .
 inclDeprecBool ifFalse:[
   deprecSet := Object _selectorsInBaseCategory:#'Deprecated Notification'.
 ].
 list := self selectorsForEnvironment: envId .
 res := SortedCollection new .
 list do:[:sym | | sel |
    (aString == nil or:[ (sym includesString: aString)]) ifTrue:[
       sel := sym .
       (primsBoolean or:[ deprecSet ~~ nil ]) ifTrue:[
         (self compiledMethodAt: sym environmentId: envId otherwise: nil) ifNotNil:[ :meth|
           primsBoolean ifTrue:[ meth _isPrimitive ifFalse:[ sel := nil ]].
           sel ifNotNil:[
             deprecSet ifNotNil:[ (meth _selectorPool * deprecSet) size ~~ 0 ifTrue:[
               sel := nil . numDeprecated := numDeprecated + 1 ]].
           ].
         ]
       ].
       sel ifNotNil:[ res add: sel ].
    ].
 ].
 numDeprecated > 0 ifTrue:[
   res := Array withAll: res .
   res add:'(Omitted ' , numDeprecated asString, ' deprecated methods)'.
 ].
 ^ res
%

category: 'Updating the Method Dictionary'
method: Behavior
_sessionMethodsChanged
  "set bit in VM's cbCodeChanged word so it can propagate to
   other VMs upon commit. 
   System class >> _sessionMethodsChanged*  returns union of bit
   with respect to a view change."
  ^ self _codeChangedForEnv: -1 .
%

category: 'Debugging Support'
method: Behavior
_setMethodBreak: aSelector stepPoint: anInt

^ self _setMethodBreak: aSelector stepPoint: anInt env: 0
%

category: 'Debugging Support'
method: Behavior
_setMethodBreak: aSelector stepPoint: anInt env: envId

"Returns true to indicate success.  Otherwise it returns a string describing
 the error."

  | selectorSym |

  (aSelector isByteKindOf: CharacterCollection) ifFalse: [ ^ 'Illegal selector' ].
  selectorSym := Symbol _existingWithAll: aSelector .
  selectorSym ifNil:[ ^ 'selector is not an existing Symbol'].
  (#( 	_and: and: _or: or:
	_downTo:by:do: _downTo:do:
	_gsReturnNoResult _gsReturnNothingEnableEvents
	_isArray _isExceptionClass _isExecBlock _isFloat _isInteger _isNumber
	_isOneByteString _isRange _isRegexp _isRubyHash _isScaledDecimal
	_isSmallInteger _isSymbol
	_leaveProtectedMode __inProtectedMode
	_stringCharSize
	#'==' #'~~'
	ifFalse: ifFalse:ifTrue:
	ifNil: ifNil:ifNotNil: ifNotNil: ifNotNil:ifNil:
	ifTrue: ifTrue:ifFalse:
	isKindOf:
	repeat timesRepeat:
	to:by:do: to:do:
	untilFalse untilFalse: untilTrue untilTrue:
	whileFalse whileFalse: whileTrue whileTrue:
    )
    includesIdentical: aSelector) ifTrue:[
      ^ 'You may not set a method break on an optimized selector' ].

  (self == SmallInteger
      and: [ #(  #+  #-  #>=  #*  #=  ) includesIdentical: aSelector])
        ifTrue:[ ^ 'You may not set a method break on an optimized selector' ].

  (anInt _isSmallInteger) ifFalse:[ ^ 'Step point must be a SmallInteger' ].
  (self includesSelector: selectorSym environmentId: envId)
      ifFalse: [ ^ 'Selector does not exist in class' ].

  ((self compiledMethodAt: selectorSym environmentId: envId) setBreakAtStepPoint: anInt)
    ifNil:[ ^ 'Step point does not exist in method' ].

  ^ true
%

category: 'Private'
method: Behavior
_setSelfCanBeSpecial

  "Set the SelfCanBeSpecial bit in the receiver.
   For use only after altering the superclass hierarchy in such a way
   as to insert new classes between a class for which isSpecial is true
   and Object."

format := format bitOr: 16r2000 .
%

category: 'Debugging Support'
method: Behavior
_sourceCodeAndOffsets: aSelector
  ^ self _sourceCodeAndOffsets: aSelector environmentId: 0
%

category: 'Debugging Support'
method: Behavior
_sourceCodeAndOffsets: aSelector environmentId: envId

"Returns an Array with two elements.  The first element is a String
 representing the source code for the argument, aSelector.  The second element
 is an InvariantArray (that holds SmallIntegers) is a list of offsets into
 sourceString, corresponding in order to the step points.  If aSelector (a
 String) is not a selector in the receiver's method dictionary, returns nil."

| method |
method := self compiledMethodAt: aSelector environmentId: envId otherwise: nil .
method ifNotNil:[ ^ { method _sourceString . method _sourceOffsets } ] .
^ nil
%

category: 'Private'
method: Behavior
_stripAllMethodSources

"Returns true if all method sources should be stripped for the receiver, and
 false otherwise."

"Returns true for the list of classes given in the code, false for others."

| classNameList |
" classNameList is an Array of Symbols because some of these classes
are not known at filein "
classNameList := #( ).

1 to: classNameList size do: [ :i |
  ((Globals at: (classNameList at: i)) == self)
    ifTrue: [ ^ true ]
].
^ false
%

category: 'Private'
method: Behavior
_structuralUpdatesDisallowed

"Private.

 Returns true if GemBuilder for C (GCI) direct structural update of instances
 is disallowed, false otherwise.  A result of true means that the deferred
 update mechanism is used (see GciProcessDeferredUpdates in gci.hf) by
 GemBuilder for C store operations on instances of the receiver."

^ (format bitAnd: 16r200 "no update through structural access") ~~ 0
%

category: 'Modifying Classes'
method: Behavior
_subclasses

"Returns the class variable that is the list of subclasses, or
 nil if the receiver does not keep track of subclasses."

^ nil
%

category: 'Testing Inheritance'
method: Behavior
_subclassOf: aClass
"Returns true if the receiver is identical to or is a subclass aClass.

 aClass must be a Class or a Metaclass3 .
 ClassHistories of the receiver and argument are ignored."

<primitive: 895>
self _primitiveFailed: #_subclassOf: args: { aClass } .
^ false
%

category: 'Private'
method: Behavior
_superclass
  ^ superClass
%

category: 'Private'
method: Behavior
_traversalByCallback

"Private.

 Returns true if GemBuilder for C (GCI) traversal results of instances
 are obtained by message send of aClampSpecification.traversalCallBackSelector,
 false otherwise."

^ (format bitAnd: 16r400"travByCallback") ~~ 0
%

category: 'Private Methods for Class Modification'
method: Behavior
_validateNewNamedInstVar: aSymbol

"Generate an error if the argument is the name of an already existing instance
 variable of the receiver or if the receiver is not modifiable and has not
 disallowed subclassing, otherwise return true. If execution is continued
 from the error, return false."

"reimplementation of self validateIsModifiable ."
self isModifiable ifFalse:[
  self isNsc ifTrue:[
    self _error: #rtErrClassNotModifiable .
    ^ false .
  ].
  self subclassesDisallowed ifFalse:[
    self _error: #rtErrClassNotModifiable .
    ^ false .
  ] .
  self isIndexable ifTrue:[
     self _error: #rtErrClassNotModifiable.
    ^ false
  ].
] .
(instVarNames includesIdentical: aSymbol) ifTrue:[
  self _error: #rtErrAddDupInstvar args:{ aSymbol } .
  ^ false
].
self _subclasses ifNotNil:[ :mySubclasses |
  mySubclasses do:[:aSubCls|
     (aSubCls _validateNewNamedInstVar: aSymbol) ifFalse:[ ^ false ].
  ].
].
^ true
%

category: 'Private'
method: Behavior
_validatePrivilege

^ System myUserProfile _validateCodeModificationPrivilege
%

category: 'Private'
method: Behavior
_varyingConstraint

"Returns the constraint on the unnamed part of the receiver (a kind of Class).
 If the receiver has no constraint on its unnamed part, or if it has no unnamed
 part, this method returns Object.

 As of GemStone 64bit v3.4, constraints are no longer implemented.
 This method is provided for examining classes in repositories upgraded from
 an older version."

 ^ self _constraintAt: self instSize + 1 .
%

category: 'Accessing Categories'
method: Behavior
__categorys
  ^ categorys
%

! Class extensions for 'BitSet'

!		Class methods for 'BitSet'

removeallmethods BitSet
removeallclassmethods BitSet

category: 'Instance creation'
classmethod: BitSet
new: numberOfBits
  "allocated size is always a multiple of 4 bytes,
   all bits in the result are set to zero."

^ super new: ((numberOfBits + 31) // 32) * 4
%

!		Instance methods for 'BitSet'

category: 'Accessing'
method: BitSet
at: offset
"Return the specified bit, result is 0 or 1.
 offset is a zero-based SmallInteger,
 0 specifies the least significant bit.
 offset < 0 generates an error.
 0 is returned for any offset > allocated size. "

<primitive: 748>
offset _validateClass: SmallInteger.
offset < 0 ifTrue:[ offset _error: #errArgTooSmall args:{ 0 } ].
self _primitiveFailed: #at: args: { offset }
%

category: 'Updating'
method: BitSet
at: offset put: aValue

"offset is zero-based, 0 specifies the least significant bit.
 aValue must be 0 or 1 .
 Returns previous state of the specified bit. "

<primitive: 751>
offset _validateClass: SmallInteger.
offset < 0 ifTrue:[ offset _error: #errArgTooSmall args:{ 0 } ].
(aValue == 0 or:[ aValue == 1]) ifFalse:[
  aValue _error: #rtErrArgOutOfRange args:{ 0 . 1 } ].
self _primitiveFailed: #at:put: args: { offset . aValue }
%

category: 'Updating'
method: BitSet
bitAnd: aBitSet

"clear bits in the receiver which are not also set in the argument.
 aBitSet must be an instance of the class of the receiver.
 Returns receiver. "

<primitive: 752>
aBitSet _validateInstanceOf: self class .
self _primitiveFailed: #bitAnd: args: { aBitSet }
%

category: 'Updating'
method: BitSet
bitInvert
"Inverts the  bits of the receiver."

<primitive: 755>
self _primitiveFailed: #bitInvert
%

category: 'Updating'
method: BitSet
bitOr: aBitSet

"Set bits in the receiver that are set in the argument
 Grows receiver as needed.
 aBitSet must be an instance of the class of the receiver.
 Returns receiver. "

<primitive: 753>
aBitSet _validateInstanceOf: self class .
self _primitiveFailed: #bitOr: args: { aBitSet }
%

category: 'Updating'
method: BitSet
bitXor: aBitSet
"Sets receiver to be the exclusive-or of the receiver with argument.
 Grows receiver as needed.
 aBitSet must be an instance of the class of the receiver.
 Returns receiver."
<primitive: 754>
aBitSet _validateInstanceOf: self class .
self _primitiveFailed: #bitXor: args: { aBitSet }
%

category: 'Updating'
method: BitSet
clear
  "set all bits in receiver to zero, returns receiver.
   does not change allocated size of receiver."
<primitive: 750>
self _primitiveFailed: #clear
%

category: 'Accessing'
method: BitSet
countBits
"return the number of bits set in the receiver"

<primitive: 749>
self _primitiveFailed: #countBits
%

category: 'Accessing'
method: BitSet
size

"return the allocated size in bits"

^ super size * 8
%

category: 'Updating'
method: BitSet
size: numberOfBits

 "change the allocated size. allocated size is always a multiple of 4 bytes,
  rounded up from the specified size.
  If the new size is larger than current size,
   newly allocated bits will be set to zero."

super size: ((numberOfBits + 31) // 32) * 4
%

! Class extensions for 'BlockClosure'

!		Class methods for 'BlockClosure'

removeallmethods BlockClosure
removeallclassmethods BlockClosure

category: 'Instance Creation'
classmethod: BlockClosure
new

"Disallowed."

self shouldNotImplement: #new
%

category: 'Reloading Decompiled Methods'
classmethod: BlockClosure
_new

"Private."

^ super new .
%

!		Instance methods for 'BlockClosure'

category: 'Accessing'
method: BlockClosure
argumentCount
  "Answer the number of arguments needed to evaluate the receiver."

  ^self subclassResponsibility: #argumentCount.
%

category: 'Continuations'
method: BlockClosure
callCC
   "Evaluate the receiver, which must be a one-argument block.
   The argument to the block will be a newly created continuation,
   which is an instance of GsProcess that contains a copy of the
   active process. Unless control flow is otherwise interrupted,
   return the result of evaluating the last expression in the block.

   If a reference is retained to the continuation that is created
   during the execution of this method, it may later be evaluated
   using the continuation's value: or value methods (distinct from
   the BlockClosure's value: and value methods). When the continuation
   value: is evaluated, it is effectively a GOTO to the point in
   execution that the original call to callCC returned, (optionally)
   returning a different value. The actual callCC block is only
   executed once; later evaluations of the continuation return from
   the block, but do not re-execute the block."

   ^ self value: (GsProcess continuationFromLevel: 2)
%

category: 'Copying'
method: BlockClosure
copy

"Returns the receiver. Copies of blocks are not made."

^ self.
%

category: 'Block Evaluation'
method: BlockClosure
ensure: aBlock
  "Evaluate the receiver.
  Evaluate aBlock after evaluating the receiver,
  or before any return from a block that would return to the sender.
  Returns result of evaluating the receiver.

  aBlock must be a zero-arg instance of ExecBlock,
  otherwise an error is generated."

  ^self subclassResponsibility: #ensure:.
%

category: 'Block Evaluation'
method: BlockClosure
ifCurtailed: terminationBlock
  "Evaluate the receiver and return its result.
  If abnormal termination of the receiver occurs, terminationBlock is
  evaluated. The value returned from the evaluation of terminationBlock
  is discarded.

  Activation of an exception handler from within the receiver is not in and
  of itself an abnormal termination. However, if the exception handler for
  an exception that is not resumable results in termination of the receiver
  or if its handler block contains a return statement that results in
  abnormal termination of the receiver, then terminationBlock will be evaluated
  after evaluation of the exception handler.

  If an abnormal termination result in the termination of multiple blocks
  which were evaluated using either #ensure: or #ifCurtailed: the respective
  terminationBlocks will be executed in the reverse of the order in which
  the corresponding receiver blocks were evaluated."

  | wasCurtailed result |
  wasCurtailed := true.
  ^[
     result := self value.
     wasCurtailed := false.
     result
  ] ensure:[
    wasCurtailed ifTrue: terminationBlock
  ].
%

category: 'Updating'
method: BlockClosure
instVarAt: anIndex put: aValue

"Disallowed."

self shouldNotImplement: #instVarAt:put:
%

category: 'Testing'
method: BlockClosure
isSimple

"Returns the default answer, false."

^ false
%

category: 'Block Evaluation'
method: BlockClosure
on: exceptionSelector do: handlerBlock
	"Try to evaluate the receiver, which should be a zero-argument block.
	 If an exception occurs and the expression exceptionSelector handles: theExceptionInstance
	 returns true, then evaluate the one argument block handlerBlock, passing it the exception
	 instance as its argument.

	 These forms are supported directly by the VM
		(1) on: anException do: handlerBlock
		(2) on: anExceptionSet do: handlerBlock
	 This form is handled by Smalltalk code in the body of this method.
		(3) on: anObject do: handlerBlock

	 anException must be the class Exception  or a subclass thereof;
	 anExceptionSet must be a kind of ExceptionSet;
	 handlerBlock must be an instance of ExecBloc otherwise an error is generated.

	 For forms 1, 2, and 3 if handlerBlock is not a one argument block,
	 an error is generated if exception handling attempts to invoke that handlerBlock.

	 If handlerBlock is invoked to handle an Exception which occurs during execution of the
	 receiver and handlerBlock completes normally , then the result of handlerBlock (the value
	 of the last expression in handlerBlock) will be the result of the on:do: send .  Other-than-normal
	 completion of handlerBlock is available by use of Exception's instance methods such as
	 #return, #return:, #retry, #retryUsing:, #resume, #resume:, #pass, and #outer within handlerBlock.

	 For forms 1 and 2, when searching for a handlerBlock to handle a signaled Exception, the VM
	 uses Behavior>>_subclassOf: semantics . classHistories of the class of the signaled Exception
	 and of anException or elements of anExceptionSet are ignored."

	^self subclassResponsibility: #on:do:.
%

category: 'Flow of Control'
method: BlockClosure
untilFalse

  ^self subclassResponsibility: #untilFalse
%

category: 'Flow of Control'
method: BlockClosure
untilFalse: aBlock

  ^self subclassResponsibility: #untilFalse:
%

category: 'Block Evaluation'
method: BlockClosure
value
	"Return the value of the receiver evaluated with no arguments.
	 If the receiver expects any arguments, signal an error."

	^self subclassResponsibility: #value.
%

category: 'Block Evaluation'
method: BlockClosure
value: anObject
	"Return the value of the receiver evaluated with anObject as its argument.  If
	 the receiver expects a different number of arguments, signal an error."

	^self subclassResponsibility: #value:.
%

category: 'Block Evaluation'
method: BlockClosure
value: firstObject value: secondObject
	"Return the value of the receiver evaluated with the specified arguments.  If
	 the receiver expects a different number of arguments, signal an error."

	^self subclassResponsibility: #value:value:.
%

category: 'Block Evaluation'
method: BlockClosure
valueWithArguments: argList
	"Return the value of the receiver evaluated with the specified arguments.  If
	 the receiver expects a different number of arguments, signal an error."

	^self subclassResponsibility: #valueWithArguments:.
%

category: 'Flow of Control'
method: BlockClosure
whileFalse
	"Evaluate the receiver once and then repeatedly as long as the value
	 returned by the evaluation is false."

	^self subclassResponsibility: #whileFalse.
%

category: 'Flow of Control'
method: BlockClosure
whileFalse: aBlock
	"(Reserved selector.)  Evaluates the zero-argument block aBlock repeatedly
	 while the receiver evaluates to false.  Return nil.  Generates an error if the
	 receiver is not a zero-argument block."

	^self subclassResponsibility: #whileFalse:.
%

category: 'Flow of Control'
method: BlockClosure
whileTrue
	"Evaluate the receiver once and then repeatedly as long as the value
	 returned by the evaluation is true."

	^self subclassResponsibility: #whileTrue.
%

category: 'Flow of Control'
method: BlockClosure
whileTrue: aBlock
	"(Reserved selector.)  Evaluates the zero-argument block aBlock repeatedly
	 while the receiver evaluates to true.  Return nil.  Generates an error if the
	 receiver is not a zero-argument block."

	^self subclassResponsibility: #whileTrue:.
%

category: 'Storing and Loading'
method: BlockClosure
writeTo: aPassiveObject

"Instances of BlockClosure cannot be converted to passive form.  This method
 writes nil to aPassiveObject and stops GemStone Smalltalk execution with a
 notifier."

aPassiveObject writeObject: nil.
self _error: #rtErrAttemptToPassivateInvalidObject.
%

category: 'Private'
method: BlockClosure
_deepCopyWith: copiedObjDict

"Private.  Used internally to implement deepCopy."

^ self.
%

! Class extensions for 'Boolean'

!		Class methods for 'Boolean'

removeallmethods Boolean
removeallclassmethods Boolean

category: 'Instance Creation'
classmethod: Boolean
fromStream: aStream

"If the next characters in aStream are true or false (case insensitive, leading
 spaces permitted), this method returns the appropriate Boolean.  Otherwise,
 generates an error."

| char |

self _checkReadStream: aStream forClass: CharacterCollection.

char := aStream next.
[ char == $  ] whileTrue: [ char := aStream next ].
aStream skip: -1.

((char := aStream next) isEquivalent: $T)
ifTrue:
  [ ((aStream next isEquivalent: $R)
    and: [(aStream next isEquivalent: $U)
    and: [aStream next isEquivalent: $E ]])
    ifTrue:
      [ ^ true ]
  ]
ifFalse:
  [ (char isEquivalent: $F)
    ifTrue:
      [ ((aStream next isEquivalent: $A)
        and: [(aStream next isEquivalent: $L)
        and: [(aStream next isEquivalent: $S)
        and: [aStream next isEquivalent: $E ]]])
        ifTrue:
          [ ^ false ]
      ]
  ].

self _errIncorrectFormat: aStream
%

category: 'Instance Creation'
classmethod: Boolean
fromString: aString

"If aString contains true or false, returns the appropriate Boolean.  Leading
 and trailing spaces are permitted in the String.  If aString contains any
 characters other than true or false, this method generates an error."

| s result |

s := ReadStream on: aString.
result := self fromStream: s.
[ s atEnd ]
whileFalse:
  [ (s next == $  )
    ifFalse:
      [ self _errIncorrectFormat: aString ]
  ].
^ result
%

category: 'Storing and Loading'
classmethod: Boolean
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form and returns an equivalent instance of Boolean."

"This method supports filein of Booleans in the format required by GeODE
 versions before 2.0.1."

|inst|
inst := passiveObj next == $t.
passiveObj hasRead: inst.
^inst
%

category: 'Instance Creation'
classmethod: Boolean
new

"Disallowed.  You cannot create new instances of Boolean."

self shouldNotImplement: #new
%

!		Instance methods for 'Boolean'

category: 'Logical Operations'
method: Boolean
& aBoolean

"Evaluating conjunction (AND).  Returns true if both the receiver and the
 argument aBoolean are true."

<primitive: 42>
aBoolean _validateClass: Boolean.
self _primitiveFailed: #& args: { aBoolean }.
self _uncontinuableError
%

category: 'Flow of Control'
method: Boolean
and: aBlock

"(Reserved selector.) Nonevaluating conjunction.  Returns the value of the
 zero-argument block aBlock if the receiver is true.  Otherwise, returns false
 without evaluating the argument.

 The following is an optimized control structure, not a recursive send."

^ self and: [ 
   aBlock _isExecBlock ifFalse:[ ArgumentError signal:'expected a block'].
   aBlock value
 ]
%

category: 'Converting'
method: Boolean
asBit
  "Returns 1 if self==true, 0 otherwise"

  ^ self ifTrue:[ 1 ] ifFalse:[ 0 ]
%

category: 'Formatting'
method: Boolean
asString

"Returns a String containing true or false, depending on the receiver."

self
  ifTrue:
    [ ^ String withAll: 'true' ]
  ifFalse:
    [ ^ String withAll: 'false' ]
%

category: 'Clustering'
method: Boolean
clusterDepthFirst

"Returns true.  (Because Booleans are self-defining objects, this method has no
 effect.)"

^ true
%

category: 'Storing and Loading'
method: Boolean
containsIdentity

"Private."

^true
%

category: 'Copying'
method: Boolean
copy

"Overrides the inherited method to return the receiver.  The pseudo-variables
 true and false are the only instances of Boolean, and must preserve identity."

^ self
%

category: 'Logical Operations'
method: Boolean
eqv: aBoolean

"Returns true if the receiver is identical to aBoolean."

^ self == aBoolean
%

category: 'Flow of Control'
method: Boolean
ifFalse: aBlock

"(Reserved selector.)  Returns the value of the zero-argument block aBlock if
 the receiver is false.  Otherwise, returns nil without evaluating the
 argument."

"The following is an optimized control structure, not a recursive send."

^ self ifFalse: [
    aBlock _isExecBlock ifFalse:[ ArgumentError signal:'expected a block'].
   ^ aBlock value 
 ]
%

category: 'Flow of Control'
method: Boolean
ifFalse: falseBlock ifTrue: trueBlock

"(Reserved selector.)  Returns the value of the zero-argument block falseBlock
 if the receiver is false.  Otherwise, returns the value of the zero-argument
 block trueBlock without evaluating falseBlock."

"The following is an optimized control structure, not a recursive send."

self ifFalse:[
  falseBlock _isExecBlock ifFalse:[ ArgumentError signal:'expected a block'].
  ^ falseBlock value
] ifTrue: [
  trueBlock _isExecBlock ifFalse:[ ArgumentError signal:'expected a block'].
  ^ trueBlock value
]
%

category: 'Flow of Control'
method: Boolean
ifTrue: aBlock

"(Reserved selector.)  Returns the value of the zero-argument block aBlock if
 the receiver is true.  Otherwise, returns nil without evaluating the argument."

"The following is an optimized control structure, not a recursive send."

^ self ifTrue: [
   aBlock _isExecBlock ifFalse:[ ArgumentError signal:'expected a block'].
   ^ aBlock value
]
%

category: 'Flow of Control'
method: Boolean
ifTrue: trueBlock ifFalse: falseBlock

"(Reserved selector.)  Returns the value of the zero-argument block falseBlock
 if the receiver is false.  Otherwise, returns the value of the zero-argument
 block trueBlock without evaluating falseBlock."

"The following is an optimized control structure, not a recursive send."

^ self ifTrue: [
   trueBlock _isExecBlock ifFalse:[ ArgumentError signal:'expected a block'].
   ^ trueBlock value
  ] ifFalse: [ 
   falseBlock _isExecBlock ifFalse:[ ArgumentError signal:'expected a block'].
   ^ falseBlock value
  ]
%

category: 'Testing'
method: Boolean
isSpecial

"Returns true if the receiver is a special object."

^ true
%

category: 'Logical Operations'
method: Boolean
not

"Negation.  Returns true if the receiver is false.  Returns false if the
 receiver is true."

^ self == false
%

category: 'Flow of Control'
method: Boolean
or: aBlock

"(Reserved selector.) Nonevaluating disjunction.  Returns the value of the
 zero-argument block aBlock if the receiver is false.  Otherwise, returns true
 without evaluating the argument.

 The following is an optimized control structure, not a recursive send."

^ self or:[ 
    aBlock _isExecBlock ifFalse:[ ArgumentError signal:'expected a block'].
    aBlock value 
 ]
%

category: 'Json'
method: Boolean
printJsonOn: aStream

	aStream nextPutAll: (self ifTrue: ['true'] ifFalse: ['false']).
%

category: 'Formatting'
method: Boolean
printString

"Returns a String whose contents are a displayable representation of the
 receiver."

"GemStone does not allow the creation of new kinds of Boolean, so there is no
 point in creating a stream and sending printOn:."

^self asString
%

category: 'Storing and Loading'
method: Boolean
writeTo: passiveObj

"Converts the receiver to its passive form and writes that information on
 passiveObj."

passiveObj nextPut: (self ifTrue: [ $* ] ifFalse: [ $~ ])

%

category: 'Logical Operations'
method: Boolean
xor: aBoolean

"Exclusive OR.  Returns true if the receiver is not identical to aBoolean."

<primitive: 45>
aBoolean _validateClass: Boolean.
self _primitiveFailed: #xor: args: { aBoolean }.
self _uncontinuableError
%

category: 'Flow of Control'
method: Boolean
_and: aBlock

"(Reserved selector.)  Nonevaluating conjunction.  Returns the value of the
 zero-argument block aBlock if the receiver is true.  Otherwise, returns false
 without evaluating the argument.

 The following is an optimized control structure, not a recursive send."

^ self and: [ 
    aBlock _isExecBlock ifFalse:[ ArgumentError signal:'expected a block'].
    aBlock value
  ]
%

category: 'Decompiling without Sources'
method: Boolean
_asSource

^ self asString
%

category: 'New Indexing Comparison'
method: Boolean
_classSortOrdinal

^ 20
%

category: 'New Indexing Comparison - for Compare'
method: Boolean
_idxForCompareBooleanGreaterThanOrEqualToSelf: aBoolean

"second half of a double dispatch call from Boolean>>_idxForCompareGreaterThanOrEqualTo:. Note that aBoolean should be the receiver in any >= comparison"

aBoolean = self
  ifTrue: [ ^true].
^aBoolean == true
%

category: 'New Indexing Comparison - for Compare'
method: Boolean
_idxForCompareBooleanGreaterThanSelf: aBoolean

"second half of a double dispatch call from Boolean>>_idxForCompareGreaterThan:. Note that aBoolean should be the receiver in any > comparison"

self = aBoolean
    ifTrue: [ ^ false ].
^aBoolean == true
%

category: 'New Indexing Comparison - for Compare'
method: Boolean
_idxForCompareBooleanLessThanOrEqualToSelf: aBoolean

"second half of a double dispatch call from Boolean>>_idxForCompareLessThanOrEqualTo:. Note that aBoolean should be the receiver in any <= comparison"

aBoolean = self
  ifTrue: [ ^true].
^aBoolean == false
%

category: 'New Indexing Comparison - for Compare'
method: Boolean
_idxForCompareBooleanLessThanSelf: aBoolean

"second half of a double dispatch call from Boolean>>_idxForCompareLessThan:. Note that aBoolean should be the receiver in any < comparison"

self = aBoolean
    ifTrue: [ ^ false ].
^aBoolean == false
%

category: 'New Indexing Comparison'
method: Boolean
_idxForCompareGreaterThan: arg

""

^arg _idxForCompareBooleanGreaterThanSelf: self
%

category: 'New Indexing Comparison'
method: Boolean
_idxForCompareGreaterThanOrEqualTo: arg

""

^arg _idxForCompareBooleanGreaterThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: Boolean
_idxForCompareLessThan: arg

""

^arg _idxForCompareBooleanLessThanSelf: self
%

category: 'New Indexing Comparison'
method: Boolean
_idxForCompareLessThanOrEqualTo: arg

""

^arg _idxForCompareBooleanLessThanOrEqualToSelf: self
%

category: 'New Indexing Comparison - for Sort'
method: Boolean
_idxForSortBooleanGreaterThanOrEqualToSelf: anObject

"second half of a double dispatch call from Boolean>>_idxForSortGreaterThanOrEqualTo:. Note that aBoolean should be the receiver in any >= comparison"

anObject = self
  ifTrue: [ ^ true ].
^anObject
%

category: 'New Indexing Comparison - for Sort'
method: Boolean
_idxForSortBooleanGreaterThanSelf: anObject

"second half of a double dispatch call from Boolean>>_idxForSortGreaterThan:. Note that aBoolean should be the receiver in any > comparison"

anObject = self
  ifTrue: [ ^ false ].
^anObject
%

category: 'New Indexing Comparison - for Sort'
method: Boolean
_idxForSortBooleanLessThanOrEqualToSelf: anObject

"second half of a double dispatch call from Boolean>>_idxForSortLessThanOrEqualTo:. Note that aBoolean should be the receiver in any <= comparison"

anObject = self
  ifTrue: [ ^ true ].
^anObject not
%

category: 'New Indexing Comparison - for Sort'
method: Boolean
_idxForSortBooleanLessThanSelf: anObject

"second half of a double dispatch call from Boolean>>_idxForSortLessThan:. Note that aBoolean
should be the receiver in any < comparison"

anObject = self
  ifTrue: [ ^ false ].
^anObject not
%

category: 'New Indexing Comparison'
method: Boolean
_idxForSortGreaterThan: arg

""

^arg _idxForSortBooleanGreaterThanSelf: self
%

category: 'New Indexing Comparison'
method: Boolean
_idxForSortGreaterThanOrEqualTo: arg

""

^arg _idxForSortBooleanGreaterThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: Boolean
_idxForSortLessThan: arg

""

^arg _idxForSortBooleanLessThanSelf: self
%

category: 'New Indexing Comparison'
method: Boolean
_idxForSortLessThanOrEqualTo: arg

""

^arg _idxForSortBooleanLessThanOrEqualToSelf: self
%

category: 'Flow of Control'
method: Boolean
_or: aBlock

"(Reserved selector.)  Nonevaluating disjunction.  Returns the value of the
 zero-argument block aBlock if the receiver is false.  Otherwise, returns true
 without evaluating the argument.

 The following is an optimized control structure, not a recursive send."

^ self or:[ 
    aBlock _isExecBlock ifFalse:[ ArgumentError signal:'expected a block'].
    aBlock value 
  ]
%

category: 'Logical Operations'
method: Boolean
| aBoolean

"Evaluating disjunction (OR).  Returns true if either the receiver or the
 argument aBoolean is true."

<primitive: 43>
aBoolean _validateClass: Boolean.
self _primitiveFailed: #| args: { aBoolean }.
self _uncontinuableError
%

! Class extensions for 'Break'

!		Instance methods for 'Break'

removeallmethods Break
removeallclassmethods Break

category: 'Instance initialization'
method: Break
initialize
  gsNumber := ERR_Break.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'Breakpoint'

!		Class methods for 'Breakpoint'

removeallmethods Breakpoint
removeallclassmethods Breakpoint

category: 'Instance initialization'
classmethod: Breakpoint
trappable: aBoolean
  "Default is that Breakpoints are signaled by the VM
   direct to the GCI, and they are not trappable in Smalltalk,
   and Breakpoint>>initialize is not run.

   To override the default, execute
     Breakpoint trappable: true
   and they will be signalled as normal trappable exceptions.
   Returns a Boolean, previous state from the VM .
  "
  <primitive: 829>
  ^ self _primitiveFailed: #trappable:
%

!		Instance methods for 'Breakpoint'

category: 'Instance initialization'
method: Breakpoint
initialize
  gsNumber := ERR_Breakpoint.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Instance initialization'
method: Breakpoint
resignalAs: anException
  "not working yet."
  ^ super resignalAs: anException
%

category: 'Instance initialization'
method: Breakpoint
resume
  "Resume execution at the step point where this Breakpoint was signalled
   and attempt to execute the bytecode at that step point."

  ^ self resume: nil
%

category: 'Instance initialization'
method: Breakpoint
resume: aValue
  "Resume execution at the step point where this Breakpoint was signalled
   and attempt to execute the bytecode at that step point."

  ^ super resume: aValue
%

category: 'Instance initialization'
method: Breakpoint
return
  "Return from the on:do: which installed a handler for this Breakpoint.
   Set the current GsProcess' state so that the breakpoint is reenabled."

  GsProcess _current clearLastBreakpointHistory .
  ^ super return
%

category: 'Instance initialization'
method: Breakpoint
return: aValue
  "Return from the on:do: which installed a handler for this Breakpoint.
   Set the current GsProcess' state so that the breakpoint is reenabled."

  GsProcess _current clearLastBreakpointHistory .
  ^ super return: aValue
%

! Class extensions for 'CannotReturn'

!		Instance methods for 'CannotReturn'

removeallmethods CannotReturn
removeallclassmethods CannotReturn

category: 'Instance initialization'
method: CannotReturn
initialize
  gsNumber := ERR_CannotReturn.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'Character'

!		Class methods for 'Character'

removeallmethods Character
removeallclassmethods Character

category: 'Deprecated Character Data Table'
classmethod: Character
activateCharTablesFromFile: file

"Install #CharacterDataTables from the passivated contents of a file.

Don't forget to commit to make the change permanent.

You must be SystemUser to execute this method.

See passivateCharTablesToFile: for the other side of this mechanism.

Use $GEMSTONE/goodies/CharacterTableUnicode.dat
to enable full 16-bit Unicode support.

See $GEMSTONE/goodies/UnicodeData.gs for
more info on Unicode Standards support.
"
self deprecated:  'Character class>>activateCharTablesFromFile: Deprecated as of GS/64 3.1, use Unicode classes' .

" Check that we're SystemUser "
System myUserProfile userId = 'SystemUser' ifFalse: [
   self _halt: 'Only SystemUser may execute this method' ].

(Globals at: #CharacterDataTables ifAbsent: [
    Globals at: #CharacterDataTables put: nil ] ).
Globals at: #CharacterDataTables put:
    ( PassiveObject fromServerTextFile: file ) activate.
^ self _loadCharTables
%

category: 'Deprecated Character Data Table'
classmethod: Character
categoryId: aSymbol

"Given a character category symbol, return the numeric id.  Note the
 indexes are not the same as used in ICU.

See Character>>categorySymbol: for symbol meanings. "
self deprecated:  'Charater class>>categoryId: Deprecated as of GS/64 3.1, use Unicode classes' .

^ #( #Lu #Ll #Lt #Lm #Lo #Mn #Mc #Me #Nd #Nl #No #Pc #Pd #Ps
   #Pe #Pi #Pf #Po #Sm #Sc #Sk #So #Zs #Zl #Zp #Cc #Cf #Cs #Co #Cn)
   indexOf: aSymbol
%

category: 'Deprecated Character Data Table'
classmethod: Character
categorySymbol: id

"Given a character category id, return the category symbol. Note the ids are
 not the same as used in ICU.

Category symbols are taken from the Unicode Standard:

#Lu  - Letter, Uppercase
#Ll - Letter, Lowercase
#Lt  - Letter, Titlecase
#Lm - Letter, Modifier
#Lo - Letter, Other
#Mn - Mark, Nonspacing
#Mc - Mark, Spacing Combining
#Me - Mark, Enclosing
#Nd - Number, Decimal Digit
#Nl - Number, Letter
#No - Number, Other
#Pc - Punctuation, Connector
#Pd - Punctuation, Dash
#Ps - Punctuation, Open/Start
#Pe - Punctuation, Close/End
#Pi - Punctuation, Initial Quote
#Pf - Punctuation, Final Quote
#Po - Punctuation, Other
#Sm - Symbol, Math
#Sc - Symbol, Currency
#Sk - Symbol, Modifier
#So - Symbol, Other
#Zs - Separator, Space
#Zl - Separator, Line
#Zp - Separator, Paragraph
#Cc - Other, Control
#Cf - Other, Format
#Cs - Other, Surrogate
#Co - Other, Private Use
#Cn - Other, Not Assigned
"
self deprecated:  'Character class>>categorySymbol: Deprecated as of GS/64 3.1, use Unicode classes' .

^ #( #Lu #Ll #Lt #Lm #Lo #Mn #Mc #Me #Nd #Nl #No #Pc #Pd #Ps
   #Pe #Pi #Pf #Po #Sm #Sc #Sk #So #Zs #Zl #Zp #Cc #Cf #Cs #Co #Cn)
   at: id
%

category: 'Deprecated Character Data Table'
classmethod: Character
charTables

" Reconstruct structured character data tables from the raw byte arrays
  stored in Globals at: #CharacterDataTables

Returns:

An Array of elements, arranged according to collate order,
each element an Array of 4 or 5 entries:

1.  The character for this entry.
2.  The symbolic character category code.
3.  Uppercase character ( if a letter ) / Numerator ( if numeric ).
4.  Lowercase character ( if a letter ) / Denominator ( if numeric/fraction ).
5.  (Optional)  Titlecase character ( if a letter )

"

| tables result dispatchBA indexBA categoryBA mainBA titleCaseBA
  titleCaseData titleCaseEntry size entry category id  chCls |
self deprecated:  'Character class>>charTables Deprecated as of GS/64 3.1, use Unicode classes' .

result := Array new.
tables := Globals at: #CharacterDataTables otherwise: nil .
tables ifNil:[ tables := Character _fetchCharTables ].

dispatchBA := tables at: 1.
indexBA := tables at: 2.
categoryBA := tables at: 3.
mainBA := tables at: 4.
titleCaseBA := tables at: 5.

" Construct TitleCase Data for later use.. "
titleCaseData := { } .
0 to: ( titleCaseBA size / 8 - 1 ) do: [ :i |
   entry := { } .
   entry add: ( titleCaseBA unsigned32At: ( i * 8 + 1 )).
   entry add: ( titleCaseBA unsigned32At: ( i * 8 + 5 )).
   titleCaseData add: entry ].

" Now reconstruct character data table "
size := mainBA size / 8.
chCls := Character .
0 to: size - 1 do: [ :i |
   entry := { } .
   id :=  ( indexBA unsigned32At: ( i * 4 + 1 )).
   entry add: ( chCls withValue: id ).
   category := self categorySymbol: ( categoryBA unsigned8At: ( i + 1 )).
   entry add: category.
   ( #( #Lu #Ll #Lt ) includes: category )
      ifTrue: [
         entry add:
            ( chCls withValue: ( mainBA unsigned32At: ( i * 8 + 1 ))).
         entry add:
            ( chCls withValue: ( mainBA unsigned32At: ( i * 8 + 5 ))).
         titleCaseEntry :=
            titleCaseData detect: [ :x | ( x at: 1 ) = id ] ifNone: [ nil ].
         ( titleCaseEntry == nil ) ifFalse: [
            entry add: ( chCls withValue: ( titleCaseEntry at: 2 )) ] ]
      ifFalse: [
         entry add: ( mainBA unsigned32At: ( i * 8 + 1 )).
         entry add: ( mainBA unsigned32At: ( i * 8 + 5 )) ].
   result add: entry ].
^ result
%

category: 'Instance Creation'
classmethod: Character
codePoint: anInteger

"Returns the Character with the specified value.
 Allowable range is 0 <= anInteger <= 16r10FFFF "

<primitive: 72>

anInteger _validateClass: SmallInteger.
OutOfRange new name:'anInteger' min: 0 max: 16r10FFFF actual: anInteger ; signal
%

category: 'Printable Characters'
classmethod: Character
digits

"Returns an InvariantArray containing Characters representing
 digits 0 through 9."

^#($0 $1 $2 $3 $4 $5 $6 $7 $8 $9)
%

category: 'Instance Creation'
classmethod: Character
fromStream: aStream

"Returns the next Character in the stream aStream."

self _checkReadStream: aStream forClass: String.
^ aStream next.
%

category: 'Instance Creation'
classmethod: Character
fromString: aString

"If aString is a one-Character String, returns the Character in aString.
 Otherwise, generates an error."

aString _validateClass: String.
(aString size == 1)
  ifTrue: [ ^ aString at: 1 ]
  ifFalse: [ self _errIncorrectFormat: aString ]
%

category: 'Deprecated Character Data Table'
classmethod: Character
installCharTables: table

"Converts a structured character data table into appropriately formated
byte arrays and then places them into Globals at: #CharacterDataTables
for use in this and subsequent sessions.  This operation *does not*
do a commit -- follow up with a commit if you wish to make this change
valid for subsequent sessions.

WARNINGS:

Installing incorrectly formatted character table data will break
character/string operations, including command line processing to a point
where the system will be impossible to use.  In this case, clear the
installation by doing the following:

1.  From the OS, set the host machine environmental parameter
    GS_DISABLE_CHARACTER_TABLE_LOAD to some value
    (it's the presence of this parameter that enables the mechanism).

2.  Login a new topaz session

3.  Execute Globals at: #CharacterDataTables put: nil.

4.  Commit

Note that changing the collation order of characters in new
CharacterDataTables will break any indexes that are keyed off of
Strings/DoubleByteStrings.  Before changing the tables, remove all such
indexes, install the new tables, and then reconstruct the indexes.

You must be SystemUser to execute this method.

Table Format:

An Array of elements, arranged according to character collate order,
each element an Array of 4 or 5 entries:

1.  The character for this entry.
2.  The symbolic character category code.
    See Character>>categorySymbol: for a list.
3.  Uppercase character ( if a letter ) / Numerator ( if numeric ).
4.  Lowercase character ( if a letter ) / Denominator ( if numeric/fraction ).
5.  (Optional)  Titlecase character

"

| tables dispatchBA indexBA categoryBA mainBA titleCaseBA
  mainSize dispatchSize titleCaseSize titleCaseIndex entry item id  |
self deprecated:  'Character class>>installCharTables:  Deprecated as of GS/64 3.1, use Unicode classes' .

" Check that we're SystemUser "
System myUserProfile userId = 'SystemUser' ifFalse: [
   self _halt:'Only SystemUser may execute this method' ].

" Generate the ByteArrays "
mainSize := table size.
dispatchSize := 0.
titleCaseSize := 0.
indexBA := ByteArray new: ( mainSize * 4 ).
categoryBA := ByteArray new: mainSize.
mainBA := ByteArray new: ( mainSize * 8 ).
" First pass: do most of the work,
  get info on sizes for dispatch and titleCase tables "
0 to: mainSize - 1 do: [ :i |
   entry := table at: i + 1.
   id := ( entry at: 1 ) codePoint.
   ( id > dispatchSize ) ifTrue: [ dispatchSize := id ].
   ( entry size > 4 ) ifTrue: [ titleCaseSize := titleCaseSize + 1 ].
   indexBA unsigned32At: ( i * 4 + 1 ) put: id.
   categoryBA unsigned8At: ( i + 1 ) put:
      ( Character categoryId: ( entry at: 2 ) ).
   ((item := entry at: 3) == nil) ifFalse: [
      ( item isKindOf: Character )
         ifTrue: [ mainBA unsigned32At: ( i * 8 + 1 ) put: item codePoint ]
         ifFalse: [ mainBA signed32At: ( i * 8 + 1 ) put: item ] ].
   ((item := entry at: 4) == nil) ifFalse: [
      ( item isKindOf: Character )
         ifTrue: [ mainBA unsigned32At: ( i * 8 + 5 ) put: item codePoint ]
         ifFalse: [ mainBA signed32At: ( i * 8 + 5 ) put: item ] ] ].

" Second pass: fill in dispatch and titleCase tables "
dispatchSize := dispatchSize + 1.
dispatchBA := ByteArray new: ( dispatchSize * 4 ).
titleCaseBA := ByteArray new: ( titleCaseSize * 8 ).
titleCaseIndex := 0.
1 to: mainSize do: [ :i |
   entry := table at: i.
   id := ( entry at: 1 ) codePoint.
   dispatchBA unsigned32At: ( id * 4 + 1 ) put: ( i - 1 ).
   ( entry size > 4 ) ifTrue: [
      titleCaseBA unsigned32At:
         ( titleCaseIndex * 8 + 1 ) put:  id.
      titleCaseBA unsigned32At:
         ( titleCaseIndex * 8 + 5 ) put: ( entry at: 5 ) codePoint.
      titleCaseIndex := titleCaseIndex + 1 ] ].

" Setup the table array "
tables := Array new: 5.
tables at: 1 put: dispatchBA.
tables at: 2 put: indexBA.
tables at: 3 put: categoryBA.
tables at: 4 put: mainBA.
tables at: 5 put: titleCaseBA.

" Install on #CharacterDataTables "
(Globals at: #CharacterDataTables ifAbsent: [
   Globals at: #CharacterDataTables put: nil ] ).
Globals at: #CharacterDataTables put: tables.

" Now install in this session's internal tables "
self _dispatch: dispatchBA index: indexBA category: categoryBA
   main: mainBA titleCase: titleCaseBA.

^ tables
%

category: 'Non-Printable Characters'
classmethod: Character
lfValue

"Returns value of the ASCII line-feed Character."

^  10
%

category: 'Printable Characters'
classmethod: Character
lowercaseRoman

"Returns an InvariantArray containing all lower-case Roman ASCII
 characters in alphabetic order."

^#($a $b $c $d $e $f $g $h $i $j $k $l $m $n $o $p $q $r $s $t $u
   $v $w $x $y $z)
%

category: 'Instance Creation'
classmethod: Character
maximumCodePoint
  "Returns the maximum code point per Unicode standards"

  ^ 16r10FFFF
%

category: 'Instance Creation'
classmethod: Character
new

"Disallowed.  You may not create new instances of Character."

self shouldNotImplement: #new
%

category: 'Deprecated Character Data Table'
classmethod: Character
passivateCharTablesToFile: file

"Write the passivated contents of the #CharacterDataTables (if present)
to a file.

Useful for efficiently porting the CharacterDataTable to another stone.
See activateCharTablesFromFile: for the other side of this mechanism. "

self deprecated:  'Character class>>passivateCharTablesToFile: Deprecated as of GS/64 3.1, use Unicode classes' .

(Globals at: #CharacterDataTables ifAbsent: [ ^nil ] )
   passivate toServerTextFile: file
%

category: 'Non-Printable Characters'
classmethod: Character
space

"Returns the ASCII space Character."
"recompiled in charact2.gs"

^ $  .
%

category: 'Printable Characters'
classmethod: Character
uppercaseRoman

"Returns an InvariantArray containing all upper-case Roman ASCII
 characters in alphabetic order."

^#($A $B $C $D $E $F $G $H $I $J $K $L $M $N $O $P $Q $R $S $T $U
   $V $W $X $Y $Z)
%

category: 'Instance Creation'
classmethod: Character
withSortValue: anInt

"Will be deprecated.
 Legacy method returning character at a particular point in collation,
 not Unicode compatible"

<primitive: 546>
self _primitiveFailed: #withSortValue: args: { anInt }
%

category: 'Instance Creation'
classmethod: Character
withValue: anInteger

"Returns the Character with the specified value.
 Allowable range is 0 <= anInteger <= 16r10FFFF "

<primitive: 72>

anInteger _validateClass: SmallInteger.
OutOfRange new name:'anInteger' min: 0 max: 16r10FFFF actual: anInteger ; signal
%

category: 'Deprecated Character Data Table'
classmethod: Character
_dispatch: dispatchTable index: indexTable category: categoryTable
    main: mainTable titleCase: titleCaseTable

" Copy byte array tables to internal C tables for String comparisions

  dispatchTable , indexTable
     are ByteArrays containing big-endian 16 bit values

  mainTable, titleCaseTable
     are ByteArrays containing pairs of big-endian 16 bit values

  categoryTable is a ByteArrays containing 8 bit values .

  if dispatchTable == nil,  then other arguments are ignored, and
  the C tables are reset to the hardcoded defaults defined when
  the VM executable/shared library was built .
"

<primitive: 643>
self _primitiveFailed: #_dispatch:index:category:main:titleCase:
     args: { dispatchTable . indexTable .
             categoryTable . mainTable . titleCaseTable }
%

category: 'Deprecated Character Data Table'
classmethod: Character
_dumpCharTables

" Dump contents of character tables to stdout  "

   <primitive: 645>
self _primitiveFailed: #_dumpCharTables
%

category: 'Deprecated Character Data Table'
classmethod: Character
_fetchCharTables

" Fetch contents of character tables "

<primitive: 644>
self _primitiveFailed: #_fetchCharTables
%

category: 'Deprecated Character Data Table'
classmethod: Character
_loadCharTables

"Load the character data tables recorded in
( Globals at: #CharacterDataTables ) into the session's internal memory.

This data should be in the form of an Array containing 5 ByteArrays.

See Character>>installCharTables for information on how to configure this. "

| table dispatchBA indexBA categoryBA mainBA titleCaseBA |
self deprecated:  'Character class>>_loadCharTables Deprecated as of GS/64 3.1, use Unicode classes' .

table :=  Globals at: #CharacterDataTables otherwise: nil .
table ifNil:[ ^ false ].

dispatchBA := table at: 1.
indexBA := table at: 2.
categoryBA := table at: 3.
mainBA := table at: 4.
titleCaseBA := table at: 5.

self _dispatch: dispatchBA index: indexBA category: categoryBA
      main: mainBA titleCase: titleCaseBA.

^ table
%

category: 'Deprecated Character Data Table'
classmethod: Character
_resetCharTablesToCDefaults

" Resets the character data tables in the VM to the hardcoded defaults
  defined when the VM executable/shared library was built .
  This defaults are the same as those used at session initialization if
   (Globals at: #CharacterDataTables otherwise: nil) == nil
"

self deprecated:  'Character class>>_resetCharTablesToCDefaults  Deprecated as of GS/64 3.1, use Unicode classes' .

self _dispatch: nil index: nil category: nil main: nil titleCase: nil
%

!		Instance methods for 'Character'

category: 'Comparisons'
method: Character
< aCharacter

"Legacy method.
 Returns true if the Legacy sort order of the receiver is
 less than that of aCharacter."

<primitive: 61>

(aCharacter class == Character)
  ifTrue:[ self _primitiveFailed: #< args: { aCharacter }.
           self _uncontinuableError ].
aCharacter _validateClass: AbstractCharacter.
^ aCharacter > self
%

category: 'Comparisons'
method: Character
<= aCharacter

"Legacy method.
 Returns true if the Legacy sort order of the receiver is
 less than or equal to that of aCharacter."

<primitive: 181>

(aCharacter class == Character)
  ifTrue:[ self _primitiveFailed: #<= args: { aCharacter }.
           self _uncontinuableError ].
aCharacter _validateClass: AbstractCharacter.
^ aCharacter >= self
%

category: 'Comparisons'
method: Character
= aCharacter

"Returns true if the receiver and aCharacter have the same code point."

<primitive: 60>

(aCharacter isKindOf: AbstractCharacter)
  ifTrue: [ ^ aCharacter = self ]
  ifFalse: [ ^ false ]
%

category: 'Comparisons'
method: Character
> aCharacter

"Legacy method.
 Returns true if the Legacy sort order of the receiver is
 greater than that of aCharacter. "

<primitive: 182>

(aCharacter class == Character)
  ifTrue:[ self _primitiveFailed: #GT: args: { aCharacter }.
           self _uncontinuableError ].
aCharacter _validateClass: AbstractCharacter.
^ aCharacter < self
%

category: 'Comparisons'
method: Character
>= aCharacter

"Legacy method.
 Returns true if the Legacy sort order of the receiver is
 greater than or equal to that of aCharacter."

<primitive: 183>

(aCharacter class == Character)
  ifTrue:[ self _primitiveFailed: #>= args: { aCharacter }.
           self _uncontinuableError ].
aCharacter _validateClass: AbstractCharacter.
^ aCharacter <= self
%

category: 'Converting'
method: Character
asCharacter

"Returns the receiver."

^ self
%

category: 'Accessing'
method: Character
asciiValue

"Returns the Unicode value of the receiver (a SmallInteger)."

<primitive: 71>

self _primitiveFailed: #asciiValue .
self _uncontinuableError
%

category: 'Converting'
method: Character
asDigit

"Returns the digit value (0-9) of the receiver.  If the receiver is not
 a digit, this returns 0."

(self < $0 or:[ self > $9]) ifTrue: [^0].
^self codePoint - $0 codePoint
%

category: 'Converting'
method: Character
asFoldcase

"The result of calling u_foldCase(GCI_OOP_TO_CHAR(self), U_FOLD_CASE_DEFAULT)
 in libicu is returned as a Character

 The foldCase is the equivalent to use non-case-sensitive uses of strings or
 characters.
 It is recommended to use the asFoldcase method implemented
 in string classes MultiByteString and String because
 those methods take into account effects of adjacent characters
 on case folding."

^ self _unicodeStatus: 22
%

category: 'Accessing'
method: Character
asInteger

"Returns the Unicode value of the receiver (a SmallInteger)."

<primitive: 71>

self _primitiveFailed: #asInteger .
self _uncontinuableError
%

category: 'Converting'
method: Character
asLowercase
" Returns Unicode lowercase  equivalent of the receiver.
  If the receiver has no lowercase  equivalent, returns the receiver.

  calls u_tolower in libicu .
  same as java.lang.Character.toLowerCase() .

  Differs from the legacy method asLowercaseOld for code points
  for about 1000 code points between 256 and 67000 .

  This method only returns the simple, single-code point case mapping.
  Full case mappings should be used whenever possible because they produce
  better results by working on whole strings.
  They take into account the string context and the language and can map
  to a result string with a different length as appropriate.
  See asLowercaseForLocale: in classes String and MultiByteString .
"

<primitive: 963>
self _primitiveFailed: #asLowercase
%

category: 'Converting-Legacy'
method: Character
asLowercaseOld

"Legacy method. Not reliable for codepoints over 255 unless
 (deprecated) Character Data Tables installed.
 Returns a Character that is the lower-case character corresponding
 to the receiver.  If the receiver is lower-case or has no case, this
 returns the receiver itself."

<primitive: 74>

self _primitiveFailed: #asLowercaseOld .
self _uncontinuableError
%

category: 'Formatting'
method: Character
asString

"Returns a one-Character String or DoubleByteString containing the receiver."

<primitive: 56>

self _primitiveFailed: #asString .
self _uncontinuableError
%

category: 'Converting'
method: Character
asSymbol

"Returns a one-Character Symbol that represents the receiver."

^ self asString asSymbol
%

category: 'Converting'
method: Character
asTitlecase
" Returns the titlecase of the receiver.  This is either the uppercase, or
  the distinct titlecase character for the (few) characters that have a
  separate titlecase form. If the receiver has no case or is already title
  case, returns the receiver.

  calls u_totitle in libicu .
  same as java.lang.Character.toTitleCase() .

  This method only returns the simple, single-code point case mapping.
  Full case mappings should be used whenever possible because they produce
  better results by working on whole strings.
  They take into account the string context and the language and can map
  to a result string with a different length as appropriate.
  See asTitlecaseForLocale: in classes String and MultiByteString .
"
<primitive: 964>
self _primitiveFailed: #asUnicodeTitlecase
%

category: 'Converting'
method: Character
asUppercase

" Returns Unicode uppercase equivalent of the receiver.
  If the receiver has no uppercase equivalent, returns the receiver.

  calls u_toupper in libicu .
  same as java.lang.Character.toUpperCase() .

  Differs from the legacy method asUppercaseOld for
  code points 181, 255, and about 1000 code points >= 256 .

  This method only returns the simple, single-code point case mapping.
  Full case mappings should be used whenever possible because they produce
  better results by working on whole strings.
  They take into account the string context and the language and can map
  to a result string with a different length as appropriate.
  See asUppercaseForLocale: in classes String and MultiByteString .
"

<primitive: 968>
self _primitiveFailed: #asUppercase
%

category: 'Converting-Legacy'
method: Character
asUppercaseOld

"Legacy method. Not reliable for codepoints over 255 unless
 (deprecated) Character Data Tables installed.
 Returns a Character that is the upper-case character corresponding
 to the receiver.  If the receiver is upper-case or has no case, this
 returns the receiver itself."

<primitive: 73>

self _primitiveFailed: #asUppercaseOld .
self _uncontinuableError
%

category: 'Accessing'
method: Character
codePoint

"Returns the Unicode value of the receiver (a SmallInteger)."

<primitive: 71>

self _primitiveFailed: #codePoint .
self _uncontinuableError
%

category: 'Unicode'
method: Character
compareTo: aCharacter collator: anIcuCollator

"Returns -1, 0, or 1 depending on how a Unicode16 string holding
 the receiver compares to a Unicode16 string holding the argument using
 the specified IcuCollator.
 anIcuCollator == nil is interpreted as   IcuCollator default ."

<primitive: 980>
 "primitive returns nil if aCharacter is not a Character"
  anIcuCollator ifNil:[ (System __sessionStateAt: 20) ifNil:[
     ^ self compareTo: aCharacter collator: IcuCollator default
  ]].
  self _primitiveFailed: #compareTo:collator: args: { aCharacter . anIcuCollator }
%

category: 'Private'
method: Character
containsIdentity

"Private."

^true
%

category: 'Copying'
method: Character
copy

"Returns the receiver.  (Does not create a new Character.)"

^self
%

category: 'Converting'
method: Character
digitValue

"If the result of calling u_charDigitValue(GCI_OOP_TO_CHAR(self)) in libicu
 is -1, returns nil.
 Otherwise the result from u_charDigitValue is returned as a SmallInteger.

 Results differ from legacy method digitValueOld for 450 code points
 between 1632 and 120831 "

^ self _unicodeStatus: 20
%

category: 'Converting'
method: Character
digitValueInRadix: radixSmallInt

"Returns a SmallInteger representing the value of the receiver
 in the given radix.
 Returns nil if if the receiver is not a digit in the given radix.
 If radixSmallInt < 2 or > 36, signals an OutOfRange error .

 The result is computed by calling u_digit(GCI_OOP_TO_CHAR(self), radix)
 in libicu."

<primitive: 966>

radixSmallInt _validateClass: SmallInteger .
(radixSmallInt < 2 or:[ radixSmallInt > 36]) ifTrue:[
   OutOfRange new
     name: 'radixSmallInt' min: 2 max: 36 actual: radixSmallInt ;
     signal
].
ArgumentError signal:'no valid digit translation'.
 ^ self _primitiveFailed: #digitValueInRadix: args: { radixSmallInt }
%

category: 'Converting-Legacy'
method: Character
digitValueInRadixOld: radix

"Legacy method.
 Returns a SmallInteger representing the value of the receiver, a digit, or
 returns nil if the receiver is not a digit in the given radix."

| val up |
radix == 10 ifTrue: [ ^self digitValueOld ].
radix < 10 ifTrue: [
  val := self digitValueOld .
  val >= radix ifTrue: [ ^nil ].
  ^val
].
val := self digitValueOld .
val ~~ nil ifTrue: [ ^val ].
up := self asUppercaseOld.
($A <= up and: [ up <= (self class withValue: ($A codePoint + radix - 11)) ])
ifTrue: [ ^(up codePoint - $A codePoint) + 10 ]
ifFalse: [ ^nil ]
%

category: 'Converting-Legacy'
method: Character
digitValueOld

"Legacy method.
 Returns a SmallInteger representing the value of the receiver,
 a digit, or returns nil if the receiver is not a digit."

(self isDigitOld)
  ifTrue:[ ^ self codePoint - $0 codePoint]
  ifFalse:[ ^ nil]
%

category: 'Formatting'
method: Character
displayWidth

"Returns the width necessary to display the receiver.
 For a Character, this method always returns 1."

^ 1
%

category: 'Case-Insensitive Comparisons'
method: Character
equalsNoCase: aCharacter

"Returns true if the receiver is the same Character as the argument regardless
 of case or internal representation.
 using equivalent of asUppercase before comparing."

<primitive: 979>
"If the primitive fails,
 then aCharacter must not be an instance of Character"

^ aCharacter isEquivalent: self
%

category: 'Case-Insensitive Comparisons'
method: Character
equalsNoCaseOld: aCharacter

"Legacy method. Not reliable for codepoints over 255 unless
 (deprecated) Character Data Tables installed.
 Returns true if the receiver is the same Character as the argument regardless
 of case or internal representation,
 using equivalent of asUppercaseOld before comparing."

<primitive: 75>
"If the primitive fails,
 then aCharacter must not be an instance of Character"

^ aCharacter isEquivalent: self
%

category: 'Grease'
method: Character
greaseInteger
"Returns the Unicode value of the receiver (a SmallInteger)."
<primitive: 71>
self _primitiveFailed: #greaseInteger .
self _uncontinuableError
%

category: 'Testing'
method: Character
hasMirror

"Return true if the receiver has a mirrored equivalent, E.g. s either the open
 or close of mirrored pairs such as ${ $}, $< $>, etc.

 Calls u_isMirrored(GCI_OOP_TO_CHAR(self)) in libicu .
 "

^ self _unicodeStatus: 19
%

category: 'Testing'
method: Character
isAlphabetic

"The result of calling u_isUAlphabetic(GCI_OOP_TO_CHAR(self)) in libicu is
 returned as a Boolean.  Result is TRUE if receiver code point has the
 Alphabetic Unicode property."

^ self _unicodeStatus: 1
%

category: 'Testing'
method: Character
isAlphaNumeric

"The result of calling u_isalnum(GCI_OOP_TO_CHAR(self)) in libicu is returned
 as a Boolean.
 Differs subtantially from the legacy method isAlphaNumericOld
 for codePoints >= 256 "

^ self _unicodeStatus: 6
%

category: 'Testing-Legacy'
method: Character
isAlphaNumericOld

"Legacy method. Not reliable for codepoints over 255 unless
 (deprecated) Character Data Tables installed.
 Returns true if the receiver is a Roman letter or digit.  Returns false
 otherwise."

 | cat |
 cat := self _category .
 ^ cat == 9 or:[ cat < 4 ]
%

category: 'Testing'
method: Character
isDigit

"The result of calling u_isdigit(GCI_OOP_TO_CHAR(self)) in libicu is returned
 as a Boolean.
 Returns true for characters with Unicode general category 'Nd'
 (decimal digit numbers).
 Same as java.lang.Character.isDigit().

 Differs from the legacy method isDigitOld for 450 code points between
 1632 and 120831, which are considered digits in the Unicode standard,
 and for which isDigitOld returns false."

^ self _unicodeStatus: 4
%

category: 'Testing-Legacy'
method: Character
isDigitOld

"Legacy method.
 Returns true if the receiver is a digit.  Returns false otherwise."

^ self _category == 9
%

category: 'Case-Insensitive Comparisons'
method: Character
isEquivalent: aCharacter

"Returns true if the receiver is the same Character as the argument regardless
 of case or internal representation.
 using equivalent of asUppercase before comparing."

<primitive: 979>
"If the primitive fails,
 then aCharacter must not be an instance of Character"

^ aCharacter isEquivalent: self
%

category: 'Case-Insensitive Comparisons'
method: Character
isEquivalentOld: aCharacter

"Legacy method. Not reliable for codepoints over 255 unless
 (deprecated) Character Data Tables installed.
 Returns true if the receiver is the same Character as the argument regardless
 of case or internal representation,
 using equivalent of asUppercaseOld before comparing."

<primitive: 75>
"If the primitive fails,
 then aCharacter must not be an instance of Character"

^ aCharacter isEquivalent: self
%

category: 'Unicode'
method: Character
isGraphic

"Returns true if the receiver is a graphic character; a printable character,
 excluding separators. Return false for characters with general categories Cc (control
 codes), Cf (format controls), Cs (surrogates), Cn (unassigned), and Z (separators).

 Calls u_isgraph(GCI_OOP_TO_CHAR(self)) in libicu"

^ self _unicodeStatus: 9
%

category: 'Testing'
method: Character
isHexDigit

"Returns true if the receiver is a hexadecimal digit.

 Returns true for characters with Unicode general category 'Nd' (decimal digit numbers).
 as well as Latin letters a-f and A-F in both ASCII and Fullwidth ASCII.
 (That is, for letters with code points
  16r0041..16r0046, 16r0061..16r0066, 16rFF21..16rFF26, 16rFF41..16rFF46.)

 In order to narrow the definition of hexadecimal digits to only ASCII
 characters, use  (c codePoint <= 16r7F and:[ unicodeIsXDigit ) .

 calls u_isxdigit in libicu .
"

^ self _unicodeStatus: 7
%

category: 'Testing'
method: Character
isLetter

"The result of calling u_isalpha(GCI_OOP_TO_CHAR(self)) in libicu is
 returned as a Boolean.
 Result is true for code points Unicode general categories 'L' (letters).

 Same as isAlphabetic for code points <= 836 .
 Same as java.lang.Character.isLetter() .

 Differs substantially from legacy isLetterOld for code points >= 256"

^ self _unicodeStatus: 5
%

category: 'Testing-Legacy'
method: Character
isLetterOld

"Legacy method. Not reliable for codepoints over 255 unless
 (deprecated) Character Data Tables installed.
 Returns true if the receiver is a Roman letter.  Returns false otherwise."

^ self _category < 4
%

category: 'Testing'
method: Character
isLowercase
"Returns true if the receiver's code point has
 the Unicode general category  'Ll' (lowercase letter).

 Calls u_islower in libicu .
 Same as java.lang.Character.isLowerCase() .

 This misses some characters that are also lowercase but have
 a different general category value.
 In order to include those, use unicodeIsULowercase .
 isLowercase and unicodeIsULowercase differ for code points 170, 186,
 and about 180 code points >= 256 .

 Differs from legacy method isLowercaseOld
 for code points 170, 186, and about 1700 code points >= 256 .
"

^ self _unicodeStatus: 25
%

category: 'Testing-Legacy'
method: Character
isLowercaseOld

"Legacy method. Not reliable for codepoints over 255 unless
 (deprecated) Character Data Tables installed.
 Returns true if the receiver is a lower-case character.  Returns false
 otherwise."

^ self _category == 2
%

category: 'Unicode'
method: Character
isNumeric
  "The result of calling
   u_getIntPropertyValue(GCI_OOP_TO_CHAR(self), UCHAR_NUMERIC_TYPE) in libicu
   is returned as a Boolean .
   Will return true for characters having either Numeric_Type Numeric
   (such as Han characters in Chinese-style number format) or Numeric_Type Decimal."

^ self _unicodeStatus: 27
%

category: 'Testing-Legacy'
method: Character
isNumericOld

"Legacy method.
 Returns true if the receiver contains numeric content ( category is #Nd #Nl #No).
 Returns false otherwise."

| category |
category := self _category.
category < 9 ifTrue: [ ^ false ].
category < 12 ifTrue: [ ^ true ].
^ false
%

category: 'Testing'
method: Character
isPrintable

 "Return true if the receiver is a printable character; this includes characters
  with any general category other than 'C' (controls).

  Calls u_isprint(GCI_OOP_TO_CHAR(self)) in libicu i.
"

^ self _unicodeStatus: 17
%

category: 'Testing'
method: Character
isPunctuation

"Return a boolean, true if the receiver is a punctuation character, false otherwise.

 Calls u_ispunct(GCI_OOP_TO_CHAR(self)) in libicu"

^ self _unicodeStatus: 8
%

category: 'Testing'
method: Character
isSeparator

"Returns true if codepoint of the receiver satisfies
    u_isISOControl(ch) || (ch == 32) || (ch == 160)
 using libicu.

 which is the same as isSeparator in previous releases."

^ self _unicodeStatus: 29
%

category: 'Testing'
method: Character
isSpecial

"Returns true if the receiver is a special object."

^ true
%

category: 'Testing'
method: Character
isTitlecase
"Returns true if the receiver's code point has the Unicode general category
 'Lt' (titlecase letter).

 Note that only titlecase characters return true. While asTitlecase converts
 to uppercase or titlecase, isTitlecase only returns true for titlecase.

 Calls u_istitle in libicu .
 Same as java.lang.Character.isTitleCase() .

 Differs from legacy method isTitlecaseOld for titlecase characters, when
 character data tables not installed; about 30 code points between 400 and 8100.
 The first code point for which isTitlecase returns true is 453.
"

^ self _unicodeStatus: 26
%

category: 'Testing-Legacy'
method: Character
isTitlecaseOld

"Legacy method.
 Returns true if the receiver is a title-case character.  Returns false
 otherwise."

^ self _category == 3
%

category: 'Testing'
method: Character
isUppercase
"Returns true if the receiver's code point has
 the Unicode general category  'Lu' (uppercase letter).

 Calls u_isupper in libicu .
 Same as java.lang.Character.isUpperCase().

 This misses some characters that are also uppercase but
 have a different general category value.  There are about 50 such
 differences between codepoints 8500 and 9500 .
 In order to include those, use unicodeIsUUppercase .

 Differs from legacy method isUppercaseOld
 for about 1400 code points >= 256 .
"

^ self _unicodeStatus: 24
%

category: 'Testing-Legacy'
method: Character
isUppercaseOld

"Legacy method. Not reliable for codepoints over 255 unless
 (deprecated) Character Data Tables installed.
 Returns true if the receiver is an upper-case character.  Returns false
 otherwise."

^ self _category == 1
%

category: 'Testing-Legacy'
method: Character
isVowel

"Legacy method. Vowel is not a unicode property.  Results may not be correct
 for code points over 255.
 Returns true if the receiver is a vowel ('Y' is considered to be a vowel).
 Returns false otherwise.

 This code assumes that the Legacy collation sequence places all uppercase variations
 of a given letter (including various diacritical marks) immediately following the
 plain version of the letter.
"
| ucs arr chCls |
ucs := self asUppercaseOld sortValue.
arr := #( $A $E $I $O $U $Y ) .
chCls := Character .
1 to: arr size do:[:j | | v |
  v := arr at: j .
  ucs < v sortValue  ifTrue: [ ^ false ].
  ucs < ( chCls withValue: v codePoint  + 1 ) sortValue  ifTrue:[^ true ]
].
^ false
%

category: 'Converting'
method: Character
mirror

"If the argument is the open or close of a mirrored pair, such as $( $) or ${ $},
 return the other character, the mirror of the receiver. If the receiver does
 not have a mirror, return the receiver.

 Calls u_charMirror(GCI_OOP_TO_CHAR(self)) in libicu .
 "

^ self _unicodeStatus: 21
%

category: 'Converting'
method: Character
numericValue

 "Calls u_getNumericValue(GCI_OOP_TO_CHAR(self)) in libicu
  and returns an Integer, Fraction, Float, or nil.

  Differs from digitValue in that numericValue  method handles
  characters having Numeric_Type Numeric (such as Han characters in Chinese-style
  number format) in addition to Numeric_Type Decimal."

^ self _unicodeStatus: 28
%

category: 'Formatting'
method: Character
printOn: aStream

"Puts a displayable representation of the receiver on the given stream."

aStream nextPut: $$ .
aStream nextPut: self
%

category: 'Formatting'
method: Character
printString

"Returns a String whose contents are a displayable representation of the
 receiver."

"GemStone does not allow the creation of new kinds of Character, so there
 is no point in creating a stream and sending printOn:."

| result |
result := String new: 2 .
result at: 1 put: $$ .
result at: 2 put: self  .
^ result
%

category: 'Accessing'
method: Character
sortValue

"Will be deprecated.
 Legacy method providing index into collation sequence, not Unicode compatible"

<primitive: 545>
self _primitiveFailed: #sortValue
%

category: 'Unicode'
method: Character
unicodeCategory

  "Returns a Symbol , the 2-letter Unicode name of
   general category for the code point.
   Result is based on u_charType(GCI_OOP_TO_CHAR(self)) in libicu"

  ^ self _unicodeStatus: 30
%

category: 'Unicode'
method: Character
unicodeIsBase

"The result of calling u_isbase(GCI_OOP_TO_CHAR(self)) in libicu is
 returned as a Boolean.

 From libicu uchar.h :
   Returns true for general categories 'L' (letters), 'N' (numbers),
   'Mc' (spacing combining marks), and 'Me' (enclosing marks).

   Note that this is different from the Unicode definition in
   chapter 3.5, conformance clause D13,
   which defines base characters to be all characters (not Cn)
   that do not graphically combine with preceding characters (M)
   and that are neither control (Cc) or format (Cf) characters.
"

^ self _unicodeStatus: 18
%

category: 'Unicode'
method: Character
unicodeIsBlank

"The result of calling u_isblank(GCI_OOP_TO_CHAR(self)) in libicu is returned as a Boolean"

^ self _unicodeStatus: 10
%

category: 'Unicode'
method: Character
unicodeIsControl

"The result of calling u_iscntrl(GCI_OOP_TO_CHAR(self)) in libicu is returned as a Boolean"

^ self _unicodeStatus: 15
%

category: 'Unicode'
method: Character
unicodeIsDefined

"The result of calling u_isdefined(GCI_OOP_TO_CHAR(self)) in libicu is returned as a Boolean"

^ self _unicodeStatus: 11
%

category: 'Unicode'
method: Character
unicodeIsISOControl

"The result of calling u_isISOControl(GCI_OOP_TO_CHAR(self)) in libicu is returned as a Boolean"

^ self _unicodeStatus: 16
%

category: 'Unicode'
method: Character
unicodeIsJavaSpace

"The result of calling u_isJavaSpaceChar(GCI_OOP_TO_CHAR(self)) in libicu is returned as a Boolean"

^ self _unicodeStatus: 13
%

category: 'Unicode'
method: Character
unicodeIsSpace

"The result of calling u_isspace(GCI_OOP_TO_CHAR(self)) in libicu is returned as a Boolean"

^ self _unicodeStatus: 12
%

category: 'Unicode'
method: Character
unicodeIsULowercase

"The result of calling u_isULowercase(GCI_OOP_TO_CHAR(self)) in libicu is returned as a Boolean"

^ self _unicodeStatus: 2
%

category: 'Unicode'
method: Character
unicodeIsUUppercase

"The result of calling u_isUUppercase(GCI_OOP_TO_CHAR(self)) in libicu
 is returned as a Boolean.

 Result is true if the receiver has the Uppercase Unicode property.
 Different than isUppercase.
"

^ self _unicodeStatus: 3
%

category: 'Unicode'
method: Character
unicodeIsWhitespace

"The result of calling u_isWhitespace(GCI_OOP_TO_CHAR(self)) in libicu is returned as a Boolean"

^ self _unicodeStatus: 14
%

category: 'Unicode'
method: Character
unicodeType

"Return the general cateory SmallInteger for the receiver, by calling
 u_charType(GCI_OOP_TO_CHAR(self)) in libicui.  Same as java.lang.Character.getType().

 Values returned from this method do not have the same category equivalent as
 GemStone legacy category; do not use result with _categorySymbol:.
"

^ self _unicodeStatus: 23
%

category: 'Storing and Loading'
method: Character
writeTo: passiveObj

"Converts the receiver to its passive form and writes that information on
 passiveObj."

| av |
av := self codePoint .
av <= 16rFF ifTrue:[
  passiveObj nextPut: $$; nextPut: self
] ifFalse:[  | ccls b3 b2 b1 b0 |
  ccls := Character .
  b0 := av bitAnd: 16rFF .
  av <= 16rFFFF ifTrue:[
    b1 := av bitShift: -8 .
    passiveObj nextPut: $! ; nextPut: (ccls withValue: b1);
                nextPut: (ccls withValue: b0) .
  ] ifFalse:[
    av := av bitShift: -8 .
    b1 := av bitAnd: 16rFF .
    av := av bitShift: -8 .
    b2 := av bitAnd: 16rFF .
    av := av bitShift: -8 .
    b3 := av bitAnd: 16rFF .
    passiveObj nextPut: $& ; nextPut: (ccls withValue: b3);
          nextPut: (ccls withValue: b2) ;
          nextPut: (ccls withValue: b1) ;
          nextPut: (ccls withValue: b0) .
  ]
]
%

category: 'Private'
method: Character
_asSource

"Private."

^ self printString
%

category: 'Private'
method: Character
_category

"Returns the numeric category id for this character.  This numeric
 index is specific to the GemStone legacy implementation, and is not
 the same as the ICU index from unicodeType. Use with _categoryAsSymbol
 to determine the correct unicode category. Will not return correct
 results for Characters over 255 unless deprecated Character data tables
 are installed."

<primitive: 640>
self _primitiveFailed: #_category
%

category: 'Private'
method: Character
_categoryAsSymbol

" Returns the category symbol for this character.

Category symbols are taken from the Unicode Standard. Note that
the index of these is not the same as ICU.

#Lu  - Letter, Uppercase
#Ll - Letter, Lowercase
#Lt  - Letter, Titlecase
#Lm - Letter, Modifier
#Lo - Letter, Other
#Mn - Mark, Nonspacing
#Mc - Mark, Spacing Combining
#Me - Mark, Enclosing
#Nd - Number, Decimal Digit
#Nl - Number, Letter
#No - Number, Other
#Pc - Punctuation, Connector
#Pd - Punctuation, Dash
#Ps - Punctuation, Open/Start
#Pe - Punctuation, Close/End
#Pi - Punctuation, Initial Quote
#Pf - Punctuation, Final Quote
#Po - Punctuation, Other
#Sm - Symbol, Math
#Sc - Symbol, Currency
#Sk - Symbol, Modifier
#So - Symbol, Other
#Zs - Separator, Space
#Zl - Separator, Line
#Zp - Separator, Paragraph
#Cc - Other, Control
#Cf - Other, Format
#Cs - Other, Surrogate
#Co - Other, Private Use
#Cn - Other, Not Assigned

"

^ #( #Lu #Ll #Lt #Lm #Lo #Mn #Mc #Me #Nd #Nl #No #Pc #Pd #Ps
   #Pe #Pi #Pf #Po #Sm #Sc #Sk #So #Zs #Zl #Zp #Cc #Cf #Cs #Co #Cn)
at: self _category
%

category: 'Private'
method: Character
_isPasswordSymbol

| cat |
cat := self _category .
^ (cat >= 12) and:[ cat <= 21 ]
%

category: 'Private'
method: Character
_numericValue

"Legacy method.
 Returns the numeric value (if one exists) for this character, or nil,
 using the legacy character tables."
self deprecated:  'Character _numericValue  Deprecated as of GS/64 3.2, use numericValue'.
^ self __numericValue
%

category: 'Private'
method: Character
_type

"Returns 1 for alpha, 2 for digit, and 3 for special."

| cat |
cat := self _category .
cat == 9 ifTrue: [ ^ 2].
cat < 4 ifTrue: [ ^ 1].
^ 3
%

category: 'Private'
method: Character
_typeAsSymbol

"Private."

^ #( #alpha #digit #special ) at: self _type
%

category: 'Unicode'
method: Character
_unicodeStatus: opcode
"
    opcode  libicu function
    1      u_isUAlphabetic(ch) returns a Boolean
    2      u_isULowercase(ch)  returns a Boolean
    3      u_isUUppercase(ch)  returns a Boolean
    4      u_isdigit(ch)    returns a Boolean
    5      u_isalpha(ch)    returns a Boolean
    6      u_isalnum(ch)    returns a Boolean
    7      u_isxdigit(ch)   returns a Boolean
    8      u_ispunct(ch)    returns a Boolean
    9      u_isgraph(ch)    returns a Boolean
    10     u_isblank(ch)    returns a Boolean
    11     u_isdefined(ch)  returns a Boolean
    12     u_isspace(ch)    returns a Boolean
    13     u_isJavaSpaceChar(ch) returns a Boolean
    14     u_isWhiteSpace(ch)    returns a Boolean
    15     u_iscntrl(ch)         returns a Boolean
    16     u_isISOControl(ch)    returns a Boolean
    17     u_isprint(ch)         returns a Boolean
    18     u_isbase(ch)          returns a Boolean
    19     u_isMirrored(ch)  returns a Boolean
    20     u_charDigitValue  returns a SmallInteger
    21     u_charMirror      returns a Character
    22     u_foldCase        returns a Character
    23     u_charType        returns a SmallInteger
    24     u_isupper        returns a Boolean
    25     u_islower        returns a Boolean
    26     u_istitle        returns a Boolean
    27     u_getIntPropertyValue(c, UCHAR_NUMERIC_TYPE), returns a Boolean
    28     u_getNumericValue  returns a Float, SmallDouble or nil
    29     u_isISOControl(ch) || (ch == 32) || (ch == 160)   smalltalk isSeparator
    30     symbolic name of u_charType(ch), returns a Symbol
"
<primitive: 965>
opcode _validateClass: SmallInteger .
self _primitiveFailed: #_unicodeStatus: args: { opcode }
%

category: 'Private'
method: Character
__numericValue
"Returns the numeric value (if one exists) for this character, or nil,
 using the legacy character tables."
<primitive: 641>
self _primitiveFailed: #__numericValue
%

! Class extensions for 'CharacterCollection'

!		Class methods for 'CharacterCollection'

removeallmethods CharacterCollection
removeallclassmethods CharacterCollection

category: 'Formatting'
classmethod: CharacterCollection
charSize

"(Subclass responsibility.)
 Returns number of bytes that make up a character for instances of this class."

^ self subclassResponsibility: #charSize
%

category: 'Session Control'
classmethod: CharacterCollection
disableUnicodeComparisonMode

  "Causes subsequent sessions, after login, to use Legacy methods for
   #< #> #= and various methods in the indexing
   when a receiver is a kind of String or MultiByteString.

   Signals an Error if the current user is not SystemUser .

   Returns a Boolean , the previous state of
     (Globals at: #StringConfiguration) == Unicode16 ."

   | prev |
   System myUserProfile userId = 'SystemUser' ifFalse:[
     Error signal:'Only SystemUser should execute this method'.
   ].
   prev := (Globals at: #StringConfiguration otherwise: nil) == Unicode16 .
   Globals at: #StringConfiguration put: String .
   System commit .
   ^ prev
%

category: 'Session Control'
classmethod: CharacterCollection
enableUnicodeComparisonMode

  "Causes subsequent sessions, after login, to use Unicode methods for
   #< #> #= and various methods in the indexing
   when a receiver is a kind of String or MultiByteString.

   Signals an Error if the current user is not SystemUser .

   Returns a Boolean , the previous state of
     (Globals at: #StringConfiguration) == Unicode16 ."

   | prev |
   System myUserProfile userId = 'SystemUser' ifFalse:[
     Error signal:'Only SystemUser should execute this method'.
   ].
   prev := (Globals at: #StringConfiguration otherwise: nil) == Unicode16 .
   Globals at: #StringConfiguration put: Unicode16 .
   System commit .
   ^ prev
%

category: 'Deprecated'
classmethod: CharacterCollection
fromServerTextFile: aFileSpec

self deprecated: 'CharacterCollection class >> fromServerTextFile: deprecated long before v3.0. Use an instance of GsFile
 to access the file system.'.
 "For multi-byte characters, assumes the byte order of the file
 matches the current session's cpu's in-memory byte order.  "

^ self new _fromServerTextFile: aFileSpec
%

category: 'Instance Creation'
classmethod: CharacterCollection
fromStream: aStream width: anInteger

"Returns a new instance of the receiver's class that contains the next
 anInteger Characters of aStream."

| result |

self _checkReadStream: aStream forClass: CharacterCollection.
result:= self new: anInteger.
1 to: anInteger do: [ :i | result at: i put: aStream next ].
^ result
%

category: 'Session Control'
classmethod: CharacterCollection
isInUnicodeComparisonMode

  "Returns a Boolean, true if Unicode versions of methods
   #< #> #= and various methods in the indexing
   are used when a receiver is a kind of String or MultiByteString"

  ^ Unicode16 usingUnicodeCompares
%

category: 'Instance Creation'
classmethod: CharacterCollection
new
"(Subclass responsibility.)"

^ self subclassResponsibility: #new
%

category: 'Instance Creation'
classmethod: CharacterCollection
new: anInteger
"(Subclass responsibility.)"

^ self subclassResponsibility: #new:
%

category: 'Instance Creation'
classmethod: CharacterCollection
withAll: aSequenceableCollection

"Returns a new instance of the receiver that contains the elements in the
 argument aSequenceableCollection."

| result |
result:= self new.
aSequenceableCollection accompaniedBy: result do: [:res :each | res add: each].
^result
%

category: 'Indexing Support'
classmethod: CharacterCollection
_idxBasicCanCompareWithCharacterCollectionInstance: aCharacterCollection
  "Returns true if <aCharacterCollection> may be inserted into a basic BtreeNode
   whose #lastElementClass is the receiver (see RangeEqualityIndex
   class>>isBasicClass:)."

  "If using Unicode compares, then *String and Unicode* instances may be compared.
   If not using Unicode compares then *String and Unicode* instances may not be compared."

  ^ true
%

category: 'Indexing Support'
classmethod: CharacterCollection
_idxBasicCanCompareWithUnicodeInstance: aUnicodeString
  "Returns true if <aUnicodeString> may be inserted into a basic BtreeNode whose
   #lastElementClass is the receiver (see RangeEqualityIndex class>>isBasicClass:)."

  "If using Unicode compares, then *String and Unicode* instances may be compared.
   If not using Unicode compares then *String and Unicode* instances may not be compared."

  ^ Unicode16 usingUnicodeCompares
%

category: 'Instance Creation'
classmethod: CharacterCollection
_newString

"Returns a new instance of the receiver, or instance of String as appropriate.
 Reimplemented in subclasses as needed to handle canonical symbols."

^ self new
%

category: 'Private'
classmethod: CharacterCollection
_newString: aSize

"Returns a new instance of the receiver, or instance of String as appropriate.
 Reimplemented in subclasses as needed to handle canonical symbols."

^ self new: aSize
%

!		Instance methods for 'CharacterCollection'

category: 'Concatenating'
method: CharacterCollection
, aCharOrCharCollection

"Returns a new instance of the receiver's class that contains the elements of
 the receiver followed by the elements of aCharOrCharCollection.

 Warning: Creating a new instance and copying the receiver take time.  If you
 can safely modify the receiver, it can be much faster to use the addAll:
 method.  See the documentation of the Concatenating category of class
 SequenceableCollection for more details."

| newCollection |

newCollection := self copy.
newCollection addAll: aCharOrCharCollection.
^ newCollection
%

category: 'Comparing'
method: CharacterCollection
< aCharCollection

"Returns true if the receiver collates before the argument.  Returns false
 otherwise.
 (Subclass responsibility.)"

^ self subclassResponsibility: #<
%

category: 'Comparing'
method: CharacterCollection
<= aCharCollection

"Returns true if the receiver collates before the argument or if all of the
 corresponding Characters in the receiver and argument are equal.
 Returns false otherwise.
 (Subclass responsibility.)"

^ self subclassResponsibility: #<=
%

category: 'Comparing'
method: CharacterCollection
= aCharCollection

"Returns true if all of the corresponding Characters in the receiver and
 argument are equal.  Returns false otherwise.
 (Subclass responsibility.)"

^ self subclassResponsibility: #=
%

category: 'Comparing'
method: CharacterCollection
> aCharCollection

"Returns true if the receiver collates after the argument.  Returns false
 otherwise.
 (Subclass responsibility.)"

^ self subclassResponsibility: #>
%

category: 'Comparing'
method: CharacterCollection
>= aCharCollection

"Returns true if the receiver collates after the argument or if all of the
 corresponding Characters in the receiver and argument are equal.  Returns
 false otherwise.
 (Subclass responsibility.)"

^ self subclassResponsibility: #>=
%

category: 'Adding'
method: CharacterCollection
add: aCharOrCharColl

"Appends all of the elements of aCharOrCharColl to the receiver and returns
 aCharOrCharColl.  Returns aCharOrCharColl"

| index |

(aCharOrCharColl isKindOf: Collection) ifTrue:[
    index := self size.
    aCharOrCharColl accompaniedBy: self do: [:me :aChar |
      index := index + 1.
      me at: index put: aChar.
    ].
    ^ aCharOrCharColl.
].

^ self at: (self size + 1) put: aCharOrCharColl.
%

category: 'Adding'
method: CharacterCollection
addAll: aCharOrCharCollection

"Equivalent to add: aCharOrCharCollection."

^ self add: aCharOrCharCollection
%

category: 'Adding'
method: CharacterCollection
addCodePoint: aSmallInteger

 ^ self add: (Character codePoint: aSmallInteger)
%

category: 'Adding'
method: CharacterCollection
addLast: aCharOrCharCollection

"Equivalent to add: aCharOrCharCollection."

^ self add: aCharOrCharCollection
%

category: 'Updating'
method: CharacterCollection
addLineDelimiters

"Returns a copy of the receiver that contains each occurrence of the backslash
 Character replaced by the line-feed Character."

^ self copyReplaceAll: '\' with: (self class with: Character lf).
%

category: 'Formatting'
method: CharacterCollection
addLineWith: aCharacterCollection centeredToWidth: anInt

"Add a centered line of width anInt to the receiver.
 Returns the receiver."

| numSpaces aSpace|
aSpace := Character space.
numSpaces := anInt - aCharacterCollection size.
numSpaces <= 0 "No room, just do the add"
  ifTrue:[self addAll: aCharacterCollection.]
  ifFalse:[ | leftSpaces |
    leftSpaces := numSpaces // 2.
    leftSpaces timesRepeat:[self add: aSpace].
    self addAll: aCharacterCollection.
    (numSpaces - leftSpaces) timesRepeat:[self add: aSpace].
].
^self
%

category: 'Decrypting'
method: CharacterCollection
aesDecryptWith128BitKey: aKey salt: aSalt
"Decrypts the receiver using 128 bit AES decryption and places the result into
 a new instance of the class of the receiver.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self aesDecryptWith128BitKey: aKey salt: aSalt into: nil
%

category: 'Decrypting'
method: CharacterCollection
aesDecryptWith128BitKey: aKey salt: aSalt into: destObjOrNil
"Decrypts the receiver using 128 bit AES decryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey salt: aSalt opCode: -1 into: destObjOrNil
%

category: 'Decrypting'
method: CharacterCollection
aesDecryptWith192BitKey: aKey salt: aSalt
"Decrypts the receiver using 192 bit AES decryption and places the result into
 a new instance of the class of the receiver.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self aesDecryptWith192BitKey: aKey salt: aSalt into: nil
%

category: 'Decrypting'
method: CharacterCollection
aesDecryptWith192BitKey: aKey salt: aSalt into: destObjOrNil
"Decrypts the receiver using 192 bit AES decryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey salt: aSalt opCode: -2 into: destObjOrNil
%

category: 'Decrypting'
method: CharacterCollection
aesDecryptWith256BitKey: aKey salt: aSalt
"Decrypts the receiver using 256 bit AES decryption and places the result into
 a new instance of the class of the receiver.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self aesDecryptWith256BitKey: aKey salt: aSalt into: nil
%

category: 'Decrypting'
method: CharacterCollection
aesDecryptWith256BitKey: aKey salt: aSalt into: destObjOrNil
"Decrypts the receiver using 256 bit AES decryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey salt: aSalt opCode: -3 into: destObjOrNil
%

category: 'Encrypting'
method: CharacterCollection
aesEncryptWith128BitKey: aKey salt: aSalt
"Encrypts the receiver using 128 bit AES encryption and places the result into
 a new instance of the class of the receiver.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self aesEncryptWith128BitKey: aKey salt: aSalt into: nil
%

category: 'Encrypting'
method: CharacterCollection
aesEncryptWith128BitKey: aKey salt: aSalt into: destObjOrNil
"Encrypts the receiver using 128 bit AES encryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey salt: aSalt opCode: 1  into: destObjOrNil
%

category: 'Encrypting'
method: CharacterCollection
aesEncryptWith192BitKey: aKey salt: aSalt
"Encrypts the receiver using 192 bit AES encryption and places the result into
 a new instance of the class of the receiver.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self aesEncryptWith192BitKey: aKey salt: aSalt into: nil
%

category: 'Encrypting'
method: CharacterCollection
aesEncryptWith192BitKey: aKey salt: aSalt into: destObjOrNil
"Encrypts the receiver using 192 bit AES encryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey salt: aSalt opCode: 2 into: destObjOrNil
%

category: 'Encrypting'
method: CharacterCollection
aesEncryptWith256BitKey: aKey salt: aSalt
"Encrypts the receiver using 256 bit AES encryption and places the result into
 a new instance of the class of the receiver.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self aesEncryptWith256BitKey: aKey salt: aSalt into: nil
%

category: 'Encrypting'
method: CharacterCollection
aesEncryptWith256BitKey: aKey salt: aSalt into: destObjOrNil
"Encrypts the receiver using 256 bit AES encryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey salt: aSalt opCode: 3 into: destObjOrNil
%

category: 'Authenticated Decrypting'
method: CharacterCollection
aesGcmDecryptWith128BitKey: aKey salt: aSalt into: destObjOrNil tag: tag extraData: eData
"Decrypts the receiver using 128 bit AES-GCM decryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey
       salt: aSalt
       opCode: -7
       into: destObjOrNil
       tag: tag
       extraData: eData
%

category: 'Authenticated Decrypting'
method: CharacterCollection
aesGcmDecryptWith192BitKey: aKey salt: aSalt into: destObjOrNil tag: tag extraData: eData
"Decrypts the receiver using 192 bit AES-GCM decryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey
       salt: aSalt
       opCode: -8
       into: destObjOrNil
       tag: tag
       extraData: eData
%

category: 'Authenticated Decrypting'
method: CharacterCollection
aesGcmDecryptWith256BitKey: aKey salt: aSalt into: destObjOrNil tag: tag extraData: eData
"Decrypts the receiver using 256 bit AES-GCM decryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey
       salt: aSalt
       opCode: -9
       into: destObjOrNil
       tag: tag
       extraData: eData
%

category: 'Authenticated Encrypting'
method: CharacterCollection
aesGcmEncryptWith128BitKey: aKey salt: aSalt into: destObjOrNil tag: tag extraData: eData
"Encrypts the receiver using 128 bit AES-GCM encryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey salt: aSalt opCode: 7 into: destObjOrNil tag: tag extraData: eData
%

category: 'Authenticated Encrypting'
method: CharacterCollection
aesGcmEncryptWith192BitKey: aKey salt: aSalt into: destObjOrNil tag: tag extraData: eData
"Encrypts the receiver using 192 bit AES-GCM encryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey
       salt: aSalt
       opCode: 8
       into: destObjOrNil
       tag: tag
       extraData: eData
%

category: 'Authenticated Encrypting'
method: CharacterCollection
aesGcmEncryptWith256BitKey: aKey salt: aSalt into: destObjOrNil tag: tag extraData: eData
"Encrypts the receiver using 256 bit AES-GCM encryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey
       salt: aSalt
       opCode: 9
       into: destObjOrNil
       tag: tag
       extraData: eData
%

category: 'Decrypting'
method: CharacterCollection
aesOcbDecryptWith128BitKey: aKey salt: aSalt into: destObjOrNil tag: tag extraData: eData
"Decrypts the receiver using 128 bit AES-OCB decryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey
       salt: aSalt
       opCode: -4
       into: destObjOrNil
       tag: tag
       extraData: eData
%

category: 'Authenticated Decrypting'
method: CharacterCollection
aesOcbDecryptWith192BitKey: aKey salt: aSalt into: destObjOrNil tag: tag extraData: eData
"Decrypts the receiver using 192 bit AES-OCB decryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey
       salt: aSalt
       opCode: -5
       into: destObjOrNil
       tag: tag
       extraData: eData
%

category: 'Authenticated Decrypting'
method: CharacterCollection
aesOcbDecryptWith256BitKey: aKey salt: aSalt into: destObjOrNil tag: tag extraData: eData
"Decrypts the receiver using 256 bit AES-OCB decryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey
       salt: aSalt
       opCode: -6
       into: destObjOrNil
       tag: tag
       extraData: eData
%

category: 'Authenticated Encrypting'
method: CharacterCollection
aesOcbEncryptWith128BitKey: aKey salt: aSalt into: destObjOrNil tag: tag extraData: eData
"Encrypts the receiver using 128 bit AES-OCB encryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey
       salt: aSalt
       opCode: 4
       into: destObjOrNil
       tag: tag
       extraData: eData
%

category: 'Authenticated Encrypting'
method: CharacterCollection
aesOcbEncryptWith192BitKey: aKey salt: aSalt into: destObjOrNil tag: tag extraData: eData
"Encrypts the receiver using 192 bit AES-OCB encryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey
       salt: aSalt
       opCode: 5
       into: destObjOrNil
       tag: tag
       extraData: eData
%

category: 'Authenticated Encrypting'
method: CharacterCollection
aesOcbEncryptWith256BitKey: aKey salt: aSalt into: destObjOrNil tag: tag extraData: eData
"Encrypts the receiver using 256 bit AES-OCB encryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey
       salt: aSalt
       opCode: 6
       into: destObjOrNil
       tag: tag
       extraData: eData
%

category: 'Converting'
method: CharacterCollection
asArrayOf32PathTerms
  "Returns an Array of path substrings held by the receiver.  The receiver
 is assumed to be a period-separated list of substrings.  These substrings
 are extracted and collected in an Array.  If the receiver contains no
 periods, the Array will hold a copy of the receiver.  The $\ Character
 is no longer recognized as an escape character.

 Raises an error if an element is not a valid path term."

  | nextName result |
  result := {}.
  nextName := self speciesForConversion new.
  self
    do: [ :c |
      c == $.
        ifTrue: [
          nextName _isValid32PathTermName
            ifFalse: [ ^ self _error: #'rtErrInvalidIndexPathExpression' args: { nextName } ].
          result add: nextName asSymbol.
          nextName := self speciesForConversion new ]
        ifFalse: [ nextName add: c ] ].
  nextName size ~~ 0
    ifTrue: [
      nextName _isValid32PathTermName
        ifFalse: [ ^ self _error: #'rtErrInvalidIndexPathExpression' args: { nextName } ].
      result add: nextName asSymbol ]
    ifFalse: [
      result size == 0
        ifTrue: [ result add: nextName asSymbol ] ].
  ^ result
%

category: 'Converting'
method: CharacterCollection
asArrayOfKeywords

"Returns an Array of keyword substrings held by the receiver.  The receiver
 is assumed to be a colon-separated list of substrings.  These substrings
 are extracted and collected in an Array.  If the receiver contains no
 colons, the Array will hold a copy of the receiver."

| c nextName result |

result := { }.
nextName := self speciesForConversion new.
1 to: self size do: [ :i |
  c := self at: i.
  nextName add: c.
  c == $: ifTrue: [
    result add: nextName.
    nextName := self speciesForConversion new.
  ].
].
nextName size ~~ 0 ifTrue: [
  result add: nextName
]
ifFalse: [
  result size == 0 ifTrue: [result add: nextName]
].
^result
%

category: 'Converting'
method: CharacterCollection
asArrayOfPathTerms
  "Returns an Array of path substrings held by the receiver.  The receiver
 is assumed to be a period-separated list of substrings.  These substrings
 are extracted and collected in an Array.  If the receiver contains no
 periods, the Array will hold a copy of the receiver.  The $\ Character
 is no longer recognized as an escape character.

 Raises an error if an element is not a valid path term."

  | nextName result |
  result := {}.
  nextName := self speciesForConversion new.
  self
    do: [ :c |
      c == $.
        ifTrue: [
          nextName _isValidPathTermName
            ifFalse: [ ^ self _error: #'rtErrInvalidIndexPathExpression' args: {nextName} ].
          result add: nextName asSymbol.
          nextName := self speciesForConversion new ]
        ifFalse: [ nextName add: c ] ].
  nextName size ~~ 0
    ifTrue: [
      nextName _isValidPathTermName
        ifFalse: [ ^ self _error: #'rtErrInvalidIndexPathExpression' args: {nextName} ].
      result add: nextName asSymbol ]
    ifFalse: [
      result size == 0
        ifTrue: [ result add: nextName asSymbol ] ].
  (result at: 1) = #'*'
    ifTrue: [
      "* not allowed as first term"
      ^ self _error: #'rtErrInvalidIndexPathExpression' args: {(result at: 1)} ].
  ^ result
%

category: 'Converting'
method: CharacterCollection
asArrayOfSubstrings

"Returns an Array of substrings held by the receiver. The receiver
 is assumed to be a separator-separated list of substrings.  These substrings
 are extracted and collected in an Array.  If the receiver contains no
 separators, the Array will hold a copy of the receiver.  Separators not meant
 to separate substrings may be escaped with a $\ Character."

| nextName result esc sz |

result := { } .
(sz := self size) == 0 ifTrue: [
  ^result
].
nextName := self speciesForConversion new.
esc := false.
1 to: sz do: [ :i | | c |
  c := self at: i.
  esc ifTrue: [
    nextName add: c.
    esc := false.
  ] ifFalse: [
    c == $\ ifTrue: [ esc := true ]
    ifFalse: [
	c isSeparator ifTrue: [
          nextName size ~~ 0 ifTrue: [result add: nextName].
          nextName := self speciesForConversion new.
        ] ifFalse: [
          nextName add: c
        ].
    ].
  ].
].

esc ifTrue:[ nextName add: $\ ].

(nextName size ~~ 0 or: [result size == 0]) ifTrue:[ result add: nextName ].

^result
%

category: 'Converting'
method: CharacterCollection
asBase64String
"Return a String which represents the receiver represented in base64 format."

^ self asBase64StringOnOneLine: true
%

category: 'Converting'
method: CharacterCollection
asBase64StringOnOneLine: aBoolean
"Return a String which represents the receiver represented in base64 format.
 If aBoolean is true, the resulting is one long line which does not contain
 newline characters.  If aBoolean is false, newline characters are inserted
 such that each line does not exceed 64 characters."

<primitive: 1062>
aBoolean _validateClass: Boolean .
self _primitiveFailed: #asBase64StringOnOneLine:
%

category: 'Converting'
method: CharacterCollection
asDecimalFloat

"Returns a DecimalFloat whose value is represented by the receiver."

^ DecimalFloat fromString: self
%

category: 'Converting'
method: CharacterCollection
asDoubleByteString

"Returns a DoubleByteString representation of the receiver."

^ DoubleByteString withAll: self asString .       "fix 39372"
%

category: 'Converting'
method: CharacterCollection
asFloat

"Returns a SmallDouble or Float whose value is represented by the receiver."

^ Float fromString: self
%

category: 'Converting'
method: CharacterCollection
asHexString

"Returns a String containing a hexadecimal printed representation of the
 contents of the receiver.  For example, the message 'abc' asHexString
 returns the String '616263'.

 The receiver must be a byte format object."

<primitive: 467>
self _validateByteClass: CharacterCollection .
self _primitiveFailed: #asHexString .
%

category: 'Message Authentication Codes'
method: CharacterCollection
asHmacSumWithDigestKind: opCode key: keyString

"Computes the keyed-hash message authentication code (HMAC) of the receiver using
 the message digest algorithm indicated by opCode and secret key keyString.

 opCode must be one of the following:

 opCode   Digest Algorithm
 =========================
  1          md5
  2          sha1
  3          sha2-256
  4          sha2-512
  5          sha3-224
  6          sha3-256
  7          sha3-384
  8          sha3-256
 =========================

 secretKey must be an instance or subclass of a ByteArray or String and must
 have a character size of one, i.e.: its class must answer 1 when sent the
 message #_bytesPerWord. secretKey must have a size between 1 and 64 bytes.

 Answers the HMAC of the receiver as a LargeInteger."

<primitive: 1060>
opCode _validateClass: SmallInteger .
((opCode < 1) or:[ opCode > 8]) ifTrue:[ opCode _error: #rtErrArgOutOfRange args:{ 1 . 8 } ].
keyString _validateClasses: { String . ByteArray } .
((keyString _basicSize > 64) or:[keyString _basicSize < 1])
  ifTrue:[ keyString _error: #rtErrBadSize args: { 64 . keyString _basicSize } ] .
self _primitiveFailed: #asHmacSumWithDigestKind:key: args: { opCode . keyString } .
%

category: 'Converting'
method: CharacterCollection
asInteger

"Returns an Integer whose value is represented by the receiver."

^ Integer fromString: self
%

category: 'Converting'
method: CharacterCollection
asLowercase

"Returns a new instance of the receiver's class, with all upper-case
 characters in the receiver changed to lower-case.

 (Subclass responsibility.)"

^ self subclassResponsibility: #asLowercase
%

category: 'Message Authentication Codes'
method: CharacterCollection
asMd5HmacStringWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the MD5 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a String of hexadecimal characters."

^ (self asMd5HmacWithKey: keyString) asHexStringWithLength: 32
%

category: 'Message Authentication Codes'
method: CharacterCollection
asMd5HmacWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the MD5 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a LargeInteger."

^ self asHmacSumWithDigestKind: 1 key: keyString
%

category: 'Message Digests'
method: CharacterCollection
asMd5String

"Compute the 128 bit MD5 message digest for the receiver and return it as
 a 32 character string of hexadecimal characters."

^ self md5sum asHexStringWithLength: 32
%

category: 'Converting'
method: CharacterCollection
asMultiByteString: example

"Returns a Double/QuadByteString representation of the receiver,
 depending on the class of the example string."

^ example class withAll: self
%

category: 'Converting'
method: CharacterCollection
asNumber

"Returns the receiver converted to a kind of number.  If the receiver contains
 all digits (with optional radix notation), returns a kind of Integer.  If the
 receiver has a slash, returns a Fraction.  Otherwise conversion to a Float is
 attempted.  An error may result if the receiver does not contain the proper
 format for a kind of Number."

 | str strm |
 str := self trimWhiteSpace.
 str size == 0 ifTrue: [ ^0 ].
 strm := ReadStreamPortable on: str .
 (self indexOf: $/ startingAt: 1) ~~ 0 ifTrue:[
    ^ Fraction fromStream: strm .
 ].
 ^ Number fromStream: strm .
%

category: 'Converting'
method: CharacterCollection
asQuadByteString

"Returns a QuadByteString representation of the receiver."

^ QuadByteString withAll: self.
%

category: 'Message Authentication Codes'
method: CharacterCollection
asSha1HmacStringWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the sha1 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a String of hexadecimal characters."

^ (self asSha1HmacWithKey: keyString) asHexStringWithLength: 40
%

category: 'Message Authentication Codes'
method: CharacterCollection
asSha1HmacWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the sha1 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a LargeInteger."

^ self asHmacSumWithDigestKind: 2 key: keyString
%

category: 'Message Digests'
method: CharacterCollection
asSha1String

"Compute the 160 bit SHA1 message digest for the receiver and return it as
 a 40 character string of hexadecimal characters."

^ self sha1Sum asHexStringWithLength: 40
%

category: 'Message Authentication Codes'
method: CharacterCollection
asSha256HmacStringWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the sha256 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a String of hexadecimal characters."

^ (self asSha256HmacWithKey: keyString) asHexStringWithLength: 64
%

category: 'Message Authentication Codes'
method: CharacterCollection
asSha256HmacWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the sha256 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a LargeInteger."

^ self asHmacSumWithDigestKind: 3 key: keyString
%

category: 'Message Digests'
method: CharacterCollection
asSha256String

"Compute the 256 bit SHA-2 message digest for the receiver and return it as
 a 64 character string of hexadecimal characters."

^ self sha256Sum asHexStringWithLength: 64
%

category: 'Message Authentication Codes'
method: CharacterCollection
asSha3_224HmacStringWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the SHA3 224 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a String of hexadecimal characters."

^ (self asSha3_224HmacWithKey: keyString) asHexStringWithLength: 56
%

category: 'Message Authentication Codes'
method: CharacterCollection
asSha3_224HmacWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the SHA3 224 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a LargeInteger."

^ self asHmacSumWithDigestKind: 5 key: keyString
%

category: 'Message Digests'
method: CharacterCollection
asSha3_224String

"Compute the SHA3 224 bit message digest for the receiver and return it as
 a 56 character string of hexadecimal characters."

^ self sha3_224Sum asHexStringWithLength: 56
%

category: 'Message Authentication Codes'
method: CharacterCollection
asSha3_256HmacStringWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the SHA3 256 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a String of hexadecimal characters."

^ (self asSha3_256HmacWithKey: keyString) asHexStringWithLength: 64
%

category: 'Message Authentication Codes'
method: CharacterCollection
asSha3_256HmacWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the SHA3 256 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a LargeInteger."

^ self asHmacSumWithDigestKind: 6 key: keyString
%

category: 'Message Digests'
method: CharacterCollection
asSha3_256String

"Compute the SHA3 256 bit message digest for the receiver and return it as
 a 64 character string of hexadecimal characters."

^ self sha3_256Sum asHexStringWithLength: 64
%

category: 'Message Authentication Codes'
method: CharacterCollection
asSha3_384HmacStringWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the SHA3 384 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a String of hexadecimal characters."

^ (self asSha3_384HmacWithKey: keyString) asHexStringWithLength: 96
%

category: 'Message Authentication Codes'
method: CharacterCollection
asSha3_384HmacWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the SHA3 384 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a LargeInteger."

^ self asHmacSumWithDigestKind: 7 key: keyString
%

category: 'Message Digests'
method: CharacterCollection
asSha3_384String

"Compute the SHA3 384 bit message digest for the receiver and return it as
 a 96 character string of hexadecimal characters."

^ self sha3_384Sum asHexStringWithLength: 96
%

category: 'Message Authentication Codes'
method: CharacterCollection
asSha3_512HmacStringWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the SHA3 512 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a String of hexadecimal characters."

^ (self asSha3_512HmacWithKey: keyString) asHexStringWithLength: 128
%

category: 'Message Authentication Codes'
method: CharacterCollection
asSha3_512HmacWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the SHA3 512 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a LargeInteger."

^ self asHmacSumWithDigestKind: 8 key: keyString
%

category: 'Message Digests'
method: CharacterCollection
asSha3_512String

"Compute the SHA3 512 bit message digest for the receiver and return it as
 a 128 character string of hexadecimal characters."

^ self sha3_512Sum asHexStringWithLength: 128
%

category: 'Message Authentication Codes'
method: CharacterCollection
asSha512HmacStringWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the sha512 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a String of hexadecimal characters."

^ (self asSha512HmacWithKey: keyString) asHexStringWithLength: 128
%

category: 'Message Authentication Codes'
method: CharacterCollection
asSha512HmacWithKey: keyString
"Computes the keyed-hash message authentication code (HMAC) of the receiver
 using the sha512 message digest algorithm and secret key keyString. Answers
 the resulting HMAC of the receiver as a LargeInteger."

^ self asHmacSumWithDigestKind: 4 key: keyString
%

category: 'Message Digests'
method: CharacterCollection
asSha512String

"Compute the 512 bit SHA-2 message digest for the receiver and return it as
 a 128 character string of hexadecimal characters."

^ self sha512Sum asHexStringWithLength: 128
%

category: 'Deprecated'
method: CharacterCollection
asSmallFloat

"SmallFloat is deprecated. Return a SmallDouble or Float whose value
 is represented by the receiver."

self deprecated: 'CharacterCollection>>asSmallFloat deprecated v3.0. Use asFloat or Float class>>#fromString: '.
^ (Float fromString: self) asSmallFloat
%

category: 'Converting'
method: CharacterCollection
asString

"Returns a String representation of the receiver."

^ String withAll: self
%

category: 'Converting'
method: CharacterCollection
asSymbolKind

"Returns a canonical symbol containing the same Characters as the receiver."

CharacterCollection subclassResponsibility: #asSymbolKind
%

category: 'Converting'
method: CharacterCollection
asUppercase

"Returns a new instance of the receiver's class, with all lower-case
 characters in the receiver changed to upper-case.

 (Subclass responsibility.)"

^ self subclassResponsibility: #asUppercase
%

category: 'Comparing'
method: CharacterCollection
at: anIndex equals: aCharCollection

"Returns true if aCharCollection is contained in the receiver starting at
 anIndex.  Returns false otherwise.

 Note that this method returns true only if aCharCollection begins exactly at
 the position designated by anIndex.  To locate a pattern beginning on or after
 anIndex, see the method findPattern:startingAt: in category 'Searching'.

 (Subclass responsibility.)"

^ self subclassResponsibility: #at:equals:
%

category: 'Authenticated Encrypting'
method: CharacterCollection
authenticatedEncryptionExample

| key salt enc dec hamlet extraData tag|
key  := ByteArray withRandomBytes: 32 .
salt := ByteArray withRandomBytes: 12 .

hamlet :=
'Alas, poor Yorick! I knew him, Horatio: a fellow
of infinite jest, of most excellent fancy: he hath
borne me on his back a thousand times; and now, how
abhorred in my imagination it is! my gorge rims at
it. Here hung those lips that I have kissed I know
not how oft. Where be your gibes now? your
gambols? your songs? your flashes of merriment,
that were wont to set the table on a roar? Not one
now, to mock your own grinning? quite chap-fallen?
Now get you to my ladys chamber, and tell her, let
her paint an inch thick, to this favour she must
come; make her laugh at that. Prithee, Horatio, tell
me one thing.' .

extraData := 'Hamlet: Act V, Scene i'.

tag := ByteArray new.
enc := hamlet aesOcbEncryptWith256BitKey: key salt: salt
       into: ByteArray new tag: tag extraData: extraData .
dec := enc aesOcbDecryptWith256BitKey: key salt: salt
       into: String new tag: tag extraData: extraData .
^ dec = hamlet
%

category: 'Comparing'
method: CharacterCollection
between: min and: max
	"Answer whether the receiver is less than or equal to the argument max and
	 greater than or equal to the argument min."

	^self >= min and: [self <= max].
%

category: 'Authenticated Decrypting'
method: CharacterCollection
chacha20Poly1305DecryptWithKey: aKey salt: aSalt into: destObjOrNil tag: tag extraData: eData
"Decrypts the receiver using CHACHA20-Poly1305 decryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey
       salt: aSalt
       opCode: -10
       into: destObjOrNil
       tag: tag
       extraData: eData
%

category: 'Authenticated Encrypting'
method: CharacterCollection
chacha20Poly1305EncryptWithKey: aKey salt: aSalt into: destObjOrNil tag: tag extraData: eData
"Encrypts the receiver using CHACHA20-Poly1305 encryption and places the result into
 destObjOrNil.

 See the comments in method:
   #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
         into: destObjOrNil tag: nil extraData: nil
 for more information on encryption and decryption."

^ self _primEncryptDecryptWithKey: aKey
       salt: aSalt
       opCode: 10
       into: destObjOrNil
       tag: tag
       extraData: eData
%

category: 'Formatting'
method: CharacterCollection
charSize

"Returns number of bytes that make up a character for this string class.
 (Subclass responsibility.)"

^ self subclassResponsibility: #charSize
%

category: 'Accessing'
method: CharacterCollection
codePointAt: anIndex

CharacterCollection subclassResponsibility: #'codePointAt:'
%

category: 'Updating'
method: CharacterCollection
codePointAt: anIndex put: aValue

CharacterCollection subclassResponsibility: #'codePointAt:put:'
%

category: 'Comparing'
method: CharacterCollection
compareTo: aString collator: anIcuCollator

"Returns -1, 0 or 1,  when receiver is less than,
 equal to, or greater than argString .
 argString must be a String, MultiByteString, or a Utf8 .
 anIcuCollator == nil is interpreted as   IcuCollator default ."

^ self compareTo: aString collator: anIcuCollator useMinSize: false
%

category: 'Comparing'
method: CharacterCollection
compareTo: argString collator: anIcuCollator useMinSize: aMinSize
  "Returns -1, 0 or 1,  when receiver is less than,
 equal to, or greater than argString .
 argString must be a String, MultiByteString, or a Utf8.
 anIcuCollator == nil is interpreted as   IcuCollator default .

 If aMinSize==true, compare stops at (self size min: argString size),
 which is Squeak semantics for comparison .

 If aMinSize is a SmallInteger >= 1, compare stops at
 aMinSize min: (self size min: argString size) ."

  ^ self subclassResponsibility: #'compareTo:collator:useMinSize:'
%

category: 'Searching'
method: CharacterCollection
containsDigit

"Answers true if the receiver contains at least one digit, otherwise
 answers false."

1 to: self size do:[:n| (self at: n) isDigit ifTrue:[ ^ true ] ] .
^ false
%

category: 'Searching'
method: CharacterCollection
containsLowercase

"Answers true if the receiver contains at least one lower case character,
 otherwise answers false.
 Differs from legacy method containsLowercaseOld for
 code points 170, 186, and about 1700 code points >= 256"

1 to: self size do:[:n| (self at: n) isLowercase ifTrue:[ ^ true ] ] .
^ false
%

category: 'Legacy'
method: CharacterCollection
containsLowercaseOld

"Answers true if the receiver contains at least one lower case character,
 otherwise answers false."

1 to: self size do:[:n| (self at: n) isLowercaseOld ifTrue:[ ^ true ] ] .
^ false
%

category: 'Private'
method: CharacterCollection
containsPasswordSymbol

"Answers true if the receiver contains at least one symbol character, with
 symbol in this case meaning any Character that is neither a letter nor a
 digit; otherwise answers false."

1 to: self size do:[:n| (self at: n) _isPasswordSymbol ifTrue:[ ^ true ] ] .
^ false
%

category: 'Searching'
method: CharacterCollection
containsUppercase

"Answers true if the receiver contains at least one upper case character,
 otherwise answers false."

1 to: self size do:[:n| (self at: n) isUppercase ifTrue:[ ^ true ] ] .
^ false
%

category: 'Repository Conversion'
method: CharacterCollection
convert
"If receiver is a committed object and receiver's class on disk was
 a subclass of one of
   ObsDoubleByteString, ObsDoubleByteSymbol, ObsQuadByteString
 the receiver is added to the writeSet of current transaction,
 otherwise has no effect.
 Returns true if receiver was added to the writeSet, false otherwise."

<primitive: 744>
self _primitiveFailed:#convert
%

category: 'Copying'
method: CharacterCollection
copy
  "optimization, no need for postCopy"
<primitive: 885>
self _primitiveFailed: #copy .
self _uncontinuableError
%

category: 'Copying'
method: CharacterCollection
copyFrom: startIndex to: stopIndex

"Returns a new SequenceableCollection containing the elements of the receiver
 between startIndex and stopIndex, inclusive.  The result is of the same class
 as the receiver, unless the receiver is a Symbol or DoubleByteSymbol,
 in which case the result class is respectively String or DoubleByteString.

 Both startIndex and stopIndex must be positive integers not larger than the
 size of the receiver, with startIndex <= stopIndex.
 If startIndex > stopIndex and both are positive, an empty collection is returned.
 "

| result |
(startIndex > stopIndex) ifTrue: [
  stopIndex < 0 ifTrue:[ self _error: #rtErrBadCopyFromTo args: { stopIndex } ].
  ^ self class new
].
(startIndex < 1)
   ifTrue: [ ^ self _errorIndexOutOfRange: startIndex].

((stopIndex > self size) or: [(stopIndex < 0)])
   ifTrue: [ ^ self _errorIndexOutOfRange: stopIndex].

result := (self class _newString: (stopIndex - startIndex + 1)).
result replaceFrom: 1 to: 1 + stopIndex - startIndex
	with: self startingAt: startIndex  .
^ result
%

category: 'Copying'
method: CharacterCollection
copyReplaceAll: subString with: newSubString

"Returns a copy of the receiver with all occurrences of the given substring
 replaced with newSubString."

| copy csize matches ssize idx |

copy := self copy.
matches := { } .
ssize := subString size.
ssize == 0 ifTrue: [^copy].
idx := 1 - ssize.
csize := copy size.
[ idx := idx + subString size.
  idx <= csize and: [
    idx := copy findString: subString startingAt: idx.
    idx ~~ 0
  ]
] whileTrue: [
  matches add: idx
].
matches reverseDo: [:p |
  copy removeFrom: p to: p+ssize-1.
  copy insertAll: newSubString at: p
].
^copy
%

category: 'Copying'
method: CharacterCollection
copyReplaceChar: aCharacter with: secondCharacter

"Returns a copy of the receiver with all occurrences of the given
 Character replaced with secondCharacter."

| copy |
secondCharacter _validateClass: Character .
copy := self speciesForCollect withAll: self .
1 to: copy size do:[:n |
 (copy at: n) == aCharacter ifTrue:[ copy at: n put: secondCharacter].
].
^ copy
%

category: 'Copying'
method: CharacterCollection
copyWithout: anObject

"Returns a copy of the receiver that does not contain the given object.
 Comparisons are by equality."

| copy element sz |

copy := self class _newString .

sz := 0.
1 to: self size do: [:i |
  element := self at: i.
  (element = anObject)
    ifFalse: [
      sz := sz + 1.
      copy at: sz put: element.
      ]
  ].

^copy
%

category: 'Converting'
method: CharacterCollection
copyWrappedTo: rightMargin

"Returns a String with the receiver's contents word-wrapped to the given
 right margin."

| res col ch lf |

self size < rightMargin ifTrue: [^self].
lf := Character lf.
res := self species new.
col := 0.
1 to: self size do: [ :i |
  ch := self at: i.
  res add: ch.
  ch == lf ifTrue: [ col := 0 ]
  ifFalse: [
    col := col + 1.
    (col > rightMargin and: [ch isSeparator]) ifTrue: [
      res add: lf.
      col := 1.
    ].
  ].
].
^ res
%

category: 'Formatting'
method: CharacterCollection
describeClassName

"Returns a copy of the receiver with the Character $a prepended to the
 receiver's contents.  This method is used for formatting class names in object
 descriptions, where the receiver is a string containing the name of a class.
 For example, the String 'UserClass', when sent the message describeClassName,
 returns 'aUserClass'."

| result |
result := self speciesForPrint new .
self size ~~ 0 ifTrue:[   "inline simplified isVowel using Unicode asUppercase"
  ( #( $A $E $I $O $U $Y ) includesIdentical: (self at: 1) asUppercase ) ifTrue:[
    result add: 'an' ; addAll: self .
    ^ result
  ]
].
result add: $a ; add: self.
^ result
%

category: 'Encrypting'
method: CharacterCollection
encryptionExample

| key salt enc dec hamlet |
key  := ByteArray withRandomBytes: 32 .
salt := ByteArray withRandomBytes: 16 .

hamlet :=
'Alas, poor Yorick! I knew him, Horatio: a fellow
of infinite jest, of most excellent fancy: he hath
borne me on his back a thousand times; and now, how
abhorred in my imagination it is! my gorge rims at
it. Here hung those lips that I have kissed I know
not how oft. Where be your gibes now? your
gambols? your songs? your flashes of merriment,
that were wont to set the table on a roar? Not one
now, to mock your own grinning? quite chap-fallen?
Now get you to my ladys chamber, and tell her, let
her paint an inch thick, to this favour she must
come; make her laugh at that. Prithee, Horatio, tell
me one thing.' .


enc := hamlet aesEncryptWith256BitKey: key salt: salt .
dec := enc aesDecryptWith256BitKey: key salt: salt into: String new.
^ dec = hamlet
%

category: 'Testing'
method: CharacterCollection
equalsNoCase: aCharCollection
    "Returns true if the receiver is equivalent to aCharCollection.
    The receiver is equivalent to aCharCollection if the receiver
    contains the same Characters as aCharCollection regardless of case
    or internal representation.  For example, if $a is in
    aCharCollection, it is equivalent to any representation of an 'a'
    in the receiver's character set.

    (Subclass responsibility.)"

^ self subclassResponsibility: #equalsNoCase:
%

category: 'Case-Sensitive Searching'
method: CharacterCollection
findLastSubString: subString startingAt: startIndex

"startIndex should be >= 1 and <= self size . Search is backwards
 through the receiver. Returns 0 if no match found. "
^  self _findLastString: subString startingAt: startIndex ignoreCase: false
%

category: 'Searching'
method: CharacterCollection
findPattern: aPattern startingAt: anIndex

"This method searches the receiver, beginning at anIndex, for a substring that
 matches aPattern.  If a matching substring is found, this method returns the
 index of the first Character of the substring.  Otherwise, this returns 0.

 The argument aPattern is an Array containing zero or more CharacterCollections
 plus zero or more occurrences of the special Characters asterisk or
 question-mark.  See the description of the matchPattern: method for more
 information about this argument.

 Performs a case-sensitive search."

^ self _findPattern: aPattern startingAt: anIndex ignoreCase: false
%

category: 'Searching'
method: CharacterCollection
findPatternNoCase: aPattern startingAt: anIndex

"This method searches the receiver, beginning at anIndex, for a substring that
 matches aPattern.  If a matching substring is found, this method returns the
 index of the first Character of the substring.  Otherwise, this returns 0.

 The argument aPattern is an Array containing zero or more CharacterCollections
 plus zero or more occurrences of the special Characters asterisk or
 question-mark.  See the description of the matchPattern: method for more
 information about this argument.

 Performs a case-insensitive search."

^ self _findPattern: aPattern startingAt: anIndex ignoreCase: true
%

category: 'Case-Sensitive Searching'
method: CharacterCollection
findString: subString startingAt: startIndex

"If a receiver contains subString beginning at some point at or after
 startIndex, this returns the index at which subString begins.  If the
 receiver does not contain subString, this returns 0.

 The search is case-sensitive."

^ self _findString: subString startingAt: startIndex ignoreCase: false
%

category: 'Case-Insensitive Searching'
method: CharacterCollection
findStringNoCase: subString startingAt: startIndex

"If a receiver contains subString beginning at some point at or after
 startIndex, this returns the index at which subString begins.  If the
 receiver does not contain subString, this returns 0.

 The search is case-insensitive."

^ self _findString: subString startingAt: startIndex ignoreCase: true
%

category: 'Hashing'
method: CharacterCollection
hash

"Returns a positive Integer based on a case-sensitive hash of the contents
 of the receiver.  The algorithm implemented is described in:

 [Pearson 90]
 Pearson, Peter K., Fast Hashing of Variable-Length Text Strings,
 Communications of the ACM 33, 6, (June 1990), 677-680.

 This implementation inherited by JapaneseString.  String and
 MultiByteString each reimplement hash."

<primitive: 31>
self _primitiveFailed: #hash .
self _uncontinuableError
%

category: 'LDAP Support'
method: CharacterCollection
hasLdapWildcardPattern

"Answer true if the receiver contains one and only one '%' character,
 and the character after the '%' is 's'."

(self select:[:e| e == $%]) size == 1
  ifFalse:[^ false ].

^(self findPattern: #('%' 's') startingAt: 1) ~~ 0
%

category: 'Case-Insensitive Searching'
method: CharacterCollection
includesString: aString

"Returns true if aString is contained as a subString within the receiver,
 using a case-insensitive search.  Returns false otherwise."

^ (self _findString: aString startingAt: 1 ignoreCase: true) ~~ 0
%

category: 'Searching'
method: CharacterCollection
indexOf: pattern matchCase: flag startingAt: startIndex

"Searches the receiver, beginning at anIndex, for a substring that
 matches aPattern.  If a matching substring is found, returns the
 index of the first Character of the substring.  Otherwise, returns 0.

 The argument pattern is an Array containing zero or more CharacterCollections
 plus zero or more occurrences of the special Characters asterisk or
 question-mark.  See the description of the matchPattern: method for more
 information about this argument.

 If the flag argument is true, a case-sensitive search is performed.  Otherwise,
 a case-insensitive search is performed."

^ self _findPattern: pattern startingAt: startIndex ignoreCase: (flag not).
%

category: 'Searching'
method: CharacterCollection
indexOfSubCollection: aSubColl startingAt: anIndex

"Returns the index of the first element of the receiver where that element and
 the subsequent ones are equal to those in aSubColl. The search is begun in the
 receiver at starting at anIndex. Returns zero if no match is found."

(aSubColl size == 0) ifTrue: [ ^ 0 ].
^ self findString: aSubColl startingAt: anIndex .
%

category: 'Searching'
method: CharacterCollection
indexOfSubCollection: aSubColl startingAt: anIndex ifAbsent: anExceptionBlock

"Returns the index of the first element of the receiver where that element and
 the subsequent ones are equal to those in aSubColl. The search is begun in the
 receiver at starting at anIndex. Returns the value of evaluating
 anExceptionBlock if no match is found."

| idx |
(aSubColl size == 0) ifTrue: [ ^ anExceptionBlock value ].
idx := self findString: aSubColl startingAt: anIndex .
idx == 0 ifTrue:[ ^ anExceptionBlock value ].
^ idx
%

category: 'Deprecated'
method: CharacterCollection
insert: aCharOrCharCollection at: anIndex

self deprecated: 'CharacterCollection>>insert:at: deprecated long before v3.0. Use insertAll:at: instead.'.
^ self insertAll: aCharOrCharCollection at: anIndex.
%

category: 'Adding'
method: CharacterCollection
insertAll: aCharOrCharCollection at: anIndex

"Inserts aCharOrCharCollection into the receiver at the specified index and
 returns aCharOrCharCollection."

(aCharOrCharCollection isKindOf: CharacterCollection)
   ifTrue: [ ^ super insertAll: aCharOrCharCollection at: anIndex ].

(aCharOrCharCollection isKindOf: AbstractCharacter)
  ifTrue: [
    self replaceFrom: anIndex + 1 to: 1 + self size with: self startingAt: anIndex .
    self at: anIndex put: aCharOrCharCollection.
    ^aCharOrCharCollection.
    ].

^ aCharOrCharCollection _error: #rtErrInvalidArgClass
                        args: { AbstractCharacter . CharacterCollection }.
%

category: 'Adding'
method: CharacterCollection
insertObject: anObject at: anIndex

"Inserts anObject into the receiver at index anIndex and returns
 aCollection."

"reimplemented to use at:put: instead of _basicAt:put:"
| selfSize |

anIndex <= 0 ifTrue:[ ^ self _errorIndexOutOfRange: anIndex ].
selfSize := self size.
anIndex > selfSize ifTrue:[
  anIndex > (selfSize + 1) ifTrue:[ ^ self _errorIndexOutOfRange: anIndex].
  ^ self at: anIndex put: anObject.
  ].

"Not adding to the end of the receiver. Create a gap for anObject to
 be copied into."
self replaceFrom: anIndex + 1 to: 1 + selfSize with: self startingAt: anIndex .

^ self at: anIndex put: anObject.
%

category: 'Testing'
method: CharacterCollection
isDigits

"Returns true if the receiver contains only digits.  Returns false if the
 receiver contains non-digit Characters."

1 to: self size do: [:i |
    (self at: i) isDigit ifFalse: [^false]
].
^true
%

category: 'Testing'
method: CharacterCollection
isEquivalent: aCharCollection

"Returns true if the receiver is equivalent to aCharCollection.  The receiver
 is equivalent to aCharCollection if the receiver contains the same Characters
 as aCharCollection regardless of case or internal representation.  For
 example, if $a is in aCharCollection, it is equivalent to any representation
 of an 'a' in the receiver's character set.

 (Subclass responsibility.)"

^ self subclassResponsibility: #isEquivalent:
%

category: 'Testing'
method: CharacterCollection
isInfix

"Returns true if the receiver is an infix (binary) selector.  Returns false
 otherwise."

| binaryChars mySize fch |

binaryChars := '+-\*~<>=|/&@%,?!'.   "fixed bug 14109"

mySize := self size.
mySize == 0 ifTrue: [ ^ false ].

fch := self at: 1 .
(fch == $_  or:[ (binaryChars indexOf: fch startingAt: 1) ~~ 0] ) ifFalse:[
  ^ false
].
2 to: mySize do: [ :i |
  ((binaryChars indexOf: (self at: 1) startingAt: 1) == 0) ifTrue:[
     ^ false
  ].
].
^ true.
%

category: 'Testing'
method: CharacterCollection
isKeyword

"Returns true if the receiver is a keyword; that is, a legal keyword method
 selector. Returns false otherwise."

| mySize limit ch idx |

mySize := self size.
mySize > 1  ifFalse: [ ^ false ].
idx := 1 .
[ idx <= mySize ] whileTrue:[
  ch := self at: idx .  "first char of next keyword within selector"
  ( ch == $_ or:[ ch isLetter ] ) ifFalse: [ ^ false ].
  idx := idx + 1 .
  limit := mySize .
  [ idx <= limit ] whileTrue:[
    ch := self at: idx .
    ch == $:  ifTrue:[
      idx == mySize ifTrue:[ ^ true "hit ending $: " ].
      limit := 0 . "end of a keyword, exit inner loop"
    ] ifFalse:[
      (ch == $_  or:[ ch isAlphaNumeric]) ifFalse:[ ^ false ].
    ] .
    idx := idx + 1
  ]
].
^ false
%

category: 'Testing'
method: CharacterCollection
isMinusAndDigits

"Returns true if the receiver contains a minus sign followed only by digits.
 Returns false if the receiver has any other Characters."

((self at: 1) == $- ) ifFalse:[ ^ false ].
2 to: self size do: [:i |
    (self at: i) isDigit ifFalse:[ ^ false]
].
^true
%

category: 'Testing'
method: CharacterCollection
isValidIdentifier

"Returns true if the receiver is a valid GemStone Smalltalk variable name,
 and false otherwise."

^ self _validIdentifier: 1024"GCI_MAX_SYMBOL_SIZE"
%

category: 'Updating'
method: CharacterCollection
lf

"Appends a line-feed to the receiver and returns the receiver."

self addCodePoint: 10
%

category: 'Formatting'
method: CharacterCollection
linesIndentedBy: anInt

"Returns a copy of the receiver in which all lines have been indented
 by anInt spaces."

| c newStr indentStr lfInd lf targEndInd selfCurrInd sz destIdx sp |

indentStr := self class new: anInt.
indentStr atAllPut: (sp := Character space).
lf := Character lf.
lfInd := self indexOf: lf.
lfInd == 0 ifTrue:[ | res |
  (res := indentStr) addAll: self .
  ^ res
].

newStr := self class new.
selfCurrInd := 1.
c := self copy.

[(lfInd := c indexOf: lf) ~~ 0 ] whileTrue: [
  targEndInd := newStr size + indentStr size.
  newStr addAll: indentStr.
 "self copyFrom: selfCurrInd to: lfInd into: newStr startingAt: targEndInd + 1. "
  destIdx := targEndInd + 1 .
  newStr replaceFrom: destIdx to: destIdx + lfInd - selfCurrInd
	  with: self startingAt: selfCurrInd .
  selfCurrInd := lfInd + 1.
  c at: lfInd put: sp .
].
selfCurrInd < (sz := self size) ifTrue: [
  newStr addAll: indentStr.
 "self copyFrom: selfCurrInd to: sz into: newStr startingAt: newStr size + 1."
  destIdx := newStr size + 1 .
  newStr replaceFrom: destIdx to: destIdx + sz - selfCurrInd
         with: self startingAt: selfCurrInd .
].
^newStr
%

category: 'Comparing'
method: CharacterCollection
match: prefix

"Returns true if the argument prefix is a prefix of the receiver, and
 false if not.  The comparison is case-sensitive."

self size == 0 ifTrue: [ ^ prefix size == 0 ].
^ self at: 1 equals: prefix
%

category: 'Comparing'
method: CharacterCollection
matchesAnyOf: aCollectionOfCharacterColls

"Returns true if the receiver returns true to the message match: with any of
 the objects in the given collection; returns false otherwise.  Examples:

   'xyz' matchesAnyOf: #('xyz' 'abc*')
     true
   'xyz' matchesAnyOf: #('ayz' 'abc')
     false
   'x#z' matchesAnyOf: #('x@z' '*')
     false

 The deprecated class JISString does not support this method."

aCollectionOfCharacterColls do: [:coll |
  (self match: coll) ifTrue: [ ^true ]
].
^false
%

category: 'Comparing'
method: CharacterCollection
matchPattern: aPattern

"Returns true if the receiver matches aPattern, false if it doesn't.  An exact
 match is required.  For partial matching, use the 'Searching' method
 findPattern:startingAt: instead.

 The argument aPattern is a kind of Array containing zero or more
 CharacterCollections, plus zero or more occurrences of the special Characters
 $* or $?.  If either $* or $? occurs in aPattern, it acts as a wild card.
 The Character $? matches any single Character in the receiver, and $* matches
 any sequence of zero or more Characters in the receiver.  For example,

 'weimaraner' matchPattern: #('w' $* 'r')

 returns true, because the Character $* is interpreted as a wild card.

 If either of these special Characters occurs in the receiver, it is
 interpreted literally.  For example,

 'w*r' matchPattern: #('weimaraner')

 returns false - because the Character $* occurs in the receiver, it is
 interpreted as a literal asterisk (not as a wild card)."

| match         "indicates if the current pattern matched"
  pattern       "the Array of pattern elements"
  selfSize      "the size of the receiver"
  patternSize   "the number of elements in the pattern Array"
  startIndexArr "an Array of indexes into the receiver; this Array is
                 parallel to the pattern Array (each element in this Array
                 corresponds to the starting index for each pattern element)"
  index         "index into the pattern Array and the startIndexArr Array"
  thisPattern   "the current element of the pattern to match"
  selfIndex     "an index into the receiver"
|

 "The pattern Array must be processed so that there are no *? pairs in it.
 They must all be converted to ?* pairs for the algorithm to work correctly."
 pattern:= Array withAll: aPattern.
 patternSize:= pattern size.
 index := 1.
 [ index < patternSize ]
 whileTrue:
 [ (((pattern at: index) isEquivalent: $* )
    and:[ (pattern at: index+1) isEquivalent: $? ])
   ifTrue:[
     pattern at: index put: $?.
     pattern at: index+1 put: $*.
     index := 1 max: index-1.
   ]
   ifFalse:
   [ index := index + 1 ].
 ].

 "initialize"
 selfSize := self size.
 startIndexArr:= Array new: (patternSize + 1).
                 "last element is set, but not used"
 index := 1.
 startIndexArr at: 1 put: 1.

 "if no patterns to match, exit early"
 (patternSize == 0)
 ifTrue:
    [^ selfSize == 0 ].

 [index <= patternSize]
 whileTrue:
 [ thisPattern := pattern at: index.
   selfIndex := startIndexArr at: index.
   match := true.

   (thisPattern isKindOf: CharacterCollection) "pattern to match is a string"
   ifTrue:
   [ (selfIndex + thisPattern size - 1) > selfSize
     ifTrue: "this pattern too big to match rest of receiver"
        [^ false ].

     index = patternSize "processing the final pattern"
     ifTrue:
     [ ((index > 1) and: [ (pattern at: index - 1) isEquivalent: $* ])
       ifTrue: "is not the first pattern _and_ previous pattern was a $*"
       [ ((selfSize == 0) and: [thisPattern size == 0])
         ifTrue: [^ true].
         ^(self findString: thisPattern
                startingAt: selfSize - thisPattern size + 1) ~~ 0
         "find the pattern far enough back in the string so that only
         the final chars match"
       ]
       ifFalse: "processing first pattern _or_ previous pattern was not $*"
       [ (match:= (selfIndex + thisPattern size - 1 = selfSize))
         ifTrue: "exactly enough chars in self to match thisPattern"
         [ (selfSize == 0)
           ifTrue: [match:= (thisPattern size == 0)]
           ifFalse: [match:= self at: selfIndex equals: thisPattern ].
         ]
       ].
     ]
     ifFalse: "not processing the final pattern"
     [ ((index > 1) and: [ (pattern at: index - 1) isEquivalent: $* ])
       ifTrue: "not first pattern _and_ previous pattern was $*"
       [ (((selfSize == 0) and: [thisPattern size == 0]) or:
         [(selfIndex:= self findString: thisPattern
                           startingAt: selfIndex) ~~ 0])
         ifTrue: "thisPattern was found"
         [ startIndexArr at: index put: selfIndex.
           startIndexArr at: index + 1 put: selfIndex + thisPattern size.
         ]
         ifFalse: "thisPattern was not found"
            [^ false ]
       ]
       ifFalse: "first pattern _or_ previous pattern was not $*"
       [ (((selfSize == 0) and: [thisPattern size == 0]) or:
         [(self at: selfIndex equals: thisPattern)])
         ifTrue:
            [startIndexArr at: index + 1 put: selfIndex+ thisPattern size]
         ifFalse:
            [match := false ].
       ].
     ]
   ]
   ifFalse: "thisPattern is not a string"
   [ (thisPattern isEquivalent: $*)
     ifTrue:
        [startIndexArr at: (index + 1) put: selfIndex]
     ifFalse:
     [ (thisPattern isEquivalent: $?)
       ifTrue:
       [ selfIndex > selfSize
         ifTrue: "no char to match; already at end of self"
            [^ false ].
         startIndexArr at: (index + 1) put: (selfIndex + 1).
         index = patternSize "processing the last pattern"
         ifTrue:
            [match := selfIndex = selfSize.].
       ]
       ifFalse: "next pattern is neither a $* or $?"
       [ ^ aPattern _error: #rtErrBadPattern].
     ].
   ].  "end ifTrue:ifFalse"

   match
   ifTrue:
      [index := index + 1 ] "advance to the next term in the pattern"
   ifFalse:
   [ "If there is a preceding $* term in the pattern, backup to the
      term following it, and advance position in the string by 1."
      [ index := index - 1.
        index < 2 ifTrue:
           [^ false ].
        (pattern at: index - 1) isEquivalent: $*
      ] untilTrue.
      startIndexArr at: index put: ((startIndexArr at: index) + 1).
   ].
].  "end whileTrue:"

^ true
%

category: 'Comparing'
method: CharacterCollection
max: another

"If the receiver is greater than the argument, return the receiver.
 Otherwise return the argument."

	^self > another
		ifTrue: [self]
		ifFalse: [another].
%

category: 'Searching'
method: CharacterCollection
maxConsecutiveSubstring

"Returns the largest substring within the receiver that contains Characters with
 consecutive ASCII values.  For example, the message

   'abxabcdxabc' maxConsecutiveSubstring

 yields the result 'abcd'.

 If there are no such substrings larger than 2 Characters, returns a String that
 contains the first Character in the receiver."

| mySize bigSize bigStart aStart aSize lastVal thisVal |

mySize := self size .
mySize < 2 ifTrue:[ ^ self ].
bigSize := 1 .
bigStart := 1 .
aStart := 1 .
aSize := 1 .
lastVal := (self at: 1) codePoint  .
2 to: mySize do:[:j |
  lastVal := lastVal + 1 .
  thisVal := (self at: j) codePoint .
  thisVal == lastVal
    ifTrue:[ aSize := aSize + 1 ]
    ifFalse:[
      aSize > bigSize ifTrue:[
	bigStart := aStart .
	bigSize := aSize .
	].
      aSize := 1 .
      lastVal := thisVal .
      aStart := j .
      ].
   ].
aSize > bigSize ifTrue:[
  bigStart := aStart.
  bigSize := aSize.
  ].
^ self copyFrom: bigStart to: (bigStart + bigSize - 1) .
%

category: 'Searching'
method: CharacterCollection
maxRepeatingSubstring

"Returns the largest substring within the receiver that contains repetitions of
 a Character, using case-sensitive comparison.  For example, the message

   'aaxbbbBxccc' maxRepeatingSubstring

 yields the result 'bbb'.

 If there are no such substrings larger than 1 Character, returns a String that
 contains the first Character in the receiver."

| mySize bigSize bigStart aStart aSize lastChar |

mySize := self size .
mySize < 2 ifTrue:[ ^ self ].
bigSize := 1 .
bigStart := 1 .
aStart := 1 .
aSize := 1 .
lastChar := self at: 1 .
2 to: mySize do:[:j | | thisChar |
  thisChar := self at: j .
  thisChar == lastChar
    ifTrue:[ aSize := aSize + 1 ]
    ifFalse:[
      aSize > bigSize ifTrue:[
	bigStart := aStart .
	bigSize := aSize .
	].
      aSize := 1 .
      lastChar := thisChar .
      aStart := j .
      ].
   ].
aSize > bigSize ifTrue:[
  bigStart := aStart.
  bigSize := aSize.
  ].
^ self copyFrom: bigStart to: (bigStart + bigSize - 1) .
%

category: 'Searching'
method: CharacterCollection
maxSameTypeSubstring

"Returns the largest substring within the receiver that contains either all
 digits, all alphabetic characters, or all special characters.  For example, the
 message

   'axv2435,.-' maxSameTypeSubstring

 yields the result '2435'.

 If there are no such substrings larger than 1 Character, returns a String that
 contains the first Character in the receiver.

 This method may generate an error if the receiver is a JapaneseString."

| mySize bigSize bigStart aStart aSize lastType |

mySize := self size .
mySize < 2 ifTrue:[ ^ self ].
bigSize := 1 .
bigStart := 1 .
aStart := 1 .
aSize := 1 .
lastType := (self at: 1) _type  .
2 to: mySize do:[:j | | thisType |
  thisType := (self at: j) _type .
  thisType == lastType
    ifTrue:[ aSize := aSize + 1 ]
    ifFalse:[
      aSize > bigSize ifTrue:[
	bigStart := aStart .
	bigSize := aSize .
	].
      aSize := 1 .
      lastType := thisType .
      aStart := j .
      ].
   ].
aSize > bigSize ifTrue:[
  bigStart := aStart.
  bigSize := aSize.
  ].
^ self copyFrom: bigStart to: (bigStart + bigSize - 1) .
%

category: 'Message Digests'
method: CharacterCollection
md5sum

"Return the 128 bit MD5 checksum of the receiver as a LargeInteger.

 Computation is per RFC 1321 , http://www.ietf.org/rfc/rfc1321.txt,
 using L. Peter Deutsch's C implementation from
 http://sourceforge.net/projects/libmd5-rfc/

 For DoubleByteString and QuadByteString, the computation is based
 on viewing the string as a ByteArray holding big-endian characters.
"

^ self _asMessageDigestKind: 1
%

category: 'Comparing'
method: CharacterCollection
min: another

"If the receiver is less than the argument, return the receiver.
 Otherwise return the argument."

	^self < another
		ifTrue: [self]
		ifFalse: [another].
%

category: 'Accessing'
method: CharacterCollection
numArgs

"Returns the number of arguments the receiver would take, were the receiver
 a message selector."

| idx count sz |
(sz := self size) == 0 ifTrue:[ ^ 0 ].
(self at: sz) == $: ifTrue:[
  count := 1 .
  idx := 0 .
  [ idx := self indexOf: $: startingAt: idx + 1 .
    idx ~~ sz
  ] whileTrue:[
    count := count + 1
  ].
  ^ count
] ifFalse:[
  self isInfix ifTrue: [ ^ 1 ].
  ^ 0
]
%

category: 'Json'
method: CharacterCollection
printJsonOn: aStream

	aStream nextPut: $".
	1 to: self size do: [:j |
		| codePoint char |
		codePoint := (char := self at: j) codePoint.
    (32 <= codePoint and: [codePoint <= 127]) ifTrue: [
      char == $" "34" ifTrue: [aStream nextPutAll: '\"' ] ifFalse: [
      char == $\ "92" ifTrue: [aStream nextPutAll: '\\' ] ifFalse: [
        aStream nextPut: char
      ]]
    ] ifFalse:[
      (codePoint <= 13 and:[ codePoint >= 8]) ifTrue:[
		    codePoint == 8 	ifTrue: [aStream nextPutAll: '\b'	. codePoint := nil. ] ifFalse: [
		    codePoint == 9 	ifTrue: [aStream nextPutAll: '\t'	. codePoint := nil. ] ifFalse: [
		    codePoint == 10 	ifTrue: [aStream nextPutAll: '\n'. codePoint := nil. 	] ifFalse: [
		    codePoint == 12 	ifTrue: [aStream nextPutAll: '\f'. codePoint := nil. 	] ifFalse: [
		    codePoint == 13 	ifTrue: [aStream nextPutAll: '\r'. codePoint := nil. 	]
      ]]]]].
      codePoint ifNotNil:[ | hex |
        hex := '0123456789ABCDEF'.
			  aStream nextPutAll: '\u' ;
             nextPut: (hex at: 1 + ((codePoint bitShift: -12) bitAnd: 16rF)) ;
             nextPut: (hex at: 1 + ((codePoint bitShift: -8) bitAnd: 16rF)) ;
             nextPut: (hex at: 1 + ((codePoint bitShift: -4) bitAnd: 16rF)) ;
             nextPut: (hex at: 1 + (codePoint bitAnd: 16rF)) .
      ]
	  ].
  ].
	aStream nextPut: $".
%

category: 'Formatting'
method: CharacterCollection
printOn: aStream

"Puts a displayable representation of the receiver on the given stream."

"Should be reimplemented for more efficiency in subclasses."

aStream nextPutAll: self quoted .
%

category: 'Formatting'
method: CharacterCollection
printOn: aStream recursionSet: anIdentitySet
	"Put a displayable representation of the receiver on the given stream
	 since CharacterCollections cannot have recursive references."

	self printOn: aStream
%

category: 'Formatting'
method: CharacterCollection
printString

"Returns a CharacterCollection whose contents are a displayable representation of the
 receiver."

^ self printStringWithMaxSize: 100000
%

category: 'Formatting'
method: CharacterCollection
printStringWithMaxSize: n

"Returns a CharacterCollection whose contents are a displayable representation of the
 receiver, limited to a specified number of characters <n>.

 If the number of characters in the displayable representation  exceeds <n>,
 display only the first <n>-1 characters and then display '. . .'. "

| ws |

ws := PrintStream printingOn: self speciesForPrint new maxSize: n.
self printOn: ws.
^ws contents
%

category: 'Formatting'
method: CharacterCollection
quoted

"Returns a copy of the receiver enclosed in single-quote marks, with contained
 single-quote Characters doubled.  The copy is of the same class as the
 receiver."

| sz result targetIdx lastIdx idx |
sz := self size.
result := self class _newString: sz + 2.
result at: 1 put: $'.
targetIdx := 2.
lastIdx := 1.
[ (idx := self indexOf: $' startingAt: lastIdx) == 0 ] whileFalse: [
 "self copyFrom: lastIdx to: idx into: result startingAt: targetIdx . "
  result replaceFrom: targetIdx to: targetIdx + idx - lastIdx
	 with: self startingAt: lastIdx .

  targetIdx := targetIdx + (idx - lastIdx) + 2.
  result at: targetIdx - 1 put: $'.
  lastIdx := idx + 1
].
lastIdx <= sz ifTrue: [
 "self copyFrom: lastIdx to: sz into: result startingAt: targetIdx "
  result replaceFrom: targetIdx to: targetIdx + sz - lastIdx
         with: self startingAt: lastIdx .
].
result at: targetIdx + (sz - lastIdx) + 1 put: $'.
^result
%

category: 'Comparing'
method: CharacterCollection
sameAs: aCharCollection

"Returns true if the receiver is equivalent to aCharCollection.  The receiver
 is equivalent to aCharCollection if the receiver contains the same Characters
 as aCharCollection regardless of case or internal representation.  For
 example, if $a is in aCharCollection, it is equivalent to any representation
 of an 'a' in the receiver's character set.
 From ANSI.  Used in Seaside. "

(aCharCollection isKindOf: CharacterCollection) ifFalse: [ ^false ].

self size ~~ aCharCollection size ifTrue: [ ^false ].

^ self isEquivalent: aCharCollection
%

category: 'Message Digests'
method: CharacterCollection
sha1Sum

"Return the 160 bit SHA1 checksum of the receiver as a LargeInteger.

 Computation is per FIPS PUB 180-3:
   http://csrc.nist.gov/publications/fips/fips180-3/fips180-3_final.pdf

 For DoubleByteString and QuadByteString, the computation is based
 on viewing the string as a ByteArray holding big-endian characters.
"

^ self _asMessageDigestKind: 2
%

category: 'Message Digests'
method: CharacterCollection
sha256Sum

"Return the 256 bit SHA256 checksum of the receiver as a LargeInteger.

 Computation is per FIPS PUB 180-3:
   http://csrc.nist.gov/publications/fips/fips180-3/fips180-3_final.pdf

 For DoubleByteString and QuadByteString, the computation is based
 on viewing the string as a ByteArray holding big-endian characters.
"

^ self _asMessageDigestKind: 3
%

category: 'Message Digests'
method: CharacterCollection
sha3_224Sum

"Return the SHA3 224 bit checksum of the receiver as a LargeInteger.

 Computation is per FIPS PUB 202
   https://nvlpubs.nist.gov/nistpubs/fips/nist.fips.202.pdf

 For DoubleByteString and QuadByteString, the computation is based
 on viewing the string as a ByteArray holding big-endian characters.
"

^ self _asMessageDigestKind: 5
%

category: 'Message Digests'
method: CharacterCollection
sha3_256Sum

"Return the SHA3 256 bit checksum of the receiver as a LargeInteger.

 Computation is per FIPS PUB 202
   https://nvlpubs.nist.gov/nistpubs/fips/nist.fips.202.pdf

 For DoubleByteString and QuadByteString, the computation is based
 on viewing the string as a ByteArray holding big-endian characters.
"

^ self _asMessageDigestKind: 6
%

category: 'Message Digests'
method: CharacterCollection
sha3_384Sum

"Return the SHA3 384 bit checksum of the receiver as a LargeInteger.

 Computation is per FIPS PUB 202
   https://nvlpubs.nist.gov/nistpubs/fips/nist.fips.202.pdf

 For DoubleByteString and QuadByteString, the computation is based
 on viewing the string as a ByteArray holding big-endian characters.
"

^ self _asMessageDigestKind: 7
%

category: 'Message Digests'
method: CharacterCollection
sha3_512Sum

"Return the SHA3 512 bit checksum of the receiver as a LargeInteger.

 Computation is per FIPS PUB 202
   https://nvlpubs.nist.gov/nistpubs/fips/nist.fips.202.pdf

 For DoubleByteString and QuadByteString, the computation is based
 on viewing the string as a ByteArray holding big-endian characters.
"

^ self _asMessageDigestKind: 8
%

category: 'Message Digests'
method: CharacterCollection
sha512Sum

"Return the 512 bit SHA512 checksum of the receiver as a LargeInteger.

 Computation is per FIPS PUB 180-3:
   http://csrc.nist.gov/publications/fips/fips180-3/fips180-3_final.pdf

 For DoubleByteString and QuadByteString, the computation is based
 on viewing the string as a ByteArray holding big-endian characters.
"

^ self _asMessageDigestKind: 4
%

category: 'Digital Signature Creation - EC'
method: CharacterCollection
signWithEcPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Signs the receiver with the given elliptic curve private key. Returns a ByteArray containing the resulting
 signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #EC ; _validateIsPrivateKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: 0 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - DSA'
method: CharacterCollection
signWithSha1AndDsaPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA1 and signs the resulting hash with the given
 DSA private key. Returns a ByteArray containing the resulting signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #DSA ; _validateIsPrivateKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: 2 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - RSA'
method: CharacterCollection
signWithSha1AndRsaPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA1 and signs the resulting hash with the given
 RSA private key using #PKCS1 padding. Returns a ByteArray containing the
 resulting signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #RSA ; _validateIsPrivateKey ; _validateIsNotRsaPss .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: 2 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - RSA'
method: CharacterCollection
signWithSha1AndRsaPssPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA1 and signs the resulting hash with the given
 RSA private key and PSS padding. Returns a ByteArray containing the resulting
 signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #RSA ; _validateIsPrivateKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: -2 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - DSA'
method: CharacterCollection
signWithSha256AndDsaPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA2 256 and signs the resulting hash with the given
 DSA private key. Returns a ByteArray containing the resulting signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #DSA ; _validateIsPrivateKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: 3 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - RSA'
method: CharacterCollection
signWithSha256AndRsaPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA 256 and signs the resulting hash with the given
 RSA private key using #PKCS1 padding. Returns a ByteArray containing the resulting
 signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #RSA ; _validateIsPrivateKey ; _validateIsNotRsaPss .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: 3 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - RSA'
method: CharacterCollection
signWithSha256AndRsaPssPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA2 256 and signs the resulting hash with the given
 RSA private key and PSS padding. Returns a ByteArray containing the resulting
 signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #RSA ; _validateIsPrivateKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: -3 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - RSA'
method: CharacterCollection
signWithSha3_224AndRsaPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA3 224 and signs the resulting hash with the given
 RSA private key using #PKCS1 padding. Returns a ByteArray containing the resulting
 signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #RSA ; _validateIsPrivateKey ; _validateIsNotRsaPss .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: 5 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - RSA'
method: CharacterCollection
signWithSha3_224AndRsaPssPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA3 224 and signs the resulting hash with the given
 RSA private key and PSS padding. Returns a ByteArray containing the resulting
 signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #RSA ; _validateIsPrivateKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: -5 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - RSA'
method: CharacterCollection
signWithSha3_256AndRsaPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA3 256 and signs the resulting hash with the given
 RSA private key using #PKCS1 padding. Returns a ByteArray containing the resulting
 signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #RSA ; _validateIsPrivateKey ; _validateIsNotRsaPss .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: 6 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - RSA'
method: CharacterCollection
signWithSha3_256AndRsaPssPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA3 256 and signs the resulting hash with the given
 RSA private key and PSS padding. Returns a ByteArray containing the resulting
 signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #RSA ; _validateIsPrivateKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: -6 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - RSA'
method: CharacterCollection
signWithSha3_384AndRsaPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA3 384 and signs the resulting hash with the given
 RSA private key using #PKCS1 padding. Returns a ByteArray containing the resulting
 signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #RSA ; _validateIsPrivateKey ; _validateIsNotRsaPss .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: 7 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - RSA'
method: CharacterCollection
signWithSha3_384AndRsaPssPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA3 384 and signs the resulting hash with the given
 RSA private key and PSS padding. Returns a ByteArray containing the resulting
 signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #RSA ; _validateIsPrivateKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: -7 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - RSA'
method: CharacterCollection
signWithSha3_512AndRsaPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA3 512 and signs the resulting hash with the given
 RSA private key using #PKCS1 padding. Returns a ByteArray containing the resulting
 signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #RSA ; _validateIsPrivateKey ; _validateIsNotRsaPss .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: 8 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - RSA'
method: CharacterCollection
signWithSha3_512AndRsaPssPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA3 512 and signs the resulting hash with the given
 RSA private key and PSS padding. Returns a ByteArray containing the resulting
 signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #RSA ; _validateIsPrivateKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: -8 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - DSA'
method: CharacterCollection
signWithSha512AndDsaPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA2 512 and signs the resulting hash with the given
 DSA private key. Returns a ByteArray containing the resulting signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #DSA ; _validateIsPrivateKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: 4 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - RSA'
method: CharacterCollection
signWithSha512AndRsaPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA2 512 and signs the resulting hash with the given
 RSA private key using #PKCS1 padding. Returns a ByteArray containing the resulting
 signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #RSA ; _validateIsPrivateKey ; _validateIsNotRsaPss .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: 4 signature: aByteArrayOrNil
%

category: 'Digital Signature Creation - RSA'
method: CharacterCollection
signWithSha512AndRsaPssPrivateKey: aGsTlsPrivateKey into: aByteArrayOrNil
"Hashes the receiver using SHA2 512 and signs the resulting hash with the given
 RSA private key and PSS padding. Returns a ByteArray containing the resulting
 signature.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPrivateKey _validateAlgorithm: #RSA ; _validateIsPrivateKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPrivateKey digestKind: -4 signature: aByteArrayOrNil
%

category: 'Updating'
method: CharacterCollection
space

"Appends a space to the receiver and returns the receiver."

self add: $ .
%

category: 'Private'
method: CharacterCollection
speciesForConversion

"Return the class of the receiver.  Subclasses should reimplement this method."

^ self class.
%

category: 'Converting'
method: CharacterCollection
subStrings

"Returns an Array of CharacterCollections where element represents a word in
 the receiver.  A word is a group of Characters separated by one or more
 separators."

^ self asArrayOfSubstrings.
%

category: 'Converting'
method: CharacterCollection
subStrings: separators
	"Returns an Array of CharacterCollections in which each element represents a
	 substring separated by any of the Characters in separators.  For compatibility with
	 existing code, separators may be a single Character or a collection of Characters.
	 The result will include empty substrings when two adjacent separators exist, as well as if
	 a separator is the first or last element."

	^separators size == 0
		ifTrue: [self subStringsDelimitedBy: separators]
		ifFalse: [self subStringsDelimitedByAny: separators].
%

category: 'Converting'
method: CharacterCollection
subStringsDelimitedBy: aCharacter
	"Returns an Array of CharacterCollections in which each element represents a
	 substring separated by aCharacter.  The result will include empty substrings when
	 two adjacent separators exist, as well as if a separator is the first or last element."

	| result startIndex endIndex sz |
	result := {}.
	startIndex := 1.

	[endIndex := self indexOf: aCharacter startingAt: startIndex.
	endIndex == 0]
			whileFalse:
				[endIndex == startIndex ifTrue: [result add: self class _newString].
				endIndex > startIndex
					ifTrue: [result add: (self copyFrom: startIndex to: endIndex - 1)].
				startIndex := endIndex + 1].
	startIndex > (sz := self size)
		ifTrue: [result add: self class _newString]
		ifFalse: [result add: (self copyFrom: startIndex to: sz)].
	^result
%

category: 'Converting'
method: CharacterCollection
subStringsDelimitedByAny: separators
	"Returns an Array of CharacterCollections in which each element represents a
	 substring separated by any of the Characters in separators.
	 The result will include empty substrings when two adjacent separators exist,
	 as well as if a separator is the first or last element."

	| nextName result |
	result := {}.
	nextName := self speciesForConversion new.
	self do:
			[:c |
			(separators includes: c)
				ifTrue:
					[result add: nextName.
					nextName := self speciesForConversion new]
				ifFalse: [nextName add: c]].
	result add: nextName.
	^result
%

category: 'Deprecated'
method: CharacterCollection
toServerTextFile: aFileSpec

self deprecated: 'CharacterCollection >> toServerTextFile: deprecated long before v3.0. Use an instance of GsFile
 to access the file system.'.
self _toServerTextFile: aFileSpec
%

category: 'Converting'
method: CharacterCollection
trimBlanks

"Returns a CharacterCollection containing the same Characters as the receiver,
 but with leading and trailing blanks removed."

self size == 0 ifTrue: [ ^ self ].
^ (self trimLeadingBlanks) trimTrailingBlanks.
%

category: 'Converting'
method: CharacterCollection
trimLeadingBlanks

"Returns a CharacterCollection containing the same Characters as the receiver,
 but with leading blanks removed."

| idx blank sz |
(sz := self size) == 0 ifTrue: [ ^ self ].
blank := Character space.
((self at: 1) == blank) ifFalse: [ ^ self ].
idx := 2.
[ true ] whileTrue:[
  idx <= sz ifFalse:[ ^ '' ].
  (self at: idx) == blank ifFalse:[ ^ self copyFrom: idx to: sz].
  idx := idx + 1
]
%

category: 'Converting'
method: CharacterCollection
trimLeadingSeparators

"Returns a CharacterCollection containing the same Characters as the receiver,
 but with leading separators removed."

| idx sz |
(sz := self size) == 0 ifTrue:[ ^ self ].
(self at: 1) isSeparator ifFalse: [ ^ self ].
idx := 2.
[ true ] whileTrue:[
  idx <= sz ifFalse:[ ^ '' ].
  (self at: idx) isSeparator ifFalse:[ ^ self copyFrom: idx to: sz].
  idx := idx + 1
]
%

category: 'Converting'
method: CharacterCollection
trimSeparators

"Returns a CharacterCollection containing the same Characters as the receiver,
 but with leading and trailing separators removed."

self size == 0 ifTrue: [ ^ self ].
^ (self trimLeadingSeparators) trimTrailingSeparators.
%

category: 'Converting'
method: CharacterCollection
trimTrailingBlanks

"Returns a CharacterCollection containing the same Characters as the receiver,
 but with trailing blanks removed."

| idx blank sz |

(sz := self size) == 0 ifTrue: [ ^ self ].
blank := Character space.
(self at: sz ) == blank ifFalse: [ ^ self ].
idx := sz  - 1.
[ true ] whileTrue:[
  idx < 1 ifTrue:[ ^ '' ] .
  (self at: idx) == blank ifFalse:[ ^ self copyFrom: 1 to: idx ].
  idx := idx - 1
]
%

category: 'Converting'
method: CharacterCollection
trimTrailingSeparators

"Returns a CharacterCollection containing the same Characters as the receiver,
 but with trailing separators removed."

| idx sz |

(sz := self size) == 0 ifTrue: [ ^ self ].
(self at: sz ) isSeparator ifFalse: [ ^ self ].
idx := sz  - 1.
[ true ] whileTrue:[
  idx < 1 ifTrue:[ ^ '' ] .
  (self at: idx) isSeparator ifFalse:[ ^ self copyFrom: 1 to: idx ].
  idx := idx - 1
]
%

category: 'Formatting'
method: CharacterCollection
trimWhiteSpace

"Returns a copy of the receiver with leading and trailing white space removed."

| first limit selfSize |

((selfSize := self size) == 0) ifTrue: [
   ^ self class _newString
].

limit := selfSize + 1.

first := 1 .
(self at: 1) isSeparator ifTrue: [ | j |
  first := nil .
  j := 2.
  [ j == limit ] whileFalse: [
      (self at: j) isSeparator ifTrue: [
         j := j + 1.
      ] ifFalse:[
         first := j.
         j := limit .
       ].
  ].
  first ifNil: [ ^ self class _newString ].
].

(self at: selfSize) isSeparator ifTrue: [ | k |
  k := selfSize - 1.
  [ k == 0 ] whileFalse: [
     (self at: k) isSeparator ifFalse:[
       ^ self copyFrom: first to: k
     ].
     k := k - 1.
  ].
  ^ self class _newString
].

first == 1 ifTrue:[  ^ self copy ].
^ self copyFrom: first to: selfSize .
%

category: 'Digital Signature Creation - EC'
method: CharacterCollection
verifyWithEcPublicKey: aGsTlsPublicKey signature: aByteArray
"Verifies the receiver using the given elliptic curve key and signature.  Returns true
 if the signature is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #EC ; _validateIsPublicKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: 0 signature: aByteArray
%

category: 'Digital Signature Verification - DSA'
method: CharacterCollection
verifyWithSha1AndDsaPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA1 and verifies the resulting hash using the
 given DSA public key and signature. Returns true if the signature is correct.
 Otherwise raises an exception.

 See the method:
  # _primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #DSA ; _validateIsPublicKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: 2 signature: aByteArray
%

category: 'Digital Signature Verification - RSA'
method: CharacterCollection
verifyWithSha1AndRsaPssPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA1 and verifies the resulting hash using the given
 RSA public key and signature with PSS padding. Returns true if the signature
 is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #RSA ; _validateIsPublicKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: -2 signature: aByteArray
%

category: 'Digital Signature Verification - RSA'
method: CharacterCollection
verifyWithSha1AndRsaPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA1 and verifies the resulting hash using the given
 RSA public key and signature with #PKCS1 padding. Returns true if the signature
 is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #RSA ; _validateIsPublicKey ; _validateIsNotRsaPss .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: 2 signature: aByteArray
%

category: 'Digital Signature Verification - DSA'
method: CharacterCollection
verifyWithSha256AndDsaPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA256 and verifies the resulting hash using the
 given DSA public key and signature. Returns true if the signature is correct.
 Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #DSA ; _validateIsPublicKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: 3 signature: aByteArray
%

category: 'Digital Signature Verification - RSA'
method: CharacterCollection
verifyWithSha256AndRsaPssPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA2 256 and verifies the resulting hash using the given
 RSA public key and signature with PSS padding. Returns true if the signature
 is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #RSA ; _validateIsPublicKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: -3 signature: aByteArray
%

category: 'Digital Signature Verification - RSA'
method: CharacterCollection
verifyWithSha256AndRsaPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA2 256 and verifies the resulting hash using the given
 RSA public key and signature with #PKCS1 padding. Returns true if the signature
 is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #RSA ; _validateIsPublicKey ; _validateIsNotRsaPss .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: 3 signature: aByteArray
%

category: 'Digital Signature Verification - RSA'
method: CharacterCollection
verifyWithSha3_224AndRsaPssPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA3 224 and verifies the resulting hash using the given
 RSA public key and signature with PSS padding. Returns true if the signature
 is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #RSA ; _validateIsPublicKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: -5 signature: aByteArray
%

category: 'Digital Signature Verification - RSA'
method: CharacterCollection
verifyWithSha3_224AndRsaPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA3 224 and verifies the resulting hash using the given
 RSA public key and signature with #PKCS1 padding. Returns true if the signature
 is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #RSA ; _validateIsPublicKey ; _validateIsNotRsaPss .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: 5 signature: aByteArray
%

category: 'Digital Signature Verification - RSA'
method: CharacterCollection
verifyWithSha3_256AndRsaPssPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA3 256 and verifies the resulting hash using the given
 RSA public key and signature with PSS padding. Returns true if the signature
 is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #RSA ; _validateIsPublicKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: -6 signature: aByteArray
%

category: 'Digital Signature Verification - RSA'
method: CharacterCollection
verifyWithSha3_256AndRsaPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA3 256 and verifies the resulting hash using the given
 RSA public key and signature with #PKCS1 padding. Returns true if the signature
 is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #RSA ; _validateIsPublicKey ; _validateIsNotRsaPss .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: 6 signature: aByteArray
%

category: 'Digital Signature Verification - RSA'
method: CharacterCollection
verifyWithSha3_384AndRsaPssPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA3 384 and verifies the resulting hash using the given
 RSA public key and signature with PSS padding. Returns true if the signature
 is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #RSA ; _validateIsPublicKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: -7 signature: aByteArray
%

category: 'Digital Signature Verification - RSA'
method: CharacterCollection
verifyWithSha3_384AndRsaPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA3 384 and verifies the resulting hash using the given
 RSA public key and signature with #PKCS1 padding. Returns true if the signature
 is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #RSA ; _validateIsPublicKey ; _validateIsNotRsaPss .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: 7 signature: aByteArray
%

category: 'Digital Signature Verification - RSA'
method: CharacterCollection
verifyWithSha3_512AndRsaPssPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA3 512 and verifies the resulting hash using the given
 RSA public key and signature with PSS padding. Returns true if the signature
 is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #RSA ; _validateIsPublicKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: -8 signature: aByteArray
%

category: 'Digital Signature Verification - RSA'
method: CharacterCollection
verifyWithSha3_512AndRsaPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA3 512 and verifies the resulting hash using the given
 RSA public key and signature with #PKCS1 padding. Returns true if the signature
 is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #RSA ; _validateIsPublicKey ; _validateIsNotRsaPss .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: 8 signature: aByteArray
%

category: 'Digital Signature Verification - DSA'
method: CharacterCollection
verifyWithSha512AndDsaPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA512 and verifies the resulting hash using the
 given DSA public key and signature. Returns true if the signature is correct.
 Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #DSA ; _validateIsPublicKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: 4 signature: aByteArray
%

category: 'Digital Signature Verification - RSA'
method: CharacterCollection
verifyWithSha512AndRsaPssPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA2 512 and verifies the resulting hash using the given
 RSA public key and signature with PSS padding. Returns true if the signature
 is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #RSA ; _validateIsPublicKey .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: -4 signature: aByteArray
%

category: 'Digital Signature Verification - RSA'
method: CharacterCollection
verifyWithSha512AndRsaPublicKey: aGsTlsPublicKey signature: aByteArray
"Hashes the receiver using SHA2 512 and verifies the resulting hash using the given
 RSA public key and signature with #PKCS1 padding. Returns true if the signature
 is correct. Otherwise raises an exception.

 See the method:
   #_primAsymSignVerifyWithKey:digestKind:signature:
 for more information."

aGsTlsPublicKey _validateAlgorithm: #RSA ; _validateIsPublicKey ; _validateIsNotRsaPss .
^ self _primAsymSignVerifyWithKey: aGsTlsPublicKey digestKind: 4 signature: aByteArray
%

category: 'Formatting'
method: CharacterCollection
width: anInteger

"Pads the receiver with spaces to create an object of size anInteger.
 If anInteger is positive, the spaces are added to the right of the receiver.
 If anInteger is negative, the spaces are added to the left of the receiver.
 If the size of the receiver is already greater than anInteger, the receiver
 is left unchanged."

| onRight s change |

change := anInteger abs - self size.
change > 0
ifTrue:
  [ onRight := anInteger > 0.
    s := (self speciesForConversion) new: change.
    s atAllPut: $ .
    onRight
    ifTrue:
      [ self addAll: s ]
    ifFalse:
      [ self insertAll: s at: 1 ]
  ]
%

category: 'Copying'
method: CharacterCollection
withCRs

"Supplied for Smalltalk-80 compatibility.  This is equivalent to withLFs."

^self withLFs
%

category: 'Formatting'
method: CharacterCollection
wrapTo: col

"Word-wrap the receiver to column col, treating tab Characters as modulo-8."

^ self _wrapTo: col indentingWith: ''
%

category: 'Message Digests'
method: CharacterCollection
_asMessageDigestKind: opCode

"
opCode   Digest Algorithm   Digest Bits
=======================================
  1          md5              128
  2          sha1             160
  3          sha2-256         256
  4          sha2-512         512
  5          sha3-224         224
  6          sha3-256         256
  7          sha3-384         384
  8          sha3-256         512
=======================================

"
<primitive: 666>
opCode _validateClass: SmallInteger .
((opCode < 1) or:[ opCode > 8])
  ifTrue:[ opCode _error: #rtErrArgOutOfRange args:{ 1 . 8 } ].
^ self _primitiveFailed: #_asMessageDigestKind:
%

category: 'Private'
method: CharacterCollection
_asUtf8WithEol
  "Return a Utf8 copy of the receiver, with a linefeed
   appended if receiver does not end with a linefeed.  Used by topaz"
  | lastCp sz str |
  sz := self size .
  sz ~~ 0 ifTrue:[ lastCp := self codePointAt: sz ].
  str := self .
  lastCp == 10 ifFalse:[
    (str := self copy) codePointAt: sz + 1 put: 10
  ].
  ^ str encodeAsUTF8
%

category: 'Accessing'
method: CharacterCollection
_at: anIndex

"Private.  Reimplemented to return a kind of Character."

^ self at: anIndex
%

category: 'Private'
method: CharacterCollection
_charCollCompare: aCharCollection

"Returns -1 if self < aCharCollection, returns 0 if self = aCharCollection,
 and returns 1 if self > aCharCollection."

| selfSize argSize selfElement argElement |

aCharCollection _validateClass: CharacterCollection.

selfSize:= self size.
argSize:= aCharCollection size.

1 to: (selfSize min: argSize) do: [:i |

  selfElement := self at: i.
  argElement  := aCharCollection at: i.

  (selfElement < argElement) ifTrue: [ ^ -1 ].
  (selfElement > argElement) ifTrue: [ ^ 1 ]
  ].

"All elements of self and argument are equal."

(selfSize < argSize) ifTrue: [ ^ -1 ].
(selfSize > argSize) ifTrue: [ ^ 1 ].

^ 0 "equal"
%

category: 'New Indexing Comparison'
method: CharacterCollection
_classSortOrdinal

^ 10
%

category: 'Comparing'
method: CharacterCollection
_coerceToUnicode
  ^ self asUnicodeString
%

category: 'Private'
method: CharacterCollection
_deepCopyWith: copiedObjDict

| copy |

copy := copiedObjDict at: self otherwise: nil.
copy ifNotNil:[ ^ copy ].

^ self copy.
%

category: 'Compatibility'
method: CharacterCollection
_encodeAsUTF8intoString

^ self encodeAsUTF8IntoString
%

category: 'Private'
method: CharacterCollection
_findLastString: subString startingAt: startIndex ignoreCase: aBoolean

"If a receiver contains subString beginning at some point at or before
 startIndex, this returns the index at which subString begins.  If the
 receiver does not contain subString, this returns 0.
 Search is backwards. "

 self  subclassResponsibility: #_findLastString:startingAt:ignoreCase: .

 self _uncontinuableError
%

category: 'Searching'
method: CharacterCollection
_findPattern: aPattern startingAt: anIndex ignoreCase: caseInsens

"This method searches the receiver, beginning at anIndex, for a substring that
 matches aPattern.  If a matching substring is found, this method returns the
 index of the first Character of the substring.  Otherwise, this returns 0.

 The argument aPattern is an Array containing zero or more CharacterCollections
 plus zero or more occurrences of the special Characters asterisk or
 question-mark.  See the description of the matchPattern: method for more
 information about this argument.

 If caseInsens is true, a case-insensitive search is performed.  Otherwise,
 caseInsens should be false and a case-sensitive search is performed."

| i             "loop counter"
  pattern       "the argument aPattern, converted to an Array"
  selfSize      "the size of the receiver"
  patternSize   "the number of elements in the pattern Array"
  startIndex    "an Array that corresponds to the pattern Array, with each
                 element containing the starting index into the receiver of
                 the corresponding pattern"
  index         "the index into both the pattern and the startIndex Arrays"
  next          "the current pattern (element of the pattern Array) to
                 be matched"
  cursor        "the index into the receiver"
|

aPattern _validateClass: Array.
(anIndex <= 0) ifTrue: [ ^ self _errorIndexOutOfRange: anIndex ].

pattern := Array withAll: aPattern.
patternSize := pattern size.

" First, the pattern Array must be processed so that there are no *? pairs
  it in.  They must all be converted to ?* pairs for the algorithm to work
  correctly."
i := 1.
[ i < patternSize ]
whileTrue:
  [ ( ((pattern at: i) isEquivalent: $*) and:[(pattern at: i+1) isEquivalent: $?])
    ifTrue:
      [ pattern at: i put: $?.
        pattern at: i+1 put: $*.
        i := 1 max: i-1.
      ]
    ifFalse:
      [ i := i + 1 ].
  ].

"initialize"
selfSize := self size.
startIndex := Array new: (patternSize + 1)."last element is set, but not used"
index := 1.
anIndex > selfSize
   ifTrue: [startIndex at: 1 put: (selfSize + 1)] "Fix for bug 15038"
   ifFalse: [startIndex at: 1 put: anIndex].

(patternSize == 0) "no pattern to match"
ifTrue:
  [ startIndex <= selfSize
    ifTrue:
      [ ^ startIndex ]
    ifFalse:
      [ ^ 0 ].
  ].

[index <= patternSize]
whileTrue:
  [ next := pattern at: index.
    cursor := startIndex at: index.

    (next isKindOf: CharacterCollection) "pattern element is a string"
    ifTrue:
      [ ((cursor > selfSize) or:
        [(cursor + next size - 1) > selfSize]) "pattern element too big to "
        ifTrue: "match beginning at cursor location"
          [ ^ 0 ].
        ((index == 1) or: [ (pattern at: index - 1) isEquivalent: $* ])
        ifTrue: "is first pattern or end of *; can skip chars to find match"
          [ cursor := self _findString: next startingAt: cursor
				ignoreCase: caseInsens .
            (cursor == 0)
            ifTrue:
              [ ^ 0 ]
            ifFalse:
              [ startIndex at: index put: cursor.
                startIndex at: index + 1 put: cursor + next size.
              ]
          ]
        ifFalse: "can't skip chars to find match"
          [ (self _at: cursor equals: next ignoreCase: caseInsens )
            ifTrue:
              [ startIndex at: index + 1 put: cursor + next size ]
            ifFalse:
              [
                [ (index := index - 1) < 1
                  ifTrue:
                    [ ^ 0 ].
                  ((pattern at: index) isKindOf: CharacterCollection) or:
                  [ (index == 1) or: [ (pattern at: index - 1) isEquivalent: $* ] ]
                ]
                untilTrue.
                startIndex at: index put: ((startIndex at: index) + 1).
                index := index - 1.
              ].
          ].
      ]
    ifFalse: "pattern element not a string"
      [ (next isEquivalent: $*) "pattern element is *"
        ifTrue:
          [ startIndex at: (index + 1) put: cursor
          ]
        ifFalse:
          [ (next isEquivalent: $?)  "pattern element is ?"
            ifTrue:
              [ cursor > selfSize
                ifTrue:
                  [ ^ 0 ].
                startIndex at: (index + 1) put: (cursor + 1)
              ]
            ifFalse: "found a pattern element other than ?, * or string"
              [ ^ aPattern _error: #rtErrBadPattern
              ].
          ].
      ].

    index := index + 1
  ].

^ startIndex at: 1
%

category: 'Private'
method: CharacterCollection
_findString: subString startingAt: startIndex ignoreCase: aBoolean

CharacterCollection
  subclassResponsibility: #_findString:startingAt:ignoreCase: .

self _uncontinuableError
%

category: 'Instance Creation'
method: CharacterCollection
_fromServerTextFile: aFileSpec

"Import the contents of aFileSpec into the receiver.
 For multi-byte characters, assumes the byte order of the file
 matches the current session's cpu's in-memory byte order.
"
<primitive: 302>

aFileSpec _validateByteClass: CharacterCollection .
aFileSpec _error: #hostErrFileImport args: #()
%

category: 'Indexing Support'
method: CharacterCollection
_idxBasicCanCompareWithClass: aClass
  "Returns true if the receiver may be inserted into a basic BtreeNode whose
   #lastElementClass is <aClass> (see RangeEqualityIndex class>>isBasicClass:)."

  "If using Unicode compares, then *String and Unicode* instances may be compared.
   If not using Unicode compares then *String and Unicode* instances may not be compared."

  ^ aClass _idxBasicCanCompareWithCharacterCollectionInstance: self
%

category: 'New Indexing Comparison - for Compare'
method: CharacterCollection
_idxForCompareCharacterCollectionGreaterThanOrEqualToSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForCompareGreaterThanOrEqualTo:. Note that aCharacterCollection should be the receiver in any >= comparison"

^(aCharacterCollection _idxPrimCompareLessThan: self) not
%

category: 'New Indexing Comparison - for Compare'
method: CharacterCollection
_idxForCompareCharacterCollectionGreaterThanSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForCompareGreaterThan:. Note that aCharacterCollection should be the receiver in any > comparison"

^aCharacterCollection _idxPrimCompareGreaterThan: self
%

category: 'New Indexing Comparison - for Compare'
method: CharacterCollection
_idxForCompareCharacterCollectionLessThanOrEqualToSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForCompareLessThanOrEqualTo:. Note that aCharacterCollection should be the receiver in any <= comparison"

^(aCharacterCollection _idxPrimCompareGreaterThan: self) not
%

category: 'New Indexing Comparison - for Compare'
method: CharacterCollection
_idxForCompareCharacterCollectionLessThanSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForCompareLessThan:. Note that aCharacterCollection should be the receiver in any < comparison"

^aCharacterCollection _idxPrimCompareLessThan: self
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForCompareEqualTo: arg
  ""

  ^ arg _idxForCompareEqualToCharacterCollection: self
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForCompareEqualTo: aCharacterCollection collator: anIcuCollator
  "treat receiver and aCharacterCollection as Unicode, since anIcuCollator is supplied"

  ^ aCharacterCollection
    _idxForCompareEqualToUnicode: self
    collator: anIcuCollator
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForCompareGreaterThan: arg

""

^arg _idxForCompareCharacterCollectionGreaterThanSelf: self
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForCompareGreaterThan: aCharacterCollection collator: anIcuCollator
  "treat receiver and aCharacterCollection as Unicode, since anIcuCollator is supplied"

  ^ aCharacterCollection
    _idxForCompareGreaterThanUnicode: self
    collator: anIcuCollator
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForCompareGreaterThanOrEqualTo: arg

""

^arg _idxForCompareCharacterCollectionGreaterThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForCompareGreaterThanOrEqualTo: aCharacterCollection collator: anIcuCollator

  ^ aCharacterCollection
    _idxForCompareGreaterThanOrEqualToUnicode: self
    collator: anIcuCollator
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForCompareLessThan: arg

""

^arg _idxForCompareCharacterCollectionLessThanSelf: self
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForCompareLessThan: aCharacterCollection collator: anIcuCollator

  ^ aCharacterCollection
    _idxForCompareLessThanUnicode: self
    collator: anIcuCollator
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForCompareLessThanOrEqualTo: arg

""

^arg _idxForCompareCharacterCollectionLessThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForCompareLessThanOrEqualTo: aCharacterCollection collator: anIcuCollator

  ^ aCharacterCollection
    _idxForCompareLessThanOrEqualToUnicode: self
    collator: anIcuCollator
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForCompareNotEqualTo: arg

""

^ (self _idxForCompareEqualTo: arg) not
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForCompareNotEqualTo: aCharacterCollection collator: anIcuCollator
  "REL_SIG_STRING_SIZE = 900"

  ^ (self _idxForCompareEqualTo: aCharacterCollection collator: anIcuCollator)
    not
%

category: 'New Indexing Comparison - for Sort'
method: CharacterCollection
_idxForSortCharacterCollectionGreaterThanOrEqualToSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForSortGreaterThanOrEqualTo:. Note that aCharacterCollection should be the receiver in any >= comparison"

^(aCharacterCollection _idxPrimCompareLessThan: self) not
%

category: 'New Indexing Comparison - for Sort'
method: CharacterCollection
_idxForSortCharacterCollectionGreaterThanSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForSortGreaterThan:. Note that aCharacterCollection should be the receiver in any > comparison"

^aCharacterCollection _idxPrimCompareGreaterThan: self
%

category: 'New Indexing Comparison - for Sort'
method: CharacterCollection
_idxForSortCharacterCollectionLessThanOrEqualToSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForSortLessThanOrEqualTo:. Note that aCharacterCollection should be the receiver in any <= comparison"

^(aCharacterCollection _idxPrimCompareGreaterThan: self) not
%

category: 'New Indexing Comparison - for Sort'
method: CharacterCollection
_idxForSortCharacterCollectionLessThanSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForSortLessThan:. Note that aCharacterCollection should be the receiver in any < comparison"

^aCharacterCollection _idxPrimCompareLessThan: self
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForSortEqualTo: arg

""

^arg _idxForSortEqualToCharacterCollection: self
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForSortEqualTo: aCharacterCollection collator: anIcuCollator

  ^ aCharacterCollection
    _idxForSortEqualToCharacterCollection: self
    collator: anIcuCollator
%

category: 'New Indexing Comparison - for Sort'
method: CharacterCollection
_idxForSortEqualToCharacterCollection: aCharacterCollection
  "second half of a double dispatch call from CharacterCollection>>_idxForSortEqualTo:."

  ^ self _idxPrimCompareEqualTo: aCharacterCollection
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForSortGreaterThan: arg

""

^arg _idxForSortCharacterCollectionGreaterThanSelf: self
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForSortGreaterThan: aCharacterCollection collator: anIcuCollator

  ^ aCharacterCollection
    _idxForSortGreaterThanCharacterCollection: self
    collator: anIcuCollator
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForSortGreaterThanOrEqualTo: arg

""

^arg _idxForSortCharacterCollectionGreaterThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForSortGreaterThanOrEqualTo: aCharacterCollection collator: anIcuCollator

  ^ aCharacterCollection
    _idxForSortGreaterThanOrEqualToCharacterCollection: self
    collator: anIcuCollator
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForSortLessThan: arg

""

^arg _idxForSortCharacterCollectionLessThanSelf: self
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForSortLessThan: aCharacterCollection collator: anIcuCollator

  ^ aCharacterCollection
    _idxForSortLessThanCharacterCollection: self
    collator: anIcuCollator
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForSortLessThanOrEqualTo: arg

""

^arg _idxForSortCharacterCollectionLessThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForSortLessThanOrEqualTo: aCharacterCollection collator: anIcuCollator

  ^ aCharacterCollection
    _idxForSortLessThanOrEqualToCharacterCollection: self
    collator: anIcuCollator
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForSortNotEqualTo: arg

""

^(arg _idxForSortEqualTo: self) not
%

category: 'New Indexing Comparison'
method: CharacterCollection
_idxForSortNotEqualTo: aCharacterCollection collator: anIcuCollator

  ^ (self _idxForSortEqualTo: aCharacterCollection collator: anIcuCollator) not
%

category: 'New Indexing Comparison - for Sort'
method: CharacterCollection
_idxForSortNotEqualToCharacterCollection: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForSortNotEqualTo:."

^ (self _idxPrimCompareEqualTo: aCharacterCollection) not
%

category: 'New Indexing Comparison - prims'
method: CharacterCollection
_idxPrimCompareEqualTo: arg
  "This comparison operation is used for the indexing subsystem to
 determine an ordering for insertion into indexing objects.

 This method collates letters AaBb..Zz."

  "The comparison should be compatible with the case-insensitive semantics
 of the String method with selector #= .
 Same primitive as String>>lessThan: "

  self subclassResponsibility: #_idxPrimCompareEqualTo:
%

category: 'New Indexing Comparison - prims'
method: CharacterCollection
_idxPrimCompareGreaterThan: arg
  "This comparison operation is used for the indexing subsystem to
 determine an ordering for insertion into indexing objects.

 This method collates letters AaBb..Zz."

  "The comparison should be compatible with the case-insensitive semantics
 of the String method with selector #< .
 Same primitive as String>>lessThan: "

  self subclassResponsibility: #_idxPrimCompareGreaterThan:
%

category: 'New Indexing Comparison - prims'
method: CharacterCollection
_idxPrimCompareLessThan: arg
  "This comparison operation is used for the indexing subsystem to
 determine an ordering for insertion into indexing objects.

 This method collates letters AaBb..Zz."

  "The comparison should be compatible with the case-insensitive semantics
 of the String method with selector #< .
 Same primitive as String>>lessThan: "

  self subclassResponsibility: #_idxPrimCompareLessThan:
%

category: 'Converting'
method: CharacterCollection
_isValid32PathTermName
  "Returns true if the receiver is a valid term in a path expression."

  | first sz maxSize |
  self = '*'
    ifTrue: [ ^ true ].
  maxSize := 64.
  ((sz := self size) > maxSize or: [ sz == 0 ])
    ifTrue: [ ^ false ].
  first := self at: 1.
  (first == $# or: [ first == $_ or: [ first isLetter ] ])
    ifFalse: [ ^ false ].
  2 to: sz do: [ :i |
    | c |
    c := self at: i.
    (c == $_ or: [ c == $| or: [ c isAlphaNumeric ] ])
      ifFalse: [ ^ false ] ].
  ^ true
%

category: 'Indexing Support'
method: CharacterCollection
_isValidPathTermName
  "Returns true if the receiver is a valid term in a path expression."

  ^ self _isValid32PathTermName
%

category: 'Private'
method: CharacterCollection
_keySizeInBytesForOpCode: opCode

| code codes128Bit codes192Bit codes256Bit |
code := opCode abs .
codes128Bit := { 1 . 4 . 7 } .
codes192Bit := { 2 . 5 . 8 } .
codes256Bit := { 3 . 6 . 9 . 10 } .

(codes128Bit includesIdentical: code)
  ifTrue:[ ^ 16 ].
(codes192Bit includesIdentical: code)
  ifTrue:[ ^ 24 ].
(codes256Bit includesIdentical: code)
  ifTrue:[ ^ 32 ].
%

category: 'Private'
method: CharacterCollection
_lineNumberFor:  offset
  "Counts LF characters in the receiver up to offset and
   returns an Array of the form { line number,  offset in line } "
  | cnt lf eolPos |
  cnt := 1 . lf := Character lf .
  eolPos := 0 .
  1 to: (offset min: self size) do:[:n|
    (self at: n) == lf ifTrue:[ eolPos := n . cnt := cnt + 1 ].
  ].
  ^ { cnt . offset - eolPos }
%

category: 'Testing'
method: CharacterCollection
_literalEqual: anotherLiteral
  "Petit Parser definition:
     For two literals to be _literalEqual, their class must be identical and
     otherwise equal.
     For CharacterCollections equality is sufficent
   In GemStone we allow Unicode7 to compare to String, etc."

  ^ self _unicodeEqual: anotherLiteral
%

category: 'Private'
method: CharacterCollection
_primAsymSignVerifyWithKey: aKey digestKind: opCode signature: aByteArray

"Signs or verifies the message contained in the receiver using public key
 encryption. aKey must be an instance of GsTlsPrivateKey (which indicates a
 signing operation) or GsTlsPublicKey (which indicates a verify operation).
 For signing keys that require a message digest, anInt indicates one of the
 supported message digests for the specified key type. For signing keys
 that do not require a message digest, anInt must be zero.  See the tables
 below. The same message digest must be used for both signing and
 verifying.

 For signing operations, aByteArray must be a variant instance of ByteArray
 or nil, in which case a new instance of ByteArray will be created to
 contain the signature. For verifying operations, aByteArray must be a
 non-empty instance of ByteArray containing the signature from the signing.

 For secure signing and verifying with RSA keys only, a padding scheme must
 be used. The default RSA padding scheme is RSA_PKCS1_PADDING
 (PKCS #1 v1.5 padding), which is the most common type. However the newer
 PSS (Probabilistic Signature Scheme) is more secure and is recommended
 whenever possible. See RFC 3447 for additional information on PSS padding.
 Other RSA padding schemes, including no padding, are not supported due
 to known security vulnerabilities. The padding scheme selected must be the
 same for the signature and verification else the verification will fail.

 To sign or verify with RSA_PKCS1_PSS_PADDING, negate the digest opCode.
 For example, a message signed with an RSA private key using opCode 2 uses
 SHA1 message digest and (the default) RSA_PKCS1_PADDING. Signing a message
 with an RSA private key using opCode -2 uses SHA1 message digest and
 RSA_PKCS1_PSS_PADDING padding.

 RSA keys of type RSA-PSS may only use RSA_PKCS1_PSS_PADDING. PSS padding
 will be used for such keys in all cases, even if RSA_PKCS1_PADDING is
 requested. RSA-PSS private keys will answer #EVP_PKEY_RSA_PSS when sent
 the #sslAlgorithm message.

 Signing operations return aByteArray containing the signature on success
 and raise a CryptoError exception on error. Verify operations return true
 if the verification succeeds and raise a CryptoError exception if
 verification fails or an error occurs.

 Note that not all private/public key pairs support digital signatures.

 Key Type  Supported Digest Types
 ================================
 DSA       SHA1,SHA2
 ECDSA	   SHA1,SHA2
 RSA	   SHA1,SHA2,SHA3
 EC        None
 Ed25519   None
 Ed448     None
 ==============================


 Digest Type	OpCode
 =====================
 None           0
 SHA1		2
 SHA2-256	3
 SHA2-512	4
 SHA3-224	5
 SHA3-256	6
 SHA3-384	7
 SHA3-512	8
 =====================
"

<primitive: 1089>
opCode _validateClass: SmallInteger .
aKey _validateClasses: { GsTlsPrivateKey . GsTlsPublicKey } .
(aKey isKindOf: GsTlsPrivateKey)
  ifTrue:[ "signing, arg may be nil"
    aByteArray ifNotNil:[ aByteArray _validateClass: ByteArray ].
  ]
  ifFalse:[
     "verifying, arg must be a ByteArray"
    aByteArray _validateClass: ByteArray
].
self _validateKey: aKey withDigestKind: opCode .
^ self _primitiveFailed: #_primAsymSignVerifyWithKey:digestKind:signature:
%

category: 'Private'
method: CharacterCollection
_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode into: destObjOrNil
"Private method for invoking non-AEAD encrypt/decrypt modes. See the method:
    #_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
    into: destObjOrNil tag: tag extraData: eData
 for more information."

^ self _primEncryptDecryptWithKey: aKey
       salt: aSalt
       opCode: opCode
       into: destObjOrNil
       tag: nil
       extraData: nil
%

category: 'Private'
method: CharacterCollection
_primEncryptDecryptWithKey: aKey salt: aSalt opCode: opCode
into: destObjOrNil tag: tag extraData: eData

"Private method for encrypting or decrypting the receiver.
 The following encryption schemes are currently supported:

 ==================================================
                          Key     Salt    Tag
 opCode Cipher   Mode  bits/Bytes Size   Bytes
 ==================================================
   1     AES     CBC     128/16    16     N/A
   2     AES     CBC     192/24    16     N/A
   3     AES     CBC     256/32    16     N/A
   4     AES     OCB     128/16    12     16
   5     AES     OCB     192/24    12     16
   6     AES     OCB     256/32    12     16
   7     AES     GCM     128/16    12     16
   8     AES     GCM     192/24    12     16
   9     AES     GCM     256/32    12     16
  10   CHACHA20 Poly1305 256/32    12     16
 ==================================================

 AES encryption/decryption (Advanced Encryption Standard) is performed
 using the OpenSSL open source package and the AES specification,
 available at:
   http://csrc.nist.gov/publications/fips/fips197/fips-197.pdf

 CBC is an acronym for cipher block chaining. See the referenced AES
 specification document for further details.

 OCB is an acronym for Offset Cookbook Mode. See RFC 7253 for further
 details.

 GCM is an acronym for Galois Counter Mode. See RFC 5288 for further
 details.

 OCB, GCM and Poly1305 are AEAD modes. AEAD is an acronym for
 Authenticated Encryption with Associated Data. AEAD provides data
 authenticity, confidentiality, and integrity. It also supports
 Additional Authenticated Data (AAD). AAD is not encrypted and therefore
 not kept confidential, however AAD authenticity and integrity are
 guaranteed. AAD is not included in the encrypted payload but must be
 provided in order to decrypt the data. AAD is optional and the eData
 argument may be nil if AAD is not required.

 AEAD encryption/decryption is performed using the OpenSSL open source
 package and is implemented to conform to ISO/IEC standard 19772:2009.
 See https://www.iso.org/standard/46345.html for further information.

 opCode must be an instance of SmallInteger and be one of the values from
 the first column of the above table. A positive value indicates the data
 is to be encrypted and a negative value indicates decryption.

 aKey and aSalt must be instances of ByteArray of the correct size
 per the above table, otherwise an error is raised.

 destObjOrNil must be nil or an instance of a non-invariant byte object.
 If destObjOrNil is nil, the result of the operation will be placed into a
 new instance of ByteArray (encryption) or String (decryption). Otherwise
 the result will be placed into the given byte object starting at offset 1.
 The size of destObjOrNil will be modified to correctly contain all
 encrypted or decrypted data, and may differ from the size of the receiver
 due to the automatic addition or removal of padding by the cipher
 algorithm.

 When encrypting a receiver that has a character size greater than one, data
 is placed into big-endian byte order before encryption.

 When decrypting into a destObjOrNil object that a character size greater
 than one, data is converted to big-endian byte order after decryption.

 During AEAD encryption, a tag is generated which is used during decryption
 to ensure data integrity. The tag data will be stored into the tag
 argument, which must an instance of a variant byte object. During AEAD
 decryption, tag must be a byte object containing the tag bytes returned
 during encryption. For non-AEAD modes, tag must be nil.

 During AEAD encryption, eData must be nil or a byte object with a character
 size of one containing additional data to be used in generating the tag
 value. eData is NOT added to the encrypted payload. During decryption,
 eData must be a byte object with a character size of one containing the
 same bytes provided during encryption or nil if no byte object was
 provided. For non-AEAD modes, eData must be nil.

 Successful encryption or decryption returns encrypted or decrypted data,
 which is stored into destObjOrNil if it was provided or a new byte object
 if it was not. An exception is raised if encryption or decryption fails.
"

<primitive: 953>
self _validateEncryptionOpCode: opCode ;
     _validateEncryptionKey: aKey forOpCode: opCode ;
     _validateEncryptionSalt: aSalt forOpCode: opCode ;
     _validateEncryptionExtraData: eData forOpCode: opCode .
destObjOrNil ifNotNil:[ destObjOrNil _validateIsBytes ; validateIsVariant ] .
self _validateEncryptionTag: tag forOpCode: opCode .
^ self _primitiveFailed: #_primEncryptDecryptWithKey:salt:opCode:into:tag:extraData:
%

category: 'Private'
method: CharacterCollection
_saltSizeForOpCode: opCode

^ opCode abs >= 4
    ifTrue:[ 12 ]
   ifFalse:[ 16 ]
%

category: 'Deprecated'
method: CharacterCollection
_toServerTextFile: aFileSpec

 "Writes the receiver to the specified file
 using the current session's cpu's in-memory byte order of any multi-byte characters.

 The argument aFileSpec must be convertable to a Utf8 , otherwise an error is signaled
 by the primitive."

<primitive: 301>
self deprecated: 'CharacterCollection>>_toServerTextFile: deprecated long before v3.0.  Use an instance of GsFile
 to access the file system.'.

aFileSpec _validateByteClass: CharacterCollection .
self _error: #hostErrFileExport args: { aFileSpec }
%

category: 'Private'
method: CharacterCollection
_validate: anObj isSize: expectedSize

anObj _basicSize == expectedSize
  ifFalse:[ anObj _error: #rtErrBadSize args: { expectedSize . anObj _basicSize } ] .
%

category: 'Private'
method: CharacterCollection
_validateEncryptionExtraData: eData forOpCode: opCode
opCode abs > 3  "AEAD encrypt/decrypt"
  ifTrue:[ eData ifNotNil:[ eData _validateIsBytes ]]
 ifFalse:[ eData _validateClass: UndefinedObject ]
%

category: 'Private'
method: CharacterCollection
_validateEncryptionKey: aKey forOpCode: opCode
aKey _validateClass: ByteArray .
self _validate: aKey isSize: (self _keySizeInBytesForOpCode: opCode)
%

category: 'Private'
method: CharacterCollection
_validateEncryptionOpCode: opCode
opCode _validateClass: SmallInteger .
(((opCode < -10) or:[opCode > 10]) or:[opCode == 0])
  ifTrue:[opCode _error: #rtErrArgOutOfRange args:{ -10 . 10 } ] .
^ true
%

category: 'Private'
method: CharacterCollection
_validateEncryptionSalt: salt forOpCode: opCode
salt _validateClass: ByteArray .
self _validate: salt isSize: (self _saltSizeForOpCode: opCode)
%

category: 'Private'
method: CharacterCollection
_validateEncryptionTag: tag forOpCode: opCode
| absCode |
absCode := opCode abs .
absCode > 3 "AEAD encrypt/decrypt"
  ifTrue:[
    tag _validateClass: ByteArray .
    opCode < 3 ifTrue:[ tag validateIsVariant ]			"AEAD Encrypting"
              ifFalse:[ self _validate: tag isSize: 16 ] . 	"AEAD Decrypting"
  ] ifFalse:[ tag _validateClass: UndefinedObject ] "Not AEAD"
%

category: 'Private'
method: CharacterCollection
_validateInteger: anInt inRangeFrom: lowest to: highest
anInt _validateClass: SmallInteger .
((anInt < lowest) or:[anInt > highest])
  ifTrue:[ anInt _error: #rtErrArgOutOfRange args:{ lowest . highest } ] .
^ true
%

category: 'Private'
method: CharacterCollection
_validateKey: aKey withDigestKind: opCode
|alg|
alg := aKey algorithm . "#RSA, #DSA, #DH, #EC, etc"
alg == #RSA "SHA1,2,3 allowed"
  ifTrue:[ ^ self _validateInteger: opCode abs inRangeFrom: 2 to: 8 ].
alg == #DSA "SHA1 and SHA2 only"
  ifTrue:[ ^ self _validateInteger: opCode inRangeFrom: 2 to: 4 ].
alg == #EC "No digest required/allowed"
  ifTrue:[ ^ self _validateInteger: opCode inRangeFrom: 0 to: 0 ].

^ CryptoError signal: 'Invalid key kind for signing or verifying'
%

category: 'Testing'
method: CharacterCollection
_validIdentifier: maxSize
| first sz |

((sz := self size) > maxSize or:[sz == 0]) ifTrue:[ ^false ].

first := self at: 1.
(first == $_ or:[first isLetter]) ifFalse:[ ^false ].

2 to: sz do: [:i | | c |
  c := self at: i.
  (c == $_  or:[ c isAlphaNumeric]) ifFalse:[ ^false ].
].
^ true
%

category: 'Formatting'
method: CharacterCollection
_wrapTo: col indentingWith: indentStr

"Returns a new instance of the class of the receiver.

 Word-wrap the receiver to column col, treating tab Characters as modulo-8.
 Whenever a line-feed is inserted, prepend indentStr to the subsequent line."

| ch curcol linestart wordstart lf tab str sz |

lf := Character lf.
tab := Character tab.
curcol := 1.
wordstart := 0.
linestart := 1.
str := self class _newString.

1 to: (sz := self size) do: [ :i |
  ch := self at: i.
  ch == lf ifTrue: [
    str add: (self copyFrom: linestart to: i).
    linestart := i + 1.
    wordstart := 0.
    curcol := 1.
  ]
  ifFalse: [
    ch isSeparator ifTrue: [
      wordstart := 0
    ]
    ifFalse: [
      wordstart == 0 ifTrue: [
        wordstart := i
      ].
    ].

    ch == tab ifTrue: [
      curcol := (curcol + 8) \\ 8
    ]
    ifFalse: [
      curcol := curcol + 1
    ].

    curcol > col ifTrue: [
      (wordstart == 0 or: [linestart == wordstart]) ifTrue: [
	str add: (self copyFrom: linestart to: i).
	linestart := i + 1.
	curcol := 1.
      ]
      ifFalse: [
	str add: (self copyFrom: linestart to: wordstart - 1).
	linestart := wordstart.
	curcol := i - wordstart + 1.
      ].
      str add: lf.
      str add: indentStr .
      curcol := curcol + indentStr size .
    ].
  ].
].

linestart <= sz ifTrue: [
  str add: (self copyFrom: linestart to: sz )
].

^str.
%

! Class extensions for 'Class'

!		Class methods for 'Class'

removeallmethods Class
removeallclassmethods Class

category: 'Filein Support'
classmethod: Class
_resolveReservedClass: reservedOop name: className

  " returns an Array of size 2.
     result at:1 is already existing class or nil .
     result at: 2 is a String describing Globals modifications made"

  | oldCls result |
  result := Array new: 2 .
  result at: 2 put: String new .
  reservedOop ifNotNil:[
    oldCls := Object _objectForOop: reservedOop .
  ] ifNil:[
    oldCls := (System myUserProfile resolveSymbol: className asSymbol) ifNotNil:[:assoc | assoc value ]
  ].
  result at:1 put: oldCls .
  ^ result
%

!		Instance methods for 'Class'

category: 'Class Instance Variables'
method: Class
addClassInstanceVariable: civNameString

"Adds the given class instance variable to the receiver's metaclass.
 Generates an error if the receiver is either not modifiable or does
 not disallow subclasses."

self _validatePrivilege ifTrue:[
  self class addInstVarNames: { civNameString }
]
%

category: 'Updating Variables'
method: Class
addClassVarName: aString

"Add aString to the class variable list for the receiver, if the
 class variable is not already defined."

| aSym definingClass |
self _validatePrivilege ifTrue:[
  aSym := aString asSymbol .
  aSym validateIsIdentifier .
  definingClass := self _classDefiningClassVar: aSym .  "fix bug 10480"
  definingClass ifNotNil:[
    definingClass == self ifTrue:[
      "if the receiver already defines the class variable,
         do nothing and return silently    (fix bug 8094) "
      ^ self
      ].
    LookupError new object: definingClass; key: aSym ; reason: #classErrClassVarNameExists ;
		details: 'class variable already exists'; signal .
    ^ self
  ].
  self _addClassVar: aSym value: nil .
]
%

category: 'Updating'
method: Class
addNewVersion: aClass

"Make aClass a new version of the receiver.  That is, add aClass to the
 receiver's history, and set aClass's history to be the same as the
 receiver's history.  The existing history of aClass will have aClass
 removed from it."

self _validatePrivilege ifTrue:[
  aClass classHistory removeVersion: aClass.
  aClass classHistory: classHistory.
  classHistory newVersion: aClass
].
%

category: 'Updating Variables'
method: Class
addSharedPool: aDictionary

"Add aDictionary to the end of the shared pool list for the receiver. "
| poolDicts |
self _validatePrivilege ifTrue:[
  (aDictionary _validateClass:  SymbolDictionary ) ifTrue:[
     poolDicts := poolDictionaries .
     (poolDicts ~~ nil and:[ poolDicts includesIdentical: aDictionary]) ifTrue:[
        ^ self _error: #classErrPoolDictExists args: { aDictionary }
     ].
     poolDicts ifNil:[ poolDicts := { } . poolDictionaries := poolDicts ]
       ifNotNil:[ poolDicts isInvariant ifTrue:[
                    poolDicts := Array withAll: poolDicts . poolDictionaries := poolDicts
                ]].
     poolDicts add: aDictionary
  ].
].
%

category: 'Listing Instances'
method: Class
allInstances

"Returns an Array that contains all instances of the receiver.
 Note: This method scans the entire GemStone repository, and may therefore
 take some time to execute.

 Large result sets may cause out-of-memory issues. To avoid problems,
 use Repository >> allReferences:, which returns a GsBitmap that does not
 require keeping objects in temporary object space.

 This method aborts the current transaction; if an abort would cause unsaved
 changes to be lost, it signals an error, #rtErrAbortWouldLoseData."

| list |

list := IdentitySet new.
list addAll: ((SystemRepository listInstancesInMemory: {self}) at: 1).
list addAll: ((SystemRepository listInstances: { self }) at: 1).
^list asArray
%

category: 'Queries'
method: Class
allSubclasses

	^ClassOrganizer new allSubclassesOf: self.
%

category: 'Queries'
method: Class
allSuperclasses

	^ClassOrganizer new allSuperclassesOf: self.
%

category: 'Class Instance Variables'
method: Class
atClassInstVar: varName

"Returns the value of the given class instance variable in the receiver.
 Generates an error if the argument does not name a class instance variable
 in the receiver.  In general, it is more efficient to implement a direct
 accessing method for a class instance variable."

| idx cls varSym |
varSym := Symbol _existingWithAll: varName .
varSym == nil ifTrue:[
  self _error: #classErrNotAVar args: { varName } .
  ^ nil
  ].
cls := self class.
idx := cls.instVarNames indexOf: varSym.
(idx < 0 or: [idx <= cls class instSize ]) ifTrue: [
  self _error: #classErrNotAVar args: { varName } .
  ^ nil
].
^ self instVarAt: idx
%

category: 'Class Instance Variables'
method: Class
atClassInstVar: varName put: newValue
	"Changes the value of the given class instance variable in the receiver,
 without regard to the variance or invariance of the receiver.  Generates an
 error if the argument does not name a class instance variable in the receiver.
 Returns the argument 'newValue'."

	self _validatePrivilege
		ifTrue:
			[| idx cls varSym |
			varSym := Symbol _existingWithAll: varName.
			varSym == nil
				ifTrue:
					[self _error: #classErrNotAVar args: {varName}.
					^nil].
			cls := self class.
			idx := cls.instVarNames indexOf: varSym.
			(idx < 0 or: [idx <= cls class instSize])
				ifTrue:
					[self _error: #classErrNotAVar args: {varName}.
					^nil].
			self _unsafeAt: idx put: newValue.
			^newValue].
	^nil
%

category: 'Deprecated'
method: Class
byteSubclass: aString
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
category: aCategoryName
inDictionary: aDictionary
instancesInvariant: invarBoolean

| cl |
self deprecated: 'The preferred methods are in the Subclass Creation category.'.
cl := self byteSubclass: aString
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  category: aCategoryName
  inDictionary: aDictionary
  instancesInvariant: invarBoolean .
cl category: aCategoryName asString.
^cl
%

category: 'Subclass Creation'
method: Class
byteSubclass: aString
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
category: aCategoryName
inDictionary: aDictionary
options: optionsArray

| cl |
cl := self byteSubclass: aString
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  options: optionsArray .
cl category: aCategoryName asString.
^cl
%

category: 'Subclass Creation'
method: Class
byteSubclass: aString
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary

^ self byteSubclass: aString
    classVars: anArrayOfClassVars
    classInstVars: anArrayOfClassInstVars
    poolDictionaries: anArrayOfPoolDicts
    inDictionary: aDictionary
    newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
    description: nil
    options: #()
%

category: 'Deprecated'
method: Class
byteSubclass: aString
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
description: aDescription
isInvariant: invarBoolean

 | opts |
self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
 opts := invarBoolean ifTrue:[ { #instancesInvariant } ]
           ifFalse:[ self instancesInvariant ifTrue:[ ^ self _error: #classErrInvariantSuperClass ].
                     #() ] .
^self
  byteSubclass: aString
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
  description: aDescription
  options: opts
%

category: 'Deprecated'
method: Class
byteSubclass: aString
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
inClassHistory: aClassHistory
description: aDescription
isInvariant: invarBoolean

	| theFormat opts |
	self
		deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category ('
				, aString , ').'.
	(aDictionary _validateClass: SymbolDictionary) ifFalse: [^nil].
	self instSize ~~ 0 ifTrue: [^self _error: #classErrByteObjInstVars].
	self isNsc
		ifTrue:
			[^aString _error: #classErrBadFormat
				with: 'cannot create byte subclass of Nsc class'].
	theFormat := (format bitAnd: 16r3 bitInvert) bitOr: 16r1 + 16r4.
	opts := invarBoolean
				ifTrue: [{#instancesInvariant}]
				ifFalse:
					[self instancesInvariant
						ifTrue: [^self _error: #classErrInvariantSuperClass].
					#()].
	^self
		_subclass: aString
		instVarNames: #()
		format: theFormat
		classVars: anArrayOfClassVars
		classInstVars: anArrayOfClassInstVars
		poolDictionaries: anArrayOfPoolDicts
		inDictionary: aDictionary
		inClassHistory: aClassHistory
		description: aDescription
		options: opts
%

category: 'Deprecated'
method: Class
byteSubclass: aString
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
instancesInvariant: invarBoolean

 | opts |
 self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods
  are in the Subclass Creation category (' , aString , ').'.

 opts := invarBoolean ifTrue:[ { #instancesInvariant } ]
           ifFalse:[ self instancesInvariant ifTrue:[ ^ self _error: #classErrInvariantSuperClass ].
                     #() ] .
^self
  byteSubclass: aString
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
  description: nil
  options: opts
%

category: 'Deprecated'
method: Class
byteSubclass: aString
	classVars: anArrayOfClassVars
	classInstVars: anArrayOfClassInstVars
	poolDictionaries: anArrayOfPoolDicts
	inDictionary: aDictionary
	newVersionOf: oldClass
	description: aDescription
	isInvariant: invarBoolean

	| hist theFormat opts descr |
	self
		deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category ('
				, aString , ').'.
	aDictionary _validateClass: SymbolDictionary.
	self instSize ~~ 0 ifTrue: [^self _error: #classErrByteObjInstVars].
	self isNsc
		ifTrue:
			[^aString _error: #classErrBadFormat
				with: 'cannot create byte subclass of Nsc class'].
	theFormat := (format bitAnd: 16r3 bitInvert) bitOr: 16r1 + 16r4.
	opts := invarBoolean
				ifTrue: [{#instancesInvariant}]
				ifFalse:
					[self instancesInvariant
						ifTrue: [^self _error: #classErrInvariantSuperClass].
					#()].
	descr := aDescription.
	oldClass
		ifNotNil:
			[(self
				_equivalentSubclass: oldClass
				superCls: self
				name: aString
				newOpts: opts
				newFormat: theFormat
				newInstVars: #()
				newClassInstVars: anArrayOfClassInstVars
				newPools: anArrayOfPoolDicts
				newClassVars: anArrayOfClassVars
				inDict: aDictionary
				isKernel: false)
					ifTrue:
						[oldClass _commentOrDescription: aDescription.
						^oldClass	"avoid creation of a new version"].
			hist := oldClass classHistory.
			descr
				ifNil:
					[descr := [oldClass commentForFileout] on: Error
								do: [:ex | 'old comment not available']]].
	^self
		_subclass: aString
		instVarNames: #()
		format: theFormat
		classVars: anArrayOfClassVars
		classInstVars: anArrayOfClassInstVars
		poolDictionaries: anArrayOfPoolDicts
		inDictionary: aDictionary
		inClassHistory: hist
		description: descr
		options: opts
%

category: 'Subclass Creation'
method: Class
byteSubclass: aString
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
newVersionOf: oldClass
description: aDescription
options: optionsArray

	"Creates and returns a new byte subclass of the receiver.  You are not
 permitted to modify the new class after it is created.  If the receiver is not
 some kind of String class, then instances of the new class store and return
 SmallIntegers in the range 0 - 255.

 This method generates an error if instances of the receiver are of special
 storage format, if they are NSCs, or if they have instance variables.

 optionsArray is an Array of Symbols containing zero or more of
   #noInheritOptions,  #subclassesDisallowed, #disallowGciStore, #modifiable ,
   #traverseByCallback #selfCanBeSpecial #logCreation
 and at most one of
   #dbTransient, #instancesNonPersistent, #instancesInvariant
 and at most one of the word size definitions for byte format (affects swizzling)
   #'2byteWords' #'4byteWords' #'8byteWords'
   #'signed2byteWords' #'signed4byteWords' #'signed8byteWords'
 If present, #noInheritOptions must be the first element and it causes
 none of subclassesDisallowed, disallowGciStore, traverseByCallback,
         dbTransient, instancesNonPersistent, instancesInvariant
 to be inherited from the superclass, nor copied from the
 current version of the class.
 #selfCanBeSpecial is never inherited and is needed only when modifying
 superclass hierarchy above classes with special format.
 The option #logCreation, if present in the options array, causes logging
 with GsFile(C)>>gciLogSever:  of class creation / equivalence.

 Returns oldClass if it would be equivalent to the requested new class.
 (See the class comment for Class). "

	| hist fmt descr |
	aDictionary
		ifNotNil: [ (aDictionary _validateClass: SymbolDictionary) ifFalse: [^nil] ].
	self isNsc
		ifTrue:
			[^aString _error: #classErrBadFormat
				with: 'cannot create byte subclass of Nsc class'].
	fmt := (format bitAnd: 16r3 bitInvert) bitOr: 16r1 + 16r4.
	descr := aDescription.
	oldClass
		ifNotNil:
			[(self
				_equivalentSubclass: oldClass
				superCls: self
				name: aString
				newOpts: optionsArray
				newFormat: fmt
				newInstVars: #()
				newClassInstVars: anArrayOfClassInstVars
				newPools: anArrayOfPoolDicts
				newClassVars: anArrayOfClassVars
				inDict: aDictionary
				isKernel: false )
					ifTrue:
						[oldClass _commentOrDescription: aDescription.
						^oldClass	"avoid creation of a new version" ].
			hist := oldClass classHistory.
			descr ifNil: [descr := [ oldClass commentForFileout ] on: Error do:[:ex | 'old comment not available']]].
	^self
		_subclass: aString
		instVarNames: #()
		format: fmt
		classVars: anArrayOfClassVars
		classInstVars: anArrayOfClassInstVars
		poolDictionaries: anArrayOfPoolDicts
		inDictionary: aDictionary
		inClassHistory: hist
		description: descr
		options: optionsArray
%

category: 'Deprecated'
method: Class
byteSubclass: aString
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
newVersionOf: oldClass
instancesInvariant: invarBoolean

 | opts |
self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
 opts := invarBoolean ifTrue:[ { #instancesInvariant } ]
           ifFalse:[ self instancesInvariant ifTrue:[ ^ self _error: #classErrInvariantSuperClass ].
                     #() ] .
 ^ self byteSubclass: aString
    classVars: anArrayOfClassVars
    classInstVars: anArrayOfClassInstVars
    poolDictionaries: anArrayOfPoolDicts
    inDictionary: aDictionary
    newVersionOf: oldClass
    description: (oldClass ifNotNil:[ [ oldClass commentForFileout ] on: Error do:[:ex | 'old comment not available']])
    options: opts
%

category: 'Subclass Creation'
method: Class
byteSubclass: aString
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
options: optionsArray


"Creates and returns a new byte subclass of the receiver.  You are not
 permitted to modify the new class after it is created.  If the receiver is not
 some kind of String class, then instances of the new class store and return
 SmallIntegers in the range 0 - 255.

 If aString is the name of a Class that is visible to the current user, this
 method creates the new class as a new version of the existing class, and they
 then share the same class history.  However, if no class named aString is
 visible to the user, then the new class is no relation to any existing class,
 and it has a new class history.

 This method generates an error if instances of the receiver are of special
 storage format, if they are NSCs, or if they have instance variables.

 optionsArray is an Array of Symbols containing zero or more of
   #noInheritOptions,  #subclassesDisallowed, #disallowGciStore, #modifiable ,
   #traverseByCallback #selfCanBeSpecial #logCreation
 and at most one of
   #dbTransient, #instancesNonPersistent, #instancesInvariant
 and at most one of the word size definitions for byte format (affects swizzling)
   #'2byteWords' #'4byteWords' #'8byteWords'
   #'signed2byteWords' #'signed4byteWords' #'signed8byteWords'
 If present, #noInheritOptions must be the first element and it causes
 none of subclassesDisallowed, disallowGciStore, traverseByCallback,
         dbTransient, instancesNonPersistent, instancesInvariant
 to be inherited from the superclass, nor copied from the
 current version of the class.
 #selfCanBeSpecial is never inherited and is needed only when modifying
 superclass hierarchy above classes with special format.
 The option #logCreation, if present in the options array, causes logging
 with GsFile(C)>>gciLogSever:  of class creation / equivalence.
"

^ self byteSubclass: aString
    classVars: anArrayOfClassVars
    classInstVars: anArrayOfClassInstVars
    poolDictionaries: anArrayOfPoolDicts
    inDictionary: aDictionary
    newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
    description: nil
    options: optionsArray
%

category: 'Deprecated'
method: Class
byteSubclass: aString
classVars: anArrayOfClassVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
inClassHistory: aClassHistory
description: aDescription
isInvariant: invarBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^self byteSubclass: aString
  classVars: anArrayOfClassVars
  classInstVars: #()
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  inClassHistory: aClassHistory
  description: aDescription
  isInvariant: invarBoolean
%

category: 'Deprecated'
method: Class
byteSubclass: aString
classVars: anArrayOfClassVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
isInvariant: invarBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^ self byteSubclass: aString
    classVars: anArrayOfClassVars
    classInstVars: #()
    poolDictionaries: anArrayOfPoolDicts
    inDictionary: aDictionary
    instancesInvariant: invarBoolean
%

category: 'Subclass Creation'
method: Class
byteSubclass: aString
classVars: anArrayOfClassVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
options: optionsArray

^ self byteSubclass: aString
    classVars: anArrayOfClassVars
    classInstVars: #()
    poolDictionaries: anArrayOfPoolDicts
    inDictionary: aDictionary
    options: optionsArray
%

category: 'Instance Migration'
method: Class
cancelMigration

"Disables class migration by clearing the migrationDestination instance
 variable."

self migrationDestination: nil.
%

category: 'Category'
method: Class
category: newCategory

"Sets the classCategory variable of the receiver.
 The argument should be a kind of CharacterCollection or nil."

	self _rwCategory: newCategory
%

category: 'Accessing'
method: Class
classHistory

"Returns the classHistory instance variable for this class, which may be nil."

^ classHistory
%

category: 'Updating'
method: Class
classHistory: aClassHistory

"Set the class history of the receiver. Returns the receiver."

self _validatePrivilege ifTrue:[
  aClassHistory ifNotNil:[
    (aClassHistory _validateInstanceOf: ClassHistory) ifFalse:[ ^ nil ].
  ].
  classHistory := aClassHistory .
  self _refreshClassCache: false .
].
%

category: 'Accessing'
method: Class
classHistoryAt: aSmallInteger

"Returns the specified version of this class."

^ classHistory at: aSmallInteger .
%

category: 'Clustering'
method: Class
clusterBehavior

"Clusters elements of the receiver and its metaclass that are used
 for GemStone Smalltalk execution.

 It is recommended that when several classes are being clustered in a
 transaction, send clusterBehavior to all classes to be clustered, then send
 clusterDescription."

super clusterBehavior ifTrue:[ ^ true ].
self class clusterBehavior .
^ false
%

category: 'Clustering'
method: Class
clusterDescription

"Clusters elements of the receiver and its metaclass that are not required
 for GemStone Smalltalk execution.

 It is recommended that when several classes are being clustered in a
 transaction, send clusterBehavior to all classes to be clustered, then send
 clusterDescription."

| result |
super clusterDescription ifTrue:[ ^ true ].
result := false .
extraDict ifNotNil:[
  result := extraDict cluster .
  result ifFalse:[ extraDict associationsDo:[:assoc| assoc cluster]].
].
result ifFalse:[ timeStamp ifNotNil:[ result := timeStamp cluster ]].
userId ifNotNil:[ userId cluster ].
result ifFalse:[ result := self class clusterDescription ].
^ result
%

category: 'Documentation'
method: Class
comment

" As of GS/64 3.1, comments are now recorded in the class extraDict
  dictionary under the key #comment.  Comment information formerly
  recorded as a GsClassDocumentation under the key #description are
  converted to a string and placed under #comment during DB
  conversion/upgrade. "

  | str |
  (self _extraDictAt: #comment) ifNotNil:[:cmt | ^ cmt ].
  str := 'No class-specific documentation for ' , self name .
  str add: ', hierarchy is:
'; add: (self hierarchy: 0) .
  ^ str
%

category: 'Documentation'
method: Class
comment: aString

  (aString isKindOf: CharacterCollection) ifFalse: [
    ArgumentTypeError signal: 'Comment must be a String' ].
  self _extraDictAt: #comment put: aString
%

category: 'Documentation'
method: Class
commentForFileout

"Returns a non-empty class comment or nil."

| str |
str := self _extraDictAt: #comment .
str size = 0 ifTrue:[ ^ nil ].
^ str
%

category: 'Private'
method: Class
copyVariables
| chSize chist priorVersion priorVars civNames priorCivNames toIgnoreCount |
(chSize := (chist := classHistory) size) = 1 ifTrue:[
  ^ self
].
priorVersion := chist at: chSize - 1.
(priorVars := priorVersion _classVars) notNil ifTrue: [
  | cvars |
  (cvars := classVars) notNil ifTrue: [
    priorVars associationsDo: [:anAssociation | |aKey |
      (cvars includesKey: (aKey := anAssociation key)) ifTrue: [
        cvars removeKey: aKey .
        cvars addAssociation: anAssociation.
      ].
    ].
  ].
].
priorCivNames := priorVersion class allInstVarNames.
toIgnoreCount := Class allInstVarNames size.
toIgnoreCount < priorCivNames size ifTrue: [
	priorCivNames := priorCivNames
		copyFrom: toIgnoreCount + 1
		to: priorCivNames size.
	civNames := self class allInstVarNames.
	priorCivNames do: [:each |
		(civNames includesIdentical: each) ifTrue: [
			self atClassInstVar: each put: (priorVersion atClassInstVar: each).
		].
	].
].
%

category: 'Versions'
method: Class
currentVersion

  "return the most recent version of the receiver"
  classHistory ifNotNil:[ :hist | | sz |
    (sz := hist size) ~~ 0 ifTrue:[ ^ hist at: sz ].
  ].
  ^ self
%

category: 'Browser Methods'
method: Class
definition
	"Returns a String containing a GemStone Smalltalk definition for the receiver
 (that is, a subclass creation message).  This method uses the UserProfile
 of the owner of the current session as the correct context."
	"For use with the Topaz run command."

	^self _definitionInContext: System myUserProfile
%

category: 'Accessing'
method: Class
description

"Returns the description of this class."

self deprecated: 'Class>>description deprecated v3.1. Replaced by comment.'.
^ self _description
%

category: 'Documentation'
method: Class
description: aDescription

"Update the description of this Class.  Returns the argument."
self deprecated: 'Deprecated as of GS/64 3.1'.
^ self _description: aDescription
%

category: 'Accessing'
method: Class
extraDict

"Returns the SymbolDictionary held in extraDict that holds miscellaneous
 information about the receiver.  Result may be nil. "

^ extraDict
%

category: 'Updating'
method: Class
extraDict: aSymbolDictionary

"Set the value of the extraDict instance variable."

self _validatePrivilege ifTrue:[
  aSymbolDictionary ifNotNil:[
    (aSymbolDictionary _validateClass: SymbolDictionary) ifFalse:[ ^ self].
  ].
  extraDict := aSymbolDictionary
].
%

category: 'Accessing'
method: Class
extraDictForStore

"Returns the SymbolDictionary held in extraDict that holds miscellaneous
 information about the receiver.  Creates the dictionary if needed."

^ extraDict ifNil:[ extraDict := SymbolDictionary new ].
%

category: 'Deprecated'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfStrings
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
category: aCategoryName
inDictionary: aDictionary
instancesInvariant: invarBoolean
isModifiable: modifyBoolean

| cl |
self deprecated: 'The preferred methods are in the Subclass Creation category.'.
cl := self indexableSubclass: aString
   instVarNames: anArrayOfStrings
   classVars: anArrayOfClassVars
   classInstVars: anArrayOfClassInstVars
   poolDictionaries: anArrayOfPoolDicts
   inDictionary: aDictionary
   instancesInvariant: invarBoolean
   isModifiable: modifyBoolean .
cl category: aCategoryName asString.
^cl
%

category: 'Subclass Creation'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfStrings
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
category: aCategoryName
inDictionary: aDictionary
options: optionsArray

| cl |
cl := self
  indexableSubclass: aString
  instVarNames: anArrayOfStrings
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  options: optionsArray  .
cl category: aCategoryName asString.
^cl
%

category: 'Subclass Creation'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary

"Executes
   oldClass := aDictionary at:(aString asSymbol) otherwise: nil
 and returns oldClass if it would be equivalent to the requested new class.
 (See the class comment for Class). "

^ self indexableSubclass: aString
    instVarNames: anArrayOfInstvarNames
    classVars: anArrayOfClassVars
    classInstVars: anArrayOfClassInstVars
    poolDictionaries: anArrayOfPoolDicts
    inDictionary: aDictionary
    newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
    description: nil
    options: #()
%

category: 'Deprecated'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: aConstraint
instancesInvariant: invarBoolean
description: aDescription
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^self
  indexableSubclass: aString
  instVarNames: anArrayOfInstvarNames
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  constraints: aConstraint
  instancesInvariant: invarBoolean
  newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
  description: aDescription
  isModifiable: modifyBoolean
%

category: 'Deprecated'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: ignored
instancesInvariant: invarBoolean
inClassHistory: aClassHistory
description: aDescription
isModifiable: modifyBoolean

| opts fmt |
self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
"Any specified constraints are ignored."
(self isBytes) ifTrue: [ ^ aString _error: #classErrBadFormat with: 'cannot create indexable subclass of byte class'].
(self isNsc) ifTrue: [ ^ aString _error: #classErrBadFormat with: 'cannot create indexable subclass of Nsc class'].
opts := { } .
invarBoolean ifTrue:[ opts add: #instancesInvariant ]
           ifFalse:[ self instancesInvariant ifTrue:[ ^ self _error: #classErrInvariantSuperClass ]].
modifyBoolean ifTrue:[ opts add: #modifiable ].
fmt := format bitOr: 16r4 "add indexable bit" .

^ self _subclass: aString
        instVarNames: anArrayOfInstvarNames
        format: fmt
        classVars: anArrayOfClassVars
        classInstVars: anArrayOfClassInstVars
        poolDictionaries: anArrayOfPoolDicts
        inDictionary: aDictionary
        inClassHistory: aClassHistory
        description: aDescription
        options: opts
%

category: 'Deprecated'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: aConstraint
instancesInvariant: invarBoolean
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^self
  indexableSubclass: aString
  instVarNames: anArrayOfInstvarNames
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  constraints: aConstraint
  instancesInvariant: invarBoolean
  newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
  description: nil
  isModifiable: modifyBoolean
%

category: 'Deprecated'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: ignored
instancesInvariant: invarBoolean
newVersionOf: oldClass
description: aDescription
isModifiable: modifyBoolean

	| opts fmt hist descr |
	self
		deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category ('
				, aString , ').'.
	"Any specified constraints are ignored."
	self isBytes
		ifTrue:
			[^aString _error: #classErrBadFormat
				with: 'cannot create indexable subclass of byte class'].
	self isNsc
		ifTrue:
			[^aString _error: #classErrBadFormat
				with: 'cannot create indexable subclass of Nsc class'].
	opts := {}.
	invarBoolean
		ifTrue: [opts add: #instancesInvariant]
		ifFalse:
			[self instancesInvariant
				ifTrue: [^self _error: #classErrInvariantSuperClass]].
	modifyBoolean ifTrue: [opts add: #modifiable].
	fmt := format bitOr: 16r4.	"add indexable bit"
	descr := aDescription.
	oldClass
		ifNotNil:
			[(self
				_equivalentSubclass: oldClass
				superCls: self
				name: aString
				newOpts: opts
				newFormat: fmt
				newInstVars: anArrayOfInstvarNames
				newClassInstVars: anArrayOfClassInstVars
				newPools: anArrayOfPoolDicts
				newClassVars: anArrayOfClassVars
				inDict: aDictionary
				isKernel: false)
					ifTrue:
						["avoid creation of a new version"
						oldClass _commentOrDescription: aDescription.
						^oldClass].
			hist := oldClass classHistory.
			descr
				ifNil:
					[descr := [oldClass commentForFileout] on: Error
								do: [:ex | 'old comment not available']]].
	^self
		_subclass: aString
		instVarNames: anArrayOfInstvarNames
		format: fmt
		classVars: anArrayOfClassVars
		classInstVars: anArrayOfClassInstVars
		poolDictionaries: anArrayOfPoolDicts
		inDictionary: aDictionary
		inClassHistory: hist
		description: descr
		options: opts
%

category: 'Deprecated'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: aConstraint
instancesInvariant: invarBoolean
newVersionOf: oldClass
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^ self indexableSubclass: aString
    instVarNames: anArrayOfInstvarNames
    classVars: anArrayOfClassVars
    classInstVars: anArrayOfClassInstVars
    poolDictionaries: anArrayOfPoolDicts
    inDictionary: aDictionary
    constraints: aConstraint
    instancesInvariant: invarBoolean
    newVersionOf: oldClass
    description: nil
    isModifiable: modifyBoolean
%

category: 'Deprecated'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
instancesInvariant: invarBoolean
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^self
  indexableSubclass: aString
  instVarNames: anArrayOfInstvarNames
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  instancesInvariant: invarBoolean
  newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
  isModifiable: modifyBoolean
%

category: 'Deprecated'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
instancesInvariant: invarBoolean
newVersionOf: oldClass
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^ self indexableSubclass: aString
    instVarNames: anArrayOfInstvarNames
    classVars: anArrayOfClassVars
    classInstVars: anArrayOfClassInstVars
    poolDictionaries: anArrayOfPoolDicts
    inDictionary: aDictionary
    constraints: #()
    instancesInvariant: invarBoolean
    newVersionOf: oldClass
    description: (oldClass ifNotNil:[ [ oldClass commentForFileout ] on: Error do:[:ex | 'old comment not available']])
    isModifiable: modifyBoolean
%

category: 'Subclass Creation'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
newVersionOf: oldClass
description: aDescription
options: optionsArray
	"Creates and returns a new indexable subclass of the receiver.  Instances of the
 new class are represented as pointer objects.

 This method generates an error if instances of the receiver are of special
 storage format or if they are NSCs.

 optionsArray is an Array of Symbols containing zero or more of
   #noInheritOptions,  #subclassesDisallowed, #disallowGciStore, #modifiable ,
   #traverseByCallback #selfCanBeSpecial #logCreation
 and at most one of
   #dbTransient, #instancesNonPersistent, #instancesInvariant
 If present, #noInheritOptions must be the first element and it causes
 none of subclassesDisallowed, disallowGciStore, traverseByCallback,
         dbTransient, instancesNonPersistent, instancesInvariant
 to be inherited from the superclass, nor copied from the
 current version of the class.
 #selfCanBeSpecial is never inherited and is needed only when modifying
 superclass hierarchy above classes with special format.
 The option #logCreation, if present in the options array, causes logging
 with GsFile(C)>>gciLogSever:  of class creation / equivalence.

 Returns oldClass if it would be equivalent to the requested new class.
 (See the class comment for Class). "

	| hist fmt descr |
	self isBytes
		ifTrue:
			[^aString _error: #classErrBadFormat
				with: 'cannot create indexable subclass of byte class'].
	self isNsc
		ifTrue:
			[^aString _error: #classErrBadFormat
				with: 'cannot create indexable subclass of Nsc class'].
	fmt := format bitOr: 16r4.	"add indexable bit"
	descr := aDescription.
	oldClass
		ifNotNil:
			[(self
				_equivalentSubclass: oldClass
				superCls: self
				name: aString
				newOpts: optionsArray
				newFormat: fmt
				newInstVars: anArrayOfInstvarNames
				newClassInstVars: anArrayOfClassInstVars
				newPools: anArrayOfPoolDicts
				newClassVars: anArrayOfClassVars
				inDict: aDictionary
				isKernel: false )
					ifTrue:
						[oldClass _commentOrDescription: aDescription.
						^oldClass	"avoid creation of a new version"].
			hist := oldClass classHistory.
			descr ifNil: [descr := [ oldClass commentForFileout ] on: Error do:[:ex | 'old comment not available']]].
	^self
		_subclass: aString
		instVarNames: anArrayOfInstvarNames
		format: fmt
		classVars: anArrayOfClassVars
		classInstVars: anArrayOfClassInstVars
		poolDictionaries: anArrayOfPoolDicts
		inDictionary: aDictionary
		inClassHistory: hist
		description: descr
		options: optionsArray
%

category: 'Subclass Creation'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
options: optionsArray

"Creates and returns a new indexable subclass of the receiver.  Instances of the
 new class are represented as pointer objects.

 This method generates an error if instances of the receiver are of special
 storage format or if they are NSCs.

 optionsArray is an Array of Symbols containing zero or more of
   #noInheritOptions,  #subclassesDisallowed, #disallowGciStore, #modifiable ,
   #traverseByCallback #selfCanBeSpecial #logCreation
 and at most one of
   #dbTransient, #instancesNonPersistent, #instancesInvariant
 If present, #noInheritOptions must be the first element and it causes
 none of subclassesDisallowed, disallowGciStore, traverseByCallback,
         dbTransient, instancesNonPersistent, instancesInvariant
 to be inherited from the superclass, nor copied from the
 current version of the class.
 #selfCanBeSpecial is never inherited and is needed only when modifying
 superclass hierarchy above classes with special format.
 The option #logCreation, if present in the options array, causes logging
 with GsFile(C)>>gciLogSever:  of class creation / equivalence.

 Executes
   oldClass := aDictionary at:(aString asSymbol) otherwise: nil
 and returns oldClass if it would be equivalent to the requested new class.
 (See the class comment for Class). "

^ self indexableSubclass: aString
    instVarNames: anArrayOfInstvarNames
    classVars: anArrayOfClassVars
    classInstVars: anArrayOfClassInstVars
    poolDictionaries: anArrayOfPoolDicts
    inDictionary: aDictionary
    newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
    description: nil
    options: optionsArray
%

category: 'Deprecated'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: aConstraint
instancesInvariant: invarBoolean
inClassHistory: aClassHistory
description: aDescription
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^self indexableSubclass: aString
  instVarNames: anArrayOfInstvarNames
  classVars: anArrayOfClassVars
  classInstVars: #()
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  constraints: aConstraint
  instancesInvariant: invarBoolean
  inClassHistory: aClassHistory
  description: aDescription
  isModifiable: modifyBoolean
%

category: 'Deprecated'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: aConstraint
instancesInvariant: invarBoolean
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^ self indexableSubclass: aString
    instVarNames: anArrayOfInstvarNames
    classVars: anArrayOfClassVars
    classInstVars: #()
    poolDictionaries: anArrayOfPoolDicts
    inDictionary: aDictionary
    constraints: aConstraint
    instancesInvariant: invarBoolean
    isModifiable: modifyBoolean
%

category: 'Deprecated'
method: Class
indexableSubclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: aConstraint
isInvariant: invarBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^ self indexableSubclass: aString
     instVarNames: anArrayOfInstvarNames
     classVars: anArrayOfClassVars
     classInstVars: #()
     poolDictionaries: anArrayOfPoolDicts
     inDictionary: aDictionary
     constraints: aConstraint
     instancesInvariant: invarBoolean
     isModifiable: false
%

category: 'Queries'
method: Class
instancesInMemory
"Returns an Array that contains all instances of the receiver.
 that are in the temporary object memory of this session."

^ (SystemRepository listInstancesInMemory: { self }) at: 1
%

category: 'Displaying'
method: Class
instanceString

"Returns a symbol that can be used to name an instance of the receiver."

^(('AEIOUYaeiouy' indexOf: (name at: 1) startingAt: 1) ~~ 0
   ifTrue: ['an']
   ifFalse: ['a'])
 , name
%

category: 'Displaying'
method: Class
instanceSymbol

"Returns a symbol that can be used to name an instance of the receiver."

^ ((('AEIOUYaeiouy' indexOf: (name at: 1) startingAt: 1) ~~ 0
  ifTrue: ['An']
  ifFalse: ['A'])
    , name) asSymbol
%

category: 'Class Instance Variables'
method: Class
instVarAt: anIndex put: aValue
  | sz |
  self _validatePrivilege ifTrue:[
    anIndex <= Class instSize ifTrue:[
      ImproperOperation new object: self ;
         signal: 'instVarAt:put: not allowed to a class instance variable defined for Class'
    ].
    anIndex > (sz := self class instSize) ifTrue:[
       OutOfRange new name:#anIndex max: sz actual: anIndex ; signal .
    ].
    self _unsafeAt: anIndex put: aValue .
    ^ aValue
  ].
  ^ nil
%

category: 'Versions'
method: Class
isVersionOf: anotherClass

"Returns whether the receiver and the given class share the same class
 history."

| hist |
^ ( hist := classHistory) ~~ nil and:[ anotherClass classHistory == hist ].
%

category: 'Locking'
method: Class
lockableParts

"Returns an Array of the receiver's contents that are locked by browsers
 and folders."

| parts ed |
parts := super lockableParts.
(ed := extraDict) ifNotNil:[
	parts addLast: ed.
	ed do: [:each | parts addLast: each].
].
^ parts reject: [:each | each == nil or:[ each == #()] ].
%

category: 'Modifying Classes'
method: Class
makeInstancesDbTransient

  "Takes effect immediately and will cause any instances that get
   committed to be DbTransient.

   For a DbTransient object, instVars on disk are always nil.
   The first transaction which causes the object to be reachable
   from committed state will commit the object with all instVars on
   disk set to nil .  Thereafter any stores to instVars of the committed
   object in any session do not cause the object to be written
   by the transaction.

   Clustering a committed DbTransient object will
   cause the object to be rewritten to disk with all instVars nil
   when the transaction commits.  So clustering is the only way
   that concurrency conflict could occur involving a DbTransient object.

   The format of the receiver must be    non-indexable, pointer
   otherwise an error is generated.

   You cannot change a non-modifiable class to or from dbTransient ."

  <primitive: 2001>   "enter protected mode"
  | prot |
  prot := System _protectedMode .
  [
    self validateIsModifiable .
    self _makeInstancesDbTransient: true .
  ] ensure: [
    prot _leaveProtectedMode
  ].
%

category: 'Modifying Classes'
method: Class
makeInstancesNonPersistent

  "Takes effect immediately and will prevent committing new instances
   of the receiver in the current transaction.

   To change a non-modifiable class from persistent to non-persistent ,
   see  ClassOrganizer >> makeInstancesNonPersistent: . "

  <primitive: 2001>   "enter protected mode"
  | prot |
  prot := System _protectedMode .
  [ self validateIsModifiable .
    self _makeInstancesNonPersistent .
  ] ensure:[
    prot _leaveProtectedMode
  ].
%

category: 'Modifying Classes'
method: Class
makeInstancesNotDbTransient

  "Takes effect immediately and cancels any previous makeInstancesDbTransient.

   The receiver must be a modifiable class."

  <primitive: 2001>   "enter protected mode"
  | prot |
  prot := System _protectedMode .
  [
    self validateIsModifiable .
    self _makeInstancesDbTransient: false .
  ] ensure:[
    prot _leaveProtectedMode .
  ].
%

category: 'Modifying Classes'
method: Class
makeInstancesPersistent

  "Takes effect immediately.
   To change a non-modifiable class from non-persistent to persistent,
   see  ClassOrganizer >> makeInstancesPersistent: . "

  <primitive: 2001>   "enter protected mode"
  | prot |
  prot := System _protectedMode .
  [ self isModifiable ifFalse:[ self validateIsModifiable . ^ self ].
    self _makeInstancesPersistent .
  ] ensure:[
    prot _leaveProtectedMode
  ].
%

category: 'Method Timestamps'
method: Class
methodStampDictName
  ^ #GSMethodStampDict
%

category: 'Instance Migration'
method: Class
migrateTo: aClass

"Enables class migration by setting the migrationDestination instance
 variable."

self _validatePrivilege ifTrue:[
  self migrationDestination: aClass.
].
%

category: 'Accessing'
method: Class
migrationDestination

"Returns the migrationDestination instance variable of this class."

^ destClass
%

category: 'Updating'
method: Class
migrationDestination: aClass

"Update the migrationDestination instance variable.  Returns the argument."

self _validatePrivilege ifTrue:[
  aClass ifNotNil:[ aClass _validateClass: Class ].
  destClass := aClass .
  self _refreshClassCache: false .
].
^ aClass
%

category: 'Repository Conversion'
method: Class
needsRecompileFor30

 "Returns true if the receiver needs to have methods recompiled."

 ^ super needsRecompileFor30 or:[ self class needsRecompileFor30]
%

category: 'Repository Conversion'
method: Class
needsRecompileFor33

 "Returns true if the receiver needs to have methods recompiled."

 ^ super needsRecompileFor33 or:[ self class needsRecompileFor33]
%

category: 'Instance Creation'
method: Class
new

"Returns an instance of the receiver with no indexed variables."
<primitive: 51>
self _primitiveFailed: #new .
self _uncontinuableError
%

category: 'Instance Creation'
method: Class
new: anInteger

"Returns an instance of the receiver with the specified number of indexed
 variables.  Generates an error if the receiver is not indexable or if
 anInteger is not a positive SmallInteger.

 For new byte objects, all indexed variables are set to zero;
 for new pointer objects, all indexed variables are set to nil."

<primitive: 53>
(self isIndexable) ifFalse:[ self _errorNotIndexable .  ^ self new ].
(anInteger _isInteger)
  ifFalse:[ anInteger _validateClass: Integer . ^ self new ]
  ifTrue:[
    (anInteger < 0) ifTrue:[ anInteger _error: #rtErrArgNotPositive .
                            ^ self new].
    anInteger _error: #rtErrArgOutOfRange .
    ^ self new
  ].
self _primitiveFailed: #new: args: { anInteger }.
self _uncontinuableError
%

category: 'Pragmas'
method: Class
pragmaDictName

  ^ #GSMethodPragmaDict
%

category: 'Repository Conversion'
method: Class
recompileAllMethods

"Recompile all methods for execution in a Gs64 v3.0 or later system."

super recompileAllMethods .
self class recompileAllMethods
%

category: 'Browser Methods'
method: Class
recompileWithDicts: symbolList

"Recompiles all the receiver's instance and class methods for envId 0.
 Returns the CompiledMethods that fail to compile properly."
| failed |
failed := { }.
self _validatePrivilege ifTrue:[
  | cls envId |
  cls := self .
  envId := 0 .
  2 timesRepeat: [
    cls env: envId unifiedCategoriesDo:[ :categName :selectorList |
      selectorList copy do: [ :aSel| | oldMeth |
        [ oldMeth := cls compiledMethodAt: aSel environmentId: envId .
          cls compileMethod: oldMeth sourceString dictionaries: symbolList
		          category: categName environmentId: envId
        ] onException: CompileError do:[:ex |
          failed add: oldMeth
        ].
      ].
    ].
    cls := self class .
  ].
].
^ failed.
%

category: 'Updating Variables'
method: Class
removeClassVarName: aString

"Remove the class variable named aString from the receiver.
 The value of the removed Association is set to nil.
 Signals an error if there is no such class variable, or if the
 Association for that class variable is invariant."

self _validatePrivilege ifTrue:[
  (Symbol _existingWithAll: aString) ifNotNil:[ :aSym |
    classVars ifNotNil:[ :cvs | | assoc |
      assoc := cvs associationAt: aSym otherwise: nil .
      assoc ifNotNil:[
         assoc value: nil .
         cvs removeKey: aSym  .
         ^ self
      ].
    ].
  ].
  ^ LookupError new reason: #classErrClassVarNotFound; key: aString ; object: self;
	signal
].
%

category: 'Updating Variables'
method: Class
removeSharedPool: aDictionary

"Remove aDictionary from the shared pool list for the receiver.  Generates an
 error if aDictionary is not a shared pool for the receiver.

 You may use this method only if, when the receiver was created, the argument
 to poolDictionaries: was an Array rather than a literal Array, which would
 create an InvariantArray.  (See Class >> subclass:.)"

self _validatePrivilege ifTrue:[
  poolDictionaries ifNotNil:[ | idx |
    (idx := poolDictionaries indexOfIdentical: aDictionary) ~~ 0 ifTrue:[
       poolDictionaries removeAtIndex: idx .
       ^ self
    ].
  ].
  ^ LookupError new reason: #classErrPoolDictNotFound;  key: aDictionary  ;
	object: self ; signal .
].
%

category: 'Deprecated'
method: Class
subclass: aString
inDictionary: aDictionary
constraints: constraintSpec

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^ self subclass: aString
    instVarNames: #()
    classVars:  #()
    classInstVars:  #()
    poolDictionaries: { }
    inDictionary: aDictionary
    constraints: constraintSpec
    instancesInvariant: false
    isModifiable: false
%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
classInstVars: anArrayOfClassInstVars
inDictionary: aDictionary
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^ self
    subclass: aString
    instVarNames: anArrayOfInstvarNames
    classVars:  #()
    classInstVars: anArrayOfClassInstVars
    poolDictionaries: { }
    inDictionary: aDictionary
    constraints:  #()
    instancesInvariant: false
    isModifiable: modifyBoolean
%

category: 'Subclass Creation'
method: Class
subclass: aString
instVarNames: anArrayOfStrings
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
category: aCategoryName
inDictionary: aDictionary
| cl |
cl := self subclass: aString
  instVarNames: anArrayOfStrings
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  options: #() .
cl category: aCategoryName asString.
^cl
%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfStrings
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
category: aCategoryName
inDictionary: aDictionary
constraints: aConstraint
instancesInvariant: invarBoolean
isModifiable: modifyBoolean

| cl |
self deprecated: 'The preferred methods are in the Subclass Creation category.'.
cl := self subclass: aString
    instVarNames: anArrayOfStrings
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  constraints: aConstraint
  instancesInvariant: invarBoolean
  isModifiable: modifyBoolean .
cl category: aCategoryName asString.
^cl
%

category: 'Subclass Creation'
method: Class
subclass: aString
instVarNames: anArrayOfStrings
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
category: aCategoryName
inDictionary: aDictionary
options: optionsArray
| cl |
cl := self subclass: aString
  instVarNames: anArrayOfStrings
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  options: optionsArray .
cl category: aCategoryName asString.
^cl
%

category: 'Subclass Creation'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary

"Executes
   oldClass := aDictionary at:(aString asSymbol) otherwise: nil
 and returns oldClass if it would be equivalent to the requested new class.
 (See the class comment for Class). "

^ self subclass: aString
    instVarNames: anArrayOfInstvarNames
    classVars: anArrayOfClassVars
    classInstVars: anArrayOfClassInstVars
    poolDictionaries: anArrayOfPoolDicts
    inDictionary: aDictionary
    newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
    description: nil
    options: #()
%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: aConstraint
instancesInvariant: invarBoolean
description: aDescription
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^self
  subclass: aString
  instVarNames: anArrayOfInstvarNames
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  constraints: aConstraint
  instancesInvariant: invarBoolean
  newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
  description: aDescription
  isModifiable: modifyBoolean
%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: ignored
instancesInvariant: invarBoolean
inClassHistory: aClassHistory
description: aDescription
isModifiable: modifyBoolean

| opts |
self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
opts := { } .
invarBoolean ifTrue:[ opts add: #instancesInvariant ]
           ifFalse:[ self instancesInvariant ifTrue:[ ^ self _error: #classErrInvariantSuperClass ]].
modifyBoolean ifTrue:[ opts add: #modifiable ].
^ self _subclass: aString
          instVarNames: anArrayOfInstvarNames
          format: format
          classVars: anArrayOfClassVars
          classInstVars: anArrayOfClassInstVars
          poolDictionaries: anArrayOfPoolDicts
          inDictionary: aDictionary
          inClassHistory: aClassHistory
          description: aDescription
          options: opts
%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: aConstraint
instancesInvariant: invarBoolean
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^self
  subclass: aString
  instVarNames: anArrayOfInstvarNames
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  constraints: aConstraint
  instancesInvariant: invarBoolean
  newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
  description: nil
  isModifiable: modifyBoolean
%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: ignored
instancesInvariant: invarBoolean
newVersionOf: oldClass
description: aDescription
isModifiable: modifyBoolean

	| opts hist descr |
	self
		deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category ('
				, aString , ').'.
	"Any specified constraints are ignored."
	opts := {}.
	invarBoolean
		ifTrue: [opts add: #instancesInvariant]
		ifFalse:
			[self instancesInvariant
				ifTrue: [^self _error: #classErrInvariantSuperClass]].
	modifyBoolean ifTrue: [opts add: #modifiable].
	descr := aDescription.
	oldClass
		ifNotNil:
			[(self
				_equivalentSubclass: oldClass
				superCls: self
				name: aString
				newOpts: opts
				newFormat: self format
				newInstVars: anArrayOfInstvarNames
				newClassInstVars: anArrayOfClassInstVars
				newPools: anArrayOfPoolDicts
				newClassVars: anArrayOfClassVars
				inDict: aDictionary
				isKernel: false)
					ifTrue:
						[oldClass _commentOrDescription: aDescription.
						^oldClass	"avoid creation of a new version"].
			hist := oldClass classHistory.
			descr
				ifNil:
					[descr := [oldClass commentForFileout] on: Error
								do: [:ex | 'old comment not available']]].
	^self
		_subclass: aString
		instVarNames: anArrayOfInstvarNames
		format: format
		classVars: anArrayOfClassVars
		classInstVars: anArrayOfClassInstVars
		poolDictionaries: anArrayOfPoolDicts
		inDictionary: aDictionary
		inClassHistory: hist
		description: descr
		options: opts
%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: aConstraint
instancesInvariant: invarBoolean
newVersionOf: oldClass
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^ self subclass: aString
    instVarNames: anArrayOfInstvarNames
    classVars: anArrayOfClassVars
    classInstVars: anArrayOfClassInstVars
    poolDictionaries: anArrayOfPoolDicts
    inDictionary: aDictionary
    constraints: aConstraint
    instancesInvariant: invarBoolean
    newVersionOf: oldClass
    description: nil
    isModifiable: modifyBoolean

%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: ignored
options: optionsArr

	| hist descr oldClass |
	self
		deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category ('
				, aString , ').'.
	"Any specified constraints are ignored."
	oldClass := self _classNamed: aString inDictionary: aDictionary.
	oldClass
		ifNotNil:
			[(self
				_equivalentSubclass: oldClass
				superCls: self
				name: aString
				newOpts: optionsArr
				newFormat: self format
				newInstVars: anArrayOfInstvarNames
				newClassInstVars: anArrayOfClassInstVars
				newPools: anArrayOfPoolDicts
				newClassVars: anArrayOfClassVars
				inDict: aDictionary
				isKernel: false) ifTrue: [^oldClass	"avoid creation of a new version"].
			hist := oldClass classHistory.
			descr := [oldClass commentForFileout] on: Error
						do: [:ex | 'old comment not available']].
	^self
		_subclass: aString
		instVarNames: anArrayOfInstvarNames
		format: format
		classVars: anArrayOfClassVars
		classInstVars: anArrayOfClassInstVars
		poolDictionaries: anArrayOfPoolDicts
		inDictionary: aDictionary
		inClassHistory: hist
		description: descr
		options: optionsArr
%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
instancesInvariant: invarBoolean
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^self
  subclass: aString
  instVarNames: anArrayOfInstvarNames
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  instancesInvariant: invarBoolean
  newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
  isModifiable: modifyBoolean
%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
instancesInvariant: invarBoolean
newVersionOf: oldClass
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^ self subclass: aString
    instVarNames: anArrayOfInstvarNames
    classVars: anArrayOfClassVars
    classInstVars: anArrayOfClassInstVars
    poolDictionaries: anArrayOfPoolDicts
    inDictionary: aDictionary
    constraints: #()
    instancesInvariant: invarBoolean
    newVersionOf: oldClass
    description: (oldClass ifNotNil:[ [ oldClass commentForFileout ] on: Error do:[:ex | 'old comment not available']])
    isModifiable: modifyBoolean
%

category: 'Subclass Creation'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
options: optionsArray

"Creates and returns a new subclass of the receiver.

 optionsArray is an Array of Symbols containing zero or more of
   #noInheritOptions,  #subclassesDisallowed, #disallowGciStore, #modifiable ,
   #traverseByCallback #selfCanBeSpecial #logCreation
 and at most one of
   #dbTransient, #instancesNonPersistent, #instancesInvariant
 If present, #noInheritOptions must be the first element and it causes
 none of subclassesDisallowed, disallowGciStore, traverseByCallback,
         dbTransient, instancesNonPersistent, instancesInvariant
 to be inherited from the superclass, nor copied from the
 current version of the class.
 #selfCanBeSpecial is never inherited and is needed only when modifying
 superclass hierarchy above classes with special format.
 The option #logCreation, if present in the options array, causes logging
 with GsFile(C)>>gciLogSever:  of class creation / equivalence.

 Executes
   oldClass := aDictionary at:(aString asSymbol) otherwise: nil
 and returns oldClass if it would be equivalent to the requested new class.
 (See the class comment for Class). "


^ self subclass: aString
    instVarNames: anArrayOfInstvarNames
    classVars: anArrayOfClassVars
    classInstVars: anArrayOfClassInstVars
    poolDictionaries: anArrayOfPoolDicts
    inDictionary: aDictionary
    newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
    description: nil
    options: optionsArray
%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: aConstraint
instancesInvariant: invarBoolean
inClassHistory: aClassHistory
description: aDescription
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^self subclass: aString
  instVarNames: anArrayOfInstvarNames
  classVars: anArrayOfClassVars
  classInstVars: #()
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  constraints: aConstraint
  instancesInvariant: invarBoolean
  inClassHistory: aClassHistory
  description: aDescription
  isModifiable: modifyBoolean
%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: aConstraint
instancesInvariant: invarBoolean
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^self
   subclass: aString
   instVarNames: anArrayOfInstvarNames
   classVars: anArrayOfClassVars
   classInstVars: #()
   poolDictionaries: anArrayOfPoolDicts
   inDictionary: aDictionary
   constraints: aConstraint
   instancesInvariant: invarBoolean
   isModifiable: modifyBoolean
%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: aConstraint
isInvariant: invarBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^self
  subclass: aString
  instVarNames: anArrayOfInstvarNames
  classVars: anArrayOfClassVars
  classInstVars: #()
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  constraints: aConstraint
  instancesInvariant: invarBoolean
  isModifiable: false
%

category: 'Subclass Creation'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
inDictionary: aDictionary

"Creates and returns a new subclass of the receiver.

 This method is a shortcut for convenience only.  It might not be retained in
 future GemStone releases.  Use it interactively or pedagogically, but avoid
 it in production code.

 The new class has no class variables, no class instance variables,
 and no pool dictionaries.
 Instances of the new class are variant, but the new class itself is not
 modifiable.

 Executes
   oldClass := aDictionary at:(aString asSymbol) otherwise: nil
 and returns oldClass if it would be equivalent to the requested new class.
 (See the class comment for Class). "

^self
  subclass: aString
  instVarNames: anArrayOfInstvarNames
  classVars:  #()
  classInstVars:  #()
  poolDictionaries: { }
  inDictionary: aDictionary
  newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
  description: nil
  options: #()
%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
inDictionary: aDictionary
constraints: constraintSpec

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^self
  subclass: aString
  instVarNames: anArrayOfInstvarNames
  classVars:  #()
  poolDictionaries: { }
  inDictionary: aDictionary
  constraints: constraintSpec
  instancesInvariant: false
  isModifiable: false
%

category: 'Deprecated'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
inDictionary: aDictionary
isModifiable: modifyBoolean

self deprecated: 'Obsolete in GemStone/S 64.  The preferred methods are in the Subclass Creation category (' , aString , ').'.
^self
  subclass: aString
  instVarNames: anArrayOfInstvarNames
  classVars:  #()
  classInstVars:  #()
  poolDictionaries: { }
  inDictionary: aDictionary
  constraints:  #()
  instancesInvariant: false
  isModifiable: modifyBoolean
%

category: 'Subclass Creation'
method: Class
subclass: aString
instVarNames: anArrayOfInstvarNames
inDictionary: aDictionary
options: optionsArray

"Creates and returns a new subclass of the receiver.

 This method is a shortcut for convenience only.  It might not be retained in
 future GemStone releases.  Use it interactively or pedagogically, but avoid
 it in production code.

 The new class has no class variables, no class instance variables,
 and no pool dictionaries.
 Instances of the new class are variant, but the new class itself is not
 modifiable.

 optionsArray is an Array of Symbols containing zero or more of
   #noInheritOptions,  #subclassesDisallowed, #disallowGciStore, #modifiable ,
   #traverseByCallback #selfCanBeSpecial #logCreation
 and at most one of
   #dbTransient, #instancesNonPersistent, #instancesInvariant
 If present, #noInheritOptions must be the first element and it causes
 none of subclassesDisallowed, disallowGciStore, traverseByCallback,
         dbTransient, instancesNonPersistent, instancesInvariant
 to be inherited from the superclass, nor copied from the
 current version of the class.
 #selfCanBeSpecial is never inherited and is needed only when modifying
 superclass hierarchy above classes with special format.
 The option #logCreation, if present in the options array, causes logging
 with GsFile(C)>>gciLogSever:  of class creation / equivalence.

 Executes
   oldClass := aDictionary at:(aString asSymbol) otherwise: nil
 and returns oldClass if it would be equivalent to the requested new class.
 (See the class comment for Class). "

^self
  subclass: aString
  instVarNames: anArrayOfInstvarNames
  classVars:  #()
  classInstVars:  #()
  poolDictionaries: { }
  inDictionary: aDictionary
  newVersionOf: (self _classNamed: aString inDictionary: aDictionary)
  description: nil
  options: optionsArray
%

category: 'Queries'
method: Class
subclasses

	^ClassOrganizer new subclassesOf: self.
%

category: 'Accessing'
method: Class
thisClass
  ^ self
%

category: 'Accessing'
method: Class
timeStamp

"Returns the timestamp instance variable of this class, a DateTime showing when
 the class was created."

^ timeStamp
%

category: 'Updating'
method: Class
timeStamp: aDateTime

"Set the value of the timeStamp instance variable.
 For use only when creating a class, while the class is not yet invariant."

self _validatePrivilege ifTrue:[
  self isInvariant ifFalse:[
    aDateTime ifNotNil:[ (aDateTime _validateClass: DateTime) ifFalse:[ ^ self]].
    timeStamp := aDateTime
  ] ifTrue:[
    self validateIsVariant .
  ].
].
%

category: 'Accessing'
method: Class
userId

"Returns the userId instance variable of this class, the ID of the user who
 created this class."

^ userId
%

category: 'Updating'
method: Class
userId: aString

"Set the value of the userId instance variable.
 For use only when creating a class, while the class is not yet invariant."

(self _validatePrivilege) ifTrue:[
  self isInvariant ifFalse:[
    aString ifNotNil:[ (aString _validateClass: CharacterCollection ) ifFalse:[ ^ self]].
    userId := aString
  ] ifTrue:[
    self validateIsVariant .
  ].
].
%

category: 'Accessing'
method: Class
versionedName
| ofs str |
(classHistory atOrNil: classHistory size) == self ifTrue:[ ^ name ].
(ofs := classHistory indexOfIdentical: self) ~~ 0 ifTrue:[
  (str := String new) addAll: name ; add: $[ ; add: ofs asString; add:$] .
  ^ str
].
Error signal: 'oop ' , self asOop asString, ', not found in classHistory(oop ' ,
    classHistory asOop asString , $) .
%

category: 'Accessing'
method: Class
versionNumber
 "Returns a SmallInteger"
  | ofs |
  (ofs := classHistory indexOfIdentical: self) ~~ 0 ifTrue:[
     ^ ofs
  ].
  ^ 1
%

category: 'Private'
method: Class
_addByteSwizzle: aSymbol toFormat: fmt newClassName: subClsName signed: isSigned
 | swiz newFmt oldSwiz signedMask |
  "uses GC_BEHAV_byteSwizKind_shift"
  aSymbol == #'2byteWords' ifTrue:[ swiz := 1 ].
  aSymbol == #'4byteWords' ifTrue:[ swiz := 2 ].
  aSymbol == #'8byteWords' ifTrue:[ swiz := 3 ].
  aSymbol == #'signed2byteWords' ifTrue:[ swiz := 1 ].
  aSymbol == #'signed4byteWords' ifTrue:[ swiz := 2 ].
  aSymbol == #'signed8byteWords' ifTrue:[ swiz := 3 ].
  swiz ifNil:[ Error signal:'invalid argument ' , aSymbol].

  oldSwiz := (format bitShift: -31"GC_BEHAV_byteSwizKind_shift") bitAnd: 3 .
  oldSwiz > 0 ifTrue:[
    oldSwiz = swiz ifFalse:[
       subClsName _error: #classErrBadFormat
        with: ' option ', aSymbol printString,' incompatible with superclass'.
    ].
  ].
  newFmt := fmt bitOr: (swiz bitShift: 31"GC_BEHAV_byteSwizKind_shift" ) .
  signedMask := 1 bitShift: 33"GC_BEHAV_signedWords_shift" .
  isSigned ifTrue:[ newFmt := newFmt bitOr: signedMask ]
          ifFalse:[ newFmt := newFmt bitAnd: (signedMask bitInvert) ].
  ^ newFmt
%

category: 'Private'
method: Class
_adjustClassVars: anArray
  classVars ifNil:[
    1 to: anArray size do:[:j | self _addClassVar: (anArray at: j) value: nil ]
  ] ifNotNil:[
    | argSet existingSet toRemove toAdd assocs |
    existingSet := IdentitySet new .
    assocs := { } .
    argSet := IdentitySet withAll: anArray  .
    "GsDevKit and Seaside need to be able to override classVarNames for
     implementation of SharedPool ,
     so must use   self classVarNames    here."
    self classVarNames do: [:classVarName | | a |
      a := classVars associationAt: classVarName.
      assocs add: a . existingSet add: a key ].
    toRemove := existingSet - argSet  .
    classHistory size > 1 ifTrue:[  | cvd | "create a new dictionary"
      cvd := SymbolDictionary new objectSecurityPolicy: classVars objectSecurityPolicy ; yourself.
      assocs do:[:a | (toRemove includes: a key) ifFalse:[ cvd addAssociation: a ]].
      self _setClassVars: cvd old: classVars .
    ] ifFalse:[
      toRemove do:[ :aKey | self removeClassVarName: aKey ].
    ].
    toAdd := argSet - existingSet .
    toAdd do:[ :aKey | self _addClassVar: aKey value: nil ].
  ].
%

category: 'Private'
method: Class
_adjustOptions: opts formatArr: fmtArray
  | fmt newFmt modifiableBool fmtToStore |
  fmt := format .
  fmtToStore := fmt .
  newFmt := fmtArray at: 1 .
  modifiableBool := fmtArray at: 2 .
  ((fmt bitAnd: 16r20) ~~ 0 and:[ (newFmt bitAnd: 16r20) == 0 ]) ifTrue:[
     "clear subclassesDisallowed bit."
     fmtToStore := fmtToStore bitAnd: 16r20 bitInvert .
  ].
  ((fmt bitAnd: 16r800) == 0 and:[ (newFmt bitAnd: 16r800) ~~ 0 ]) ifTrue:[
    "set instancesNonPersistent bit"
    fmtToStore := fmtToStore bitOr: 16r800 .
  ].
  (fmt bitAnd: 16r400) ~~ (newFmt bitAnd: 16r400) ifTrue:[
     "change state of traverseByCallback"
     fmtToStore := (fmtToStore bitAnd:( 16r400 bitInvert)) bitOr:( newFmt bitAnd: 16r400) .
  ].
  (fmt bitAnd: 16r200) ~~ (newFmt bitAnd: 16r200) ifTrue:[
     "change state of disallowGciStore"
     fmtToStore := (fmtToStore bitAnd:( 16r200 bitInvert)) bitOr:( newFmt bitAnd: 16r200) .
  ].
  (fmt bitAnd: 16r2000) ~~ (newFmt bitAnd: 16r2000) ifTrue:[
     "change state of selfCanBeSpecial"
     fmtToStore := (fmtToStore bitAnd:( 16r2000 bitInvert)) bitOr:( newFmt bitAnd: 16r2000) .
  ].
  fmtToStore ~~ fmt ifTrue:[
		 self _unsafeAt: 2 "offset of format" put: fmtToStore .
		 (fmtArray at: 3) ifTrue:[ "logBool"
				self _logServer: 'modified format of class ' , self name,' to ', fmtToStore asString .
		 ].
  ].
  "clear modifiable"
  (self isModifiable and:[ modifiableBool == false ]) ifTrue:[
     self immediateInvariant .
  ].
%

category: 'Private'
method: Class
_beVariantWhile: aBlock

"Executes the given block while the receiver is in a variant (modifiable) form.
 The block should not contain a return or the receiver will be left in a
 variant state.

 Great caution should be exercised in using this method as it allows
 modification of classes in uncontrolled ways that can lead to corrupted class
 objects."

self _validatePrivilege ifTrue:[
  | was |
  was := self isInvariant.
  was ifTrue: [ self __makeVariant; _refreshClassCache: false  ].
  [
    [
      aBlock value
    ] onException: Error do:[:ex |
      was ifTrue: [
	super immediateInvariant; _refreshClassCache: false.
	was := false
      ].
      ex outer .
    ]
  ] ensure:[
    was ifTrue:[ super immediateInvariant; _refreshClassCache: false ].
  ]
].
%

category: 'Browser Methods'
method: Class
_bytesPerWord
  "Returns a SmallInteger, the number of bytes per word in instances of the receiver.
   Returns 0 for special format, 8 for oop or nsc format,
   and 1, 2, 4 or 8 for various byte format classes."

  self isBytes ifTrue:[ | swiz |
    swiz := (format bitShift: -31"GC_BEHAV_byteSwizKind_shift") bitAnd: 3"GC_BEHAV_byteSwizKind_mask".
    ^ 1 bitShift: swiz .
  ].
  self _isSpecial ifTrue:[ ^ 0 ].
  ^ 8
%

category: 'Private'
method: Class
_category: newCategory
"Sets the classCategory variable of the receiver.
 The argument should be a kind of CharacterCollection or nil."
"Implemenation shared by base image and Rowan"

newCategory ifNil:[
	classCategory := nil.
	^ self ].

(newCategory _validateClass: CharacterCollection ) ifFalse:[ ^ nil ].

classCategory := newCategory asString
%

category: 'Category'
method: Class
_classCategory

"Returns the classCategory of the receiver."

^ classCategory
%

category: 'Subclass Creation'
method: Class
_classDefiningClassVar: aSymbol

"Returns the receiver or the superclass that defines a class variable with
 name aSymbol, otherwise returns nil."

| aClass |
aClass := self .
[
  aClass _classVars ifNotNil:[ :cvs |
    ( cvs associationAt: aSymbol otherwise: nil ) ifNotNil:[
      ^ aClass
    ].
  ].
  aClass := aClass superClass .
  aClass == nil
] untilTrue .
^ nil
%

category: 'Class Membership'
method: Class
_classHistoryIncludesIdentical: aClass

 ^ (classHistory indexOfIdentical: aClass) ~~ 0
%

category: 'Accessing'
method: Class
_classInstVars

"Returns an Array of the receiver's class instance variables."

| civs |

civs := self class allInstVarNames.
civs removeFrom: 1 to: (self class superClass instSize).
^civs
%

category: 'Private'
method: Class
_classNamed: aString inDictionary: aDictionary
  | v sym |
  aDictionary ifNil:[ ^ nil ].
  (sym := Symbol _existingWithAll: aString) ifNil:[ ^ nil ].
  v := aDictionary at: sym otherwise: nil .
  ^ (v isKindOf: Class) ifTrue:[ v ] ifFalse:[ nil ].
%

category: 'Private'
method: Class
_classVarsChangableTo: anArray
  ^ self _classVarsChangableTo: anArray log: false
%

category: 'Private'
method: Class
_classVarsChangableTo: anArray log: logBool
  | ok |
  ok := true .
  anArray ifNotNil:[ | argSet |
    anArray _isArray ifFalse:[
      (anArray _validateClass: Array) ifFalse:[ ^ nil ].
    ].
    argSet := IdentitySet new .
    1 to: anArray size do: [:index| | aVarName aSym definingClass |
      aVarName := anArray at: index .
      (aVarName _isOneByteString or:[ aVarName _validateClass: CharacterCollection]) ifTrue:[
	aSym := aVarName asSymbol .
	aSym validateIsIdentifier "fix bug 9666" .
	definingClass := self _classDefiningClassVar: aSym .  "fix bug 10480"
	(definingClass ~~ nil and:[ definingClass ~~ self]) ifTrue:[  
           ok := false .
           logBool ifTrue:[ self _logServer: aSym , ' defined in ', definingClass name ].
        ].
        argSet add: aSym .  "exists or ok to add"
        anArray at: index put: aSym .
      ].
    ].
    classVars ifNotNil:[ :cvs | | toRemove |
      toRemove := cvs keys - argSet  .
      toRemove do:[:aKey |
        (classVars associationAt: aKey) isInvariant ifTrue:[ 
          ok := false .
          logBool ifTrue:[ self _logServer: aKey , ' an invariant class variable to remove '].
        ]
      ].
    ].
  ].
  ^ true
%

category: 'Private'
method: Class
_commentOrDescription: newComment
	"For backward compatibility, accept a GsClassDocumentation (although this is deprecated)
	or a string. Copy any GsClassDocumentation to avoid bug 41763."
newComment
	ifNotNil:
		[(newComment isKindOf: GsClassDocumentation)
			ifTrue:
				[| newDesc |
				newDesc := newComment copy.
				newDesc itsClass: self.
				self _description: newDesc]
			ifFalse: [self comment: newComment]].
%

category: 'Private'
method: Class
_definition
	"Returns a String. Sent by topaz "

	^self _definitionInContext: System myUserProfile
%

category: 'Browser Methods'
method: Class
_definitionInContext: aUserProfile

^ self _definitionInContext: aUserProfile withConstraints: false
%

category: 'Browser Methods'
method: Class
_definitionInContext: aUserProfile withConstraints: withConstraintsBool

"Returns a description of the receiver using object names taken from the given
 UserProfile."

| result newByteSubclass anArray lfsp
  firstElement poolDicts civs supercls aSize iVs |
supercls := self superClass .
result := String new.
result addAll: (supercls == nil ifTrue: ['nil'] ifFalse: [supercls name]).
newByteSubclass := false.
(lfsp := Character lf asString) addAll: '  ' .

(self isBytes and: [supercls isBytes not]) ifTrue: [
  result addAll: ' byteSubclass: '''; addAll: name; addLast: $'.
  newByteSubclass := true.
] ifFalse: [
  (self isIndexable and:[ supercls == nil or:[ supercls isIndexable not]]) ifTrue: [
    result addAll: ' indexableSubclass: '''; addAll: name; addLast: $'.
  ] ifFalse: [
    result addAll: ' subclass: '''; addAll: name; addLast: $'.
  ]
].
" instVarNames: #( <list of strings> ) "
iVs := self instVarNames .
(newByteSubclass not or:[ iVs size > 0 ]) ifTrue: [
  result addAll: lfsp;
    addAll: 'instVarNames: #(';
    addAll: (self _instVarNamesWithSeparator: (lfsp , '                 '));
    add: $).
].
" classVars: #( <list of strings> ) "
result addAll: lfsp; addLast: 'classVars: #('.
self _sortedClassVarNames accompaniedBy: result do: [:res :aKey |
  res addLast: $  .
  (aKey includesValue: $')
    ifTrue:[ res addAll: aKey _asSource ]
    ifFalse:[ res addAll: aKey ].
  ].
result addLast: $).

" classInstVars: #( <list of strings> ) "
result addAll: lfsp; addLast: 'classInstVars: #('.
civs := self class allInstVarNames.
civs removeFrom: 1 to: (self class superClass instSize).
civs accompaniedBy: result do: [:res :civName |
  res addLast: $  .
  (civName includesValue: $')
    ifTrue:[ res addAll: civName _asSource ]
    ifFalse:[ res addAll: civName ].
].
result addLast: $).

" poolDictionaries: { <list of dictionary names> } "
result addAll: lfsp; addAll: 'poolDictionaries: ' .
(poolDicts := self sharedPools) size > 0 ifTrue:[
  result addAll: '{ ' .
  firstElement := true.
  poolDicts do: [:each |
    firstElement ifFalse: [ result add: ' . '].
    anArray := aUserProfile dictionariesAndSymbolsOf: each.
    (aSize := anArray size) == 0 ifTrue:[ result add: '"(not named)"' ]
      ifFalse:[ aSize = 1 ifTrue:[ result add:((anArray at:1) at: 2) ]
                      ifFalse:[ result add: '"(multiple names)"' ]].
    firstElement := false.
  ].
  result add: ' }' .
] ifFalse:[
  result add: '#()'
].

" inDictionary: <name of containing dictionary> "
result addAll: lfsp; addAll: 'inDictionary: ' ;
   addAll: (self _dictionaryNameForFileout: aUserProfile) .

withConstraintsBool ifTrue:[
 (newByteSubclass not and: [self _hasConstraints]) ifTrue: [
    result add: self _definitionOfConstraints; lf
  ].
].

"options:"
result add:  lfsp; add: self _optionsStringForDefinition .
result add: Character lf .
^result
%

category: 'Browser Methods'
method: Class
_definitionOfConstraints

" Returns a string of the form
     constraints: { <Array of instance-variable-symbol/class-name pairs> }
 or
     constraints: <class name>

 As of GemStone 64bit v3.4, constraints are no longer implemented.
 This method is provided for examining classes in repositories upgraded from
 an older version."

| result  aConstraint firstElement |
result := String new.
result lf; add: '  constraints: '.
( constraints isKindOf: Array ) ifTrue: [
    result addAll: '{ '.
    firstElement := true.
    1 to: self instSize do: [ :x |
      aConstraint := constraints atOrNil: x .
      ((aConstraint ~~ nil _and: [aConstraint ~~ Object])
          _and:[ superClass == nil
            _or:[ (superClass _namedIvConstraintAt: x) ~~ aConstraint ]] )
      ifTrue: [
        " if not the first constraint, prefix with a period to separate
          from the last constraint "
        firstElement ifFalse: [
          result add: ' . '; lf; add: '                '
        ]
        ifTrue: [
          firstElement := false
        ].
        result add: '{ #'; add: (instVarNames at: x) ;
              add: ' . '; add: aConstraint name; addLast: $} .
      ]
    ].
    aConstraint:= self _varyingConstraint .
    ( (aConstraint ~~ Object) _and:
        [(superClass _varyingConstraint) ~~ aConstraint] )
    ifTrue:[
      firstElement ifFalse: [
          result add: ' . '; lf; add: '                '
      ]
      ifTrue: [
        firstElement := false
      ].
      result add: '   "the elements"  '; add: aConstraint name
    ].
    result add: ' }'.
  ]
  ifFalse: [
    constraints class class == Metaclass3 ifTrue: [
      result add: constraints name.
    ]
    ifFalse: [
      result add: ' nil'
    ].
  ].

^result
%

category: 'Private'
method: Class
_description
	"Returns the description of this class.
	Deprecated as of GS/64 3.1: subsumed by the new #comment field."

	self deprecated: 'Class>>_description deprecated v3.1. Replaced by comment'.
	^self _extraDictAt: #description
%

category: 'Private'
method: Class
_description: aDescription
	"Update the description of this Class.  Returns the argument.
 As of GS/64 3.1, this is deprecated in favor of #comment:"

	self deprecated: 'Deprecated as of GS/64 3.1'.
	self _extraDictAt: #description put: aDescription.
	^aDescription
%

category: 'Browser Methods'
method: Class
_dictionaryNameForFileout: aUserProfile
  | anArray |
  anArray := aUserProfile dictionariesAndSymbolsOf: self.
  (anArray size) == 0 ifTrue: [ | hist j |
    hist := self classHistory .
    anArray := nil .
    j := hist size .
    [ j >= 1 ] whileTrue:[
      anArray := aUserProfile dictionariesAndSymbolsOf: (hist at: j) .
      anArray size ~~ 0 ifTrue:[ j := 0"exit loop" ] .
      j := j - 1 .
    ]
  ].
  (anArray size) ~~ 0 ifTrue:[
    anArray := aUserProfile dictionariesAndSymbolsOf: ((anArray at: 1) at: 1).
    anArray size == 0 ifTrue:[
      ^ '(dictionary not in your dictionaries)'
    ] ifFalse: [ | dName |
      (dName := (anArray at: 1) at: 2) isValidIdentifier ifTrue: [
        ^ dName
      ] ifFalse: [  "this code moved from sessionmethods.topaz to here for v3.0"
        ^ ( '(GsCurrentSession currentSession symbolList objectNamed: ' , dName printString )
	       add: $) ; yourself .
      ]
    ]
  ].
  ^ 'UserGlobals' .
%

category: 'Private'
method: Class
_equivalentSubclass: oldClass
	superCls: actualSelf
	name: aString
	newOpts: optionsArray
	newFormat: theFormat
	newInstVars: anArrayOfInstvarNames
	newClassInstVars: anArrayOfClassInstVars
	newPools: anArrayOfPoolDicts
	newClassVars: anArrayOfClassVars
	inDict: aDictionary
	isKernel: isKernelBool

 "oldClass is equivalent to the subclass that would be created using
  the other arguments if:
     instVar names match exactly ,
     and class instVar names match exactly ,
     and the classVars in oldClass can be modified to add/remove Associations
        to match anArrayOfClassVars ,
     and pool dictionaries match exactly

  With respect to options and format, oldClass is equivalent if
    The state of format bits dbTransient, instancesInvariant match exactly ,
    and subclassesDisallowed cannot be set in the new subclass if it not set in oldClass ,
    and modifiable  cannot be set if it is not set in oldClass  ,
    and (SELF_CAN_BE_SPECIAL, NSC_DUPLICATES, INDEXABLE, IMPLEMENTATION, NO_STRUCT_UPDATE bits)
        of the formats must match exactly.

  If all other equivalence tests pass, the following changes to oldClass may be
  made to match the arguments and avoid creating a new subclass:
    instancesNonPersistent bit may be set (but not cleared) in the format of the oldClass
    subclassesDisallowed bit may be cleared (but not set) in format of oldClass
    traverseByCallback bit may be set or cleared in format of oldClass
    oldClass may be changed from modifiable to not modifiable (by sending immediateInvariant)
    classVars may be added to oldClass
    classVars having modifiable Associations may be removed from oldClass
"

	| oldOk fmtArr nam supr opts ivs civs poolds cvars cvarsArray logBool |
	fmtArr := self _validateOptions: optionsArray withFormat: theFormat newClassName: aString .
  logBool := fmtArr at: 3 .
	(oldClass isKindOf: Class) ifFalse: [oldClass _validateClass: Class].
	nam := oldClass name asString = aString asString.
	supr := oldClass superClass == actualSelf.
	opts := oldClass _optionsChangableTo: fmtArr.
	ivs := oldClass _instVarsEqual: anArrayOfInstvarNames.
	civs := oldClass class _instVarsEqual: anArrayOfClassInstVars.
	poolds := oldClass _poolDictsEqual: anArrayOfPoolDicts.
	cvars := oldClass
				_classVarsChangableTo: (cvarsArray := anArrayOfClassVars copy) log: logBool .
	oldOk := nam
				and: [supr and: [(opts==true) and: [ivs and: [civs and: [poolds and: [cvars]]]]]].
	oldOk ifTrue: [| oldVal newName |
			aDictionary ifNotNil: [
				newName := aString asSymbol.
				oldVal := aDictionary at: newName otherwise: nil.
        "allow nil as a value in aDictionary to satisfy forward refs during loading"
        (oldVal ~~ nil and:[ oldVal ~~ oldClass ]) ifTrue:[
             ImproperOperation
							signal: 'no new subclass needed, but aDictionary at: oldClass name ~~ oldClass'.
						^false
         ]
      ].
			oldClass _adjustOptions: optionsArray formatArr: fmtArr.
			oldClass _adjustClassVars: cvarsArray.
			logBool ifTrue:[self _logServer: 'class ' , aString , ' equivalent to requested class'].
			^true
    ] ifFalse:[ 
       logBool ifTrue:[
         self _logServer: '_equivalentSubclass false, nam:' , nam asString ,
                ' supr:' , supr asString , ' opts:' , opts asString ,
                ' ivs:' , ivs asString , ' civs:' , civs asString ,
                ' poolds:' , poolds asString , ' cvars:' , cvars asString]
    ].
	^false
%

category: 'Private'
method: Class
_extraDictAt: key

  "Return value for key in extraDict.
   Return nil if extraDict or the key are not present. "

  ^ extraDict
      ifNil: [ nil ]
      ifNotNil: [ :ed | ed at: key otherwise: nil ]
%

category: 'Private'
method: Class
_extraDictAt: key put: value

  "Add value for key to extraDict.  Create extraDict if not present. "

  extraDict ifNil: [
    extraDict := SymbolDictionary new.
    extraDict objectSecurityPolicy: self objectSecurityPolicy ].

  ^ extraDict at: key put: value
%

category: 'Private'
method: Class
_extraDictRemoveKey: key

  " Remove key/value from extraDict.
    Dont care if extraDict or the key itself are not present. "

  extraDict ifNotNil:[ :ed | ed removeKey: key ifAbsent: [] ].
%

category: 'Filein Support'
method: Class
_finishNewReservedClass: newClass old: oldCls resolv: resolveRes
  oldCls == nil ifTrue:[
    (((newClass class) superClass) == (self class)) ifFalse:[
       self _halt: 'Inconsistent class hierarchy'
    ].
     ^ 'created class: '  , newClass definition
  ] ifFalse:[
     ^ 'class already exists: ' , (resolveRes at: 2)
	    , ', ' , oldCls definition
  ].
%

category: 'Private'
method: Class
_gbsTraversalCallback

"Private.  When GemBuilder Smalltalk traverses a Class, this method
 is called to return a description of the class."

^self definition
%

category: 'Browser Methods'
method: Class
_hasConstraints

"Determine if there are any constraints on any instance variable; return true if any
exist, false otherwise.
As of GemStone 64bit v3.4, constraints are no longer implemented.
This method is provided for examining classes in repositories upgraded from
an older version."

| aConstraint constrs |
constrs := constraints .
 constrs _isArray  ifTrue: [
    1 to: self instSize do: [:x |
      aConstraint := constrs atOrNil: x .
      ((aConstraint ~~ nil _and: [aConstraint ~~ Object])
          _and: [ superClass == nil
          _or: [ (superClass _namedIvConstraintAt: x) ~~ aConstraint ]] )
                   ifTrue: [^true]
        ].
    aConstraint := self _varyingConstraint .
    ( (aConstraint ~~ Object) _and:
        [(superClass _varyingConstraint) ~~ aConstraint] )
    ifTrue: [^true].
  ] ifFalse: [
    constraints class class == Metaclass3 ifTrue: [^true]
  ].

^false
%

category: 'Private'
method: Class
_insertCivAt: offset

"insert space for a new class instance variable at the specified offset.
 Each call will cause a markSweep GC of the VM local object memory."

<primitive: 486>

self _primitiveFailed: #_insertCivAt: args: { offset }.
self _uncontinuableError
%

category: 'Repository Conversion'
method: Class
_makeClassObsolete: aSymbolDictionary
"remove association from <aSymbolDictionary>
 rename receiver
 if assoc is invariant:
   create new association for obsolete class
 if not:
   reuse the original association for obsolete class
"

 | newName assoc |
 assoc := aSymbolDictionary associationAt: self name.
 newName := ('Obsolete', self name asString) asSymbol.
 aSymbolDictionary removeAssociation: assoc.
 self _unsafeAt: 11 put: newName.
 assoc isInvariant
   ifTrue: [ aSymbolDictionary at: self name asSymbol put: self ]
   ifFalse: [
      assoc key: self name asSymbol.
      aSymbolDictionary addAssociation: assoc ].
 self removeAllMethods.
 self class removeAllMethods.
%

category: 'Modifying Classes'
method: Class
_makeInstancesDbTransient: aBool

<protected>
self _validatePrivilege ifTrue:[
  aBool ifTrue:[
    (self isPointers and:[ self isIndexable not]) ifFalse:[
      ^ ImproperOperation new details:'Only non-indexable pointer objects may be DbTransient';
           object: self ; signal
    ].
    format := format bitOr: 16r1000 .
  ] ifFalse:[
    format := format bitAnd: (16r1000 bitInvert)
  ].
  self _refreshClassCache: false .
].
%

category: 'Private'
method: Class
_makeInstancesNonPersistent

<protected>
self _validatePrivilege ifTrue:[
  format := format bitOr: 16r800 .
  self _refreshClassCache: false .
].
%

category: 'Private'
method: Class
_makeInstancesPersistent

<protected>
self _validatePrivilege ifTrue:[
  self _validateInstancesPersistent ifTrue:[
    self superClass instancesNonPersistent ifTrue:[
      ^ ImproperOperation new reason: #rtErrSuperclassIsNP;
	details: 'superclass is non-persistent';
	object: self ; signal .
    ]
  ]
].
format := (format bitOr: 16r800 ) bitXor: 16r800 .
self _refreshClassCache: false .
%

category: 'Browser Methods'
method: Class
_modifiableDefinitionInDictionary: dict named: dictName

"Returns a description of the receiver that is modifiable,
 and that places the class in the given dictionary name.
 Byte classes will not use the modifiable form of class definition as
 they have no information that can be modified."

  | result newByteSubclass lfsp firstElement poolDicts cat nm resolver supercls |

  supercls := self superClass .
  nm := supercls == nil ifTrue: [ 'nil' ] ifFalse: [
      dict keyAtValue: supercls ifAbsent: [ supercls name ]
      ].
  (result := String new) add: $( ; addAll: nm .
  newByteSubclass := false.
  lfsp := (Character lf asString) addAll: '  '.

  (self isBytes and: [supercls isBytes not]) ifTrue:
    [
    result addAll: ' byteSubclass: '.
    newByteSubclass := true.
    ]
  ifFalse:
    [
    (self isIndexable and: [supercls isIndexable not]) ifTrue:
      [ result addAll: ' indexableSubclass: ' ]
    ifFalse:
      [ result addAll: ' subclass: ' ].
    ].

  nm := dict keyAtValue: self ifAbsent: [ self name ].
  result addAll: (String withAll: nm) quoted.

 " instVarNames: #( <list of strings> ) "
  newByteSubclass ifFalse: [
    result addAll: lfsp;
      addAll: 'instVarNames: #(';
      addAll: (self _instVarNamesWithSeparator: (lfsp , '                 '));
      add: $).
  ].

  " classVars: #( <list of strings> ) "

  result addAll: lfsp; addLast: 'classVars: #('.
  self _sortedClassVarNames accompaniedBy: result do: [:res :aKey |
    res addLast: $  .
    (aKey includesValue: $')
      ifTrue:[ res addAll: aKey _asSource ]
      ifFalse:[ res addAll: aKey ].
    ].

  result addLast: $).

  " classInstVars: #( <list of strings> ) "

  result addAll: lfsp; addLast: 'classInstVars: #('.
  self _classInstVars accompaniedBy: result do:[ :res :civName |
    res addLast: $  .
    (civName includesValue: $')
      ifTrue:[ res addAll: civName _asSource ]
      ifFalse:[ res addAll: civName ].
    ].
  result addLast: $).

  " poolDictionaries: { <list of dictionary names> } "

  result addAll: lfsp; addAll: 'poolDictionaries: '.
  (poolDicts := self sharedPools ) size > 0 ifTrue:[
    firstElement := true.
    resolver := System myUserProfile .
    result add: ' {' .
    poolDicts do: [:each | | anArray aSize |
      firstElement ifFalse: [ result add: ' . '].
      anArray := resolver dictionariesAndSymbolsOf: each.
      (aSize := anArray size) == 0 ifTrue:[ result add: '"(not named)"' ]
        ifFalse:[ aSize = 1 ifTrue:[ result add:((anArray at:1) at: 2) ]
                      ifFalse:[ result add: '"(multiple names)"' ]].
      firstElement := false.
    ].
    result add: ' }' .
  ] ifFalse:[
    result add: '#()'
  ].

  " inDictionary: <name of containing dictionary> "
  result addAll: lfsp; addAll: 'inDictionary: '; addAll: dictName.

  "constraints omitted"

  "options:"
  result add:  lfsp; add: self _optionsStringForDefinition .

  result add: $) .
  (cat := classCategory ) ifNotNil:[
    result addAll: ' category: '; addAll: (String withAll: cat) quoted
  ].
  result addLast: Character lf .
  ^result
%

category: 'Filein Support'
method: Class
_newKernelByteSubclass: clsName
classVars: classVarArg
poolDictionaries: poolDicts
inDictionary: aDict
options: options
reservedOop: reservedOopNum

	"Old (pre-GS64) behavior:
   Created a new class always, to allow upgrade
   scripts to change the definition of a kernel class.  Preserved
   the identity of both the class and the metaClass if the class
   already existed.

 New (Gemstone64) behavior:
   Does not allow redefinition of a
   class in the upgrade case.  Only create class if it does not exist
   by the given name in globals.  The _unsafeSet11Oop primitive does
   not currently allow changing the identity of an already existing
   object.

 Returns the new or existing class .

 If reservedOopNum is ~~ nil, reservedOopNum must be a positive Integer
 that is a legal gs64v1.1 objectId and the methods   Object >> _unsafeSetOop:
 and   Object(C) >> _objectForOop:   must be installed."

  self ifNil:[ Error signal:'superclass not yet defined'].
	self _validatePrivilege ifTrue:
			[| newClass className oldCls resolveRes dictNames result |
      dictNames := #( ObsoleteClasses GsCompilerClasses GemStone_Legacy_Streams).
			(aDict == Globals or:[ dictNames includesIdentical: aDict name])
				ifFalse:[ aDict _error: #rtErrInvalidArgument.  ^nil].
			className := Symbol withAll: clsName.
			resolveRes := Class _resolveReservedClass: reservedOopNum name: className.
			oldCls := resolveRes at: 1.
			oldCls ifNil:[
        GsObjectSecurityPolicy setCurrent: Object objectSecurityPolicy while:[
           newClass := self
										byteSubclass: className
										classVars: classVarArg
										classInstVars: #()
										poolDictionaries: poolDicts
										inDictionary: aDict
										newVersionOf: nil
										description: nil
										options: options.
							reservedOopNum ifNotNil:[
								newClass := newClass _unsafeSetOop: reservedOopNum .
							  (aDict associationAt: className) immediateInvariant ].
         ].
         result := newClass .
       ] ifNotNil:[ | fmt |
					fmt := (format bitAnd: 16r3 bitInvert) bitOr: 16r1 + 16r4.
					(self
						_equivalentSubclass: oldCls
						superCls: self
						name: className
						newOpts: ((options copy)
								add: #logCreation;
								yourself)
						newFormat: fmt
						newInstVars: #()
						newClassInstVars: #()
						newPools: poolDicts
						newClassVars: classVarArg
						inDict: aDict
						isKernel: true)
							ifFalse: [ArgumentError signal: 'existing class not equivalent to arguments'].
          result := oldCls .
      ].
			self _finishNewReservedClass: newClass old: oldCls resolv: resolveRes .
      ^ result
  ].
	^ nil
%

category: 'Filein Support'
method: Class
_newKernelIndexableSubclass: clsName
instVarNames: ivArg
classVars: classVarArg
classInstVars: classInstVarArg
poolDictionaries: poolDicts
inDictionary: aDict
options: options
reservedOop: reservedOopNum

	"Used by filein.
Old (pre-GS64) behavior:
   Created a new class always, to allow upgrade
   scripts to change the definition of a kernel class.  Perserved
   the identity of both the class and the metaClass if the class
   already existed.
 New (Gemstone64) behavior:
   Does not allow redefinition of a
   class in the upgrade case.  Only create class if it does not exist
   by the given name in globals.  The _unsafeSet11Oop primitive does
   not currently allow changing the identity of an already existing
   object.

 Returns the new class, the old class or nil

 If reservedOopNum is ~~ nil, reservedOopNum must be a positive Integer
 that is a legal gs64v1.1 objectId and the methods   Object >> _unsafeSet11Oop:
 and   Object(C) >> _objectForOop:   must be installed."

  self ifNil:[ Error signal:'superclass not yet defined'].
	self _validatePrivilege
		ifTrue:
			[| newClass className oldCls resolveRes fmt dictNames result |
      dictNames := #( ObsoleteClasses GsCompilerClasses GemStone_Legacy_Streams).
			(aDict == Globals or:[ dictNames includesIdentical: aDict name])
					ifFalse:
					[aDict _error: #rtErrInvalidArgument.
					^false].
			self isBytes
				ifTrue:
					[^clsName _error: #classErrBadFormat
						with: 'cannot create indexable subclass of byte class'].
			self isNsc
				ifTrue:
					[^clsName _error: #classErrBadFormat
						with: 'cannot create indexable subclass of Nsc class'].
			fmt := format bitOr: 16r4.	"add indexable bit"
			className := Symbol withAll: clsName.
			resolveRes := Class _resolveReservedClass: reservedOopNum name: className.
			oldCls := resolveRes at: 1.
			oldCls ifNil:[
          aDict at: className put: nil.
					GsObjectSecurityPolicy setCurrent: Object objectSecurityPolicy while:[
            newClass := self
										_subclass: className
										instVarNames: ivArg
										format: fmt
										classVars: classVarArg
										classInstVars: classInstVarArg
										poolDictionaries: poolDicts
										inDictionary: aDict
										inClassHistory: nil
										description: nil
										options: options.
							reservedOopNum ifNotNil:[
								newClass := newClass _unsafeSetOop: reservedOopNum .
							  (aDict associationAt: className) immediateInvariant ].
           ].
           result := newClass .
         ] ifNotNil: [(self
						_equivalentSubclass: oldCls
						superCls: self
						name: className
						newOpts: ((options copy)
								add: #logCreation;
								yourself)
						newFormat: fmt
						newInstVars: ivArg
						newClassInstVars: classInstVarArg
						newPools: poolDicts
						newClassVars: classVarArg
						inDict: aDict
						isKernel: true)
							ifFalse: [ArgumentError signal: 'existing class not equivalent to arguments'].
           result := oldCls
         ].
			self _finishNewReservedClass: newClass old: oldCls resolv: resolveRes .
      ^ result
      ].
	^false
%

category: 'Filein Support'
method: Class
_newKernelSubclass: clsName
instVarNames: ivArg
classVars: classVarArg
classInstVars: anArrayOfClassInstVars
poolDictionaries: poolDicts
inDictionary: aDict
options: options
reservedOop: reservedOopNum

^ self _newKernelSubclass: clsName
    subclassOf: self
    instVarNames: ivArg
    classVars: classVarArg
    classInstVars: anArrayOfClassInstVars
    poolDictionaries: poolDicts
    inDictionary: aDict
    options: options
    reservedOop: reservedOopNum
%

category: 'Filein Support'
method: Class
_newKernelSubclass: clsName
subclassOf: actualSelf
instVarNames: ivArg
classVars: classVarArg
classInstVars: anArrayOfClassInstVars
poolDictionaries: poolDicts
inDictionary: aDict
options: options
reservedOop: reservedOopNum

	"Old (pre-GS64) behavior:
   Created a new class always, to allow upgrade
   scripts to change the definition of a kernel class.  Preserved
   the identity of both the class and the metaClass if the class
   already existed.

 New (Gemstone64) behavior:
   Does not allow redefinition of a
   class in the upgrade case.  Only create class if it does not exist
   by the given name in globals.  The _unsafeSet11Oop primitive does
   not currently allow changing the identity of an already existing
   object.

 Returns a String, either the definition of the new class or a
 message that the class already exists.

 If reservedOopNum is ~~ nil, reservedOopNum must be a positive Integer
 that is a legal gs64v1.1 objectId and the methods   Object >> _unsafeSet11Oop:
 and   Object(C) >> _objectForOop:   must be installed."
  self ifNil:[ Error signal:'superclass not yet defined'].
	self _validatePrivilege ifTrue:[
     | newClass className oldCls resolveRes dictNames result |
      dictNames := #( ObsoleteClasses GsCompilerClasses GemStone_Legacy_Streams).
			(aDict == Globals or:[ dictNames includesIdentical: aDict name])
				ifFalse:
					[aDict _error: #rtErrInvalidArgument.
					^false].
			className := Symbol withAll: clsName.
			resolveRes := Class _resolveReservedClass: reservedOopNum name: className.
			oldCls := resolveRes at: 1.
			oldCls ifNil:[
          aDict at: className put: nil.
					GsObjectSecurityPolicy setCurrent: Object objectSecurityPolicy while:[
            newClass := self
										_subclass: className
										instVarNames: ivArg
										format: format
										classVars: classVarArg
										classInstVars: anArrayOfClassInstVars
										poolDictionaries: poolDicts
										inDictionary: aDict
										inClassHistory: nil
										description: nil
										options: options.
							reservedOopNum ifNotNil:[ "change object identifier"
								newClass := newClass _unsafeSetOop: reservedOopNum .
							  (aDict associationAt: className) immediateInvariant 
              ].
             ].
             result := newClass .
       ] ifNotNil: [
          (self
						_equivalentSubclass: oldCls
						superCls: actualSelf
						name: className
						newOpts: ((options copy)
								add: #logCreation;
								yourself)
						newFormat: oldCls format
						newInstVars: ivArg
						newClassInstVars: anArrayOfClassInstVars
						newPools: poolDicts
						newClassVars: classVarArg
						inDict: aDict
						isKernel: true)
							ifFalse: [ArgumentError signal: 'existing class not equivalent to arguments'].
         result := oldCls .
       ].
			self _finishNewReservedClass: newClass old: oldCls resolv: resolveRes .
      ^ result
   ].
	^ nil
%

category: 'Browser Methods'
method: Class
_optionsArray
  | result optCount swiz |
  result := { } .
  optCount := 0 .
  self instancesDbTransient ifTrue:[ result add: #dbTransient . optCount := optCount + 1 ].
  self _structuralUpdatesDisallowed ifTrue:[ result add: #disallowGciStore  ].
  self instancesInvariant ifTrue:[ result add:  #instancesInvariant  . optCount := optCount + 1 ].
  self instancesNonPersistent ifTrue:[ result add:  #instancesNonPersistent  . optCount := optCount + 1 ].
  optCount > 1 ifTrue:[
    self _error: #classErrBadFormat
        with:'only one of #dbTransient #instancesNonPersistent  #instancesInvariant allowed' .
  ].
  self isModifiable ifTrue:[ result add: #modifiable  ].
  self selfCanBeSpecial ifTrue:[ result add: #selfCanBeSpecial  ].
  self subclassesDisallowed ifTrue:[ result add: #subclassesDisallowed  ].
  self _traversalByCallback ifTrue:[ result add: #traverseByCallback  ].
  swiz := (format bitShift: -31"GC_BEHAV_byteSwizKind_shift") bitAnd: 3"GC_BEHAV_byteSwizKind_mask" .
  swiz > 0 ifTrue:[ | signed |
    signed := (format bitShift: -33"GC_BEHAV_signedWords_shift") bitAnd: 1 .
    signed == 0 ifTrue:[ result add: ( #( #'2byteWords' #'4byteWords' #'8byteWords' ) at: swiz)]
             ifFalse:[ result add: ( #( #'signed2byteWords' #'signed4byteWords' #'signed8byteWords' ) at: swiz) ]
  ].
  "See also  Class >> _rwOptionsArray  in Rowan ."
  ^ result
%

category: 'Browser Methods'
method: Class
_optionsArrayForDefinition
  "Returns an Array like that from Class>>_optionsArray for use in
   recreating the receiver or exporting a definition to Rowan files."
  | inheritable myOpts supOpts myInheritable supInheritable notInher result |
  inheritable := IdentitySet withAll: #( subclassesDisallowed disallowGciStore traverseByCallback
                                         dbTransient instancesNonPersistent instancesInvariant ).
  myOpts := IdentitySet withAll: self _optionsArray .
  supOpts := IdentitySet new .
  self superclass ifNotNil:[:sc | supOpts addAll: sc _optionsArray ].
  myInheritable := myOpts * inheritable .
  supInheritable := supOpts * inheritable .
  notInher := supInheritable - myInheritable .
  notInher size > 0 ifTrue:[
    result := { #noInheritOptions } .
    result addAll: (SortedCollection withAll: myOpts) .
  ] ifFalse:[
    myOpts := myOpts - inheritable + (myInheritable - supInheritable) .
    result := Array withAll: (SortedCollection withAll: myOpts).
  ].
  ^ result
%

category: 'Private'
method: Class
_optionsChangableTo: formatArr

  "formatArr describes format and options that would be produced by
   a new class creation.
   Returns true or a String describing why a new version of the class would be required."
  | fmt newFmt str list |
  fmt := format .
  str := String new .
  newFmt := formatArr at: 1 .
  ((fmt bitAnd: 16r800) ~~ 0 and:[ (newFmt bitAnd: 16r800) == 0 ]) ifTrue:[
     str addAll: 'instancesNonPersistent cannot be cleared,' 
  ].
  list := #( 16r8 instancesInvariant 16r1000 'dbTransient'
             16r40 'nscDuplicates' 16r7 implementationFormat ).
  1 to: list size by: 2 do:[:j | | mask |
    mask := list at: j .
    (fmt bitAnd: mask) ~~ (newFmt bitAnd: mask) ifTrue:[
      str addAll:'difference in ', (list at: j+1),', '.
    ].
  ].
  ((fmt bitAnd: 16r800) == 0 and:[ (newFmt bitAnd: 16r800) ~~ 0 ]) ifTrue:[
     "setting instancesNonPersistent"
     (newFmt bitAnd: 16r1008) ~~ 0 ifTrue:[
       str addAll:
       'instancesNonPersistent not allowed with dbTransient or instancesInvariant,'.
     ].
  ].
  ((fmt bitAnd: 16r20) == 0 and:[ (newFmt bitAnd: 16r20) ~~ 0 ]) ifTrue:[
     str addAll:'cannot set subclassesDisallowed if previously cleared,'.
  ].
  self isModifiable ifFalse:[
    (formatArr at: 2 "modifiableBool") ifTrue:[ str addAll:'cannot reenable modifiable,' ].
  ].
  ((fmt bitOr: newFmt) bitAnd: 16r7FFFC000) ~~ 0 ifTrue:[
     str addAll:'isMetaClass,' 
  ].
  (fmt bitShift: -31) = (newFmt bitShift: -31) ifFalse:[ "GC_BEHAV_byteSwizKind_shift"
     str addAll:'difference in byteSwizKind,'.
  ].
  str size == 0 ifTrue:[ ^ true ].
  ^ str.
%

category: 'Browser Methods'
method: Class
_optionsStringForDefinition
  | result arr |
  result :=  'options: #(' copy .
  arr := self _optionsArrayForDefinition . "fix 48681"
  1 to: arr size do:[:j | | sym |
    result add: $  .
    sym := arr at: j .
    (sym at: 1) isDigit ifTrue:[ result add: $#; add: $' ; add: sym ; add: $' ]
                      ifFalse:[ result add: sym ].
  ].
  result add: $)  .
  ^ result
%

category: 'Private'
method: Class
_poolDictsEqual: anArray
  "pool dictionaries should be identical to ensure that compiled-in Associations can be shared"
  "fix bug 42279"

  | sharedPoolsArray |
  sharedPoolsArray := self sharedPools.
  sharedPoolsArray size ~= anArray size ifTrue: [ ^false ].
  1 to: sharedPoolsArray size do: [:index |
    (sharedPoolsArray at: index) == (anArray at: index) ifFalse: [ ^false ]].
  ^true
%

category: 'Private'
method: Class
_recompileAllMethods

"Unconditionally recompile all env 0 methods."

super _recompileAllMethods .
self class _recompileAllMethods
%

category: 'Private'
method: Class
_removeClassVar: aSymbol

"Remove the class variable named aSymbol from the receiver.
 Signals an error if there is no such class variable."

self _validatePrivilege ifTrue:[
  (Symbol _existingWithAll: aSymbol) ifNotNil:[ :aSym |
    classVars ifNotNil:[ :cvs | | assoc |
      assoc := cvs associationAt: aSym otherwise: nil .
      assoc ifNotNil:[
         cvs removeKey: aSym  .
         ^ self
      ].
    ].
  ].
  ^ LookupError new reason: #classErrClassVarNotFound; key: aSymbol ; object: self;
	signal
].
%

category: 'Modifying Classes'
method: Class
_setClassVars: aDict old: previousDict

  classVars ~~ aDict ifTrue:[
    previousDict ~~ classVars ifTrue:[ self error:'invalid store to classVars'].
    classVars := aDict .
    self class _setClassVars: aDict old: previousDict
  ].
%

category: 'Browser Methods'
method: Class
_signedWords

  "Return true if instances of the receiver are byte format with signed words,
   false otherwise."

  self isBytes ifTrue:[
     ^ ((format bitShift: -33"GC_BEHAV_signedWords_shift") bitAnd: 1) == 1
  ].
  ^ false .
%

category: 'Subclass Creation'
method: Class
_subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
newVersionOf: oldClass
description: aDescription
options: optionsArray

 "optionsArray is an Array of Symbols containing zero or more of
   #noInheritOptions,  #subclassesDisallowed, #disallowGciStore, #modifiable ,
   #traverseByCallback #selfCanBeSpecial #logCreation
 and at most one of
   #dbTransient, #instancesNonPersistent, #instancesInvariant
 If present, #noInheritOptions must be the first element and it causes
 none of subclassesDisallowed, disallowGciStore, traverseByCallback,
         dbTransient, instancesNonPersistent, instancesInvariant
 to be inherited from the superclass, nor copied from the
 current version of the class.
 #selfCanBeSpecial is never inherited and is needed only when modifying
 superclass hierarchy above classes with special format.
 The option #logCreation, if present in the options array, causes logging
 with GsFile(C)>>gciLogSever:  of class creation / equivalence.

 Returns oldClass if it would be equivalent to the requested new class.
 (See the class comment for Class). "

	| hist descr |
	descr := aDescription.
	oldClass
		ifNotNil:
			[ (self
				_equivalentSubclass: oldClass
				superCls: self
				name: aString
				newOpts: optionsArray
				newFormat: oldClass format
				newInstVars: anArrayOfInstvarNames
				newClassInstVars: anArrayOfClassInstVars
				newPools: anArrayOfPoolDicts
				newClassVars: anArrayOfClassVars
				inDict: aDictionary
				isKernel: false)
					ifTrue:
						[oldClass _commentOrDescription: aDescription.
						^oldClass	"avoid creation of a new version"].
			hist := oldClass classHistory.
			descr
				ifNil:
					[descr := [oldClass commentForFileout] on: Error
								do: [:ex | 'old comment not available']]].
	^self
		_subclass: aString
		instVarNames: anArrayOfInstvarNames
		format: format
		classVars: anArrayOfClassVars
		classInstVars: anArrayOfClassInstVars
		poolDictionaries: anArrayOfPoolDicts
		inDictionary: aDictionary
		inClassHistory: hist
		description: descr
		options: optionsArray
%

category: 'Modifying Classes'
method: Class
_subclasses

"Returns the class variable that is the list of subclasses, or
 nil if the receiver does not keep track of subclasses."

^ subclasses
%

category: 'Modifying Classes'
method: Class
_subclasses: anIdentitySet

"Modifies the class variable that is the list of subclasses."

self _validatePrivilege ifTrue:[
  anIdentitySet ifNotNil:[ (anIdentitySet _validateClass: IdentitySet) ifFalse:[ ^ nil ]].
  subclasses := anIdentitySet
].
%

category: 'Private'
method: Class
_topazFileoutDefn
  "used by topaz fileout command"
  | str nm cat comm |
  nm := self name .
  (str := 'run' copy) lf ;
       add: self _definition .
   str last ~~ Character lf ifTrue:[ str lf ].
   str  add: $% ; lf .
  cat := self category .
  comm := self commentForFileout .
  cat ifNotNil:[
    str add: 'run'; lf ;
       add: nm ; add: ' category: '; add: self category quoted ; lf ;
       add: $% ; lf
  ].
  comm ifNotNil:[
    str add: 'run'; lf ;
       add: nm ; add: ' comment: ' ; add: self comment quoted ; lf ;
    add: $% ; lf .
  ].
  ^ str
%

category: 'Private'
method: Class
_validateOptions: optionsArray  withFormat: theFormat newClassName: subClsName
  "theFormat is requested format and optionsArray is
   requested options for a new class .
   Returns an Array,
    { (theFormat with possible bits added) .  modifiableBBoolean . logBoolean } .
   signals an Error if the optionsArray contains an unregconized option."
| iOptCount fmt modifiableBool logBool swizSym nSwiz signed |
modifiableBool := false .
logBool := false .
nSwiz := 0 .
fmt := theFormat bitAnd: 16r2000 bitInvert "never inherit selfCanBeSpecial" .
iOptCount := 0 .
1 to: optionsArray size do:[:j | | oSym |
  oSym := optionsArray at: j .
  oSym == #noInheritOptions ifTrue:[
    j == 1 ifFalse:[ subClsName _error: #classErrBadFormat with: '#noInheritOptions must be first element of options'].
    "do not inherit dbTransient,instancesNonPersistent,instancesInvariant,subclassesDisallowed,
                    disallowGciStore,traverseByCallback "
    fmt := fmt bitAnd: (16r1E28  bitInvert) ] ifFalse:[
  oSym == #dbTransient ifTrue:[ iOptCount := iOptCount + 1 . fmt := fmt bitOr: 16r1000 ] ifFalse:[
  oSym == #instancesNonPersistent  ifTrue:[ iOptCount := iOptCount + 1 . fmt := fmt bitOr: 16r800 ] ifFalse:[
  oSym == #instancesInvariant ifTrue:[ iOptCount := iOptCount + 1 . fmt := fmt bitOr: 16r8 ] ifFalse:[
  oSym == #subclassesDisallowed ifTrue:[ fmt := fmt bitOr: 16r20 ] ifFalse:[
  oSym == #disallowGciStore ifTrue:[ fmt := fmt bitOr: 16r200 ] ifFalse:[
  oSym == #traverseByCallback ifTrue:[ fmt := fmt bitOr: 16r400 ] ifFalse:[
  oSym == #selfCanBeSpecial ifTrue:[ fmt := fmt bitOr: 16r2000 ] ifFalse:[
  oSym == #modifiable ifTrue:[ modifiableBool := true ] ifFalse:[
  oSym == #logCreation ifTrue:[ logBool := true ] ifFalse:[
  oSym == #'2byteWords' ifTrue:[ swizSym := oSym  . nSwiz := nSwiz + 1 . signed:=false ] ifFalse:[
  oSym == #'4byteWords' ifTrue:[ swizSym := oSym  . nSwiz := nSwiz + 1 . signed:=false ] ifFalse:[
  oSym == #'8byteWords' ifTrue:[ swizSym := oSym  . nSwiz := nSwiz + 1 . signed:=false ] ifFalse:[
  oSym == #'signed2byteWords' ifTrue:[ swizSym := oSym  . nSwiz := nSwiz + 1 . signed:=true ] ifFalse:[
  oSym == #'signed4byteWords' ifTrue:[ swizSym := oSym  . nSwiz := nSwiz + 1 . signed:=true ] ifFalse:[
  oSym == #'signed8byteWords' ifTrue:[ swizSym := oSym  . nSwiz := nSwiz + 1 . signed:=true ] ifFalse:[
    subClsName _error: #classErrBadFormat with: 'unrecognized option ' , oSym printString
  ]]]]]]]]]]]]]]]].
  "See also  Class >> _rwOptionsArray  in Rowan ."
].
iOptCount > 1 ifTrue:[
  subClsName _error: #classErrBadFormat
        with:'only one of #dbTransient #instancesNonPersistent  #instancesInvariant allowed' .
].
nSwiz > 1 ifTrue:[
  subClsName _error: #classErrBadFormat
        with: 'only one of #''2byteWords'' #''4byteWords'' #''8byteWords'' #''signed2byteWords'' #''signed4byteWords'' #''signed8byteWords'' allowed'
].
swizSym ifNotNil:[
  (fmt bitAnd: 3) ~~ 1  ifTrue:[
    subClsName _error: #classErrBadFormat
        with:'A *byteWords options wa specified but new class would not be byte format'.
  ].
  fmt := self _addByteSwizzle: swizSym toFormat: fmt newClassName: subClsName signed: signed .
].
^ { fmt . modifiableBool . logBool  }
%

category: 'Subclass Creation'
method: Class
_validName: str

"Returns whether the given string contains Characters valid for a class name.
 The string need not be a Symbol."

| ch |

(str size == 0 or: [str size > 64]) ifTrue: [
  ^false
].

ch := str at: 1.
(ch == $_  or: [ch isLetter]) ifFalse: [
  ^false
].

2 to: str size do: [ :i |
  ch := str at: i.
  (ch == $_  or: [ch isAlphaNumeric]) ifFalse: [
    ^false
  ].
].

^true.
%

category: 'Browser Methods'
method: Class
_versionedName
"used by topaz"

^ [ self versionedName ] onException: Error do:[:ex| ^ self describe]
%

category: 'Private'
method: Class
__definition
	"Returns a String.
	For insteractive use to show constraints in a class
  in an upgraded repository."

	^self _definitionInContext: System myUserProfile withConstraints: true
%

category: 'Private'
method: Class
__makeVariant

"Makes the receiver variant."

<primitive: 272>
self _primitiveFailed: #__makeVariant .
self _uncontinuableError
%

! Class extensions for 'ClassHistory'

!		Class methods for 'ClassHistory'

removeallmethods ClassHistory
removeallclassmethods ClassHistory

category: 'Instance Creation'
classmethod: ClassHistory
new

"Create a new ClassHistory."

| result |
result := super new .
result _setNoStubbing  .
^ result
%

category: 'Instance Creation'
classmethod: ClassHistory
new: anInt

"Disallowed"

self shouldNotImplement: #new: .
self _uncontinuableError
%

category: 'Instance Creation'
classmethod: ClassHistory
with: anObj

"Disallowed"

self shouldNotImplement: #with: .
self _uncontinuableError
%

category: 'Instance Creation'
classmethod: ClassHistory
with: anObj with: obj2

"Disallowed"

self shouldNotImplement: #with:with: .
self _uncontinuableError
%

category: 'Instance Creation'
classmethod: ClassHistory
with: anObj with: obj2 with: obj3

"Disallowed"

self shouldNotImplement: #with:with:with: .
self _uncontinuableError
%

category: 'Instance Creation'
classmethod: ClassHistory
with: anObj with: obj2 with: obj3 with: obj4

"Disallowed"

self shouldNotImplement: #with:with:with:with: .
self _uncontinuableError
%

category: 'Instance Creation'
classmethod: ClassHistory
withAll: aCollection

"Disallowed"

self shouldNotImplement: #withAll: .
self _uncontinuableError
%

!		Instance methods for 'ClassHistory'

category: 'Accessing'
method: ClassHistory
at: aTimeOrIndex

"Returns the Class that was current at the given time.  The time may be
 specified absolutely using a DateTime, or relatively using an integer.  If a
 DateTime is specified, returns the version of the class that was active at
 that time, or nil if the time is before the earliest version.

 If an Integer is specified, it is used to chronologically select the
 version, with 1 indicating the first version created, 2 the version, and so on.
 If the index is less than one or greater than the number of versions in
 the history, an error is generated."

| candidate aClass|

(aTimeOrIndex class == DateTime)
  ifFalse: [ ^ super at: aTimeOrIndex ]
  ifTrue: [
    "candidate is initially nil.  Iterate through the history from
    the beginning.  If the entry's time is before the specified time,
    it is possibly the class that was active so save it as the best
    candidate.  If the entry's time is after the specified time, it
    cannot be the answer and no subsequent ones can be, so returns the
    current candidate."

    1 to: self size do: [ :j |
      aClass := self at: j.
      (aClass timeStamp <= aTimeOrIndex)
        ifTrue: [ candidate := aClass ]
        ifFalse: [ ^ candidate ].
    ].
    ^ candidate
  ]
%

category: 'Accessing'
method: ClassHistory
current

"Returns the current, or most recent class."

^ super at: self size
%

category: 'Accessing'
method: ClassHistory
currentVersion

"Returns the most recent version in the receiver's collection of versions."

| sz |
(sz := self size) == 0 ifTrue: [ ^nil ].
^self at: sz
%

category: 'Accessing'
method: ClassHistory
description

"Returns the description of this ClassHistory."

^ description
%

category: 'Updating'
method: ClassHistory
description: aString

"Updates the description of this ClassHistory."

description := aString
%

category: 'Accessing'
method: ClassHistory
name

"Returns the name of this ClassHistory."

^ name
%

category: 'Updating'
method: ClassHistory
name: aString

"Updates the name of this ClassHistory."

name := aString asString asSymbol
%

category: 'Updating'
method: ClassHistory
newVersion: aClass

"Installs the given class as the receiver's most current version.  Does not
 install the receiver in the given class as its version history.
 Returns the class object.

 Generates an error if the receiver's basicSize would exceed 2034 "

| currSize idxSize |
currSize := self basicSize .
currSize >= 2034 ifTrue:[   "virtual machine constant"
  "VM restriction, classHistory cannot be a large object"
  idxSize := self size .
  self _error: #objErrMaxSize args:{ idxSize . idxSize + 1 }.
  self _uncontinuableError
  ]
ifFalse:[
  ^ self add: aClass
  ]
%

category: 'Updating'
method: ClassHistory
removeVersion: aClass

"Removes the given class from the receiver's list of versions."

| idx |
idx := self indexOf: aClass.
idx ~~ 0 ifTrue: [
  self removeFrom: idx to: idx
].
%

category: 'Class Membership'
method: ClassHistory
species

"Returns a class similar to, or the same as, the receiver's class which
 can be used for containing derived copies of the receiver."

^ Array
%

! Class extensions for 'ClientForwarderSend'

!		Instance methods for 'ClientForwarderSend'

removeallmethods ClientForwarderSend
removeallclassmethods ClientForwarderSend

category: 'Handling'
method: ClientForwarderSend
defaultAction

"Return an error to the controlling GCI client. Stack is saved
 and available as an argument to the GCI error struct.

 instVars of the receiver go into the GCI error struct as follows
 self.receiver -->  GciErrSType.args[0]
 self.clientObj --> GciErrSType.args[1]
 self.selector -->  GciErrSType.args[2]
 self.gsArgs -->    GciErrSType.args[3]   (instVar defined in AbstractException)
"

<primitive: 33>  "this frame removed from stack before saving stack."
self _uncontinuableError
%

category: 'Instance initialization'
method: ClientForwarderSend
initialize
  gsNumber := ERR_ClientForwarderSend.
  gsResumable := true .
  gsTrappable := false . "should always go to the client"
%

category: 'Instance initialization'
method: ClientForwarderSend
receiver: aSelf clientObj: aClientObj selector: aSymbol args: anArray
  receiver := aSelf .
  clientObj := aClientObj .
  selector := aSymbol .
  gsArgs := anArray
%

! Class extensions for 'ClusterBucket'

!		Class methods for 'ClusterBucket'

removeallmethods ClusterBucket
removeallclassmethods ClusterBucket

category: 'Listing Instances'
classmethod: ClusterBucket
allInstances

"Returns the collection of all instances of the receiver."

^ AllClusterBuckets
%

category: 'Accessing'
classmethod: ClusterBucket
bucketWithId: aPositiveSmallInt

"Returns the instance with the specified ID if one exists.  Generates an error
 if anInt is less than 1 or outside of the range of existing cluster buckets."

^ AllClusterBuckets at: aPositiveSmallInt
%

category: 'Instance Creation'
classmethod: ClusterBucket
new

"Creates an instance of the receiver and adds the new instance to
 AllClusterBuckets."

| newId result max |
max := System _maxClusterId .
(AllClusterBuckets size >= max) ifTrue: [
  self _error: #rtErrMaxClusterId args: { max }.
  ^ AllClusterBuckets at: 1 "returns default bucket"
  ] .
newId := AllClusterBuckets size + 1 .
result := super new .
result _clusterId: newId ;
       keepClusteredOnModify: false ;
  "create an empty string for the description, so description can be
   modified later without putting the ClusterBucket in a write set."
       description: String new .

AllClusterBuckets addLast: result .
^ result
%

category: 'Instance Creation'
classmethod: ClusterBucket
newForExtent: extentId

"DEPRECATED.
 In Gemstone64 v2.0, the relationship between a ClusterBucket and
 an extent is weak.  Clustering an object into a ClusterBucket will
 attempt to use the specified extentId , if the session already has a
 free page within that extent.
 Otherwise  DBF_ALLOCATION_MODE in Stone's config file will
 take precedence when the session asks stone for more free pages.

 Creates an instance of the receiver for clustering objects in the extent
 extentId.  The extentId argument is a positive SmallInteger in the range of
 1 to (SystemRepository numberOfExtents)."

self deprecated: 'ClusterBucket>>newForExtent: deprecated in v3.2, use new instead'.
^ self new extentId: extentId
%

!		Instance methods for 'ClusterBucket'

category: 'Accessing'
method: ClusterBucket
clusterId

"Returns the value of the private instance variable.  This instance variable
 should only be assigned by ClusterBucket | new."

^ _clusterId
%

category: 'Accessing'
method: ClusterBucket
description

"Returns the value of the description instance variable."

^ description
%

category: 'Updating'
method: ClusterBucket
description: anObject

"Assigns anObject (typically some kind of String object) as the description of
 the receiver."

description := anObject
%

category: 'Accessing'
method: ClusterBucket
extentId

"Returns the value of the extentId instance variable.

 In Gemstone64 v2.0, the relationship between a ClusterBucket and
 an extent is weak.  Clustering an object into a ClusterBucket will
 attempt to use the specified extentId , if the session already has a
 free page within that extent.
 Otherwise  DBF_ALLOCATION_MODE in Stone's config file will
 take precedence when the session asks stone for more free pages."


^ extentId
%

category: 'Updating'
method: ClusterBucket
extentId: anExtentId

"DEPRECATED.  In Gemstone64 v2.0, the relationship between a ClusterBucket and
 an extent is weak.  Clustering an object into a ClusterBucket will
 attempt to use the specified extentId , if the session already has a
 free page within that extent.
 Otherwise  DBF_ALLOCATION_MODE in Stone's config file will
 take precedence when the session asks stone for more free pages.

 An argument of nil specifies don't care behavior.  Positive arguments are
 an offset into the result of Repository | fileNames, thus specifying an extent.
 Reference to an extent which does not exist will generate an error at the
 time of executing this method.

 At the time of object modification, a non-nil extentId specifies which extent
 to attempt to put the object in (per DEPRECATED notice above).
 If the extent no longer exists, or is full,
 at the time of clustering or object modification, don't care behavior occurs."


self deprecated: 'ClusterBucket>>extentId: deprecated in v3.2, use new instead'.
SystemRepository validateExtentId: anExtentId .
extentId := anExtentId .
self _clearClusterCache .
%

category: 'Accessing'
method: ClusterBucket
keepClusteredOnModify

"This feature is not implemented in this release."

^ keepClusteredOnModify
%

category: 'Updating'
method: ClusterBucket
keepClusteredOnModify: aBoolean

"This feature is deferred until a future release.  Argument value of true
 is not supported in this release."

keepClusteredOnModify := false .
self _clearClusterCache .
%

category: 'Updating'
method: ClusterBucket
_clearClusterCache

"Private.  Any method that modifies an instance variable of a
 ClusterBucket that affects clustering behavior must send this method.  The
 instance variable description is not included in such instance variables."

<primitive: 235>
self _primitiveFailed: #_clearClusterCache .
self _uncontinuableError
%

category: 'Updating'
method: ClusterBucket
_clusterId: aSmallInteger

"Private.  For use only during instance creation."

_clusterId := aSmallInteger .
self _clearClusterCache .
%

! Class extensions for 'ClusterBucketArray'

!		Instance methods for 'ClusterBucketArray'

removeallmethods ClusterBucketArray
removeallclassmethods ClusterBucketArray

category: 'Clustering'
method: ClusterBucketArray
cluster

"Instances of ClusterBucketArray, especially AllClusterBuckets
 must always be clustered in the default bucket."

^ super clusterInBucket: 1
%

category: 'Clustering'
method: ClusterBucketArray
clusterInBucket: aClusterBucketOrId

"Instances of ClusterBucketArray, especially AllClusterBuckets
 must always be clustered in the default bucket."

^ super clusterInBucket: 1
%

! Class extensions for 'Collection'

!		Class methods for 'Collection'

removeallmethods Collection
removeallclassmethods Collection

category: 'Instance Creation'
classmethod: Collection
with: aValue

"Returns an instance of the receiver containing the argument."

| inst |
inst := self new.
inst add: aValue.
^inst
%

category: 'Instance Creation'
classmethod: Collection
with: aValue with: val2

"Returns an instance of the receiver containing the arguments."

| inst |
inst := self new.
inst add: aValue; add: val2.
^inst
%

category: 'Instance Creation'
classmethod: Collection
with: aValue with: val2 with: val3

"Returns an instance of the receiver containing the arguments."

| inst |
inst := self new.
inst add: aValue; add: val2; add: val3.
^inst
%

category: 'Instance Creation'
classmethod: Collection
with: aValue with: val2 with: val3 with: val4

"Returns an instance of the receiver containing the arguments."

| inst |
inst := self new.
inst add: aValue; add: val2; add: val3; add: val4.
^inst
%

category: 'Instance Creation'
classmethod: Collection
withAll: aCollection

"Returns an instance of the receiver containing the elements of the argument."

| result |

result:= self new.
result addAll: aCollection.
^result
%

!		Instance methods for 'Collection'

category: 'Comparing'
method: Collection
= aCollection

"Returns true if all of the following conditions are true:

 1.  The receiver and aCollection are of the same class.
 2.  The two collections are of the same size.
 3.  The corresponding elements of the receiver and aCollection are equal."

(self == aCollection)
  ifTrue: [ ^ true ].

(self class == aCollection class)
  ifFalse: [ ^ false ].

(self size == aCollection size)
  ifFalse: [ ^ false ].

self do: [ :anElement |
  (aCollection includes: anElement)
    ifFalse: [ ^ false ].
  ].

^ true.
%

category: 'Adding'
method: Collection
add: newObject

"Makes newObject one of the receiver's elements and returns newObject."

Collection subclassResponsibility: #add:
%

category: 'Adding'
method: Collection
addAll: aCollection

"Adds all of the elements of aCollection to the receiver and returns
 aCollection."

(self == aCollection) ifTrue: [ ^ self addAll: (aCollection copy) ].
aCollection accompaniedBy: self do: [ :me :each | me add: each ].
^ aCollection.
%

category: 'Enumerating'
method: Collection
allSatisfy: aBlock

	"Return true if <aBlock> evaluates to true for every element of the receiver.
	Return true if the receiver is empty. Otherwise return false."

	self do: [:each |
		(aBlock value: each) ifFalse: [^false].
	].
	^true.
%

category: 'Enumerating'
method: Collection
any
	"Return an arbitrary element of the receiver. Error if the receiver is empty."

	self do: [:each | ^each].
	self _error: #assocErrNoElementsDetected.
%

category: 'Enumerating'
method: Collection
anySatisfy: aBlock

        "Return true if <aBlock> evaluates to true for any element of the receiver.
        Otherwise return false. Return false if the receiver is empty."

	self do: [:each |
		(aBlock value: each) ifTrue: [^true].
	].
	^false.
%

category: 'Converting'
method: Collection
asArray

"Returns an Array with the contents of the receiver."

^Array withAll: self
%

category: 'Converting'
method: Collection
asBag

"Returns a Bag with the contents of the receiver."

^Bag withAll: self
%

category: 'Converting'
method: Collection
asByteArray

"Returns an Array with the contents of the receiver."

^ByteArray withAll: self
%

category: 'Converting'
method: Collection
asIdentityBag

"Returns an IdentityBag with the contents of the receiver."

^IdentityBag withAll: self
%

category: 'Converting'
method: Collection
asIdentitySet

"Returns an IdentitySet with the contents of the receiver."

^IdentitySet withAll: self
%

category: 'Converting'
method: Collection
asOrderedCollection

"Returns an OrderedCollection with the contents of the receiver."

^OrderedCollection withAll: self
%

category: 'Converting'
method: Collection
asSet

"Returns a Set with the contents of the receiver."

^Set withAll: self
%

category: 'Converting'
method: Collection
asSortedCollection

"Returns a SortedCollection with the contents of the receiver."

^SortedCollection withAll: self
%

category: 'Converting'
method: Collection
asSortedCollection: sortBlock

"Returns a SortedCollection with the contents of the receiver, using the
 given sort block."

| coll |
coll := SortedCollection sortBlock: sortBlock.
coll addAll: self.
^coll
%

category: 'Converting'
method: Collection
asSortedOrderedCollection

"Returns an OrderedCollection that has been sorted with a SortedCollection
 and having the contents of the receiver."

^OrderedCollection withAll: (SortedCollection withAll: self)
%

category: 'Clustering'
method: Collection
clusterDepthFirst

"This method clusters the receiver and its named and unnamed instance variables
 in depth-first order.  Returns true if the receiver has already been clustered
 during the current transaction; returns false otherwise."

self cluster
  ifTrue: [ ^ true ]
  ifFalse:[
    1 to: self namedSize do: [ :i | (self instVarAt: i) clusterDepthFirst ].
    self do: [ :each | each clusterDepthFirst ].
    ^ false
    ].
%

category: 'Enumerating'
method: Collection
collect: aBlock

"Evaluates aBlock with each of the receiver's elements as the argument.
 Collects the resulting values into a collection of class specified by
 sending #speciesForCollect message to the receiver. Returns the new
 collection. The argument aBlock must be a one-argument block.

 For SequenceableCollections, the result preserves the ordering of the
 receiver.  That is, if element a comes before element b in the receiver,
 then element a is guaranteed to come before b in the result."

|result|

result := self speciesForCollect new .
self accompaniedBy: result do: [ :res :each | res add: (aBlock value: each) ].
^ result
%

category: 'Enumerating'
method: Collection
detect: aBlock

"Evaluates aBlock repeatedly, with elements of the receiver as the argument.
 Returns the first element for which aBlock evaluates to true.  If none of the
 receiver's elements evaluates to true, generates an error.  The argument
 aBlock must be a one-argument block."

^ self detect: aBlock
       ifNone: [^ self _error: #assocErrNoElementsDetected args: { aBlock }]
%

category: 'Enumerating'
method: Collection
detect: aBlock ifNone: exceptionBlock

"Evaluates aBlock repeatedly, with elements of the receiver as the argument.
 Returns the first element for which aBlock evaluates to true.  If none of the
 receiver's elements evaluates to true, this evaluates the argument
 exceptionBlock and returns its value.  The argument aBlock must be a
 one-argument block, and exceptionBlock must be a zero-argument block."

self do: [:each| (aBlock value: each) ifTrue: [^each]].
^exceptionBlock value
%

category: 'Enumerating'
method: Collection
do: aBlock

"Evaluates the one-argument block aBlock using each element of the
 receiver in order.  Returns the receiver."

"Must be reimplemented in subclasses such as Dictionary which have different
 format or implementation."

1 to: self size do: [:i |
  aBlock value: (self at: i)
].
^ self
%

category: 'Enumerating'
method: Collection
do: operationBlock separatedBy: aBlock
	"Evaluates the one-argument block operationBlock using each element of the
	 receiver in order.  aBlock is evaluated between successive elements.
	 Returns the receiver. "

	| index |
	index := 0.
	self do:
			[:each |
			(index := index + 1) > 1 ifTrue: [aBlock value].
			operationBlock value: each].
	^self
%

category: 'Error Handling'
method: Collection
errorDifferentSizeCollections

"Reports an error indicating that the size of the receiver collection is
 different from the size of the argument collection."

^ self _error: #objErrDiffSizeColl
%

category: 'Error Handling'
method: Collection
errorInvalidArgClass: argument classes: classArray

"Reports an error indicating that the class of argument is not one of those
 specified in classArray."

^ self _error: #rtErrInvalidArgClass args: { argument . classArray }.
%

category: 'Repository Conversion'
method: Collection
getIndexInfo
  "Returns indexing information for the receiver."

  ^ {}
%

category: 'Comparing'
method: Collection
hash

"Returns a numeric hash key for the receiver."

| hashValue |

hashValue := (97133 bitXor: (self size)) bitXor: (self class asOop).
"For large collections, the hash value is just a function of its size
 and class"
(self size > 64) ifTrue: [ ^ hashValue abs ].

self do: [ :anElement |
  (anElement isKindOf: Collection)
    ifTrue: [ hashValue := hashValue bitXor: anElement size ]
    ifFalse: [ hashValue := hashValue bitXor: anElement hash ].
  ].
^ hashValue abs.
%

category: 'Searching'
method: Collection
identicalOccurrencesOf: anObject

"Returns the number of the receiver's elements that are identical (==) to
 anObject."

| result |

result := 0.
self do: [ :element |
  (anObject == element) ifTrue: [ result := result + 1 ]
  ].
^result
%

category: 'Searching'
method: Collection
includes: anObject

"Returns true if anObject is equal to one of the elements of the receiver.
 Returns false otherwise."

self do: [ :element | (anObject = element) ifTrue: [ ^true ]].
^false
%

category: 'Searching'
method: Collection
includesIdentical: anObject

"Returns true if anObject is identical to one of the elements of the receiver.
 Returns false otherwise."

self do: [ :element | (anObject == element) ifTrue: [ ^true ]].
^false
%

category: 'Searching'
method: Collection
includesValue: anObject

"Returns true if anObject is equal to one of the elements of the receiver.
 Returns false otherwise."

"Note: This method is the same as #includes:"

^ self includes: anObject.
%

category: 'Enumerating'
method: Collection
inject: aValue into: aBlock

"Accumulates a running value associated with evaluating the argument,
 aBlock, with the current value and the each element of the receiver
 as block arguments.  The initial value is the value of the argument, aValue.
 For example: total := #(1 2 3 4) inject: 0 into: [:sum:int | sum + int]"

| val |

val := aValue.
self do: [:element | val := aBlock value: val value: element ].
^val
%

category: 'Testing'
method: Collection
isEmpty

"Returns true if the receiver is empty.  Returns false otherwise."

^self size == 0
%

category: 'Testing'
method: Collection
notEmpty

"Returns true if the receiver is not empty.  Returns false otherwise."

^self size ~~ 0
%

category: 'Searching'
method: Collection
occurrencesOf: anObject

"Returns the number of the receiver's elements that are equal to anObject."

| count |

count := 0.
self do: [ :element| (anObject = element) ifTrue: [count := count + 1]].
^count.
%

category: 'Json'
method: Collection
printJsonOn: aStream

	| delimiter |
	delimiter := ''.
	aStream nextPut: $[.
	self do: [:each |
		aStream nextPutAll: delimiter.
		each printJsonOn: aStream.
		delimiter := ','.
	].
	aStream nextPut: $].
%

category: 'Formatting'
method: Collection
printNonRecursiveRepresentationOn: aStream recursionSet: anIdentitySet
	"Put a displayable representation of the receiver on the given stream
	 while avoiding recursion from object reference loops."

	| count sz |
	super printOn: aStream recursionSet: anIdentitySet.
	aStream nextPutAll: '( ' .
	count := 1 .
	sz := self size .
	self do:[:anElement |
	  aStream position > 700 ifTrue:[
		"prevent infinite recursion when printing cyclic structures, and
		 limit the size of result when printing large collections."
		aStream nextPutAll: '...)' .
		^ self
		] .
	  anElement printOn: aStream recursionSet: anIdentitySet.
	  count < sz ifTrue:[ aStream nextPutAll: ', ' ].
	  count := count + 1 .
	  ].
	aStream nextPut: $) .
%

category: 'Formatting'
method: Collection
printOn: aStream

"Puts a displayable representation of the receiver on the given stream."

^ self printNonRecursiveOn: aStream
%

category: 'Hashing'
method: Collection
rehash
	"Re-establish any hash invariants of the receiver."
%

category: 'Enumerating'
method: Collection
reject: aBlock

"Evaluates aBlock with each of the receiver's elements as the argument.  Stores
 the values for which aBlock is false into a collection of the same class as
 the receiver, and returns the new collection.  The argument aBlock must be a
 one-argument block.

 For SequenceableCollections, the result preserves the ordering of the
 receiver.  That is, if element a comes before element b in the receiver,
 then element a is guaranteed to come before b in the result."

|result|
result := self species new.
self do: [:each|
  (aBlock value: each) ifFalse: [result add: each]
  ].
^ result
%

category: 'Removing'
method: Collection
remove: oldObject

"Removes from the receiver an object that is equivalent to oldObject and
 returns oldObject.  If several elements of the receiver are equivalent to
 oldObject, only one instance is removed.  If oldObject has no equivalent
 elements in the receiver, raises an error."

^ self remove: oldObject ifAbsent: [self _errorNotFound: oldObject]
%

category: 'Removing'
method: Collection
remove: oldObject ifAbsent: anExceptionBlock

"Removes from the receiver an object that is equivalent to oldObject and
 returns oldObject.  If several elements of the receiver are equivalent to
 oldObject, only one instance is removed.  If oldObject has no equivalent
 elements in the receiver, anExceptionBlock is evaluated and the result of the
 evaluation is returned."

^ self subclassResponsibility: #remove:ifAbsent:
%

category: 'Removing'
method: Collection
removeAll: aCollection

"For each element in aCollection, removes from the receiver one element that is
 equivalent to the element in aCollection.  Returns aCollection if successful."

aCollection == self ifTrue:[
  self size: 0 .
  ^ aCollection
  ].
aCollection accompaniedBy: self do: [ :me :element | me remove: element ].
^ aCollection.
%

category: 'Removing'
method: Collection
removeAllIdentical: aCollection

"For each element in aCollection, removes from the receiver one element that is
 identical to the element in aCollection.  Returns aCollection if successful."

aCollection accompaniedBy: self do: [:me :element | me removeIdentical: element ].
^ aCollection.
%

category: 'Removing'
method: Collection
removeAllPresent: aCollection

"Removes from the receiver one occurrence of each element of aCollection that is
 also an element of the receiver.  Differs from removeAll: in that, if some
 elements of aCollection are not present in the receiver, no error is generated.
 Returns aCollection."

aCollection do: [ :anElement |
  self removeIfPresent: anElement
  ].

^ aCollection
%

category: 'Removing'
method: Collection
removeAllSuchThat: aBlock
   "Remove all elements of the receiver for which aBlock returns true.
    Answer the removed elements.
    This is a fallback implementation. It is not efficient."

  | toBeRemoved |
  toBeRemoved := self select: aBlock.
  self removeAllIdentical: toBeRemoved.
  ^ toBeRemoved
%

category: 'Removing'
method: Collection
removeIdentical: oldObject

"Removes from the receiver an object that is identical to oldObject, and
 returns oldObject.  If several elements of the receiver are identical to
 oldObject, only one instance is removed.  If oldObject is not present in
 the receiver, raises an error."

^ self removeIdentical: oldObject ifAbsent: [self _errorNotFound: oldObject]
%

category: 'Removing'
method: Collection
removeIdentical: oldObject ifAbsent: anExceptionBlock

"Removes from the receiver an object that is identical to oldObject and
 returns oldObject.  If several elements of the receiver are identical to
 oldObject, only one instance is removed.  If oldObject is not present in
 the receiver, anExceptionBlock is evaluated and the result of the
 evaluation is returned."

^ self subclassResponsibility: #removeIdentical:ifAbsent:
%

category: 'Enumerating'
method: Collection
select: aBlock

"Evaluates aBlock with each of the receiver's elements as the argument.
 Stores the values for which aBlock is true into a collection of the
 same class as the receiver, and returns the new collection.
 The argument aBlock must be a one-argument block.

 For SequenceableCollections, the result preserves the ordering of the
 receiver.  That is, if element a comes before element b in the receiver,
 then element a is guaranteed to come before b in the result.

 The new collection that this method returns does not retain any indexes of the
 receiver.  If you want to perform indexed selections on the new collection,
 you must build all of the necessary indexes.  For more information, see the
 GemStone Programming Guide."

| result |

result:= self species new.
self do: [ :each | (aBlock value: each) ifTrue: [result add: each]].
^result
%

category: 'Sorting'
method: Collection
sortAscending

"Returns an Array containing the elements of the receiver sorted in ascending
 order."

^ self sortAscending: '' persistentRoot: nil.
%

category: 'Sorting'
method: Collection
sortAscending: aSortSpec

"Returns an Array containing the elements of the receiver, sorted in ascending
 order, as determined by the values of the instance variables represented by
 aSortSpec.  The argument aSortSpec must be either a String representing a
 single path, or an Array holding up to 16 such Strings (each representing a
 path).  If aSortSpec is an Array, the first path in the Array is the primary
 sort key, and the remaining paths are taken in order as subordinate keys.

 Each path in aSortSpec must follow the rules for equality indexes.
 In addition, if any path in aSortSpec is an empty path (that is, a zero-length
 String), the sort is performed upon the elements of the receiver itself,
 rather than upon the instance variables of those elements."

^self sortAscending: aSortSpec persistentRoot: nil
%

category: 'Sorting'
method: Collection
sortAscending: aSortSpec persistentRoot: persistentArrayOrNil

"Returns an Array containing the elements of the receiver, sorted in ascending
 order, as determined by the values of the instance variables represented by
 aSortSpec.  The argument aSortSpec must be either a String representing a
 single path, or an Array holding up to 16 such Strings (each representing a
 path).  If aSortSpec is an Array, the first path in the Array is the primary
 sort key, and the remaining paths are taken in order as subordinate keys.

 Each path in aSortSpec must follow the rules for equality indexes.
 In addition, if any path in aSortSpec is an empty path (that is, a zero-length
 String), the sort is performed upon the elements of the receiver itself,
 rather than upon the instance variables of those elements."

| arg |

arg := { } .

(aSortSpec isKindOf: CharacterCollection)
  ifTrue: [ arg add: aSortSpec ; add: #ASCENDING ]
  ifFalse: [
     aSortSpec accompaniedBy: arg do: [ :aArg :each | aArg add: each ; add: #ASCENDING ]
  ].

^ self sortWith: arg persistentRoot: persistentArrayOrNil
%

category: 'Sorting'
method: Collection
sortDescending

"Returns an Array containing the elements of the receiver sorted in descending
 order."

^ self sortDescending: '' persistentRoot: nil.
%

category: 'Sorting'
method: Collection
sortDescending: aSortSpec

"Returns an Array containing the elements of the receiver, sorted in descending
 order, as determined by the values of the instance variables represented by
 aSortSpec.  The argument aSortSpec must be either a String representing a
 single path, or an Array holding up to 16 such Strings (each representing a
 path).  If aSortSpec is an Array, the first path in the Array is the primary
 sort key, and the remaining paths are taken in order as subordinate keys.

 Each path in aSortSpec must follow the rules for equality indexes.
 In addition, if any path in aSortSpec is an empty path (that is, a zero-length
 String), the sort is performed upon the elements of the receiver itself,
 rather than upon the instance variables of those elements."

^ self sortDescending: aSortSpec persistentRoot: nil
%

category: 'Sorting'
method: Collection
sortDescending: aSortSpec persistentRoot: persistentArrayOrNil

"Returns an Array containing the elements of the receiver, sorted in descending
 order, as determined by the values of the instance variables represented by
 aSortSpec.  The argument aSortSpec must be either a String representing a
 single path, or an Array holding up to 16 such Strings (each representing a
 path).  If aSortSpec is an Array, the first path in the Array is the primary
 sort key, and the remaining paths are taken in order as subordinate keys.

 Each path in aSortSpec must follow the rules for equality indexes.
 In addition, if any path in aSortSpec is an empty path (that is, a zero-length
 String), the sort is performed upon the elements of the receiver itself,
 rather than upon the instance variables of those elements."

| arg |

arg := { } .

(aSortSpec isKindOf: CharacterCollection)
  ifTrue: [ arg add: aSortSpec ; add: #DESCENDING ]
  ifFalse: [
    aSortSpec accompaniedBy: arg do: [:aArg :each | aArg add: each ; add: #DESCENDING ]
  ].

^ self sortWith: arg persistentRoot: persistentArrayOrNil
%

category: 'Sorting'
method: Collection
sortWith: aSortPairArray

"Returns an Array containing the elements of the receiver, sorted according to
 the contents of aSortPairArray.  The argument aSortPairArray is an Array of
 Strings that represent path/direction pairs, in the following form:

 aCollection sortWith: #('a.b' 'ASCENDING' 'a.c' 'DESCENDING' ...)

 That Array may contain up to 16 path/direction pairs.  The first path in the
 Array is the primary sort key, and the remaining paths are taken in order as
 subordinate keys.

 In aSortPairArray, each path String must follow the rules for equality
 indexes.  Each direction String must be either 'ASCENDING' or
 'DESCENDING' (case-insensitive); otherwise, an error is generated.

 In addition, if any path in aSortPairArray is an empty path (that is, a
 zero-length String), the sort is performed upon the elements of the receiver
 itself, rather than upon the instance variables of those elements."

^ self sortWith: aSortPairArray persistentRoot: nil
%

category: 'Sorting'
method: Collection
sortWith: aSortPairArray persistentRoot: persistentArrayOrNil

"Returns an Array containing the elements of the receiver, sorted according to
 the contents of aSortPairArray.  The argument aSortPairArray is an Array of
 Strings that represent path/direction pairs, in the following form:

 aCollection sortWith: #('a.b' 'ASCENDING' 'a.c' 'DESCENDING' ...)

 That Array may contain up to 16 path/direction pairs.  The first path in the
 Array is the primary sort key, and the remaining paths are taken in order as
 subordinate keys.

 In aSortPairArray, each path String must follow the rules for equality
 indexes.  Each direction String must be either 'ASCENDING' or
 'DESCENDING' (case-insensitive); otherwise, an error is generated.

 In addition, if any path in aSortPairArray is an empty path (that is, a
 zero-length String), the sort is performed upon the elements of the receiver
 itself, rather than upon the instance variables of those elements.

 If persistentArrayOrNil is notNil, then it is expected to be an empty persistent array and
 the array will be used to persist large temporary data structures created during
 the sorting operation. IndexManager>>autoCommit must be true in order for periodic
 commits to  be made during the sorting operation. When the sort operation is complete
 the persistent array will be emptied and a final commit performed. The persistentArrayOrNil
 and  IndexManager>>autoCommit should be used when a collection is so large that it
 isn't practical to allocate enough temporary memory."

| paths booleans thePath directionStr directionBool result spaSize |

paths := { } .
booleans := { } .

((spaSize := aSortPairArray size) ~~ 0 and:[ spaSize even])
    ifFalse: [ ^ aSortPairArray _error: #assocErrSortOddLengthArray ].

1 to: spaSize by: 2 do: [ :i |
    thePath := aSortPairArray at: i.
    thePath _isOneByteString ifFalse:[
      thePath _validateClasses: { String }.
      ].
    directionStr := (aSortPairArray at: i + 1) .
    directionBool := true .
    directionStr == #ASCENDING
      ifFalse:[
        (directionStr == #DESCENDING )
          ifTrue:[ directionBool := false ]
          ifFalse:[
            (Symbol _existingWithAll: directionStr asUppercase) ifNotNil:[ :directionSym|
               directionSym == #ASCENDING
               ifFalse:[
                 directionSym == #DESCENDING
                   ifTrue: [ directionBool := false ]
                   ifFalse: [ directionStr _error: #assocErrBadDirection ].
	       ].
	    ] ifNil:[  directionStr _error: #assocErrBadDirection ].
	  ].
       ].
    paths add: thePath.
    booleans add: directionBool .
].
result := self _sortPaths: paths directions: booleans persistentRoot: persistentArrayOrNil.
^ result
%

category: 'Private'
method: Collection
_asCollectionForSorting

"Redefine this method for classes that understand _at: or have different
 semantics for the do: method."

| result |

result := { } .
self accompaniedBy: result do: [:res :element | res add: element ].
^ result.
%

category: 'Private'
method: Collection
_asIdentityBag

"Returns the receiver."

^ self asIdentityBag
%

category: 'Private'
method: Collection
_deepCopyWith: copiedObjDict

| copy myClass |

copy := copiedObjDict at: self otherwise: nil.
copy ifNotNil: [ ^ copy ].

myClass := self class.
copy := myClass new.
copiedObjDict at: self put: copy.

self _deepCopyNamedIvsWith: copiedObjDict to: copy .

1 to: self _basicSize do: [ :n | | anElement |
  anElement := self _at: n.
  copy add: (anElement _deepCopyWith: copiedObjDict)
].
^ copy.
%

category: 'Error Handling'
method: Collection
_errorNotFound: anObject

"Sends an error message indicating that the expected object was not found."

^ self _error: #objErrNotInColl args: { anObject }
%

category: 'Private'
method: Collection
_findRangeIndexWithPath: pathArray

"Redefine this method for classes that support indexes."

^ nil
%

category: 'Private'
method: Collection
_indexedPaths

"Returns the indexed paths for the receiver.  This is a private method that
 should only be used for sorting/indexing."

"This method should be reimplemented for classes where _indexedPaths is
 defined."

^ nil
%

category: 'Sorting'
method: Collection
_sortPaths: thePaths directions: theBooleans

""

^ self _sortPaths: thePaths directions: theBooleans  persistentRoot: nil
%

category: 'Sorting'
method: Collection
_sortPaths: thePaths directions: theBooleans persistentRoot: persistentArrayOrNil
  ""

  | thePath pathArrays indexObjs result |
  pathArrays := Array new: thePaths size.	" do some error checking "
  1 to: thePaths size do: [ :i |
    thePath := thePaths at: i.
    thePath size > 1024
      ifTrue: [
        " check the string size "
        ^ thePath _error: #'assocErrPathTooLong' ].
    (thePath includesValue: $*)
      ifTrue: [
        " check if sorting over set-valued instance variable (not allowed) "
        ^ self _error: #'rtErrBagInvalidSortSpecification' args: {thePath} ].
    pathArrays at: i put: thePath asArrayOfPathTerms ].
  indexObjs := self _indexObjectsFor: pathArrays.
  result := (indexObjs at: 1)
    _sortOn: indexObjs
    directions: theBooleans
    persistentRoot: persistentArrayOrNil.
  ^ result
%

category: 'Private'
method: Collection
_validateNotEmpty

^ self isEmpty
    ifTrue:[ self _error: #objErrCollectionEmpty]
    ifFalse:[ true ]
%

! Class extensions for 'CollisionBucket'

!		Instance methods for 'CollisionBucket'

removeallmethods CollisionBucket
removeallclassmethods CollisionBucket

category: 'Accessing'
method: CollisionBucket
keyValueDictionary

"Returns the value of the instance variable."

^keyValueDictionary
%

category: 'Updating'
method: CollisionBucket
keyValueDictionary: aDict

"Updates the value of the keyValueDictionary instance variable."

keyValueDictionary := aDict
%

category: 'Searching'
method: CollisionBucket
_firstPair

"Returns an Array containing the receiver's first key/value pair.
 If the receiver is empty, returns an Array containing nils."

   numElements == 0 ifFalse: [  | idx |
      idx := 1 .
      "Search for the first non-nil key"
      1 to: self tableSize do: [ :n | | aKey |
         (aKey := self _at: idx ) ifNotNil: [    "inline keyAt:"
            ^ { aKey . (self _at: idx + 1) }
         ].
         idx := idx + 2
      ]
   ].
   ^ { nil . nil } "No first pair was found"
%

category: 'Private'
method: CollisionBucket
_removeAll

"Dereferences the receiver from its parent and shrinks the receiver.
 Used while rebuilding a KeyValueDictionary."

keyValueDictionary := nil.
numElements := 0.
"gs64 v3.0 don't send  size: 0"
%

category: 'Private'
method: CollisionBucket
_removePairAt: anOffset

(self _at: anOffset)  ifNil:[
  ^ Error signal:'key has already been removed'.
].
self _at: anOffset put: nil ;
     _at: anOffset + 1 put: nil .
numElements := numElements - 1 .
%

! Class extensions for 'CompileError'

!		Instance methods for 'CompileError'

removeallmethods CompileError
removeallclassmethods CompileError

category: 'Instance initialization'
method: CompileError
initialize
  gsNumber := ERR_CompileError.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Formatting'
method: CompileError
sourceString
^ gsArgs ifNotNil:[:gsa | gsa atOrNil: 2 ]
%

category: 'Formatting'
method: CompileError
_firstCompilerErrorNumber

^ (self errorDetails at: 1) at: 1
%

! Class extensions for 'CompileWarning'

!		Class methods for 'CompileWarning'

removeallmethods CompileWarning
removeallclassmethods CompileWarning

category: 'Instance creation'
classmethod: CompileWarning
signal: aString method: aMethod
  | ex |
  (ex := self new) args: { aString . aMethod } ;
     signal
%

!		Instance methods for 'CompileWarning'

category: 'Instance initialization'
method: CompileWarning
buildMessageText
^ self buildMessageText:(
  [ | str |
    str := String new .
    gsArgs ifNotNil:[:d| | meth |
      meth := d at: 2 .
      str add:( d at: 1) ; add: ' in ' ;
          add: meth inClass name ;
          add: ' >> ' ; 
          add: meth selector .
    ].
    str
  ] onException: Error do:[:ex |
    ex return: nil
  ]
)
%

category: 'Instance initialization'
method: CompileWarning
initialize
  gsNumber := ERR_CompileWarning .
  gsResumable := true .
  gsTrappable := true .
%

category: 'Documentation'
method: CompileWarning
method
  ^ gsArgs at: 2
%

category: 'Documentation'
method: CompileWarning
warningString
  ^ gsArgs at: 1
%

! Class extensions for 'ControlInterrupt'

!		Instance methods for 'ControlInterrupt'

removeallmethods ControlInterrupt
removeallclassmethods ControlInterrupt

category: 'Instance initialization'
method: ControlInterrupt
initialize
  gsNumber := ERR_ControlInterrupt.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'CryptoError'

!		Instance methods for 'CryptoError'

removeallmethods CryptoError
removeallclassmethods CryptoError

category: 'Instance initialization'
method: CryptoError
initialize
  gsNumber := ERR_CryptoError.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'Date'

!		Class methods for 'Date'

removeallmethods Date
removeallclassmethods Date

category: 'Inquiries'
classmethod: Date
dayOfWeek: dayName

"Returns a SmallInteger that gives the numeric index of the day of the week
 described by dayName. The index is a number between 1 and 7 inclusive,
 where 1 signifies Sunday. dayName must be the full symbolic name of the
 day. Raises an error if dayName is not a valid week day name."

 "Example: Date dayOfWeek: 'Tuesday'."

| aDay |
aDay := dayName asString asLowercase.
aDay at: 1 put: ((aDay at: 1) asUppercase).
^ WeekDayNames value indexOf: aDay
    ifAbsent: [ dayName _error: #rtErrInvalidArgument
    args: { 'argument is not a week day name' }].
%

category: 'Instance Creation'
classmethod: Date
fromStream: aStream

"Creates and returns an instance of the receiver by reading a String from
 aStream.  The String expresses the date in the default format (DD/MM/YYYY).
 Generates an error if the String does not conform to the format."

^ self fromStream: aStream usingFormat: #(1 2 3 $/ 1 1)
%

category: 'Instance Creation'
classmethod: Date
fromStream: aStream usingFormat: anArray

"Creates and returns an instance of the receiver by reading a String from
 aStream.  The String expresses the date in the format specified by anArray.
 The expression is terminated either by a space Character or by the end of the
 Stream.  Generates an error if the String does not conform to the format,
 or if anArray contains an incorrect formatting specification.

 See the class documentation of Date for a complete description of the
 String-formatting specification Array.

 If the month format (5th element) indicates either an abbreviation (2) or an
 entire name (3), then this method tries to determine the month by decoding a
 Character substring.  That substring may include any number of characters, but
 must exactly match a legal month name (or the initial portion of that month
 name).  If the substring matches more than one month, the first month matched
 is used (the search begins with January)."

| dayInt monthInt yearInt dateDelim result parseField parseBlocks |

"This block returns a string up from the input stream up to the specified
 delimiter.  If also allows an end-of-file if that parameter is set true.
 It then skips over the delimiter if it is found.
"
parseField := [ :delim :allowEof | | str |
  str := aStream contents class new.
  [ ((aStream peek isEquivalent: delim) not) and:[aStream atEnd not] ]
    whileTrue: [ str add: aStream next ].

  (aStream atEnd) ifTrue:[
     allowEof ifFalse:[ Date _error: #rtErrBadFormat args: { aStream } ].
     ]
  ifFalse:[ aStream next "skip over delimiter" ].
  str
  ].

parseBlocks := {
  "parse day"
  [ :delim | | nextField |
    nextField := parseField value: delim value: (delim == $  ).
    dayInt := Integer fromCompleteString: nextField
  ] .
  "parse month"
  [ :delim | | nextField |
    nextField := parseField value: delim value: (delim == $  ).
    (nextField = '' )
    ifTrue:
      [ Date _error: #rtErrBadFormat args: { aStream }].
    (anArray at: 5) == 1
    ifTrue:
      [ monthInt := Integer fromCompleteString: nextField ]
    ifFalse:
      [ monthInt := self _getMonthFrom: nextField ].
    (monthInt < 1 or:[monthInt > 12])
    ifTrue:
      [ Date _error: #rtErrBadFormat args: { aStream } ]
  ] .
  "parse year"
  [ :delim | | nextField |
    nextField := parseField value: delim value: (delim == $  ).
    yearInt := Integer fromCompleteString: nextField.
    (anArray at: 6) == 2
    ifTrue:
      [ (yearInt > 99)
        ifFalse: [yearInt := yearInt + ((Date today year) // 100 * 100) ]
      ]
  ]
 }.

self _checkReadStream: aStream forClass: CharacterCollection.

Date _checkFormat: anArray.

dateDelim := anArray at: 4.

"parse the date, with day, month, year in the specified format order"
(parseBlocks at:( anArray at: 1)) value: dateDelim .
(parseBlocks at:( anArray at: 2)) value: dateDelim .
(parseBlocks at:( anArray at: 3)) value: $  .

result := self newDay: dayInt monthNumber: monthInt year: yearInt .
^ result
%

category: 'Instance Creation'
classmethod: Date
fromString: aString

"Creates and returns an instance of the receiver from the String aString.
 The String expresses the date in the default format (DD/MM/YYYY).
 Generates an error if the String does not conform to the format."

^ self fromString: aString usingFormat: #(1 2 3 $/ 1 1)
%

category: 'Instance Creation'
classmethod: Date
fromString: aString usingFormat: anArray

"Creates and returns an instance of the receiver from the String aString.
 The String expresses the date in the format specified by anArray.  The
 expression is terminated either by a space Character or by the end of the
 String.  Generates an error if the String does not conform to the format,
 or if anArray contains an incorrect formatting specification.

 See the class documentation of Date for a complete description of the
 String-formatting specification Array.

 If the month format (5th element) indicates either an abbreviation (2) or an
 entire name (3), then this method tries to determine the month by decoding a
 character substring.  That substring may include any number of Characters, but
 must exactly match a legal month name (or the initial portion of that month
 name).  If the substring matches more than one month, the first month matched
 is used (the search begins with January)."

| s result |

s := ReadStream on: aString.
result := self fromStream: s usingFormat: anArray.
[ s atEnd ]
whileFalse:
  [ (s next == $  )
    ifFalse:
      [ self _errIncorrectFormat: aString ]
  ].
^ result
%

category: 'Inquiries'
classmethod: Date
indexOfMonthName: monthName

"Return an Integer corresponding to the month with the name monthName.
 If monthName is not a recognized name, return 0, otherwise return an
 Integer between 1 and 12 inclusive, where 1 signifies January."

| monthArray |
monthArray := MonthNames value.
1 to: 12 do: [:index | (monthArray at: index) = monthName ifTrue: [^ index]] .
^ 0
%

category: 'Inquiries'
classmethod: Date
isLeap: year

"Returns true if year is a leap year; false otherwise."

"a year is a leap year if: (it is evenly divisible by 4 and it is not a
 century year) or (it is a century year and evenly divisible by 400)"

((year \\ 100) == 0)
   ifTrue: [^ ((year \\ 400) == 0)].
^ ((year \\ 4) == 0)
%

category: 'Storing and Loading'
classmethod: Date
loadFrom: passiveObj

"Creates and returns an active instance of the receiver from the passive form
 of the object."
| inst yr dy |
yr := passiveObj readObject .
yr  _isSmallInteger ifFalse:[ Error signal:'unexpected ', yr class name ].
dy := passiveObj readObject .
dy  _isSmallInteger ifFalse:[ Error signal:'unexpected ', dy class name ].
self == SmallDate ifTrue:[
  inst := self newDay: dy year: yr .
  inst isSpecial ifFalse:[ Error signal:'a SmallDate should be special'].
] ifFalse:[
  inst := self _basicNew _year: yr day: dy .
  passiveObj hasRead: inst .
].
^ inst
%

category: 'Instance Creation'
classmethod: Date
migrateNew

"Override default migrateNew behavior with #_basicNew."

^ self _basicNew
%

category: 'Inquiries'
classmethod: Date
nameOfMonth: anIndex

"Returns a String that gives the name, in the user's native language, of the
 month of the year whose numeric index is anIndex.  The index is a number
 between 1 and 12 inclusive, where 1 signifies January."

^ (MonthNames value) at: anIndex.
%

category: 'Instance Creation'
classmethod: Date
new

"Disallowed.  To create a new Date, use another instance creation method."

self shouldNotImplement: #new
%

category: 'Instance Creation'
classmethod: Date
new: anInteger

"Disallowed.  To create a new Date, use another instance creation method."

self shouldNotImplement: #new:
%

category: 'Instance Creation'
classmethod: Date
newDay: dayInt month: monthString year: yearInt

"Creates and returns an instance of the receiver from the specified values.
 Generates an error if any of the values are out of range."

 | monthInt |

 monthInt := self _getMonthFrom: monthString .
 (monthInt < 1 or: [ monthInt > 12 ]) ifTrue:[
   Date _error: #rtErrBadFormat args: { { dayInt . monthString . yearInt } }.
 ].
^ self _newDay: dayInt monthNumber: monthInt year: yearInt .
%

category: 'Instance Creation'
classmethod: Date
newDay: day monthNumber: month year: year

(month < 1 or:[ month > 12]) ifTrue: [
  ArgumentError signal: 'Incorrect specified month: ' , month asString
].
(day < 1 or:[ day > (self numberOfDaysIn: month year: year)]) ifTrue: [
  ArgumentError signal: 'Incorrect specified day: ', day asString.
].
^ self _newDay: day monthNumber: month year: year
%

category: 'Instance Creation'
classmethod: Date
newDay: day year: year

"Creates and returns an instance of the receiver from the specified values.
 Generates an error if any of the values are out of range."

^ self _newDay: day monthNumber: 1 year: year.
%

category: 'Instance Creation'
classmethod: Date
numberOfDaysIn: month year: aYear

| x |
x := #( 31 nil 31 30 31 30 31 31 30 31 30 31 ) atOrNil: month .
x ifNotNil:[ ^ x ] .
month == 2 ifFalse:[ ArgumentError signal: 'Incorrect specified month: ' , month asString ].
(((aYear \\ 100) == 0)
   ifTrue: [ ((aYear \\ 400) == 0)]
   ifFalse: [ ((aYear \\ 4) == 0) ])
  ifTrue: [^ 29].
^ 28
%

category: 'Instance Creation'
classmethod: Date
today

  "Returns an instance of the receiver representing the current Date, taking into
   consideration the current GemStone TimeZone.

   See Date class >>_today for 3.2.x and earlier implementation that gets date
   directly from the OS and does not reflect repository TimeZone."

   | parts |
   parts := DateAndTime now asFloatParts.
     "parts is { year. dayOfYear. monthIndex. dayOfMonth. hour. minute. second } "
   ^self newDay: (parts at: 4) monthNumber: (parts at: 3) year: (parts at: 1).
%

category: 'Instance Creation'
classmethod: Date
_checkFormat: anArray

"Private.  Verifies that anArray is a valid string-formatting specification for
 the receiver.  Generates an error if it is not."

| v |

anArray _validateClass: Array.
(anArray size < 6)
  ifTrue:[ Date _error: #rtErrBadFormatSpec args: { anArray } ].

"Check for a combination of the digits 1, 2, and 3"
((anArray at: 1) + (anArray at: 2) + (anArray at: 3) == 6 and:
        [(anArray at: 1) * (anArray at: 2) * (anArray at: 3) == 6])
  ifFalse:[ Date _error: #rtErrBadFormatSpec args: { anArray } ].

(anArray at: 4) _validateClass: AbstractCharacter.

((v := anArray at: 5) == 1 or: [v == 2 or: [v == 3]])
  ifFalse:[ Date _error: #rtErrBadFormatSpec args: { anArray } ].

((anArray at: 6) == 1 or: [(anArray at: 6) == 2])
  ifFalse:[ Date _error: #rtErrBadFormatSpec args: { anArray } ].
%

category: 'Instance Creation'
classmethod: Date
_getMonthFrom: aCharCollection

"Private.  Returns the SmallInteger that corresponds to the month of the year
 that matches aCharCollection.

 The argument may include any number of characters, but must exactly match a
 legal month name (or the initial portion of that month name).  If the argument
 matches more than one month, the first month matched is used.  The search
 begins with January."

| whichMonth monthArray argSize matchMonth |

monthArray := MonthNames value.
matchMonth:= [:monthStr | | strSize match i |
   i:= 1.
   match:= false.
   strSize:= monthStr size.
   [ ((i <= argSize) and:[i <= strSize]) and:
     [match:= (aCharCollection at: i) isEquivalent: (monthStr at: i)]]
   whileTrue: [
      i:= i + 1.
   ].
   match
].

   argSize:= aCharCollection size.
   whichMonth:= 1.
   [ (whichMonth <= 12) and:
     [(matchMonth value: (monthArray at: whichMonth)) not]
   ]
   whileTrue:
      [whichMonth := whichMonth + 1].

   (whichMonth <= 12)
      ifTrue: [ ^whichMonth].
   ^ 0
%

category: 'Instance Creation'
classmethod: Date
_newDay: day monthNumber: month year: year

"Creates and returns an instance of SmallDate if possible, otherwise
 returns an instance of the receiver from the specified values.
 Generates an error if any of the values are out of range.
 If self == Date or self == SmallDate,  result is a SmallDate if possible."

<primitive: 316>

day _validateClass: SmallInteger .
month _validateClass: SmallInteger .
year _validateClass: SmallInteger .

^ self _primitiveFailed: #_newDay:monthNumber:year:
       args: { day . month . year }
%

category: 'Instance Creation'
classmethod: Date
_today

"Creates and returns an instance of the receiver from the system calendar
 on the machine that is running the Gem process, which is assumed to
 represent the current date. 
 If self == Date or self == SmallDate,  result is a SmallDate if possible."

<primitive: 315>
^ self _primitiveFailed: #today
%

!		Instance methods for 'Date'

category: 'Comparing'
method: Date
< aDate

"Returns true if the receiver represents a date before that of the argument,
 and false if it doesn't.  Generates an error if the argument is not
 a Date."

| argYear myYear |

argYear := aDate year.
(myYear := self year) == argYear
  ifTrue: [ ^ self dayOfYear < aDate dayOfYear ]
  ifFalse: [ ^ myYear < argYear ].
%

category: 'Comparing'
method: Date
= aDate

"Returns true if the receiver represents the same date as that of the
 argument, and false if it doesn't."

(self == aDate) ifTrue: [ ^ true ].
(aDate isKindOf: Date ) ifFalse: [ ^false ].
^ (self year == aDate year) and: [self dayOfYear == aDate dayOfYear]
%

category: 'Comparing'
method: Date
> aDate

"Returns true if the receiver represents a date after that of the argument,
 and false if it doesn't.  Generates an error if the argument is not
 a Date."

| argYear myYear |

argYear := aDate year.
(myYear := self year) == argYear
  ifTrue: [ ^ self dayOfYear > aDate dayOfYear ]
  ifFalse: [^  myYear > argYear ].
%

category: 'Arithmetic'
method: Date
addDays: anInteger

"Returns a Date that describes a date anInteger days later than that
 of the receiver."

^ (self class) newDay: (self dayOfYear + anInteger) year: self year.
%

category: 'Arithmetic'
method: Date
addMonths: anInteger

"Returns a Date that describes a date anInteger months later than that of the
 receiver.

 This method attempts to keep the day of the month the same.  If the
 new month has fewer days than the receiver's original month, then it
 truncates to the last day of the new month."

| yr month day newYear newMonth newDay newDate generatedDay |

yr := self year.
month := self month.
day := self day.

newMonth := month + anInteger.
newYear := yr + ((newMonth - 1) // 12).
newMonth := (newMonth - 1) \\ 12 + 1.
newDate := self class _newDay: day monthNumber: newMonth year: newYear.
generatedDay := newDate day.
(generatedDay ~= day)
  ifTrue: [
    newDay := newDate _daysInMonth: newMonth.
    newDate := self class _newDay: newDay monthNumber: newMonth year: newYear
    ].
^ newDate.
%

category: 'Arithmetic'
method: Date
addYears: anInteger

"Returns a Date that describes a date anInteger years later than that of the
 receiver."

| yr month day newYear newDay newDate generatedDay |

yr := self year.
month := self month.
day := self day.

newYear := yr + anInteger.
newDate := self class _newDay: day monthNumber: month year: newYear.
generatedDay := newDate day.
(generatedDay ~= day)
  ifTrue: [
    newDay := newDate _daysInMonth: month.
    newDate := self class _newDay: newDay monthNumber: month year: newYear
    ].
^ newDate.
%

category: 'Converting'
method: Date
asDays

"Returns an Integer that represents the receiver in units of days since
 January 1, 1901."

| numYears numDays |

numYears := self year - 1901.
numDays := (numYears * 365) + (numYears // 4) +
           ((numYears + 300) // 400) - (numYears // 100) + self dayOfYear - 1.
^ numDays.
%

category: 'Formatting'
method: Date
asString

"Returns a String that expresses the receiver in the default format
 (DD/MM/YYYY)."

| t result |

t := self _yearMonthDay.
result := (t at: 3) _digitsAsString .
result addAll: '/';
  addAll: (t at: 2) _digitsAsString;
  addAll: '/';
  addAll: (t at: 1) _digitsAsString.
^ result
%

category: 'Formatting'
method: Date
asStringUsingFormat: anArray

"Returns a String that expresses the receiver in the format defined by anArray.
 Generates an error if anArray contains an incorrect formatting specification.

 See the class documentation of Date for a complete description of the
 String-formatting specification Array."

| t dateSeparator monthName aString day yearNumber |

t := self _yearMonthDay.

Date _checkFormat: anArray.

dateSeparator := (anArray at: 4) asString.

((anArray at: 5) == 2) "get the month name according to the format"
   ifTrue: [monthName := self _monthAbbrev: (t at: 2)]
   ifFalse: [((anArray at: 5) == 3) "month as number is default"
      ifTrue: [monthName := Date nameOfMonth: (t at: 2)]
      ifFalse: [monthName := (t at: 2) _digitsAsString]].

((anArray at: 6) == 2)
   ifTrue: [yearNumber := ((t at: 1) \\ 100) _digitsAsString]
   ifFalse: [yearNumber := (t at: 1) asString].  "YYYY is default"

day := (t at:3) _digitsAsString.
((anArray at: 1) == 2) "month first"
   ifTrue: [aString := monthName , dateSeparator]
   ifFalse: [((anArray at: 1) == 3) "yearNumber first"
      ifTrue: [aString := yearNumber , dateSeparator]
      ifFalse: [aString := day , dateSeparator]].  "day first is default"

((anArray at: 2) == 1) "day second"
   ifTrue: [aString addAll: day; addAll: dateSeparator] "yearNumber second"
   ifFalse: [((anArray at: 2) == 3) "month second is default"
      ifTrue: [aString addAll: yearNumber; addAll: dateSeparator]
      ifFalse: [aString addAll: monthName; addAll: dateSeparator]].

((anArray at: 3) == 1) "day third"
   ifTrue: [aString addAll: day]
   ifFalse: [((anArray at: 3) == 2) "month third"
      ifTrue: [aString addAll: monthName]
      ifFalse: [aString addAll: yearNumber]].  "yearNumber third is default"

^ aString
%

category: 'Accessing'
method: Date
at: anIndex put: aValue

"Disallowed.  You may not change the value of a Date."

self shouldNotImplement: #at:put:
%

category: 'Accessing'
method: Date
day

"Returns a SmallInteger that gives the day of the month described by the
 receiver."

^  (self _yearMonthDay) at: 3
%

category: 'Accessing'
method: Date
dayOfMonth

"Returns a SmallInteger that gives the day of the month described by the
 receiver."

^ self day.
%

category: 'Accessing'
method: Date
dayOfWeek

"Returns a SmallInteger that gives the numeric index of the day of the week
 described by the receiver.  The index is a number between 1 and 7 inclusive,
 where 1 signifies Sunday."

^ self julianDay - 2299295 - 1 \\ 7 + 1
%

category: 'Accessing'
method: Date
dayOfYear

"Returns a SmallInteger that gives the day of the year described by the
 receiver."

^ dayOfYear
%

category: 'Accessing'
method: Date
daysInMonth

"Returns a SmallInteger that gives the number of days in the month
 described by the receiver."

^ self _daysInMonth: self month

%

category: 'Accessing'
method: Date
daysInYear

"Returns a SmallInteger that gives the number of days in the year
 described by the receiver."

(self leap) ifTrue: [^ 366].
^ 365
%

category: 'Comparing'
method: Date
hash

"Returns an Integer hash code for the receiver."

^ ((self year hash) bitShift: -1) bitXor: (self dayOfYear hash)
%

category: 'Accessing'
method: Date
julianDay

"Returns the Julian Day of the receiver, a SmallInteger that gives the number of
 days since January 1, 4713 B.C., as defined in Communications of the ACM,
 algorithm #199."

<primitive: 46 >

^ self _primitiveFailed: #julianDay
%

category: 'Accessing'
method: Date
leap

"Returns true if the receiver describes a leap year and false if it does not."

| yr |
  "a year is a leap year if: (it is evenly divisible by 4 and it is not a
   century year) or (it is a century year and evenly divisible by 400)"

yr := self year .
((yr \\ 100) == 0)
   ifTrue: [^ ((yr \\ 400) == 0)].
^ ((yr \\ 4) == 0)
%

category: 'Accessing'
method: Date
month

"Returns a SmallInteger that gives the numeric index of the month of the year
 described by the receiver.  The index is a number between 1 and 12 inclusive,
 where 1 signifies January."

^ self monthIndex.
%

category: 'Accessing'
method: Date
monthIndex

"Returns a SmallInteger that gives the numeric index of the month of the year
 described by the receiver.  The index is a number between 1 and 12 inclusive,
 where 1 signifies January."

^ (self _yearMonthDay) at: 2
%

category: 'Accessing'
method: Date
monthName

"Returns a String that gives the name of the month of the year described by the
 receiver, in the user's native language."

^ MonthNames value at: self month
%

category: 'Accessing'
method: Date
monthOfYear

"Returns a SmallInteger that gives the numeric index of the month of the year
 described by the receiver.  The index is a number between 1 and 12 inclusive,
 where 1 signifies January."

^ self month.
%

category: 'Inquiries'
method: Date
next: dayName
"Returns the next date whose weekday name is dayName"

^ self addDays:
   ((self class dayOfWeek: dayName) - 1 - self weekdayIndex - 1) \\ 7 + 1
%

category: 'Inquiries'
method: Date
nextMonth: monthName
"Returns the next Date which describes a date later than the receiver and has
 month named monthName."

| tmp index |
index := self class indexOfMonthName: monthName asString .
^ self month == index
    ifTrue: [self addYears: 1]
   ifFalse: [tmp := self.
             [tmp month == index]
                whileFalse: [tmp := tmp addMonths: 1].
              tmp]
%

category: 'Inquiries'
method: Date
previous: dayName

"Returns the previous date whose weekday name is dayName."

^ self subtractDays: self dayOfWeek -
       (self class dayOfWeek: dayName) -
       1 \\ 7 + 1.
%

category: 'Inquiries'
method: Date
previousMonth: monthName
"Returns the next Date which describes a date earlier than the receiver and has
 month named monthName."

| tmp index |
index := self class indexOfMonthName: (monthName asString).
^ self month == index
    ifTrue: [self subtractYears: 1]
    ifFalse: [tmp := self.
              [tmp month == index]
                 whileFalse: [tmp := tmp subtractMonths: 1].
               tmp]
%

category: 'Formatting'
method: Date
printOn: aStream

"Puts a displayable representation of the receiver on aStream."

aStream nextPutAll: self asString .
%

category: 'Accessing'
method: Date
size: anInteger

"Disallowed.  You may not change the size of a Date."

self shouldNotImplement: #size:
%

category: 'Arithmetic'
method: Date
subtractDate: aDate

"Returns a positive Integer that counts the number of times midnight occurs
 between the times described by the receiver and aDate."

^ (self asDays - aDate asDays) abs
%

category: 'Arithmetic'
method: Date
subtractDays: anInteger

"Returns a Date that describes a date anInteger days earlier than that
 of the receiver."

^ self class newDay: self dayOfYear - anInteger year: self year
%

category: 'Arithmetic'
method: Date
subtractMonths: anInteger

"Returns a Date that describes a date anInteger months earlier than that of the
 receiver.

 This method attempts to keep the day of the month the same.  If the
 new month has fewer days than the receiver's original month, then it
 truncates to the last day of the new month."

^ self addMonths: (anInteger negated).
%

category: 'Arithmetic'
method: Date
subtractYears: anInteger

"Returns a Date that describes a date anInteger years earlier than that of the
 receiver."

^ self addYears: (anInteger negated).
%

category: 'Formatting'
method: Date
USDateFormat

"Returns a String that expresses the date of the receiver. The date is in
 United States format, month first (MM/DD/YY)."

^self asStringUsingFormat: #(2 1 3 $/ 1 2 $: false )
%

category: 'Inquiries'
method: Date
weekdayIndex
"Return a SmallInteger between 1 and 7 representing the day of the week of the
 receiver where Monday is 1 and Sunday is 7."

^ (self asDays + 1) \\ 7 + 1
%

category: 'Accessing'
method: Date
weekDayName

"Returns a String that gives the name of the day of the week described by the
 receiver, in the user's native language."

^ WeekDayNames value at: (self dayOfWeek)
%

category: 'Storing and Loading'
method: Date
writeTo: passiveObj

"Writes the passive form of the receiver into passiveObj."

passiveObj writeClass: self class.
self year writeTo: passiveObj.
self dayOfYear writeTo: passiveObj.
passiveObj space
%

category: 'Accessing'
method: Date
year

"Returns a SmallInteger that gives the year described by the receiver."

^ year
%

category: 'New Indexing Comparison'
method: Date
_classSortOrdinal

^ 50
%

category: 'Accessing'
method: Date
_daysInMonth: month

"Returns a SmallInteger that gives the number of days in the month
 specified by the Integer month."

((month == 1) or: [(month == 3) or: [(month == 5) or: [(month == 7) or:
   [(month == 8) or: [(month == 10) or: [(month == 12)]]]]]])
   ifTrue: [^ 31].
((month == 4) or: [(month == 6) or: [(month == 9) or: [(month == 11)]]])
   ifTrue: [^ 30].
(self leap)
   ifTrue: [^ 29].
^ 28
%

category: 'Formatting'
method: Date
_monthAbbrev: anIndex

"Private.  Returns a three-letter String that gives the abbreviation, in the
 user's native language, of the name of the month whose numeric index is
 anIndex.  The index is a number between 1 and 12 inclusive, where 1 signifies
 January."

|theMonth itsAbbrev|

theMonth := Date nameOfMonth: anIndex.  "get its full name"
itsAbbrev := String new.
1 to: 3 do: "take the first three letters"
   [:aChar | itsAbbrev := itsAbbrev , (theMonth at: aChar)].
^ itsAbbrev
%

category: 'Private'
method: Date
_year: yr day: dy
  "used by PassiveObject "
   year := yr .
   dayOfYear := dy
%

category: 'Accessing'
method: Date
_yearMonthDay

"Private.  Returns a three-element Array of SmallIntegers containing the year,
 the index of the month, and the day of the month described by the receiver."

<primitive: 239 >

^ self _primitiveFailed: #_yearMonthDay
%

! Class extensions for 'DateTime'

!		Class methods for 'DateTime'

removeallmethods DateTime
removeallclassmethods DateTime

category: 'Instance Creation'
classmethod: DateTime
basicNew

"Returns a variant instance"

^ (self newGmtWithYear: 1901 dayOfYear: 0 seconds: 0 timeZone: TimeZone current) copy
%

category: 'Instance Creation'
classmethod: DateTime
fromStream: aStream

"Creates and returns an instance of the receiver by reading a String from
 aStream.  The String expresses local time in the default format
 (DD/MM/YYYY HH:MM:SS).  Generates an error if the String does not conform to
 the format."

^ self fromStream: aStream usingFormat: #(1 2 3 $/ 1 1 $: true true false)
%

category: 'Instance Creation'
classmethod: DateTime
fromStream: aStream usingFormat: anArray

"Creates and returns an instance of the receiver by reading a String from
 aStream.  The String expresses local time in the format specified by
 anArray.  The expression is terminated either by a space Character or by the
 end of the Stream.  Generates an error if the String does not conform to the
 format, or if anArray contains an incorrect formatting specification.

 See the class documentation of DateTime for a complete description of the
 String-formatting specification Array.

 If the month format (5th element) indicates either an abbreviation (2) or an
 entire name (3), then this method tries to determine the month by decoding a
 character substring.  That substring may include any number of Characters, but
 must exactly match a legal month name (or the initial portion of that month
 name).  If the substring matches more than one month, the first month matched
 is used (the search begins with January).

 If the specification indicates that seconds should not be included (9th
 element is false), and aString includes seconds, this method generates an
 error."

| aDateTime localTimeZone |

localTimeZone := TimeZone current.
aDateTime := (self fromStreamGmt: aStream usingFormat: anArray)
             subtractSeconds: (localTimeZone secondsFromGmt).
aDateTime isDst
  ifTrue:  [ ^ aDateTime subtractSeconds: (localTimeZone secondsForDst)]
  ifFalse: [ ^ aDateTime ].
%

category: 'Instance Creation'
classmethod: DateTime
fromStreamGmt: aStream

"Creates and returns an instance of the receiver by reading a String from
 aStream.  The String expresses Greenwich Mean Time in the default format
 (DD/MM/YYYY HH:MM:SS).  Generates an error if the String does not conform to
 the format."

^ self fromStreamGmt: aStream usingFormat: #(1 2 3 $/ 1 1 $: true true false)
%

category: 'Instance Creation'
classmethod: DateTime
fromStreamGmt: aStream usingFormat: anArray

"Creates and returns an instance of the receiver by reading a String from
 aStream.  The String expresses Greenwich Mean Time in the format specified by
 anArray.  The expression is terminated either by a space Character or by the
 end of the Stream.  Generates an error if the String does not conform to the
 format, or if anArray contains an incorrect formatting specification.

 See the class documentation of DateTime for a complete description of the
 String-formatting specification Array.

 If the month format (5th element) indicates either an abbreviation (2) or an
 entire name (3), then this method tries to determine the month by decoding a
 character substring.  That substring may include any number of Characters, but
 must exactly match a legal month name (or the initial portion of that month
 name).  If the substring matches more than one month, the first month matched
 is used (the search begins with January).

 If the specification indicates that seconds should not be included (9th
 element is false), and aString includes seconds, this method generates an
 error."

| dayInt monthInt yearInt hourInt minInt secInt timeDelim dateDelim ampm
  ampmPresent secondsPresent timePresent result parseDay
  blkNo parseMonth parseYear parseField |

"This block returns a string up from the input stream up to the specified
 delimiter.  If also allows an end-of-file if that parameter is set true.
 It then skips over the delimiter if it is found.
"
parseField := [ :delim :allowEof | | str |
                str := aStream contents class new.
                [ ((aStream peek isEquivalent: delim) == false ) and:[ aStream atEnd ==false ] ]
                whileTrue:
                  [ str add: aStream next ].
                (aStream atEnd)
                ifTrue:
                  [ allowEof
                    ifFalse:
                      [ self _error: #rtErrBadFormat args: { aStream } ]
                  ]
                ifFalse:
                  [ aStream next "skip over delimiter" ].
                str
             ].

parseDay:= "parse day"
  [ :delim | | nextField |
    nextField := parseField value: delim value: (delim == $  ).
    dayInt := Integer fromCompleteString: nextField
  ].
parseMonth:= "parse month"
  [ :delim | | nextField |
    nextField := parseField value: delim value: (delim == $  ).
    (nextField =  '' )
    ifTrue:
      [ self _error: #rtErrBadFormat args: { aStream }].
    (anArray at: 5) == 1
    ifTrue:
      [ monthInt := Integer fromCompleteString: nextField ]
    ifFalse:
      [ monthInt := Date _getMonthFrom: nextField ].
    (monthInt < 1 or:[ monthInt > 12 ] )
    ifTrue:
      [ self _error: #rtErrBadFormat args: { aStream } ]
  ].
parseYear := "parse year"
  [ :delim | | nextField |
    nextField := parseField value: delim value: (delim == $  ).
    yearInt := Integer fromCompleteString: nextField.
    (anArray at: 6) == 2
    ifTrue:
      [ (yearInt > 99)
        ifFalse: [yearInt := yearInt + ((self now yearGmt) // 100 * 100) ]
      ]
  ].

self _checkReadStream: aStream forClass: CharacterCollection.

self _checkFormat: anArray.

dateDelim := anArray at: 4.
timeDelim := anArray at: 7.
timePresent := anArray at: 8.

"parse the date, with day, month, year in the specified format order"
true ifTrue:[ | delim |
  delim:= {  dateDelim . dateDelim .  $ }.
  1 to: 3 do: [:i | blkNo:= anArray at: i.
            (blkNo == 1) ifTrue: [parseDay value: (delim at: i)].
            (blkNo == 2) ifTrue: [parseMonth value: (delim at: i)].
            (blkNo == 3) ifTrue: [parseYear value: (delim at: i)]
  ].
].
timePresent ifTrue:[ "read the time"
    secondsPresent := anArray at: 9.
    ampmPresent := anArray at: 10.
    hourInt := Integer fromCompleteString: (parseField value: timeDelim value: false).
    minInt := Integer fromCompleteString:
                     (parseField value: (secondsPresent
                                         ifTrue: [timeDelim]
                                         ifFalse: [$ ])
                                 value: (secondsPresent not and:[ampmPresent not]) ).
    secondsPresent ifTrue: [
      secInt := Integer fromCompleteString: (parseField value: $  value: ampmPresent not)]
    ifFalse:[ secInt := 0 ].

    ampmPresent ifTrue: [
        hourInt < 0 ifTrue: [
          self _error: #rtErrBadFormat args: { aStream }].
        hourInt > 12 ifTrue: [
          self _error: #rtErrBadFormat args: { aStream }].
	(ampm := String new) add: (aStream next); add: aStream next.
        (ampm isEquivalent: 'PM') ifTrue: [
	  hourInt := hourInt + 12.
	  hourInt == 24 ifTrue: [
            hourInt := 12].
          ]
        ifFalse: [
	  (ampm isEquivalent: 'AM') ifFalse: [
            self _error: #rtErrBadFormat args: { aStream } ].
	  hourInt == 12 ifTrue: [
            hourInt := 0].
          ].
      ]
] ifFalse:[ "ignore the time"
    hourInt := 0.
    minInt := 0.
    secInt := 0
].
result := self newGmtWithYear: yearInt
                month: monthInt
                day: dayInt
                hours: hourInt
                minutes: minInt
                seconds: secInt.

"This is an easy way to test that all of the values specified were in
 in range.  If any of them were not, the result will be different
 than what we specified."

(result asPartsGmt = { yearInt . monthInt . dayInt . hourInt . minInt . secInt })
ifFalse:
  [ self _error: #rtErrBadFormat args: { aStream } ].

^ result
%

category: 'Instance Creation'
classmethod: DateTime
fromString: aString

"Creates and returns an instance of the receiver from the String aString.
 The String expresses local time in the default format
 (DD/MM/YYYY HH:MM:SS).  Generates an error if the String does not conform to
 the format."

^ self fromString: aString usingFormat: #(1 2 3 $/ 1 1 $: true true false)
%

category: 'Instance Creation'
classmethod: DateTime
fromString: aString usingFormat: anArray

"Creates and returns an instance of the receiver from the String aString.
 The String expresses local time in the format specified by anArray.
 The expression is terminated either by a space Character or by the end of the
 String.  Generates an error if the String does not conform to the format,
 or if anArray contains an incorrect formatting specification.

 See the class documentation of DateTime for a complete description of the
 String-formatting specification Array.

 If the month format (5th element) indicates either an abbreviation (2) or an
 entire name (3), then this method tries to determine the month by decoding a
 character substring.  That substring may include any number of Characters, but
 must exactly match a legal month name (or the initial portion of that month
 name).  If the substring matches more than one month, the first month matched
 is used (the search begins with January).

 If the specification indicates that seconds should not be included (9th
 element is false), and aString includes seconds, this method generates an
 error."

| aDateTime localTimeZone |

localTimeZone := TimeZone current.
aDateTime := (self fromStringGmt: aString usingFormat: anArray)
             subtractSeconds: (localTimeZone secondsFromGmt).
aDateTime isDst
  ifTrue:  [ ^ aDateTime subtractSeconds: (localTimeZone secondsForDst)]
  ifFalse: [ ^ aDateTime ].
%

category: 'Instance Creation'
classmethod: DateTime
fromStringGmt: aString

"Creates and returns an instance of the receiver from the String aString.
 The String expresses Greenwich Mean Time in the default format
 (DD/MM/YYYY HH:MM:SS).  Generates an error if the String does not conform to
 the format."

^ self fromStringGmt: aString usingFormat: #(1 2 3 $/ 1 1 $: true true false)
%

category: 'Instance Creation'
classmethod: DateTime
fromStringGmt: aString usingFormat: anArray

"Creates and returns an instance of the receiver from the String aString.
 The String expresses Greenwich Mean Time in the format specified by anArray.
 The expression is terminated either by a space Character or by the end of the
 String.  Generates an error if the String does not conform to the format,
 or if anArray contains an incorrect formatting specification.

 See the class documentation of DateTime for a complete description of the
 String-formatting specification Array.

 If the month format (5th element) indicates either an abbreviation (2) or an
 entire name (3), then this method tries to determine the month by decoding a
 character substring.  That substring may include any number of Characters, but
 must exactly match a legal month name (or the initial portion of that month
 name).  If the substring matches more than one month, the first month matched
 is used (the search begins with January).

 If the specification indicates that seconds should not be included (9th
 element is false), and aString includes seconds, this method generates an
 error."

| s result |

s := ReadStream on: aString.
result := self fromStreamGmt: s usingFormat: anArray.
[ s atEnd ]
whileFalse:
  [ (s next == $  )
    ifFalse:
      [ self _errIncorrectFormat: aString ]
  ].
^ result
%

category: 'Storing and Loading'
classmethod: DateTime
loadFrom: passiveObj

"Creates and returns an active instance of the receiver from the passive form
 of the object, which expresses itself in Greenwich Mean Time."

| inst year dayCount ms zone marker |
marker := passiveObj objectPositionMarker.
passiveObj version >= 510
  ifTrue: [
     passiveObj readNamedIV.
     year := passiveObj ivValue.
     passiveObj readNamedIV.
     dayCount := passiveObj ivValue.
     passiveObj readNamedIV.
     ms := passiveObj ivValue.
     passiveObj readNamedIV.
     zone := passiveObj ivValue.
     passiveObj skipNamedInstVars.

     inst := self _newGmtWithYear: year dayOfYear: dayCount milliseconds: ms
               timeZone: zone .
    ]
  ifFalse: [
    inst := self _newGmtWithYear: passiveObj readObject
               month: passiveObj readObject
               day: passiveObj readObject
               seconds: passiveObj readObject
               timeZone: TimeZone current.
    passiveObj version >= 500
      ifFalse: [ "convert 4.1 local time to GMT"
        inst := inst addSeconds: Time gmtOffsetSeconds
        ].
    ].
passiveObj hasRead: inst marker: marker .
^inst.
%

category: 'Instance Creation'
classmethod: DateTime
migrateNew

"Override default migrateNew behavior with #_basicNew."

^ self _basicNew
%

category: 'General Inquiries'
classmethod: DateTime
nameOfMonth: anIndex

"Returns a String that gives the name, in the user's native language, of the
 month of the year whose numeric index is anIndex.  The index is a number
 between 1 and 12 inclusive, where 1 signifies January."

^ (MonthNames value) at: anIndex.
%

category: 'Instance Creation'
classmethod: DateTime
new

"Disallowed. To create a new DateTime, use another instance creation method."

self shouldNotImplement: #new
%

category: 'Instance Creation'
classmethod: DateTime
new: anInteger

"Disallowed. To create a new DateTime, use another instance creation method."

self shouldNotImplement: #new:
%

category: 'Private'
classmethod: DateTime
newGmtWithDate: aDate seconds: secsGmt

"Creates and returns an instance of the receiver from the specified values,
 in the current time zone."

^ self newGmtWithDate: aDate seconds: secsGmt timeZone: (TimeZone current).
%

category: 'Private'
classmethod: DateTime
newGmtWithDate: aDate seconds: secsGmt timeZone: aTimeZone

"Creates and returns an instance of the receiver from the specified values,
 in the time zone specified."

^ self newGmtWithYear: (aDate year) dayOfYear: (aDate dayOfYear)
  seconds: secsGmt  timeZone: aTimeZone.
%

category: 'Instance Creation'
classmethod: DateTime
newGmtWithDate: aDate time: aTime timeZone: aTimeZone

"Creates and returns an instance of the receiver from the specified values,
 in the time zone specified."

^ self newGmtWithYear: (aDate year) dayOfYear: (aDate dayOfYear)
  seconds: (aTime asSeconds) timeZone: aTimeZone.
%

category: 'Instance Creation'
classmethod: DateTime
newGmtWithYear: year dayOfYear: dayCount milliseconds: ms

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

^ self newGmtWithYear: year dayOfYear: dayCount milliseconds: ms
  timeZone: (TimeZone current).
%

category: 'Instance Creation'
classmethod: DateTime
newGmtWithYear: year dayOfYear: dayCount milliseconds: ms timeZone: aTimeZone

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

<primitive: 314>

aTimeZone _validateClass: TimeZone.
(year _isSmallInteger and: [ (dayCount _isSmallInteger) ])
  ifFalse: [
    self _error: #rtErrDateTimeOutOfRange
    ].

(year >= 1901)
  ifFalse: [ ^ year _error: #rtErrBadDateTimeArgs ].
((dayCount <= 366) and: [ dayCount > 0 ])
  ifFalse: [ ^ dayCount _error: #rtErrBadDateTimeArgs ].

(ms _isInteger)
  ifTrue: [ | extraDays newMs |
    extraDays := ms // 86400000.
    newMs := ms \\ 86400000.
    ^ (self newGmtWithYear: year dayOfYear: dayCount milliseconds: newMs
      timeZone: aTimeZone) addDays: extraDays.
    ]
  ifFalse: [ ^ ms _error:  #rtErrBadDateTimeArgs ].

^ self _primitiveFailed: #newGmtWithYear:dayOfYear:milliseconds:timeZone:
       args: { year . dayCount . ms . aTimeZone }
%

category: 'Instance Creation'
classmethod: DateTime
newGmtWithYear: year dayOfYear: dayCount seconds: seconds

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

^ self newGmtWithYear: year dayOfYear: dayCount seconds: seconds
       timeZone: (TimeZone current).
%

category: 'Instance Creation'
classmethod: DateTime
newGmtWithYear: year dayOfYear: dayCount seconds: seconds timeZone: aTimeZone

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

^ self newGmtWithYear: year dayOfYear: dayCount
       milliseconds: (seconds * 1000) timeZone: aTimeZone.
%

category: 'Instance Creation'
classmethod: DateTime
newGmtWithYear: yearInt month: monthInt day: dayInt hours: hourInt
  minutes: minuteInt seconds: secondInt

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

^ self newGmtWithYear: yearInt month: monthInt day: dayInt hours: hourInt
       minutes: minuteInt seconds: secondInt timeZone: (TimeZone current).
%

category: 'Instance Creation'
classmethod: DateTime
newGmtWithYear: yearInt month: monthInt day: dayInt hours: hourInt
  minutes: minuteInt seconds: secondInt ms: msInt

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

| dt |
dt := self newGmtWithYear: yearInt
           month: monthInt
           day: dayInt
           hours: hourInt
           minutes: minuteInt
           seconds: secondInt
           timeZone: (TimeZone current) .
^dt addMs: msInt
%

category: 'Instance Creation'
classmethod: DateTime
newGmtWithYear: yearInt month: monthInt day: dayInt hours: hourInt
  minutes: minuteInt seconds: secondInt timeZone: aTimeZone

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

<primitive: 312>

   "Check the class of the arguments"
yearInt _validateClass: SmallInteger.
monthInt _validateClass: SmallInteger.
dayInt _validateClass: SmallInteger.
hourInt _validateClass: SmallInteger.
minuteInt _validateClass: SmallInteger.
secondInt _validateClass: Integer.
aTimeZone _validateClass: TimeZone.

(yearInt >= 1901)
   ifFalse: [yearInt _error: #rtErrBadDateTimeArgs].
((hourInt >= 0) and: [hourInt <= 23])
   ifFalse: [hourInt _error: #rtErrBadDateTimeArgs].
((minuteInt >= 0) and: [minuteInt <= 59])
   ifFalse: [minuteInt _error: #rtErrBadDateTimeArgs].
((secondInt >= 0) and: [secondInt <= 59])
   ifFalse: [secondInt _error: #rtErrBadDateTimeArgs].
((monthInt >=1) and: [monthInt <= 12])
   ifFalse: [monthInt _error: #rtErrBadDateTimeArgs].
dayInt _error: #rtErrBadDateTimeArgs.
^ self _primitiveFailed: #newGmtWithYear:month:day:hours:minutes:seconds:timeZone:
       args: { yearInt . monthInt . dayInt . hourInt .
               minuteInt . secondInt . aTimeZone }
%

category: 'Instance Creation'
classmethod: DateTime
newGmtWithYear: yearInt month: monthInt day: dayInt milliseconds: millisecondsInt

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

^ self _newGmtWithYear: yearInt
       month: monthInt
       day: dayInt
       milliseconds: millisecondsInt
       timeZone: TimeZone current
%

category: 'Private'
classmethod: DateTime
newWithDate: aDate milliseconds: ms

"Creates and returns an instance of the receiver from the specified values,
 in the current time zone."

^ self newWithDate: aDate milliseconds: ms timeZone: (TimeZone current).
%

category: 'Private'
classmethod: DateTime
newWithDate: aDate milliseconds: ms timeZone: aTimeZone

"Creates and returns an instance of the receiver from the specified values,
 in the time zone specified."

| aDateTime |

aDateTime := (self newGmtWithYear: (aDate year) dayOfYear: (aDate dayOfYear)
                   milliseconds: ms timeZone: aTimeZone  )
             subtractSeconds: (aTimeZone secondsFromGmt).

(aDateTime isDstIn: aTimeZone)
  ifTrue: [ ^ aDateTime subtractSeconds: (aTimeZone secondsForDst) ]
  ifFalse: [ ^ aDateTime ].
%

category: 'Private'
classmethod: DateTime
newWithDate: aDate seconds: secs

"Creates and returns an instance of the receiver from the specified values,
 in the current time zone."

^ self newWithDate: aDate seconds: secs timeZone: (TimeZone current).
%

category: 'Private'
classmethod: DateTime
newWithDate: aDate seconds: secs timeZone: aTimeZone

"Creates and returns an instance of the receiver from the specified values,
 in the time zone specified."

| aDateTime |

aDateTime := (self newGmtWithYear: (aDate year) dayOfYear: (aDate dayOfYear)
             seconds: secs timeZone: aTimeZone)
             subtractSeconds: (aTimeZone secondsFromGmt).

(aDateTime isDstIn: aTimeZone)
  ifTrue: [ ^ aDateTime subtractSeconds: (aTimeZone secondsForDst) ]
  ifFalse: [ ^ aDateTime ].
%

category: 'Instance Creation'
classmethod: DateTime
newWithDate: aDate time: aTime

"Creates and returns an instance of the receiver from the specified values,
 in the current time zone."

^ self newWithDate: aDate time: aTime timeZone: (TimeZone current).
%

category: 'Instance Creation'
classmethod: DateTime
newWithDate: aDate time: aTime timeZone: aTimeZone

"Creates and returns an instance of the receiver from the specified values,
 in the time zone specified."

| aDateTime |

aDateTime := (self newGmtWithYear: (aDate year) dayOfYear: (aDate dayOfYear)
             seconds: (aTime asSeconds) timeZone: aTimeZone)
             subtractSeconds: (aTimeZone secondsFromGmt).

(aDateTime isDstIn: aTimeZone)
  ifTrue: [ ^ aDateTime subtractSeconds: (aTimeZone secondsForDst) ]
  ifFalse: [ ^ aDateTime ].
%

category: 'Instance Creation'
classmethod: DateTime
newWithYear: year dayOfYear: dayCount seconds: seconds

"Creates and returns an instance of the receiver from the specified values,
 which express local time."

^ self newWithYear: year dayOfYear: dayCount seconds: seconds
       timeZone: (TimeZone current).
%

category: 'Instance Creation'
classmethod: DateTime
newWithYear: year dayOfYear: dayCount seconds: seconds timeZone: aTimeZone

"Creates and returns an instance of the receiver from the specified values,
 in the time zone specified."

| aDateTime |

aDateTime := (self newGmtWithYear: year dayOfYear: dayCount
              seconds: seconds timeZone: aTimeZone)
             subtractSeconds: (aTimeZone secondsFromGmt).

(aDateTime isDstIn: aTimeZone)
  ifTrue: [ ^ aDateTime subtractSeconds: (aTimeZone secondsForDst) ]
  ifFalse: [ ^ aDateTime ].
%

category: 'Instance Creation'
classmethod: DateTime
newWithYear: yearInt month: monthInt day: dayInt hours: hourInt
  minutes: minuteInt seconds: secondInt

"Creates and returns an instance of the receiver from the specified values,
 which express local time."

^ self newWithYear: yearInt month: monthInt day: dayInt hours: hourInt
       minutes: minuteInt seconds: secondInt timeZone: (TimeZone current).
%

category: 'Instance Creation'
classmethod: DateTime
newWithYear: yearInt month: monthInt day: dayInt hours: hourInt
  minutes: minuteInt seconds: secondInt timeZone: aTimeZone

"Creates and returns an instance of the receiver from the specified values,
 as represented in the time zone specified."

| aDateTime |

aDateTime := (self newGmtWithYear: yearInt month: monthInt day: dayInt
                   hours: hourInt minutes: minuteInt seconds: secondInt
                   timeZone: aTimeZone)
             subtractSeconds: (aTimeZone secondsFromGmt).

(aDateTime isDstIn: aTimeZone)
  ifTrue:  [ ^ aDateTime subtractSeconds: (aTimeZone secondsForDst)]
  ifFalse: [ ^ aDateTime ].
%

category: 'Instance Creation'
classmethod: DateTime
now: aTimeZone

"Creates and returns an instance of the receiver representing the current
 time in the time zone specified."

<primitive: 237>
^ self _primitiveFailed: #now
%

category: 'Deprecated'
classmethod: DateTime
today

"Creates and returns an instance of the receiver representing the 00:00 hrs,
 today, local time."

self deprecated: 'DateTime>>today deprecated v3.0, use Date today or DateTime now'.
^ self today: (TimeZone current).
%

category: 'Deprecated'
classmethod: DateTime
today: aTimeZone

"Creates and returns an instance of the receiver representing the 00:00 hrs,
 today, local time."

self deprecated: 'DateTime class>>today: deprecated v3.0, use #now:'.
^ self newWithDate: (Date today) seconds: 0 timeZone: aTimeZone.
%

category: 'Instance Creation'
classmethod: DateTime
whenStoneStarted

"Return a new instance of the receiver which corresponds to when the
 repository was started."

^ self now subtractSeconds: System secondsSinceStoneStarted
%

category: 'Private'
classmethod: DateTime
_checkFormat: anArray

"Private. Verifies that anArray is a valid string-formatting specification for
 the receiver.  Generates an error if it is not."

"anArray is a format Array as documented in DateTime>>asStringUsingFormat:"

| v doTime |

anArray _isArray ifFalse:[ anArray _validateClass: Array ].
(anArray size < 8 or:[ anArray size > 12])
  ifTrue:[ self _error: #rtErrBadFormatSpec args: { anArray } ].

"Check for a combination of the digits 1, 2, and 3"
((anArray at: 1) + (anArray at: 2) + (anArray at: 3) == 6 and:
        [(anArray at: 1) * (anArray at: 2) * (anArray at: 3) == 6])
  ifFalse:[ self _error: #rtErrBadFormatSpec args: { anArray } ].

v := anArray at: 4 .
v class ~~ Character ifTrue:[ v _validateClass: CharacterCollection ].

((v := anArray at: 5) == 1 or: [v == 2 or: [v == 3]])
  ifFalse:[ self _error: #rtErrBadFormatSpec args: { anArray } ].

((anArray at: 6) == 1 or: [(anArray at: 6) == 2])
  ifFalse:[ self _error: #rtErrBadFormatSpec args: { anArray } ].

(doTime := anArray at: 8) _validateClass: Boolean.
doTime ifTrue:[
  anArray size >= 10
    ifFalse:[ self _error: #rtErrBadFormatSpec args: { anArray } ].
  v := anArray at: 7 .
  v class ~~ Character ifTrue:[ v _validateClass: CharacterCollection].
  (anArray at: 9) _validateClass: Boolean.
  (anArray at: 10) _validateClass: Boolean.
  ].

(anArray size > 10)
  ifTrue: [ | doTimeZone |
    (doTimeZone := anArray at: 11) _validateClass: Boolean.
    doTimeZone ifTrue: [
      (anArray size > 11) ifTrue: [ (anArray at: 12) _validateClass: Boolean ]
      ].
    ].
%

category: 'Private'
classmethod: DateTime
_newGmtWithYear: year dayOfYear: dayCount milliseconds: ms

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

^ self _newGmtWithYear: year dayOfYear: dayCount milliseconds: ms
       timeZone: (TimeZone current).
%

category: 'Private'
classmethod: DateTime
_newGmtWithYear: year dayOfYear: dayCount milliseconds: ms timeZone: aTimeZone

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

| extraDays newMillisecs newDayOfYear |

(ms abs >= 86400000)
  ifTrue: [
    extraDays := ms // 86400000.
    newDayOfYear := dayCount + extraDays.
    newMillisecs := ms \\ 86400000.
    ]
  ifFalse: [
    newDayOfYear := dayCount.
    newMillisecs := ms.
    ].

^ self newGmtWithYear: year dayOfYear: newDayOfYear milliseconds: newMillisecs
       timeZone: aTimeZone.
%

category: 'Private'
classmethod: DateTime
_newGmtWithYear: year dayOfYear: dayCount seconds: seconds

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

^ self _newGmtWithYear: year dayOfYear: dayCount seconds: seconds
       timeZone: (TimeZone current).
%

category: 'Private'
classmethod: DateTime
_newGmtWithYear: year dayOfYear: dayCount seconds: seconds timeZone: aTimeZone

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

| extraDays newSeconds newDayOfYear |

(seconds abs >= 86400)
  ifTrue: [
    extraDays := seconds // 86400.
    newDayOfYear := dayCount + extraDays.
    newSeconds := seconds \\ 86400.
    ]
  ifFalse: [
    newDayOfYear := dayCount.
    newSeconds := seconds.
    ].

^ self newGmtWithYear: year dayOfYear: newDayOfYear seconds: newSeconds
%

category: 'Private'
classmethod: DateTime
_newGmtWithYear: yearInt month: monthInt day: dayInt
milliseconds: millisecInt timeZone: aTimeZone

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

<primitive: 238>

aTimeZone _validateClass: TimeZone.
( yearInt _isSmallInteger and:
[ (monthInt _isSmallInteger) and:
[ (dayInt _isSmallInteger) and: [millisecInt _isSmallInteger]]] )
  ifFalse: [
    self _error: #rtErrDateTimeOutOfRange
    ].

(yearInt >= 1901)
  ifFalse: [ yearInt _error: #rtErrBadDateTimeArgs ].
((monthInt >=  1) and: [ monthInt <= 12 ])
  ifFalse: [ ^ monthInt _error: #rtErrBadDateTimeArgs ].
((dayInt >=  1) and: [ dayInt <= 31 ])
  ifFalse: [ ^ dayInt _error: #rtErrBadDateTimeArgs ].
((millisecInt > 0) and: [ millisecInt <= 86399999 ])
  ifFalse: [ ^ millisecInt _error:  #rtErrBadDateTimeArgs ].

^ self _primitiveFailed: #_newGmtWithYear:month:day:milliseconds:timeZone:
       args: { yearInt . monthInt . dayInt . millisecInt . aTimeZone }
%

category: 'Private'
classmethod: DateTime
_newGmtWithYear: yearInt month: monthInt day: dayInt
seconds: secondInt

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

^ self _newGmtWithYear: yearInt month: monthInt day: dayInt
       seconds: secondInt timeZone: (TimeZone current).
%

category: 'Private'
classmethod: DateTime
_newGmtWithYear: yearInt month: monthInt day: dayInt
seconds: secondInt timeZone: aTimeZone

"Creates and returns an instance of the receiver from the specified values,
 which express Greenwich Mean Time."

^ self _newGmtWithYear: yearInt month: monthInt day: dayInt
       milliseconds: (secondInt * 1000) timeZone: aTimeZone.
%

category: 'Private'
classmethod: DateTime
_newJulianDay: julianDay localTime: milliseconds

"Returns a new instance of the receiver based on the specified values. The
 values julianDay and milliseconds represent the local time."

| aTimeZone aDateTime |

aTimeZone := TimeZone current.
aDateTime := (self _newJulianDay: julianDay millisecond: milliseconds
              timeZone: aTimeZone)
             subtractSeconds: (aTimeZone secondsFromGmt).

(aDateTime isDstIn: aTimeZone)
  ifTrue:  [ ^ aDateTime subtractSeconds: (aTimeZone secondsForDst)]
  ifFalse: [ ^ aDateTime ].
%

category: 'Private'
classmethod: DateTime
_newJulianDay: julianDay millisecond: ms

"Returns a new instance of the receiver based on the specified values. The
 values julianDay and milliseconds should represent Greenwich Mean Time."

^ self _newJulianDay: julianDay millisecond: ms timeZone: (TimeZone current).
%

category: 'Private'
classmethod: DateTime
_newJulianDay: julianDay millisecond: ms timeZone: aTimeZone

"Returns a new instance of the receiver based on the specified values. The
 values julianDay and milliseconds should represent Greenwich Mean Time."

<primitive: 236>
julianDay _validateClass: SmallInteger.
ms _validateClass: SmallInteger.
aTimeZone _validateClass: TimeZone.
{ julianDay . ms . aTimeZone } _error: #rtErrBadDateTimeArgs.
^ self _primitiveFailed: #_newJulianDay:milliseconds:timeZone
       args: { julianDay . ms . aTimeZone }
%

category: 'Private'
classmethod: DateTime
_newJulianDay: julianDay second: sec

"Private. Creates and returns an instance of the receiver from the specified
 values, which express Greenwich Mean Time. Generates an error if any of the
 values are out of range."

^ self _newJulianDay: julianDay second: sec timeZone: (TimeZone current).
%

category: 'Private'
classmethod: DateTime
_newJulianDay: julianDay second: sec timeZone: aTimeZone

"Private. Creates and returns an instance of the receiver from the specified
 values, which express Greenwich Mean Time. Generates an error if any of the
 values are out of range."

^ self _newJulianDay: julianDay millisecond: (sec * 1000)
    timeZone: aTimeZone.
%

!		Instance methods for 'DateTime'

category: 'Comparing'
method: DateTime
< aDateTime

"Returns true if the receiver represents a moment in time before that of the
 argument, and false if it doesn't.  Generates an error if the argument is not
 a DateTime."

| argYear argDayOfYear |

argYear := aDateTime yearGmt.
(year < argYear) ifTrue: [ ^true ].
(year > argYear) ifTrue: [ ^false ].

"The years are the same"

argDayOfYear := aDateTime dayOfYearGmt .
(dayOfYear < argDayOfYear) ifTrue: [ ^true ].
(dayOfYear > argDayOfYear) ifTrue: [ ^false ].

"The days are the same"

^ milliseconds < aDateTime millisecondsGmt
%

category: 'Comparing'
method: DateTime
= aDateTime

"Returns true if the receiver represents the same moment in time as that of the
 argument, and false if it does not."

self == aDateTime ifTrue:[ ^ true ].

(aDateTime isKindOf: self class) ifFalse:
    [^false].
^(year = aDateTime yearGmt) and:
     [(dayOfYear = aDateTime dayOfYearGmt) and:
          [milliseconds = aDateTime millisecondsGmt ]]
%

category: 'Comparing'
method: DateTime
> aDateTime

"Returns true if the receiver represents a moment in time after that of the
 argument, and false if it doesn't.  Generates an error if the argument is not
 a DateTime."

| argYear argDayOfYear |

argYear := aDateTime yearGmt.
(year > argYear) ifTrue: [ ^true ].
(year < argYear) ifTrue: [ ^false ].

"The years are the same"

argDayOfYear := aDateTime dayOfYearGmt.
(dayOfYear > argDayOfYear) ifTrue: [ ^true ].
(dayOfYear < argDayOfYear) ifTrue: [ ^false ].

"The days are the same"

^ milliseconds > aDateTime millisecondsGmt
%

category: 'Arithmetic'
method: DateTime
addDays: anInteger

"Returns a DateTime that describes a moment in time anInteger days
 later than that of the receiver."

| newDate  |

newDate := self asDate addDays: anInteger.
^ self class newWithDate: newDate milliseconds: self timeSinceMidnightMs .
%

category: 'Arithmetic'
method: DateTime
addHours: aNumber

"Returns a DateTime that describes a moment in time aNumber hours
 later than that of the receiver."

^ (self class) _newGmtWithYear: year dayOfYear: dayOfYear
                  milliseconds: (milliseconds + (aNumber * 3600000)) asInteger
                  timeZone: timeZone.
%

category: 'Arithmetic'
method: DateTime
addMilliseconds: aNumber

"Returns a DateTime that describes a moment in time aNumber milliseconds
 later than that of the receiver."

^ (self class) _newGmtWithYear: year dayOfYear: dayOfYear
		milliseconds: (milliseconds + aNumber ) asInteger
                timeZone: timeZone.
%

category: 'Arithmetic'
method: DateTime
addMinutes: aNumber

"Returns a DateTime that describes a moment in time aNumber minutes
 later than that of the receiver."

^ (self class) _newGmtWithYear: year dayOfYear: dayOfYear
           milliseconds: (milliseconds + (aNumber * 60000)) asInteger
           timeZone: timeZone.
%

category: 'Arithmetic'
method: DateTime
addMonths: anInteger

"Returns a DateTime that describes a moment in time anInteger months
 later than that of the receiver.

 This method attempts to keep the day of the month the same.  If the
 new month has fewer days than the receiver's original month, then it
 truncates to the last day of the new month."

| newDate |

newDate := self asDate addMonths: anInteger.
^ self class newWithDate: newDate milliseconds: self timeSinceMidnightMs .
%

category: 'Arithmetic'
method: DateTime
addMs: aNumber

"Return a DateTime that describes a moment in time aNumber milliseconds
 later than that of the receiver."

^ self class _newGmtWithYear: year
             dayOfYear: dayOfYear
             milliseconds: (milliseconds + aNumber) asInteger
             timeZone: timeZone
%

category: 'Arithmetic'
method: DateTime
addSeconds: aNumber

"Returns a DateTime that describes a moment in time aNumber seconds
 later than that of the receiver."

^ (self class) _newGmtWithYear: year dayOfYear: dayOfYear
		milliseconds: (milliseconds + (aNumber * 1000)) asInteger
                timeZone: timeZone.
%

category: 'Arithmetic'
method: DateTime
addWeeks: anInteger

"Returns a DateTime that describes a moment in time anInteger weeks
 later than that of the receiver."

^ self addDays: (anInteger * 7).
%

category: 'Arithmetic'
method: DateTime
addYears: anInteger

"Returns a DateTime that describes a moment in time anInteger years
 later than that of the receiver."

| newDate |

newDate := self asDate addYears: anInteger.
^ self class newWithDate: newDate milliseconds: self timeSinceMidnightMs .
%

category: 'Converting'
method: DateTime
asDate

"Returns a Date that represents the receiver, as expressed in local time."

^ self asDateIn: TimeZone current.
%

category: 'Converting'
method: DateTime
asDateIn: aTimeZone
	"Returns a Date that represents the receiver,
	as expressed in the specified time zone.
	Bug 40674 plb 2010.05.20"

	| offset aDateTime |
	offset := self _localOffset: aTimeZone.
	aDateTime := offset = 0
		ifTrue: [self]
		ifFalse: [self addSeconds: offset].
	^self dateClass
		newDay: aDateTime dayOfYearGmt
		year: aDateTime yearGmt.
%

category: 'Converting'
method: DateTime
asDateTime

"Returns the receiver."

^ self.
%

category: 'Converting'
method: DateTime
asDays

"Returns an Integer that represents the receiver in units of days since
 January 1, 1901, local time."

^ self asDate asDays.
%

category: 'Converting'
method: DateTime
asDaysGmt

"Returns an Integer that represents the receiver in units of days since
 January 1, 1901, Greenwich Mean Time."

| numYears numDays |

numYears := year - 1901.
numDays := (numYears * 365) + (numYears // 4) +
           ((numYears + 300) // 400) - (numYears // 100) + dayOfYear - 1.
^ numDays.
%

category: 'Converting'
method: DateTime
asMillisecondsGmt

"Returns an Integer that represents the receiver in units of milliseconds since
 midnight January 1, 1901, Greenwich Mean Time."

^ ((self asDaysGmt) * 86400000) + milliseconds
%

category: 'Converting'
method: DateTime
asParts

"Returns an Array of six SmallIntegers (year month day hours minutes seconds)
 that expresses the receiver in local time."

^ (self addSeconds: (self _localOffset)) asPartsGmt  .
%

category: 'Converting'
method: DateTime
asPartsGmt

"Returns an Array of six SmallIntegers (year month day hours minutes seconds)
 that expresses the receiver in Greenwich Mean Time."

| result |

result := self _yearMonthDayGmt.  "year/month/day"
result addLast: (milliseconds // 3600000).  "hours"
result addLast: (milliseconds \\ 3600000) // 60000.  "minutes"
result addLast: (milliseconds // 1000  \\ 60).  "seconds"
^ result
%

category: 'Converting'
method: DateTime
asPosixSeconds

"Returns a SmallDouble that represents the receiver in units of seconds since
 midnight January 1, 1970, Coordinated Universal Time."

	^self asMillisecondsGmt - 2177452800000 / 1000.0
%

category: 'Converting'
method: DateTime
asSeconds

"Returns an Integer that represents the receiver in units of seconds since
 midnight January 1, 1901, local time."

^ self asSecondsGmt + self _localOffset.
%

category: 'Converting'
method: DateTime
asSecondsGmt

"Returns an Integer that represents the receiver in units of seconds since
 midnight January 1, 1901, Greenwich Mean Time."

^ ((self asDaysGmt) * 86400) + (milliseconds // 1000)
%

category: 'Formatting'
method: DateTime
asString

"Returns a String that expresses the receiver in local time
 in the default format (DD/MM/YYYY HH:MM:SS)."

^self asStringUsingFormat: #(1 2 3 $/ 1 1 $: true true false)
%

category: 'Formatting'
method: DateTime
asStringGmt

"Returns a String that expresses the receiver in Greenwich Mean Time
 in the default format (DD/MM/YYYY HH:MM:SS)."

| t result |

t := self _yearMonthDayGmt.
result := (t at: 3) _digitsAsString .
result addAll: '/';
  addAll: (t at: 2) _digitsAsString;
  addAll: '/';
  addAll: (t at: 1) _digitsAsString;
  addAll: ' ';
  addAll: (milliseconds // 3600000) _digitsAsString;
  addAll: ':';
  addAll: ((milliseconds // 1000 \\ 3600) // 60) _digitsAsString;
  addAll: ':';
  addAll: (milliseconds // 1000 \\ 60 ) truncated _digitsAsString .
^ result
%

category: 'Formatting'
method: DateTime
asStringGmtMs
"Returns a String that expresses the receiver in Greenwich Mean Time
 in the default format (DD/MM/YYYY HH:MM:SS.mmm)."

| result |
result := self asStringGmt .
result add: $. ;
  addAll: (milliseconds \\ 1000) _digitsAsString3  .
^ result
%

category: 'Formatting'
method: DateTime
asStringGmtUsingFormat: anArray

"Returns a String that expresses the receiver in Greenwich Mean Time
 in the format defined by anArray.  Generates an error if anArray
 contains an incorrect formatting specification.

 See the class documentation of DateTime for a complete description of the
 String-formatting specification Array."

| aString |

((anArray size >= 11) and: [ anArray at: 11 ])
  ifTrue: [
    aString := self asStringStdUsingFormat: anArray.
    aString add: Character space.
    aString addAll: 'Greenwich Mean Time'.
    ]
  ifFalse: [
    aString := self asStringStdUsingFormat: anArray.
    ].

^ aString.
%

category: 'Formatting'
method: DateTime
asStringISO8601
  "yyyy-mm-ddThh:mm:ss+zzzz"

  | string offset offsetHours offsetMinutes |
  string := self asStringUsingFormat: #(3 2 1 $- 1 1 $: true true false true true).
  string at: 11 put: $T.
  string := string copyFrom: 1 to: 19.
  offset := timeZone offsetAtLocal: self.
  string add: (offset < 0
    ifTrue: [$-]
    ifFalse: [$+]).
  offset := offset abs // 60.
  offsetHours := offset // 60.
  offsetMinutes := offset \\ 60.
  offsetHours < 10 ifTrue: [string add: $0].
  string addAll: offsetHours printString.
  offsetMinutes < 10 ifTrue: [string add: $0].
  string addAll: offsetMinutes printString.
  ^ string
%

category: 'Formatting'
method: DateTime
asStringMs
"Returns a String that expresses the receiver in local time
 in the default format (DD/MM/YYYY HH:MM:SS.mmm)."

| result |
result := self asString .
result add: $. ;
  addAll: (milliseconds \\ 1000) _digitsAsString3  .
^ result
%

category: 'Private'
method: DateTime
asStringStdUsingFormat: anArray

"Returns a String that expresses the receiver according to its
 contents without regard to the time zone parameter."

|t dateSeparator timeSeparator monthName aString
 hour hourInt min sec day yearNumber |

t := self _yearMonthDayGmt.
self class _checkFormat: anArray.
dateSeparator := (anArray at: 4) asString.

timeSeparator := (anArray at: 7) asString.

((anArray at: 5) == 2) "get the month name according to the format"
   ifTrue: [monthName := self _monthAbbrev: (t at: 2)]
   ifFalse: [((anArray at: 5) == 3) "month as number is default"
      ifTrue: [monthName := Date nameOfMonth: (t at: 2)]
      ifFalse: [monthName := (t at: 2) _digitsAsString]].

((anArray at: 6) == 2)
   ifTrue: [yearNumber := ((t at: 1) \\ 100) _digitsAsString]
   ifFalse: [yearNumber := (t at: 1) asString].  "YYYY is default"

day := (t at:3) _digitsAsString.
((anArray at: 1) == 2) "month first"
   ifTrue: [aString := monthName , dateSeparator]
   ifFalse: [((anArray at: 1) == 3) "yearNumber first"
      ifTrue: [aString := yearNumber , dateSeparator]
      ifFalse: [aString := day , dateSeparator]].  "day first is default"

((anArray at: 2) == 1) "day second"
   ifTrue: [aString addAll: day; addAll: dateSeparator] "yearNumber second"
   ifFalse: [((anArray at: 2) == 3) "month second is default"
      ifTrue: [aString addAll: yearNumber; addAll: dateSeparator]
      ifFalse: [aString addAll: monthName; addAll: dateSeparator]].

((anArray at: 3) == 1) "day third"
   ifTrue: [aString addAll: day]
   ifFalse: [((anArray at: 3) == 2) "month third"
      ifTrue: [aString addAll: monthName]
      ifFalse: [aString addAll: yearNumber]].  "yearNumber third is default"

hourInt := (milliseconds // 3600000) .
hour := hourInt _digitsAsString.
min := (milliseconds // 1000 \\ 3600 // 60) _digitsAsString.
sec := (milliseconds // 1000 \\ 60) _digitsAsString.

(anArray at: 8) ifTrue: [ "print the time"
  aString add: $ .
  (anArray at: 10) ifTrue: [ "12-hour format"
    (hourInt > 12) ifTrue: [
      aString addAll: (hourInt - 12) _digitsAsString;
              addAll: timeSeparator;
              addAll: min.

      (anArray at: 9) ifTrue: [
        aString addAll: timeSeparator;
                addAll: sec
        ].
      ]
    ifFalse: [
      aString addAll: (hourInt == 0 ifTrue: ['12'] ifFalse: [hour]);
              addAll: timeSeparator;
              addAll: min.

      (anArray at: 9) ifTrue: [
        aString addAll: timeSeparator;
                addAll: sec.
        ].
      ].

    aString addAll: (hourInt >= 12 ifTrue: [' PM'] ifFalse: [' AM']).
    ]
  ifFalse: [
    aString addAll: hour;
            addAll: timeSeparator;
            addAll: min.

    (anArray at: 9) ifTrue: [
      aString addAll: timeSeparator;
              addAll: sec.
      ].
    ].
  ].

^ aString
%

category: 'Formatting'
method: DateTime
asStringUsingFormat: anArray

"Returns a String that expresses the receiver in local time
 in the format defined by anArray.  Generates an error if anArray
 contains an incorrect formatting specification.

 See the class documentation of DateTime for a complete description of the
 String-formatting specification Array."

| localDt aString |

((anArray size >= 11) and: [ anArray at: 11 ])
  ifTrue: [
    | aTimeZone |
    ((anArray size >= 12) and: [ anArray at: 12 ])
      ifTrue: [ aTimeZone := self timeZone ]
      ifFalse: [ aTimeZone := TimeZone current ].
    localDt := self addSeconds: (self _localOffset: aTimeZone).
    aString := localDt asStringStdUsingFormat: anArray.
    aString add: Character space.
    (self isDstIn: aTimeZone)
      ifTrue: [ aString addAll: aTimeZone dstPrintString ]
      ifFalse: [ aString addAll: aTimeZone standardPrintString ].
    ]
  ifFalse: [
    localDt := self addSeconds: self _localOffset.
    aString := localDt asStringStdUsingFormat: anArray.
    ].

^ aString.
%

category: 'Converting'
method: DateTime
asTime

"Returns a Time which represents the time since midnight represented by the
 receiver, as expressed in local time."

^ self timeClass fromMilliseconds: self timeSinceMidnightMs
%

category: 'Accessing'
method: DateTime
at: anIndex put: aValue

"Disallowed. Changes to a DateTime a not allowed."

^ self shouldNotImplement: #at:put:
%

category: 'Repository Conversion'
method: DateTime
convertToDateTime

"Returns the receiver."

^ self.
%

category: 'Converting'
method: DateTime
dateClass

"Returns the class of the object to be created when the receiver is being
 converted to a date."

^ Date
%

category: 'Accessing'
method: DateTime
day

"Returns a SmallInteger that gives the day of the month described by the
 receiver, expressed in local time."

^ self asDate day.
%

category: 'Accessing'
method: DateTime
dayGmt

"Returns a SmallInteger that gives the day in the month described by the
 receiver, expressed in Greenwich Mean Time."

^  (self _yearMonthDayGmt) at: 3
%

category: 'Accessing'
method: DateTime
dayOfMonth

"Returns a SmallInteger that gives the day of the month described by the
 receiver, expressed in local time."

^self day
%

category: 'Accessing'
method: DateTime
dayOfMonthGmt

"Returns a SmallInteger that gives the day of the month described by the
 receiver, expressed in Greenwich Mean Time."

^ self dayGmt
%

category: 'Accessing'
method: DateTime
dayOfWeek

"Returns a SmallInteger that gives the numeric index of the day of the week
 described by the receiver, expressed in local time.  The index is a
 number between 1 and 7 inclusive, where 1 signifies Sunday."

^ self asDate dayOfWeek.
%

category: 'Accessing'
method: DateTime
dayOfWeekGmt

"Returns a SmallInteger that gives the numeric index of the day of the week
 described by the receiver, expressed in Greenwich Mean Time.  The index is a
 number between 1 and 7 inclusive, where 1 signifies Sunday."

    ^(DateTime fromString: (self asStringGmt)) dayOfWeek
%

category: 'Accessing'
method: DateTime
dayOfYear

"Returns a SmallInteger that gives the day of the year described by the
 receiver, expressed in local time."

^ self asDate dayOfYear.
%

category: 'Accessing'
method: DateTime
dayOfYearGmt

"Returns a SmallInteger that gives the day of the year described by the
 receiver, expressed in Greenwich Mean Time."

^ dayOfYear
%

category: 'Deprecated'
method: DateTime
daysInMonth

"Returns a SmallInteger that gives the number of days in the month
 described by the receiver, expressed in local time."

self deprecated: 'DateTime>>daysInMonth deprecated v3.0, use Date>>daysInMonth'.
^ self asDate daysInMonth.
%

category: 'Deprecated'
method: DateTime
daysInMonthGmt

"Returns a SmallInteger that gives the number of days in the month described by
 the receiver, expressed in Greenwich Mean Time."

self deprecated: 'DateTime>>daysInMonthGmt deprecated v3.0, use Date protocols'.
^ self _daysInMonth: self monthOfYearGmt
%

category: 'Accessing'
method: DateTime
daysInYear

"Returns a SmallInteger that gives the number of days in the year
 described by the receiver."

(self leap) ifTrue: [^ 366].
^ 365
%

category: 'Comparing'
method: DateTime
hash

"Returns an Integer hash code for the receiver."

^ (((year hash) bitShift: -1) bitXor: (dayOfYear hash)) bitXor: (self secondGmt hash).
%

category: 'Accessing'
method: DateTime
hour

"Returns the hour in the 24-hour clock represented by the receiver in local
 time."

^ self _secondsLocal // 3600
%

category: 'Accessing'
method: DateTime
hourGmt

"Returns the hour in the 24-hour clock represented by the receiver, as
 expressed in Greenwich Mean Time"

^ milliseconds // 3600000
%

category: 'Accessing'
method: DateTime
hours

"Returns a SmallInteger (between zero and 23 inclusive) that gives the number of
 hours represented by the receiver since midnight, local time."

^ self hour
%

category: 'Accessing'
method: DateTime
hoursGmt

"Returns a SmallInteger (between zero and 23 inclusive) that gives the number of
 hours represented by the receiver since midnight, Greenwich Mean Time."

^ self hourGmt
%

category: 'Private'
method: DateTime
isDst

^ self isDstIn: (TimeZone current)
%

category: 'Private'
method: DateTime
isDstIn: aTimeZone

| startOfDst endOfDst aYear|

(aTimeZone secondsForDst == 0)
  ifTrue: [ ^ false "No Dst"].

aYear := self yearGmt.
(aTimeZone yearStartDst > aYear)
  ifTrue: [ ^ false "Dst did not begin until later on"].

(startOfDst := aTimeZone startOfDstFor: aYear) == nil ifTrue: [^false].
(endOfDst   := aTimeZone endOfDstFor: aYear) == nil ifTrue: [^false].

(startOfDst < endOfDst)
  ifTrue: [ "Northern Hemisphere"
    ^ ((self >= startOfDst) and: [ self < endOfDst ])
    ]
  ifFalse: [ "Southern Hemisphere"
    ^ ((self >= startOfDst) or: [ self < endOfDst ])
    ]
%

category: 'Accessing'
method: DateTime
julianDay

"Returns the Julian Day of the receiver, expressed in local time. The julian
 day is a SmallInteger that gives the number of days since January 1, 4713 B.C.,
 as defined in Communications of the ACM, algorithm #199."

^ self asDate julianDay.
%

category: 'Deprecated'
method: DateTime
julianSecond

"Returns a SmallInteger (between zero and 86399 inclusive) that gives the number
 of seconds represented by the receiver since midnight, local time."

self deprecated: 'DateTime>>julianSecond  deprecated v3.0, use #timeSinceMidnight'.
^ self timeSinceMidnight.
%

category: 'Accessing'
method: DateTime
leap

"Returns true if the receiver describes a leap year, expressed in local time,
 and false if it does not."

^ self asDate leap.
%

category: 'Accessing'
method: DateTime
leapGmt

"Returns true if the receiver describes a leap year, expressed in Greenwich Mean
 Time, and false if it does not."

| yr |

"a year is a leap year if: (it is evenly divisible by 4 and it is not a
 century year) or (it is a century year and evenly divisible by 400)"

yr := (self _yearMonthDayGmt) at: 1.
((yr \\ 100) == 0) ifTrue: [^ ((yr \\ 400) == 0)].
^ ((yr \\ 4) == 0)
%

category: 'Accessing'
method: DateTime
millisecondsGmt

"Returns the value of the milliseconds instance variable."

^ milliseconds.
%

category: 'Accessing'
method: DateTime
minute

"Returns a SmallInteger (between zero and 59 inclusive) that gives the minute
 of the time represented by the receiver in local time."

^ (self _secondsLocal \\ 3600) // 60
%

category: 'Accessing'
method: DateTime
minuteGmt

"Returns a SmallInteger (between zero and 59 inclusive) that gives the minute
 of the time represented by the receiver in Greenwich Mean Time."

^ milliseconds \\ 3600000 // 60000
%

category: 'Accessing'
method: DateTime
minutes

"Returns a SmallInteger (between zero and 59 inclusive) that gives the number of
 minutes represented by the receiver since the previous hour, local time."

^ self minute
%

category: 'Accessing'
method: DateTime
minutesGmt

"Returns a SmallInteger (between zero and 59 inclusive) that gives the number of
 minutes represented by the receiver since the previous hour, Greenwich Mean
 Time."

^ self minuteGmt
%

category: 'Accessing'
method: DateTime
month

"Returns a SmallInteger that gives the numeric index of the month of the year
 described by the receiver, expressed in local time.  The index is a number
 between 1 and 12 inclusive, where 1 signifies January."

^ self asDate month
%

category: 'Accessing'
method: DateTime
monthGmt

"Returns a SmallInteger that gives the numeric index of the month of the year
 described by the receiver, expressed in Greenwich Mean Time. The index is a
 number between 1 and 12 inclusive, where 1 signifies January."

^  (self _yearMonthDayGmt) at: 2
%

category: 'Accessing'
method: DateTime
monthName

"Returns a String that gives the name of the month of the year described by the
 receiver, expressed in local time, in the user's native language."

^ self asDate monthName.
%

category: 'Accessing'
method: DateTime
monthNameGmt

"Returns a String that gives the name of the month of the year described by the
 receiver, expressed in Greenwich Mean Time, in the user's native language."

^ MonthNames value at: self monthGmt
%

category: 'Accessing'
method: DateTime
monthOfYear

"Returns a SmallInteger that gives the numeric index of the month of the year
 described by the receiver, expressed in local time.  The index is a number
 between 1 and 12 inclusive, where 1 signifies January."

^ self month
%

category: 'Accessing'
method: DateTime
monthOfYearGmt

"Returns a SmallInteger that gives the numeric index of the month of the year
 described by the receiver, expressed in Greenwich Mean Time.  The index is a
 number between 1 and 12 inclusive, where 1 signifies January."

^ self monthGmt
%

category: 'Formatting'
method: DateTime
printJsonOn: aStream
	"JSON does not specify the format for dates and times, so we use ISO 8601"

	aStream nextPutAll: self asStringISO8601.
%

category: 'Formatting'
method: DateTime
printOn: aStream

"Puts a displayable representation of the receiver, expressed in local
 time, on aStream."

aStream nextPutAll: self asString .
%

category: 'Accessing'
method: DateTime
second

"Returns a SmallInteger (between zero and 59 inclusive) that gives the number of
 seconds represented by the receiver since the previous minute."

^ self _secondsLocal \\ 60.
%

category: 'Accessing'
method: DateTime
secondGmt

"Returns a SmallInteger (between zero and 59 inclusive) that gives the number
 of seconds represented by the receiver, as expressed in Greenwich Mean Time,
 since the previous minute."

^ milliseconds // 1000 \\ 60.
%

category: 'Accessing'
method: DateTime
seconds

"Returns a SmallInteger (between zero and 59 inclusive) that gives the number of
 seconds represented by the receiver since the previous minute."

^ self second.
%

category: 'Accessing'
method: DateTime
secondsGmt

"Returns a SmallInteger (between zero and 59 inclusive) that gives the number
 of seconds represented by the receiver, as expressed in Greenwich Mean Time,
 since the previous minute."

^ self secondGmt.
%

category: 'Accessing'
method: DateTime
size: anInteger

"Disallowed.  You may not change the size of a DateTime."

self shouldNotImplement: #size:
%

category: 'Arithmetic'
method: DateTime
subtractDate: aDateTime

"Returns a positive Integer that counts the number of times midnight local time
 occurs between the times described by the receiver and aDateTime."

^ (self asDate) subtractDate: (aDateTime asDate)
%

category: 'Arithmetic'
method: DateTime
subtractDateGmt: aDateTime

"Returns a positive Integer that counts the number of times that midnight
 Greenwich Mean Time occurs between the times described by the receiver and
 aDateTime."

^ (self asDays - aDateTime asDays) abs
%

category: 'Arithmetic'
method: DateTime
subtractDays: anInteger

"Returns a DateTime that describes a moment in time anInteger days
 earlier than that of the receiver."

^ self addDays: (anInteger negated).
%

category: 'Arithmetic'
method: DateTime
subtractHours: aNumber

"Returns a DateTime that describes a moment in time aNumber hours
 earlier than that of the receiver."

^ self addHours: (aNumber negated).
%

category: 'Arithmetic'
method: DateTime
subtractMinutes: aNumber

"Returns a DateTime that describes a moment in time aNumber minutes
 earlier than that of the receiver."

^ self addMinutes: (aNumber negated).
%

category: 'Arithmetic'
method: DateTime
subtractMonths: anInteger

"Returns a DateTime that describes a moment in time anInteger months
 earlier than that of the receiver.

 This method attempts to keep the day of the month the same.  If the
 new month has fewer days than the receiver's original month, then it
 truncates to the last day of the new month."

^ self addMonths: (anInteger negated).
%

category: 'Arithmetic'
method: DateTime
subtractSeconds: aNumber

"Returns a DateTime that describes a moment in time aNumber seconds
 earlier than that of the receiver."

^ self addSeconds: (aNumber negated).
%

category: 'Arithmetic'
method: DateTime
subtractTime: aDateTime

"Returns an Array of three positive Integers that count the hours, minutes, and
 seconds, respectively, between the times of day described by the receiver and
 aDateTime.

 The computation ignores the dates of both the receiver and aDateTime, and
 assumes that the receiver is the later time.  Hence, if the time of day in the
 receiver is less than the time of day in aDateTime, then the receiver's time of
 day is assumed to occur on the day following that of aDateTime."

| parts h m s |

parts := self asParts.
h := parts at: 4.
m := parts at: 5.
s := parts at: 6.

parts := aDateTime asParts.
h < (parts at: 4) ifTrue:[ h := h + 24 ].
h := h - (parts at: 4).
m := m - (parts at: 5).
s := s - (parts at: 6).

s < 0 ifTrue: [
  s := s + 60.
  m := m - 1
].
s > 60 ifTrue: [
  s := s - 60.
  m := m + 1
].
m < 0 ifTrue: [
  m := m + 60.
  h := h - 1
].
m > 60 ifTrue: [
  m := m - 60.
  h := h + 1
].
^{ h . m . s }
%

category: 'Arithmetic'
method: DateTime
subtractWeeks: anInteger

"Returns a DateTime that describes a moment in time anInteger weeks
 earlier than that of the receiver."

^ self subtractDays: (anInteger * 7).
%

category: 'Arithmetic'
method: DateTime
subtractYears: anInteger

"Returns a DateTime that describes a moment in time anInteger years
 earlier than that of the receiver."

^ self addYears: (anInteger negated).
%

category: 'Deprecated'
method: DateTime
timeAsSeconds

"Returns a SmallInteger (between zero and 86399 inclusive) that gives the number
 of seconds represented by the receiver since midnight, local time."

self deprecated: 'DateTime>>timeAsSeconds deprecated v3.0, use #timeSinceMidnight'.
^ self timeSinceMidnight.
%

category: 'Deprecated'
method: DateTime
timeAsSecondsGmt

"Returns a SmallInteger (between zero and 86399 inclusive) that gives the number
 of seconds represented by the receiver since midnight, Greenwich Mean Time."

self deprecated: 'DateTime>>timeAsSecondsGmt deprecated v3.0, use #timeSinceMidnightGmt'.
^ self timeSinceMidnightGmt.
%

category: 'Converting'
method: DateTime
timeClass

^ Time
%

category: 'Deprecated'
method: DateTime
timeCompare: aDateTime
 "Provided for compatibility.

  Answer 0 is the receiver's time component is the same as that of
  aDateTime. Otherwise, -1 is the receiver's is less than, or 1 if
  the receiver's is greater than."

| otherMs |
self deprecated: 'DateTime>>timeCompare: deprecated v3.0, use other comparison methods'.
otherMs := aDateTime millisecondsGmt.
milliseconds = otherMs ifTrue: [^0].
milliseconds < otherMs ifTrue: [^-1].
^1
%

category: 'Converting'
method: DateTime
timeSinceMidnight

"Returns a SmallInteger (between zero and 86399 inclusive) that gives the number
 of seconds represented by the receiver since midnight, local time."

| secs tz |

secs := self timeSinceMidnightGmt + (tz := TimeZone current) secondsFromGmt.
self isDst ifTrue: [ secs := secs + tz secondsForDst ].
secs := (secs + 86400) \\ 86400.
^ secs.
%

category: 'Converting'
method: DateTime
timeSinceMidnightGmt

"Returns a SmallInteger (between zero and 86399 inclusive) that gives the number
 of seconds represented by the receiver since midnight, Greenwich Mean Time."

^ milliseconds // 1000.
%

category: 'Converting'
method: DateTime
timeSinceMidnightMs

"Returns a SmallInteger (between zero and 86399999 inclusive) that gives the number
 of milliseconds represented by the receiver since midnight, local time."

| ms tz|
ms := milliseconds + ((tz := TimeZone current) secondsFromGmt * 1000) .
self isDst ifTrue:[ ms := ms + (tz secondsForDst * 1000) ].
ms := (ms + 86400000) \\ 86400000.
^ ms.
%

category: 'Accessing'
method: DateTime
timeZone

^ timeZone
%

category: 'Converting'
method: DateTime
timeZone: aTimeZone

"Returns a new DateTime with the specified TimeZone. The new DateTime
 represents the same point in time as the receiver. A point in time
 is usually different when viewed as local time in different time zones."

^ self class newGmtWithYear: (self yearGmt) dayOfYear: (self dayOfYearGmt)
       milliseconds: (self millisecondsGmt) timeZone: aTimeZone.
%

category: 'Formatting'
method: DateTime
US12HrFormat

"Returns a String that expresses the receiver in local time.  The date is in
 United States format (month first) and the time of day is based on the 12-hour
 clock (MM/DD/YY HH:MM:SS pm)."

^self asStringUsingFormat: #(2 1 3 $/ 1 2 $: true false true)
%

category: 'Formatting'
method: DateTime
US24HrFormat

"Returns a String that expresses the receiver in local time.  The date is in
 United States format (month first) and the time of day is based on the 24-hour
 clock (MM/DD/YY HH:MM:SS)."

^self asStringUsingFormat: #(2 1 3 $/ 1 2 $: true false false)
%

category: 'Formatting'
method: DateTime
USDateFormat

"Returns a String that expresses the date of the receiver, expressed in local
 time. The date is in United States format, month first (MM/DD/YY)."

^self asStringUsingFormat: #(2 1 3 $/ 1 2 $: false)
%

category: 'Accessing'
method: DateTime
weekDayName

"Returns a String that gives the name of the day of the week described by the
 receiver, expressed in local time."

^ self asDate weekDayName
%

category: 'Accessing'
method: DateTime
weekDayNameGmt

"Returns a String that gives the name of the day of the week described by the
 receiver, expressed in Greenwich Mean Time."

^  WeekDayNames value at: (self dayOfWeekGmt).
%

category: 'Accessing'
method: DateTime
year

"Returns a SmallInteger that gives the year described by the receiver,
 expressed in local time."

^ self asDate year.
%

category: 'Accessing'
method: DateTime
yearGmt

"Returns a SmallInteger that gives the year described by the receiver, expressed
 in Greenwich Mean Time."

^ year
%

category: 'New Indexing Comparison'
method: DateTime
_classSortOrdinal

^ 60
%

category: 'Private'
method: DateTime
_daysInMonth: month

"Returns a SmallInteger that gives the number of days in the month
 specified by the Integer month."

((month == 1) or: [(month == 3) or: [(month == 5) or: [(month == 7) or:
   [(month == 8) or: [(month == 10) or: [(month == 12)]]]]]])
   ifTrue: [^ 31].
((month == 4) or: [(month == 6) or: [(month == 9) or: [(month == 11)]]])
   ifTrue: [^ 30].
(self leap)
   ifTrue: [^ 29].
^ 28
%

category: 'Private'
method: DateTime
_localOffset

^ self _localOffset: TimeZone current.
%

category: 'Private'
method: DateTime
_localOffset: aTimeZone

| offset |

offset := aTimeZone secondsFromGmt.
(self isDstIn: aTimeZone)
  ifTrue: [ offset := offset + aTimeZone secondsForDst ].
^ offset.
%

category: 'Formatting'
method: DateTime
_monthAbbrev: anIndex

"Private.  Returns a three-letter String that gives the abbreviation, in the
 user's native language, of the name of the month whose numeric index is
 anIndex.  The index is a number between 1 and 12 inclusive, where 1 signifies
 January."

|theMonth itsAbbrev|

theMonth := self class nameOfMonth: anIndex.  "get its full name"
itsAbbrev := String new.
1 to: 3 do: "take the first three letters"
   [:aChar | itsAbbrev := itsAbbrev , (theMonth at: aChar)].
^ itsAbbrev
%

category: 'Deprecated'
method: DateTime
_monthName: anIndex

"Private.  Returns a String that gives the name, in the user's native language,
 of the month of the year whose numeric index is anIndex.  The index is a number
 between 1 and 12 inclusive, where 1 signifies January."

self deprecated: 'DateTime>>_monthName: deprecated v3.0, use nameOfMonth:'.
^ (MonthNames value) at: anIndex.
%

category: 'Private'
method: DateTime
_secondsLocal

"Returns the number of seconds since midnight in the receiver, as expressed in
 local time."

^ ((milliseconds // 1000) + self _localOffset) \\ 86400.
%

category: 'Private'
method: DateTime
_yearMonthDay

"Private. Returns a three-element Array of SmallIntegers containing the year,
 index of the month, and the day of the month represented by the receiver,
 expressed in local time."

^ (self addSeconds: (self _localOffset)) _yearMonthDayGmt.
%

category: 'Private'
method: DateTime
_yearMonthDayGmt

"Private. Returns a three-element Array of SmallIntegers containing the year,
 index of the month, and the day of the month represented by the receiver,
 expressed in Greenwich Mean Time."

<primitive: 239 >
^ self _primitiveFailed: #_yearMonthDayGmt
%

! Class extensions for 'Deprecated'

!		Instance methods for 'Deprecated'

removeallmethods Deprecated
removeallclassmethods Deprecated

category: 'Instance initialization'
method: Deprecated
initialize
  gsNumber := ERR_Deprecated.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'DoubleByteString'

!		Class methods for 'DoubleByteString'

removeallmethods DoubleByteString
removeallclassmethods DoubleByteString

category: 'Formatting'
classmethod: DoubleByteString
charSize

"Returns number of bytes that make up a character for this string class"

^ 2
%

category: 'Instance Creation'
classmethod: DoubleByteString
new

"Returns a new instance of the receiver."

^ self _basicNew
%

category: 'Instance Creation'
classmethod: DoubleByteString
new: aSize

"Returns a new instance of the receiver, of sufficient size to hold aSize
 Characters of two bytes each."

^ self _basicNew:  aSize + aSize
%

category: 'Instance Creation'
classmethod: DoubleByteString
withAll: aString

"Returns an instance of the receiver or an instance of QuadByteString,
 using the minimum bytes per character required to represent the argument."

<primitive: 456>
^ QuadByteString withAll: aString
%

!		Instance methods for 'DoubleByteString'

category: 'Concatenating'
method: DoubleByteString
, aCharOrCharColl

"Returns a new instance of the receiver's class that contains the elements of
 the receiver followed by the elements of aCharOrCharColl.  The argument
 must be a String, a DoubleByteString, or an AbstractCharacter.

 The result may not be an instance of the class of the receiver if one of the
 following rules applies:

 1) If the receiver is a DoubleByteSymbol, the result is a DoubleByteString.

 Warning: Creating a new instance and copying the receiver take time.  If you
 can safely modify the receiver, it can be much faster to use the addAll:
 method.  See the documentation of the Concatenating category of class
 SequenceableCollection for more details."

<primitive: 614 >
| result cSize |
cSize := aCharOrCharColl stringCharSize .
cSize == 4 ifTrue:[
  result:= QuadByteString withAll: self.
  result addAll: aCharOrCharColl .
  ^ result .
].
aCharOrCharColl class == Character ifTrue:[
  aCharOrCharColl codePoint <= 16rFFFF ifTrue:[
    result := self copy
  ] ifFalse:[
    result := QuadByteString withAll: self
  ].
  result at: (result size + 1) put: aCharOrCharColl .
  ^ result
].
aCharOrCharColl _validateClasses: { AbstractCharacter . CharacterCollection }.
^ super , aCharOrCharColl
%

category: 'Adding'
method: DoubleByteString
add: aCharOrCharColl

"Appends all of the elements of aCharOrCharColl to the receiver and returns
 aCharOrCharColl."

<primitive: 437>
| cSize |
aCharOrCharColl class == Character ifTrue:[
  aCharOrCharColl codePoint > 16rFFFF ifTrue:[
    ^ self _convertToQuadByte add: aCharOrCharColl
  ].
  ^ self _primitiveFailed: #add: args: { aCharOrCharColl }
].
(cSize := aCharOrCharColl stringCharSize) == 4 ifTrue:[
  aCharOrCharColl _asDoubleByteString ifNotNil:[ :aString |
    ^ self addAll: aString
  ] ifNil:[
    ^ self _convertToQuadByte addAll: aCharOrCharColl
  ]
].
(aCharOrCharColl isKindOf: AbstractCharacter) ifTrue:[
  ^ self add: aCharOrCharColl asCharacter .
].
aCharOrCharColl _validateClass: CharacterCollection .
^ self add: aCharOrCharColl asDoubleByteString
%

category: 'Adding'
method: DoubleByteString
addAll: aCharOrCharColl

"Equivalent to add: aCharOrCharColl."

<primitive: 437>
| cSize |
(cSize := aCharOrCharColl stringCharSize) == 4 ifTrue:[
  aCharOrCharColl _asDoubleByteString ifNotNil:[ :aString |
    ^ self addAll: aString
  ] ifNil:[
    ^ self _convertToQuadByte addAll: aCharOrCharColl
  ]
].
aCharOrCharColl class == Character ifTrue:[
  aCharOrCharColl codePoint > 16rFFFF ifTrue:[
    ^ self _convertToQuadByte add: aCharOrCharColl
  ].
  ^ self _primitiveFailed: #add: args: { aCharOrCharColl }
].
(aCharOrCharColl isKindOf: AbstractCharacter) ifTrue:[
  ^ self add: aCharOrCharColl asCharacter .
].
(aCharOrCharColl isKindOf: CharacterCollection) ifTrue:[
  ^ self add: aCharOrCharColl asDoubleByteString.
].
aCharOrCharColl do: [:each | self add: each].
^aCharOrCharColl.
%

category: 'Adding'
method: DoubleByteString
addCodePoint: aSmallInteger

<primitive: 1048>
^ self add: (Character codePoint: aSmallInteger).
%

category: 'Adding'
method: DoubleByteString
addLast: aCharOrCharColl

"Equivalent to add: aCharOrCharColl."

<primitive: 437>
| cSize |
aCharOrCharColl class == Character ifTrue:[
  aCharOrCharColl codePoint > 16rFFFF ifTrue:[
    ^ self _convertToQuadByte add: aCharOrCharColl
  ].
  ^ self _primitiveFailed: #add: args: { aCharOrCharColl }
].
(cSize := aCharOrCharColl stringCharSize) == 4 ifTrue:[
  aCharOrCharColl _asDoubleByteString ifNotNil:[ :aString |
    ^ self addAll: aString
  ] ifNil:[
    ^ self _convertToQuadByte addAll: aCharOrCharColl
  ]
].
(aCharOrCharColl isKindOf: AbstractCharacter) ifTrue:[
  ^ self add: aCharOrCharColl asCharacter .
].
aCharOrCharColl _validateClass: CharacterCollection .
^ self add: aCharOrCharColl asDoubleByteString
%

category: 'Deprecated'
method: DoubleByteString
asciiLessThan: aString

"Returns true if the receiver collates before the argument using the
 ASCII collating table, which collates AB...Z...ab..z."

self deprecated: 'DoubleByteString>>asciiLessThan: deprecated in 3.2'.

^ self lessThan: aString collatingTable: DoubleByteAsciiCollatingTable
%

category: 'Concatenating'
method: DoubleByteString
asDoubleByteString
 ^ self
%

category: 'Accessing'
method: DoubleByteString
at: anIndex

"Returns the Character at anIndex."

<primitive: 435>

(anIndex _isSmallInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #at: args: { anIndex } .
self _uncontinuableError
%

category: 'Updating'
method: DoubleByteString
at: anIndex put: aChar

"Stores aChar at the specified location and returns aChar."

<primitive: 436>

(aChar class == Character) ifTrue:[
  (aChar codePoint) > 16rFFFF ifTrue:[
    ^ self _convertToQuadByte at:anIndex put:aChar
  ].
] ifFalse:[
  aChar _validateClass: AbstractCharacter .
  ^ self at: anIndex put: aChar asCharacter
].
(anIndex _isSmallInteger) ifTrue: [
  ((anIndex > (self size + 1)) or: [anIndex <= 0]) ifTrue: [
    ^ self _errorIndexOutOfRange: anIndex
    ].
] ifFalse:[
  ^ self _errorNonIntegerIndex: anIndex
] .
self _primitiveFailed: #at:put: args: { anIndex . aChar }
%

category: 'Formatting'
method: DoubleByteString
charSize

"Returns number of bytes that make up a character for this string class."

^ 2
%

category: 'Accessing'
method: DoubleByteString
codePointAt: anIndex

"Returns the integer value of the Character at anIndex in the receiver."

<primitive: 1072>

(anIndex _isSmallInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #codePointAt: args: { anIndex } .
self _uncontinuableError
%

category: 'Updating'
method: DoubleByteString
codePointAt: anIndex put: aValue

"Stores aValue at the specified location, with autogrow, but
 without auto-conversion of receiver to larger character size.
 Returns aValue.
"
<primitive: 1074>
aValue _validateClass: SmallInteger .
(aValue < 0 or:[ aValue > 65535]) ifTrue:[
  aValue _error: #rtErrArgOutOfRange args:{ 0 . 65535 }
].
(anIndex _isSmallInteger) ifTrue: [
  ((anIndex > (self size + 1)) or: [anIndex < 1]) ifTrue: [
    ^ self _errorIndexOutOfRange: anIndex
  ]
] ifFalse: [
  ^ self _errorNonIntegerIndex: anIndex
] .
self _primitiveFailed: #codePointAt:put: args: { anIndex . aValue }
%

category: 'Encoding'
method: DoubleByteString
encodeAsUTF16

"Encode receiver in UTF16 format.  Returns a Utf16 ."

<primitive: 1084>
self _primitiveFailed: #encodeAsUTF16
%

category: 'Encoding'
method: DoubleByteString
encodeAsUTF8

"Encode receiver in UTF8 format. Returns a Utf8 ."

<primitive: 468>
self _primitiveFailed: #encodeAsUTF8
%

category: 'Encoding'
method: DoubleByteString
encodeAsUTF8IntoString
"Encode receiver in UTF8 format.  Returns a String. For Seaside"

<primitive: 994>
self _primitiveFailed: #encodeAsUTF8IntoString
%

category: 'Adding'
method: DoubleByteString
insertAll: aCharOrCharColl at: anIndex

"Inserts aCharOrCharColl at the specified index.  Returns aCharOrCharColl."

<primitive: 454>
| cSize |
anIndex _isSmallInteger ifFalse:[ anIndex _validateClass: Integer ].
((anIndex <= 0) or: [anIndex > (self size + 1)])
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].

(cSize := aCharOrCharColl stringCharSize) == 4 ifTrue:[ | aString |
  (aString := aCharOrCharColl _asDoubleByteString )
    ifNil:[ ^ self _convertToQuadByte insertAll: aCharOrCharColl at: anIndex]
    ifNotNil:[ ^ self insertAll: aString at: anIndex ].
].
aCharOrCharColl _validateClasses: { AbstractCharacter . CharacterCollection }.
^self insertAll: aCharOrCharColl asDoubleByteString at: anIndex.
%

category: 'Copying'
method: DoubleByteString
replaceFrom: startIndex to: stopIndex with: charCollection startingAt: repIndex

"Replaces the elements of the receiver between the indexes startIndex and
 stopIndex inclusive with the elements of aSeqCollection starting at repIndex.
 If aSeqCollection is identical to the receiver, the source and
 destination blocks may overlap.

 Returns the receiver."

<primitive: 439>
| cSize argInfo recInfo src16 |
startIndex _isSmallInteger ifFalse:[ startIndex _validateClass: SmallInteger ].
stopIndex _isSmallInteger ifFalse:[ stopIndex _validateClass: SmallInteger ].
repIndex _isSmallInteger ifFalse:[ repIndex _validateClass: SmallInteger ].
argInfo := charCollection _stringCharSize .
recInfo := self _stringCharSize .
(recInfo bitAnd: 16r8) ~~ 0 ifTrue:[  "receivier is a Utf string"
  (cSize := argInfo bitAnd: 16r7) > 1 ifTrue:[
    src16 := charCollection _asUnicode16 .
    (src16 ~~ nil and:[ src16 ~~ charCollection]) ifTrue:[
      ^ self replaceFrom: startIndex to: stopIndex with: src16 startingAt: repIndex
    ].
    ^ self _convertToQuadByte
     replaceFrom: startIndex to: stopIndex with: charCollection startingAt: repIndex
  ].
].
(cSize := argInfo bitAnd: 16r7) == 4 ifTrue:[
   src16 := charCollection _asDoubleByteString .
   (src16 ~~ nil and:[ src16 ~~ charCollection]) ifTrue:[
      ^ self replaceFrom: startIndex to: stopIndex with: src16 startingAt: repIndex
   ].
   ^ self _convertToQuadByte replaceFrom: startIndex to: stopIndex
                        with: charCollection startingAt: repIndex
].
^ super replaceFrom: startIndex to: stopIndex with: charCollection startingAt: repIndex
%

category: 'Accessing'
method: DoubleByteString
size

"Returns the number of Characters in the receiver."

^ self _basicSize // 2.
%

category: 'Updating'
method: DoubleByteString
size: anInteger

"Changes the size of the receiver to anInteger.

 If anInteger is less than the current size of the receiver, the receiver is
 shrunk accordingly.  If anInteger is greater than the current size of the
 receiver, the receiver is extended and new elements are initialized to
 codePoint zero."

^ super _basicSize: anInteger + anInteger
%

category: 'Private'
method: DoubleByteString
_basicAt: anInteger put: aValue

"Disallowed"
self shouldNotImplement: #_basicAt:put:
%

category: 'Private'
method: DoubleByteString
_basicSize: anInteger

"Disallowed"
self shouldNotImplement: #_basicSize:
%

category: 'Converting'
method: DoubleByteString
_convertToQuadByte

"Change the receiver from a DoubleByteString to a QuadByteString,
 or from a Unicode16 to Unicode32.  Changes the
 class of the receiver and the receiver's storage structure.
 Returns the receiver."

<primitive: 445>
%

! Class extensions for 'DoubleByteSymbol'

!		Class methods for 'DoubleByteSymbol'

removeallmethods DoubleByteSymbol
removeallclassmethods DoubleByteSymbol

category: 'Class Membership'
classmethod: DoubleByteSymbol
isSubclassOf: aClassHistoryOrClass

  (self _classHistoryIncludesIdentical: aClassHistoryOrClass) ifTrue:[ ^ true].
  ^ super isSubclassOf: aClassHistoryOrClass
%

category: 'Storing and Loading'
classmethod: DoubleByteSymbol
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into a new instance of the receiver.
 Returns the new instance."

| theSize str marker result |

"since Symbols can't refer to other objects, the 'hasRead:' message
 may be sent after values have been filled in."

theSize := passiveObj readSize .
str := String new: theSize .
marker := passiveObj objectPositionMarker .
passiveObj next: theSize bytesTo: str ;
           next .
result :=  (str _changeClassToMultiByte: DoubleByteString) asSymbol .
passiveObj hasRead: result marker: marker .
^ result
%

category: 'Instance Creation'
classmethod: DoubleByteSymbol
migrateNew

"Override default migrateNew behavior with #_basicNew."

^ self _basicNew
%

category: 'Instance Creation'
classmethod: DoubleByteSymbol
new

"Disallowed.  To create a new DoubleByteSymbol, use the class method
 #withAll: instead."

self shouldNotImplement: #new
%

category: 'Instance Creation'
classmethod: DoubleByteSymbol
new: size

"Disallowed.  To create a new DoubleByteSymbol, use the class method #withAll:
 instead."

self shouldNotImplement: #new:
%

category: 'Instance Creation'
classmethod: DoubleByteSymbol
withAll: aString

"Returns a canonical symbol that has the same Characters as aString."

<primitive: 300>
^ Symbol withAll: aString
%

category: 'Instance Creation'
classmethod: DoubleByteSymbol
_basicNew
^ self shouldNotImplement: #_basicNew
%

category: 'Instance Creation'
classmethod: DoubleByteSymbol
_basicNew: aSize
^ self shouldNotImplement: #_basicNew:
%

category: 'Class Membership'
classmethod: DoubleByteSymbol
_classHistoryIncludesIdentical: aClass
  ^ aClass == Symbol or:[ aClass == DoubleByteSymbol or:[ aClass == QuadByteSymbol]]
%

category: 'Instance Creation'
classmethod: DoubleByteSymbol
_existingWithAll: aString

"Return an existing canonical symbol that has the same value as 'aString'.
 If no such Symbol or DoubleByteSymbol already exists, returns nil."

<primitive: 310>
^ Symbol _existingWithAll: aString
%

category: 'Instance Creation'
classmethod: DoubleByteSymbol
_newString

"Return a new instance of DoubleByteString."

^ DoubleByteString new
%

category: 'Instance Creation'
classmethod: DoubleByteSymbol
_newString: aSize

"Return a new instance of DoubleByteString of the specified size."

^ DoubleByteString new: aSize
%

!		Instance methods for 'DoubleByteSymbol'

category: 'Concatenating'
method: DoubleByteSymbol
, aCharOrCharCollection

"Returns a new instance of DoubleByteString that contains the elements of
 the receiver followed by the elements of aCharOrCharCollection.
 A DoubleByteString is returned rather than a DoubleByteSymbol to avoid
 the expense of unnecessary creation and canonicalization of Symbols."

| result |

result := DoubleByteString withAll: self .
result  addAll: aCharOrCharCollection.
^ result
%

category: 'Converting'
method: DoubleByteSymbol
-> anObject

"Returns a SymbolAssociation with the receiver as the key and the given object
 as the value."

^ SymbolAssociation newWithKey: self value: anObject
%

category: 'Comparing'
method: DoubleByteSymbol
= anObject

"Returns true if anObject is equal to the receiver.  Since symbols
 are canonicalized, this method does the check based on the
 identities of the receiver and the argument."

^ self == anObject.
%

category: 'Compatiblity'
method: DoubleByteSymbol
argumentCount

 ^ self numArgs
%

category: 'Converting'
method: DoubleByteSymbol
asDoubleByteString

"Returns a copy of the receiver as a DoubleByteString."

^ DoubleByteString withAll: self.
%

category: 'Converting'
method: DoubleByteSymbol
asString

"Returns a copy of the receiver as a String."

^ String withAll: self.
%

category: 'Converting'
method: DoubleByteSymbol
asSymbol

"Returns the receiver.  All Symbols and DoubleByteSymbols are canonical."

^ self
%

category: 'Converting'
method: DoubleByteSymbol
asSymbolKind

"Returns the receiver.  All Symbols and DoubleByteSymbols are canonical."

^ self
%

category: 'Clustering'
method: DoubleByteSymbol
cluster

"Has no effect.  Clustering of DoubleByteSymbols is only performed by
 the clusterAllSymbols method in class System . "

^ true
%

category: 'Clustering'
method: DoubleByteSymbol
clusterInBucket: aClusterBucketOrId

"Has no effect.  Clustering of Symbols is only performed by
 the clusterAllSymbols method in class System . "

^ true
%

category: 'Copying'
method: DoubleByteSymbol
copy

"Returns self.  Copies of (canonical) double byte symbols are not allowed."

^ self
%

category: 'Copying'
method: DoubleByteSymbol
copyReplacing: oldObject withObject: newObject
	"Returns a String comprising a copy of the receiver in which all occurrences
	 of objects equal to oldObject have been replaced by newObject."

	^self asDoubleByteString copyReplacing: oldObject withObject: newObject
%

category: 'Comparing'
method: DoubleByteSymbol
hash

"Returns a numeric hash key for the receiver."

^ self identityHash
%

category: 'Converting'
method: DoubleByteSymbol
keywords

"Returns an Array of the keywords in the receiver, treating
 any colon-delimited segment as if it was a legal keyword."

^ Symbol _keywords: self
%

category: 'Testing'
method: DoubleByteSymbol
precedence

"Returns the precedence of the receiver, were it a message selector, with
 1=unary, 2=binary and 3=keyword."

^ self isInfix
    ifTrue: [ 2 ]
    ifFalse: [ self isKeyword ifTrue: [ 3 ]
                              ifFalse: [ 1 ] ]
%

category: 'Formatting'
method: DoubleByteSymbol
printOn: aStream

"Puts a displayable representation of the receiver on the given stream.
 That representation conforms to GemStone Smalltalk parsing rules."

aStream nextPut: $# .
super printOn: aStream
%

category: 'Copying'
method: DoubleByteSymbol
shallowCopy

"Returns self.  Copies of (canonical) double byte symbols are not allowed."

^ self
%

category: 'Class Membership'
method: DoubleByteSymbol
species

"Returns a class similar to, or the same as, the receiver's class which
 can be used for containing derived copies of the receiver."

^ DoubleByteString
%

category: 'Private'
method: DoubleByteSymbol
speciesForConversion

^ DoubleByteString .
%

category: 'Formatting'
method: DoubleByteSymbol
withNoColons

"Returns a String containing the value of the receiver with all colons removed.

 A String is returned rather than a symbol to avoid the expense of unnecessary
 creation and canonicalization of symbols."

^ self copyWithout: $:   .
%

category: 'Decompiling without Sources'
method: DoubleByteSymbol
_asSource

| result |
result := String new .
result add: $#  .
result addAll: super _asSource .
^ result
%

category: 'New Indexing Comparison'
method: DoubleByteSymbol
_idxForCompareEqualTo: arg

""

^arg _idxForCompareEqualToDoubleByteSymbol: self
%

category: 'New Indexing Comparison'
method: DoubleByteSymbol
_idxForCompareEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ aCharacterCollection _idxForCompareEqualToDoubleByteSymbol: self
%

category: 'New Indexing Comparison - for Compare'
method: DoubleByteSymbol
_idxForCompareEqualToCharacterCollection: aCharacterCollection
  "second half of a double dispatch call from CharacterCollection>>_idxForCompareEqualTo:."

  ^ false
%

category: 'New Indexing Comparison - for Compare'
method: DoubleByteSymbol
_idxForCompareEqualToUnicode: aUnicodeString collator: aCollator
"second half of a double dispatch call from CharacterCollection>>_idxForCompareEqualTo:collator:."

  ^ false
%

category: 'New Indexing Comparison'
method: DoubleByteSymbol
_idxForSortEqualTo: arg
  ""

  ^ arg _idxForSortEqualToSymbol: self
%

category: 'New Indexing Comparison'
method: DoubleByteSymbol
_idxForSortEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ self asString
    _idxForSortEqualTo: aCharacterCollection
    collator: anIcuCollator
%

category: 'New Indexing Comparison'
method: DoubleByteSymbol
_idxForSortEqualToSymbol: aSymbol
  "second half of a double dispatch call from Symbol>>_idxForSortEqualTo:."

  ^ self asString _idxPrimCompareEqualTo: aSymbol asString
%

category: 'New Indexing Comparison'
method: DoubleByteSymbol
_idxForSortNotEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ (self _idxForSortEqualTo: aCharacterCollection collator: anIcuCollator) not
%

category: 'Comparing'
method: DoubleByteSymbol
~= aCharCollection

"This method can be optimized for symbols since they are canonical."

^ self ~~ aCharCollection
%

! Class extensions for 'EndOfStream'

!		Instance methods for 'EndOfStream'

removeallmethods EndOfStream
removeallclassmethods EndOfStream

category: 'Instance initialization'
method: EndOfStream
initialize
  gsNumber := ERR_EndOfStream.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'EqualityCollisionBucket'

!		Instance methods for 'EqualityCollisionBucket'

removeallmethods EqualityCollisionBucket
removeallclassmethods EqualityCollisionBucket

category: 'Searching'
method: EqualityCollisionBucket
at: aKey put: aValue keyValDict_coll: aKeyValDict
 "Stores the aKey/aValue pair in the receiver.
  Returns self size if this at:put: added a new key, 0 if this at:put:
  replaced the value of an existing key."
 | emptySlotIdx startTableSize thisKey numElem |

  startTableSize := self tableSize .
  aKey ifNil:[ ^ self _error: #rtErrNilKey ] .
  (numElem := numElements) == 0 ifTrue:[
    emptySlotIdx := 1
  ] ifFalse:[ | idx |
    "search for aKey, or for the first empty slot "
    idx := 1 .
    1 to: startTableSize do:[:n |
      thisKey := self _at: idx . "inline keyAt:"
      thisKey ifNotNil:[
        (aKey = thisKey) ifTrue:[ "Key found.  Store given value"
          self _at: idx + 1 put: aValue .  "inline at:putValue:"
          aKeyValDict _markDirty .
          ^ 0
        ].
      ] ifNil:[
        emptySlotIdx ifNil:[ emptySlotIdx := idx ].
      ].
      idx := idx + 2 .
    ] .
    "Key not found so add key and value"
    emptySlotIdx ifNil:[ " bucket is full so grow it "
      emptySlotIdx := self _basicSize + 1 .
      self size: emptySlotIdx + 7  .  "accommodate 4 more key,value pairs"
    ] .
  ].
  numElem := numElem + 1 .
  numElements := numElem .
  self _at: emptySlotIdx put: aKey. "inline at:putKey:"
  self _at: emptySlotIdx + 1 put: aValue . "inline at:putValue"
  ^ numElem
%

category: 'Private'
method: EqualityCollisionBucket
compareKey: key1 with: key2

"Not used by implementation of EqualityCollisionBucket"

self shouldNotImplement: #compareKey:with:
%

category: 'Searching'
method: EqualityCollisionBucket
searchForKey: argKey

"Returns the index of argKey, or if not found, nil."
   | idx |
   argKey ifNil:[ ^ self _error: #rtErrNilKey ] .
   idx := 1 .
   1 to: self tableSize do: [ :n | | aKey |
      aKey := self _at: idx . "inline keyAt:"
      aKey ifNotNil:[
        argKey = aKey ifTrue:[ ^ n ].
      ].
      idx := idx + 2
   ].
   ^ nil "Key not found"
%

! Class extensions for 'Error'

!		Class methods for 'Error'

removeallmethods Error
removeallclassmethods Error

category: 'Signaling'
classmethod: Error
signalFatalSigTerm

  "Terminates the session with a Fatal error 4052.  Allows
   Smalltalk or Ruby handler to implement SIGTERM semantics."

  ^ self new _number: GS_ERR_SIGTERM ; signalNotTrappable
%

!		Instance methods for 'Error'

category: 'Instance initialization'
method: Error
initialize
  gsNumber := ERR_Error.
  gsResumable := false .
  gsTrappable := true .
%

! Class extensions for 'Exception'

!		Instance methods for 'Exception'

removeallmethods Exception
removeallclassmethods Exception

category: 'Instance initialization'
method: Exception
initialize
  gsNumber := ERR_Exception.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'ExceptionSet'

!		Class methods for 'ExceptionSet'

removeallmethods ExceptionSet
removeallclassmethods ExceptionSet

category: 'instance creation'
classmethod: ExceptionSet
with: selector1 with: selector2
  "Private - Answer a new instance of the receiver containing
  the <exceptionSelector> arguments, selector1 and selector2"

  | res |
  res := self new .
  res , selector1 .
  res , selector2 .
  ^ res
%

!		Instance methods for 'ExceptionSet'

category: 'ANSI - exceptionSelector'
method: ExceptionSet
, anotherExceptionOrSet
  "Return an exception set that contains all of the exception selectors in both
  the receiver and the argument.
  This is commonly used to specify a set of exception selectors for an
  'exception handler.'

  The argument must be a subclass of Exception or a kind of ExceptionSet .
  Returns the receiver."

  anotherExceptionOrSet _isExceptionClass ifTrue:[
    self at: self size + 1 put: anotherExceptionOrSet  . "inline self add:"
    ^ self
  ].
  (anotherExceptionOrSet isKindOf: ExceptionSet) ifTrue:[ | sz |
    sz := self size .
    1 to: anotherExceptionOrSet size do:[:j |
      "inline self add: "
      sz := sz + 1 .
      self at: sz put: (anotherExceptionOrSet at:j)
    ].
  ].
  anotherExceptionOrSet _validateClass: ExceptionSet.
%

category: 'ANSI support'
method: ExceptionSet
add: anException

 ^self at: self size + 1 put: anException
%

category: 'ANSI support'
method: ExceptionSet
at: anOffset put: anException

 anException _isExceptionClass ifFalse:[
   anException _error: #rtErrBadArgKind args: { Error }.
   self _uncontinuableError .
 ].
 ^ super at: anOffset put: anException
%

category: 'ANSI - exceptionSelector'
method: ExceptionSet
handles: anExceptionClass

  1 to: self size do:[:j |
    ((self at: j) handles: anExceptionClass) ifTrue:[ ^ true ] .
  ].
  ^ false
%

category: 'initializing'
method: ExceptionSet
initialize

  ^ self
%

category: 'accessing'
method: ExceptionSet
selectors
  "Return the list of Exceptions contained in the receiver."
  | res |
  res := OrderedCollection new: self size .
  1 to: self size do:[:j |
    res at: j put: (self at: j) .
  ].
  ^ res
%

category: 'Exceptions'
method: ExceptionSet
selectors: anOrderedCollection
  "Store all elements of anOrderedCollection into receiver's varying instVars,
   growing or shrinking receiver to match size of anOrderedCollection."

  self size: anOrderedCollection size .
  1 to: anOrderedCollection size do:[:j |
    self at: j put: (anOrderedCollection at:j)
  ].
%

! Class extensions for 'ExecBlock'

!		Class methods for 'ExecBlock'

removeallmethods ExecBlock
removeallclassmethods ExecBlock

category: 'Storing and Loading'
classmethod: ExecBlock
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into a new instance of the receiver.
 Returns the new instance."

| src result marker meth o symbolList |

"Returns a new instance of the receiver read from the given PassiveObject"

marker := passiveObj objectPositionMarker.
src := passiveObj readObject.
symbolList := GsCurrentSession currentSession symbolList .
meth := src _compileInContext: (o := Object new) symbolList: symbolList.
result := meth _executeInContext: o.
(result == nil or: [(result _isExecBlock) not]) ifTrue: [
  "error in compiling"
  self _halt: 'Error in recreating a ' , name.
  ^nil
  ]
ifFalse: [
  passiveObj hasRead: result marker: marker.
  ^result
  ]
%

category: 'Accessing'
classmethod: ExecBlock
_cost

"cost is not define at the class level, each instance
 has its own cost ."

self shouldNotImplement: #_cost
%

!		Instance methods for 'ExecBlock'

category: 'Debugging Support'
method: ExecBlock
argsAndTemps

"Returns an Array of Symbols which are the names of arguments and
 temporaries for this block,  not including inner blocks"

^ self method argsAndTemps
%

category: 'Accessing'
method: ExecBlock
argumentCount

  "Returns a SmallInteger, the number of arguments to the receiver.
   argumentCount is provided to conform to ANSI Smalltalk."
  <primitive: 458>
  self _primitiveFailed: #argumentCount.
  ^0.
%

category: 'Updating'
method: ExecBlock
at: anIndex put: anObj
  "Disallowed"
  self shouldNotImplement: #at:put:
%

category: 'Clustering'
method: ExecBlock
clusterDepthFirst

 self cluster ifTrue:[ ^ true ].
 self method cluster .   "cannot use instVarAt to access"
        "other instVars are specials or transient"
 ^ false
%

category: 'Block Evaluation'
method: ExecBlock
cull: anArg
  "Return the value of the receiver evaluated with 0 or 1 arguments. If the block
   expects 1 argument pass anArg as the value of the argument"

  ^ self argumentCount == 0
    ifTrue: [ self value ]
    ifFalse: [ self value: anArg ]
%

category: 'Block Evaluation'
method: ExecBlock
cull: firstArg cull: secondArg
  "Return the value of the receiver evaluated with between 0 and 2 arguments,
   discarding arguments not needed by the receiver."

  | nargs |
  (nargs := self argumentCount) < 2 ifTrue:[
     nargs == 1 ifTrue:[ ^ self value: firstArg ].
     ^ self value .
  ].
  ^ self value: firstArg value: secondArg
%

category: 'Block Evaluation'
method: ExecBlock
cull: firstArg cull: secondArg cull: thirdArg
  "Return the value of the receiver evaluated with between 0 and 3 arguments,
   discarding arguments not needed by the receiver."

  | nargs |
  (nargs := self argumentCount) < 3 ifTrue:[
     nargs == 2 ifTrue:[ ^ self value: firstArg value: secondArg ].
     nargs == 1 ifTrue:[ ^ self value: firstArg ].
     ^ self value .
  ].
  ^ self value: firstArg value: secondArg value: thirdArg
%

category: 'Block Evaluation'
method: ExecBlock
cull: firstArg cull: secondArg cull: thirdArg cull: fourthArg
  "Return the value of the receiver evaluate with between 0 and 4 arguments,
   discarding arguments not needed by the receiver."

  | nargs |
  (nargs := self argumentCount) <= 2 ifTrue:[
     nargs == 2 ifTrue:[ ^ self value: firstArg value: secondArg ].
     nargs == 1 ifTrue:[ ^ self value: firstArg ].
     ^ self value .
  ].
  nargs == 3 ifTrue:[
     ^ self value: firstArg value: secondArg value: thirdArg .
  ].
  ^ self value: firstArg value: secondArg value: thirdArg value: fourthArg.
%

category: 'Block Evaluation'
method: ExecBlock
ensure: aBlock
"Evaluate the receiver.
 Evaluate aBlock after evaluating the receiver,
 or before any return from a block that would return to the sender.
 Returns result of evaluating the receiver.

 aBlock must be a zero-arg instance of ExecBlock, otherwise an
 error is generated. "

<primitive: 2017>  "marks frame with ENSURE_Mark_NIL, always fails"
| result |
result := self value .  "execute the receiver"
self _removeEnsure ifNotNil:[:b | b value "normal execution of argument"].
^ result
%

category: 'Processes - Blue Book'
method: ExecBlock
fork
  "forks the receiver as a new process at the current scheduling priority"

  ^ GsProcess _forkBlock: self with: #() env: 0 prio: nil
%

category: 'Processes - Blue Book'
method: ExecBlock
forkAt: priority
  "forks the receiver as a new process at the given priority"

  | proc |
  proc := GsProcess _forkBlock: self with: #() env: 0 prio: priority .
  ^ proc
%

category: 'Processes'
method: ExecBlock
forkAt: priority with: blockArgs
  "forks the receiver as a new process at the given priority"

  | proc |
  proc := GsProcess _forkBlock: self with: blockArgs env: 0 prio: priority.
  ^ proc
%

category: 'Processes'
method: ExecBlock
forkWith: blockArgs
  "forks the receiver as a new process at the current scheduling priority"

  ^ GsProcess _forkBlock: self with: blockArgs env: 0 prio: nil
%

category: 'Accessing'
method: ExecBlock
instVarAt: anIndex

  anIndex == 1 ifTrue:[
    "translate GsNativeCode to GsNMethod if needed"
    ^ self method
  ].
  ^ super instVarAt: anIndex
%

category: 'Accessing'
method: ExecBlock
isSimple

^ self _cost = 1
%

category: 'Accessing'
method: ExecBlock
lastRubyArgIsStar
  ^ (iFields1 bitAnd: 16r100) ~~ 0
%

category: 'Accessing'
method: ExecBlock
method
  "Returns the GsNMethod containing the code for this block.

   A primitive is needed here because the value of the _method instVar
   can be an instance of GsNativeCode for an in-memory block."

<primitive: 492>
self _primitiveFailed: #method .
%

category: 'Repository Conversion'
method: ExecBlock
needsRecompile

"Returns true if the receiver's block needs recompilation, false otherwise."

^  self method needsRecompile
%

category: 'Processes - Blue Book'
method: ExecBlock
newProcess
  "creates a new suspended process holding the receiver"

  ^ GsProcess _newForBlock: self with: #() env: 0
%

category: 'Processes - Blue Book'
method: ExecBlock
newProcessWith: argArray
  "creates a new suspended process holding the receiver to be evaluated
   with the given arguments"

  ^GsProcess _newForBlock: self with: argArray env: 0
%

category: 'Accessing'
method: ExecBlock
noRubyDeclaredArgs
  ^ (iFields1 bitAnd: 16r200) ~~ 0
%

category: 'Accessing'
method: ExecBlock
numArgs

  "Returns a SmallInteger, the number of arguments to the receiver.
   numArgs is GemStone legacy API. "
  <primitive: 458>
  self _primitiveFailed: #numArgs .
  ^ 0
%

category: 'Accessing'
method: ExecBlock
numberArgs

  "Returns a SmallInteger, the number of arguments to the receiver."
  self deprecated: 'ExecBlock>>numberArgs deprecated v3.2; use #argumentCount'.
  ^ self argumentCount
%

category: 'Accessing'
method: ExecBlock
numberTemps

^ (iFields1 bitShift: -12) bitAnd: 16rFFFF
%

category: 'Updating'
method: ExecBlock
objectSecurityPolicy: anObjectSecurityPolicy

"Assign the receiver to the given security policy.
 For complex blocks, also changes the security policy for variable contexts."

| vc |
super objectSecurityPolicy: anObjectSecurityPolicy .
(vc := staticLink) ifNotNil:[ vc objectSecurityPolicy: anObjectSecurityPolicy ].
%

category: 'Block Evaluation'
method: ExecBlock
on: exceptionSelector do: handlerBlock
 "Try to evaluate the receiver, which should be a zero-argument block.
  If an exception occurs and the expression
      exceptionSelector handles: theExceptionInstance
  returns true, then evaluate the one argument block handlerBlock ,
  passing it the exception instance as its argument.

  Two forms are supported directly by the VM, and the 'fast path code'
  below is used.
    (1) on: anException do: handlerBlock
    (2) on: anExceptionSet do: handlerBlock
  A third form is handled by Smalltalk code in the body of this method,
  and for this form only, #handles is sent to anObject to determine
  whether an exception should be handled .
    (3) on: anObject     do: handlerBlock

  anException must be the class Exception  or a subclass thereof ;
  anExceptionSet must be a kind of ExceptionSet;
  handlerBlock must be an instance of ExecBloc otherwise an error is generated.

  For forms 1,2,3 if handlerBlock expects more than 1 argument,
  an error is generated if exception handling attempts to
  invoke that handlerBlock.

  If handlerBlock is invoked to handle an Exception which occurs during
  execution of the receiver and handlerBlock completes normally , then
  the result of handlerBlock (value of last expression in handlerBlock)
  will be the result of the on:do: send .  Other-than-normal
  completion of handlerBlock is available by use of
  Exception's instance methods such as
    #return  #return:  #retry #retryUsing: #resume #resume: #pass #outer
  within handlerBlock

  For forms 1 and 2, when searching for a handlerBlock to handle a signaled Exception,
  the VM uses Behavior>>_subclassOf: semantics . classHistories of the
  class of the signaled Exception and of anException or elements of anExceptionSet
  are ignored.
"

<primitive: 2030> "always fails"
| fastPath | "fastPath := true by primitive if form 1 or 2 detected"
             "fastPath := nil  by primitive if form 3 detected."
fastPath ifNotNil:[ "fast path code"
 ^ self value
].
"Any changes to this method's code before this line may also
 require changes to code in comgen.c conditional on BcPrim_ENTER_onDo .
"
^ self onException: AbstractException do:[:ex |
    (exceptionSelector handles: ex) ifTrue:[
      handlerBlock argumentCount == 0
         ifTrue:[ handlerBlock value ]
        ifFalse:[ handlerBlock value: ex ]
    ] ifFalse:[
      ex pass
    ]
  ]
%

category: 'Block Evaluation - Private'
method: ExecBlock
onException: anException do: handlerBlock
 "An optimized form of on:do: .  anException may not be any
  object which responds to  handles:

  Private.

  Try to evaluate the receiver, which should be a zero-argument block.
  If an exception occurs which is matched by anExceptionClass,
  evaluate the one argument block handlerBlock , passing it the exception
  instance as its argument.

  These forms are supported:
    (1) on: anException do: handlerBlock
    (2) on: anArray     do: handlerBlock
    (3) on: anExceptionSet do: handlerBlock
    (4) on: anArray     do: anArrayOfBlocks

   This form is not supported
    (5) on: anObject     do: handlerBlock

  anException must be the class Exception  or a subclass thereof ;
  anExceptionSet must be a kind of ExceptionSet;
  anArray must be a kind of Array ;
  handlerBlock must be an instance of ExecBloc ; if any of these
  is violated, the send of on:do: will fail with an error.

  anArrayOfBlocks must be an Array containing instances of ExecBloc ,
  the elements are not checked during the on:do: send , and are only
  checked if they are attempted to be evaluated during exception handling.

  In forms 2,3,4 elements of anArray or anExceptionSet which
  are not a kind of Exception, or an Array containing kinds of Exception,
  will silently have no effect on catching exceptions.

  In form 4, elements of anArray may in turn be Arrays of Exceptions.
  In form 4, the offset of the first element of anArray containing a
  match to a signaled exception defines the offset in anArrayOfBlocks
  at which to find the handlerBlock to invoke.

  For forms 1,2,3 if handlerBlock expects more than 1 argument,
  an error is generated if exception handling attempts to
  invoke that handlerBlock.

  For form 4, elements of anArrayOfBlocks must be instances of ExecBlock,
  otherwise the corresponding entry in anArray will silently have no effect
  on catching exceptions.  If an element of anArrayOfBlocks is an ExecBlock
  taking more than 1 argument, an error is generated if exception
  handling attempts to invoke that block.

  If handlerBlock is invoked to handle an Exception which occurs during
  execution of the receiver and handlerBlock completes normally , then
  the result of handlerBlock (value of last expression in handlerBlock)
  will be the result of the on:do: send .  Other-than-normal
  completion of handlerBlock is available by use of
  Exception's instance methods such as
    #return  #return:  #retry #retryUsing: #resume #resume: #pass #outer
  within handlerBlock

  When searching for a handlerBlock to handle a signaled Exception, the VM
  uses Behavior>>_subclassOf: semantics . classHistories of the
  class of the signaled Exception and of anException or elements of anExceptionSet
  are ignored.
"

<primitive: 2018> "marks frame with Exception_Mark_NIL, always fails"
^ self value
%

category: 'Block Evaluation - Private'
method: ExecBlock
onSynchronous: anExceptionClass do: handlerBlock

"Same as onException:do: , except that asynchronous Exceptions
 (those exceptions which have trappable==1) will never be handled
 by handlerBlock .

 Allows asynchronous Exceptions to be handled by a higher
 level handler, even if the async Exception would match anExceptionClass.

 Private.
"

<primitive: 2029>
^ self value
%

category: 'Repository Conversion'
method: ExecBlock
recompile
  "Attempts to recompile the receiver's method assuming it is a simple block
   without references to self."

| src newBlk doitMeth litVars mask |
self needsRecompile ifFalse:[ ^ self "recompile not needed"].
staticLink ifNotNil:[
  Error signal:'cannot recompile a complex block with reference to home VariableContext'.
].
ccallin ifNotNil:[
  Error signal:'cannot recompile a block for a CCallin'.
].
src := self _sourceString .
litVars := self method _literalVariablesForRecompile .
doitMeth := src _compileInContext: nil
       symbolList: GsSession currentSession symbolList
       oldLitVars: litVars environmentId: 0 flags: 4"GCI_COMPILE_evaluateWithoutSelf".
newBlk := doitMeth _executeInContext: nil .
mask := 16r40000ffff0ff . "nTemps, nArgs, isCopying"
(newBlk _iFields1 bitAnd: mask) = (iFields1 bitAnd: mask) ifFalse:[
  Error signal:'in recompile: iFields1 differs, newBlock 16r',
     newBlk _iFields1 asHexString , ' old 16r', iFields1 asHexString .
].
self _unsafeAt: 1 put: newBlk method .  "_method := newBlk method"
%

category: 'Flow of Control'
method: ExecBlock
repeat
"(Reserved selector.)  Evaluate the receiver repeatedly, ending only if the
 block forces some stopping condition.

 The following is a control structure optimization, not a recursive send."

 [ self value] repeat
%

category: 'Accessing'
method: ExecBlock
selfOffsetInVC

" Return a one-based offset to self for use as arg to primitiveAt:
  If there is no reference to self in the receiver,
  returns  1 , otherwise returns a value > VAR_CONTEXT_NAMED_SIZE"

^ ((iFields1 bitShift: -28) bitAnd: 16rFFFF) + 1

%

category: 'Accessing'
method: ExecBlock
selfValue

"return self in an active copy , or nil if self is not referenced."
| aVc ofs |
aVc := staticLink .
aVc ifNotNil:[ | vcOfs |
  vcOfs := self selfOffsetInVC .
  vcOfs > 2"VAR_CONTEXT_NAMED_SIZE" ifTrue:[
    [ aVc parent ~~ nil ] whileTrue:[ aVc := aVc parent ].
    vcOfs > aVc basicSize ifTrue:[ ^ nil ].
    ^ aVc _primitiveAt: vcOfs
  ].
].
ofs := self _selfOffsetInSelf .
ofs ~~ 0 ifTrue:[
  ofs := ofs - 1 .
  ^ self _primitiveAt: ofs .
].
^ nil
%

category: 'Updating'
method: ExecBlock
size: anInteger
  "Disallowed"
  self shouldNotImplement: #size:
%

category: 'Accessing'
method: ExecBlock
staticLink

^ staticLink
%

category: 'Flow of Control'
method: ExecBlock
untilFalse

"(Reserved selector.)  Evaluates the receiver repeatedly until the evaluation's
 result is false.  Return nil.  Generates an error if the receiver is not a
 zero-argument block."

"The following is a control structure optimization, not a recursive send."

^ [self value] untilFalse
%

category: 'Flow of Control'
method: ExecBlock
untilFalse: aBlock

"(Reserved selector.)  Evaluates the receiver once ,
 and then repeats the evaluation while aBlock returns true .
 Returns nil.  Generates an error if the receiver is not a
 zero-argument block."

"The following is a control structure optimization, not a recursive send."

^ [self value] untilFalse:[ aBlock value]
%

category: 'Flow of Control'
method: ExecBlock
untilTrue

"(Reserved selector.)  Evaluates the receiver repeatedly until the evaluation's
 result is true.  Return nil.  Generates an error if the receiver is not a
 zero-argument block."

"The following is a control structure optimization, not a recursive send."

^ [self value] untilTrue
%

category: 'Flow of Control'
method: ExecBlock
untilTrue: aBlock

"(Reserved selector.)  Evaluates the receiver once ,
 and then repeats the evaluation while aBlock returns false .
 Returns nil.  Generates an error if the receiver is not a
 zero-argument block."

"The following is a control structure optimization, not a recursive send."

^ [self value] untilTrue:[ aBlock value]
%

category: 'Block Evaluation'
method: ExecBlock
value

"Return the value of the receiver evaluated with no arguments.
 If the block expects any arguments, an error is generated.

 #value is optimized by the compiler.
 This method is in the image for use by perform: and for
 failure paths from the optimized bytecode "

^ self valueWithArguments: #()
%

category: 'Block Evaluation'
method: ExecBlock
value: anObject

"Return the value of the receiver evaluated with anObject as its argument.  If
 the block expects a different number of arguments, an error is generated.

 #value: is optimized by the compiler.
 This method is in the image for use by perform: and for
 failure paths from the optimized bytecode "


^ self valueWithArguments: { anObject }
%

category: 'Block Evaluation'
method: ExecBlock
value: anArg on: anExceptionClass do: handlerBlock
  "Evaluate the one-argument receiver with the specified on:do: .
   Useful for avoiding complex blocks in the sender of value:on:do: "

  ^ [ self value: anArg ] onException: anExceptionClass do: handlerBlock
%

category: 'Block Evaluation'
method: ExecBlock
value: anArg onSynchronous: anExceptionClass do: handlerBlock
  "Evaluate the one-argument receiver with the specified onSynchronous:do: .
   Useful for avoiding complex blocks in the sender of value:onSynchronous:do: "

  ^ [ self value: anArg ] onSynchronous: anExceptionClass do: handlerBlock
%

category: 'Block Evaluation'
method: ExecBlock
value: firstObject value: secondObject

"Return the value of the receiver evaluated with the two objects as its
 arguments.  If the block expects a different number of arguments, an error is
 generated.

 #value:value: is optimized by the compiler.
 This method is in the image for use by perform: and for
 failure paths from the optimized bytecode "

^ self valueWithArguments: {  firstObject . secondObject }
%

category: 'Block Evaluation'
method: ExecBlock
value: firstObject value: secondObject value: thirdObject

"Return the value of the receiver evaluated with the three objects as its
 arguments.  If the block expects a different number of arguments, an error is
 generated.

 #value:value:value: is optimized by the compiler.
 This method is in the image for use by perform: and for
 failure paths from the optimized bytecode "

^ self valueWithArguments: {  firstObject . secondObject . thirdObject }
%

category: 'Block Evaluation'
method: ExecBlock
value: first value: second value: third value: fourth

"Return the value of the receiver evaluated with the four objects as
 its arguments.  If the block expects a different number of arguments,
 an error is generated.

 #value:value:value:value: is optimized by the compiler.
 This method is in the image for use by perform: and for
 failure paths from the optimized bytecode "

^ self valueWithArguments: {  first . second . third . fourth }
%

category: 'Block Evaluation'
method: ExecBlock
value: first value: second value: third value: fourth value: fifth

"Return the value of the receiver evaluated with the five objects as
 its arguments.  If the block expects a different number of arguments,
 an error is generated.

 #value:value:value:value:value: is optimized by the compiler.
 This method is in the image for use by perform: and for
 failure paths from the optimized bytecode "

^ self valueWithArguments: {  first . second . third . fourth . fifth }
%

category: 'Deprecated'
method: ExecBlock
valueNowOrOnUnwindDo: aBlock

self deprecated: 'ExecBlock>>valueNowOrOnUnwindDo: deprecated v3.2; use #ensure:'.
^self _valueNowOrOnUnwindDo: aBlock.
%

category: 'Block Evaluation'
method: ExecBlock
valueWithArguments: argList

"Return the value of the receiver evaluated with the elements of the Array
 argList as arguments.  If the block expects a different number of arguments,
 an error is generated."

<primitive: 2003>  "compiler emits special bytecode"
^ self _primitiveFailed: #valueWithArguments: args: { argList }
%

category: 'Debugging Support'
method: ExecBlock
vcTempNames

"Returns an Array of Symbols which are the names for the temps
 defined in the receiver's VariableContext.  The ordering of names
 in the result matches ordering of temps in the varying instVars
 of the receiver."

| vc allNames allOfs res vcNamedSize myMeth lastTmpOfs |
vc := staticLink .
vc == nil ifTrue:[ ^ #() ].
vcNamedSize := vc class instSize .
allNames := self argsAndTemps .
myMeth := self method .
allOfs := myMeth _argsAndTempsOffsets .
res := Array new: vc size .
lastTmpOfs := allOfs size - myMeth _numCopyingBlockArgs .
self argumentCount + 1 to: lastTmpOfs do:[:j| | anOfs ofs |
  anOfs := allOfs at: j .
  anOfs > 0 ifTrue:[
    (anOfs bitAnd: 16rFF) == 0 ifTrue:[ "lexLevel == 0"
      ofs := anOfs bitShift: -8 .
      res at: (ofs + 1 - vcNamedSize) put:( allNames at: j) .
    ] "else temp is defined in a parent block"
  ] "else temp is on stack"
].
^ res
%

category: 'Flow of Control'
method: ExecBlock
whileFalse
"(Reserved selector.)  Evaluate the receiver once and then repeatedly as long
 as the value returned by the evaluation is false.

 The following is a control structure optimization, not a recursive send."

^ [ self value] whileFalse
%

category: 'Flow of Control'
method: ExecBlock
whileFalse: aBlock

"(Reserved selector.)  Evaluates the zero-argument block aBlock repeatedly
 while the receiver evaluates to false.  Return nil.  Generates an error if the
 receiver is not a zero-argument block."

"The following is a control structure optimization, not a recursive send."

^ [self value] whileFalse:[
    aBlock _isExecBlock ifFalse:[ ArgumentError signal:'expected a block'].
    aBlock value
  ]
%

category: 'Flow of Control'
method: ExecBlock
whileTrue
"(Reserved selector.)  Evaluate the receiver once and then repeatedly as long
 as the value returned by the evaluation is true.

 The following is a control structure optimization, not a recursive send."

^ [ self value] whileTrue
%

category: 'Flow of Control'
method: ExecBlock
whileTrue: aBlock

"(Reserved selector.)  Evaluates the zero-argument block aBlock repeatedly
 while the receiver evaluates to true.  Return nil.  Generates an error if the
 receiver is not a zero-argument block."

"The following is a control structure optimization, not a recursive send."

^ [self value] whileTrue: [
    aBlock _isExecBlock ifFalse:[ ArgumentError signal:'expected a block'].
    aBlock value
  ]
%

category: 'Storing and Loading'
method: ExecBlock
writeTo: aPassiveObject

"Converts the receiver to its passive form and writes that information on
 aPassiveObject.

 Simple blocks can usually be passivated and then reactivated.  Complex blocks
 can be passivated but may have to be massaged to be reactivated.  References
 to 'self' in complex blocks will resolve to an instance of Object when the
 block is activated, and any arguments or temporaries from enclosing scopes
 will be nil."

aPassiveObject writeClass: self class.
aPassiveObject writeObject: self _sourceString; cr
%

category: 'Decompiling without Sources'
method: ExecBlock
_asSource

"return a stripped source representation of the block."

"not implemented yet"
self shouldNotImplement: #_asSource
%

category: 'Updating'
method: ExecBlock
_basicSize: anInteger
  "Disallowed"
  self shouldNotImplement: #_basicSize:
%

category: 'Accessing'
method: ExecBlock
_cost

"return 1 for simple, 2 for complex , 3 for complex with return to home"

^ (iFields1 bitShift: -44) bitAnd: 16rF
%

category: 'Accessing'
method: ExecBlock
_debugSourceForBlock
  | str meth |
  "used by topaz  LIST @<blockObjId>  , and by RubyBridge class"

  meth := self method .
  str := meth _sourceStringForBlock .
  ^ str
%

category: 'Private'
method: ExecBlock
_deepCopyWith: copiedObjDict

"Private.  Used internally to implement deepCopy."

^ self.
%

category: 'Accessing'
method: ExecBlock
_fileAndLine
  "Return an Array, { fileName . lineNumber } ,
   or nil if method  does not have that information.
   Smalltalk methods return nil."

  | mth fl |
  mth := self method .
  (fl := mth homeMethod _fileAndLine) ifNotNil:[
    ^ { (fl at: 1) . (fl at: 2) + mth _lineDeltaForBlock }
  ].
  ^ nil
%

category: 'Private'
method: ExecBlock
_gbsTraversalCallback

"Private.  When GemBuilder Smalltalk traverses an ExecutableBlock, this method
 is called to place the block's source string in the traversal buffer."

^self _sourceString
%

category: 'Repository Conversion'
method: ExecBlock
_iFields1

"Private"

^ iFields1
%

category: 'Accessing'
method: ExecBlock
_isCopyingBlock

^ ((iFields1 bitShift: -46) bitAnd: 1) ~~ 0
%

category: 'Accessing'
method: ExecBlock
_isRubyLambda

^ iFields1 < 0
%

category: 'Private'
method: ExecBlock
_removeEnsure

"The sender's frame is changed from  ensure:[] to  ensure: nil .
 Returns nil or an ensure block that has not been started."
<primitive: 1004>

self _primitiveFailed: #_removeEnsureAtFP:
%

category: 'Accessing'
method: ExecBlock
_selfOffsetInSelf

"result is zero or a positive zero-based absolute offset"

^ (iFields1 bitShift: -48) bitAnd: 16rF
%

category: 'Modification Tracking'
method: ExecBlock
_setModificationTrackingTo: tracker

"Private.

 No modification tracking is required for blocks,
 even if they are not invariant."

^self
%

category: 'Accessing'
method: ExecBlock
_sourceString

"Returns a String that will create a block similar to the receiver when
 the string is compiled.  References to variables in other contexts or
 to the pseudovariable 'self' will not recompile properly if the source
 string is used to create a new block similar to the receiver."

| result |
result := String new.
result addAll: '"This is source for a block.  "
' .

result addAll:' ^ ' ;
       addAll: self method _sourceStringForBlock .
^result
%

category: 'Private'
method: ExecBlock
_sourceStringForExecute

"Returns a String that will create a block similar to the receiver when
 the string is compiled.  References to variables in other contexts or
 to the pseudovariable 'self' will not recompile properly if the source
 string is used to create a new block similar to the receiver.

 Used by GsExternalSession which further formats the string
"

^ self method _sourceStringForBlock .
%

category: 'Deprecated'
method: ExecBlock
_valueNowOrOnUnwindDo: aBlock

self deprecated: 'ExecBlock>>_valueNowOrOnUnwindDo: deprecated v3.2; use #ensure:'.
^self _valueNowOrOnUnwindDoPrim: aBlock.
%

category: 'Deprecated'
method: ExecBlock
_valueNowOrOnUnwindDoPrim: aBlock
"Obsolete in GemStone/64. Use ensure: instead.

 aBlock must be a zero-arg instance of ExecBlock, otherwise an
 error is generated. "

<primitive: 2017>  "marks frame with ENSURE_Mark_NIL, always fails"
| result |
result := self value .  "execute the block"
aBlock value .       "normal execution of the unwind block"
^ result
%

category: 'Private'
method: ExecBlock
_valueOnUnwind

"This method should be invoked only from within the virtual machine. Other
 use from a Smalltalk program will corrupt the Smalltalk execution
 stack."

"Used to implement valueNowOrOnUnwind:"

self value .  "execute the block"
self _gsReturnNoResult "exit from method with no result,
			special selector optimized by compiler"
%

! Class extensions for 'ExternalError'

!		Instance methods for 'ExternalError'

removeallmethods ExternalError
removeallclassmethods ExternalError

category: 'Instance initialization'
method: ExternalError
initialize
  gsNumber := ERR_ExternalError.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'FloatingPointError'

!		Class methods for 'FloatingPointError'

removeallmethods FloatingPointError
removeallclassmethods FloatingPointError

category: 'Private'
classmethod: FloatingPointError
enableAllExceptions

^ self enableExceptions: self _exceptionList
%

category: 'Private'
classmethod: FloatingPointError
enabledExceptions
  "Returns an Array containing zero or more of the Symbols
    #divideByZero #overflow #underflow #invalidOperation #inexactResult
  reflecting the most recent call to either of
     FloatingPointError class>>enableAllExceptions
     FloatingPointError class>>enableExceptions:
  "
  ^ self _enabledBitsAsArray: (self _enableExceptions: -1)  .
%

category: 'Accessing'
classmethod: FloatingPointError
enableExceptions: argument

"Argument may be one of the symbols
   #divideByZero #overflow #underflow #invalidOperation #inexactResult
 or an Array containing zero or more of those Symbols ,
 or nil which means enable none of the exceptions.

 The specified exceptions will be checked for
 after each arithmetic primitive in Float and SmallDouble and if
 any of those execptions occurs, a FloatingPointError will be signaled.
 Overrides the settings of a previous call to enableExceptions: .
 Returns a SmallInteger, the previously enabled exception bits."

 | bits |
 argument ifNil:[ ^ self _enableExceptions: 0 ].
 argument _isSymbol ifTrue:[
   ^ self _enableExceptions: (self _symbolToBit: argument)
 ].
 argument _isArray ifTrue:[
   bits := 0 .
   argument do:[:aSym |
     bits := bits bitOr:( self _symbolToBit: aSym)
   ].
   ^ self _enableExceptions: bits
 ].
 argument _validateClasses: { Array . Symbol }.
%

category: 'Private'
classmethod: FloatingPointError
_checkFpStatus
  "If any of the currently enabled exceptions have occurred,
   clear those bits from the VM state and signal an error.
   Currently enabled exceptions are returned by
     FloatingPointError class >> enabledExceptions .
   This method is called from primitive failure paths in in each floating
   point primitive."

| flags |
flags := self _getExceptions: false .
flags ~~ 0 ifTrue:[
  self new actual: flags ; signal
].
%

category: 'Private'
classmethod: FloatingPointError
_enabledBitsAsArray: bits
  "The result is in sorted order."
  | res |
  res := { } .
  bits ifNotNil:[
    (bits bitAnd: 1) ~~ 0 ifTrue:[ res add: #divideByZero ].
    (bits bitAnd: 16r10) ~~ 0 ifTrue:[ res add: #inexactResult ].
    (bits bitAnd: 8) ~~ 0 ifTrue:[ res add: #invalidOperation ].
    (bits bitAnd: 2) ~~ 0 ifTrue:[ res add: #overflow ].
    (bits bitAnd: 4) ~~ 0 ifTrue:[ res add: #underflow ].
  ].
  ^ res
%

category: 'Private'
classmethod: FloatingPointError
_enableExceptions: aSmallInteger

"aSmallInteger may have one or more bits in the range 16r1F,
 per the constants in FloatingPointError class >> _initializeConstants .
 other bits are ignored.  The specified exceptions will be checked for
 after each arithmetic primitive in Float and SmallDouble and if
 any of those execptions occurs, a FloatingPointError will be signaled.
 Returns the value which was passed in the previous call to this primitive.

 If aSmallInteger == -1 , then returns the positive value
 which was passed in the previous call to this primitive without changing
 the state of the enabled exceptions.
"

<primitive: 122>

aSmallInteger _validateClass: SmallInteger .
self _primitiveFailed: #enableExceptions: args: { aSmallInteger }
%

category: 'Private'
classmethod: FloatingPointError
_exceptionList

^  #( #divideByZero #overflow #underflow #invalidOperation #inexactResult)
%

category: 'Private'
classmethod: FloatingPointError
_getExceptions: aBoolean

"Fetch and clear bits representing floating point exceptions that have occurred
 since last execution of this primitive.  Result is a SmallInteger.

 aBoolean == false  means get bits enabled by the last call to
 FloatingPointError class>>enableExceptions: , and clear all bits.
 aBoolean == true means get and clear all bits .

 The bits in the result are in the range 16r1F and
 are defined by the constants in FloatingPointError class>>_initializeConstants ."

<primitive: 129>

aBoolean _validateClass: Boolean .
self _primitiveFailed: #_getExceptions: args: { aBoolean }
%

category: 'Private'
classmethod: FloatingPointError
_initializeConstants
   "VM changes needed if you change any of these bit definitions.
    Also fix sends of _enableExceptions: in image."
   self _addInvariantClassVar: #divideByZero value: 1 ;
         _addInvariantClassVar: #overflow value: 2  ;
         _addInvariantClassVar: #underflow value: 4  ;
         _addInvariantClassVar: #invalidOperation  value: 8  ;
         _addInvariantClassVar: #inexactResult  value: 16r10 .
%

category: 'Private'
classmethod: FloatingPointError
_symbolToBit: aSymbol

  | bit |
  bit := classVars at: aSymbol otherwise: nil .
  bit ifNil:[ Error signal:'invalid name of a floating point exception bit, ' , aSymbol].
  ^ bit
%

!		Instance methods for 'FloatingPointError'

category: 'Private'
method: FloatingPointError
actual: aValue
  actual := aValue
%

category: 'Formatting'
method: FloatingPointError
asString
  | str arr cls |
  str := 'a ' copy .
  str add: (cls := self class) name .
  arr := cls _enabledBitsAsArray: actual .
  arr size == 0 ifTrue:[
    gsNumber == ERR_LargeIntegerOverflow
      ifTrue:[ str add:' overflow during LargeInteger arithmetic'].
  ] ifFalse:[
    1 to: arr size do:[:n | str add: $  ; add: (arr at: n) ].
  ].
  ^ str .
%

category: 'Accessing'
method: FloatingPointError
exceptionList
 "Return an Array of Symbols , the list of exceptions
  which produced this instance.  The result will contain one or more of
     #divideByZero #overflow #underflow #invalidOperation #inexactResult  .
  and is sorted alphabetically.
 "
  ^ self class _enabledBitsAsArray: actual
%

category: 'Instance initialization'
method: FloatingPointError
initialize
  gsNumber := ERR_FloatingPointError.
  gsResumable := true .
  gsTrappable := true .
  actual := 0 .
%

! Class extensions for 'FloatingPointException'

!		Instance methods for 'FloatingPointException'

removeallmethods FloatingPointException
removeallclassmethods FloatingPointException

category: 'Instance initialization'
method: FloatingPointException
initialize
  gsNumber := ERR_FloatingPointException.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'Fraction'

!		Class methods for 'Fraction'

removeallmethods Fraction
removeallclassmethods Fraction

category: 'Storing and Loading'
classmethod: Fraction
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into a new instance of the receiver.
 Returns the new instance."

| inst num den |
inst := Fraction _basicNew .
passiveObj hasRead: inst.
passiveObj readNamedIV.
num := passiveObj ivValue.
passiveObj readNamedIV.
den := passiveObj ivValue.

passiveObj skipNamedInstVars.

inst _numerator: num denom: den .
^inst.
%

!		Instance methods for 'Fraction'

category: 'Accessing'
method: Fraction
denominator

"Returns the denominator of the receiver."

^ denominator
%

category: 'Accessing'
method: Fraction
instVarAt: anIndex put: aValue

"Disallowed.  You may not change the value of a Fraction."

self shouldNotImplement: #instVarAt:put:
%

category: 'Accessing'
method: Fraction
numerator

"Returns the numerator of the receiver."

^ numerator
%

category: 'Copying'
method: Fraction
postCopy
  ^ self immediateInvariant
%

category: 'Storing and Loading'
method: Fraction
writeTo: passiveObj

"Converts the receiver to its passive form and writes that information on
 passiveObj."

"Reimplemented from Number since the receiver has a non-literal representation."

^super basicWriteTo: passiveObj
%

category: 'Private'
method: Fraction
_numerator: n denom: d
  numerator := n .
  denominator := d .
  ^ self immediateInvariant
%

! Class extensions for 'GciTransportError'

!		Instance methods for 'GciTransportError'

removeallmethods GciTransportError
removeallclassmethods GciTransportError

category: 'Instance initialization'
method: GciTransportError
initialize
  gsNumber := ERR_GciTransportError.
  gsResumable := false .
  gsTrappable := true .
%

! Class extensions for 'GsCurrentSession'

!		Class methods for 'GsCurrentSession'

removeallmethods GsCurrentSession
removeallclassmethods GsCurrentSession

category: 'Instance Creation'
classmethod: GsCurrentSession
currentSession

"Returns the sole instance of GsCurrentSession that represents this login
 session."

<primitive: 311>
self _primitiveFailed: #currentSession
%

category: 'Initialization'
classmethod: GsCurrentSession
initialize
 "If the GciLogin flags contained bit GCI_CLIENT_DOES_SESSION_INIT.
  the VM expects the GCI application to do
    GciPerform(OOP_CLASS_GSCURRENT_SESSION, 'initialize', NULL, 0);
  after a successful login, so that errors in initialize are debuggable.
  Otherwise the VM will do the message send itself during the login.

  topaz passes GCI_CLIENT_DOES_SESSION_INIT. to GciLogin and does the
  GciPerform itself.
 "

 | res |
 (res := self currentSession)
    "No application changes here, they belong in
     the instance method GsCurrentSession >> initialize"
    initialize .
 ^ res
%

category: 'Instance Creation'
classmethod: GsCurrentSession
new

"Disallowed.

 The only instance of GsCurrentSession that is permitted in a session is created
 automatically when a user logs in to GemStone.  Its default SymbolList is a
 copy of the user's SymbolList.  You can obtain that instance by sending the
 message GsSession currentSession."

self shouldNotImplement: #new.
self _uncontinuableError .
%

!		Instance methods for 'GsCurrentSession'

category: 'Transaction Control'
method: GsCurrentSession
abortTransaction

"Rolls back all modifications made to committed GemStone objects and provides
 the session with a new view of the most current committed state of the
 repository.

 These operations are performed whether or not the session was previously in a
 transaction.  If the transaction mode is set to #autoBegin, then a new
 transaction is started.  If the transaction mode is set to #manualBegin, then
 a new transaction is not started."

System abortTransaction
%

category: 'Transaction Control'
method: GsCurrentSession
beginTransaction

"Starts a new transaction for the session.  If the session is already
 in a transaction, aborts the current transaction and starts a new transaction.

 If the session changed any permanent objects without committing them, their
 state is aborted."

System beginTransaction
%

category: 'Session Configuration Access'
method: GsCurrentSession
clientVersionAt: aSymbol

"Returns the value of the GsSession's client version information parameter named
 aSymbol.  Returns nil if no version parameter named aSymbol exists."

^ System clientVersionAt: aSymbol
%

category: 'Transaction Control'
method: GsCurrentSession
commitAndReleaseLocks

"Attempts to commit the transaction for the session.

 This method is the same as 'commitTransaction' except for the handling of
 locks.  If the commit succeeds, this method releases all locks for the session
 and returns true.  Otherwise, it returns false and does not release locks.

 This method also clears the commit release locks and commit-or-abort release
 locks sets.  See the 'Releasing Locks' method category in class System for
 more information."

^ System commitAndReleaseLocks
%

category: 'Transaction Control'
method: GsCurrentSession
commitTransaction

"Attempts to update the persistent state of GemStone to include changes made
 by this session.

 If the commit operation succeeds, then this method returns true, and the
 current transaction's changes, if any, become a part of the persistent
 repository.  After the repository update, the session exits the current
 transaction.  If the transaction mode is autoBegin, then the session enters
 a new transaction.  If the transaction mode is #manualBegin, then the session
 remains outside of a transaction.

 If conflicts prevent the repository update, then this method returns false.
 Call the transactionConflicts method to determine the nature of the
 conflicts.  If the session is outside of a transaction, then this method
 raises the error #rtErrPrimOutsideTrans.

 This method also updates the session's view of GemStone.  If the commit
 operation succeeds, then all objects in the session's view are consistent with
 the current state of GemStone.  If the commit fails, then this method retains
 all the changes that were made to objects within the current transaction.
 However, commits made by other sessions are visible to the extent that changes
 in this transaction do not conflict with them."

^ System commitTransaction
%

category: 'Session Configuration Access'
method: GsCurrentSession
configurationAt: aSymbol

"Returns the value of the configuration parameter named aSymbol (for example,
 #GEM_HALT_ON_ERROR).  Raises an error if aSymbol is not a valid parameter
 name."

^ System configurationAt: aSymbol
%

category: 'Session Configuration Access'
method: GsCurrentSession
configurationAt: aSymbol put: anObject

"Sets the value of the configuration parameter named aSymbol to anObject.
 Raises an error if aSymbol is not a valid parameter name or anObject is an
 inappropriate value for the parameter."

^ System configurationAt: aSymbol put: anObject
%

category: 'Session Configuration Access'
method: GsCurrentSession
configurationParameters

"Returns a Set of Symbols containing the names of all valid configuration
 parameters for this GsSession."

^ ConfigurationParameterDict keys
%

category: 'Transaction Control'
method: GsCurrentSession
continueTransaction

"Updates the session's view to the most recently committed state of GemStone
 without rolling back modifications made to committed objects in the session.
 The read and write sets of the session are carried forward and continue to
 accumulate until the session either commits or aborts.  Changes made by this
 session to committed objects are not visible to other sessions until the
 session commits.

 Returns true if accumulated modifications to the committed state of GemStone
 would not cause concurrency conflict as of the new view; otherwise returns
 false.  If it returns false, you can call the transactionConflicts method to
 determine the nature of the conflicts.

 This method can be used whether or not the session is outside of a transaction.
 Of course, the session cannot commit the accumulated changes unless it is
 inside a transaction.

 If transaction mode is #manualBegin, then continueTransaction does not alter
 the inside/outside of transaction state of the session.

 Modifications made by other committed transactions are accumulated for
 retrieval by GciDirtyObjs() and GciDirtySavedObjs() just as they are
 accumulated for commitTransaction or abortTransaction.

 This method has no effect on object locks held by the session.  Locks in the
 release locks sets are not released."

^ System continueTransaction
%

category: 'Method lookup control'
method: GsCurrentSession
enableSessionMethods: aBoolean env: envId
  "sets omPtr->sessionMethodsEnabled := aBoolean  in VM
   and returns previous state ; this controls compilation logic
   but has no effect on method lookup.  Caller responsible for
   using transientMethodDictForEnv:put: to install session methods.
   Clears method lookup caches for all in-memory classes, and
   clears all send-site caches for all in-memory methods. "

<primitive: 650>
aBoolean _validateClass: Boolean .
envId _validateClass: SmallInteger .
self _primitiveFailed:#enableSessionMethods: args: { aBoolean . envId }
%

category: 'Smalltalk Execution'
method: GsCurrentSession
execute: aString

"Executes aString containing GemStone Smalltalk code in the session represented
 by the receiver.  Symbol resolution is from the default symbol list."

^ ( aString _compileInContext: nil symbolList: self symbolList )
     _executeInContext: nil
%

category: 'Smalltalk Execution'
method: GsCurrentSession
execute: aString symbolList: aSymbolList

"Executes aString containing GemStone Smalltalk code in the session represented
 by the receiver.  Symbol resolution is from the given symbol list."

^ ( aString _compileInContext: nil symbolList: aSymbolList )
     _executeInContext: nil
%

category: 'Transaction Control'
method: GsCurrentSession
inTransaction

"Returns true to indicate that the session is in a transaction, false
 otherwise."

^ (System _zeroArgPrim: 5) ~~ 0   "fix 48290"
%

category: 'Session Control'
method: GsCurrentSession
logoutAndGoIdle

"The session performs a relogin as the Nameless user and goes idle by
 setting the transactionMode to #transactionless."

| namelessUserProfile |
namelessUserProfile := AllUsers userWithId: 'Nameless' ifAbsent: [self error: 'There is no Nameless user profile.'].

System currentObjectSecurityPolicy: namelessUserProfile defaultObjectSecurityPolicy.
System abortTransaction.
System _generationScavenge.
self reloginAsUser: 'Nameless' withPassword: ''.
self transactionMode: #transactionless.

%

category: 'Deprecated'
method: GsCurrentSession
nativeLanguage

"Returns the String that designates the language that controls error message
 generation in the current session."

| language |
self deprecated: 'GsCurrentSession>>nativeLanguage deprecated v3.0; base system no longer uses nativeLanguage'.
language := self objectNamed: #'NativeLanguage'.
language ifNil: [^#'English'].
^language.
%

category: 'Accessing the Symbol List'
method: GsCurrentSession
objectNamed: aSymbol

"Returns the first object in the receiver's symbol list that has the given
 name.  If no object with the given name is found, returns nil."

| assn |

assn := (transientSymbolList ifNil:[ symbolList]) resolveSymbol: aSymbol.
assn ifNil: [ ^nil ].
^assn value
%

category: 'Session Control'
method: GsCurrentSession
reloginAsUser: aUserId withEncryptedPassword: aPassword

"The session aborts its current transaction, if any, and attempts a relogin
 with the specified userid and encrypted password.  Errors in login leave
 the session in the idle state, as if logoutAndGoIdle had been successfully
 executed."

| result |
result := System _reloginAsUser: aUserId password: aPassword encrypted: true.
(System hasUserAction: #_topazReloginCallback)
  ifTrue: [ System userAction: #_topazReloginCallback ].

result ifFalse: [System _error: #rtErrWarningPasswordExpire]
       ifTrue: [System transactionMode: #autoBegin]
%

category: 'Session Control'
method: GsCurrentSession
reloginAsUser: aUserId withPassword: aPassword

"The session aborts its current transaction, if any, and attempts a relogin
 with the specified userid and password.  Errors in login leave the session
 in the idle state, as if logoutAndGoIdle had been successfully executed."

| result |
result := System _reloginAsUser: aUserId password: aPassword encrypted: false.

(System hasUserAction: #_topazReloginCallback)
  ifTrue: [ System userAction: #_topazReloginCallback ].

result ifFalse: [System _error: #rtErrWarningPasswordExpire]
       ifTrue: [System transactionMode: #autoBegin]
%

category: 'Accessing the Symbol List'
method: GsCurrentSession
resolveSymbol: aSymbol

"Searches the receiver's symbol list for an Association whose key is equal to
 aString, and returns that Association.  If no such Association is found in the
 symbol list, returns nil.

 Implemented to use the current session's transient copy of the symbol list.
 This method is the default mechanism for symbol-resolution during
 compilation of GemStone Smalltalk methods."

^ (transientSymbolList ifNil:[ symbolList]) resolveSymbol: aSymbol
%

category: 'Session Configuration Access'
method: GsCurrentSession
serverVersionAt: aSymbol

"Returns the value of the GsSession's Stone version information parameter named
 aSymbol."

^ System stoneVersionAt: aSymbol
%

category: 'Session Configuration Access'
method: GsCurrentSession
sessionVersionAt: aSymbol

"Returns the value of the GsSession's Gem version information parameter
 named aSymbol."

^ System gemVersionAt: aSymbol
%

category: 'Deprecated'
method: GsCurrentSession
signalFromSession

"Deprecated.
 Return a GsInterSessionSignal object containing information about a signal from
 another session, or nil if there is no signal waiting."

| sigArray result |
self deprecated:'GsCurrentSession>>signalFromSession replaced by InterSessionSignal(C)>>poll' .
sigArray := System _signalFromGemStoneSession .
sigArray ifNil:[ ^ nil ].
result := GsInterSessionSignal signal: (sigArray at: 2)
			      message: (sigArray at: 3) .
result session: (GsSession sessionWithSerialNumber: (sigArray at: 1)).
^ result
%

category: 'Accessing the Symbol List'
method: GsCurrentSession
symbolList

"If a transientSymbolList has been installed, return it.
 Otherwise return the symbolList installed at session login, which
 is  System myUserProfile symbolList .

If you have not executed any of
  System class >> refreshTransientSymbolList
  GsCurrentSession >> transientSymbolList:
  GsCurrentSession >> transientSymbolList
in the current session, then
  GsCurrentSession currentSession symbolList == System myUserProfile symbolList
"

^ transientSymbolList ifNil:[ symbolList]
%

category: 'Transaction Control'
method: GsCurrentSession
transactionMode

"Returns the current transaction mode for the current GemStone session, either
 #autoBegin, #manualBegin or #transactionless.  The default is #autoBegin."

^ System _zeroArgPrim: 4
%

category: 'Transaction Control'
method: GsCurrentSession
transactionMode: newMode

"Sets a new transaction mode for the current GemStone session and exits the
 previous mode by aborting the current transaction.  Valid arguments are
 #autoBegin, #manualBegin or #transactionless.
 The mode transactionless is intended primarily for idle sessions. Users
 may scan database objects, but are at risk of obtaining inconsistent views.
 When in transactionless mode, the session is exempt from voting on
 possibleDeadObjs and will not be terminated by STN_GEM_TIMEOUT expiring.
"

System transactionMode: newMode
%

category: 'Accessing the Symbol List'
method: GsCurrentSession
transientSymbolList
  "If transientSymbolList is nil , set it to be a copy of the
      current UserProfile's symbolList .
   Returns the receiver's transientSymbolList .

   transientSymbolList is nil unless one of
		GsCurrentSession >> transientSymbolList
    GsCurrentSession >> transientSymbolList:  
    System class >> refreshTransientSymbolList
  have been executed in the current session."

  ^ transientSymbolList 
		ifNil: [
			"System class refreshTransientSymbolList directly updates the transientSymbolList instance variable"
			System refreshTransientSymbolList.
			transientSymbolList ]
%

category: 'Accessing the Symbol List'
method: GsCurrentSession
transientSymbolList: aSymbolList
 "Installs aSymbolList as the transientSymbolList of the receiver.
  Requires CodeModification privilege .

  The transientSymbolList is nil unless one of
		GsCurrentSession >> transientSymbolList
    GsCurrentSession >> transientSymbolList:  
    System class >> refreshTransientSymbolList
 have been executed in the current session."
  (aSymbolList _validateClass: SymbolList) ifTrue:[
    System myUserProfile _validateCodeModificationPrivilege .
    transientSymbolList := aSymbolList 
  ]
%

category: 'Session Configuration Access'
method: GsCurrentSession
versionParameters

"Returns a Set of Strings containing the names of all valid version parameters
 for this GsSession."

^ VersionParameterDict keys
%

category: 'Reduced Conflict Support'
method: GsCurrentSession
_RcHasConflicts

"Do the hasConflicts check for Rc objects but do not selectively abort."

| scanArray redoObject conflicts|

scanArray := System _getRedoAndConflictObjects.
" if no redo objects were found, cannot resolve conflicts "
scanArray ifNil: [ ^ true ].

1 to: scanArray size by: 2 do: [ :i |
  redoObject := scanArray at: i.
  conflicts := scanArray at: i + 1.

  (redoObject _validateRcConflictsWith: conflicts)
    ifFalse: [ ^ true ]
].
^false
%

category: 'Reduced Conflict Support'
method: GsCurrentSession
_scanRedoLogForConflicts

"Scan the redo log for entries on RC objects that could conflict
on replay.  If we find one, return true, indicating that we do not
know if we can replay successfully without selectively aborting."

| redoLog |
(redoLog := System _redoLog) == nil
  ifTrue: [ ^ false ].

redoLog redoObjects keysAndValuesDo: [ :redoObj :logEntries |
  (redoObj _validateRcConflictsWith: #())
    ifFalse: [ ^ true ]
].
^ false
%

category: 'Accessing the Symbol List'
method: GsCurrentSession
_transientSymbolList
	"Direct access to transentSymbolList .
  The transientSymbolList is nil unless one of
		GsCurrentSession >> transientSymbolList
    GsCurrentSession >> transientSymbolList:  
    System class >> refreshTransientSymbolList
 have been executed in the current session."

  ^ transientSymbolList
%

! Class extensions for 'GsExceptionHandler'

!		Class methods for 'GsExceptionHandler'

removeallmethods GsExceptionHandler
removeallclassmethods GsExceptionHandler

category: 'Disallowed'
classmethod: GsExceptionHandler
basicNew
  "Instances are created only by methods in Exception. See also #new ."

  self shouldNotImplement: #basicNew .
  ^ nil
%

category: 'Disallowed'
classmethod: GsExceptionHandler
new
  "instances may be created only by using
     Exception(C)>>category:number:do:
     Exception(C)>>installStaticException:category:number:
     Exception(C)>>installStaticException:category:number:subtype:
     Exception(C)>>addDefaultHandler:
  "
  self shouldNotImplement: #new .
  ^ nil
%

category: 'Management'
classmethod: GsExceptionHandler
removeActivationException: aGsExceptionHandler

"Search the current GemStone Smalltalk call stack for a method or block context
 that has anException installed, and remove it.  The stack is searched by
 starting with the top method or block context and moving down.

 If aGsExceptionHandler was found on the stack, returns aGsExceptionHandler,
 otherwise signals a RuntimeError.  "

aGsExceptionHandler class == GsExceptionHandler ifTrue:[
  aGsExceptionHandler _remove ifNil:[
    LookupError signal: 'aGsExceptionHandler was not found on the stack'
  ].
  ^ aGsExceptionHandler .
].
aGsExceptionHandler _validateClass: GsExceptionHandler
%

!		Instance methods for 'GsExceptionHandler'

category: 'Accessing'
method: GsExceptionHandler
block

"Returns the value of the instance variable 'theBlock'."

^ theBlock
%

category: 'Accessing'
method: GsExceptionHandler
category

^ GemStoneError
%

category: 'Accessing'
method: GsExceptionHandler
exceptionClass
  ^ exClass
%

category: 'Accessing'
method: GsExceptionHandler
isLegacy
  ^ exClass == nil
%

category: 'Accessing'
method: GsExceptionHandler
next

"Returns the next exception to be invoked (the value of the next instance
 variable)."

^next
%

category: 'Accessing'
method: GsExceptionHandler
number

"Returns the value of the instance variable 'number'."

^number
%

category: 'Management'
method: GsExceptionHandler
remove
"Search the current GemStone Smalltalk call stack for a frame
 that has the receiver installed, and remove it if found.
 If not found on the stack, search the list of static handlers and
 remove it if found.

 Returns receiver if found, nil if not found ."

  ^ self _remove ifNil:[ AbstractException removeStaticException: self ]
%

category: 'Accessing'
method: GsExceptionHandler
subtype

"Returns the value of the receiver's subtype instance variable."

^subtype
%

category: 'Accessing'
method: GsExceptionHandler
_ansiBlock
  "If receiver is an ANSI static handler, return the receivers block,
   else return nil."
  exClass ifNotNil:[ ^ theBlock ].
  ^ nil
%

category: 'Management'
method: GsExceptionHandler
_remove

"Search the current GemStone Smalltalk call stack for a method or block context
 that has the receiver installed, and remove it.  The stack is searched by
 starting with the top method or block context and moving down.

 Returns receiver if found, nil if not found ."

<primitive: 375>
self _primitiveFailed: #remove
%

! Class extensions for 'GsFile'

!		Class methods for 'GsFile'

removeallmethods GsFile
removeallclassmethods GsFile

category: 'Instance Creation'
classmethod: GsFile
basicNew

"disallowed"

self shouldNotImplement: #basicNew
%

category: 'Private'
classmethod: GsFile
classUserAction: actionName onClient: onClient

"Executes a GsFile user action, passing it the following arguments:
    self
    any arguments to the primitive (see other uses of primitive 396)
 Maximum of 6 arguments (6 with: keywords) for this primitive.
 actionName must be a Symbol .
 Checks NoGsFileOnServer or NoGsFileOnClient privilege"

<primitive: 396>

^ self _primitiveFailed: #classUserAction:onClient:
       args: { actionName . onClient }
%

category: 'Private'
classmethod: GsFile
classUserAction: actionName onClient: onClient with: arg1

"See GsFile | classUserAction:onClient: for documentation."

<primitive: 396>

^ self _primitiveFailed: #classUserAction:onClient:with:
       args: { actionName . onClient . arg1 }
%

category: 'Private'
classmethod: GsFile
classUserAction: actionName onClient: onClient with: arg1 with: arg2

"See GsFile | classUserAction:onClient: for documentation."

<primitive: 396>

^ self _primitiveFailed: #classUserAction:onClient:with:
       args: { actionName . onClient . arg1 . arg2 }
%

category: 'Drastic Measures'
classmethod: GsFile
closeAll

"Closes all open files on the client machine except stdin/stdout/stderr.
 Returns the receiver if successful, nil if not."

"make sure the sources file has been explicitly closed"

^ self classUserAction: #GsfCloseAll onClient: true
%

category: 'Drastic Measures'
classmethod: GsFile
closeAllOnServer

"Closes all open files on the server machine except stdin/stdout/stderr.
 Returns the receiver if successful, nil if not."

^ self classUserAction: #GsfCloseAll onClient: false
%

category: 'Directory Operations'
classmethod: GsFile
contentsAndTypesOfDirectory: dirSpec onClient: onClient

"Returns an Array of objects describing the contents of the given directory.
 The argument bool indicates the location of the directory on the client or
 the server.  Successive pairs of elements of the Array are each the name of an
 entry, and a Boolean - true if the entry is a file, and false if not.
 The order of results is based on the file system.

 Sample:  { 'file.c'  true . 'subdir' . false . ... }

 Returns anArray if successful, nil if not."
| dirContents result |
onClient ifTrue:[
  dirContents := self contentsOfDirectory: dirSpec onClient: true .
  dirContents ifNil:[ ^ nil ].
  result := { }  .
  1 to: dirContents size do:[:n | | str |
    str := dirContents at: n .
    result add: str ;
           add: (self _fileKind: str onClient: true ) == 0  .
  ] .
] ifFalse:[  | unicodeEnabled |
  "use faster primitives available in VM for server file access"
  dirContents := self _contentsOfServerDirectory: dirSpec expandPath: true utf8Results: true .
  dirContents _isSmallInteger ifTrue:[ ^ nil ].
  result := { }  .
  unicodeEnabled := Unicode16 _unicodeCompareEnabled.
  1 to: dirContents size do:[:n | | elem aGsFileStat |
    result add: (elem := self _decodeUtfResult: (dirContents at: n) unicode: unicodeEnabled) .
    (aGsFileStat := self stat: elem isLstat: false) _isSmallInteger ifTrue:[
      result add: false "probably a symlink to a non-existant file"
    ] ifFalse:[
      result add: aGsFileStat isDirectory not
    ]
  ].
].
^ result
%

category: 'Directory Operations'
classmethod: GsFile
contentsOfDirectory: dirSpec onClient: clientBool

"Returns an Array describing the contents of the given directory.
 Elements of result are Unicode strings if the system is in Unicode Comparison
 Mode (String Configuration isInUnicodeComparsonMode = true), otherwise they are
 traditional Strings.

 The argument bool indicates the location of the directory on the client or
 the server.
 The ordering of the result is based on the file system implementation.
 Returns anArray if successful, nil if not."

| unicodeEnabled files uDir res |
unicodeEnabled := Unicode16 _unicodeCompareEnabled.
uDir := self _utfPath: dirSpec forClient: clientBool .
files :=  self classUserAction: #GsfDirectory onClient: clientBool
       with: uDir
       with: true.
files ifNil:[ ^ files ].
res := { } .
1 to: files size do:[:n |
  res add: (self _decodeUtfResult: (files at: n) unicode: unicodeEnabled) .
].
^ res
%

category: 'Directory Operations'
classmethod: GsFile
createClientDirectory: aPathName

"Creates the named directory on the client machine's file system.
 Returns the receiver if the directory was created, nil if an error occurs."

^ self _createDirectory: aPathName onClient: true mode: nil .
%

category: 'Directory Operations'
classmethod: GsFile
createServerDirectory: aPathName

"Creates the named directory on the server machine.
 Returns the receiver if the directory was created, nil if an error occurs."

^ self _createDirectory: aPathName onClient: false mode: nil .
%

category: 'Directory Operations'
classmethod: GsFile
createServerDirectory: aPathName mode: modeInt

"Creates the named directory on the server machine.
 Returns the receiver if the directory was created, nil if an error occurs.
 modeInt should be a SmallInteger, the value of which will be used
 as the second arg to mkdir() .
 If modeInt == nil, a value of 8r770 will be used.
 Note that mkdir() takes the modeInt and removes bits set in the process' umask.
"

^ self _createDirectory: aPathName onClient: false mode: modeInt .
%

category: 'File Operations'
classmethod: GsFile
exists: aPathName

"Returns true if the given path points to a file on the client,
 false if not, and nil if an error occurs trying to find out."

^ self _exists: aPathName onClient: true
%

category: 'File Operations'
classmethod: GsFile
existsOnServer: aPathName

"Returns true if the given path points to a file, on the server,
 false if not, and nil if an error occurs trying to find out."

^ self _exists: aPathName onClient: false
%

category: 'Garbage Collection'
classmethod: GsFile
finalizeAll

"Closes all open files on the client machine that were opened by a
 instance of GsFile but that are no longer being used by an instance
 of GsFile. The need for finalization is caused by instances being
 garbage collected before they are sent close.
 Returns nil if an error occurs.
 Returns the total number of GsFile instances that were not finalized
 because they were still in use."

System _generationScavenge.
^ self classUserAction: #GsfFinalizeAll onClient: true
%

category: 'Garbage Collection'
classmethod: GsFile
finalizeAllOnServer

"Closes all open files on the server machine that were opened by a
 instance of GsFile but that are no longer being used by an instance
 of GsFile. The need for finalization is caused by instances being
 garbage collected before they are sent close.
 Returns nil if an error occurs.
 Returns the total number of GsFile instances that were not finalized
 because they were still in use."

System _generationScavenge.
^ self classUserAction: #GsfFinalizeAll onClient: false
%

category: 'Directory Operations'
classmethod: GsFile
gciClientIsWindows

"Returns true if GCI client is on a Microsoft Windows operating system,
 false otherwise."

^ self _directoryPrim: 6 with: nil with: nil
%

category: 'Writing'
classmethod: GsFile
gciLog: aString onClient: clientBool

"Passes the contents of the given collection to a GciGetLogger() function.
 The clientBool must be a Boolean , true means use GciGetLogger() on
 the client process, false means server process .
 If the collection has size > 0 and does not end in an Ascii LineFeed,
 a LineFeed is appended to the bytes passed to the logging function.

 If clientBool==true,
 a check for client process disconnected is made at start of the GsfGciLog
 useraction invocation, and if the client has disconnected, this invocation
 will behave as if it were GsFile(C)>>gciLogServer: .
"

^ clientBool ifTrue:[ self gciLogClient: aString ]
	    ifFalse:[ self gciLogServer: aString]
%

category: 'Writing'
classmethod: GsFile
gciLogClient: aString

"Passes the contents of the given collection to the GciGetLogger() function
 of the client process.

 If the client is a topaz process, the default logger writes to the
 current output file as controlled by topaz OUTPUT PUSH statements,
 else to stdout.

 If the collection has size > 0 and does not end in an Ascii LineFeed,
 a LineFeed is appended to the bytes passed to the logging function.

 A check for client process disconnected is made at start of the GsfGciLog
 useraction invocation, and if the client has disconnected, this invocation
 will behave as if it were GsFile class >>gciLogServer: .

 If this method is invoked from within a non-blocking Gci execution or
 a GciTs execution , it will behave as if GsFile class>>gciLogServer: .
"

^ self classUserAction: #GsfGciLog  onClient: true with: aString
%

category: 'Writing'
classmethod: GsFile
gciLogServer: aString

"Passes the contents of the given String to the GciGetLogger() function
 of the server process.
 In a gem process, the default logger writes to stdout, i.e. to the gem
 log file.  In the linked login of a topaz -l process,
 the default logger writes to the current output file as controlled by
 topaz OUTPUT PUSH statements, else to stdout.
 If the collection has size > 0 and does not end in an Ascii LineFeed,
 a LineFeed is appended to the bytes passed to the logging function."

<primitive: 1003>
"primitive accepts String , ByteArray, Utf8 args"
aString _validateClass: MultiByteString .
[ self gciLogServer: aString encodeAsUTF8 ] onException: Error do:[:ex |
  self classUserAction: #GsfGciLog  onClient: false with: aString
].
^ self
%

category: 'Reading'
classmethod: GsFile
getContentsOfServerFile: filePathAndName
"Return the contents of the named file in a single operation.
 Use caution when reading a large file as doing so may cause
 an out of memory error."
 
  | gsf result |
  gsf := GsFile openReadOnServer: filePathAndName.
  gsf ifNil:[ IOError signal: GsFile serverErrorString ] .
  result := gsf contents.
  gsf close.
  ^ result
%

category: 'Directory Operations'
classmethod: GsFile
isClientDirectory: aPathName
"Returns true if 'aPathName' names an existing directory on the client
 machine.
 Returns false if it is not a directory.
 Returns nil if an error occurs."

^ self _isDirectory: aPathName onClient: true .
%

category: 'Directory Operations'
classmethod: GsFile
isServerDirectory: aPathName
"Returns true if 'aPathName' names an existing directory on the server
 machine.
 Returns false if it is not a directory.
 Returns nil if an error occurs."

^ self _isDirectory: aPathName onClient: false .
%

category: 'File Operations'
classmethod: GsFile
isSymbolicLink: aPathName onClient: clientBool 
  | expPath stats |
  expPath := self _expandFilenamePreserveLinks: aPathName isClient: clientBool.
  stats := self stat: expPath isLstat: true .
  ^ (stats mode bitAnd: 8r170000"S_IFMT") = 8r120000"S_IFLNK" 
%

category: 'Error Reporting'
classmethod: GsFile
lastErrorString

"Returns the currently posted error string, for class operations on the
 client, or nil if no error has occurred.  Clears the error string."

^ self classUserAction: #GsfClassError onClient: true with: nil
%

category: 'File Operations'
classmethod: GsFile
lastModificationOfClientFile: aPathName

"Gets the date and time the named file on the client machine was last
 modified. Returns a DateTime if successful, nil if an error occurs."

^ self _fileModTime: aPathName onClient: true .
%

category: 'File Operations'
classmethod: GsFile
lastModificationOfServerFile: aPathName

"Gets the date and time the named file on the server machine was last
 modified. Returns a DateTime if successful, nil if an error occurs."

^ self _fileModTime: aPathName onClient: false .
%

category: 'Private'
classmethod: GsFile
new

"disallowed"

self shouldNotImplement: #new
%

category: 'Instance Creation'
classmethod: GsFile
open: aPathName mode: openMode

"Creates an instance of the receiver and opens a file on the client machine.

 The openMode argument must be a String that that is equal to one of the
 following: 'r', 'w', 'a', 'r+', 'w+', 'a+', 'rb', 'wb', 'ab', 'r+b', 'w+b',
 'a+b', 'rb+', 'wb+', 'ab+'.  The mode has the same meaning as it does for the
 C library function, fopen().

 Returns a GsFile if successful, nil if an error occurs."

^ self open: aPathName mode: openMode onClient: true
%

category: 'Instance Creation'
classmethod: GsFile
open: aPathName mode: openMode  onClient: clientBool

"Creates an instance of the receiver and opens a file on the client machine
 (if clientBool is true) or the server machine (if clientBool is false).

 The openMode argument must be a String that that is equal to one of the
 following: 'r', 'w', 'a', 'r+', 'w+', 'a+', 'rb', 'wb', 'ab', 'r+b', 'w+b',
 'a+b', 'rb+', 'wb+', 'ab+'.  The mode has the same meaning as it does for the
 C library function, fopen().

 Returns a GsFile if successful, nil if an error occurs."

^self open: aPathName mode: openMode  onClient: clientBool withCompression: false
%

category: 'Instance Creation (compression)'
classmethod: GsFile
open: aPathName mode: openMode  onClient: clientBool withCompression: compBool

"Creates an instance of the receiver and opens a file on the client machine
 (if clientBool is true) or the server machine (if clientBool is false).

 The openMode argument must be a String that that is equal to one of the
 following: 'r', 'w', 'a', 'r+', 'w+', 'a+', 'rb', 'wb', 'ab', 'r+b', 'w+b',
 'a+b', 'rb+', 'wb+', 'ab+'.  The mode has the same meaning as it does for the
 C library function, fopen().

 The classmethod GsFile>>validFileModes may be used to generate a list
 of all valid file modes.

 If compBool is true, the file is opened using gzip gzopen() call rather than
 fopen().  In this case, the file is assumed to be a compressed file, although
 gzopen() will also succeed when used to open uncompressed files.  When reading
 a compressed file, it is automatically decompressed as it is read.  Similarly,
 data written to a compressed file is automatically compressed before it is
 written to the file.

 For compressed files, all open mode arguments listed above that do not contain
 a plus sign ('+') are valid.  In addition, any valid write mode argument
 (those contaning 'a' or 'w') may have a '1' or '9' appended.
 Appending a '1' means compress using the best possible speed while appending
  a '9' means compress to attain the best possible compression.

 The classmethod GsFile>>validFileModesForCompression may be used to
 generate a list of all valid file modes for compressed files.

 Returns a GsFile if successful, nil if an error occurs."

| inst |
inst := self _basicNew.
inst _open: aPathName mode: openMode onClient: clientBool.
^ compBool ifTrue:[inst gzOpen] ifFalse:[inst open]
%

category: 'Instance Creation'
classmethod: GsFile
openAppend: aPathName

"Opens a text file on the client machine for writing.
 If the file does not exist it is created.
 The file position indicator is positioned at the end of the file
 before each write operation.
 Read operations are not allowed.
 Returns aGsFile if successful, nil if an error occurs."

^self open: aPathName mode: 'a' onClient: true
%

category: 'Instance Creation'
classmethod: GsFile
openAppendOnServer: aPathName

"Opens a text file on the server machine for writing.
 If the file does not exist it is created.
 The file position indicator is positioned at the end of the file
 before each write operation.
 Read operations are not allowed.
 Returns aGsFile if successful, nil if an error occurs."

^self open: aPathName mode: 'a' onClient: false
%

category: 'Instance Creation (compression)'
classmethod: GsFile
openCompressed: aPathName mode: openMode

"Creates an instance of the receiver and opens a file on the client machine
 using the gzip compression routing gzopen().

 The openMode argument must be one of the strings return by the method
 GsFile>>validFileModesForCompression.

 Returns a GsFile if successful, nil if an error occurs."

^ self open: aPathName mode: openMode onClient: true withCompression: true
%

category: 'Instance Creation (compression)'
classmethod: GsFile
openCompressed: aPathName mode: openMode  onClient: clientBool

"Creates an instance of the receiver and opens a file on the client machine
 (if clientBool is true) or the server machine (if clientBool is false).

 The openMode argument must be one of the strings return by the method
 GsFile>>validFileModesForCompression.

 Returns a GsFile if successful, nil if an error occurs."

^self open: aPathName mode: openMode  onClient: clientBool withCompression: true
%

category: 'Instance Creation'
classmethod: GsFile
openOnServer: aPathName mode: openMode

"Creates an instance of the receiver and opens a file on the server machine.

 The openMode argument must be a String that that is equal to one of the
 following: 'r', 'w', 'a', 'r+', 'w+', 'a+', 'rb', 'wb', 'ab', 'r+b', 'w+b',
 'a+b', 'rb+', 'wb+', 'ab+'.  The mode has the same meaning as it does for the
 C library function, fopen().

 Returns a GsFile if successful, nil if an error occurs."

^ self open: aPathName mode: openMode onClient: false
%

category: 'Instance Creation (compression)'
classmethod: GsFile
openOnServerCompressed: aPathName mode: openMode

"Creates an instance of the receiver and opens a file on the server machine.

 The openMode argument must be one of the strings return by the method
 GsFile>>validFileModesForCompression.

 Returns a GsFile if successful, nil if an error occurs."

^ self open: aPathName mode: openMode onClient: false withCompression: true
%

category: 'Instance Creation'
classmethod: GsFile
openRead: aPathName

"Opens an existing text file on the client machine for reading.
 Write operations are not allowed.
 Returns aGsFile if successful, nil if an error occurs."

^self open: aPathName mode: 'r' onClient: true
%

category: 'Instance Creation (compression)'
classmethod: GsFile
openReadCompressed: aPathName

"Opens an existing text file on the client machine for reading.
 Write operations are not allowed.
 Returns aGsFile if successful, nil if an error occurs."

^self open: aPathName mode: 'rb' onClient: true withCompression: true
%

category: 'Instance Creation'
classmethod: GsFile
openReadOnServer: aPathName

"Opens an existing text file on the server machine for reading.
 Write operations are not allowed.
 Returns aGsFile if successful, nil if an error occurs."

^self open: aPathName mode: 'r' onClient: false
%

category: 'Instance Creation (compression)'
classmethod: GsFile
openReadOnServerCompressed: aPathName

"Opens an existing text file on the server machine for reading.
 Write operations are not allowed.

 If the file was compressed with gzip, all data will be
 uncompressed as it is read.

 Returns aGsFile if successful, nil if an error occurs."

^self open: aPathName mode: 'rb' onClient: false withCompression: true
%

category: 'Instance Creation'
classmethod: GsFile
openUpdate: aPathName

"Opens an existing text file on the client machine for reading and writing.
 Returns aGsFile if successful, nil if an error occurs."

^self open: aPathName mode: 'r+' onClient: true
%

category: 'Instance Creation'
classmethod: GsFile
openUpdateOnServer: aPathName

"Opens an existing text file on the server machine for reading and writing.
 Returns aGsFile if successful, nil if an error occurs."

^self open: aPathName mode: 'r+' onClient: false
%

category: 'Instance Creation'
classmethod: GsFile
openWrite: aPathName

"Opens a text file on the client machine for writing.
 If the file exists it is truncated to zero length.
 If the file does not exist it is created.
 Read operations are not allowed.
 Returns aGsFile if successful, nil if an error occurs."

^self open: aPathName mode: 'w' onClient: true
%

category: 'Instance Creation (compression)'
classmethod: GsFile
openWriteCompressed: aPathName

"Opens a text file on the client machine for writing.
 If the file exists it is truncated to zero length.
 If the file does not exist it is created.
 Read operations are not allowed.
 Returns aGsFile if successful, nil if an error occurs."

^self open: aPathName mode: 'wb' onClient: true withCompression: true
%

category: 'Instance Creation'
classmethod: GsFile
openWriteOnServer: aPathName

"Opens a text file on the server machine for writing.
 If the file exists it is truncated to zero length.
 If the file does not exist it is created.
 Read operations are not allowed.
 Returns aGsFile if successful, nil if an error occurs."

^self open: aPathName mode: 'w' onClient: false
%

category: 'Instance Creation (compression)'
classmethod: GsFile
openWriteOnServerCompressed: aPathName

"Opens a text file on the server machine for writing.
 If the file exists it is truncated to zero length.
 If the file does not exist it is created.
 Read operations are not allowed.
 All data will be compressed in gzip format before being
 written to the file.
 Returns aGsFile if successful, nil if an error occurs."

^self open: aPathName mode: 'wb' onClient: false withCompression: true
%

category: 'Directory Operations'
classmethod: GsFile
removeClientDirectory: aPathName

"Removes the named directory from the client machine's file system.
 Returns the receiver if the directory was deleted, nil if an error occurs.
 Note that this method will fail if the directory is not empty."

^ self _removeDirectory: aPathName onClient: true .
%

category: 'File Operations'
classmethod: GsFile
removeClientFile: aPathName

"Removes the named file from the client machine's file system.
 Returns the receiver if the file was deleted, nil if an error occurs."

^ self _removeFile: aPathName onClient: true .
%

category: 'Directory Operations'
classmethod: GsFile
removeServerDirectory: aPathName

"Removes the named directory from the server machine.
 Returns the receiver if the directory was deleted, nil if an error occurs.
 Note that this method will fail if the directory is not empty."

^ self _removeDirectory: aPathName onClient: false .
%

category: 'File Operations'
classmethod: GsFile
removeServerFile: aPathName

"Removes the named file from the server machine.  Returns the receiver if the
 file was deleted, nil if an error occurs."

^ self _removeFile: aPathName onClient: false .
%

category: 'File Operations'
classmethod: GsFile
renameFile: oldName to: newName

"Rename the given file to the new name.  Returns 0 if successful, nil or a non-zero
  SmallInteger if not.  The resulting SmallInteger is the UNIX error number (errno)
  returned by the C function rename().  See the man page for rename on your system
  for errno value definitions."

| oldN newN |
oldN := self _utfPath: oldName forClient: true .
newN := self _utfPath: newName forClient: true .
^ self classUserAction: #GsfRename onClient: true with: oldN with: newN
%

category: 'File Operations'
classmethod: GsFile
renameFileOnServer: oldName to: newName

"Rename the given file to the new name.  Returns 0 if successful, nil or a non-zero
  SmallInteger if not.  The resulting SmallInteger is the UNIX error number (errno)
  returned by the C function rename().  See the man page for rename on your system
  for errno value definitions."

| oldN newN |
oldN := self _utfPath: oldName forClient: false .
newN := self _utfPath: newName forClient: false .

^ self classUserAction: #GsfRename onClient: false with: oldN with: newN
%

category: 'Directory Operations'
classmethod: GsFile
serverCurrentDirectory

"Returns a String, the current directory of this topaz -l  or gem process."
^ self _directoryPrim: 2 with: nil with: nil
%

category: 'Error Reporting'
classmethod: GsFile
serverErrorString

"Returns the currently posted error string, for class operations on the
 server, or nil if no error has occurred.  Clears the error string."

^ self classUserAction: #GsfClassError onClient: false with: nil
%

category: 'File Operations'
classmethod: GsFile
serverRealPath: aPathName
  "Return the result of calling the posix function realpath for aPathName."
  | res |
  res := self _directoryPrim: 3 with: aPathName with: nil .
  res _isSmallInteger ifTrue:[  
    SystemCallError new errno: res ; signal .
  ].
  ^ res
%

category: 'File Operations'
classmethod: GsFile
sizeOf: aPathName

"Returns the size in bytes of the given client file, false if
 file does not exist, or nil if any other error occurs."

^ self _sizeOf: aPathName onClient: true
%

category: 'File Operations'
classmethod: GsFile
sizeOfOnServer: aPathName

"Returns the size in bytes of the given server file, false if
 file does not exist, or nil if any other error occurs."

^ self _sizeOf: aPathName onClient: false
%

category: 'File Operations'
classmethod: GsFile
stat: aName isLstat: aBoolean

"uses the server file system access.
 Returns a SmallInteger errno value if an error occurs or
 or if aName is not a valid file or directory. Otherwise
 returns a new instance of GsFileStat. "

^ self _stat: (self _utfPath: aName forClient: false) isLstat: aBoolean
%

category: 'Standard Files'
classmethod: GsFile
stderr

"Returns an instance of the receiver that is set up to write to the standard
 error output of the client process, or nil if an error occurs."

^self _getStdFile: 2 onClient: true
%

category: 'Standard Files'
classmethod: GsFile
stderrServer

"Returns an instance of the receiver that is set up to write to the standard
 error of the server process, or nil if an error occurs."

^self _getStdFile: 2 onClient: false
%

category: 'Standard Files'
classmethod: GsFile
stdin

"Returns an instance of the receiver that is set up to read the standard input
 of the client process, or nil if an error occurs."

^self _getStdFile: 0 onClient: true
%

category: 'Standard Files'
classmethod: GsFile
stdinServer

"Returns an instance of the receiver that is set up to read the standard input
 of the server process, or nil if an error occurs."

^self _getStdFile: 0 onClient: false
%

category: 'Standard Files'
classmethod: GsFile
stdout

"Returns an instance of the receiver that is set up to write to the standard
 output of the client process, or nil if an error occurs."

^self _getStdFile: 1 onClient: true
%

category: 'Standard Files'
classmethod: GsFile
stdoutServer

"Returns an instance of the receiver that is set up to write to the standard
 output of the server process, or nil if an error occurs."

^self _getStdFile: 1 onClient: false
%

category: 'File Modes'
classmethod: GsFile
validFileModes

"Answers an array of strings, each of which is a valid mode for
 opening files without compression."

^self classUserAction: #GsfAllValidFileModes onClient: true with: false
%

category: 'File Modes'
classmethod: GsFile
validFileModesForCompression

"Answers an array of strings, each of which is a valid mode for
 opening files with compression."

^self classUserAction: #GsfAllValidFileModes onClient: true with: true
%

category: 'Private'
classmethod: GsFile
_basicNew

"creates an instance registered with VM for finalization of cData"

<primitive: 674>
self _primitiveFailed: #_basicNew
%

category: 'Private'
classmethod: GsFile
_clientVersionAt: verParamId

"Returns client process version information.  verInfoId should be one of the
 values in the global dictionary VersionParameterDict."

^ self classUserAction: #GsfVersionParam onClient: true with: verParamId
%

category: 'Private'
classmethod: GsFile
_contentsOfServerDirectory: aPathName expandPath: aBoolean utf8Results: u8Boolean

"Returns an Array, or a SmallInteger errno.
 aPathName must be a String, Unicode string , Utf8 or Utf16  .
 The result is an Array of file names in the specified directory.
 If aBoolean is true, the result includes the expansion of aPathName
 as a prefix of each element of the result.
 If u8Boolean is true, elements of result not representable with 7 bit code points
 will be Utf8 .
 "

^ self _primContentsOfServerDirectory: (self _utfPath: aPathName forClient: false)
         expandPath: aBoolean utf8Results: u8Boolean
%

category: 'Private'
classmethod: GsFile
_createDirectory: aPathName onClient: clientBool mode: modeInt

"Returns receiver if creation succeeds.  Returns nil if an error occurs,
 in which case the class error buffer contains the error."
| path |
path := self _utfPath: aPathName forClient: clientBool .
^ self classUserAction: #GsfMkdir onClient: clientBool with: path with: modeInt
%

category: 'Private'
classmethod: GsFile
_decodeUtfResult: obj unicode: unicodeCompareEnabled
 | unic |
 unic := unicodeCompareEnabled ifNil:[ Unicode16 _unicodeCompareEnabled ].
 obj ifNil:[ ^ obj ].
 ((obj isKindOfClass: Utf8) or:[ obj isKindOfClass: Utf16]) ifTrue:[
   ^ unic ifTrue:[ obj decodeToUnicode] ifFalse:[ obj decodeToString].
 ].
 ^ unic ifTrue:[ obj asUnicodeString ] ifFalse:[ obj ].
%

category: 'Private'
classmethod: GsFile
_directoryPrim: opcode with: arg1 with: arg2

"opcode 0    chdir   arg1 is a String or Utf8 , arg2 ignored
 opcode 1    rmdir   arg1 is a String or Utf8 , arg2 ignored
 opcode 2    getcwd,   arg1 and arg2 ignored
 opcode 3    realpath   arg1 is a String or Utf8, arg2 ignored.
 opcode 4    mkdir   arg1 is a String or Utf8 , arg2 ignored
 opcode 5    umask   arg1 ignored, arg1 a 32 bit SmallInteger 
                          arg2 < 0 , result = umask(0); umask(result) .
                          arg2 >= 0  result = umask(arg2)
 opcode 6    gciClientIsWindows   arg1 and arg2 ignored "
<primitive: 765>
^ self _primitiveFailed: #_directoryPrim:with:with: args: { opcode. arg1 . arg2 }
%

category: 'Private'
classmethod: GsFile
_exists: aPathName onClient: clientBool

"Returns true if the given path points to a file, false if not, and nil if an
 error other than file-does-not-exist occurs trying to find out."

| result path |
path := self _utfPath: aPathName forClient: clientBool .
"GsfSize returns -1 for non-existant file , nil for other error ,
 >= 0 for a file that exists "
result := self classUserAction: #GsfSize onClient: clientBool with: path .
result == -1 ifTrue:[ ^ false "file does not exist" ].
result == nil ifTrue:[ ^ nil "some other error" ] .
^ true
%

category: 'Private'
classmethod: GsFile
_expandEnvVariable: varName isClient: clientBool

"Expands the environment variable named varName in either the GemBuilder for C
 or Gem process, returning a String. varName should be a kind of String.

 Returns nil if any of the following are true
    varName is not a byte format object.
    there is no environment variable defined with name  varName,
    the value of the environment variable is more than approximately 8000 bytes,
    clientBool is true and the size of varName exceeds approximately 8000 bytes."
| vn res |
vn := self _utfPath: varName forClient: clientBool .
res := self classUserAction: #GsfExpandEnvVar onClient: clientBool with: vn with: 0 .
^ self _decodeUtfResult: res unicode: nil .
%

category: 'Private'
classmethod: GsFile
_expandFilename: fName isClient: clientBool

"Expands fName using HostExpandFileName including following symbolic links.
 in either GCI or Gem process, returning a String. fName should be a kind of String.

 Returns nil if any of the following are true
    fName is not a byte format object.
    For each environment variable name varName within fName ,
      there is no environment variable defined with name  varName,
      the value of the environment variable is more than approximately 8000 bytes,
      clientBool is true and the size of varName exceeds approximately 8000 bytes."

| nam res |
nam := self _utfPath: fName forClient: clientBool .
res := self classUserAction: #GsfExpandEnvVar onClient: clientBool with: nam with: 1 .
^ self _decodeUtfResult: res unicode: nil
%

category: 'Private'
classmethod: GsFile
_expandFilenamePreserveLinks: fName isClient: clientBool

"Expands fName using HostExpandFileName without following symbolic links.
 in either GCI or Gem process, returning a String. fName should be a kind of String.

 Returns nil if any of the following are true
    varName is not a byte format object.
    there is no environment variable defined with name  varName,
    the value of the environment variable is more than approximately 8000 bytes,
    clientBool is true and the size of varName exceeds approximately 8000 bytes."

| nam res |
nam := self _utfPath: fName forClient: clientBool .
res := self classUserAction: #GsfExpandEnvVar onClient: clientBool with: nam with: 2 .
^ self _decodeUtfResult: res unicode: nil
%

category: 'Private'
classmethod: GsFile
_fileKind: aPathName onClient: clientBool

"Returns a SmallInteger representing the kind of the file, or returns nil if an
 error occurs, in which case the class error buffer contains the error.

 The file kinds are enumerated as follows.  Symbolic links are reported as
 the type of the file pointed to.

    0. file
    1. directory
    2. character device
    3. block device
    4. symbolic link (not applicable)
    5. other
    6. error
    7. FIFO
    8. stdin, stdout, or stderr
    9. stdin or stdout connected to a terminal (isatty(fileno(f) == 1)  "
| path |
path := self _utfPath: aPathName forClient: clientBool .
^ self classUserAction: #GsfFileKind onClient: clientBool with: path
%

category: 'Private'
classmethod: GsFile
_fileModTime: aPathName onClient: clientBool

"Returns DateTime of files last modification time if successful.
 Returns nil if an error occurs,
 in which case the class error buffer contains the error."
| path |
path := self _utfPath: aPathName forClient: clientBool .
^ self classUserAction: #GsfModTime onClient: clientBool with: path
%

category: 'Private'
classmethod: GsFile
_fstat: aFileDescriptor isLstat: aBoolean

"uses the server file system access.
 Returns a SmallInteger errno value if an error occurs or
 or if aFileDescriptor is not a valid file descriptor . Otherwise
 returns a new instance of GsFileStat. "
<primitive: 758>
aFileDescriptor _validateClass: SmallInteger .
aBoolean _validateClass: Boolean .
self _primitiveFailed: #_fstat:isLstat: args: { aFileDescriptor . aBoolean }
%

category: 'Private'
classmethod: GsFile
_getStdFile: stdId onClient: clientBool

"Returns an instance of the receiver that is set up to operate on a standard
 file of the client or nil if an error occurs.  stdId can be 0 (stdin),
 1 (stdout), or 2 (stderr)."

|result stdIdCache virtualId |

(stdIdCache := System __sessionStateAt: 18) ifNil:[
  stdIdCache := { nil . nil . nil . nil . nil . nil } .
  System __sessionStateAt: 18 put: stdIdCache.
].
clientBool ifTrue:[   virtualId := stdId + 1 ]
          ifFalse:[  virtualId := stdId + 4 ].
result := stdIdCache at: virtualId .
result == nil ifTrue: [ | status |
  "It has not yet been cached so create it"
  result := self _basicNew .
  result _newStdFile: stdId isClient: clientBool .
  "tell the client about it"
  status := result userAction: #GsfCreateStdFile onClient: clientBool with: stdId.
  status == nil ifFalse:[
    stdIdCache at: virtualId put: result.
    status == 1 ifTrue:[ result _newEmptyClientCData ].
  ] ifTrue: [
    result := nil.
  ].
].

^ result
%

category: 'Private'
classmethod: GsFile
_isDirectory: aPathName onClient: clientBool

"Returns true if 'aPathName' names an existing directory.
 Returns false if it is not a directory.
 Returns nil if an error occurs."
| path |
path := self _utfPath: aPathName forClient: clientBool .
^ self classUserAction: #GsfIsDir onClient: clientBool with: path
%

category: 'Private'
classmethod: GsFile
_log: string
  "Convenience method to print to stdout just like log: does."

  self stdout log: string
%

category: 'Private'
classmethod: GsFile
_primContentsOfServerDirectory: aPathName expandPath: aBoolean utf8Results: u8Boolean

"Returns an Array, or a SmallInteger errno.
 aPathName must be a String,  Utf8 or Utf16.
 The result is an Array of file names in the specified directory.
 If aBoolean is true, the result includes the expansion of aPathName
 as a prefix of each element of the result.
 If u8Boolean is true, elements of result not representable with 7 bit code points
 will be Utf8 ."
<primitive: 431>
aPathName _validateClasses: { String . Utf8 }.
aBoolean _validateClass: Boolean .
u8Boolean _validateClass: Boolean .
self _primitiveFailed: #_primContentsOfServerDirectory:
     args: { aPathName . aBoolean . u8Boolean}
%

category: 'Private'
classmethod: GsFile
_removeDirectory: aPathName onClient: clientBool

"Returns receiver if removal succeeds.  Returns nil if an error occurs,
 in which case the class error buffer contains the error."
| path |
path := self _utfPath: aPathName forClient: clientBool .
^ self classUserAction: #GsfRmdir onClient: clientBool with: path
%

category: 'Private'
classmethod: GsFile
_removeFile: aPathName onClient: clientBool

"Returns receiver if removal succeeds.  Returns nil if an error occurs,
 in which case the class error buffer contains the error."
| path |
path := self _utfPath: aPathName forClient: clientBool .
^ self classUserAction: #GsfUnlink onClient: clientBool with: path
%

category: 'Private'
classmethod: GsFile
_setEnvVariable: varName value: valueStr isClient: clientBool

"Both arguments must be either a String or a MultiByteString representable in
 8000 bytes of Utf8 or Utf16 encoding.
 If valueStr is nil or of size 0 , implements unsetenv semantics.
"

| nam val res |
nam := self _utfPath: varName forClient: clientBool .
valueStr ifNotNil:[
  val := self _utfPath: valueStr forClient: clientBool .
  val size = 0 ifTrue:[ val := nil  "unsetenv"].
].
"check of NoGsFileOnServer or NoGsFileOnClient privilege is done by C code of prim 396"
res := self classUserAction: #GsfSetEnvVar onClient: clientBool with: nam with: val .
res ifNil:[ Error signal: 'GsfSetEnvVar failed' ].
^ self
%

category: 'Error Reporting'
classmethod: GsFile
_setLastError: aString onClient: clientBool

self classUserAction: #GsfClassError onClient: clientBool with: aString
%

category: 'Private'
classmethod: GsFile
_sizeOf: aPathName onClient: clientBool

"Returns size of the file in bytes.  Returns nil if an error occurs,
 in which case the class error buffer contains the error."

| result path |
path := self _utfPath: aPathName forClient: clientBool .

"GsfSize returns -1 for non-existant file , nil for other error ,
 >= 0 for a file that exists "
result := self classUserAction: #GsfSize onClient: clientBool with: path .
result == nil ifTrue:[ ^ nil ] .
result == -1 ifTrue:[ ^ false "file does not exist" ].
^ result
%

category: 'Private'
classmethod: GsFile
_stat: aName isLstat: aBoolean

"uses the server file system access.
 Returns a SmallInteger errno value if an error occurs or
 or if aName is not a valid file or directory. Otherwise
 returns a new instance of GsFileStat. "
<primitive: 757>
aName _validateClasses: { String . Utf8 } .
aBoolean _validateClass: Boolean .
self _primitiveFailed: #_stat:isLstat: args: { aName . aBoolean }
%

category: 'Compressed File Operations'
classmethod: GsFile
_utfPath: aPath forClient: clientBool
  "Returns a String or Unicode7 with 7bit characters; or a Utf8 or Utf16"
  | isClient |
  (clientBool == false or:[ clientBool == 0 or:[ clientBool == 2]]) ifTrue:[
    isClient := false .
  ] ifFalse:[
    (clientBool == true or:[ clientBool == 1 or:[ clientBool == 3]]) ifTrue:[
      isClient := true .
    ] ifFalse:[
      ArgumentError signal:'invalid clientBool argument, a', clientBool class name
    ]
  ].
  (isClient and:[ self gciClientIsWindows]) ifTrue:[
    ^ aPath encodeAsUTF16  "file operation executing in Windows client"
  ].
  aPath _stringCharSize > 0 ifTrue:[
    aPath _asUnicode7 ifNotNil:[:p |
      "all chars are 7 bit"
      aPath isUnicodeString ifFalse:[ ^ String withAll: p ].
    ].
  ].
  ^ aPath encodeAsUTF8  "file operation on Unix client or in gem process"
%

!		Instance methods for 'GsFile'

category: 'Writing'
method: GsFile
+ collection

"Writes the contents of the given collection to the receiver's file at
 the current position. The argument must be a Collection with byte format.
 Returns a count of bytes added, or nil if an error occurs."

self deprecated: 'GsFile>>+ deprecated v3.2. Use the , method instead.'.
^  self nextPutAll: collection
%

category: 'Writing'
method: GsFile
, collection

"Writes the contents of the given collection to the receiver's file at
 the current position. The argument must be a Collection with byte format.
 Returns a count of bytes added, or nil if an error occurs."

^  self nextPutAll: collection
%

category: 'Comparing'
method: GsFile
= aFile

"Returns true if the receiver and aFile represent the same file system file.
 Returns false otherwise.

 Result is always false if either receiver or argument are not open."

(self == aFile) ifTrue:[ ^ true].
aFile == nil ifTrue:[ ^ false].
(aFile isKindOf: self class) ifFalse:[ ^ false].
(self isOpen and:[ aFile isOpen]) ifTrue:[
   self isClient = aFile isClient  ifTrue:[
    pathName = aFile pathName ifFalse:[
      ^ fileDescriptor = aFile fileDescriptor
    ].
    ^ true
  ].
].
^ false
%

category: 'Writing'
method: GsFile
add: char

"Writes the given Character to the receiver's file at the current position.
 Returns true, or nil if an error occurs."

^ self nextPut: char
%

category: 'Writing'
method: GsFile
addAll: collection

"Writes the contents of the given collection to the receiver's file at
 the current position. The argument must be a Collection
 with byte format.  Returns a count of bytes added, or nil if an error occurs."

^ self nextPutAll: collection
%

category: 'Stream Compatibility'
method: GsFile
atBeginning
"Answer true if the stream is positioned at the beginning"

^self position == 0
%

category: 'Positioning'
method: GsFile
atEnd

"Returns true if the receiver is currently positioned at the end of its
 file, false if not, or nil if an error occurs."

<primitive: 724>  "primitive succeeds only if file is on server"
(isClient _isSmallInteger and:[ self isCommitted == false]) ifTrue:[ ^ self _setNotOpenError ].
^ self userAction: #GsfAtEnd onClient: isClient
%

category: 'File Operations'
method: GsFile
close

"Closes the receiver's file.  Returns the receiver, or nil if an error occurs.
 If receiver is a standard file, has no effect and returns receiver."

 | res stdIdCache |
 self isOpen ifFalse: [^ self ].
 stdIdCache := System __sessionStateAt: 18.
 stdIdCache == nil ifFalse: [
   (stdIdCache includesIdentical: self) ifTrue:[
   fileDescriptor := -1 .
     ^ self
   ]
 ].
 fileDescriptor := -1 .
 res := self userAction: #GsfClose onClient: isClient .
 res == nil ifFalse:[ self _setNotOpened ].
 ^ res
%

category: 'Reading'
method: GsFile
contents

"Returns a String containing the contents of the receiver from the current
 position to the end of the file.  Returns nil if an error occurs."

^ self isCompressed
    ifTrue:[self _contentsUncompressed: String new ]
    ifFalse:[self _contents: String new ]
%

category: 'Reading'
method: GsFile
contentsAsUtf8

"Returns a Utf8 containing the contents of the receiver from the current
 position to the end of the file.  Returns nil if an error occurs."

^ self isCompressed
    ifTrue:[self _contentsUncompressed: Utf8 new ]
    ifFalse:[self _contents: Utf8 new ]
%

category: 'Writing'
method: GsFile
cr

"If the receiver is a text file then writes an end-of-line sequence to it.
 If the receiver is a binary file then writes a carriage return to it.
 Returns a count of bytes added, or nil if an error occurs."

(self _isBinary) ifTrue: [ | isCl |
  ((isCl := isClient) _isSmallInteger and:[ self isCommitted == false]) ifTrue:[ ^ self _setNotOpenError ].
  ^ self userAction: #GsfPutC onClient: isCl with: 13 "ASCII carriage return"
] ifFalse: [
  "ANSI-C text files will write eoln when they see a single lf character."
  ^ self lf
].
%

category: 'ANSI Compatibility'
method: GsFile
do: aBlock
"To be compatible with ANSI gettableStream (5.9.2.2):
Each member of the receiver's future sequence values is, in turn, removed from the future
sequence values; appended to the past sequence values; and, passed as the argument to an
evaluation of operand. The argument, operation, is evaluated as if sent the message #value:.
The number of evaluations is equal to the initial size of the receiver's future sequence
values. If there initially are no future sequence values, operation is not evaluated.
The future sequence values are used as arguments in their sequence order. The result is
undefined if any evaluation of operand changes the receiver's future sequence values"

	self positionA to: self fileSize - 1 do: [:i |
		aBlock value: self next.
	].
%

category: 'Writing'
method: GsFile
ff

"Writes a form-feed (page break) to the receiver's file.
 Returns true, or nil if an error occurs."

^ self nextPut:  Character newPage
%

category: 'Accessing'
method: GsFile
filePointer
"Returns a CPointer representing the underlying FILE * in C.
 Valid for open server files only. Returns nil the file is compressed,
 on the client, or if an error occurs. Care must be taken to not access
 the result of this method after the receiver has been closed."

^ self _zeroArgPrim: 3
%

category: 'File Operations'
method: GsFile
fileSize

"Returns the size in bytes of the receiver, or nil if an error occurs.

 Note that the value returned is independent of the open mode used to
 create the receiver."

^ self class _sizeOf: pathName onClient: isClient
%

category: 'File Operations'
method: GsFile
flush

"Flushes all written bytes to the file.  Returns the receiver, or nil if an
 error occurs. On Unix, calls fflush() . "
| isCl |
((isCl := isClient) _isSmallInteger and:[ self isCommitted == false]) ifTrue:[
   ^ self _setNotOpenError
].
^ self userAction: #GsfFlush onClient: isCl
%

category: 'Locking'
method: GsFile
getStatusForLockKind: kind atOffset: offset forBytes: bytes
"Tests if the receiver could be locked with the given arguments but without
 actually acquiring the lock.  Returns information about a conflicting lock,
 if any.

 The kind argument must be a symbol that indicates the lock kind requested, and
 must be either #readLock or #writeLock.  offset indicates the position within
 the receiver in bytes.  bytes indicates the number of bytes starting at offset
 (0 means to the end of the file).

 Returns an Array of 5 elements as described below.  If the first element is
 false, then elements 2 through 5 contain information about a conflicting
 lock (there may be more than one) which prevented the lock from being available.
 If the first element is true, then element 2 will be nil and elements 3
 through 5 will be -1.

 1 - a Boolean: true if the lock would have been granted, otherwise false.
 2 - a Symbol (#readLock or #writeLock): conflicting lock kind.
 3 - a SmallInteger: start position of the conflicting lock.
 4 - a SmallInteger: number of bytes held by the conflicting lock.
 5 - a SmallInteger: process ID of the owner of the conflicting lock.

 Raises an IOError if an error occurs.

 This method is not supported and will always raise an error if the receiver
 references a client file and the GCI client is running on the Microsoft
 Windows platform."

| result |
result := self userAction: #GsfLockQuery onClient: self isClient
               with: kind with: offset with: bytes .
^ result class == Array
     ifTrue:[ result ]
    ifFalse:[ IOError signal: result ]
%

category: 'Compressed File Operations'
method: GsFile
gzOpen

"If the receiver is not open, open it using the existing mode.
 Returns the receiver, or nil if an error occurs.
 If an error occurs,  there is no error information associated with
 the instance; you must use    GsFile lastErrorString   to retrieve the error. "

^self _openUsingGzip: true
%

category: 'Compressed File Operations'
method: GsFile
gzOpen: aPathName mode: openMode

"Opens the receiver's file with the given mode.  If the file is already open,
 it is closed and reopened.

 The openMode argument must be a String that that is equal to one of the
 following: 'r', 'w', 'a', 'r+', 'w+', 'a+', 'rb', 'wb', 'ab', 'r+b', 'w+b',
 'a+b', 'rb+', 'wb+', 'ab+'.  The mode has the same meaning as it does for the
 C library function, fopen().

 Returns the receiver if successful, or nil if an error occurs.
 If an error occurs,  there is no error information associated with
 the instance; you must use   GsFile lastErrorString to retrieve the error. "

self isOpen ifTrue:[ self close ].
mode := openMode.
pathName := aPathName.
^self gzOpen
%

category: 'Comparing'
method: GsFile
hash

"Returns a SmallInteger related to the value of the receiver.  If two instances
 of GsFile are equal (as compared by the = method), then they must have the same
 hash value.
 Gemstone 64 v3.0, hash algorithm has changed for GsFile
 "

^ pathName hash
%

category: 'File Operations'
method: GsFile
head: lineCount

"Returns a String containing the first lineCount lines from the
 receiver's file, or nil if an error occurs."

| result line |

self position: 0.
result := String new.
lineCount timesRepeat: [
  line := self nextLine.
  line ~~ nil ifTrue: [
    result addAll: line.
  ]
  ifFalse: [
    ^result
  ]
].
^result
%

category: 'Testing'
method: GsFile
isClient

"Returns true if the receiver's file is a client file, or nil if an error
 occurs."
| isCl |
(isCl := isClient) _isSmallInteger ifTrue:[ ^ (isCl bitAnd:1) == 1 ].
^ isCl
%

category: 'Testing'
method: GsFile
isCompressed

"Returns true if the file was opened in compressed mode or false
 if it was not or an uncompressed file was opened in compressed mode.

 Returns nil if the receiver is not in a valid open state."

^ (self _zeroArgPrim: 1) ifNil:[
    self userAction: #GsfIsCompressed onClient: isClient
  ]
%

category: 'ANSI Compatibility'
method: GsFile
isEmpty
"To be compatible with ANSI sequencedStream (5.9.1.3):
Returns true if both the set of past and future sequence values of the receiver are empty.
Otherwise returns false."

	^self fileSize == 0.
%

category: 'Testing'
method: GsFile
isExternal

"Is the source for this stream is external to GemStone Smalltalk."

^true
%

category: 'Testing'
method: GsFile
isOpen

"Returns true if the receiver's file is open, false otherwise"
| isCl |
((isCl := isClient) _isSmallInteger and:[ self isCommitted == false]) ifTrue:[
  ^ false
].
isClient ifNil:[ ^ false "an instance created with _basicNew" ].
^ self userAction: #GsfIsOpen onClient: isCl
%

category: 'Standard Files'
method: GsFile
isTerminal

"Returns true if receiver is stdin or stdout, and is connected to a terminal,
  i.e. in the client process , in C,  istty(fileno(f)) == 1 "

^ (pathName at: 1 equals:'std')
   and:[ (GsFile _fileKind: pathName onClient: self isClient) == 9 ]
%

category: 'Error Reporting'
method: GsFile
lastErrorCode

"Returns the currently posted error code, or zero if no error has occurred.
 A result of -1  means
   'attempt to access a GsFile that was never opened or has been closed' .
 Does not clear the error code or error string.

 If the last error was a failure during reopen of a closed file,
 this method returns zero;  you must use lastErrorString to get information
 on such reopen  failures.   This situation applies to errors produce by
 sending  open   or   open:mode:    to an instance of GsFile. "

 isClient _isSmallInteger ifFalse:[
   "a pre v2.1 committed or open instance, get result from C"
   ^ self userAction: #GsfErrorCode onClient: isClient
 ].
 self isCommitted ifTrue:[
   "v2.1 committed instance, get result from C"
   ^ self userAction: #GsfErrorCode onClient: isClient
 ].
 "at this point we have a v2.1 transient instance."
 isClient >= 2 ifTrue:[
   ^ -1 "attempt to access a GsFile that was never opened or has been closed."
 ].
 ^ 0
%

category: 'Error Reporting'
method: GsFile
lastErrorString

"Returns the currently posted error string, or nil if no error has occurred.
 Clears the error string and error code.
 If the last error was a failure during reopen of a closed file,
 will retrieve and clear the error string from the class."

 isClient _isSmallInteger ifFalse:[
   "a pre v2.1 committed or open instance, get result from C"
   ^ self userAction: #GsfErrorString onClient: isClient
 ].
 self isCommitted ifTrue:[
   "v2.1 committed instance in any state, get result from C"
   ^ self userAction: #GsfErrorString onClient: isClient
 ].
 "at this point we have a v2.1 transient instance."
 isClient >= 2 ifTrue:[
   "change state from 'closed, access error' to 'closed' and return err string."
   isClient := isClient - 2 .
   ^ 'attempt to access a GsFile that was never opened or has been closed'.
 ].
 "get class' error string for last unsuccessful open/reopen."
 ^ self class lastErrorString .
%

category: 'Accessing'
method: GsFile
lastModified
"Returns a DateTime that represents the last time the receiver's file
 was modified. Returns nil if an error occurs."

^ GsFile _fileModTime: (self pathName) onClient: isClient .
%

category: 'Writing'
method: GsFile
lf

"Append an end-of-line to the receiver.
 Returns true if successful, nil if an error occurred."

^ self nextPut: 10"ASCII line-feed Character"
%

category: 'Writing'
method: GsFile
log: string

"Writes the contents of the given collection to the receiver's file at the
 current position.  Appends a newline to the file if the string does not end
 with one.  The argument must be a kind of Collection with byte format.

 Returns the receiver if successful; returns nil otherwise."

| lf result |

result := self nextPutAll: string.
(result ~~ nil and:[ result > 0]) ifTrue:[
  lf := Character lf.
  string last == lf ifFalse: [ result := self nextPut: lf ]
  ].
result == nil ifTrue: [ ^nil ].
self flush.
^ self
%

category: 'Accessing'
method: GsFile
maxSize
  "For compatibility with Stream , return a large value"

  ^ SmallInteger maximumValue
%

category: 'Accessing'
method: GsFile
mode

"Returns the access mode of the receiver's file."

^mode
%

category: 'Accessing'
method: GsFile
name

"Returns the receiver's file path name."

^ pathName
%

category: 'Reading'
method: GsFile
next

"Returns a Character representing the next byte from the receiver's file.
 Returns nil if an error occurs, or if receiver is at EOF."

<primitive: 1014>
"primitive fails if receiver not an open server file, or if EINTR occurs"
| isCl res |
((isCl :=isClient) _isSmallInteger and:[ self isCommitted == false]) ifTrue:[
  ^ self _setNotOpenError
].
[
  res := self userAction: #GsfGetC onClient: isCl .
  res _isSmallInteger   "repeat loop if EINTR , to allow ctl-C detection"
] untilFalse .
^ res
%

category: 'Reading'
method: GsFile
next: numberOfBytes

"Returns a String containing the next numberOfBytes Characters from the
 receiver's file, or nil if an error occurs."

| result count |
result := String new.
count := self _read: numberOfBytes into: result .
(count == 0 or:[ count == nil]) ifTrue:[ ^ nil ].
^ result
%

category: 'Reading'
method: GsFile
next: amount byteStringsInto: byteObj

"Reads bytes written by printBytes: into the given byte object.
 Returns count of bytes read, or nil if an error occurs."
| isCl |
((isCl := isClient) _isSmallInteger and:[ self isCommitted == false]) ifTrue:[
   ^ self _setNotOpenError
].
^ self userAction: #GsfLoadByteStrings onClient: isCl with: byteObj with: amount
%

category: 'Reading'
method: GsFile
next: numberOfBytes into: aByteObject

"Reads the next numberOfBytes into the given collection object. The
 object's size is truncated to the amount of data actually read.
 Returns a count of bytes read, or nil if EOF or an error occurs."

| cnt |
cnt := self _read: numberOfBytes into: aByteObject .
cnt == 0 ifTrue:[ ^ nil ].
cnt ifNotNil:[
  cnt < aByteObject size ifTrue:[ aByteObject size: cnt ].
].
^cnt
%

category: 'Reading'
method: GsFile
next: numberOfItems ofSize: bytesPerItem into: byteObj

"Reads bytes for the next numberOfItems of the given bytesPerItem into
 the given collection object. The object's size is truncated to the
 amount of data actually read.  bytesPerItem must between 1 and 4096 inclusive.

 Returns a count of bytes read, or nil if an error occurs."
| count |
count := self _read: numberOfItems ofSize: bytesPerItem into: byteObj .
(count == 0 or:[ count == nil]) ifTrue:[ ^ nil ].
count < byteObj size ifTrue:[ byteObj size: count ].
^ count
%

category: 'Reading'
method: GsFile
nextByte

"Returns the next byte (integer) from the receiver's file.
 Returns nil if an error occurs, or if receiver is at EOF."

| c |
c := self next .
c ifNil:[ ^ nil ].
^ c codePoint
%

category: 'Reading'
method: GsFile
nextLine

"Returns a String containing the next line from the receiver's file. The String
 will be terminated with a newline, unless the end of file is reached and there
 is no line terminator.  If receiver is positioned at the end of the file, or
 a read error occurs, returns nil.

 There is no limit on line size."

^ self nextLineTo: Character lfValue prompt: nil
%

category: 'Reading'
method: GsFile
nextLineInto: str startingAt: pos

"Deprecated,  new code should use  nextLineTo: .
 Reads the next line from the receiver's file into the given collection object,
 starting at the given position in the collection. The collection will be
 terminated with a newline, unless the end of file is reached and there is no
 line terminator.  If the receiver is positioned at the end of the file, nothing
 is written.  Returns a count of bytes read, or nil if an error occurs."

"There is no limit on line size."

| res isCl |
self deprecated: 'GsFile>>nextLineInto:startingAt: deprecated in v3.2, use nextLineTo: instead'.
((isCl := isClient) _isSmallInteger and:[ self isCommitted == false]) ifTrue:[
   ^ self _setNotOpenError
].
res := self userAction: #GsfGetLine onClient: isCl with: str with: pos
		with: Character lfValue with: nil .
res ~~ nil ifTrue:[ ^ res size - pos + 1 ].
^ res
%

category: 'Reading'
method: GsFile
nextLineTo: eolValue
  ^ self nextLineTo: eolValue prompt: nil
%

category: 'Reading'
method: GsFile
nextLineTo: eolValue prompt: promptString
| res |
[
  "if receiver is stdin, GsfGetLine will return one of
    nil (for EOF), a String, or a SmallInteger (if interrupted by a ctl-C) .
   otherwise GsfGetLine returns nil or a String .
   By rerunning this loop if a SmallInteger is obtained, we give a chance
   for the SIGINT handler to signal a soft-break error
  "
  res := self _nextLineTo: eolValue prompt: promptString .
  res _isSmallInteger
] untilFalse .
^ res
%

category: 'Reading'
method: GsFile
nextLineToChar: eolCharacter

^ self nextLineTo: eolCharacter codePoint prompt: nil
%

category: 'ANSI Compatibility'
method: GsFile
nextMatchFor: anObject
"To be compatible with ANSI gettableStream (5.9.2.2):
The first object is removed from the receiver's future sequence value and appended to
the end of the receiver's past sequence values. The value that would result from sending
#= to the object with anObject as the argument is returned.
The results are undefined if there are no future sequence values in the receiver."

	^self next = anObject.
%

category: 'Writing'
method: GsFile
nextPut: aByte

"Writes the given byte to the receiver's file at the current position.
 aByte must be a Character or a SmallInteger in the range 0..255.

 If aByte is a SmallInteger, it will be interpreted logically as

 Character withValue: aByte

 Returns true, or nil if an error occurs."

<primitive: 1011>
"handle file not open on server"
| isCl |
((isCl :=isClient) _isSmallInteger and:[ self isCommitted == false]) ifTrue:[
  ^ self _setNotOpenError
].
^ self userAction: #GsfPutC onClient: isCl with: aByte
%

category: 'Writing'
method: GsFile
nextPutAll: collection

"Writes the contents of the given collection to the receiver's file at the
 current position. The argument must be a kind of Collection with byte
 format.  Returns a count of bytes added, or nil if an error occurs."

^ self write: (collection _basicSize) from: collection
%

category: 'Writing'
method: GsFile
nextPutAllBytes: collection

"Writes the byte contents of the given collection to the receiver's file at the
 current position.  The argument must be a kind of Collection with byte
 format.  Returns a count of bytes added, or nil if an error occurs."

^ self nextPutAll: collection
%

category: 'Writing'
method: GsFile
nextPutAllUtf8: aStringOrCharacter

"Writes the contents of the given String, MultiByteString or Utf8 to the
 receiver's file using UTF8 encoding .
 Returns a count of bytes added, or nil if an error occurs."

| utf |
aStringOrCharacter class == Character ifTrue:[
  utf := Unicode16 with: aStringOrCharacter.
  utf := utf encodeAsUTF8 .
] ifFalse:[
  utf := aStringOrCharacter encodeAsUTF8 .
].
^ self write: utf size from: utf .
%

category: 'Writing'
method: GsFile
nextPutAsUtf8: aString
  "Will be deprecated.   Use nextPutAllUtf8: "
  ^ self nextPutAllUtf8: aString
%

category: 'File Operations'
method: GsFile
open

"If the receiver is not open, open it using the existing mode.
 Returns the receiver, or nil if an error occurs.
 If an error occurs,  there is no error information associated with
 the instance; you must use    GsFile lastErrorString   to retrieve the error. "

^self _openUsingGzip: false
%

category: 'File Operations'
method: GsFile
open: aPathName mode: openMode

"Opens the receiver's file with the given mode.  If the file is already open,
 it is closed and reopened.

 The openMode argument must be a String that that is equal to one of the
 following: 'r', 'w', 'a', 'r+', 'w+', 'a+', 'rb', 'wb', 'ab', 'r+b', 'w+b',
 'a+b', 'rb+', 'wb+', 'ab+'.  The mode has the same meaning as it does for the
 C library function, fopen().

 Returns the receiver if successful, or nil if an error occurs.
 If an error occurs,  there is no error information associated with
 the instance; you must use   GsFile lastErrorString to retrieve the error. "

self isOpen ifTrue:[
  self close  .
  self isOpen ifTrue:[
    ^ self "a standard file that cannot be closed"
  ].
].
mode := openMode.
pathName := aPathName .
^self open
%

category: 'Accessing'
method: GsFile
pathName

"Returns the receiver's file path name."

^pathName
%

category: 'Reading'
method: GsFile
peek

"Returns the a Character representing the next byte in the receiver's file,
 without advancing the current pointer.
 Returns nil if an error occurs, or peek hits EOF."

| result |
[
  (result := self _peek: 1 ) == false
] whileTrue . "loop to handle EINTR"
result == true ifTrue:[
  | pos |  "handle client file"
  pos := self position.
  result := self next.
  self position: pos.
].
^ result
%

category: 'Reading'
method: GsFile
peek2

"Returns the next byte plus one in the receiver's file, without advancing the
 current pointer.  Returns nil if an error occurs, or peek hits EOF."
| result |
[
  (result := self _peek: 2 ) == false
] whileTrue . "loop to handle EINTR"
result == true ifTrue:[
  | pos |  "handle client file"
  pos := self position.
  result := self next; next.
  self position: pos.
].
^result
%

category: 'ANSI Compatibility'
method: GsFile
peekFor: expectedObject
"To be compatible with ANSI gettableStream (5.9.2.8):
Returns the result of sending #= to the first object in the receiver's future sequence
values with anObject as the argument. Returns false if the receiver has no future
sequence values."

	| foundObject flag |
	foundObject := self peek.
	(flag := expectedObject = foundObject) ifTrue: [
		self next.
	].
	^flag.
%

category: 'Positioning'
method: GsFile
position

"Returns the current position of the receiver's file,
 or nil if an error occurs.  Result is zero based."
 ^ (self _zeroArgPrim: 2) ifNil:[
   | isCl |
   ((isCl := isClient) _isSmallInteger and:[ self isCommitted == false]) ifTrue:[
     ^ self _setNotOpenError
   ].
   ^ self userAction: #GsfPosition onClient: isCl
 ].
%

category: 'Positioning'
method: GsFile
position: offset

"Changes the receiver's position in its file to the given offset, which may be
 negative or zero.  Returns the new position, or nil if an error occurs."

^ self _seekTo: offset opcode: 0 .
%

category: 'Positioning'
method: GsFile
positionA
	"For compatibility with PositionableStream"

	^self position.
%

category: 'Positioning'
method: GsFile
positionA: offset
	"For compatibility with PositionableStream"

	^self position: offset.
%

category: 'Positioning'
method: GsFile
positionL
	"For compatibility with PositionableStream"

	^self position.
%

category: 'Positioning'
method: GsFile
positionL: offset
	"For compatibility with PositionableStream"

	^self position: offset.
%

category: 'Positioning'
method: GsFile
positionW
	"For compatibility with PositionableStream"

	^self position.
%

category: 'Positioning'
method: GsFile
positionW: offset
	"For compatibility with PositionableStream"

	^self position: offset.
%

category: 'Private'
method: GsFile
postCopy
	"Do cleanup on new copy. Ensure that it is closed."

super postCopy.
isClient := self isClient ifTrue:[ 1 ] ifFalse:[ 0 ]
%

category: 'Writing'
method: GsFile
printBytes: byteObj

"Prints the bytes from the given byte object in decimal notation with
 line breaks to keep output lines from being too long.

 Returns the receiver, or nil if an error occurs.

 See also the method next:byteStringsInto:."

(isClient _isSmallInteger and:[ self isCommitted == false]) ifTrue:[ ^ self _setNotOpenError ].
^ self userAction: #GsfPrintBytes onClient: isClient with: byteObj
%

category: 'Reading'
method: GsFile
read: numberOfBytes into: byteObj

"Reads up to the given number of bytes into the given byte object (for
 example, a String) starting at index 1.
 Returns the number of bytes read, or nil if an error occurs,
 or 0 if EOF on the receiver.
 The destination object is grown as needed, but is not shrunk. "

^ self _read: numberOfBytes into: byteObj
%

category: 'Locking'
method: GsFile
readLockAtOffset: offset forBytes: bytes
"Requests a read lock on a portion of the receiver without blocking.  Returns
 true if successful, false if the lock was denied. Raises an IOError if an error
 occurred.

 See the method #_acquireLockKind:atOffset:forBytes:waitTime: for more
 information about file locks."

 ^ self readLockAtOffset: offset forBytes: bytes waitTime: 0
%

category: 'Locking'
method: GsFile
readLockAtOffset: offset forBytes: bytes waitTime: milliseconds
"Requests a read lock on a portion of the receiver and blocks for up to
 milliseconds to acquire the lock.  Returns true if successful, false if the lock
 was denied. Raises an IOError if an error occurred.

 See the method #_acquireLockKind:atOffset:forBytes:waitTime: for more
 information about file locks."

 ^ self _acquireLockKind: #readLock atOffset: offset forBytes: bytes waitTime: milliseconds
%

category: 'Locking'
method: GsFile
readLockContents
"Requests a read lock on the entire contents the receiver without blocking.
 Returns true if successful, false if the lock was denied. Raises an IOError if
 an error occurred.

 See the method #_acquireLockKind:atOffset:forBytes:waitTime: for more
 information about file locks."

 ^ self readLockAtOffset: 0 forBytes: 0
%

category: 'ANSI Compatibility'
method: GsFile
reset
"To be compatible with ANSI sequencedStream (5.9.1.6):
Sets the receiver's future sequence values to be the current past sequence values appended
with the current future sequence values. Make the receiver's past sequence values be empty."

	self positionA: 0.
%

category: 'Positioning'
method: GsFile
rewind

"Repositions the receiver's file to the beginning.  Returns 0, or nil if an
 error occurs."

self position: 0
%

category: 'Positioning'
method: GsFile
seekFromBeginning: offset

"Moves the receiver's position in its file to the given offset from the
 beginning of the file, which may be positive or zero, but not negative.
 Returns the new position, or nil if an error occurs."

^ self _seekTo: offset opcode: 0
%

category: 'Positioning'
method: GsFile
seekFromCurrent: offset

"Changes the receiver's position in its file by the given offset, which
 may be negative or zero.  Returns the new position, or nil if an error occurs."

^ self _seekTo: offset opcode: 1
%

category: 'Positioning'
method: GsFile
seekFromEnd: offset

"Moves the receiver's position in its file to the given offset from the
 end of the file, which may be negative or zero, but not positive.
 Returns the new position, or nil if an error occurs.  Not supported
 for compressed files."

^ self _seekTo: offset opcode: 2
%

category: 'ANSI Compatibility'
method: GsFile
setToEnd
"To be compatible with ANSI sequencedStream (5.9.1.7):
All of the receiver's future sequence values are appended, in sequence, to the receiver's
past sequence values. The receiver then has no future sequence values."

	self positionA: self fileSize.
%

category: 'Reading'
method: GsFile
skip: count

"Changes the receiver's position in its file by the given offset, which may be
 zero, or negative.  Returns the new position, or nil if an error occurs."

| pos |
pos := self position.
^ self position: pos + count
%

category: 'ANSI Compatibility'
method: GsFile
skipTo: anObject
"To be compatible with ANSI gettableStream (5.9.2.10):
Each object in the receiver's future sequence values up to and including the first
occurrence of an object that is equivalent to anObject is removed from the future
sequence values and appended to the receiver's past sequence values. If an object that
is equivalent to anObject is not found in the receiver's future sequence values, all
of the objects in future sequence values are removed from future sequence values and
appended to past sequence values. If an object equivalent to anObject is not found
false is returned. Otherwise return true."

	self positionA to: self fileSize - 1 do: [:i |
		self next = anObject ifTrue: [^true].
	].
	^false.
%

category: 'Writing'
method: GsFile
space

"Append an space character to the receiver.
 Returns true if successful, nil if an error occurred."

^ self nextPut: 32 " $   codePoint "
%

category: 'File Operations'
method: GsFile
sync

"Flushes all written bytes to the file's storage device.  Returns the receiver, or nil if an
 error occurs. On Unix, calls fflush() followed by fsync() . "
| isCl |
((isCl := isClient) _isSmallInteger and:[ self isCommitted == false]) ifTrue:[
   ^ self _setNotOpenError
].
^ self userAction: #GsfSync onClient: isCl
%

category: 'Writing'
method: GsFile
tab

"Append a tab to the receiver.
 Returns true if successful, nil if an error occurred."

^ self nextPut: 9  "ASCII tab"
%

category: 'Locking'
method: GsFile
unlockAtOffset: offset forBytes: bytes

"Releases a previously acquired lock on a portion the file referenced by the
 receiver.  offset indicates the position within the receiver in bytes and bytes
 indicates the number of bytes to unlock (0 means unlock the file from offset to
 the end of the file).

 Please refer the comments in the following method for more details on file
 locks:
   GSFile _acquireLockKind: atOffset: forBytes: waitTime:

 Returns true if the lock was released or if the region was not locked by the process.
 Raises an IOError if an error occurs."

| result |
result := self userAction: #GsfUnlock onClient: self isClient with: offset with: bytes .
^ result class == Boolean
     ifTrue:[ result ]
    ifFalse:[ IOError signal: result ]
%

category: 'Locking'
method: GsFile
unlockContents
"Releases a previously acquired lock on contents of the receiver .

 Returns true if the lock was released successfully or if the contents were not
 locked by the process.  Raises an IOError if an error occurs.

 See the method #_acquireLockKind:atOffset:forBytes:waitTime: for more
 information about file locks."

^ self unlockAtOffset: 0 forBytes: 0
%

category: 'ANSI Compatibility'
method: GsFile
upTo: anObject
"To be compatible with ANSI gettableStream (5.9.2.10):
Each object in the receiver's future sequence values up to and including the first
occurrence of an object that is equivalent to anObject is removed from the future
sequence values and appended to the receiver's past sequence values. A collection,
containing, in order, all of the transferred objects except the object (if any) that
is equivalent to anObject is returned. If the receiver's future sequence values is
initially empty, an empty collection is returned."

	| result |
	result := String new.
	self positionA to: self fileSize - 1 do: [:i |
		| char |
		(char := self next) = anObject ifTrue: [^result].
		result add: char.
	].
	^result.
%

category: 'Private'
method: GsFile
userAction: actionName onClient: onClient

"Executes a GsFile user action, passing it the following arguments:
    self
    any arguments to the primitive (see other uses of primitive 396)
 Maximum of 6 arguments (6 with: keywords) for this primitive.
 actionName must be a Symbol "

<primitive: 396>

^ self _primitiveFailed: #userAction:onClient: args: { actionName . onClient }
%

category: 'Private'
method: GsFile
userAction: actionName onClient: onClient with: arg1

"See GsFile | userAction:onClient: for documentation."

<primitive: 396>

^ self _primitiveFailed: #userAction:onClient:with:
       args: { actionName . onClient . arg1 }
%

category: 'Private'
method: GsFile
userAction: actionName onClient: onClient with: arg1 with: arg2

"See GsFile | userAction:onClient: for documentation."

<primitive: 396>

^ self _primitiveFailed: #userAction:onClient:with:with:
       args: { actionName . onClient . arg1 . arg2 }
%

category: 'Private'
method: GsFile
userAction: actionName onClient: onClient with: arg1 with: arg2 with: arg3

"See GsFile | userAction:onClient: for documentation."

<primitive: 396>

^ self _primitiveFailed: #userAction:onClient:with:with:with:
       args: { actionName . onClient . arg1 . arg2 . arg3 }
%

category: 'Private'
method: GsFile
userAction: actionName onClient: onClient with: arg1 with: arg2 with: arg3 with: arg4

"See GsFile | userAction:onClient: for documentation."

<primitive: 396>

^ self _primitiveFailed: #userAction:onClient:with:with:with:with:
       args: { actionName . onClient . arg1 . arg2 . arg3 . arg4 }
%

category: 'Writing'
method: GsFile
write: amount from: byteObj

"Write the given number of bytes from the given byte object.  Returns the
 number of bytes written, or nil if an error occurs."

<primitive: 875>  "primitive succeeds only if file is open on server"
| isCl |
((isCl:=isClient) _isSmallInteger and:[ self isCommitted == false]) ifTrue:[
  ^ self _setNotOpenError
].
^ self userAction: #GsfWrite onClient: isCl
       with: byteObj with: 1 with: amount
%

category: 'Writing'
method: GsFile
write: byteObj itemCount: itemCount ofSize: bytesPerItem

"Writes itemCount items of size bytesPerItem from the given byte object to
 the receiver's file.  bytesPerItem must be between 1 and 4096 inclusive.
 Returns a count of bytes written, or nil if an error occurs."

^ self write: itemCount * bytesPerItem from: byteObj
%

category: 'Locking'
method: GsFile
writeLockAtOffset: offset forBytes: bytes
"Requests a write lock on a portion of the receiver without blocking.
 Returns true if successful, false if the lock was denied. Raises an IOError if
 an error occurred.

 See the method #_acquireLockKind:atOffset:forBytes:waitTime: for more
 information about file locks."

 ^ self writeLockAtOffset: offset forBytes: bytes waitTime: 0
%

category: 'Locking'
method: GsFile
writeLockAtOffset: offset forBytes: bytes waitTime: milliseconds
"Requests a write lock on a portion of the receiver and blocks for up to
 milliseconds to acquire the lock.  Returns true if successful, false if the
 lock was denied. Raises an IOError if an error occurred.

 See the method #_acquireLockKind:atOffset:forBytes:waitTime: for more
 information about file locks."

 ^ self _acquireLockKind: #writeLock atOffset: offset forBytes: bytes waitTime: milliseconds
%

category: 'Locking'
method: GsFile
writeLockContents
"Requests a write lock on the entire contents the receiver without blocking.
 Returns true if successful, false if the lock was denied. Raises an IOError
 if an error occurred.

 See the method #_acquireLockKind:atOffset:forBytes:waitTime: for more
 information about file locks."

 ^ self writeLockAtOffset: 0 forBytes: 0
%

category: 'Locking'
method: GsFile
_acquireLockKind: kind atOffset: offset forBytes: bytes waitTime: milliseconds

"Requests a lock on a portion the file referenced by the receiver.  kind
 is a symbol that indicates the lock kind and must be either #readLock or
 #writeLock.  A read lock is a shared lock such that a given byte in a file
 may have multiple read locks on it at the same time.  A write lock is an
 exclusive lock which precludes all other read locks and write locks.

 offset indicates the position within the receiver in bytes.
 offset is zero-based; the first by in the file is at offset 0.  bytes
 indicates the number of bytes to lock (0 means lock the file from offset to the
 end of the file).

 waitTime is the number of milliseconds to wait when requesting a lock and
 the lock is unavailable.  0 means return immediately (do not wait).  -1 means
 wait forever.  Specifying a waitTime greater than zero causes the session to
 rapidly poll for the lock to be available.

 For processes running on UNIX based systems, file locks are advisory, which
 means locks are not enforced by the operating system and applications may
 freely ignore them.

 For processes running on Microsoft Windows-based systems, file locks are
 mandatory, which means locks are enforced by the operating system and
 applications cannot ignore them.

 *** Warning ***
 On UNIX-based systems only, closing any handle referencing a given physical
 file forces *all* locks on that file held by the process to be released.  For
 this reason, locks should not be used on any file which the process opens more
 than once.

 This method is highly dependent upon the underlying operating system
 implementation of advisory file locks.  On UNIX-based systems, the fcntl(F_SETLK)
 call is used.  On Microsoft Windows systems, the LockFileEx() call is used.
 Consult the documentation for your host system to learn about the behavior of
 file locking for your system.

 Some systems do not support read locks if the file is opened in write-only
 mode or write locks if the file is open in read-only mode.  Attempting to lock
 files opened in these manners may raise an error indicating a bad file
 descriptor.

 Returns true if the lock was acquired, false if the lock was denied.
 Raises an IOError if an error occurs."

| result |
result := self userAction: #GsfLock onClient: self isClient with: kind with: offset with: bytes with: milliseconds .
^ result class == Boolean
     ifTrue:[ result ]
    ifFalse:[ IOError signal: result ]
%

category: 'Reading'
method: GsFile
_contents: resultObj

"Read the contents of the receiver from the current
 position to the end of the file into resultObj.
 Returns nil if an error occurs, otherwise returns resultObj."

| mySize myPosition |

mySize := self fileSize .
(mySize == nil or:[ mySize == 0]) ifTrue:[
  "a file not reporting size like stdin, or of size zero like /dev/zero"
  | buf |
  buf := resultObj class new .
  [ true ] whileTrue:[ | status |
    status := self _next:1024 into: buf .
    status ifNil:[ ^ resultObj ].
    resultObj addAll: buf
  ]
].
myPosition := self position .
(mySize + myPosition) == 0 ifTrue:[ ^ resultObj ]. "a zero length file"
myPosition == nil ifTrue:[ ^ nil ] .
(self next: ( mySize - myPosition ) into: resultObj ) ifNil:[ ^ nil ].
^ resultObj
%

category: 'Private'
method: GsFile
_contentsUncompressed: resultObj

| buf |
buf := resultObj class new .
[ true ] whileTrue:[ | count |
  count := self _read: 4096 into: buf .
  count == 0 ifTrue:[ ^ resultObj ].
  count ifNil:[ ^ nil ].
  count < 4096 ifTrue:[
    buf size: count  "truncate to match amount read"
  ].
  resultObj addAll: buf
].
%

category: 'Fileout'
method: GsFile
_fileOutAll: aString

  self nextPutAll: aString encodeAsUTF8
%

category: 'Private'
method: GsFile
_isBinary

"Returns true if receiver is a binary file. Otherwise returns false."
| m |
(m := mode) ifNil: [ ^ false].
^ m includesValue: $b
%

category: 'Private'
method: GsFile
_isReadable

"Returns true if receiver is open for read, otherwise returns false."
| m |
(m := mode) ifNil:[ ^ false].
^ (m includesValue: $r) or:[ m includesValue: $+ ]
%

category: 'Private'
method: GsFile
_newEmptyClientCData

"initialized cData for a client file, if cData not already
 initialized for a server file."

<primitive: 675>
self _primitiveFailed: #_newEmptyClientCData
%

category: 'Private'
method: GsFile
_newStdFile: stdId isClient: clientBool

  "Initialize the receiver to represent a standard file."

  clientBool ifTrue:[
    isClient := true .
    fileDescriptor := -1 .
  ] ifFalse:[
    isClient := false .
    fileDescriptor := stdId .
  ].
  stdId == 0 ifTrue: [mode := 'r'. pathName := 'stdin']
          ifFalse:[mode := 'w'.
                   stdId == 1 ifTrue: [pathName := 'stdout']
                             ifFalse:[pathName := 'stderr'].
                  ].

%

category: 'Reading'
method: GsFile
_next: numberOfBytes basicInto: aByteObject

"GsFile doesn't distinguish Strings from EUCStrings, so no reimplementation
 needed."

^ self next: numberOfBytes into: aByteObject
%

category: 'Reading'
method: GsFile
_next: numberOfBytes into: resultObj

"Returns a String containing the next numberOfBytes Characters from the
 receiver's file, or nil if an error occurs."

| count |
count := self _read: numberOfBytes into: resultObj .
(count == 0 or:[ count == nil]) ifTrue:[ ^ nil ].
^ resultObj
%

category: 'Reading'
method: GsFile
_nextLineTo: eolValue prompt: promptString

"Reads the next line from the receiver's file into a new String .

 eolValue is either a SmallInteger 0..255, or a String with
 size >= 1 and <= 128 .

 The String will be terminated with the codePoint per eolValue
 unless the end of file is reached and there is no line terminator.
 Returns the new String, or nil if an error occurs.
 Result is nil if file is at EOF.

 If promptString is non-nil, and receiver is stdin, and
   isatty(fileno(stdin)) == 1 , then the topaz line editor will be used
 to read the input
 Returns -2 if EINTR occurred."

<primitive: 723>  "primitive succeeds for server file"
| isCl |
((isCl := isClient) _isSmallInteger and:[ self isCommitted == false]) ifTrue:[
  ^ self _setNotOpenError
].
^ self userAction: #GsfGetLine onClient: isCl with: nil
                with: 1 with: eolValue with: promptString.
%

category: 'Private'
method: GsFile
_nextPutAllBytes: collection
  "used by PassiveObject"
| str |
(str := String new) addAllBytes: collection . "str is big-endian"
^ self nextPutAll: str .
%

category: 'Private'
method: GsFile
_open: aPath mode: aMode  onClient:  clientBool

  "Initialize the receiver as specified by the arguments."

  "Does not actually open the underlying file."

  clientBool ifTrue:[ isClient := 3 ] ifFalse:[ isClient := 2 ].
  pathName := aPath .
  mode := aMode .
%

category: 'Compressed File Operations'
method: GsFile
_openUsingGzip: aBoolean

"If the receiver is not open, open it using the existing mode.
 If aBoolean is true, the gzip function gzopen() is used to open
 the file.  gzopen() can be used to open an existing file which was
 not compressed with gzip.  In that case, the file will be read
 without decompression.

 Returns the receiver, or nil if an error occurs.
 If an error occurs,  there is no error information associated with
 the instance; you must use:
     GsFile lastErrorString
 to retrieve the error. "


| res path |
(pathName == nil or:[mode == nil]) ifTrue: [ ^nil ].
(self isOpen) ifTrue:[ ^self ].
mode stringCharSize == 1 ifFalse:[
   ArgumentError signal: 'mode must be a String or Unicode7'. ^nil.
].
path := GsFile _utfPath: pathName forClient: isClient .
res := self userAction: #GsfOpen onClient: isClient
	         with: path with: mode with: aBoolean.
res ifNotNil:[
  self _setIsOpen .
  res == 1 ifTrue:[ self _newEmptyClientCData ].
  self isClient ifTrue:[
    fileDescriptor := -1 .
  ].
  fileDescriptor := self userAction: #GsfGetFileDesc onClient: false .
  " maglev only ifFalse:[
    IO _rememberFileDescriptor: fileDescriptor obj: self .
  ]."
  ^ self .
].
^ res
%

category: 'Private'
method: GsFile
_peek: numBytes

"Returns nil for error on server file, false if EINTR occurred,
 true if receiver is a client file.
 Primitive fails for invalid numBytes argument."
<primitive: 1013>
(numBytes == 1 or:[ numBytes == 2]) ifFalse:[
   ArgumentError signal:'invalid number of bytes for peek'].
^ self _primitiveFailed: #_peek: args: { numBytes }
%

category: 'Reading'
method: GsFile
_read: numberOfBytes into: byteObj

"Reads up to the given number of bytes into the given byte object
 starting at index 1.   byteObj must be a String or ByteArray .
 Returns the number of bytes read, or nil if an error occurs,
 or 0 if EOF on the receiver.
 The destination object is grown as needed, but is not shrunk"

<primitive: 874>
"primitive succeeds only if file is open on server, and no EINTR occurred"
^ self _read: numberOfBytes ofSize: 1 into: byteObj .  "client file or EINTR"
%

category: 'Reading'
method: GsFile
_read: numberOfItems ofSize: bytesPerItem into: byteObj

"Reads bytes for the next numberOfItems of the given bytesPerItem into
 the given collection object. The object's size is truncated to the
 amount of data actually read.  bytesPerItem must between 1 and 4096 inclusive.

 Returns a count of bytes read, 0 if hit EOF, or nil if an error occurs."
| count isCl |
( (isCl := isClient) _isSmallInteger and:[ self isCommitted == false]) ifTrue:[
   ^ self _setNotOpenError
].
[ count := self userAction: #GsfRead onClient: isCl
       with: byteObj with: bytesPerItem with: numberOfItems .
  count == -2
] whileTrue . "loop to handle EINTR and soft break"
^ count
%

category: 'Private'
method: GsFile
_seekTo: offset opcode: seekKind

"seekKind  function
   0        seek to  start of file + offset
   1        seek  offset from current position
   2        seek to  end of file - offset

 Returns the new position, or nil if an error occurs."
<primitive: 1010>
"primitive failed, handle client file"
| isCl |
((isCl:=isClient) _isSmallInteger and:[ self isCommitted == false]) ifTrue:[
  ^ self _setNotOpenError
].
^ self userAction: #GsfSeek onClient: isCl with: offset with: seekKind
%

category: 'Private'
method: GsFile
_setIsOpen

  "for a transient instance, change state to 'open' "
  self isCommitted ifFalse:[
    isClient := (isClient bitAnd: 1) == 1  "convert 0/1, 2/3 to false/true"
  ]
%

category: 'Private'
method: GsFile
_setNotOpened

  "for a transient instance,  change state to 'closed' "
  self isCommitted ifFalse:[
    isClient _isSmallInteger ifFalse:[
       isClient ifTrue:[ isClient:= 1 ] ifFalse:[ isClient:=0 ].
    ].
  ].
%

category: 'Private'
method: GsFile
_setNotOpenError

  "set state of a transient instance to 'closed, access error' "

  self isCommitted ifFalse:[
    isClient _isSmallInteger ifFalse:[
      self error:'invalid state' .
      self _uncontinuableError
    ].
    isClient < 2 ifTrue:[ isClient := isClient + 2 ].
  ].
  ^ nil
%

category: 'Private'
method: GsFile
_zeroArgPrim: opcode

"returns nil for a client file or file error.

  opcode 1   isCompressed
  opcode 2   position
  opcode 3   filePointer
"
<primitive: 1012>
self _primitiveFailed: #_zeroArgPrim: args: { opcode }
%

category: 'Comparing'
method: GsFile
~= aFile

"Returns false if the receiver and aFile represent the same file system file.
 Returns true otherwise."

^ (self = aFile) not
%

! Class extensions for 'GsMethodDictionary'

!		Instance methods for 'GsMethodDictionary'

removeallmethods GsMethodDictionary
removeallclassmethods GsMethodDictionary

category: 'Enumerating'
method: GsMethodDictionary
accompaniedBy: anObj keysAndValuesDo: aBlock

"Evaluates aBlock with each of the receiver's key/value pairs as the
 2nd and 3rd arguments.
 aBlock must be a 3 argument block, with arguments anObj, key value ."

| aKey tSize |
"process hash slots"
tSize := tableSize * 2 .
1 to: tSize by: 2 do: [ :offset |
  nil == (aKey := self _basicAt: offset) ifFalse:[
    aBlock value: anObj value: aKey value: (self _basicAt: offset + 1) .
    ].
  ].
"process collision chains"
tSize + 1 to: self _basicSize by: 3 do:[:offset |
  (aKey := self _basicAt: offset) == nil ifFalse:[
    aBlock value: anObj value: aKey value: (self _basicAt: offset + 1) .
    ].
  ].
%

category: 'Enumerating'
method: GsMethodDictionary
associationsDo: aBlock

"Evaluates aBlock with each of the receiver's key/value pairs as the argument by
 creating a SymbolAssociation for each key/value pair.  The argument aBlock must
 be a one-argument block.  Returns the receiver."

self keysAndValuesDo: [:aKey :aValue |
  aBlock value: (SymbolAssociation newWithKey: aKey value: aValue)
  ].
%

category: 'Accessing'
method: GsMethodDictionary
at: aKey ifAbsent: aBlock

"Returns the value whose key is identical to aKey.  If no such key/value pair
 exists, returns the result of evaluating the zero-argument block aBlock."

| offset hashKey |

nil == aKey ifTrue:[ ^self _reportKeyNotFound: aKey with: aBlock ].

(hashKey := self _basicAt:
       ( offset := (self hashFunction: aKey) * 2 - 1 )
  ) == aKey
ifTrue:[
  ^ self _basicAt: offset + 1 . "the value"
  ]
ifFalse:[
  nil == hashKey ifTrue:[
    "search collision chain"
    offset := self _basicAt: (offset + 1). "get one based offset to start of chain"
    [ offset == nil ] whileFalse:[
      (self _basicAt: offset )  == aKey ifTrue:[
        ^ self _basicAt: offset + 1
        ].
      offset := self _basicAt: offset + 2 .
      ] .
    ].
  ].
^ self _reportKeyNotFound: aKey with: aBlock .
%

category: 'Accessing'
method: GsMethodDictionary
at: aKey otherwise: aValue

"Returns the value whose key is identical to aKey.  If no such key/value pair
 exists, returns the given alternate value."

<primitive: 857>
keyConstraint ifNotNil:[ aKey _validateClass: keyConstraint ].
valueConstraint ifNotNil:[ aValue _validateClass: valueConstraint ].
self _primitiveFailed: #at:otherwise: args: { aKey . aValue }
%

category: 'Updating'
method: GsMethodDictionary
at: aKey put: aValue

"Stores the aKey/aValue pair in the hash dictionary.  Rebuilds the hash table
 if the addition caused the number of collisions to exceed the limit allowed.

 If aKey is not compatible with the key constraint of the receiver, or aValue is
 not compatible with the value constraint of the receiver, an error is
 generated."

self _validatePrivilege ifTrue:[
  self _atKey: aKey put: aValue .
].
^ aValue
%

category: 'Private'
method: GsMethodDictionary
atHash: hashIndex putKey: aValue

"Disallowed."

self shouldNotImplement: #atHash:putKey:
%

category: 'Private'
method: GsMethodDictionary
atHash: hashIndex putKey: aKey value: aValue

"Disallowed."

self shouldNotImplement: #atHash:putKey:value:
%

category: 'Private'
method: GsMethodDictionary
atHash: hashIndex putValue: aValue

"Disallowed."

self shouldNotImplement: #atHash:putValue:
%

category: 'Clustering'
method: GsMethodDictionary
clusterDepthFirst

"This method clusters the receiver and its values in depth-first order.  The
 keys are not clustered because they are Symbols.

 Has no effect and returns true if the receiver was previously clustered in the
 current transaction."

self cluster
  ifTrue:[ ^ true ]
  ifFalse: [
      "none of the named instance variables should be clustered"

      self valuesDo:[:aMethod | aMethod clusterDepthFirst ].
      ^ false
    ]
%

category: 'Private'
method: GsMethodDictionary
collisionBucketClass

"Returns the class of object to create when keys collide.
 GsMethodDictionary does not use collision buckets."

^ nil
%

category: 'Hashing'
method: GsMethodDictionary
hashFunction: aKey

"The hash function performs an operation on the value of the key aKey and
 returns some Integer between 1 and tableSize, inclusive."

^(aKey basicIdentityHash \\ self tableSize) + 1
%

category: 'Initializing'
method: GsMethodDictionary
initialize: newSize

"Initializes the instance variables of the receiver to be an empty
 IdentityKeyValueDictionary of the specified size."

self _validatePrivilege ifTrue:[
  super initialize: newSize .
]
%

category: 'CodeModification Override'
method: GsMethodDictionary
instVarAt: anIndex put: aValue

self _validatePrivilege ifTrue:[
  ^ super instVarAt: anIndex put: aValue
]
%

category: 'Accessing'
method: GsMethodDictionary
keyAtValue: anObject ifAbsent: aBlock

"Returns the key of the first value identical to anObject.  If no
 match is found, this method evaluates the block aBlock and returns its
 result."

self keysAndValuesDo:[ :aKey :aValue |
  anObject == aValue ifTrue:[ ^ aKey ]
  ].
^ aBlock value
%

category: 'Accessing'
method: GsMethodDictionary
keyConstraint

"Returns the key constraint of the receiver."

^ keyConstraint
%

category: 'Updating'
method: GsMethodDictionary
keyConstraint: aClass

"Sets the key constraint of the receiver to aClass.  Generates an error if
 the receiver is not empty."

self _validatePrivilege ifTrue:[
  numElements ~~ 0 ifTrue:[ ^ self _error: #rtErrCannotChgConstraint ].
  keyConstraint := aClass
]
%

category: 'Enumerating'
method: GsMethodDictionary
keysAndValuesDo: aBlock

"Evaluates aBlock with each of the receiver's key/value pairs as the
 arguments.  The argument aBlock must be a two-argument block.  The
 first argument is the key and the second argument is the value of
 each key/value pair.  Returns the receiver."

| aKey tSize |

"process hash slots"
tSize := tableSize * 2 .
1 to: tSize by: 2 do: [ :offset |
  (aKey := self _basicAt: offset) ifNotNil:[
    aBlock value: aKey value: (self _basicAt: offset + 1) .
    ].
  ].
"process collision chains"
tSize + 1 to: self _basicSize by: 3 do:[:offset |
  (aKey := self _basicAt: offset) ifNotNil:[
    aBlock value: aKey value: (self _basicAt: offset + 1) .
    ].
  ].
%

category: 'Enumerating'
method: GsMethodDictionary
keysDo: aBlock

"Evaluates aBlock with each of the receiver's keys as the
 argument.  The argument aBlock must be a one-argument block.
 Returns the receiver."

| aKey tSize |

"process hash slots"
tSize := tableSize * 2 .
1 to: tSize by: 2 do: [ :offset |
  (aKey := self _basicAt: offset) ifNotNil:[
    aBlock value: aKey .
    ].
  ].
"process collision chains"
tSize + 1 to: self _basicSize by: 3 do:[:offset |
  (aKey := self _basicAt: offset) ifNotNil:[
    aBlock value: aKey
    ].
  ].
%

category: 'Copying'
method: GsMethodDictionary
postCopy

"Because there are no collision buckets to copy, we can ignore inherited behavior."

^self
%

category: 'Formatting'
method: GsMethodDictionary
printOn: aStream

"Puts a displayable representation of the receiver on the given stream."

"copy the implementation from Object."

aStream nextPutAll: self asString
%

category: 'Hashing'
method: GsMethodDictionary
rebuildTable: newSize

"Rebuilds the method dictionary by populating a larger method dictionary
 first and doing a (primitive) become:"

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  | newGsMethodDict |

  tableSize = newSize ifTrue:[
    ^ self "no change in table size"
  ].
  collisionLimit == 536870911 ifTrue:[
    ^ self              "avoid recursive rebuild"
  ].

  newGsMethodDict := self class new: (newSize * 2).
  newGsMethodDict valueConstraint: valueConstraint.
  newGsMethodDict keyConstraint: keyConstraint.

  self keysAndValuesDo: [ :aKey :aValue |
    newGsMethodDict at: aKey put: aValue.
    ].
  "receiver is not expected to have a dependencyList, so an unprotected become: should be safe"
  newGsMethodDict _becomeDictionary: self.
] ensure:[
  prot _leaveProtectedMode
]
%

category: 'Removing'
method: GsMethodDictionary
removeAll

"Remove all key/value pairs from the receiver."

| tSize |
self _validatePrivilege ifTrue:[
  self _basicSize: 0 . "dereference all keys and values"
  tableSize :=  (tSize := 29) .
  self _basicSize: tSize + tSize  .  "reinitialize hash table to all nils"
  numElements := 0 .
  numCollisions := 0
]
%

category: 'Removing'
method: GsMethodDictionary
removeKey: aKey ifAbsent: aBlock

"Removes the key/value pair whose key is identical to aKey.  If no such
 key/value pair exists, returns the result of evaluating the zero-argument
 block aBlock."

 | oldVal |

 self _validatePrivilege ifTrue:[
   nil == aKey ifTrue:[ ^self _error: #rtErrNilKey ].
   oldVal := self _atKey: aKey put: nil .
   oldVal == nil ifTrue:[
     ^ self _reportKeyNotFound: aKey with: aBlock
   ].
   ^ oldVal
 ]
%

category: 'Removing'
method: GsMethodDictionary
removeKey: aKey otherwise: notFoundValue

"Removes the key/value pair whose key is identical to aKey.  If no such
 key/value pair exists, returns notFoundValue . "

 | oldVal |

 self _validatePrivilege ifTrue:[
   nil == aKey ifTrue:[ ^self _error: #rtErrNilKey ].
   oldVal := self _atKey: aKey put: nil .
   oldVal == nil ifTrue:[
     ^ notFoundValue
   ].
   ^ oldVal
 ]
%

category: 'CodeModification Override'
method: GsMethodDictionary
squeakBasicAt: anIndex put: aValue

  ^ self _basicAt: anIndex put: aValue
%

category: 'Statistics'
method: GsMethodDictionary
statistics

"A GsMethodDictionary has no collision buckets, so the statistics defined
 for KeyValueDictionary have no meaning."

^ nil
%

category: 'Accessing'
method: GsMethodDictionary
valueConstraint

"Returns the value constraint of the receiver."

^ valueConstraint
%

category: 'Updating'
method: GsMethodDictionary
valueConstraint: aClass

"Sets the value constraint of the receiver to aClass.  Generates an error if
 the receiver is not empty."

self _validatePrivilege ifTrue:[
  numElements ~~ 0 ifTrue:[ ^ self _error: #rtErrCannotChgConstraint ].
  valueConstraint := aClass
]
%

category: 'Private'
method: GsMethodDictionary
_atKey: aKey put: aValue

"aValue == nil means remove KV pair , and result is
 the value removed, or nil if not found.

 aValue ~~ nil means add KV pair, and return aValue.
 aValue must not be nil."

<primitive: 721>
self _primitiveFailed: #_atKey:put: args: { aKey . aValue }
%

category: 'CodeModification Override'
method: GsMethodDictionary
_basicAt: anIndex put: aValue

self _validatePrivilege ifTrue:[
  ^ super _basicAt: anIndex put: aValue
]
%

category: 'CodeModification Override'
method: GsMethodDictionary
_basicSize: anInteger

self _validatePrivilege ifTrue:[
  ^ super _basicSize: anInteger
]
%

category: 'Private'
method: GsMethodDictionary
_collisionBucketsDo: aBlock
  "a GsMethodDictionary has no collision buckets"
  ^ self
%

category: 'Private'
method: GsMethodDictionary
_initializeWithoutClear: newSize

"Private. Initializes the instance variables of the receiver to be an empty
 KeyValueDictionary of the specified size. Does not clear the contents
 of the receiver - assumes they are all nil."

tableSize := newSize.
numElements := 0.
numCollisions := 0.
collisionLimit := newSize // 4 .
^self
%

category: 'Testing'
method: GsMethodDictionary
_isLarge

"Returns true if the object is implemented as a tree of private smaller objects.
 Note that GsMethodDictionary does not use collision buckets.
"
^ ((self _status: false) bitShift: -5) ~~ 0
%

category: 'Private'
method: GsMethodDictionary
_nodesObjectSecurityPolicy: anObjectSecurityPolicy
  "a GsMethodDictionary has no collision buckets"
  ^ self
%

category: 'CodeModification Override'
method: GsMethodDictionary
_primitiveAt: anIndex put: aValue

self _validatePrivilege ifTrue:[
  ^ super _primitiveAt: anIndex put: aValue
]
%

category: 'Private'
method: GsMethodDictionary
_resetParentRef

"Private. After a become:, the parent refs of the collisionBuckets must
 be reset to point to the correct parent."

"GsMethodDictionarys don't use collision buckets"

^self

%

category: 'Canonical Symbol Support'
method: GsMethodDictionary
_selectiveAbort

"Performs an abort operation on the receiver."

"Receiver is just an ordinary small or large object as far as selective abort is
 concerned, so execute the implementation in Object.  Must avoid the
 reimplementation in IdentityKeyValueDictionary, so can't use super."

^ self _primitiveSelectiveAbort
%

category: 'CodeModification Override'
method: GsMethodDictionary
_unsafeAt: anIndex put: aValue

self _validatePrivilege ifTrue:[
  ^ super _unsafeAt: anIndex put: aValue
]
%

category: 'CodeModification Override'
method: GsMethodDictionary
_validatePrivilege

( self isMemberOf: GsMethodDictionary ) ifTrue: [
  ^   System myUserProfile _validateCodeModificationPrivilege
].
^ true
%

! Class extensions for 'GsNativeCode'

!		Class methods for 'GsNativeCode'

removeallmethods GsNativeCode
removeallclassmethods GsNativeCode

category: 'Disallowed'
classmethod: GsNativeCode
new

"Disallowed.  You cannot create new instances of GsNativeCode."

self shouldNotImplement: #new
%

category: 'Disallowed'
classmethod: GsNativeCode
new: aSize

"Disallowed.  You cannot create new instances of GsNativeCode."

self shouldNotImplement: #new:
%

!		Instance methods for 'GsNativeCode'

category: 'Disallowed'
method: GsNativeCode
at: anOffset put: aVal

"Disallowed."
self shouldNotImplement:#at:put:
%

category: 'Disallowed'
method: GsNativeCode
basicAt: anOffset put: aValue

"Disallowed."
self shouldNotImplement:#basicAt:put:
%

category: 'Disallowed'
method: GsNativeCode
copy
"Disallowed."
self shouldNotImplement:#copy
%

category: 'Disallowed'
method: GsNativeCode
objectSecurityPolicy: anObjectSecurityPolicy

"Disallowed."
self shouldNotImplement:#objectSecurityPolicy:
%

category: 'Disallowed'
method: GsNativeCode
passivate

"Disallowed."
self shouldNotImplement:#passivate
%

category: 'Disallowed'
method: GsNativeCode
size: aSize

"Disallowed."
self shouldNotImplement:#size:
%

category: 'Disallowed'
method: GsNativeCode
squeakBasicAt: anOffset put: aValue

"Disallowed."
self shouldNotImplement:#squeakBasicAt:put:
%

category: 'Disallowed'
method: GsNativeCode
_at: anOffset put: aVal

"Disallowed."
self shouldNotImplement:#_at:put:
%

category: 'Disallowed'
method: GsNativeCode
_basicAt: anOffset put: aVal

"Disallowed."
self shouldNotImplement:#_basicAt:put:
%

category: 'Disallowed'
method: GsNativeCode
_basicSize: aSize

"Disallowed."
self shouldNotImplement:#_basicSize:
%

category: 'Private'
method: GsNativeCode
_myNmethod

<primitive: 677>
^ self _primitiveFailed: #_myNmethod
%

category: 'Disallowed'
method: GsNativeCode
_objectSecurityPolicy: anObjectSecurityPolicy

"Disallowed."
self shouldNotImplement:#objectSecurityPolicy:
%

category: 'Disallowed'
method: GsNativeCode
_primitiveAt: anOffset put: aVal

"Disallowed."
self shouldNotImplement:#_primitiveAt:put:
%

! Class extensions for 'GsNMethod'

!		Class methods for 'GsNMethod'

removeallmethods GsNMethod
removeallclassmethods GsNMethod

category: 'Debugging Support'
classmethod: GsNMethod
clearAllBreaks

"Clear all method breakpoints that have been set in any methods."

self _setBreakAtIp: -1 operation: 2 frame: nil process: nil
%

category: 'Debugging Support'
classmethod: GsNMethod
clearBreakInClass: aClass selector: aSelector stepPoint: aStepPoint

"Clear the breakpoint at aStepPoint in method aSelector of class aClass."

(aClass compiledMethodAt: aSelector) clearBreakAtStepPoint: aStepPoint
%

category: 'Converting ArrayBuilders'
classmethod: GsNMethod
convertArrayBuildersIn: directoryName list: listName
  "given a directoryName and a file named 'listName' within that
   directory, assuming listName has one fileName per line,
   convert each file in the list.  within the list,
   leading/trailing whitespace on each line is ignored,
   and lines starting with # are ignored."

| dPath listF nC nNotC fName line fPath |
dPath := directoryName copy .
dPath last == $/ ifFalse:[ dPath add: $/ ].
listF := GsFile openRead:  dPath , listName  .
listF ifNil:[ ArgumentError signal:'file open failed for: ', (dPath , listName)].
nC := 0 . nNotC := 0 .
[ true ] whileTrue:[
  (line := listF nextLine) ifNil:[
    listF close .
    ^ nC asString, ' changed ' , nNotC asString , ' not changed '
  ].
  fName := line trimWhiteSpace .
  (fName at: 1) == $# ifFalse:[
    fPath := dPath , fName .
    [
      (self convertArrayBuildersInTopazScript: fPath)
         ifTrue:[ nC := nC + 1 . GsFile gciLogClient:'  changed ', fName ]
        ifFalse:[ nNotC := nNotC + 1 ].
    ] onException: Error do:[:ex |
      GsFile gciLogClient:'Error during ' , fName , ' ; ', ex description .
      nil pause .
    ]
  ].
].
%

category: 'Converting ArrayBuilders'
classmethod: GsNMethod
convertArrayBuildersInString: aString
  "Convert any ArrayBuilder productions in aString to
   CurlyArrayBuilder per $GEMSTONE/doc/bnf.txt .
   Returns aString if no ArrayBuilder found,
   otherwise returns a modified copy of aString."

  ^ self _convertArrayBuilders: aString stream: (ReadStreamPortable on: aString)
%

category: 'Converting ArrayBuilders'
classmethod: GsNMethod
convertArrayBuildersInTopazScript: aFileName
  "Convert any ArrayBuilder productions in source String within
   specified file to CurlyArrayBuilder per $GEMSTONE/doc/bnf.txt.
   The file is overwritten in place. Return true if file changed,
   false otherwise."
  | f origContents result cmd lineNum line src startLnum |
  f := GsFile openRead: aFileName .
  f ifNil:[ ArgumentError signal:'file open failed for: ', aFileName ].
  origContents := f contents .
  f close .
  f := GsFile openRead: aFileName .
  f ifNil:[ ArgumentError signal:'file open failed for: ', aFileName ].
  result :=  String new .
  lineNum := 0 .
  [ true ] whileTrue:[
    (line := f nextLine) ifNil:[
      f close .
      result = origContents ifFalse:[
        f := GsFile openWrite: aFileName .
        f ifNil:[ ArgumentError signal:'open for write failed for: ', aFileName ].
        (f nextPutAll: result) = result size ifFalse:[
          Error signal:'write failed to ' , aFileName
        ].
        f close .
        ^ true
      ].
      ^ false
    ].
    lineNum := lineNum + 1 .
    result addAll: line .
    (cmd := line trimWhiteSpace) size > 0 ifTrue:[
      "per abbreviations allowed in topaz command decoding in src/tpaux.c"
      (      (cmd at: 1 equalsNoCase:'m')
        or:[ (cmd at: 1 equalsNoCase:'cl')
        or:[ (cmd at: 1 equalsNoCase:'doi')
        or:[ (cmd at: 1 equalsNoCase:'interp')
        or:[ (cmd at: 1 equalsNoCase:'pri')
        or:[ (cmd at: 1 equalsNoCase:'run')
        or:[ (cmd at: 1 equalsNoCase:'rubyc')
        or:[ (cmd at: 1 equalsNoCase:'rubyr')
        or:[ (cmd at: 1 equalsNoCase:'rubym')
        or:[ (cmd at: 1 equalsNoCase:'ru') and:[ (cmd at: 1 equalsNoCase:'rub') not ]
           ]]]]]]]]] ) ifTrue:[
	 src := String new .   startLnum := lineNum .
	 [ line := f nextLine .
           lineNum := lineNum + 1 .
	   line == nil or:[ (line at: 1) == $% ]
	 ] whileFalse:[
	   src addAll: line .
	 ].
         [
	   result addAll:( self convertArrayBuildersInString: src ).
         ] onException: Error do:[:ex |
           GsFile gciLogClient:'Error for source starting at line ', startLnum asString .
           ex pass .
         ].
	 line ifNotNil:[ result addAll: line  "the % line "].
      ].
    ].
  ].
%

category: 'Debugging Support'
classmethod: GsNMethod
disableBreakInClass: aClass selector: aSelector stepPoint: aStepPoint

"Disable the breakpoint previously set at aStepPoint in method aSelector of
 class aClass."

(aClass compiledMethodAt: aSelector) disableBreakAtStepPoint: aStepPoint
%

category: 'Dynamic Literal Variables'
classmethod: GsNMethod
dynamicLiteralVariables: aBoolean

"Returns a Boolean, the previous state of the
  compiler control variable.

 aBoolean enables generation of dynamic literal variables
 in all subsequent compilations of smalltalk methods.
 Ruby methods (environment 1) are not affected.

 If dynamic literal variables are enabled,
 a literal variable whose association
 is not an instance of SymbolAssociation and which is
 a kind of SymbolAssociation, and which is not invariant,
 will have code emitted to send #_value to the association
 on fetchs, and #'_value:' on  on stores.

 _value and _value: have default implementations in Association.
"

^ self _oneArgPrim: 2 with: (aBoolean ifTrue:[1] ifFalse:[0])
%

category: 'Debugging Support'
classmethod: GsNMethod
enableBreakInClass: aClass selector: aSelector stepPoint: aStepPoint

"Set or reenable the breakpoint previously set at aStepPoint in method
 aSelector of class aClass."

(aClass compiledMethodAt: aSelector) setBreakAtStepPoint: aStepPoint
%

category: 'Compiler Access'
classmethod: GsNMethod
generateFromIR: aGsComMethNode

 "Invokes the code generator to generate an instance of GsNMethod from the
  specified IR graph.
  May be used for anonymous methods or for methods to be installed in a class.
  Caller of this method is responsible for installing in the
  method dictionary of an appropriate class.

  Returns a GsNMethod if the compilation
  succeeded with no warnings or errors, or an Array of of the form
    { (GsNMethod , or nil if the compilation has errors) .
       (nil or an Array of error descriptors as described for
        compileMethod:dictionaries:category: ) .
       (nil or a String describing warnings)
     } .

  Each element of the 'Array of error descriptors'
  is an Array of size 3 or 4, containing the elements:
   1. The GemStone error number.
   2. Offset into the source string where the error occurred.
   3. Error message text, if available, or nil.
   4. Internal compiler error text, if the error is internal.
 "
TraceIR ~~ 0 ifTrue:[
  TraceIR == 1 ifTrue:[
    aGsComMethNode fileName ifNotNil:[
      GsFile gciLogServer: '--IR: ' , aGsComMethNode summary
    ].
  ].
  TraceIR > 1 ifTrue:[ | strm |
    strm := IndentingStream newPrinting .
    aGsComMethNode printFormattedOn: strm .
    GsFile gciLogServer: strm contents .
    "  following code to be enabled after filein for debugging only,"
    "  SessionTemps not defined at this point in slowfilein  "
    " false ifTrue:[ |  hist |
        hist := SessionTemps current at:#AllIrs otherwise: nil .
        hist == nil ifTrue:[
           SessionTemps current at:#AllIrs put:( hist := { } ).
        ].
        hist addLast: aGsComMethNode .
      ].
    "
  ].
].
^ self _generateFromIR: aGsComMethNode
%

category: 'Constants'
classmethod: GsNMethod
maxArgs
  "Returns the maximum number of arguments to a method."

  ^ GEN_MAX_ARGS
%

category: 'Method Lookup Cache Statistics'
classmethod: GsNMethod
methodLookupCachesReport: minSize

| raw arr rpt lf lines line overflows |
(minSize _isSmallInteger not or:[ minSize < 0 ]) ifTrue:[
  ArgumentError signal:'minSize must be a SmallInteger >= 0'.
].
raw := self _mluCachesReport: minSize .
lf := Character lf .
(rpt := '--- selected session stats' copy) add: lf .

arr := raw at: 1 . "selected method lookup stats ( name, value pairs)"
lines := { } .
1 to: arr size by: 2 do:[:j | | val |
  val := arr at: j + 1 .
  (line := '  ' copy) add: (arr at: j); add: '  ' ; add: val asString .
  lines add: line .
].
(SortedCollection withAll: lines)  do:[:s | rpt add: s ; add: lf . ].

rpt add: '--- Class , descending size of per-class method lookup cache '; add: lf .
arr := raw at: 3 . "size of classes' method lookup cache"
lines := { } .
1 to: arr size by: 2 do:[:m | | cls sz |
  sz := arr at: (m + 1 ).
  sz >= minSize ifTrue:[
    line := String new .
    (cls := arr at: m ) isMeta ifTrue:[ line add:'meta' ].
    "ifFalse:[ cls isRubySingletonClass ifTrue:[ line add:'singleton ' ]]."
    line add: cls thisClass name ; add: '  ' ; add: sz asString .
    line := Association newWithKey: line value: sz .
    lines add: line .
  ].
].
(lines sortDescending:'value') do:[:assoc | rpt add: '  '; add: assoc key; add: lf ].

arr := raw at: 2 . "send-site caches details"
lines := { } .
overflows := { } .
arr := raw at: 2 .
1 to: arr size by: 5 do:[:k | | cacheSize |
  cacheSize := arr at: k + 4 .
  (cacheSize < 0 or:[ cacheSize >= minSize]) ifTrue:[
    | cls selEnvId selector ipOfs meth isBlk |
    meth := arr at: k .
    ipOfs := arr at: k + 1 .
    selector := arr at: k + 2 .
    selEnvId := arr at: k + 3 .
    meth isMethodForBlock ifTrue:[  meth := meth homeMethod . isBlk := true ].
    line := String withAll: (cls := meth inClass) thisClass name .
    line add:( meth environmentId > 0
               ifTrue:[ cls isMeta ifTrue:[ $. ] ifFalse:[ $# ] ]
               ifFalse:[ cls isMeta ifTrue:[ '(C) >> '] ifFalse:[ ' >> ']]) .
    line add: meth selector ; add: '  ' .
    isBlk ifNotNil:[ line add: ' (in block) ' ].
    line add:'  IP:'; add: ipOfs asString ; add: '  ';
        add:' '; add: selector ; add: ' , ' .
    cacheSize < 0 ifTrue:[
      line add: 'polymorphic overflow'. overflows add: line
    ] ifFalse:[
      line add: 'send-site size: '; add: cacheSize asString . lines add: line
    ]
  ].
].
rpt add: '--- Overflowed send sites '; add: lf .
(overflows sortAscending:'') do:[:s | rpt add: '  '; add: s ; add: lf ].
rpt add: '--- Send sites with size >= ' ; add: minSize asString ; add: lf .
(lines sortAscending:'') do:[:s | rpt add: '  '; add: s ; add: lf  ].

^ rpt
%

category: 'Instance Creation'
classmethod: GsNMethod
new

"Disallowed.  You cannot create new instances of GsNMethod."

self shouldNotImplement: #new
%

category: 'Instance Creation'
classmethod: GsNMethod
new: anInteger

"Disallowed.  You cannot create new instances of GsNMethod."

self shouldNotImplement: #new:
%

category: 'Class organizer support'
classmethod: GsNMethod
optimizedSelectors
^ #( #~~
    #==
    #_and:
    #_downTo:by:do:
    #_downTo:do:
    #_isArray
    #_isExceptionClass
    #_isExecBlock
    #_isFloat
    #_isInteger
    #_isNumber
    #_isOneByteString
    #_isRange
    #_isRegexp
    #_isRubyHash
    #_isScaledDecimal
    #_isSmallInteger
    #_isSymbol
    #_leaveProtectedMode
    #_or:
    #_stringCharSize
    #and:
    #ifFalse:
    #ifFalse:ifTrue:
    #ifNil:
    #ifNil:ifNotNil:
    #ifNotNil:
    #ifNotNil:ifNil:
    #ifTrue:
    #ifTrue:ifFalse:
    #isKindOf:
    #or:
    #repeat
    #timesRepeat:
    #to:by:do:
    #to:do:
    #untilFalse
    #untilFalse:
    #untilTrue
    #untilTrue:
    #whileFalse
    #whileFalse:
    #whileTrue
    #whileTrue: )
%

category: 'Compiler Access'
classmethod: GsNMethod
traceIR: anInt

 "set value of class variable controlling printing of IR inputs to
  generateFromIR: .  printing is with GsFile gciLogServer:  .

  anInt == 0 means  no tracing ,
        == 1 means  log source line number of each method node
        == 2 means  full print of IR graph for each method
"

  TraceIR := anInt .
%

category: 'Debugging Support'
classmethod: GsNMethod
_addMarkerIds: anArray

""

| placesMarked markerLine markerLineSize space addToEnd markPosition
  aStr neededSize subStr |

placesMarked:= anArray at: 1.
markerLine:= anArray at: 2.
space:= Character space.

"have the source marked at each error with ^; now add marker identifier"
addToEnd:= false.
1 to: (placesMarked size) do: [:i |
   markPosition:= (placesMarked at: i) at: 1.
   aStr:= ((placesMarked at: i) at: 2) asString.
   neededSize:= markPosition + aStr size.
   markerLineSize := markerLine size .
   (markerLineSize < neededSize) ifTrue: [
       markerLine size: neededSize.
       markerLineSize + 1 to: neededSize do: [:k |
          markerLine at: k put: space].
       markerLineSize:= neededSize.
   ].

   (addToEnd) ifFalse: [
      subStr:= markerLine copyFrom: markPosition + 1
                                to: (markPosition + aStr size).
      subStr do: [:each | (each == $ ) ifFalse: [addToEnd := true]].
   ].
   (addToEnd) ifTrue: [
       markerLine add: aStr.
       (i == placesMarked size) ifFalse: [ markerLine add: ',']
   ] ifFalse: [  | destIdx |
      destIdx := markPosition + 1 .
      markerLine replaceFrom: destIdx to: destIdx + aStr size - 1 with: aStr startingAt: 1 .
   ]
].
(68 - markerLine size) timesRepeat:[ markerLine add: $ ].
(75 - markerLine size) timesRepeat:[ markerLine add: $* ] .
markerLine add: Character lf.
^ true
%

category: 'Private'
classmethod: GsNMethod
_allBreakpointsPerform: aSelector

"For all breakpoint in the result of GsNMethod (C) | _breakReport:,
 perform  aSelector .  "

| reportArray |
reportArray := (self _breakReport: true ) at: 2 .

1 to: reportArray size do:[:j| | aBrk aMethod stepPoint |
  aBrk := reportArray at: j .
  aMethod := aBrk at: 5 .
  stepPoint := aBrk at: 4 .
  aMethod perform: aSelector with: stepPoint .
].
%

category: 'Debugging Support'
classmethod: GsNMethod
_allMethodBreakpoints

"Returns an Array of Arrays of the form

  { { <breakpointNumber>. <aGsNMethod>. <ipOffset>.
        ...
        <breakpointNumber>. <aGsNMethod>. <ipOffset>}.
      ...
    { <breakpointNumber>. <aGsNMethod>. <ipOffset>
         ...
       {
   }

 The interior Arrays are as described for GsNMethod | _allBreakpoints.

 Breakpoints installed by single-step are not reported.
 If no breakpoints are set, returns an Array of size zero."

^ self _oneArgPrim: 0 with: 0
%

category: 'Private'
classmethod: GsNMethod
_breakpoint: aNumber perform: aSelector

"For the breakpoint listed with number aNumber
 in the result of GsNMethod (C) | _breakReport:,
 perform  aSelector .
 Return true if the breakpoint found, false otherwise."

| reportArray |
reportArray := (self _breakReport: true ) at: 2 .

1 to: reportArray size do:[:j| | aBrk |
  aBrk := reportArray at: j .
  (aBrk at:1) = aNumber ifTrue:[  | aMethod stepPoint |
    aMethod := aBrk at: 5 .
    stepPoint := aBrk at: 4 .
    aMethod perform: aSelector with: stepPoint .
    ^ true
  ].
].
^ false
%

category: 'Debugging Support'
classmethod: GsNMethod
_breakReport: withLineNumbersBool

"Returns an Array describing all method breakpoints currently set.
 The Array contains a string report of all breakpoints and an Array
 of 6 element Arrays .

 The Array is { <break report string as displayed in topaz, one line
		  per breakpoint> .
               { { <breakNumber>. <class> . <selector>. <stepPoint>. <GsNMethod> . <disabledBoolean>}.
                     ...
                 {<breakNumber>. <class> . <selector>. <stepPoint>. <GsNMethod> . <disabledBoolean>}
               }
               }"

| allBreaksRaw sortedBreaks report descriptors |

allBreaksRaw := GsNMethod _allMethodBreakpoints .
allBreaksRaw size == 0 ifTrue:[ ^ { 'No breaks set' , Character lf . { } } ] .

sortedBreaks := SortedCollection new .
allBreaksRaw do:[: methodBreakArray |
  1 to: methodBreakArray size by: 3 do:[ :j |
    | brkNum aMeth ipOfs hmMeth assoc stepPt |
    brkNum := methodBreakArray at: j .
    aMeth :=  methodBreakArray at: j + 1 .
    ipOfs :=  methodBreakArray at: j + 2 .

    hmMeth  := aMeth homeMethod .
    stepPt := hmMeth _stepPointForMeth: aMeth ip: ipOfs .
    stepPt == nil ifTrue:[ self error:'could not translate IP to a step point'].
    assoc := Association newWithKey: brkNum
	value: { brkNum . hmMeth inClass . hmMeth selector . stepPt . hmMeth } .
    sortedBreaks add: assoc .
  ].
].

report := String new .
descriptors := { }  .
sortedBreaks do:[ :assoc |
  | aBreakArray breakNumber aMethod theClass selector className stepPoint disabled env |
  aBreakArray := assoc value .
  breakNumber := aBreakArray at: 1 .
  aMethod := aBreakArray at: 5 .
  theClass := aBreakArray at: 2 .
  className := theClass name .
  selector := aBreakArray at: 3 .
  stepPoint := aBreakArray at: 4 .
  withLineNumbersBool ifTrue:[
    report addAll: breakNumber asString; addAll:': ' .
  ] .
  report addAll: className; addAll: ' >> ' ;
	 addAll: selector;  addAll: ' @ ' ;
	 addAll: stepPoint asString .
  (env := aMethod environmentId) ~~ 0 ifTrue:[
    report addAll: ' env ' , env asString .
  ].
  (disabled := stepPoint < 0) ifTrue:[
    stepPoint := stepPoint negated .
    report addAll: ' (disabled)'
  ].
  report add: Character lf .
  aBreakArray at: 4 put: stepPoint .
  descriptors add: aBreakArray .
  aBreakArray at: 6 put: disabled .
].
^ { report . descriptors }
%

category: 'Debugging Support'
classmethod: GsNMethod
_buildMarkedSourceFrom: sourceStrArg sourceSize: aSize markers: markerArray

^ self _buildMarkedSourceFrom: sourceStrArg sourceSize: aSize markers: markerArray tabSize: 8
%

category: 'Debugging Support'
classmethod: GsNMethod
_buildMarkedSourceFrom: sourceStrArg sourceSize: aSize markers: markerArray tabSize: tabSize

"Given a source string, its size (passed in for efficiency), and a marker
 Array, returns an instance of sourceStr's class containing the marked source."

| lineFeed tab space
  placesMarked     "an Array of markers marked on the current line"
  markerLineIndex "index into the current marker line"
  result          "the result of this method"
  markerLine      "the current marker line"
  aChar           "one Character of the source"
  displayWidth    "the number of positions it takes to display aChar"
  lineSz
  sourceStr
|

 "initialize"
 lineFeed := Character lf.
 tab:= Character tab.
 space:= Character space.

 placesMarked:= { } .
 markerLineIndex:= 1.
 sourceStr := sourceStrArg .
 sourceStr == nil ifTrue:[ sourceStr := String withAll:' "source not available" '].
 result:= sourceStr class new .
 result addAll: '   ' .
 lineSz := 0 .
 markerLine:= String new .
 markerLine add: $  .
 1 to: aSize do: [:i |
   aChar:= sourceStr at: i.  "fetch a char"
   displayWidth := 1 .
   "Add the char to the result"
   (aChar == tab) ifTrue: [
      displayWidth:= tabSize - (lineSz \\ tabSize).
      displayWidth timesRepeat: [result add: space].
      lineSz := lineSz + displayWidth .
   ] ifFalse: [
      result add: aChar.
      lineSz := lineSz + displayWidth .
      ((i == aSize) and: [aChar ~~ lineFeed]) ifTrue: [
        result add: lineFeed .
      ].
   ].

   ((markerArray at: i) == nil) ifTrue: [ "no marker at this position"
      displayWidth timesRepeat:[ markerLine add: space].
   ] ifFalse: [ "found an error at this position"
      placesMarked add: { markerLineIndex + 1 . markerArray at: i }.
      markerLine add: $^ .
      displayWidth - 1 timesRepeat: [markerLine add: space].
   ].
   markerLineIndex:= markerLineIndex + displayWidth.

   ((aChar == lineFeed) or: [i == aSize]) ifTrue: [ "we are at end of line"
      "add error identifiers to marker line "
      (placesMarked size ~~ 0) ifTrue: [
         self _addMarkerIds: { placesMarked . markerLine . markerLineIndex }.
         result add: $  ; add: $* .
         result add: markerLine.
       ] .
      (i == aSize) ifFalse: [
         result addAll: '   ' .
         lineSz := 0 .
      ].
      markerLine size: 1.
      markerLineIndex:= 1.
      placesMarked size: 0.
   ]
 ].
 ^result
%

category: 'Debugging Support'
classmethod: GsNMethod
_buildMarkersFrom: sourceOffsets ofSize: sizeArg

"Given an Array of source offsets, build an Array of size sizeArg containing
 the index into anArray at the position corresponding to anArray's element.
 The remainder of the Array contains nil.  Negative offsets denote disabled
 breakpoints."

| markerArray anOffset posOffset aSize |
aSize := 1 max: sizeArg .                          "fix bug 14976"
markerArray:= Array new: aSize.
1 to: sourceOffsets size do: [:i |
  anOffset := sourceOffsets at: i .
  anOffset == nil ifFalse:[
    posOffset:= (anOffset abs max: 1) min: aSize.  "limit within range"
    (markerArray at: posOffset) ifNotNil:[ "one retry for fix 45431"
      posOffset := posOffset + 1 min: aSize .
    ].
    (markerArray at: posOffset) ifNil:[
       anOffset < 0 ifTrue:[ markerArray at: posOffset put: i negated ]
		    ifFalse:[ markerArray at: posOffset put: i ]
       ]
    ]
  ].
^markerArray
%

category: 'Debugging Support'
classmethod: GsNMethod
_clearAllStepBreaks

"Legacy code, no longer used.
 Clear all single step breakpoints from all loaded methods."
self _setBreakAtIp: -1 operation: 3 frame: nil process: nil
%

category: 'Converting ArrayBuilders - Private'
classmethod: GsNMethod
_convertArrayBuilders: aString stream: stream
  | result ch |
  result := aString class new .
  [ true ] whileTrue:[
    ch := stream nextOrNil .
    ch ifNil:[ ^ aString = result ifTrue:[ aString ] ifFalse:[ result]].
    self _parseChar: ch stream: stream to: result
  ].
%

category: 'Converting ArrayBuilders - Private'
classmethod: GsNMethod
_convertArrayBuildersIfNeeded: aString
 "called from within VM when parsing a doit, or
  when compiling a method and session methods are not activated.
  Invoked from code in Behavior when session methods in use.
  VM has already found GemConvertArrayBuilder==true  "

  | stream |
^ [
    stream := ReadStreamPortable on: aString .
    self _convertArrayBuilders: aString stream: stream .
  ] onException: Error do:[:ex | | pos msg comErrInfo |
    pos := 1 .
    msg := 'unknown ArrayBuilder conversion error' .
    [ | txt |
      pos := stream ifNotNil:[ stream position ].
      (txt := ex messageText) ifNotNil:[ msg := 'ArrayBuilder conversion, ', txt ].
    ] onException: Error do:[:exx |  "ignore"].
    comErrInfo := { { 1071 . pos . msg } } .
    { nil . comErrInfo . nil }
  ].
%

category: 'Converting ArrayBuilders - Private'
classmethod: GsNMethod
_convertArrayBuildersIn: directoryName list: listName refDir: refDirPath
  "refDirPath is to a separate checkout previously converted ;
   its listName files control the operation.
   Compare results of conversion of files in directoryName
   with files in refDirPath , and raise an error if files differ.
"

| dPath listF nC nNotC line fName fPath |
dPath := directoryName copy .
dPath last == $/ ifFalse:[ dPath add: $/ ].
listF := GsFile openRead:  refDirPath , listName  .
listF ifNil:[ ArgumentError signal:'file open failed for: ', (dPath , listName)].
nC := 0 . nNotC := 0 .
[ true ] whileTrue:[
  (line := listF nextLine) ifNil:[
    listF close .
    ^ nC asString, ' changed ' , nNotC asString , ' not changed '
  ].
  fName := line trimWhiteSpace .
  (fName at: 1) == $# ifFalse:[
    fPath := dPath , fName .
    [
      (self convertArrayBuildersInTopazScript: fPath) ifTrue:[  | refFil newFil |
        nC := nC + 1 . GsFile gciLogClient:'  changed ', fName  .
        refFil := GsFile openRead: refDirPath , fName .
        newFil := GsFile openRead: fPath .
        refFil contents = newFil contents ifFalse:[
          GsFile gciLogClient:'difference in ', fPath
        ].
        refFil close .  newFil close .
      ] ifFalse:[
         nNotC := nNotC + 1
      ].
    ] onException: Error do:[:ex |
      GsFile gciLogClient:'Error during ' , fName , ' ; ', ex description .
      nil pause .
    ]
  ].
].
%

category: 'Debugging Support'
classmethod: GsNMethod
_deleteAllBreaks

"Delete all breakpoints listed in the result of GsNMethod (C) | _breakReport:."

self _allBreakpointsPerform: #clearBreakAtStepPoint:
%

category: 'Debugging Support'
classmethod: GsNMethod
_deleteBreakNumber: aNumber

"Delete the breakpoint listed with number aNumber in the result of
 GsNMethod (C) | _breakReport:.
 Return true if breakpoint deleted, false if not found.
 Used by Topaz"

^ self _breakpoint: aNumber perform: #clearBreakAtStepPoint:
%

category: 'Debugging Support'
classmethod: GsNMethod
_disableAllBreaks

"Disable all breakpoints listed in the result of GsNMethod (C) | _breakReport:."

self _allBreakpointsPerform: #disableBreakAtStepPoint:
%

category: 'Debugging Support'
classmethod: GsNMethod
_disableBreakNumber: aNumber

"disable the breakpoint listed with number aNumber in the result of
 GsNMethod (C) | _breakReport: .
 Return true if breakpoint disabled, false if not found.
 Used by Topaz"

^ self _breakpoint: aNumber perform: #disableBreakAtStepPoint:
%

category: 'Debugging Support'
classmethod: GsNMethod
_enableAllBreaks

"Enable all breakpoints listed in the result of GsNMethod (C) | _breakReport:."

self _allBreakpointsPerform: #setBreakAtStepPoint:
%

category: 'Debugging Support'
classmethod: GsNMethod
_enableBreakNumber: aNumber

"Enable the breakpoint listed with number aNumber in the result of
 GsNMethod (C) | _breakReport: .
 Return true if breakpoint enabled, false if not found.
 Used by Topaz"

^ self _breakpoint: aNumber perform: #setBreakAtStepPoint:
%

category: 'Method Lookup Cache Statistics'
classmethod: GsNMethod
_findImplementation: aSelector dir: directoryName list: listName
  "Return a report String describing where in topaz scripts
   aSelector is implemented. aSelector may be either a Symbol
   or a message pattern String like   'kw1: arg1 kw2: arg2' .
   Both directoryName and listName are Strings.
   directoryName is a path to the file listName which
   contains a list of files to search.  Lines in the file beginning
   with $# are comment lines."
| dPath listF rpt line fName fPath|
rpt := String new .
dPath := directoryName copy .
dPath last == $/ ifFalse:[ dPath add: $/ ].
listF := GsFile openRead:  dPath , listName  .
listF ifNil:[ ArgumentError signal:'file open failed for: ', (dPath , listName)].
[ true ] whileTrue:[
  (line := listF nextLine) ifNil:[
    listF close .
    ^ rpt .
  ].
  fName := line trimWhiteSpace .
  (fName at: 1) == $# ifFalse:[
    fPath := dPath , fName .
    [
      rpt addAll:(self _findImplementation: aSelector inTopazScript: fPath fileName: fName)
    ] onException: Error do:[:ex |
      GsFile gciLogServer: 'ERROR during ' , fName  .
      ex pass
    ].
  ].
].
%

category: 'Method Lookup Cache Statistics'
classmethod: GsNMethod
_findImplementation: aSelector inTopazScript: aPath fileName: fName
  "Return a report String describing where in topaz script aPath
   aSelector is implemented. aSelector may be either a Symbol
   or a message pattern String like   'kw1: arg1 kw2: arg2'  "
  | f keyWords numColons result lineNum selSym lf tab prevSrc |
  f := GsFile openRead: aPath .   lf := Character lf . tab := Character tab .
  f ifNil:[ ArgumentError signal:'file open failed for: ', fName ].
  aSelector _isSymbol ifFalse:[ | pat idx nxt lim sel |
    pat := String new .  "extract selector from message pattern"
    sel := aSelector trimWhiteSpace .
    lim := sel size .
    idx := 1 .
    [ (nxt := sel indexOf: $:  startingAt: idx)  ~~ 0 ] whileTrue:[
      pat addAll:(sel copyFrom: idx to: nxt) .
      idx := nxt + 1 .
      [ idx <= lim and:[ (sel at: idx) isSeparator] ] whileTrue:[
         idx := idx + 1   "skip whitespace after colon"
      ].
      [ idx <= lim and:[ (sel at: idx) isSeparator not ] ] whileTrue:[
         idx := idx + 1   "skip until end of argument name"
      ].
      [ idx <= lim and:[ (sel at: idx) isSeparator] ] whileTrue:[
         idx := idx + 1   "skip whitespace after argumentName"
      ].
    ].
    selSym := pat asSymbol .
  ] ifTrue:[
    selSym := aSelector
  ].
  (selSym includesValue: $: )
       ifTrue:[ keyWords := selSym keywords . numColons := keyWords size ]
      ifFalse:[ keyWords := { selSym } . numColons := 0 ].
  result :=  String new .
  lineNum := 0 .
  [ true ] whileTrue:[ | line methSrc cmd |
    (line := f nextLine) ifNil:[ f close .  ^ result ].
    lineNum := lineNum + 1 .
    cmd := line trimWhiteSpace .
    cmd size > 0 ifTrue:[
      "per abbreviations allowed in topaz command decoding in src/tpaux.c"
      ((     cmd at: 1 equalsNoCase:'m')
       or:[ (cmd at: 1 equalsNoCase:'cl')
       or:[ (cmd at: 1 equalsNoCase:'doi')
       or:[ (cmd at: 1 equalsNoCase:'interp')
       or:[ (cmd at: 1 equalsNoCase:'pri')
       or:[ (cmd at: 1 equalsNoCase:'run')
       or:[ (cmd at: 1 equalsNoCase:'rubyc')
       or:[ (cmd at: 1 equalsNoCase:'rubyr')
       or:[ (cmd at: 1 equalsNoCase:'rubym')
       or:[ (cmd at: 1 equalsNoCase:'ru') and:[ (cmd at: 1 equalsNoCase:'rub') not ]
           ]]]]]]]]] ) ifTrue:[  "have doit or method of some kind"

         ((cmd at: 1 equalsNoCase:'m')
         or:[ cmd at: 1 equalsNoCase:'cl']) ifTrue:[ "have method: or classmethod:" | keywd |
           [
             (line := f nextLine) ifNil:[ f close .  ^ result ].
	     lineNum := lineNum + 1 .
             (line at: 1) == $% ifFalse:[ line := line trimWhiteSpace ].
             line size == 0
           ] whileTrue .
           keywd := keyWords at: 1 .
	   (line at:1 equals: keywd) ifTrue:[  | found methStartLine |
             methSrc := line copy .   found := false .
	     methStartLine := lineNum .
             numColons == 0 ifTrue:[
               found :=  line = keywd asString .
             ] ifFalse:[ | ofs |
	       ofs := keywd size + 1 .
	       [ | pos endOfSel nextL |
		 pos := f position .
		 (nextL := f nextLine) ifNil:[ f close .  ^ result ].
		 methSrc addAll: nextL .
		 lineNum := lineNum + 1 .
		 (nextL at: 1) == $% ifTrue:[ f position: pos ].
		 (endOfSel := (nextL occurrencesOf: $: ) == 0) ifFalse:[
		   line addAll: nextL trimWhiteSpace .
		 ].
		 endOfSel
	       ] whileFalse .
	       (line occurrencesOf: $: ) == numColons ifTrue:[ | j kwlim |
		 found := true .
		 j := 2 . kwlim := numColons .
		 [ j <= kwlim ] whileTrue:[ | idx |
		   keywd := keyWords at: j .
		   (idx := line findString: keywd startingAt: ofs) == 0 ifTrue:[
		     kwlim := 0 . found := false
		   ] ifFalse:[  "line contains keywd"
		     j := j + 1 .  ofs := idx + keywd size .
		   ]
		 ].
               ].
             ].
	     found ifTrue:[
	       result size == 0 ifTrue:[ result add: lf ].
	       result add: fName , ':' , methStartLine asString , '  ', selSym , lf .
	     ] ifFalse:[
               methSrc := nil .
             ].
	   ].
         ].
         [(line at:1) == $% ] whileFalse:[
           (line := f nextLine) ifNil:[ f close . ^ result ].
           methSrc ifNotNil:[ methSrc addAll: line ].
           lineNum := lineNum + 1 .
         ].
         methSrc ifNotNil:[
           (prevSrc ~~ nil and:[ (methSrc = prevSrc) not]) ifTrue:[
	      result addAll:'     <different source>', lf
           ].
           prevSrc := methSrc .
           methSrc := nil .
         ].
      ].
    ].
  ]
%

category: 'Private'
classmethod: GsNMethod
_firstPortableIpoffset

  "returns the VM constant OC_GSNMETHOD_FIRST_INSTR_OFFSET"
  ^ self _oneArgPrim: 3 with: 0
%

category: 'Compiler Access'
classmethod: GsNMethod
_generateFromIR: aGsComMethNode

 "Invokes the code generator to generate an instance of GsNMethod from the
  specified IR graph.
  May be used for anonymous methods or for methods to be installed in a class.
  Caller of this method is responsible for installing in the
  method dictionary of an appropriate class.

  Returns a GsNMethod if the compilation
  succeeded with no warnings or errors, or an Array of of the form
    { (GsNMethod , or nil if the compilation has errors) .
       (nil or an Array of error descriptors as described for
        compileMethod:dictionaries:category: ) ,
       (nil or a String describing warnings)
    }  .

  Each element of the 'Array of error descriptors'
  is an Array of size 3 or 4, containing the elements:
   1. The GemStone error number.
   2. Offset into the source string where the error occurred.
   3. Error message text, if available, or nil.
   4. Internal compiler error text, if the error is internal.
 "

<primitive: 679>
self _primitiveFailed: #generateFromIR: args: { aGsComMethNode }
%

category: 'Private'
classmethod: GsNMethod
_gsReturnToC

"Return from Smalltalk execution to C code.
 The C code may be a primitive within the VM,
 a user action or FFI function which invoked Smalltalk ,
 or a GCI client process which made a GCI call to run Smalltalk.

 To be sent only from within the virtual machine. Do not do a send or perform
 of this method in any Smalltalk or GCI code. "

<primitive: 2013>
"If primitive succeeds control returns to the GCI, useraction/FFI, or the VM.
 If primitive fails, the scheduler needs to regain control
 for process termination."

self _terminateNonForkedProcess .
%

category: 'Debugging Support'
classmethod: GsNMethod
_hasBreakpoints
  "Returns a Boolean. "
  ^ self _oneArgPrim: 4 with: 0
%

category: 'Method Lookup Cache Statistics'
classmethod: GsNMethod
_mluCachesReport: minSize

"Returns an Array of the form
   { { String, value , ... }     (various stats)
     { Class , lookupCacheSize , ... )
     { GsNMethod, IP , selector, envId, send_site_cache_size , ... }
   }"

^ self _oneArgPrim: 1 with:  minSize
%

category: 'Private'
classmethod: GsNMethod
_noopReturnTos

"Used in implementation of partial continuation evaluation only."
<primitive: 2032>
"Execution returns to the calling frame"
%

category: 'Debugging Support'
classmethod: GsNMethod
_oneArgPrim: opcode with: aSmallInteger
"opcode == 0 , _allMethodBreakpoints
 opcode == 1 , send-site caches report

 opcode == 2 , set ComGenStateSType.dynamicSmalltalkLitVars,
               arg is 0 or 1,
               result is previous value, aBoolean
 opcode == 3 , _firstPortableIpoffset , arg is 0, result SmallInteger. 
 opcode == 4,  _hasBreakpoints, arg not used, result Boolean"

<primitive: 194>
self _primitiveFailed: #_oneArgPrim args: { opcode . aSmallInteger }

%

category: 'Converting ArrayBuilders - Private'
classmethod: GsNMethod
_parseChar: ch stream: stream to: result
    ch == $# ifTrue:[
      stream peek == $[  ifTrue:[
        "don't put $# in result"
        stream next .  "  throw away  $[ "
        self _parseOldArrayBuilder: stream to: result
      ] ifFalse:[
        result add: ch .
        self _parseSymbolLiteral: stream to: result
      ]
    ] ifFalse:[
      result add: ch .
      ch == $$ ifTrue:[ | nxt |  "character literal"
        (nxt := stream nextOrNil) ifNotNil:[ result add: nxt ]. ] ifFalse:[
      ch == $( ifTrue:[  "parenthesized expression"
        self _parseExpression: stream to: result end: $) ] ifFalse:[
      ch == $[ ifTrue:[  "block"
        self _parseExpression: stream to: result end: $] ] ifFalse:[
      ch == ${ ifTrue:[  "select block or CurlyArrayBuilder "
        self _parseExpression: stream to: result end: $} ] ifFalse:[
      ch == $'  ifTrue:[
        self _parseStringLiteral: stream to: result   ] ifFalse: [
      ch == $"  ifTrue:[
        self _parseComment: stream to: result
      ]]]]]]
    ].
%

category: 'Converting ArrayBuilders - Private'
classmethod: GsNMethod
_parseComment: stream to: result
  | ch |
  [ true ] whileTrue:[
    ch := stream nextOrNil .
    ch ifNil:[ ArgumentError signal:'premature end of stream in _parseComment' ].
    result add: ch .
    ch == $"  ifTrue:[ ^ self "end of comment" ]
  ]
%

category: 'Converting ArrayBuilders - Private'
classmethod: GsNMethod
_parseExpression: stream to: result end: endCh
  | ch |
  [ true ] whileTrue:[
    ch := stream nextOrNil .
    ch ifNil:[ ArgumentError signal:'premature end of stream in _parseExpression' ].
    ch == endCh ifTrue:[
      result add: ch .
      ^ self
    ].
    self _parseChar: ch stream: stream to: result
  ]
%

category: 'Converting ArrayBuilders - Private'
classmethod: GsNMethod
_parseOldArrayBuilder: stream to: result
  | ch |
  result add: ${  .
  stream peek == $  ifFalse:[ result add: $  "space after { for readability"].
  [ true ] whileTrue:[
    ch := stream nextOrNil .
    ch ifNil:[ ArgumentError signal:'premature end of stream in _parseOldArrayBuilder' ].
    ch == $,  ifTrue:[  "ensure space before/after .  per syntax"
       result last == $  ifFalse:[ result add: $  ].
       result add: $.  .
       stream peek == $  ifFalse:[ result add: $  ].
    ] ifFalse:[
      ch == $]  ifTrue:[
        result last == $  ifFalse:[ result add: $  "space before } for readability"].
        result add: $} .
        ^ self
      ] ifFalse:[
        self _parseChar: ch stream: stream to: result
      ]
    ]
  ]
%

category: 'Converting ArrayBuilders - Private'
classmethod: GsNMethod
_parseStringLiteral: stream to: result
  | ch |
  [ true ] whileTrue:[
    ch := stream nextOrNil .
    ch ifNil:[ ArgumentError signal:'premature end of stream in _parseStringLiteral' ].
    result add: ch .
    ch == $' ifTrue:[
      stream peek == $' ifFalse:[ ^ self  "end of literal"].
      stream next .  "handle  adjacent single quotes"
      result add: $' .
    ].
  ].
%

category: 'Converting ArrayBuilders - Private'
classmethod: GsNMethod
_parseSymbolLiteral: stream to: result
  "Caller has just consumed a # .
   Just parse enough to distinguish commas within a binary selector Symbol
    literal from commas that are separators or selectors on their own"
  | ch selectorChars |
  ch := stream peek .
  ch ifNil:[ ^ self ].
  selectorChars := '+-\*~<>=|/&@%,?!' .
  (selectorChars indexOf: ch startingAt:1) ~~ 0 ifTrue:[
    "we have a binary selector Symbol literal"
    ch := stream next .
    result add: ch .
    ch := stream peek .
    ch ifNil:[ ^ self ].
    (selectorChars indexOf: ch startingAt:1) ~~ 0 ifTrue:[
      "ch is part of the binary selector"
      ch := stream next .
      result add: ch .
    ].
  ].
%

category: 'Debugging Support'
classmethod: GsNMethod
_setBreakAtIp: ipOffset operation: opcode frame: fpOffset process: aGsProcess

"Apply the following action to all methods that contain breakpoints.

 opcode  action
   2     delete all method breakpoints and disabled method breakpoints
   3     delete single step breakpoints
   (note function of prim 190 varies depending on receiver)

 Other opcodes that are defined for the instance method version of this
 primitive are illegal when used as a class method.

 ipOffset must be == -1 "

<primitive: 190>
self _primitiveFailed: #_setBreakAtIp:operation:frame:process:
  args: { ipOffset . opcode . fpOffset . aGsProcess }
%

category: 'Debugging Support'
classmethod: GsNMethod
_sourceWithErrors: compilerError fromString: aString

 ^ self _sourceWithErrors: compilerError fromString: aString tabSize: 8
%

category: 'Debugging Support'
classmethod: GsNMethod
_sourceWithErrors: compilerError fromString: aString tabSize: tabSize

"This method returns an instance of aString's class containing the text in a
 string with compiler errors marked, plus the error text for each error.

 The argument compilerError is the result Array from either the
 Behavior | compileMethod:dictionaries:category:  method or the
 GsNMethod | _recompileWithSource: method.

 The argument aString is the source string which was an input to either the
 Behavior | compileMethod:dictionaries:category: method or the
 GsNMethod | _recompileWithSource: method."

| lineFeed result aStringSize offsets errNumbers thisErr pos
  markerArray errDict errMsgs auxMsgs errsz |

"initialize"
lineFeed := Character lf.
offsets := Array new: (errsz:= compilerError size) .
errNumbers := Array new: errsz  .
errMsgs := Array new: errsz  .
auxMsgs := Array new: errsz  .
aString == nil ifTrue:[
  result := String new  .
  result addAll:' "method source not available" '; add: lineFeed .
].

"get an Array of source offsets with errors, and an Array of error numbers"
1 to: errsz   do: [:i | | errNum |
   thisErr := compilerError at: i.
   offsets at: i put: (thisErr at: 2 "source offset").
   errNumbers at: i put: (errNum := thisErr at: 1 "error number") .
   result ~~ nil ifTrue:[
     result addAll:'error '; add: errNum asString; addAll: ', '
   ].
   thisErr size >= 3 ifTrue:[ | eMsg |
     errMsgs at: i put: (eMsg := thisErr at: 3"error message String") .
     (result ~~ nil and:[ eMsg ~~ nil]) ifTrue:[
        result addAll: eMsg asString ; add: $,
     ].
     thisErr size >= 4 ifTrue:[
       auxMsgs at: i put: (eMsg := thisErr at: 4 "additional message text").
       (result ~~ nil and:[ eMsg ~~ nil]) ifTrue:[
          result addAll: eMsg asString ; add: $,
       ].
     ].
   ].
   result ~~ nil ifTrue:[ result add: lineFeed ].
].
result ~~ nil ifTrue:[ ^ result ].

aStringSize:= aString size.

"build an Array parallel to the source that contains nil if no error at
 that source position, and an index into offsets if there is an error at
 that source position"
markerArray:= self _buildMarkersFrom: offsets ofSize: aStringSize.

result:= self _buildMarkedSourceFrom: aString
                          sourceSize: aStringSize
                             markers: markerArray tabSize: tabSize .

"add error strings"
errDict := GemStoneError at: System myUserProfile nativeLanguage.
1 to: errNumbers size do: [:i | | msg |
  result add: lineFeed.
  result addAll: i asString.
  result addAll: ': ['.
  pos := errNumbers at: i.
  result addAll: pos asString.
  result addAll: '] '.
  msg := errMsgs at: i .
  msg == nil ifTrue:[
    pos > errDict size
      ifTrue: [ msg := '(unknown error number)']
      ifFalse: [ msg := (errDict at: pos) asString].
    ].
  result addAll: msg .
  (auxMsgs at: i) ~~ nil ifTrue:[ result addAll: (auxMsgs at: i) ].
  ].
result add: lineFeed.

^result
%

category: 'Private'
classmethod: GsNMethod
_terminateNonForkedProcess
  2 timesRepeat:[ self _class ]. "loop to detect/handle termination interrupt"
  TerminateProcess new details:'Cannot continue execution'; signalNotTrappable
%

!		Instance methods for 'GsNMethod'

category: 'Accessing'
method: GsNMethod
argsAndTemps

"Returns an Array of Symbols which are the names of arguments and
 temporaries for this method,  not including inner blocks"

 | offset numArgsTmps fi |
 numArgsTmps := self _numArgsTempsCblkargs .
 numArgsTmps < 1 ifTrue:[ ^  #()  ].

 offset := self isMethodForBlock ifTrue:[ DEBUGINFO_BLK_HDRSIZE ] ifFalse:[ 
   (fi := self _debugInfoFileInfo) == 1 ifTrue:[ 
     DEBUGINFO_RowanMTH_HDRSIZE 
   ] ifFalse:[
     fi == 0 ifTrue:[ 
        self _hasPragmaInfo ifTrue:[ DEBUGINFO_BLK_HDRSIZE ] 
                           ifFalse:[ DEBUGINFO_MTH_HDRSIZE ]
     ] ifFalse:[ 
       Error signal:' _debugInfoFileInfo value ', fi asString,' unsupported'. 
       0
     ]   
   ]
 ].
 offset := offset + 1 .
 ^ debugInfo copyFrom: offset to: (offset + numArgsTmps - 1)
%

category: 'Accessing'
method: GsNMethod
at: anIndex

"Returns value of specified varying instVar of receiver.

 Attempt to fetch from the send-site caches area of a method that
 has been loaded for execution will return nil.
 Use GsNMethod>>literals to get the complete literals pool.
"

anIndex < 1 ifTrue:[ self _errorIndexOutOfRange: anIndex ].
^ self _primitiveAt: (anIndex + GsNMethod_InstSize)
%

category: 'Accessing'
method: GsNMethod
at: anOffset put: aValue

"For use only on methods not yet invariant, such as during compilation
 of a CCallout for Ruby.  Most instances of GsNMethod are produced
 as invariant by primitives 679 or 228 .  Stores aValue into specified
 varying instVar of the receiver."

<primitive: 572>
anOffset _validateClass: SmallInteger .
self _errorIndexOutOfRange: anOffset .
self _primitiveFailed: #at:put:  args: { anOffset . aValue }
%

category: 'Accessing'
method: GsNMethod
basicSize

"Returns varying size of receiver."

^ self _primitiveSize - GsNMethod_InstSize
%

category: 'Disassembly'
method: GsNMethod
blockLiterals

"Return an Array of Block Literals that the receiver contains,
 including all inner blocks. Returns nil if the receiver
 contains no blocks.
 In-line blocks never have any block literals."

| arr |
self _numBlockLiterals ~~ 0 ifTrue:[
  arr := { } .
  self _addBlockLiteralsTo: arr .
] ifFalse:[
  arr := #() .
].
^ arr
%

category: 'Debugging Support'
method: GsNMethod
clearAllBreaks

"Clear all method breakpoints in the receiver."

self _allBreaksOp: 2 frame: nil process: nil
%

category: 'Debugging Support'
method: GsNMethod
clearBreakAtStepPoint: aStepPoint

"Clear method breakpoint at specified step point.
 Returns nil if aStepPoint is not legal otherwise returns receiver."

self _breakOperation: 2 forStepPoint: aStepPoint
%

category: 'Clustering'
method: GsNMethod
clusterDepthFirst

"This method clusters the receiver, its bytecodes, its selector pool, and its
 selector in depth-first order.  Returns true if the receiver has already been
 clustered during the current transaction; returns false otherwise."

self _clusterDepthFirst ifFalse:[ | blks |
  blks := self blockLiterals .
  blks do:[ :aBlk |
      aBlk clusterDepthFirst
  ].
  ^ false
].
^ true
%

category: 'Copying'
method: GsNMethod
copy

"Disallowed.  You may not create new instances of GsNMethod."

self shouldNotImplement: #copy
%

category: 'Debugging Support'
method: GsNMethod
disableAllBreaks

"Disable all method breakpoints in the receiver."


self _allBreaksOp: 4 frame: nil process: nil
%

category: 'Debugging Support'
method: GsNMethod
disableBreakAtStepPoint: aStepPoint

"Disable method breakpoint at specified step point.
 Returns nil if aStepPoint is not legal otherwise returns receiver."

self _breakOperation: 4 forStepPoint: aStepPoint
%

category: 'Stripping Sources'
method: GsNMethod
emptySource

"Returns nil in place of the source string.  The #emptySource selector may be
 used as an argument to the stripWith: keyword of the method
 GsNMethod>>decompileForCategory:classRef:stripWith:classMethod:, where it causes
 the string 'source not available...' to be used as the source string when
 reloading the decompiled method."

^ nil
%

category: 'Accessing'
method: GsNMethod
environmentId

"Return a SmallInteger,
 the 8 bit unsigned compilation environment identifier of this method."

 ^ (selector bitShift: 0 - SI_SELECTORID_ENV_shift) bitAnd: SELECTORID_ENV_mask
%

category: 'Stripping Sources'
method: GsNMethod
fullSource

"Returns the complete source string.  The #fullSource selector may be used as an
 argument to the stripWith: keyword of the method
 GsNMethod>>decompileForCategory:classRef:stripWith:classMethod:."

^ self sourceString
%

category: 'Disassembly'
method: GsNMethod
hasBlockLiteralsOfCost: aClassOrCost

"Return true if the receiver contains a block as costly as, or
 more costly than aClassOrCost, return false otherwise.

 For the purposes of this analysis, we have:
   simple block  cost == 1  (old class SimpleBlock)
   complex block cost == 2  (old class ComplexBlock)
   complex block with return to home, cost == 3 (ComplexVCBlock)

 Note that SimpleBlock, ComplexBlock, ComplexVCBlock
 are supported for compatiblity only . New code should use
 the integer costs since there is now only one block class, ExecBlock .
 In-line blocks never have an actual ExecBlock and are never included
 in this analysis.
 "

| blockLits |
blockLits := self _blockLiterals .
blockLits size ~~ 0 ifTrue:[  | argCost |
  aClassOrCost _isSmallInteger ifTrue:[
    argCost := aClassOrCost
  ] ifFalse:[
    argCost := aClassOrCost _cost .
  ].
  1 to: blockLits size do:[:k| | aBlk |
    aBlk := blockLits at: k .
    aBlk _cost >= argCost ifTrue:[ ^ true ] .
  ].
  1 to: blockLits size do:[:k| | aBlk |
    aBlk := blockLits at: k .
    "recursively check inner blocks"
    (aBlk method hasBlockLiteralsOfCost: argCost) ifTrue:[ ^ true].
  ].
].
^ false
%

category: 'Accessing'
method: GsNMethod
homeMethod

"Returns the home method if the receiver is method for a block,
 otherwise returns self."

^ self isMethodForBlock ifTrue:[ inClass ] ifFalse:[ self ]
%

category: 'Accessing'
method: GsNMethod
inClass

"Returns the class in which the receiver was compiled.
 Returns nil for an anonymous method."

self isMethodForBlock ifTrue:[
  ^ inClass"the home method"   inClass
] ifFalse:[
  ^ inClass   "result is nil for anonymous method"
]
%

category: 'Accessing'
method: GsNMethod
instVarAt: anIndex

"If the receiver has an instance variable at anIndex, returns
 its value.  Generates an error if anIndex is not a SmallInteger or
 is out of bounds, or if the receiver has no instance variables."

| numInstVars |
(anIndex _isSmallInteger) ifTrue: [
  numInstVars:= self class instSize .
  ((anIndex >= 1) and:[anIndex <= numInstVars]) ifFalse:[
        ^ self _errorIndexOutOfRange: anIndex "out of bounds"
  ].
  ^ self _primitiveAt: anIndex
] ifFalse: [
  ^ self _errorNonIntegerIndex: anIndex
].
%

category: 'Disassembly'
method: GsNMethod
instVarsAccessed

"Returns an IdentitySet of instVarNames that the method accesses,
 including accesses by all inner blocks."

^ self _allInstVarsAccessed: -2 into: IdentitySet new
%

category: 'Disassembly'
method: GsNMethod
instVarsRead

"Returns an IdentitySet of instVarNames that the method accesses,
 including accesses by all inner blocks "

^ self _allInstVarsAccessed: 0 into: IdentitySet new
%

category: 'Disassembly'
method: GsNMethod
instVarsWritten

"Returns an IdentitySet of instVarNames that the method accesses,
 including accesses by all inner blocks "

^ self _allInstVarsAccessed: -1 into: IdentitySet new
%

category: 'Accessing'
method: GsNMethod
invocationCount

"no longer supported"
^ 0
%

category: 'Accessing'
method: GsNMethod
isMethodForBlock

"Returns true if the receiver is the method for an ExecBloc."

^ ((iFields1 bitShift: 0 - IsMethodForBlock_shift) bitAnd: 1) == 1
%

category: 'Reporting'
method: GsNMethod
isSenderOf: aSymbol

"Returns true if the receiver sends the message aSymbol.  Returns false
 otherwise."

| ofs |
ofs := self _sourceOffsetOfFirstSendOf: aSymbol .
^ ofs ~~ nil
%

category: 'Stripping Sources'
method: GsNMethod
isSourceStripped

    "Answer true if the source code has been stripped for the
    receiver. Otherwise answer false. Determine this by asking the
    class if the method's selector is one of the known stripped
    selectors."

 "Stripping not implemented yet for GsNMethod"
 ^ false
%

category: 'Accessing'
method: GsNMethod
literals

"Returns an Array containing the literal pool of the receiver.,
 including literal pools of all inner blocks. "

| res blks |
res := self _literals .
blks := self blockLiterals .
1 to: blks size do:[:k | | aBlk|
  aBlk := blks at: k .
  res addAll: aBlk method _literals
].
^ res
%

category: 'Accessing'
method: GsNMethod
literalsOffset

"Returns the one-based offset to the start of the non-selector literals.
 Returns zero if there are no literals other than those in the selectorPool."

| ofs high |
ofs := (iFields1 bitShift: 0 - LiteralsOffset_shift) bitAnd: LiteralsOffset_mask .
ofs ~~ 0 ifTrue:[
  (ofs bitAnd: LiteralsOffset_lrgBit) ~~ 0 ifTrue:[
    ofs := ofs bitXor: LiteralsOffset_lrgBit .
    high := debugInfo at: DebugInfo_Fields2_offset .
    high := (high bitShift: 0 - HighLiteralsOffset_shift ) bitAnd: HighLiteralsOffset_mask .
    high := high bitShift: LiteralsOffset_smallBits .
    ofs := ofs bitOr: high .
  ].
  ofs := ofs + 1  "convert to one-based"
].
^ ofs
%

category: 'Accessing'
method: GsNMethod
loadedSizeBytes

 "Return the number of bytes of code generation memory that
  the receiver will occupy when loaded for execution, not
  including the native code."

  "header + namedIvs = 3 + 4 = 7 words"

  ^ (7 + self size + (self numSends * 2)) * 8
%

category: 'Accessing'
method: GsNMethod
methodCompilerVersion

"Returns the method compiler version.
 4 indicates a method compiled by Gs64 v3.3 or above method compiler,
 3 indicates a method compiled by Gs64 v3.0 or above method compiler,
 2 indicates a method compiled by Gs64 v2.0 or above method compiler,
 1 indicates a method compiled in a previous version and processed by
   repository conversion.

 Any other value indicates a method from a previous version that
 did not get converted. "

^ (iFields1 bitShift: MethCompilerVers_shift negated) bitAnd: MethCompilerVers_mask
%

category: 'Repository Conversion'
method: GsNMethod
needsRecompile

"Returns true if the receiver needs recompilation, false otherwise."

 ^ self methodCompilerVersion < 4
%

category: 'Accessing'
method: GsNMethod
numArgs

"Returns the number of arguments expected by the method."

^ iFields1 bitAnd: NArgs_mask
%

category: 'Accessing'
method: GsNMethod
numSends

"Returns the number of non-special sends in the method."

^ (iFields1 bitShift: 0 - NumSends_shift) bitAnd: NumSends_mask
%

category: 'Pragmas'
method: GsNMethod
pragmas
  "Returns a possibly empty Array of Pragmas."
  (self _debugInfoAccess: 11 at: -1) ifNotNil:[:parray |
    parray == #() ifTrue:[ ^ #() ].
    ^ self class createPragmaFrom: parray for: self
  ].
  self environmentId ~~ 0 ifTrue:[ ^ #() ].
  self inClass ifNotNil:[:cls | ^ cls _pragmasForMethod: self ]
%

category: 'Formatting'
method: GsNMethod
printOn: aStream
  | classOrNil selectorOrNil |
  aStream nextPutAll: self class name; nextPut: $  .
  self isMethodForBlock ifTrue: [aStream nextPutAll: '[] in '].
  classOrNil := self inClass.
  selectorOrNil := self homeMethod selector.
  aStream nextPutAll: (classOrNil ifNil:[ '<nil>' ] ifNotNil:[ classOrNil name]);
          nextPutAll: '>>';
          nextPutAll: (selectorOrNil ifNil:[ '<anonymous>'] ifNotNil:[ selectorOrNil]).
%

category: 'Repository Conversion'
method: GsNMethod
recompile
  "Returns self if recompile not needed, otherwise
   returns a new instance of GsNMethod using session's default symbolList"

  self needsRecompile ifFalse:[ ^ self "recompile not needed"].
  ^ self recompileIntoMethodDict: nil intoCategories: nil .
%

category: 'Repository Conversion'
method: GsNMethod
recompileIntoMethodDict: aMethodDict intoCategories: aCategDict
  ^ self recompileIntoMethodDict: aMethodDict intoCategories: aCategDict symbolList: nil
%

category: 'Repository Conversion'
method: GsNMethod
recompileIntoMethodDict: aMethodDict intoCategories: aCategDict symbolList: aSymbolList

"Recompiles the method for execution in a Gs64 v3.0 or later system.

 Literal variables whose key is in ObsoleteClasses
 are replaced by the appropriate association from the ObsoleteClassesDict .
 Other literal variables are looked up in the literal pool of the receiver,
 before searching class variables, class pool dictionaries, or
 the current symbolList.  Thus recompilation should work without knowing
 what symbolList was used when the receiver was created.

 The result is a GsNMethod if compilation succeeds, otherwise
 an error is generated.  environmentId zero is used for all compilations.

 If aMethodDict is not nil, and the compilation succeeds,
 the resulting method is added to aMethodDict instead of to
 the receiver's method dictionary.  This is used to add methods
 to per-session dictionaries.

 If aMethodDict is not nil and aCategDict is not nil and
 the compilation succeeds, the resulting method is added aCategDict
 instead of the receiver's categories.

 If the receiver is an anonymous method, the sender of this method is
 reponsible for saving the result."

| cls litVars newSrc |
litVars := self _literalVariablesForRecompile .
newSrc := self sourceString .
(System gemConfigurationAt:#GemConvertArrayBuilder) ifTrue:[
  newSrc := GsNMethod convertArrayBuildersInString: newSrc .
].
cls := self inClass .
cls ifNil:[
  ^ newSrc _compileInContext: nil symbolList: aSymbolList oldLitVars: litVars
		environmentId: 0 flags: 0
] ifNotNil:[ | categ |
 categ := cls categoryOfSelector: self selector .
  ^ cls _checkCompileResult:
     ( cls _primitiveCompileMethod: newSrc symbolList: aSymbolList
         category: categ oldLitVars: litVars
         intoMethodDict: aMethodDict intoCategories: aCategDict
         environmentId: 0 )
     source: newSrc .
]
%

category: 'Stripping Sources'
method: GsNMethod
removeAllSourceButFirstComment

"Installs a new source string for the receiver so that only the method signature
 and the first comment (if it exists) are left.  For use in stripping a method
 in place in GemStone.  Bypasses the invariance of the receiver."

self _validatePrivilege ifTrue:[
  "v3.0, debuginfo ok to modify if committed."
  self _debugInfo _unsafeAt: DebugInfo_source_offset put: self _sourceToFirstComment
].
%

category: 'Accessing'
method: GsNMethod
selector

"Returns the Symbol which is the selector of this method."

| selIv selectorObjId |
" in blocks and anonymous methods, selector instVar contains envId and #'' "
self isMethodForBlock ifTrue:[ ^ nil ].
inClass == nil ifTrue:[ ^ nil "anonymous method"].
selIv := selector .
"note bit operations are on integer value of SmallInteger, not on a OopType"
selectorObjId := ((selIv bitAnd: 16r1fffffffffe0 ) bitShift:3) bitOr: 1 .
^ Object _objectForOop: selectorObjId .
%

category: 'Debugging Support'
method: GsNMethod
setBreakAtStepPoint: aStepPoint

"Set method breakpoint at specified step point.
 Returns nil if aStepPoint is not legal otherwise returns receiver."

^ self _breakOperation: 0 forStepPoint: aStepPoint
%

category: 'Accessing'
method: GsNMethod
size
"Returns varying size of receiver."

^ self _primitiveSize - GsNMethod_InstSize
%

category: 'Disassembly'
method: GsNMethod
sourceOffsetsOfInstVar: aString

"Returns an Array of sourceOffsets of access to the specified instVar.
 Returns nil if there is no instVar named aString in the receiver's class."
| ofs sym |
(sym := Symbol _existingWithAll: aString ) ifNil:[ ^ nil ].
ofs := self inClass offsetOfInstVar: sym .
ofs == 0 ifTrue:[ ^ nil ].
^ Array withAll:(self _allInstVarsAccessed: ofs into: IdentitySet new)
%

category: 'Accessing'
method: GsNMethod
sourceString

"Returns a CharacterCollection that contains the source code of the receiver."

^ debugInfo at: DebugInfo_source_offset
%

category: 'Stripping Sources'
method: GsNMethod
sourceToFirstComment

"Returns a new source string for the receiver that contains only the method
 signature and the first comment (if it exists).  Does not modify the
 receiver.  The #sourceToFirstComment selector may be used as an argument to
 the stripWith: keyword of the method
 GsNMethod>>decompileForCategory:classRef:stripWith:classMethod:."

| i tmpString sz srcStr nArgs selectorSym |

i := 0.
srcStr := self sourceString .
sz := srcStr size.
selectorSym := self selector .
(selectorSym occurrencesOf: $:) timesRepeat: [
    i := srcStr indexOf: $: startingAt: i + 1.
].
" check if it's a binary selector (it has an argument but no colons) "
nArgs := self numArgs .
(i == 0 and: [ nArgs = 1 ])
  ifTrue: [ i := selectorSym size ].
" i is the offset of the last colon in the signature, or the last character
of a binary selector, or zero "
i := i + 1.

" scan past any white space "
[ (srcStr at: i) isSeparator ] whileTrue: [ i := i + 1 ].

i > sz
  ifTrue: [ ^ srcStr copy ].

" scan past any non-white space to get to end of argument to last keyword"
[ i <= sz and: [ (srcStr at: i) isSeparator not ]] whileTrue: [ i := i + 1 ].
" i is now the offset of the first white space past the signature "

i > sz
  ifTrue: [ ^ srcStr copy ].

" scan past any white space "
[ i <= sz and: [ (srcStr at: i) isSeparator ] ] whileTrue: [ i := i + 1 ].
" i is now the offset of the initial comment or first line of code "

i > sz
  ifTrue: [ ^ srcStr copy ].

" if i is the offset of the initial comment, jump to the end of the comment "
(srcStr at: i) == $"
  ifTrue: [ i := srcStr indexOf: $" startingAt: i + 1 ]
  ifFalse: [ i := i - 1 ].

" create the string to replace the original source with "
tmpString := srcStr copyFrom: 1 to: i .
tmpString addAll: '

< source code not available >'.

^ tmpString
%

category: 'Accessing'
method: GsNMethod
squeakBasicAt: anIndex
 ^ self _basicAt: anIndex
%

category: 'CodeModification Override'
method: GsNMethod
squeakBasicAt: anIndex put: aValue

self shouldNotImplement:#squeakBasicAt:put:
%

category: 'Accessing'
method: GsNMethod
squeakBasicSize
^ self basicSize
%

category: 'Storing and Loading'
method: GsNMethod
writeTo: aPassiveObject

"Instances of GsNMethod cannot be converted to passive form.  This method writes
 nil to aPassiveObject and stops GemStone Smalltalk execution with a notifier."

aPassiveObject writeObject: nil.
self _error: #rtErrAttemptToPassivateInvalidObject.
%

category: 'Disassembly'
method: GsNMethod
_addBlockLiteralsTo: anArray
  "Add all block literals of receiver plus inner blocks to anArray"
  | lastBlkIdx numBlks |
  numBlks := self _numBlockLiterals .
  numBlks ~~ 0 ifTrue:[
    lastBlkIdx := self _lastBlockLiteralOffset .
    lastBlkIdx - numBlks + 1 to: lastBlkIdx do:[:n| | aBlk | "recursively process blocks"
      aBlk := self at: n .
      anArray add: aBlk .
      aBlk method _addBlockLiteralsTo: anArray
    ].
  ].
%

category: 'Debugging Support'
method: GsNMethod
_allBreakpoints

"Returns an Array of the form

       { breakpointNumber1 . method . ipOffset1 .
          ...
          breakpointNumberN . method . ipOffsetN .
       }

 The ipOffsets are instruction offsets.
 Negative ipOffsets denote disabled breakpoints.

 Returns nil if no method breakpoints set in the receiver.
 Single step breakpoints managed by GciStep() are not reported."

| res blks |
res := self __allBreakpoints .
blks := self blockLiterals .
blks do:[ :aBlk | | blkRes |
    blkRes := aBlk method __allBreakpoints .
    blkRes size ~~ 0 ifTrue:[
      res == nil ifTrue:[ res := { }  ].
      res addAll: blkRes
    ].
].
^ res
%

category: 'Debugging Support'
method: GsNMethod
_allBreakpointsSourceOffsets

| brksArr result |
brksArr := self _allBreakpoints .         "includes methods of inner blocks"
brksArr == nil ifTrue:[ ^ nil ].
result := { }  .
1 to: brksArr size by: 3 do:[:k | | meth allIps stepPoint anIp |
  meth := brksArr at: k + 1 .
  anIp := brksArr at: k + 2 .
  allIps := meth _debugInfoAccess: 2 at: -1 .
  stepPoint := allIps indexOf: anIp .
  stepPoint ~~ 0 ifTrue:[  "convert to source offset"
    result add:( meth _debugInfoAccess: 1 at: stepPoint )
  ] ifFalse:[
    result add: 1 . "unexpected path"
  ].
].
^ result
%

category: 'Debugging Support'
method: GsNMethod
_allBreaksOp: anInt frame: fpOffset process: aGsProcess

| blks |
self _setBreakAtIp: -1 operation: anInt frame: fpOffset process: aGsProcess .
blks := self blockLiterals .
"Don't use do:  here,  we might be setting single step breakpoints
  within a do: implementation..."
1 to: blks size do:[ :j |
    (blks at: j) method _setBreakAtIp: -1
	operation: anInt frame: fpOffset process: aGsProcess
].
%

category: 'Debugging Support'
method: GsNMethod
_allDebugInfo: kind

| res blks |
res := self _debugInfoAccess: kind  at: -1 .
blks := self blockLiterals .
blks do:[ :aBlk | | blkRes |
    blkRes := aBlk method _debugInfoAccess: kind  at: -1 .
    blkRes size ~~ 0 ifTrue:[ res addAll: blkRes ].
].
^ res
%

category: 'Debugging Support'
method: GsNMethod
_allDebugInfoWithMeths: kind

| res arr blks |
res := { }  .
arr := self _debugInfoAccess: kind  at: -1 .
1 to: arr size do:[:j | res addLast: { self . (arr at: j) } ].
blks := self blockLiterals .
blks do:[ :aBlk | | blkRes aMeth |
    aMeth := aBlk method .
    blkRes := aMeth _debugInfoAccess: kind  at: -1 .
    1 to: blkRes size do:[:j | res addLast: { aMeth . (blkRes at: j) }].
].
^ res
%

category: 'Disassembly'
method: GsNMethod
_allInstVarsAccessed: kind into: anIdentitySet

"Adds to result the instVarNames that the method accesses,
 including accesses by all inner blocks , as specified by kind.
 Returns anIdentitySet .

  kind == 0 --> read ,   result is an IdentitySet of instVar names
  kind == -1 --> written,  result is an IdentitySet of instVar names
  kind == -2 --> read or written, result is an IdentitySet of instVar names
  kind is one based offset of an instVar,  result is IdentitySet of source offsets of access to that instVar"

| blks |
self _instVarsAccessed: kind into: anIdentitySet .
blks := self blockLiterals .
blks do:[ :aBlk |
  aBlk method _instVarsAccessed: kind into: anIdentitySet .
].
^ anIdentitySet
%

category: 'Accessing'
method: GsNMethod
_argsAndTempsOffsets

"Returns an Array of SmallIntegers .
 Each SmallInteger has bits
     16rFF , lexical level , number of VC.parent refs to follow to the defining VC
     high order bits are a signed   offset*256 .
 The Array represents
   numArgs positive zero-based offsets with respect to FP ( level == 0)
   numTemps offsets ,
      negative offsets are zero-based with respect to FP (level == 0)
      positive offsets are zero based wrt.  instVar 0 of a VC
        the bits 16r000FF are number of VC.parent refs to follow to VC
   Does not include any info for  inner blocks"

 ^ self _debugInfoAccess: 8 at: -1
%

category: 'Accessing'
method: GsNMethod
_at: anIndex

"Returns value of specified varying instVar of receiver.
 Attempt to fetch from the send-site caches area of a method that
 has been loaded for execution will return nil.
 Use GsNMethod>>literals to get the complete literals pool.
"

anIndex < 1 ifTrue:[ self _errorIndexOutOfRange: anIndex ].
^ self _primitiveAt: (anIndex + GsNMethod_InstSize)
%

category: 'CodeModification Override'
method: GsNMethod
_at: anIndex put: aValue

self shouldNotImplement:#_at:put:
%

category: 'Accessing'
method: GsNMethod
_basicAt: anIndex

"Returns value of specified varying instVar of receiver.
 Attempt to fetch from the send-site caches area of a method that
 has been loaded for execution will return nil.
 Use GsNMethod>>literals to get the complete literals pool.
"

anIndex < 1 ifTrue:[ self _errorIndexOutOfRange: anIndex ].
^ self _primitiveAt: (anIndex + GsNMethod_InstSize)
%

category: 'CodeModification Override'
method: GsNMethod
_basicAt: anIndex put: aValue

self shouldNotImplement:#_basicAt:put:
%

category: 'Disassembly'
method: GsNMethod
_blockLiterals

"Return an Array of block literals that the receiver contains,
 or nil if the receiver contains no block literals.
 In-line blocks never have any block literals.
 Literals for inner blocks are not included."

| arr lastBlkIdx numBlks |
numBlks := self _numBlockLiterals .
numBlks ~~ 0 ifTrue:[  | resIdx |
  arr := Array new: numBlks .
  lastBlkIdx := self _lastBlockLiteralOffset .
  resIdx := 1 .
  lastBlkIdx - numBlks + 1 to: lastBlkIdx do:[:n|
    arr at: resIdx put:( self at: n ) .
    resIdx := resIdx + 1 .
  ].
].
^ arr
%

category: 'Debugging Support'
method: GsNMethod
_blockSourceOffsets
  "If receiver is a home method, return an array containing source offsets
   of all step point in receiver and its blocks,
   otherwise return an Array of size equal all source offsets in the home method,
   and containing source offsets of step points within the receiver,
   and nils for all other step points of the home method and its blocks."
| homeMth res blks |
homeMth := self homeMethod .
homeMth == self ifTrue:[ ^ self _sourceOffsets ].
res := homeMth _debugInfoAccess: 1  at: -1 .
res := Array new:(res size) . "convert to nils"
blks := homeMth blockLiterals .
blks do:[ :aBlk | | blkMth blkRes |
    blkMth := aBlk method .
    blkRes := blkMth _debugInfoAccess: 1  at: -1 .
    blkMth == self ifTrue:[ res addAll: blkRes "just these step points"]
        ifFalse:[ res size: (res size + blkRes size) "append nils"].
].
^ res
%

category: 'Debugging Support'
method: GsNMethod
_breakOperation: anInt forStepPoint: aStepPoint

"Returns nil if aStepPoint is not legal otherwise returns receiver.
 anInt is an opcode as defined for second arg to
 _setBreakAtIp:operation:frame:process:
"

| info |
info := self _meth_ip_ForStepPoint: aStepPoint abs .
info == nil ifTrue:[ ^ nil ].
(info at:1) _setBreakAtIp: (info at: 2)  operation: anInt frame: nil process: nil .
%

category: 'Debugging Support'
method: GsNMethod
_breakPointKind: anIp

"This method infers the kind of action associated with a given bytecode."

self error:'GsNMethod>>_breakPointKind:  not implemented'.
%

category: 'Debugging Support'
method: GsNMethod
_buildIpMarkerArray

"This method builds a marker Array for the receiver's source code string.
 containing IPs of all step points, not including step points in inner blocks.

 The result Array is the same size as the source string and
 contains IP numbers at offsets corresponding to the source string."

| srcOffsets ipsArr srcSize mrkSize markerArray |

srcOffsets := self _debugInfoAccess: 1 at: -1 .
ipsArr := self _debugInfoAccess:2 at: -1 .
srcSize := self sourceString size .

mrkSize := 1 max: srcSize .                          "fix bug 14976"
markerArray:= Array new: mrkSize .
1 to: srcOffsets size do: [:i | | anOffset anIp posOffset |
  anOffset := srcOffsets at: i .
  anIp := (ipsArr at: i ) .
  posOffset := (anOffset abs max: 1) min: mrkSize.  "limit within range"
  (markerArray at: posOffset) ifNotNil:[  "one retry for fix 45431"
     posOffset := posOffset + 1 min: mrkSize
  ].
  (markerArray at: posOffset) ifNil:[
     markerArray at: posOffset put: anIp
  ]
].
^ markerArray
%

category: 'Debugging Support'
method: GsNMethod
_buildMarkerArray: allSteps ofSize: aSize

"This method returns a marker Array for the receiver's source code string,
 each element of the result is the source offset of a step point.

 allSteps == true , show all steps
          == false, show steps where a breakpoint currently exists
          a SmallInteger, show just that step point

 The result Array is the same size as the source string and
 contains step numbers at offsets corresponding to the source string."

| srcOffsets |

srcOffsets := self _blockSourceOffsets  .
(allSteps _isSmallInteger ) ifTrue:[  | numSteps stepToDisplay |
  stepToDisplay := allSteps .
  numSteps := srcOffsets size .
  stepToDisplay < 1 ifTrue:[ stepToDisplay := 1 ].
  stepToDisplay > numSteps ifTrue:[ stepToDisplay:= numSteps ].
  1 to: numSteps do:[ :j |
     j == stepToDisplay ifFalse:[ srcOffsets at: j put: nil ].
  ].
] ifFalse:[
  allSteps ifFalse:[ self _setBreakpointsInSourceOffsets: srcOffsets ].
].

^ GsNMethod _buildMarkersFrom: srcOffsets ofSize: aSize
%

category: 'Reporting'
method: GsNMethod
_classAndSelectorNameWidth: anInt

"Return a String of the form className | selector with the className substring
 padded to width anInt.
 Used by ProfMonitor"

|text sel|
self isMethodForBlock ifTrue:[ | homeMeth homeCls |
  homeMeth := self homeMethod .
  homeCls := homeMeth inClass .
  homeCls ifNotNil:[
    text := 'block in ', homeCls name .
  ] ifNil:[
    text := 'block in executed code' copy .
  ].
  sel := homeMeth selector .
] ifFalse:[  | inCls |
  (inCls := inClass) ifNil: [
    text := 'executed code' copy .
  ] ifNotNil:[
    text := String withAll: inClass name .
    sel := self selector .
  ].
].
text width: anInt .
sel ifNotNil:[ text addAll: ' >> '; addAll: sel ].
^ text
%

category: 'Debugging Support'
method: GsNMethod
_clearAllStepBreaks

"Legacy code, no longer used"

self _allBreaksOp: 3 frame: nil process: nil
%

category: 'Clustering'
method: GsNMethod
_clusterDepthFirst

"This method clusters the receiver, its bytecodes, its selector pool, and its
 selector in depth-first order.  Returns true if the receiver has already been
 clustered during the current transaction; returns false otherwise."

| savedBucket systm nonBlkLits |

self cluster ifTrue: [ ^ true ] .
systm := System .
savedBucket := systm clusterBucket.
systm clusterBucket: 5.  "kernel classes description bucket"
debugInfo cluster.   "sourceString part of debugInfo now"
self sourceString cluster .
systm clusterBucket: savedBucket.
nonBlkLits := self _literals .
1 to: nonBlkLits size do:[:k| | aLit |
  aLit := nonBlkLits at: k .
  aLit _isSymbol ifFalse:[
    (aLit class ~~ SymbolAssociation and:[ systm canWrite: aLit ]) ifTrue:[
      aLit clusterDepthFirst
    ].
  ].
].
^ false
%

category: 'Private for Class Modification'
method: GsNMethod
_copyForRecompilation

self _validatePrivilege ifTrue:[
  ^ super copy .
].
^ nil
%

category: 'Private for Class Modification'
method: GsNMethod
_copyToForceRecompilation

"Returns a copy of the receiver, with bytecodes that will force
 recompilation of the receiver prior to subsequent execution.

 Cannot be used to recompile the method for an individual block,
 you must recompile the home method for the block.
"

| newMethod dbgInfo |
self _validatePrivilege ifFalse:[ ^ nil ].
self isMethodForBlock ifTrue:[
  ^ self error:'cannot recompile method for a block, must recompile the home method.'
].
newMethod := (GsNMethod compiledMethodAt: #_methodNeedingRecompilation)
                _copyForRecompilation .
dbgInfo := { 0 . self sourceString . 0 } .
dbgInfo immediateInvariant .
newMethod _debugInfo: dbgInfo .
newMethod _inClass: inClass .
newMethod immediateInvariant . "required to pass assertions in object manager"
^ newMethod
%

category: 'Accessing'
method: GsNMethod
_debugInfo
 "Returns an Array with varying elements.
  See $GEMSTONE/upgrade/opalcls.ht for details . "

  ^ debugInfo
%

category: 'Private for Class Modification'
method: GsNMethod
_debugInfo: anArray

self _validatePrivilege ifTrue:[
  debugInfo := anArray
].
%

category: 'Debugging Support'
method: GsNMethod
_debugInfoAccess: kind at: ofs

"Return debugging information per kind and ofs for the  receiver.
 Does not search any methods for blockLiterals of the receiver.

 Signals a Notification and returns nil if the requested item is
 not available because the receiver needs recompilation.

  kind = 1  sourceOffsets (ofs > 0 , source Offset for specified step point
                           ofs == -1, all sourceOffsets
                  for ofs > 0 , if ofs beyond last step point, returns
                  SmallInteger maximumValue  )

  kind = 2  all ipSteps  (ofs == -1)
  kind = 3  offset into ipSteps for instruction PRECEEDING the
                absolute instruction offset specified by ofs
  kind = 4  offset into ipSteps for instruction FOLLOWING the
                absolute instruction offset specified by ofs
  kind = 5  _sourceOffsetOfFirstSendOf: , ofs is aSymbol

  kind = 6  _ipForStepPoint:   ofs is a stepPoint, result is an instr offset,
		or nil if stepPoint does not exist in the receiver.

  kind = 7  _selectorPool   result is a IdentitySet of all selectors sent
                         (ofs == -1)
  kind = 8  argAndTmpOffsets , zero based word offsets  (ofs == -1)
            for the args,  offsets are positive with respect to FP
            for the temps,
              negative offsets denote offsets with respect to FP
              positive offsets encode the fields
                 16rFFF00 zero-based offset wrt varying instVar 0 of VC,
                 16r000FF number of VC.parent refs to follow to the defining VC

  kind = 9  _lastBlockLiteralOffset   (ofs == -1)

  kind = 10 _sourceOffsetsOfSends  (ofs not used)
            returns Array of pairs { anOffset , aSymbol, ... anOffset, aSymbol}

  kind = 11  returns Array of pragma info or nil.  (ofs == -1)
             returns the canonical empty Array if receiver is method for a block
"

<primitive: 672>
self methodCompilerVersion < 4 ifTrue:[
  Notification signal: (self _classAndSelectorNameWidth: 0) ,
		      ' method from Gs64 v3.2.x needs recompile' .
  ^ nil .
].
self _primitiveFailed: #_debugInfoAccess:at: args: { kind . ofs }
%

category: 'Debugging Support'
method: GsNMethod
_debugInfoFileInfo
  "result is a SmallInteger 0, 1, 2"
 | info1 |
 info1 := debugInfo at: DebugInfo_fields1_offset .
 ^ ((info1 bitShift: 0 - DbgI1fileInfo_shift ) bitAnd: DbgI1fileInfo_mask) 
%

category: 'Debugging Support'
method: GsNMethod
_descrForStack
^ self _descrForStackPadTo: 0
%

category: 'Debugging Support'
method: GsNMethod
_descrForStackPadTo: minSize

^ self _descrForStackPadTo: minSize rcvr: _remoteNil
%

category: 'Debugging Support'
method: GsNMethod
_descrForStackPadTo: minSize rcvr: aRcvr

| result hmMeth inCls isBlock |
result := String new.
self isMethodForBlock ifTrue:[
  isBlock := true.
  result add: '[] in '.
  hmMeth := inClass.
  inCls := hmMeth inClass
] ifFalse:[
  isBlock := false.
  hmMeth := self.
  inCls := inClass
].
inCls ifNil: [result add: 'Executed Code ']
   ifNotNil:[
     aRcvr ~~ _remoteNil ifTrue:[| rcvCls |
       (isBlock not and: [(rcvCls := Reflection classOf: aRcvr) ~~ inCls]) ifTrue:[
         result add: rcvCls name; add: ' ('; add: inCls name; add: $)
       ] ifFalse:[result add: inCls name]
     ] ifFalse: [result add: inCls name].
     result add: ' >> '; add: (hmMeth selector ifNil:['(executed code)'])
].
[ result size < minSize ] whileTrue:[ result add: $ ].    "pad with spaces"
inClass ifNotNil:[ | env |
   env := hmMeth environmentId.
   env ~~ 0 ifTrue:[ result add: ' (envId '; add: env asString; add: $)]
].
^result
%

category: 'Deprecated'
method: GsNMethod
_emptySource

self deprecated: '_emptySource obsolete, Use #emptySource'.
^ self emptySource
%

category: 'Execution'
method: GsNMethod
_executeInContext: anObject

"The receiver should be a zero argument GsNMethod obtained by invoking
 String/_compileInContext:symbolList: and the argument should be the argument
 passed to the _compileInContext: keyword for that compilation.

 Returns the result of executing the receiver with anObject as self."

<primitive: 2008>
^ self _primitiveFailed: #_executeInContext: args: { anObject }
%

category: 'Accessing'
method: GsNMethod
_fileAndLine
  "Return an Array, { fileName . lineNumber } ,
   or nil if method  does not have that information.
   Smalltalk methods return nil."

  ^ nil
%

category: 'Debugging Support'
method: GsNMethod
_firstSourceOffset

 | arr |
 arr := self _debugInfoAccess: 1 at: -1.
 arr size > 0 ifTrue:[ ^ arr at: 1 ].
 ^ 1
%

category: 'Deprecated'
method: GsNMethod
_fullSource

self deprecated: '_fullSource obsolete, Use #fullSource'.
^ self fullSource
%

category: 'Debugging Support'
method: GsNMethod
_hasPragmaInfo
  "Returns true or false.
   Result always false for methods compiled before Gs64 v3.5"

^ (((debugInfo at: DebugInfo_fields2_offset)
      bitShift: 0 - debugI2_hasPragmas_shift) bitAnd: 1) ~~ 0
%

category: 'GBS compatibility'
method: GsNMethod
_inClass
  ^ self inClass
%

category: 'Private for Class Modification'
method: GsNMethod
_inClass: aClass

"for use only by _copyToForceRecompilation"

self _validatePrivilege ifTrue:[
  inClass := aClass
]
%

category: 'Disassembly'
method: GsNMethod
_instVarsAccessed: kind into: result

"Adds to result
  the names of the instVars which are read or written by the method,
  per the kind argument, not including accesses by any inner blocks .
 or Adds to result the source offsets of accesses to the specified instVar

  kind == 0 --> read ,
  kind == -1 --> written,
  kind == -2 --> read or written
  kind is one based offset of an instVar"

<primitive: 671>
self _primitiveFailed: #_instVarsAccessed:into: args: { kind . result }
%

category: 'Accessing'
method: GsNMethod
_ipSteps

"Returns an Array containing the step points for the portable code for the
 method."

| res |
res := self _allDebugInfo: 2  .
res size == 0 ifTrue:[ ^ #( 1) "a primitive method" ].
^ res
%

category: 'Debugging Support'
method: GsNMethod
_isPrimitive
| firstOfs |
firstOfs := GSNMETHOD_FIRST_INSTR_OFFSET .
^ self _opcodeAtIsCallPrimitive: firstOfs
%

category: 'Debugging Support'
method: GsNMethod
_isProtected

| firstOfs |
firstOfs := GSNMETHOD_FIRST_INSTR_OFFSET .
^ ((self _opcodeKindAt: firstOfs) bitAnd: 16r2) ~~ 0
%

category: 'Debugging Support'
method: GsNMethod
_lastBlockLiteralOffset
  "Returns offset of last block literal.
   If there are no block literals, result will be offset of the last literal.
   If there are no literals , result will be offset of last instruction word"
  ^ self _debugInfoAccess: 9 at: -1
%

category: 'Accessing'
method: GsNMethod
_lineDeltaForBlock

"Return the line number of the start of the block relative
 to the start of the home method source."

| ofs src |
ofs := self _sourceStartForBlock .
ofs <= 1 ifTrue:[ ^ 0 ].
src := debugInfo at: DebugInfo_source_offset .

^ (self _lineNumberForOffset: ofs inString: src) - 1
%

category: 'Debugging Support'
method: GsNMethod
_lineNumberForIp: targetIp

"Returns the line number in the receiver's source string for the
 specified IP value within the receiver.
 Assumes that the IP is from a frame that is Not at top of Stack."

| stepPoint |
stepPoint := self _previousStepPointForIp: targetIp .
^ self _lineNumberForStep: stepPoint
%

category: 'Private'
method: GsNMethod
_lineNumberForOffset: sourceOffset inString: srcStr
  " find the first end-of-line which is at or after the
    sourceOffset of the step point."

  | offset lf cr lineNumber |
  lf := Character withValue: 10 .
  cr := Character withValue: 13 .
  offset := 1 .
  lineNumber := 0 .

  (srcStr indexOf: cr startingAt: offset) == 0 ifTrue:[
    "no CR's in source"
    [ offset <= sourceOffset ] whileTrue:[
      lineNumber := lineNumber + 1 .
      offset := srcStr indexOf: lf startingAt: offset .
      offset == 0 ifTrue:[ ^ lineNumber ].
      offset := offset + 1 .
    ].
  ] ifFalse:[
    "at least one CR present in source"
    [ offset <= sourceOffset ] whileTrue:[ | lfOfs crOfs |
      lineNumber := lineNumber + 1 .
      lfOfs := srcStr indexOf: lf startingAt: offset .
      crOfs := srcStr indexOf: cr startingAt: offset .
      crOfs ~~ 0 ifTrue:[
        offset := lfOfs ~~ 0 ifTrue:[ crOfs min: lfOfs ] ifFalse:[ crOfs ]
      ] ifFalse:[
        offset := lfOfs ~~ 0 ifTrue:[ lfOfs ] ifFalse:[ 0 ].
      ].
      offset == 0 ifTrue:[ ^ lineNumber ].
      (crOfs ~~ 0 and:[ lfOfs == (crOfs + 1) ]) ifTrue:[
        offset := offset + 2 .  "skip CR LF pair"
      ] ifFalse:[
        offset := offset + 1 .  "skip CR or LF "
      ]
    ].
  ].
  ^ lineNumber .
%

category: 'Debugging Support'
method: GsNMethod
_lineNumberForStep: aStepPoint

"Returns the line number in the receiver's source string for the
 step point with number aStepPoint.
 Returns 1 if aStepPoint is out of range."

  | sourceOffset |
  sourceOffset := self _sourceOffsetsAt: aStepPoint .
  sourceOffset == nil ifTrue:[ ^ 1 ].

  ^ self _lineNumberForOffset: sourceOffset inString: self sourceString
%

category: 'Debugging Support'
method: GsNMethod
_lineNumberForTosIp: targetIp

"Returns the line number in the receiver's source string for the
 specified IP value within the receiver.
 Assumes that the IP is from a frame that IS at top of Stack ."

| stepPoint |
stepPoint := self _nextStepPointForIp: targetIp .
^ self _lineNumberForStep: stepPoint
%

category: 'Accessing'
method: GsNMethod
_literals

"Returns an Array containing the literal pool of the receiver ,
 excluding inner blocks, and including the selector pool."

| litOfs res last |
res := { }  .
(litOfs := self literalsOffset) ~~ 0 ifTrue:[
  last := self _lastBlockLiteralOffset - self _numBlockLiterals .
  litOfs to: last do: [:k || aLit |
    aLit := self at: k .
    res add: aLit .
  ].
].
res addAll: self _selectorPool .
^ res
%

category: 'Class organizer support'
method: GsNMethod
_literalsIncludesSymbol: aSymbol value: anObject
  "if anObject == nil , it is not searched for as a value"
  | lastBlkIdx numBlks litOfs |
  (self _selectorPool includes: aSymbol) ifTrue:[ ^ true ].
  (litOfs := self literalsOffset) ~~ 0 ifTrue:[
    lastBlkIdx := self _lastBlockLiteralOffset .
    numBlks := self _numBlockLiterals .
    anObject ifNil:[
      litOfs to: (lastBlkIdx - numBlks) do:[:k| | lit |
        lit := self at: k.
        (lit == aSymbol
          or:[ ((lit isKindOf: SymbolAssociation) and:[ lit key == aSymbol ])
           or:[ lit _isArray and:[ lit includesIdentical: aSymbol ]]]
        ) ifTrue:[ ^ true ]
      ].
    ] ifNotNil:[
      litOfs to: (lastBlkIdx - numBlks) do:[:k| | lit |
        lit := self at: k.
        (lit == aSymbol  
          or:[ lit == anObject
           or:[ ((lit isKindOf: SymbolAssociation) 
                  and:[ lit key == aSymbol or:[ lit _value == anObject]])
            or:[ lit _isArray and:[ lit includesIdentical: aSymbol ]]]]
         ) ifTrue:[ ^ true ]
      ].
    ].
    lastBlkIdx - numBlks + 1 to: lastBlkIdx do:[:n| | aBlk |
      "recursively search blocks"
      aBlk := self at: n .
      (aBlk method _literalsIncludesSymbol: aSymbol value: anObject) ifTrue:[ ^ true ] .
    ].
  ].
  ^ false
%

category: 'Class organizer support'
method: GsNMethod
_literalsIncludesValue: anObject
  | litOfs lastBlkIdx numBlks |
  (self _selectorPool includes: anObject) ifTrue:[ ^ true ].
  (litOfs := self literalsOffset) ~~ 0 ifTrue:[
    lastBlkIdx := self _lastBlockLiteralOffset .
    numBlks := self _numBlockLiterals .
    litOfs to: (lastBlkIdx - numBlks) do:[:k| | lit |
      lit := self at: k.
      (lit == anObject 
        or:[ ((lit isKindOf: SymbolAssociation) and:[ lit _value == anObject]) 
         or:[ lit _isArray and:[ lit includesIdentical: anObject ]]]
      ) ifTrue:[ ^ true ]
    ].
    lastBlkIdx - numBlks + 1 to: lastBlkIdx do:[:n| | aBlk | "recursively search blocks"
      aBlk := self at: n .
      (aBlk method _literalsIncludesValue: anObject) ifTrue:[ ^ true ]
    ].
  ].
  ^ false
%

category: 'Repository Conversion'
method: GsNMethod
_literalVariablesForRecompile
  "Returns an Array of the literal variables referenced in the receiver.
   The result has key,value pairs of  the form  aSymbol,  aSymbolAssociation"
| litVars obsDict |
litVars := { }  .
obsDict := self class _classVars at: #ObsoleteClassesDict
              ifAbsent:[ "handle recompile of default block for SortedCollection in upgrade"
                 SymbolDictionary new
              ].
self literals do: [:aLit |
  | litName newLit srcName lookupKey obsArr clsName litVal |
  aLit _isSymbol ifFalse:[
    (aLit isKindOf: SymbolAssociation ) ifTrue:[
      litName := aLit key . litVal := aLit value
    ] ifFalse:[
      (aLit isClass and:[ aLit isMeta not ]) ifTrue:[ "was optimized ref to a class"
         litName := aLit name . clsName := litName . litVal := aLit
      ].
    ].
    litName ifNotNil:[
      (obsArr := obsDict at: litName otherwise: nil) ifNotNil:[
         (litVal ~~ nil and:[ litVal == (obsArr at:1)]) ifTrue:[
            srcName := obsArr at: 2 .
            lookupKey := obsArr at: 3 .
            newLit := Globals associationAt: lookupKey otherwise: nil .
         ].
       ].
       newLit ifNotNil:[ litVars add: srcName ; add: newLit ]
         ifNil:[
            clsName ifNotNil:[
              (Globals associationAt: clsName otherwise: nil) ifNotNil:[ :assoc|
                assoc _value == aLit ifTrue:[ litVars add: clsName ; add: assoc ]
              ].
            ] ifNil:[ litVars add: litName ; add: aLit "use old assoc"]
        ].
    ] ifNil:[  "an optimized literal that was not a class"
      | assoc |
      assoc := self _literalValToAssocations: aLit .
      assoc _isArray ifTrue:[ assoc do:[:a | litVars add: a key ; add: a ]]
                    ifFalse:[ litVars add: assoc key ; add: assoc ].
    ].
  ].
].
^ litVars
%

category: 'Error Handling'
method: GsNMethod
_methodNeedingRecompilation

"Generate an error for a method that needs to be recompiled.

 During schema modification, this method is copied,
 the sourceString in the copy replaced with the sourceString
 of a method needing recompilation, and the copy then installed
 in the modified class' method dictionary as the value for the
 selector needing recompilation.  See also _copyToForceRecompilation . "

| srcPrefix prefixSize thisMeth srcStr |
thisMeth := GsProcess _methodInFrameContents:(GsProcess _frameContentsAt:1).
srcStr := thisMeth sourceString .
prefixSize := srcStr size min: 40 .
srcPrefix := srcStr copyFrom:1 to: prefixSize .
srcPrefix addAll: '...' .
self _error: #rtErrUncompiledMethod args:{ srcPrefix . thisMeth inClass }.
self _uncontinuableError
%

category: 'Debugging Support'
method: GsNMethod
_meth_ip_ForStepPoint: aStepPoint

"Returns an Array , { aMethod . anInstructionOffset }
 for the specified step point .
 Instruction offsets are absolute byte offsets relative to start of the method.
 Returns nil if aStepPoint is not a legal step point for the receiver"

| allIps |
allIps := self _allDebugInfoWithMeths: 2 .
(aStepPoint >= 1 and:[ aStepPoint <= allIps size]) ifTrue:[
  ^ allIps at: aStepPoint
].
^ nil
%

category: 'Accessing'
method: GsNMethod
_nArgs

"Returns the number of arguments expected by the method."

^ iFields1 bitAnd: NArgs_mask
%

category: 'Debugging Support'
method: GsNMethod
_natIpToPort: aNativeIpOffset
  "For the given offset in native code, interpreted as a return address,
   return the corresponding offset in intepreted code.
  Returns 0 if the receiver does not have native code,
    -1 if the argument is out of range,
    value < -1 if translation not exact. "

  ^ self _nativeIpOffsetToPortable: aNativeIpOffset asReturn: true
%

category: 'Debugging Support'
method: GsNMethod
_nativeIpOffsetToPortable: aNativeIpOffset asReturn: aBoolean

"for the given offset in native code, return the corresponding
 offset in intepreted code.
 If aBoolean == true, result is as an IP to which the callee will return.
 Returns 0 if the receiver does not have native code,
    -1 if the argument is out of range,
    value < -1 if translation not exact
 "
<primitive: 673>
self _primitiveFailed: #_nativeIpOffsetToPortableRtnAddr:asReturn:
     args: { aNativeIpOffset . aBoolean }
%

category: 'Debugging Support'
method: GsNMethod
_nextStepPointForIp: anIp

"Returns the step point which represents
 the step point at or just after the given instruction pointer offset.  If
 anIp is after the last step point, returns an integer one greater than
 the last legal step point.

 The anIp argument is an absolute instruction offset ."
 ^ ( self _debugInfoAccess: 4  at: anIp ) + self _stepPointOffset .
%

category: 'Debugging Support'
method: GsNMethod
_numArgs

"Returns the number of arguments expected by the method."

^ iFields1 bitAnd: NArgs_mask
%

category: 'Debugging Support'
method: GsNMethod
_numArgsTempsCblkargs
"return (number of args + num temps) + copyingBlockIdxSize "
| info1 info2 |
info1 := debugInfo at: DebugInfo_fields1_offset  .
info2 := debugInfo at: DebugInfo_fields2_offset  .
^ ((info1 bitShift: 0 - NumArgsPlusTemps_shift) bitAnd: NumArgsPlusTemps_mask) +
  ((info2 bitShift: 0 - CopyingBlockIdxSize_shift) bitAnd: CopyingBlockIdxSize_mask)
%

category: 'Debugging Support'
method: GsNMethod
_numBlockLiterals

^ (debugInfo at: DebugInfo_Fields2_offset) bitAnd: NumBlockLiterals_mask
%

category: 'Debugging Support'
method: GsNMethod
_numCopyingBlockArgs

^ 0
" maglev only:
^ ((debugInfo at: DebugInfo_fields2_offset) bitShift: 0 - CopyingBlockIdxSize_shift)
     bitAnd: CopyingBlockIdxSize_mask
"
%

category: 'Debugging Support'
method: GsNMethod
_numIpSteps

^ ((debugInfo at: DebugInfo_fields1_offset) bitShift: 0 - NumIpSteps_shift)
 	bitAnd: NumIpSteps_mask
%

category: 'Debugging Support'
method: GsNMethod
_numSourceOffsets

^ self _numIpSteps
%

category: 'Debugging Support'
method: GsNMethod
_opcodeAtIsCallPrimitive: ipOffset

" ipOffset is obtained from result of GsProcess >> _frameContentsAt:"

^ ((self _opcodeKindAt: ipOffset abs ) bitAnd: 16r1) ~~ 0
%

category: 'Debugging Support'
method: GsNMethod
_opcodeAtIsSend: ipOffset

" ipOffset is obtained from result of GsProcess >> _frameContentsAt:"

^ ((self _opcodeKindAt: ipOffset abs ) bitAnd: 16r4) ~~ 0
%

category: 'Disassembly'
method: GsNMethod
_opcodeInfoAt: ipOffset

"ipOffset must be a valid absolute byte offset to an instruction in the receiver.
 If the specified instruction represents a currently installed breakpoint,
 the result is for the instruction that the breakpoint is replacing.

 returns a SmallInteger with the following bit fields
           16rFF  instruction size in bytes , including the opcode word and any
	  	  in-line literal words , an unsigned 8-bit int
       16rFFFF00  instVar offset for instVarAccess
      16r1000000  boolean, 1 means opcode is an instVarAccess
      16r2000000  boolean, 1 means opcode is a pushBlock
   16rFFF0000000  opcode
 "
<primitive: 670>
self _primitiveFailed: #_opcodeInfoAt: args: { ipOffset } .
^ 1
%

category: 'Private'
method: GsNMethod
_opcodeKindAt: ipOffset

"primitive will fail if ipOffset is out of range .
 argument ipOffset is zero based  and must be positive.

 result contains these bits
   16r1 if instruction at ipOffset is a call to a primitive,
   16r2 if instruction requires protected mode
   16r4 if instruction is a send opcode."
<primitive: 189>
self _primitiveFailed: #_opcodeKindAt: args: { ipOffset }
%

category: 'Pragmas'
method: GsNMethod
_pragmasArray
  "Return the raw pragmas array for a method."

  (self _debugInfoAccess: 11 at: -1) ifNotNil:[:parray | ^ parray ].
  ^ #()
%

category: 'Debugging Support'
method: GsNMethod
_previousStepPointForIp: anIp

"Returns the step point which represents
 the step point preceding the given instruction pointer offset.

 The anIp argument is an absolute instruction offset .
 The argument isQuick is ignored."

^ ( self _debugInfoAccess: 3  at: anIp ) + self _stepPointOffset .
%

category: 'Accessing'
method: GsNMethod
_primitiveAt: anIndex

"Returns the value of an instance variable in the receiver.
 The argument anIndex must be a SmallInteger   >= 1 and
 < (named size plus varying size) of receiver.

 Attempt to fetch from the send-site caches area of a method that
 has been loaded for execution will return nil.
 Use GsNMethod>>literals to get the complete literals pool.
"

<primitive: 806>
anIndex _validateClass: SmallInteger .
self _errorIndexOutOfRange: anIndex .
self _primitiveFailed: #_primitiveAt:  args: { anIndex }
%

category: 'CodeModification Override'
method: GsNMethod
_primitiveAt: anIndex put: aValue

self shouldNotImplement:#_primitiveAt:put:
%

category: 'Debugging Support'
method: GsNMethod
_primitiveNumber

"Return the primitive number of the method or -1 if the method
 is not a primitive."
| firstOfs |
firstOfs := GSNMETHOD_FIRST_INSTR_OFFSET .
(self _opcodeAtIsCallPrimitive: firstOfs) ifTrue:[
 ^ ((self _opcodeInfoAt: firstOfs) bitShift: -40) bitAnd: 16rFFF
].
^ -1
%

category: 'Accessing'
method: GsNMethod
_primitiveSize
"Returns total the number of instance variables in the receiver.
 For a GsNMethod in code_gen memory, the size includes the
 words in the send_cache area, and will be larger than the
 on-disk size by  (2*self numSends) ."

<primitive: 805>
self _primitiveFailed: #_primitiveSize .
self _uncontinuableError
%

category: 'Private'
method: GsNMethod
_recompile
  "Unconditionally recompile the receiver."
  ^ self recompileIntoMethodDict: nil intoCategories: nil .
%

category: 'Debugging Support'
method: GsNMethod
_recompileWithSource: aString

"Recompiles the receiver using the new source string aString.  If the
 compilation has no errors, installs a new method in the receiver's class's
 method dictionary, and returns true, otherwise signals a CompileError .
 May signal a CompileWarning after installing a new method.

 No check is made as to whether the new method has the
 same selector as the receiver.

 The environmentId of the receiver is preserved.

 Raises an error and does not recompile if the receiver is an anonymous method."
| env |
env := self environmentId .
self _validatePrivilege ifFalse:[ ^ nil ].
self isMethodForBlock ifTrue:[
  self _halt: 'Attempt to recompile method for a block. You must recompile the home method.' .
  ^ nil
].
inClass ifNil:[
  self _halt: 'Attempt to recompile anonymous method.' .
  ^ nil
].
inClass compileMethod: aString
        dictionaries: GsCurrentSession currentSession symbolList
	      category: (inClass categoryOfSelector: self selector environmentId: env)
        environmentId: env .
^ true
%

category: 'Class organizer support'
method: GsNMethod
_referencedStringsInto: aSet
  "Add all literal Strings of receiver to aSet. Returns receiver."
  | litOfs lastBlkIdx numBlks |
  (litOfs := self literalsOffset) ~~ 0 ifTrue:[
    lastBlkIdx := self _lastBlockLiteralOffset .
    numBlks := self _numBlockLiterals .
    litOfs to: (lastBlkIdx - numBlks) do:[:k| | lit |
      lit := self at: k.
      lit _isSymbol ifFalse:[
        (lit isKindOf: CharacterCollection) ifTrue:[
          aSet add: lit
        ].
      ].
    ].
    lastBlkIdx - numBlks + 1 to: lastBlkIdx do:[:n| | aBlk | "recursively search blocks"
      aBlk := self at: n .
      aBlk method _referencedStringsInto: aSet
    ].
  ].
%

category: 'Deprecated'
method: GsNMethod
_removeAllSourceButFirstComment

self deprecated: '_removeAllSourceButFirstComment obsolete, Use #removeAllSourceButFirstComment'.
^ self removeAllSourceButFirstComment
%

category: 'Private'
method: GsNMethod
_rowanPackageInfo
  "Returns nil or whatever Rowan stored "
  self _debugInfoFileInfo == 1 ifTrue:[
    ^ debugInfo at: DEBUGINFO_packageInfo 
  ].
  ^ nil  
%

category: 'Private'
method: GsNMethod
_rowanPackageInfo: aValue
  self _debugInfoFileInfo == 1 ifFalse:[
    self isMethodForBlock ifTrue:[
      Error signal:'Cannot store package info into a GsNMethod for a block'.
    ].
    Error signal:'Method must be recompiled to allocate slot for package info'. 
  ].
  debugInfo _unsafeAt: DEBUGINFO_packageInfo put: aValue . "unsafeAt to override invariance"
%

category: 'GBS compatibility'
method: GsNMethod
_selector

^ self selector
%

category: 'Reporting'
method: GsNMethod
_selectorPool

"Returns an IdentitySet containing the selectors sent by the receiver.
 The result includes selectors sent by inner blocks of the receiver."

^ self _allDebugInfo: 7
%

category: 'Debugging Support'
method: GsNMethod
_setAllStepBreaks: includeMethodStartBcs frame: fpOffset process: aGsProcess

"includeMethodStartBcs is a Boolean, true means include setting of breakpoints
 on method-entry bytecodes which check the interruptWord ."

| opcFlags |
opcFlags := 1 .
includeMethodStartBcs ifTrue:[ opcFlags := 16r101 ].
self _allBreaksOp: opcFlags frame: fpOffset process: aGsProcess .
%

category: 'Debugging Support'
method: GsNMethod
_setBreakpointsInSourceOffsets: sourceOffsets

"Given all the source offsets for the receiver, replace with nil those
 that do not correspond to a breakpoint.  Negate those which correspond
 to a disabled breakpoint."

| allBreaks |

allBreaks := self _allBreakpointsSourceOffsets .
allBreaks == nil ifTrue:[ ^ sourceOffsets size: 0 ].

1 to: sourceOffsets size do:[:j | | aStepPoint  |
   aStepPoint := sourceOffsets at: j .  "aStepPoint is a source offset"
   (allBreaks indexOf: aStepPoint) = 0 ifTrue:[ | negStepPoint |
     negStepPoint := aStepPoint negated .
     (allBreaks indexOf: negStepPoint) = 0 ifTrue:[
       sourceOffsets at: j put: nil
     ] ifFalse:[
       sourceOffsets at: j put: negStepPoint
     ].
   ]
].
^ self
%

category: 'Debugging Support'
method: GsNMethod
_setStepOverBreaks: includeMethodStartBcs frame: fpOffset process: aGsProcess

^ self _setStepOverBreaks: includeMethodStartBcs frame: fpOffset process: aGsProcess
       stepThrough: false
%

category: 'Debugging Support'
method: GsNMethod
_setStepOverBreaks: includeMethodStartBcs frame: fpOffset process: aGsProcess
   stepThrough: thruBool
| opcFlags |
opcFlags := 1 .
includeMethodStartBcs ifTrue:[ opcFlags := 16r101 ].
self _setBreakAtIp: -1 operation: opcFlags frame: fpOffset process: aGsProcess .
thruBool ifTrue:[ | blks |
  blks := self homeMethod blockLiterals .
  blks do:[ :aBlk | | meth |
      meth := aBlk method .
      meth ~~ self ifTrue:[
        meth _setBreakAtIp: -1 operation: 1 frame: nil process: aGsProcess
      ].
  ].
].
%

category: 'Debugging Support'
method: GsNMethod
_sourceAtIp: anIp

"Return the source string with marker for step point closest to
 the specified IP .
 Assumes that the IP is from a frame that is Not at top of Stack."

| aStep |
aStep := self _previousStepPointForIp: anIp .
^ self _sourceWithSteps: aStep
%

category: 'Debugging Support'
method: GsNMethod
_sourceAtTosIp: anIp

"Return the source string with marker for step point closest to
 the specified IP .
 Assumes that the IP is from a frame that IS at top of Stack."

| aStep |
aStep := self _nextStepPointForIp: anIp .
^ self _sourceWithSteps: aStep
%

category: 'Accessing'
method: GsNMethod
_sourceFileLineComment
  "Smalltalk methods are not expected to not have file/line info"
  ^ ''
%

category: 'Reporting'
method: GsNMethod
_sourceOffsetOfFirstSendOf: aSymbol

"Returns the source offset of the step point for the first send in the receiver
 that sends aSymbol.  If the receiver is not a sender of aSymbol, or if aSymbol
 is not a Symbol, returns nil."

| ofs blks |
ofs := self _debugInfoAccess: 5  at: aSymbol .
ofs ~~ nil ifTrue:[ ^ ofs ].
blks := self blockLiterals .
blks do:[ :aBlk | | blkOfs |
    blkOfs := aBlk method _debugInfoAccess: 5  at: aSymbol .
    blkOfs ~~ nil ifTrue:[ ^ blkOfs ].
].
^ nil .
%

category: 'Debugging Support'
method: GsNMethod
_sourceOffsets

"Returns an Array of SmallIntegers describing all the step points of
 the receiver and any blocks.
 Each element of the result is the source offset of a step point.
 The result is not sorted by source offset."

| res |
res := self _allDebugInfo: 1 .
res size == 0 ifTrue:[ ^ #( 1) "a primitive method" ].
^ res
%

category: 'Debugging Support'
method: GsNMethod
_sourceOffsetsAt: aStepPoint

"Returns the source offset for the step point with number aStepPoint.
 Returns beginning or ending source offset for a step point out of range.

 Step points are expressed as source offsets (Gs64 v3.0 and above). "
 | ofs max stp |
 aStepPoint < 1 ifTrue:[ ^ 1 ].
 stp := aStepPoint .
 self isMethodForBlock ifTrue:[ stp := stp - self _stepPointOffset ].
 ofs := self _debugInfoAccess: 1 at: stp .
 ofs ifNil:[ ^ 1 ].
 max := self sourceString size .
 ofs > max ifTrue:[  ofs := max ].
 ^ ofs
%

category: 'Debugging Support'
method: GsNMethod
_sourceOffsetsOfSends

"Returns an Array,  { anOffset, aSelectorSymbol, ... anOffset, aSelectorSymbol}
 containing source offsets and selectors of all of the sends in the receiver.
 The results are not sorted by source offset."

^ self _allDebugInfo: 10
%

category: 'Accessing'
method: GsNMethod
_sourceStartForBlock

"Return the 1-based offset in the home methods source of
 the start of the block."
| word |
self isMethodForBlock ifFalse:[ ^ 1].
word := debugInfo at: DebugInfo_FirstLastSrcOffset_offset .
word < 0 ifTrue:[ ^ 1 "ruby line numbers no longer expected"].

^ word bitAnd: 16r3fffffff "firstOfs"
%

category: 'Accessing'
method: GsNMethod
_sourceString

"Returns sourceString."

^  self sourceString
%

category: 'Accessing'
method: GsNMethod
_sourceStringForBlock

"Return the portion of the home method's source for this block"

| src word firstOfs lastOfs isLineNum |

self isMethodForBlock ifFalse:[ ^ nil ].
src := debugInfo at: DebugInfo_source_offset .
word := debugInfo at: DebugInfo_FirstLastSrcOffset_offset .
isLineNum := false .
word < 0 ifTrue:[ isLineNum := true.  word := 0 - word ].
firstOfs := word bitAnd: 16r3fffffff .
lastOfs := word bitShift: -30 .
isLineNum ifTrue:[ | lines res lf sz | "ruby line numbers"
  lines := src subStrings: (lf := Character lf) .
  res := String new .
  firstOfs to: (lastOfs min: lines size) do:[:n | res addAll: (lines at: n); add: lf ].
  res addAll:'...'; add: lf .
  lines := self homeMethod sourceString subStrings: lf .
  sz := lines size .
  (lines at: sz) size == 0 ifTrue:[ sz := sz - 1 ].
  res add: (lines at: sz) ; add: lf . "append comment 'starts at line N of file...' "
  ^ res
] ifFalse:[
  ^ src copyFrom:firstOfs to: lastOfs .
]
%

category: 'Accessing'
method: GsNMethod
_sourceStringWithFileName
 "Return source string with file name and starting line number appended.
  Ruby methods usually have a file and line; Smalltalk methods do not.
  Used by topaz."

  | sStr |
  sStr := self sourceString .
  ^ [ | res flStr |
      res := sStr .
      (flStr := self _sourceFileLineComment) size > 0 ifTrue:[ | lf |
	lf := Character lf .
	res := res copy .
	(res at: res size) == lf ifFalse:[ res add: lf ].
	res add: flStr .
      ].
      res
    ] onException: Error do:[:ex | sStr ]
%

category: 'Accessing'
method: GsNMethod
_sourceStringWithLineNumbers
 "Return source string with lines prefixed with line numbers.
  Used by topaz."
 | src res n nStr idx lf prev |
 src := self sourceString .
 res := src class new .
 lf := Character lf .
 n := 1 .
 [ nStr := n asString .
   (3 - nStr size) timesRepeat:[ res add: $  ].
   res add: nStr .
   2 timesRepeat:[ res add: $  ].
   n := n + 1 .
   idx ifNil:[ prev := 1 ] ifNotNil:[ prev := idx + 1 ].
   idx := src indexOf: lf startingAt: prev .
   idx == 0 ifTrue:[ res add: (src copyFrom: prev to: src size) ]
         ifFalse:[ res add: (src copyFrom: prev to: idx) ].
   idx == src size ifTrue:[ idx := 0 ].
   idx == 0
 ] untilTrue .
 ^ res
%

category: 'Deprecated'
method: GsNMethod
_sourceToFirstComment

self deprecated: '_sourceToFirstComment obsolete, Use #sourceToFirstComment'.
^ self sourceToFirstComment
%

category: 'Debugging Support'
method: GsNMethod
_sourceWithStepIps

^ self _sourceWithStepIps: 8
%

category: 'Debugging Support'
method: GsNMethod
_sourceWithStepIps: tabSize

"This method returns the source string with intermixed control information
 indicating IP values of each step point.

 Result shows the step points for the receiver only, not any inner blocks.
 "

| markerArray "A parallel Array to the source string.  It is filled with nils
               except for locations pointed to by the sourceOffsets Array."
  aSize        "the size of the source"
  srcStr result |

srcStr := self sourceString .
aSize:= srcStr size.
markerArray := self _buildIpMarkerArray .
result := GsNMethod _buildMarkedSourceFrom: srcStr
                             sourceSize: aSize
                                markers: markerArray tabSize: tabSize .
^ result
%

category: 'Debugging Support'
method: GsNMethod
_sourceWithSteps: allSteps

  ^ self _sourceWithSteps: allSteps tabSize: 8
%

category: 'Debugging Support'
method: GsNMethod
_sourceWithSteps: allSteps tabSize: tabSize

^ self _sourceWithSteps: allSteps tabSize: tabSize window: 0
%

category: 'Debugging Support'
method: GsNMethod
_sourceWithSteps: allSteps tabSize: tabSize window: winSize
"This method returns the source string with intermixed control information
 indicating where step points are.

 allSteps == true , show all steps
          == false, show steps where a breakpoint currently exists
          a SmallInteger, show just that step point

 winSize == 0 means list entire method ,
         non zero and  allSteps is a SmallInteger causes specified
         number of lines to be listed, centered on specified step point,
         plus the fileName/lineNumber comment line if present.
 "
| markerArray "A parallel Array to the source string.  It is filled with nils
               except for locations pointed to by the sourceOffsets Array."
  srcSize       "the size of the source"
  srcStr fullResult res |

srcStr := winSize == 0 ifTrue:[ self _sourceStringWithFileName ]
                       ifFalse:[ self sourceString ].
srcSize:= srcStr size.
markerArray := self _buildMarkerArray: allSteps ofSize: srcSize.
fullResult := GsNMethod _buildMarkedSourceFrom: srcStr
                             sourceSize: srcSize markers: markerArray tabSize: tabSize .
res := fullResult .
(winSize > 0 and:[ allSteps _isSmallInteger]) ifTrue:[
   | targLine lineCnt lines lf halfWin |
  targLine := 0 .   halfWin := winSize // 2 .
  lf := Character lf .
  lineCnt := 0 .
  1 to: srcSize do:[ :n |
    ((markerArray at: n) ~~ nil and:[ targLine == 0]) ifTrue:[ targLine := lineCnt + 1 ].
    (srcStr at: n)  == lf ifTrue:[  lineCnt := lineCnt + 1 ].
  ].
  lines := fullResult subStrings: lf .
  lineCnt := lines size .
  (lineCnt > winSize and:[ lineCnt > 10]) ifTrue:[
    | start end |
    res := fullResult class new .
    start := (targLine - halfWin) max: 1 .
    end := (targLine + halfWin) min: lineCnt .
    start > 2 ifTrue:[
      1 to: 2 do:[:k | res addAll: (lines at: k); add: lf  ].
      res addAll:'...'; add: lf .
    ].
    start to: end do:[:k | res addAll: (lines at: k) ; add: lf ].
    end < (lineCnt - 2) ifTrue:[
      res addAll:'...'; add: lf .
    ].
  ]
].
winSize ~~ 0 ifTrue:[
  res add: self _sourceFileLineComment .
].
^ res
%

category: 'Accessing'
method: GsNMethod
_stackBase

"Private."

self error:'GsNMethod>>_stackBase  not implemented'.
%

category: 'Debugging Support'
method: GsNMethod
_stepPointForIp: ipOffset level: aLevel useNext: nextBool

"Result is step point for specified instruction offset and level in a stack
 ipOffset is an absolute instruction offset ."

aLevel = 1 ifTrue:[  "top of stack"
   ^ self _nextStepPointForIp: ipOffset
 ] ifFalse:[
   nextBool ifTrue:[
     ^ self _nextStepPointForIp: ipOffset
   ] ifFalse:[
     ^ self _previousStepPointForIp: ipOffset
   ].
 ]
%

category: 'Debugging Support'
method: GsNMethod
_stepPointForMeth: aMethod ip: ipOffset
"Given a aMethod which may be self or method for an inner block of self,
 and anIp , an ipOffset within aMethod (possibly negated),
 Return the stepPointer number within self, or nil ."
| absIp allIps  |
absIp := ipOffset abs .
allIps := self _allDebugInfoWithMeths: 2 .
1 to: allIps size do:[:j | | anIpArr|
  anIpArr :=  allIps at: j .
  ((anIpArr at: 1) == aMethod and:[ (anIpArr at: 2) = absIp]) ifTrue:[
     ipOffset > 0 ifTrue:[ ^ j ] ifFalse:[ ^ j negated ].
  ].
].
^ nil
%

category: 'Debugging Support'
method: GsNMethod
_stepPointOffset
  "If receiver is method for a block, return the value to add to a steppoint
   for the block to obtain the steppoint as viewed from the home method,
   otherwise return zero."

  self isMethodForBlock ifTrue:[ | offsets |
    offsets := self _blockSourceOffsets .
    1 to: offsets size do:[:n | (offsets at: n) ifNotNil:[ ^ n - 1]].
    "Error signal:'step point not found in _stepPointOffset'." "for debugging"
  ].
  ^ 0
%

category: 'CodeModification Override'
method: GsNMethod
_unsafeAt: anIndex put: aValue

self shouldNotImplement:#_unsafeAt:put:
%

category: 'CodeModification Override'
method: GsNMethod
_validatePrivilege

^ System myUserProfile _validateCodeModificationPrivilege
%

category: 'Debugging Support'
method: GsNMethod
__allBreakpoints

"Returns an Array of the form

       { breakpointNumber1 . self . ipOffset1 .
          ...
          breakpointNumberN . self . ipOffsetN .
       }

 The ipOffsets are instruction offsets.
 Negative ipOffsets denote disabled breakpoints.

 Returns nil if no method breakpoints set in the receiver.
 Single step breakpoints managed by GciStep() are not reported."

<primitive: 193>
self _primitiveFailed: #__allBreakpoints
%

category: 'Debugging Support'
method: GsNMethod
__setBreakAtIp: ipOffset operation: opcFlags frame: fpOffset process: aGsProcess

"Set breakpoint at specified ipOffset.  ipOffset is an instruction offset,
 which is a absolute byte offset within the receiver.

 opcFlags is a SmallInteger , fields have these masks
    opcode 		   16rFF
    includeMethodStartBcs 16r100 , a Boolean , only affects opcode==1
        when ipOffset == -1 .

 opcode  action
   0	 set or reenable method breakpoint
   1     set single step breakpoint (takes precedence over method break)
   2     delete method breakpoint or disabled method breakpoint
   3     delete single step breakpoint
   4     disable method breakpoint, no action if breakpoint not set

  If ipOffset == -1, then apply the action to all step points within the
  method.

  For opcode 1 only, to restrict breakpoint to a specific frame
  and/or GsProcess , fpOffset and/or aGsProcess must be non-nil. "

<primitive: 190>
self _primitiveFailed: #__setBreakAtIp:operation:frame:process:
	args: { ipOffset . opcFlags . fpOffset . aGsProcess }
%

! Class extensions for 'GsObjectSecurityPolicy'

!		Class methods for 'GsObjectSecurityPolicy'

removeallmethods GsObjectSecurityPolicy
removeallclassmethods GsObjectSecurityPolicy

category: 'Instance Creation'
classmethod: GsObjectSecurityPolicy
newInRepository: aRepository

"Returns a new GsObjectSecurityPolicy in the Repository aRepository.  If the
 maximum number (65534) of ObjectSecurityPolicies has already been created for
 aRepository, this generates an error.
 The new GsObjectSecurityPolicy is owned by the UserProfile of the current session,
 and has the default authorization of W----- (owner can read and write).

 After execution of this method, the current transaction must be committed
 before objects can be created in the GsObjectSecurityPolicy, object assigned to the
 objectSecurityPolicy, or the GsObjectSecurityPolicy used as argument to a
 defaultObjectSecurityPolicy: keyword in creation of UserProfiles.

 Requires the ObjectSecurityPolicyCreation privilege and WriteAccess to
 DataCuratorObjectSecurityPolicy.
 "

<primitive: 291>
aRepository _validateClass: Repository.
self _primitiveFailed: #newInRepository: args: { aRepository } .
self _uncontinuableError
%

category: 'Execution'
classmethod: GsObjectSecurityPolicy
setCurrent: anObjectSecurityPolicy while: aBlock

| oldPolicy |

oldPolicy := System currentObjectSecurityPolicy.
oldPolicy == anObjectSecurityPolicy ifTrue: [ "no need to change/restore policy "
  ^ aBlock value
].
System currentObjectSecurityPolicy: anObjectSecurityPolicy .
^ [
    [
      aBlock value
    ] onException: Error do:[:ex |
      (oldPolicy == nil or:[ oldPolicy isKindOf: GsObjectSecurityPolicy]) ifTrue: [
        System currentObjectSecurityPolicy: oldPolicy .
      ].
      ex outer
    ]
  ] ensure:[
    System currentObjectSecurityPolicy: oldPolicy
  ].
%

!		Instance methods for 'GsObjectSecurityPolicy'

category: 'Formatting'
method: GsObjectSecurityPolicy
asString

"Returns a formatted String that contains the following information:

 The name of the Repository containing the receiver and the index of
 the receiver in that Repository.

 * The user ID of the receiver's owner.

 * The Symbol that defines each user group which is authorized to read or
   write in the receiver.

 * Information about whether the receiver's owner, each group, or all
   users are authorized to read or write in the receiver.

 For example, the message 'System myUserProfile defaultObjectSecurityPolicy asString'
 returns a String that resembles the following:

 anObjectSecurityPolicy, Number 1 in Repository SystemRepository, Owner SystemUser write, World read

 If the receiver is not a GsObjectSecurityPolicy in the SystemRepository, the String contains
 'NOT IN REPOSITORY' in place of the index."

| result theGroupAuthString |
theGroupAuthString := String new.  " string of group authorizations"
"groups and build a single string to describe group authorizations."
groupsRead size ~~ 0 ifTrue:[
  (SortedCollection withAll: groupsRead ) do: [:groupName |
     theGroupAuthString addAll: ' Group '; addAll: (groupName asString);
                 add: ' '; addAll: 'read' ; add: $, .
  ].
].
groupsWrite size ~~ 0 ifTrue:[
  (SortedCollection withAll: groupsWrite ) do: [:groupName |
     theGroupAuthString addAll: ' Group '; addAll: (groupName asString);
        add: ' '; addAll: 'write' ; add: $, .
  ].
].
result := String new.
self _name ifNotNil:[:nam| result addAll: nam ]
           ifNil:[ result addAll: 'anObjectSecurityPolicy'].
result addAll: '(#';
       addAll: self objectSecurityPolicyId  asString ;
       addAll: ' in Repository ';
       addAll: self repository name ; add: $, ;
       addAll: ' Owner ';
       addAll: self owner userId ;   add: $   ;
       addAll: self ownerAuthorization ; add: $, ;
       addAll: theGroupAuthString;
       addAll: ' World ';
       addAll: self worldAuthorization .
self trackReads ifTrue:[ result addAll:', trackReads' ].
result  add: $) .
^ result
%

category: 'Accessing Authorization'
method: GsObjectSecurityPolicy
authorizationForGroup: groupOrString

"Returns a Symbol that defines whether the specified group is authorized to
 write (and read) in this GsObjectSecurityPolicy (#write), to read only
 (#read), or neither (#none)."

| theGroupObj |

"Find the UserProfileGroup object and handle the case where it was passed to us
 instead of the name of the group."
theGroupObj := (groupOrString class == UserProfileGroup)
  ifTrue: [ groupOrString ]
  ifFalse: [ UserProfileGroup groupWithName: groupOrString otherwise: nil ].
theGroupObj == nil ifTrue:[ ^ #none ].

groupsWrite size ~~ 0 ifTrue:[
  (groupsWrite includesIdentical: theGroupObj ) ifTrue:[ ^ #write ]
  ].
groupsRead size ~~ 0 ifTrue:[
  (groupsRead includesIdentical: theGroupObj) ifTrue:[ ^ #read ]
  ].
^ #none
%

category: 'Accessing Authorization'
method: GsObjectSecurityPolicy
authorizationForUser: aUserProfile

"Returns a Symbol that describes the authorization that the given user has for
 the receiver."

| userGroups curAuth |

"Consider world authorization"
curAuth := worldAuthorization.
curAuth == 2 ifTrue: [ ^ #write ].

"Owner authorization applicable?"
itsOwner == aUserProfile ifTrue: [
  ownerAuthorization == 2 ifTrue:[ ^ #write ].
  curAuth := curAuth bitOr: ownerAuthorization .
  ].

"Consider group authorizations..."
userGroups := aUserProfile groups .
userGroups size ~~ 0 ifTrue:[
  groupsWrite size ~~ 0 ifTrue:[
    (groupsWrite * userGroups) size ~~ 0 ifTrue:[ ^ #write ].
    ].
  groupsRead size ~~ 0 ifTrue:[
    (groupsRead * userGroups) size ~~ 0 ifTrue:[ curAuth := curAuth bitOr: 1 ].
    ].
  ].

curAuth == 1 ifTrue:[ ^ #read ].
^#none
%

category: 'Clustering'
method: GsObjectSecurityPolicy
clusterDepthFirst

"This method clusters the receiver.  (Overrides the inherited method, which
 also clusters all instance variables.)  Returns true if the receiver has
 already been clustered during the current transaction, false otherwise."

  self cluster ifTrue:[ ^ true ].
  groupsRead cluster .
  groupsWrite cluster .
^ false
%

category: 'Accessing Authorization'
method: GsObjectSecurityPolicy
currentUserCanRead

"Returns true if the current user has read authorization for the
 receiver, false if not."

^ self _currentUserAccess >= 1
%

category: 'Accessing Authorization'
method: GsObjectSecurityPolicy
currentUserCanWrite

"Returns true if the current user has write authorization for the
 receiver, false if not."

^ self _currentUserAccess == 2
%

category: 'Updating Authorization'
method: GsObjectSecurityPolicy
group: groupOrString authorization: anAuthorizationSymbol

"Redefines the authorization for the specified group to one of the following
 authorization Symbols:

 * #write (members of the group can both read and write in this GsObjectSecurityPolicy).
 * #read (read only).
 * #none (neither read nor write permission).

 This method generates an error if groupOrString is not either a UserProfileGroup or the
 name of a UserProfileGroup in global AllGroups, or if anAuthorizationSymbol is not one
 of (#read, #write, #none).

 Requires the ObjectSecurityPolicyProtection privilege, if the objectSecurityPolicy is not owned
 by the GemStone UserProfile under which this session is running."

| theGroupObj |

"Test for errInvalidAuthorization "
( #( #write #read #none) includesValue: anAuthorizationSymbol)
ifFalse: [  ^ self _error: #segErrBadAuthKind args: { anAuthorizationSymbol } ].

"Find the UserProfileGroup object and handle the case where it was passed to us
 instead of the name of the group."
theGroupObj := (groupOrString class == UserProfileGroup)
  ifTrue: [ groupOrString ]
  ifFalse: [ UserProfileGroup _validateGroupString: groupOrString ].

"modify the group collections for this objectSecurityPolicy as appropriate."
anAuthorizationSymbol == #write ifTrue:[
  self _authAt: -3 put: theGroupObj "readGroups remove: theGroupObj ifAbsent:[]" .
  self _authAt: 4 put: theGroupObj  "writeGroups _addSymbol: theGroupObj " .
  ]
ifFalse:[
  anAuthorizationSymbol == #read ifTrue:[
    self _authAt: -4 put: theGroupObj "writeGroups remove: theGroupObj ifAbsent:[]" .
    self _authAt: 3 put: theGroupObj  "readGroups _addSymbol: theGroupObj " .
    ]
  ifFalse:[ "none"
    self _authAt: -3 put: theGroupObj "readGroups remove: theGroupObj ifAbsent:[]" .
    self _authAt: -4 put: theGroupObj "writeGroups remove: theGroupObj ifAbsent:[]" .
    ].
  ].
%

category: 'Deprecated'
method: GsObjectSecurityPolicy
groupNo: groupIndex group: aGroupString authorization: anAuthorizationSymbol

self deprecated: 'GsObjectSecurityPolicy>>groupNo:group:authorization: obsolete, deprecated v3.0. Use #group:authorization:.'.
^ self group: aGroupString authorization: anAuthorizationSymbol
%

category: 'Accessing'
method: GsObjectSecurityPolicy
groups

"Returns an IdentitySet, the set of UserProfileGroups that are
 explicitly authorized to read or write in this GsObjectSecurityPolicy."

| result |
groupsRead ~~ nil ifTrue:[ result := groupsRead ]
                 ifFalse:[ result := IdentitySet new ].
groupsWrite ~~ nil ifTrue:[ result := result + groupsWrite ].
^ result
%

category: 'Accessing Authorization'
method: GsObjectSecurityPolicy
groupsWithAuthorization: anAccessSymbol

"Returns an IdentitySet of UserProfileGroups with the authorization of
 anAccessSymbol (one of #write, #read, or #none) for the receiver."

| result |

( #( #write #read #none) includesValue: anAccessSymbol)
  ifFalse: [  ^ self _error: #segErrBadAuthKind args: { anAccessSymbol } ].

result := IdentitySet new .
AllGroups valuesDo:[ :aGroup |
  (self authorizationForGroup: aGroup groupName) == anAccessSymbol ifTrue:[
    result add: aGroup
    ].
  ].
^ result
%

category: 'Accessing'
method: GsObjectSecurityPolicy
name
 | n |
 n := self dynamicInstVarAt: #name.
 n ifNil:[
    itsId == 4 ifTrue:[ ^ 'GsIndexingObjectSecurityPolicy' ].
    itsId == 5 ifTrue:[ ^ 'SecurityDataObjectSecurityPolicy' ].
    ^ 'ObjectSecurityPolicy' , itsId asString
 ].
 ^ n
%

category: 'Updating'
method: GsObjectSecurityPolicy
name: aString
  self dynamicInstVarAt: #name put: aString
%

category: 'Accessing'
method: GsObjectSecurityPolicy
number

"Returns the index of the receiver in its Repository"

^ itsId
%

category: 'Accessing'
method: GsObjectSecurityPolicy
objectSecurityPolicyId

"Returns the index of the receiver in its Repository"

^ itsId
%

category: 'Accessing'
method: GsObjectSecurityPolicy
owner

"Returns the UserProfile of the receiver's owner."

^ itsOwner
%

category: 'Updating'
method: GsObjectSecurityPolicy
owner: aUserProfile

"Redefines the receiver's owner to be the user represented by aUserProfile.
 To execute this method, you must be authorized to write in the receiver's
 GsObjectSecurityPolicy (customarily, the DataCurator GsObjectSecurityPolicy)."

itsOwner := aUserProfile
%

category: 'Accessing Authorization'
method: GsObjectSecurityPolicy
ownerAuthorization

"Returns a Symbol that defines whether the GsObjectSecurityPolicy's owner is authorized to
 write (and read) in this GsObjectSecurityPolicy (#write), to read only (#read), or neither
 (#none)."

^ #( #none #read #write) at: ownerAuthorization + 1
%

category: 'Updating Authorization'
method: GsObjectSecurityPolicy
ownerAuthorization: anAuthorizationSymbol

"Redefines the authorization for the GsObjectSecurityPolicy's owner to one of the following
 authorization Symbols:

 * #write (the GsObjectSecurityPolicy's owner can both read and write in this GsObjectSecurityPolicy).

 * #read (read only).

 * #none (neither read nor write permission).

 Generates an error if anAuthorizationSymbol is not one of (#read, #write,
 #none).

 Requires the ObjectSecurityPolicyProtection privilege, if the objectSecurityPolicy is not owned
 by the GemStone UserProfile under which this session is running."

 | authInt |
 authInt := #( #none #read #write ) indexOf: anAuthorizationSymbol .
 authInt == 0 ifTrue:[
   ^ self _error: #segErrBadAuthKind args: { anAuthorizationSymbol }
   ].
 self _authAt: 5 put: (authInt - 1)
%

category: 'Accessing'
method: GsObjectSecurityPolicy
repository

"Returns the Repository containing the receiver."

^ itsRepository
%

category: 'Deprecated'
method: GsObjectSecurityPolicy
segmentId

self deprecated: 'GsObjectSecurityPolicy>>segmentId is deprecated v3.0 and provided for backwards compatibility.
Use #objectSecurityPolicyId instead.'.

^self objectSecurityPolicyId
%

category: 'Execution'
method: GsObjectSecurityPolicy
setCurrentWhile: aBlock

"Sets the receiver to be the current security policy while the given block
 executes.  Catches all errors and reinstalls the current security policy
 to avoid having the receiver be left as the current security policy.  Returns
 the result of evaluating the zero-argument block."

^ self class setCurrent: self while: aBlock
%

category: 'Accessing'
method: GsObjectSecurityPolicy
trackReads
 "Returns true if read tracking is enabled for objects in this security policy.
 Read tracking occurs when an object is faulted into temporary object memory of a session.
 See also DisableObjectReadLogging privilege in UserProfile,
 DynamicDisableObjectReadLogging privilege in UserProfile,
 System class >> setObjectReadTracking: , System class >> objectReadTrackingEnabled,
 Object >> trackRead, and stone config items STN_OBJECT_READ_LOG*  ."

 ^ trackReads == true
%

category: 'Updating'
method: GsObjectSecurityPolicy
trackReads: aBoolean 
"If aBoolean == true, enables read tracking for objects in this security policy.
 If aBoolean == false, disables read tracking for objects in this security policy.

 If aBoolean == true, an error is signalled if any element of AllUsers 
 has a userId not compatible with writing that userId to a .csv file. 
 Read tracking is also subject to the STN_OBJECT_READ_LOG* items in the stone config file,
 the DisableObjectReadLogging privilege in UserProfile,
 the DynamicDisableObjectReadLogging privilege in UserProfile,
 and the method System class >> setObjectReadTracking: . "

 aBoolean _validateClass: Boolean .
 aBoolean ifTrue:[
   AllUsers do:[:upro | UserProfile _validateUserId: upro userId ].
 ].
 trackReads := aBoolean
%

category: 'Accessing Authorization'
method: GsObjectSecurityPolicy
usersWithAuthorization: anAccessSymbol

"Returns a IdentitySet containing all users with the authorization of
 anAccessSymbol (one of #write, #read, or #none) for the receiver."

| result |

( #( #write #read #none) includesValue: anAccessSymbol)
  ifFalse: [  ^ self _error: #segErrBadAuthKind args: { anAccessSymbol } ].

result := IdentitySet new .
AllUsers do:[:aUserProfile |
  (self authorizationForUser: aUserProfile) == anAccessSymbol ifTrue:[
     result add: aUserProfile
     ].
  ].
^ result
%

category: 'Accessing Authorization'
method: GsObjectSecurityPolicy
worldAuthorization

"Returns a Symbol that defines whether all system users are authorized to write
 (and read) in this GsObjectSecurityPolicy (#write), to read only (#read), or neither
 (#none)."

^ #( #none #read #write) at: worldAuthorization + 1
%

category: 'Updating Authorization'
method: GsObjectSecurityPolicy
worldAuthorization: anAuthorizationSymbol

"Redefines the authorization for all users to one of the following
 authorization Symbols:

 * #write (all users can both read and write in this GsObjectSecurityPolicy).

 * #read (read only).

 * #none (neither read nor write permission).

 Generates an error if anAuthorizationSymbol is not one of (#read, #write,
 #none).

 Requires the ObjectSecurityPolicyProtection privilege, if the objectSecurityPolicy is not owned
 by the GemStone UserProfile under which this session is running."

 | authInt |
 authInt := #( #none #read #write ) indexOf: anAuthorizationSymbol .
 authInt == 0 ifTrue:[
   ^ self _error: #segErrBadAuthKind args: { anAuthorizationSymbol }
   ].
 self _authAt: 6 put: (authInt - 1)
%

category: 'Storing and Loading'
method: GsObjectSecurityPolicy
writeTo: aPassiveObject

"Instances of GsObjectSecurityPolicy cannot be converted to passive form.  This method writes
 nil to aPassiveObject and stops GemStone Smalltalk execution with a notifier."

  aPassiveObject writeObject: nil.
  self _error: #rtErrAttemptToPassivateInvalidObject.
%

category: 'Private'
method: GsObjectSecurityPolicy
_authAt: anOffset put: aValue

"offset   action
 ------   ---------
  5       assign to ownerAuthorization instance variable
  6       assign to worldAuthorization instance variable
  3       add to groupsRead IdentitySet, create set if needed.
  4       add to groupsWrite IdentitySet, create set if needed.
 -3       remove from groupsRead IdentitySet
 -4       remove from groupsWrite IdentitySet
  7       assign new value to groupsRead (for repository conversion)
  8       assign new value to groupsWrite (for repository conversion)

  Gs64 v2.2 ,  offsets 7 and 8 not supported, was used by _migrateGroups.
"

<primitive: 292>
self _primitiveFailed: #_authAt:put: args: { anOffset . aValue }.
self _uncontinuableError
%

category: 'Private'
method: GsObjectSecurityPolicy
_currentUserAccess

"returns current user's access to objects in the receiver  ,
 0 == none,  1 == read , 2 == write
"
<primitive: 591>
self _primitiveFailed: #_currentUserAccess.
self _uncontinuableError
%

category: 'Copying'
method: GsObjectSecurityPolicy
_deepCopyWith: copiedObjDict

"Private. Used internally to implement deepCopy."

^ self.
%

category: 'Repository Conversion'
method: GsObjectSecurityPolicy
_migrateGroups: migrateBlock

"Converts the groups collections from IdentitySets of Strings to
 IdentitySets of UserProfileGroups."

(Globals at:#DataCuratorObjectSecurityPolicy) setCurrentWhile:[
  migrateBlock value: groupsRead ; value: groupsWrite
].
%

category: 'Accessing'
method: GsObjectSecurityPolicy
_name
  ^ self dynamicInstVarAt: #name
%

! Class extensions for 'GsProcess'

!		Class methods for 'GsProcess'

removeallmethods GsProcess
removeallclassmethods GsProcess

category: 'Instance Creation'
classmethod: GsProcess
continuationFromLevel: numLevels

"Returns an instance of the receiver, that is a copy of the current
 process'  stack , from the stack base to the frame that is the
 specified number of levels above the frame executing this primitive.
 anInt must be >= 1 .

 When  value:   is sent to the result,
 execution resumes in the frame at the top of the instance's saved stack,
 and with top-of-stack value replaced by the argument to value: .

 The result contains copies of all method temps and args from the
 stack it was created from ;  value: sent to the result restores
 the contents of of all method temps and args to the identical values when
 the instance was created. "

<primitive: 47>
System continuationsEnabled ifFalse:[
  "see enableContinuations, disableContinuations in System."
  self error:'continuations are disabled in this session' .
].
numLevels _validateClass: SmallInteger .
numLevels < 1 ifTrue:[ self _errorIndexOutOfRange: numLevels ].
self _primitiveFailed: #continuationFromLevel: args: { numLevels } .
self _uncontinuableError
%

category: 'Debugging Support'
classmethod: GsProcess
cpuOsKind

"Returns an integer specifying the CPU and OS on which
 this VM is running. The result is one of
   1 - reserved for use within VM
   2 - sparc Solaris
   3 - x86_64 Linux
   4 - PowerPc AIX
   5 - x86_64 Apple Unix
   6 - x86_64 Solaris
   7 - Itanium HPUX
"
<primitive: 345>
self _primitiveFailed: #cpuOsKind
%

category: 'Partial Continuations'
classmethod: GsProcess
installPartialContinuation: aGsProcess atLevel: targetLevel value: anObject

" aGsProcess must be a partial continuation, i.e., the result from
  partialContnuationFromLevel:to:, or an error will be generated.

  Replaces targetLevel frames of stack with partialContinuation
  and resumes execution with top-of-stack to be replaced by anObject."

<primitive: 2031>
self _uncontinuableError  "should never reach here"
%

category: 'Instance Creation'
classmethod: GsProcess
new

"Disallowed."

self shouldNotImplement: #new
%

category: 'Partial Continuations'
classmethod: GsProcess
partialContinuationFromLevel: levelsToTrim to: targetLevel

"Returns an instance of the receiver, that is a copy of the stack
 for the current process from frame levelsToTrim+1 to frame targetLevel-1.
 This method (partialContinuationFromLevel:to: will be at top of stack
 which is frame 1 , when the partialContinuation is created.
 levelsToTrim and targetLevel must be >= 1 .
 targetLevel must be >= levelsToTrim+2 .

 The result is expected to be used as an argument to
 #installPartialContinuation:atLevel:value: .

 The result contains copies of all method temps and args from the
 stack it was created from and when used as an argument to
 #installPartialContinuation:atLevel:value:, the contents of all
 method temps and args are restored to the identical values when
 the instance was created."

<primitive: 900>
levelsToTrim _validateClass: SmallInteger .
levelsToTrim < 1 ifTrue:[ self _errorIndexOutOfRange: levelsToTrim ].
targetLevel _validateClass: SmallInteger .
targetLevel < 1 ifTrue:[ self _errorIndexOutOfRange: targetLevel ].
targetLevel > (levelsToTrim + 1) ifFalse:[ self _errorIndexOutOfRange: targetLevel ].
self _primitiveFailed: #partialContinuationFromLevel:to:
     args: { levelsToTrim . targetLevel } .
self _uncontinuableError
%

category: 'Debugging Support'
classmethod: GsProcess
stackReportToLevel: aLevel

"Returns a String describing the currently active stack, starting with
 to the sender of this method (which is considered level 1).  The aLevel

 The format of the result is subject to change with each release of GemStone."

| framesArr aFrame report level lf |
framesArr := { }  .
level := 1 .
[ level <= aLevel and:[ (aFrame := self _frameContentsAt: level + 1) ~~ nil] ]
whileTrue:[
  framesArr at: level put: aFrame.
  level := level + 1.
  ].
report := String new .
lf := Character lf .
1 to: framesArr size do:[:j | | gsMethod |
  report add: j asString; add:' ' .
  aFrame := framesArr at: j .
  gsMethod := aFrame at: 1 .
  gsMethod ifNil:[
    report add:'<Reenter marker>'; add: lf .
  ] ifNotNil:[ | stepPoint aRcvr |
    aRcvr := aFrame at: 10 .
    report add:  (gsMethod _descrForStackPadTo: 0 rcvr: aRcvr ) .
    stepPoint := gsMethod _previousStepPointForIp: (aFrame at: 2) .
    report add:' @' ; add: stepPoint asString ;
      add: ' line ';
      add: (gsMethod _lineNumberForStep: stepPoint) asString ;
      add:'  [GsNMethod '; add: gsMethod asOop asString ; add:']';
      add: lf .
    ].
  ].
^ report
%

category: 'Debugging Support'
classmethod: GsProcess
usingNativeCode

  "returns true if the currently executing process is using native code"
<primitive: 855>
self _primitiveFailed: #usingNativeCode
%

category: 'Private'
classmethod: GsProcess
_current
  "Returns currently active process .
   The result should be used for accessing thread-local data only.
   Instance methods related to accessing the stack frames do not work
   for the active process."

<primitive: 693>
self _primitiveFailed: #_current
%

category: 'Private'
classmethod: GsProcess
_currentOrNil
  "Returns currently active process if one exists. Returns nil if
   current execution was initiated by GCI and no GsProcess has yet been created.
   The result should be used for accessing thread-local data only.
   Instance methods related to accessing the stack frames do not work
   for the active process."

<primitive: 1110>
self _primitiveFailed: #_currentOrNil
%

category: 'Private'
classmethod: GsProcess
_forkBlock: aBlock with: blockArgs env: envId prio: aPriority
  "Answer an ready instance of the receiver representing a potential
   process on the given block and with the active priority."
  | proc |
  (blockArgs _isArray ) ifFalse:[
    blockArgs _error: #rtErrInvalidArgClass args: { Array } .
    self _uncontinuableError .
  ].
  (proc := self _basicNew) _init: aBlock args: blockArgs env: envId
      stackSerial: self _current _stackSerialNum
      forked:  ModeInfo_forked .
  self _scheduler _scheduleNewProcess: proc prio: aPriority  .
  ^ proc
%

category: 'Debugging Support'
classmethod: GsProcess
_frameContentsAt: aLevel

"Returns an Array describing the specified stack level in the currently
 executing process.

 aLevel == 1 means top of stack with respect to the
               sender of _frameContentsAt: .

 If aLevel is out of range, returns nil.

 The contents of the result Array are the same as for the instance method
 GsProcess | _frameContentsAt:, except for :
    element  9 (argAndTempNames) is nil
             8 (self) is nil
             4  varContext is always nil .
 "

<primitive: 195>
self _primitiveFailed: #_frameContentsAt: args: { aLevel }
%

category: 'Debugging Support'
classmethod: GsProcess
_fullStackReport
  "used in ernie framework"
| curr res |
curr := GsProcess _current .
[ res := curr stackReportToLevel: 300 withArgsAndTemps: true andMethods: false ] forkAt: 39 .
Processor yield .
^ res asString .
%

category: 'Private'
classmethod: GsProcess
_maxProcessStacks

  ^ MaxProcessStacks "from C constant OM_MAX_PROCESS_STACKS"
%

category: 'Debugging Support'
classmethod: GsProcess
_methodInFrameContents: aFrameContentsArray

^ aFrameContentsArray at: 1 "virtual machine constant"
%

category: 'Private'
classmethod: GsProcess
_newForBlock: aBlock with: blockArgs env: envId
  "Answer a suspended instance of the receiver representing a potential
   process on the given block and with the active priority."
  | proc |
  (blockArgs _isArray ) ifFalse:[
    blockArgs _error: #rtErrInvalidArgClass args: { Array } .
    self _uncontinuableError .
  ].
  (proc := self _basicNew) _init: aBlock args: blockArgs env: envId
       stackSerial: self _current _stackSerialNum
      forked:  ModeInfo_forked .
  self _scheduler _suspendNewProcess: proc .
  ^ proc
%

category: 'Debugging Support'
classmethod: GsProcess
_receiverAt: aLevel

"Returns the receiver at the given level of the currently executing
 process' stack.

 aLevel == 1 means top of stack with respect to the
               sender of _selfAt: .

 If aLevel is out of range, returns nil."

|frame|
(frame := self _frameContentsAt: aLevel) ifNil:[ ^ nil ].
^ frame at: 10 "virtual machine constant"
%

category: 'Private'
classmethod: GsProcess
_scheduler

"Returns   ProcessorScheduler scheduler "
<primitive: 457>

self _primitiveFailed: #_scheduler
%

category: 'Debugging Support'
classmethod: GsProcess
_selfAt: aLevel

"Returns self at the given level of the currently executing process' stack.

 aLevel == 1 means top of stack with respect to the
               sender of _selfAt: .

 If aLevel is out of range, returns nil."

|frame|
(frame := self _frameContentsAt: aLevel + 1) ifNil:[ ^ nil ].
^ frame at: 8 "virtual machine constant"
%

category: 'Debugging Support'
classmethod: GsProcess
_topazAsString: anObject
  ^ [ anObject asString
    ] onSynchronous: AbstractException do:[:ex |
      [  | cls |
         cls := Reflection classOf: anObject .
         cls == UnauthorizedObjectStub ifTrue:[ anObject __asString]
                           ifFalse:[ 'a ', cls name ]
      ] onSynchronous: AbstractException do:[:exb |
        '<error during asString>'
      ]
    ]
%

category: 'Debugging Support'
classmethod: GsProcess
_topazExceptionName: anException
  ^ [ | ecls |
      (ecls := anException class) _isExceptionClass ifFalse:[ ^ nil ].
      ecls name  .
    ] onSynchronous: AbstractException do:[:ex |
      ex return: '<error during _topazExceptionName:>'
    ].
%

!		Instance methods for 'GsProcess'

category: 'Private'
method: GsProcess
beForked
  "Private.
   Prevents termination of the receiver from causing a return to the GCI.
   if the scheduler has something else to run."

  modeInfo := modeInfo bitOr: ModeInfo_forked
%

category: 'In-Process Debugger Support'
method: GsProcess
clearLastBreakpointHistory
  "If a Breakpoint is handled by handler block installed with on:do: or
   onException:do:, and neither #return nor #resume is sent to the instance
   of Breakpoint, then
      GsProcess _current clearLastBreakpointHistory
   is needed in order for the next Breakpoint to be signalled properly."

  lastBreakpt := nil
%

category: 'Process Properties'
method: GsProcess
clientData
  "answer the client data associated with the receiver.
   See also environmentAt:  "

  ^clientData
%

category: 'Process Properties'
method: GsProcess
clientData: anobject
  "set the client data object associated with the receiver.
   See also environmentAt:put:  "

  clientData := anobject
%

category: 'Debugging Support'
method: GsProcess
convertToPortableStack
  "Ensure that the receiver is executing in interpreted mode,
   used so that subsequent breakpoint operations will work."

  self == GsProcess _currentOrNil ifTrue:[
    GsProcess usingNativeCode ifTrue:[
      ^ Error signal:'cannot alter stack of the currently running GsProcess'
    ]
  ] ifFalse:[
    self localStackDepth "ensure stack pushed to object memory".
    self _nativeStack ifTrue:[
      self _convertToPortableStack: -1 .
    ].
  ]
%

category: 'Debugging Support'
method: GsProcess
convertToPortableStack: forTrimToLevel

  "Ensure that the receiver is executing in interpreted mode,
   used so that subsequent breakpoint operations and execution
   restart will work.
   forTrimToLevel should be -1 if not being called for stack trim,
   otherwise a positive stack level. "

  self == GsProcess _currentOrNil ifTrue:[
    GsProcess usingNativeCode ifTrue:[
      ^ Error signal:'cannot alter stack of the currently running GsProcess'
    ]
  ] ifFalse:[
    self localStackDepth "ensure stack pushed to object memory".
    self _nativeStack ifTrue:[
      self _convertToPortableStack: forTrimToLevel .
    ].
  ]
%

category: 'Copying'
method: GsProcess
copy

"Disallowed."

self shouldNotImplement: #copy
%

category: 'Accessing'
method: GsProcess
createdByApplication
  "Returns true if the receiver was created by the application.
   Returns false if the receiver was created by a Smalltalk method."

  ^(args == nil)
%

category: 'Accessing'
method: GsProcess
debugActionBlock
   ^ debugActionBlock
   
%

category: 'Updating'
method: GsProcess
debugActionBlock: aBlock
  aBlock ifNotNil:[ 
    aBlock _validateClass: ExecBlock.
    aBlock argumentCount <= 1 ifFalse:[
      ArgumentError signal:'block must take 0 or 1 arguments'.
    ].
  ].
  debugActionBlock := aBlock
%

category: 'Debugging Support'
method: GsProcess
disableDebugging
  "causes the receiver to ignore any breakpoints which may be set."
  ^ self _setModeinfoBit: ModeInfo_debugEnabled value: 0
%

category: 'Debugging Support'
method: GsProcess
enableDebugging
  "causes the receiver to ignore any breakpoints which may be set."
  ^ self _setModeinfoBit: ModeInfo_debugEnabled value: 1 .
%

category: 'Process Properties'
method: GsProcess
environment
  | env |
  (env := environment) ifNil:[
     env := Dictionary new .
     environment := env
  ].
  ^ env
%

category: 'Process Properties'
method: GsProcess
environment: aDictionary

  environment := aDictionary
%

category: 'Process Properties'
method: GsProcess
environmentAt: key

  | env |
 (env := environment) ifNil:[ ^ nil ].
 ^ env at: key otherwise:  nil
%

category: 'Process Properties'
method: GsProcess
environmentAt: key ifAbsent: aBlock
  | env |
  (env := environment) ifNil:[ ^ aBlock value ].
  ^ env at: key ifAbsent:[ aBlock value ]
%

category: 'Process Properties'
method: GsProcess
environmentAt: key put: value

  ^ (environment ifNil:[ self environment]) at: key put: value
%

category: 'Debugging Support'
method: GsProcess
gciStepIntoFromLevel: aLevel
  "For use by a Gci application.
   Breakpoint and step point errors will be returned as errors to the GCI.
   Do not use when implementing in-process debugging.

  Step execution so that it stops at the beginning of the next method
  invoked.  If a return causes the
  context at the specified level to be removed from the
  stack, execution will stop immediately after that return.
  If the receiver completes result of the completion is returned."
  aLevel < 1 ifTrue:[ ArgumentError signal:'level argument must be >= 1' ].
  ^ self _stepOverInFrame: aLevel - 1 mode: true replace: false tos: nil
%

category: 'Debugging Support'
method: GsProcess
gciStepOverFromLevel: aLevel
  "For use by a Gci application.
   Breakpoint and step point errors will be returned as errors to the GCI.
   Do not use when implementing in-process debugging.

  Step execution so that it stops after the next message send
  in the specified stack level.  If a return causes the
  context at the specified level to be removed from the
  stack, execution will stop immediately after that return.
  If the receiver completes result of the completion is returned."
  aLevel < 1 ifTrue:[ ArgumentError signal:'level argument must be >= 1' ].
  ^ self _stepOverInFrame: aLevel  mode: true replace: false tos: nil
%

category: 'Debugging Support'
method: GsProcess
gciStepThruFromLevel: aLevel
  "For use by a Gci application.
   Breakpoint and step point errors will be returned as errors to the GCI.
   Do not use when implementing in-process debugging.

  Step execution so that it stops after the next message send
  in the specified stack level.  If a return causes the
  context at the specified level to be removed from the
  stack, execution will stop immediately after that return.
  If the receiver completes result of the completion is returned."

  | flags |
  aLevel < 1 ifTrue:[ ArgumentError signal:'level argument must be >= 1' ].
  flags := (aLevel bitAnd: 16rFFFFFFFF) bitOr: 16r100000000"thru flag" .
  ^ self _stepOverInFrame: flags mode: true replace: false tos: nil
%

category: 'Debugging Support'
method: GsProcess
ignoreNextBreakpoint
  "Use this prior to resuming from a breakpoint if your
   debugger is not using GciContinue to resume execution."

  ^ self _setModeinfoBit: ModeInfo_breakptsIgnore_mask value: 1 .
%

category: 'Updating'
method: GsProcess
instVarAt: anIndex put: aValue

"Disallowed."

self shouldNotImplement: #instVarAt:put:
%

category: 'Accessing'
method: GsProcess
interruptFlag

"Return the interruptFlag value"
^ interruptFlag_stEntryCount bitShift: -32
%

category: 'Continuations'
method: GsProcess
isContinuation

"Return true if the receiver is a continuation."

^ (modeInfo bitAnd: ModeInfo_continuationMask) == ModeInfo_isContinuation
%

category: 'Debugging Support'
method: GsProcess
isForked
  ^ (modeInfo bitAnd: ModeInfo_forked) ~~ 0
%

category: 'Continuations'
method: GsProcess
isPartialContinuation

"Return true if the receiver is a continuation."

^ (modeInfo bitAnd: ModeInfo_continuationMask) == ModeInfo_isPartialContinuation
%

category: 'Changing Process State'
method: GsProcess
join: limitSeconds

  "Waits up to limitSeconds for receiver to terminate.
   Returns self if thread already terminated, nil
   if the time limit expires before receiver terminates"

  | curr delay mi |
  limitSeconds _isSmallInteger ifFalse:[
    ArgumentError signal:'expected a SmallInteger' .
  ].
  limitSeconds < 0 ifTrue:[
    ArgumentError signal:'argument must be >= zero ' .
  ].
  self == (curr := self _currentOrNil) ifTrue:[
    ThreadError signal:'tried to join itself' .
    ^ nil
  ].
  mi := modeInfo .
  (mi bitAnd: ModeInfo_terminated ) ~~ 0 ifTrue:[ ^ self ].
  limitSeconds ~~ 0 ifTrue:[
    (mi bitAnd: ModeInfo_terminationStarted) ~~ 0 ifTrue:[ | t limMs |
      limMs := 1000 * limitSeconds .
      t := 0 .
      [ t < limMs ] whileTrue:[
        delay := Delay forMilliseconds: 20.
        delay highPriorityWait  .
        self _isTerminated ifTrue:[ ^ self ].
        t := t + 20 .
      ]
    ] ifFalse:[
      delay := Delay forMilliseconds: 1000 * limitSeconds .
      self _addJoiner: delay .
      delay highPriorityWait  .
      self _removeJoiner: delay .
      self _isTerminated ifTrue:[ ^ self ].
    ]
  ].
  ^ nil
%

category: 'Accessing'
method: GsProcess
localMethodAt: aLevel

"Returns the GsNMethod that is active at aLevel in the receiver, where
 aLevel == 1 is the top of the stack.  Generates an error if aLevel less than
 zero or greater than stackDepth.  Returns nil if there is a reenter marker at
 the specified level."

| fpIdx dp |

(aLevel < 1 or:[ aLevel > (dp := self localStackDepth) ]) ifTrue:[
  aLevel _error: #rtErrArgOutOfRange args:{ 1 . dp } .
  ].
fpIdx := self _frameOffsetAt: aLevel .
fpIdx ifNil:[ ^ nil].
^ self _fetchMethod:( arStack at: (fpIdx + FP_codePtr_OFS )) .
%

category: 'Accessing'
method: GsProcess
localStackDepth

"Returns the local stack depth of the receiver, or 0 if receiver is running.
 If receiver not running but has an in-memory stack, forces the receiver out
 to object memory.  If receiver not running but is locked in memory,
 this method will unlock it.
 Returns 0 if receiver was found in memory but has been terminated, or
 has never been run.
"

<primitive: 719>  "process push to OM, primitive always fails"
| uacd |
(uacd := uaCount_depth) ifNil:[
  (self _isTerminated or:[arStack == nil]) ifTrue:[ ^ 0 ].
  ^ 0
].
^ uacd bitAnd: 16rFFFFFFFF
%

category: 'Accessing'
method: GsProcess
lockedInMemory

  ^ self _storeBit: 0 value: nil .
%

category: 'Updating'
method: GsProcess
lockInMemory

 "Returns true if successful.
  Returns false if too many processes would be locked in stack memory,
  or if the receiver is not currently in stack memory."

 ^ self _storeBit: 0 value: true
%

category: 'Accessing'
method: GsProcess
methodAt: aLevel

"Returns the GsNMethod that is active at aLevel in the receiver, where
 aLevel == 1 is the top of the stack.  Generates an error if aLevel less than
 zero or greater than stackDepth.  Returns nil if there is a reenter marker at
 the specified level."

^ self localMethodAt: aLevel
%

category: 'Accessing'
method: GsProcess
name
  ^ name ifNil:[ '' ]
%

category: 'Updating'
method: GsProcess
name: aString
  name := aString 
%

category: 'Accessing'
method: GsProcess
onQueue
  ^ onQueue
%

category: 'Accessing'
method: GsProcess
parentProcess
  ^ parentProcess "a GsProcess"
%

category: 'Formatting'
method: GsProcess
printOn: aStream

"Puts a displayable representation of the receiver on the given stream."

aStream nextPutAll: self _reportString
%

category: 'Formatting'
method: GsProcess
printString

"Returns a String whose contents are a displayable representation of the
 receiver."

^ self _reportString
%

category: 'Accessing'
method: GsProcess
priority
  "Answer the scheduling priority of the receiver"

  | p_u |
  (p_u := priority_use) ifNil:[ ^ 15 "inline userSchedulingPriority"].
  ^ p_u bitShift: -40
%

category: 'Updating'
method: GsProcess
priority: anInt
  | oldPriority p_u |
      "inline _priorityRangeCheck"
  (anInt < 1 or:[ anInt > 40])  "inline lowestPriority, highestPriority" ifTrue:[
    anInt _error: #rtErrArgOutOfRange args:{ 1 . 40 } .
  ].
  (p_u := priority_use) ifNil:[
    oldPriority := 15 "inline userSchedulingPriority" .
  ] ifNotNil:[
    oldPriority :=  p_u bitShift: -40 .
  ].
  anInt ~~ oldPriority ifTrue:[
    self _setPriority: anInt.
    self _scheduler _changePriority: self from: oldPriority  .
    waitingOn ifNotNil:[
      waitingOn _changePriority: self from: oldPriority .
    ]
  ].
%

category: 'Accessing'
method: GsProcess
result
  "Retrieve the result from a completed thread. Returns nil
  if receiver still running, or did not complete normally."
  | res |
  self _isTerminated ifFalse:[ ^ nil ].
  (res := blockResult) == _remoteNil ifTrue:[ ^ nil ].
  ^ res
%

category: 'Changing Process State'
method: GsProcess
resume
  "Puts the receiver in the queue of ready processes at its priority"
  (modeInfo bitAnd: ModeInfo_continuationMask) ~~ 0 ifTrue:[
    self error:'cannot resume an instance that is a continuation'
  ].
  self _scheduler _resumeProcess: self .
%

category: 'Updating'
method: GsProcess
setInterruptFlag: anInt

" set the interruptFlag value. Receiver is expected to have been suspended."
| val |
stackId >= 0 ifTrue:[ self error:'cannot update C stack state correctly' ] .
val := interruptFlag_stEntryCount bitAnd:(16rFFFFFFFF). "keep stEntryCount"
val := val bitOr:(( anInt bitAnd:16rFFFFFFF) bitShift: 32) .
interruptFlag_stEntryCount := val
%

category: 'Changing Process State'
method: GsProcess
signalException: anException

  | prevBit |
  "signal an exception to the receiver."

  self == self _currentOrNil ifTrue:[
    "this path not expected to be used by Ruby"
    ^ anException signal
  ].
  signalledException := anException .
  prevBit := self _storeBit: 2  value: true . "set INT_THREAD_SIGNAL_EXC_MASK bit"
  self resume .
%

category: 'Accessing'
method: GsProcess
stackDepth

"Returns the stack depth of the receiver."

| result |
result := self localStackDepth .
^ result
%

category: 'Accessing'
method: GsProcess
stackId

  "returns -1 if not in a stack area for execution.
   Only the VM can update stackId ."

  ^ stackId
%

category: 'Debugging Support'
method: GsProcess
stackReportToLevel: aLevel withArgsAndTemps: includeArgsAndTemps andMethods: includeMethods
  "Returns a String describing the stack of the receiver.
   The report includes frames from level 1 (top of stack) through the lesser of aLevel and
   the stack depth.

   The format of the result is subject to change with each release of GemStone.
   aLevel should be a SmallInteger >= 1 .
   includeArgsAndTemps and includeMethods should be Booleans."

^ self stackReportToLevel: aLevel withArgsAndTemps: includeArgsAndTemps
 	  andMethods: includeMethods includeSource: false
%

category: 'Debugging Support'
method: GsProcess
stackReportToLevel: aLevel withArgsAndTemps: includeArgsAndTemps andMethods: includeMethods includeSource: includeSource

^ self stackReportToLevel: aLevel withArgsAndTemps: includeArgsAndTemps
      andMethods: includeMethods includeSource: includeSource
      lineLimit: 100"characters per line"
%

category: 'Debugging Support'
method: GsProcess
stackReportToLevel: aLevel withArgsAndTemps: includeArgsAndTemps andMethods: includeMethods includeSource: includeSource lineLimit: lineLim
  "Returns a String describing the stack of the receiver.
   The report includes frames from level 1 (top of stack) through the lesser of aLevel and
   the stack depth.

   The format of the result is subject to change with each release of GemStone.
   aLevel should be a SmallInteger >= 1 .
   includeArgsAndTemps and includeMethods should be Booleans.
   includeSource may be: false or #none   no source ,
                         true or #execution  Execution sources,
                         #blocks    Sources for blocks and methods not in method dictionary
			                   2  or #all  method sources.
   lineLim is a SmallInteger specifing the maximum length in Characters 
       of each line of the report."

     | framesArr aFrame report gsMethod level inclSrc rptBlk tmpBlk |
     rptBlk := [:str | str size > lineLim
                   ifTrue:[ report add: (str copyFrom: 1 to: lineLim) ]
                   ifFalse:[ report add: str ]] .
     tmpBlk := [:anObj | | s | s := String new .
          anObj isSpecial ifFalse:[
            anObj isCommitted ifTrue:[ s add: 'oop:'; add: anObj asOop asString; add:' ' ].
            anObj class isIndexable ifTrue:[ s add: 'size:'; add: anObj size asString; add:' '].
          ].
          s size > 0 ifTrue:[ s add:' '].
          s add: anObj describe.
          s  ].
     inclSrc := includeSource .
     (inclSrc == true or:[ inclSrc == #execution]) ifTrue:[ inclSrc := 1 ] ifFalse:[
     (inclSrc == false or:[ inclSrc == #none]) ifTrue:[ inclSrc := 0 ] ifFalse:[
     (inclSrc == #blocks)                ifTrue:[ inclSrc := 2 ] ifFalse:[
     (inclSrc == #all or:[ inclSrc == 2 ]) ifTrue:[ inclSrc := 3 ] ifFalse:[
         ArgumentError signal: 'unrecognized includeSource arg ', includeSource asString]]]].
     framesArr := { } .
     level := 1.
     [ level <= aLevel and: [(aFrame := self _frameContentsAt: level) ~~ nil]] whileTrue:[
       framesArr at: level put: aFrame.
       level := level + 1
     ].
     report := String new.
     1 to: framesArr size do: [:j |
       report add: j asString; add: ' '.
       aFrame := framesArr at: j.
       (gsMethod := aFrame at: 1) ifNil: [report add: '<Reenter marker>'; lf
       ] ifNotNil: [
         | stepPoint |
         report add: gsMethod _descrForStack.
         stepPoint := gsMethod _stepPointForIp: (aFrame at: 2) level: j
             useNext: (self _nativeStack or:[ self _calleeIsAsync: j]) .
         report
           add: ' @'; add: stepPoint asString; add: ' line ';
           add: (gsMethod _lineNumberForStep: stepPoint) asString.
         includeMethods ifTrue:[
          report add: '  [GsNMethod '; add: gsMethod asOop asString; add: $]
         ].
         report lf.
         includeArgsAndTemps ifTrue:[ | argsAndTempsNames argsAndTempsValues |
          report add: '    receiver '.
             rptBlk value: (tmpBlk value: (aFrame at: 10)) .  report lf .
           argsAndTempsNames := aFrame at: 9.
           argsAndTempsValues := aFrame copyFrom: 11 to: aFrame size.
           [ argsAndTempsNames with: argsAndTempsValues do:[:nam :value |
                report add: '    '; add: nam; add: ' '.
                rptBlk value: ( tmpBlk value: value ) .
                report  lf
             ]
           ] on: ArgumentError do: [:ex |
             report
               add: '    <<<args and temps mismatch (';
               add: argsAndTempsNames size printString;
               add: ' versus ';
               add: argsAndTempsValues size printString;
               add: ', frame size = ';
               add: aFrame size printString;
               add: ') - ';
               add: '    names: ';
               add: argsAndTempsNames printString;
               add: '    values: '.
             rptBlk value: argsAndTempsValues describe .
             report add: '>>>'; lf .
            ]
          ].
          (self _stackReportIncludeSourceFor: gsMethod inclSrc: inclSrc) ifTrue:[
            gsMethod isMethodForBlock ifTrue:[
              report add: '  Block source:'; add:  (aFrame at: 10) _sourceString; lf
            ] ifFalse:[
              report add: (gsMethod inClass ifNil:[ '  Executed'] ifNotNil:['  Method']);
                     add: ' source: ';
                     add: gsMethod sourceString ; lf .
            ]
          ]
       ].
     ].
     ^report
%

category: 'Accessing'
method: GsProcess
startSeconds
  "Returns start time of the reciever, a SmallDouble, number of seconds since Jan 1, 2001
   with resolution of microseconds."
  ^ startSeconds
%

category: 'In-Process Debugger Support'
method: GsProcess
stepIntoFromLevel: aLevel

    "Step execution over next message in stack frame specified by aLevel.
     To be used within implementation of an in-process debugger,
     after the receiver has signaled a Breakpoint from a previous single step
     or from a method breakpoint.
     After executing this method,
     the sender must yield  so the receiver can run.
     Receiver needs to have an on:do: handler for Breakpoint which
     yields to the debuggger process and then resumes from the Breakpoint.

     The step_into interrupt bit in the VM will be set by the returnNothing
     bytecode when the receiver returns from AbstractException>>_signalAsync."

  self stepOverFromLevel: aLevel .
  self _setModeinfoBit: ModeInfo_rtnNothingStepInto value: 1 .
%

category: 'In-Process Debugger Support'
method: GsProcess
stepOverFromLevel: aLevel
    "Step execution over next message in stack frame specified by aLevel.
     To be used within implementation of an in-process debugger.
     After executing this method,
     the sender must yield  so the receiver can run.
     Receiver needs to have an on:do: handler for Breakpoint which
     yields to the debuggger process and then resumes from the Breakpoint."

  | dp |
  self == GsProcess _currentOrNil ifTrue:[
    ^ Error signal:'cannot single-step the currently running GsProcess'
  ].
  dp := self localStackDepth . "force to object memory"
  aLevel _validateClass: SmallInteger .
  (aLevel < 1 or:[ aLevel > dp]) ifTrue:[
     ^ ArgumentError new name:'alevel' min: 1 max: dp actual: aLevel ; signal.
  ].
  self _stepOverFromLevel: aLevel
%

category: 'In-Process Debugger Support'
method: GsProcess
stepThroughFromLevel: aLevel
  "Step execution over the next message send in stack frame specified
   by aLevel, stopping in blocks which share a home method with the
   specified frame.
   To be used within implementation of an in-process debugger.
   After executing this method,
   the sender must yield  so the receiver can run.
   Receiver needs to have an on:do: handler for Breakpoint which
   yields to the debuggger process and then resumes from the Breakpoint."

  | dp |
  self == GsProcess _currentOrNil ifTrue:[
    ^ Error signal:'cannot single-step the currently running GsProcess'
  ].
  dp := self localStackDepth . "force to object memory"
  aLevel _validateClass: SmallInteger .
  (aLevel < 1 or:[ aLevel > dp]) ifTrue:[
     ^ ArgumentError new name:'alevel' min: 1 max: dp actual: aLevel ; signal.
  ].
  self _stepOverFromLevel: (16r100000000 bitOr: aLevel)
%

category: 'Changing Process State'
method: GsProcess
suspend
  "Suspends the receiver from processing and does not
   schedule it for further processing."

  (modeInfo bitAnd: ModeInfo_continuationMask) ~~ 0 ifTrue:[
    self error:'cannot suspend an instance that is a continuation'
  ].
  self _scheduler _suspendProcess: self .
%

category: 'Accessing'
method: GsProcess
threadRubyEnvId

 ^ modeInfo bitAnd: ModeInfo_threadEnvMask
%

category: 'Updating'
method: GsProcess
unlockInMemory

 "If receiver is locked in stack memory, allow it to be preempted.
  Returns true if receiver was locked in stack memory."

  ^ self _storeBit:0 value: false
%

category: 'Continuations'
method: GsProcess
value

"See value: for documentation "

^ self value: nil

%

category: 'Continuations'
method: GsProcess
value: anArg

"The receiver must be a continuation, i.e. a result
 from continuationFromLevel: , or an error will be generated.

 Resumes execution of the continuation with top-of-stack
 replaced by anArg . "

<primitive: 2016>
self _uncontinuableError  "should never reach here"
%

category: 'Accessing'
method: GsProcess
waitingOn
  ^ waitingOn
%

category: 'Private'
method: GsProcess
_activate
  "Wake up the receiver"
  | sched wasInCrit |
  sched := self _scheduler .
  wasInCrit := sched _enterCritical .
  signalTime := 0 .
  sched _scheduleProcess: self.
  wasInCrit == 0 ifTrue:[ sched _exitCritical ] .
%

category: 'Private'
method: GsProcess
_addJoiner: aDelay

| js |
(js := joiners ) ifNil:[
  joiners := { aDelay }.
  ^ self .
].
js at: js size + 1 put: aDelay
%

category: 'Debugging Support'
method: GsProcess
_calleeIsAsync: aLevel
  self _currentOrNil == self ifTrue:[ ^ false "an approximation"].
  aLevel > 1 ifTrue:[
    (self _frameDescrAt: aLevel -1) ifNotNil:[:calleeDesc |
      (self _fetchMethod:(arStack at:(calleeDesc at:2) + FP_codePtr_OFS)) ifNotNil:[:calleeMeth|
        | sel |
        sel := calleeMeth selector .
        (sel == #_signalAsync or:[ sel == #_signalGcFinalize or:[ sel == #_signalTimeout]]) ifTrue:[
           ^ calleeMeth inClass == AbstractException
        ]
      ]
    ]
  ].
  ^ false
%

category: 'Accessing'
method: GsProcess
_cannotReturn: aValue

"Raises an error message in the event that the virtual machine cannot
 resume after the current method or block context."

^ self _error: #rtErrCantReturn
%

category: 'Private'
method: GsProcess
_canWaitOnSocket

  ^ true
%

category: 'Private'
method: GsProcess
_checkIfDebuggable
  "Check to make sure the receiver is debuggable. Currently this
   means that it is in the debug or ready states.
   If it is not debuggable then raise an error."

  | status |

  status := self _statusString.
  (status = 'ready' or:[status = 'debug' or:[ status = 'active' ]]) ifFalse: [
    ImproperOperation new _number: 2376 ; reason: #rtErrGsProcessNotDebuggable;
  object: self ; signal
  ].
%

category: 'Private Debugging Support'
method: GsProcess
_clearAllStackBreaks

  1 to: self stackDepth - 1 do:[:n | self _clearStackBreakAt: n ]
%

category: 'Private Debugging Support'
method: GsProcess
_clearAllStepBreaksAt: aLevel

"Legacy code, not used in the current debugging implementation.
 Clears single step breakpoints at the specified level on the stack.  If
 aLevel is 1, clears the step-into bit in the virtual machine flag word, which
 could be left over from a step-into flag argument to a GciPerform* or
 GciExecute* call."

| meth |
aLevel == 1 ifTrue:[  | flags |
  flags := self interruptFlag .
  flags := flags bitAnd:( 16r200 bitInvert) .
  self setInterruptFlag: flags  .
].
meth := (self == GsProcess _currentOrNil) ifTrue:[ GsProcess _frameContentsAt: 1 ]
                                     ifFalse:[ self localMethodAt: aLevel ].
meth ifNotNil:[ meth _clearAllStepBreaks ]
%

category: 'Private Debugging Support'
method: GsProcess
_clearStackBreakAt: aLevel

"Clear stack breakpoint at specified level.

 Debugger implementations should use GciStep rather than invoking
 this method directly."

| ipOffset dp |
self _nativeStack ifTrue:[
  ^ self _error: #rtErrNativeCodeNoBreakpts
].
(aLevel < 1 or:[ aLevel > (dp := self stackDepth) ]) ifTrue:[
  aLevel _error: #rtErrArgOutOfRange args:{ 1 . dp } .
  ].
(self __returnAddrIdxAt: aLevel) ifNotNil:[ :rtnAddrIdx |
  ipOffset := arStack at: rtnAddrIdx .
  ipOffset < 0 ifTrue:[
    arStack at: rtnAddrIdx put: ipOffset abs
  ].
].
%

category: 'Debugger Support'
method: GsProcess
_continue
    "Continue execution. Return the result of execution if the receiver
     completes. Otherwise if a breakpoint is hit returns the receiver."

  | result status sched wasInCrit |

  self _checkIfDebuggable.
  sched := self _scheduler .
  wasInCrit := sched _enterCritical .
  status := self _statusString.
  (status = 'ready') ifTrue: [
    self _unscheduleProcess .
  ].
  self _storeBit: 1 value: true . "debuggingInProgress:=1"
  wasInCrit == 0 ifTrue:[ sched _exitCritical ].

  result := self _primContinue: nil .

  wasInCrit := sched _enterCritical .
  (status = 'ready') ifTrue: [
    (result == self) ifTrue: [
      (self _isTerminated) ifFalse: [
        sched _scheduleProcess: self .
      ].
    ].
  ].
  self _storeBit: 1 value: false . "debuggingInProgress:=0"
  wasInCrit == 0 ifTrue:[ sched _exitCritical ].
  ^result
%

category: 'Private'
method: GsProcess
_convertToPortableStack: forTrimToLevel
  "convert receiver's stack from native code to portable code format.
   Receiver must be suspended and stack in object memory.
  forTrimToLevel should be -1 if not being called for stack trim,
  otherwise a positive stack level. "

  self _nativeStack ifTrue:[
    [
      | newIps ipOfs fpOfs ar_size lev |
      newIps := { }  .
      ipOfs := 1 .
      fpOfs := topFpOffset .
      ar_size := arStack size .
      lev := 1 .
      [ fpOfs < 0 ] whileTrue:[ | fpIdx codePtr meth natIp portIp |

        fpIdx := ar_size  + (self _toWordOfs: fpOfs) + 1 .
        codePtr := arStack at:( fpIdx + FP_codePtr_OFS ).
        meth := self _fetchMethod:  codePtr  .
        natIp := arStack  at: ipOfs .
        portIp := meth _nativeIpOffsetToPortable: natIp asReturn: true .
        portIp <= 0 ifTrue:[
          (forTrimToLevel >= 1 and:[ lev <= forTrimToLevel ]) ifTrue:[
            portIp := GsNMethod _firstPortableIpoffset .
          ] ifFalse:[
            "examples: error 2085 from an ifTrue bytecode..."
            self error:'native to portable IP not exact in frame ', lev asString .
          ]
        ].
        arStack  at: ipOfs put: portIp .
        ipOfs := fpIdx + FP_rtnAddr_OFS .

        fpOfs := arStack at: fpIdx"+ FP_savedFP_OFS==0" . "get callerFpOfs"
        lev := lev + 1 .
      ].
      self _setInterpretedStack .
    ] onSynchronous: Error do:[ :ex |
      InternalError new details: ex description ;
           reason: 'nativeStackNotConvertableToInterpreted' ; signal
    ].
  ].
%

category: 'Private'
method: GsProcess
_current
  "Returns currently active process .
   The result should be used for accessing thread-local data only.
   Instance methods related to accessing the stack frames do not work
   for the active process."

<primitive: 693>
self _primitiveFailed: #_current
%

category: 'Private'
method: GsProcess
_currentOrNil
  "Returns currently active process if one exists. Returns nil if
   current execution was initiated by GCI and no GsProcess has yet been created.
   The result should be used for accessing thread-local data only.
   Instance methods related to accessing the stack frames do not work
   for the active process."

<primitive: 1110>
self _primitiveFailed: #_currentOrNil
%

category: 'Process Properties'
method: GsProcess
_environment

  ^ environment
%

category: 'Changing Process State'
method: GsProcess
_executeEnsureBlocks
  | pairs |
  pairs := self _getEnsureBlocks .
  1 to: pairs size by: 2 do: [:k |
    self _removeEnsureAtFP:( pairs at: k) .
    (pairs at: k + 1 ) value .
  ].
%

category: 'Accessing'
method: GsProcess
_fetchMethod: aCodePtr

"translate a codePtr to a GsNMethod"
| codeCls  |
codeCls := aCodePtr class .
codeCls == GsNMethod ifTrue:[
 ^ aCodePtr
] ifFalse:[
  self _halt: 'should not be here' .
  ^ nil
].
%

category: 'Debugging Support'
method: GsProcess
_frameAt: aLevel offsetOfTempNamed: aString

"For the method activation at level aLevel, returns a SmallInteger which
 is the 1-based offset of the method argument or temporary with name aString.
 Returns 0 if no temp exists at aLevel with name aString.

 Generates an error if aLevel is out of range."

| frameContents aSym tempNames dp |

(aLevel < 1 or:[ aLevel > (dp := self localStackDepth) ]) ifTrue:[
  aLevel _error: #rtErrArgOutOfRange args:{ 1 . dp } .
  ].

aSym := Symbol _existingWithAll: aString .
aSym ifNil:[ ^ 0 ].
frameContents := self _frameContentsAt: aLevel .
tempNames := frameContents at: 9 "virtual machine constant" .
^ tempNames indexOf: aSym
%

category: 'Debugging Support'
method: GsProcess
_frameAt: aLevel tempAt: anOffset put: aValue

"In the method activation at level aLevel, alter the method argument or
 temporary at anOffset to have value aValue.

 Generates an error if aLevel or anOffset is out of range."

| frDesc fpIdx codePtr theMeth nArgs argAndTempNames vcOfs
  theVc argsTmpsOfss absIp lastTmpIdx sd |
(aLevel < 1 or:[ aLevel > (sd := self localStackDepth) ]) ifTrue:[
  aLevel _error: #rtErrArgOutOfRange args:{ 1 . sd } .
].
frDesc := self _frameDescrAt: aLevel .
fpIdx := frDesc at: 2 .
codePtr := arStack at:( fpIdx + FP_codePtr_OFS ).
theMeth := self _fetchMethod: codePtr  .
  nArgs := theMeth numArgs .
argAndTempNames := theMeth  argsAndTemps .
vcOfs := fpIdx + FP_VC_OFS .
theVc := arStack at: vcOfs .
(theVc class == VariableContext) ifFalse:[ theVc := nil ].
argAndTempNames := theMeth  argsAndTemps .
argAndTempNames ifNil:[ argAndTempNames := { } ] .
argsTmpsOfss := theMeth _argsAndTempsOffsets .
anOffset <= nArgs ifTrue:[ | anOfs |
  anOfs := argsTmpsOfss at: anOffset  .
  arStack at:(fpIdx + (anOfs bitShift:-8) ) put: aValue .
  ^ self
].
absIp := (self _ipOffsetFromFrameDescr: frDesc) abs .
absIp <= OC_GSNMETHOD_FIRST_INSTR_OFFSET ifTrue:[
  ArgumentError signal:'method temps not allocated yet' .
  ^ nil
].
lastTmpIdx := argsTmpsOfss size - theMeth _numCopyingBlockArgs .
anOffset <= lastTmpIdx ifTrue:[ | aOfs |
  aOfs := argsTmpsOfss at: anOffset  .
  aOfs < 0 ifTrue:[
    " temp on evaluation stack"
    arStack at: (fpIdx + (aOfs bitShift:-8) ) put: aValue .
    ^ self
  ] ifFalse:[
    aOfs > 0 ifTrue:[ | lev ofs aVc |
      " temp allocated in a VariableContext,
         allow for theVc==nil to handle corrupted stacks during development. "
      ofs := aOfs bitShift: -8 .
      lev := aOfs bitAnd: 16rFF .
      aVc := theVc .
      [ lev > 0 ] whileTrue:[
	aVc := aVc ifNotNil:[ aVc parent ] .
	lev := lev - 1 .
      ].
      aVc ifNotNil:[
         aVc _primitiveAt: (ofs + 1) put: aValue . ^ self
      ] ifNil:[ ArgumentError signal:'theVc not found'. ^ nil ].
    ] ifFalse:[
      ArgumentError signal:'internal error, invalid offset for VC access'.
      ^ nil
    ].
  ].
].
anOffset <= argsTmpsOfss size ifTrue:[ "handle copying Block args"
  | aOfs rcvr |
  rcvr := arStack at:(fpIdx + FP_lastArg_OFS + nArgs ) .
  aOfs := argsTmpsOfss at: anOffset .
  rcvr at: aOfs + 1 put: aValue .
  ^ self
].
ArgumentError signal:'method temp or arg not found' .
^ nil
%

category: 'Formatting'
method: GsProcess
_frameContentsAt: aLevel

"Private.  Returns an Array describing the specified level in the receiver.
 aLevel == 1 is top of stack.  If aLevel is less than 1 or greater than
 stackDepth, returns nil.

 The result Array contains:
 offset item
 -----  -----
   1    gsMethod  (a GsNMethod)
   2    ipOffset    (absolute instruction offset in portable code
                     negative means a stack breakpoint is present)
   3    frameOffset (always nil , not compatible with Gs64 v2.x)
   4    varContext
   5    saveProtectedMode (always nil in v3.0)
   6    markerOrException (always nil in v3.0)
   7    homeMethod if gsMethod is for a block, otherwise nil .
   8    self   (possibly nil in a ComplexBlock)
   9    argAndTempNames   (an Array of Symbols or Strings)
  10    receiver
  11... arguments and temp values, if any"

"Notes to GemStone implementors:

 If result of this method is changed, you must change tpaux.c in the
 topaz C sources, other methods in this class, and the code for primitive 195.

 Note that if execution stops at a breakpoint at the entry to a method,
 the method temporaries may not be allocated yet and so some or all of
 the method temporaries may be missing from the result."

| result fpIdx theMeth theVc argsTmpsOfss ofsIdx frDesc vcOfs
  codePtr theIp absIp |

(aLevel < 1 or:[ aLevel > self localStackDepth ]) ifTrue:[
  ^ nil .
].
frDesc := self _frameDescrAt: aLevel .
frDesc ifNil:[ ^ nil ].
fpIdx := frDesc at: 2 .
result := Array new: 10 "virtual machine constant" .
arStack ifNil:[ ^ nil ].
codePtr := arStack at:( fpIdx + FP_codePtr_OFS ).
theMeth := self _fetchMethod: codePtr  .

result at: 1 put:  theMeth .
theIp := (self _ipOffsetFromFrameDescr: frDesc) .
result at: 2 put: theIp .
absIp := theIp abs .
result at: 3 put: nil.  "previous semantics of frameOffset not usable"

vcOfs := fpIdx + FP_VC_OFS .
arStack ifNotNil:[ theVc := arStack at: vcOfs ].

(Reflection classOf: theVc) == VariableContext ifTrue:[
  result at: 4 put: theVc
] ifFalse: [
  theVc := nil "IP at start of method, VC not allocated yet"
].
"result at: 5 , leave as nil,  saveProtectedMode not available"
theMeth ifNotNil:[
  | homeMeth nArgs receiver theSelf argAndTempNames lastTmpIdx |
  homeMeth := theMeth homeMethod .
  homeMeth ~~ theMeth ifTrue:[ result at: 7 put: homeMeth ].

  nArgs := theMeth numArgs .
  receiver := arStack at:(fpIdx + FP_lastArg_OFS + nArgs ) .
  result at: 10 put: receiver .
  (receiver _isExecBlock and:[theMeth inClass ~~ ExecBlock]) ifTrue:[
    theSelf := receiver selfValue  .
  ] ifFalse:[
    theSelf := receiver
  ].
  result at: 8 "virtual machine constant" put: theSelf.

  argAndTempNames := theMeth  argsAndTemps .
  argAndTempNames size == 0 ifTrue:[ argAndTempNames := { } ] .
  result at: 9 put: argAndTempNames .

  argsTmpsOfss := theMeth _argsAndTempsOffsets .
  lastTmpIdx := argsTmpsOfss size - theMeth _numCopyingBlockArgs .
  ofsIdx := 1 .
  1 to: nArgs do:[:k | | anOfs xofs |
    anOfs := argsTmpsOfss at: ofsIdx  .
    xofs := anOfs bitShift:-8 .
    result addLast:( arStack ifNotNil:[:ars | ars at:(fpIdx + xofs )] ) .
    ofsIdx := ofsIdx + 1 .
  ].
  absIp <= OC_GSNMETHOD_FIRST_INSTR_OFFSET ifTrue:[
    "temps not allocated yet"
    argAndTempNames size > nArgs ifTrue:[
      argAndTempNames size: nArgs
    ].
  ] ifFalse:[ | aOfs tmpVal lastStkTmpIdx calleeLastArgIdx calleeFpIdx tIdx |
    [ ofsIdx <= lastTmpIdx ] whileTrue:[
      aOfs := argsTmpsOfss at: ofsIdx  .
      tmpVal := nil .
      aOfs < 0 ifTrue:[ | idx |
  " temp on evaluation stack"
        idx := fpIdx + (aOfs bitShift:-8) .
        lastStkTmpIdx ifNil:[ lastStkTmpIdx := idx ]
           ifNotNil:[ lastStkTmpIdx := lastStkTmpIdx min: idx ].
  tmpVal := arStack ifNotNil:[:ars | ars at: idx ].
      ] ifFalse:[
  aOfs > 0 ifTrue:[  | ofs lev aVc |
          ofs := aOfs bitShift: -8 .
          lev := aOfs bitAnd: 16rFF .
          aVc := theVc .
          [ lev > 0 ] whileTrue:[
            aVc := aVc ifNotNil:[ aVc parent ] .
            lev := lev - 1 .
          ].
    " temp allocated in a VariableContext,
            allow for theVc==nil to handle corrupted stacks during development. "
    tmpVal := aVc ifNotNil:[ aVc _primitiveAt: (ofs + 1)] ifNil:[ nil]
  ] ifFalse:[
    self error:'illegal offset'.
  ].
      ].
      result addLast: tmpVal .
      ofsIdx := ofsIdx + 1 .
    ].
   "maglev only"
   "[ ofsIdx <= argsTmpsOfss size ] whileTrue:[ //handle copying Block args
      aOfs := argsTmpsOfss at: ofsIdx .
      tmpVal := receiver at: aOfs + 1 .
      result addLast: tmpVal .
      ofsIdx := ofsIdx + 1 .
    ]. "
    calleeFpIdx := frDesc at: 3 .
    calleeLastArgIdx := calleeFpIdx + FP_lastArg_OFS .
    tIdx := 1 .
    lastStkTmpIdx ifNil:[ lastStkTmpIdx := fpIdx + FP_codePtr_OFS ].
    (lastStkTmpIdx - 1) to: calleeLastArgIdx by: -1 do:[:idx |
      tmpVal := arStack ifNotNil:[:ars | ars at: idx ].
      result addLast: tmpVal .
      argAndTempNames add: '.t' , (tIdx asString ) .
      tIdx := tIdx + 1 .
    ].
  ].
].
^ result
%

category: 'Accessing'
method: GsProcess
_frameDescrAt: aLevel

"Private.
 Returns nil if aLevel is less than zero or greater than stackDepth,
 otherwise returns an Array  { aLevel . fpIdx . calleeFpIdx } ,
 where fpIdx is one-based frameOffset at aLevel in the receiver,
 and  where aLevel == 1 is top of stack.
 If aLevel == 1, calleeFpIdx will be zero.

 (self.arStack at: fpIdx) is the savedFP for aLevel.
 if aLevel > 1, (self.arStack at: calleeIdx) is the savedFP for aLevel - 1.
 "
| fpIdx calleeFpIdx cache dp |

(aLevel < 1 or:[ aLevel > (dp := self localStackDepth) ]) ifTrue:[
  ^ nil.
].
(cache := dbgFpsCache) ifNil:[  | ar_size idx ar_stk depth |
  (ar_stk := arStack) ifNil:[ ^ nil ].
  ar_size := ar_stk size .
  fpIdx := ar_size + (self _toWordOfs:topFpOffset) + 1.
  depth := dp .
  cache := Array new: depth .
  cache at: 1 put: fpIdx .
  idx := 1 .
  [ idx < depth ] whileTrue:[ | callerFpOfs |
    idx := idx + 1 .
    callerFpOfs := ar_stk at: fpIdx"+ FP_savedFP_OFS==0" .
    calleeFpIdx := fpIdx .
    fpIdx := ar_size + (self _toWordOfs:callerFpOfs) + 1.
    cache at: idx put: fpIdx .
  ].
  dbgFpsCache := cache .
].
fpIdx := cache at: aLevel .
calleeFpIdx := aLevel > 1 ifTrue:[ cache at: aLevel - 1 ] ifFalse:[ 0 ] .
^ { aLevel . fpIdx . calleeFpIdx }
%

category: 'Accessing'
method: GsProcess
_frameOffsetAt: aLevel

"Private.
 Returns nil if aLevel is less than zero or greater than stackDepth,
 otherwise returns  idx ,
 where idx is one-based frameOffset at aLevel in the receiver,
 and  where aLevel == 1 is top of stack.

 In self.arStack at idx is the savedFP for aLevel."

| arr |
arr := self _frameDescrAt: aLevel .
arr ifNil:[ ^ nil ].
^ arr at: 2
%

category: 'Private'
method: GsProcess
_getEnsureBlocks

"For a GsProcess returns an Array of pairs,
     FP offset,  block that was arg to an ensure:
 for the ensure blocks currently on the stack of the receiver"

<primitive: 826>  "primitive fails if receiver not in stack memory"
| ar_stk pairs fpOfs ar_size |
pairs := { }  .
(ar_stk := arStack) ifNotNil:[  "handle a suspended process not in stack memory"
  fpOfs := topFpOffset .
  ar_size := ar_stk size .
  [ fpOfs < 0 ] whileTrue:[ | fpIdx arOfs anFP |
    fpIdx := ar_size  + (anFP := self _toWordOfs: fpOfs) + 1 .
    arOfs := fpIdx + FP_markerNil_OFS .
    arOfs >= 1 ifTrue:[ | marker aBlk |
      marker := ar_stk at: arOfs .
      marker == OOP_ENSURE_Mark_NIL ifTrue:[
        aBlk := ar_stk at:(fpIdx + FP_lastArg_OFS).
        aBlk _isExecBlock ifTrue:[
          pairs add: anFP ; add: aBlk .
        ] ifFalse:[
          self _halt:'illegal class of argument' .
        ].
      ].
    ].
    fpOfs := ar_stk at: fpIdx"+ FP_savedFP_OFS==0" . "get callerFpOfs"
  ].
].
^ pairs
%

category: 'Process Groups'
method: GsProcess
_group
  "Return the Smalltalk process group (a SmallInteger) the receiver is in,
   assigning to a new group if needed ."
  | g |
  (g := group) ifNil:[ ^ self _newGroup].
  ^ g
%

category: 'Process Groups'
method: GsProcess
_groupOrNil
  "Return the process group the receiver is in,
   or return nil if receiver has not been assigned to a group."

  ^ group
%

category: 'Process Groups'
method: GsProcess
_groupSameAs: anInteger
  "Return true if the receiver's group is the same as anInteger.
   Otherwise return false."

  "Note we don't use 'self _group' because that would initialize
   group which we don't need done for a same as test."

  ^group = anInteger
%

category: 'Debugging Support'
method: GsProcess
_gsiDebuggerDetailedReportAt: aLevel

"If aLevel is less than 1 or greater than stackDepth of the receiver,
 returns nil.  Otherwise, returns an Array containing:

 offset item
 ------ ----
   1    gsMethod  (a GsNMethod)
   2    receiver
   3    self
   4    selector  (a Symbol)
   5    quickStepPoint (SmallInteger index into sourceOffsets, or nil)
   6    sourceOffsets  (Array of SmallIntegers)
   7    argAndTempNames  (Array of Symbols)
   8    argAndTempValues (Array) (may be smaller than argAndTempNames if
   execution halted at entry to a method, may be larger than
         argAndTempNames if compiler has allocated additional
   unnamed stack temporaries. For aLevel == 1, Last element is TOS )
   9    sourceString
  10    ipOffset   in portable code (SmallInteger)
  11    markerOrException  (SmallInteger in a reenter marker,
         or an Exception, or nil )

 The quickStepPoint is the step point in the method, if the method has
 50 or less step points.  Otherwise, the quickStepPoint will be nil
 and the expression

 gsMethod _stepPointForIp: ipOffset level: aLevel
    useNext: (self _nativeStack or:[ self _calleeIsAsync: aLevel])

 may be used to obtain search for and obtain the step point."
| frameContents |

(frameContents := self _frameContentsAt: aLevel) ifNil:[ ^ nil ].
^ self _gsiDetailedReportWithFrameContents: frameContents
             forLevel: aLevel
%

category: 'Debugging Support'
method: GsProcess
_gsiDetailedReportWithFrameContents: frameContents forLevel: aLevel

"Private."
"See _gsiDebuggerDetailedReportAt: for documentation."

| result receiver gsMethod argAndTempNames argAndTempValues aself |

result := Array new: 11 .

result
  at: 1 put: (gsMethod := frameContents at: 1  "virtual machine constant") ; "gsMethod"
  at: 2 put: (receiver := frameContents at: 10 "virtual machine constant" ); "receiver"
  at: 3 put: (aself := frameContents at: 8 "virtual machine constant" ); "self"
  at: 4 put: (gsMethod == nil
    ifTrue: [#'<UserAction>']
    ifFalse: [ gsMethod _selector]); "selector"
  at: 5 put: (gsMethod == nil
    ifTrue: [ 0 ]
    ifFalse: [ gsMethod
      _stepPointForIp: (frameContents at: 2 "virtual machine constant") abs
      level: aLevel useNext: (self _nativeStack or:[ self _calleeIsAsync: aLevel])
    ]) ;
  at: 6 put: (gsMethod == nil
    ifTrue: [ #() ]
    ifFalse: [ gsMethod _blockSourceOffsets ]). "source offsets"

argAndTempNames := frameContents at: 9 "virtual machine constant" .

argAndTempValues := (frameContents size >= 11 "virtual machine constant")
    ifTrue:[ frameContents copyFrom: 11 to: frameContents size]
    ifFalse:[ { }  ].

result  at: 7 put: argAndTempNames ;
        at: 8 put: argAndTempValues ;
        at: 9 put: (gsMethod == nil
          ifTrue: [ 'source not available' ]
          ifFalse: [ gsMethod sourceString  ]);
        at: 10 put: (frameContents at: 2 "virtual machine constant") abs ;  "ipOffset"
        at: 11 put: (frameContents at: 6 "virtual machine constant"). "markerOrException"

^ result
%

category: 'Debugging Support'
method: GsProcess
_gsiStackReportFromLevel: startLevel toLevel: stopLevel

"Returns an Array describing the receiver.  For each stack level in the
 receiver, the result contains 2 elements,  a String describing that
 level, and an Array containing the result of the _gsiDebuggerDetailedReportAt:
 method for that level.

 Level 1 is the top of the stack.  If startLevel is out of range, or if
 stopLevel is less than startLevel, returns nil.  If stopLevel is beyond the end
 of the stack, returns information up to the end of the stack."

|result actualStop frameContents dp |

stopLevel < startLevel ifTrue:[ ^ nil ].
startLevel > (dp := self localStackDepth) ifTrue:[ ^ nil ].

actualStop := (stopLevel > dp )
  ifTrue: [ dp ]
  ifFalse: [ stopLevel ].

result := { } .
startLevel to: actualStop do:[:j|
  frameContents := self _frameContentsAt: j .
  frameContents
    ifNil:[ result addLast: nil; addLast:nil ]
    ifNotNil:[ | frameStr |
      frameStr := String new .
      frameStr addAll: (self _reportWithFrameContents: frameContents level: j) .
      result addLast: frameStr ;
             addLast:(self _gsiDetailedReportWithFrameContents: frameContents
                            forLevel: j ) .
      ].
  ].
^ result
%

category: 'Private'
method: GsProcess
_init: aBlock args: aArgs env: environmentId stackSerial: serialNum forked: forkedBit
  "returns receiver.
   caller must _scheduleNewProcess or _suspendNewProcess "
  stackId := -1 .
  "scheduler will set priority_use to match active process"

  modeInfo := ModeInfo_debugEnabled bitOr: forkedBit .
  block := aBlock.
  blockResult := _remoteNil .
  args := aArgs.
  msgEnvironmentId := environmentId .
  serialNum _isSmallInteger ifFalse:[
      InternalError new details:'GsProcess invalid stackSerialNum' ; signalNotTrappable.
  ].
  stackSerialNum := serialNum .
  environmentId == 0 ifTrue:[ | asz bNargs |
    (asz := args size) ~~ (bNargs := block argumentCount) ifTrue: [
      "Generate an error because we don't have the correct number of args."
      OutOfRange new _number: 2373; expectedNumArgs: bNargs actual: asz ;
     object: block ; details:'wrong number of arguments for invoking a block' ;
          signal.
      ^ nil
    ].
  ].
  debugActionBlock := GsProcess _current debugActionBlock . "inherit from parent process"
  startSeconds := System _timeGmtFloat - 978307200.0 . "aSmallDouble, secondsSince2001"
  parentProcess := self _current .
  "self _setClientData . debugging only"
%

category: 'Accessing'
method: GsProcess
_ipOffsetAt: aLevel

"return the portable code IP offset for the specified frame"
^ self _ipOffsetFromFrameDescr: (self _frameDescrAt: aLevel)

%

category: 'Accessing'
method: GsProcess
_ipOffsetFromFrameDescr: frDescr

"return the portable code IP offset for frame described by
 frDescr.  frDescr must be non-nil result from _frameDescrAt:

 if receiver is a stack from native code, result maybe approximate
"

| ipOff aLevel notTopFrame |
aLevel := frDescr at: 1.
(notTopFrame := aLevel >  1) ifTrue:[ | calleeFpIdx |
  calleeFpIdx := frDescr at: 3 .
  ipOff := arStack at:( calleeFpIdx + FP_rtnAddr_OFS ) .
] ifFalse:[
  ipOff := arStack at: 1 "savedIP at TOS" .
].
(ipOff == nil or:[ ipOff < 0]) ifTrue:[
  "due to biasing of return addresses for native code, can have negative
   offset for start of method, so treat as zero for stack display.
   can have nil for reenter markers. "
  ipOff := 0
].
self _nativeStack ifTrue:[ | fpIdx meth anIp |
  fpIdx := frDescr at: 2 .
  meth := self _fetchMethod:( arStack at:( fpIdx + FP_codePtr_OFS )).
  " arg to asReturn: is false here because callers (or topaz) are using one of
      GsProcess>>_StepPointAt:
      GsNMethod>>_stepPointForIp:level:useNext:
    to convert the IP to a step point
  "
  anIp := meth _nativeIpOffsetToPortable: ipOff asReturn: false .
  anIp <= 0 ifTrue:[
    anIp < -1 ifTrue:[
      anIp := anIp negated.  "approximate result"
    ] ifFalse:[
      self error:'ipOff=', anIp asString, ' out of range' .
    ].
  ].
  ipOff := anIp .
].
^ ipOff .
%

category: 'Private'
method: GsProcess
_isRubyThread
  ^ (modeInfo bitAnd: ModeInfo_isRubyThread ) ~~ 0
%

category: 'Private'
method: GsProcess
_isTerminated

  "returns true if the receiver has been terminated."

  ^ (modeInfo bitAnd: ModeInfo_terminated ) ~~ 0
%

category: 'Process Groups'
method: GsProcess
_joinGroup: aGroup
  "Have the receiver join the specified processor group. Return self.
   For Smalltalk,  aGroup is an Integer,
   For Ruby aGroup is a RubyThreadGroup .
  "
  group := aGroup .
%

category: 'Debugging Support'
method: GsProcess
_localStepPointAt: aLevel

"Used by topaz debugger,
 returns an array { stepPoint . aGsNMethod }
 or nil if no step point available "

| fpIdx theMeth  ipOffset stepPt frDesc dp |
(aLevel < 1 or:[ aLevel > (dp := self localStackDepth) ]) ifTrue:[
  aLevel _error: #rtErrArgOutOfRange args:{ 1 . dp } .
  ].
(frDesc := self _frameDescrAt: aLevel) ifNil:[ ^ nil ].
fpIdx := frDesc at: 2 .
theMeth := self _fetchMethod:( arStack at:( fpIdx + FP_codePtr_OFS)) .
theMeth ifNil:[ ^  nil  ].
ipOffset := self _ipOffsetFromFrameDescr: frDesc .
stepPt := theMeth _stepPointForIp: ipOffset level: aLevel
        useNext: (self _nativeStack or:[ self _calleeIsAsync: aLevel]) .
stepPt ifNil:[ ^ nil ].
^ { stepPt . theMeth }
%

category: 'Debugging Support'
method: GsProcess
_methodInFrameContents: aFrameContentsArray

^ aFrameContentsArray at: 1 "virtual machine constant"
%

category: 'Private'
method: GsProcess
_nativeStack

  "returns true if the receiver represents execution in native code.

   Only usable on instances created by a VM stack save. "

  ^ ((modeInfo bitAnd: ModeInfo_nativeStack_mask ) bitShift: 0 - ModeInfo_nativeStack_shift) > 1"GC_GSPROCESS_cpuKind_interp"
%

category: 'Process Groups'
method: GsProcess
_newGroup
  "Add the receiver to a new Smalltalk process group. Return the group value."
  | g |
  g := self _scheduler _newGroup.
  group := g .
  ^ g
%

category: 'Private'
method: GsProcess
_onQueue: anObj
  " for use by ProcessorScheduler only"
  onQueue := anObj
%

category: 'Private'
method: GsProcess
_primContinue: argUnused

<primitive: 2012>
"no image code here, compiler emits Bc_RETURN_TOS here"
%

category: 'Private'
method: GsProcess
_primStep: breakpointsToIgnore from: argUnused with: saveArray mode: raiseExcept

<primitive: 2011>
^ self  "normal execution path"
%

category: 'Updating'
method: GsProcess
_raisePriority
  "Raises priority of receiver to  ((priority +2) max: 40) .
   Returns the previous priority of receiver.  "
  | old p won |
  old := self priority .
  old <= 20 ifTrue:[   "inline highestPriority logic"
    p := old + 20
  ] ifFalse:[
    p := 40
  ].
  p ~~ old ifTrue:[
    self _setPriority: p .
    self _scheduler _changePriority: self from: p .
    (won := waitingOn) ifNotNil:[
      won _changePriority: self from: old .
    ]
 ].
 ^ old
%

category: 'Private'
method: GsProcess
_reapSignal: signalSource
  "Make the receiver ready to run."
  "caller enters critical region"

  waitingOn := nil .
  self _scheduler _scheduleProcess: self.
%

category: 'Debugging Support'
method: GsProcess
_receiverInFrameContents: aFrameContentsArray

^ aFrameContentsArray at: 10 "virtual machine constant"
%

category: 'Private'
method: GsProcess
_removeEnsureAtFP: anFP

 "Dereference the ensure block in specified frame,
  so it won't be executed more than once.
  Frame changed  from  ensure:[] to  ensure: nil ."

 <primitive: 958>
 "handle receiver not in stack memory "
 | ar_stk ar_size fpIdx arOfs |
 anFP _isSmallInteger ifTrue:[
   ar_stk := arStack .
   ar_size := ar_stk size .
   fpIdx := ar_size  + anFP + 1.
   arOfs := fpIdx + FP_markerNil_OFS .
   arOfs >= 1 ifTrue:[ | marker |
     marker := ar_stk atOrNil: arOfs .
     marker == OOP_ENSURE_Mark_NIL ifTrue:[
      ar_stk at:(fpIdx + FP_lastArg_OFS) put: nil .
      ^ self
     ]
   ]
 ].
 self _primitiveFailed: #_removeEnsureAtFP: args: { anFP }
%

category: 'Private'
method: GsProcess
_removeJoiner: aDelay

| js ofs |
(js := joiners ) ifNotNil:[
  ofs := (js := joiners ) indexOfIdentical: aDelay .
  ofs ~~ 0 ifTrue:[ js at: ofs put: nil ].
].
%

category: 'Debugging Support'
method: GsProcess
_reportAt: aLevel

"Return a one line String describing the specified level in the receiver.
 aLevel == 1 means top of stack with respect

 If aLevel is out of range,  nil is returned. "

| fc isCurr |
(isCurr :=  self == GsProcess _currentOrNil) ifTrue:[
   (fc := GsProcess _frameContentsAt: aLevel ) ifNil:[ ^ nil ].
   fc at: 8 put: (GsProcess _selfAt: aLevel) . "get self"
] ifFalse:[
   (fc := self _frameContentsAt: aLevel) ifNil:[ ^ nil ].
].
^ self _reportWithFrameContents: fc level: aLevel  .
%

category: 'Debugging Support'
method: GsProcess
_reportOfSize: aSize

"Returns an Array describing the receiver up to aSize entries in length.  Each
 element in the result is a String produced by GsProcess | _reportAt:."

| result dp |
result := { }  .
dp := self == GsProcess _currentOrNil ifTrue:[ System stackDepth ]
                                ifFalse:[ self stackDepth ].
1 to: (dp min: aSize) do:[:j |
   (self _reportAt: j) ifNotNil:[:r | result add: r ]
].
^ result
%

category: 'Debugging Support'
method: GsProcess
_reportString

"Returns a String with each Activation object in the receiver, up to 2000
 levels, described on separate lines."

| s arr |
s := String new.
s add: self class name.
s add: '(oop=' ; add: self asOop printString ; add: ', ' .

s add: 'status=' ; add: (self _statusString) ; add: ', '.

clientData ifNotNil:[
  s add: 'clientData=' ; add: (clientData printString) ; add: ', '.
].

s add: 'priority=' ; add: (self priority printString) ; add: ', '.

group ifNotNil:[
  s add: 'group=' ; add: (group printString) ; add: ', '.
].

block ifNotNil:[
  s add: 'block=' ; add: (block printString) ; add: ', '.
  s add: 'args=' ; add: (args printString) ; add: ', '.
].
s lf .

arr := self _reportOfSize: 2000.
arr size > 0 ifTrue:[
  1 to: arr size do:[:j | s add: '   ' ; add: (arr at: j) ; lf ].
  s add: $) ; lf .
] ifFalse:[
  s add: $)
].
^ s
%

category: 'Debugging Support'
method: GsProcess
_reportWithFrameContents: frameContents level: aLevel
"Return a one line String describing the argument.
 Example result:
   SmallInteger (Object) >> doesNotUnderstand:  @9 line 10 "

| gsMethod receiver aself result ip stepPt |

gsMethod := self _methodInFrameContents: frameContents .
receiver := self _receiverInFrameContents: frameContents .

aself := self _selfInFrameContents: frameContents .

gsMethod ifNil:[ ^ self == GsProcess _currentOrNil ifTrue:['GCI'] ifFalse:['User Action' ]].

result := String new .
result add: (gsMethod _descrForStackPadTo: 0 rcvr: receiver) .
ip := (frameContents at: 2) abs .
stepPt := gsMethod _stepPointForIp: ip level: aLevel
        useNext: (self _nativeStack or:[ self _calleeIsAsync: aLevel]) .
result add: ' @' ; add: stepPt asString ;
       add: ' line ' ;
       add: (gsMethod _lineNumberForStep:  stepPt) asString .
^ result
%

category: 'Private'
method: GsProcess
_scheduler

"Returns   ProcessorScheduler scheduler "
<primitive: 457>

self _primitiveFailed: #_scheduler
%

category: 'Debugging Support'
method: GsProcess
_selfInFrameContents: aFrameContentsArray

^ aFrameContentsArray at: 8 "virtual machine constant"
%

category: 'Private Debugging Support'
method: GsProcess
_setBreaksForStepInto

"Set breakpoints so that a subsequent IntContinue on the receiver will
 execute a step-into.  step-into across a fork of a new GsProcess
 is not supported.  (The step breakpoint in the forked block will
 be ignored since it not in the current GsProcess).

 For use only from within implementation of GciStep.

 Return a 3 element Array,
     { <the GsNMethod in which single-step breaks were set> .
       <the level at which stack break was set, or zero> .
       <a SmallInteger, number of breakpoints to ignore> }

Algorithm
Step Into (is always from TOS == level 1)
  1. set all step breakpoints in TOS.codeOop
  2. if next instruction to execute is BC_CALL_PRIMITIVE,
      set all step breakpoints in level 2 and set stack breakpoint in level 3
    else
      set stack breakpoint at level 2
"

| tosFrame aMethod stackBreakLevel result breakpointsToIgnore ipOfs |

self convertToPortableStack .

result := Array new: 3 .
tosFrame := self _frameContentsAt: 1 .
aMethod := tosFrame at: 1 "virtual machine constant" .
ipOfs := tosFrame at: 2 "virtual machine constant" "ipOffset" .
(aMethod _opcodeAtIsCallPrimitive: ipOfs) ifTrue:[
  "next instruction to be executed is a call-primitive"
  aMethod := self localMethodAt: 2 .
  aMethod _setAllStepBreaks: true frame: nil process: self .
  result at: 1 put: aMethod.
  breakpointsToIgnore := 0.
  stackBreakLevel := 3  .
] ifFalse:[
  "next instruction is not a primitive call"
  aMethod _setAllStepBreaks: false frame: nil process: self .
  result at: 1 put: aMethod .
  breakpointsToIgnore := 0 .  "fix 41539"
  stackBreakLevel := 2 .
].
           " use <  in following compare to not set stack break in _gsReturnToC"
stackBreakLevel < self stackDepth
  ifTrue:[ self _setStackBreakAt: stackBreakLevel ]
  ifFalse:[ stackBreakLevel := 0 "no stack breaks to set" ].
result at: 2 put: stackBreakLevel .
result at: 3 put: breakpointsToIgnore.
^ result
%

category: 'Private Debugging Support'
method: GsProcess
_setBreaksForStepOverFromLevel: flags

"Set breakpoints so that a subsequent IntContinue on the receiver will
 execute a step-over.
 For use only from within implementation of GciStep.
 Return an Array, { <the GsNMethod in which single-step breaks were set> .
        <the level at which stack break was set, or zero> .
                    <number of breakpoints to ignore> }."
| aFrame aMethod stackBreakLevel result breakpointsToIgnore aLevel thruBool fpOffset
  trc doTrace |
doTrace ifNotNil:[ trc := String new ].
self convertToPortableStack .
result := Array new: 3 .
aLevel :=    flags bitAnd:  16rFFFFFFFF .
thruBool := (flags bitAnd: 16r100000000) ~~ 0 .
aFrame := self _frameContentsAt: aLevel .
aMethod := aFrame at: 1 "virtual machine constant" .
fpOffset := self _toWordOfs: ( aLevel == 1 ifTrue:[ topFpOffset ]
                       ifFalse:[ ((self _frameDescrAt: aLevel) at: 3) ]).
aLevel >= 2 ifTrue:[
  stackBreakLevel := aLevel + 1.
  breakpointsToIgnore := 0 .
  aMethod _setStepOverBreaks: false frame: nil process: self stepThrough: thruBool .
  doTrace ifNotNil:[ trc add: $A ].
] ifFalse:[ "aLevel is 1" | hmm |
  "set breaks in TOS method, excluding first step point"
  aMethod _setStepOverBreaks: false frame: fpOffset process: self stepThrough: thruBool .
  result at: 1 put: aMethod .
  breakpointsToIgnore := 0 .  "no special logic after lastBreakpt instVar added"
  (thruBool and:[ (hmm := aMethod homeMethod) ~~ aMethod ]) ifTrue:[ | depth j |
     depth := self localStackDepth.
     j := 2 .
     stackBreakLevel := 0 .
     [ j < depth ] whileTrue:[
       ((self _frameContentsAt: j) at: 1) == hmm ifTrue:[
          stackBreakLevel := j . "stack break for return to/across home method"
          j := depth "break out of whileTrue" . doTrace ifNotNil:[trc add: $B ].
       ].
       j := j + 1 .
     ].
  ] ifFalse:[
    stackBreakLevel := aLevel + 1 . doTrace ifNotNil:[trc add: $C ].
  ]
].
     " use <  in following compare to not set stack break in _gsReturnToC"
doTrace ifNotNil:[
 GsFile gciLogServer: 'dp ' , self localStackDepth asString ,
        ' stackBreakLevel ' , stackBreakLevel asString ].
(stackBreakLevel > 0 and:[ stackBreakLevel <  self localStackDepth ])
  ifTrue:[ self _setStackBreakAt: stackBreakLevel . doTrace ifNotNil:[trc add: $D] ]
  ifFalse:[ stackBreakLevel := 0 "no stack breaks to set" . doTrace ifNotNil:[trc add: $E] ].
doTrace ifNotNil:[
  GsFile gciLogServer:
  'level ' , aLevel asString , ' aMethod ' , aMethod asOop asString ,
   ' hmm ' , aMethod homeMethod asOop asString ,
  ' breakpointsToIgnore ' , breakpointsToIgnore asString ,
	' stackBreakLevel ' , stackBreakLevel asString , ' trc ' , trc ].
result at: 3 put: breakpointsToIgnore .
result at: 2 put: stackBreakLevel .
^ result
%

category: 'Private'
method: GsProcess
_setInterpretedStack

  | mask val |
  mask := ModeInfo_nativeStack_mask bitInvert .
  val :=  GC_GSPROCESS_cpuKind_interp bitShift: ModeInfo_nativeStack_shift  .
  modeInfo := (modeInfo bitAnd: mask) bitOr: val  .
%

category: 'Private'
method: GsProcess
_setModeinfoBit: aMask value: aValue

  "Sets bit specified by aSmallInteger in  the modeInfo instVar.
   aMask must be a SmallInteger, one of
     ModeInfo_terminated - set terminated and terminationStarted bits ;
       if the process has an in-memory stack, unlocks it and clears its use count;
       stores nil in instVars arStack,clientData,ioSelectResult .
       if isRubyThread bit == 0, stores nil in  environment instVar.
     ModeInfo_terminationStarted - sets specified bit
     ModeInfo_isRubyThread  - sets specified bit
     ModeInfo_breakptsIgnore_mask - sets specified bit field.
     ModeInfo_debugEnable - sets specified bit field

   aValue must be a SmallInteger >= 0 , and not larger than the specified bit field.
  "
  <primitive: 708>
  aMask _validateClass: SmallInteger .
  aValue _validateClass: SmallInteger .
  self _primitiveFailed: #_setModeinfoBit:value: args: { aMask . aValue }
%

category: 'Private'
method: GsProcess
_setPriority: anInt
  "Update priority of the receiver, and of receiver's in-memory
   stack if the receiver  has one.
  "
  <primitive: 706>
  anInt _validateClass: SmallInteger .
  (anInt < 1 or:[ anInt > 40]) ifTrue:[
     anInt _error: #rtErrArgOutOfRange args:{ 1 . 40 } ].
  self _primitiveFailed: #_setPriority: args: { anInt }
%

category: 'Private Debugging Support'
method: GsProcess
_setStackBreakAt: aLevel

"Sets a stack breakpoint that is hit when a return from a
 block would return into or across the specified level on the stack.
 When a stack breakpoint is hit, the breakpoint is automatically cleared,
 the return from block is executed, and execution stops with a breakpoint
 error.  If both an unwind block (as installed by ensure:)
 and a stack breakpoint are present at the same level in the stack
 the stack breakpoint will be cleared and execution will stop with a
 breakpoint error before executing the unwind block."

| ipOffset dp |
self convertToPortableStack .

" disallow stack break at aLevel == self stackDepth
  to prevent setting set stack break in _gsReturnToC"

(aLevel < 1 or:[ aLevel >= (dp := self stackDepth) ]) ifTrue:[
  aLevel _error: #rtErrArgOutOfRange args:{ 1 . dp - 1 } .
  ].
(self __returnAddrIdxAt: aLevel) ifNotNil:[ :rtnAddrIdx |
  ipOffset := arStack at: rtnAddrIdx .
  arStack at: rtnAddrIdx put: ipOffset abs negated
].
%

category: 'Private'
method: GsProcess
_signalAll
  "Wake up the receiver"  "caller is in critical region"

  waitingOn := nil .
  self _scheduler  _scheduleProcess: self.
%

category: 'Private'
method: GsProcess
_signalJoiners
| js sched wasInCrit |
js := joiners .
js ifNotNil:[
  joiners := nil .
  sched := self _scheduler.
  wasInCrit := sched _enterCritical .
  1 to: js size do:[:n | | aDelay |
    aDelay := js at: n .
    aDelay ifNotNil:[
      sched _delayUnschedule: aDelay .  "Inline Delay>>signal"
      aDelay _activate
    ].
  ].
  wasInCrit == 0 ifTrue:[ sched _exitCritical ].
].
%

category: 'Private'
method: GsProcess
_signalTime
  ^ signalTime
%

category: 'Private'
method: GsProcess
_signalTime: time

  signalTime := time.
%

category: 'Debugging Support'
method: GsProcess
_stackReport
  "For use with topaz"
  ^ self stackReportToLevel: 1000 withArgsAndTemps: false andMethods: true
		includeSource: false
%

category: 'Debugging Support'
method: GsProcess
_stackReportIncludeSourceFor: gsMethod inclSrc: inclSrc

  "inclSrc is a SmallInteger produced by
     stackReportToLevel:withArgsAndTemps:andMethods:includeSource:lineLimit:  .
   Returns true if source should be displayed."

  | cls sel |
  inclSrc <= 0 ifTrue:[ ^ false "none" ].
  inclSrc >= 3 ifTrue:[ ^ true  "all"  ].
  inclSrc == 1 ifTrue:[ ^  gsMethod inClass == nil  "anonymous methods"].

  "inclSrc == 2 "
  (cls := gsMethod inClass ) ifNil:[ ^ true "an anonymous method"].
  sel := gsMethod isMethodForBlock ifTrue:[ gsMethod homeMethod selector ]
                                ifFalse:[ gsMethod selector ].
  sel ifNil:[ ^ true "failsafe" ].
  (cls compiledMethodAt: sel environmentId: gsMethod environmentId otherwise: nil usePackages: true)
      ifNotNil:[ ^ false "exists in a method dict, don't report sources"].
  ^ true  "block from block execution, etc"
%

category: 'Private'
method: GsProcess
_stackSerialNum
  ^ stackSerialNum
%

category: 'Private'
method: GsProcess
_start
  "Called from C to start a new process.
   NOTE, exception handling differs in extent0.dbf and extent0.ruby.dbf .
   This version of _start from extent0.dbf .

   This method will never return. Once the receiver completes it should
   find another process to run.
   This method is preloaded during VM startup.  You must logout/login
   to have any changes to this method take effect."

  | res curr |
  (modeInfo bitAnd: ModeInfo_continuationMask) ~~ 0 ifTrue:[
    self error:'cannot start an instance that is a continuation'
  ].
  block ifNotNil:[
    res := block valueWithArguments: args
  ].
  "if this process resumed a continuation, then current process may
   no longer be self at this point."
  curr := self _terminateCurrentWithResult: res . "terminate currently running process"
  joiners ifNotNil:[
    curr == self ifTrue:[ self _signalJoiners ]
               ifFalse:[ self error:'cannot _signalJoiners after resuming continuation'].
  ].
  self _scheduler _runNextProcess .
%

category: 'Private'
method: GsProcess
_statusString
  "Returns a string that describes the receiver's status as a thread."

  ^ self _scheduler _statusString: self.
%

category: 'Private'
method: GsProcess
_stepCleanup: saveArray
  "Legacy code, no longer used."
  | newStackDepth flags |

  self _storeBit: 1 value: false . "debuggingInProgress:=0"

  newStackDepth := self stackDepth.

  self _clearAllStackBreaks .
  GsNMethod _clearAllStepBreaks .

  "clear INT_STEP_INTO_FROM_TOS_MASK "
  flags := self interruptFlag .
  flags := flags bitAnd: (16r200 bitInvert).
  self setInterruptFlag: flags .
%

category: 'Private Debugging Support'
method: GsProcess
_stepOverFromLevel: flags
  "  (flags bitAnd:  16rFFFFFFFF) is the stack level to step from .
     (flags bitAnd: 16r100000000) is the step through boolean ."
  self == GsProcess _currentOrNil ifTrue:[
    ^ Error signal:'cannot single-step the currently running GsProcess'
  ].
  self localStackDepth . "force to object memory"
  self convertToPortableStack .

  "No longer needed, VM cleared stack and step BPs before signalling
   a single step Breakpoint."
    "self _clearAllStackBreaks . "
    "GsNMethod _clearAllStepBreaks ."

  "clear any leftover INT_STEP_INTO_FROM_TOS_MASK"
  self setInterruptFlag: (self interruptFlag bitAnd:(16r200 bitInvert)) .

  self _setBreaksForStepOverFromLevel: flags .
%

category: 'Private Debugging Support'
method: GsProcess
_stepOverInFrame: flags
    "Used by GBS , which expects breakpoint and step point errors
     to be returned to the GCI.
     Do not use when implementing in-process debugging.
     Step execution so that it stops after the next message send
     in the specified stack level.  If a return causes the
     context at the specified level to be removed from the
     stack, execution will stop immediately after that return.
     Exceptions for step-point and breakpoint errors are suppressed.
     Returns the receiver unless the receiver completes in which
     case the result of the completion is returned.

     (flags bitAnd:  16rFFFFFFFF) is the stack level to step from .
     (flags bitAnd: 16r100000000) is the step through boolean .
    "

  ^ self _stepOverInFrame: flags mode: false replace: false tos: nil
%

category: 'Private'
method: GsProcess
_stepOverInFrame: flags mode: raiseException replace: replaceTosBool tos: newTosValue
    "Used in the implementation of GciStep and within the server
     image. topaz no longer uses this directly.
     Do not use when implementing in-process debugging.
     Step execution so that it stops after the next message send
     in the specified stack level.  If a return causes the
     context at the specified level to be removed from the
     stack, execution will stop immediately after that return.
     If 'raiseException' then debugger exceptions are raised
     so that a GCI app can catch them.
     Returns the receiver unless the receiver completes in which
     case the result of the completion is returned.
     Execution of this method must be in interpreted mode.

     flags == 0 is stepInto semantics.
     flags == 1 is typical step over in top frame .
     flags == 16r100000001 is typical step through (stopping in blocks of the method)
     (flags bitAnd:  16rFFFFFFFF) is the stack level to step from .
     (flags bitAnd: 16r100000000) is the step through boolean .
    "

  | actualLevel result saveArray breakpointsToIgnore
    mySaveArray saveStackDepth status sched dp currProc doTrace wasInCrit |
  self isContinuation ifTrue:[
    Error signal:'cannot use stepping to resume a continuation' .
  ].
  self _checkIfDebuggable .

  self convertToPortableStack .
  self _nativeStack ifTrue:[
    Error signal:'unable to convert GsProcess to be stepped to portable stack'
  ].
  (currProc := GsProcess _currentOrNil) _nativeStack ifTrue:[ | cvtProc |
    cvtProc := [ currProc convertToPortableStack ] fork .
    [ cvtProc _isTerminated ] whileFalse:[
      self _scheduler yield .
    ].
    currProc _nativeStack ifTrue:[
      Error signal:'unable to convert current GsProcess to portable for stepping another GsProcess'.
    ].
  ].
  sched := self _scheduler .
  wasInCrit := sched _enterCritical .
  status := self _statusString.
  (status = 'ready') ifTrue: [
    self _unscheduleProcess .
  ].
  dp := self localStackDepth .
  flags < 0 ifTrue:[
    flags _error: #rtErrArgOutOfRange args:{ 0 . dp } .
  ].
  actualLevel := flags bitAnd:  16rFFFFFFFF .
  (actualLevel > dp) ifTrue:[
    actualLevel _error: #rtErrArgOutOfRange args:{ 0 . dp } .
  ].
  saveStackDepth := dp .
  (actualLevel == 0) ifTrue: [  | interrFlags |
    saveArray := self _setBreaksForStepInto.
    breakpointsToIgnore := (saveArray at: 3).
    "set INT_STEP_INTO_FROM_TOS_MASK bit"
    interrFlags := self interruptFlag .
    interrFlags := flags bitOr: 16r200.
    self setInterruptFlag: interrFlags .
  ] ifFalse: [
    saveArray := self _setBreaksForStepOverFromLevel: flags .
    breakpointsToIgnore := (saveArray at: 3).
        	" deleted INT_STEP_OVER_FROM_TOS_MASK logic "
  ].

  (replaceTosBool) ifTrue: [
    mySaveArray := { (saveArray at: 1) . (saveArray at: 2) . saveStackDepth .
                       newTosValue }.
  ] ifFalse: [
    mySaveArray := { (saveArray at: 1) . (saveArray at: 2) . saveStackDepth }.
  ].

  self _storeBit: 1 value: true . "debuggingInProgress:=1"
  wasInCrit == 0 ifTrue:[ sched _exitCritical ].

  doTrace ifNotNil:[
    GsFile gciLogServer:'breakpointsToIgnore ' , breakpointsToIgnore asString ].
  result := self _primStep: breakpointsToIgnore
                 from: nil with: mySaveArray mode: raiseException.

  wasInCrit := sched _enterCritical .
  (status = 'ready') ifTrue: [
    (result == self) ifTrue: [
      (self _isTerminated) ifFalse: [
        sched _scheduleProcess: self .
      ].
    ].
  ].
  wasInCrit == 0 ifTrue:[ sched _exitCritical ].
  "_stepCleanup may run a method that was stepped into (Object>>at:), so run with
   #_performNoDebug:with: to ensure that the cleanup is performed"
  self _performNoDebug: #_stepCleanup:  env: 0  with: mySaveArray.

  ^result
%

category: 'Private Debugging Support'
method: GsProcess
_stepOverInFrame: flags return: anObject
    "Used by GBS , which expects breakpoint and step point errors
     to be returned to the GCI.
     Do not use when implementing in-process debugging.
     Step execution so that it stops after the next message send
     in the specified stack frame.  If a return causes the
     context at the specified frame to be removed from the
     stack, execution will stop immediately after that return.
     The argument anObject will be the return value of the
     current top of stack - useful for passing back the return
     value of a client forwarder send.
     Returns the receiver unless the receiver completes in which
     case the result of the completion is returned.
     Execution of this method must be in interpreted mode.

     (flags bitAnd:  16rFFFFFFFF) is the stack level to step from .
     (flags bitAnd: 16r100000000) is the step through boolean .
    "
  ^ self _stepOverInFrame: flags mode: false replace: true tos: anObject
%

category: 'Debugging Support'
method: GsProcess
_stepPointAt: aLevel

"Used by topaz debugger,
 returns a stepPoint , or nil if no step point available "

^ (self _localStepPointAt: aLevel) at: 1
%

category: 'Debugging Support'
method: GsProcess
_stepPointStringAt: aLevel

"Used by topaz debugger."

| stepPtArr result stepPoint theMeth |
stepPtArr := self _localStepPointAt: aLevel .
stepPtArr ifNil:[ ^ '' ].

stepPoint := stepPtArr at: 1 .
theMeth := stepPtArr at: 2 .
result := String withAll:'@' .
result addAll: stepPoint asString ;
    addAll: ' line ' ;
    addAll: (theMeth _lineNumberForStep: stepPoint) asString .
^ result

%

category: 'Private'
method: GsProcess
_storeBit: bitNum value: aValue
  "  aValue must be true,false, or nil(bitNum 0 only)
   bitNum 0 ==  set/clear  lockedInMemory returns previous value ;
               aValue == nil means no change and return current value.
   bitNum 1 ==  store  debuggingInProgress, per aValue
                returns receiver ;
   bitNum 2 == set/clear THREAD_SIGNAL_EXC interrupt bit
        returns previous value, for non-current process only.
   bitNum 3 ==  set PROC_TERMINATE_MASK interrupt bit for non-current process

   The primitive accesses/updates either C memory state or object memory
   as required by whether process is in stack memory or not.
 "
  <primitive: 707>

  self _primitiveFailed: #_storeBit:value: args: { bitNum . aValue }
%

category: 'Private'
method: GsProcess
_targetProcess
  "Returns the GsProcess that is waiting for the receiver."

  ^ self
%

category: 'Private'
method: GsProcess
_terminateCurrentWithResult: aResult

"Sets terminated and terminationStarted bits in the currently running
  process (which may be different from the receiver),
  stores the result, and clears its waitingOn, and block.
  For the current in-memory stack, unlocks it and clears its use count.
  Stores nil in instVars arStack,clientData,ioSelectResult .
  If modeInfo.isRubyThread bit == 0, stores nil in  environment instVar.

  Returns currently running process.
"

  <primitive: 720>
  self _primitiveFailed: #_terminateCurrentWithResult: args: { aResult }
%

category: 'Private'
method: GsProcess
_terminationStarted

  "returns true if receiver has started executing ensure: blocks"
  ^ (modeInfo bitAnd: ModeInfo_terminationStarted ) ~~ 0
%

category: 'Private Debugging Support'
method: GsProcess
_topazDescrForFrame: frameId padTo: minSize
  "Used by topaz stack display code .
   Returns nil if frameId is out of range, or an Array
    { aGsNMethod . aString } .
   aGsNMethod in the result is nil if frame is a reenter marker.
  "
  | frame aMethod |
  frame := self _frameContentsAt: frameId .
  frame ifNil:[ ^ nil ].
  aMethod := frame at: 1 .
  aMethod ifNil:[ ^ { nil . 'reenter marker' } ].
  ^ { aMethod .
      aMethod _descrForStackPadTo: minSize rcvr: (frame at: 10 "rcvr")
    }
%

category: 'Debugging Support'
method: GsProcess
_topazMethodAt: aLevel

  "Returns nil if aLevel is out of range"

| fpIdx dp |
(aLevel < 1 or:[ aLevel > (dp := self localStackDepth) ]) ifTrue:[
  ^ nil
  ].
fpIdx := self _frameOffsetAt: aLevel .
fpIdx ifNil:[ ^ nil].
^ self _fetchMethod:( arStack at: (fpIdx + FP_codePtr_OFS )) .
%

category: 'Private Debugging Support'
method: GsProcess
_topazStepOverInFrame: flags
  "Used by topaz which
   expects breakpoint and step point errors to be returned to the GCI .
   Do not use when implementing in-process debugging.

   Step execution so that it stops after the next message send
   in the specified stack level.  If a return causes the
   context at the specified level to be removed from the
   stack, execution will stop immediately after that return.
   Exceptions for step-point and breakpoint errors are NOT suppressed.
   If the receiver completes result of the completion is returned.

   (flags bitAnd:  16rFFFFFFFF) is the stack level to step from .
    (flags bitAnd: 16r100000000) is the step through boolean."

  ^ self _stepOverInFrame: flags mode: true replace: false tos: nil
%

category: 'Private'
method: GsProcess
_toWordOfs: aByteOffset

^ aByteOffset bitShift: -3
%

category: 'Debugging Support'
method: GsProcess
_trimStackToLevel: aLevel

"Deletes stack entries from 1 to (aLevel - 1) inclusive, thus making aLevel the
 new top-of-stack(TOS).  At new TOS, looks in the implementation class of the
 method at TOS, using the selector of the method at TOS.  If that class's method
 dictionary contains a different GsNMethod, the GsNMethod currently in the method
 dictionary is installed as the method at the TOS.  The saved instruction
 pointer for TOS is set to the entry of the method at TOS.

 Limitations:
   If the receiver was executing in native code, and the stack
   is trimmed, any resumption of execution will be in interpreted mode.

   If the new top-of-stack is an anonymous method, it is not possible to
   determine whether that method has been recompiled, and the method at new top
   of stack will not be changed.  Debuggers should use the
   GsNMethod | _recompileWithSource method to implement recompilation from a
   debugger pane.  _recompileWithSource: raises an error on attempts to
   recompile an anonymous method.

   Raises an error if the new top-of-stack would represent the entry to an
   ExecBlock and the home method of the block has been recompiled.

   Raises an error if the new top-of-stack would represent a return
   across an FFI , Ruby C extension, or UserAction call.

 If aLevel = 1, reset instruction pointer to beginning of the method.

 Debuggers must not cache or directly manipulate VariableContexts
 when examining or altering stacks.

 Provides the ability for a debugger to restart execution after recompiling
 a method that is active on the stack of the receiver."

 | fpIdx aMeth oldHome oldmClass newTosIdx oldDepth newDepth newTosFpOfs
   prevFpIdx envId |
  self convertToPortableStack: aLevel .

  "check for argument aLevel out of range"
  oldDepth := self localStackDepth .
  aLevel > oldDepth  ifTrue:[ ^ self ].
  newDepth := oldDepth - (aLevel - 1) .

  1 to: aLevel do:[:n|
    prevFpIdx := fpIdx .
    fpIdx := self _frameOffsetAt: n  .
    aMeth := self _fetchMethod:( arStack at:( fpIdx + FP_codePtr_OFS)) .
    aMeth ifNil:[ Error signal:'Cannot trim stack across a reenter marker'].
  ].

  "check to see whether new TOS method has been recompiled, and if so,
   install the new method."
  oldHome := aMeth homeMethod .
  oldmClass := oldHome inClass .
  envId := oldHome environmentId .
  oldmClass ifNotNil:[
    "not an anonymous method"
    | newHome oldCptr |
    newHome := oldmClass compiledMethodAt: oldHome selector environmentId: envId
			otherwise: aMeth .
    newHome == oldHome ifFalse:[
      aMeth == oldHome ifFalse:[
        Error signal:'Cannot trim stack to a block in recompiled method'.
      ].
      "install recompiled method"
      oldCptr := arStack at: (fpIdx + FP_codePtr_OFS) .
      oldCptr class == GsNMethod ifTrue:[
        arStack at: (fpIdx + FP_codePtr_OFS) put: newHome .
      ] ifFalse:[
        self _halt:'should not be here'
      ].
    ].
  ].
  newTosIdx := fpIdx + FP_codePtr_OFS .
  "push savedIP which is byte offset to first instr in the method"
  newTosIdx := newTosIdx - 1 .
  arStack at: newTosIdx put: OC_GSNMETHOD_FIRST_INSTR_OFFSET .
  aLevel > 1 ifTrue:[
    "don't need  _tosIsIpNil in arStack object"
    newTosFpOfs := arStack at: prevFpIdx .
    topFpOffset := newTosFpOfs .  "adjust offset to Top FP"
    "adjust the depth of the receiver."
     uaCount_depth := ((uaCount_depth bitShift: -32)  bitShift:32)
         bitOr: (newDepth bitAnd: 16rFFFFFFFF) ].
  newTosIdx > 1 ifTrue:[
    "delete the unwanted stack frames.
     Since FP linkages in arStack are relative to   arStack size
       there is no other fixup needed. "
    arStack removeFrom:1 to: newTosIdx - 1 .
  ].
  dbgFpsCache := nil . "clear frame offsets cache"
  self clearLastBreakpointHistory .   "fix 49694"
%

category: 'Private'
method: GsProcess
_unscheduleProcess

  "remove the given process from the queues it is in.
   caller puts scheduler in critical region."
  | sched won st |
  sched := self _scheduler .
  (st := signalTime) ifNotNil: [
     sched _delayUnschedule: self.
  ].
  (won := waitingOn) ifNotNil:[
    won _unscheduleProcess: self .
  ].
  sched _unschedule: self.
  st ifNotNil: [
    signalTime := nil
  ].
  won ifNotNil:[
    waitingOn := nil
  ].
%

category: 'Private'
method: GsProcess
_wait
  "Suspend the receiver who should be the active process"
  self _scheduler _reschedule .
  ^ self
%

category: 'Private'
method: GsProcess
_waitingOn: anObj
  " for use by ProcessorScheduler only"
  waitingOn := anObj
%

category: 'Private Debugging Support'
method: GsProcess
__returnAddrIdxAt: aLevel

| fpIdx theMeth frDesc |
(frDesc := self _frameDescrAt: aLevel ) ifNil:[ ^ nil ].
fpIdx := frDesc at: 2 .
theMeth := self _fetchMethod:( arStack at:( fpIdx + FP_codePtr_OFS)) .
theMeth ifNil:[ ^  nil  ].
aLevel > 1 ifTrue:[ | calleeFpIdx |
  calleeFpIdx :=  frDesc at: 3 .
  ^ calleeFpIdx + FP_rtnAddr_OFS .
] ifFalse:[
  ^ 1 "savedIP at TOS"  .
]
%

! Class extensions for 'GsSession'

!		Class methods for 'GsSession'

removeallmethods GsSession
removeallclassmethods GsSession

category: 'Instance Creation'
classmethod: GsSession
currentSession

"Returns the sole instance of GsCurrentSession that represents this login
 session."

<primitive: 311>
self _primitiveFailed: #currentSession
%

category: 'Testing'
classmethod: GsSession
isSolo

  ^ self currentSession isSolo
%

category: 'Private'
classmethod: GsSession
serialOfSession: aSessionId

"Returns a SmallInteger, the unique serial number of the session that is
 currently logged in as aSessionId, or -1 if that session is not logged in.

 Requires SessionAccess privilege if the session described by aSessionId is
 not the current session."

| descr |
descr := System _descriptionOfSessionSerialNum: 0 sessionId: aSessionId .
^ descr at: 9
%

category: 'Private'
classmethod: GsSession
sessionIdOfSerial: aSerialNumber

"Returns the current session ID of the session uniquely identified by
 aSerialNumber, or -1 if that session is not logged in.

 Requires SessionAccess privilege if the session described by aSerialNumber is
 not the current session."

| descr |
descr := System _descriptionOfSessionSerialNum: aSerialNumber sessionId: -1 .
^ descr at: 10
%

category: 'Instance Creation'
classmethod: GsSession
sessionWithSerialNumber: anInteger

"Returns an instance of a kind of GsSession that represents the GemStone session
 on the same repository, whose serial number is anInteger.  Returns nil if no
 logged-in session has that serial number.

 Requires SessionAccess privilege if the session described by anInteger is
 not the current session."

| dict aSess currSess descr systm |
currSess := GsSession currentSession.
anInteger == currSess sessionSerialNum ifTrue:[ ^ currSess ].
systm := System .
dict := systm __sessionStateAt: 9 .
dict == nil ifTrue:[
  dict := IdentityKeyValueDictionary new .
  dict at: (currSess sessionSerialNum) put: currSess .
  systm __sessionStateAt: 9 put: dict .
  ].
descr := systm _descriptionOfSessionSerialNum: anInteger sessionId: -1 .
(descr at: 1 "UserProfile" ) == nil ifTrue:[
  dict removeKey: anInteger otherwise: nil .
  ^ nil .
  ].
aSess := dict at: anInteger otherwise: nil .
aSess ~~ nil ifTrue:[ ^ aSess ].
aSess := super new _initFromSessionDescription: descr .
dict at: anInteger put: aSess.
^ aSess
%

category: 'Private Symbol Access'
classmethod: GsSession
_newSymbolsAsArray

"Returns an Array containing the enumeration of all Symbols
 in   GsSession _newSymbolsDictionary ."

<primitive: 2001>  "enter protected mode"
| res prot |
prot := System _protectedMode .
[ | dict |
  dict := self _newSymbolsDictionary .
  res := { }  .
  dict keysDo:[:aSym | res add: aSym ].
] ensure:[
  prot _leaveProtectedMode
].
^ res
%

category: 'Private Symbol Access'
classmethod: GsSession
_newSymbolsDictionary

"Returns the session-private dictionary that is a cache of Symbols
 created during the session's current transaction
 on behalf of this session by the Symbol Creation Session."

<protected primitive: 152>
self _primitiveFailed: #_newSymbolsDictionary
%

category: 'Private Symbol Access'
classmethod: GsSession
_newSymbolsSize

"Returns the number of Symbols in
    GsSession _newSymbolsDictionary ."

<primitive: 2001>  "enter protected mode"
| res prot |
prot := System _protectedMode .
[ | dict |
  dict := self _newSymbolsDictionary .
  res := dict size .
] ensure:[
  prot _leaveProtectedMode
].
^ res
%

!		Instance methods for 'GsSession'

category: 'Signalling'
method: GsSession
enableInterSessionSignalling: aBoolean

"If aBoolean is true, enables receipt of intersession signals in the session
 represented by the receiver.  Otherwise, disables receipt of such signals."

System enableSignaledGemStoneSessionError
%

category: 'Testing'
method: GsSession
hasRemoteSessions

"Returns true if the receiver has remote sessions, false otherwise.

 If the receiver is itself a remote session, returns nil."

| rsess |

rsess := self remoteSessions.
rsess ifNil:[ ^ nil "don't know the answer" ].
^ rsess size ~~ 0
%

category: 'Testing'
method: GsSession
isCurrent

"Returns true if the receiver represents the current login session of this Gem;
 returns false otherwise."

^ self == GsSession currentSession
%

category: 'Testing'
method: GsSession
isRemote

"Returns true if the receiver represents a remote federated session."

^ self ~~ GsSession currentSession
%

category: 'Testing'
method: GsSession
isSolo

"Returns true if receiver represents a login that used the GCI_LOGIN_SOLO flag.

 A SOLO session is created by a linked login in a topaz -l process
 using the GEM_SOLO_EXTENT from the topaz's config file.
 In a remote login (gem forked by netldi), the gem must have an
 appropriate GEM_SOLO_EXTENT value in its config.
 GEM_SOLO_EXTENT can be in the gem config file, or passed on the topaz -l
 or gemnetobject command line with  -C 'GEM_SOLO_EXTENT=anExtent0.dbf' .

 The repository specified by GEM_SOLO_EXTENT must be a single extent
 repository, last written by a clean shutdown of a stone.
 If the extent on the filesystem has either write or execute permission
 for any of user,group,world  then session will open the extent with an
 exclusive file lock, to prevent opening an extent that is also open
 by a stone.  Set the extent to read-only permission on the filesystem
 to allow sharing by multiple Solo sessions.  The amount of internal
 free space in the extent does not affect the Solo sessions.

 Each Solo gem or topaz -l process also opens a read-write 10MB
 /tmp/gemRO_<pid>_extent1.dbf which is deleted on logout
 or process exit.

 Object creation in a Solo session is all in temporary object memory.
 Use -T on the topaz -l command line or in the gemnet NRS to specify
 appropriate temp obj memory config, or use appropriate config file
 options in the -C argument or in a gem config file.

 In a Solo session,
   A large number of classmethods in System
   will signal an ImproperOperation (number 2050)
   if the operation would require connection to a stone.

   Many methods in Repository signal an ImproperOperation including
      markForCollection, reclaimAll, fullBackup*, restoreFromBackup*

   Nested transactions work normally, but outer level commit
   signals an ImproperOperation unless the commit is readonly.
"

^ sessionSerialNum == 0
%

category: 'Accessing'
method: GsSession
remoteSessions

"Returns a collection of GsSession instances representing all the remote
 sessions spawned by the session represented by the receiver.  If the receiver
 has no remote sessions, the resulting collection is empty.  This list is
 maintained in the session transient state."

| sessionDict |

self == GsSession currentSession ifFalse:[ ^ nil "result is unknown" ].
" sessionDict maps parameters to sessions "
sessionDict := System __sessionStateAt: 8. "xxx not yet updated"
sessionDict == nil ifTrue:[ ^ #() ].
^ sessionDict values
%

category: 'Signalling'
method: GsSession
sendSignalObject: aGsInterSessionSignal

"Sends the signal and message of aGsInterSessionSignal to the session
 represented by the receiver."

System _sendSignal: aGsInterSessionSignal signal
       toSess: sessionSerialNum
       withMessage: aGsInterSessionSignal message
%

category: 'Accessing'
method: GsSession
serialNumber

"Returns the serial number issued to the session represented by the receiver
 when that session logged in."

^ sessionSerialNum
%

category: 'Accessing'
method: GsSession
sessionSerialNum

"Returns the value of the sessionSerialNum instance variable of the receiver."

^ sessionSerialNum
%

category: 'Session Control'
method: GsSession
stop

"Aborts the current transaction and terminates the session.  If  the receiver is
 the current session, no operation is performed.  If the UserProfile of the
 current session lacks SystemControl privilege, an exception occurs."

self == GsSession currentSession ifTrue:[ ^ self "do nothing" ].

System _stopSession: sessionSerialNum kind: 0 timeout: 0
%

category: 'Accessing'
method: GsSession
userProfile

"Returns the UserProfile attached to the session."

^ userProfile
%

category: 'Private'
method: GsSession
_initFromSessionDescription: anArray

"Initializes the receiver using  anArray, which must be a result from
 System (C) | _descriptionOfSessionSerialNum:sessionId:."

sessionSerialNum := anArray at: 9 .
userProfile := anArray at: 1
%

! Class extensions for 'GsStackBuffer'

!		Class methods for 'GsStackBuffer'

removeallmethods GsStackBuffer
removeallclassmethods GsStackBuffer

category: 'Instance Creation'
classmethod: GsStackBuffer
new

"Disallowed."

self shouldNotImplement: #new
%

category: 'Instance Creation'
classmethod: GsStackBuffer
new: aSize

"Disallowed."

self shouldNotImplement: #new:
%

!		Instance methods for 'GsStackBuffer'

category: 'Copying'
method: GsStackBuffer
copy

"Disallowed."

self shouldNotImplement: #copy
%

category: 'Copying'
method: GsStackBuffer
replaceFrom: startIndex to: stopIndex with: aSeqCollection

"Disallowed."
^ self shouldNotImplement: #replaceFrom:to:with:
%

category: 'Copying'
method: GsStackBuffer
replaceFrom: startIndex to: stopIndex with: aSeqCollection startingAt: repIndex

"Disallowed."
^ self shouldNotImplement: #replaceFrom:to:with:startingAt:
%

category: 'Copying'
method: GsStackBuffer
replaceFrom: startIndex to: stopIndex withObject: anObject

"Disallowed."
^ self shouldNotImplement: #replaceFrom:to:withObject:
%

! Class extensions for 'Halt'

!		Instance methods for 'Halt'

removeallmethods Halt
removeallclassmethods Halt

category: 'Instance initialization'
method: Halt
initialize
  gsNumber := ERR_Halt.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'IdentityBag'

!		Class methods for 'IdentityBag'

removeallmethods IdentityBag
removeallclassmethods IdentityBag

category: 'Modifying Classes'
classmethod: IdentityBag
isModifiable

"Returns true if the receiver and its Array of named instance variables are
 modifiable."

^ (self isInvariant or:[ instVarNames isInvariant] ) not
%

category: 'Instance Creation'
classmethod: IdentityBag
new: anInteger

"Returns an instance of the receiver with sufficient space to hold
 approximately anInteger number of elements. "

 <primitive: 41>
 self _primitiveFailed: #new: args: { anInteger } .
 self _uncontinuableError
%

category: 'Private'
classmethod: IdentityBag
_clearEnumCache

"Remove all entries from the enumeration cache"

<primitive: 547>
self _primitiveFailed: #_clearEnumCache .
self _uncontinuableError
%

!		Instance methods for 'IdentityBag'

category: 'Set Arithmetic'
method: IdentityBag
* aBagOrSet

"Intersection.  Returns a kind of IdentityBag containing only the elements that
 are present in both the receiver and the argument aBagOrSet.

 The class of the result is the lowest class in the hierarchy of which both the
 receiver and argument are some kind.

 If the result is a kind of Set, then each element that occurs in both the
 receiver and aBagOrSet occurs exactly once in the result.  If the result is a
 IdentityBag that is not an IdentitySet, and if an element occurs m times in the
 receiver and n times in the argument aBagOrSet, then the result contains the
 lesser of m or n occurrences of that element."

^ (aBagOrSet _isRcIdentityBag or: [ aBagOrSet _isIdentityBag not ])
  ifTrue: [ self _primIntersect: aBagOrSet _asIdentityBag ]
  ifFalse: [ self _primIntersect: aBagOrSet ]
%

category: 'Set Arithmetic'
method: IdentityBag
+ aBagOrSet

"Union.  Returns a kind of IdentityBag containing exactly the elements that are
 present in either the receiver or the argument aBagOrSet.

 The class of the result is the lowest class in the hierarchy of which both the
 receiver and argument are some kind.

 If the result is a kind of IdentitySet, then each element that occurs in either
 the receiver or aBagOrSet occurs exactly once in the result.  If the result is
 an IdentityBag that is not an IdentitySet, and if an element occurs m times in
 the receiver and n times in the argument aBagOrSet, then the result contains
 m + n occurrences of that element."

^ (aBagOrSet _isRcIdentityBag or: [ aBagOrSet _isIdentityBag not ])
  ifTrue: [ self _primUnion: aBagOrSet _asIdentityBag ]
  ifFalse: [ self _primUnion: aBagOrSet ]
%

category: 'Set Arithmetic'
method: IdentityBag
- aBagOrSet

"Difference.  Returns a kind of IdentityBag containing exactly those elements of
 the receiver that have a greater number of occurrences in the receiver than in
 the argument aBagOrSet.  If an element occurs m times in the receiver and n
 times in aBagOrSet (where m >= n), then the result will contain m - n
 occurrences of that element.

 The class of the result is the class of the receiver."

^ (aBagOrSet _isRcIdentityBag or: [ aBagOrSet _isIdentityBag not ])
  ifTrue: [ self _primDifference: aBagOrSet _asIdentityBag ]
  ifFalse: [ self _primDifference: aBagOrSet ]
%

category: 'Comparing'
method: IdentityBag
= aBag

"Returns true if all of the following conditions are true:

 1.  The receiver and aBag are of the same class.
 2.  The two collections are of the same size.

 3.  Their public named instance variables are identical.
 4.  The elements of the receiver and aBag are identical.
 5.  Each element occurs the same number of times in the receiver and in aBag.

 Returns false otherwise."

<primitive: 600>
self _primitiveFailed: #= args: { aBag } .
^false
%

category: 'Adding'
method: IdentityBag
add: newObject

"Adds newObject to the receiver.  Has no effect if newObject is nil.
 Returns newObject. "

<primitive: 208>  "ObsoleteDictionary >> _basicAdd: is also prim 208"
self _primitiveFailed: #add: args: { newObject } .
self _uncontinuableError
%

category: 'Adding'
method: IdentityBag
add: anObject withOccurrences: aSmallInteger

"Adds anObject as an element of the receiver aSmallInteger number of times.

 Has no effect if anObject is nil."

  ^ self _add: anObject withOccurrences: aSmallInteger forReplay: false
%

category: 'Adding'
method: IdentityBag
addAll: aCollection

"Adds all of the elements of aCollection to the receiver."

^ aCollection _isRcIdentityBag
  ifTrue: [ super addAll: aCollection ]
  ifFalse: [ self _addAll: aCollection forReplay: false ]
%

category: 'Converting'
method: IdentityBag
asArray

"Returns a new Array containing all elements of the receiver."
| a |
a := Array new .
a _addAllFromNsc: self .
^ a
%

category: 'Accessing'
method: IdentityBag
at: anIndex

"Returns the element of the receiver that is currently located at position
 anIndex.

 The elements of an IdentityBag are inherently unordered, and can change
 position (index) when the IdentityBag is altered.  Thus, after an IdentityBag
 is altered, a given element may reside at a different index than before, and a
 given index may house a different element.  You should not infer an ordering
 for an IdentityBag's elements when you access them by index.

 This method is useful primarily as a code optimizer for iterating over all the
 elements of an IdentityBag (using a loop that runs the index from 1 to
 the size of the IdentityBag).

 The IdentityBag must not change during the iteration.  But the iteration may
 run faster than it would if you use other alternatives such as the do:
 method."

<primitive: 64>
(anIndex _isInteger)
   ifTrue: [ ^ self _error: #objErrBadOffsetIncomplete args: { anIndex } ]
ifFalse: [ ^ self _errorNonIntegerIndex: anIndex ].
self _primitiveFailed: #at: args: { anIndex } .
self _uncontinuableError
%

category: 'Updating'
method: IdentityBag
at: anIndex put: anObject

"Disallowed.  Generates an error, since the elements of an IdentityBag are not
 externally accessible through numeric indices."

^ self _errorNotIndexable
%

category: 'Searching'
method: IdentityBag
collect: aBlock

"Evaluates aBlock with each of the receiver's elements as the argument.
 Collects the resulting values into a collection of class specified by
 sending #speciesForCollect message to the receiver and returns the new
 collection.  The argument aBlock must be a one-argument block."

| result mySize |

result:= self speciesForCollect new.
mySize := self size.
1 to: mySize do: [ :i | result add: (aBlock value:(self _at:i)) ].
^ result
%

category: 'Deprecated'
method: IdentityBag
copyFrom: index1 count: aCount into: aColl startingAt: destIndex

"Deprecated, non-ANSI.
 Copies elements of the receiver from (receiver at: index1)
  to (receiver at: (index1 + aCount - 1)), inclusive
 into aColl starting at destIndex, overwriting the previous contents.

 If (index1 + aCount - 1) is beyond the end of the receiver, then
 the copy will stop with the element  (receiver at: (receiver size)) .

 aColl must be a kind of Array or OrderedCollection .
 Returns the number of elements copied."

 | count |
  self deprecated: 'IdentityBag >> copyFrom:count:into:startingAt: deprecated v3.0' .
  count :=  aCount min: ( self size - index1 + 1) .
  aColl replaceFrom: destIndex to: destIndex + count - 1
	  with: self startingAt: index1 .
  ^ count

%

category: 'Deprecated'
method: IdentityBag
copyFrom: index1 to: index2 into: aColl startingAt: destIndex

"Deprecated, non-ANSI.
 Copies the elements of the receiver from (receiver at: index1)
  to (receiver at: index2), inclusive,
 into aColl starting at destIndex, overwriting the previous contents.
 aColl must be a kind of Array or OrderedCollection .
 Returns the receiver. "

 self deprecated: 'IdentityBag >> copyFrom:to:into:startingAt: deprecated v3.0' .
 aColl replaceFrom: destIndex to: destIndex + (index2 - index1)
	with: self startingAt: index1 .
 ^ self
%

category: 'Enumerating'
method: IdentityBag
do: aBlock

"Evaluates aBlock with each of the receiver's elements as the argument.
 The argument aBlock must be a one-argument block."

| theSize |
theSize := self size.
1 to: theSize do: [:i | aBlock value: (self _at: i) ].
^ self
%

category: 'Repository Conversion'
method: IdentityBag
fixReferencesInFirstPass

"Return whether this object should have references to Float, SmallFloat,
 LargeInteger  fixed in the first pass.
 false means fix them in the 2nd pass.  IdentityBag objects can be part
 of an RcIdentityBag, so we do those later"

^self class ~~ IdentityBag
%

category: 'Comparing'
method: IdentityBag
hash

"Returns an Integer hash code for the receiver."

| mySize |
(mySize := self size) = 0
  ifTrue: [ ^12794 ]
  ifFalse: [
    ^(((self _at: 1) identityHash bitShift: -1)
      bitXor: (self _at: mySize) identityHash)
        bitXor: mySize
  ]
%

category: 'Searching'
method: IdentityBag
includes: anObject

"Returns true if the argument anObject is an element of the receiver.
 Returns false otherwise.  (Compare with includesValue:, which is based
 on equality.)"

<primitive: 65>
self _primitiveFailed: #includes: args: { anObject } .
self _uncontinuableError
%

category: 'Searching'
method: IdentityBag
includesIdentical: anObject

"Returns true if the argument anObject is an element of the receiver.
 Returns false otherwise.  (Compare with includesValue:, which is based
 on equality.)"

<primitive: 65>
self _primitiveFailed: #includes: args: { anObject } .
self _uncontinuableError
%

category: 'Searching'
method: IdentityBag
includesValue: anObject

"Returns true if the receiver contains an object of the same value as the
 argument, anObject.  Returns false otherwise.  (Compare with includes:,
 which is based on identity.)"

1 to: self size do: [ :each |
  (anObject = (self _at: each)) ifTrue: [ ^true ]
].
^ false
%

category: 'Set Arithmetic'
method: IdentityBag
intersect: aBagOrSet into: anotherBagOrSet

"April 5, 2000 Norm Green - Created for OOCL
Same as the * method, except the result of the intersection
is put into anotherBagOrSet"

<primitive: 533>
aBagOrSet _validateClass: IdentityBag.
anotherBagOrSet _validateClass: IdentityBag.
self _primitiveFailed: #intersect:into: args: { aBagOrSet . anotherBagOrSet } .
self _uncontinuableError
%

category: 'Set Arithmetic'
method: IdentityBag
intersectAll: anArrayOfSetsOrBags into: aBagOrSet

"April 5, 2000 Norm Green - Created for OOCL

Starts by sorting by ascending size anArrayOfSetsOrBags.  The intersect the first
element with the receiver.  Take that result and intersect it with the second element.
Continue to all intersections are completed and place the result set into aBagOrSet.
Returns aBagOrSet.  If the array is empty, aBagOrSet will be a copy of the receiver.

This method will fail if any of the following is true:
 -the array size is greater than 2031.
 -any element of the array is not an IdentitySet or IdentityBag.
 -any element of the array is empty.
 -aBagOrSet is not an IdentityBag or IdentitySet"

<primitive: 532>
anArrayOfSetsOrBags _validateClass: Array.
aBagOrSet _validateClass: IdentityBag.
self _primitiveFailed: #intersectAll:into:
     args: { anArrayOfSetsOrBags . aBagOrSet } .
self _uncontinuableError
%

category: 'Searching'
method: IdentityBag
occurrencesOf: anObject

"Returns the number of the receiver's elements that are identical (==) to
 anObject."

<primitive: 67>
self _primitiveFailed: #occurrencesOf: args: { anObject } .
self _uncontinuableError
%

category: 'Removing'
method: IdentityBag
remove: anObject

"Removes anObject from the receiver and returns anObject.  If anObject is
 present several times in the receiver, only one occurrence is removed.
 Generates an error if anObject is not in the receiver."

<primitive: 207>  "primitive fails if anObject not found"
^ self _errorNotFound: anObject
%

category: 'Removing'
method: IdentityBag
remove: anObject ifAbsent: exceptionBlock

"Removes from the receiver an object that is identical to anObject and
 returns anObject.  If several elements of the receiver are identical to
 anObject, only one instance is removed.  If anObject is not present in
 the receiver, evaluates anExceptionBlock and returns the result of the
 evaluation."

<primitive: 209>  "primitive fails if anObject not found"
^ exceptionBlock value
%

category: 'Removing'
method: IdentityBag
remove: anObject otherwise: notFoundValue

"Removes from the receiver an object that is identical to anObject and
 returns anObject.  If several elements of the receiver are identical to
 anObject, only one instance is removed.  If anObject is not present in
 the receiver, returns notFoundValue."

<primitive: 209>  "primitive fails if anObject not found"
^ notFoundValue
%

category: 'Removing'
method: IdentityBag
removeAll: aCollection

"Removes one occurrence of each element of aCollection from the
 receiver and returns the receiver.  Generates an error if any
 element of aCollection is not present in the receiver.

 The primitive _removeAll:errIfAbsent: includes an optimization
 for the case    where aCollection == self ."

^ aCollection _isRcIdentityBag
  ifTrue: [ self _removeAll: aCollection _asIdentityBag errIfAbsent: true forReplay: false ]
  ifFalse: [ self _removeAll: aCollection errIfAbsent: true forReplay: false ]
%

category: 'Removing'
method: IdentityBag
removeAllPresent: aCollection

"Removes from the receiver one occurrence of each element of aCollection that is
 also an element of the receiver.  Differs from removeAll: in that, if some
 elements of aCollection are not present in the receiver, no error is generated.
 Returns aCollection."

aCollection _isRcIdentityBag
  ifTrue: [ self _removeAll: aCollection _asIdentityBag errIfAbsent: false forReplay: false ]
  ifFalse: [ self _removeAll: aCollection errIfAbsent: false forReplay: false ].
^aCollection
%

category: 'Removing'
method: IdentityBag
removeIdentical: anObject

"Removes anObject from the receiver and returns anObject.  If anObject is
 present several times in the receiver, only one occurrence is removed.
 Generates an error if anObject is not in the receiver."

^ self remove: anObject
%

category: 'Removing'
method: IdentityBag
removeIdentical: anObject ifAbsent: exceptionBlock

"Removes from the receiver an object that is identical to anObject and
 returns anObject.  If several elements of the receiver are identical to
 anObject, only one instance is removed.  If anObject is not present in
 the receiver, evaluates anExceptionBlock and returns the result of the
 evaluation."

<primitive: 209>  "primitive fails if anObject not found"
^ exceptionBlock value
%

category: 'Removing'
method: IdentityBag
removeIdentical: anObject otherwise: notFoundValue

"Removes from the receiver an object that is identical to anObject and
 returns anObject.  If several elements of the receiver are identical to
 anObject, only one instance is removed.  If anObject is not present in
 the receiver, returns notFoundValue."

<primitive: 209>  "primitive fails if anObject not found"
^ notFoundValue
%

category: 'Removing'
method: IdentityBag
removeIfPresent: anObject

"Removes anObject from the receiver and returns anObject.  If anObject is
 present several times in the receiver, only one occurrence is removed.
 Returns nil if anObject is missing from the receiver."

<primitive: 207>  "primitive fails if anObject not found"
^nil
%

category: 'Set Arithmetic'
method: IdentityBag
sizeOfIntersectionWith: aBagOrSet

"April 5, 2000 Norm Green - Created for OOCL
Perform the intersection and return only the size of the result set"
<primitive: 531>
aBagOrSet _validateClass: IdentityBag.
self _primitiveFailed: #sizeOfIntersectionWith: args: { aBagOrSet } .
self _uncontinuableError
%

category: 'Set Arithmetic'
method: IdentityBag
sizeOfIntersectionWithAll: anArrayOfBagsOrSets

"April 7, 2000 Norm Green - Created for OOCL
Starts by sorting by ascending size anArrayOfSetsOrBags.  The intersect the first
element with the receiver.  Take that result and intersect it with the second element.
Continue to all intersections are completed. Return the size of the resultant set.
If the array is empty, return the size of the receiver.

This method will fail if any of the following is true:
 -the array size is greater than 2031.
 -any element of the array is not an IdentitySet or IdentityBag.
 -any element of the array is empty.

All intermediate objects are disposed (no garbage is created)"

<primitive: 534>
anArrayOfBagsOrSets _validateClass: Array.
self _primitiveFailed: #sizeOfIntersectionWithAll:
     args: { anArrayOfBagsOrSets } .
self _uncontinuableError
%

category: 'Adding'
method: IdentityBag
_add: anObject withOccurrences: aSmallInteger forReplay: aBoolean

"Adds anObject as an element of the receiver aSmallInteger number of times.

 Has no effect if anObject is nil."

<primitive: 549>
aSmallInteger _validateClass: SmallInteger .
aBoolean _validateClass: Boolean .
self _primitiveFailed: #add:withOccurrences:forReplay:
     args: { anObject . aSmallInteger } .
self _uncontinuableError
%

category: 'Adding'
method: IdentityBag
_addAll: aCollection forReplay: aBoolean

"Adds all of the elements of aCollection to the receiver.  Occurrences
 of nil in aCollection are not added to the receiver.

 The primitive code handles aCollection if it is a kind of
 Array or OrderedCollection, without faulting in elements of aCollection.
 Otherwise the accompaniedBy:do: implementation in this method is executed."

<primitive: 225>
(self == aCollection) ifTrue: [ ^ self addAll: (aCollection copy) ].
aBoolean _validateClass: Boolean .
aCollection _validateClass: Collection.
aCollection accompaniedBy: self do: [:me :each | me add: each].
^ aCollection
%

category: 'Adding'
method: IdentityBag
_addForReplay: newObject

"Adds newObject to the receiver.  Has no effect if newObject is nil.
 Returns newObject. "

<primitive: 1044>
self _primitiveFailed: #'_addForReplay:' args: { newObject } .
self _uncontinuableError
%

category: 'Adding'
method: IdentityBag
_addIfAbsent: anObject

"Adds anObject to the receiver.
 Returns true if anObject added, false if it was already present in receiver.
 If newObject is nil has no effect, and returns false."

<primitive: 804>
self _primitiveFailed: #_addIfAbsent: args: { anObject } .
self _uncontinuableError
%

category: 'Accessing'
method: IdentityBag
_at: anInteger

"Private.  Functionally equivalent to 'at:'."

"Used to implement the 'do:' method, which cannot be implemented as a
 primitive itself, since blocks cannot be sent as arguments to primitives."

<primitive: 64>
(anInteger _isInteger)
ifTrue: [ ^ self _error: #objErrBadOffsetIncomplete args: { anInteger } ]
ifFalse: [ ^ self _errorNonIntegerIndex: anInteger ].
self _primitiveFailed: #_at: args: { anInteger } .
self _uncontinuableError
%

category: 'Reduced Conflict Support'
method: IdentityBag
_checkForRcIndexesBeforeRcSelectiveAbort
"_selectiveAbort may not be performed if there are indexes on the receiver and those indexes
 themselves are not RC."

_indexedPaths ifNil: [ ^ self ].
_indexedPaths _allIndexesAreRc
  ifFalse: [ self _error: #rtErrSelectiveAbort ]
%

category: 'Searching'
method: IdentityBag
_indexOf: anElement

"Returns the numerical index into the receiver of the given element (or nil if
 the element cannot be found)."

"Uses _at: to do the search and matches on equality."

1 to: self size do: [ :each |
  (self _at: each) = anElement ifTrue: [ ^each ]
  ].
^nil
%

category: 'Testing'
method: IdentityBag
_isIdentityBag

"Return whether the receiver is an identity-based collection."

^ true
%

category: 'Searching'
method: IdentityBag
_offsetOf: anObject

"Returns the first SmallInteger offset such that
 (self _at: (self _offSetOf: anObject)) == anObject.

 If the argument is not in the receiver, returns 0.  The result of this method
 is only valid while no insertions or deletions are made to the IdentityBag."

"Implemented with sequential scan of the receiver."

<primitive: 62>
self _primitiveFailed: #_offsetOf: args: { anObject } .
self _uncontinuableError
%

category: 'Set Arithmetic'
method: IdentityBag
_primDifference: aBagOrSet

""

<primitive: 223>
aBagOrSet _validateClass: IdentityBag.
self _primitiveFailed: #_primDifference: args: { aBagOrSet } .
self _uncontinuableError
%

category: 'Set Arithmetic'
method: IdentityBag
_primIntersect: aBagOrSet

<primitive: 222>
aBagOrSet _validateClass: IdentityBag.
self _primitiveFailed: #_primIntersect: args: { aBagOrSet } .
self _uncontinuableError
%

category: 'Updating'
method: IdentityBag
_primitiveAt: anIndex put: anObject

"Disallowed.  Generates an error, since the elements of an IdentityBag are not
 externally accessible through numeric indices."

^ self _errorNotIndexable
%

category: 'Set Arithmetic'
method: IdentityBag
_primUnion: aBagOrSet

""

<primitive: 221 >
aBagOrSet _validateClass: IdentityBag.
self _primitiveFailed: #_primUnion: args: { aBagOrSet } .
self _uncontinuableError
%

category: 'Private'
method: IdentityBag
_rcAdd: anObject withOccurrences: aSmallInteger

"Adds anObject as an element of the receiver aSmallInteger number of times,
 adding root node and modified nodes of the receiver to the RcReadSet.

 Has no effect if anObject is nil."

<primitive: 44>
self _primitiveFailed: #_rcAdd: args: { anObject . aSmallInteger }.
self _uncontinuableError
%

category: 'Reduced Conflict Support'
method: IdentityBag
_rcDifference: aBag

"Returns the set arithmetic difference between the receiver and aBag.  Also
 places the root object and interior nodes that are accessed in the
 reduced-conflict read set (RcReadSet) for both the receiver and aBag."

<primitive: 224>
aBag _validateClass: IdentityBag.
self _primitiveFailed: #_rcDifference: args: { aBag } .
self _uncontinuableError
%

category: 'Reduced Conflict Support'
method: IdentityBag
_rcOccurrencesOf: anObject

"Returns the number of occurrences of anObject in the receiver.  Places the
 root of the receiver and any interior nodes that are accessed in the
 reduced-conflict read set(RcReadSet) ."

<primitive: 68>
self _primitiveFailed: #_rcOccurrencesOf: args: { anObject } .
self _uncontinuableError
%

category: 'Removing'
method: IdentityBag
_rcRemove: anObject

"Removes anObject from the receiver and returns anObject.  If anObject is
 present several times in the receiver, only one occurrence is removed.
 Generates an error if anObject is not in the receiver.
 Adds root node and any modified nodes to the RcReadSet. "

<primitive: 596>  "primitive fails if anObject not found"
^ self _errorNotFound: anObject
%

category: 'Removing'
method: IdentityBag
_remove: anObject

"Removes anObject from the receiver and returns anObject.  If anObject is
 present several times in the receiver, only one occurrence is removed.
 Generates an error if anObject is not in the receiver."

"Needed by certain subclasses where the super remove: message picks up an
 undesired reimplementation."

<primitive: 207>  "primitive fails if anObject not found"
^ self _errorNotFound: anObject
%

category: 'Private'
method: IdentityBag
_removeAll: aCollection errIfAbsent: aBool forReplay: replayBool
  ^ self _removeAll: aCollection errIfAbsent: aBool forReplay: replayBool rc: false 
%

category: 'Private'
method: IdentityBag
_removeAll: aCollection errIfAbsent: aBool forReplay: replayBool rc: rcBool

"For each element in aCollection, removes the one element that is identical
 from the receiver.  Returns aCollection if successful.

 The primitive includes an optimization
 for the case    where aCollection == self ."

<primitive: 226>

aBool _validateClass: Boolean .
replayBool _validateClass: Boolean .
rcBool _validateClass: Boolean .

(aCollection _isArray)
  ifFalse: [
    self _removeAll: (aCollection asArray) errIfAbsent: aBool forReplay: replayBool rc: rcBool.
    ^ aCollection
    ].

(aCollection isKindOf: CharacterCollection) ifTrue:[
  replayBool ifTrue:[ Error signal:'replay not supported' ].
  rcBool ifTrue:[
    aBool ifTrue:[ Error signal:'errIfAbsent not supported for rc remove'].
    1 to: aCollection size do:[:j| self _rcRemove: (aCollection _at: j) ].
  ] ifFalse:[
   aBool
    ifTrue:[
      1 to: aCollection size do:[:j| self remove: (aCollection _at: j) ].
    ] ifFalse:[
      1 to: aCollection size do:[:j| self remove: (aCollection _at: j) otherwise: nil ].
    ].
  ^ aCollection
  ].
 ].

aCollection _validateClass: Collection .
self _primitiveFailed: #_removeAll:errIfAbsent:forReplay:
       args: { aCollection . aBool . replayBool} .
self _uncontinuableError
%

category: 'Removing'
method: IdentityBag
_removeForReplay: anObject

<primitive: 1037>  "returns true if successful, false if object not found"
%

category: 'Private'
method: IdentityBag
_removeFromEnumCache

"Explicitly remove the receiver from the enumeration cache ,
 to facilitate garbage collection of temporary IdentityBags.

 Enumeration of the last element of an IdentityBag will
 automatically remove the receiver from the enumeration cache, so this
 method would be of use only if enumeration was terminated before
 reaching the end of an IdentityBag."

<primitive: 548>
self _primitiveFailed: #_removeFromEnumCache .
self _uncontinuableError
%

category: 'Reduced Conflict Support'
method: IdentityBag
_selectiveAbort

"Performs an abort operation on the receiver. That is, if the object is
 committed, it removes any changes made by the current transaction and allows
 access to the committed state of the object.  This is not allowed if the object
 participates in an index."

_indexedPaths ~~ nil
  ifTrue: [ self _error: #rtErrSelectiveAbort ].
^ super _selectiveAbort
%

category: 'Set Arithmetic'
method: IdentityBag
_union: aBag

"Union with minimal result.
 An object must be present in either the receiver or the argument in order
 to appear in the result.
 For each object in the result the number of occurrances in the result
 is the maximum of the number of occurrances in the receiver or argument.

 aBag must be a kind of IdentityBag."

<primitive: 1043>

aBag _validateClass: IdentityBag.
self _primitiveFailed: #_union: args: { aBag } .
self _uncontinuableError
%

! Class extensions for 'IdentityCollisionBucket'

!		Class methods for 'IdentityCollisionBucket'

removeallmethods IdentityCollisionBucket
removeallclassmethods IdentityCollisionBucket

category: 'Instance Creation'
classmethod: IdentityCollisionBucket
new

^ self new: 0
%

!		Instance methods for 'IdentityCollisionBucket'

category: 'Private'
method: IdentityCollisionBucket
at: aKey put: aValue keyValDict_coll: aKeyValDict

"Private.  Adds the key-value pair (aKey, aValue) to the collision bucket.
 Returns 0 if this at:put: replaced the value for an existing  key ,
 Returns self size if this at:put: added a value ."

| marker |

aKey == nil ifTrue:[ ^ self _error: #rtErrNilKey ] .

marker := self binarySearchForInsertKey: aKey.
marker < 0 ifTrue:[ "replace existing key"
  self at: marker negated putValue: aValue.
  aKeyValDict _markDirty .
  ^ 0
  ].
(numElements >= 974) ifTrue:[
  aKeyValDict _rebuildAt: aKey put: aValue .
  ^ 0
  ].
^ self insertAt: marker key: aKey value: aValue.
%

category: 'Searching'
method: IdentityCollisionBucket
binarySearchForInsertKey: aKey

"Returns the negated index of aKey if found, or the offset at which to
 insert aKey."

"Does a binary search."

<primitive: 448>
aKey == nil ifTrue:[ ^ self _error: #rtErrNilKey ] .
self _primitiveFailed: #binarySearchForInsertKey: args: { aKey }
%

category: 'Private'
method: IdentityCollisionBucket
compareKey: key1 with: key2

"Returns true if key1 is identical to key2, and false otherwise."

^ key1 == key2
%

category: 'Private'
method: IdentityCollisionBucket
insertAt: marker key: aKey value: aValue

"Private.  Inserts the given key-value pair (aKey, aValue) at the
 position marker."

  self insertAll: { aKey . aValue } at: (marker + marker - 1).
  numElements := numElements + 1.
  ^ numElements
%

category: 'Removing'
method: IdentityCollisionBucket
removeKey: aKey ifAbsent: aBlock

"Removes the key/value pair having the key aKey.  If aKey is not found,
 returns the result of evaluating the zero-argument block aBlock."

  | keyIndex valueOffset aValue |
  keyIndex := self searchForKey: aKey.
  keyIndex == nil
    ifFalse: [
      numElements := numElements - 1.
      aValue := self valueAt: keyIndex.
      valueOffset := keyIndex + keyIndex .
      self removeFrom: valueOffset - 1 to: valueOffset .
      ^aValue
      ]
    ifTrue: [ ^ self _reportKeyNotFound: aKey with: aBlock ]
%

category: 'Removing'
method: IdentityCollisionBucket
removeKey: aKey otherwise: notFoundValue

"Removes the key/value pair having the key aKey.  If aKey is not found,
 returns the notFoundValue. "

  | keyIndex valueOffset aValue |
  keyIndex := self searchForKey: aKey.
  keyIndex == nil
    ifFalse: [
      numElements := numElements - 1.
      aValue := self valueAt: keyIndex.
      valueOffset := keyIndex + keyIndex .
      self removeFrom: valueOffset - 1 to: valueOffset .
      ^aValue
      ]
    ifTrue: [ ^ notFoundValue ].
%

category: 'Searching'
method: IdentityCollisionBucket
searchForKey: aKey

"Returns the index of aKey, or nil if not found."

"Does a binary search."

<primitive: 391>
aKey == nil ifTrue:[ ^ self _error: #rtErrNilKey ] .
self _primitiveFailed: #searchForKey: args: { aKey }
%

category: 'Private'
method: IdentityCollisionBucket
_validateSortedKeys

"Private."

| prevOop thisOop |
prevOop := 0 .
1 to: numElements * 2 by: 2 do:[:j |
   thisOop := (self basicAt: j) asOop .
   thisOop <= prevOop
      ifTrue:[ self _halt:'IdentityCollisionBucket not sorted' ].
   prevOop := thisOop
   ].
%

! Class extensions for 'IdentityDictionary'

!		Instance methods for 'IdentityDictionary'

removeallmethods IdentityDictionary
removeallclassmethods IdentityDictionary

category: 'Enumerating'
method: IdentityDictionary
accompaniedBy: anObj keysAndValuesDo: aBlock

"Evaluates aBlock with each of the receiver's key/value pairs as the
 2nd and 3rd arguments.
 aBlock must be a 3 argument block, with arguments anObj, key value ."

| aKey collisionBkt |
1 to: tableSize * 2 by: 2 do: [ :tableIndex |
  nil == (aKey := self _at: tableIndex) ifTrue: [
    nil == (collisionBkt := self _at: (tableIndex + 1)) ifFalse: [
      1 to: collisionBkt _basicSize by: 2 do: [ :j |
        nil == (aKey := collisionBkt _at: j) ifFalse: [
          aBlock value: anObj value: aKey value: (collisionBkt _at: j + 1) _value .
          ].
        ].
      ].
    ]
  ifFalse: [
    aBlock value: anObj value: aKey value: (self _at: tableIndex + 1) _value .
    ].
  ].
%

category: 'Adding'
method: IdentityDictionary
add: anAssociation

"Requires an Association as the argument.  If the receiver already includes an
 Association whose key is identical to that of anAssociation, this method redefines
 the value portion of that Association."

^ self addAssociation: anAssociation
%

category: 'Updating'
method: IdentityDictionary
addAssociation: anAssociation

"Adds the argument to the receiver."

anAssociation class == Association ifFalse:[
  anAssociation _validateClass: Association
  ].
anAssociation objectSecurityPolicy: self objectSecurityPolicy .
^ self _at: anAssociation key put: anAssociation
%

category: 'Accessing'
method: IdentityDictionary
associationAt: aKey

"Returns the Association with key aKey.  Generates an error if
 no such Association exists."

| anAssoc |
anAssoc := self _at: aKey otherwise: nil .
anAssoc ifNil:[ ^ self _errorKeyNotFound: aKey ].
^ anAssoc
%

category: 'Accessing'
method: IdentityDictionary
associationAt: aKey ifAbsent: aBlock

"Returns the Association with key aKey.  If no such Association exists, returns
 the result of evaluating the zero-argument block aBlock."

| anAssoc |

anAssoc := self _at: aKey otherwise: nil .
anAssoc ifNil:[ ^ aBlock value] .
^ anAssoc
%

category: 'Accessing'
method: IdentityDictionary
associationAt: aKey otherwise: defaultValue

"Returns the Association with key aKey.  If no such Association exists, returns
 the given default value."

| anAssoc |
anAssoc := self _at: aKey otherwise: nil .
anAssoc ifNil:[ ^ defaultValue ].
^ anAssoc
%

category: 'Enumerating'
method: IdentityDictionary
associationsDo: aBlock

"Evaluates aBlock with each of the receiver's Associations.
 The argument aBlock must be a one-argument block.  Returns the receiver."

| collisionBkt |

1 to: tableSize * 2 by: 2 do: [ :tableIndex |
  nil == (self _at: tableIndex) ifTrue: [
    nil == (collisionBkt := self _at: (tableIndex + 1)) ifFalse: [
      1 to: collisionBkt _basicSize by: 2 do: [ :j |
        nil == (collisionBkt _at: j) ifFalse: [
          aBlock value: (collisionBkt _at: j + 1) .
          ].
        ].
      ].
    ]
  ifFalse: [
    aBlock value: (self _at: tableIndex + 1) .
    ].
  ].
%

category: 'Accessing'
method: IdentityDictionary
at: aKey

"Returns the value of the Association with key aKey.  Generates an error if no
 such Association exists."

| anAssoc |
anAssoc:= self _at: aKey otherwise: nil .
anAssoc ifNil:[ anAssoc := self _errorKeyNotFound: aKey ].
^anAssoc value
%

category: 'Accessing'
method: IdentityDictionary
at: aKey ifAbsent: aBlock

"Returns the value that corresponds to aKey.  If no such key/value pair
 exists, returns the result of evaluating the zero-argument block aBlock."

| anAssoc |
anAssoc := self _at: aKey otherwise: nil .
anAssoc ifNil:[ ^ aBlock value ] .
^ anAssoc value
%

category: 'Accessing'
method: IdentityDictionary
at: aKey otherwise: aValue

"Returns the value that corresponds to aKey.  If no such key/value pair exists,
 returns the given alternate value."

| anAssoc |
anAssoc := self _at: aKey otherwise: nil .
anAssoc ifNil:[ ^ aValue] .
^ anAssoc value
%

category: 'Updating'
method: IdentityDictionary
at: aKey put: aValue

"If the receiver already contains a Association with the given key, this
 method makes aValue the value of that Association.  Otherwise, it creates a
 new Association with the given key and value and adds it to the receiver.
 Returns aValue."

| anAssoc |

anAssoc := self _at: aKey otherwise: nil .
anAssoc == nil
  ifTrue:[ | newAssoc |
    newAssoc := Association newWithKey: aKey value: aValue.
    newAssoc objectSecurityPolicy: self objectSecurityPolicy .
    self _at: aKey put: newAssoc.
    ^aValue
    ].

tableSize := tableSize.	"make sure IdentityDictionary is dirty, not just Association (#42383)"
anAssoc value: aValue.
^aValue
%

category: 'Private'
method: IdentityDictionary
atHash: hashIndex putKey: aKey value: aValue

aValue ifNotNil:[ aValue isAssociation ifFalse:[
    (aValue isKindOf: CollisionBucket) ifFalse:[
       aValue _error: #rtErrInvalidArgClass args:
         { Association . CollisionBucket . UndefinedObject }
      ].
    ].
  ].
^ super atHash: hashIndex putKey: aKey value: aValue.
%

category: 'Private'
method: IdentityDictionary
atHash: hashIndex putValue: aValue

"Updates the hash table by storing aValue at the specified hashIndex."

aValue ifNotNil:[ aValue isAssociation ifFalse:[
    (aValue isKindOf: CollisionBucket) ifFalse:[
       aValue _error: #rtErrInvalidArgClass args:
         { Association . CollisionBucket . UndefinedObject }
      ].
    ].
  ].
^super atHash: hashIndex putValue: aValue.
%

category: 'Storing and Loading'
method: IdentityDictionary
basicWriteTo: passiveObj

"Converts the receiver to its passive form and writes that information on
 passiveObj."

| s cls c |
  cls := self class.
  passiveObj writeClass: cls.

  passiveObj writeSize: (s := self size) .

  passiveObj writeNamedIvsFrom: self class: cls .
  passiveObj endNamedInstVars.

  c := 0.
  self associationsDo:[ :anAssoc |
    passiveObj writeObject: anAssoc .
    c := c + 1.
    c > 99 ifTrue: [
      passiveObj lf.
      c := 0.
      ].
    ].

  passiveObj cr
%

category: 'Searching'
method: IdentityDictionary
collectAssociations: aBlock

"Evaluates aBlock with each of the receiver's Associations as the argument.
 Collects the resulting values into a new dictionary and returns that
 dictionary.  The argument aBlock must be a one-argument block."

|result|

result:= self speciesForCollect new: self tableSize.
self associationsDo: [ :anAssoc |
  result at: (anAssoc key) put: ( aBlock value: anAssoc )
  ] .
^ result
%

category: 'Private'
method: IdentityDictionary
compareKey: key1 with: key2

"Returns true if key1 is equivalent to key2; returns false otherwise."

^ key1 == key2
%

category: 'Hashing'
method: IdentityDictionary
hashFunction: aKey

"The hash function should perform some operation on the value of the
 key (aKey) which returns a value in the range 1..tableSize."

^ (aKey identityHash \\ self tableSize) + 1
%

category: 'Searching'
method: IdentityDictionary
includesAssociation: anAssociation

"Returns true if the receiver contains an element identical to anAssociation.
 Returns false otherwise."

self associationsDo: [ :element |
  (anAssociation == element) ifTrue: [ ^ true ].
  ].

^ false.
%

category: 'Searching'
method: IdentityDictionary
includesIdenticalAssociation: anAssociation

"Returns true if anAssociation is identical to one of the Associations of
 the receiver.  Returns false otherwise."

self associationsDo: [ :assoc | (anAssociation == assoc) ifTrue: [ ^true ]].
^false
%

category: 'Searching'
method: IdentityDictionary
includesKey: aKey

"Reimplemented from KeyValueDictionary for efficiency."

^ ( super at: aKey otherwise: nil ) ~~ nil
%

category: 'Accessing'
method: IdentityDictionary
keyAtValue: anObject ifAbsent: aBlock

"Returns the key of the first Association whose value matches the given object,
 anObject.  If no match is found, this method evaluates the block aBlock and
 returns its result."

| aKey collisionBkt |

1 to: tableSize * 2 by: 2 do: [ :tableIndex |
  (aKey := self _at: tableIndex) ifNil: [
    (collisionBkt := self _at: (tableIndex + 1)) ifNotNil: [
      1 to: collisionBkt _basicSize by: 2 do: [ :j |
        (aKey := collisionBkt _at: j) ifNotNil: [
          anObject == (collisionBkt _at: j + 1) _value ifTrue: [
            ^ aKey
            ].
          ].
        ].
      ].
  ] ifNotNil: [
    anObject == (self _at: tableIndex + 1) _value ifTrue: [
      ^ aKey
      ].
    ].
  ].

^aBlock value.
%

category: 'Enumerating'
method: IdentityDictionary
keysAndAssociationsDo: aBlock

"Evaluates aBlock with each of the receiver's key/Association pairs as the
 arguments.  The argument aBlock must be a two-argument block.  The
 first argument is the key and the second argument is the Association of
 each pair.  Returns the receiver."

| aKey collisionBkt |

1 to: tableSize * 2 by: 2 do: [ :tableIndex |
  (aKey := self _at: tableIndex) ifNil: [
    (collisionBkt := self _at: (tableIndex + 1)) ifNotNil: [
      1 to: collisionBkt _basicSize by: 2 do: [ :j |
        (aKey := collisionBkt _at: j) ifNotNil: [
          aBlock value: aKey value: (collisionBkt _at: j + 1) .
          ].
        ].
      ].
  ] ifNotNil: [
    aBlock value: aKey value: (self _at: tableIndex + 1) .
  ].
].
%

category: 'Enumerating'
method: IdentityDictionary
keysAndValuesDo: aBlock

"Evaluates aBlock with each of the receiver's key/value pairs as the
 arguments.  The argument aBlock must be a two-argument block.  The
 first argument is the key and the second argument is the value of
 each key/value pair.  Returns the receiver."

"Reimplemented to send value to each Association."

| aKey collisionBkt |

1 to: tableSize * 2 by: 2 do: [ :tableIndex |
  (aKey := self _at: tableIndex) ifNil: [
    (collisionBkt := self _at: (tableIndex + 1)) ifNotNil: [
      1 to: collisionBkt _basicSize by: 2 do: [ :j |
        (aKey := collisionBkt _at: j) ifNotNil: [
          aBlock value: aKey value: (collisionBkt _at: j + 1) _value .
          ].
        ].
      ].
    ]
  ifNotNil: [
    aBlock value: aKey value: (self _at: tableIndex + 1) _value .
    ].
  ].
%

category: 'Storing and Loading'
method: IdentityDictionary
loadVaryingFrom: passiveObj size: varyingSize

"Reads the varying part of the receiver from the given passive object.
 Does not record the receiver as having been read.  Does not read the
 receiver's named instance variables, if any."

1 to: varyingSize do: [:i |
  self add: passiveObj readObject"anAssociation"
].
%

category: 'Updating'
method: IdentityDictionary
objectSecurityPolicy: aGsObjectSecurityPolicy

"Assigns the receiver and all its components to the given ObjectSecurityPolicy.
 Returns the receiver."

super objectSecurityPolicy: aGsObjectSecurityPolicy .  "self plus buckets"
self associationsDo: [ :anAssoc |
  anAssoc objectSecurityPolicy: aGsObjectSecurityPolicy
].
^ self.
%

category: 'Copying'
method: IdentityDictionary
postCopy
	"Cleanup of new copy after shallowCopy."

	super postCopy.
	self postCopyAssociations
%

category: 'Copying'
method: IdentityDictionary
postCopyAssociations
	"Cleanup of new copy after shallowCopy.
	 Dictionaries which actually store the associations need to copy
	 the associations to prevent contention with the original."

	self associationsDo: [:each | self addAssociation: each copy].
%

category: 'Hashing'
method: IdentityDictionary
rebuildTable: newSize

"Rebuilds the hash table by saving the current state, initializing and
 changing the size of the table, and adding the key value pairs saved
 back to the dictionary."

|saveTable index saveCollLimit|

collisionLimit == 536870911 ifTrue:[
  ^ self  "avoid recursive rebuild"
  ].

saveTable := Array new: (self size * 2).
index := 0.
self keysAndAssociationsDo: [ :aKey :anAssoc |
  index := index + 1.
  saveTable at: index put: aKey.
  index := index + 1.
  saveTable at: index put: anAssoc
  ].
self tableSize: newSize.
self collisionLimit: newSize * 20  . "since buckets are sorted"
saveCollLimit := collisionLimit .
collisionLimit := 536870911 . "prevent recursive rebuild"
1 to: index by: 2 do: [ :i |
  self _at: (saveTable at: i) put: (saveTable at: i + 1)
  ].
collisionLimit := saveCollLimit .
				"deleted   'reduce garbage' code"
^self
%

category: 'Removing'
method: IdentityDictionary
removeAssociation: anAssocation

"Removes an element from the receiver equal to anAssociation and returns
 anAssociation.  If no such element is present, this method generates an error."

^self removeAssociation: anAssocation ifAbsent: [
  ^ self _errorNotFound: anAssocation
].
%

category: 'Removing'
method: IdentityDictionary
removeAssociation: anAssocation ifAbsent: aBlock

"Removes anAssocation from the receiver.  If no such element is present,
 evaluates the zero-argument block aBlock and returns the result of that
 evaluation."

"We must remove this particular Association (anAssocation), not
 just any Association with the same key as anAssocation."

^ self _removeKey: anAssocation key ifAbsent: aBlock
%

category: 'Removing'
method: IdentityDictionary
removeKey: aKey

"Removes the Association with key identical to aKey from the receiver and
 returns the value portion of that Association.  If no Association is present
 with key identical to aKey, reports an error."

|anAssoc|
anAssoc:= self _removeKey: aKey ifAbsent: nil.
^anAssoc value
%

category: 'Removing'
method: IdentityDictionary
removeKey: aKey ifAbsent: aBlock

"Removes the key/value pair with key aKey from the receiver and returns
 the value.  If no key/value pair is present with key aKey, evaluates
 the zero-argument block aBlock and returns the result of that evaluation."

|anAssoc|
anAssoc:= self _removeKey: aKey ifAbsent: [^aBlock value].
^anAssoc value
%

category: 'Accessing'
method: IdentityDictionary
values

"Returns an OrderedCollection containing the receiver's values."

|result|
result:= OrderedCollection new.
self valuesDo: [ :value | result add: value ].
^ result.
%

category: 'Private'
method: IdentityDictionary
_deferredGciUpdateWith: valueArray

"Private."

1 to: valueArray size do:[:j | self add: (valueArray at: j) ].
%

category: 'Searching'
method: IdentityDictionary
_keysWithValue: aValue

"Returns a (possibly empty) set of keys associated with the value, aValue."

| result |

result:= IdentitySet new.  "an IdentitySet: we should not have duplicate keys"
self keysAndValuesDo: [ :aKey :dictValue  |
  (aValue = dictValue) ifTrue: [result add: aKey ]
  ].
^ result
%

category: 'Locking Support'
method: IdentityDictionary
_lockableValues

"Returns an Array of the receiver's Associations."

| result |

result := { }  .
self associationsDo: [ :anAssoc | result add: anAssoc] .
^ result
%

category: 'Private'
method: IdentityDictionary
_rebuildAt: aKey put: aValue
  self rebuildTable: self _nextTableSize .
  ^ self _at: aKey put: aValue
%

! Class extensions for 'IdentityKeyValueDictionary'

!		Instance methods for 'IdentityKeyValueDictionary'

removeallmethods IdentityKeyValueDictionary
removeallclassmethods IdentityKeyValueDictionary

category: 'Accessing'
method: IdentityKeyValueDictionary
at: aKey ifAbsent: aBlock

"Returns the value that corresponds to aKey.  If no such key/value pair
 exists, returns the given alternate value."

| hofs hashKey collisionBkt |

aKey ifNotNil:[
  hofs := (aKey identityHash \\ tableSize) . hofs := hofs + hofs .
  hashKey := self _atZ: hofs  .
  hashKey ifNil: [
      collisionBkt := self _atZ: hofs + 1 .
      collisionBkt ifNotNil: [ ^ collisionBkt at: aKey ifAbsent: aBlock ]
  ] ifNotNil: [
    (aKey == hashKey) ifTrue: [ ^ self _atZ: hofs + 1 ]
  ].
].
aBlock ifNil:[^ self _errorKeyNotFound: aKey ] .
^ aBlock value
%

category: 'Accessing'
method: IdentityKeyValueDictionary
at: aKey otherwise: defaultValue

"Returns the value that corresponds to aKey.  If no such key/value pair
 exists, returns the given alternate value."

| hofs hashKey collisionBkt |

aKey ifNotNil:[
  hofs := (aKey identityHash \\ tableSize) . hofs := hofs + hofs .
  hashKey := self _atZ: hofs  .
  hashKey ifNil: [
    collisionBkt := self _atZ: hofs + 1 .
    collisionBkt ifNotNil: [ ^collisionBkt at: aKey otherwise: defaultValue ]
  ] ifNotNil: [
    (aKey == hashKey) ifTrue: [ ^ self _atZ: hofs + 1 ]
  ].
].
^ defaultValue
%

category: 'Updating'
method: IdentityKeyValueDictionary
at: aKey put: aValue

^ self _at: aKey put: aValue
%

category: 'Private'
method: IdentityKeyValueDictionary
collisionBucketClass

"Returns the class of object to create when keys collide."

^ IdentityCollisionBucket
%

category: 'Hashing'
method: IdentityKeyValueDictionary
compareKey: key1 with: key2

"No longer used by implementation of IdentityKeyValueDictionary"

self shouldNotImplement: #compareKey:with:
%

category: 'Comparing'
method: IdentityKeyValueDictionary
hash

"Returns a numeric hash key for the receiver."

| hashValue |

hashValue := 97633 bitXor: (self size).
"For large dictionaries, the hash value is just a function of its size"
(self size > 64) ifTrue: [ ^ hashValue abs ].
self keysDo: [ :aKey |
   "Skip if the key is a dictionary."
   (aKey isKindOf: AbstractDictionary)
     ifFalse: [
       hashValue := hashValue bitXor: aKey identityHash
       ]
     ].
^ hashValue abs
%

category: 'Hashing'
method: IdentityKeyValueDictionary
hashFunction: aKey

"No longer used by implementation of IdentityKeyValueDictionary"

self shouldNotImplement: #hashFunction:
%

category: 'Accessing'
method: IdentityKeyValueDictionary
keys

"Returns an IdentitySet containing the receiver's keys."

| result |
result := IdentitySet new .
self keysDo:[ :aKey | result add: aKey ].
^ result
%

category: 'Hashing'
method: IdentityKeyValueDictionary
rebuildTable: newSize

super rebuildTable: newSize .
self collisionLimit: newSize * 500 . "differs from KeyValueDictionary"
%

category: 'Hashing'
method: IdentityKeyValueDictionary
rehash
	"Re-establish any hash invariants of the receiver.
	 Identity hashes cannot change."
%

category: 'Removing'
method: IdentityKeyValueDictionary
removeKey: aKey ifAbsent: aBlock
  ^ self _removeKey: aKey ifAbsent: aBlock
%

category: 'Removing'
method: IdentityKeyValueDictionary
removeKey: aKey otherwise: defaultValue

  ^ self _removeKey: aKey ifAbsent:[ ^ defaultValue ]
%

category: 'Private'
method: IdentityKeyValueDictionary
_at: aKey otherwise: defaultValue

"Returns the value that corresponds to aKey.  If no such key/value pair
 exists, returns the given alternate value."

| hofs hashKey collisionBkt |

aKey ifNotNil:[
  hofs := (aKey identityHash \\ tableSize) . hofs := hofs + hofs .
  hashKey := self _atZ: hofs  .
  hashKey ifNil: [
    collisionBkt := self _atZ: hofs + 1 .
    collisionBkt ifNotNil: [ ^collisionBkt at: aKey otherwise: defaultValue ]
  ] ifNotNil: [
    (aKey == hashKey) ifTrue: [ ^ self _atZ: hofs + 1 ]
  ].
].
^ defaultValue
%

category: 'Private'
method: IdentityKeyValueDictionary
_at: aKey put: aValue

"Stores the aKey/aValue pair in the hash dictionary.  Rebuilds the hash table
 if the addition caused the number of collisions to exceed the limit allowed."

| hofs hashKey hashValue collisionBkt cbStatus |
aKey ifNil: [ ^ self _error: #rtErrNilKey ].
hofs := (aKey identityHash \\ tableSize) . hofs := hofs + hofs .
hashKey := self _atZ: hofs  .
hashKey ifNil: [
  collisionBkt := self _atZ: hofs + 1 .
  collisionBkt ifNil: [
    self _atZ: hofs putKey: aKey value: aValue .
    numElements := numElements + 1
  ] ifNotNil: [
    cbStatus := collisionBkt at: aKey put: aValue keyValDict_coll: self.
    cbStatus ~~ 0 ifTrue:[
      numElements := numElements + 1.
      cbStatus ~~ 1 ifTrue:[
        numCollisions := numCollisions + 1
      ].
    ].
  ]
] ifNotNil: [
  aKey == hashKey ifTrue: [
    self _atZ: hofs  putKey: aKey value: aValue .
  ] ifFalse: [
    hashValue := self _atZ: hofs + 1 .
    collisionBkt := self collisionBucketClass new.
    collisionBkt objectSecurityPolicy: self objectSecurityPolicy .
    collisionBkt keyValueDictionary: self.
    self _atZ: hofs putKey: nil value: collisionBkt .
    collisionBkt at: hashKey put: hashValue keyValDict_coll: self.
    numElements := numElements + 1.
    cbStatus := collisionBkt at: aKey put: aValue keyValDict_coll: self.
    cbStatus > 1 ifTrue:[
      numCollisions := numCollisions + 1
    ]
  ]
].
(numCollisions > collisionLimit) ifTrue: [
  self rebuildTable: self _nextTableSize 
].
^aValue
%

category: 'Private'
method: IdentityKeyValueDictionary
_initializeWithoutClear: newSize

"Private. Initializes the instance variables of the receiver to be an empty
 KeyValueDictionary of the specified size. Does not clear the contents
 of the receiver - assumes they are all nil."

tableSize := newSize.
numElements := 0.
numCollisions := 0.
collisionLimit := newSize  * 500 .
  "This is where it differs from KeySoftValueDictionary. By setting the default
   collision limit to be high in comparison to the size of the dictionary,
   the dictionary will only be rebuilt when the collision buckets have
   become big."
^self
%

category: 'Private'
method: IdentityKeyValueDictionary
_removeKey: aKey ifAbsent: aBlock

"Removes the key/value pair with key aKey from the receiver and returns
 the value.  If no key/value pair is present with key aKey, evaluates
 the zero-argument block aBlock and returns the result of that evaluation."

| hofs thisKey collisionBkt aValue lastPair |

aKey ifNil:[ ^ self _error: #rtErrNilKey ].
hofs := (aKey identityHash \\ tableSize) . hofs := hofs + hofs .
(thisKey := (self _atZ: hofs )) ifNil:[ "It might be in a collision bucket"
      collisionBkt := self _atZ: hofs + 1.
      collisionBkt ifNil:[
        aBlock ifNil:[^ self _errorKeyNotFound: aKey ] .
        ^ aBlock value
      ].
      aValue := collisionBkt removeKey: aKey ifAbsent: [
        aBlock ifNil:[^ self _errorKeyNotFound: aKey ] .
        ^ aBlock value
      ].
      "If we got this far, a key/value pair was removed from collisionBkt"
      numCollisions := numCollisions - 1.
      (collisionBkt size <= 1) ifTrue: [
         "We don't need the collision bucket anymore.
          Put its last key/value pair in our table."
         lastPair := collisionBkt _firstPair.
         self _atZ: hofs  putKey: (lastPair at:1) value: (lastPair at:2)
      ]
] ifNotNil: [ "There is a key/value pair in this slot"
      (aKey == thisKey) ifFalse: [ ^ self _reportKeyNotFound: aKey with: aBlock ] .
      "The key matches - remove it"
      aValue := self _atZ: hofs + 1.
      self _atZ: hofs putKey: nil value: nil .
].
numElements := numElements - 1.
^aValue
%

category: 'Canonical Symbol Support'
method: IdentityKeyValueDictionary
_selectiveAbort

""

| aKey collisionBkt |

1 to: tableSize * 2 by: 2 do: [ :tableIndex |
  nil == (aKey := self _at: tableIndex)
    ifTrue: [ nil == (collisionBkt := self _at: (tableIndex + 1))
      ifFalse: [collisionBkt _selectiveAbort]].
  ].

super _selectiveAbort
%

! Class extensions for 'IdentitySet'

!		Instance methods for 'IdentitySet'

removeallmethods IdentitySet
removeallclassmethods IdentitySet

category: 'Adding'
method: IdentitySet
add: anObject withOccurrences: anInteger

"Disallowed.  Each element of an IdentitySet must be unique."

self shouldNotImplement: #add:withOccurrences:
%

category: 'Adding'
method: IdentitySet
addAll: aCollection

"Adds each element of aCollection to the receiver only if the element is not
 already present in the receiver.  Occurrences of nil in aCollection are not
 added to the receiver."

^ super addAll: aCollection
%

category: 'Adding'
method: IdentitySet
addValue: anObject

"Adds anObject to the receiver only if it is not already an element of the
 receiver, and if the receiver does not contain an equivalent object.  Has no
 effect if anObject is nil.  Returns anObject."

(self includesValue: anObject) ifFalse:[ self add: anObject ].
^ anObject
%

category: 'Converting'
method: IdentitySet
asIdentitySet

"Returns an IdentitySet with the contents of the receiver."

^ self.
%

category: 'Converting'
method: IdentitySet
asSet

^Set withAll: self
%

category: 'Updating Indexes - Private'
method: IdentitySet
_cleanUpDependencies

"The receiver is having all of its indexes removed.  Remove dependency
 list entries for objects along the path of the indexes.

 Optimized version for IdentitySet where we don't worry about duplicates."

| rootTerms iList rootTermsSize |

iList := self _indexedPaths.
rootTerms := iList rootTerms.
rootTermsSize := rootTerms size .

1 to: self size do: [ :i | | obj |
  obj := self _at: i.
  1 to: rootTermsSize do: [ :j |
        (rootTerms at: j) cleanupDependencyListFor: obj
   ].
].

iList hasTrackingObjects
  ifTrue: [ iList removeAllIndexesFor: self ]
  ifFalse: [ self _indexedPaths: nil ].

" resize the Array of path terms to avoid object audit errors "
rootTerms size: 0.
%

category: 'Set Arithmetic'
method: IdentitySet
_union: aBagOrSet

^ self + aBagOrSet
%

! Class extensions for 'ImproperOperation'

!		Instance methods for 'ImproperOperation'

removeallmethods ImproperOperation
removeallclassmethods ImproperOperation

category: 'Instance initialization'
method: ImproperOperation
args: anArray
  anArray _isArray ifTrue:[ | sz |
    object := anArray atOrNil: 1 .
    (sz := anArray size) > 1 ifTrue:[
      gsArgs := anArray copyFrom:2 to: sz
    ].
  ]
%

category: 'Instance initialization'
method: ImproperOperation
initialize
  gsNumber := ERR_ImproperOperation.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Instance initialization'
method: ImproperOperation
object: anObject
  object := anObject
%

category: 'Instance initialization'
method: ImproperOperation
_legacyHandlerArgs
  | arr |
  (arr := { object } ) addAll: super _legacyHandlerArgs .
  ^ arr
%

! Class extensions for 'IndexingErrorPreventingCommit'

!		Class methods for 'IndexingErrorPreventingCommit'

removeallmethods IndexingErrorPreventingCommit
removeallclassmethods IndexingErrorPreventingCommit

category: 'Private'
classmethod: IndexingErrorPreventingCommit
resignal: anException objWithIndex: anObject
  " check for recursive signal "

  (anException isKindOf: IndexingErrorPreventingCommit)
    ifTrue: [
      | mgr |
      (mgr := IndexManager) current autoCommit
        ifTrue: [
          anObject
            ifNotNil: [ anObject removeIncompleteIndex ]
            ifNil: [ mgr removeAllIncompleteIndexes ] ].
      anException pass ]
    ifFalse: [
      | epcEx |
      (epcEx := IndexingErrorPreventingCommit new) args: {anException}.
      epcEx messageText: anException description.
      epcEx _preventCommits.
      anException resignalAs: epcEx ]
%

!		Instance methods for 'IndexingErrorPreventingCommit'

category: 'Instance initialization'
method: IndexingErrorPreventingCommit
initialize
  gsNumber := ERR_IndexingErrorPreventingCommit.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Private'
method: IndexingErrorPreventingCommit
_preventCommits
  | details |
  (details := gsDetails) ifNil:[
    details := 'anIndexingErrorPreventingCommit'.
    gsDetails := details.
  ].
  System disableCommitsUntilAbortWithReason: details .
%

category: 'Private'
method: IndexingErrorPreventingCommit
_signalWith: inCextensionArg

  self _preventCommits .
  ^ super _signalWith: inCextensionArg
%

! Class extensions for 'Integer'

!		Class methods for 'Integer'

removeallmethods Integer
removeallclassmethods Integer

category: 'Instance Creation'
classmethod: Integer
fromCompleteString: aString

"Returns an instance of the appropriate subclass, reconstructed from aString.
 Leading blanks and trailing blanks are permitted.  Trailing non-digits
 generate an error.

 Smalltalk radix syntax for non-base-10 numbers is supported."

<primitive: 343>

" non-primitive implementation handles error conditions and arguments that
  are not a kind of String "
| s result |

s := ReadStream on: aString.
result := self fromStream: s.
[ s atEnd ]
whileFalse:
  [ (s next == $  )
    ifFalse:
      [ ^ self _errIncorrectFormat: aString ]
  ].
^ result

%

category: 'Instance Creation'
classmethod: Integer
fromHexString: aString

"Returns an instance of the appropriate subclass of the receiver whose
 value is read from the given string."

| ch factor result idx size digit legalChars |

idx := 1.
size := aString size.
size ~~ 0 ifTrue: [
  ch := aString at: idx.
  [ ch == $   ] whileTrue: [
    idx := idx + 1.
    ch := aString at: idx
  ].

  ch == $- ifTrue: [
    idx := idx + 1.
    factor := -1
  ]
  ifFalse: [
    ch == $+ ifTrue: [
      idx := idx + 1.
    ]
  ].
].

result := 0.

legalChars := #($a $b $c $d $e $f $A $B $C $D $E $F) .
(idx > size or: [(ch := aString at: idx) isDigit == false
  and: [ (legalChars includesIdentical: ch) == false]]) ifTrue: [
  ^ self _errIncorrectFormat: aString
].

[ idx > size or: [ (ch := aString at: idx) isDigit == false
  and: [(legalChars includesIdentical: ch) == false]
  ] ] whileFalse: [
  digit := ch > $9 ifTrue: [ch asUppercase codePoint - $A codePoint + 10]
                    ifFalse: [ ch codePoint - $0 codePoint ].
  result := result * 16 + digit.
  idx := idx + 1.
].

(ch isLetter
  and: [(legalChars includesIdentical: ch) == false])
  ifTrue: [
  ^ self _errIncorrectFormat: aString
].

factor ~~ nil ifTrue: [
  result := result * factor
].

^result
%

category: 'Instance Creation'
classmethod: Integer
fromStream: aStream
	"Reads bytes from aStream and returns an instance of the appropriate subclass.
	 Starting at the current position of aStream, leading blanks and trailing blanks
	 are permitted.  Trailing non-digits terminate the conversion without raising any errors.

	 ANSI and Smalltalk-80 radix syntax for is supported (sign before or after the radix, respectively)."

	| ch factor result radix signFound dCount |
	self _checkReadStream: aStream forClass: CharacterCollection.
	ch := aStream next.
	[ch == $  ] whileTrue: [ch := aStream next].
	aStream skip: -1.
	result := 0.
	factor := 1.
	signFound := false.
	(aStream peek == $- )
		ifTrue:
			[aStream next.
			signFound := true.
			factor := -1.]
		ifFalse:
			[(aStream peek == $+ )
				ifTrue:
					[aStream next.
					signFound := true]].

  dCount := 0 .
	[ (ch := aStream peek) ~~ nil and:[ ch isDigit] ] whileTrue:
		[ aStream next.
			result := result * 10 + ch digitValue .
      dCount := dCount + 1 ].
  dCount == 0 ifTrue:[ ^self _errIncorrectFormat: aStream].
	(ch == $r or: [ch == $#])
		ifTrue:
			["found a radix separator"
			aStream next.
			radix := result.
			result := 0.
			(aStream peek == $- )
				ifTrue:
					[signFound
						ifTrue: [^self _errIncorrectFormat: aStream].
					aStream next.
					signFound := true.
					factor := -1]
				ifFalse:
					[(aStream peek == $+ )
						ifTrue:
							[signFound
								ifTrue: [^self _errIncorrectFormat: aStream].
							aStream next.
							signFound := true]].
			[ aStream peek ifNil:[ false ] ifNotNil:[:x| x isSeparator == false] ] whileTrue:
					[ch := aStream next.
					result := result * radix + (ch digitValueInRadix: radix)]].
	^result * factor
%

category: 'Instance Creation'
classmethod: Integer
fromString: aString

"Returns an instance of the appropriate subclass, reconstructed from aString.
 Leading blanks and trailing blanks are permitted.  Trailing non-digits
 terminate the conversion without raising any errors.

 Smalltalk radix syntax for non-base-10 numbers is supported."

<primitive: 267>

" non-primitive implementation handles error conditions and arguments that
  are not a kind of String "
| s result |

s := ReadStream on: aString.
result := self fromStream: s.
^ result
%

category: 'Private'
classmethod: Integer
_finishFromStream: aStream signFound: aSignFound factor: aFactor radix: radix
  "BinaryExponent = ( 'e' | 'E' | 'd' | 'D' | 'q' ) ['-' | '+'] Digits
   DecimalExponent = ( 'f' | 'F' ) ['-' | '+'] Digits
   Digit = '0' | '1' | '2' | ... | '9'
   Digits = Digit {Digit}
   Exponent = BinaryExponent | DecimalExponent | ScaledDecimalExponent | FixedPointExponent
   FractionalPart = '.' Digits [Exponent]
   FixedPointExponent = 'p' [ ['-' | '+'] Digits ]
   Number = RadixedLiteral | NumericLiteral
   Numeric = Digit | 'A' | 'B' | ... | 'Z'
   NumericLiteral = Digits ( [FractionalPart] | [Exponent] )
   Numerics = Numeric { Numeric }
   RadixedLiteral = Digits ( '#' | 'r' ) ['-'] Numerics
   ScaledDecimalExponent = 's' [ ['-' | '+'] Digits ]
"
  "parse Numerics (plus option sign) for RadixedLiteral"

  | number ch signFound factor |
  signFound := aSignFound.
  factor := aFactor.
  number := 0.
  (aStream peek == $- )
    ifTrue: [
      signFound
        ifTrue: [ ^ self _errIncorrectFormat: aStream ].
      aStream next.
      signFound := true.
      factor := -1 ]
    ifFalse: [
      (aStream peek == $+ )
        ifTrue: [
          signFound
            ifTrue: [ ^ self _errIncorrectFormat: aStream ].
          aStream next.
          signFound := true ] ].
  [ aStream peek ifNil:[ false ] ifNotNil:[:x | x isSeparator == false] ]
    whileTrue: [
      ch := aStream next.
      number := number * radix + (ch digitValueInRadix: radix) ].
  ^ number * factor
%

category: 'Instance Creation'
classmethod: Integer
_new: length

"Returns a instance of LargeInteger with specified size and value = 0"

<primitive: 266>
length _validateClass: SmallInteger .
length _error: #rtErrArgOutOfRange
%

category: 'Instance Creation'
classmethod: Integer
_new: length neg: aBoolean

| res |
res := self _new: length .
aBoolean ifTrue:[ res _digitAt:0 put: 1 "set sign bit" ].
^ res
%

category: 'Private'
classmethod: Integer
_newWithValue: aSmallInt

"Provided for use in testing the image .
 Returns a subnormal LargeInteger equal to the argument.

 The absolute value of aSmallInt must be a SmallInteger . "

| res arg isNeg idx |
isNeg := aSmallInt < 0 .
arg := aSmallInt .
isNeg ifTrue:[ arg := arg negated ].
arg _isSmallInteger ifFalse:[ arg _validateClass: SmallInteger] .
res := self _new: 2 neg: isNeg .
idx := 1 .
[arg == 0] whileFalse:[ | aDig |
  aDig := arg bitAnd: 16rffffffff .
  res _digitAt: idx put: aDig .
  idx := idx + 1 .
  arg := arg bitShift: -32 .
].
^ res immediateInvariant .
%

category: 'Constants'
classmethod: Integer
_selectedPrimeGreaterThan: anInteger
  ^ self _selectedPrimeGreaterThan: anInteger limit: 160481189
%

category: 'Constants'
classmethod: Integer
_selectedPrimeGreaterThan: anInteger limit: aLimit

"Returns a prime number that is greater than or equal to anInteger.
 The prime number is not necessarily the smallest prime number that is
 greater than or equal to anInteger.  For anInteger greater than our
 highest selected prime, returns an odd number."

| primes prime hi lo mid |
primes := #( 3 5 7 11 17 19 23 29 37 53 73 107 157 233 347 503 751
853 907 929 947 977 991 1009 1013 1019 1021 1039 
1511 
1723 1829 1901 1933 1969 1979 1987 2003 
2011 2017 2027 2029
3001 4001 5003 6007 7001 8009 9001
10007 11003 12007 13001 14009 15013 16001 17011 18013 19001
20011 21001 22003 23003 24001 25013 26003 27011 28001 29009
30011 31013 32003 33013 34019 35023 36007 37003 38011 39019
40009 41011 42013 43003 44017 45007 46021 47017 48017 49003
50021 51001 52009 53003 54001 55001 56003 57037 58013 59009
60013 61001 62003 63029 64007 65003 66029 67003 68023 69001
70001 71011 72019 73009 74017 75011 76001 77003 78007 79031
80021 81001 82003 83003 84011 85009 86011 87011 88001 89003
90001 91009 92003 93001 94007 95003 96001 97001 98009 99013
100003 101009 102001 103001 104003 224737 350377 479909
611953 746773 882377 1020379 1159523 1299709 2750159 4256233
5800079 7368787 8960453 10570841 12195257 13834103 15485863
32452843 49979687 67867967 86028121 104395301 122949823
141650939 160481183 ).

hi := primes size.
(anInteger > (primes at: hi))
  ifTrue: [
    ^ anInteger odd
      ifTrue: [ anInteger ]
      ifFalse: [ anInteger + 1 ]
  ].

lo := 0.
[       "Binary search for closest selected prime that is >= anInteger"
  mid := (hi + lo) // 2.
  prime := primes at: mid.
  anInteger <= prime
    ifTrue: [ hi := mid ]
    ifFalse: [ lo := mid ].
  hi - lo > 1
] untilFalse.

prime := primes at: hi .
prime > aLimit ifTrue:[ ^ primes at: hi - 1 ].
^ prime 
%

!		Instance methods for 'Integer'

category: 'Arithmetic'
method: Integer
* anInteger

"Returns the product of the receiver and anInteger."

<primitive: 260>

^ self _retry: #* coercing: anInteger
%

category: 'Arithmetic'
method: Integer
+ anInteger

"Returns the sum of the receiver and anInteger."

<primitive: 258>

^ self _retry: #+ coercing: anInteger
%

category: 'Arithmetic'
method: Integer
- anInteger

"Returns the difference between the receiver and anInteger."

<primitive: 259>

^ self _retry: #- coercing: anInteger
%

category: 'Arithmetic'
method: Integer
/ anInteger

"Returns the result of dividing the receiver by anInteger."

<primitive: 261>
(anInteger == 0)
  ifTrue: [^ self _errorDivideByZero].
(anInteger _isInteger)
  ifTrue: [^(Fraction numerator: self denominator: anInteger)]
  ifFalse: [^ self _retry: #/ coercing: anInteger]
%

category: 'Arithmetic'
method: Integer
// anInteger

"Divides the receiver by anInteger.  Returns the integer quotient, with
 truncation toward negative infinity.  For example,

 9//4 = 2
 -9//4 = -3

 The selector \\ returns the remainder from this division."

<primitive: 263>
(anInteger == 0) ifTrue: [^ self _errorDivideByZero].
^ self _retry: #// coercing: anInteger
%

category: 'Comparing'
method: Integer
< aNumber

"Returns true if the receiver is less than aNumber; returns false otherwise."

<primitive: 20>
aNumber _isFloat ifTrue:[ ^ self asFloat < aNumber ].
^ self _retry: #< coercing: aNumber
%

category: 'Comparing'
method: Integer
<= aNumber

"Returns true if the receiver is less than or equal to aNumber; returns false
 otherwise."

<primitive: 21>
aNumber _isFloat ifTrue:[ ^ self asFloat <= aNumber ].
^ self _retry: #<= coercing: aNumber
%

category: 'Comparing'
method: Integer
= aNumber

"Returns true if the receiver is equal to aNumber; returns false otherwise."

<primitive: 24>
aNumber _isFloat ifTrue:[ ^ self asFloat = aNumber ].
aNumber _isNumber ifTrue:[ ^ self _retry: #= coercing: aNumber]
		   ifFalse:[ ^ false ]
%

category: 'Comparing'
method: Integer
> aNumber

"Returns true if the receiver is greater than aNumber; returns false
 otherwise."

<primitive: 22>
aNumber _isFloat ifTrue:[ ^ self asFloat > aNumber ].
^ self _retry: #> coercing: aNumber
%

category: 'Comparing'
method: Integer
>= aNumber

"Returns true if the receive is greater than aNumber; returns false
 otherwise."

<primitive: 23>
aNumber _isFloat ifTrue:[ ^ self asFloat >= aNumber ].
^ self _retry: #>= coercing: aNumber
%

category: 'Bit Manipulation'
method: Integer
allMask: anInteger

"Treats the argument anInteger as a bit mask.  Returns true if all of the bits
 that are 1 in the argument are 1 in the receiver; returns false otherwise."

^ anInteger = (self bitAnd: anInteger)
%

category: 'Bit Manipulation'
method: Integer
anyMask: anInteger

"Treats the argument anInteger as a bit mask.  Returns true if any of the bits
 that are 1 in the argument are 1 in the receiver; returns false otherwise."

^ 0 ~= (self bitAnd: anInteger )
%

category: 'Converting'
method: Integer
asCharacter

"Returns the Character whose value equals the receiver.  Allowable range
 for the receiver is 0 to 65535, inclusive."

^ Character withValue: self
%

category: 'Converting'
method: Integer
asDecimalFloat
"Returns a DecimalFloat representing the receiver."

<primitive: 137>

| factor sum radix |

sum := 0.0F0 .
radix := 4294967295.0F0 .
factor := self sign asFloat.
1 to: self size do: [:i |
  sum := (self _digitAt: i) * factor + sum.
  factor := factor * radix
].
^sum
%

category: 'Converting'
method: Integer
asFloat

"Returns a SmallDouble or Float representing the receiver."

<primitive: 138>

self _primitiveFailed: #asFloat .
self _uncontinuableError
%

category: 'Converting'
method: Integer
asFraction

"Returns self, per ANSI standard"

^ self
%

category: 'Converting'
method: Integer
asHexString

^ self _asHexString: '0123456789abcdef'
%

category: 'Converting'
method: Integer
asHexStringWithLength: length

"Behaves like the asHexString method except that the result
 string is padded with zeros if it is shorter than length
 characters until its size reaches the specified length.

 Has no effect if the hex string size is equal to or
 greater than length."

| string diff |
string :=  self asHexString .
diff := length - string size .
diff > 0
  ifTrue:[ | result |
       result := String new.
       diff timesRepeat:[ result add: $0 ].
       result addAll: string.
       ^ result
].
^ string
%

category: 'Converting'
method: Integer
asInteger

"Returns the receiver."

^ self
%

category: 'Formatting'
method: Integer
asString

"Returns a string representing the receiver.  Positive
 values do not include a leading + ."


<primitive: 265>

self _primitiveFailed: #asString .
self _uncontinuableError
%

category: 'Bit Manipulation'
method: Integer
bitAnd: anInteger

"Returns an Integer whose bits are the logical and of the receiver's bits and
 the bits of anInteger."

<primitive: 740>
anInteger _validateClass: Integer .
self _primitiveFailed: #bitAnd: args: { anInteger }
%

category: 'Bit Manipulation'
method: Integer
bitAt: argInt

"Returns the bit at the ith positin of the receiver, where 1 is the least
 significant bit.
 Gs64 v3.0 changed from 0 based to 1 based argument to meet ANSI standard."

| word digitIndex i |
argInt _isSmallInteger ifFalse:[ argInt _validateClass: SmallInteger ].
argInt < 1 ifTrue:[ argInt _error: #errArgTooSmall args:{ 1 } ].
i := argInt - 1 . "i is zero based"
digitIndex := (i bitShift: -5) + 1.  "32 bits per digit, get a small integer"
word := self _digitAt: digitIndex.
(self _digitAt:0) ~~ 0 ifTrue:[  | index | "rcvr is negative"
  "convert word from ones-complement to twos-complement"
  word := 16rFFFFFFFF - word.
  index := 1.
  [ (index < digitIndex) and:[(self _digitAt: index) == 0] ] whileTrue: [
    index := index + 1
  ].
  index == digitIndex ifTrue: [ word := word + 1 ].
].

^ (word bitShift: 0 - (i bitAnd: 16r1F) ) bitAnd: 1
%

category: 'Bit Manipulation'
method: Integer
bitAt: index put: bitValue
	"Return an integer whose binary representation is identical to the receiver with the exception that
	 the value of the bit at position index is equal to the low order bit of bitValue.
	 The least significant bit of the receiver is designated as position 1, with indices increasing to the left.
	 ANSI says the result is undefined if either the receiver or bitValue is a negative integer.
	 GemStone uses two-complement negative numbers and accepts either or both being negative.
	 It is an error if index is less than or equal to zero.
	 Gs64 v3.0 changed from 0 based to 1 based argument to meet ANSI standard."

	| mask |
	index _isSmallInteger ifFalse: [index _validateClass: SmallInteger].
	index < 1 ifTrue: [^index _error: #errArgTooSmall args: {1}].
	mask := 1 bitShift: index - 1.
	bitValue == 1 ifTrue: [^self bitOr: mask].
	bitValue == 0 ifTrue: [^self bitAnd: -1 - mask	"inline bitInvert"].
	^self bitAt: index put: (bitValue bitAt: 1).
%

category: 'Bit Manipulation'
method: Integer
bitInvert

"Returns an Integer whose bits are the one's-complement of the receiver."

^ -1 - self
%

category: 'Bit Manipulation'
method: Integer
bitOr: anInteger

"Returns an Integer whose bits are the logical or of the receiver's bits and
 the bits of anInteger."

<primitive: 741>
anInteger _validateClass: Integer.
self _primitiveFailed: #bitOr: args: { anInteger }
%

category: 'Bit Manipulation'
method: Integer
bitShift: aSmallInteger

"Returns an Integer whose value (in two's-complement representation) is the
 receiver's value (also in two's-complement representation) shifted by
 anInteger bits.

 If anInteger is positive, the shift is to the left, and zero-bits enter at
 the right.  If anInteger is negative, the shift is to the right, and the sign
 bit is repeated at the left."

 | res |
 res := self _bitShift: aSmallInteger .
 (aSmallInteger < 0 and:[ self < 0]) ifTrue:[
   (self bitAnd: (1 bitShift: 0 - aSmallInteger) - 1 ) == 0 ifFalse:[
     res := res - 1
   ].
 ].
 ^ res
%

category: 'Bit Manipulation'
method: Integer
bitXor: anInteger

"Returns an Integer whose bits are the logical xor of the receiver's bits and
 the bits of anInteger."

<primitive: 743>
anInteger _validateClass: Integer .
self _primitiveFailed: #bitXor: args: { anInteger }
%

category: 'Truncation and Rounding'
method: Integer
ceiling

"Returns the receiver."

^ self
%

category: 'Accessing'
method: Integer
denominator

"For an Integer, always returns 1."

^1
%

category: 'Arithmetic'
method: Integer
divMod: aNumber into: anArray

"anArray should be an Array of size 2.
 Performs
   anArray at: 1 put: (self // aNumber) .
   anArray at: 2 put: (self \\ aNumber) .
 Uses the large integer divide C code which produces both quotient
 and remainder in a single pass .
 Returns anArray .
"
<primitive: 949>
(aNumber = 0) ifTrue:[^ self _errorDivideByZero].
anArray _isArray ifFalse:[ anArray _validateClass: Array ].
^ self _retry: #divMod:into: coercing: aNumber into: anArray
%

category: 'Testing'
method: Integer
even
  ^ (self bitAt: 1) == 0
%

category: 'Arithmetic'
method: Integer
factorial

"Returns the factorial of the receiver.  Returns 1 if the
 receiver is less than or equal to 1."

| x result |
result := 1  .
x := result .
self timesRepeat:[ result := result * x .  x := x + 1 ] .
^ result .
%

category: 'Truncation and Rounding'
method: Integer
floor

"Returns the receiver."

^ self
%

category: 'Truncation and Rounding'
method: Integer
fractionPart
  "Returns the fraction remaining after the receiver is truncated toward zero."
  ^ 0
%

category: 'Divisibility'
method: Integer
gcd: anInteger

"Answer the greatest common divisor of the receiver and anInteger."

"Answer the largest non-negative integer that divides both the receiver
and anInteger with no remainder.
Answer 0 if the receiver and anInteger are both zero."

| divisor dividend remainder |
anInteger _isInteger ifFalse:[ anInteger  _validateClass: Integer ].
dividend := self negative ifTrue:[ 0 - self] ifFalse:[ self ].
divisor := anInteger negative ifTrue:[ 0 - anInteger ] ifFalse:[ anInteger].
remainder := divisor .
[remainder = 0] whileFalse: [
  remainder := dividend \\ divisor.
  dividend := divisor.
  divisor := remainder
].
^dividend
%

category: 'Comparing'
method: Integer
hash

"Returns a numeric hash index.  For an Integer representable in 30 bits,
 returns a SmallInteger equal to the receiver, otherwise converts
 receiver to a Float and returns the hash of the Float. "

<primitive: 530>
self _primitiveFailed: #hash .
self _uncontinuableError
%

category: 'Bit Manipulation'
method: Integer
highBit

"(Subclass responsibility.)  Returns the index of the high-order bit that is set
 in the binary representation of the receiver.  (If the receiver is a negative
 integer, takes its absolute value first.)  If the receiver is zero, this
 returns nil.
 As of Gs64 v3.2  the least significant bit has index 1 ,
 in previous versions the least significant bit had index 0 .

 Example:   4 highBit == 3  "

Integer subclassResponsibility: #highBit
%

category: 'Testing'
method: Integer
isPowerOf2

"Answer true if the receiver is a power of 2.  Otherwise answer false."
self == 0
  ifTrue:[^ false].

^ (self bitAnd: (self - 1)) == 0
%

category: 'Divisibility'
method: Integer
lcm: anInteger
	"Answer the least common multiple of the receiver and anInteger.
	 Answer the smallest non-negative integer which is evenly divided by
	 both the receiver and operand. Answer 0 if both are zero."

	^(self = 0 and: [anInteger = 0])
		ifTrue: [0]
		ifFalse: [(self // (self gcd: anInteger) * anInteger) abs].
%

category: 'Bit Manipulation'
method: Integer
noMask: anInteger

"Treats the argument anInteger as a bit mask.  Returns true if none of the bits
 that are 1 in the argument are 1 in the receiver; returns false otherwise."

^ 0 = (self bitAnd: anInteger)
%

category: 'Accessing'
method: Integer
numerator

"For an Integer, always returns the receiver."

^self
%

category: 'Testing'
method: Integer
odd
  ^ (self bitAt: 1) == 1
%

category: 'Json'
method: Integer
printJsonOn: aStream

	self printOn: aStream.
%

category: 'Formatting'
method: Integer
printOn: aStream base: b

"Prints a representation of the receiver on aStream in base b.  The base b
 must be 2 <= b <= 36.  Returns the receiver."

^ self printOn: aStream base: b showRadix: true.
%

category: 'Formatting'
method: Integer
printOn: aStream base: b showRadix: aBoolean

"Prints a representation of the receiver on aStream in base b.  The base b
 must be 2 <= b <= 36.  Returns the receiver."

aStream nextPutAll: (self printStringRadix: b showRadix: aBoolean)
%

category: 'Formatting'
method: Integer
printStringRadix: base
	"Return a string containing a sequence of characters that represents the numeric value of the
	 receiver in the radix specified by the argument. The sequence of characters must be
	 recognizable using the radixDigits production of the Smalltalk Lexical Grammar as if the numeric
	 value of the radixSpecifier was base. If the receiver is negative, a minus sign ('-') is prepended
	 to the sequence of characters. The result is undefined if base is less than two or greater than 36."

	^self printStringRadix: base showRadix: false
%

category: 'Formatting'
method: Integer
printStringRadix: base showRadix: aBoolean
	"Returns a String that describes the receiver in the specified radix.
	 The base b must be 2 <= b <= 36."

	| str b |
	self < 0
		ifTrue: [^'-' , (self negated printStringRadix: base showRadix: aBoolean)].
	b := base.
	(b between: 2 and: 36)
		ifFalse: [^self _error: #rtErrInvalidArgument args: {b}].
	b == 10
		ifTrue: [str := self asString]
		ifFalse:
			[b == 16
				ifTrue: [str := self _asHexString: '0123456789ABCDEF']
				ifFalse:
					[| abs digits quorem |
					abs := self.
					digits := #($0 $1 $2 $3 $4 $5 $6 $7 $8 $9 $A $B $C $D $E $F $G $H $I $J $K $L $M $N $O $P $Q $R $S $T $U $V $W $X $Y $Z).
					quorem := {nil. nil}.
					str := String new.
					abs == 0
						ifTrue: [str add: $0]
						ifFalse:
							[[abs > 0] whileTrue:
									[abs quoRem: b into: quorem.
									str add: (digits at: (quorem at: 2) + 1).
									abs := quorem at: 1]].
					str := str reverse]].
	aBoolean
		ifTrue:
			[| res |
			(res := b asString)
				add: $r;
				addAll: str.
			^res].
	^str
%

category: 'Arithmetic'
method: Integer
quo: anInteger

"Divides the receiver by anInteger.  Returns the integer quotient, with
 truncation toward zero.  For example,
   -9 quo: 4 = -2
 The selector rem: returns the remainder from this division.
 If you need both quotient and remainder, use  Integer>>quoRem:into:  "

<primitive: 262>
(anInteger == 0) ifTrue: [^ self _errorDivideByZero].
^ self _retry: #quo: coercing: anInteger
%

category: 'Arithmetic'
method: Integer
quoRem: aNumber into: anArray

"anArray should be an Array of size 2.
 Performs
   anArray at: 1 put: (self quo: aNumber) .
   anArray at: 2 put: (self rem: aNumber) .
 Uses the large integer divide C code which produces both quotient
 and remainder in a single pass .
 Returns anArray .
"
<primitive: 789>
(aNumber = 0) ifTrue: [^ self _errorDivideByZero].
anArray _isArray ifFalse:[ anArray _validateClass: Array ].
^ self _retry: #quoRem:into: coercing: aNumber into: anArray
%

category: 'Arithmetic'
method: Integer
rem: aNumber

"Returns the integer remainder defined in terms of quo: (division
 of the receiver by aNumber, with truncation toward zero)."

aNumber _isInteger ifTrue:[ | arr |
  (aNumber == 0) ifTrue: [^ self _errorDivideByZero].
  arr := { nil . nil }.
  self quoRem: aNumber into: arr .
  ^ arr at: 2
].
^ super rem: aNumber
%

category: 'Truncation and Rounding'
method: Integer
roundAndCoerceTo: aNumber

"Returns the multiple of aNumber that is nearest in value to the receiver."

aNumber = 0
  ifTrue: [^0]
  ifFalse: [^(self / aNumber) rounded * aNumber]
%

category: 'Truncation and Rounding'
method: Integer
rounded

"Returns the receiver."

^ self
%

category: 'Truncation and Rounding'
method: Integer
roundedHalfToEven
	"Returns the integer nearest in value to the receiver. If the receiver is
	exactly halfway between two integers, return the even one."

	^self
%

category: 'Accessing'
method: Integer
size: anInteger

"Disallowed.  You may not change the size of an Integer."

self shouldNotImplement: #size:
%

category: 'Truncation and Rounding'
method: Integer
truncateAndCoerceTo: aNumber

"Returns the multiple of aNumber that is closest to the receiver, on
 the same side of the receiver as zero is located.  In particular,
 returns the receiver if the receiver is a multiple of aNumber."

aNumber = 0
  ifTrue: [^0]
  ifFalse: [^(self quo: aNumber) * aNumber]
%

category: 'Truncation and Rounding'
method: Integer
truncated

"Returns the receiver."

^ self
%

category: 'Arithmetic'
method: Integer
\\ anInteger

"Returns the modulus defined in terms of //.  Returns a Number with the same
 sign as the argument anInteger.  For example,

 9\\4 = 1
 -9\\4 = 3
 9\\-4 = -3"

<primitive: 264>
(anInteger == 0)
  ifTrue: [^ self _errorDivideByZero].
^ super \\ anInteger
%

category: 'Converting'
method: Integer
_asHexString: hexChars

"Returns a String representing the receiver as a base-16 number."

| val len dig mask shift str |

self < 0 ifTrue:[
  val := 0 - self . str := '-' copy .
] ifFalse:[
  self == 0 ifTrue:[ ^ '0' ].
  str := String new .
  val := self .
].
len := val _digitLength .
dig := val _digitAt: len . "most significant 32bit digit"
mask := 16rF0000000 .
shift := -28 .
[ mask ~~ 0 and:[ (dig bitAnd: mask) == 0] ] whileTrue:[
   mask := mask bitShift: -4 .  "skip leading zeros"
   shift := shift + 4 .
].
[ mask ~~ 0 ] whileTrue:[
  str add: (hexChars at: 1 + (( dig bitShift: shift ) bitAnd: 16rF)).
  mask := mask bitShift: -4 .
  shift := shift + 4 .
].
len - 1 _downTo: 1 do:[:n |
  dig := val _digitAt: n .
  mask := 16rF0000000 .
  shift := -28 .
  [ mask ~~ 0 ] whileTrue:[
    str add: (hexChars at: 1 + (( dig bitShift: shift ) bitAnd: 16rF)).
    mask := mask bitShift: -4 .
    shift := shift + 4 .
  ].
].
str size == 0 ifTrue: [
  str add: $0 .
].
^ str
%

category: 'Private'
method: Integer
_bitShift: aSmallInteger
  <primitive: 742>
  aSmallInteger _validateClass: SmallInteger.
  self _primitiveFailed: #_bitShift: args: { aSmallInteger }
%

category: 'Private'
method: Integer
_bottomBit
 "sender must ensure receiver is positive,
  used by raisedTo methods"

  ^ (self _digitAt: 1) bitAnd: 1
%

category: 'Converting'
method: Integer
_coerce: aNumber

"Coerces an Integer."

aNumber _isSmallInteger ifTrue:[
  ^ Integer _newWithValue: aNumber
].
^ aNumber truncated
%

category: 'Accessing'
method: Integer
_decimalDigitsLength: useApproximationBoolean

"Returns the number of decimal digits required to represent the absolute value of
 the receiver, ignoring any leading zero bits to left of least significant bit.
 Result is always >= 1

 If useApproximationBoolean == true, the result is approximated for
 receivers not representable as a SmallInteger.
 The approximation is
    nd := ((self _lastDigit _decimalDigitsLength: false)
           + (self _digitLength - 1) * 9.64 ) truncated .
 Where log base 10(16rFFFFFFFF) == 9.63...  .
 The approximate result is within +/- 1 of the exact result for results
 <= 2860 ."

<primitive: 852>
self _primitiveFailed: #_decimalDigitsLength: args: { useApproximationBoolean }
%

category: 'Accessing'
method: Integer
_digitAt: anIndex

"Returns the specified 32bit unsigned word from a sign-magnitude representation
 of the receiver.
 anIndex should be a SmallInteger >= 0 .
 _digitAt:0 is the sign word , which contains either 0 or 1, 1 means negative.
 anIndex beyond most significant digit will return 0.
 The result will be a SmallInteger >= 0 "

<primitive: 18>

anIndex _validateClass: SmallInteger.
anIndex < 0 ifTrue:[ anIndex _error: #errArgTooSmall args:{ 0 } ].
self _primitiveFailed: #_digitAt: args: { anIndex } .
self _uncontinuableError
%

category: 'Accessing'
method: Integer
_digitLength

"Returns the number of base 4294967296 digits in the receiver, without counting
 any leading zeros or the sign digit."

<primitive: 19>

self _primitiveFailed: #_digitLength .
self _uncontinuableError
%

category: 'Converting'
method: Integer
_generality

"Returns the generality of Integer."

^ 40
%

category: 'Private'
method: Integer
_isLeapYear

"Returns true if the year represented by the value of the receiver is
 a leap year, false otherwise.  Valid at least for years 1900 and later."

 "Useful in debugging Date and DateTime."

(self \\ 100) = 0 ifTrue:[
   ^ (self \\ 400) = 0
   ].
^ (self \\ 4) = 0
%

category: 'Accessing'
method: Integer
_lastDigit

"Returns the last base 4294967296 digit of the receiver."

^self _digitAt: self _digitLength
%

category: 'Testing'
method: Integer
_validateIsPowerOf2

self isPowerOf2
  ifFalse:[ self _error: #rtErrNotPowerOfTwo args: { self } ] .
%

category: 'Comparing'
method: Integer
~= aNumber

"Returns true if the receiver is not equal to aNumber; returns false
 otherwise."

<primitive: 25>
aNumber _isFloat ifTrue:[ ^ self asFloat ~= aNumber ].
aNumber _isNumber ifTrue:[ ^ self _retry: #~= coercing: aNumber]
		   ifFalse:[ ^ true ]
%

! Class extensions for 'IntegerKeyValueDictionary'

!		Instance methods for 'IntegerKeyValueDictionary'

removeallmethods IntegerKeyValueDictionary
removeallclassmethods IntegerKeyValueDictionary

category: 'Accessing'
method: IntegerKeyValueDictionary
at: aKey ifAbsent: aBlock

"Returns the value that corresponds to aKey.  If no such key/value pair
 exists, returns the given alternate value."

| hofs hashKey collisionBkt |

aKey ifNotNil:[
  hofs := (aKey hash \\ tableSize) . hofs := hofs + hofs .
  hashKey := self _atZ: hofs  .
  hashKey ifNil: [
      collisionBkt := self _atZ: hofs + 1 .
      collisionBkt ifNotNil: [ ^ collisionBkt at: aKey ifAbsent: aBlock ]
  ] ifNotNil: [
    (aKey = hashKey) ifTrue: [ ^ self _atZ: hofs + 1 ]
  ].
].
aBlock ifNil:[^ self _errorKeyNotFound: aKey ] .
^ aBlock value
%

category: 'Accessing'
method: IntegerKeyValueDictionary
at: aKey otherwise: defaultValue

"Returns the value that corresponds to aKey.  If no such key/value pair
 exists, returns the given alternate value."

| hofs hashKey collisionBkt |

aKey ifNotNil:[
  hofs := (aKey hash \\ tableSize) . hofs := hofs + hofs .
  hashKey := self _atZ: hofs  .
  hashKey ifNil: [
    collisionBkt := self _atZ: hofs + 1 .
    collisionBkt ifNotNil: [ ^collisionBkt at: aKey otherwise: defaultValue ]
  ] ifNotNil: [
    (aKey = hashKey) ifTrue: [ ^ self _atZ: hofs + 1 ]
  ].
].
^ defaultValue
%

category: 'Updating'
method: IntegerKeyValueDictionary
at: aKey put: aValue

"Stores the aKey/aValue pair in the KeyValueDictionary.  Generates an error if
 aKey is not a kind of Integer."

aKey _isInteger ifFalse: [ ^ aKey _error: #rtErrBadArgKind args: { Integer }].

^ self _at: aKey put: aValue
%

category: 'Private'
method: IntegerKeyValueDictionary
collisionBucketClass

"Returns the class of object to create when keys collide."

^ EqualityCollisionBucket
%

category: 'Hashing'
method: IntegerKeyValueDictionary
compareKey: key1 with: key2

"No longer used by implementation of IntegerKeyValueDictionary"

self shouldNotImplement: #compareKey:with:
%

category: 'Hashing'
method: IntegerKeyValueDictionary
hashFunction: aKey

"No longer used by implementation of IntegerKeyValueDictionary"

self shouldNotImplement: #hashFunction:
%

category: 'Accessing'
method: IntegerKeyValueDictionary
keyAt: aKey otherwise: defaultValue

"Returns the key that corresponds to aKey.  If no such key/value pair
 exists, returns the given alternate value. "

| hofs hashKey collisionBkt |

aKey ifNotNil:[
  hofs := (aKey hash \\ tableSize) . hofs := hofs + hofs .
  hashKey := self _atZ: hofs  .
  hashKey ifNil: [
    collisionBkt := self _atZ: hofs + 1 .
    collisionBkt ifNotNil:[ ^ collisionBkt keyAt: aKey otherwise: defaultValue ]
  ] ifNotNil: [
    (aKey = hashKey) ifTrue:[ ^ hashKey ].
  ].
].
^ defaultValue

%

category: 'Hashing'
method: IntegerKeyValueDictionary
rehash
	"Re-establish any hash invariants of the receiver.
	 Integer hashes cannot change."
%

category: 'Removing'
method: IntegerKeyValueDictionary
removeKey: aKey ifAbsent: aBlock
  ^ self _removeKey: aKey ifAbsent: aBlock
%

category: 'Removing'
method: IntegerKeyValueDictionary
removeKey: aKey otherwise: defaultValue

  ^ self _removeKey: aKey ifAbsent:[ ^ defaultValue]
%

category: 'Private'
method: IntegerKeyValueDictionary
_at: aKey put: aValue

"Stores the aKey/aValue pair in the hash dictionary.  Rebuilds the hash table
 if the addition caused the number of collisions to exceed the limit allowed."

| hofs hashKey hashValue collisionBkt cbStatus |
aKey ifNil: [ ^ self _error: #rtErrNilKey ].
hofs := (aKey hash \\ tableSize) . hofs := hofs + hofs .
hashKey := self _atZ: hofs  .
hashKey ifNil: [
  collisionBkt := self _atZ: hofs + 1 .
  collisionBkt ifNil: [
    self _atZ: hofs putKey: aKey value: aValue .
    numElements := numElements + 1
  ] ifNotNil: [
    cbStatus := collisionBkt at: aKey put: aValue keyValDict_coll: self.
    cbStatus ~~ 0 ifTrue:[
      numElements := numElements + 1.
      cbStatus ~~ 1 ifTrue:[
        numCollisions := numCollisions + 1
      ].
    ].
  ]
] ifNotNil: [
  aKey = hashKey ifTrue: [
    self _atZ: hofs putKey: aKey value: aValue .
  ] ifFalse: [
    hashValue := self _atZ: hofs + 1 .
    collisionBkt := self collisionBucketClass new.
    collisionBkt objectSecurityPolicy: self objectSecurityPolicy .
    collisionBkt keyValueDictionary: self.
    self _atZ: hofs putKey: nil value: collisionBkt .
    collisionBkt at: hashKey put: hashValue keyValDict_coll: self.
    numElements := numElements + 1.
    cbStatus := collisionBkt at: aKey put: aValue keyValDict_coll: self.
    cbStatus > 1 ifTrue:[
      numCollisions := numCollisions + 1
    ]
  ]
].
(numCollisions > collisionLimit) ifTrue: [
  self rebuildTable: self _nextTableSize .
].
^aValue
%

category: 'Private'
method: IntegerKeyValueDictionary
_removeKey: aKey ifAbsent: aBlock

"Removes the key/value pair with key aKey from the receiver and returns
 the value.  If no key/value pair is present with key aKey, evaluates
 the zero-argument block aBlock and returns the result of that evaluation."

| hofs thisKey collisionBkt aValue lastPair |

aKey ifNil:[ ^ self _error: #rtErrNilKey ].
hofs := (aKey hash \\ tableSize) . hofs := hofs + hofs .
(thisKey := (self _atZ: hofs )) ifNil:[ "It might be in a collision bucket"
      collisionBkt := self _atZ: hofs + 1.
      collisionBkt ifNil:[
        aBlock ifNil:[^ self _errorKeyNotFound: aKey ] .
        ^ aBlock value
      ].
      aValue := collisionBkt removeKey: aKey ifAbsent: [
        aBlock ifNil:[^ self _errorKeyNotFound: aKey ] .
        ^ aBlock value
      ].
      "If we got this far, a key/value pair was removed from collisionBkt"
      numCollisions := numCollisions - 1.
      (collisionBkt size <= 1) ifTrue: [
         "We don't need the collision bucket anymore.
          Put its last key/value pair in our table."
         lastPair := collisionBkt _firstPair.
         self _atZ: hofs putKey: (lastPair at:1) value: (lastPair at:2) .
      ]
] ifNotNil: [ "There is a key/value pair in this slot"
      (aKey = thisKey) ifFalse: [
        aBlock ifNil:[^ self _errorKeyNotFound: aKey ] .
        ^ aBlock value
      ].
      "The key matches - remove it"
      aValue := self _atZ: hofs + 1.
      self _atZ: hofs putKey: nil value: nil .
].
numElements := numElements - 1.
^aValue
%

! Class extensions for 'InternalError'

!		Instance methods for 'InternalError'

removeallmethods InternalError
removeallclassmethods InternalError

category: 'Instance initialization'
method: InternalError
initialize
  gsNumber := ERR_InternalError.
  gsResumable := false .
  gsTrappable := true .
%

! Class extensions for 'InterSessionSignal'

!		Class methods for 'InterSessionSignal'

removeallmethods InterSessionSignal
removeallclassmethods InterSessionSignal

category: 'Signalling'
classmethod: InterSessionSignal
disableSignalling

"Prevent instances of InterSessionSignal from being signalled by the VM"

 System _updateSignalErrorStatus: 3 toState: false
%

category: 'Signalling'
classmethod: InterSessionSignal
enableSignalling

"Enable instances of InterSessionSignal to be signalled by the VM .
 must be resent by exception handling code after handling each
 instance of InterSessionSignal that was signalled by the VM."

 System _updateSignalErrorStatus: 3 toState: true
%

category: 'Signalling'
classmethod: InterSessionSignal
poll
"Returns a new instance of InterSessionSignal if a signal is
 available from another session, or nil ."

| sigArray ex |
sigArray := System _signalFromGemStoneSession .
sigArray ifNil:[ ^ nil ].
(ex := self new) fromSession:  (sigArray at: 1)
              signal: (sigArray at: 2) message: (sigArray at: 3)
              numPending: ( sigArray at: 4).
^ ex
%

category: 'Signalling'
classmethod: InterSessionSignal
pollAndSignal
  "Signal a new instance of InterSessionSignal if a signal is
    available from another session, otherwise do nothing."

  self poll ifNotNil:[ :ex | ex signal ].
%

category: 'Signalling'
classmethod: InterSessionSignal
sendSignal: aSmallInt to: aSessionId withMessage: aString

"Send a signal to another VM which can result in an InterSessionSignal
 being signalled within that VM."

^ System sendSignal: aSmallInt to: aSessionId withMessage: aString
%

category: 'Signalling'
classmethod: InterSessionSignal
signallingEnabled

"Return true if signaling enabled, return false otherwise  .
 See InterSessionSignal(C)>>enableSignalling "

^ System _signalErrorStatus: 3
%

!		Instance methods for 'InterSessionSignal'

category: 'Formatting'
method: InterSessionSignal
buildMessageText
^ self buildMessageText:(
  [ | str |
    (str := String withAll:'fromSession=') add: sendingSession asString;
      add: ' signal:'; add: signal asString ;
      add: ' message: ' ; add: (gsArgs atOrNil: 1) asString .
    gsDetails ifNotNil:[:d| str add:', '; add: d  asString ].
    str
  ] onException: Error do:[:ex |
    ex return: nil
  ]
)
%

category: 'Handling'
method: InterSessionSignal
defaultAction

"Return an error to the controlling GCI client .
 See AbstractException>>defaultAction for documentation."

<primitive: 33>
self _uncontinuableError
%

category: 'Handling'
method: InterSessionSignal
fromSession: sessionIdInt signal: signalInt message: msgString
  gsNumber := ERR_InterSessionSignal .
  sendingSession := sessionIdInt .
  signal := signalInt .
  gsArgs := { msgString . 0} .
%

category: 'Handling'
method: InterSessionSignal
fromSession: sessionIdInt signal: signalInt message: msgString numPending: anInt
  gsNumber := ERR_InterSessionSignal .
  sendingSession := sessionIdInt .
  signal := signalInt .
  gsArgs := { msgString . anInt } .
%

category: 'Instance initialization'
method: InterSessionSignal
initialize
  gsNumber := ERR_InterSessionSignal.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Accessing'
method: InterSessionSignal
numPending
  "Returns a SmallInteger the number pending in stone beyond this one
   at the point this one was obtained from stone."
  | res |
  res := gsArgs ifNotNil:[:a | a atOrNil: 2 ].
  res ifNil:[ res := 0 ].
  ^ res
%

category: 'Signalling'
method: InterSessionSignal
replyToSenderWithSignal: aSmallInteger withString: aString

"Reply to the VM which caused this instance "

^ System sendSignal: aSmallInteger to: self sendingSessionId withMessage: aString
%

category: 'Deprecated'
method: InterSessionSignal
sendingSession
  self deprecated: 'Exception>>sendingSession deprecatedi v3.0, use #sendingSessionId or #sendingSessionSerial'.
  ^ sendingSession
%

category: 'Accessing'
method: InterSessionSignal
sendingSessionId
  ^ GsSession sessionIdOfSerial: sendingSession
%

category: 'Accessing'
method: InterSessionSignal
sendingSessionSerial
  ^ sendingSession
%

category: 'Accessing'
method: InterSessionSignal
sentInt
  ^ signal
%

category: 'Accessing'
method: InterSessionSignal
sentMessage
  ^ gsArgs ifNotNil:[:a | a atOrNil: 1 ].
%

category: 'Formatting'
method: InterSessionSignal
_legacyHandlerArgs
  | arr |
  (arr := { sendingSession . signal } ) addAll: super _legacyHandlerArgs .
  ^ arr
%

! Class extensions for 'InvariantString'

!		Instance methods for 'InvariantString'

removeallmethods InvariantString
removeallclassmethods InvariantString

category: 'Formatting'
method: InvariantString
asString

"Returns a copy of the receiver as an instance of class String."

^ String withAll: self
%

category: 'Formatting'
method: InvariantString
species
  "(R) Returns a class similar to, or the same as, the receiver's class which
   can be used for containing derived copies of the receiver."

  ^ String
%

category: 'Formatting'
method: InvariantString
_quoted

"Returns a copy of the receiver enclosed in single-quote marks, with contained
 single-quote Characters doubled.  The copy will be of class String.  This is
 usually done to prepare a storable version of the receiver."

| result c rs sz |

sz := self size.
result := String new: sz + 2.
result at: 1 put: $'.
rs := 1.
1 to: sz do: [:i |
  (c := self at: i) == $' ifTrue: [
    rs := rs + 1.
    result at: rs put: $'.
  ].
  rs := rs + 1.
  result at: rs put: c.
].
rs := rs + 1.
result at: rs put: $'.
^result
%

! Class extensions for 'IOError'

!		Instance methods for 'IOError'

removeallmethods IOError
removeallclassmethods IOError

category: 'Instance initialization'
method: IOError
initialize
  gsNumber := ERR_IOError.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'KeyValueDictionary'

!		Class methods for 'KeyValueDictionary'

removeallmethods KeyValueDictionary
removeallclassmethods KeyValueDictionary

category: 'Storing and Loading'
classmethod: KeyValueDictionary
loadFrom: passiveObj mappingToClass: newClass

"Reads from passiveObj the passive form of an object with named instance
 variable format.  Converts the object to its active form by loading the
 information into a new instance of the receiver.  Returns the new instance."

| newDict size |
(passiveObj version >= 500 or:[ newClass == self]) ifTrue:[
  size := passiveObj readSize.
  newDict := self new.
  newDict loadFrom: passiveObj size: size.
  ]
ifFalse:[
  self halt: 'no support for activation of objects written by 4.1'
].
^ newDict
%

category: 'Instance Creation'
classmethod: KeyValueDictionary
new

"Creates an instance of KeyValueDictionary with a default table size."

^self new: 5
%

category: 'Instance Creation'
classmethod: KeyValueDictionary
new: tableSize

"Creates an instance of KeyValueDictionary based upon the specified table size."

| newSize newDict |

newSize := self _tableSizeFor: tableSize.
newDict := self _basicNew: (newSize + newSize).

newDict _initializeWithoutClear: newSize.
^ newDict.
%

!		Instance methods for 'KeyValueDictionary'

category: 'Enumerating'
method: KeyValueDictionary
accompaniedBy: anObj keysAndValuesDo: aBlock

"Evaluates aBlock with each of the receiver's key/value pairs as the
 2nd and 3rd arguments.
 aBlock must be a 3 argument block, with arguments anObj, key value ."

| aKey collisionBkt |

1 to: tableSize * 2 by: 2 do: [ :tableIndex |
  (aKey := self _at: tableIndex) ifNil: [
    (collisionBkt := self _at: (tableIndex + 1)) ifNotNil: [
      1 to: collisionBkt _basicSize by: 2 do: [ :j |
        (aKey := collisionBkt _at: j) ifNotNil: [
          aBlock value: anObj value: aKey value: (collisionBkt _at: j + 1).
	].
      ].
    ].
  ] ifNotNil: [
    aBlock value: anObj value: aKey value: (self _at: tableIndex + 1).
  ].
].
%

category: 'Enumerating'
method: KeyValueDictionary
associationsDo: aBlock

"Evaluates aBlock with each of the receiver's key/value pairs as the
 argument by creating an Association for each key/value pair.  The
 argument aBlock must be a one-argument block."

self keysAndValuesDo: [:aKey :aValue |
   aBlock value: (Association newWithKey: aKey value: aValue)
].
^self
%

category: 'Accessing'
method: KeyValueDictionary
at: aKey ifAbsent: aBlock

"Returns the value that corresponds to aKey.  If no such key/value pair
 exists, returns the result of evaluating the zero-argument block aBlock."

| hash hashKey collisionBkt |

aKey ifNil:[ ^self _reportKeyNotFound: aKey with: aBlock ].
hash := self hashFunction: aKey.
hashKey := self keyAt: hash.
hashKey ifNil:[
   (collisionBkt := self valueAt: hash) ifNotNil:[ ^collisionBkt at: aKey ifAbsent: aBlock ]
] ifNotNil:[
  (self compareKey: aKey with: hashKey) ifTrue: [ ^self valueAt: hash ]
].
^ self _reportKeyNotFound: aKey with: aBlock
%

category: 'Accessing'
method: KeyValueDictionary
at: aKey otherwise: aValue

"Returns the value that corresponds to aKey.  If no such key/value pair
 exists, returns the given alternate value."

| hash hashKey collisionBkt |

aKey ifNil:[ ^ aValue ].
hash := self hashFunction: aKey.
(hashKey := self keyAt: hash) ifNil: [
  (collisionBkt := self valueAt: hash) ifNotNil:[
    ^collisionBkt at: aKey otherwise: aValue
  ]
] ifNotNil: [
  (self compareKey: aKey with: hashKey) ifTrue: [ ^self valueAt: hash ]
].
^ aValue
%

category: 'Updating'
method: KeyValueDictionary
at: aKey put: aValue

"Stores the aKey/aValue pair in the hash dictionary.  Rebuilds the hash table
 if the addition caused the number of collisions to exceed the limit allowed."

"Code is duplicated in _at:put:"

| hash hashKey hashValue collisionBkt |
aKey ifNil: [ ^ self _error: #rtErrNilKey ].
hash := self hashFunction: aKey.
hashKey := self keyAt: hash.
hashKey ifNil:[
  collisionBkt := self valueAt: hash.
  collisionBkt ifNil:[
    self atHash: hash putKey: aKey value: aValue .
    numElements := numElements + 1
  ] ifNotNil: [  | cbStatus |
    cbStatus := collisionBkt at: aKey put: aValue keyValDict_coll: self.
    cbStatus ~~ 0 ifTrue:[
      numElements := numElements + 1.
      cbStatus ~~ 1 ifTrue:[
        numCollisions := numCollisions + 1
      ].
    ].
  ]
] ifNotNil:[
  (self compareKey: aKey with: hashKey) ifTrue: [
    self atHash: hash putKey: aKey value: aValue .
  ] ifFalse: [  | cbStatus |
    hashValue := self valueAt: hash.
    collisionBkt := self collisionBucketClass new.
    collisionBkt objectSecurityPolicy: self objectSecurityPolicy .
    collisionBkt keyValueDictionary: self.
    self atHash: hash putKey: nil value: collisionBkt .
    collisionBkt at: hashKey put: hashValue keyValDict_coll: self.
    numElements := numElements + 1.
    cbStatus := collisionBkt at: aKey put: aValue keyValDict_coll: self.
    cbStatus > 1 ifTrue:[
      numCollisions := numCollisions + 1
    ]
  ]
].
(numCollisions > collisionLimit) ifTrue: [
  self rebuildTable: self _nextTableSize .
].
^aValue
%

category: 'Private'
method: KeyValueDictionary
atHash: hashIndex putKey: aKey

"Updates the hash table by storing aKey at the specified hashIndex.

 Note: This method overwrites the key value at the given hashIndex.
 Returns aKey."

"hashIndex <= 0 checked by primitive _basicAt:put:"
hashIndex > tableSize ifTrue: [
  hashIndex _error: #rtErrArgOutOfRange args:{ 1 . tableSize }
].
self _basicAt: (hashIndex + hashIndex - 1) put: aKey
%

category: 'Private'
method: KeyValueDictionary
atHash: hashIndex putKey: aKey value: aValue

"Updates the hash table by storing aKey at the specified hashIndex
 and storing aValue at hashIndex+1 .

 Note: This method overwrites the key at the given hashIndex.
 Returns aKey."

"hashIndex <= 0 checked by primitive _basicAt:put:"
| idx |
hashIndex > tableSize ifTrue: [
  hashIndex _error: #rtErrArgOutOfRange args:{ 1 . tableSize }
].
idx := hashIndex + hashIndex  .
self _basicAt: idx - 1 put: aKey ;
     _basicAt:   idx  put: aValue
%

category: 'Private'
method: KeyValueDictionary
atHash: hashIndex putValue: aValue

"Updates the hash table by storing aValue at the specified hashIndex.
 Returns aValue."

"hashIndex <= 0 checked by primitive _basicAt:put:"
hashIndex > tableSize ifTrue:[
  ^ hashIndex _error: #rtErrArgOutOfRange args:{ 1 . tableSize }
].
^self _basicAt: (hashIndex + hashIndex) put: aValue
%

category: 'Storing and Loading'
method: KeyValueDictionary
basicWriteTo: passiveObj

"Converts the receiver to its passive form and writes that information on
 passiveObj."

| s cls ivs c |
  cls := self class.
  passiveObj writeClass: cls.

  ivs := cls _instVarNames .

  passiveObj writeSize: (s := self size) .

  passiveObj writeNamedIvsFrom: self class: cls .
  passiveObj endNamedInstVars.

  c := 0.
  self keysAndValuesDo:[ :aKey :aValue |
    passiveObj writeObject: aKey; writeObject: aValue .
    c := c + 1.
    c > 49 ifTrue: [
      passiveObj lf.
      c := 0.
      ].
    ].

  passiveObj cr
%

category: 'Updating'
method: KeyValueDictionary
become: anObject

self ~~ anObject ifTrue:[
  super become: anObject .
  self _resetParentRef .
  anObject _resetParentRef .
].
%

category: 'Clustering'
method: KeyValueDictionary
clusterDepthFirst

"This method clusters the receiver and its named instance variables and the
 key/value pairs in depth-first order.  Returns true if the receiver has
 already been clustered during the current transaction; returns false
 otherwise."

self cluster ifTrue: [ ^ true ].

1 to: self namedSize do:[ :i |
  (self instVarAt: i) clusterDepthFirst
  ].
1 to: tableSize * 2 by: 2 do: [ :tableIndex | | aKey collisionBkt |
  (aKey := self _at: tableIndex) ifNil:[
    (collisionBkt := self _at: (tableIndex + 1)) ifNotNil: [
      collisionBkt cluster .
      1 to: collisionBkt _basicSize by: 2 do: [ :j |
        (aKey := collisionBkt _at: j) ifNotNil:[
          aKey clusterDepthFirst.
          (collisionBkt _at: j + 1) clusterDepthFirst.
        ].
      ].
    ].
  ] ifNotNil: [
    aKey clusterDepthFirst .
    (self _at: tableIndex + 1) clusterDepthFirst.
  ].
].
^ false
%

category: 'Private'
method: KeyValueDictionary
collisionBucketClass

"Returns the class of object to create when keys collide."

^ CollisionBucket
%

category: 'Accessing'
method: KeyValueDictionary
collisionLimit

"Returns the value of the collisionLimit instance variable."

^collisionLimit
%

category: 'Updating'
method: KeyValueDictionary
collisionLimit: aLimit

"Sets the collision limit to aLimit."

"the value 536870911 is special and may only be installed by instance methods.
 This value is used to prevent recursive rebuild."

aLimit _isSmallInteger ifFalse:[  "gemstone64 explicit constraint check"
  ^ aLimit _error: #rtErrBadArgKind args: { SmallInteger }
  ].
aLimit > 0 ifFalse:[ ^ aLimit _error: #rtErrArgNotPositive ].
aLimit >= 536870911
  ifTrue:[ collisionLimit := 536870910 ]
  ifFalse:[ collisionLimit := aLimit ]
%

category: 'Private'
method: KeyValueDictionary
compareKey: key1 with: key2

"Returns whether key1 is equivalent to key2."

^ key1 = key2
%

category: 'Enumerating'
method: KeyValueDictionary
do: aBlock

"Iteratively evaluates the one argument block aBlock, using the value of
 each Association or key-value pair as the argument of the block."

self keysAndValuesDo: [:aKey :aValue | aBlock value: aValue].
^self
%

category: 'Initializing'
method: KeyValueDictionary
fillFrom: index1 resizeTo: newSize with: anObject

"If newSize > 0,
   change size of receiver to newSize and
   store anObject into instVars 1..newSize
 else
   store anObject into instVars 1..(0 - newSize)
"
<primitive: 607>
index1 _validateClass: SmallInteger .
newSize _validateClass: SmallInteger .
(index1 > newSize abs )
  ifTrue:[ index1 _error: #rtErrBadCopyFromTo args: { newSize }].

(index1 < 1) ifTrue:[ self _errorIndexOutOfRange: index1].
self _primitiveFailed: #fillFrom:resizeTo:with:
     args: { index1 . newSize . anObject }
%

category: 'Repository Conversion'
method: KeyValueDictionary
fixReferencesInFirstPass

"Return whether this object should have references to Float, SmallFloat,
 and LargeInteger fixed in the first pass.
 false means fix them in the 2nd pass.  KeyValueDictionary object can
 be part of a Set or Bag, so we do those later"

^self class ~~ KeyValueDictionary
%

category: 'Hashing'
method: KeyValueDictionary
hashFunction: aKey

"The hash function should perform some operation on the value of the
 key (aKey) which returns a value in the range 1..tableSize."

^(aKey hash \\  tableSize) + 1
%

category: 'Searching'
method: KeyValueDictionary
includesIdenticalAssociation: anAssociation

"Returns true if anAssociation is identical to one of the Associations of
 the receiver.  Returns false otherwise.

 KeyValueDictionaries do not store Associations. So this message is not
 meaningful for this class."

^ self shouldNotImplement: #includesIdenticalAssociation.
%

category: 'Initializing'
method: KeyValueDictionary
initialize: itsSize

"Initializes the instance variables of the receiver to be an empty
 KeyValueDictionary with the specified hash table size."

| newSize |

newSize := itsSize .

(newSize <= 0)
  ifTrue: [
    newSize _error: #rtErrArgNotPositive.
    newSize := 3
    ].

self _basicSize: 0.  "Set size to 0 and restore to nil any entries"
self _basicSize: newSize + newSize .
self _initializeWithoutClear: newSize.
^self
%

category: 'Private'
method: KeyValueDictionary
keyAt: hashOffset

"Private.  Returns the key at the specified hash offset."

^ self _at: ((hashOffset + hashOffset) - 1)
%

category: 'Accessing'
method: KeyValueDictionary
keyAt: aKey otherwise: aValue

"Returns the key that corresponds to aKey.  If no such key/value pair
 exists, returns the given alternate value.

 Note that the keyAt: method is private and does not have behavior that is
 compatible with this method."

| hash hashKey collisionBkt |

aKey ifNil:[ ^ self _error: #rtErrNilKey ] .
hash := self hashFunction: aKey.
(hashKey := self keyAt: hash) ifNil:[
  (collisionBkt := self valueAt: hash) ifNotNil:[
     ^ collisionBkt keyAt: aKey otherwise: aValue
  ]
] ifNotNil: [
  (self compareKey: aKey with: hashKey) ifTrue: [ ^ hashKey ]
].
^ aValue
%

category: 'Private'
method: KeyValueDictionary
keyAtHash: hashIndex

"hashIndex <= 0 checked by primitive _basicAt:"
hashIndex > tableSize ifTrue:[
  hashIndex _error: #rtErrArgOutOfRange args:{ 1 . tableSize }
].
^ self _basicAt: (hashIndex + hashIndex - 1)
%

category: 'Accessing'
method: KeyValueDictionary
keyAtValue: anObject ifAbsent: aBlock

"Returns the key of the first value equal to the given object, anObject.
 If no match is found, evaluates and returns the result of the block aBlock."

| aKey collisionBkt |

1 to: tableSize * 2 by: 2 do: [ :tableIndex |
  (aKey := self _at: tableIndex) ifNil: [
    (collisionBkt := self _at: (tableIndex + 1)) ifNotNil: [
      1 to: collisionBkt _basicSize by: 2 do: [ :j |
        (aKey := collisionBkt _at: j) ifNotNil: [
          anObject = (collisionBkt _at: j + 1) ifTrue: [ ^aKey ].
	].
      ].
    ].
  ] ifNotNil: [
    anObject = (self _at: tableIndex + 1) ifTrue: [ ^aKey ].
  ].
].

^aBlock value.
%

category: 'Accessing'
method: KeyValueDictionary
keys

"Returns an IdentitySet containing the receiver's keys."

| result |
result := IdentitySet new . 
self keysDo:[ :aKey | result add: aKey ].
^ result
%

category: 'Enumerating'
method: KeyValueDictionary
keysAndValuesDo: aBlock

"Evaluates aBlock with each of the receiver's key/value pairs as the
 arguments.  The argument aBlock must be a two-argument block.  The
 first argument is the key and the second argument is the value of
 each key/value pair."

| aKey collisionBkt |

1 to: tableSize * 2 by: 2 do: [ :tableIndex |
  (aKey := self _at: tableIndex) ifNil: [
    (collisionBkt := self _at: (tableIndex + 1)) ifNotNil: [
      1 to: collisionBkt _basicSize by: 2 do: [ :j |
        (aKey := collisionBkt _at: j) ifNotNil: [
          aBlock value: aKey value: (collisionBkt _at: j + 1).
	].
      ].
    ].
  ] ifNotNil: [
    aBlock value: aKey value: (self _at: tableIndex + 1).
  ].
].
%

category: 'Accessing'
method: KeyValueDictionary
keysAsArray

"Returns an Array containing the receiver's keys."

| result |
result := Array new .
self keysDo:[ :aKey | result add: aKey ].
^ result
%

category: 'Enumerating'
method: KeyValueDictionary
keysDo: aBlock

"For each key/value pair in the receiver, evaluates the one-argument block
 aBlock with the key as the argument."

| aKey collisionBkt |

1 to: tableSize * 2 by: 2 do: [ :tableIndex |
  (aKey := self _at: tableIndex) ifNil: [
    (collisionBkt := self _at: (tableIndex + 1)) ifNotNil: [
      1 to: collisionBkt _basicSize by: 2 do: [ :j |
        (aKey := collisionBkt _at: j) ifNotNil: [
          aBlock value: aKey .
	].
      ].
    ].
  ] ifNotNil: [
    aBlock value: aKey
  ].
].
%

category: 'Storing and Loading'
method: KeyValueDictionary
loadNamedIVsFrom: passiveObj

"Reads named instance variables from the given passive object.  The
 first instance variable should already have been parsed and be
 available in the passiveObj argument."

| name offset nameSym aValue |

passiveObj version >= 500 ifTrue:[ ^ super loadNamedIVsFrom: passiveObj ].

[ name := passiveObj ivName.
  name ifNotNil:[
    nameSym := Symbol _existingWithAll: name .
    nameSym ifNotNil:[
      offset := self class _ivOffsetOf: nameSym.
      offset ifNotNil:[
        aValue := passiveObj ivValue .
        offset == 3 "collisionLimit" ifTrue:[
          aValue := (tableSize * 100) min: 500000000 .
        ].
        self instVarAt: offset put: aValue .
      ] ifNil:[
	self dynamicInstVarAt: nameSym put: passiveObj ivValue
      ]
    ] ifNil:[
      self dynamicInstVarAt: name asSymbol put: passiveObj ivValue
    ].
    passiveObj readNamedIV
  ] ifNil: [
    false
  ]
] untilFalse.

passiveObj skipNamedInstVars.
%

category: 'Storing and Loading'
method: KeyValueDictionary
loadVaryingFrom: passiveObj size: varyingSize

"Reads the varying part of the receiver from the given passive object.
 Does not record the receiver as having been read.  Does not read the
 receiver's named instance variables, if any."

passiveObj version >= 500 ifTrue:[
  1 to: varyingSize do: [:i |
    self at: passiveObj readObject"aKey" put: passiveObj readObject"aValue"
    ].
  ]
ifFalse:[ super loadVaryingFrom: passiveObj size: varyingSize ].
%

category: 'Accessing'
method: KeyValueDictionary
numCollisions

"Returns the total number of collisions, the sum of the number of key/value
 pairs in all collision buckets."

^ numCollisions
%

category: 'Accessing'
method: KeyValueDictionary
numElements

"Same as the size method."

^numElements
%

category: 'Copying'
method: KeyValueDictionary
postCopy

"Cleanup of new copy after shallowCopy."

| collisionBkt |
super postCopy.
1 to: tableSize do: [ :index |
  (self keyAt: index) ifNil:[
     (collisionBkt := self valueAt: index) ifNotNil: [
        collisionBkt := collisionBkt copy.
        collisionBkt keyValueDictionary: self.
        self atHash: index putValue: collisionBkt.
     ]
  ]
].
%

category: 'Private'
method: KeyValueDictionary
rebuildIfNeeded

"Private.  If the receiver has too many collisions, rebuild it with a larger
 hash table.  Returns true if the receiver was rebuilt, false otherwise."

numCollisions > collisionLimit ifTrue:[
  [ numCollisions > collisionLimit ] whileTrue: [
    self rebuildTable: self _nextTableSize .
    ].
  ^ true .
  ].
^ false
%

category: 'Hashing'
method: KeyValueDictionary
rebuildTable: newSize

"Rebuilds the hash table by saving the current state, initializing and
 changing the size of the table, and adding the key value pairs saved
 back to the hash dictionary."

| saveTable index saveCollLimit prevCnt |

collisionLimit == 536870911 ifTrue:[
  ^ self  "prevent recursive rebuild"
  ].
true ifTrue:[ prevCnt := self _numBuckets. ].
saveTable := Array new: (self size * 2).
index := 0.
self keysAndValuesDo: [ :aKey :aValue |
  index := index + 1.
  saveTable at: index put: aKey.
  index := index + 1.
  saveTable at: index put: aValue
  ].
self tableSize: newSize.  "installs new collision limit "
saveCollLimit := collisionLimit . "save new collision limit "
collisionLimit := 536870911 . "prevent recursive rebuild"
1 to: index by: 2 do: [ :i |
  self at: (saveTable at: i "key") put: (saveTable at: i + 1 "value")
  ].
collisionLimit := saveCollLimit .
true ifTrue:[ | cnt |
  cnt := 0 .
  self _collisionBucketsDo:[:bkt | cnt := cnt + 1 ]. 
  "GsFile gciLogServer: self asOop asString,
      ' prevBuckets ', prevCnt asString, 
      ' rebuild tableSize ', tableSize asString,
      ' size ', self size asString,' basicSize ', self basicSize asString ,
      ' buckets ', cnt asString ."
  ]. 
^self
%

category: 'Hashing'
method: KeyValueDictionary
rehash
	"Re-establish any hash invariants of the receiver."

	self rebuildTable: self tableSize
%

category: 'Removing'
method: KeyValueDictionary
removeAllKeys

  "Remove all key-value pairs from the receiver."

  self initialize: self tableSize
%

category: 'Removing'
method: KeyValueDictionary
removeKey: aKey ifAbsent: aBlock

"Removes the key/value pair with key aKey from the receiver and returns
 the value.  If no key/value pair is present with key aKey, evaluates
 the zero-argument block aBlock and returns the result of that evaluation."

| hash thisKey collisionBkt aValue lastPair |

aKey ifNil:[ ^ self _error: #rtErrNilKey ].
hash := self hashFunction: aKey.
(thisKey := (self keyAt: hash)) ifNil:[
             "It might be in a collision bucket"
      collisionBkt := self valueAt: hash.
      collisionBkt ifNil:[ ^ self _reportKeyNotFound: aKey with: aBlock ] .
      aValue := collisionBkt removeKey: aKey ifAbsent: [
           ^ self _reportKeyNotFound: aKey with: aBlock "fix 11884"
      ].
      "If we got this far, a key/value pair was removed from collisionBkt"
      numCollisions := numCollisions - 1.
      (collisionBkt size <= 1) ifTrue: [
         "We don't need the collision bucket anymore.
          Put its last key/value pair in our table."
         lastPair := collisionBkt _firstPair.
         self atHash: hash putKey: (lastPair at: 1) value: (lastPair at: 2) .
         collisionBkt _removeAll
      ]
] ifNotNil:[ "There is a key/value pair in this slot"
      (self compareKey: aKey with: thisKey)
         ifFalse: [ ^ self _reportKeyNotFound: aKey with: aBlock ] .
      "The key matches - remove it"
      aValue := self valueAt: hash.
      self atHash: hash putKey: nil value: nil
].

numElements := numElements - 1.
^aValue
%

category: 'Removing'
method: KeyValueDictionary
removeKey: aKey otherwise: notFoundValue

"Removes the key/value pair with key aKey from the receiver and returns
 the value.  If no key/value pair is present with key aKey,
 returns the notFoundValue ."

| hash thisKey collisionBkt aValue lastPair |

aKey ifNil:[ ^ self _error: #rtErrNilKey ].
hash := self hashFunction: aKey.
(thisKey := (self keyAt: hash)) ifNil:[
	  "It might be in a collision bucket"
      collisionBkt := self valueAt: hash.
      collisionBkt ifNil: [ ^ notFoundValue ] .
      aValue := collisionBkt removeKey: aKey ifAbsent:[ ^ notFoundValue ].
      "If we got this far, a key/value pair was removed from collisionBkt"
      numCollisions := numCollisions - 1.
      (collisionBkt size <= 1) ifTrue: [
         "We don't need the collision bucket anymore.
          Put its last key/value pair in our table."
         lastPair := collisionBkt _firstPair.
         self atHash: hash putKey: (lastPair at: 1) value: (lastPair at: 2).
         collisionBkt _removeAll
      ]
] ifNotNil:[ "There is a key/value pair in this slot"
      (self compareKey: aKey with: thisKey) ifFalse: [ ^ notFoundValue ].
      "The key matches - remove it"
      aValue := self valueAt: hash.
      self atHash: hash putKey: nil value: nil .
].
numElements := numElements - 1.
^aValue
%

category: 'Accessing'
method: KeyValueDictionary
size

"Returns the number of key/value pairs in the receiver."

^numElements
%

category: 'Statistics'
method: KeyValueDictionary
statistics

"Returns a Dictionary containing statistics that can be useful in determining
 the performance of a hash dictionary.  The dictionary contains the following
 information:

 TotalCollisionBuckets - The number of collision buckets required to
                         implement the hash dictionary.

 AveragePairsPerBucket - The average number of key/value pairs in each bucket.

 LargestBucket         - The bucket having the most key/value pairs.  This
                         bucket contains the most keys for which the hash
                         function did not provide a good distribution over the
                         range of values in the table."

| hashKey collisionBkt bktCount pairCount maxBkt maxBktSize result|

bktCount := 0.
pairCount :=0.
maxBktSize := 0.
1 to: tableSize do: [ :tableIndex |
  hashKey := self keyAt: tableIndex.
  collisionBkt := self valueAt: tableIndex.
  (nil == hashKey and:[ nil ~~ collisionBkt]) ifTrue:[
    bktCount := bktCount + 1.
    pairCount := pairCount + collisionBkt size.
    (collisionBkt size > maxBktSize) ifTrue: [
      maxBktSize := collisionBkt size.
      maxBkt := collisionBkt
      ]
    ]
  ].
result := SymbolDictionary new.
result at: #TotalCollisionBuckets put: bktCount.
(bktCount ~~ 0)
  ifTrue: [result at: #AveragePairsPerBucket put: pairCount // bktCount]
  ifFalse: [result at: #AveragePairsPerBucket put: 0].
result at: #LargestBucket put: maxBkt.
^result
%

category: 'Accessing'
method: KeyValueDictionary
tableSize

"Returns the size of the primary hash table."

^tableSize
%

category: 'Initializing'
method: KeyValueDictionary
tableSize: newSize

"Sets the table size to a new value."

"When used during rehashing a dictionary the sender must have already
 copied all key/value pairs to a temporary object to save the contents
 of the dictionary."

(newSize <= 0)
  ifTrue: [newSize _error: #rtErrArgNotPositive  .  ^ self ].

"gs64 v3.0 , removed iteration to send _removeAll to all buckets"

self fillFrom: 1 resizeTo:  newSize + newSize  with: nil .
numElements := 0.
numCollisions := 0.
  "some subclasses override this limit in reimplementation of rebuildTable:"
collisionLimit := newSize <= 1000 ifTrue:[ newSize ] ifFalse:[ newSize * 5 ].
tableSize := newSize .
%

category: 'Private'
method: KeyValueDictionary
valueAt: hash

"Returns the value at the specified hash offset."

^self _basicAt: (hash + hash)
%

category: 'Private'
method: KeyValueDictionary
valueAtHash: hashIndex

"hashIndex <= 0 checked by primitive _basicAt:"
hashIndex > tableSize ifTrue:[
  hashIndex _error: #rtErrArgOutOfRange args:{ 1 . tableSize }
].
^self _basicAt: (hashIndex + hashIndex)
%

category: 'Enumerating'
method: KeyValueDictionary
valuesDo: aBlock

"For each key/value pair in the receiver, evaluates the one-argument block
 aBlock with the value as the argument."

self keysAndValuesDo: [:aKey :aValue |
   aBlock value: aValue
].
%

category: 'Private'
method: KeyValueDictionary
_checkNumCollisions

"Provided for use in testing only. Checks the consistency of
 the collision count in the dictionary against the contents of its buckets.
 Returns self of the dictionary is ok, otherwise generates an error."

| tableLimit mySize totalCollis totalElem |
tableLimit := tableSize * 2 .
totalCollis := 0 .
totalElem := 0 .
1 to: tableLimit by: 2 do: [ :tableIndex | | aKey |
  aKey := self _at: tableIndex .
  aKey == nil ifTrue: [ | collisionBkt |
    collisionBkt := self _at: (tableIndex + 1) .
    collisionBkt ~~ nil ifTrue: [ | cbSiz |
      (cbSiz := collisionBkt numElements) > 0 ifTrue:[
        totalCollis := totalCollis + (cbSiz - 1).
        totalElem := totalElem + cbSiz.
      ] ifFalse: [
        cbSiz < 0 ifTrue:[ nil error:'underflow' ].
      ]
    ].
  ] ifFalse:[
    totalElem := totalElem + 1 .
  ].
].
totalCollis = numCollisions ifFalse:[ nil error:'bad numCollisions'].
mySize := self size .
totalElem = mySize ifFalse:[ nil error:'bad size'].
^ self
%

category: 'Private'
method: KeyValueDictionary
_collisionBucketsDo: aBlock

"Private.  Executes the given one-argument block for each collision bucket
 in the receiver."

| collisionBkt |

1 to: tableSize * 2 by: 2 do: [ :tableIndex |
  (self _at: tableIndex) ifNil:[
     (collisionBkt := self _at: (tableIndex + 1)) ifNotNil:[ aBlock value: collisionBkt ]
  ]
]
%

category: 'Private'
method: KeyValueDictionary
_deferredGciUpdateWith: valueArray

"Private."

1 to: valueArray size by: 2 do:[:j |
  self at: (valueArray at: j) put: (valueArray at: j + 1)
  ].
%

category: 'Initializing'
method: KeyValueDictionary
_gciInitialize

"Private."

tableSize == nil
  ifTrue: [ self initialize: 5 ]
  ifFalse: [ self initialize: tableSize ].
^ true
%

category: 'Searching'
method: KeyValueDictionary
_idxOccurrencesOf: aValue

"Dictionaries can only contain Associations."

^ 0
%

category: 'Private'
method: KeyValueDictionary
_initializeWithoutClear: newSize

"Private. Initializes the instance variables of the receiver to be an empty
 KeyValueDictionary of the specified size. Does not clear the contents
 of the receiver - assumes they are all nil."

tableSize := newSize.
numElements := 0.
numCollisions := 0.
"OC_KEY_VALUE_DICT_COLL_LIMIT_RATIO is 1"
newSize <= 100 ifTrue:[ collisionLimit := newSize ] ifFalse:[
newSize < 1000 ifTrue:[ collisionLimit := newSize * 3 ] ifFalse:[
  collisionLimit := newSize * 5 
]]. 

^self
%

category: 'Testing'
method: KeyValueDictionary
_isLarge

"Returns true if the object is implemented as a tree of private smaller objects"

^ numCollisions ~~ 0
%

category: 'Locking Support'
method: KeyValueDictionary
_lockableValues

"Returns an Array of the receiver's keys and values."

| result |

result := { }  .
self keysAndValuesDo: [:aKey :aValue |
  result add: aKey .
  result add: aValue .
] .
^ result
%

category: 'Private'
method: KeyValueDictionary
_markDirty

  "used to make the root object of a dictionary written, when
   replacing a value for an existing key in a collision bucket."

  numElements := numElements
%

category: 'Private'
method: KeyValueDictionary
_nextTableSize
  | limit sz namedSize |
  namedSize := self namedSize .
  limit := (2034 - namedSize) // 2 .
  "GsFile gciLogServer:' _nextTableSize start tableSize ', tableSize asString,
             ' limit ', limit asString . "
  sz := tableSize * 4 .
  sz <= limit ifTrue:[ 
    ^ Integer _selectedPrimeGreaterThan: sz - 1 limit: limit .
  ].
  tableSize < (limit - 250) ifTrue:[ 
    "avoid growing to a large object this time"
    ^ Integer _selectedPrimeGreaterThan: limit -1 limit: limit . 
  ].
  ^ Integer _selectedPrimeGreaterThan: tableSize * 2 .
%

category: 'Private'
method: KeyValueDictionary
_nodesObjectSecurityPolicy: anObjectSecurityPolicy

"Assigns collision buckets to the given ObjectSecurityPolicy.
 Returns the receiver."

self _collisionBucketsDo:[ :aBucket |
  aBucket objectSecurityPolicy: anObjectSecurityPolicy
].
%

category: 'Private'
method: KeyValueDictionary
_numBuckets
 | cnt |
  cnt := 0 .
  self _collisionBucketsDo:[:bkt | cnt := cnt + 1 ].
  ^ cnt 
%

category: 'Private'
method: KeyValueDictionary
_rebuild

"Rebuild to the next higher size because the internal data structures like
 the collision buckets would like it to happen."

  self rebuildIfNeeded ifFalse:[
    self rebuildTable: tableSize . " rehash using same table size"
  ].
  ^ self.
%

category: 'Private'
method: KeyValueDictionary
_rebuildAt: aKey put: aValue
  "Sent by IdentityCollisionBucket >> at:put:keyValDict_coll:"
  self rebuildTable: self _nextTableSize .
  self at: aKey put: aValue
%

category: 'Private'
method: KeyValueDictionary
_resetParentRef

"Private. After a become:, the parent refs of the collisionBuckets must
 be reset to point to the correct parent."

self _collisionBucketsDo: [:collisionBkt |
  collisionBkt keyValueDictionary: self].
%

! Class extensions for 'LanguageDictionary'

!		Instance methods for 'LanguageDictionary'

removeallmethods LanguageDictionary
removeallclassmethods LanguageDictionary

category: 'Accessing'
method: LanguageDictionary
value

"Returns the value associated with the user's native language Symbol.  That
 Symbol is taken from the user's UserProfile object.  If there is no entry for
 the given language, an error is generated."

|myLanguage "the Symbol representing the user's native language"|

myLanguage := System myUserProfile nativeLanguage.
^ self at: myLanguage
       ifAbsent: [^ self _errorKeyNotFound: myLanguage]
%

! Class extensions for 'LockError'

!		Instance methods for 'LockError'

removeallmethods LockError
removeallclassmethods LockError

category: 'Instance initialization'
method: LockError
args: anArray
  anArray _isArray ifTrue:[ | sz |
    object := anArray atOrNil: 1 .
    (sz := anArray size) > 1 ifTrue:[
      gsArgs := anArray copyFrom:2 to: sz
    ].
  ]
%

category: 'Formatting'
method: LockError
buildMessageText
^ self buildMessageText:(
  [
    gsDetails ifNotNil:[:s| s asString ]
  ] onException: Error do:[:ex |
    ex return: nil
  ]
)
%

category: 'Instance initialization'
method: LockError
initialize
  gsNumber := ERR_LockError.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Accessing'
method: LockError
object
  ^ object
%

category: 'Instance initialization'
method: LockError
object: anObj

  object := anObj
%

category: 'Instance initialization'
method: LockError
_legacyHandlerArgs
  | arr |
  (arr := { object } ) addAll: super _legacyHandlerArgs .
  ^ arr
%

! Class extensions for 'LookupError'

!		Instance methods for 'LookupError'

removeallmethods LookupError
removeallclassmethods LookupError

category: 'Instance initialization'
method: LookupError
args: anArray
  anArray _isArray ifTrue:[ | sz |
    object := anArray atOrNil: 1 .
    key := anArray atOrNil: 2 .
    (sz := anArray size) > 2 ifTrue:[
      gsArgs := anArray copyFrom: 3 to: sz
    ].
  ]
%

category: 'Formatting'
method: LookupError
buildMessageText
^ self buildMessageText:(
  [ | str |
    gsDetails ifNotNil:[ :gsd |
      str := String new .
      object ifNotNil:[:o | str add: ' accessing a '; add: o class name ].
      key ifNotNil:[:k|
        (str := String withAll:', ') add: k asString
      ].
      str add: ', ' ; add: gsd  asString .
    ].
    str
  ] onException: Error do:[:ex |
    ex return: nil
  ]
)
%

category: 'Instance initialization'
method: LookupError
initialize
  gsNumber := ERR_LookupError.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Instance initialization'
method: LookupError
key: anObject
  key := anObject
%

category: 'Instance initialization'
method: LookupError
_legacyHandlerArgs
  | arr |
  arr := { object . key } .
  gsArgs ifNotNil:[:a |  arr addAll: a ].
  ^ arr
%

! Class extensions for 'Magnitude'

!		Class methods for 'Magnitude'

removeallmethods Magnitude
removeallclassmethods Magnitude

category: 'Instance Creation'
classmethod: Magnitude
fromStream: aStream

"(Subclass responsibility.)  Returns an instance of the receiver."

Magnitude subclassResponsibility: #fromStream:
%

category: 'Instance Creation'
classmethod: Magnitude
fromString: aString

"Returns an instance of the appropriate subclass, reconstructed from aString.
 The String must contain only Characters representing the object to be created,
 although leading and trailing blanks are permitted."

| s result |

s := ReadStream on: aString.
result := self fromStream: s.
[ s atEnd ]
whileFalse:
  [ (s next == $  )
    ifFalse:
      [ self _errIncorrectFormat: aString ]
  ].
^ result
%

category: 'Indexing Support'
classmethod: Magnitude
_idxCanOptimizeComparison
  "Answer true if it is possible to optimize index-related comparisons by replacing
   _idxForCompare* calles with a primitive comparison call instead.

     _idxForCompareEqualTo:              #=
     _idxForCompareGreaterThan:          #>
     _idxForCompareGreaterThanOrEqualTo: #>=
     _idxForCompareLessThan:             #<
     _idxForCompareLessThanOrEqualTo:    #<=
     _idxForCompareNotEqualTo:           #~=
   "

  "In general all subclasses of Magnitude can optimize comparisons."

  ^ true
%

!		Instance methods for 'Magnitude'

category: 'Comparing'
method: Magnitude
< aMagnitude

"(Subclass responsibility.)  Returns true if the receiver is less than
 aMagnitude; returns false otherwise."

Magnitude subclassResponsibility: #<
%

category: 'Comparing'
method: Magnitude
<= aMagnitude

"Returns true if the receiver is less than or equal to aMagnitude;
 returns false otherwise."

^(self > aMagnitude) == false
%

category: 'Comparing'
method: Magnitude
= aMagnitude

"(Subclass responsibility.)  Returns true if the receiver is equal to
 aMagnitude; returns false otherwise."

Magnitude subclassResponsibility: #=
%

category: 'Comparing'
method: Magnitude
> aMagnitude

"Returns true if the receiver is greater than aMagnitude;
 returns false otherwise."

^ aMagnitude < self
%

category: 'Comparing'
method: Magnitude
>= aMagnitude

"Returns true if the receiver is greater than or equal to aMagnitude;
 returns false otherwise."

^(self < aMagnitude) == false
%

category: 'Comparing'
method: Magnitude
between: min and: max

"Returns true if the receiver is less than or equal to the argument max and
 greater than or equal to the argument min.  Returns false otherwise."

^ (self >= min) and:[ self <= max ]
%

category: 'Comparing'
method: Magnitude
hash

"(Subclass responsibility.)  Returns a numeric hash index."

"Two Numbers for which = is true, must have the same hash value."

Magnitude subclassResponsibility: #hash
%

category: 'Comparing'
method: Magnitude
max: aMagnitude

"If the receiver is greater than the argument, return the receiver.
 Otherwise return the argument."

self > aMagnitude
        ifTrue: [^self].
^aMagnitude
%

category: 'Comparing'
method: Magnitude
min: aMagnitude

"If the receiver is less than the argument, return the receiver.
Otherwise return the argument."

self < aMagnitude
        ifTrue: [^self].
^aMagnitude
%

category: 'Private'
method: Magnitude
_deepCopyWith: copiedObjDict

"Private. Used internally to implement deepCopy."

| copy |

self isSpecial ifTrue: [ ^ self ].

copy := copiedObjDict at: self otherwise: nil.
copy ~~ nil ifTrue: [ ^ copy ].

copy := self copy.
copiedObjDict at: self put: copy.

^ copy.
%

category: 'Indexing Support'
method: Magnitude
_idxOptimizedCompareWithClass: aClass
  "Returns true if the receiver may be inserted into a BtreePlusNode whose
   #lastElementClass is <aClass> and whose comparisons are optimized."

  "Sent when index option optimizeComparison is true and aClass responds trut to #_idxCanOptimizeComparison"

  ^ true
%

! Class extensions for 'MessageNotUnderstood'

!		Instance methods for 'MessageNotUnderstood'

removeallmethods MessageNotUnderstood
removeallclassmethods MessageNotUnderstood

category: 'Accessing - failedMessage'
method: MessageNotUnderstood
arguments

  ^ gsArgs ifNil:[ #() ].
%

category: 'Accessing - failedMessage'
method: MessageNotUnderstood
arguments: anArray

  gsArgs := anArray
%

category: 'Instance initialization'
method: MessageNotUnderstood
buildMessageText
^ self buildMessageText:(
  [
   | str |
    (str := String withAll:'a ') add: receiver class name ;
       add: ' does not understand  ' ; add: selector printString .
    envId ~~ 0 ifTrue:[ str add:' (env ' ; add: envId asString; add: $) ].
    str
  ] onException: Error do:[:ex |
    ex return: nil
  ]
 )
%

category: 'Accessing - failedMessage'
method: MessageNotUnderstood
defaultAction
  | result |
  (result := super defaultAction) == self ifFalse:[ ^ result ].

  ^ receiver perform: selector env: envId withArguments: gsArgs
%

category: 'Accessing - failedMessage'
method: MessageNotUnderstood
environmentId
  ^ envId
%

category: 'Accessing - compatibility'
method: MessageNotUnderstood
gsArguments
  | arr |
  arr :=  { receiver . selector }  .
  gsArgs ifNotNil:[:a | arr add: a ] ifNil:[ arr add: #() ].
  ^ arr
%

category: 'Instance initialization'
method: MessageNotUnderstood
initialize
  gsNumber := ERR_MessageNotUnderstood.
  gsResumable := true .
  gsTrappable := true .
  envId := 0 .
%

category: 'Accessing'
method: MessageNotUnderstood
message: aFailedMessage

   gsArgs := aFailedMessage arguments .
   selector :=  aFailedMessage selector .
%

category: 'Accessing'
method: MessageNotUnderstood
receiver
  "Answer the object that was the receiver of the message that failed."

  ^ receiver
%

category: 'Accessing'
method: MessageNotUnderstood
receiver: anObject
  "Set the object which did not understand the message to be anObject"

  receiver := anObject
%

category: 'Instance initialization'
method: MessageNotUnderstood
receiver: anObject selector: aSymbol args: anArray envId: envIdArg

  "inline super initialize"
  gsNumber := ERR_MessageNotUnderstood.
  gsResumable := true .
  gsTrappable := true .

  selector := aSymbol .
  envId := envIdArg ifNil:[ 0 ].
  receiver := anObject .
  gsArgs:= anArray .
%

! Class extensions for 'Metaclass3'

!		Instance methods for 'Metaclass3'

removeallmethods Metaclass3
removeallclassmethods Metaclass3

category: 'Class Instance Variables'
method: Metaclass3
addInstVarNames: instVarNamesArray

"Adds the instance variables specified in the argument to the receiver and any
 of its subclasses.  Generates an error upon encountering a name that is not a
 valid instance variable name or that is already an instance variable of the
 receiver.

 Instance variables that are added to a metaclass are called
 Class Instance Variables."

| ivnames oldEnd tCls |

self isMeta ifFalse:[ ^ self error:'receiver is not a metaClass' ].
tCls  := self thisClass .
(tCls isInvariant not or: [tCls subclassesDisallowed]) ifFalse: [
  ^ self _error: #rtErrClassNotModifiable
  ].
instVarNamesArray size == 0 ifTrue: [ ^self ].
"allow an error handler to proceed from the errors signaled by the validation
 method and thus skip offending instance variable names"
ivnames := { } .
instVarNamesArray do:[ :aStr | ivnames add: aStr asSymbol ].
ivnames := ivnames select: [:e |
  self _validateNewClassInstVar: e
  ].
oldEnd := self instSize + 1.
self _inheritCIVs: ivnames at: oldEnd.
%

category: 'Accessing'
method: Metaclass3
classHistory
  self isMeta ifTrue:[ ^ destClass classHistory ].
  ^ super classHistory
%

category: 'Updating the Method Dictionary'
method: Metaclass3
compileAccessingMethodsFor: anArrayOfSymbols

"Reimplemented to treat class instance variables specially,
 if isMeta is true for receiver .
 The new methods have environmentId == 0 .  "

| allVarNames varName symlst |
self isMeta ifFalse:[
  ^ super compileAccessingMethodsFor: anArrayOfSymbols
].
symlst := SymbolList new.
varName := 'newValue'.
allVarNames := self allInstVarNames collect:[:s | s asString ].
[allVarNames includesValue: varName] whileTrue: [
  varName :=  'z' , varName .
].
anArrayOfSymbols do: [ :var | | methodtext lf tab |
  lf := Character lf .
  tab := Character tab .
  (methodtext := String new) add: var; add: lf; add: tab; add: '^' ; add: var ; add: lf .
  [ self compileMethod: methodtext dictionaries: symlst
       category: #Accessing environmentId: 0
  ] onException: CompileError do:[:ex |
     self _error: #classErrNotAVar args: { var }
  ].
  (allVarNames indexOf: var) > Metaclass3 instSize ifTrue:[
    "compile a method that lets the variable be modified if the user has
     the proper authority"
    (methodtext := String new)
      add: var; add: ': '; add: varName; add: lf;
      add: tab; add: 'self atClassInstVar: #'''; add: var;
      add: ''' put: '; add: varName; add: lf .
    [ self compileMethod: methodtext dictionaries: symlst
           category: #Updating environmentId: 0
    ] onException: CompileError do:[:ex |
      self _error: #classErrNotAVar args: { var }
    ].
  ] ifFalse: [
    (methodtext := String new ) add: var; add: ': ' ; add: varName ; add: lf ;
      add: tab; add: var ; add: ' := ' ; add: varName ; add: lf .
    [ self compileMethod: methodtext dictionaries: symlst
             category: #Updating environmentId: 0
    ] onException: CompileError do:[:ex |
        self _error: #classErrNotAVar args: { var }
    ].
  ].
].
%

category: 'Browser Methods'
method: Metaclass3
definition
	"We are created as a side-effect of creating the class!"

	^self thisClass definition.
%

category: 'Accessing'
method: Metaclass3
extraDict
  self isMeta ifTrue:[ ^ destClass extraDict ].
  ^ nil
%

category: 'Accessing'
method: Metaclass3
extraDictForStore
  self isMeta ifTrue:[ ^ destClass extraDictForStore ].
  ^ nil
%

category: 'Accessing'
method: Metaclass3
instanceString
  self isMeta ifTrue:[ ^ destClass name ] ifFalse:[ ^ name ]
%

category: 'Accessing'
method: Metaclass3
instanceSymbol
  self isMeta ifTrue:[ ^ destClass name ] ifFalse:[ ^ name ]
%

category: 'Accessing'
method: Metaclass3
isMeta

"Returns true if the receiver is a meta class "

 ^ format ifNil:[ false "cope with malformed classes" ]
      ifNotNil:[:fmt| (fmt bitAnd: GC_Metaclass3_metaClass) ~~ 0]
%

category: 'Accessing'
method: Metaclass3
methodStampDictName
  self isMeta ifTrue:[ ^ #GSMetaMethodStampDict ].
  ^ super methodStampDictName.
%

category: 'Accessing'
method: Metaclass3
name
 self isMeta ifTrue:[ ^ destClass name , ' class' ]
           ifFalse:[ ^ super name ]
%

category: 'Instance Creation'
method: Metaclass3
new
  "Disallowed.  To create a new class or metaclass, use
   Class | subclass:instVarNames:... instead."
  self shouldNotImplement: #new
%

category: 'Instance Creation'
method: Metaclass3
new: aSize
  "Disallowed.  To create a new class or metaclass, use
   Class | subclass:instVarNames:... instead."
  self shouldNotImplement: #new:
%

category: 'Accessing'
method: Metaclass3
pragmaDictName

  self isMeta ifTrue:[ ^ #GSMetaMethodPragmaDict ].
  ^ super pragmaDictName
%

category: 'Accessing'
method: Metaclass3
theNonMetaClass
  "used by Monticello"

  "inline  self thisClass"
  (format bitAnd: GC_Metaclass3_metaClass) ~~ 0 ifTrue:["a meta class"  ^ destClass ] .
  ^ self
%

category: 'Accessing'
method: Metaclass3
thisClass

"If receiver is a meta class, returns the class, else returns self."

(format bitAnd: GC_Metaclass3_metaClass) ~~ 0 ifTrue:["a meta class"  ^ destClass ] .
^ self
%

category: 'Accessing'
method: Metaclass3
versionedName
 self isMeta ifTrue:[ ^ destClass versionedName , ' class' ]
           ifFalse:[ ^ super name ]
%

category: 'Accessing'
method: Metaclass3
_classCategory
  ^ self thisClass _classCategory
%

category: 'Accessing'
method: Metaclass3
_gbsTraversalCallback

"Private.  When GemBuilder Smalltalk traverses a metaclass, this method
 is called to return a description of the metaclass."

 self isMeta ifTrue:[ ^ self printString ].
 ^ super _gbsTraversalCallback  "probably MNU"
%

category: 'Class Instance Variables'
method: Metaclass3
_inheritCIVs: ivnames at: inheritedIVIndex
	"Adds the instance variables specified in the argument to the receiver at the
 given location.

 Instance variables that are added to a metaclass are called
 Class Instance Variables."

	| names vname subClss tCls numNewIvs |
	self isMeta ifFalse: [^self error: 'receiver is not a metaClass'].
	(numNewIvs := ivnames size) == 0 ifTrue: [^self].
	numNewIvs + self instSize > GEN_MAX_INSTANCE_VARS
		ifTrue: [self error: 'new instVars would exceed max number of instVars'].
	names := Array withAll: instVarNames.
	tCls := self thisClass.
	1 to: numNewIvs
		do:
			[:i |
			vname := (ivnames at: i) asSymbol.
			names insertObject: vname at: inheritedIVIndex + i - 1.
			self _incrementInstVars: 1.
			self _unsafeAt: 4 put: (InvariantArray withAll: names).
			"force the class to increase in size by storing one position off the end"
			self _refreshClassCache: false.
			tCls _insertCivAt: inheritedIVIndex + i - 1].
	tCls _refreshClassCache: false.
	tCls subclassesDisallowed
		ifFalse:
			[subClss := self _subclasses.
			subClss ~~ nil
				ifTrue: [subClss do: [:e | e _inheritCIVs: ivnames at: inheritedIVIndex]]].
	"finally, recompile my methods to ensure that the instance variable indexes are
 correct"
	self _recompileMethodsAfterNewIvOffset: inheritedIVIndex
%

category: 'Accessing'
method: Metaclass3
_name
 | cls |
 self isMeta ifFalse:[ ^ super _name ].
 (cls := destClass) ifNotNil:[ ^ cls _name , ' class' ].
 (cls := primaryCopy) ifNotNil:[ ^ 'virtual_', cls _name "This path only used for Ruby" ].
 ^ 'unknownMetaclass3'
%

category: 'Private'
method: Metaclass3
_primSubclass: nameSym
instVarNames: arrayOfIvNames
format: anInteger
classVars: aSymbolDictionary
poolDictionaries: anArrayOfPoolDicts
classInstanceVars: arrayOfCivNames

	"subclass creation primitive.
 if aSymbolDictionary is empty or nil , the resulting class will have
  classVars == nil .
 if anArrayOfPoolDicts is empty or nil , the resulting class will have
   poolDictionaries == nil

 Gemstone64, this primitive enforces the restriction that
 a class cannot be more than 450 classes below Object when
 following the superClass chain upwards ."
	<primitive: 1042>

	nameSym _validateClass: Symbol.
	anInteger _validateClass: SmallInteger.
	arrayOfIvNames _validateClass: Array.
	arrayOfCivNames _validateClass: Array.
	aSymbolDictionary
		ifNotNil: [aSymbolDictionary _validateClass: SymbolDictionary].
	anArrayOfPoolDicts ifNotNil: [anArrayOfPoolDicts _validateClass: Array].
	^self
		_primitiveFailed: #_primSubclass:instVarNames:format:classVars:poolDictionaries:
		args:
			{nameSym.
			arrayOfIvNames.
			anInteger.
			aSymbolDictionary.
			anArrayOfPoolDicts.
			arrayOfCivNames}
%

category: 'Modifying Classes'
method: Metaclass3
_setClassVars: aDict old: previousDict

  classVars ~~ aDict ifTrue:[
    previousDict ~~ classVars ifTrue:[ self error:'invalid store to classVars'].
    classVars := aDict .
    self thisClass _setClassVars: aDict old: previousDict
  ]
%

category: 'Private'
method: Metaclass3
_subclass: nameSym
instVarNames: arrayOfIvNames
format: anInteger
classVars: aSymbolDictionary
poolDictionaries: anArrayOfPoolDicts
classInstanceVars: arrayOfCivNames

	self _validateNewClassName: nameSym.
	^self
		_primSubclass: nameSym
		instVarNames: arrayOfIvNames
		format: anInteger
		classVars: aSymbolDictionary
		poolDictionaries: anArrayOfPoolDicts
		classInstanceVars: arrayOfCivNames
%

category: 'Accessing'
method: Metaclass3
_subclasses

"Returns the class variable that is the list of subclasses, or
 nil if the receiver does not keep track of subclasses."

| subclss result |

self isMeta ifTrue:[
  subclss := self thisClass _subclasses .
  subclss == nil ifTrue:[ ^ nil ].
  result := IdentitySet new .
  subclss do:[ :aClass | result add: aClass class ].
  ^ result
].
^ super _subclasses
%

category: 'Class Instance Variables'
method: Metaclass3
_validateNewClassInstVar: ivname

"Returns true if the name passes all tests.  Generates errors if the name fails
 due to a) being an invalid identifier or b) being a duplicate of an existing
 instance variable, either in this metaclass or in one of this metaclass's
 instance's subclasses metaclasses."

| subs |
((ivname size == 0
  or: [ivname isValidIdentifier not])
  or:[ ivname _isSymbol not]) ifTrue: [
  self _error: #classErrBadIdentifier args: { ivname }.
  ^false
  ].
(instVarNames includesIdentical: ivname) ifTrue: [
  self _error: #rtErrAddDupInstvar args: { ivname }.
  ^false
  ].
self thisClass isModifiable ifTrue: [
  subs := self _subclasses .
  subs ~~ nil ifTrue: [
    subs do: [:sub |
      (sub _validateNewClassInstVar: ivname) ifFalse: [ ^false ]
      ].
    ].
  ].
^true
%

category: 'Modifying Classes'
method: Metaclass3
_validateNewClassName: aName
  "Return true if aName is a legal class name, otherwise signal an error.
   If execution continued from the error, return false."

(aName _isSymbol) ifTrue:[
  (aName _validIdentifier: 1024"GCI_MAX_SYMBOL_SIZE") ifTrue:[ ^ true ].
  ( Exception _new: 2149 args: { aName } ) signal .
  ^ false
] ifFalse:[
  "primitive will generate not-a-symbol error"
  ^ false
].
%

! Class extensions for 'MigrationError'

!		Class methods for 'MigrationError'

removeallmethods MigrationError
removeallclassmethods MigrationError

category: 'Accessing'
classmethod: MigrationError
errorNumber
  ^ ERR_MigrationError
%

!		Instance methods for 'MigrationError'

category: 'Instance initialization'
method: MigrationError
args: anArray
  anArray _isArray ifTrue:[ | sz |
    object := anArray atOrNil: 1 .
    (sz := anArray size) > 1 ifTrue:[
      gsArgs := anArray copyFrom:2 to: sz
    ].
  ]
%

category: 'Formatting'
method: MigrationError
buildMessageText
^ self buildMessageText:(
  [ | str |
    (str := String withAll:'object=a ') add: object class versionedName .
    gsDetails ifNotNil:[:d| str add:', '; add: d  asString ].
    str
  ] onException: Error do:[:ex |
    ex return: nil
  ]
)
%

category: 'Instance initialization'
method: MigrationError
initialize
  gsNumber := ERR_MigrationError .
  gsResumable := false .
  gsTrappable := true .
%

category: 'Accessing'
method: MigrationError
object
  ^ object
%

category: 'Instance initialization'
method: MigrationError
object: anObject
  object := anObject
%

! Class extensions for 'Module'

!		Class methods for 'Module'

removeallmethods Module
removeallclassmethods Module

category: 'Smalltalk Printing'
classmethod: Module
sprintf: formatString with: argsArray

"printf per Ruby Pickaxe book page 529..532  ."
<primitive: 767>
formatString _validateClass: String .
argsArray _validateClass: Array  .
self _primitiveFailed: #sprintf:with: args: { formatString . argsArray }
%

!		Instance methods for 'Module'

category: 'Instance Migration'
method: Module
allInstancesInMemory

^ ((SystemRepository listInstancesInMemory: {self}) at: 1)
%

category: 'Browser Methods'
method: Module
changeNameTo: aString
  "Sets the receiver's 'name' instance variable."

  self _validatePrivilege ifTrue:[
    aString size == 0 ifTrue: [ ^ self error: 'class names must have size > 0'].
    name := Symbol withAll: aString.
  ]
%

category: 'Accessing'
method: Module
classHistory
  "Modules have a classHistory instVar, but it is not used"
  ^ nil
%

category: 'Clustering'
method: Module
clusterBehavior

 super clusterBehavior ifTrue:[ ^ true ].
 classHistory ifNotNil:[ classHistory cluster ].
 ^ false
%

category: 'Browsing'
method: Module
definition

  InternalError new
    details:'Unexpected instance of Module in a Smalltalk repository' ;
    signalNotTrappable  .

  ^ 'Module newModule name: ', name printString , '
       "does not include code to install in a dictionary" '
%

category: 'Browser Methods'
method: Module
hierarchy

"Returns a String that enumerates the receiver's superclasses (up to Object)
 and the instance variables defined by the receiver and each of its
 superclasses."

^ self hierarchy: 0
%

category: 'Browser Methods'
method: Module
hierarchy: envId

"Returns a String that enumerates the receiver's superclasses (up to Object)
 and the instance variables defined by the receiver and each of its
 superclasses."

^ self _superClassHierarchy: envId includeOops: false includeVirtual: false
%

category: 'Instance Migration'
method: Module
instVarMappingTo: anotherClass

"See also class InstVarMappingArray for new functionality specific to
 dynamic instVars.
 Returns an instance-variable mapping from the receiver's named instance
 variables to those in the given class.  If an entry is 0, the other
 class does not have the corresponding instance variable.

 For migrating instances of oldClass to instances of newClass,
 create a mapping via
     newClass instVarMappingTo: oldClass
 "

| otherivn result numIvs |
otherivn := anotherClass allInstVarNames .
numIvs := self instSize .
result := Array new: numIvs .
1 to: numIvs do: [:i |
  result at: i put: (otherivn indexOfIdentical: (instVarNames at: i))
].
^result
%

category: 'Queries'
method: Module
isMeta

"Returns whether the receiver is a kind of metaclass."

^ false
%

category: 'Locking'
method: Module
lockableParts

"Returns an Array of the receiver's contents that are locked by browsers
 and folders."

| mcls parts |
mcls := self class .
parts := { self .
  instVarNames .
  classVars .  "if not nil will be shared with the metaClass"
  categorys .
  methDicts .

  mcls .
  mcls.instVarNames .
  mcls.categorys .
  mcls.methDicts .
  mcls.poolDictionaries }.
parts := parts reject:[:ea | ea == nil or:[ ea == #()] ]. 
^parts
%

category: 'Accessing'
method: Module
name
  ^ name
%

category: 'Accessing'
method: Module
name: aName
  self isInvariant ifTrue:[
     ArgumentTypeError signal:'illegal attempt to change name of a Module'.
  ].
  name := aName asSymbol
%

category: 'Authorization'
method: Module
objectSecurityPolicy: anObjectSecurityPolicy
	"Assigns the receiver and its non-shared components to the given security policy.
 The security policies of class variable values are not changed.  The current user
 must have write access to both the old and new security policies for this method
 to succeed."
	"enter protected mode"
	<primitive: 2001>
	| prot |
	prot := System _protectedMode.
	[| anArray aCls categBlk parts |
		parts := self lockableParts asIdentitySet.
		parts remove: self; remove: self class.
		self _objectSecurityPolicy: anObjectSecurityPolicy.
		self class _objectSecurityPolicy: anObjectSecurityPolicy.
		parts accompaniedBy: anObjectSecurityPolicy do: [:policy :each |
			  "categorys, methDicts instVars for self and self class
         handled here.  For Class, lockableParts includes extraDict. "
			each objectSecurityPolicy: policy
    ].
	  anArray := {}.
	  aCls := self.
	  categBlk := [:arr |
			arr ifNotNil:[
        anArray add: arr.
				arr _isArray ifTrue:[
          anArray addAll: arr.
					arr do: [:dict | dict ifNotNil:[ anArray addAll: dict values]]]
				ifFalse: [anArray addAll: arr values]]].
		2 timesRepeat:[
 			aCls persistentMethodDictsDo:[:aDict | anArray add: aDict; addAll: aDict values].
			aCls persistentNameSpacesDo: [:aRns | anArray add: aRns].
			categBlk value: aCls __categorys.  "categs array may contain nils"
			aCls _poolDictionaries do: [:d | anArray add: d].
			aCls := self class
		].
		anArray add: classHistory.
		anArray asIdentitySet accompaniedBy: anObjectSecurityPolicy do: [:policy :anObj |
			(anObj isSpecial or: [anObj _isSymbol or: [anObj == #()]])
				ifFalse: [anObj objectSecurityPolicy: policy]]
	] ensure: [prot _leaveProtectedMode]
%

category: 'Accessing the Class Hierarchy'
method: Module
superclassForEnv: envId

"for receiver and specified environment > 0, return first non-nil of:
    persistent super class
    env 0 super class (smalltalk superclass)
 If envId == 0, equivalent to  self superClass .
"

<primitive: 676>
envId _validateClass: SmallInteger .
(envId < 0 or:[ envId > 255]) ifTrue:[ "COMPILE_ENV_MAX"
  self _errorIndexOutOfRange: envId
].
self _primitiveFailed: #superclassForEnv: args: { envId }
%

category: 'Queries'
method: Module
thisClass

"Returns the receiver.
 For a Class the receiver's metaclass returns this Class as well.
 This method is useful to get the base version of a Class if one is
 holding either the Class or its metaclass."

^ self
%

category: 'Accessing'
method: Module
transientMethodDictForEnv: envId
"result will be nil if no methods exist for specified environmentId."

^ transientMethDicts atOrNil: (envId*4 + 1) .
%

category: 'Accessing the Class Hierarchy'
method: Module
transientSuperclassForEnv: envId

 "Result is normally nil for all environments in a Smalltalk VM."

 ^ transientMethDicts ifNotNil:[:tmd | tmd atOrNil: (envId*4 + 3 )]
                      ifNil:[ nil ]
%

category: 'Updating the Class Hierarchy'
method: Module
transientSuperclassForEnv: envId put: aClass

"Intended for use only in a Ruby VM.
 Smalltalk VM method lookup does not use the transient superclass."

| ofs |
ofs := envId*4 + 3 .
transientMethDicts size < ofs ifTrue:[ transientMethDicts size: ofs ].
transientMethDicts at: ofs put: aClass .
%

category: 'Accessing'
method: Module
versionedName
| ofs str |
classHistory last == self ifTrue:[ ^ name ].
(ofs := classHistory indexOfIdentical: self) ~~ 0 ifTrue:[
  (str := String new) addAll: name ; add: $[ ; add: ofs asString; add:$] .
  ^ str
].
Error signal: self asOop asString , ', oop not found in classHistory(oop ' ,
    classHistory asOop asString , $)
%

category: 'Browser Methods'
method: Module
_instVarNamesWithSeparator: sep

"Returns a string showing my instance variables, with the given
 separator string inserted after every three names."

| result i theIvs numIvs each |

result := String new: 0.
i := 0.
theIvs := self instVarNames.
numIvs := theIvs size.
1 to: numIvs do:[:j|
  each := theIvs at: j .
  (i := i + 1) > 3 ifTrue:
    [
    result addAll: sep.
    i := 0.
    ].
  result addLast: $  .
  (each includesValue: $')
    ifTrue:[ result addAll: each _asSource ]
    ifFalse:[ result addAll: each ].
  ].
^result
%

category: 'Accessing'
method: Module
_name
  ^ name
%

category: 'Modifying Classes'
method: Module
_setClassVars: aDict old: previousDict

  classVars ~~ aDict ifTrue:[
    previousDict ~~ classVars ifTrue:[ self error:'invalid store to classVars'].
    classVars := aDict .
  ].
%

category: 'Browser Methods'
method: Module
_superClassHierarchy: envId includeOops: withOopsBool

"Returns a String showing the superClass hierarchy of the receiver.
 Used by implementation of topaz. "

| lf depth clsList rptList result |
lf := Character lf.
depth := 0.
(clsList := self _allSuperList: envId ) addLast: self .
rptList := { }  .
depth := 0 .
clsList do:[ :aCls | | aLine ivNames |
  aLine := String new .
  1 to: depth do: [:i | aLine addAll: '  '].
  aLine add: (aCls name  ifNil:[ '(nil name)' ]) .
  ivNames := aCls _instVarNamesWithSeparator:  ''   .
  ivNames size ~~ 0 ifTrue: [
     aLine add: $(; add: ivNames ; add: $) .
  ].
  depth := depth + 1 .
  rptList add: aLine .
].
withOopsBool ifTrue:[ | width padStr |
  width := 0 .
  rptList do:[:aLine | | sz |
    ((sz := aLine size) > width and:[ sz < 40 ]) ifTrue:[ width := sz ]
  ].
  width := width + 4 .
  padStr := String new .
  width timesRepeat:[ padStr add: $  . ].
  1 to: rptList size do:[:idx | | aCls aLine lsz |
    aLine := rptList at: idx .
    lsz := aLine size .
    lsz < width ifTrue:[
      aLine size: width .
      aLine replaceFrom: lsz + 1 to: width with: padStr startingAt: lsz + 1 .

    ] ifFalse:[  aLine add: $  ].
    aLine add: (aCls := clsList at: idx) asOop asString .
  ].
].
result := String new .
rptList do:[ :aLine | result add: aLine ; add: lf .].
^ result
%

category: 'Browser Methods'
method: Module
_superClassHierarchy: envId includeOops: withOopsBool includeVirtual: vBool
  ^ self _superClassHierarchy: envId includeOops: withOopsBool 
%

category: 'Browser Methods'
method: Module
_versionedName
"used by topaz"

^ [ self versionedName ] onException: Error do:[:ex| ^ self describe]
%

! Class extensions for 'MultiByteString'

!		Class methods for 'MultiByteString'

removeallmethods MultiByteString
removeallclassmethods MultiByteString

category: 'Formatting'
classmethod: MultiByteString
charSize

"Returns number of bytes that make up a character for this string class."

^ self subclassResponsibility: #charSize
%

category: 'Storing and Loading'
classmethod: MultiByteString
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into a new instance of the receiver.
 Returns the new instance."

| inst |

inst := String new: passiveObj readSize . "yes, make a String !!"
inst _loadFrom: passiveObj.
inst _changeClassToMultiByte: self .  "Now change to double/quad byte"
passiveObj hasRead: inst .
^inst
%

category: 'Instance Creation'
classmethod: MultiByteString
new

^ self subclassResponsibility: #new
%

category: 'Instance Creation'
classmethod: MultiByteString
new: aSize

^ self subclassResponsibility: #new:
%

!		Instance methods for 'MultiByteString'

category: 'Case-Insensitive Comparisons'
method: MultiByteString
< aCharCollection

"Returns true if the receiver collates before the argument.
 Returns false otherwise.

 The comparison is case-insensitive.
 Signals an Error if argument is a kind of Unicode7, Unicode16 or Unicode32 .

 The ASCII letters A and a collate the same.
   'A' < 'a' returns false.
   'A' < 'b' returns true. "

<primitive: 440>
(aCharCollection _stringCharSize bitAnd: 16r7) ~~ 0 ifTrue:[
  ^ aCharCollection > self
].
aCharCollection _validateClass: CharacterCollection .
^ aCharCollection > self
%

category: 'Case-Insensitive Comparisons'
method: MultiByteString
<= aCharCollection

"Returns true if the receiver collates before the argument or if all of the
 corresponding Characters in the receiver and argument are equal.
 Returns false otherwise.

 The comparison is consistent with that defined for the < method."

^(self > aCharCollection) == false
%

category: 'Case-Sensitive Comparisons'
method: MultiByteString
= aCharCollection

"Returns true if corresponding Characters in the receiver and argument are equal
 and aCharCollection is comparable with the receiver, and aCharCollection is
 not a kind of Symbol.  Returns false otherwise.

 The comparison for 'equal' is case-sensitive .
 Signals an Error if argument is a kind of Unicode7, Unicode16 or Unicode32 .

 Note that 'kind of Symbol' means either an instance of Symbol or instance of
 MultiByteSymbol."

<primitive: 449>
| argInfo |
argInfo := aCharCollection _stringCharSize .
(argInfo bitAnd: 16r8) ~~ 0 ifTrue:[  "arg is Unicode"
  ^ (self compareTo: aCharCollection collator: nil useMinSize: false) == 0
].
(aCharCollection isKindOf: Utf8) ifTrue:[
  ^ (self compareTo: aCharCollection collator: nil useMinSize: false) == 0
].
(aCharCollection isKindOf: CharacterCollection) ifTrue:[ ^ aCharCollection = self].
^ false
%

category: 'Case-Insensitive Comparisons'
method: MultiByteString
> aCharCollection

"Returns true if the receiver collates after the argument.  Returns false
 otherwise.

 The comparison is case-insensitive.
 Signals an Error if argument is a kind of Unicode7, Unicode16 or Unicode32."

<primitive: 442>

aCharCollection _validateClass: CharacterCollection.
^ aCharCollection < self
%

category: 'Case-Insensitive Comparisons'
method: MultiByteString
>= aCharCollection

"Returns true if the receiver collates after the argument or if all of the
 corresponding Characters in the receiver and argument are equal.
 Returns false otherwise.

 The comparison is consistent with that defined for the < method."

^ (self < aCharCollection) == false
%

category: 'Converting'
method: MultiByteString
asByteArray

	^ByteArray fromString: self
%

category: 'Deprecated'
method: MultiByteString
asciiLessThan: aString

^ self subclassResponsibility: #asciiLessThan:
%

category: 'Deprecated'
method: MultiByteString
asDoubleByteSymbol

"Returns a Symbol, DoubleByteSymbol or QuadByteSymbol
    representation of the receiver.
 Deprecated."

self deprecated:'MultiByteString >> asDoubleByteSymbol is deprecated, use asSymbol' .
^ self asSymbol .
%

category: 'Converting'
method: MultiByteString
asLowercase

"Returns a new instance of the receiver's class, with all upper-case Characters
 in the receiver changed to lower-case.

 Lower-case Characters are defined by the Unicode standard and
 u_tolower from libicu is called for each character in the receiver.

 If receiver is a DoubleByteSymbol, returns a DoubleByteString.
 If receiver is a QuadByteSymbol, returns a QuadByteString."

<primitive: 821>
self _primitiveFailed: #asLowercase .
self _uncontinuableError
%

category: 'Converting'
method: MultiByteString
asLowercaseOld

"Returns a new instance of the receiver's class, with all upper-case Characters
 in the receiver changed to lower-case.

 Lower-case Characters are the a to z,
 including letters with diacriticals, up to codePoint 255.
 Conversion of codepoints above 255 , requires that
 (deprecated)character data tables be installed.

 If receiver is a DoubleByteSymbol, returns a DoubleByteString.
 If receiver is a QuadByteSymbol, returns a QuadByteString."

<primitive: 984>
self _primitiveFailed: #asLowercaseOld .
self _uncontinuableError
%

category: 'Converting'
method: MultiByteString
asString

"Returns a (single byte) String representation of the receiver if all of the
 Characters in the receiver can be represented as single-byte Characters.
 Otherwise, returns the receiver."

| result |
(result := self _asString) ifNil:[ ^ self ].
^ result
%

category: 'Converting'
method: MultiByteString
asSymbol

"Returns a canonical Symbol representation of the receiver."

^ Symbol withAll: self .
%

category: 'Converting'
method: MultiByteString
asSymbolKind

"Returns a (single byte) Symbol representation of the receiver."

^ Symbol withAll: self
%

category: 'Converting'
method: MultiByteString
asUnicodeString

"If receiver is an Unicode16 or Unicode32, return the receiver.
 Otherwise return an instance of Unicode7 , Unicode16 or Unicode32 ,
 using the class with smallest character size needed to
 represent a copy of the receiver.
 Raises an ArgumentError if receiver contains a codePoint > 16r10FFFF,
 or a codePoint in the range 16rD800 to 16rDFFF "

<primitive: 927>

self _primitiveFailed:#asUnicodeString
%

category: 'Converting'
method: MultiByteString
asUppercase

"Returns a new instance of the receiver's class, with all lower-case Characters
 in the receiver changed to upper-case.

 Upper-case Characters are defined by the Unicode standard and
 u_toupper from libicu is called for each character in the receiver.

 If receiver is a DoubleByteSymbol, returns a DoubleByteString.
 If receiver is a QuadByteSymbol, returns a QuadByteString."

<primitive: 455>
self _primitiveFailed: #asUppercase .
self _uncontinuableError
%

category: 'Converting'
method: MultiByteString
asUppercaseOld

"Returns a new instance of the receiver's class, with all lower-case Characters
 in the receiver changed to upper-case.

 Upper-case Characters are the A to Z,
 including letters with diacriticals, up to codePoint 255.
 Conversion of codepoints above 255 , requires that
 (deprecated)character data tables be installed.

 If receiver is a DoubleByteSymbol, returns a DoubleByteString.
 If receiver is a QuadByteSymbol, returns a QuadByteString."

<primitive: 983>
self _primitiveFailed: #asUppercaseOld .
self _uncontinuableError
%

category: 'Accessing'
method: MultiByteString
at: anIndex

"Returns the Character at anIndex."

^ self subclassResponsibility: #at:
%

category: 'Case-Sensitive Comparisons'
method: MultiByteString
at: anIndex equals: aCharCollection

"Returns true if aCharCollection is contained in the receiver, starting at
 anIndex.  Returns false otherwise.  The comparison is case-sensitive."

<primitive: 668>
anIndex _validateClass: SmallInteger.
((anIndex <= 0) or: [(anIndex > self size)])
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex ].
aCharCollection _validateClass: CharacterCollection.
^ self _primitiveFailed: #at:equals: args: { anIndex . aCharCollection }
%

category: 'Case-Insensitive Comparisons'
method: MultiByteString
at: anIndex equalsNoCase: aCharCollection

"Returns true if aCharCollection is contained in the receiver, starting at
 anIndex.  Returns false otherwise.  The comparison is case-insensitive."

<primitive: 438>
anIndex _validateClass: SmallInteger.
((anIndex <= 0) or: [(anIndex > self size)])
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex ].
aCharCollection _validateClass: CharacterCollection.
^ self _primitiveFailed: #at:equalsNoCase: args: { anIndex . aCharCollection }
%

category: 'Updating'
method: MultiByteString
at: anIndex put: aChar

") Stores aChar at the specified location and returns aChar."

^ self subclassResponsibility: #at:put:
%

category: 'Formatting'
method: MultiByteString
charSize

"Returns number of bytes that make up a character for this string class."

^ self subclassResponsibility: #charSize
%

category: 'Comparing'
method: MultiByteString
codePointCompareTo: aString

"aString must be a CharacterCollection or ByteArray,
 and must not be a Utf8 or Utf16 .
 Return -1 , 0, or 1  based on how self compares to aString
 using codePoint comparison of each character. "

<primitive: 844>
^ self _primitiveFailed: #codePointCompareTo: args: { aString }
%

category: 'Testing'
method: MultiByteString
containsSeparator

"Returns true if the receiver contains a separator Character."

1 to: self size do: [:i |
  (self at: i) isSeparator ifTrue: [^true].
].
^false
%

category: 'Copying'
method: MultiByteString
copyFrom: startIndex to: stopIndex

"Returns a new Array containing the elements of the receiver
 between startIndex and stopIndex, inclusive.  The result is of the same class
 as the receiver.

 If startIndex > stopIndex then an empty collection is returned.
 Otherwise both startIndex and stopIndex must be positive integers not larger than the
 size of the receiver, with startIndex <= stopIndex.
"

<primitive: 818>
(startIndex < 1) ifTrue: [ ^ self _errorIndexOutOfRange: startIndex].

((stopIndex > self size) or: [(stopIndex < 0)])
   ifTrue: [ ^ self _errorIndexOutOfRange: stopIndex].

self _primitiveFailed: #copyFrom:to: args: { startIndex . stopIndex }
%

category: 'Encoding'
method: MultiByteString
encodeUsing: encodingArray

  "A new instance of the class of the receiver will be returned where
   each character in the receiver is replaced by the character or string
   from encodingArray that is located at the position in encodingArray for
   the character.  The codePoint of the character is used as an index
   into the encodingArray,  value 16r0 accesses   encodingArray at:1 .
   An Error is signaled if any of the following are true
    the codePoint is >= size of the encodingArray

    the value from the encodingArray is a Character with codePoint too large
    to fit in an instance of the receiver's class.

    the value from the encodingArray is a CharacterCollection with
    bytes-per-character bigger than bytes-per-character of the receiver

    the value from the encodingArray is neither a String, MultiByteString,
    nor Character . "
  <primitive: 654>
  encodingArray _validateInstanceOf: Array  .
  self _primitiveFailed: #encodeUsing: args: { encodingArray }
%

category: 'Deprecated'
method: MultiByteString
equals: aString collatingTable: aTable

"Deprecated. Returns true if the receiver collates the same as the argument.
 See lessThan:collatingTable: for description of aTable.
 The collating sequence is defined by aTable."

self deprecated: 'MultiByteString>>equals:collatingTable: deprecated in 3.2'.
^ self _primEquals: aString collatingTable: aTable
%

category: 'Case-Insensitive Comparisons'
method: MultiByteString
equalsNoCase: aCharCollection

"Returns true if corresponding Characters in the receiver and argument are equal
 and aCharCollection is comparable with the receiver, and aCharCollection is
 not a kind of Symbol.  Returns false otherwise.

 The comparison for equal is case-insensitive.

 Note that 'kind of Symbol' means either an instance of Symbol or instance of
 DoubleByteSymbol."

<primitive: 450>
aCharCollection _stringCharSize ~~ 0 ifTrue:[
  self _primitiveFailed: #equalsNoCase:  args: { aCharCollection } .
  ^ self _uncontinuableError
].
(aCharCollection isKindOf: CharacterCollection) ifTrue: [
  ^ self equalsNoCase: (String withAll: aCharCollection)
] ifFalse:[
  ^ false
].
%

category: 'Execution'
method: MultiByteString
evaluate

"Compiles the receiver as an unbound method and executes it using the current
 default symbol list."

^ (self _compileInContext: nil
       symbolList: GsSession currentSession symbolList)
  _executeInContext: nil
%

category: 'Execution'
method: MultiByteString
evaluateInContext: anObject symbolList: aSymbolList

"Compiles the receiver as an instance method for the class of anObject, using
 aSymbolList as the symbol list.  Executes the resulting GsMethod using anObject
 as self and returns the result of the execution.  Generates an error if
 compilation errors occur."

^ (self _compileInContext: anObject symbolList: aSymbolList)
    _executeInContext: anObject
%

category: 'Deprecated'
method: MultiByteString
greaterThan: aString collatingTable: aTable

"Deprecated. Returns true if the receiver collates after the argument.
 See lessThan:collatingTable: for description of aTable."

self deprecated: 'MultiByteString>>greaterThan:collatingTable: deprecated in 3.2'.
^self _primGreaterThan: aString collatingTable: aTable
%

category: 'Deprecated'
method: MultiByteString
greaterThanOrEqual: aString collatingTable: aTable

"Deprecated. Returns true if the receiver collates after or the same as the argument.
 The collating sequence is defined by aTable."

self deprecated: 'MultiByteString>>greaterThanOrEqual:collatingTable: deprecated in 3.2'.
^ (self lessThan: aString collatingTable: aTable) not
%

category: 'Hashing'
method: MultiByteString
hash

"Returns a positive Integer based on a case-sensitive hash of the contents
 of the receiver."

"The algorithm implemented is described in:

 [Pearson 90]
 Pearson, Peter K., Fast Hashing of Variable-Length Text Strings,
 Communications of the ACM 33, 6, (June 1990), 677-680."

<primitive: 282>
self _primitiveFailed: #hash .
self _uncontinuableError
%

category: 'Case-Sensitive Searching'
method: MultiByteString
includes: aCharacter

"Returns true if the receiver contains aCharacter, false otherwise.
 The search is case-sensitive."

^(aCharacter isKindOf: AbstractCharacter) and:
	[self includesValue: aCharacter].
%

category: 'Case-Sensitive Searching'
method: MultiByteString
includesValue: aCharacter

"Returns true if the receiver contains aCharacter.  The search is
 case-sensitive."

<primitive: 452>

"the following code is executed if aCharacter is a JISCharacter "

aCharacter _validateClass: AbstractCharacter .
^ self includesValue: aCharacter asCharacter .
%

category: 'Case-Sensitive Searching'
method: MultiByteString
indexOf: aCharacter startingAt: startIndex

"Returns the index of the first occurrence of aCharacter in the receiver,
 not preceding startIndex.  If the receiver does not contain aCharacter,
 returns zero.

 The search is case sensitive."

<primitive: 453>
startIndex _validateClass: Integer .
startIndex < 1 ifTrue:[ ^ self _errorIndexOutOfRange: startIndex].
^ self indexOf: aCharacter asCharacter startingAt: startIndex
%

category: 'Case-Insensitive Comparisons'
method: MultiByteString
isEquivalent: aCharCollection
    "Returns true if the receiver is equivalent to aCharCollection.
    The receiver is equivalent to aCharCollection if the receiver
    contains the same Characters as aCharCollection regardless of case
    or internal representation.  For example, if $a is in
    aCharCollection, it is equivalent to any representation of an 'a'
    in the receiver's character set."

<primitive: 450>
aCharCollection _stringCharSize ~~ 0 ifTrue:[
  self _primitiveFailed: #isEquivalent:  args: { aCharCollection } .
  ^ self _uncontinuableError
].
(aCharCollection isKindOf: CharacterCollection) ifTrue: [
  ^ self isEquivalent: (String withAll: aCharCollection)
] ifFalse:[
  ^ false
].
%

category: 'Encoding'
method: MultiByteString
javaScriptEncode
  "Returns a String.
   The primitive encapulates all of the encoding logic
   from Seaside JSStream>>encodeString:on: .
   Assumes Character>>greaseInteger always equivalent to Character>>codePoint
  "
<primitive: 1046>
^ self _primitiveFailed: #javaScriptEncode
%

category: 'Deprecated'
method: MultiByteString
lessThan: aString collatingTable: aTable

"Deprecated. Returns true if the receiver collates before the argument.
 The collating sequence is defined by aTable which must be
 a string with the same character size as the receiver."

self deprecated: 'MultiByteString>>lessThan:collatingTable: deprecated in 3.2'.
^self _primLessThan: aString collatingTable: aTable
%

category: 'Deprecated'
method: MultiByteString
lessThanOrEqual: aString collatingTable: aTable

"Deprecated. Returns true if the receiver collates before or the same as the argument.
 The collating sequence is defined by aTable."

self deprecated: 'MultiByteString>>lessThanOrEqual:collatingTable: deprecated in 3.2'.
^ (self greaterThan: aString collatingTable: aTable) not
%

category: 'Accessing'
method: MultiByteString
numArgs

"Returns the number of arguments the receiver would take, if the receiver were
 a message selector."

self isInfix ifTrue: [ ^1 ].
self isKeyword ifTrue: [ ^self identicalOccurrencesOf: $: ].
^0
%

category: 'Formatting'
method: MultiByteString
printOn: aStream

"Puts a displayable representation of the receiver on the given stream."

"For classes whose instances can be literals, the result should contain
 formatting information.  For example, the following expression should
 evaluate to true:

 #abc asString = String withAll: '#abc'"

| idx lastIdx sz |

aStream _nextPut: $' .

idx := self indexOf: $' startingAt: 1 .
(idx == 0 or:[ idx > aStream maxSize ]) ifTrue:[
  aStream nextPutAll: self .
  ]
ifFalse:[
  lastIdx := 1.
  [ (idx == 0) or: [aStream isFull]] whileFalse: [
    aStream nextPutAll: ( self copyFrom: lastIdx to: idx) .
    aStream nextPut: $' .
    lastIdx := idx + 1 .
    idx := self indexOf: $' startingAt: lastIdx .
    ].
  sz := self size .
  ((lastIdx <= sz) and: [aStream isFull == false]) ifTrue: [
    aStream nextPutAll: (self copyFrom: lastIdx to: sz) .
    ].
  ].
aStream _nextPut: $'
%

category: 'Removing'
method: MultiByteString
removeFrom: startIndex to: stopIndex

"Deletes the elements of the receiver from startIndex to stopIndex.  The
 size of the receiver is decreased by stopIndex - startIndex + 1.

 Both startIndex and stopIndex must be positive integers not larger than the
 size of the receiver, with startIndex <= stopIndex."

<primitive: 271>
(stopIndex < startIndex)
ifTrue:
   [ ^ startIndex _error: #rtErrBadCopyFromTo args: { stopIndex }].
((stopIndex > self size) or: [(stopIndex < 1)])
   ifTrue: [ ^ self _errorIndexOutOfRange: stopIndex].
(startIndex < 1)
   ifTrue: [ ^ self _errorIndexOutOfRange: startIndex].
^ self _primitiveFailed: #removeFrom:to: args: { startIndex . stopIndex }
%

category: 'Copying'
method: MultiByteString
reverse

"Returns a copy of the receiver with its elements in reverse order."
| copy |
copy := self species new: self size .
^ copy _reverseFrom: self
%

category: 'Accessing'
method: MultiByteString
size

"Returns the number of Characters in the receiver"

^ self subclassResponsibility: #size
%

category: 'Updating'
method: MultiByteString
size: anInteger


"Changes the size of the receiver to anInteger Characters."

^ self subclassResponsibility: #size:
%

category: 'Encoding'
method: MultiByteString
sizeForEncodeAsUTF8

"Returns a SmallInteger, the size of the result of encodeAsUTF8."

<primitive: 520>
self _primitiveFailed: #sizeForEncodeAsUTF8
%

category: 'Accessing'
method: MultiByteString
valueAt: anIndex

"Returns the value of the (multi-byte) Character at anIndex."

^ (self at: anIndex) codePoint
%

category: 'Copying'
method: MultiByteString
withLFs

"Returns a copy of the receiver with all back-slashes replaced by line-feeds."

^self copyReplaceAll: '\' with: (self class with: Character lf)
%

category: 'Accessing'
method: MultiByteString
wordAt: anIndex

"Returns the integer value of the Character at anIndex in the receiver."

^ (self at: anIndex) codePoint
%

category: 'Updating'
method: MultiByteString
wordAt: anIndex put: aValue

"Stores the integer value aValue in the character cell of the receiver
 specified by anIndex. Return aValue."

self at: anIndex put: (Character withValue: aValue) .
^ aValue
%

category: 'Storing and Loading'
method: MultiByteString
writeTo: passiveObj

"Converts the receiver to its passive form and writes that information on
 passiveObj."

passiveObj writeClass: self class;
  writeSize: self _basicSize;
    nextPutAllBytes: self; space
%

category: 'Decompiling without Sources'
method: MultiByteString
_asSource

^ self quoted
%

category: 'Converting'
method: MultiByteString
_asString

"Returns a (single byte) String representation of the receiver if all of the
 Characters in the receiver can be represented as single-byte Characters.
 Returns nil if any of the Characters in the receiver cannot be represented as
 single-byte Characters."

<primitive: 229>

^ self _primitiveFailed: #_asString .
%

category: 'Converting'
method: MultiByteString
_asUnicode16

"Return an instance of Unicode7 or Unicode16 using the
minimum bytes per character required to represent the receiver.
Return nil if the receiver is not representable as Unicode7 nor Unicode16."

<primitive: 926>

self _primitiveFailed:#_asUnicode16
%

category: 'Converting'
method: MultiByteString
_asUnicode7

<primitive: 925>
"Return an instance of Unicode7 if receiver can be
represented as an Unicode7 string, else return nil."

self _primitiveFailed:#_asUnicode7
%

category: 'Compatibility'
method: MultiByteString
_at: anIndex equals: aCharCollection ignoreCase: aBoolean

"Returns true if aCharCollection is contained in the receiver, starting at
 anIndex.  Returns false otherwise.  The comparison is case-insensitive."

^ aBoolean ifTrue:[ self at: anIndex equalsNoCase: aCharCollection ]
	  ifFalse:[ self at: anIndex equals: aCharCollection ]
%

category: 'Indexing Support'
method: MultiByteString
_changingSizeTo: newSize

"Notifies any modification tracking objects that the receiver (an indexable
 object) is having its size changed."

"the object manager reports size changes in number of bytes; convert this
 to number of characters."

^ super _changingSizeTo: newSize // (self charSize)
%

category: 'Execution'
method: MultiByteString
_compileInContext: anObject symbolList: aSymbolList

"Compiles the receiver as an instance method for the class of
 anObject, using aSymbolList as the symbol list, and using the
 default compilation environment.
 The receiver is parsed as a doit, i.e. it does not have a selector.

 Returns a GsMethod that is not in the method dictionary of any class, or
 else generates an error if there are compilation errors."

^ self _compileInContext: anObject symbolList: aSymbolList oldLitVars: nil
      environmentId: 0 flags: 0
%

category: 'Execution'
method: MultiByteString
_compileInContext: anObject symbolList: aSymbolList
oldLitVars: litVarArray  environmentId: anEnvironmentId flags: flagsInt

"Compiles the receiver as an instance method for the class of
 anObject, using aSymbolList as the symbol list.
 The receiver is parsed as a doit, i.e. it does not have a selector.

 If litVarArray is not nil, it must be an Array of Symbol,SymbolAssociation pairs
 and this Array will be searched prior to searching aSymbolList to
 resolve literal variables within the method.

 anEnvironmentId must be a SmallInteger >= 0 and <= 255
 specifying a compilation environment.

 flagsInt must be a SmallInteger >= 0 and <= 255 and contains the bits
 specified by GCI_COMPILE_* in $GEMSTONE/include/gcicmn.ht .

 Returns a GsMethod that is not in the method dictionary of any class, or
 else generates an error if there are compilation errors."

<primitive: 150>
aSymbolList _validateClass: SymbolList .
^ self _primitiveFailed: #_compileInContext:symbolList:oldLitVars:environmentId:flags:
       args: { anObject . aSymbolList . litVarArray . anEnvironmentId . flagsInt}
%

category: 'Private'
method: MultiByteString
_findLastString: subString startingAt: startIndex ignoreCase: aBoolean

"Searches backwards through receiver    starting at startIndex  .
 Returns the index at which subString begins.  If the
 receiver does not contain subString, this returns 0."

<primitive: 451>

startIndex _validateClass: SmallInteger.
(startIndex < 1) | (startIndex > self size)
  ifTrue: [ ^ self _error: #objErrBadOffsetIncomplete args: { startIndex } ].
subString _validateClasses:{ String }.
aBoolean _validateClass: Boolean .
^ self _primitiveFailed: #_findLastString:startingAt:ignoreCase:
       args: { subString . startIndex . aBoolean }
%

category: 'Private'
method: MultiByteString
_findString: subString startingAt: startIndex ignoreCase: aBoolean

"If a receiver contains subString beginning at some point at or after
 startIndex, this returns the index at which subString begins.  If the
 receiver does not contain subString, this returns 0."

<primitive: 30>
startIndex _validateClass: SmallInteger.
(startIndex < 1)
  ifTrue: [ ^ self _error: #objErrBadOffsetIncomplete args: { startIndex } ].
subString _validateClasses:{ String }.
aBoolean _validateClass: Boolean .
^ self _primitiveFailed: #_findString:startingAt:ignoreCase:
       args: { subString . startIndex . aBoolean }
%

category: 'New Indexing Comparison - prims'
method: MultiByteString
_idxPrimCompareEqualTo: aCharCollection
  "This comparison operation is used for the indexing subsystem to
 determine an ordering for insertion into indexing objects.

 This method collates letters AaBb..Zz."

  "The comparison should be compatible with the case-insensitive semantics
 of the String method with selector #= .
 Same primitive as String>>lessThan: "

  <primitive: 981>
  aCharCollection isUnicodeString
    ifTrue: [
      "Unicode"
      ArgumentError signal:'Unicode argument disallowed in String comparison' ]
    ifFalse: [
      aCharCollection ifNil:[ ^ false ].
      aCharCollection _validateClass: CharacterCollection.
      "JapaneseString or MixedString"
      ^ self = aCharCollection ]
%

category: 'New Indexing Comparison - prims'
method: MultiByteString
_idxPrimCompareGreaterThan: aCharCollection

"This comparison operation is used for the indexing subsystem to
 determine an ordering for insertion into indexing objects such as B-trees.

 This method collates letters AaBb..Zz."

"The comparison should be compatible with the case-insensitive semantics
 of the String method with selector #> .
 Same primitive as String>>greaterThan: ."

<primitive: 443>
aCharCollection _validateClass: CharacterCollection.
^ aCharCollection isUnicodeString
    ifTrue: [ ArgumentError signal:'Unicode argument disallowed in String comparison' ]
    ifFalse: [
      "JapaneseString or MixedString"
      self > aCharCollection ]
%

category: 'New Indexing Comparison - prims'
method: MultiByteString
_idxPrimCompareLessThan: aCharCollection

"This comparison operation is used for the indexing subsystem to
 determine an ordering for insertion into indexing objects.

 This method collates letters AaBb..Zz."

"The comparison should be compatible with the case-insensitive semantics
 of the String method with selector #< .
 Same primitive as String>>lessThan: "

<primitive: 441>

aCharCollection _validateClass: CharacterCollection.
^ aCharCollection isUnicodeString
    ifTrue: [ ArgumentError signal:'Unicode argument disallowed in String comparison' ]
    ifFalse: [
      "JapaneseString or MixedString"
      self < aCharCollection ]
%

category: 'Deprecated'
method: MultiByteString
_primEquals: aString collatingTable: aTable

"Deprecated. Returns true if the receiver collates the same as the argument.
 See lessThan:collatingTable: for description of aTable.
 The collating sequence is defined by aTable."

<primitive: 446 >
aTable _validateSameCharSizeAs: self .
aString _validateClasses:{ String }.
self _primitiveFailed: #equals:collatingTable: args: { aString . aTable }
%

category: 'Deprecated'
method: MultiByteString
_primGreaterThan: aString collatingTable: aTable

"Deprecated. Returns true if the receiver collates after the argument.
 See lessThan:collatingTable: for description of aTable."

<primitive: 433 >
aTable _validateSameCharSizeAs: self .
aString _validateClasses:{ String }.
self _primitiveFailed: #greaterThan:collatingTable: args: { aString . aTable }
%

category: 'Deprecated'
method: MultiByteString
_primLessThan: aString collatingTable: aTable

"Deprecated. Returns true if the receiver collates before the argument.
 The collating sequence is defined by aTable which must be
 a string with the same character size as the receiver."

<primitive: 359 >
aTable _validateSameCharSizeAs: self .
aString _validateClasses:{ String }.
self _primitiveFailed: #lessThan:collatingTable: args: { aString . aTable }
%

category: 'Copying'
method: MultiByteString
_reverseFrom: aString

"copies elements of aString into receiver in reverse order.
 Size of aString must be same as size of receiver.
 Returns receiver.
 aString==self is allowed. "

<primitive: 793>
aString stringCharSize == self stringCharSize ifFalse:[
  aString _error: #rtErrBadArgKind args:{ self class }.
].
aString size == self size ifFalse:[
  aString _error: #rtErrBadSize args:{ self size . aString size }.
].
self _primitiveFailed: #_reverseFrom args: { aString }
%

category: 'Private'
method: MultiByteString
_validateSameCharSizeAs: aString

"Returns nil if self has same character size as aString,
 otherwise generates an error."

| cSize cls |
self stringCharSize ~~ (cSize := aString stringCharSize) ifTrue:[
  cSize == 1 ifTrue:[ cls := String ].
  cSize == 2 ifTrue:[ cls := DoubleByteString ].
  cSize == 4 ifTrue:[ cls := QuadByteString ].
  self _errorExpectedClass: cls
].
^ nil
%

! Class extensions for 'NameError'

!		Class methods for 'NameError'

removeallmethods NameError
removeallclassmethods NameError

category: 'Signaling'
classmethod: NameError
signalNotImplemented
  "Signal that the sender of this method is 'not implemented' "
   ^ self new selector: ((GsProcess methodAt: 2) selector) ; 
              signal:'not implemented'.
%

!		Instance methods for 'NameError'

category: 'Instance initialization'
method: NameError
args: anArray
  anArray _isArray ifTrue:[ | sz |
    selector := anArray atOrNil: 1 .
    (sz := anArray size) > 1 ifTrue:[
      gsArgs := anArray copyFrom:2 to: sz
    ].
  ]
%

category: 'Formatting'
method: NameError
buildMessageText
^ self buildMessageText:(
  [ | str |
    str := String new .
    selector ifNotNil:[:s| str add: s asString ] .
    gsDetails ifNotNil:[:d| str add:', '; add: d  asString ].
    str
  ] onException: Error do:[:ex |
    ex return: nil
  ]
)
%

category: 'Instance initialization'
method: NameError
initialize
  gsNumber := ERR_NameError.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Accessing - failedMessage'
method: NameError
selector
  ^ selector
%

category: 'Accessing - failedMessage'
method: NameError
selector: aSymbol
  selector := aSymbol
%

category: 'Instance initialization'
method: NameError
_legacyHandlerArgs
  | arr |
  (arr := { selector } ) addAll: super _legacyHandlerArgs .
  ^ arr
%

! Class extensions for 'Notification'

!		Class methods for 'Notification'

removeallmethods Notification
removeallclassmethods Notification

category: 'Documentation'
classmethod: Notification
disableSignalling
  "revert to the default behavior of Notification which is to ignore them."
  System __sessionStateAt: 22 put:  nil
%

category: 'Documentation'
classmethod: Notification
enableSignalling
  "enable signalling of Notifications to the GCI client."
  System __sessionStateAt: 22 put: true .
%

category: 'Documentation'
classmethod: Notification
signallingEnabled

  "returns true if Notifications will be signalled to the GCI client,
   false otherwise. "
  ^ (System __sessionStateAt: 22) ~~ nil
%

!		Instance methods for 'Notification'

category: 'Handling'
method: Notification
defaultAction

  (System __sessionStateAt: 22) ifNotNil:[
    self _signalGciError "deliver error to GCI, does not return"
  ].
  ^ nil "take no action,  the value 'nil' is returned as the value
      of the message that signaled the exception."
%

category: 'Instance initialization'
method: Notification
initialize
  gsNumber := ERR_Notification.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Private'
method: Notification
_errNumPrefix

^ '(notification '
%

! Class extensions for 'Number'

!		Class methods for 'Number'

removeallmethods Number
removeallclassmethods Number

category: 'Instance Creation'
classmethod: Number
fromStream: aStream
  "BinaryExponent = ( 'e' | 'E' | 'd' | 'D' | 'q') ['-' | '+'] Digits
   DecimalExponent = ( 'f' | 'F' ) ['-' | '+'] Digits
   Digit = '0' | '1' | '2' | ... | '9'
   Digits = Digit {Digit}
   Exponent = BinaryExponent | DecimalExponent | ScaledDecimalExponent | FixedPointExponent
   FractionalPart = '.' Digits [Exponent]
   FixedPointExponent = 'p' [ ['-' | '+'] Digits ]
   Number = RadixedLiteral | NumericLiteral
   Numeric = Digit | 'A' | 'B' | ... | 'Z'
   NumericLiteral = Digits ( [FractionalPart] | [Exponent] )
   Numerics = Numeric { Numeric }
   RadixedLiteral = Digits ( '#' | 'r' ) ['-'] Numerics
   ScaledDecimalExponent = 's' [ ['-' | '+'] Digits ]
"
  | ch integerPart factor signFound dCount |
  self _checkReadStream: aStream forClass: CharacterCollection.
  ch := aStream next.
  [ ch unicodeIsWhitespace] whileTrue: [ ch := aStream next ].
  aStream skip: -1.
  integerPart := 0.
  factor := 1.
  signFound := false.
  (aStream peek isEquivalent: $-)
    ifTrue: [
      aStream next.
      signFound := true.
      factor := -1 ]
    ifFalse: [
      (aStream peek isEquivalent: $+)
        ifTrue: [
          aStream next.
          signFound := true ] ].
  dCount := 0 .
  [ (ch := aStream peek) ~~ nil and: [ ch isDigit ] ]
    whileTrue: [
      aStream next.
      integerPart := integerPart * 10 + ch digitValue .
      dCount := dCount + 1 ].
  dCount == 0 ifTrue:[ ^ self _errIncorrectFormat: aStream ].

  (ch == $r or: [ ch == $# ])
    ifTrue: [
      "found a radix separator"
      aStream next.
      ^ Integer
        _finishFromStream: aStream
        signFound: signFound
        factor: factor
        radix: integerPart ].
  (ch isEquivalent: (Locale decimalPoint at: 1))
    ifTrue: [
      "found a decimal point"
      aStream next.
      ^ BinaryFloat
        _finishFromStream: aStream
        signFound: signFound
        factor: factor
        integerPart: integerPart ].
  (#( $e $E $d $D $q $s $p $f $F ) includesIdentical: ch) ifTrue:[
      aStream next.
      ^ BinaryFloat
        _finishFromStream: aStream
        signFound: signFound
        factor: factor
        integerPart: integerPart
        fractionalPart: 0
        exponent: ch ].
  ^ integerPart * factor
%

category: 'Instance Creation'
classmethod: Number
migrateNew

"Override default migrateNew behavior with #_basicNew."

^ self _basicNew
%

category: 'Instance Creation'
classmethod: Number
new

"Disallowed."

self shouldNotImplement: #new
%

category: 'Instance Creation'
classmethod: Number
new: anInteger

"Disallowed."

self shouldNotImplement: #new:
%

category: 'Custom Numeric Literals'
classmethod: Number
parseLiterals: aCharacter exponentRequired: aBoolean
  "Add a new numeric literal class to those recognized by the compiler.

   aCharacter specifies an new exponent letter to be recognized, and
   must be a Unicode alphabetic codePoint < 127 .

   If aBoolean==false the literal's exponent is parsed as for ScaledDecimal,
      Exponent =  aCharacter [ ['-' ] Digits ]

   If aBoolean==true, the literal's exponent is parsed as for binary floats,
      Exponent = aCharacter ['-'] Digits

   The compiler will send fromString: to the class which was the receiver of
   parseLiterals:exponentRequired:  to construct an instance of a new
   numeric literal class. The fromString: implementation must be able to
   parse the string and return a new instance of the receiver.

   Should be invoked from session initialization; once invoked
   the new literal will be recognized until the session logs out.

   To deinstall a literal class from the compiler.
   invoke with   aBoolean == nil  ; if the receiver and aCharacter
   do not match an installed literal class, an Error is signalled.

   An Error is signaled if the receiver or aCharacter matches a
   predefined literal specified in the GemStone BNF.
   Thus aCharacter may not be one of   e E d D f F p q r s  .

   Returns an Array of triples listing additional literals recognized by
   the compiler, of the form  { exponentCharacter, aClass, aBoolean  ... }

   If aCharacter==nil, has no effect and returns the Array of literals recognized."

  <primitive: 997>
  aBoolean ifNotNil:[ aBoolean _validateClass: Boolean ].
  aCharacter _validateClass: Character .
  ^ self _primitiveFailed: #parseLiterals:exponentRequired: args: { aCharacter . aBoolean }
%

category: 'Modifying Classes'
classmethod: Number
_addInvariantClassVar: aSymbol value: aVal
  | dict assoc |
  self _validatePrivilege ifFalse:[ ^ nil ].
  (dict := classVars) ifNil:[
    dict := self _createClassVarsDict .
  ].
  assoc := dict associationAt: aSymbol otherwise: nil .
  assoc ifNotNil:[ | oldVal |
    oldVal := assoc _value .
    "special comparison logic to handle NaN classVars in Float classes"
    (oldVal class == aVal class and:[ oldVal asString = aVal asString ]) ifTrue:[
      ^ self "no change"
    ].
  ].
  ^ super _addInvariantClassVar: aSymbol value: aVal
%

category: 'Instance Creation'
classmethod: Number
_finishFromStream: aStream signFound: signFound factor: factor integerPart: integerPart fractionalPart: fractionalPart exponent: exponent exponentPart: exponentPart
  "BinaryExponent = ( 'e' | 'E' | 'd' | 'D' | 'q' ) ['-' | '+'] Digits
   DecimalExponent = ( 'f' | 'F' ) ['-' | '+'] Digits
   Digit = '0' | '1' | '2' | ... | '9'
   Digits = Digit {Digit}
   Exponent = BinaryExponent | DecimalExponent | ScaledDecimalExponent | FixedPointExponent
   FractionalPart = '.' Digits [Exponent]
   FixedPointExponent = 'p' [ ['-' | '+'] Digits ]
   Number = RadixedLiteral | NumericLiteral
   Numeric = Digit | 'A' | 'B' | ... | 'Z'
   NumericLiteral = Digits ( [FractionalPart] | [Exponent] )
   Numerics = Numeric { Numeric }
   RadixedLiteral = Digits ( '#' | 'r' ) ['-'] Numerics
   ScaledDecimalExponent = 's' [ ['-' | '+'] Digits ]
"

  "create appropriate subclass from the pieces ... by hook or by crook"

| aString |

 aString :=   (factor * integerPart) asString , (Locale decimalPoint at: 1) asString , fractionalPart asString , exponent asString , exponentPart asString.
(exponent == $s) ifTrue: [^ScaledDecimal fromString: aString].
(exponent == $p) ifTrue: [^FixedPoint fromString: aString].
(exponent isEquivalent: $f) ifTrue: [^DecimalFloat fromString: aString].
  ^ BinaryFloat fromString: aString
%

category: 'Custom Numeric Literals'
classmethod: Number
_fromLiteralString: aString
  "Sent from C by the compiler to construct an instance of a literal which has
   been registered by invoking parseLiterals:exponentRequired:
   Returns a kind of Number, or returns a String describing an error."
^ [
    self fromString: aString
  ] onSynchronous: Exception do:[:ex | | msg sfx |
    msg := ex asString .
    (msg isKindOfClass: String) ifFalse:[ msg := 'error during _fromLiteralString:'].
    sfx := ', for a String, ' .
    (msg at:( msg size - sfx size + 1) equals: sfx) ifTrue:[
      msg size: msg size - sfx size .
    ].
    ^ '; ', msg
  ]
%

category: 'Private'
classmethod: Number
_kindList

""

"   1       2          3          4      5              6 "
^ #(#normal #subnormal #infinity #zero #quietNaN #signalingNaN)
%

!		Instance methods for 'Number'

category: 'Arithmetic'
method: Number
* aNumber

"(Subclass responsibility.)  Returns the result of multiplying the receiver by
 aNumber."

Number subclassResponsibility: #*
%

category: 'Arithmetic'
method: Number
+ aNumber

"(Subclass responsibility.)  Returns the sum of the receiver and aNumber."

Number subclassResponsibility: #+
%

category: 'Arithmetic'
method: Number
- aNumber

"(Subclass responsibility.)  Returns the difference between the receiver and
 aNumber."

Number subclassResponsibility: #-
%

category: 'Arithmetic'
method: Number
/ aNumber

"(Subclass responsibility.)  Returns the result of dividing the receiver by
 aNumber."

Number subclassResponsibility: #/
%

category: 'Arithmetic'
method: Number
// aNumber

"Divides the receiver by aNumber.  Returns the integer quotient, with
 truncation toward negative infinity.  For example,

 9 // 4 = 2
 -9 // 4 = -3
 -0.9 // 0.4 = -3

 The selector \\ returns the remainder from this division."

^ (self / aNumber) floor
%

category: 'Arithmetic'
method: Number
abs

"Returns a Number that is the absolute value of the receiver."

(self < 0)
   ifTrue: [^self negated].
^self
%

category: 'Arithmetic'
method: Number
arcCos

"Returns the arc-cosine of the receiver in radians."

^ self asFloat arcCos
%

category: 'Arithmetic'
method: Number
arcSin

"Returns the arc-sine of the receiver in radians."

^ self asFloat arcSin
%

category: 'Arithmetic'
method: Number
arcTan

"Returns the arc-tangent of the receiver in radians."

^ self asFloat arcTan
%

category: 'Converting'
method: Number
asDecimalFloat

"(Subclass responsibility.)  Returns a DecimalFloat representing the receiver."

Number subclassResponsibility: #asDecimalFloat

%

category: 'Converting'
method: Number
asFixedPoint: scale

| f |
f := self asFraction .
f _isInteger ifTrue:[ ^ FixedPoint numerator: f denominator: 1 scale: scale].
^ f asFixedPoint: scale
%

category: 'Converting'
method: Number
asFloat

"(Subclass responsibility.)  Returns a SmallDouble or Float representing the receiver."

Number subclassResponsibility: #asFloat
%

category: 'Converting'
method: Number
asFloatD
	"We only have one precision of floating point."

	^self asFloat.
%

category: 'Converting'
method: Number
asFloatE
	"We only have one precision of floating point."

	^self asFloat.
%

category: 'Converting'
method: Number
asFloatQ
	"We only have one precision of floating point."

	^self asFloat.
%

category: 'Converting'
method: Number
asFraction

"(Subclass responsibility.)  Returns a Fraction that represents the receiver."

Number subclassResponsibility: #asFraction
%

category: 'Converting'
method: Number
asInteger

"Returns the integer that is closest to the receiver, on the same side
 of the receiver as zero is located.  In particular, returns the
 receiver if the receiver is an integer.

 Truncates to be compatible with client Smalltalk implementations and
 differs from ANSI which specifies rounding ."

^self truncated
%

category: 'Converting'
method: Number
asScaledDecimal: scale

"Returns a ScaledDecimal representing the receiver, with
 the specified scale.

 If the receiver is an Integer, or responds to asFraction with an
 Integer result, the result will have the specified scale.  This
 is to conform with other Smalltalk implementations. We do not
 conform to ANSI , which specifieds a ScaledDecimal result
 with scale 0 in this case."

| f |
f := self asFraction .
f _isInteger ifTrue:[ ^ ScaledDecimal for: f scale: scale  ].
^ f asScaledDecimal: scale.
%

category: 'Deprecated'
method: Number
asSmallFloat

"SmallFloat is deprecated.
 This method returns a SmallDouble or Float equal to the receiver."

self deprecated: 'asSmallFloat deprecated in v3.0 and later, use asFloat'.
^ self asFloat asSmallFloat
%

category: 'Truncation and Rounding'
method: Number
ceiling

"Returns the integer that is closest to the receiver, on the same side
 of the receiver as positive infinity.  In particular, returns the
 receiver if the receiver is an integer."

(self <= 0.0)
   ifTrue: [^self truncated]
   ifFalse: [^self negated floor negated truncated]
%

category: 'Repository Conversion'
method: Number
convert
"If receiver is a committed object and receiver's class on disk was one of
   ObsSmallFloat, ObsFloat, ObsLargePositiveInteger, ObsLargeNegativeInteger
 the receiver is added to the writeSet of current transaction,
 otherwise has no effect.
 Returns true if receiver was added to the writeSet, false otherwise."

<primitive: 744>
self _primitiveFailed:#convert
%

category: 'Arithmetic'
method: Number
cos

"Returns the cosine of the receiver which is treated as an
 angle expressed in radians."

^ self asFloat cos
%

category: 'Arithmetic'
method: Number
degreesToRadians

"Assuming the receiver represents an angle in degrees, returns the angle
 in radians."

^ self / 57.29577951308232
%

category: 'Accessing'
method: Number
denominator

"(Subclass responsibility.)  Returns the denominator of a Fraction representing
 the receiver."

Number subclassResponsibility: #denominator
%

category: 'Arithmetic'
method: Number
divMod: aNumber into: anArray

"anArray should be an Array of size 2.
 Returns anArray "

 anArray at: 1 put: (self // aNumber) .
 anArray at: 2 put: (self \\ aNumber) .
 ^ anArray
%

category: 'Flow of Control'
method: Number
downTo: aNumber by: stepValue do: aBlock

"Iteratively evaluates the one-argument block aBlock, using the block's single
 argument as the iteration control variable.  Initially, that control variable
 is set to the receiver.
 The argument stepValue must be a Number greater than zero and
 aNumber must be a Number.

 The block is evaluated while the control variable is greater than or equal to
 aNumber.  After each evaluation, the control variable is decremented by
 stepValue.  If the receiver is less than aNumber, the block is not evaluated
 at all.  Returns the receiver."

| j |
j := self .
stepValue > 0 ifFalse:[ stepValue _error: #rtErrArgNotPositive ].
[ j >= aNumber ] whileTrue:[
   aBlock value: j .
   j := j - stepValue
   ]
%

category: 'Flow of Control'
method: Number
downTo: aNumber do: aBlock

"Iteratively evaluates the one-argument block aBlock, using the block's single
 argument as the iteration control variable.  Initially, that control variable
 is set to the receiver.  The block is evaluated while the control variable is
 greater than or equal to aNumber (which must be a kind of Number).  After each
 evaluation, the control variable is decremented by 1.  If the receiver is less
 than aNumber, the block is not evaluated at all.  Returns the receiver."

self downTo: aNumber by: 1 do: aBlock
%

category: 'Testing'
method: Number
even

"Returns true if the receiver is an even integer, false otherwise."

^self \\ 2 = 0
%

category: 'Arithmetic'
method: Number
exp

"Returns e raised to the power of the receiver."

^ self asFloat exp
%

category: 'Truncation and Rounding'
method: Number
floor

"Returns the integer that is closest to the receiver, on the same side
 of the receiver as negative infinity.  In particular, returns the
 receiver if the receiver is an integer."

|result|

result := self truncated.
(self >= 0)
   ifTrue: [^result].
(self = result)
   ifTrue: [^result]
   ifFalse: [^ (result - 1) truncated]
%

category: 'Arithmetic'
method: Number
floorLog: x

"Returns an Integer that is the floor of the receiver's log base x."

^ (self log: x) floor.
%

category: 'Truncation and Rounding'
method: Number
fractionPart
 "Returns the fraction remaining after the receiver is truncated toward zero."
  ^ self - self truncated
%

category: 'Comparing'
method: Number
hash

"Returns a SmallInteger related to the value of the receiver."

"Two Numbers for which = is true, must have the same hash value."

"This method should be reimplemented in Float and SmallInteger."

^ self asFloat hash
%

category: 'Truncation and Rounding'
method: Number
integerPart
  "Returns the integer nearest in value to the receiver,
   in the direction toward zero."
  ^ self truncated
%

category: 'Testing'
method: Number
isExceptionalFloat
"return true if the receiver is not a normal number"

| k |
k := self _getKind .
(k == 1 or:[ k == 4]) ifTrue:[ ^ false ].
^ true
%

category: 'Accessing'
method: Number
kind

"Returns a Symbol from the following list:

   #normal
   #subnormal
   #infinity
   #zero
   #quietNaN
   #signalingNaN

 The symbol tells what kind of floating-point number the receiver is.
 Refer to IEEE standards 754 and 854 for more information.  Numbers that
 are not instances of a floating point class are always normal."

^Number _kindList at: (self _getKind)
%

category: 'Generality'
method: Number
lessGeneralThan: anOperand

"Returns true if the receiver has lower generality than anOperand,
 false otherwise."

^ self _generality < anOperand _generality
%

category: 'Arithmetic'
method: Number
ln

"Returns the natural logarithm of the receiver."

^ self asFloat ln
%

category: 'Arithmetic'
method: Number
log: x

"Returns the base x logarithm of the receiver."

^ self ln / x ln
%

category: 'Generality'
method: Number
moreGeneralThan: anOperand

"Returns true if the receiver has higher generality than anOperand,
 false otherwise."

^ self _generality > anOperand _generality
%

category: 'Arithmetic'
method: Number
negated

"Returns a Number that is the negation of the receiver."

^0 - self
%

category: 'Testing'
method: Number
negative

"Returns true if the receiver is less than zero, false if the receiver is zero
 or greater."

^self < 0
%

category: 'Accessing'
method: Number
numerator

"(Subclass responsibility.)  Returns the numerator of a Fraction representing
 the receiver."

Number subclassResponsibility: #numerator
%

category: 'Testing'
method: Number
odd

"Returns true if the receiver is an odd integer, false otherwise."

^ self even == false
%

category: 'Testing'
method: Number
positive

"Returns true if the receiver is greater than or equal to zero, false if the
 receiver is less than zero."

^self >= 0
%

category: 'Json'
method: Number
printJsonOn: aStream

	aStream nextPutAll: self asFloat asStringLocaleC.
%

category: 'Formatting'
method: Number
printOn: aStream

"Appends a printable representation of the receiver to aStream and returns
 the receiver."

aStream nextPutAll: (self asString).
^ self.
%

category: 'Formatting'
method: Number
printString

"Returns a String whose contents are a displayable representation of the
 receiver."

^self asString
%

category: 'Arithmetic'
method: Number
quo: aNumber

"Divides the receiver by aNumber.  Returns the integer quotient, with
 truncation toward zero.  For example,

 -9 quo: 4 = -2
 -0.9 quo: 0.4 = -2

 The selector rem: returns the remainder from this division."

^ (self / aNumber) truncated
%

category: 'Arithmetic'
method: Number
quoRem: aNumber into: anArray

"anArray should be an Array of size 2.
 Returns anArray "

 anArray at: 1 put: (self quo: aNumber) .
 anArray at: 2 put: (self rem: aNumber) .
 ^ anArray .
%

category: 'Arithmetic'
method: Number
radiansToDegrees

"Assuming the receiver represents an angle in radians, returns the angle
 in degrees."

^ self * 57.29577951308232
%

category: 'Arithmetic'
method: Number
raisedTo: aNumber

"Returns the receiver raised to the power of the argument."

(aNumber _isInteger) ifTrue: [
  ^ self raisedToInteger: aNumber
] ifFalse: [
  ^ self asFloat raisedTo: aNumber
]
%

category: 'Arithmetic'
method: Number
raisedToInteger: aNumber

"Returns the receiver raised to the power of the argument.
 aNumber must be a kind of Integer."

(aNumber _isInteger) ifFalse:[
  aNumber _validateClass: Integer
].
aNumber < 0 ifTrue:[
  ^ 1 / (self _raisedToPositiveInteger: (0 - aNumber))
].
^ self _raisedToPositiveInteger: aNumber
%

category: 'Arithmetic'
method: Number
reciprocal

"Returns 1 divided by the receiver.  Generates an error if the receiver is 0."

(self = 0)
    ifTrue:[^self _errorDivideByZero].  "division by zero error"
^ 1 / self
%

category: 'Arithmetic'
method: Number
rem: aNumber

"Returns the integer remainder defined in terms of quo: (division
 of the receiver by aNumber, with truncation toward zero)."

^ (self - ((self quo: aNumber) * aNumber))
%

category: 'Truncation and Rounding'
method: Number
rounded

"Returns the integer nearest in value to the receiver."

^(self + (self sign / 2)) truncated
%

category: 'Truncation and Rounding'
method: Number
roundedHalfToEven

"Returns the integer nearest in value to the receiver. If the receiver is
exactly halfway between two integers, return the even one."

^self asFraction roundedHalfToEven
%

category: 'Truncation and Rounding'
method: Number
roundedNoFpe

^ self rounded
%

category: 'Truncation and Rounding'
method: Number
roundTo: aNumber

"Returns the multiple of aNumber that is nearest in value to the receiver."

aNumber = 0
  ifTrue: [^0]
  ifFalse: [^(self / aNumber) rounded * aNumber]
%

category: 'Accessing'
method: Number
sign

"Returns 1 if the receiver is greater than zero, -1 if the receiver is less
 than zero, and zero if the receiver is zero."

(self > 0) ifTrue:[ ^ 1 ].
(self < 0) ifTrue:[ ^ -1 ].
^0
%

category: 'Arithmetic'
method: Number
sin

"Returns the sine of the receiver which is treated as an
 angle expressed in radians."

^ self asFloat sin
%

category: 'Arithmetic'
method: Number
sqrt

"Returns the square root of the receiver.  This method currently returns a
 Float.  This will eventually change, so that exact squares of Integers and
 Fractions return Integers and Fractions, for instance."

^ self asFloat sqrt
%

category: 'Arithmetic'
method: Number
squared

"Returns the multiplicative product of the receiver and itself."

^ self * self
%

category: 'Testing'
method: Number
strictlyPositive

"Returns true if the receiver is greater than zero and false if it is less than
 or equal to zero."

^ self > 0
%

category: 'Arithmetic'
method: Number
tan

"Returns the tangent of the receiver which is treated as an
 angle expressed in radians."

^ self asFloat tan
%

category: 'Flow of Control'
method: Number
timesRepeat: aBlock

"(Reserved selector.)  If the receiver is greater than zero, evaluates the
 zero-argument block aBlock the number of times represented by the receiver.
 (If the receiver is zero or negative, aBlock is not executed.).

 A method which sends timesRepeat: will have a step point generated for
 the send of the timesRepeat:  and a step point for the loop index increment
 and test instructions.

 The following is present for use by perform: , not a recursive send."

self timesRepeat: [aBlock value]
%

category: 'Intervals'
method: Number
to: aNumber

"Returns an Interval for the numbers between the receiver and aNumber, where
 each number is the previous number plus 1."

^ Interval from: self to: aNumber.
%

category: 'Intervals'
method: Number
to: aNumber by: stepValue

"Returns an Interval for the numbers between the receiver and aNumber, where
 each number is the previous number plus stepValue."

stepValue = 0 ifTrue: [ArgumentError signal: 'stepValue may not be zero!'].
^ Interval from: self to: aNumber by: stepValue.
%

category: 'Truncation and Rounding'
method: Number
truncated

"(Subclass responsibility.)  Returns the integer nearest in value to
 the receiver, in the direction toward zero."

Number subclassResponsibility: #truncated
%

category: 'Truncation and Rounding'
method: Number
truncatedNoFpe

^ self truncated
%

category: 'Truncation and Rounding'
method: Number
truncateTo: aNumber

"Returns the multiple of aNumber that is closest to the receiver, on
 the same side of the receiver as zero is located.  In particular,
 returns the receiver if the receiver is a multiple of aNumber."

aNumber = 0
  ifTrue: [^0]
  ifFalse: [^(self quo: aNumber) * aNumber]
%

category: 'Storing and Loading'
method: Number
writeTo: passiveObj

"Converts the receiver to its passive form and writes that information on
 passiveObj."

passiveObj nextPutAll: self describe.
passiveObj space
%

category: 'Arithmetic'
method: Number
\\ aNumber

"Returns the modulo remainder defined in terms of //.  Returns a Number with
 the same sign as the argument aNumber.  For example,

 9 \\ 4 = 1
 -9 \\ 4 = 3
 9 \\ -4 = -3
 0.9 \\ 0.4 = 0.1"

^ self - (self // aNumber * aNumber)
%

category: 'Decompiling without Sources'
method: Number
_asSource

""

^ self asString
%

category: 'New Indexing Comparison'
method: Number
_classSortOrdinal

^ 90
%

category: 'Converting'
method: Number
_coerce: aNumber

"(Subclass responsibility.)  Returns aNumber as an instance of the class of the
 receiver, or an instance of any subclass of the receiver.  This method must be
 defined by all subclasses of Number."

Number subclassResponsibility: #_coerce
%

category: 'Flow of Control'
method: Number
_downTo: aNumber by: delta do: aBlock

"(Reserved selector.)
 Iteratively evaluates the one-argument block aBlock, using the block's single
 argument as the iteration control variable.  Initially, that control variable
 is set to the receiver.  The block is evaluated while the control variable is
 greater than or equal to aNumber (which must be a kind of Number).  After each
 evaluation, the control variable is decremented by delta.  If the receiver is less
 than aNumber for a positive delta, the block is not evaluated at all.  Returns the
 receiver.

 The following is present for use by perform: , not a recursive send."

self _downTo: aNumber by: delta do: aBlock
%

category: 'Flow of Control'
method: Number
_downTo: aNumber do: aBlock

"(Reserved selector.)
 Iteratively evaluates the one-argument block aBlock, using the block's single
 argument as the iteration control variable.  Initially, that control variable
 is set to the receiver.  The block is evaluated while the control variable is
 greater than or equal to aNumber (which must be a kind of Number).  After each
 evaluation, the control variable is decremented by 1.  If the receiver is less
 than aNumber, the block is not evaluated at all.  Returns the receiver.

 The following is present for use by perform: , not a recursive send."

self _downTo: aNumber do: aBlock
%

category: 'Error Handling'
method: Number
_errorDivideByZero

"Generates a divide by 0 error."

^ ZeroDivide new _number: 2026 ; reason: 'numErrIntDivisionByZero';
    dividend: self ;
    signal
%

category: 'Converting'
method: Number
_generality

"(Subclass responsibility.)  Returns the integer that represents the ordering
 of the receiver in the generality hierarchy."

" Class           result of _generality
   SmallInteger     20
   Integer	    40
   ScaledDecimal    60
   FixedPoint       65
   Fraction         70
   SmallFloat       80
   SmallDouble      85
   Float            90
   DecimalFloat    100
"

Number subclassResponsibility: #_generality
%

category: 'Accessing'
method: Number
_getKind

"Returns the kind of the receiver based on the list in Number>>kind.  Only floats
 have a kind other than 1."

^ 1
%

category: 'Indexing Support'
method: Number
_idxBasicCanCompareWithClass: aClass
  "Returns true if the receiver may be inserted into a basic BtreeNode whose
   #lastElementClass is <aClass> (see RangeEqualityIndex class>>isBasicClass:)."

  (super _idxBasicCanCompareWithClass: aClass)
    ifTrue: [ ^ true ].
  ^ aClass isSubclassOf: Number
%

category: 'New Indexing Comparison'
method: Number
_idxForCompareEqualTo: arg

""

^arg _idxForCompareEqualToNumber: self
%

category: 'New Indexing Comparison - for Compare'
method: Number
_idxForCompareEqualToNumber: aNumber

"second half of a double dispatch call from Number>>_idxForCompareEqualTo:."

^aNumber = self
%

category: 'New Indexing Comparison'
method: Number
_idxForSortEqualTo: arg

""

^arg _idxForSortEqualToNumber: self
%

category: 'New Indexing Comparison - for Sort'
method: Number
_idxForSortEqualToNumber: aNumber

"second half of a double dispatch call from Number>>_idxForSortEqualTo:."

( self _isNaN and: [ aNumber _isNaN ] ) ifTrue: [ ^ true ].
^ self = aNumber
%

category: 'New Indexing Comparison'
method: Number
_idxForSortGreaterThan: arg

""

^arg _idxForSortNumberGreaterThanSelf: self
%

category: 'New Indexing Comparison'
method: Number
_idxForSortGreaterThanOrEqualTo: arg

""

^arg _idxForSortNumberGreaterThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: Number
_idxForSortLessThan: arg

""

^arg _idxForSortNumberLessThanSelf: self
%

category: 'New Indexing Comparison'
method: Number
_idxForSortLessThanOrEqualTo: arg

""

^arg _idxForSortNumberLessThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: Number
_idxForSortNotEqualTo: arg

""

^arg _idxForSortNotEqualToNumber: self
%

category: 'New Indexing Comparison - for Sort'
method: Number
_idxForSortNotEqualToNumber: aNumber

"second half of a double dispatch call from Number>>_idxForSortNotEqualTo:."

( self _isNaN and: [ aNumber _isNaN ] ) ifTrue: [ ^ false ].
^ self ~= aNumber
%

category: 'New Indexing Comparison - for Sort'
method: Number
_idxForSortNumberGreaterThanOrEqualToSelf: aNumber

"second half of a double dispatch call from Number>>_idxForSortGreaterThanOrEqualTo:. Note that aNumber should be the receiver in any >= comparison"

( self _isNaN and: [ aNumber _isNaN ] ) ifTrue: [ ^ true ].
aNumber _isNaN ifTrue: [ ^ true ].
self _isNaN ifTrue: [ ^ false ].
^aNumber >= self
%

category: 'New Indexing Comparison - for Sort'
method: Number
_idxForSortNumberGreaterThanSelf: aNumber

"second half of a double dispatch call from Number>>_idxForSortGreaterThan:. Note that aNumber should be the receiver in any > comparison"

( self _isNaN and: [ aNumber _isNaN ] ) ifTrue: [ ^ false ].
aNumber _isNaN ifTrue: [ ^ true ].
self _isNaN ifTrue: [ ^ false ].
^aNumber > self
%

category: 'New Indexing Comparison - for Sort'
method: Number
_idxForSortNumberLessThanOrEqualToSelf: aNumber

"second half of a double dispatch call from Number>>_idxForSortLessThanOrEqualTo:. Note that aNumber should be the receiver in any <= comparison"

( self _isNaN and: [ aNumber _isNaN ] ) ifTrue: [ ^ true ].
aNumber _isNaN ifTrue: [ ^ false ].
self _isNaN ifTrue: [ ^ true ].
^aNumber <= self
%

category: 'New Indexing Comparison - for Sort'
method: Number
_idxForSortNumberLessThanSelf: aNumber

"second half of a double dispatch call from Number>>_idxForSortLessThan:. Note that aNumber should be the receiver in any < comparison"

( self _isNaN and: [ aNumber _isNaN ] ) ifTrue: [ ^ false ].
aNumber _isNaN ifTrue: [ ^ false ].
self _isNaN ifTrue: [ ^ true ].
^aNumber < self
%

category: 'Indexing Support'
method: Number
_idxOptimizedCompareWithClass: aClass
  "Returns true if the receiver may be inserted into a BtreePlusNode whose
   #lastElementClass is <aClass> and whose comparisons are optimized."

  "Sent when index option optimizeComparison is true and aClass responds true to #_idxCanOptimizeComparison"

  "Optimized comparisons allowed for Numbers that are not NaN"

  ^ (self _idxBasicCanCompareWithClass: aClass) and: [ self _isNaN not]
%

category: 'Indexing Support'
method: Number
_isNaN

"Returns whether the receiver is quiet NaN or signaling NaN.
 This method is only to be used by the indexing subsystem."

^ false
%

category: 'Private'
method: Number
_nonZeroGte: aNumber

"Sent by code emitted for to:by:do: , if the
 bytecode for _nonZeroGte fails .
 Signals an ArgumentError if self = 0 ,
 otherwise returns self >= aNumber "

0 = self ifTrue:[
   ArgumentError new
    object: self ;
    signal:'argument to by: of to:by:do: must be nonzero'
].
^ self >= aNumber
%

category: 'Private'
method: Number
_raisedToPositiveInteger: aNumber
  "Sender must ensure aNumber is an Integer >= 0"

aNumber <= 1 ifTrue:[
  aNumber = 0 ifTrue:[ ^ 1 ]
              ifFalse:[ ^ self ].
].
aNumber _bottomBit == 1 ifTrue:[  "aNumber is odd"
  ^ self * (self _raisedToPositiveInteger: aNumber - 1 )
].
^ (self * self) _raisedToPositiveInteger: (aNumber bitShift: -1)
%

category: 'Converting'
method: Number
_retry: aSelector coercing: aNumber

"A difference in representation between the receiver and aNumber has prevented
 aSelector from being executed.  Coerce either the receiver or aNumber to the
 higher of their generalities and attempt to execute the method represented by
 aSelector again.

 This is the basis of the type coercion logic for all Numbers."
| gen argGen |
" numeric primitives #=, #~= check for   (anArg isKindOf: Number)
  and if that result is  false ,   the compare result is returned
  directly by those primitives
"
gen := self _generality .
argGen := aNumber _generality .
(gen < argGen ) ifTrue: [
  ^ (aNumber _coerce: self) perform: aSelector with: aNumber
].
(gen > argGen ) ifTrue: [
  ^self perform: aSelector with: (self _coerce: aNumber)
].
self _uncontinuableError.  "coercion failed"
%

category: 'Converting'
method: Number
_retry: aSelector coercing: aNumber into: anArray

"A difference in representation between the receiver and aNumber has prevented
 aSelector from being executed.  Coerce either the receiver or aNumber to the
 higher of their generalities and attempt to execute the method represented by
 aSelector again.

 This is the basis of the type coercion logic for all Numbers."
| gen argGen |
gen := self _generality .
argGen := aNumber _generality .
(gen < argGen ) ifTrue: [
  ^ (aNumber _coerce: self) perform: aSelector with: aNumber with: anArray
].
(gen > argGen ) ifTrue: [
  ^self perform: aSelector with: (self _coerce: aNumber) with: anArray
].
self _uncontinuableError.  "coercion failed"
%

category: 'Deprecated'
method: Number
_sqrt

self deprecated: '_sqrt obsolete,  Use #sqrt instead.'.
^ self sqrt
%

! Class extensions for 'NumericError'

!		Instance methods for 'NumericError'

removeallmethods NumericError
removeallclassmethods NumericError

category: 'Instance initialization'
method: NumericError
initialize
  gsNumber := ERR_NumericError.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'Object'

!		Class methods for 'Object'

removeallmethods Object
removeallclassmethods Object

category: 'Encoded OOPs'
classmethod: Object
getNewOopFromOldOop: anInteger

"Convert the specified old OOP in Gs64 v1.1 OOP format to
 an OOP in Gs64 v2.0 format.  For OOPs representing objects other
 than SmallIntegers or SmallDoubles the result will always be a SmallInteger;
 otherwise it is possible for the result to be a LargeInteger.
 The argument may be a positive SmallInteger or LargeInteger "

^self _getOopOrObjectFromOldOop: anInteger opCode: 1
%

category: 'Encoded OOPs'
classmethod: Object
getNewOopWithLow: anInt high: anotherInt

"Convert the specified double encoded OOP into an Integer representing
 the OOP of the object.  The object with the return OOP is not verified
 to exist."

^self _getOopWithOldLow: anInt high: anotherInt opCode: 1
%

category: 'Encoded OOPs'
classmethod: Object
getObjectFromOldOop: anInteger

"Convert the specified old OOP in Gs64 v1.1 OOP format to
 a Gs64 v2.0 object identifier. Return  the object represented by
 the new object identifier , or nil if it does not exist."

^self _getOopOrObjectFromOldOop: anInteger opCode: 0
%

category: 'Encoded OOPs'
classmethod: Object
getObjectWithOldLow: anInt high: anotherInt

"Convert the specified Gs64 v1.1 double encoded OOP into the object represented
 by that OOP.  nil is returned if the object does not exist."

^self _getOopWithOldLow: anInt high: anotherInt opCode: 0
%

category: 'Encoded OOPs'
classmethod: Object
getOldOopWithOldLow: anInt high: anotherInt

"Returns a SmallInteger representing the Old OOP
value of the specified double encoded OOP.  No checking is done to
ensure the object with OOP returned actually exists.  Answer nil if the
arguments are invalid or are not instances of SmallInteger."

(anInt < 0 or:[ anInt > 65535]) ifTrue:[ ^ nil ].
(anotherInt < 0 or:[ anotherInt > 65535]) ifTrue:[ ^ nil ].
^ (anotherInt bitShift:16)  bitOr: anInt .
%

category: 'Storing and Loading'
classmethod: Object
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object with named instance
 variable format.  Converts the object to its active form by loading the
 information into a new instance of the receiver.  Returns the new instance."

| inst |
"create a new instance and fill it full of information contained
 in the passiveObj"
self isVariable ifTrue: [
  inst := self _basicNew: passiveObj readSize.
  ]
ifFalse: [
  inst := self _basicNew
  ].
inst loadFrom: passiveObj.
^inst
%

category: 'Debugging'
classmethod: Object
objectForOop:  aPositiveInt

" Return the object with the given oop, or an error if no such object
  exists.  The oop may specify a special object.

  There are risks associated with oop fishing, particularly when using
  oops across transaction boundaries and between sessions. The numeric
  oop is not a reference, and GC of otherwise unreferenced objects means
  that the object previously corresponding to a particular oop may no
  longer exist or be a different object than expected.

  This method is provided for convenience for debugging and for experienced
  users who understand the internal behavior and the risks involved."

| result |

result := self _objectForOop: aPositiveInt.
result ifNil:[
  (aPositiveInt == 20"nil asOop" ) ifTrue: [ ^ nil ].
   ^ aPositiveInt _error: #rtErrInvalidArgument 
].
^ result
%

category: 'Debugging'
classmethod: Object
_classOf: anObject

"return    anObject class  "
<primitive: 464>  "equivalent to primitive: 610, needed by ProtoObject"

self _primitiveFailed: #_classOf: args: { anObject }
%

category: 'Encoded OOPs'
classmethod: Object
_getOopOrObjectFromOldOop: anInt opCode: anOpCode

"Convert the specified old (Gs64 v1.1 OOP format) OOP to either
 an object or the new (Gs64 v2.0 OOP format) OOP, depending
 on the opCode.  An opCode of zero means return the
 object if it exists or nil if it does not.  A non-zero opCode
 means return a positive Integer representing the new OOP."
<primitive: 539>

self _primitiveFailed: #_getOopOrObjectFromOldOop:opCode:
     args: { anInt . anOpCode }
%

category: 'Encoded OOPs'
classmethod: Object
_getOopWithOldLow: anInt high: anotherInt opCode: anOpCode

"Convert the specified Gs64 v1.1 double encoded OOP into either an object
 or an Integer representing the Gs64 v2.0 OOP of the object.
 If the opCode is zero, then the object is returned if it
 exists.  nil is returned if the object does not exist.
 If the opCode is not zero, then a SmallInteger representing the OOP of the object is
 returned.  The object with this OOP is not verified to exist."

<primitive: 537>
anInt _validateClass: SmallInteger.
anotherInt _validateClass: SmallInteger.
anOpCode _validateClass: SmallInteger.
(anInt > 65535) ifTrue:[self _errorIndexOutOfRange: anInt].
(anotherInt > 65535) ifTrue:[self _errorIndexOutOfRange: anotherInt].
self _primitiveFailed: #_getOopWithOldLow:high:opcode:
     args: { anInt . anotherInt . anOpCode }
%

category: 'Private'
classmethod: Object
_objectForOop: aPositiveInt

" return the object with the given objectId, or nil if no such
  object exists.  The objectId may specify a special object.

  The argument must be >= 0 and must be either
  a SmallInteger or a LargeInteger, otherwise the primitive
  will fail."

<primitive: 38>
(aPositiveInt isKindOf: Integer)
    ifFalse: [ ^ aPositiveInt _error: #rtErrBadArgKind args: { Integer }].
(aPositiveInt < 0)
    ifTrue: [ ^ aPositiveInt _error: #rtErrArgNotPositive ].
^ self _primitiveFailed: #_objectForOop: args: { aPositiveInt }
%

category: 'Private'
classmethod: Object
_oop11toOop20: a11oop

"Given the Integer value of a objectId from Gemstone64 v1.1,
 return the value of the Gemstone v2.0 objectId .  The objectId
 is assumed must be of a Pom object, not a special object."

(a11oop bitAnd: 1) == 0 ifTrue:[ nil error:'argument is not a Pom oop'].
^  ((a11oop bitShift:-1) bitShift:8 ) bitOr: 1
%

category: 'Private'
classmethod: Object
_oop20toOop11: a20oop

"Given the Integer value of a objectId from Gemstone64 v2.0,
 return the value of the Gemstone v1.1 objectId .  The objectId
 is assumed must be of a Pom object, not a special object."

(a20oop bitAnd: 1) == 0 ifTrue:[ nil error:'argument is not a Pom oop'].
^  ((a20oop bitShift:-8) bitShift:1 ) bitOr: 1
%

!		Instance methods for 'Object'

category: 'Converting'
method: Object
-> anObject

"Returns an Association with the receiver as the key and the given object as
 the value."

^Association newWithKey: self value: anObject
%

category: 'Comparing'
method: Object
= anObject

"Returns true if the receiver and the argument have the same value.
 This method is defined here for identity, and is commonly
 reimplemented in a subclass to check for equality.

 Ordinarily, a class which reimplements the = method should also
 reimplement hash."

^self == anObject "default is identity check, unless overridden to"
                  "check for equality"
%

category: 'Comparing'
method: Object
== anObject

"(Optimized selector.)  Returns true if the receiver and the argument are the
 same object.

 This selector is optimized by the compiler and may not be reimplemented
 in any subclass.  This implementation is so that perform will work."

^ self == anObject
%

category: 'Enumerating'
method: Object
accompaniedBy: anObj do: aBlock

"Evaluate the two argument block aBlock with anObj and each element of
  the receiver.  Return the receiver. "

self do:[ :elem | aBlock value: anObj value: elem ]
%

category: 'Indexing Support'
method: Object
addObjectToBtreesWithValues: anArray

"Add the receiver to the B-trees of any equality indexes that it participates
 in, using the given Array of index object/value pairs."

^ self _errorsDuringPreventCommit:[ self _addObjectToBtreesWithValues: anArray ]
%

category: 'Deprecated'
method: Object
asciiLessThan: anObject

"Deprecated. For objects that are not Characters or CharacterCollections, returns
 the result of an ordinary less-than compare.  This method is reimplemented in
 Character, String and MultiByteString to provide comparison based on the ASCII
 collating order."

self deprecated: 'Object>>asciiLessThan: deprecated in 3.2.'.
^ self < anObject
%

category: 'Converting'
method: Object
asOop

"Returns the value of the receiver's object-oriented pointer (OOP) as a
 positive Integer .  This is the receiver's unique identifier that
 distinguishes it from all other objects.  For non-special objects
 and for instances of 
   Boolean Character SmallDate SmallDateAndTime 
   SmallFraction SmallScaledDecimal SmallTime UndefinedObject
 the result will always be a SmallInteger.  For instances of SmallInteger
 and SmallDouble the result may be a LargeInteger.

 The result is different from the result obtained in Gemstone64 v1.x"

<primitive: 86>
self _primitiveFailed: #asOop
%

category: 'Encoded OOPs'
method: Object
asOopNumber

"If the receiver is a non-special object, return the receiver's
 oopNumber as defined under OOP_TAG_POM_OOP in the file
 $GEMSTONE/include/gcioop.ht  , otherwise return nil . "

| oop |
oop := self asOop .
oop _isSmallInteger ifFalse:[ ^ nil ].
(oop bitAnd:7) == 1 ifFalse:[ ^ nil ].
^ oop bitShift: -8
%

category: 'Updating'
method: Object
assignToObjectSecurityPolicyId: anObjectSecurityPolicyId

"Assign the receiver to objectSecurityPolicy specified by anObjectSecurityPolicyId which should
 be a SmallInteger >= 0.  Does not affect components of the receiver. "

| seg |
anObjectSecurityPolicyId > 0 ifTrue:[ seg := SystemRepository at: anObjectSecurityPolicyId ]
	       ifFalse:[ "assign to nil , world Write" ].
^ self objectSecurityPolicy: seg .
%

category: 'Deprecated'
method: Object
assignToSegment: anObjectSecurityPolicy

self deprecated: 'Object>>assignToSegment: deprecated v3.0. Use #objectSecurityPolicy: instead.'.
^ self objectSecurityPolicy: anObjectSecurityPolicy
%

category: 'Formatting'
method: Object
asString

"Returns a String that indicates the class of the receiver; for
 example, if the receiver is an instance of class Monkey, it returns
 a String of the form 'aMonkey'.  This method is often
 overridden in subclasses to provide behavior tailored to the class.

 The result should not contain any formatting information.
 For example the following expression should evaluate to true:

   #abc asString = String withAll: 'abc'

 This method is used by Object | describe.  Thus GemStone error
 handling and Topaz are dependent upon this method being functional.

 Must conform to rules for reimplementation of Object | describe."

^ self class name describeClassName .
%

category: 'Formatting'
method: Object
asUnicodeString

^ Unicode7 withAll: self asString
%

category: 'Accessing'
method: Object
at: anIndex

"Returns the value of an indexed variable in the receiver.
 The argument anIndex must not be larger than the size of the
 receiver, and must not be less than 1.

 Generates an error if anIndex is not a SmallInteger or is out of
 bounds, or if the receiver is not indexable."

<primitive: 32>

(self class isIndexable) "not an indexable object"
  ifFalse: [^ self _errorNotIndexable].
(anIndex _isInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #at: args: { anIndex } .
self _uncontinuableError
%

category: 'Updating'
method: Object
at: anIndex put: aValue

"Stores the argument aValue in the indexed variable of the
 receiver indicated by anIndex.  The argument anIndex must not be
 larger than 1 + the size of the receiver, and must not be less than 1.

 Returns aValue.

 Generates an error if anIndex is not a SmallInteger or is out of
 bounds, if the receiver is not indexable, or if the
 receiver is not of the right class to store the given value.

 The primitive is equivalent to GciStoreIdxOop or GciStoreByte,
 depending on implementation of the receiver."

<primitive: 268>

(anIndex _isInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
(self class isIndexable) "not an indexable object"
  ifFalse: [ ^ self _errorNotIndexable].
((anIndex < 1) | (anIndex > (self size + 1))) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].
(self class isBytes)
ifTrue: [ ((aValue class ~~ SmallInteger) or: [ (aValue < 0) | (aValue > 255) ])
    ifTrue: [^ aValue _error: #rtErrExpectedByteValue].
  ].

self _primitiveFailed: #at:put: args: { anIndex . aValue } .
self _uncontinuableError
%

category: 'Accessing'
method: Object
basicAt: anIndex

"Returns the object at the given location in the receiver.  Subclasses
 should not reimplement this method."

^self _basicAt: anIndex
%

category: 'Updating'
method: Object
basicAt: anIndex put: aValue

"Puts the given object into the given location in the receiver.  Subclasses
 should not reimplement this method."

^self _basicAt: anIndex put: aValue
%

category: 'Comparing'
method: Object
basicIdentityHash

"This method returns some Integer related to the identity of the receiver.  If
 two objects compare identically (==) to each other, the results of sending
 basicIdentityHash to each of those objects is equal."

"Faster than identityHash.  Used in GsMethodDictionary ."


<primitive: 321>

self _primitiveFailed: #basicIdentityHash .
self _uncontinuableError
%

category: 'Storing and Loading'
method: Object
basicLoadFrom: passiveObj

"Reads from passiveObj the passive form of an object with named instance
 variable format.  Converts the object to its active form by loading the
 information into the receiver."

passiveObj hasRead: self.
(passiveObj readNamedIV) ifFalse: [
  "may be an old format, with assumed instance variable positions"
  ^self basicLoadFromOld: passiveObj
  ].
self loadNamedIVsFrom: passiveObj.
self class isVariable ifTrue: [
  self loadVaryingFrom: passiveObj
].
%

category: 'Storing and Loading'
method: Object
basicLoadFrom: passiveObj size: varyingSize

"Read the structure from the given passiveObj, with named instance variable
 format.  This is similar to basicLoadFrom:, but is used for objects whose size
 can not be preallocated at instantiation time (such as a Set)."

passiveObj hasRead: self.
self basicLoadFromNoRead: passiveObj size: varyingSize
%

category: 'Storing and Loading'
method: Object
basicLoadFromNoRead: passiveObj size: varyingSize

"Private."

(passiveObj readNamedIV) ifFalse: [
  "may be an old format, with assumed instance variable positions"
  ^self basicLoadFromOld: passiveObj
].
self loadNamedIVsFrom: passiveObj.
self class isVariable ifTrue: [
  self loadVaryingFrom: passiveObj size: varyingSize
].
%

category: 'Storing and Loading'
method: Object
basicLoadFromOld: passiveObj

"Read my structure from the given passiveObj."

self class firstPublicInstVar to: self namedSize do: [:i |
  self instVarAt: i put: passiveObj readObject
].

self class isVariable ifTrue: [
  self loadVaryingFrom: passiveObj
].
%

category: 'Accessing'
method: Object
basicPhysicalSize

"Returns the number of bytes required to represent the receiver in memory.  If
 the receiver is in special format (which implies that its representation is the
 same as its OOP), returns zero.

 The basicPhysicalSize method returns the same result as the default
 implementation (in class Object) of the physicalSize method.  It makes that
 default implementation available even when the physicalSize method is
 reimplemented in a subclass.  The basicPhysicalSize method should not itself
 be reimplemented."

<primitive: 364>
self _primitiveFailed: #basicPhysicalSize
%

category: 'Accessing'
method: Object
basicSize

"Returns the number of named instance variables plus the number of indexed
 instance variables in the receiver.  This result is equivalent to
 GciFetchSize(self).

 This method is implemented as a primitive, for improved performance."

<primitive: 88>

self _primitiveFailed: #basicSize .
self _uncontinuableError
%

category: 'Storing and Loading'
method: Object
basicWriteTo: passiveObj

"Converts the receiver to its passive form and writes that information on
 passiveObj."

| s variable cls c |
  cls := self class.
  passiveObj writeClass: cls.

  variable := cls isVariable.
  variable ifTrue: [ passiveObj writeSize: (s := self _basicSize) ].

  passiveObj writeNamedIvsFrom: self class: cls .
  passiveObj endNamedInstVars.

  variable ifTrue: [
    c := 0.
    1 to: s do: [:i |
      passiveObj writeObject: (self _at: i).
      c := c + 1.
      c > 99 ifTrue: [
        passiveObj lf.
        c := 0.
      ].
    ]
  ].
  passiveObj cr
%

category: 'Updating'
method: Object
become: anObject

"Swaps the identities of the receiver and the argument.

 Intended only for experienced GemStone Smalltalk programmers who need to
 migrate instances of one class to another.

 The sender is responsible for checking the consistency of the class histories
 of the argument and the receiver.  This method makes no such checks.

 The argument, the receiver, or both are permitted to be invariant.

 Neither the argument nor the receiver may be special objects (instances of
 classes such as SmallInteger, Character, or Boolean).  Also, neither may be
 instances of a class that is a kind of
   Behavior, BlockClosure, CByteArray, CFunction, CLibrary, CPointer,
   GsCompilerIRNode, GsFile, GsNMethod,, GsProcess, GsSocket, Regexp,
   VariableContext
 Neither the argument nor the receiver may be a kind of Bag that has indexes
 built on it.  If either the receiver or the argument (or both) participate in
 an index, then either both must be in byte format or neither must be in byte
 format.  That is, one cannot be in byte format if the other is not also.  To
 avoid the error conditions triggered by presence of indexes, remove the
 indexes from the relevant NSCs prior to invoking this method.

 Neither the argument nor the receiver may exist as self below the sender of a
 become: message on the active GemStone Smalltalk stack.

 Any clusterIds that belong to an object on disk remain with the object.
 That is, the clusterIds do not follow the identities when they are swapped.

 The ObjectSecurityPolicies of the objects do not follow the identities
 when they are swapped.

 As of Gs64 v3.0 ,  tags are no longer swapped between the objects,
 they are treated same as instance variables. "

<primitive: 2001>  "enter protected mode"
| prot |
prot := System _protectedMode .
[
  | refsToRcvr refsToObj rcvrInIndex objInIndex dlForRcvr dlForObj newSelf
    myCls argCls |
  self == anObject ifTrue: [ ^ self ].

  (self _onStackAsSelfBelow: 5) ifTrue:[ self _error: #rtErrCantBecomeSelfOnStack ].
  (anObject _onStackAsSelfBelow: 5) ifTrue:[ anObject _error: #rtErrCantBecomeSelfOnStack].

  (myCls := self class) _instancesNpDbtransient ==
  (argCls := anObject class) _instancesNpDbtransient ifFalse:[
    myCls instancesNonPersistent == argCls instancesNonPersistent ifFalse:[
      self _error: #rtErrCantBecomeGeneric args: { 'instancesNonPersistent' }.
    ].
    myCls instancesDbTransient == argCls instancesDbTransient ifFalse:[
      self _error: #rtErrCantBecomeGeneric args: { 'instancesDbTransient' }.
    ]
  ].
  " get index participants that reference the receiver "
  refsToRcvr := self _getIndexReferencesInto: { } .

  " get index participants that reference anObject "
  refsToObj := anObject _getIndexReferencesInto: { } .

  (refsToRcvr size + refsToObj size) ~~ 0 ifTrue:[
    refsToRcvr size ~~ 0 ifTrue:[
      rcvrInIndex := (refsToRcvr detect:[ :o | o _isIndexObject ] ifNone: [ nil ])
    ].
    refsToObj size ~~ 0 ifTrue:[
      objInIndex := (refsToObj detect:[ :o | o _isIndexObject ] ifNone: [ nil ])
    ].
    " if both participate in an index, check their formats "
    (rcvrInIndex ~~ nil or: [ objInIndex ~~ nil ]) ifTrue:[
	(( myCls isBytes and: [ argCls isBytes not ] ) or:
	[ myCls isBytes not and: [ argCls isBytes ] ] ) ifTrue: [
          ^ ArgumentError new reason: #rtErrCantBecomeOneIdx ; _number: 2119; details:
'Become not allowed, one object participates in an index and the other has different format';
           signal
	]
    ].
    UnorderedCollection _disableRcIndexLogging.
    self _removeIndexParticipation: refsToRcvr for: anObject.
    anObject _removeIndexParticipation: refsToObj for: self.
    "Make sure Dependency lists are gone or the primitive will fail"
    (dlForObj := DependencyList for: anObject) == nil
	  ifFalse:[DependencyList set: nil for: anObject.].
    (dlForRcvr := DependencyList for: self ) == nil
	  ifFalse:[DependencyList set: nil for: self.].

    self _primitiveBecome: anObject forDict: false .

    " remember, the variable 'anObject' and 'self' have switched identities "

    "Restore the DependencyLists.  Filter out empty ones, which are invalid anyway"
    (dlForObj size ~~ 0)
	  ifTrue:[DependencyList set: dlForObj for: anObject.].
    (dlForRcvr size ~~ 0)
	  ifTrue: [DependencyList set: dlForRcvr for: self.].

    newSelf := self .   "not safe to send to self in this method after the become"
    newSelf _restoreIndexParticipation: refsToRcvr.
    anObject _restoreIndexParticipation: refsToObj.

    UnorderedCollection _enableRcIndexLogging.
  ] ifFalse:[   "no index participation"
    self _primitiveBecome: anObject forDict: false .
  ]
] ensure:[
  prot _leaveProtectedMode
].
%

category: 'Finalization'
method: Object
beEphemeron: aBoolean

"Sets or clears receiver's isEphemeron bit and returns previous state of the bit.
 If aBoolean is false, clears isEphemeron bit if it was set, otherwise has
 no effect.
 If aBoolean is true, an ArgumentError is signaled and the object is not changed if
 any of the following are true:
   receiver is a byte format object
   receiver is an Nsc format object
   receiver has no instVars  (self basicSize == 0)
   receiver is a large object (self basicSize > 2034)
   receiver is a committed object
   receiver is a kind of VariableContext, GsNMethod, GsProcess, ExecBlock,
	Behavior, or SoftReference .
 If an object has the isEphemeron bit set,
   an error is generated if a size change would convert it to a large object;

   the isEphemeron bit is silently cleared if an attempt is made to
   commit the object, the isEphemeron bit remains cleared if the commit
   fails due to concurrency conflicts.

   the isEphemeron bit will silently have no effect after the basicSize
   of the object is changed to zero .

   the isEphemeron bit will silently have no effect if the first instVar
   references a special or committed object.

 See Object >> mourn  for documentation of finalization behavior. "

<primitive: 827>
aBoolean _validateClass: Boolean .
self _primitiveFailed: #beEphemeron: args: { aBoolean }
%

category: 'Updating'
method: Object
beTranlogged

"For a temporary object, ensures that if it is first added to the closure
 of an attempted commit by reference from a not-tranlogged object,
 the temporary object will be committed as a tranlogged object.
 Has no effect on special objects, or committed objects which are
 tranlogged.
 Signals an error if the receiver is committed and would answer false to
 isTranlogged , or is part of the closure of a failed commit and
 was made reachable by a not-tranlogged object.
 Returns the receiver."

<primitive: 1020>
^ self _primitiveFailed: #beTranlogged
%

category: 'Testing'
method: Object
canBeWritten

"Returns true if the current user has write authorization for the
 receiver, false if not."

^ ((self _status: false)  bitAnd: 16r10) ~~ 0
%

category: 'Class Membership'
method: Object
changeClassTo: aClass

"Redefines the class of the receiver to be aClass.  For this method to execute
 successfully, all of the following conditions must be true:

 1.  The receiver's class must have the same implementation as aClass
     (byte array, pointer array, or non-sequenceable collection).
 2.  If aClass is a kind of IdentitySet, then the class of the receiver
     must also be a kind of IdentitySet.  (This method cannot be used to
     change an IdentityBag to an IdentitySet.)
 3.  The argument aClass must not be a kernel class for which instance creation
     is disallowed.
 4.  The argument aClass must not be a GsMethod, GsMethodDictionary,
     SymbolDictionary, or SymbolList.
 5.  The receiver cannot as self below the sender of a
     changeClassTo: message on the active GemStone Smalltalk stack.
 6.  If the receiver has more instvars than the named instVars defined by
     aClass,  aClass must support varying instvars , and some of the instvars
     in the receiver will become varying instvars .
     If aClass defines more named instvars than the receiver, some of the
     receiver's varying instvars, if any, will become named instvars .
 7.  If aClass defines more named instvars than the total instvars in the
     receiver, extra instvars are initialized to nil .

 Generates an error if any of these conditions is not true."

^ self _changeClassTo: aClass preserveVarying: false
%

category: 'Deprecated'
method: Object
changeToSegment: anObjectSecurityPolicy

self deprecated: 'Object>>changeToSegment: deprecated v3.0. Use #objectSecurityPolicy: instead.'.
^ self objectSecurityPolicy: anObjectSecurityPolicy
%

category: 'Class Membership'
method: Object
class

"Returns the object that is the receiver's class."

<primitive: 610 >
^ self _primitiveFailed: #class .
%

category: 'Clustering'
method: Object
cluster

"This method clusters an object using the current default ClusterBucket.  It
 does not force an object to disk.  Rather an object not yet on disk
 is tagged so that the object remembers what cluster bucket is supposed to be
 used at such time as the object actually goes disk.  If the object is large,
 then all nodes of the object are clustered into the same bucket.

 If the object is a kind of UnorderedCollection with indexes, this method does
 not cluster indexes.  To cluster indexing objects, use the clusterIndexes
 method.  Alternatively, see the UnorderedCollection | clusterDepthFirst method.

 No action is taken to cluster objects referenced by user-defined tags.

 Has no effect and returns true if the receiver was previously clustered
 in the current transaction or if receiver is a GsNMethod loaded for execution; 
 otherwise returns false after clustering the receiver. "

^ self clusterInBucket: System currentClusterId
%

category: 'Clustering'
method: Object
clusterBucket

"Returns the cluster bucket for the object.  Returns nil if the object is a
 temporary object that is neither on disk nor previously clustered."

<primitive: 90>
self _primitiveFailed: #clusterBucket .
self _uncontinuableError
%

category: 'Clustering'
method: Object
clusterDepthFirst

"This method clusters the receiver and its instance variables in depth-first
 order.  Clustering is performed on all of the objects that can be reached via a
 transitive traversal starting at the receiver.  If an object is referenced by
 more than one clustering operation during the current transaction (that is,
 since the last commit or abort), it is located nearest the first reference.

 This routine assumes that if we need to cluster inside this object that the
 object contains only named instance variables or indexed instance variables.
 It cannot have unordered instance variables since that would have to be a kind
 of Bag and this method is overridden in Collection.

 Note that this implementation does not include clustering of any user-defined
 tags.

 After clustering, returns true if the receiver is a byte object,
 otherwise returns false.

 Has no effect and returns true if the receiver was previously clustered
 in the current transaction, or if the receiver is a special object."

self cluster
  ifTrue:
    [ ^ true ]
  ifFalse:
    [ 1 to: self namedSize do:
      [ :i | (self instVarAt: i) clusterDepthFirst ].
      1 to: self _basicSize do: [ :i | (self at: i) clusterDepthFirst ].
      ^ false
    ]
%

category: 'Clustering'
method: Object
clusterInBucket: aClusterBucketOrId

"This method does not force an object to disk.  Rather an object not already on
 disk is tagged so that the object remembers what cluster bucket is supposed to
 be used at such time as the object actually goes disk.

 If the object is large, then all nodes of the object are clustered
 into the same bucket.

 If the object is a kind of UnorderedCollection with indexes, this method does
 not cluster indexes.  To cluster indexing objects, use the clusterIndexes
 method.  Alternatively, see the UnorderedCollection | clusterDepthFirst method.

 No action is taken to cluster objects referenced by user-defined tags.

 If the receiver was previously clustered in the current transaction
 or is a GsNMethod loaded for execution, has no effect and returns true;

 else if the receiver is a special object has no effect and returns true;

 else if current session does not have write authorization to receiver,
 and receiver is in SystemObjectSecurityPolicy or
 DataCuratorObjectSecurityPolicy, has no effect and returns true;

 else if current session does not have write authorization to receiver,
 signals an Error ;

 else if the receiver is a byte object and was not previously clustered
 in the current transaction  , clusters the receiver and returns true ;

 else if the receiver was not previously clustered in the current transaction,
 clusters the receiver and returns false. "

<primitive: 279>
self _primitiveFailed: #clusterInBucket: args: { aClusterBucketOrId } .
self _uncontinuableError
%

category: 'Storing and Loading'
method: Object
containsIdentity

"Private."

^false
%

category: 'Copying'
method: Object
copy

"Returns a copy of the receiver which shares the receiver's instance
 variables.

 Gs64 v3.0 change:
   Dynamic instVars (i.e. the Tags) of the receiver ARE copied.
 See primitive 55 for Gs64 v2.x behavior."

^self shallowCopy postCopy
%

category: 'Instance Migration'
method: Object
copyDynamicInstVars: aList from: anObject
  "For dynamic instVar name/value pairs from anObject,
   if the name of the dynamic instVar is an element of anIdentitySet,
   add that name/value pair to the receiver.
   aList should be an  Array or IdentitySet of Symbols. "
  | pairs |
  pairs := anObject dynamicInstVarPairs .
  1 to: pairs size by: 2 do:[:n | | aName |
    aName := pairs at: n .
    (aList includesIdentical: aName) ifTrue:[
      self dynamicInstVarAt: aName put: (pairs at: n+1)
    ].
  ].
%

category: 'Instance Migration'
method: Object
copyDynamicInstVarsFrom: anObject
  "Copy all dynamic instVar name/value pairs from anObject to the receiver."
  | pairs |
  pairs := anObject dynamicInstVarPairs .
  1 to: pairs size by: 2 do:[:n |
    self dynamicInstVarAt: (pairs at: n) put: (pairs at: n+1)
  ].
%

category: 'Copying'
method: Object
deepCopy

"Returns a deep copy of the receiver.  That is, if the receiver is a collection
 or a complex object, the copy has copies of the original's elements and
 any component parts.  Those copies are also deep copies.  In other words,
 deep copying is recursive.

                                 CAUTION:

 Use this method with care.  It makes copies of all objects that can be reached
 from the receiver, which in some cases could be very large."

| copiedObjDict copy |

copiedObjDict := IdentityKeyValueDictionary new.
copy := self _deepCopyWith: copiedObjDict.
^copy.
%

category: 'Deprecated Notification'
method: Object
deprecated
  "Methods that send this message are considered deprecated. Deprecated methods
   are supported for the current version, but may be removed in a future release."

  self deprecated: nil.
%

category: 'Deprecated Notification'
method: Object
deprecated: aString
  "Methods that send this message are considered deprecated, with <aString> providing
   details on the deprecation.  <aString> should include the class and method name,
   the version in which the method is deprecated, and what to do instead.
   Deprecated methods are supported for the current version, but may be removed in
   a future release."

  DeprecationEnabled ifNotNil:[ ^ Deprecated signal: aString ].
  ^ self
%

category: 'Deprecated Notification'
method: Object
deprecatedNotification: aString
  "Methods that send this message are considered deprecated, with <aString> providing
   details on the deprecation.  See Object>>deprecated: for details."

  DeprecationEnabled ifNil:[ ^ self "ignore" ].
  Deprecated signal: aString.
%

category: 'Formatting'
method: Object
describe

"Returns an instance of a subclass of CharacterCollection describing the
 receiver.  This method is required by Topaz and by GemStone internal error
 handling.  Any reimplementation must conform to the following rules:

 * This method must not return nil.

 * This method must return an instance of a subclass of CharacterCollection."

^ [ self printString ] onException: Error do:[:ex | '<error during Object>>describe>' ]
%

category: 'Formatting'
method: Object
describe1K

"Returns an instance of a subclass of CharacterCollection describing the
 receiver.   Result has a max size of 1024 elements.
 This method is required by Topaz and by GemStone internal error
 handling.  Any reimplementation must conform to the following rules:

 * This method must not return nil.

 * This method must return an instance of a subclass of CharacterCollection."

^ [ | ws |
    ws := PrintStream printingOn: String new maxSize: 1024 .
    self printOn: ws.
    ws _collection
  ] onException: Error do:[:ex | '<error during Object>>describe1K>' ]
%

category: 'Error Handling - Compatibility'
method: Object
doesNotUnderstand: aMessageDescriptor

"The method is for compatiblity with Gs64 v2.x, and assumes you are using
   only method environment 0  for all of your Smalltalk code."

| ex sel args |
(ex := MessageNotUnderstood _basicNew)
  receiver: self selector: (sel := aMessageDescriptor at: 1)
		args: (args := aMessageDescriptor at: 2) envId: 0 .
^ex signal .	"fix for #40871"

"Legacy behavior was that you could fall through from an exception handler
without an explicit resume/return. The fix for 40871 will break applications
that rely on the old behavior."

"If we continue from the error, re-try the send of the message that was
 not understood."
"^ self perform: sel env: 0 withArguments: args "
%

category: 'Error Handling'
method: Object
doesNotUnderstand: aSymbol args: anArray envId: envId

"Generates an error reporting that the receiver cannot respond to a message.
 because no compiled method was found for aSymbol in method environment
 envId.   envId is a SmallInteger, 0 for Smalltalk , 1 for Ruby ,
 2..255 for future use by Smalltalk package managers .
"
| ex |
(ex := MessageNotUnderstood _basicNew)
  receiver: self selector: aSymbol args: anArray envId: envId .
^ex signal .	"fix for #40871"

"Legacy behavior was that you could fall through from an exception handler
without an explicit resume/return. The fix for 40871 will break applications
that rely on the old behavior."

"If we continue from the error, re-try the send of the message that was
 not understood."

"^ self perform: aSymbol env: envId withArguments: anArray "
%

category: 'Dynamic Smalltalk Instance Variables'
method: Object
dynamicInstanceVariables
  "Returns an Array of Symbols which are names of the
   dynamic instance variables of the receiver."

  ^ self _instvarNamesAfter: self namedSize 
%

category: 'Dynamic Smalltalk Instance Variables'
method: Object
dynamicInstVarAt: aSymbol
 "Return the value of specified dynamic instance variable, or nil if receiver
  has no such dynamic instance variable.  Does not consider instVars defined
  by the receiver's class."

<primitive: 881>
aSymbol _validateClass: Symbol .
aSymbol size == 0 ifTrue:[ ArgumentError signal:'instVar names cannot be empty symbol'].
self _primitiveFailed: #dynamicInstVarAt: args: { aSymbol }
%

category: 'Dynamic Smalltalk Instance Variables'
method: Object
dynamicInstVarAt: aSymbol put: aValue
 "Store the value of specified dynamic instance variable,
  If aValue ~~ _remoteNil, creates the dynamic instance variable if needed.
  aValue == _remoteNil effectively removes the instance variable.
  Does not consider instVars defined by the receiver's class.
  Generates an error if receiver is an ExecBlock, Behavior, GsNMethod, or
  special object.
  Returns aValue .

  Do not use this method and also use _tagAt:put: on the same object."

<primitive: 880>
aSymbol _validateClass: Symbol .
aSymbol size == 0 ifTrue:[ ArgumentError signal:'instVar names cannot be empty Symbol'].
self _primitiveFailed: #dynamicInstVarAt:put: args: { aSymbol . aValue }
%

category: 'Dynamic Smalltalk Instance Variables'
method: Object
dynamicInstVarPairs
"Returns the dynamic instance variarbles of the receiver
 in the form of an Array of name/value pairs.
 If the Array is of size zero, it may be the invariant Array  #() . "

<primitive: 1107>
self _primitiveFailed: #dynamicInstVarPairs .
%

category: 'Comparing'
method: Object
equalsNoCase: aCharCollection

"Returns true if the receiver and the argument have the same value.
 Returns false otherwise.

 Reimplemented in String and MultiByteString to provide case-insensitive
 comparison.  This implementation is the default for non-string objects."

^ self = aCharCollection
%

category: 'Error Handling'
method: Object
error: messageString

"Raises an error.  This method is intended for use in raising
 application-defined or user-defined errors. Returns the receiver."

^ UserDefinedError new _number: 2318;
        reason: #halt ; details: messageString ; signal
%

category: 'Listing References'
method: Object
findAllReferences

 "Returns an Array containing all references to the receiver.
 If an object contains multiple references to the receiver, that object occurs
 only once in the result.

 Large result sets may cause out-of-memory issues. To avoid problems,
 use Repository >> allReferences:, which returns a GsBitmap that does not
 require keeping objects in temporary object space.

 This method aborts the current transaction; if an abort would cause unsaved
 changes to be lost, it signals an error, #rtErrAbortWouldLoseData.

 The result contains both permanent and temporary objects.  The temporary
 objects found may vary from run to run. "

  ^ self findReferencesWithLimit: 0
%

category: 'Listing References'
method: Object
findNonStubbedReferencesInMemory

"Returns an Array of objects in current sessions temporary object memory
 that reference the receiver by a direct memory reference, i.e.
 those references that have not been stubbed to an objectId if the
 receiver is a committed object.

 The result may contain both permanent and temporary objects and may
 vary from run to run.  Does not abort the current transaction.

 Large result sets may cause out-of-memory issues. To avoid problems,
 use Repository >> allReferences:, which returns a GsBitmap that does not
 require keeping objects in temporary object space.
"

| argArray tmpArray resultArr |
argArray := { self } .
tmpArray := SystemRepository _listReferencesInMemory: argArray nonStubbedOnly: true .
resultArr := tmpArray at: 1.
resultArr removeIdentical: argArray ifAbsent:[ ].
^ resultArr
%

category: 'Listing References'
method: Object
findReferences

"Searches GemStone for objects that reference the receiver, and returns an Array
 of any such objects.  The search continues until all such objects have been
 found, or until the result contains 20 elements.  (The method
 findReferencesWithLimit: allows you to specify an arbitrarily large limit for
 the result Array.)

 If an object contains multiple references to the receiver, that object occurs
 only once in the result.

 This method aborts the current transaction; if an abort would cause unsaved
 changes to be lost, it signals an error, #rtErrAbortWouldLoseData.

 The result contains both permanent and temporary objects.  The temporary
 objects found may vary from run to run. 

 Large result sets may cause out-of-memory issues. To avoid problems,
 use Repository >> allReferences:, which returns a GsBitmap that does not
 require keeping objects in temporary object space.
"

^self findReferencesWithLimit: 20
%

category: 'Listing References'
method: Object
findReferencesInMemory

"Returns an Array of objects in current sessions temporary object memory
 that reference the receiver.  If receiver is a committed object,
 the result includes both direct memory references from temporary or
 modified committed objects, and references by objectId from other
 in-memory committed objects.  See also findNonStubbedReferencesInMemory .

 The search continues until all such objects have been found, or until the
 size of the result reaches the specified maximum aSmallInt.

 The result may contain both permanent and temporary objects and may
 vary from run to run.  Does not abort the current transaction.

 Large result sets may cause out-of-memory issues. To avoid problems,
 use Repository >> allReferences:, which returns a GsBitmap that does not
 require keeping objects in temporary object space.
"

| argArray tmpArray resultArr |
argArray := { self } .
tmpArray := SystemRepository listReferencesInMemory: argArray .
resultArr := tmpArray at: 1.
resultArr removeIdentical: argArray ifAbsent:[ ].
^ resultArr
%

category: 'Listing References'
method: Object
findReferencesWithLimit: aSmallInt

"Returns an Array of objects in GemStone that reference the receiver.
 The search continues until all such objects have been found, or until the
 size of the result reaches the specified maximum aSmallInt.

 Large result sets may cause out-of-memory issues. To avoid problems,
 use Repository >> allReferences:, which returns a GsBitmap that does not
 require keeping objects in temporary object space.

 If aSmallInt is <=  0, the result size is unlimited.

 The result contains both permanent and temporary objects.  The temporary
 objects found may vary from run to run.

 This method aborts the current transaction; if an abort would cause unsaved
 changes to be lost, it signals an error, #rtErrAbortWouldLoseData.

 Note that this method may take a considerable length of time to
 execute, and the result may occupy a large amount of disk space.
 (Compare with findReferences, which limits the result to 20 elements.). "

| argArray tmpArray resultArr limit |

System needsCommit
  ifTrue: [ self _error: #rtErrAbortWouldLoseData ] .

argArray := { self } .
limit :=  aSmallInt > 0 ifTrue:[ aSmallInt + 1] ifFalse:[ 0 ] .
tmpArray := SystemRepository listReferences: argArray withLimit: limit.

resultArr := tmpArray at: 1.
resultArr removeIdentical: argArray ifAbsent:[ ].
aSmallInt > 0 ifTrue:[
  resultArr size > aSmallInt ifTrue:[ resultArr size: aSmallInt ].
].
^ resultArr
%

category: 'Repository Conversion'
method: Object
fixIndexableRefsAfterConversion

"Default method for fixing references to ObsLargePositiveInteger and
 ObsLargeNegativeInteger instances that can now be represented as
 SmallInteger and Floats and SmallFloats which can now be represented as
 a SmallDouble."

|myClass|

myClass := self class.

myClass isIndexable
  ifTrue:[
    1 to: self _basicSize do:[:n| |iv |
      iv := self _basicAt: n.
	iv needsFixingAfterConversion
		ifTrue:[self _basicAt: n put: (iv + 0)].
  ].
].
^true
%

category: 'Repository Conversion'
method: Object
fixInstVarRefsAfterConversion

"Default method for fixing references to ObsLargePositiveInteger and
 ObsLargeNegativeInteger instances that can now be represented as
 a SmallInteger and Floats and SmallFloats which can now be
 represented as a SmallDouble."

|myClass|
myClass := self class.
"Check the named inst vars first..."
1 to: myClass allInstVarNames size do:[:n|
  |iv |
  iv := self instVarAt: n.
  iv needsFixingAfterConversion
	ifTrue:[self instVarAt: n put: (iv + 0)].
].
^true
%

category: 'Repository Conversion'
method: Object
fixReferencesInFirstPass

"Return whether this object should have references to Float,
 SmallFloat, ObsLargePositiveInteger and ObsLargeNegativeInteger
 fixed in the first pass.  false means fix them in the 2nd pass."

^true
%

category: 'Encoded OOPs'
method: Object
getOldHighEncodedOop

"Return a SmallInteger which is the high 16 bits of the Gs64 v1.1 OOP for the object.
 Return nil if the object is not representable with a 4 byte OOP .
 Special objects will always return nil.
"
| n |
n := self _getOldOop .
n ifNil:[ ^ nil ].
^ n bitShift: 16
%

category: 'Encoded OOPs'
method: Object
getOldLowEncodedOop

"Return a SmallInteger which is the low 16 bits of the Gs64 v1.1 OOP for the object.
 Return nil if the object is not representable with a 4 byte OOP .
 Special objects will always return nil.
"

| n |
n := self _getOldOop .
n == nil ifTrue:[ ^ nil ].
^ n bitAnd: 16rFFFF
%

category: 'Encoded OOPs'
method: Object
getOopFromInstVar: anOffset

"Return the oop for the object referenced by this one at the given
offset without reading the referenced object.  Note that in the case of
collections, the offset could reference either a named instance variable
or an indexable instance variable.  Named inst vars start at offset 1
and run up to the number of named inst vars.  Indexable inst vars follow
named inst vars in the sequence.

The receiver must be a format-oop
object;  kinds of IdentityBag , CharacterCollection, ByteArray are not allowed.

For example, to access the first element of an Array subclass with 3
named inst vars, an offset of 4 would be used."

<primitive: 637>
anOffset _validateClass: Integer .
(anOffset < 1 or:[anOffset > self _primitiveSize]) ifTrue:[
  self _errorIndexOutOfRange: anOffset .
].
self _primitiveFailed: #getOopFromInstVar: args: { anOffset } .
%

category: 'Error Handling'
method: Object
halt

"Signals a Halt exception. Returns the receiver."

  Halt signal
%

category: 'Error Handling'
method: Object
halt: messageString

"Raises an error.  This method is intended for use in raising
 application-defined or user-defined errors. Returns the receiver."

  Halt signal: messageString
%

category: 'Comparing'
method: Object
hash

"This method returns some Integer related to the contents of the
 receiver.  If two objects compare equal (=) to each other, the results
 of sending hash to each of those objects must also be equal.
 Ordinarily, a class which reimplements the = method should also reimplement
 hash.  Notice that an instance of class Object actually bases equality on
 identity, since it has no contents.  Therefore, this implementation of
 hash actually returns an Integer related to the identity of the
 receiver."

<primitive: 609 >

self _primitiveFailed: #hash .
self _uncontinuableError
%

category: 'Comparing'
method: Object
identityHash

"This method returns some Integer related to the identity of the receiver.  If
 two objects compare identically (==) to each other, the results of sending
identityHash to each of those objects will be equal."

<primitive: 609 >

self _primitiveFailed: #identityHash .
self _uncontinuableError
%

category: 'Flow of Control'
method: Object
ifNil: aBlock

"(Reserved selector.)  Returns the value of the zero-argument block aBlock
 if the receiver is == nil, otherwise, returns the receiver.

 This implementation is used by sends for which the argument is not
 a literal block.
 The following is an optimized control structure, not a recursive send."

^ self ifNil:[ aBlock value ] ifNotNil:[ self ]
%

category: 'Flow of Control'
method: Object
ifNil: nilBlock ifNotNil: notNilBlock

"(Reserved selector.)  Returns the value of the zero-argument block nilBlock
 if the receiver is == nil.  Otherwise, returns the value of the notNilBlock
 without evaluating nilBlock.
 The notNilBlock may be a zero or one arg block.  For a one arg block
 the receiver is assigned to the block argument if the notNilBlock is executed.

 This implementation is used by sends for which the arguments are not
 literal blocks.
 The following is an optimized control structure, not a recursive send."

  self ifNil:[^ nilBlock value].
  notNilBlock numArgs == 0 ifTrue:[ ^ notNilBlock value].
  ^ notNilBlock value: self
%

category: 'Flow of Control'
method: Object
ifNotNil: aBlock

"(Reserved selector.)  Returns the value of the block aBlock
 if the receiver is ~~ nil, otherwise, returns the receiver.
 aBlock may be a zero or one arg block.  For a one arg block
 the receiver is assigned to the block argument if aBlock is executed.

 This implementation is used by sends for which the argument is not
 a literal block.
 The following is an optimized control structure, not a recursive send."

 self ifNotNil:[
    aBlock numArgs == 0 ifTrue:[ ^ aBlock value].
    ^ aBlock value: self
 ].
  ^ self
%

category: 'Flow of Control'
method: Object
ifNotNil: notNilBlock ifNil: nilBlock

"(Reserved selector.)  Returns the value of the zero-argument block notNilBlock
 if the receiver is ~~ nil.  Otherwise, returns the value of the zero-argument
 block nilBlock without evaluating notNilBlock.
 The notNilBlock may be a zero or one arg block.  For a one arg block
 the receiver is assigned to the block argument if the notNilBlock is executed.

 This implementation is used by sends for which the arguments are not
 literal blocks.
 The following is an optimized control structure, not a recursive send."

 self ifNotNil:[
    notNilBlock numArgs == 0 ifTrue:[ ^ notNilBlock value].
    ^ notNilBlock value: self
 ].
 ^ nilBlock value
%

category: 'Invariance'
method: Object
immediateInvariant

"Makes the receiver immediately invariant.  (By comparison, when
 invariance is specified during subclass creation, instances become
 invariant when they are first committed.)  Returns receiver.

 There is no protocol to reverse the effect of this method.  If the
 receiver is a temporary object , there is no way to undo the effect
 of this method.  If the receiver was committed before the start
 of the current transaction, the effect of this method can be undone
 by aborting the transaction."

<primitive: 269>
self _primitiveFailed: #immediateInvariant .
self _uncontinuableError
%

category: 'Deprecated'
method: Object
in: aCollection

"If there is an element of aCollection identical to the receiver,
 returns true.  Otherwise, returns false."

self deprecated: 'Object>>in: deprecated before v3.0, use #includesIdentical: in collection'.
^ aCollection includesIdentical: self "this means identity, not equality"
%

category: 'Initialization'
method: Object
initialize
  "Default implementation"  
  ^ self
%

category: 'Accessing'
method: Object
instVarAt: anIndex

"If the receiver has an instance variable at anIndex, returns
 its value.  Generates an error if anIndex is not a SmallInteger or
 is out of bounds, or if the receiver has no instance variables."

<primitive: 611 >

| numInstVars |
(anIndex _isInteger) ifTrue: [
   numInstVars:= self namedSize .
   (numInstVars == 0) ifTrue:[ ^ self _error: #rtErrNoInstVars].
   ((anIndex >= 1) and:[anIndex <= numInstVars]) ifFalse:[
        ^ self _errorIndexOutOfRange: anIndex "out of bounds"
   ]
] ifFalse: [
  ^ self _errorNonIntegerIndex: anIndex
].
self _primitiveFailed: #instVarAt: args: { anIndex } .
self _uncontinuableError
%

category: 'Updating'
method: Object
instVarAt: anIndex put: aValue

"Stores the argument aValue in the instance variable indicated by anIndex.
 Generates an error if anIndex is not a SmallInteger or is out of bounds,
 or if the receiver has no instance variables,
 or if  (self class _structuralUpdatesDisallowed == true) .
 Returns aValue .

 This primitive is equivalent to GciStoreNamedOop."

<primitive: 604 >
| numIvs |
numIvs := self namedSize .
numIvs == 0 ifTrue: [ ^ self _error: #rtErrNoInstVars].
(anIndex _isSmallInteger)
  ifTrue: [((anIndex < 1) | (anIndex > numIvs ))
        ifTrue: [ ^ self _errorIndexOutOfRange: anIndex]] "out of bounds"
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].

self _primitiveFailed: #instVarAt:put: args: { anIndex . aValue } .
self _uncontinuableError
%

category: 'Testing'
method: Object
isAssociation
  ^ false
%

category: 'Testing'
method: Object
isBehavior

"Returns true if the receiver is a kind of Behavior (that is, a Class or
 metaclass object).  Returns false otherwise."

^false
%

category: 'Class Membership'
method: Object
isByteKindOf: aClass

"Returns true if the class of the receiver is a kind of aClass,
 and if the receiver has byte format."

(self isKindOf: aClass) ifFalse:[ ^ false ] .

^ self class isBytes
%

category: 'Testing'
method: Object
isClass

"Returns true if the receiver is a kind of Behavior (that is, a Class or
 metaclass object).  Returns false otherwise."

^ self isBehavior.
%

category: 'Clustering'
method: Object
isClustered

" Returns true if the object has been clustered in the current
  transaction, false otherwise"

<primitive: 278>
self _primitiveFailed: #isClustered
%

category: 'Testing'
method: Object
isCommitted

"Returns true if the receiver was a committed object at the time the current
 transaction began.  Returns false otherwise."

^((self _status: true) bitAnd: 2) ~~ 0
%

category: 'Deprecated'
method: Object
isConnected

"Always returns false.  this method is obsolete"
self deprecated: 'Object>>isConnected is deprecated before v3.0; will always returns false'.
^ false
%

category: 'Finalization'
method: Object
isEphemeron

"Returns true if receiver's isEphemeron bit is set, false otherwise"

<primitive: 828>
self _primitiveFailed: #isEphemeron
%

category: 'Testing'
method: Object
isEquivalent: anObject

"Returns true if the receiver is equivalent to anObject.  This
 is used to test the equivalence of Characters and Strings.  At
 this level, two objects are equivalent if they are identical."

^self == anObject
%

category: 'Formatting'
method: Object
isInPrintingRecursionSet: anIdentitySet
	"Answer whether we are recursing through the receiver.
	 In general, the identity test is sufficient."

	^anIdentitySet includes: self
%

category: 'Repository Conversion'
method: Object
isInternalObject
"Deterine if the receiver is an instance of one of a special internal class.
 See also PrivateObject>>isInternalObject . "

  ^ false
%

category: 'Invariance'
method: Object
isInvariant

"Returns true if the receiver is currently invariant, false otherwise."

<primitive: 36>
self _primitiveFailed: #isInvariant .
self _uncontinuableError
%

category: 'Class Membership'
method: Object
isKindOf: aClassHistoryOrClass

"(Optimized selector.)  Returns true if the class of the receiver is identical
 to, or is a subclass of any class in aClassHistoryOrClass; otherwise, returns
 false.

 If the aClassHistoryOrClass argument is actually a class rather than a class
 history, then this method uses the class history of the argument, instead of
 the class itself.

 For compatiblity with legacy code,
   String >> isKindOf:
 behaves as if String has the classHistory  { String . MultiByteString }.
 and
   Symbol >> isKindOf:
   DoubleByteSymbol >> isKindOf:
   QuadByteSymbol >> isKindOf:
 all behave as if the class of the receiver
 has classHistory { Symbol . DoubleByteSymbol . QuadByteSymbol } .

 This selector is optimized by the compiler and may not be reimplemented
 in any subclass.  This implementation is so that perform: will work."

^ self isKindOf: aClassHistoryOrClass
%

category: 'Class Membership'
method: Object
isKindOfClass: aClass

"Returns true if aClass is identical to the class of the receiver
 or aClass is a superclass of the class of the receiver."

<primitive: 34>

aClass _errorExpectedClass: Class.
"just assume the class was wrong; we can't ask if it isKindOf: Class"
self _primitiveFailed: #isKindOfClass: args: { aClass } .
self _uncontinuableError
%

category: 'Class Membership'
method: Object
isMemberOf: aClass

"Returns true if the receiver is an instance of aClass, or if aClass is
 contained in the class history of the receiver's class; otherwise, returns
 false."

| rcvrClass |
rcvrClass := self class .
rcvrClass == aClass ifTrue:[ ^ true ] .
(rcvrClass _classHistoryIncludesIdentical: aClass) ifTrue:[ ^ true] .
^ false
%

category: 'Class Membership'
method: Object
isMemberOfClass: aClass

"Returns true if the receiver is an instance of aClass."

^self class == aClass
%

category: 'Queries'
method: Object
isMeta

"Returns false.  This method is reimplemented in metaclasses to return true."

^ false.
%

category: 'Testing'
method: Object
isNil

"Returns true if the receiver is nil, false otherwise.
 If isNil is not reimplemented in the application, then
   == nil
 is faster than
   isNil
 "

^ self == nil
%

category: 'Testing'
method: Object
isSpecial

"Returns true if the receiver is a special object (for example, AbstractCharacter,
 Boolean, SmallInteger, or nil).  Returns false otherwise."

^ false
%

category: 'Testing'
method: Object
isSymbol

"Returns true if the receiver is a Symbol, DoubleByteSymbol, or QuadByteSymbol.
 false otherwise."

^ self _isSymbol
%

category: 'Testing'
method: Object
isTranlogged

"Returns true if the receiver was a committed object at the time the current
 transaction began and is not a notTranlogged object, or was part of the
 closure of a failed commit and did not become reachabley by reference
 from a not tranlogged object."

^ ((self _status: true) bitAnd: 16r20) ~~ 0
%

category: 'Class Membership'
method: Object
isUnicodeString
  "Returns true if receiver is a kind of Unicode7, Unicode16, or Unicode32"
^ (self _stringCharSize bitAnd: 16r8) ~~ 0
%

category: 'Deprecated'
method: Object
isUtfString
  "Returns true if receiver is a kind of Unicode7, Unicode16, or Unicode32"
self deprecated: 'Object>>isUtfString deprecated v3.1.0.1, use isUnicodeString'.
^ (self _stringCharSize bitAnd: 16r8) ~~ 0
%

category: 'Testing'
method: Object
isWritten

"Returns true if the receiver was a committed object at the start of the
 current transaction and as been written by the current transaction.
 Returns false otherwise."

^((self _status: true) bitAnd: 16rA ) == 16rA
%

category: 'Class Membership'
method: Object
legacyChangeClassTo: aClass

"Same as changeClassTo: except condition 7 is not enforced.
 If aClass defines more named instvars than the receiver,
 and the receiver has varying instvars, the additional named
 instvars are inserted before the varying instvars , rather
 than some of the varying instvars becoming named instvars."


^ self _changeClassTo: aClass preserveVarying: true
%

category: 'Storing and Loading'
method: Object
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object with named instance
 variable format.  Converts the object to its active form by loading the
 information into the receiver."

^self basicLoadFrom: passiveObj
%

category: 'Storing and Loading'
method: Object
loadNamedIVsFrom: passiveObj

"Reads named instance variables from the given passive object.  The
 first instance variable should already have been parsed and be
 available in the passiveObj argument."

| name offset nameSym |

[ name := passiveObj ivName.
  name ifNotNil:[
    nameSym := Symbol _existingWithAll: name .
    nameSym ifNotNil:[
      offset := self class _ivOffsetOf: nameSym.
      offset ifNotNil:[ self instVarAt: offset put: passiveObj ivValue ]
                ifNil:[ self dynamicInstVarAt: nameSym put: passiveObj ivValue].
    ] ifNil:[
      self dynamicInstVarAt: name asSymbol put: passiveObj ivValue
    ].
    passiveObj readNamedIV
  ] ifNil:[
    false
  ]
] untilFalse.

passiveObj skipNamedInstVars.
%

category: 'Storing and Loading'
method: Object
loadVaryingFrom: passiveObj

"Reads the varying part of the receiver from the given passive object.
 Does not record the receiver as having been read.  Does not read the
 receiver's named instance variables, if any."

1 to: self _basicSize do: [:i |
  self _basicAt: i put: passiveObj readObject
].
%

category: 'Storing and Loading'
method: Object
loadVaryingFrom: passiveObj size: varyingSize

"Reads the varying part of the receiver from the given passive object.
 Does not record the receiver as having been read.  Does not read the
 receiver's named instance variables, if any."

1 to: varyingSize do: [:i |
  self _basicAt: i put: passiveObj readObject
].
%

category: 'Instance Migration'
method: Object
migrate

"Migrate the instance from its current class to its class's target class.
 If its class has no target class, do nothing."

| targetClass newInstance invar |

targetClass := self class migrationDestination.
targetClass ~~ nil ifTrue: [
  invar := self isInvariant .
  newInstance := targetClass migrateNew.
  newInstance objectSecurityPolicy: self objectSecurityPolicy .
  newInstance migrateFrom: self.
  invar ifTrue:[ newInstance immediateInvariant ].
  newInstance become: self.
  " remember, self and newInstance have changed identities (thus, we are
    really returning the newly created instance of targetClass) "
  ^ self
]
%

category: 'Instance Migration'
method: Object
migrateFrom: anotherObject instVarMap: aMap

"Takes information from the given object and puts it in the receiver.
 This message is sent to an object when its class is being migrated to
 another class to account for changes in a schema.

 aMap may be an Array or an InstVarMappingArray .
 the varying portion of aMap is a precalculated indirection table
 associating the receiver's instance variables with instance variables
 in the other object.  If a table entry is 0, the other object is assumed to not have that
 instance variable.

 If aMap is an InstVarMappingArray, it contains a boolean controlling whether
 dynamic instance variables are copied from old to new object.

 This method should be reimplemented if it is desired to copy some dynamic
 instVar values to named instVars, or copy some named instVars to dynamic,
 or to handle renaming of instVars , or otherwise customize the migration."

| cls |
cls := self class.
cls firstPublicInstVar to: aMap size do:[:i | | idx |
  "copy the named instVars per aMap"
  idx := aMap at: i.
  idx == 0 ifFalse:[ self _unsafeAt: i put: (anotherObject instVarAt: idx)].
].
aMap class == Array ifTrue:[
  self copyDynamicInstVarsFrom: anotherObject
] ifFalse:[
  "aMap is an InstVarMappingArray"
  aMap preserveDynamic ifTrue:[
    self copyDynamicInstVarsFrom: anotherObject
  ].
].
self migrateIndexable: anotherObject myClass: cls otherClass: anotherObject class.
%

category: 'Instance Migration'
method: Object
migrateIndexable: anotherObject myClass: cls otherClass: othercls

"Handle migrating the indexable component of an object."

| instSize |

instSize := cls instSize.
(cls isIndexable and: [ othercls isIndexable])
  ifTrue:[ " both classes are indexable "
    cls isBytes
      ifTrue:[
        ( (cls isKindOf: CharacterCollection) and:
        [ othercls isKindOf: CharacterCollection ] )
          ifTrue:[ " both are CharacterCollections "
            "anotherObject copyFrom:1 to: anotherObject size into: self startingAt: 1."
             self replaceFrom: 1 to: anotherObject size with: anotherObject startingAt: 1
          ]
         ifFalse:[
           instSize := cls instSize.
           self _basicSize: anotherObject _basicSize.
           1 to: anotherObject _basicSize do: [:j |
             self _unsafeAt: j+instSize put: (anotherObject _basicAt: j).
           ].
         ].
      ]
      ifFalse: [
        instSize := cls instSize.
        self _basicSize: anotherObject _basicSize.
        1 to: anotherObject _basicSize do: [:j |
            self _unsafeAt: j+instSize put: (anotherObject _at: j).
        ].
      ].
  ].
%

category: 'Finalization'
method: Object
mourn

 "Default implementation which does nothing.

  An ephemeron (any object with the isEphemeron bit set) is treated
  specially during garbage collection. An ephemeron is typically used
  to provide finalization for some 'key' object. The ephemeron's first
  instVar is a reference to the key, and any other instVars refer to
  'values'. Values often contain indirect references to the key.
  When in-memory mark-sweep determines that an object is only reachable
  through ephemerons for which it is the key, the ephemerons 'fire.'
  Equivalently, an ephemeron fires when in-memory mark-sweep finds that
  *if* all instVars of all ephemerons with its key were set to nil, the
  key would be unreferenced and therefore subject to garbage collection.

  When GC 'fires' an ephemeron, it synchronously changes the ephemeron's
  isEphemeron bit from true to false. It also arranges for the object to
  be sent #mourn later in a high-priority finalization GsProcess.

  Subclasses that are designed to be ephemerons should implement #mourn
  to do whatever makes sense for their specific function. Typically,
  #mourn performs some finalization action then sets all instvars of the
  ephemeron to nil so that the key will be collected on the next
  in-memory mark-sweep. Exceptions signaled during the execution of
  #mourn that are not handled within that implementation of #mourn are,
  by default, logged to stdout, finalization of that ephemeron is
  abandoned, and the finalization GsProcess continues to send #mourn to
  other ephemerons on the finalization queue. It will be possible to
  insert other actions, such as stack dumping, for debugging failures
  during #mourn.

  Note: Since #mourn is executed asynchronously after the GC, it is
  possible to send 'beEphemeron: false' to an object and for that
  object to subsequently be sent #mourn. This will happen if GC has
  decided to fire the ephemeron and put it on the finalization queue
  before 'beEphemeron: false' is sent. Checking the return value of
  'beEphemeron: false' could help you determine whether a #mourn
  message might yet be received, since GC resets the bit when it puts
  the ephemeron on the finalization queue. However, sending
  'beEphemeron: false' and getting a return value of true does not
  completely guarantee that no #mourn will be sent, since the
  ephemeron could conceivably have been already on the finalization queue,
  then sent 'beEphemeron: true', then sent 'beEphemeron: false'.
 "
 ^ self
%

category: 'Clustering'
method: Object
moveToDisk

" Has no effect , Returns the receiver"

^ self
%

category: 'Clustering'
method: Object
moveToDiskInBucket: aClusterBucketOrId

" Has no effect , Returns the receiver"

 ^ self
%

category: 'Accessing'
method: Object
namedSize
  "Returns a SmallInteger, the number of named instanceVariables in the receiver."
  ^ self _primitiveSize - self _basicSize
%

category: 'Repository Conversion'
method: Object
needsFixingAfterConversion
^false
%

category: 'Updating'
method: Object
nilFields

"Sets the instance variables of the receiver to nil.  This is sometimes useful
 as an aid to quicker garbage collection."

self class firstPublicInstVar to: self namedSize do: [:i |
  self instVarAt: i put: nil
].

(self class isNsc) ifTrue: [
  self removeAll: self
]
ifFalse: [
  (self class isIndexable and: [self class isPointers]) ifTrue: [
    self size: 0
  ].
].
%

category: 'Formatting'
method: Object
nonRecursivePrintString
	"Answer a String containing a displayable representation of the receiver."

	| ws str |
	str := String new.
	ws := PrintStream printingOn: str.
	self printNonRecursiveOn: ws.
	^ ws _collection "contents might have been converted to an EUCString."
%

category: 'Testing'
method: Object
notNil

"Returns true if the receiver is not nil, false otherwise.
 If notNil is not reimplemented in the application, then
   ~~ nil
 is faster than
   notNil
"

^ self ~~ nil
%

category: 'Accessing'
method: Object
objectSecurityPolicy

"Returns the GsObjectSecurityPolicy where the receiver is located.
 If the result is nil, the object has World write permission (objectSecurityPolicyId 0)."

<primitive: 592>
self _primitiveFailed: #objectSecurityPolicy .
^ nil
%

category: 'Updating'
method: Object
objectSecurityPolicy: anObjectSecurityPolicy

"Assign the receiver to the given security policy.  Override this method
 to include component objects.

 If the receiver's current security policy and anObjectSecurityPolicy are the same,
 this method has no effect, otherwise all of the following apply:

 Generates an error if the user is not authorized to write to both
 the receiver's current security policy and anObjectSecurityPolicy,
 or if the receiver is a special object (SmallInteger, AbstractCharacter,
 Boolean, SmallDouble, or UndefinedObject),
 or if anObjectSecurityPolicy is not a committed object.

 anObjectSecurityPolicy may be nil, in which case the object is given World write
 permission (objectSecurityPolicyId 0).  "

<primitive: 2001>  "enter protected mode"
| prot |
prot := System _protectedMode .
^ [
    self _objectSecurityPolicy: anObjectSecurityPolicy
  ] ensure:[
    prot _leaveProtectedMode
  ].
%

category: 'Accessing'
method: Object
objectSecurityPolicyId

"Returns the objectSecurityPolicyId of the GsObjectSecurityPolicy where the receiver is located.
 If the result is 0, the object has World write permission."
| seg |
seg := self objectSecurityPolicy .
seg ifNotNil:[ ^ seg objectSecurityPolicyId ].
^ 0
%

category: 'Storing and Loading Obsolete'
method: Object
obsoleteInstVar: instVarName value: instVarValue

"This is a placeholder method with no default behavior.  (It simply returns the
 receiver.)  It can be reimplemented in subclasses to permit user-specified
 operations.

 This message is sent when an instance of a class with the same name as the
 receiver's class is activated and it specifies a named instance variable that
 the receiver does not have.  The instance variable name and value are sent as
 arguments to this method so that the instance may do something with them if it
 desires (such as transform old values into a new format in other instance
 variables)."
%

category: 'Clustering'
method: Object
page

"This method returns an Integer identifying the disk page on which an object is
 stored.  You can use this method to check your clustering methods for
 correctness.

 The page on which an object is stored may change for any of the following
 reasons:

 1.  A clustering message is sent to the object or to another object on the
     same page, and the current transaction is committed.
 2.  The current transaction is aborted.
 3.  The object is modified, and the current transaction is committed.
 4.  Another object on the page with the object is modified,
     and the current transaction is committed.

 For self-defining objects (SmallIntegers, AbstractCharacters, Booleans,
 UndefinedObjects), this method returns zero.

 Note that this method may return nil if the receiver has not been committed to
 GemStone, regardless whether the receiver is referenced by a committed object.

 If the receiver is a recently created Symbol, this method may return zero
"

<primitive: 84>

self _primitiveFailed: #page .
self _uncontinuableError
%

category: 'Clustering'
method: Object
pageCreationTime

"Returns a DateTime that is the approximate beginning of the life of the page
 containing the receiver.

 The result represents the time that the receiver was last modified, clustered,
 moved to a new page by the Garbage Collector Gem, regenerated from a
 transaction log or full backup file during recovery or restore, whichever
 happened last.

 When an object is modified by a session, the resulting pageCreationTime is an
 approximate time of the object creation or modification, and may precede the
 time at which the modification was committed.

 If the receiver is not yet committed and has not yet been assigned to a page,
 returns the current time.

 If the receiver is special, returns the value of the class instance variable
 timeStamp of the receiver's class."

<primitive: 400>
^ self _primitiveFailed: #pageCreationTime
%

category: 'Storing and Loading'
method: Object
passivate

"Creates a passive description of the receiver that can be activated with an
 activate message to create a new object with the same value as the receiver.

 For large objects or large graphs of objects, consider using this form:

 PassiveObject passivate: anObject toStream: (GsFile openWrite: aFileName)"

^PassiveObject passivate: self
%

category: 'Error Handling'
method: Object
pause

"Generates an error.  You can use this method to establish
 breakpoints in methods, aside from any debugger breakpoints that
 may be set.  If execution continued , returns receiver. "

 Halt signalNotTrappable
%

category: 'Message Handling'
method: Object
perform: aSelectorSymbol

"Sends the receiver the unary message indicated by the argument.
 The argument is the selector of the message.  Generates an error if
 the selector is not unary.

 The perform is done in the environment in which this method is compiled."
<primitive: 2004>
^self _perform: aSelectorSymbol asSymbol env: 0 withArguments:  #()
%

category: 'Message Handling'
method: Object
perform: aSelectorSymbol env: environmentId

"Sends the receiver the unary message indicated by the argument.
 The argument is the selector of the message.  Generates an error if
 the selector is not unary.

 environmentId must be a SmallInteger >= 0 and <= 255,
 specifying a method lookup environment.
"

<primitive: 2014>
^self _perform: aSelectorSymbol asSymbol env: environmentId withArguments:  #()
%

category: 'Message Handling'
method: Object
perform: aSelectorSymbol env: environmentId withArguments: anArray

"Sends the receiver the message indicated by the arguments.
 The argument, aSelectorSymbol, is the keyword selector of the message.
 The arguments of the message are the elements of anArray.  Generates an
 error if the number of arguments expected by aSelectorSymbol is not
 the same as the number of elements in anArray.

 anArray must be an instance of Array.

 environmentId must be a SmallInteger >= 0 and <= 255,
 specifying a method lookup environment."

<primitive: 2015>
anArray _validateClass: Array.

"Now just try the primitive again, but send asSymbol to the selector to convert
 it to a Symbol."
^ self _perform: aSelectorSymbol asSymbol env: environmentId
	withArguments: anArray
%

category: 'Message Handling'
method: Object
perform: aSelectorSymbol with: anObject

"Sends the receiver the message indicated by the arguments.  The
 first argument is the keyword or binary selector of the message.  The
 second argument is the argument of the message to be sent.  Generates
 an error if the number of arguments expected by the selector is not 1.

 The perform is done in the environment in which this method is compiled."
<primitive: 2004>

^self _perform: aSelectorSymbol asSymbol env: 0 withArguments: { anObject }
%

category: 'Message Handling'
method: Object
perform: aSelectorSymbol with: firstObject with: secondObject

"Sends the receiver the message indicated by the arguments.  The first
 argument is the keyword selector of the message.  The other arguments
 are the arguments of the message to be sent.  Generates an error if the
 number of arguments expected by the selector is not 2.

 The perform is done in the environment in which this method is compiled."
<primitive: 2004>

^self _perform: aSelectorSymbol asSymbol env: 0
      withArguments: { firstObject . secondObject }
%

category: 'Message Handling'
method: Object
perform: aSelectorSymbol with: firstObject with: secondObject with: thirdObject

"Sends the receiver the message indicated by the arguments.  The first
 argument is the keyword selector of the message.  The other arguments
 are the arguments of the message to be sent.  Generates an error if the
 number of arguments expected by the selector is not 3.

 The perform is done in the environment in which this method is compiled."
<primitive: 2004>

^self _perform: aSelectorSymbol asSymbol env: 0
      withArguments: { firstObject . secondObject . thirdObject }
%

category: 'Message Handling'
method: Object
perform: aSelectorSymbol withArguments: anArray

"Sends the receiver the message indicated by the arguments.
 The argument, aSelectorSymbol, is the keyword selector of the message.
 The arguments of the message are the elements of anArray.  Generates an
 error if the number of arguments expected by aSelectorSymbol is not
 the same as the number of elements in anArray.

 anArray must be an instance of Array.
 The perform is done in the environment in which this method is compiled."

<primitive: 2005>
anArray _validateClass: Array.

"Now just try the primitive again, but send asSymbol to the selector to convert
 it to a Symbol."
^ self _perform: aSelectorSymbol asSymbol env: 0 withArguments: anArray
%

category: 'Message Handling'
method: Object
performMethod: aGsNMethod

"Executes the argument as it would be executed by a zero arg send to
 the receiver."

<primitive: 2027>
aGsNMethod _validateClass: GsNMethod .
self _primitiveFailed: #performMethod: args: { aGsNMethod }
%

category: 'Accessing'
method: Object
physicalSize

"Returns the number of bytes required to represent the receiver in memory.  If
 the receiver is in special format (which implies that its representation is the
 same as its OOP), returns zero.

 This method should be reimplemented for subclasses whose instances are (or may
 be) a composite of component parts which are objects themselves (such as B-tree
 nodes).  Since the composite object cannot be represented independently of its
 components, its physical size should include that of its components.

 However, the component objects of collection (such as an NSC) should not be
 confused with its contents or elements.  Elements or contained objects are in a
 logical relationship with the collection, whereas its components are in a
 physical relationship.  Logically related objects can be represented and stored
 independently."

<primitive: 364>
self _primitiveFailed: #physicalSize
%

category: 'Accessing'
method: Object
physicalSizeOnDisk

"Returns the number of bytes required to represent the receiver on disk.  If
 the receiver is in special format (which implies that its representation is the
 same as its OOP), returns zero.

 See also Object >> physicalSize"

<primitive: 579>
self _primitiveFailed: #physicalSizeOnDisk
%

category: 'Copying'
method: Object
postCopy

"Override to do cleanup in new copy"

^self
%

category: 'Formatting'
method: Object
printNonRecursiveOn: aStream
	"Put a displayable representation of the receiver on the given stream
	 while avoiding recursion from object reference loops."

	self printOn: aStream recursionSet: IdentitySet new.
%

category: 'Formatting'
method: Object
printNonRecursiveRepresentationOn: aStream recursionSet: anIdentitySet
	"Put a displayable representation of the receiver on the given stream
	 while avoiding recursion from object reference loops.

	 Subclasses should override this method with their own specific printing
	 implementation, analogous to what would be in a #printOn: implementation,
	 but with the exception of differences for preventing recursion.

	 Subclass implementations should call #printNonRecursiveOn:recursionSet:
	 for constituent objects in addition to whatever basic, non-recursive
	 representation they include in this method."

	self printOn: aStream
%

category: 'Formatting'
method: Object
printOn: aStream

"Puts a displayable representation of the receiver on the given stream."

"For classes whose instances can be literals, the result should contain
 formatting information.  For example, the following expression should
 evaluate to true:

 #abc asString = String withAll: '#abc'."

aStream nextPutAll: self asString
%

category: 'Formatting'
method: Object
printOn: aStream recursionSet: anIdentitySet
	"Put a displayable representation of the receiver on the given stream
	 avoiding recursion from object reference loops."

	(self isInPrintingRecursionSet: anIdentitySet)
		ifTrue: [^self printRecursiveRepresentationOn: aStream].
	anIdentitySet add: self.
	self printNonRecursiveRepresentationOn: aStream recursionSet: anIdentitySet.
	anIdentitySet remove: self
%

category: 'Formatting'
method: Object
printRecursiveRepresentationOn: aStream
	"Put a displayable representation of the receiver on the given stream.
	 The representation must not recurse. We are here because we already recursing."

	aStream nextPutAll: self class name describeClassName
%

category: 'Formatting'
method: Object
printString

"Returns a CharacterCollection whose contents are a displayable representation of the
 receiver."

"This method uses the printOn: method to create the String."

| ws str |

str := String new.

ws := PrintStream printingOn: str.

self printOn: ws.
 "contents might have been converted to another subclass of CharacterCollection."
^ ws _collection
%

category: 'Dynamic Smalltalk Instance Variables'
method: Object
removeDynamicInstVar: aSymbol

  ^ self dynamicInstVarAt: aSymbol put: _remoteNil
%

category: 'Indexing Support'
method: Object
removeObjectFromBtrees
  "Remove the receiver from the B-trees of any equality indexes that it
 participates in.  Returns an Array of index object/value pairs. The receiver
 must be a byte object."

  self class isBytes
    ifFalse: [
      ^ self
        error:
          'The method #removeObjectFromBtrees should only be used with byte objects.' ].
  ^ self _errorsDuringPreventCommit: [ self _removeObjectFromBtrees ]
%

category: 'GBS support'
method: Object
replicationSpecGbsClampCallback

  ^ #()   "default spec is an empty array"
%

category: 'Testing'
method: Object
respondsTo: aSelector

"Returns true if the receiver's class has a method with the given selector
 and false if not."

| sym |
sym := Symbol _existingWithAll: aSelector .
sym ifNil:[ ^ false ].
^ self _respondsTo: sym private: true flags: 16r10000"cache result"
%

category: 'Deprecated'
method: Object
segment

self deprecated: 'Object>>segment deprecated v3.0. Use #objectSecurityPolicy instead.'.
^self objectSecurityPolicy.
%

category: 'Accessing'
method: Object
segmentId

self deprecated: 'Object>>segmentId deprecated v3.0. Use #objectSecurityPolicyId instead.'.
^self objectSecurityPolicyId.
%

category: 'Copying'
method: Object
shallowCopy

"Returns a copy of the receiver with none of its components copied."

<primitive: 885>
self _primitiveFailed: #shallowCopy .
self _uncontinuableError
%

category: 'Error Handling'
method: Object
shouldNotImplement: aSelector

"Generates an error reporting that the receiver cannot respond to
 aSelector.  This is useful because sometimes a subclass should not
 respond to messages for which it has inherited methods from its
 superclass.  For instance, class Set should not respond to Object | at:.
 Defining Set | at: with a shouldNotImplement error hides
 the ordering information of Set from users of instances of Set."

^ self _error: #rtErrShouldNotImplement args: { aSelector }
%

category: 'Storing and Loading'
method: Object
shouldWriteDynamicInstVar: instVarName

"Returns whether the given dynamic instance variable should be written out.  The
 default is to write out all dynamic instance variables."
^true
%

category: 'Storing and Loading'
method: Object
shouldWriteInstVar: instVarName

"Returns whether the given instance variable should be written out.  The
 default is to write out all instance variables."
^true
%

category: 'Accessing'
method: Object
size

"Returns the number of unnamed instance variables in the receiver."

<primitive: 0>
self _primitiveFailed: #size .
self _uncontinuableError
%

category: 'Updating'
method: Object
size: anInteger

"Changes the varying size of the receiver to anInteger.
 Returns the receiver.

 If anInteger is less than the current size of the receiver, the
 receiver is shrunk accordingly.  If anInteger is greater than the
 current size of the receiver, the receiver is extended and new
 elements are initialized to nil.

 Generates an error if anInteger is not a SmallInteger,
 or if the receiver is not indexable."

<primitive: 603 >

(self class isIndexable not) "not an indexable object"
  ifTrue: [ ^ self _errorNotIndexable].
(anInteger _isInteger)
   ifTrue: [ ^ self _errorIndexOutOfRange: anInteger]
   ifFalse: [ ^ self _errorNonIntegerIndex: anInteger].
self _primitiveFailed: #size: args: { anInteger } .
self _uncontinuableError
%

category: 'Class Membership'
method: Object
species

"Returns a class similar to, or the same as, the receiver's class which
 can be used for containing derived copies of the receiver."

^self class
%

category: 'Class Membership'
method: Object
speciesForCollect

"Returns a class, an instance of which should be used as the result of
 collect: or other projections applied to the receiver."

^ self species
%

category: 'Class Membership'
method: Object
speciesForPrint

"Returns a class similar to, or the same as, the receiver's class which
 can be used for containing printing the receiver."

^ String
%

category: 'Class Membership'
method: Object
speciesForSelect

"Returns a class similar to, or the same as, the receiver's class which
 can be used for containing derived copies of the receiver for select:
 and reject: queries."

^ self species
%

category: 'Accessing'
method: Object
squeakBasicAt: anIndex

"Returns the specified varying instVar of an oop format receiver .
 For a byte format receiver, returns the unsigned Integer value of
 the word specified by anIndex,
 using (self class _bytesPerWord) as the size of a word .

 Signals an error if the receiver is not indexable or
 if anIndex is not a SmallInteger or is out of bounds."

<primitive: 1079>

(self class isIndexable) "not an indexable object"
  ifFalse: [ ^ self _errorNotIndexable].
(anIndex _isInteger)
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].

self _primitiveFailed: #squeakBasicAt: args: { anIndex } .
self _uncontinuableError
%

category: 'Updating'
method: Object
squeakBasicAt: anIndex put: aValue

"Stores aValue into the specified varying instVar of an oop format receiver .
 For a byte format receiver, aValue must be within range
 consistent with both  self class _bytesPerWord
 and                   self class _signedWords.

 Signals an error if the receiver is not indexable or variant,
 or if anIndex is out of bounds."

<primitive: 1080>
| cls |
(anIndex _isInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
cls := self class .
(cls isIndexable) ifFalse: [ ^ self _errorNotIndexable].
((anIndex < 1) | (anIndex > (self size + 1)))
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].

(cls isBytes) ifTrue:[  | maxV |
  aValue _validateClass: Integer .
  maxV := (1 bitShift: 8 * cls _bytesPerWord ) - 1 .
  cls _signedWords ifTrue:[
    maxV := maxV bitShift: -1 .
    (aValue < (0 - maxV - 1) or:[ aValue > maxV]) ifTrue:[
       aValue _error: #rtErrArgOutOfRange args:{ 0 . maxV}
    ].
  ] ifFalse:[
    (aValue < 0 or:[ aValue > maxV]) ifTrue:[
       aValue _error: #rtErrArgOutOfRange args:{ 0 . maxV}
    ].
  ].
].
self _primitiveFailed: #squeakBasicAt:put: args: { anIndex . aValue } .
self _uncontinuableError
%

category: 'Accessing'
method: Object
squeakBasicSize

"Returns the total number of instVars in an oop format receiver .
 For a byte format receiver, returns the size in words
 using (self class _bytesPerWord) as the size of a word ."

<primitive: 1081>

self _primitiveFailed: #basicSize .
self _uncontinuableError
%

category: 'Storing and Loading'
method: Object
storeOn: stream

"Writes a string that, when evaluated, recreates a copy of the receiver to
 the given stream.  The default is to use PassiveObjects to create the
 description."

| p lf col extra litSize c appendStr idx needVariant |

p := (self passivate) contents.
lf := Character lf.
needVariant := false.

"Format the passive description so it can be filed in properly.
 This has to take into account the max literal size restriction in
 GemStone's compiler.  It still may fail if the content string is
 too large or has too many syntactically significant characters in it."
col := litSize := extra := 0.
1 to: p size do: [:i |
  idx := i+extra.
  c := p at: idx.
  (c == $% and: [col == 0]) ifTrue: [
    p insertAll: ' ' at: idx.
    extra := extra + 1.
    litSize := litSize + 1.
    col := col + 1.
  ]
  ifFalse: [
    (c == $') ifTrue: [
      p insertAll: '''' at: idx.
      extra := extra + 1.
      litSize := litSize + 1.
      col := col + 1.
    ]
    ifFalse: [
      (c == lf) ifTrue: [col := -1]
    ]
  ].
  litSize := litSize + 1.
  col := col + 1.
  litSize > 9999 ifTrue: [
    appendStr := '''; addAll: '''.
    p insertAll: appendStr at: idx.
    extra := extra + appendStr size.
    litSize := 0.
    needVariant := true.
  ].
].

stream nextPutAll: '(PassiveObject newWithContents: '.
needVariant ifTrue: [
  stream nextPutAll: '(String new addAll: '
].
stream nextPutAll: '''';
  nextPutAll: p;
  nextPutAll: ''''.
needVariant ifTrue: [
  stream nextPutAll: '; yourself )' .
].
stream nextPutAll: ') activate'
%

category: 'Class Membership'
method: Object
stringCharSize
" Returns 1 if receiver's class is subclass of String or ByteArray,
          2 if receiver's class is subclass of   DoubleByteString,
          4 if receiver's class is subclass of   QuadByteString,
          0 otherwise"
  ^ self _stringCharSize bitAnd: 16r7
%

category: 'Error Handling'
method: Object
subclassResponsibility
	"This provides compatibility with other Smalltalks
	but preserves the GemStone/S approach"
	| meth |
  meth := GsProcess methodAt: 2 .
	self subclassResponsibility: meth selector
%

category: 'Error Handling'
method: Object
subclassResponsibility: aSelector

"This is used in an abstract superclass to detect a protocol error.
 It generates an error indicating that a concrete subclass
 should have implemented this method."

^ self _error: #rtErrSubclassResponsibility args: { aSelector }
%

category: 'Tag Management'
method: Object
tagSize
"Returns the total number of words of dynamic instVars allocated in the
 receiver."

<primitive: 779>
self _primitiveFailed: #tagSize .
self _uncontinuableError
%

category: 'Flow of Control'
method: Object
to: aNumber by: stepValue do: aBlock

"(Reserved selector.)  Iteratively evaluates the one-argument block aBlock,
 using the block's single argument as the iteration control variable.
 Initially, that control variable is set to the receiver.
 The argument stepValue must be a Number not equal to zero .
 If stepValue is equal to zero, an ArgumentError is signalled.
 aNumber must be a kind of Number.

 The block is evaluated while the control variable is less than or equal to
 aNumber, if stepValue is positive, or greater than or equal to aNumber if
 stepValue is negative  After each evaluation, the control variable is
 modified by stepValue. If the receiver is greater than aNumber and stepValue
 is positive, or less than aNumber when stepValue is negative, the block is
 not evaluated at all. Returns the receiver.

 A method which sends to:by:do: will have a step point generated for the send of
 the to:by:do: and a step point for the loop index increment and test.

 This implementation is used when the method compiler cannot in-line aBlock,
 and is used by perform."

self to: aNumber
     by: stepValue
     do: [:i | aBlock value: i]
%

category: 'Flow of Control'
method: Object
to: aNumber do: aBlock

"(Reserved selector.)  Iteratively evaluates the one-argument block aBlock,
 using the block's single argument as the iteration control variable.
 Initially, that control variable is set to the receiver.  The block is
 evaluated while the control variable is less than or equal to aNumber (which
 must be a kind of Number).  After each evaluation, the control variable is
 incremented by 1.  If the receiver is greater than aNumber, the block is not
 evaluated at all.  Returns the receiver.

 A method which sends to:do: will have a step point generated for the send of
 the to:do: and a step point for the loop index increment and test.

 This implementation is used when the method compiler cannot in-line aBlock,
 and is used by perform."

self to: aNumber do: [:i | aBlock value: i]

%

category: 'Accessing'
method: Object
trackRead
  "If receiver is not a special object, and is committed,
   and (System objectReadTrackingEnabled == true)
   add the receiver to the sessions read tracking buffer.
   This addition to the read tracking buffer is independent of whether 
     (self objectSecurityPolicy trackReads == true)
   and is independent of whether this method has already been sent to the receiver.
  
   Returns true if the object was added to the read tracking buffer, false otherwise.
  "
  <primitive: 809>
  ^ self _primitiveFailed: #trackRead
%

category: 'Testing'
method: Object
validateIsClass

"Generates an error if the receiver is not a kind of Class."

(self isKindOf: Class ) ifTrue:[^ self ] .
^ self _error: #rtErrExpectedClass
%

category: 'Testing'
method: Object
validateIsIdentifier

"Generates an error if the receiver is not a kind of Symbol, or contains
 Characters that are not allowed in a GemStone Smalltalk identifier."

<primitive: 37 >
^ self _primitiveFailed: #validateIsIdentifier
%

category: 'Testing'
method: Object
validateIsVariant

"Generates an error if the receiver is not variant."

self isInvariant ifTrue:[ ^ self _error: #rtErrObjInvariant ] .
^ self
%

category: 'Message Handling'
method: Object
with: anObject perform: aSelectorSymbol env: environmentId

"Sends the receiver the message indicated by the arguments.  The
 first argument is the keyword or binary selector of the message.  The
 second argument is the argument of the message to be sent.  Generates
 an error if the number of arguments expected by the selector is not 1.

 environmentId must be a SmallInteger >= 0 and <= 255,
 specifying a method lookup environment."

<primitive: 2014>
| sel |
sel := aSelectorSymbol asSymbol .
^self _perform: sel env: environmentId withArguments: { anObject }
%

category: 'Message Handling'
method: Object
with: argOne performMethod: aGsNMethod

"Executes the aGsNMethod as it would be executed by a one arg send to
 the receiver."

<primitive: 2027>
aGsNMethod _validateClass: GsNMethod .
self _primitiveFailed: #with:performMethod: args: { argOne . aGsNMethod }
%

category: 'Message Handling'
method: Object
with: firstObject with: secondObject perform: aSelectorSymbol env: environmentId

"Sends the receiver the message indicated by the arguments.  The first
 argument is the keyword selector of the message.  The other arguments
 are the arguments of the message to be sent.  Generates an error if the
 number of arguments expected by the selector is not 2.

 environmentId must be a SmallInteger >= 0 and <= 255,
 specifying a method lookup environment."

<primitive: 2014>

^self _perform: aSelectorSymbol asSymbol env: environmentId
      withArguments: { firstObject . secondObject }
%

category: 'Message Handling'
method: Object
with: firstObject with: secondObject with: thirdObject
  perform: aSelectorSymbol env: environmentId

"Sends the receiver the message indicated by the arguments.  The first
 argument is the keyword selector of the message.  The other arguments
 are the arguments of the message to be sent.  Generates an error if the
 number of arguments expected by the selector is not 3.

 environmentId must be a SmallInteger >= 0 and <= 255,
 specifying a method lookup environment."

<primitive: 2014>

^self _perform: aSelectorSymbol asSymbol env: environmentId
      withArguments: { firstObject . secondObject . thirdObject }
%

category: 'Message Handling'
method: Object
with: firstObject with: secondObject with: thirdObject with: fourthObject
  perform: aSelectorSymbol env: environmentId

"Sends the receiver the message indicated by the arguments.  The first
 argument is the keyword selector of the message.  The other arguments
 are the arguments of the message to be sent.  Generates an error if the
 number of arguments expected by the selector is not 4.

 environmentId must be a SmallInteger >= 0 and <= 255,
 specifying a method lookup environment."

<primitive: 2014>

^self _perform: aSelectorSymbol asSymbol env: environmentId
      withArguments: { firstObject . secondObject . thirdObject . fourthObject }
%

category: 'Storing and Loading'
method: Object
writeTo: passiveObj

"Converts the receiver to its passive form and writes that information on
 passiveObj.  The argument must an instance of PassiveObject."

^self basicWriteTo: passiveObj
%

category: 'Accessing'
method: Object
yourself

"Returns the receiver.  Often useful as the last message in a series of
 cascaded message sends to ensure that the expression returns a known value.
 It is provide for the convenience of sloppy programmers.

 Smalltalk code  code that is written to run as fast as possible
 should never yourself.  A message send and return is at least 25 machine
 instructions ; assigning to or fetching from a method temp is only
 2 machine instructions per assign/fetch in native code.

 Instead of
    ^ String new add:'abc'; yourself
 this will be faster:
    | res |
    (res := String new) add:'abc'.
    ^ res
 "

^self
%

category: 'Reduced Conflict Support'
method: Object
_abortAndReplay: conflictObjects

"Abort the receiver and replay operations on the receiver from the redo log."

| redoLog logEntries |
redoLog := System _redoLog.

" if no log entries to replay, then we're done "
redoLog == nil
  ifTrue: [
    System
      rcValueCacheAt: #'Rc-Retry-Failure-Reason'
        put: #'noRedoLog'
        for: System;
      rcValueCacheAt: #'Rc-Retry-Failure-Description'
        put: 'No redoLog for replay'
        for: System.
    ^ false ].
logEntries := redoLog getLogEntriesFor: self .
logEntries == nil
  ifTrue:[
    System
      rcValueCacheAt: #'Rc-Retry-Failure-Reason'
        put: #'noRedoLogForObject'
        for: System;
      rcValueCacheAt: #'Rc-Retry-Failure-Description'
        put: 'No redoLog for the object [',  self asOop printString, ']'
        for: System.
    ^ false ].

" cannot perform selective abort if receiver has a dependency tag "
self _hasDependencyList
  ifTrue: [
    System
      rcValueCacheAt: #'Rc-Retry-Failure-Reason'
        put: #'objHadDependency'
        for: System;
      rcValueCacheAt: #'Rc-Retry-Failure-Description'
        put: 'The object [', self asOop printString, '] has a dependency list. Replay not allowed.'
        for: System.
    ^ false ].

" refresh the state of the receiver "
self _selectiveAbort.

" tell the redo log to replay any operations on the receiver "
^ redoLog _redoOperationsForEntries: logEntries
%

category: 'Reduced Conflict Support'
method: Object
_addNamedIvsToRcRead

"Adds a small receiver, or path to the left most leaf of a large receiver
 to the rcReadSet.  Returns self.
 Receiver may not be a kind of IdentityBag nor a special object."

<primitive: 856>
self _primitiveFailed: #_addNamedIvsToRcRead
%

category: 'Accessing'
method: Object
_alias

"This method returns an alias for the receiver.  The alias is a
 kind of Integer.  It may be a SmallInteger or LargeInteger.
 The alias for any set of objects is unique."

<primitive: 609 >

self _primitiveFailed: #alias .
self _uncontinuableError
%

category: 'Private'
method: Object
_allElements

"Receiver should be a large oop or byte format Collection.
 Returns nil if receiver has NSC or special format ,
         or if receiver is a Symbol.

 Returns an Array containing all the elements of an oop format
 receiver.  The result will contain all nils that were present
 in leaf nodes, but will omit nils that would have been in
 unpopulated portions of a large object.  Thus the result should
 contain key,value pairs in order for a dictionary .

 Returns a instance of the class of the receiver containing
 all the bytes of a byte format receiver, omitting those portions
 that were never stored into.  It is assumed that Strings being
 enumerated were not sparse, i.e. they were built by normal
 concatenation or insertion and not by code like
   | s | s := String new: 100000 .
   s at: 50000 put: $X .
   'abc' copyFrom:1 to: 3 into: s startingAt: 10000 .

 Provided for use in enumerating objects to repair a repository
 after bug 43141."

<primitive: 962>
^ self _primitiveFailed: #_allElements
%

category: 'Accessing'
method: Object
_at: anIndex

"This method provides indexed access to indexable objects,
 and provides pseudo-indexed access to non-sequenceable collections,
 and provides logical indexed access to CharacterCollections whose
 Characters are composed of multiple physical bytes.

 May be reimplemented; for examples see Bag, CharacterCollection.

 The methods _at:, _basicAt:, and _basicSize operate on the logically
 indexable contents of the receiver.

 Implemented here as a direct call to a primitive for efficiency.  The
 primitive generates an error if the receiver is not indexable, or if anIndex
 is not a SmallInteger or is out of bounds."

<primitive: 32>

(self class isIndexable) "not an indexable object"
  ifFalse: [ ^ self _errorNotIndexable].
(anIndex _isInteger)
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].

self _primitiveFailed: #_at: args: { anIndex } .
self _uncontinuableError
%

category: 'Updating'
method: Object
_at: anIndex put: aValue

"Signals an error if the receiver is not indexable or variant,
 or if anIndex is not a SmallInteger or is out of bounds.

 May be reimplemented.

 The primitive is equivalent to GciStoreIdxOop or GciStoreByte,
 depending on implementation of the receiver."

<primitive: 268>

(anIndex _isInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
(self class isIndexable) "not an indexable object"
  ifFalse: [ ^ self _errorNotIndexable].
((anIndex < 1) | (anIndex > (self size + 1))) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].
(self class isBytes)
ifTrue:
  [ ((aValue class ~~ SmallInteger) or: [ (aValue < 0) | (aValue > 255) ])
    ifTrue: [^ aValue _error: #rtErrExpectedByteValue].
  ].

self _primitiveFailed: #_at:put: args: { anIndex . aValue } .
self _uncontinuableError
%

category: 'Accessing'
method: Object
_basicAt: anIndex

"Signals an error if the receiver is not
 indexable, or if anIndex is not a SmallInteger or is out of bounds.

 This method is for internal use in implementation of the kernel classes.

 The methods _at:, _basicAt:, and _basicSize operate on the logically
 indexable contents of the receiver.

 This method must not be overridden."

<primitive: 32>

(self class isIndexable) "not an indexable object"
  ifFalse: [ ^ self _errorNotIndexable].
(anIndex _isInteger)
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].

self _primitiveFailed: #_basicAt: args: { anIndex } .
self _uncontinuableError
%

category: 'Updating'
method: Object
_basicAt: anIndex put: aValue

"Signals an error if the receiver is not indexable or variant,
 or if anIndex is not a SmallInteger or is out of bounds.

 This method must not be overridden; contrast with Object|at:put:.

 The primitive is equivalent to GciStoreIdxOop or GciStoreByte,
 depending on implementation of the receiver."

<primitive: 268>

(anIndex _isInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
(self class isIndexable) "not an indexable object"
  ifFalse: [ ^ self _errorNotIndexable].
((anIndex < 1) | (anIndex > (self size + 1))) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].
(self class isBytes)
ifTrue:
  [ ((aValue class ~~ SmallInteger) or: [ (aValue < 0) | (aValue > 255) ])
    ifTrue: [^ aValue _error: #rtErrExpectedByteValue].
  ].

self _primitiveFailed: #_basicAt:put: args: { anIndex . aValue } .
self _uncontinuableError
%

category: 'Deprecated'
method: Object
_basicCopyWithoutTags

"Returns a copy of the receiver which shares the receiver's instance
 variables.  Tags, if any, are not copied."
self deprecated: 'Object>>_basicCopyWithoutTags deprecated around v3.0, Should use dynamicInstVars, not tags'.
^self _basicCopyWithoutTagsPrim.
%

category: 'Deprecated'
method: Object
_basicCopyWithoutTagsPrim

<primitive: 55>
self _primitiveFailed: #_basicCopyWithoutTagsPrim .
self _uncontinuableError
%

category: 'Accessing'
method: Object
_basicSize

"Signals an error if the receiver is not Variable-Length.
 The returned value reflects the allowable bounds in Object|_basicAt:
 and Object|_basicAt:put:.
 This method must not be overridden; contrast with Object|size.

 The methods _at:, _basicAt:, and _basicSize operate on the logically
 indexable contents of the receiver.

 The primitive is equivalent to GciFetchIdxSize(self); "

<primitive: 0>

^0 "for non-indexable objects, returns zero for their size"
%

category: 'Updating'
method: Object
_basicSize: anInteger

"Changes the size of the receiver to anInteger.
 Returns the receiver.

 If anInteger is less than the current size of the receiver, the
 receiver is shrunk accordingly.  If anInteger is greater than the
 current size of the receiver, the receiver is extended and new
 elements are initialized to nil.

 Generates an error if anInteger is not a SmallInteger,
 or if the receiver is not indexable.  This method should not be
 overridden in subclasses."

<primitive: 603 >

(self class isIndexable not) "not an indexable object"
  ifTrue: [ ^ self _errorNotIndexable].
(anInteger _isInteger)
   ifTrue: [ ^ self _errorIndexOutOfRange: anInteger]
   ifFalse: [ ^ self _errorNonIntegerIndex: anInteger].
self _primitiveFailed: #_basicSize: args: { anInteger } .
self _uncontinuableError
%

category: 'Updating'
method: Object
_becomeDictionary: anObject

<primitive: 2001>  "enter protected mode"
| prot |
prot := System _protectedMode .
[
  self ~~ anObject ifTrue:[
    self _primitiveBecome: anObject forDict: true .
    self _resetParentRef .
    anObject _resetParentRef .
  ]
] ensure:[
  prot _leaveProtectedMode
].
%

category: 'Updating'
method: Object
_becomeMinimalChecks: anObject

"Performs the become operation
 (see documentation for Object>>become: for more details)

 Does not check for receiver or anObject being on stack as self.
 Will fail if receiver or anObject participates in an index ,
 or is a kind of object not legal for become: .
 Returns self (which will have swapped identity with anObject) "

<primitive: 2001>  "enter protected mode"
| prot |
prot := System _protectedMode .
[
  self ~~ anObject ifTrue:[
    self _primitiveBecome: anObject forDict: false .
    self _resetParentRef .
    anObject _resetParentRef .
  ]
] ensure:[
  prot _leaveProtectedMode
].
%

category: 'Class Membership'
method: Object
_changeClassTo: aClass preserveVarying: aBoolean

<primitive: 2001>  "enter protected mode"
| prot |
prot := System _protectedMode .
^ [
    (self _onStackAsSelfBelow: 4) ifTrue:[
      self _error: #rtErrCantBecomeSelfOnStack
    ].
    self __changeClassTo: aClass preserveVarying: aBoolean
  ] ensure:[
    prot _leaveProtectedMode
  ].
%

category: 'Deprecated'
method: Object
_changeToSegment: anObjectSecurityPolicy

self deprecated: 'Object>>_changeToSegment: deprecated v3.0, use #objectSecurityPolicy:'.
^self objectSecurityPolicy: anObjectSecurityPolicy
%

category: 'Private'
method: Object
_checkLogEntriesForNoSelector: selector

"This method returns true only if the RcLogEntries for this transaction do
 NOT contain the specified selector.  If the selector is found then we
 take the conservative approach and fail the transaction because we cannot
 guarantee that it can succeed after performing the redo."

| redoLog logEntries |

redoLog := System _redoLog.
redoLog == nil
  ifTrue: [ ^ false ].

logEntries := redoLog getLogEntriesFor: self.
" if no log entries to replay, then we're done "
logEntries == nil
  ifTrue: [ ^ false ].

1 to: logEntries size do: [ :i |
  (logEntries at: i) selector == selector
    ifTrue: [ ^ false ]
].
^ true
%

category: 'Error Handling'
method: Object
_checkReadStream: aStream forClass: aClass

(aStream collection isKindOf: aClass)
ifFalse:
  [ ^ self _error: #rtErrBadStreamColl args: { aStream . aClass } ]
%

category: 'Class Membership'
method: Object
_class

"Returns the object that is the receiver's class.
 Provided for compatiblity with previous releases.

 _class was not supported in a Maglev Ruby repository."

<primitive: 610 >
^ self _primitiveFailed: #_class .
%

category: 'Testing'
method: Object
_class: aClass
includesSelector: aSymbol

"Used in testing only.  Generates an error if the selector aSymbol is not found
 in the method dictionary of the class aClass when the search for the selector
 begins in the class of the receiver."

| actualImplementor |
actualImplementor := self class whichClassIncludesSelector: aSymbol .
actualImplementor ~~ aClass
ifTrue:
   [ ^ self _error: #classErrSelectorLookup
       args: { actualImplementor . aClass . aSymbol }
   ]
%

category: 'Class Membership'
method: Object
_classHistoryIncludesIdentical: aClass
  self classHistory ifNotNil:[:ch | ^ (ch indexOfIdentical: aClass) ~~ 0 ] .
  ^ false
%

category: 'New Indexing Comparison'
method: Object
_classSortOrdinal

^ 100
%

category: 'New Indexing Comparison'
method: Object
_classSortsGreaterThan: anObject

^self _classSortOrdinal > anObject _classSortOrdinal
%

category: 'New Indexing Comparison'
method: Object
_classSortsLessThan: anObject

^self _classSortOrdinal < anObject _classSortOrdinal
%

category: 'Modification Tracking'
method: Object
_clearModificationTrackingTo: tracker

"Remove the given tracker from the receiver's dependency list."

DependencyList removeTracker: tracker for: self.
%

category: 'Compression'
method: Object
_compress

"If receiver is not a byte format object, or is a Symbol or Number, returns self.
 Otherwise returns a ByteArray with a dynamic instVar #sourceClass containing
 class of the receiver of _compress, and a body that is the receiver compressed
 using zlib.so.
 Use  ByteArray>>_decompress  to decompress the ByteArray."

<primitive: 507>
self _primitiveFailed: #_compress
%

category: 'Copyright'
method: Object
_copyright

"Copyright notice"

^
'Copyright (C) GemTalk Systems 1986-2023.  All Rights Reserved.'
%

category: 'Compression'
method: Object
_decompress

"Returns receiver.  See also ByteArray>>_decompress , Object>>_decompress: "

^ self
%

category: 'Compression'
method: Object
_decompress: inflateFlag

"Return self or an instance of original class with body decompressed
 using zlib.so.
 From Gs64 v3.3 onwards, receiver should always be a ByteArray.
 If receiver is a ByteArray and contains a dynamic instVar #sourceClass ,
 then the value of that instVar is used as the class of the result.

 inflateFlag:
  0 - raw, no headers (inverse of _compress)
  1 - gzip headers (interim format deprecated , was _decompressZipArchive)"


<primitive: 508>
(inflateFlag == 0 or:[ inflateFlag == 1]) ifFalse:[
   inflateFlag _error: #rtErrArgOutOfRange args:{ 0 . 1 }
].
self _primitiveFailed: #_decompress: args: { inflateFlag }
%

category: 'Private'
method: Object
_deepCopyNamedIvsWith: copiedObjDict to: copy

| myClass nFixedIvs |
myClass := self class .
myClass firstPublicInstVar to: (nFixedIvs := self namedSize ) do: [ :i |
  copy instVarAt: i put: ((self instVarAt: i) _deepCopyWith: copiedObjDict).
].
self tagSize ~~ 0 ifTrue:[ |dynIvNames sym |
  dynIvNames := self _instvarNamesAfter: nFixedIvs .
  1 to: dynIvNames size do:[:j |
    sym := dynIvNames at: j .
    copy dynamicInstVarAt: sym
           put: ((self dynamicInstVarAt: sym) _deepCopyWith: copiedObjDict)
  ].
].
%

category: 'Private'
method: Object
_deepCopyWith: copiedObjDict

"Private.

 Used internally to implement deepCopy."

| copy myClass |

self isSpecial ifTrue: [ ^ self ].

copy := copiedObjDict at: self otherwise: nil.
copy ~~ nil ifTrue: [ ^ copy ].

copy := self copy.
copiedObjDict at: self put: copy.

myClass := self class.
myClass isBytes ifTrue: [ ^ copy ].

"make deep copies of all the instance variables"
self _deepCopyNamedIvsWith: copiedObjDict to: copy .

"make deep copies of all the objects in the indexable part"
(myClass isIndexable) ifTrue: [
    1 to: self _basicSize do: [ :i | | element elementCopy |
      element := self basicAt: i.
      elementCopy := element _deepCopyWith: copiedObjDict.
      (element == elementCopy)
        ifFalse: [ copy basicAt: i put: elementCopy ].
    ].
].
^ copy
%

category: 'Private'
method: Object
_dnuError: aSymbol args: anArray reason: dnuKind
 "dnuKind constants must agree with  DNU_ constants in om.hf .
  Not using class variables for DNU_* to avoid polluting name space of Object. "
 | err |
 err := InternalError new .
 dnuKind _isSmallInteger ifTrue:[
   dnuKind == 1 ifTrue:[
     err details: 'method needs recompile, receiver a ',
	  self class name , ' , selector #', aSymbol  ;
	   tag: { self . aSymbol }.
   ] ifFalse:[
     dnuKind == 2 ifTrue:[ err details: 'method load error DNU_failure2' ].
     dnuKind == 3 ifTrue:[ err details: 'method load error DNU_failure3' ].
     dnuKind == 4 ifTrue:[ err details: 'method is not invariant' ].
     dnuKind == 5 ifTrue:[ err details: 'method load error DNU_largeLeafMissing'].
     dnuKind == 6 ifTrue:[ err details:'method load error DNU_badExtrasSize' ].
     dnuKind == 7 ifTrue:[ err details:'method load error DNU_methodDoesNotExist' ].
     err tag:{ self . aSymbol . anArray }.
   ].
 ] ifFalse:[
   err details: 'method oop:' , dnuKind asOop asString, ' needs recompile, receiver a ',
	  self class name , ' , selector #', aSymbol  ;
	   tag: { self . aSymbol }.
 ].
 ^ err signal .
%

category: 'Private'
method: Object
_doesNotUnderstand: aSymbol args: anArray envId: aSmallInt reason: dnuKind
  "This private message is sent by the virtual machine. The implementation must
 be in the persistent method dictionary for Object, since the method is preloaded
 during VM startup.  Reimplementations in a session methods dictionary will not
 be seen."

  dnuKind ~~ 0 ifTrue: [
    "a more severe problem such as method needing recompile"
    self _dnuError: aSymbol args: anArray reason: dnuKind
  ].
  aSmallInt == 0 ifTrue:[
     "this path for compatibility with Gs64 v2.x, and assumes you are using
      only method environment 0  for all of your Smalltalk code."
      ^self doesNotUnderstand: (Message selector: aSymbol arguments: anArray)
  ].
  ^self doesNotUnderstand: aSymbol args: anArray envId: aSmallInt
%

category: 'Error Handling'
method: Object
_errIncorrectFormat: aStreamOrString

""

^ self _error: #rtErrBadFormat args: { aStreamOrString }
%

category: 'Indexing Support'
method: Object
_errorCouldNotCommitDuringIndexCreation

"Raises an error because the receiver is currently in use building an index."

'Failed commit during index creation.'
  _error: #rtErrCommitProhibitingError
%

category: 'Error Handling'
method: Object
_errorExpectedClass: aClass

"Sends an error message indicating that receiver was not a kind of aClass"

self _error: #rtErrBadArgKind args: { aClass } .
self _uncontinuableError
%

category: 'Indexing Support'
method: Object
_errorIndexCreationInProgress

"Raises an error because the receiver is currently in use building an index."

'Operation could not be performed because index creation is in progress.'
  _error: #rtErrCommitProhibitingError
%

category: 'Error Handling'
method: Object
_errorIndexOutOfRange: anIndex

"Sends an error message indicating that anIndex was outside legal limits
 for the receiver."

^ self _error: #objErrBadOffsetIncomplete args: { anIndex }
%

category: 'Error Handling'
method: Object
_errorInvalidOffset: ivName

"The object does not have an instance variable with the given name."

^ self _error: #rtErrObjectInvalidOffset args: { ivName }
%

category: 'Error Handling'
method: Object
_errorNoDependencyList

"The object does not have a dependency list."

^ self _error: #rtErrObjectNoDependencyList
%

category: 'Error Handling'
method: Object
_errorNoModification

"Instances of invariant classes are immutable."

^ self _error: #objErrInvariant
%

category: 'Error Handling'
method: Object
_errorNonIntegerIndex: anIndex

"Sends an error message indicating that anIndex should have been an integer."

^ self _error: #rtErrBadSubscript args: { anIndex }
%

category: 'Error Handling'
method: Object
_errorNotIndexable

"Sends an error message indicating that the receiver is not indexable."

^ OffsetError new _number: 2012 ; reason:#objErrNotIndexable ;
    object: self ; details: 'object does not have varying instVars'; signal
%

category: 'Error Handling'
method: Object
_errorPathTermNotInDependencyList: aPathTerm

"An object did not have the path term in its dependency list."

^ self _error: #rtErrObjectPathTermNotInDependencyList args: { aPathTerm name }
%

category: 'Indexing Support'
method: Object
_errorsDuringPreventCommit: aBlock
 | result |
 [
   result := aBlock value
 ] onException: Error do:[ :ex |
   IndexingErrorPreventingCommit resignal: ex objWithIndex: nil .
 ].
 ^ result
%

category: 'Private'
method: Object
_gciInitialize
  "used by store traversal implementation"

  ^ false "illegal initialization for structural access,
           reimplementations in legal classes return true"
%

category: 'Modification Tracking'
method: Object
_getDepListAndAddTracker: tracker withId: anInteger
  "Add the given tracker to the receiver's dependency list."

  | depList depListClass dl |
  anInteger ~~ 0
    ifTrue: [
      self
        error:
          'DepList id for modification tracking must be 0 or else you risk index corruption: see bug 42640' ].
  depListClass := DependencyList.
  dl := depListClass for: self.
  dl == nil
    ifTrue: [
      depList := depListClass new: 2.
      depList _basicAt: 1 put: tracker.
      depList _basicAt: 2 put: anInteger ]
    ifFalse: [ depList := dl copyAndAddTracker: tracker withId: anInteger for: self ].
  depListClass set: (SharedDependencyLists at: depList logging: true) for: self
%

category: 'Reduced Conflict Support'
method: Object
_getInternalNodes
"PRIVATE. For large objects, return array of internal nodes."

<primitive: 639>
self _primitiveFailed: #_getInternalNodes .
self _uncontinuableError
%

category: 'Encoded OOPs'
method: Object
_getOldOop

"Return a SmallInteger which is the Gs64 v1.1 OOP for the object.
 Return nil if the object is not representable with a 4 byte OOP .
 Special objects will always return nil.
"
<primitive: 544 >
self _primitiveFailed: #_getOldOop
%

category: 'Error Handling'
method: Object
_halt: messageString

"Raises an error in response to a problem detected by kernel class methods.
 Intended for use to report seldom-occurring errors from kernel class
 methods.   The messageString arguments are not yet installed as individual
 messages in the language-dependent message dictionary."

^ InternalError new _number: 2023 ; details: messageString; signal
%

category: 'Modification Tracking'
method: Object
_hasModificationTracking

"Returns true if the receiver's dependency list has any tracking objects;
 returns false otherwise."

| result depList |
depList := DependencyList for: self.
depList == nil
  ifTrue: [ result := false ]
  ifFalse: [ result := depList includesIdentical: 0 ].
^ result
%

category: 'Modification Tracking'
method: Object
_hasModificationTrackingTo: aTrackingObject

"Returns true if the receiver's dependency list has any tracking objects;
 returns false otherwise."

| result depList |
depList := DependencyList for: self.
depList == nil
  ifTrue: [ result := false ]
  ifFalse: [ result := depList includesIdentical: aTrackingObject ].
^ result
%

category: 'Indexing Support'
method: Object
_idxBasicCanCompareWithClass: aClass
  "Returns true if the receiver may be inserted into a basic BtreeNode whose
   #lastElementClass is <aClass> (see RangeEqualityIndex class>>isBasicClass:)."

  ^ self isKindOfClass: aClass
%

category: 'ObsoleteIDX - Indexing Support'
method: Object
_idxCompareEqualTo: arg

self deprecated: 'Object>>_idxCompareEqualTo: should no longer be used.
Use the new double dispatching-based protocol in category "New Indexing Comparison".'.

^ self _idxForCompareEqualTo: arg
%

category: 'ObsoleteIDX - Indexing Support'
method: Object
_idxCompareGreaterThan: arg

self deprecated: 'Object>>_idxCompareGreaterThan: should no longer be used.
Use the new double dispatching-based protocol in category "New Indexing Comparison".'.

^ self _idxForCompareGreaterThan: arg
%

category: 'ObsoleteIDX - Indexing Support'
method: Object
_idxCompareGreaterThanOrEqual: arg

self deprecated: 'Object>>_idxCompareGreaterThanOrEqual: should no longer be used.
Use the new double dispatching-based protocol in category "New Indexing Comparison".'.

^ self _idxForCompareGreaterThanOrEqualTo: arg
%

category: 'ObsoleteIDX - Indexing Support'
method: Object
_idxCompareLessThan: arg

self deprecated: 'Object>>_idxCompareLessThan: should no longer be used.
Use the new double dispatching-based protocol in category "New Indexing Comparison".'.
^ self _idxForCompareLessThan: arg
%

category: 'ObsoleteIDX - Indexing Support'
method: Object
_idxCompareLessThanOrEqual: arg

self deprecated: 'Object>>_idxCompareLessThanOrEqual: should no longer be used.
Use the new double dispatching-based protocol in category "New Indexing Comparison".'.

^ self _idxForCompareLessThanOrEqualTo: arg
%

category: 'ObsoleteIDX - Indexing Support'
method: Object
_idxCompareNotEqualTo: arg

self deprecated: 'Object>>_idxCompareNotEqualTo: should no longer be used.
Use the new double dispatching-based protocol in category "New Indexing Comparison".'.

^ self _idxForCompareNotEqualTo: arg
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareBooleanGreaterThanOrEqualToSelf: aBoolean

"second half of a double dispatch call from Boolean>>_idxForCompareGreaterThanOrEqualTo:. Note that aBoolean should be the receiver in any >= comparison"

^false
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareBooleanGreaterThanSelf: aBoolean

"second half of a double dispatch call from Boolean>>_idxForCompareGreaterThan:. Note that aBoolean should be the receiver in any > comparison"

^false
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareBooleanLessThanOrEqualToSelf: aBoolean

"second half of a double dispatch call from Boolean>>_idxForCompareLessThanOrEqualTo:. Note that aBoolean should be the receiver in any <= comparison"

^false
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareBooleanLessThanSelf: aBoolean

"second half of a double dispatch call from Boolean>>_idxForCompareLessThan:. Note that aBoolean should be the receiver in any < comparison"

^false
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareCharacterCollectionGreaterThanOrEqualToSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForCompareGreaterThanOrEqualTo:. Note that aCharacterCollection should be the receiver in any >= comparison"

^false
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareCharacterCollectionGreaterThanSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForCompareGreaterThan:. Note that aCharacterCollection should be the receiver in any > comparison"

^false
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareCharacterCollectionLessThanOrEqualToSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForCompareLessThanOrEqualTo:. Note that aCharacterCollection should be the receiver in any <= comparison"

^false
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareCharacterCollectionLessThanSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForCompareLessThan:. Note that aCharacterCollection should be the receiver in any < comparison"

^false
%

category: 'New Indexing Comparison'
method: Object
_idxForCompareEqualTo: arg

""

^self _idxForSortEqualTo: arg
%

category: 'New Indexing Comparison'
method: Object
_idxForCompareEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ false
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareEqualToDoubleByteSymbol: aSymbol
  "second half of a double dispatch call from DoubleByteSymbol>>_idxForCompareEqualTo:."

  ^ aSymbol == self
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareEqualToNumber: aNumber

"second half of a double dispatch call from Number>>_idxForCompareEqualTo:."

^false
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareEqualToQuadByteSymbol: aSymbol
  "second half of a double dispatch call from QuadByteSymbol>>_idxForCompareEqualTo:."

  ^ aSymbol == self
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareEqualToSymbol: aSymbol
  "second half of a double dispatch call from Symbol>>_idxForCompareEqualTo:."

  ^ aSymbol == self
%

category: 'New Indexing Comparison'
method: Object
_idxForCompareGreaterThan: arg

""

^arg _idxForCompareObjectGreaterThanSelf: self
%

category: 'New Indexing Comparison'
method: Object
_idxForCompareGreaterThan: aCharacterCollection collator: anIcuCollator
  ""

  ^ false
%

category: 'New Indexing Comparison'
method: Object
_idxForCompareGreaterThanOrEqualTo: arg

""

^arg _idxForCompareObjectGreaterThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: Object
_idxForCompareGreaterThanOrEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ false
%

category: 'New Indexing Comparison'
method: Object
_idxForCompareLessThan: arg

""

^arg _idxForCompareObjectLessThanSelf: self
%

category: 'New Indexing Comparison'
method: Object
_idxForCompareLessThan: aCharacterCollection collator: anIcuCollator
  ""

  ^ false
%

category: 'New Indexing Comparison'
method: Object
_idxForCompareLessThanOrEqualTo: arg

""

^arg _idxForCompareObjectLessThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: Object
_idxForCompareLessThanOrEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ false
%

category: 'New Indexing Comparison'
method: Object
_idxForCompareNotEqualTo: arg

""

^ arg _idxForCompareNotEqualToObject: self
%

category: 'New Indexing Comparison'
method: Object
_idxForCompareNotEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ (self _idxForCompareEqualTo: aCharacterCollection collator: anIcuCollator)
    not
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareNotEqualToObject: anObject

"second half of a double dispatch call from Object>>_idxForCompareNotEqualTo:."

(self _classSortOrdinal = anObject _classSortOrdinal)
  ifTrue: [ ^anObject ~= self ].
^true
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareObjectGreaterThanOrEqualToSelf: anObject

"second half of a double dispatch call from Object>>_idxForCompareGreaterThanOrEqualTo:. Note that anObject should be the receiver in any >= comparison"

(self _classSortOrdinal = anObject _classSortOrdinal)
  ifTrue: [ ^anObject >= self ].
^false
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareObjectGreaterThanSelf: anObject

"second half of a double dispatch call from Object>>_idxForCompareGreaterThan:. Note that anObject should be the receiver in any > comparison"

(self _classSortOrdinal = anObject _classSortOrdinal)
  ifTrue: [ ^anObject > self ].
^false

%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareObjectLessThanOrEqualToSelf: anObject

""

(self _classSortOrdinal = anObject _classSortOrdinal)
  ifTrue: [ ^anObject <= self ].
^false
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareObjectLessThanSelf: anObject

""

(self _classSortOrdinal = anObject _classSortOrdinal)
  ifTrue: [ ^anObject < self ].
^false

%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareUndefinedObjectGreaterThanOrEqualToSelf: anUndefinedObject

"second half of a double dispatch call from UndefinedObject>>_idxForCompareGreaterThanOrEqualTo:."

^false
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareUndefinedObjectGreaterThanSelf: anUndefinedObject

"second half of a double dispatch call from UndefinedObject>>_idxForCompareGreaterThan:."

^false
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareUndefinedObjectLessThanOrEqualToSelf: anUndefinedObject

"second half of a double dispatch call from UndefinedObject>>_idxForCompareLessThanOrEqualTo:."

^false
%

category: 'New Indexing Comparison - for Compare'
method: Object
_idxForCompareUndefinedObjectLessThanSelf: anUndefinedObject

"second half of a double dispatch call from UndefinedObject>>_idxForCompareLessThan:."

^false
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortBooleanGreaterThanOrEqualToSelf: aBoolean

"second half of a double dispatch call from Boolean>>_idxForSortGreaterThanOrEqualTo:. Note that aBoolean should be the receiver in any >= comparison"

^aBoolean _classSortsGreaterThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortBooleanGreaterThanSelf: aBoolean

"second half of a double dispatch call from Boolean>>_idxForSortGreaterThan:. Note that aBoolean should be the receiver in any > comparison"

^aBoolean _classSortsGreaterThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortBooleanLessThanOrEqualToSelf: aBoolean

"second half of a double dispatch call from Boolean>>_idxForSortLessThanOrEqualTo:. Note that aBoolean should be the receiver in any <= comparison"

^aBoolean _classSortsLessThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortBooleanLessThanSelf: aBoolean

"second half of a double dispatch call from Boolean>>_idxForSortLessThan:. Note that aBoolean
should be the receiver in any < comparison"

^aBoolean _classSortsLessThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortCharacterCollectionGreaterThanOrEqualToSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForSortGreaterThanOrEqualTo:. Note that aCharacterCollection should be the receiver in any >= comparison"

^aCharacterCollection _classSortsGreaterThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortCharacterCollectionGreaterThanSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForSortGreaterThan:. Note that aCharacterCollection should be the receiver in any > comparison"

^aCharacterCollection _classSortsGreaterThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortCharacterCollectionLessThanOrEqualToSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForSortLessThanOrEqualTo:. Note that aCharacterCollection should be the receiver in any <= comparison"

^aCharacterCollection _classSortsLessThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortCharacterCollectionLessThanSelf: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForSortLessThan:. Note that aCharacterCollection should be the receiver in any < comparison"

^aCharacterCollection _classSortsLessThan: self
%

category: 'New Indexing Comparison'
method: Object
_idxForSortEqualTo: arg

""

^arg _idxForSortEqualToObject: self
%

category: 'New Indexing Comparison'
method: Object
_idxForSortEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ false
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortEqualToCharacterCollection: aCharacterCollection

 "second half of a double dispatch call from CharacterCollection>>_idxForSortEqualTo:."

^ false
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortEqualToNumber: aNumber

"second half of a double dispatch call from Number>>_idxForSortEqualTo:."

^ false
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortEqualToObject: anObject

"second half of a double dispatch call from Object>>_idxForSortEqualTo:."

(self _classSortOrdinal = anObject _classSortOrdinal)
  ifTrue: [ ^anObject = self ].
^false
%

category: 'New Indexing Comparison'
method: Object
_idxForSortGreaterThan: arg

""

^arg _idxForSortObjectGreaterThanSelf: self
%

category: 'New Indexing Comparison'
method: Object
_idxForSortGreaterThan: aCharacterCollection collator: anIcuCollator
  ""

  ^ false
%

category: 'New Indexing Comparison'
method: Object
_idxForSortGreaterThanOrEqualTo: arg

""

^arg _idxForSortObjectGreaterThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: Object
_idxForSortGreaterThanOrEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ false
%

category: 'New Indexing Comparison'
method: Object
_idxForSortLessThan: arg

""

^arg _idxForSortObjectLessThanSelf: self
%

category: 'New Indexing Comparison'
method: Object
_idxForSortLessThan: aCharacterCollection collator: anIcuCollator
  ""

  ^ false
%

category: 'New Indexing Comparison'
method: Object
_idxForSortLessThanOrEqualTo: arg

""

^arg _idxForSortObjectLessThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: Object
_idxForSortLessThanOrEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ false
%

category: 'New Indexing Comparison'
method: Object
_idxForSortNotEqualTo: arg

""

^arg _idxForSortNotEqualToObject: self
%

category: 'New Indexing Comparison'
method: Object
_idxForSortNotEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ (self _idxForSortEqualTo: aCharacterCollection collator: anIcuCollator) not
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortNotEqualToCharacterCollection: aCharacterCollection

"second half of a double dispatch call from CharacterCollection>>_idxForSortNotEqualTo:."

^ true
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortNotEqualToNumber: aNumber

"second half of a double dispatch call from Number>>_idxForSortNotEqualTo:."

^ true
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortNotEqualToObject: anObject

"second half of a double dispatch call from Object>>_idxForSortNotEqualTo:."

(self _classSortOrdinal = anObject _classSortOrdinal)
  ifTrue: [ ^anObject ~= self ].
^true
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortNumberGreaterThanOrEqualToSelf: aNumber

"second half of a double dispatch call from Number>>_idxForSortGreaterThanOrEqualTo:. Note that aNumber should be the receiver in any >= comparison"

^aNumber _classSortsGreaterThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortNumberGreaterThanSelf: aNumber

"second half of a double dispatch call from Number>>_idxForSortGreaterThan:. Note that aNumber should be the receiver in any > comparison"

^aNumber _classSortsGreaterThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortNumberLessThanOrEqualToSelf: aNumber

"second half of a double dispatch call from Number>>_idxForSortLessThanOrEqualTo:. Note that aNumber should be the receiver in any <= comparison"

^aNumber _classSortsLessThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortNumberLessThanSelf: aNumber

"second half of a double dispatch call from Number>>_idxForSortLessThan:. Note that aNumber should be the receiver in any < comparison"

^aNumber _classSortsLessThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortObjectGreaterThanOrEqualToSelf: anObject

"second half of a double dispatch call from Object>>_idxForSortGreaterThanOrEqualTo:. Note that anObject should be the receiver in any >= comparison"

(self _classSortOrdinal = anObject _classSortOrdinal)
  ifTrue: [ ^anObject >= self ].
^anObject _classSortsGreaterThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortObjectGreaterThanSelf: anObject

"second half of a double dispatch call from Object>>_idxForSortGreaterThan:. Note that anObject should be the receiver in any > comparison"

(self _classSortOrdinal = anObject _classSortOrdinal)
  ifTrue: [ ^anObject > self ].
^anObject _classSortsGreaterThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortObjectLessThanOrEqualToSelf: anObject

"second half of a double dispatch call from Object>>_idxForSortLessThanOrEqualTo:. Note that anObject should be the receiver in any <= comparison"

(self _classSortOrdinal = anObject _classSortOrdinal)
  ifTrue: [ ^anObject <= self ].
^anObject _classSortsLessThan: self

%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortObjectLessThanSelf: anObject

"second half of a double dispatch call from Object>>_idxForSortLessThan:. Note that anObject should be the receiver in any < comparison"

(self _classSortOrdinal = anObject _classSortOrdinal)
  ifTrue: [ ^anObject < self ].
^anObject _classSortsLessThan: self

%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortUndefinedObjectGreaterThanOrEqualToSelf: anUndefinedObject

"second half of a double dispatch call from UndefinedObject>>_idxForSortGreaterThanOrEqualTo:. Note that anUndefinedObject should be the receiver in any >= comparison"

^anUndefinedObject _classSortsGreaterThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortUndefinedObjectGreaterThanSelf: anUndefinedObject

"second half of a double dispatch call from UndefinedObject>>_idxForSortGreaterThan:. Note that anUndefinedObject should be the receiver in any > comparison"

^anUndefinedObject _classSortsGreaterThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortUndefinedObjectLessThanOrEqualToSelf: anUndefinedObject

"second half of a double dispatch call from UndefinedObject>>_idxForSortLessThanOrEqualTo:. Note that anUndefinedObject should be the receiver in any <= comparison"

^anUndefinedObject _classSortsLessThan: self
%

category: 'New Indexing Comparison - for Sort'
method: Object
_idxForSortUndefinedObjectLessThanSelf: anUndefinedObject

"second half of a double dispatch call from UndefinedObject>>_idxForSortLessThan:. Note that anUndefinedObject should be the receiver in any < comparison"

^anUndefinedObject _classSortsLessThan: self
%

category: 'Indexing Support'
method: Object
_idxGreaterThanAndLessThanValues: valueArray

"This comparison operation is by the indexing subsystem to determine if the
 receiver is within the given range of values.  The 'valueArray' argument is a
 four-element Array consisting of { val1. bool1. val2. bool2 } .  val1 is the
 lower bound value and bool1 specifies whether equal values are allowed.  val2
 is the upper value and bool2 specifies whether equal values are allowed.
 Returns true if the receiver satisfies the criteria."

(valueArray at: 2)
    ifTrue: [
        (self _idxForCompareGreaterThanOrEqualTo: (valueArray at: 1))
            ifTrue: [
                (valueArray at: 4)
                    ifTrue: [ ^ self _idxForCompareLessThanOrEqualTo: (valueArray at: 3) ]
                    ifFalse: [ ^ self _idxForCompareLessThan: (valueArray at: 3) ]
            ]
            ifFalse: [ ^ false ]
    ]
    ifFalse: [
          (self _idxForCompareGreaterThan: (valueArray at: 1))
            ifTrue: [
                (valueArray at: 4)
                    ifTrue: [ ^ self _idxForCompareLessThanOrEqualTo: (valueArray at: 3) ]
                    ifFalse: [ ^ self _idxForCompareLessThan: (valueArray at: 3) ]
            ]
            ifFalse: [ ^ false ]
    ]
%

category: 'Dynamic Smalltalk Instance Variables'
method: Object
_instvarNamesAfter: privateSize
"Returns an Array of Symbols which are names of the
 fixed and dynamic instVars of the receiver excluding
 the first privateSize named instVars.  Naming conventions for
 the instance variables are responsibility of the method compiler and other
 runtime support.  A dynamic instVar whose value is remoteNil will be
 excluded from the result."

<primitive: 770>
privateSize _validateClass: SmallInteger .
self _primitiveFailed: #_instvarNamesAfter:  "possible corrupt dynIvs"
     args: { privateSize }
%

category: 'Class Membership'
method: Object
_isArray
"(Optimized selector.)  Returns true if the receiver is an instance of
 Array or a subclass thereof, returns false otherwise."

^ self _isArray
%

category: 'Class Membership'
method: Object
_isExceptionClass

"(Optimized selector.)  Returns true if the receiver is a subclass of
 Exception , returns false otherwise."

^ self _isExceptionClass
%

category: 'Class Membership'
method: Object
_isExecBlock
"(Optimized selector.)  Returns true if the receiver is an instance of
 ExecBlock or a subclass thereof, returns false otherwise."

^ self _isExecBlock
%

category: 'Class Membership'
method: Object
_isFloat

"(Optimized selector.)  Returns true if the receiver is an
  instance of Float or SmallDouble. Returns false otherwise.

 This selector is optimized by the compiler and may not be reimplemented
 in any subclass.  This implementation is so that perform will work."

^ self _isFloat
%

category: 'Testing'
method: Object
_isIndexObject

"Returns true if the receiver is an index object; returns false otherwise."

^ false
%

category: 'Class Membership'
method: Object
_isInteger

"(Optimized selector.)  Returns true if the receiver is a kind of
 Integer. Returns false otherwise.

 This selector is optimized by the compiler and may not be reimplemented
 in any subclass.  This implementation is so that perform will work."

^ self _isInteger
%

category: 'Testing'
method: Object
_isLarge

"Returns true if the object is implemented as a tree of private smaller objects"

^ ( (self _status: false) bitShift: -8) ~~ 0
%

category: 'Class Membership'
method: Object
_isNumber

"(Optimized selector.)  Returns true if the receiver is a kind of
 Number. Returns false otherwise.

 This selector is optimized by the compiler and may not be reimplemented
 in any subclass.  This implementation is so that perform will work."

^ self _isNumber
%

category: 'Testing'
method: Object
_isObsoletePathTerm

"Returns true if the receiver is an obsolete path term; returns false
 otherwise."

^ false
%

category: 'Class Membership'
method: Object
_isOneByteString
"(Optimized selector.)  Returns true if the receiver is an instance of
 String or a subclass thereof, returns false otherwise."

^ self _isOneByteString
%

category: 'Class Membership'
method: Object
_isRange
"(Optimized selector.)  Returns true if the receiver is an instance of
 Range or a subclass thereof, returns false otherwise."

^ self _isRange
%

category: 'Testing'
method: Object
_isRcIdentityBag

"Returns true if the receiver is an RcIdentityBag; returns false otherwise."

^ false
%

category: 'Class Membership'
method: Object
_isScaledDecimal

"(Optimized selector.)  Returns true if the receiver is a kind of
 ScaledDecimal. Returns false otherwise.

 This selector is optimized by the compiler and may not be reimplemented
 in any subclass.  This implementation is so that perform will work."

^ self _isScaledDecimal
%

category: 'Class Membership'
method: Object
_isSmallInteger

"(Optimized selector.)  Returns true if the receiver is an instance of
 SmallInteger. Returns false otherwise.

 This selector is optimized by the compiler and may not be reimplemented
 in any subclass.  This implementation is so that perform will work."

^ self _isSmallInteger
%

category: 'Class Membership'
method: Object
_isSymbol

"(Optimized selector.)  Returns true if the receiver is an instance of
 Symbol, DoubleByteSymbol or QuadByteSymbol. Returns false otherwise.

 This selector is optimized by the compiler and may not be reimplemented
 in any subclass.  This implementation is so that perform will work."

^ self _isSymbol
%

category: 'Testing'
method: Object
_levels

"If the object is implemented as a tree of private smaller objects, returns
 the depth of the tree not including the leaf nodes, otherwise returns 0.

 The result will be a SmallInteger in the range 0..3  inclusive."

^ ((self _status: false) bitShift: -8)
%

category: 'Locking Support'
method: Object
_lockableValues

"Returns a kind of object usable as an argument to _lockAll: primitives."

^ self
%

category: 'Adding'
method: Object
_nextPut: anObject
"see PrintStream>>_nextPut:"

^ self nextPut: anObject
%

category: 'Private'
method: Object
_objectSecurityPolicy: anObjectSecurityPolicy

"Reassigns the receiver to anObjectSecurityPolicy.
 If the receiver's current security policy and anObjectSecurityPolicy are the same,
 this method has no effect, otherwise all of the following apply:

 Generates an error if the user is not authorized to write to both
 the receiver's current security policy and anObjectSecurityPolicy,
 or if the receiver is a special object (SmallInteger, AbstractCharacter,
 Boolean, SmallDouble, or UndefinedObject),
 or if anObjectSecurityPolicy is not a committed object.

 anObjectSecurityPolicy may be nil, in which case the object is given World write
 permission (objectSecurityPolicyId 0).  "

<protected primitive: 594>

(self isSpecial or:[ self isSymbol]) ifTrue:[
   self _error: #segErrCantMoveObj
].
anObjectSecurityPolicy _validateClass: GsObjectSecurityPolicy.

self _primitiveFailed: #_objectSecurityPolicy:
     args: { anObjectSecurityPolicy } .
self _uncontinuableError
%

category: 'Testing'
method: Object
_onStackAsSelfBelow: startLevel

"Returns true if the receiver exists as self anywhere on the current
 GemStone Smalltalk stack above specified Activation; returns false otherwise.

 startLevel is the Activation on the stack at which to begin searching
 downwards.  1 means top of stack, which will be the Activation of
 the sender of  _onStackAsSelfBelow: .  (Since _onStackAsSelfBelow: is
 a primitive, it has no Activation).

 startLevel < 1 is interpreted as  startLevel == 1.

 See Object|become: for a typical use of this method."

<primitive: 288>
startLevel _validateClass: SmallInteger.
self _primitiveFailed: #_onStackAsSelfBelow: args: { startLevel } .
^ false
%

category: 'Message Handling'
method: Object
_perform: aSelectorSymbol env: environmentId withArguments: anArray

"Sends the receiver the message indicated by the arguments.
 The argument, aSelectorSymbol, is the keyword selector of the message.
 The arguments of the message are the elements of anArray.  Generates an
 error if the number of arguments expected by aSelectorSymbol is not
 the same as the number of elements in anArray.

 environmentId must be a SmallInteger >= 0 and <= 255,
 specifying a method lookup environment."


<primitive: 2015> "fail if selector not found, or wrong number of arguments"

aSelectorSymbol _isSymbol ifFalse:[ aSelectorSymbol _validateClass: Symbol].
anArray _isArray ifFalse:[ anArray _validateInstanceOf: Array].

"now implement the failure so that forwarders can reimplement the
 failure method and send self doesNotUnderstand: ."

^ self cantPerform: aSelectorSymbol withArguments: anArray env: environmentId
%

category: 'Message Handling'
method: Object
_performNoDebug: aSelectorSymbol env: environmentId

"Send the receiver the message indicated by the arguments.  The GemStone
 Smalltalk debugger, GemStone Smalltalk breakpoints, soft breaks, and hard
 breaks are all disabled during evaluation of this send.

 If an error occurs during this send, then this session is
 not allowed to commit.  This behavior may be used to ensure consistency
 of indexes, hash dictionaries, etc.

 zero to 10 `with:' keywords are supported by the primitive.
 International selectors are not supported by this method."

<primitive: 305>

aSelectorSymbol _validateClass: String.
self _error: #rtErrCantPerform args: { aSelectorSymbol } .
^ self _primitiveFailed: #_performNoDebug:env:
       args: { aSelectorSymbol . environmentId }
%

category: 'Message Handling'
method: Object
_performNoDebug: aSelectorSymbol env: environmentId with: anArg

"Send the receiver the message indicated by the arguments.  The GemStone
 Smalltalk debugger, GemStone Smalltalk breakpoints, soft breaks, and hard
 breaks are all disabled during evaluation of this send.

 If an error occurs during this send, then this session is
 not allowed to commit.  This behavior may be used to ensure consistency
 of indexes, hash dictionaries, etc.

 zero to 10 `with:' keywords are supported by the primitive.
 International selectors are not supported by this method."

<primitive: 305>

aSelectorSymbol _validateClass: String.
self _error: #rtErrCantPerform args: { aSelectorSymbol } .
^ self _primitiveFailed: #_performNoDebug:env:with:
       args: { aSelectorSymbol . environmentId . anArg }
%

category: 'Message Handling'
method: Object
_performNoDebug: aSelectorSymbol env: environmentId with: anArg1 with: anArg2

"Send the receiver the message indicated by the arguments.  The GemStone
 Smalltalk debugger, GemStone Smalltalk breakpoints, soft breaks, and hard
 breaks are all disabled during evaluation of this send.

 If an error occurs during this send, then this session is
 not allowed to commit.  This behavior may be used to ensure consistency
 of indexes, hash dictionaries, etc.

 zero to 10 `with:' keywords are supported by the primitive.
 International selectors are not supported by this method."

<primitive: 305>

aSelectorSymbol _validateClass: String.
self _error: #rtErrCantPerform args: { aSelectorSymbol } .
^ self _primitiveFailed: #_performNoDebug:env:with:with:
       args: { aSelectorSymbol . environmentId . anArg1 . anArg2 }
%

category: 'Message Handling'
method: Object
_performNoDebug: aSelectorSymbol env: environmentId with: anArg1 with: anArg2 with: anArg3

"Sends the receiver the message indicated by the arguments.  The GemStone
 Smalltalk debugger, GemStone Smalltalk breakpoints, soft breaks, and hard
 breaks are all disabled during evaluation of this send.

 If an error occurs during this send, then this session is not allowed to
 commit.  This behavior may be used to ensure consistency of indexes and hash
 dictionaries.

 zero to 10 `with:' keywords are supported by the primitive.
 International selectors are not supported by this method."

<primitive: 305>

aSelectorSymbol _validateClass: String.
self _error: #rtErrCantPerform args: { aSelectorSymbol } .
^ self _primitiveFailed: #_performNoDebug:env:with:with:with:
       args: { aSelectorSymbol . environmentId . anArg1 . anArg2 . anArg3 }
%

category: 'Accessing'
method: Object
_primitiveAt: anIndex

"Signals an error if the if anIndex is less than 1 or greater than
 the sum of named plus indexed instance variables of the receiver.

 Equivalent to GciFetchOop(self, anIndex) if receiver is a pointer object or
 NSC object.  Equivalent to GciFetchByte(self, anIndex) if receiver is a byte
 object, in which case it returns the small integer value of the byte."

<primitive: 89>

(anIndex _isInteger)
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].

self _primitiveFailed: #_primitiveAt: args: { anIndex } .
self _uncontinuableError
%

category: 'Updating'
method: Object
_primitiveAt: anIndex put: aValue

"Signals an error if anIndex is less than 1 or greater than the sum
 of 1 plus the number of named plus the number of indexed instance
 variables of the receiver.

 Equivalent to GciStoreOop(self, anIndex, aValue) if receiver is a pointer
 object.  Equivalent to GciStoreByte(self, anIndex, aValue) if receiver is a
 byte object.

 Disallowed in Bag."

<primitive: 281>

(anIndex _isInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
((anIndex < 1) | (anIndex > (self _primitiveSize + 1))) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].
(self class isBytes)
ifTrue:
  [ ((aValue class ~~ SmallInteger) or: [ (aValue < 0) | (aValue > 255) ])
    ifTrue: [^ aValue _error: #rtErrExpectedByteValue].
  ].

self _primitiveFailed: #_primitiveAt:put: args: { anIndex . aValue } .
self _uncontinuableError
%

category: 'Accessing'
method: Object
_primitiveAtNoFault: anIndex otherwise: defaultValue

"Equivalent to _primitiveAt: except that most object faults from the page cache
 into VM object memory are avoided.

 If receiver is a pointer object and the instVar value is a committed object,
 this method usually returns defaultValue thus avoiding most object faults.
 If instVar value was recently committed by this session, a non-default value
 will be returned if that reference has not yet been stubbed by in-memory GC.
 If instVar value is a committed object in memory but needs to be refreshed due to
 crossing a transaction boundary an object fault may be performed and
 a non-default value returned.

 If receiver is a byte object, it always returns the small
 integer value of the specified byte."

<primitive: 998>
(anIndex _isInteger)
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].

self _primitiveFailed: #_primitiveAt: args: { anIndex } .
self _uncontinuableError
%

category: 'Private'
method: Object
_primitiveBecome: anObject forDict: isDictBool

"This method should only be invoked by Object/become: , or
 from a method such that neither receiver nor anObject are on stack as self.

 Returns anObject.

 This primitive will signal an error if either receiver or anObject are
   a special object,
   or a kind of one of:
     ExecBlock, Symbol, Behavior, GsNMethod,   SymbolList,
     GsSocket, Regexp, GsCompilerIRNode, CBuffer, CByteArray,
     CFunction, CLibrary, CPointer, CZstream
   or is AllSymbols and session not logged in as SystemUser ,
   or participates in an index.

 If isDictBool == false will signal an error if either receiver or anObject are a
   GsMethodDictionary or  SymbolDictionary .

 If either receiver or anObject is a GsFile with isClient==true,
  signals an error .
 If either receiver or anObject is a RubySocket being polled by the
 ProcessorScheduler, signals an error .
 If receiver or anObject is one of GsFile or RubySocket, then
 anObject must also be a GsFile or RubySocket.

 Primitive fails if one of receiver or anObject are DbTransient
 and the other is not.
"
<protected primitive: 277>
| myCls argCls |
Object _checkKindForBecome: self .
Object _checkKindForBecome: anObject .
(myCls := self class) _instancesNpDbtransient ==
 (argCls := anObject class) _instancesNpDbtransient ifFalse:[
    myCls instancesNonPersistent == argCls instancesNonPersistent ifFalse:[
      self _error: #rtErrCantBecomeGeneric args: {
         'classes of objects differ with respect to instancesNonPersistent' }.
    ].
    myCls instancesDbTransient == argCls instancesDbTransient ifFalse:[
      self _error: #rtErrCantBecomeGeneric args: {
         'classes of objects differ with respect to instancesDbTransient' }.
    ].
].
self _hasDependencyList ifTrue:[
  self _error: #rtErrCantBecomeGeneric args: { 'participates in an index' }.
].
anObject _hasDependencyList ifTrue:[
  anObject _error: #rtErrCantBecomeGeneric args: { 'participates in an index' }.
].
self _primitiveFailed: #_primitiveBecome:forDict:
     args: { anObject . isDictBool } .
self _uncontinuableError
%

category: 'Error Handling'
method: Object
_primitiveFailed: aSelector

"Methods which are implemented as primitives send _primitiveFailed:
 when a primitive fails and the failure is not attributable to any
 normal error such as bad argument kind, argument out of range, etc."

^ self _primitiveFailed: aSelector args: { }
%

category: 'Error Handling'
method: Object
_primitiveFailed: aSelector args: primArgs

"Methods which are implemented as primitives send _primitiveFailed:
 when a primitive fails and the failure is not attributable to any
 normal error such as bad argument kind, argument out of range, etc."

| args |
args := { self . aSelector } .
args add: primArgs.
^ ArgumentError new _number: 2258"#rtErrPrimFailed" ; args: args ; signal
%

category: 'Reduced Conflict Support'
method: Object
_primitiveSelectiveAbort

"Performs an abort operation on the receiver. That is, if the object is
 committed, it removes any changes made by the current transaction and allows
 access to the committed state of the object.

 The error #rtErrSelectiveAbort is thrown if the receiver has depMap entries."

<primitive: 274>

self _primitiveFailed: #_primitiveSelectiveAbort .
self _uncontinuableError
%

category: 'Accessing'
method: Object
_primitiveSize

"Returns the number of named instance variables plus the number of
 indexed instance variables in the receiver.
 Equivalent to GciFetchSize(self)."

<primitive: 88>

self _primitiveFailed: #_primitiveSize .
self _uncontinuableError
%

category: 'Reduced Conflict Support'
method: Object
_rcAt: anIndex

"Returns the object at the given index (same as _at:, _basicAt:,
 and at:).  If the receiver is not a large object, places the receiver
 in the rcReadSet.  If the receiver is a large object or NSC, places
 the root object and any large object nodes that are accessed in the
 rcReadSet.  Does not place the value that it returns in the rcReadSet."

<primitive: 92>
(self class isIndexable) "not an indexable object"
  ifFalse: [ ^ self _errorNotIndexable].
(anIndex _isInteger)
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #_rcAt: args: { anIndex } .
self _uncontinuableError
%

category: 'Reduced Conflict Support'
method: Object
_refreshAfterCommitFailure

"Returns whether the receiver should be selectively aborted when there is a
 failed attempt to commit.  Only special Reduced Objects should answer true."

^ false
%

category: 'Private'
method: Object
_resetParentRef

"Private. After a become:, child objects may need to be adjusted
 to point to the new parent.  Reimplemented in some dictionary classes."

^ self
%

category: 'Reduced Conflict Support'
method: Object
_resolveRcConflictsWith: conflictObjects

"A logical write-write conflict has occurred on the receiver.  The objects that
 had the actual physical write-write conflicts are in the conflictObjects
 Array.  Selectively abort the receiver and then attempt to replay the
 operations maintained in the System redo log.  Returns true if all the
 operations could be replayed; otherwise returns false."

^ self _abortAndReplay: conflictObjects
%

category: 'Testing'
method: Object
_respondsTo: aSymbol private: includePrivateBoolean flags: flags

"Returns true if receiver understands specified selector, false otherwise.
 flags is a Smallinteger with bit masks
    environmentId                  16rFF
    ruby lookup semantics         16r100
    ruby receiver is self        16r1000 (for future use)
    cache successes in code_gen 16r10000
    all other bits ignored
"
<primitive: 688>
aSymbol _isSymbol ifFalse:[
  ArgumentError signal:'argument to responds_to is not a Symbol'
].
self _primitiveFailed:#_respondsTo:private:flags:
     args: { aSymbol . includePrivateBoolean . flags }
%

category: 'Reduced Conflict Support'
method: Object
_selectiveAbort

"Performs an abort operation on the receiver. That is, if the object is
 committed, it removes any changes made by the current transaction and allows
 access to the committed state of the object.  Has no effect on objects
 that have not been committed in an outer level transaction .
 Signals an Error if within a nested transaction and the operation would change
 the state of the receiver.
 "

self isSpecial
  ifTrue:[ ^ self ].
^ self _primitiveSelectiveAbort.
%

category: 'Modification Tracking'
method: Object
_setModificationTrackingTo: tracker

"Adds the given tracker to the dependency list of the receiver."

self isSpecial
  ifFalse: [
    self _getDepListAndAddTracker:tracker withId: 0
  ].
%

category: 'Private'
method: Object
_setNoStubbing

"While the object is in-memory, prevent stubbing of references from
 the receiver to in-memory committed objects.

 Has no effect if receiver is special , or is byte format.
 Otherwise the receiver must be a small object or an error is generated.
 Returns the receiver."

<primitive: 521>

self _primitiveFailed: #_setNoStubbing
%

category: 'Testing'
method: Object
_status: checkClosureBool

"Returns a SmallInteger with status information about the receiver
 encoded in the bit whose masks are defined below:

 16r01 isSpecial
     Set if the receiver is an AbstractCharacter, Boolean,
     SmallInteger, or nil.
 16r02 isCommitted
     Set if the receiver existed in GemStone at the time the current
     transaction began.
     if checkClosureBool is true, and session's most recent commit
     failed and is not yet aborted, check the closureList to determine actual
     committed state of temporary objects in the closure of the commit.
 16r04 isConnected  (obsolete bit, always 0)
 16r08 isWritten
     Set if the receiver has been written since the last
     commit, abort, or begin transaction command was executed.
 16r10 isWritable
     Set if the receiver canBeWritten by the current user.
 16r20 isTranlogged - true isCommitted and tranlogged, or
     not committed and beTranlogged has been sent .
 16r40 beTranlogged has been sent to the object while in-memory.

 16r700 levels  ( > 0 means a 'large object' implemented as a tree of
		smaller objects which are not visible to Smalltalk)"

<primitive: 39>
self _primitiveFailed: #_status: args: { checkClosureBool } .
self _uncontinuableError
%

category: 'Class Membership'
method: Object
_stringCharSize
"Optimized selector with following semantics.
  | result |
  result := 0  .
  (receiver's class is subclass of String or ByteArray) ifTrue:[ result := 1].
  ( receiver's class is subclass of   DoubleByteString) ifTrue:[ result := 2 ].
  (receiver's class is subclass of   QuadByteString) ifTrue:[ result := 4 ].
  ( receiver is a kind of Unicode7, Unicode16, or Unicode32 ) ifTrue:[
       result  := result bitOr: 16r8
  ].
  ^ result
"
^ self _stringCharSize
%

category: 'Deprecated'
method: Object
_tagAt: tagNum

"Deprecated in Gs64 v3.0, use dynamicInstVarAt: instead.

 Returns the receiver's value for the specified tagNum (1 or 2).
 Returns nil if the specified tag has never been stored into. "

self deprecated: 'Object>>_tagAt: deprecated v3.0, use #dynamicInstVarAt:'.
^self _tagAtPrim: tagNum.
%

category: 'Deprecated'
method: Object
_tagAt: tagNum put: tagValue

"Sets the specified tag word to the specified value .
 Deprecated in Gs64 v3.0:
   Use dynamicInstVarAt:put: instead.
   tagNum may be from   1   to   self tagSize + 1 .
   tagSize + 1 will cause auto grow .
   Number of tags is limited to 512, (2034 - number of named instVars)
   whichever is less.

 new in Gemstone64:
   Tags may not be stored into kind of Behavior
   (i.e.  tags on Classes and  metaclasses are not supported).
   Tags may not be stored into instances of GsMethod or ExecBlock.
 new in Gemstone64 v3.0:
   If you want the old semantics of Object>>copy not copying tags,
   you must use Object>>_basicCopyWithoutTags .

 Tags may not be stored into special objects.
 Generates an error if the receiver does not support tags.
"

self deprecated: 'Object>>_tagAt:put: deprecated v3.0, use #dynamicInstVarAt:put:'.
^self _tagAtPrim: tagNum put: tagValue.
%

category: 'Deprecated'
method: Object
_tagAtPrim: tagNum

<primitive: 87>
tagNum _isSmallInteger ifFalse:[ ^ tagNum _validateClass: SmallInteger ]
   ifTrue:[ ^ OutOfRange new name:'tagNum' min: 1 max: (2 max: self tagSize) actual: tagNum ;
     _number: 2055; signal] .
^ self _primitiveFailed: #_tagAt: args: { tagNum } .
%

category: 'Deprecated'
method: Object
_tagAtPrim: tagNum put: tagValue

<primitive: 280>
self isSpecial ifTrue:[
  ImproperOperation new _number: 2054 ; signal:'tag not allowed on special object'
].
tagNum _isSmallInteger ifFalse:[ tagNum _validateClass: SmallInteger ]
   ifTrue:[ ^ OutOfRange new name:'tagNum' min: 1 max: 3 actual: tagNum ; signal ].
^ self _primitiveFailed: #_tagAt:put: args: { tagNum . tagValue } .
%

category: 'Formatting'
method: Object
_topazAsString
  ^ [ self asString
    ] onSynchronous: AbstractException do:[:ex |
        '<error during asString>'
    ]
%

category: 'Message Handling'
method: Object
_topazPerform: selStr withArgs: argsArray env: envId
  "used by topaz SENDENV command."
  | selSym |
  (selSym := Symbol _existingWithAll: selStr ) ifNil:[
     ArgumentError signal:'selector is not an existing Symbol'
  ].
  ^ self perform: selSym env: envId withArguments: argsArray
%

category: 'Error Handling'
method: Object
_uncontinuableError

"An attempt was made to continue execution past an uncontinuable error."

[true] whileTrue:[ self _error: #rtErrUncontinuable args: { } ]
%

category: 'Private'
method: Object
_unsafeAt: anIndex
"Low level access to an object, node of a large object , or node of
 an IdentityBag
 Named instance variables are accessed by this method, so 
 unnamed instance variables begin at (self class instSize + 1).

 Returns specified instVar of the object.  If object is byte format,
 returns SmallInteger value of the specified byte.
 If anIndex is < 1 , the primtive fails.
 If anIndex is beyond the end of the object, returns OOP_REMOTE_NIL"

 <primitive: 168>
 (anIndex _isInteger)
   ifFalse: [ self _errorNonIntegerIndex: anIndex .  self _uncontinuableError ].
 self _primitiveFailed: #_unsafeAt: args: { anIndex } .
 self _uncontinuableError
%

category: 'Instance Migration'
method: Object
_unsafeAt: anIndex put: aValue

"Updates the receiver at the given index.  This is to be
 used for upgrading GemStone releases and for schema migration only.  Named
 instance variables are considered to be indexed by this method, so an
 unnamed part would begin at (self class instSize + 1).

 If anIndex is less than 1 or greater than 1 plus the total number of
 instance variables in the receiver, an error is generated.

 If the receiver is a kind of Bag, then only named instance variables may be
 modified by this method.  Take care not to corrupt the private named instance
 variables defined in class Bag.

 If the receiver is a byte format object, aValue must be a SmallInteger 
 in the range 0..255, which will be stored into the byte specified by anIndex .

 This method does not check invariance.  However, Indexes are updated
 if some index is dependent upon the instance variable that is modified."

<primitive: 273>

(anIndex _isInteger)
  ifFalse: [ self _errorNonIntegerIndex: anIndex .  self _uncontinuableError ].
(self class isBytes)
ifTrue:
  [ ((aValue class ~~ SmallInteger) or: [ (aValue < 0) | (aValue > 255) ])
    ifTrue: [ ^ aValue _error: #rtErrExpectedByteValue].
  ] .
self _primitiveFailed: #_unsafeAt:put: args: { anIndex . aValue } .
self _uncontinuableError
%

category: 'Class Membership'
method: Object
_validateByteClass: aClass

"Returns true if self is a kind of aClass and also has byte format.
 Otherwise generates an error."

(self isKindOf: aClass) ifFalse:[
  self _errorExpectedClass: aClass  .
  ^ false
].
self class isBytes ifFalse:[
  self _error: #objErrNotByteKind .
  ^ false
].
^ true
%

category: 'Testing'
method: Object
_validateCharSizeIs: anInt

^ self charSize == anInt
     ifTrue:[ true ]
    ifFalse:[ ArgumentError signal: ('Expected an object with char size ',
               anInt asString, ' rather than ', self charSize asString)]
%

category: 'Class Membership'
method: Object
_validateClass: aClass

"Returns true if self is a kind of aClass.  Otherwise, generates an error."

(self isKindOf: aClass) ifTrue: [ ^ true ].
self _errorExpectedClass: aClass .
%

category: 'Class Membership'
method: Object
_validateClasses: anArray

"Returns true if self is a kind of a class in anArray.  Otherwise, generates an
 error."

1 to: anArray size do: [:i |
   (self isKindOf: (anArray at: i)) ifTrue:[ ^ true ]
].
self _error: #rtErrInvalidArgClass args: { anArray } .
^ false
%

category: 'Class Membership'
method: Object
_validateInstanceOf: aClass

"Returns true if self is instance of aClass.  Otherwise, generates an error."

(self class == aClass) ifFalse:[
   self _errorExpectedClass: aClass .
   ^ false
].
^ true
%

category: 'Class Membership'
method: Object
_validateIsBytes

"Returns true if self has byte format.
 Otherwise generates an error."

self class isBytes ifFalse:[
  self _error: #objErrNotByteKind .
  ^ false
].
^ true
%

category: 'Testing'
method: Object
_validateIsClass

"Generates an error if the receiver is not a kind of Behavior, with
 error message specific to class modification attempt."

(self isKindOf: Behavior ) ifTrue:[ ^ true ] .
self _error: #rtErrConstrNotAClass .
^ false
%

category: 'Class Membership'
method: Object
_validateKindOfClass: aClass

"Returns true if self is a kind of aClass, ignoring the classHistory
 of aClass.  Otherwise, generates an error."

(self isKindOfClass: aClass) ifTrue: [ ^ true ].
self _errorExpectedClass: aClass .
%

category: 'Class Membership'
method: Object
_validateKindOfClasses: anArray

"Returns true if self is a kind of any class in anArray, ignoring the classHistory
 of the class.  Otherwise, generates an error."

anArray detect:[:eachClass| self isKindOfClass: eachClass]
  ifNone:[self _errorExpectedClass: anArray first ].

^ true
%

category: 'Queries'
method: Object
_validateRcConflictsWith: conflictObjects

"The default validation for Rc conflicts is to return true.
 This satisfies the cases of changes to the root object"

^ true
%

category: 'Private'
method: Object
__changeClassTo: aClass preserveVarying: aBoolean

"Redefines the class of the receiver to be aClass.
 Checks 1..5 documented for changeClassTo:  are done in the primitive."

<protected primitive: 270>
aClass _validateClass: Class.  "argument must be a Class"
aBoolean _validateClass: Boolean .
self _error: #rtErrCantChangeClass args: { aClass . ''  }
%

category: 'Comparing'
method: Object
~= anObject

"Returns true if the receiver and the argument do not have the same
 value."

^ (self = anObject) == false
%

category: 'Comparing'
method: Object
~~ anObject

"(Optimized selector.)  Returns true if the receiver and the argument are not
 the same object.

 This selector is optimized by the compiler and may not be reimplemented
 in any subclass. This implementation is so that perform will work."

^ self ~~ anObject
%

! Class extensions for 'ObjectsCommittedNotification'

!		Class methods for 'ObjectsCommittedNotification'

removeallmethods ObjectsCommittedNotification
removeallclassmethods ObjectsCommittedNotification

category: 'Signalling'
classmethod: ObjectsCommittedNotification
disableSignalling

"Prevent instances of ObjectsCommittedNotification from being signalled by the VM."

 System _updateSignalErrorStatus: 1 toState: false
%

category: 'Signalling'
classmethod: ObjectsCommittedNotification
enableSignalling

"Enable instances of ObjectsCommittedNotification to be signalled by the VM .
 must be resent by exception handling code after handling each
 instance of ObjectsCommittedNotification that was signalled by the VM."

 System _updateSignalErrorStatus: 1 toState: true
%

category: 'Signalling'
classmethod: ObjectsCommittedNotification
signallingEnabled

"Return true if signaling enabled, return false otherwise  .
 See ObjectsCommittedNotification(C)>>enableSignalling "

^ System _signalErrorStatus: 1
%

!		Instance methods for 'ObjectsCommittedNotification'

category: 'Handling'
method: ObjectsCommittedNotification
defaultAction

"Return an error to the controlling GCI client .
 See AbstractException>>defaultAction for documentation."

<primitive: 33>
self _uncontinuableError
%

category: 'Instance initialization'
method: ObjectsCommittedNotification
initialize
  gsNumber := ERR_ObjectsCommittedNotification.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'OffsetError'

!		Instance methods for 'OffsetError'

removeallmethods OffsetError
removeallclassmethods OffsetError

category: 'Instance initialization'
method: OffsetError
args: anArray
  anArray _isArray ifTrue:[ | sz o |
    object := (o := anArray atOrNil: 1) .
    (actual := anArray atOrNil: 2 ) ifNotNil:[
      maximum := (anArray atOrNil: 3) ifNil:[ o ifNotNil:[ o size ]].
      (sz := anArray size) > 3 ifTrue:[
        gsArgs := anArray copyFrom: 4 to: sz
      ].
    ].
  ]
%

category: 'Formatting'
method: OffsetError
buildMessageText
^ self buildMessageText:(
  [ | str |
    str := String new .
    object ifNotNil:[:o | str add: 'accessing a '; add: o class name ].
    maximum ifNotNil:[:m |
      str := String new .
      str add:'max:'; add: m asString .
      actual ifNotNil:[:a| str add: ' actual:'; add: a asString].
    ].
    gsDetails ifNotNil:[:d| str add:', '; add: d  asString ].
    str
  ] onException: Error do:[:ex |
    ex return: nil
  ]
)
%

category: 'Instance initialization'
method: OffsetError
initialize
  gsNumber := ERR_OffsetError.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Instance initialization'
method: OffsetError
offset: anOffset maximum: aMax
  actual := anOffset .
  maximum := aMax
%

category: 'Instance initialization'
method: OffsetError
_legacyHandlerArgs
  | arr |
  arr := { object . maximum . actual } .
  gsArgs ifNotNil:[:a |  arr addAll: a ].
  ^ arr
%

! Class extensions for 'OrderedCollection'

!		Class methods for 'OrderedCollection'

removeallmethods OrderedCollection
removeallclassmethods OrderedCollection

category: 'Instance Creation'
classmethod: OrderedCollection
new: anInteger

"This method is supplied for compatibility with other implementations of
 Smalltalk.  Its behavior differs from other Smalltalks in that it ignores the
 extension and returns an instance with size 0.

 Other Smalltalks use an instance variable to keep track of the number of
 elements in what are fixed-size instances.  GemStone objects are dynamically
 extensible and the size of the object is used to keep track of the number of
 elements in the collection."

anInteger < 0 ifTrue:[ anInteger _error: #rtErrArgNotPositive ].
^ super new .
%

!		Instance methods for 'OrderedCollection'

category: 'Adding'
method: OrderedCollection
add: newObject

"Makes newObject one of the receiver's elements and returns newObject.  The
 new element is actually added as the last element of the receiver.  This
 method is equivalent to #addLast:."

<primitive: 203>
self _primitiveFailed: #add: args: { newObject } .
self _uncontinuableError
%

category: 'Adding'
method: OrderedCollection
add: newObject after: targetObject

"Adds newObject to the receiver immediately following the first element that is
 equal to targetObject, and returns newObject.  If targetObject is not found,
 reports an error."

| index |

index := self indexOf: targetObject
             ifAbsent: [^ self _errorNotFound: targetObject].

^ self insertObject: newObject at: (index + 1).
%

category: 'Adding'
method: OrderedCollection
add: newObject afterIndex: anIndex

"Adds newObject to the receiver at the index immediately following anIndex."

^ self insertObject: newObject at: (anIndex + 1).
%

category: 'Adding'
method: OrderedCollection
add: newObject before: targetObject

"Adds newObject to the receiver immediately before the first element that is
 equal to targetObject, and returns newObject.  If targetObject is not found,
 reports an error."

| index |

index := self indexOf: targetObject
             ifAbsent: [^ self _errorNotFound: targetObject].

^ self insertObject: newObject at: index.
%

category: 'Adding'
method: OrderedCollection
add: newObject beforeIndex: anIndex

"Inserts newObject to the receiver at the index anIndex and returns newObject."

^ self insertObject: newObject at: anIndex.
%

category: 'Adding'
method: OrderedCollection
addAll: aCollection

"Adds all of the elements of aCollection to the receiver and returns
 aCollection."

<primitive: 213>
"primitive handles arg kinds Array, OrderedCollection and IdentityBag."

^ super addAll: aCollection
%

category: 'Adding'
method: OrderedCollection
addAll: aCollection after: targetObject

"Adds each element of aCollection to the receiver immediately following the
 first element of the receiver which is equal to targetObject.  Reports an error
 if targetObject is not found.  Returns aCollection."

| index |

index := self indexOf: targetObject
             ifAbsent: [^ self _errorNotFound: targetObject].

^ self insertAll: aCollection at: (index + 1).
%

category: 'Adding'
method: OrderedCollection
addAll: aCollection afterIndex: anIndex

"Adds each element of aCollection to the receiver immediately after the element
 of the receiver at index anIndex.  Reports an error if targetObject is not
 found.   Returns aCollection."

^ self insertAll: aCollection at: (anIndex + 1).
%

category: 'Adding'
method: OrderedCollection
addAll: aCollection before: targetObject

"Adds each element of aCollection to the receiver immediately before the
 first element of the receiver which is equal to targetObject.  Reports an error
 if targetObject is not found.  Returns aCollection."

| index |

index := self indexOf: targetObject
             ifAbsent: [^ self _errorNotFound: targetObject].

^ self insertAll: aCollection at: index.
%

category: 'Adding'
method: OrderedCollection
addAll: aCollection beforeIndex: anIndex

"Adds each element of aCollection to the receiver immediately before the
 element of the receiver at index anIndex.  Reports an error if targetObject is
 not found.  Returns aCollection."

^ self insertAll: aCollection at: anIndex.
%

category: 'Adding'
method: OrderedCollection
addAllFirst: aCollection

"Inserts the given objects at the beginning of the receiver.  Returns the
 argument, aCollection."

self insertAll: aCollection at: 1.
^aCollection
%

category: 'Adding'
method: OrderedCollection
addAllLast: aCollection

"Appends the objects in aCollection to the receiver.  Returns aCollection."

self addAll: aCollection.
^aCollection
%

category: 'Adding'
method: OrderedCollection
addFirst: anObject

"Inserts the given object at the beginning of the receiver.  Returns the
 inserted object."

self insertObject: anObject at: 1.
^anObject
%

category: 'Adding'
method: OrderedCollection
addLast: newObject

"Adds newObject as the last element of the receiver and returns newObject."

<primitive: 203>
self _primitiveFailed: #addLast: args: { newObject } .
self _uncontinuableError
%

category: 'Copying'
method: OrderedCollection
copyFrom: startIndex to: stopIndex

"Returns a new collection containing the elements of the receiver
 between startIndex and stopIndex, inclusive.  The result is of the same class
 as the receiver.

 If startIndex > stopIndex then an empty collection is returned.
 Otherwise both startIndex and stopIndex must be positive integers not larger than the
 size of the receiver, with startIndex <= stopIndex.
"

<primitive: 817>
(startIndex < 1) ifTrue: [ ^ self _errorIndexOutOfRange: startIndex].

((stopIndex > self size) or: [(stopIndex < 0)])
   ifTrue: [ ^ self _errorIndexOutOfRange: stopIndex].

self _primitiveFailed: #copyFrom:to: args: { startIndex . stopIndex }
%

category: 'Comparing'
method: OrderedCollection
hasIdenticalContents: anArray

"Returns true if all of the following conditions are true:

 1.  The receiver and anArray are of the same class.
 2.  The two Arrays are the same size.
 3.  The corresponding elements of the receiver and anArray are equal.

 Returns false otherwise."

<primitive: 612 >

self _primitiveFailed: #hasIdenticalContents: args: { anArray }.
self _uncontinuableError
%

category: 'Searching'
method: OrderedCollection
includesIdentical: anObject

"Returns true if anObject is identical to one of the elements of the
 receiver.  Returns false otherwise."

<primitive: 59>
self _primitiveFailed: #includesIdentical: args: { anObject }
%

category: 'Searching'
method: OrderedCollection
indexOfIdentical: anObject

"Private.  Returns the index of the first element in the receiver that is
 identical to the argument.  If the receiver does not have any elements that are
 identical to the argument, returns zero."

<primitive: 494>
self _primitiveFailed: #indexOfIdentical: args: { anObject }
%

category: 'Copying'
method: OrderedCollection
replaceFrom: startIndex to: stopIndex with: aSeqCollection startingAt: repIndex

"Replaces the elements of the receiver between the indexes startIndex and
 stopIndex inclusive with the elements of aSeqCollection starting at repIndex.
 If aSeqCollection is identical to the receiver, the source and
 destination blocks may overlap.

 Returns the receiver."

<primitive: 608>
startIndex _isSmallInteger ifFalse:[ startIndex _validateClass: SmallInteger ].
stopIndex _isSmallInteger ifFalse:[ stopIndex _validateClass: SmallInteger ].
repIndex _isSmallInteger ifFalse:[ repIndex _validateClass: SmallInteger ].

^ super replaceFrom: startIndex to: stopIndex with: aSeqCollection startingAt: repIndex
%

category: 'Adding'
method: OrderedCollection
_addAllFromNsc: aBag

""

<primitive: 202>
self _primitiveFailed: #_addAllFromNsc: args: { aBag } .
self _uncontinuableError
%

! Class extensions for 'OutOfRange'

!		Instance methods for 'OutOfRange'

removeallmethods OutOfRange
removeallclassmethods OutOfRange

category: 'Instance initialization'
method: OutOfRange
args: anArray
  anArray _isArray ifTrue:[ | sz |
    object := anArray atOrNil: 1 .
    minimum := anArray atOrNil: 2 .
    maximum := anArray atOrNil: 3 .
    actual := anArray atOrNil: 4 .
    (sz := anArray size) > 4 ifTrue:[
      gsArgs := anArray at: 5 .
    ].
  ]
%

category: 'Formatting'
method: OutOfRange
buildMessageText
^ self buildMessageText:(
  [
    | str |
    str := String new .
    object ifNotNil:[:o |
       o _isOneByteString ifTrue:[ str add: 'for '; add: o quoted]
         ifFalse:[ str add: 'for a '; add: o class name].
    ].
    gsArgs "argName" ifNotNil:[:a|
          str size > 0 ifTrue:[ str add:', ' ].
          str add: ' the value for '; add: a asString ;
		      add: ' is ' ; add: actual asString ]
       ifNil:[ actual ifNotNil: [:a | str add: a asString ] ].
    minimum ifNotNil: [
	   maximum ifNotNil:[ str add: ' should be between ';
                   add: minimum asString; add: ' and '; add: maximum asString ]
		      ifNil: [str add: ' minimum is '; add: minimum asString]
    ] ifNil: [
	     maximum ifNotNil: [str add: ' maximum is '; add: maximum asString]
    ].
    gsDetails ifNotNil: [:arg | str add: ', '; add: arg asString] .
    str
  ] onException: Error do:[:ex |
    ex return: nil
  ]
)
%

category: 'Instance initialization'
method: OutOfRange
expectedNumArgs: aNumber actual: aValue
  minimum := aNumber .
  maximum := aNumber .
  actual := aValue
%

category: 'Instance initialization'
method: OutOfRange
initialize
  gsNumber := ERR_OutOfRange.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Instance initialization'
method: OutOfRange
name: aName max: aMax actual: aValue
  gsArgs := aName .
  maximum := aMax .
  actual := aValue
%

category: 'Instance initialization'
method: OutOfRange
name: aName min: aMin actual: aValue
  gsArgs := aName .
  minimum := aMin .
  actual := aValue
%

category: 'Instance initialization'
method: OutOfRange
name: aName min: aMin max: aMax actual: aValue
  gsArgs := aName .
  minimum := aMin .
  maximum := aMax .
  actual := aValue
%

category: 'Instance initialization'
method: OutOfRange
_legacyHandlerArgs
  | arr |
  arr := { object . minimum . maximum . actual } .
  gsArgs ifNotNil:[:a |  arr add: a ].
  ^ arr
%

! Class extensions for 'PostgresError'

!		Instance methods for 'PostgresError'

removeallmethods PostgresError
removeallclassmethods PostgresError

category: 'Instance initialization'
method: PostgresError
initialize
  gsNumber := ERR_PostgresError.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'PrivateObject'

!		Class methods for 'PrivateObject'

removeallmethods PrivateObject
removeallclassmethods PrivateObject

category: 'Instance Creation'
classmethod: PrivateObject
basicNew
  "disallowed"
  self shouldNotImplement: #basicNew
%

category: 'Instance Creation'
classmethod: PrivateObject
new
  "disallowed"
  self shouldNotImplement: #new
%

category: 'Instance Creation'
classmethod: PrivateObject
_basicNew
  "disallowed"
  self shouldNotImplement: #_basicNew
%

!		Instance methods for 'PrivateObject'

category: 'Converting'
method: PrivateObject
asOop

"Returns the value of the receiver's object-oriented pointer (OOP) as a
 positive Integer .  This is the receiver's unique identifier that
 distinguishes it from all other objects.  For non-special objects
 and for instances of Character, Boolean, UndefinedObject, and JISCharacter
 the result will always be a SmallInteger.  For instances of SmallInteger
 and SmallDouble the result may be a LargeInteger.

 The result is different from the result obtained in Gemstone64 v1.x"

<primitive: 86>
self _primitiveFailed: #asOop .
^ nil
%

category: 'Encoded OOPs'
method: PrivateObject
asOopNumber

"If the receiver is a non-special object, return the receiver's
 oopNumber as defined under OOP_TAG_POM_OOP in the file
 $GEMSTONE/include/gcioop.ht  , otherwise return nil . "

| oop |
oop := self asOop .
oop _isSmallInteger ifFalse:[ ^ nil ].
(oop bitAnd:7) == 1 ifFalse:[ ^ nil ].
^ oop bitShift: -8
%

category: 'Formatting'
method: PrivateObject
asString

^ 'a ' , self class name
%

category: 'Accessing'
method: PrivateObject
basicPhysicalSize

"Returns the number of bytes required to represent the receiver in memory.  If
 the receiver is in special format (which implies that its representation is the
 same as its OOP), returns zero.

 The basicPhysicalSize method returns the same result as the default
 implementation (in class Object) of the physicalSize method.  It makes that
 default implementation available even when the physicalSize method is
 reimplemented in a subclass.  The basicPhysicalSize method should not itself
 be reimplemented."

<primitive: 364>
self _primitiveFailed: #basicPhysicalSize .
^ 0
%

category: 'Accessing'
method: PrivateObject
basicSize

"Instance variables of PrivateObjects are not visible to Smalltalk."
^ 0
%

category: 'Testing'
method: PrivateObject
canBeWritten

"Returns true if the current user has write authorization for the
 receiver, false if not."

^ ((self _status: false)  bitAnd: 16r10) ~~ 0
%

category: 'Class Membership'
method: PrivateObject
class

"Returns the object that is the receiver's class."

<primitive: 610 >
self _primitiveFailed: #class .
^ nil
%

category: 'Formatting'
method: PrivateObject
describe

"Returns an instance of a subclass of CharacterCollection describing the
 receiver.  This method is required by Topaz and by GemStone internal error
 handling.  Any reimplementation must conform to the following rules:

 * This method must not return nil.

 * This method must return an instance of a subclass of CharacterCollection.

  Final version of this method is in bomlastconv.gs .
"
^ 'a ' , self class name
%

category: 'Formatting'
method: PrivateObject
describe1K
  "Final version of this method is in bomlastconv.gs ."
  ^ self describe
%

category: 'Error Handling - Compatibility'
method: PrivateObject
doesNotUnderstand: aMessageDescriptor

"The method is for compatiblity with Gs64 v2.x, and assumes you are using
   only method environment 0  for all of your Smalltalk code."

| ex sel args |
(ex := MessageNotUnderstood _basicNew)
  receiver: self selector: (sel := aMessageDescriptor at: 1)
		args: (args := aMessageDescriptor at: 2) envId: 0 .
^ex signal .	"fix for #40871"

"Legacy behavior was that you could fall through from an exception handler
without an explicit resume/return. The fix for 40871 will break applications
that rely on the old behavior."

"If we continue from the error, re-try the send of the message that was
 not understood."
"^ self perform: sel env: 0 withArguments: args "
%

category: 'Error Handling'
method: PrivateObject
doesNotUnderstand: aSymbol args: anArray envId: envId

"Generates an error reporting that the receiver cannot respond to a message.
 because no compiled method was found for aSymbol in method environment
 envId.   envId is a SmallInteger, 0 for Smalltalk , 1 for Ruby ,
 2..255 for future use by Smalltalk package managers .
"
| ex |
(ex := MessageNotUnderstood _basicNew)
  receiver: self selector: aSymbol args: anArray envId: envId .
^ex signal .	"fix for #40871"

"Legacy behavior was that you could fall through from an exception handler
without an explicit resume/return. The fix for 40871 will break applications
that rely on the old behavior."

"If we continue from the error, re-try the send of the message that was
 not understood."

"^ self perform: aSymbol env: envId withArguments: anArray "
%

category: 'Listing References'
method: PrivateObject
findAllReferences

 "Returns an Array containing all references to the receiver.
 If an object contains multiple references to the receiver, that object occurs
 only once in the result.

 Large result sets may cause out-of-memory issues. To avoid problems,
 use Repository >> allReferences:, which returns a GsBitmap that does not
 require keeping objects in temporary object space.

 This method aborts the current transaction; if an abort would cause unsaved
 changes to be lost, it signals an error, #rtErrAbortWouldLoseData.

 The result contains both permanent and temporary objects.  The temporary
 objects found may vary from run to run. "

  ^ self findReferencesWithLimit: 0
%

category: 'Listing References'
method: PrivateObject
findReferences

"Searches GemStone for objects that reference the receiver, and returns an Array
 of any such objects.  The search continues until all such objects have been
 found, or until the result contains 20 elements.  (The method
 findReferencesWithLimit: allows you to specify an arbitrarily large limit for
 the result Array.)

 Large result sets may cause out-of-memory issues. To avoid problems,
 use Repository >> allReferences:, which returns a GsBitmap that does not
 require keeping objects in temporary object space.

 If an object contains multiple references to the receiver, that object occurs
 only once in the result.

 This method aborts the current transaction; if an abort would cause unsaved
 changes to be lost, it signals an error, #rtErrAbortWouldLoseData.

 The result contains both permanent and temporary objects.  The temporary
 objects found may vary from run to run. "

^self findReferencesWithLimit: 20
%

category: 'Listing References'
method: PrivateObject
findReferencesInMemory

"Returns an Array of objects in current sessions temporary object memory
 that reference the receiver.  If receiver is a committed object,
 the result includes both direct memory references from temporary or
 modified committed objects, and references by objectId from other
 in-memory committed objects.  See also findNonStubbedReferencesInMemory .

 The search continues until all such objects have been found, or until the
 size of the result reaches the specified maximum aSmallInt.

 The result may contain both permanent and temporary objects and may
 vary from run to run.  Does not abort the current transaction.

 Large result sets may cause out-of-memory issues. To avoid problems,
 use Repository >> allReferences:, which returns a GsBitmap that does not
 require keeping objects in temporary object space.
"

| argArray tmpArray resultArr |
argArray := { self } .
tmpArray := SystemRepository listReferencesInMemory: argArray .
resultArr := tmpArray at: 1.
resultArr removeIdentical: argArray ifAbsent:[ ].
^ resultArr
%

category: 'Listing References'
method: PrivateObject
findReferencesWithLimit: aSmallInt

"Returns an Array of objects in GemStone that reference the receiver.
 The search continues until all such objects have been found, or until the
 size of the result reaches the specified maximum aSmallInt.

 Large result sets may cause out-of-memory issues. To avoid problems,
 use Repository >> allReferences:, which returns a GsBitmap that does not
 require keeping objects in temporary object space.

 If aSmallInt is <=  0, the result size is unlimited.

 The result contains both permanent and temporary objects.  The temporary
 objects found may vary from run to run.

 This method aborts the current transaction; if an abort would cause unsaved
 changes to be lost, it signals an error, #rtErrAbortWouldLoseData.

 Note that this method may take a considerable length of time to
 execute, and the result may occupy a large amount of disk space.
 (Compare with findReferences, which limits the result to 20 elements.). "

| argArray tmpArray resultArr limit |

System needsCommit
  ifTrue: [ PrivateObject _error: #rtErrAbortWouldLoseData ] .

argArray := { self } .
limit :=  aSmallInt > 0 ifTrue:[ aSmallInt + 1] ifFalse:[ 0 ] .
tmpArray := SystemRepository listReferences: argArray withLimit: limit.

resultArr := tmpArray at: 1.
resultArr removeIdentical: argArray ifAbsent:[ ].
aSmallInt > 0 ifTrue:[
  resultArr size > aSmallInt ifTrue:[ resultArr size: aSmallInt ].
].
^ resultArr
%

category: 'Comparing'
method: PrivateObject
identityHash

"This method returns some Integer related to the identity of the receiver.  If
 two objects compare identically (==) to each other, the results of sending
identityHash to each of those objects will be equal."

<primitive: 609 >
self _primitiveFailed: #identityHash .
^ nil
%

category: 'Testing'
method: PrivateObject
isBehavior
  ^ false
%

category: 'Testing'
method: PrivateObject
isClass

  ^ false
%

category: 'Testing'
method: PrivateObject
isCommitted

"Returns true if the receiver was a committed object at the time the current
 transaction began.  Returns false otherwise."

^((self _status: true) bitAnd: 2) ~~ 0
%

category: 'Testing'
method: PrivateObject
isInternalObject
  ^ true
%

category: 'Testing'
method: PrivateObject
isMeta
  ^ false
%

category: 'Testing'
method: PrivateObject
isNil
  ^ false
%

category: 'Testing'
method: PrivateObject
isSpecial
  ^ false
%

category: 'Testing'
method: PrivateObject
isTranlogged

"Returns true if the receiver was a committed object at the time the current
 transaction began and is not a notTranlogged object, or was part of the
 closure of a failed commit and did not become reachabley by reference
 from a not tranlogged object."

^ ((self _status: true) bitAnd: 16r20) ~~ 0
%

category: 'Testing'
method: PrivateObject
isWritten

"Returns true if the receiver was a committed object at the start of the
 current transaction and as been written by the current transaction.
 Returns false otherwise."

^((self _status: true) bitAnd: 16rA ) == 16rA
%

category: 'Testing'
method: PrivateObject
notNil
  ^ true
%

category: 'Accessing'
method: PrivateObject
objectSecurityPolicy

"Returns the GsObjectSecurityPolicy where the receiver is located.
 If the result is nil, the object has World write permission (objectSecurityPolicyId 0)."

<primitive: 592>
self _primitiveFailed: #objectSecurityPolicy .
^ nil
%

category: 'Accessing'
method: PrivateObject
objectSecurityPolicyId

"Returns the objectSecurityPolicyId of the GsObjectSecurityPolicy where the receiver is located.
 If the result is 0, the object has World write permission."
| seg |
seg := self objectSecurityPolicy .
seg ifNotNil:[ ^ seg objectSecurityPolicyId ].
^ 0
%

category: 'Clustering'
method: PrivateObject
pageCreationTime

"Returns a DateTime that is the approximate beginning of the life of the page
 containing the receiver.

 The result represents the time that the receiver was last modified, clustered,
 moved to a new page by the Garbage Collector Gem, regenerated from a
 transaction log or full backup file during recovery or restore, whichever
 happened last.

 When an object is modified by a session, the resulting pageCreationTime is an
 approximate time of the object creation or modification, and may precede the
 time at which the modification was committed.

 If the receiver is not yet committed and has not yet been assigned to a page,
 returns the current time.

 If the receiver is special, returns the value of the class instance variable
 timeStamp of the receiver's class."

<primitive: 400>
self _primitiveFailed: #pageCreationTime .
^ nil
%

category: 'Error Handling'
method: PrivateObject
pause

"Generates an error.  You can use this method to establish
 breakpoints in methods, aside from any debugger breakpoints that
 may be set.  If execution continued , returns receiver. "

 Halt signalNotTrappable
%

category: 'Accessing'
method: PrivateObject
physicalSize

"Returns the number of bytes required to represent the receiver in memory.  If
 the receiver is in special format (which implies that its representation is the
 same as its OOP), returns zero.

 This method should be reimplemented for subclasses whose instances are (or may
 be) a composite of component parts which are objects themselves (such as B-tree
 nodes).  Since the composite object cannot be represented independently of its
 components, its physical size should include that of its components.

 However, the component objects of collection (such as an NSC) should not be
 confused with its contents or elements.  Elements or contained objects are in a
 logical relationship with the collection, whereas its components are in a
 physical relationship.  Logically related objects can be represented and stored
 independently."

<primitive: 364>
self _primitiveFailed: #physicalSize .
^ 0
%

category: 'Accessing'
method: PrivateObject
physicalSizeOnDisk

"Returns the number of bytes required to represent the receiver on disk.  If
 the receiver is in special format (which implies that its representation is the
 same as its OOP), returns zero.

 See also Object >> physicalSize"

<primitive: 579>
self _primitiveFailed: #physicalSizeOnDisk .
^ 0
%

category: 'Formatting'
method: PrivateObject
printString

^ 'a ' , self class name
%

category: 'Accessing'
method: PrivateObject
size

"Instance variables of PrivateObjects are not visible to Smalltalk."
^ 0
%

category: 'Accessing'
method: PrivateObject
squeakBasicSize

^ 0
%

category: 'Tag Management'
method: PrivateObject
tagSize

^ 0
%

category: 'Accessing'
method: PrivateObject
_basicSize

"Instance variables of PrivateObjects are not visible to Smalltalk."
^ 0
%

category: 'Private'
method: PrivateObject
_dnuError: aSymbol args: anArray reason: dnuKind
 "dnuKind constants must agree with  DNU_ constants in om.hf .
  Not using class variables for DNU_* to avoid polluting name space of Object. "
 | err |
 err := InternalError new .
 dnuKind _isSmallInteger ifTrue:[
   dnuKind == 1 ifTrue:[
     err details: 'method needs recompile, receiver a ',
	  self class name , ' , selector #', aSymbol  ;
	   tag: { self . aSymbol }.
   ] ifFalse:[
     dnuKind == 2 ifTrue:[ err details: 'method load error DNU_failure2' ].
     dnuKind == 3 ifTrue:[ err details: 'method load error DNU_failure3' ].
     dnuKind == 4 ifTrue:[ err details: 'method is not invariant' ].
     dnuKind == 5 ifTrue:[ err details: 'method load error DNU_largeLeafMissing'].
     dnuKind == 6 ifTrue:[ err details:'method load error DNU_badExtrasSize' ].
     dnuKind == 7 ifTrue:[ err details:'method load error DNU_methodDoesNotExist' ].
     err tag:{ self . aSymbol . anArray }.
   ].
 ] ifFalse:[
   err details: 'method oop:' , dnuKind asOop asString, ' needs recompile, receiver a ',
	  self class name , ' , selector #', aSymbol  ;
	   tag: { self . aSymbol }.
 ].
 ^ err signal .
%

category: 'Private'
method: PrivateObject
_doesNotUnderstand: aSymbol args: anArray envId: aSmallInt reason: dnuKind
  "This private message is sent by the virtual machine. The implementation must
 be in the persistent method dictionary for Object, since the method is preloaded
 during VM startup.  Reimplementations in a session methods dictionary will not
 be seen."

  dnuKind ~~ 0 ifTrue: [
    "a more severe problem such as method needing recompile"
    self _dnuError: aSymbol args: anArray reason: dnuKind
  ].
  aSmallInt == 0 ifTrue:[
     "this path for compatibility with Gs64 v2.x, and assumes you are using
      only method environment 0  for all of your Smalltalk code."
      ^self doesNotUnderstand: (Message selector: aSymbol arguments: anArray)
  ].
  ^self doesNotUnderstand: aSymbol args: anArray envId: aSmallInt
%

category: 'Error Handling'
method: PrivateObject
_primitiveFailed: aSelector

"Methods which are implemented as primitives send _primitiveFailed:
 when a primitive fails and the failure is not attributable to any
 normal error such as bad argument kind, argument out of range, etc."

^ self _primitiveFailed: aSelector args: { }
%

category: 'Error Handling'
method: PrivateObject
_primitiveFailed: aSelector args: primArgs

"Methods which are implemented as primitives send _primitiveFailed:
 when a primitive fails and the failure is not attributable to any
 normal error such as bad argument kind, argument out of range, etc."

| args |
args := { self . aSelector } .
args add: primArgs.
^ ArgumentError new _number: 2258"#rtErrPrimFailed" ; args: args ; signal
%

category: 'Reduced Conflict Support'
method: PrivateObject
_primitiveSelectiveAbort
"Performs an abort operation on the receiver. That is, if the object is
 committed, it removes any changes made by the current transaction and allows
 access to the committed state of the object.

 The error #rtErrSelectiveAbort is thrown if the receiver has depMap entries."

<primitive: 274>

self _primitiveFailed: #_primitiveSelectiveAbort .
self _uncontinuableError
%

category: 'Accessing'
method: PrivateObject
_primitiveSize

"Instance variables of PrivateObjects are not visible to Smalltalk."
^ 0
%

category: 'Reduced Conflict Support'
method: PrivateObject
_refreshAfterCommitFailure

 "Returns whether the receiver should be selectively aborted when there is a
 failed attempt to commit"

 ^ false
%

category: 'Reduced Conflict Support'
method: PrivateObject
_resolveRcConflictsWith: conflictObjects

"Return false, replay not possible"
System
  rcValueCacheAt: #'Rc-Retry-Failure-Reason'
    put: #'noReplayForPrivateObjects'
    for: System;
  rcValueCacheAt: #'Rc-Retry-Failure-Description'
    put: 'Replay is not support for private object [', self asOop printString, '].'
    for: System.
^ false
%

category: 'Reduced Conflict Support'
method: PrivateObject
_selectiveAbort

"Performs an abort operation on the receiver. That is, if the object is
 committed, it removes any changes made by the current transaction and allows
 access to the committed state of the object."

^ self _primitiveSelectiveAbort.
%

category: 'Testing'
method: PrivateObject
_status: checkClosureBool

"Returns a SmallInteger with status information about the receiver
 encoded in the bit whose masks are defined below:

 16r01 isSpecial
     Set if the receiver is an AbstractCharacter, Boolean,
     SmallInteger, or nil.
 16r02 isCommitted
     Set if the receiver existed in GemStone at the time the current
     transaction began.
     if checkClosureBool is true, and session's most recent commit
     failed and is not yet aborted, check the closureList to determine actual
     committed state of temporary objects in the closure of the commit.
 16r04 isConnected  (obsolete bit, always 0)
 16r08 isWritten
     Set if the receiver has been written since the last
     commit, abort, or begin transaction command was executed.
 16r10 isWritable
     Set if the receiver canBeWritten by the current user.
 16r20 isTranlogged - true isCommitted and tranlogged, or
     not committed and beTranlogged has been sent .
 16r40 beTranlogged has been sent to the object while in-memory.

 16r700 levels  ( > 0 means a 'large object' implemented as a tree of
                smaller objects which are not visible to Smalltalk)"

<primitive: 39>
self _primitiveFailed: #_status args: { checkClosureBool } .
^ nil
%

category: 'Formatting'
method: PrivateObject
_topazAsString
  ^ [ self asString
    ] onSynchronous: AbstractException do:[:ex |
      '<error during asString>'
    ]
%

category: 'Error Handling'
method: PrivateObject
_uncontinuableError

"An attempt was made to continue execution past an uncontinuable error."

 self pause
%

category: 'Private'
method: PrivateObject
_unsafeAt: anIndex
"Low level access to an object, node of a large object , or node of
 an IdentityBag
 Named instance variables are accessed by this method, so
 unnamed instance variables begin at (self class instSize + 1).

 Returns specified instVar of the object.  If object is byte format,
 returns SmallInteger value of the specified byte.
 If anIndex is < 1 , the primtive fails.
 If anIndex is beyond the end of the object, returns OOP_REMOTE_NIL"

 <primitive: 168>
 (anIndex _isInteger)
   ifFalse: [ PrivateObject _errorNonIntegerIndex: anIndex .  self _uncontinuableError ].
 self _primitiveFailed: #_unsafeAt: args: { anIndex } .
 self _uncontinuableError
%

! Class extensions for 'ProcessorScheduler'

!		Class methods for 'ProcessorScheduler'

removeallmethods ProcessorScheduler
removeallclassmethods ProcessorScheduler

category: 'Instance Creation'
classmethod: ProcessorScheduler
new
  self shouldNotImplement: #new
%

category: 'Accessing'
classmethod: ProcessorScheduler
scheduler
  "This is equivalent to 'Processor' in the Blue Book.  It returns
   the instance of the receiver that is responsible for scheduling
   ready processes"

<primitive: 457>

^self _halt: 'Attempt to use process scheduler before it has been initialized'
%

category: 'Accessing'
classmethod: ProcessorScheduler
topazAllProcesses
 "Returns an Array of all GsProcess's known to the
  scheduler , excluding the current active process."
 | set |
 set :=  self scheduler allProcesses .
 set removeIfPresent: GsProcess _current .
 ^ set asArray .
%

category: 'Accessing'
classmethod: ProcessorScheduler
topazStatusForProcess: aGsProcess

^ self scheduler _statusString: aGsProcess
%

category: 'Debugging'
classmethod: ProcessorScheduler
_allProcessesTerminate9
  self scheduler _allProcessesTerminate9
%

category: 'Private'
classmethod: ProcessorScheduler
_isInitialized
  ^((System __sessionStateAt: 15) ~~ nil)
%

!		Instance methods for 'ProcessorScheduler'

category: 'Accessing'
method: ProcessorScheduler
activePriority
  "return the priority of the active process"
  | ap |
  ^ (ap := activeProcess) ifNil:  [ 15 "inline userSchedulingPriority" ]
                          ifNotNil: [ ap priority ]
%

category: 'Accessing'
method: ProcessorScheduler
activeProcess

"Returns the currently active process.
 Creates a new GsProcess if needed and stores it into the
  activeProcess instVar of the scheduler instance."
<primitive: 693>
self _primitiveFailed: #_currentProcess
%

category: 'Accessing'
method: ProcessorScheduler
allProcesses
  "Return an IdentitySet of all of the instances of GsProcess
   that are not terminated."

  ^ self _allProcessesInto: (IdentitySet new: 4)
%

category: 'Event Scheduling'
method: ProcessorScheduler
cancelWhenReadable: aGsSocket signal: objToNotify
  "Cancels the event specified by aGsSocket and objToNotify.
   The event is created using the whenReadable:signal: method.
   Returns true if the event has already been cancelled, reaped, or does not exist.
   Returns false if the event is cancelled by this invocation."

  ^ aGsSocket _cancelReadEventFor: objToNotify .
%

category: 'Event Scheduling'
method: ProcessorScheduler
cancelWhenWritable: aGsSocket signal: objToNotify
  "Cancels the event specified by aGsSocket and objToNotify.
   The event is created using the whenWritable:signal: method.
   Returns true if the event has already been cancelled, reaped, or does not exist.
   Returns false if the event is cancelled by this invocation."

  ^ aGsSocket _cancelWriteEventFor: objToNotify .
%

category: 'Logging'
method: ProcessorScheduler
dbgfatallog: aString
  "log a fatal error string"
  GsFile gciLogServer: aString
%

category: 'Logging'
method: ProcessorScheduler
dbglog: aString
  "log a debugging string"
  GsFile gciLogServer: aString
%

category: 'Priorities'
method: ProcessorScheduler
highestPriority
  "return the highest allowable priority"
  ^ 40   "also in C code as PROCESSSCHEDULER_highestPriority"
         "also other in-line highestPriority logic"
%

category: 'Priorities'
method: ProcessorScheduler
highIOPriority
  "Answer the priority at which the most time
   critical input/output processes should run. [Blue Book]"
  ^30
%

category: 'Priorities'
method: ProcessorScheduler
lowestPriority
  "return the lowest allowable priority"
  ^1   "also in C code as PROCESSSCHEDULER_lowestPriority"
       "also other inline lowestPriority logic"
%

category: 'Priorities'
method: ProcessorScheduler
lowIOPriority
  "Answer the priority at which most input/output
   processes should run. [Blue Book]"
  ^25
%

category: 'Accessing'
method: ProcessorScheduler
readyProcesses
  "Return all of the instances of GsProcess that are ready to run but
   are not active."
  | s wasInCrit |
  wasInCrit := self _enterCritical .
  (s := IdentitySet new: 4) _addAll: readyQueue  forReplay: false .
  wasInCrit == 0 ifTrue:[ self _exitCritical] .
  ^ s
%

category: 'Accessing'
method: ProcessorScheduler
suspendedProcesses
  "Return all of the instances of GsProcess that were suspended using
   the GsProcess>>suspend method
   or created with the ExecBlock>>newProcess method and not yet
   sent resume."

  ^ suspendedSet shallowCopy
%

category: 'Priorities'
method: ProcessorScheduler
systemBackgroundPriority
  "Answer the priority at which system background
   processes should run. [Blue Book]"
  ^5
%

category: 'Priorities'
method: ProcessorScheduler
timingPriority
  "Answer the priority at which the system processes
   keeping track of real time should run. [Blue Book]"
  ^35
%

category: 'Priorities'
method: ProcessorScheduler
userBackgroundPriority
  "Answer the priority at which background
   processes created by the user should run. [Blue Book]"
  ^10
%

category: 'Priorities'
method: ProcessorScheduler
userInterruptPriority
  "Answer the priority at which processes created
   by the user and desiring immediate service
   should run. [Blue Book]"
  ^20
%

category: 'Priorities'
method: ProcessorScheduler
userSchedulingPriority
  "Answer the priority at which the user inter-
   face processes should run. [Blue Book]"
  ^15  "this value inlined in some methods here and in GsProcess, and
	in C as PROCESSSCHEDULER_userPriority"
%

category: 'Accessing'
method: ProcessorScheduler
waitingProcesses
  "Return an IdentitySet of all of the instances of GsProcess that
   are waiting for an event."
  | res wasInCrit |
  wasInCrit := self _enterCritical .
  res := self _waitingProcessesInto: (IdentitySet new: 4) .
  wasInCrit == 0 ifTrue:[ self _exitCritical] .
  ^ res
%

category: 'Event Scheduling'
method: ProcessorScheduler
whenReadable: aGsSocket signal: objToNotify
  "Causes the receiver to signal objToNotify once when aGsSocket is
   readable or has an exceptional condition.
   Once the receiver signals the object for this event the event is
   canceled so you need to call this method again to reenable the event.
   objToNotify is signalled by sending it the message '_reapSignal: source'
   with source being the socket that is readable.
   The kernel classes that respond to _reapSignal: are:
     Semaphore, SharedQueue, and GsProcess.
   Returns self."

  aGsSocket _whenReadableNotify: objToNotify .
  "true ifTrue:[  assertion code
    (socketsPolled includesIdentical: aGsSocket) ifFalse:[ self pause ].
  ]."
%

category: 'Event Scheduling'
method: ProcessorScheduler
whenWritable: aGsSocket signal: objToNotify
  "Causes the receiver to signal objToNotify once when aGsSocket is
   writable or has an exceptional condition.
   Once the receiver signals the object for this event the event is
   canceled so you need to call this method again to reenable the event.
   objToNotify is signalled by sending it the message '_reapSignal: source'
   with source being the socket that is writable.
   The kernel classes that respond to _reapSignal: are:
     Semaphore, SharedQueue, and GsProcess.
   Returns self."

  aGsSocket _whenWritableNotify: objToNotify .
  "true ifTrue:[ assertion code
    (socketsPolled includesIdentical: aGsSocket) ifFalse:[ self pause ].
  ]."
%

category: 'Process State Change'
method: ProcessorScheduler
yield
  "schedule a different active process, if any are
   available, and put the current process in the
   appropriate ready queue."

  | readyQ wasInCrit |
  wasInCrit := self _enterCritical .
  self _reapEvents: 1 .
  readyQ := readyQueue .
  (readyQ atOrNil: 1) ifNotNil:[:aProc |
    aProc priority >= self activePriority  ifTrue: [ | oldProc newProc |
      oldProc := self activeProcess .

      newProc := aProc .
      readyQ removeFrom:1 to: 1 .
      newProc _onQueue: nil .
      oldProc ~~ newProc ifTrue:[   "fix 49249"
        self _addReadyProcess: oldProc to: readyQ .  "active process never on ready que"
        oldProc _onQueue: readyQ .
        self _switchFrom: oldProc to: newProc .   "prim does _exitCritical, updates activeProcess iv"
        ^ self
      ].
    ].
  ].
  wasInCrit == 0 ifTrue:[ self _exitCritical] .
%

category: 'Private'
method: ProcessorScheduler
_add: anObject toSet: anIdentitySet

"Returns receiver."

<primitive: 726>
self _primitiveFailed: #_add:toSet: args: { anObject . anIdentitySet }
%

category: 'Private'
method: ProcessorScheduler
_addDelay: aDelayOrProcess to: aDelayQ

"Receiver should be the ProcessorScheduler's delayQueue.
 Appends aDelayOrProcess to the receiver. Returns receiver."

<primitive: 734>  "prim fails if aDelayQ is large"
aDelayQ add: aDelayOrProcess
%

category: 'Private'
method: ProcessorScheduler
_addReadyProcess: aProcess to: aReadyQ

"Receiver should be the ProcessorScheduler's readyQueue.
 Appends aProcess to the receiver. Returns receiver."

<primitive: 733>  "prim fails if aReadyQ is large"
aReadyQ add: aProcess
%

category: 'Accessing'
method: ProcessorScheduler
_allProcessesInto: anIdentitySet

  "Add all of the instances of GsProcess that are not terminated
   to result and return aCollection."
  | polled wasInCrit |
  wasInCrit := self _enterCritical .
  anIdentitySet _addAll: readyQueue forReplay: false .
  self _waitingProcessesInto: anIdentitySet .
  anIdentitySet _addAll: suspendedSet forReplay: false .
  1 to: (polled := socketsPolled) size do:[:j |
    (polled at: j) _waitingProcessesInto: anIdentitySet .
  ].
  anIdentitySet add: self activeProcess .
  wasInCrit == 0 ifTrue:[ self _exitCritical ].
  ^ anIdentitySet
%

category: 'Accessing'
method: ProcessorScheduler
_allProcessesInto: anIdentitySet inGroup: groupId

  "Add all of the instances of GsProcess that are not terminated
   and are in specified group to aSet and return anIdentitySet."

  | set proc polled wasInCrit |
  wasInCrit := self _enterCritical .
  anIdentitySet _addAll: readyQueue forReplay: false .
  1 to: (set := readyQueue) size do:[:n|
    proc := set at: n .
    proc _groupOrNil == groupId ifTrue:[ anIdentitySet add: proc ].
  ].
  self _waitingProcessesInto: anIdentitySet inGroup: groupId .
  1 to: (set := suspendedSet) size do:[:n|
    proc := set _at: n .
    proc _groupOrNil == groupId ifTrue:[ anIdentitySet add: proc ].
  ] .
  1 to: (polled := socketsPolled) size do:[:j |
    (polled at: j) _waitingProcessesInto: anIdentitySet inGroup: groupId .
  ].
  wasInCrit == 0 ifTrue:[ self _exitCritical ].
  ^ anIdentitySet
%

category: 'Debugging'
method: ProcessorScheduler
_allProcessesTerminate9
  "Send terminate 9 to all GsProcess's known to the
   scheduler , excluding the current active process."
  | set polled |
  set :=  self allProcesses .
  set removeIfPresent: GsProcess _current .
  [
    set do:[:aProcess | aProcess terminate9 ].
    polled := socketsPolled shallowCopy .
    1 to: polled size do:[:j |
      (polled at: j) _disableEvent: nil ."disable all events and polling"
    ].
  ] onException: AbstractException do:[:ex |
    ex resume "ignore"
  ].
%

category: 'Debugging'
method: ProcessorScheduler
_allStacksReport
  "includes the stack of the currently executing process."
  | allP rpt |
  rpt := String new .
  allP := self allProcesses .
  allP do:[:aProc | rpt add: aProc _reportString ].
  ^ rpt
%

category: 'Private'
method: ProcessorScheduler
_changePriority: aProcess from: oldPriority

  | q |
  aProcess onQueue == (q := readyQueue) ifTrue:[
    q removeIdentical: aProcess otherwise: nil .
    self _addReadyProcess: aProcess to: q .
  ].
%

category: 'Private'
method: ProcessorScheduler
_checkForExpiredDelays: delayQ
    "returns number of expired delays"
    | readyCount dlysSize now |
    readyCount := 0 .
    dlysSize := delayQ size .
    now := self _timeoutNow: dlysSize .
    dlysSize ~~ 0 ifTrue:[ | idx ofs dT |
      idx := 0 .
      ofs := 1 .
      [ ofs <= dlysSize ] whileTrue:[  | aDly |
        "search  delayQ for the first item not yet expired.
         For items that have expired, activate them so they will be scheduled.
        "
        aDly := delayQ at: ofs .
        ((dT := aDly _signalTime) ~~ nil and:[ dT > now])  ifTrue: [ "this one not expired"
	  idx := ofs .
          ofs := dlysSize + 1 . "exit loop"
        ] ifFalse: [			"this one expired"
	  "self dbglog: ('sched: activating a ', aDly class name , ' time: ',
		now asString, ', _signalTime: ', aDly _signalTime asString).  "
	  aDly _activate .
          readyCount := readyCount + 1 .
          ofs := ofs + 1 .
        ].
      ].
      (idx == 0) ifTrue: [
        "all delays have expired, tell timeout thread to wait for more work."
	delayQ size: 0 .
	dlysSize := 0 .
        timeoutThreadActive ifTrue:[
          self _updateTimerThread: -1 priority: 15"inline userSchedulingPriority" .
          timeoutThreadActive := false .
        ].
        "self dbglog: 'sched: all delays expired' ."
      ] ifFalse:[
        idx ~~ 1 ifTrue: [
          delayQ removeFrom: 1 to: (idx - 1). "delete expired items"
	  dlysSize := delayQ size .
        ].
	dlysSize ~~ 0 ifTrue:[ | nextDly |
	  nextDly := delayQ at: 1 .
	  self _updateTimerThread: nextDly _signalTime priority: nextDly priority .
          timeoutThreadActive := true .
        ].
      ].
    ] .
    ^ readyCount
%

category: 'In-Process Debugger Support'
method: ProcessorScheduler
_criticalMethods

"Returns an IdentitySet of GsNMethods."

^ criticalMethods ifNil:[ self _initCriticalMethods ]
%

category: 'Delay Access'
method: ProcessorScheduler
_delaySchedule: aProcessOrDelay
  "Add aProcessOrDelay to the delayQueue ."

  self _addDelay: aProcessOrDelay to: delayQueue .
%

category: 'Delay Access'
method: ProcessorScheduler
_delayUnschedule: aDelay
  "Unschedule aDelay in the queue."

  delayQueue removeIdentical: aDelay otherwise: nil
%

category: 'Private'
method: ProcessorScheduler
_doPoll: resultArray yield: yieldInt

"Wait for a GsSocket to be ready or for a timeout to expire.
 yieldInt > 0 means yield , i.e. just poll for GsSocket(s) ready ,
 yieldInt == 0 means poll until GsSocket(s) ready or until
 timer C thread delivers EINTR to main thread when a timeout expires.

 resultArray must be an Array. It will have stored into it, each
 instance of GsSocket that had an event detected on it.
 Returns nil if there was a socket error, otherwise returns number of
 elements in resultArray, or zero if the primitive timed out."

<primitive: 192>
self _uncontinuableError
%

category: 'Private'
method: ProcessorScheduler
_enterCritical
"enter critcal region. 
 returns a SmallInteger 0==entered critical region, 1==already in critical region"

<primitive: 169> 
self _primitiveFailed: #_enterCritical
%

category: 'Private'
method: ProcessorScheduler
_exitCritical

"clears OM.schedulerInReapEvents, which is also cleared by prim 2010"

<primitive: 737>
self _primitiveFailed: #_exitCritical
%

category: 'Private'
method: ProcessorScheduler
_findReadyProcess

  "Find another process to run, waiting for events if no process is
   ready to run.  Signals a ThreadError if a deadlock would occur because
   no other processes are ready nor waiting.
   Returns the next process to run or returns nil if interrupted by OOB."

  <primitive: 736> "prim always fails, sets OM.reapEventsCount "
  | proc err oldProc rpt res readyQ |
  res := self _reapEvents: 0 .
  res ~~ self ifTrue:[
    res == 0 ifTrue:[ "no events but interrupted by OOB from stone"
      ^ nil
    ].
    "we have  res == nil"
    oldProc := self activeProcess .
    oldProc ifNotNil:[ | q |  "normally oldProc is the active process"
        (q := oldProc onQueue) ifNotNil:[
           q ~~ readyQueue ifTrue:[
             oldProc _onQueue: nil .
             self _remove: oldProc fromSet: q .  "waitingSet or suspendedSet"
           ].
        ]
    ].
    (rpt := '----- ProcessorScheduler deadlock ' copy) add: ' current process ' ;
                           add: oldProc asOop asString ; lf .
    rpt addAll: self _allStacksReport .
    GsFile gciLogServer: rpt .
    (err := ThreadError new) _number: 2366 ; reason: #rtErrSchedulerDeadlocked;
	 args: { self . oldProc . rpt } ;
         details: 'The ProcessorScheduler is deadlocked' .
    self _exitCritical .
    err signal .
  ].
  proc := (readyQ := readyQueue) atOrNil: 1 . "removeFirst"
  proc ifNotNil:[
    readyQ removeFrom: 1 to: 1 .
    proc _onQueue: nil .
    "caller must send _switchFrom:to: or _exitCritical"
  ].
  ^ proc
%

category: 'Private'
method: ProcessorScheduler
_findReadyProcessForExit: oldProc
  "Returns either nil or the next GsProcess to run.
   If result is not nil, caller must send
    _switchFrom:to:  to the scheduler to switch to returned GsProcess."

  "caller has entered critical region"
  | proc status |
  status := self _reapEvents: 0 .
  status == 0 ifTrue:[ status := self _reapEvents: 0  ].
  status ~~ self ifTrue:[
     oldProc ifNotNil:[ | q |  "normally oldProc is the active process"
       (q := oldProc onQueue) ifNotNil:[
          q ~~ readyQueue ifTrue:[
            oldProc _onQueue: nil .
            self _remove: oldProc fromSet: q .  "waitingSet or suspendedSet"
          ].
       ]
     ].
     ^ nil   "no other processes available to run"
  ].
  proc := readyQueue removeAtIndex:1 . "removeFirst"
  proc _onQueue: nil .
  ^ proc
%

category: 'Initialization'
method: ProcessorScheduler
_initialize: initSessionArg
  "This method is invoked once by the VM during session initialization.
   The single instances of ProcessorScheduler and GsCurrentSession
   are created by the VM before invoking this method.

   If initSessionArg~~nil , invokes GsCurrentSession(C) initialize

   In the following discussion, the single instance of GsCurrentSession
   is 'currSess' .

   Before invoking this method, the VM has completed these steps :
     1) default timeZone is fetched from (Globals at:#TimeZone) and
     installed in session state.   This wil be either the
     value of the class instance variable 'default' in TimeZone ,
     or the class variable #Default in the ObsoleteTimeZone class
     (if the 2007 upgrade to TimeZone has not been installed).

   Initializes the receiver's process queues.

   Invokes GsCurrentSession >> initialize  to execute any application
   defined initialization."

  readyQueue := SortedCollection sortBlock: [:p1:p2 | p1 priority >= p2 priority].
		"sort block must agree with prim 733"
  activeProcess := nil.
  delayQueue := SortedCollection sortBlock: [:d1:d2 | d1 _signalTime <= d2 _signalTime].
		"sort block must agree with prim 734"
  suspendedSet := IdentitySet new: 4 .
  waitingSet := IdentitySet new: 4 .
  lastGroup := 0.
  pollResults := Array new: 5 . pollResults size: 0 .
  socketsPolled := Array new: 5 . socketsPolled size: 0 .
  timeoutThreadActive := false .
  self _setInstVarsAlwaysValid .      "fix 44837"

  System __sessionStateAt: 15 put: self.
  (Globals at: #CharacterDataTables otherwise: nil) ifNotNil:[
    [
      (System gemEnvironmentVariable:'GS_DISABLE_CHARACTER_TABLE_LOAD') ifNil:[
        Character _loadCharTables.
      ]
    ] onException: AbstractException do:[:ex | "ignore" ].
  ].
  initSessionArg ifNotNil:[
    GsCurrentSession initialize
  ].
  "No user or application changes here.
   Customization is to be done in GsCurrentSession >> initialize"
%

category: 'Accessing'
method: ProcessorScheduler
_isSuspended: aGsProcess
  "Returns a Boolean "

  ^ suspendedSet includesIdentical: aGsProcess
%

category: 'Private'
method: ProcessorScheduler
_isWhenReadableSet: aGsSocket signal: objToNotify
  "Returns true if read event is still enabled.
   Returns false if read event was never set or has happened and been
   automatically unset."

  ^ aGsSocket hasReadWaiter: objToNotify
%

category: 'Private'
method: ProcessorScheduler
_isWhenWritableSet: aGsSocket signal: objToNotify
  "Returns true if write envent is still enabled.
   Returns false if write event was never set or has happened and been
   automatically unset."
  ^ aGsSocket hasWriteWaiter: objToNotify
%

category: 'Private'
method: ProcessorScheduler
_newGroup
  lastGroup := lastGroup + 1.
  ^lastGroup
%

category: 'Delay Access'
method: ProcessorScheduler
_now
  "Returns a SmallInteger, the time that should be used by a Delay to represent to
   current time in milliseconds."

<primitive: 651>
^ self _primitiveFailed: #_now
%

category: 'Private'
method: ProcessorScheduler
_reapEvents: yieldInt

  "Reaps any pending events that will cause a thread to be ready to run.
   yieldInt should be 0 to wait for an event, or 1 for  yielding.
   Returns nil if scheduler would deadlock,
   returns 0 if an OOB byte from stone has interrupted the waiting,
   returns self otherwise.
  "
  "senders must enter critical region by using prim 736"

  | delayQ readyQ pollCount |
  delayQ := delayQueue .
  readyQ := readyQueue .
  pollCount := 0 .
  [
    | readyCount dlysSize |
    readyCount := readyQueue size + (self _checkForExpiredDelays: delayQ) .
    dlysSize := delayQ size .
    (dlysSize + socketsPolled size ) ~~ 0 ifTrue: [
      | resSize resArray |
         "yield use case:  poll for socket activity, sleepVal== 0
          otherwise:  waitforever for socket activity, or wakeup by timeout thread."
      resSize := self _doPoll: (resArray := pollResults) yield: readyCount + yieldInt .
      pollCount := pollCount + 1 .
      resSize ~~ 0 ifTrue:[
	1 to: resSize do:[:k | | aSocket |
	  aSocket := resArray at: k .
	  aSocket _reapEvents .
        ].
        self _sizeNoShrink: resArray .
        readyCount := readyQ size.
      ].
      yieldInt ~~ 0 ifTrue:[ ^ self ].
      readyCount ~~ 0 ifTrue:[ ^ self ]
        ifFalse:[
          (self _checkForExpiredDelays: delayQ) ~~ 0 ifTrue:[ ^ self ]
      ].
      "self dbglog: 'sched: no events but interrupted' ."
      ^ 0 "no events but interrupted"
    ] ifFalse:[
      readyCount := readyQ size .
      (readyCount + yieldInt) ~~ 0 ifTrue: [
        ^ self
      ] ifFalse:[
        "readyCount == 0 ifTrue:[  self pause ]." "uncomment to debug deadlock"
        ^ nil .  "would deadlock if caller is not yield"
      ].
    ] .
    pollCount > 100 ifTrue:[  | err |
      (err := ThreadError new) _number: 2366 ; reason: #rtErrSchedulerDeadlocked;
	  args: { self . nil } ;
          details: 'The ProcessorScheduler is deadlocked, infinite loop in _reapEvents: ' .
       self _exitCritical .
       err signal
    ].
    true
  ] whileTrue "forever loop"
%

category: 'Private'
method: ProcessorScheduler
_remove: anObject fromSet: anIdentitySet

"Returns receiver. If size of anIdentitySet goes to zero
and it is committed, regenerates the in-memory state as not committed."

<primitive: 727>
self _primitiveFailed: #_remove:fromSet: args: { anObject . anIdentitySet }
%

category: 'Private'
method: ProcessorScheduler
_reschedule
  "Find a process that is ready to run. Once found suspend originalOldProcess
   and activate the new process."
  | newProc oldProc |
  oldProc := self activeProcess .
  [ (newProc := self _findReadyProcess ) ifNotNil:[
      self _switchFrom: oldProc to: newProc . "exits critical"
      "We only get here once the old process is once again active."
      true
    ] ifNil:[
      self _rescheduleContinue: oldProc .
      false
    ]
  ] whileFalse .
%

category: 'Private'
method: ProcessorScheduler
_rescheduleContinue: oldProc
  | noEx |
  [
    self _exitCritical .
    1 timesRepeat:[ self class ] . "honor interrupts"
    noEx := true .
    self _enterCritical .
  ] ensure: [  "if an Exception is not resumed, unschedule oldProc"
    noEx ifNil:[ | wasInCrit |
      wasInCrit := self _enterCritical .
      self _unschedule: oldProc .
      wasInCrit == 0 ifTrue:[ self  _exitCritical ].
    ].
  ].
%

category: 'Private'
method: ProcessorScheduler
_resumeProcess: aProcess
  "Put a suspended process back to the state it was in when it was
   suspended."

  | q suspSet readyQ |
  q := aProcess onQueue .
  q == (suspSet := suspendedSet) ifTrue:[
    suspSet removeIdentical: aProcess otherwise: nil .
    aProcess waitingOn ifNotNil:[ | ws |
      self _add: aProcess toSet: (ws := waitingSet) .
      aProcess _onQueue: ws .
    ] ifNil:[
      self _addReadyProcess: aProcess to: (readyQ := readyQueue) .
      aProcess _onQueue: readyQ .
    ].
  ] ifFalse:[ | proc |  "inline _cancelDelay"
    proc := delayQueue removeIdentical: aProcess otherwise: nil .
    proc == aProcess ifTrue:[
      proc _signalTime: 0 .
      self _addReadyProcess: proc to: (readyQ := readyQueue) .
      proc _onQueue: readyQ .
    ] ifFalse:[
      (aProcess waitingOn isKindOf: GsSocket) ifTrue:[
        proc := waitingSet removeIdentical: aProcess otherwise: nil .
        proc == aProcess ifTrue:[
          aProcess _waitingOn: nil .
          self _addReadyProcess: proc to: (readyQ := readyQueue) .
          proc _onQueue: readyQ .
        ].
      ]
    ]
  ].
%

category: 'Private'
method: ProcessorScheduler
_runNextProcess
  "Used when a GsProcess exits to kick the scheduler so it will run
   another process.  Current process state is 'terminated' . "
  <primitive: 736> "prim always fails, sets OM.reapEventsCount "

  | newProc |
  [ | cnt ex |
    cnt := 0 .
    [ newProc := self _findReadyProcess .
      newProc == nil 
    ] whileTrue:[ "handle OOB or EINTR"
      (cnt := cnt + 1 ) > 200 ifTrue:[ 
        (ex := ThreadError new) details:'_runNextProcess deadlock' .
        self _exitCritical .
        ex signalNotTrappable
      ].
    ].
    newProc == self activeProcess ifTrue:[
      (ex := ThreadError new) details:'_runNextProcess no other process' .
      self _exitCritical .
      ex signalNotTrappable
    ].
    newProc _isTerminated 
  ] whileTrue .

  self _switchFrom: nil to: newProc .  "primitive updates activeProcess instVar"
  "Should never return"

  self dbgfatallog: 'leaving _runNextProcess!!!!'.
  self _uncontinuableError
%

category: 'Private'
method: ProcessorScheduler
_scheduleNewProcess: aProcess prio: aPriority
  "Put a newly created process on the ready queue"
  | readyQ p |
  p := aPriority ifNil:[ self activePriority ].
  aProcess _setPriority: p .
  self _addReadyProcess: aProcess to: (readyQ := readyQueue) .
  aProcess _onQueue: readyQ .
%

category: 'Private'
method: ProcessorScheduler
_scheduleProcess: aProcess
  "add the given process to the ready-queue"

  | q readyQ |
  readyQ := readyQueue .
  q := aProcess onQueue .
  q ifNotNil:[       "fix 49249"
    q ~~ readyQ ifTrue:[
      aProcess _onQueue: nil .
      self _remove: aProcess fromSet: q .  "waitingSet or suspendedSet"
    ] ifFalse:[ 
      ^ self "already on readyQ" 
    ].
  ].
  self _addReadyProcess: aProcess to: readyQ .
  aProcess _onQueue: readyQ  .
%

category: 'Private'
method: ProcessorScheduler
_setInstVarsAlwaysValid

"Returns receiver.  Prevent values of instVars from being changed by abort
if they do get committed."

<primitive: 995>
self _primitiveFailed: #_setInstVarsAlwaysValid
%

category: 'Private'
method: ProcessorScheduler
_sizeNoShrink: anArray

"Sets logical size of anArray to zero; if anArray is a small object,
 the operation does not shrink the physical size of anArray .
 Returns receiver. "

<primitive: 739>
self _primitiveFailed: #_sizeNoShrink: args: { anArray }
%

category: 'Private'
method: ProcessorScheduler
_statusString: aGsProcess
  "Returns a string that describes aGsProcess's status."

  | result q waitOn |

  aGsProcess == self activeProcess ifTrue: [
     ^ 'active'.
  ].
  q := aGsProcess onQueue .
  q == readyQueue ifTrue: [
    ^ 'ready'.
  ].
  q == suspendedSet ifTrue: [
    ^ 'suspended'.
  ].
  waitOn := aGsProcess waitingOn .
  (q == waitingSet or:[ waitOn ~~ nil]) ifTrue:[
    result := String new.
    waitOn _isSymbol ifTrue:[
      result add: 'waiting on ' ; add: waitOn . "usually for Ruby IO.select"
    ] ifFalse:[
      result add: 'waiting on a '; add: waitOn class name .
    ].
    aGsProcess _signalTime ifNotNil:[
        result addLast: ', with a timeout'
    ].
  ] ifFalse:[
    q ifNotNil:[
      self halt:'invalid process state'.
    ].
    (aGsProcess _isTerminated) ifTrue: [
      result := 'terminated'.
    ] ifFalse: [
      aGsProcess _terminationStarted ifTrue:[
       result := 'terminationStarted' .
      ] ifFalse:[
        (delayQueue indexOfIdentical: aGsProcess) ~~ 0 ifTrue:[
          result := 'on delayQueue' .
        ] ifFalse:[
          "GCI application must have it in the debug state"
          result := 'debug'.
        ].
      ].
    ].
  ].
  ^ result
%

category: 'Private'
method: ProcessorScheduler
_suspendNewProcess: aProcess

 "put a newly created Process in the suspendedSet ."

  | suspSet |
  aProcess _setPriority: self activePriority .
  self _add: aProcess toSet: (suspSet := suspendedSet) .
  aProcess _onQueue: suspSet .
%

category: 'Private'
method: ProcessorScheduler
_suspendProcess: aProcess
  "suspend the process and do not schedule it for further processing"

  | suspSet |
  (aProcess == self activeProcess) ifTrue: [
    self _add: aProcess toSet: (suspSet := suspendedSet) .
    aProcess _onQueue: suspSet  .
    self _reschedule .
  ] ifFalse: [ | q |
    aProcess onQueue == (q := readyQueue) ifTrue:[
      q removeIdentical: aProcess otherwise: nil .
      self _add: aProcess toSet: (suspSet := suspendedSet) .
      aProcess _onQueue: suspSet .
    ] " ifFalse:  do nothing"
  ].
%

category: 'Private'
method: ProcessorScheduler
_switchFrom: oldProcess to: newProcess
  "Suspend the currentProcess and activate the receiver"

<primitive: 2010>
"prim clears OM.schedulerInReapEvents .
 No primitive failure code. Execution resumes upon resumption
 of oldProcess "
^ self
%

category: 'Process Access'
method: ProcessorScheduler
_terminateScheduledProcess: aGsProcess
  "remove the given process from its queues it is in"

  <primitive: 736> "enter critical region, prim always fails"
  (aGsProcess == self activeProcess) ifTrue: [ | nxt |
    nxt := self _findReadyProcessForExit: aGsProcess .
    (nxt ~~ nil and:[ nxt ~~ aGsProcess]) ifTrue:[
      self _switchFrom: nil to: nxt .       "exits critical"
      self _uncontinuableError . "should not reach here"
    ].
  ] ifFalse: [
    aGsProcess _unscheduleProcess .
  ].
  self _exitCritical .
%

category: 'Delay Access'
method: ProcessorScheduler
_timeoutNow: delayQsize

  "Returns current time in milliseconds consistent with most recently
   expired timeout.  If delayQsize == 0 , result may always be zero "
<primitive: 735>
^ self _primitiveFailed: #_timeoutNow:
%

category: 'Private'
method: ProcessorScheduler
_unschedule: aGsProcess

  "remove the given process from any scheduler state it is in."

  | q |
  q := aGsProcess onQueue .
  q ifNotNil:[
    aGsProcess _onQueue: nil .
    q == readyQueue ifTrue:[
      q removeIdentical: aGsProcess otherwise: nil.
    ] ifFalse:[
      self _remove: aGsProcess fromSet: q . "suspendedSet or waitingSet"
    ].
  ].
%

category: 'Private'
method: ProcessorScheduler
_updateTimerThread: timeMs priority: anInt

"Restart timer thread so it will wakeup at the specified signal time.
 aSignalTime == -1 means  timer thread should wait forever for more work."

<primitive: 738>
self _primitiveFailed: #_updateTimerThread: args: { timeMs . anInt }
%

category: 'Private'
method: ProcessorScheduler
_waitForMilliseconds: intervalMs
  "Causes the current thread to suspend for the specified time .
   Returns true if the timeout expired. Returns false if the thread
   woke up early for some reason."
  "Sender has done _enterCritical "
  intervalMs _isSmallInteger ifFalse:[
    intervalMs _validateClass: SmallInteger.
  ].
  intervalMs <= 0 ifTrue:[
    self yield .
    ^ false
  ] ifFalse:[ | nowMs endMs aProc proc res |
    nowMs := self _now .
    endMs := 16rfffffffffffffff "max SmallInteger" .
    (endMs - intervalMs) > nowMs ifTrue:[
       endMs := nowMs + intervalMs
    ].
    (proc := self activeProcess) _signalTime: endMs  .
    self _addDelay: proc to: delayQueue .

    self _reschedule .
    self _enterCritical .

    "we have woken up."
    aProc := self activeProcess .
    aProc ~~ proc ifTrue:[
      self _exitCritical .
      Error signal:'Unexpected: activeProcess changed during _waitForMilliseconds:'.
    ].
    res := true .
    aProc _signalTime ifNotNil:[
      "Woke up without the delay having been activated, so clean it up."
      delayQueue removeIdentical: aProc otherwise: nil . "inline _delayUnschedule:"
      aProc  _signalTime: nil.
       res := false
    ].
    self _exitCritical .
    ^ res .
  ].
%

category: 'Private'
method: ProcessorScheduler
_waitForSocket: aGsSocket
  "Causes the current thread to suspend until aGsSocket is read/writable,
   or has an error.  Timeout is infinite.  Returns true if socket is ready."

  | proc ws |
  (proc := self activeProcess) _waitingOn: aGsSocket .  "clear done by _reapSignal:"
  "assertion code"
  "true ifTrue:[
    (socketsPolled includesIdentical: aGsSocket) ifFalse:[ self pause ].
  ]."

  self _add: proc toSet: (ws := waitingSet) .
  proc _onQueue: ws .

  self _reschedule .
%

category: 'Private'
method: ProcessorScheduler
_waitForSocket: aGsSocket timeout: msToWait forWrite: writeBool

  "Causes the current thread to suspend until aGsSocket is read/writable,
   has an error, or the specified timeout occurs.
   msToWait must be > 0 . Returns receiver.
   Caller must have done _enableWrite or _enableRead for some GsSocket,
   and added current process to the socket's waiters list.
   writeBool should be nil or true .
   Caller has sent _enterCritical .
  "

  | proc aProc |
  (proc := self activeProcess) _waitingOn: aGsSocket .  "clear done by _reapSignal:"
  "assertion code"
  "true ifTrue:[
    (socketsPolled includesIdentical: aGsSocket) ifFalse:[ self pause ].
  ]."

  proc _signalTime: (self _now) + msToWait.
  self _addDelay: proc to: delayQueue .

  self _reschedule .
  self _enterCritical .
  aProc := self activeProcess.
  aProc ~~ proc ifTrue:[
    self _exitCritical .
    Error signal:'Unexpected: activeProcess changed during _waitForSocket:timeout:'.
  ].
  proc _signalTime ifNotNil: [
    delayQueue removeIdentical: proc otherwise: nil . "inline _delayUnschedule:"
    proc _signalTime: nil.
    proc _waitingOn: nil .
    writeBool ifNotNil:[ aGsSocket _cancelWriteEventFor: proc ]
		 ifNil:[ aGsSocket _cancelReadEventFor: proc ].
  ].
  self _exitCritical .
%

category: 'Accessing'
method: ProcessorScheduler
_waitingProcessesInto: anIdentitySet

  "Add all of the instances of GsProcess that
   are waiting for an event to anIdentitySet and return anIdentitySet"

  | que |
  anIdentitySet _addAll: waitingSet forReplay: false .
  (que := { } ) addAll: delayQueue .
  1 to: que size do: [ :idx |
     (que atOrNil: idx) ifNotNil:[:p |
        p _targetProcess ifNotNil:[:t| anIdentitySet add: t]
     ]
  ].
  ^ anIdentitySet
%

category: 'Accessing'
method: ProcessorScheduler
_waitingProcessesInto: aCollection inGroup: groupId

  "Add all of the instances of GsProcess that
   are waiting for an event and are in specified group
   to aCollection and return aCollection"

  | proc set |
  1 to: (set := waitingSet) size do:[:n |
    proc := set _at: n .
    proc _groupOrNil == groupId ifTrue:[ aCollection add: proc ].
  ].
  set := delayQueue .
  1 to: set size do: [ :n |
    proc := (set at: n) _targetProcess .
    proc ifNotNil:[
      proc _groupOrNil == groupId ifTrue:[ aCollection add: proc ].
    ].
  ].
  ^ aCollection
%

category: 'Private'
method: ProcessorScheduler
_waitOnSema: aSemaphore

  | proc ws |
  (proc := self activeProcess) _waitingOn: aSemaphore .
  self _add: proc toSet: (ws := waitingSet) .
  proc _onQueue: ws .

  self _reschedule .
%

category: 'Process State Change'
method: ProcessorScheduler
_yieldForTimeout
  "schedule a different active process, if any are
   available at a higher priority, and put the current process in the
   appropriate ready queue."

  | readyQ wasInCrit |
  wasInCrit := self _enterCritical .
  self _reapEvents: 1 .
  readyQ := readyQueue .
  (readyQ atOrNil: 1) ifNotNil:[:aProc |
    aProc priority > self activePriority  ifTrue: [ | oldProc newProc |
      oldProc := self activeProcess .

      newProc := aProc .
      readyQ removeFrom:1 to: 1 .
      newProc _onQueue: nil .

      oldProc ~~ newProc ifTrue:[ 
        self _addReadyProcess: oldProc to: readyQ .  "active process never on ready que"
        oldProc _onQueue: readyQ .
        self _switchFrom: oldProc to: newProc .   "prim does _exitCritical, updates activeProcess iv"
        ^ self
      ].
    ].
  ].
  wasInCrit == 0 ifTrue:[ self _exitCritical] .
%

! Class extensions for 'QuadByteString'

!		Class methods for 'QuadByteString'

removeallmethods QuadByteString
removeallclassmethods QuadByteString

category: 'Formatting'
classmethod: QuadByteString
charSize

"Returns number of bytes that make up a character for this string class."

^ 4
%

category: 'Instance Creation'
classmethod: QuadByteString
new

"Returns a new instance of the receiver."

^ self _basicNew
%

category: 'Instance Creation'
classmethod: QuadByteString
new: aSize

"Returns a new instance of the receiver, of sufficient size to hold aSize
 Characters of two bytes each."

^ self _basicNew:  aSize * 4
%

category: 'Instance Creation'
classmethod: QuadByteString
withAll: aString

"Returns an instance of the receiver containing the elements of the argument."
<primitive: 775>
^ super withAll: aString
%

!		Instance methods for 'QuadByteString'

category: 'Concatenating'
method: QuadByteString
, aCharOrCharColl

"Returns a new instance of the receiver's class that contains the elements of
 the receiver followed by the elements of aCharOrCharColl.  The argument
 must be a String, a DoubleByteString, or an AbstractCharacter.

 The result may not be an instance of the class of the receiver if one of the
 following rules applies:

 1) If the receiver is a  QuadByteSymbol, the result  is a  QuadByteString.

 Warning: Creating a new instance and copying the receiver take time.  If you
 can safely modify the receiver, it can be much faster to use the addAll:
 method.  See the documentation of the Concatenating category of class
 SequenceableCollection for more details."

<primitive: 444 >
aCharOrCharColl _validateClasses: { AbstractCharacter . CharacterCollection }.
^ super , aCharOrCharColl
%

category: 'Adding'
method: QuadByteString
add: aCharOrCharColl

"Appends all of the elements of aCharOrCharColl to the receiver and returns
 aCharOrCharColl."

<primitive: 776>
aCharOrCharColl class == Character ifTrue:[
  ^ self _primitiveFailed: #add: args: { aCharOrCharColl }
].
(aCharOrCharColl isKindOf: AbstractCharacter) ifTrue:[
  ^ self add: aCharOrCharColl asCharacter .
].
aCharOrCharColl _validateClass: CharacterCollection .
^ self add: aCharOrCharColl asQuadByteString
%

category: 'Adding'
method: QuadByteString
addAll: aCharOrCharColl

"Equivalent to add: aCharOrCharColl."

<primitive: 776>
aCharOrCharColl class == Character ifTrue:[
  ^ self _primitiveFailed: #add: args: { aCharOrCharColl }
].
(aCharOrCharColl isKindOf: AbstractCharacter) ifTrue:[
  ^ self add: aCharOrCharColl asCharacter .
].
(aCharOrCharColl isKindOf: CharacterCollection) ifTrue:[
  ^ self add: aCharOrCharColl asQuadByteString.
].
aCharOrCharColl do: [:each | self add: each].
^aCharOrCharColl.
%

category: 'Adding'
method: QuadByteString
addLast: aCharOrCharColl

"Equivalent to add: aCharOrCharColl."

<primitive: 776>
aCharOrCharColl class == Character ifTrue:[
  ^ self _primitiveFailed: #add: args: { aCharOrCharColl }
].
(aCharOrCharColl isKindOf: AbstractCharacter) ifTrue:[
  ^ self add: aCharOrCharColl asCharacter .
].
aCharOrCharColl _validateClass: CharacterCollection .
^ self add: aCharOrCharColl asQuadByteString
%

category: 'Deprecated'
method: QuadByteString
asciiLessThan: aString

"Returns true if the receiver collates before the argument using the
 ASCII collating table, which collates AB...Z...ab..z."

self deprecated: 'QuadByteString>>asciiLessThan: deprecated in 3.2.'.
^ self lessThan: aString collatingTable: QuadByteAsciiCollatingTable
%

category: 'Converting'
method: QuadByteString
asDoubleByteString

 "Returns a DoubleByteString (2 bytes per Character) representation of the receiver
 if all of the Characters in the receiver can be represented as two-byte Characters.
 Otherwise, returns the receiver."

| result |
(result := self _asDoubleByteString) ifNil:[ ^ self ].
^ result
%

category: 'Converting'
method: QuadByteString
asQuadByteString
  ^ self
%

category: 'Accessing'
method: QuadByteString
at: anIndex

"Returns the Character at anIndex."

<primitive: 655>
(anIndex _isSmallInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #at: args: { anIndex } .
self _uncontinuableError
%

category: 'Updating'
method: QuadByteString
at: anIndex put: aChar

"Stores aChar at the specified location and returns aChar."

<primitive: 667>

(aChar class == Character) ifFalse:[
  aChar _validateClass: AbstractCharacter .
  ^ self at: anIndex put: aChar asCharacter
].
(anIndex _isSmallInteger) ifTrue: [
  ((anIndex > (self size + 1)) or: [anIndex <= 0]) ifTrue: [
    ^ self _errorIndexOutOfRange: anIndex
    ].
] ifFalse:[
  ^ self _errorNonIntegerIndex: anIndex
] .
self _primitiveFailed: #at:put: args: { anIndex . aChar }
%

category: 'Formatting'
method: QuadByteString
charSize

"Returns number of bytes that make up a character for this string class."

^ 4
%

category: 'Accessing'
method: QuadByteString
codePointAt: anIndex

"Returns the integer value of the Character at anIndex in the receiver."

<primitive: 1073>

(anIndex _isSmallInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #codePointAt: args: { anIndex } .
self _uncontinuableError
%

category: 'Updating'
method: QuadByteString
codePointAt: anIndex put: aValue

"Stores codePoint aValue at the specified location, with autogrow.
 Returns aValue.
"
 self at: anIndex put: (Character withValue: aValue)
%

category: 'Encoding'
method: QuadByteString
encodeAsUTF16

"Encode receiver in UTF16 format.  Returns a Utf16 .
 Generates an error if any character value in receiver exceeds
 the maximum utf32 value 16r10FFFF."

<primitive: 1084>
self _primitiveFailed: #encodeAsUTF16
%

category: 'Encoding'
method: QuadByteString
encodeAsUTF8

"Encode receiver in UTF8 format.  Returns a Utf8 .
 Generates an error if any character value in receiver exceeds
 the maximum utf32 value 16r10FFFF."

<primitive: 468>
1 to: self size do:[:n | | v |
  v := self wordAt: n .
  v > 16r10FFFF ifTrue:[
    self error:'character at:', n asString , ' exceeds max legal utf 16r10FFFF'
  ].
].
self _primitiveFailed: #encodeAsUTF8
%

category: 'Encoding'
method: QuadByteString
encodeAsUTF8IntoString
"Encode receiver in UTF8 format.  Returns a String. For Seaside"

<primitive: 994>
1 to: self size do:[:n | | v |
  v := self wordAt: n .
  v > 16r10FFFF ifTrue:[
    self error:'character at:', n asString , ' exceeds max legal utf 16r10FFFF'
  ].
].
self _primitiveFailed: #encodeAsUTF8IntoString
%

category: 'Adding'
method: QuadByteString
insertAll: aCharOrCharColl at: anIndex

"Inserts aCharOrCharColl at the specified index.  Returns aCharOrCharColl."

<primitive: 447>
anIndex _isSmallInteger ifFalse:[ anIndex _validateClass: Integer ].
((anIndex <= 0) or: [anIndex > (self size + 1)])
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].
aCharOrCharColl _validateClasses: { AbstractCharacter . CharacterCollection }.
^self insertAll: aCharOrCharColl asDoubleByteString at: anIndex.
%

category: 'Copying'
method: QuadByteString
replaceFrom: startIndex to: stopIndex with: aSeqCollection startingAt: repIndex

"Replaces the elements of the receiver between the indexes startIndex and
 stopIndex inclusive with the elements of aSeqCollection starting at repIndex.
 If aSeqCollection is identical to the receiver, the source and
 destination blocks may overlap.

 Returns the receiver."

<primitive: 777>
| recInfo uStr |
startIndex _isSmallInteger ifFalse:[ startIndex _validateClass: SmallInteger ].
stopIndex _isSmallInteger ifFalse:[ stopIndex _validateClass: SmallInteger ].
repIndex _isSmallInteger ifFalse:[ repIndex _validateClass: SmallInteger ].
recInfo := self _stringCharSize .
(recInfo bitAnd: 16r8) ~~ 0 ifTrue:[  "receivier is a Utf string"
  (uStr := aSeqCollection asUnicodeString) ~~ aSeqCollection ifTrue:[
    ^ self replaceFrom: startIndex to: stopIndex with: uStr startingAt: repIndex .
  ].
].
^ super replaceFrom: startIndex to: stopIndex with: aSeqCollection startingAt: repIndex
%

category: 'Accessing'
method: QuadByteString
size

"Returns the number of Characters in the receiver."

^ self _basicSize // 4 .
%

category: 'Updating'
method: QuadByteString
size: anInteger

"Changes the size of the receiver to anInteger.

 If anInteger is less than the current size of the receiver, the receiver is
 shrunk accordingly.  If anInteger is greater than the current size of the
 receiver, the receiver is extended and new elements are initialized to
 codePoint zero."

^ super _basicSize: 4 * anInteger
%

category: 'Private'
method: QuadByteString
squeakBasicAt: anInteger put: aValue

"Disallowed, use codePointAt:put:"
self shouldNotImplement: #squeakBasicAt:put:
%

category: 'Converting'
method: QuadByteString
_asDoubleByteString

"Returns a DoubleByteString representation of the receiver if all of the
 Characters in the receiver can be represented as two-byte Characters.
 Returns nil if any of the Characters in the receiver cannot be represented as
 two-byte Characters."

<primitive: 702>
^ self _primitiveFailed: #_asDoubleByteString .
%

category: 'Private'
method: QuadByteString
_basicAt: anInteger put: aValue

"Disallowed, use codePointAt:put:"
self shouldNotImplement: #_basicAt:put:
%

category: 'Private'
method: QuadByteString
_basicSize: anInteger

"Disallowed"
self shouldNotImplement: #_basicSize:
%

! Class extensions for 'QuadByteSymbol'

!		Class methods for 'QuadByteSymbol'

removeallmethods QuadByteSymbol
removeallclassmethods QuadByteSymbol

category: 'Class Membership'
classmethod: QuadByteSymbol
isSubclassOf: aClassHistoryOrClass

  (self _classHistoryIncludesIdentical: aClassHistoryOrClass) ifTrue:[ ^ true].
  ^ super isSubclassOf: aClassHistoryOrClass
%

category: 'Storing and Loading'
classmethod: QuadByteSymbol
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into a new instance of the receiver.
 Returns the new instance."

| theSize str marker result |

"since Symbols can't refer to other objects, the 'hasRead:' message
 may be sent after values have been filled in."

theSize := passiveObj readSize .
str := String new: theSize .
marker := passiveObj objectPositionMarker .
passiveObj next: theSize bytesTo: str ;
           next .
result :=  (str _changeClassToMultiByte: QuadByteString) asSymbol .
passiveObj hasRead: result marker: marker .
^ result
%

category: 'Instance Creation'
classmethod: QuadByteSymbol
migrateNew

"Override default migrateNew behavior with #_basicNew."

^ self _basicNew
%

category: 'Instance Creation'
classmethod: QuadByteSymbol
new

"Disallowed."
self shouldNotImplement: #new
%

category: 'Instance Creation'
classmethod: QuadByteSymbol
new: aSize

"Disallowed."
self shouldNotImplement: #new:
%

category: 'Instance Creation'
classmethod: QuadByteSymbol
withAll: aString

"Returns a canonical symbol that has the same Characters as aString."

<primitive: 300>
^ Symbol withAll: aString
%

category: 'Instance Creation'
classmethod: QuadByteSymbol
_basicNew
^ self shouldNotImplement: #_basicNew
%

category: 'Instance Creation'
classmethod: QuadByteSymbol
_basicNew: aSize
^ self shouldNotImplement: #_basicNew:
%

category: 'Class Membership'
classmethod: QuadByteSymbol
_classHistoryIncludesIdentical: aClass
  ^ aClass == Symbol or:[ aClass == DoubleByteSymbol or:[ aClass == QuadByteSymbol]]
%

category: 'Instance Creation'
classmethod: QuadByteSymbol
_existingWithAll: aString

"Returns an existing canonical symbol that has the same value as 'aString'.
 If no such Symbol, DoubleByteSymbol or QuadByteSymbol already exists, returns nil."

"This method is useful in processing Strings that are supposed to be names
 of already existing instance variables, such as in indexing and sorting."

<primitive: 310>
aString _validateClass: CharacterCollection .
^ self _existingWithAll: aString asString .
%

category: 'Instance Creation'
classmethod: QuadByteSymbol
_newString

"Returns a new instance of String."

^ QuadByteString new
%

category: 'Instance Creation'
classmethod: QuadByteSymbol
_newString: aSize

"Returns a new instance of String."

^ QuadByteString new: aSize
%

!		Instance methods for 'QuadByteSymbol'

category: 'Concatenating'
method: QuadByteSymbol
, aCharOrCharCollection

"Returns a new instance of DoubleByteString that contains the elements of
 the receiver followed by the elements of aCharOrCharCollection.
 A DoubleByteString is returned rather than a DoubleByteSymbol to avoid
 the expense of unnecessary creation and canonicalization of Symbols."

| result |

result := QuadByteString withAll: self .
result  addAll: self; addAll: aCharOrCharCollection.
^ result
%

category: 'Converting'
method: QuadByteSymbol
-> anObject

"Returns a SymbolAssociation with the receiver as the key and the given object
 as the value."

^ SymbolAssociation newWithKey: self value: anObject
%

category: 'Comparing'
method: QuadByteSymbol
= anObject

"Returns true if anObject is equal to the receiver.  Since symbols
 are canonicalized, this method does the check based on the
 identities of the receiver and the argument."

^ self == anObject.
%

category: 'Compatiblity'
method: QuadByteSymbol
argumentCount

 ^ self numArgs
%

category: 'Converting'
method: QuadByteSymbol
asString

"Returns a copy of the receiver as a String."

^ String withAll: self.
%

category: 'Converting'
method: QuadByteSymbol
asSymbol

"Returns the receiver."

^ self
%

category: 'Converting'
method: QuadByteSymbol
asSymbolKind

"Returns the receiver.  "

^ self
%

category: 'Clustering'
method: QuadByteSymbol
cluster

"Has no effect.  Clustering of symbols is only performed by
 the clusterAllSymbols method in class System . "

^ true
%

category: 'Clustering'
method: QuadByteSymbol
clusterInBucket: aClusterBucketOrId

"Has no effect.  Clustering of symbols is only performed by
 the clusterAllSymbols method in class System . "

^ true
%

category: 'Copying'
method: QuadByteSymbol
copy

"Returns self.  Copies of (canonical) symbols are not allowed."

^ self
%

category: 'Copying'
method: QuadByteSymbol
copyReplacing: oldObject withObject: newObject
	"Returns a String comprising a copy of the receiver in which all occurrences
	 of objects equal to oldObject have been replaced by newObject."

	^self asQuadByteString copyReplacing: oldObject withObject: newObject
%

category: 'Comparing'
method: QuadByteSymbol
hash

"Returns a numeric hash key for the receiver.
 This implementation for new databases only and is
 replaced in the upgrade scripts."

^ self identityHash
%

category: 'Converting'
method: QuadByteSymbol
keywords

"Returns an Array of the keywords in the receiver, treating
 any colon-delimited segment as if it was a legal keyword."

^ Symbol _keywords: self
%

category: 'Testing'
method: QuadByteSymbol
precedence

"Returns the precedence of the receiver, were it a message selector, with
 1=unary, 2=binary and 3=keyword."

^self isInfix
    ifTrue: [ 2 ]
    ifFalse: [ self isKeyword ifTrue: [ 3 ]
                              ifFalse: [ 1 ] ]
%

category: 'Formatting'
method: QuadByteSymbol
printOn: aStream

"Puts a displayable representation of the receiver on the given stream.
 That representation conforms to GemStone Smalltalk parsing rules."

aStream nextPut: $# .
super printOn: aStream
%

category: 'Copying'
method: QuadByteSymbol
shallowCopy

"Returns self.  Copies of symbols are not allowed."

^ self
%

category: 'Class Membership'
method: QuadByteSymbol
species
  "(R) Returns a class similar to, or the same as, the receiver's class which
   can be used for containing derived copies of the receiver."

  ^ QuadByteString
%

category: 'Private'
method: QuadByteSymbol
speciesForConversion

^ QuadByteString .
%

category: 'Formatting'
method: QuadByteSymbol
withNoColons

"Returns a String containing the value of the receiver with all colons removed.

 A String is returned rather than a symbol to avoid the expense of unnecessary
 creation and canonicalization of symbols."

^ self copyWithout: $:   .
%

category: 'Decompiling without Sources'
method: QuadByteSymbol
_asSource

| result |
result := String new .
result add: $#  .
result addAll: super _asSource .
^ result
%

category: 'New Indexing Comparison'
method: QuadByteSymbol
_idxForCompareEqualTo: arg
  ""

  ^ arg _idxForCompareEqualToQuadByteSymbol: self
%

category: 'New Indexing Comparison'
method: QuadByteSymbol
_idxForCompareEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ aCharacterCollection _idxForCompareEqualToQuadByteSymbol: self
%

category: 'New Indexing Comparison - for Compare'
method: QuadByteSymbol
_idxForCompareEqualToCharacterCollection: aCharacterCollection
  "second half of a double dispatch call from CharacterCollection>>_idxForCompareEqualTo:."

  ^ false
%

category: 'New Indexing Comparison - for Compare'
method: QuadByteSymbol
_idxForCompareEqualToUnicode: aUnicodeString collator: aCollator
"second half of a double dispatch call from CharacterCollection>>_idxForCompareEqualTo:collator:."

  ^ false
%

category: 'New Indexing Comparison'
method: QuadByteSymbol
_idxForSortEqualTo: arg
  ""

  ^ super _idxForSortEqualToSymbol: arg
%

category: 'New Indexing Comparison'
method: QuadByteSymbol
_idxForSortEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ self asString
    _idxForSortEqualTo: aCharacterCollection
    collator: anIcuCollator
%

category: 'New Indexing Comparison'
method: QuadByteSymbol
_idxForSortEqualToSymbol: aSymbol
  "second half of a double dispatch call from Symbol>>_idxForSortEqualTo:."

  ^ self asString _idxPrimCompareEqualTo: aSymbol asString
%

category: 'New Indexing Comparison'
method: QuadByteSymbol
_idxForSortNotEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ (self _idxForSortEqualTo: aCharacterCollection collator: anIcuCollator) not
%

category: 'Comparing'
method: QuadByteSymbol
~= aCharCollection

"This method can be optimized for symbols since they are canonical."

^ self ~~ aCharCollection
%

! Class extensions for 'RepositoryError'

!		Instance methods for 'RepositoryError'

removeallmethods RepositoryError
removeallclassmethods RepositoryError

category: 'Instance initialization'
method: RepositoryError
initialize
  gsNumber := ERR_RepositoryError.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'RepositoryViewLost'

!		Instance methods for 'RepositoryViewLost'

removeallmethods RepositoryViewLost
removeallclassmethods RepositoryViewLost

category: 'Instance initialization'
method: RepositoryViewLost
initialize
  gsNumber := ERR_RepositoryViewLost.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'SecureSocketError'

!		Instance methods for 'SecureSocketError'

removeallmethods SecureSocketError
removeallclassmethods SecureSocketError

category: 'Instance initialization'
method: SecureSocketError
initialize
  gsNumber := ERR_SecureSocketError.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'SecurityError'

!		Instance methods for 'SecurityError'

removeallmethods SecurityError
removeallclassmethods SecurityError

category: 'Instance initialization'
method: SecurityError
initialize
  gsNumber := ERR_SecurityError.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'SelectBlock'

!		Instance methods for 'SelectBlock'

removeallmethods SelectBlock
removeallclassmethods SelectBlock

category: 'Accessing'
method: SelectBlock
argumentCount
	"A SelectBlock can only take one argument."

	^self queryBlock argumentCount.
%

category: 'Block Evaluation'
method: SelectBlock
ensure: aBlock
	"Evaluate the receiver.
	 Evaluate aBlock after evaluating the receiver,
	 or before any return from a block that would return to the sender.
	 Returns result of evaluating the receiver.

	 aBlock must be a zero-arg instance of ExecBlock, otherwise an error is generated. "

	"SelectBlocks require a single argument, so this selector does not apply."
	^self shouldNotImplement: #ensure:.
%

category: 'Block Evaluation'
method: SelectBlock
ifCurtailed: terminationBlock
	"Evaluate the receiver and return its result. If abnormal termination of the receiver occurs,
	 terminationBlock is evaluated. The value returned from the evaluation of
	 terminationBlock is discarded."

	"SelectBlocks require a single argument, so this selector does not apply."
	^self shouldNotImplement: #ifCurtailed:.
%

category: 'Accessing'
method: SelectBlock
iterationBlock

"Returns the value of the iterationBlock instance variable."

^iterationBlock
%

category: 'Block Evaluation'
method: SelectBlock
on: exceptionSelector do: handlerBlock
	"Try to evaluate the receiver, which should be a zero-argument block.
	 If an exception occurs and the expression exceptionSelector handles: theExceptionInstance
	 returns true, then evaluate the one argument block handlerBlock, passing it the exception
	 instance as its argument.

	 These forms are supported directly by the VM
		(1) on: anException do: handlerBlock
		(2) on: anExceptionSet do: handlerBlock
	 This form is handled by Smalltalk code in the body of this method.
		(3) on: anObject do: handlerBlock

	 anException must be the class Exception  or a subclass thereof;
	 anExceptionSet must be a kind of ExceptionSet;
	 handlerBlock must be an instance of ExecBloc otherwise an error is generated.

	 For forms 1, 2, and 3 if handlerBlock is not a one argument block,
	 an error is generated if exception handling attempts to invoke that handlerBlock.

	 If handlerBlock is invoked to handle an Exception which occurs during execution of the
	 receiver and handlerBlock completes normally , then the result of handlerBlock (the value
	 of the last expression in handlerBlock) will be the result of the on:do: send .  Other-than-normal
	 completion of handlerBlock is available by use of Exception's instance methods such as
	 #return, #return:, #retry, #retryUsing:, #resume, #resume:, #pass, and #outer within handlerBlock.

	 For forms 1 and 2, when searching for a handlerBlock to handle a signaled Exception, the VM
	 uses Behavior>>_subclassOf: semantics . classHistories of the class of the signaled Exception
	 and of anException or elements of anExceptionSet are ignored."

	"SelectBlocks require a single argument, so this selector does not apply."
	^self shouldNotImplement: #on:do:.
%

category: 'Accessing'
method: SelectBlock
queryBlock

"Returns the value of the queryBlock instance variable."

^queryBlock
%

category: 'Flow of Control'
method: SelectBlock
untilFalse
	^self shouldNotImplement: #untilFalse .
%

category: 'Flow of Control'
method: SelectBlock
untilFalse: aBlock
	^self shouldNotImplement: #untilFalse: .
%

category: 'Block Evaluation'
method: SelectBlock
value
	"Return the value of the receiver evaluated with no arguments.
	 If the receiver expects any arguments, signal an error."

	"SelectBlocks require a single argument, so this selector does not apply."
	^self shouldNotImplement: #value.
%

category: 'Block Evaluation'
method: SelectBlock
value: anObject

"Returns the value of the receiver evaluated with anObject as its
 argument.  A SelectBlock can only take one argument."

^iterationBlock value: anObject
%

category: 'Block Evaluation'
method: SelectBlock
value: firstObject value: secondObject
	"Return the value of the receiver evaluated with the specified arguments.  If
	 the receiver expects a different number of arguments, signal an error."

	"SelectBlocks require a single argument, so this selector does not apply."
	^self shouldNotImplement: #value:value:.
%

category: 'Block Evaluation'
method: SelectBlock
valueWithArguments: argList
	"Return the value of the receiver evaluated with the specified arguments.  If
	 the receiver expects a different number of arguments, signal an error."

	"SelectBlocks require a single argument, so this selector does not apply."
	^self shouldNotImplement: #valueWithArguments:.
%

category: 'Flow of Control'
method: SelectBlock
whileFalse
	"Evaluate the receiver once and then repeatedly as long as the value
	 returned by the evaluation is false."

	"SelectBlocks require a single argument, so this selector does not apply."
	^self shouldNotImplement: #whileFalse.
%

category: 'Flow of Control'
method: SelectBlock
whileFalse: aBlock
	"(Reserved selector.)  Evaluates the zero-argument block aBlock repeatedly
	 while the receiver evaluates to false.  Return nil.  Generates an error if the
	 receiver is not a zero-argument block."

	"SelectBlocks require a single argument, so this selector does not apply."
	^self shouldNotImplement: #whileFalse:.
%

category: 'Flow of Control'
method: SelectBlock
whileTrue
	"Evaluate the receiver once and then repeatedly as long as the value
	 returned by the evaluation is true."

	"SelectBlocks require a single argument, so this selector does not apply."
	^self shouldNotImplement: #whileTrue.
%

category: 'Flow of Control'
method: SelectBlock
whileTrue: aBlock
	"(Reserved selector.)  Evaluates the zero-argument block aBlock repeatedly
	 while the receiver evaluates to true.  Return nil.  Generates an error if the
	 receiver is not a zero-argument block."

	"SelectBlocks require a single argument, so this selector does not apply."
	^self shouldNotImplement: #whileTrue:.
%

! Class extensions for 'SequenceableCollection'

!		Class methods for 'SequenceableCollection'

removeallmethods SequenceableCollection
removeallclassmethods SequenceableCollection

category: 'Instance Creation'
classmethod: SequenceableCollection
with: aValue

"Returns an instance of the receiver containing the argument."
"Reimplemented to take advantage of the new: optimization "

| inst |
inst := self new: 1.
inst at: 1 put: aValue.
^inst
%

category: 'Instance Creation'
classmethod: SequenceableCollection
with: aValue with: val2

"Returns an instance of the receiver containing the arguments."

"Reimplemented to take advantage of the new: optimization "

| inst |

inst := self new: 2.
inst at: 1 put: aValue;
     at: 2 put: val2.

^inst
%

category: 'Instance Creation'
classmethod: SequenceableCollection
with: aValue with: val2 with: val3

"Returns an instance of the receiver containing the arguments."

"Reimplemented to take advantage of the new: optimization "

| inst |

inst := self new: 3.
inst at: 1 put: aValue;
     at:2 put: val2;
     at:3 put: val3.

^inst
%

category: 'Instance Creation'
classmethod: SequenceableCollection
with: aValue with: val2 with: val3 with: val4

"Returns an instance of the receiver containing the arguments."

"Reimplemented to take advantage of the new: optimization "

| inst |

inst := self new: 4.
inst at: 1 put: aValue;
     at:2 put: val2;
     at:3 put: val3;
     at:4 put: val4.

^inst
%

category: 'Indexing Support'
classmethod: SequenceableCollection
_canCreateQueryOnInstances
  "GsQuery may be created on most subclasses of Collection. Answer false if a GsQuery is not appropriate
   for the receiver."

  ^ true
%

!		Instance methods for 'SequenceableCollection'

category: 'Deprecated'
method: SequenceableCollection
+ aSequenceableCollection

self deprecated: 'SequenceableCollection>>+ deprecated long before v3.0. Use , instead' .
^ self , aSequenceableCollection
%

category: 'Concatenating'
method: SequenceableCollection
, aSequenceableCollection

"Returns a new instance of the receiver's class that contains the elements of
 the receiver followed by the elements of aSequenceableCollection.

 Warning: Creating a new instance and copying the receiver take time.  If you
 can safely modify the receiver, it can be much faster to use the addAll:
 method.  See the documentation of the Concatenating category for more details."

 | res mySize argSize |
 mySize := self size .
 argSize := aSequenceableCollection size .
 res := self class new: mySize + argSize .
 mySize ~~ 0 ifTrue:[
   res replaceFrom: 1 to: mySize with: self startingAt: 1 .
 ].
 argSize ~~ 0 ifTrue:[
   res replaceFrom: 1 + mySize to: mySize + argSize with: aSequenceableCollection startingAt: 1.
 ].
 ^ res
%

category: 'Comparing'
method: SequenceableCollection
= aSequenceableCollection

"Returns true if all of the following conditions are true:

 1.  The receiver and aSequenceableCollection are of the same class.
 2.  The two collections are the same size.
 3.  The corresponding elements of the receiver and aSequenceableCollection
     are equal."

| selfSize |

(self == aSequenceableCollection)
  ifTrue: [ ^ true ].

(self class == aSequenceableCollection class)
  ifFalse: [ ^ false ].

selfSize := self size.
(selfSize == aSequenceableCollection size)
  ifFalse: [ ^ false ].

1 to: selfSize do: [ :i|
  ((self at: i) = (aSequenceableCollection at: i))
    ifFalse: [ ^ false ].
  ].

^ true.
%

category: 'Enumerating'
method: SequenceableCollection
accompaniedBy: anObj do: aBlock

"Evaluate the two argument block aBlock with anObj and each element of
  the receiver.  Return the receiver. "

1 to: self size do: [:j |
  aBlock value: anObj value: (self at: j)
]
%

category: 'Adding'
method: SequenceableCollection
add: newObject

"Makes newObject one of the receiver's elements and returns newObject. The
 new element is actually added as the last element of the receiver. This
 method is therefore equivalent to #addLast:."

^ self at: ((self size) + 1) put: newObject.
%

category: 'Adding'
method: SequenceableCollection
add: newObject after: target

"Adds newObject to the receiver immediately after the first element that is
 equal to target.  An object immediately follows another if its index is one
 greater than that of the other.  Returns newObject if the operation is
 successful.  Raises an error if the operation fails."

| index |

index := self indexOf: target.
(index == 0) ifTrue: [ self _error: #rtErrObjNotFound args: { target }].
^ self insertObject: newObject at: (index + 1).
%

category: 'Adding'
method: SequenceableCollection
add: newObject before: target

"Adds newObject to the receiver immediately before the first element that is
 equal to target.  An object immediately precedes another if its index is one
 less than that of the other.  Returns newObject if the operation is successful.
 Raises an error if the operation fails."

| index |

index := self indexOf: target.
(index == 0) ifTrue: [ self _error: #rtErrObjNotFound args: { target }].
^ self insertObject:  newObject at: index.
%

category: 'Adding'
method: SequenceableCollection
addAll: aCollection afterIndex: index

"Adds all the elements of aCollection to the receiver in the traversal order
 defined by the do: method for aCollection.  Inserts the new elements into the
 receiver immediately after the element in the receiver at position index.  If
 index is equal to zero, inserts the elements of aCollection at the beginning of
 the receiver.  Returns aCollection.

 The argument index must be a non-negative integer less than or equal to the
 receiver's size."

^ self insertAll: aCollection at: (index + 1).
%

category: 'Adding'
method: SequenceableCollection
addAll: aCollection before: target

"Adds all the elements of aCollection to the receiver immediately before the
 first element that is equal to target.  An object immediately precedes another
 if its index is one less than that of the other.  Returns aCollection if the
 operation is successful.  Raises an error if the operation fails."

| index |

index := self indexOf: target.
(index == 0) ifTrue: [ self _error: #rtErrObjNotFound args: { target }].
^ self insertAll: aCollection at: index.
%

category: 'Adding'
method: SequenceableCollection
addAll: aCollection beforeIndex: index

"Adds all the elements of aCollection to the receiver in the traversal order
 defined by the do: method for aCollection.  Inserts the new elements into the
 receiver immediately before the element in the receiver at position index.  If
 index is equal to the receiver's size plus one, inserts the elements of
 aCollection at the end of the receiver.  Returns aCollection.

 The argument index must be a positive integer less than or equal to the
 receiver's size plus one."

^ self insertAll: aCollection at: index .
%

category: 'Adding'
method: SequenceableCollection
addLast: newObject

"Adds newObject as the last element of the receiver and returns newObject."

^ self at: ((self size) + 1) put: newObject.
%

category: 'Accessing'
method: SequenceableCollection
after: anObject

"Returns the object immediately following the first object which is equal to
 anObject in the receiver."

| index |

index := self indexOf: anObject
             ifAbsent: [^ self _errorNotFound: anObject].

^ self at: (index + 1).
%

category: 'Enumerating'
method: SequenceableCollection
allSatisfy: aBlock

  1 to: self size do: [ :j |
     (aBlock value: (self at:j) ) ifFalse: [ ^ false].
  ].
  ^true.
%

category: 'Enumerating'
method: SequenceableCollection
anySatisfy: aBlock

  1 to: self size do: [ :j |
    (aBlock value: (self at:j) ) ifTrue: [ ^ true ].
  ].
  ^false.
%

category: 'Comparing'
method: SequenceableCollection
at: anIndex equals: aSequenceableCollection

"Returns true if aSequenceableCollection is contained in the receiver starting
 at index anIndex. Returns false otherwise."

| seqCollSize index |
seqCollSize := aSequenceableCollection size.

((anIndex + seqCollSize - 1) > (self size))
  ifTrue: [ ^ false ].

index := anIndex.
1 to: seqCollSize do: [ :j|
  ((self at: index) = (aSequenceableCollection at: j))
    ifFalse: [ ^ false ].
  index := index + 1.
  ].

^ true.
%

category: 'Accessing'
method: SequenceableCollection
at: index ifAbsent: exceptionBlock
	"Answer the element at the position index in the receiver. If there is no position corresponding to
	index in the receiver, then answer the result of evaluating exceptionBlock."

	(index between: 1 and: self size) ifFalse: [^exceptionBlock value].
	^self at: index.
%

category: 'Updating'
method: SequenceableCollection
atAll: aCollection put: anObject

"The argument aCollection is a collection of Integers that are used as indexes
 into the receiver.  At each element in the receiver for which its index is in
 aCollection, this method replaces the element with the argument anObject.
 Returns anObject."

aCollection accompaniedBy: self do: [ :me :index | me at: index put: anObject].
^ anObject.
%

category: 'Updating'
method: SequenceableCollection
atAllPut: anObject

"Assigns anObject to each of the receiver's elements and returns anObject."

1 to: self size do: [ :index | self at: index put: anObject].
^ anObject.
%

category: 'Accessing'
method: SequenceableCollection
before: anObject

"Returns the object immediately before the first object which is equal to
 anObject in the receiver."

| index |

index := self indexOf: anObject
             ifAbsent: [^ self _errorNotFound: anObject].

^ self at: (index - 1).
%

category: 'Enumerating'
method: SequenceableCollection
collect: aBlock

"Evaluates aBlock with each of the receiver's elements as the argument.
 Collects the resulting values into a collection of class specified by
 sending #speciesForCollect message to the receiver. Returns the new
 collection. The argument aBlock must be a one-argument block.

 The result preserves the ordering of the receiver.  That is, if element a
 comes before element b in the receiver, then element a is guaranteed to come
 before b in the result."

|result mySize|

mySize := self size.
result:= self speciesForCollect new: mySize .
1 to: mySize do: [:i| result at:i put:(aBlock value: (self at:i))] .
^result
%

category: 'Copying'
method: SequenceableCollection
copyEmpty

"Returns an empty copy of the receiver."

^self species new
%

category: 'Deprecated'
method: SequenceableCollection
copyFrom: index1 count: aCount into: aColl startingAt: destIndex

"Deprecated, non-ANSI.
 Copies elements of the receiver from (receiver at: index1)
  to (receiver at: (index2 + aCount - 1)), inclusive
 into aColl starting at destIndex, overwriting the previous contents.

 If (index2 + aCount - 1) is beyond the end of the receiver, then
 the copy will stop with the element  (receiver at: (receiver size)) .

 aColl must be a kind of SequenceableCollection .

 Returns the number of elements copied."

  | count |
  self deprecated: 'SequenceableCollection>>copyFrom:count:into:startingAt: deprecated v3.0' .
  count :=  aCount min: ( self size - index1 + 1) .
  aColl replaceFrom: destIndex to: destIndex + count - 1
          with: self startingAt: index1 .
  ^ count
%

category: 'Copying'
method: SequenceableCollection
copyFrom: startIndex to: stopIndex

"Returns a new SequenceableCollection containing the elements of the receiver
 between startIndex and stopIndex, inclusive.  The result is of the same class
 as the receiver.

 Both startIndex and stopIndex must be positive integers not larger than the
 size of the receiver, with startIndex <= stopIndex.
 If startIndex > stopIndex and both are positive, an empty collection is returned.
 "

| result resSize |
(startIndex > stopIndex) ifTrue: [
  stopIndex < 0 ifTrue:[ self _error: #rtErrBadCopyFromTo args: { stopIndex } ].
  ^ self class new
].
(startIndex < 1)
   ifTrue: [ ^ self _errorIndexOutOfRange: startIndex].

( (stopIndex > self size) or: [(stopIndex < 0)])
   ifTrue: [ ^ self _errorIndexOutOfRange: stopIndex].

result := self class new: (resSize := stopIndex - startIndex + 1).
result replaceFrom: 1 to: resSize with: self startingAt: startIndex .
^ result
%

category: 'Deprecated'
method: SequenceableCollection
copyFrom: index1 to: index2 into: aSeqCollection startingAt: destIndex

"Deprecated, non-ANSI.
 Copies the elements of the receiver between index1 and index2, inclusive,
 into aSeqCollection starting at destIndex, overwriting the previous
 contents.  If aSeqCollection is the same object as the receiver, the
 source and destination blocks may overlap. Returns the receiver."

 self deprecated: 'SequenceableCollection>>copyFrom:to:into:startingAt: deprecated v3.0.' .

 "bug42397--->"
 (index1 > index2)
  ifTrue: [  self _error: #rtErrBadCopyFromTo args: { index2 } ].

(index1 < 1)
  ifTrue: [ ^ self _errorIndexOutOfRange: index1].

(index2 > self size)
  ifTrue: [ ^ self _errorIndexOutOfRange: index2].

((destIndex < 1) or: [(destIndex > (aSeqCollection size + 1))])
   ifTrue: [ ^ aSeqCollection _errorIndexOutOfRange: destIndex].

 aSeqCollection replaceFrom: destIndex to: destIndex + (index2 - index1)
        with: self startingAt: index1 .
 ^ self
%

category: 'Copying'
method: SequenceableCollection
copyReplaceAll: oldSubCollection with: newSubCollection

"Returns a modified copy of the receiver in which all sequences of elements
 within the receiver that match the elements of oldSubCollection are replaced
 by the elements of newSubCollection."

| selfSize newSubCollSize oldSubCollSize result sourceIndex targetIndex
  subIndex |

	newSubCollection class isIndexable
		ifFalse: [^newSubCollection _errorNotIndexable].

selfSize := self size.
newSubCollSize := newSubCollection size.
oldSubCollSize := oldSubCollection size.

result := self species new.

sourceIndex := 1.
targetIndex := 1.

[ sourceIndex <= selfSize ] whileTrue: [
  subIndex := self indexOfSubCollection: oldSubCollection
              startingAt: sourceIndex
              ifAbsent: [
                result replaceFrom: targetIndex to: targetIndex + (selfSize - sourceIndex)
                       with: self startingAt: sourceIndex .
                ^ result.
              ].

  (subIndex > sourceIndex) ifTrue: [
      result replaceFrom: targetIndex to: targetIndex + ((subIndex - 1) - sourceIndex)
			with: self startingAt: sourceIndex .
      targetIndex := targetIndex + (subIndex - sourceIndex).
  ].

  (newSubCollSize ~~ 0) ifTrue: [
     result replaceFrom: targetIndex to: targetIndex + (newSubCollSize  - 1)
		with: newSubCollection startingAt: 1
  ].

  targetIndex := targetIndex + newSubCollSize.
  sourceIndex := subIndex + oldSubCollSize.
  ].

^ result.
%

category: 'Copying'
method: SequenceableCollection
copyReplaceFrom: startIndex to: stopIndex with: aSequenceableCollection
	"Returns a copy of the receiver in which all elements in the
	 receiver between indexes startIndex and stopIndex inclusive have
	 been replaced by those contained in aSequenceableCollection.

	 Rules, abbreviated from the ANSI specification:
		- start < 1 or start > receiver's size + 1 (error)
		- stop < start - 1 or stop > receiver's size (error)
		- stop = start - 1 and start <= receiver's size (insert between stop and start)
		- stop = start - 1 and start = receiver's size + 1 (append at the end of the new collection)
		- otherwise, replace from start through stop with the elements from replacementElements.
	"

	| selfSize seqCollSize result |
	aSequenceableCollection class isIndexable
		ifFalse: [^aSequenceableCollection _errorNotIndexable].

	selfSize := self size.
	seqCollSize := aSequenceableCollection size.
	result := self species new.
	(startIndex < 1
		or: [startIndex > (selfSize + 1)
		or: [stopIndex < (startIndex - 1)
		or: [stopIndex > selfSize]]])
			ifTrue: [^startIndex _error: #rtErrBadCopyFromTo args: {stopIndex}].

		"The stopIndex < startIndex case is an insertion/append operation."
	stopIndex = (startIndex - 1)
		ifTrue:
			[result
				replaceFrom: 1
					to: stopIndex
					with: self
					startingAt: 1;
				replaceFrom: stopIndex + 1
					to: stopIndex + seqCollSize
					with: aSequenceableCollection
					startingAt: 1;
				replaceFrom: stopIndex + seqCollSize + 1
					to: selfSize + seqCollSize
					with: self
					startingAt: stopIndex + 1]
		ifFalse:
			"The stopIndex >= startIndex case is the 'normal' operation."
			[result
				replaceFrom: 1
					to: startIndex - 1
					with: self
					startingAt: 1;
				replaceFrom: startIndex
					to: startIndex + seqCollSize - 1
					with: aSequenceableCollection
					startingAt: 1;
				replaceFrom: result size + 1
					to: seqCollSize + selfSize - (stopIndex - startIndex + 1)
					with: self
					startingAt: stopIndex + 1].

	^result
%

category: 'Copying'
method: SequenceableCollection
copyReplaceFrom: startIndex to: stopIndex withObject: anObject

"Returns a copy of the receiver in which all elements in the receiver between
 indexes startIndex and stopIndex inclusive have been replaced by anObject."

^self
   copyReplaceFrom: startIndex
   to: stopIndex
   with: {anObject}.
%

category: 'Copying'
method: SequenceableCollection
copyReplacing: oldObject with: newObject

"Returns a copy of the receiver in which all occurrences of objects equal to
 oldObject have been replaced by newObject."

  ^self copyReplacing: oldObject withObject: newObject
%

category: 'Copying'
method: SequenceableCollection
copyReplacing: oldObject withObject: newObject

"Returns a copy of the receiver in which all occurrences of objects equal to
 oldObject have been replaced by newObject."

| result |

result := self copy.
1 to: result size do: [ :i |
  (oldObject = (result at: i))
    ifTrue: [ result at: i put: newObject ]
  ].

^ result.
%

category: 'Copying'
method: SequenceableCollection
copyWith: anObject

"Returns a copy of the receiver with anObject appended at the end."

| newCollection |

newCollection := self copy.
newCollection addLast: anObject.

^ newCollection.
%

category: 'Copying'
method: SequenceableCollection
copyWithout: anObject

"Returns a copy of the receiver that does not contain the given object.
 Comparisons are by equality."

| copy element sz |

copy := self class new.

sz := 0.
1 to: self size do: [:i |
  element := self at: i.
  (element = anObject)
    ifFalse: [
      sz := sz + 1.
      copy at: sz put: element.
      ]
  ].

^copy
%

category: 'Deprecated'
method: SequenceableCollection
deleteFrom: startIndex to: stopIndex

self deprecated: 'SequenceableCollection>>deleteFrom:to: deprecated long before v3.0. Use removeFrom:to: instead.'.
^ self removeFrom: startIndex to: stopIndex.
%

category: 'Deprecated'
method: SequenceableCollection
deleteObjectAt: anIndex

self deprecated: 'SequenceableCollection>>deleteObjectAt: deprecated long before v3.0. Use removeAtIndex: instead.'.
^ self removeAtIndex: anIndex
%

category: 'Enumerating'
method: SequenceableCollection
detect: aBlock ifNone: exceptionBlock
  "Evaluates aBlock repeatedly, with elements of the receiver as the argument.
 Returns the first element for which aBlock evaluates to true.  If none of
 the receiver's elements evaluates to true, this evaluates the argument
 exceptionBlock and returns its value.  The argument aBlock must be a
 one-argument block, and exceptionBlock must be a zero-argument block."

  | result |
  aBlock _isExecBlock ifFalse:[
    aBlock class == SelectBlock
      ifTrue: [
      "send value: to the block to create the 4 element
        associative access Array; the argument nil is ignored to
        create the Array, but all SelectBlocks have one argument"
      result := (QueryExecuter on: self)
        _boundQueryDetect: (aBlock queryBlock value: nil).
      result == #'_incompletePathTraversal'
        ifTrue: [ ^ exceptionBlock value ].
      ^ result ]].
  1 to: self size do: [ :i | | v |
    (aBlock value: (v := self at: i))
      ifTrue: [ ^ v ] ].
  ^ exceptionBlock value
%

category: 'Enumerating'
method: SequenceableCollection
doWithIndex: aBlock

"Evaluates the two argument block aBlock using each element of the receiver as
 the first argument and the corresponding index as the second argument. Returns
 the receiver."

1 to: self size do: [ :i |
  aBlock value: (self at: i) value: i.
  ].

^ self.
%

category: 'Deprecated'
method: SequenceableCollection
eq: aSequenceableCollection

self deprecated: 'SequenceableCollection>>eq: deprecated long before v3.0.  Use = instead.'.
^ self = aSequenceableCollection.
%

category: 'Searching'
method: SequenceableCollection
findFirst: aBlock

"Returns the index of the first element in the receiver which causes the one
 argument block, aBlock, to evaluate true. Returns 0 if no element in the
 receiver causes the block to evaluate true."

1 to: self size do: [:i|
  (aBlock value: (self at: i)) ifTrue: [^i]].

^0
%

category: 'Searching'
method: SequenceableCollection
findLast: aBlock

"Returns the index of the last element in the receiver which causes the one
 argument block, aBlock, to evaluate true. Returns 0 if no element in the
 receiver causes the block to evaluate true."

self size _downTo: 1 do: [:i|
  (aBlock value: (self at: i)) ifTrue: [^i]].

^0
%

category: 'Accessing'
method: SequenceableCollection
first

"Returns the first element of the receiver.  Generates an error if the receiver
 is empty."

(self size == 0)
ifTrue: "attempt to access an empty Collection"
   [ ^ self _error: #objErrCollectionEmpty].
^ self at: 1
%

category: 'Updating'
method: SequenceableCollection
first: anObject

"Stores the given object in the first position in the receiver and returns
 anObject."

^ self at: 1 put: anObject.
%

category: 'Enumerating'
method: SequenceableCollection
from: startIndex to: stopIndex do: aBlock

"Evaluates the one argument block aBlock using each element of the receiver
 starting at startIndex and ending at stopIndex. Returns the receiver."

(startIndex < 1 or:[ stopIndex > self size])
   ifTrue: [ ^ self _errorIndexOutOfRange: startIndex ].

startIndex to: stopIndex do: [ :i|
  aBlock value: (self at: i).
  ].

^ self.
%

category: 'Enumerating'
method: SequenceableCollection
from: startIndex to: stopIndex doWithIndex: aBlock

"Evaluates the two argument block aBlock using each element of the receiver
 starting at startIndex and ending at stopIndex as the first argument and the
 corresponding index as the second argument. Returns the receiver."

(startIndex < 1 or:[ stopIndex > self size])
   ifTrue: [ ^ self _errorIndexOutOfRange: startIndex ].

startIndex to: stopIndex do: [ :i|
  aBlock value: (self at: i) value: i.
  ].

^ self.
%

category: 'Enumerating'
method: SequenceableCollection
from: startIndex to: stopIndex keysAndValuesDo: aDyadicBlock

"For each index in the range <startIndex> to <stopIndex>, the 2-argument block
 <aDyadicBlock> is evaluated with the index as the first argument and the element
 at that index as the second argument."

	^startIndex to: stopIndex do: [:index | aDyadicBlock value: index value: (self at: index)]
%

category: 'Comparing'
method: SequenceableCollection
hash

"Returns a numeric hash key for the receiver."

| mySize interval hashValue |

(mySize := self size) == 0
  ifTrue: [ ^15243 ].

"Choose an interval so that we sample at most 5 elements of the receiver"
interval := ((mySize - 1) // 4) max: 1.

hashValue := 4459.
1 to: mySize by: interval do: [ :i | | anElement |
  anElement := self at: i.
  (anElement isKindOf: SequenceableCollection)
    ifTrue: [
      hashValue := (hashValue bitShift: -1) bitXor: anElement size.
      ]
    ifFalse: [
      hashValue := (hashValue bitShift: -1) bitXor: anElement hash.
      ].
  ].

^ hashValue abs
%

category: 'Comparing'
method: SequenceableCollection
hasIdenticalContents: aSequenceableCollection

"Returns true if all of the following conditions are true:

 1.  The receiver and aSequenceableCollection are of the same class.
 2.  The two collections are the same size.
 3.  The corresponding elements of the receiver and aSequenceableCollection
     are identical."

| selfSize |

(self == aSequenceableCollection)
  ifTrue: [ ^ true ].

(self class == aSequenceableCollection class)
  ifFalse: [ ^ false ].

selfSize := self size.
(selfSize == aSequenceableCollection size)
  ifFalse: [ ^ false ].

1 to: selfSize do: [ :i|
  ((self at: i) == (aSequenceableCollection at: i))
    ifFalse: [ ^ false ].
  ].

^ true.
%

category: 'Searching'
method: SequenceableCollection
identicalOccurrencesOf: anObject

"Returns the number of the receiver's elements that are identical (==) to
 anObject."

| result |

result := 0.
1 to: self size do: [ :j |
  anObject == (self at: j) ifTrue: [ result := result + 1 ]
  ].
^result
%

category: 'Searching'
method: SequenceableCollection
includes: anObject

"Returns true if anObject is equal to one of the elements of the receiver.
 Returns false otherwise."

1 to: self size do:[:j |
  anObject = (self at:j ) ifTrue:[ ^ true ]
].
^false
%

category: 'Searching'
method: SequenceableCollection
includesIdentical: anObject

"Returns true if anObject is identical to one of the elements of the
 receiver.  Returns false otherwise."

^ (self indexOfIdentical: anObject) ~~ 0
%

category: 'Searching'
method: SequenceableCollection
indexOf: anObject

"Returns the index of the first element in the receiver that is equivalent to
 the argument. If the receiver does not have any elements equivalent to the
 argument, returns 0."

^ self indexOf: anObject startingAt: 1
%

category: 'Searching'
method: SequenceableCollection
indexOf: anObject ifAbsent: anExceptionBlock

"Returns the index of the first element in the receiver that is equivalent to
 the argument. If the receiver does not have any elements equivalent to the
 argument, returns the value of evaluating anExceptionBlock."

1 to: self size do:[:i |
  anObject = (self at: i) ifTrue:[ ^ i ]
].
^ anExceptionBlock value.
%

category: 'Searching'
method: SequenceableCollection
indexOf: anObject startingAt: index

"Returns the index of the first element in the receiver that is equivalent to
 the argument. If the receiver does not have any elements equivalent to the
 argument, returns 0."

index to: self size do: [:i |
  anObject = (self at: i) ifTrue:[ ^ i ]
].
^ 0
%

category: 'Searching'
method: SequenceableCollection
indexOf: anObject startingAt: index ifAbsent: anExceptionBlock

"Returns the index of the first element in the receiver that is equivalent to
 the argument. If the receiver does not have any elements equivalent to the
 argument, returns the value of evaluating anExceptionBlock"

index to: self size do: [ :i |
  (anObject = (self at: i)) ifTrue: [ ^i ].
  ].

^ anExceptionBlock value.
%

category: 'Searching'
method: SequenceableCollection
indexOfIdentical: anObject

"Private.  Returns the index of the first element in the receiver that is
 identical to the argument.  If the receiver does not have any elements that are
 identical to the argument, returns zero."

1 to: self size do: [:i |
  (anObject == (self at: i)) ifTrue: [^i]
  ].

^ 0.
%

category: 'Searching'
method: SequenceableCollection
indexOfSubCollection: aSubColl

"Returns the index of the first element of the receiver where that element and
 the subsequent ones are equal to those in aSubColl. The search is begun in the
 receiver starting at 1."

^ self indexOfSubCollection: aSubColl startingAt: 1.
%

category: 'Searching'
method: SequenceableCollection
indexOfSubCollection: aSubColl startingAt: anIndex

"Returns the index of the first element of the receiver where that element and
 the subsequent ones are equal to those in aSubColl. The search is begun in the
 receiver starting at anIndex. Returns 0 if no match is found."

^ self indexOfSubCollection: aSubColl startingAt: anIndex ifAbsent: [0].
%

category: 'Searching'
method: SequenceableCollection
indexOfSubCollection: aSubColl startingAt: anIndex ifAbsent: anExceptionBlock

"Returns the index of the first element of the receiver where that element and
 the subsequent ones are equal to those in aSubColl. The search is begun in the
 receiver starting at anIndex. Returns the value of evaluating
 anExceptionBlock if no match is found."

| subCollSize |

subCollSize := aSubColl size.
(aSubColl size == 0) ifTrue: [ ^ anExceptionBlock value ].

anIndex to: (self size - subCollSize + 1) do: [ :i |
  (self at: i equals: aSubColl) ifTrue: [ ^ i ].
  ].

^ anExceptionBlock value.
%

category: 'Deprecated'
method: SequenceableCollection
indexOfValue: anObject

self deprecated: 'SequenceableCollection>>indexOfValue: deprecated long before v3.0. Use indexOf: instead.'.
^ self indexOf: anObject.
%

category: 'Deprecated'
method: SequenceableCollection
insert: aSequenceableCollection at: anIndex

self deprecated: 'SequenceableCollection>>insert:at: deprecated long before v3.0. Use insertAll:at: instead.'.
aSequenceableCollection _validateClass: SequenceableCollection.
^ self insertAll: aSequenceableCollection at: anIndex.
%

category: 'Adding'
method: SequenceableCollection
insertAll: aCollection at: anIndex

"Inserts all the elements of aCollection into the receiver beginning at index
 anIndex.  Returns aCollection.

 The argument anIndex must be greater than or equal to one.  If anIndex is one
 greater than the size of the receiver, appends aCollection to the receiver.  If
 anIndex is more than one greater than the size of the receiver, generates an
 error."

<primitive: 606 >
| array argCls |
argCls := aCollection class .
(aCollection isKindOf: Interval) ifTrue:[
  self insertAll: aCollection asArray at: anIndex  .
  ^ aCollection
  ].
((argCls isSubclassOf: CharacterCollection )
 or:[ argCls isSubclassOf: ByteArray ]) ifTrue:[
   array := Array new: aCollection size .
   1 to: aCollection size do:[:j|
     array at: j put: (aCollection at: j)
     ].
   self insertAll: array at: anIndex .
   ^ aCollection
   ].
aCollection _validateClasses:
  { Array . OrderedCollection . SequenceableCollection }.
(anIndex _isInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
(anIndex < 1 or: [ anIndex > (self size + 1)]) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].

^ self _primitiveFailed: #insertAll:at: args: { aCollection . anIndex }
%

category: 'Adding'
method: SequenceableCollection
insertObject: anObject at: anIndex

"Inserts anObject into the receiver at index anIndex and returns anObject.
 If receiver is a small (<= 2024 oops) committed object, does space-efficent
 tranlogging of the insertion.
"

<primitive: 398>
"primitive fails if receiver is byte format"
self insertAll: { anObject } at: anIndex .
^ anObject
%

category: 'Enumerating'
method: SequenceableCollection
keysAndValuesDo: aDyadicBlock

"The 2-argument block <aDyadicBlock> is evaluated with the index of each element
 of the receiver as the first argument and the element itself as the second argument.
 Evaluation is in indexed order with the first element being at index 1."

	^self from: 1 to: self size keysAndValuesDo: aDyadicBlock
%

category: 'Accessing'
method: SequenceableCollection
last

"Returns the last element of the receiver.  Generates an error if the receiver
 is empty."

(self size == 0)
ifTrue: "attempt to access an empty Collection"
   [ ^ self _error: #objErrCollectionEmpty].
^self at: self size
%

category: 'Updating'
method: SequenceableCollection
last: anObject

"Stores the given object in the last position in the receiver and returns
 anObject."

^ self at: (self size) put: anObject.
%

category: 'Searching'
method: SequenceableCollection
occurrencesOf: anObject

"Returns the number of the receiver's elements that are equal to anObject."

| result |

result := 0.
1 to: self size do: [ :j |
  anObject = (self at: j) ifTrue: [ result := result + 1 ]
  ].
^result
%

category: 'Searching'
method: SequenceableCollection
occurrencesOfDistinctEqualSubCollection: aCollection
    "Answer the number of *distinct* *non-overlapping* occurrences of
    aCollection in the receiver using equality not identity."

    | otherSize selfSize currentStart currentResult count |
    otherSize := aCollection size.
    selfSize := self size.
    count := 0.
    currentStart := 1.
    [(currentStart + otherSize - 1) <= selfSize] whileTrue:
        [currentResult := self indexOfSubCollection: aCollection startingAt: currentStart ifAbsent: [^count].
         count := count + 1.
         currentStart := currentResult + otherSize].
    ^count
%

category: 'Json'
method: SequenceableCollection
printJsonOn: aStream

  | delimiter |
  delimiter := ''.
  aStream nextPut: $[.
  1 to: self size do: [ :j |
    aStream nextPutAll: delimiter.
    (self at: j) printJsonOn: aStream.
    delimiter := ','.
  ].
  aStream nextPut: $].
%

category: 'Streams'
method: SequenceableCollection
readStream

"Returns a ReadStream on the receiver."

^ ReadStream on: self
%

category: 'Enumerating'
method: SequenceableCollection
reject: aBlock
  "Evaluates aBlock with each of the receiver's elements as the argument.
 Stores the values for which aBlock is false into a collection of the same
 class as the receiver, and returns the new collection.  The argument aBlock
 must be a one-argument block.

 The result preserves the ordering of the receiver.  That is, if element a
 comes before element b in the receiver, then element a is guaranteed to come
 before b in the result."

  | result |
  aBlock _isExecBlock ifFalse:[
   aBlock class == SelectBlock
    ifTrue: [
      "send value: to the block to create the 4 element
        associative access Array; the argument nil is ignored to
        create the Array, but all SelectBlocks have one argument"
      result := (QueryExecuter on: self)
        _boundQueryReject: (aBlock queryBlock value: nil).
      ^ result ]].
  result := self species new.
  1 to: self size do: [ :i | | element |
    element := self at: i.
    (aBlock value: element)
      ifFalse: [ result add: element ] ].
  ^ result
%

category: 'Removing'
method: SequenceableCollection
remove: oldObject ifAbsent: anExceptionBlock

"Removes from the receiver an object that is equivalent to oldObject.
 Returns oldObject.  If several elements of the receiver are equivalent to
 oldObject, only one instance is removed.  If oldObject has no equivalent
 elements in the receiver, evaluates anExceptionBlock and returns the result
 of the evaluation."


| index |

index := self indexOf: oldObject.
index == 0
  ifTrue: [^ anExceptionBlock value]
  ifFalse: [^ self removeAtIndex: index]
%

category: 'Deprecated'
method: SequenceableCollection
removeAll: aCollection ifAbsent: errorBlock

self deprecated: 'SequenceableCollection>>removeAll:ifAbsent: deprecated before 3.0. Use removeAllPresent:'.

"Removes all occurrences of the each of the elements of aCollection from the
 receiver.  Iterates over aCollection using do: [required].  Executes the given
 one-argument block if an object in aCollection is not in the receiver.  The
 block argument may be nil, in which case the object-not-found error will be
 ignored."

self == aCollection ifTrue:[
  self size: 0 .
  ^ aCollection
  ].
errorBlock == nil ifTrue:[
  aCollection accompaniedBy: self do: [:me :obj | | idx |
    idx := me indexOf: obj.
    [ idx ~~ 0 ] whileTrue: [
      me removeFrom: idx to: idx.
      idx := me indexOf: obj startingAt: idx
    ].
  ].
] ifFalse:[
  self _removeAll: aCollection ifAbsent: errorBlock .
].
^ aCollection
%

category: 'Removing'
method: SequenceableCollection
removeAllSuchThat: aBlock

"Removes all elements of the receiver for which aBlock returns true.  Returns
 the removed elements."

| indexes result |

indexes := { } .
result := self speciesForCollect new.
1 to: self size do: [:idx | | item |
  item := (self at: idx).
  (aBlock value: item) ifTrue: [
    result add: item.
    indexes add: idx
  ]
].

indexes reverseDo: [:each |
  self removeFrom: each to: each
].
^ result
%

category: 'Removing'
method: SequenceableCollection
removeAtIndex: anIndex

"Removes the element at the given index.  Returns the removed element."

| anObj |
anObj := self at: anIndex .
self removeFrom: anIndex to: anIndex  .
^ anObj
%

category: 'Removing'
method: SequenceableCollection
removeFirst

"Removes the first element of the receiver.  Returns the removed element."

^ self removeAtIndex: 1
%

category: 'Removing'
method: SequenceableCollection
removeFrom: startIndex to: stopIndex

"Removes the elements of the receiver from startIndex to stopIndex inclusive.
 Returns the receiver.

 The size of the receiver is decreased by stopIndex - startIndex + 1."

<primitive: 601>

(stopIndex < startIndex)
ifTrue:
   [ ^ startIndex _error: #rtErrBadCopyFromTo args: { stopIndex }].
((stopIndex > self size) or: [(stopIndex < 1)])
   ifTrue: [ ^ self _errorIndexOutOfRange: stopIndex].
(startIndex < 1)
   ifTrue: [ ^ self _errorIndexOutOfRange: startIndex].
((self isKindOf: CharacterCollection )
  or:[ self isKindOf: ByteArray ]) ifTrue:[
   | range |
   range := stopIndex - stopIndex + 1.
   startIndex to: stopIndex do: [ :i |
     self at: i put: (self at: (i + range))].
   self size: (self size - range).
   ^ self
   ].
^ self _primitiveFailed: #removeFrom:to: args: { startIndex . stopIndex }
%

category: 'Removing'
method: SequenceableCollection
removeIdentical: oldObject ifAbsent: anExceptionBlock

"Removes from the receiver an object that is identical to oldObject.
 Returns oldObject.  If several elements of the receiver are identical to
 oldObject, only one instance is removed.  If oldObject is not present
 in the receiver, evaluates anExceptionBlock and returns the result
 of the evaluation."

| idx |

idx := self indexOfIdentical: oldObject.
(idx == 0) ifTrue: [
  ^ anExceptionBlock value.
] ifFalse: [
  ^ self removeAtIndex: idx.
].
%

category: 'Removing'
method: SequenceableCollection
removeIdentical: anObject otherwise: notFoundValue

"Removes from the receiver an object that is identical to anObject and
 returns anObject.  If several elements of the receiver are identical to
 oldObject, only one instance is removed.  If anObject is not present in
 the receiver, returns notFoundValue."

| idx |

idx := self indexOfIdentical: anObject.
(idx == 0) ifTrue: [
  ^ notFoundValue
] ifFalse: [
  ^ self removeAtIndex: idx.
].
%

category: 'Removing'
method: SequenceableCollection
removeIfPresent: anElement

"Removes anObject from the receiver and returns anObject.  If anObject is
 present several times in the receiver, only one occurrence is removed.
 Returns nil if anObject is missing from the receiver."

  | index |
  index := self indexOf: anElement .
  index == 0 ifFalse:[
    self removeAtIndex: index .
    ^ anElement .
  ].
  ^ nil
%

category: 'Deprecated'
method: SequenceableCollection
removeIndex: index

self deprecated: 'SequenceableCollection>>removeIndex: Obsolete in GemStone/64.  Use the removeAtIndex: method instead.'.
^ self removeAtIndex: index.
%

category: 'Removing'
method: SequenceableCollection
removeLast

"Removes the last element of the receiver.  Returns the removed element."

| lastElement selfSize |

selfSize := self size.
lastElement := self at: selfSize.
self size: (selfSize - 1).

^ lastElement.
%

category: 'Deprecated'
method: SequenceableCollection
removeValue: oldObject

self deprecated: 'SequenceableCollection>>removeValue: Obsolete in GemStone/64.  Use the remove: method instead.'.
^ self remove: oldObject.
%

category: 'Updating'
method: SequenceableCollection
replaceFrom: startIndex to: stopIndex with: aCollection

"Replaces the elements of the receiver between the indexes startIndex and
 stopIndex with the elements of aCollection.  Returns the receiver."

| i aSize |

((aSize := aCollection size) == (stopIndex - startIndex + 1))
  ifFalse: [ ^ self errorDifferentSizeCollections].

(self == aCollection) ifTrue: [ ^ self ].

(aCollection isKindOf: SequenceableCollection) ifTrue:[
  self replaceFrom: startIndex to: startIndex + aSize - 1 with: aCollection startingAt: 1 .
  ^ self
].
startIndex > (self size + 1) ifTrue:[ self _errorIndexOutOfRange: startIndex ].
i := startIndex.
aCollection accompaniedBy: self do: [:me :element |
  me at: i put: element.
  i := i + 1.
].
^ self.
%

category: 'Updating'
method: SequenceableCollection
replaceFrom: startIndex to: stopIndex with: aCollection startingAt: repIndex

"Replaces the elements of the receiver between the indexes startIndex and
 stopIndex inclusive with the elements of aCollection starting at repIndex.
 If aCollection is identical to the receiver, the source and
 destination blocks may overlap.

 aCollection must be a kind of SequenceableCollection or a kind of IdentityBag.

 Returns the receiver."

| srcIdx srcEnd mySize |
startIndex > ((mySize := self size) + 1) ifTrue:[ self _errorIndexOutOfRange: startIndex ].
(aCollection isKindOf: SequenceableCollection) ifFalse:[
  aCollection _validateClasses: { SequenceableCollection . IdentityBag }.
].
srcIdx := repIndex .
srcEnd := srcIdx + stopIndex - startIndex .
aCollection == self ifTrue:[
  srcIdx == startIndex ifTrue:[ ^ self ].
  srcIdx < startIndex ifTrue:[ | k |  "must copy backwards"
    k := srcEnd .
    mySize < stopIndex ifTrue:[ self size: stopIndex ].
    stopIndex _downTo: startIndex do:[:j |
       self at: j put: (aCollection at:  k).
       k := k - 1 .
    ].
    ^ self
  ]
].
srcEnd > aCollection size ifTrue:[ aCollection _errorIndexOutOfRange: srcEnd].
startIndex to: stopIndex do:[:j |
  self at: j put: (aCollection at: srcIdx ) .
  srcIdx := srcIdx + 1
].
^ self
%

category: 'Updating'
method: SequenceableCollection
replaceFrom: startIndex to: stopIndex withObject: anObject

"Replaces the elements of the receiver between the indexes startIndex and
 stopIndex with anObject. Returns the receiver."

startIndex to: stopIndex do: [ :i| self at: i put: anObject].
^ self.
%

category: 'Copying'
method: SequenceableCollection
reverse

"Returns a copy of the receiver with its elements in reverse order."
| copy sz high mid |
sz := self size.
copy := self species new: sz.
copy size < sz ifTrue:[ copy size: sz "handle OrderedCollection" ].
high := sz .
mid := sz bitShift: -1 . " // 2 "
1 to: mid do:[:low |
  copy at: low put: (self at: high) .
  copy at: high put: (self at: low) .
  high := high - 1 .
] .
mid < high ifTrue:[ "middle element of odd sized Array"
  copy at: high put: (self at: high)
].
^copy
%

category: 'Enumerating'
method: SequenceableCollection
reverseDo: aBlock

"Evaluates the one-argument block aBlock using each element of the receiver,
 in reverse order, as the argument."

self size _downTo: 1 do: [ :index |
  aBlock value: (self at: index).
  ].

^ self
%

category: 'Accessing'
method: SequenceableCollection
second
	"Compatible with Squeak/Pharo"
^ self at: 2
%

category: 'Enumerating'
method: SequenceableCollection
select: aBlock
  "Evaluates aBlock with each of the receiver's elements as the argument.
 Stores the values for which aBlock is true into a collection of the same
 class as the receiver, and returns the new collection.  The argument aBlock
 must be a one-argument block.

 The result preserves the ordering of the receiver.  That is, if element a
 comes before element b in the receiver, then element a is guaranteed to come
 before b in the result."

  | result |
  aBlock _isExecBlock ifFalse:[
   aBlock class == SelectBlock
    ifTrue: [
      "send value: to the block to create the 4 element
        associative access Array; the argument nil is ignored to
        create the Array, but all SelectBlocks have one argument"
      result := (QueryExecuter on: self)
        _boundQuerySelect: (aBlock queryBlock value: nil).
      ^ result ]].
  result := self species new.
  1 to: self size do: [ :i |  | each |
    each := self at: i.
    (aBlock value: each)
      ifTrue: [ result add: each ] ].
  ^ result
%

category: 'Private'
method: SequenceableCollection
verifyAreFalse

"Returns true if all elements of the receiver are false.  Otherwise, returns
 self."

1 to: self size do: [:j |
  false == (self at:j) ifFalse: [ ^ self ]
].
^ true
%

category: 'Private'
method: SequenceableCollection
verifyAreTrue

"Returns true if all elements of the receiver are true.  Otherwise, returns
 self."

1 to: self size do: [:j |
  true == (self at:j) ifFalse: [ ^ self ]
].
^ true
%

category: 'Private'
method: SequenceableCollection
verifyElementsIn: aSeqColl

"Check for equality in corresponding elements.  Returns true if the receiver
 and the argument are equal. Otherwise returns self."

(self size == aSeqColl size)
   ifFalse: [ ^ self ].
1 to: self size do: [:i |
   (self at: i) = (aSeqColl at: i)
      ifFalse: [ ^ self ]].
^ true
%

category: 'Enumerating'
method: SequenceableCollection
with: aSequenceableCollection do: aBlock

"Evaluate a two argument block aBlock using each element of the receiver as the
 first argument and the corresponding element of aSequenceableCollection as the
 second argument.  Returns the receiver."

(self size == aSequenceableCollection size)
  ifFalse: [ ^ self errorDifferentSizeCollections ].

1 to: self size do: [ :i|
  aBlock value: (self at: i) value: (aSequenceableCollection at: i).
  ].

^ self.
%

category: 'Private'
method: SequenceableCollection
_asCollectionForSorting

""

^ self
%

category: 'Private'
method: SequenceableCollection
_removeAll: aCollection ifAbsent: errorBlock

"errorBlock must be non-nil "

  aCollection do: [:obj | | idx |
    idx := self indexOf: obj.
    idx == 0 ifTrue: [
      errorBlock value: obj .
    ].
    [ idx ~~ 0 ] whileTrue: [
      self removeFrom: idx to: idx.
      idx := self indexOf: obj startingAt: idx
    ].
  ].
%

! Class extensions for 'SignalBufferFull'

!		Instance methods for 'SignalBufferFull'

removeallmethods SignalBufferFull
removeallclassmethods SignalBufferFull

category: 'Instance initialization'
method: SignalBufferFull
initialize
  gsNumber := ERR_SignalBufferFull.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'SmallFraction'

!		Class methods for 'SmallFraction'

removeallmethods SmallFraction
removeallclassmethods SmallFraction

category: 'Storing and Loading'
classmethod: SmallFraction
loadFrom: passiveObj
  | n d inst |
  passiveObj readNamedIV.
  n := passiveObj ivValue.
  n  _isSmallInteger ifFalse:[ Error signal:'unexpected ', n class name ].
  passiveObj readNamedIV.
  d := passiveObj ivValue.
  d  _isSmallInteger ifFalse:[ Error signal:'unexpected ', d class name ].
  passiveObj skipNamedInstVars.

  inst := self numerator: n denominator: d .
  inst class == SmallFraction ifFalse:[ Error signal:'unexpected ', inst class name ].
  ^ inst
%

!		Instance methods for 'SmallFraction'

category: 'Storing and Loading'
method: SmallFraction
containsIdentity
  ^ true
%

category: 'Accessing'
method: SmallFraction
denominator
<primitive: 1000>

^ self _primitiveFailed: #denominator
%

category: 'Testing'
method: SmallFraction
isSpecial
  ^ true
%

category: 'Accessing'
method: SmallFraction
numerator
<primitive: 999>

^ self _primitiveFailed: #numerator
%

category: 'Storing and Loading'
method: SmallFraction
writeTo: passiveObj
  "reimplement basicWriteTo: to synthesize output of numerator and denominator"
  passiveObj writeClass: self class ;
     writeObject: self numerator named: #numerator ;
     writeObject: self denominator named: #denominator ;
     endNamedInstVars ;
     cr .
%

! Class extensions for 'SmallInteger'

!		Class methods for 'SmallInteger'

removeallmethods SmallInteger
removeallclassmethods SmallInteger

category: 'Queries'
classmethod: SmallInteger
maximum32bitInteger
  "Returns the maximum value representable by a 32bit signed integer"
  ^ 16r7FFFFFFF
%

category: 'Queries'
classmethod: SmallInteger
maximumValue

"Returns the maximum allowable SmallInteger value."

^ 1152921504606846975
%

category: 'Queries'
classmethod: SmallInteger
minimum32bitInteger
  "Returns the minimum value representable by a 32bit signed integer"
  ^ -16r80000000
%

category: 'Queries'
classmethod: SmallInteger
minimumValue

"Returns the minimum allowable SmallInteger value."

^ -1152921504606846976
%

!		Instance methods for 'SmallInteger'

category: 'Arithmetic'
method: SmallInteger
/ aNumber

"Returns the result of dividing the receiver by aNumber."

<primitive: 10>
(aNumber _isInteger) ifTrue:[
  (aNumber == 0) ifTrue: [^ self _errorDivideByZero].
  ^ Fraction numerator: self denominator: aNumber
].
^ super / aNumber
%

category: 'Arithmetic'
method: SmallInteger
// aNumber

"Divides the receiver by aNumber.
 Returns the integer quotient, with truncation toward negative infinity.
 For example,

 9//4 = 2
 -9//4 = -3

 The selector \\ returns the remainder from this division."

<primitive: 12>
(aNumber == 0) ifTrue: [^ self _errorDivideByZero].
^ super // aNumber
%

category: 'Comparing'
method: SmallInteger
< aNumber

"< is a special selector in SmallInteger, this implementation is used only
 by perform:  .
 Returns true if the receiver is less than aNumber;
 returns false otherwise.

 Should not be reimplemented.  The native code generator generates
 in-line implementations of SmallInteger >> < . Reimplementations
 with different semantics than primitive 3 will not take effect
 unless native code is disabled."

<primitive: 3>
aNumber _isInteger ifFalse:[
  aNumber _isFloat ifTrue:[
    ^ self asFloat < aNumber
  ].
  ^ self _retry: #< coercing: aNumber
].
^ super < aNumber
%

category: 'Comparing'
method: SmallInteger
<= aNumber

"<= is a special selector in SmallInteger, this implementation is used only
 by perform:  .
 Returns true if the receiver is less than or equal to
 aNumber; returns false otherwise.

 Should not be reimplemented.  The native code generator generates
 in-line implementations of SmallInteger >> <= . Reimplementations
 with different semantics than primitive 5 will not take effect
 unless native code is disabled."

<primitive: 5>
aNumber _isInteger ifFalse:[
  aNumber _isFloat ifTrue:[
    ^ self asFloat <= aNumber
  ].
  ^ self _retry: #<= coercing: aNumber
].
^ super <= aNumber
%

category: 'Comparing'
method: SmallInteger
= aNumber

"Returns true if the receiver is equal to
 aNumber; returns false otherwise.

 Should not be reimplemented.  The native code generator generates
 in-line implementations of SmallInteger>>= . Reimplementations
 with different semantics than primitive 7 will not take effect
 unless native code is disabled."

<primitive: 7>
aNumber _isNumber ifTrue:[
  aNumber _isFloat ifTrue:[ ^ self asFloat = aNumber].
  aNumber _isInteger ifTrue:[ ^ super = aNumber ].
  ^ self _retry: #= coercing: aNumber
].
^ false
%

category: 'Comparing'
method: SmallInteger
> aNumber

"Returns true if the receiver is greater than aNumber;
 returns false otherwise.

 Should not be reimplemented.  The native code generator generates
 in-line implementations of SmallInteger >> > . Reimplementations
 with different semantics than primitive 4 will not take effect
 unless native code is disabled."

<primitive: 4>
aNumber _isInteger ifFalse:[
  aNumber _isFloat ifTrue:[
    ^ self asFloat > aNumber
  ].
  ^ self _retry: #> coercing: aNumber
].
^ super > aNumber
%

category: 'Comparing'
method: SmallInteger
>= aNumber

">= is a special selector in SmallInteger, this implementation is used only
 by perform:  .
 Returns true if the receiver is greater than or equal to
 aNumber; returns false otherwise.

 Should not be reimplemented.  The native code generator generates
 in-line implementations of SmallInteger >> >= . Reimplementations
 with different semantics than primitive 6 will not take effect
 unless native code is disabled."

<primitive: 6>
aNumber _isInteger ifFalse:[
  aNumber _isFloat ifTrue:[
    ^ self asFloat >= aNumber
  ].
  ^ self _retry: #>= coercing: aNumber
].
^ super >= aNumber
%

category: 'Arithmetic'
method: SmallInteger
abs

"Returns a Number that is the absolute value of the receiver."

self < 0 ifTrue:[ ^ 0 - self ].
^ self
%

category: 'Bit Manipulation'
method: SmallInteger
asBitString

"Returns a string of 1 and 0 Characters representing the bits in the receiver."

| bit num bits str |

bits := { }.
bit := 1.
num := self.
[
  bits add: (num bitAnd: bit).
  num := num bitAnd: bit bitInvert.
  bit := bit bitShift: 1.
  num = 0
] untilTrue.

str := String new.
bits size _downTo: 1 do: [ :i |
  str add: ((bits at: i) > 0 ifTrue: [$1] ifFalse: [$0])
].

^str.
%

category: 'Converting'
method: SmallInteger
asDecimalFloat

"Returns a DecimalFloat representing the receiver."

<primitive: 123>

self _primitiveFailed: #asDecimalFloat .
self _uncontinuableError
%

category: 'Converting'
method: SmallInteger
asString

"Returns a String that indicates the numeric value of the
 receiver.  Positive values do not include a leading + ."

<primitive: 54>
self _primitiveFailed: #asString .
self _uncontinuableError
%

category: 'Unicode'
method: SmallInteger
asUnicodeDigitRadix: radixSmallInt
  "Returns a Character representing the receiver as a digit
   in radix radixSmallInt, or signals an Error .
   Result is computed by calling u_forDigit(self, radix) in libicu .
  "

<primitive: 967>
radixSmallInt _validateClass: SmallInteger .
(radixSmallInt < 0 or:[ radixSmallInt > 16rFF]) ifTrue:[
   OutOfRange new
     name: 'radixSmallInt' min: 0 max: 16rFF actual: radixSmallInt ;
     signal
].
ArgumentError signal:'no valid digit translation'.
 ^ self _primitiveFailed: #asUnicodeDigitRadix: args: { radixSmallInt }
%

category: 'Bit Manipulation'
method: SmallInteger
bitAnd: aNumber

"SmallInteger >> bitAnd: is a special send if both receiver and arg
 are SmallIntegers .
 Returns an Integer whose bits are the logical and of the receiver's bits and
 the bits of aNumber."

<primitive: 14>
^aNumber bitAnd: self
%

category: 'Bit Manipulation'
method: SmallInteger
bitAt: bitNumber

"Returns the bit at the ith position of the receiver, where 1 is the least
 significant bit.

 Gs64 v3.0 changed argument from 0 based to 1 based. "

| num |
bitNumber _isSmallInteger ifFalse:[ bitNumber _validateClass: SmallInteger ].
(bitNumber < 1) ifTrue: [^ bitNumber _error: #rtErrArgNotPositive].
num := bitNumber .
(bitNumber > 61) ifTrue:[  num := 61 "sign extension semantics"].
^ (self bitShift: (1 - num)) bitAnd: 1 .
%

category: 'Bit Manipulation'
method: SmallInteger
bitOr: aNumber

"Returns an Integer whose bits are the logical or of the receiver's bits and
 the bits of aNumber."

<primitive: 15>
^aNumber bitOr: self
%

category: 'Bit Manipulation'
method: SmallInteger
bitShift: shift

"Returns an Integer whose value (in two's-complement representation) is the
 receiver's value (also in two's-complement representation) shifted left by
 'shift' bits.

 Negative arguments shift right.  Zeros are shifted in from the right in left
 shifts.  The sign bit is extended in right shifts."

<primitive: 17>
shift _validateClass: SmallInteger.
^ super bitShift: shift
%

category: 'Bit Manipulation'
method: SmallInteger
bitXor: aNumber

"SmallInteger >> bitXor: is a special send if both receiver and arg
 are SmallIntegers .
 Returns an Integer whose bits are the logical xor of the receiver's bits and
 the bits of aNumber."

<primitive: 16>
^aNumber bitXor: self
%

category: 'Clustering'
method: SmallInteger
clusterDepthFirst

"Returns true.  (Because SmallIntegers are self-defining objects,
 this method has no effect.)"

^ true
%

category: 'Storing and Loading'
method: SmallInteger
containsIdentity

"Private."

^true
%

category: 'Copying'
method: SmallInteger
copy

"Overrides the inherited method; you cannot create any new SmallIntegers.
 Returns the receiver."

^ self
%

category: 'Arithmetic'
method: SmallInteger
divMod: aNumber into: anArray

"anArray should be an Array of size 2.
 Performs
   anArray at: 1 put: (self // aNumber) .
   anArray at: 2 put: (self \\ aNumber) .
 Returns anArray .
"
<primitive: 951>
(aNumber == 0) ifTrue: [^ self _errorDivideByZero].
anArray _isArray ifFalse:[ anArray _validateClass: Array ].
^ self _retry: #divMod:into: coercing: aNumber into: anArray
%

category: 'Bit Manipulation'
method: SmallInteger
highBit

"Returns the index of the high-order bit that is set
 in the binary representation of the receiver.  (If the receiver is a negative
 integer, takes its absolute value first.)  If the receiver is zero, this
 returns nil.
 As of Gs64 v3.2  the least significant bit has index 1 ,
 in previous versions the least significant bit had index 0 .

 Example:   4 highBit == 3  "

<primitive: 853>
self _primitiveFailed: #highBit
%

category: 'Comparing'
method: SmallInteger
identityHash

"Returns a numeric hash index.  For a SmallInteger, returns the receiver."

^self
%

category: 'Testing'
method: SmallInteger
isSpecial

"Returns true if the receiver is a special object."

^ true
%

category: 'Formatting'
method: SmallInteger
printString

"Returns a String whose contents are a displayable representation of the
 receiver."

"Reimplemented for efficiency.  GemStone does not allow the creation of new
 kinds of Number, so there is no point in creating a stream and sending
 printOn:"

^self asString
%

category: 'Arithmetic'
method: SmallInteger
quo: aNumber

"Divides the receiver by aNumber.  Returns the integer quotient, with
 truncation toward zero.  For example,

 -9 quo: 4 = -2

 The selector rem: returns the remainder from this division."

<primitive: 13>
(aNumber == 0)
  ifTrue: [^ self _errorDivideByZero]
  ifFalse: [^ super quo: aNumber]
%

category: 'Arithmetic'
method: SmallInteger
quoRem: aNumber into: anArray

"anArray should be an Array of size 2.
 Performs
   anArray at: 1 put: (self quo: aNumber) .
   anArray at: 2 put: (self rem: aNumber) .
 Returns anArray .
"
<primitive: 950>
(aNumber == 0) ifTrue: [^ self _errorDivideByZero].
anArray _isArray ifFalse:[ anArray _validateClass: Array ].
^ self _retry: #quoRem:into: coercing: aNumber into: anArray
%

category: 'Arithmetic'
method: SmallInteger
raisedTo: aNumber

"Returns the receiver raised to the power of the argument."
<primitive: 662>
FloatingPointError _checkFpStatus .

"primitive handles SmallInteger and Float args. "

aNumber _isInteger ifTrue:[
  ^ super raisedToInteger: aNumber
].
^ super raisedTo: aNumber
%

category: 'Arithmetic'
method: SmallInteger
raisedToInteger: aNumber

aNumber _isInteger ifFalse:[ aNumber _validateClass: Integer ].
^ self raisedTo: aNumber
%

category: 'Arithmetic'
method: SmallInteger
rem: aNumber

"Returns the integer remainder defined in terms of quo: (division
 of the receiver by aNumber, with truncation toward zero)."

<primitive: 188>
(aNumber == 0)
  ifTrue: [^ self _errorDivideByZero]
  ifFalse: [^ super rem: aNumber]
%

category: 'Arithmetic'
method: SmallInteger
\\ aNumber

"Returns the modulus defined in terms of //.  Returns a Number with the same
 sign as the argument aNumber.  For example,

 9\\4 = 1
 -9\\4 = 3
 9\\-4 = -3"

<primitive: 11>
(aNumber == 0)
  ifTrue: [^ self _errorDivideByZero]
  ifFalse: [^ super \\ aNumber]
%

category: 'Testing'
method: SmallInteger
_bottomBit
  "used by raisedTo methods"
  ^ self bitAnd: 1
%

category: 'Converting'
method: SmallInteger
_coerce: aNumber

"Reimplemented from Integer."

^aNumber truncated
%

category: 'Converting'
method: SmallInteger
_digitsAsString

"Returns self as a String of at least two digits."

| aString |

(self < 10 and:[ self >= 0])
  ifTrue: [aString := '0' , self asString]
  ifFalse: [aString := self asString].
^ aString
%

category: 'Converting'
method: SmallInteger
_digitsAsString3

  "Returns self as a String of at least three digits."
  (self < 100 and:[ self >= 0 ]) ifTrue:[
    (self < 10) ifTrue: [
      ^ '00' , self asString
    ] ifFalse:[
      ^ '0', self asString
    ].
  ] ifFalse:[
    ^ self asString
  ]
%

category: 'Converting'
method: SmallInteger
_generality

"Returns the generality of a SmallInteger"

^20
%

category: 'Private'
method: SmallInteger
_leaveProtectedMode
  "(Reserved selector) a special selector compiled to a bytecode.
   This method is for documentation only and should not be used.
   See System(C) >> _protectedMode for more information."

 ^ self _primitiveFailed:#_leaveProtectedMode
%

category: 'Private'
method: SmallInteger
_validateMin: aMin max: aMax
  (self < aMin or:[ self > aMax]) ifTrue:[
    self _error: #rtErrArgOutOfRange args:{ aMin . aMax }
  ]
%

category: 'Comparing'
method: SmallInteger
~= aNumber

"Returns true if the receiver is not equal to aNumber;
 returns false otherwise.

 Should not be reimplemented.  The native code generator generates
 in-line implementations of SmallInteger>>~= . Reimplementations
 with different semantics than primitive 8 will not take effect
 unless native code is disabled."

<primitive: 8>
aNumber _isNumber ifTrue:[
  aNumber _isFloat ifTrue:[ ^ self asFloat ~= aNumber ].
  aNumber _isInteger ifTrue:[ ^ super ~= aNumber ].
   ^ self _retry: #~= coercing: aNumber
].
^ true
%

! Class extensions for 'SocketError'

!		Instance methods for 'SocketError'

removeallmethods SocketError
removeallclassmethods SocketError

category: 'Formatting'
method: SocketError
buildMessageText
^ self buildMessageText:(
  [ | str |
    "get the details from error 2362 thrown from a primitive"
    str := 'socket error ' copy .
    1 to: gsArgs size  do:[:n | | elem |
      elem := gsArgs atOrNil: n .
      (elem _isSymbol or:[ elem _isOneByteString]) ifTrue:[ str add: elem ]
         ifFalse:[  elem _isSmallInteger ifTrue:[ str add: elem asString ]
            ifFalse:[ str add: 'a ' ; add: elem class name ]].
      n < gsArgs size ifTrue:[ str add: ', ' ]
    ].
    gsDetails ifNotNil:[:d| str add:', '; add: d  asString ].
    str
  ] onException: Error do:[:ex |
    ex return: nil
  ]
)
%

category: 'Instance initialization'
method: SocketError
initialize
  gsNumber := ERR_SocketError.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'SortedCollection'

!		Class methods for 'SortedCollection'

removeallmethods SortedCollection
removeallclassmethods SortedCollection

category: 'Repository Conversion'
classmethod: SortedCollection
buildFileNameForGem: anInt

| result |
result := String withAll: 'failedSortedCollections_' .
result addAll: anInt asString;
      addAll: '.bm' .
^ result
%

category: 'Disallowed'
classmethod: SortedCollection
byteSubclass: aString
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
inClassHistory: aClassHistory
description: aDescription
isInvariant: invarBoolean

"Disallowed for SortedCollection and its subclasses."
^ self _error: #classErrByteSubclass
%

category: 'Subclass Creation'
classmethod: SortedCollection
byteSubclass: aString
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
newVersionOf: oldClass
description: aDescription
options: options

"Disallowed for SortedCollection and its subclasses."
^ self _error: #classErrByteSubclass
%

category: 'Repository Conversion'
classmethod: SortedCollection
findAllSortedCollectionsForUser: aUserId

| co allClasses dir |
co := ClassOrganizer newWithRoot: self forUserId: aUserId .
allClasses := co allSubclassesOf: self .
allClasses add: self .
dir := self getSortedCollDir.
(GsFile exists: dir)
  ifTrue:[ GsFile removeServerDirectory: dir ].
GsFile createServerDirectory: dir.
^ SystemRepository fastListInstances: allClasses toDirectory: dir
%

category: 'Repository Conversion'
classmethod: SortedCollection
getSortedCollDir
| dir |
dir := self getUpgradeLogDir.
dir addAll: '/scbm'.
^ dir
%

category: 'Repository Conversion'
classmethod: SortedCollection
getUpgradeLogDir

| uld |
uld := System gemEnvironmentVariable: 'upgradeLogDir' .
uld ifNil:[ LookupError signal: 'Could not resolve the $upgradLogDir environment variable' ] .
((uld at: uld size) == $/)
  ifFalse:[ uld add: $/ ].
^ uld
%

category: 'Class initialization'
classmethod: SortedCollection
initDefaultSortblock

"Initialize a reference to the default sort block."

  SortedCollection _classVars at: #DefaultSortBlock put:
    SortedCollection new sortBlock
%

category: 'Repository Conversion'
classmethod: SortedCollection
initializeForConversion

System globalSessionStatAt: 0 put: 0 .
System globalSessionStatAt: 1 put: 0 .
System globalSessionStatAt: 2 put: 0 .
^ true
%

category: 'Instance Creation'
classmethod: SortedCollection
new

"Returns a new instance of the receiver with the sort block
 [ :a :b | a <= b ]."

| inst |
inst := self _basicNew .
inst sortBlock: inst _defaultBlock .
^inst
%

category: 'Instance Creation'
classmethod: SortedCollection
new: size

"Returns a new instance of the receiver with size 0 and the sort block
 [ :a :b | a <= b ].  This method is synonymous with new and is provided for
 compatibility with other Smalltalk dialects that do not have objects that are
 truly variable in size."

^ self new
%

category: 'Repository Conversion'
classmethod: SortedCollection
resortAll: anArray
  "Resort an Array of SortedCollections by sending resort to the collecion. anArray must contain
   only SortedCollections.

   Caller must handle and catch AlmostOutOfMemory errors and handle commits
 and aborts."

  | persistentRoot sys ug commitBlock |
  persistentRoot := SymbolAssociation new
    key: #'SortedCollection_Resort_Persistent_Root'
    value: nil.
  sys := System.
  ug := GsSession currentSession symbolList objectNamed: #'UserGlobals'.
  ug addAssociation: persistentRoot.
  commitBlock := [
    sys commitTransaction
      ifFalse: [
        sys abortTransaction.
        TransactionError
          signal: 'Unexpected commit failure in post conversion'  ]].
  [
  1 to: anArray size do: [ :n |
    | sc |
    sc := anArray at: n.
    sc size > 1
      ifTrue: [
        | saveArray |
        saveArray := Array new.
        persistentRoot value: saveArray.	"root saveArray in UserGlobals to allow larger than TOC collections"
        commitBlock value.
        saveArray addAll: sc.
        commitBlock value.
        sc _basicSize: 0.
        sc addAll: saveArray.
        saveArray _basicSize: 0.
        commitBlock value ] ] ]
    ensure: [
      ug removeAssociation: persistentRoot.
      commitBlock value ]
%

category: 'Instance Creation'
classmethod: SortedCollection
sortBlock: aBlock

"Returns a new instance of the receiver with the given sort block."

| inst |
inst := super new.
inst sortBlock: aBlock.
^inst
%

category: 'Instance Creation'
classmethod: SortedCollection
sortBlock: aBlock fromSortResult: sortArray

"Returns a new instance of the receiver with the given sort block and contents.
 The argument sortArray is assumed to be in the proper sort order and is
 installed as the presorted contents of the new instance."

| inst |
inst := super new.
inst sortBlock: aBlock.
1 to: sortArray size do: [:i |
  inst _basicAt: i put: (sortArray at: i)
].
^inst
%

category: 'Repository Conversion'
classmethod: SortedCollection
updateDbfHistoryForConversion

| curHist verString message ok failed |

ok := (System globalSessionStatAt: 0) asString .
failed := (System globalSessionStatAt: 1) asString .

verString := System gemVersionAt: #gsRelease .
curHist := (Globals at: #DbfHistory ifAbsent: ['']).
message := curHist, 'GemStone ', verString,
                    ' post conversion upgrade completed at ' ,
                     DateTime now asString ,
                    '.  SortedCollection conversion results: ', ok,
                    ' OK, ', failed, ' failed.'. Character lf .
Globals at: #DbfHistory put: message .
System addAllToStoneLog: message .
^ message
%

category: 'Instance Creation'
classmethod: SortedCollection
with: aValue

"Returns an instance of the receiver containing the argument."

| inst |
inst := self new .
inst add: aValue .
^ inst
%

category: 'Instance Creation'
classmethod: SortedCollection
with: aValue with: val2

"Returns an instance of the receiver containing the arguments."

| inst |
inst := self new .
inst add: aValue ; add: val2 .
^ inst
%

category: 'Instance Creation'
classmethod: SortedCollection
with: aValue with: val2 with: val3

"Returns an instance of the receiver containing the arguments."

| inst |
inst := self new .
inst add: aValue ; add: val2 ; add: val3 .
^ inst
%

category: 'Instance Creation'
classmethod: SortedCollection
with: aValue with: val2 with: val3 with: val4
	"Returns an instance of the receiver containing the arguments."

	| inst |
	inst := self new .
	inst add: aValue ; add: val2 ; add: val3; add: val4 .
	^inst.
%

category: 'Instance Creation'
classmethod: SortedCollection
withAll: aCollection

"Returns an instance of the receiver containing the elements of the argument."

| inst |
inst := self new .
inst addAll: aCollection.
^ inst
%

category: 'Instance Creation'
classmethod: SortedCollection
withAll: collection sortBlock: block

"Returns a new instance of the receiver with the given sort block and
 contents."

| instance |
instance := self sortBlock: block.
instance addAll: collection.
^instance
%

category: 'Repository Conversion'
classmethod: SortedCollection
writeFailedConversionObjsToFileForGem: gemNum bmForErrors: errBm

| fileName numOops |

 numOops := errBm size.
 numOops == 0
  ifTrue:[ ^ true ]. "Nothing to do"

 fileName := self getUpgradeLogDir .
 fileName addAll: (self buildFileNameForGem: gemNum) .

"Delete an old version, if any"
(GsFile existsOnServer: fileName)
  ifTrue:[ GsFile removeServerFile: fileName ].

errBm writeToFile: fileName.
^ true
%

category: 'Repository Conversion'
classmethod: SortedCollection
writeTotalsFile: aString counterIndex: idx

| fn count gsf |
fn := self getUpgradeLogDir.
fn addAll: aString .
gsf := GsFile openWriteOnServer: fn .
gsf ifNil:[ IOError signal: ('Error opening file ', fn) ].
count := System globalSessionStatAt: idx .
gsf nextPutAll: count asString .
gsf close .
^ count
%

category: 'Repository Conversion'
classmethod: SortedCollection
writeTotalsFiles

self writeTotalsFile: 'sc_ok.txt' counterIndex: 0 .
self writeTotalsFile: 'sc_failed.txt' counterIndex: 1 .
self writeTotalsFile: 'sc_done.txt' counterIndex: 2 .
^ true
%

category: 'Indexing Support'
classmethod: SortedCollection
_canCreateQueryOnInstances
  "Cannot create a GsQuery on the receiver"

  ^ false
%

category: 'Filein Support'
classmethod: SortedCollection
_newKernelByteSubclass: clsName
classVars: classVarArg
poolDictionaries: poolDicts
inDictionary: aDict
options: options
reservedOop: aReservedOop

"Disallowed for SortedCollection and its subclasses."

^ self _error: #classErrByteSubclass
%

!		Instance methods for 'SortedCollection'

category: 'Concatenating'
method: SortedCollection
, aSequenceableCollection
	"Returns a new instance of the receiver's class that contains the elements of
	 the receiver followed by the elements of aSequenceableCollection.

	 Warning: Creating a new instance and copying the receiver take time.  If you
	 can safely modify the receiver, it can be much faster to use the addAll:
	 method.  See the documentation of the Concatenating category for more details."

	^(self asArray, aSequenceableCollection) asSortedCollection: self sortBlock.
%

category: 'Adding'
method: SortedCollection
add: anObject

"Adds anObject to the receiver.  Increases the size of the receiver by one.
 Enforces the sorting order.  Returns anObject."

| idx |
idx := self _findIndex: anObject.
self _insertObject: anObject at: idx .
^ anObject
%

category: 'Adding'
method: SortedCollection
addAll: aCollection

"Adds the elements of aCollection to the receiver.  Increases the size of the
 receiver by the number of elements in aCollection.  Enforces the sorting order.
 Returns aCollection."

| collCopy |
aCollection == self ifTrue:[
  collCopy := Array withAll: self .
  1 to: collCopy size do:[:j | self add: (collCopy at: j) ].
]
ifFalse:[
  (self size == 0 and:[ aCollection size > 2000]) ifTrue:[
    self _mergeSortAddAll: aCollection .
  ] ifFalse:[
    (aCollection isKindOf: SequenceableCollection) ifTrue:[
      1 to: aCollection size do:[:j |
        self add:( aCollection at: j )
      ].
    ] ifFalse:[
      aCollection accompaniedBy: self do: [:me :each | me add: each] .
    ].
  ].
].
^ aCollection
%

category: 'Adding'
method: SortedCollection
addLast: anObject

"Disallowed.  Reports an error since SortedCollections have a sorting order
 that prohibits outside interference."

self shouldNotImplement: #addLast:
%

category: 'Accessing'
method: SortedCollection
at: anIndex

"Returns the value of an indexed variable in the receiver.
 The argument anIndex must not be larger than the size of the
 receiver, and must not be less than 1.

 Generates an error if anIndex is not a SmallInteger or is out of
 bounds, or if the receiver is not indexable."

<primitive: 699>
(anIndex _isInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
^ self _primitiveFailed: #at: args: { anIndex }
%

category: 'Updating'
method: SortedCollection
at: index put: anObject

"Disallowed.  Reports an error since SortedCollections have a sortBlock that
 determines the order of their contents."

^self shouldNotImplement: #at:put:
%

category: 'Updating'
method: SortedCollection
atAllPut: anObject

"Assigns anObject to each of the receiver's elements."

1 to: self size do: [:index |
   super at: index put: anObject
].
^anObject
%

category: 'Accessing'
method: SortedCollection
atOrNil: anIndex

"Return the value   self at: anIndex ,
 or return nil if anIndex is out of range . "

<primitive: 699>
anIndex _isInteger ifTrue:[
  ^ nil
].
self _errorNonIntegerIndex: anIndex .
^ self _primitiveFailed: #atOrNil: args: { anIndex }
%

category: 'Searching'
method: SortedCollection
binarySearchIncludes: anObject

"Returns true if the argument anObject is equal to an element of the receiver.
 Returns false otherwise.

 Note that this search is optimized based on the assumption that the sort criteria
 is consistent with the equality comparison. That is, if (x <= y) & (y <= x) then (x = y).
 This assumption will not hold if sorting is done on one instance variable and
 equality comparison is done on another instance variable (for example).
 SortedCollection>>#'includes:' is no longer implemented and the linear search
 is inherited from Collection.  See bug 40575."

^ (self indexOf: anObject) ~~ 0
%

category: 'Copying'
method: SortedCollection
copyFrom: startIndex to: stopIndex

"Installs the receiver's sortBlock into the copy."

^(super copyFrom: startIndex to: stopIndex) sortBlock: sortBlock
%

category: 'Copying'
method: SortedCollection
copyReplaceAll: oldSubCollection with: newSubCollection

	^(self asArray
		copyReplaceAll: oldSubCollection with: newSubCollection)
			asSortedCollection: self sortBlock
%

category: 'Copying'
method: SortedCollection
copyReplaceFrom: startIndex to: stopIndex with: aSequenceableCollection
	"Returns a copy of the receiver in which all elements in the
	 receiver between indexes startIndex and stopIndex inclusive have
	 been replaced by those contained in aSequenceableCollection.

	 Note that a SortedCollection will sort the replacement elements
	 into the resulting collection.

	 Rules, abbreviated from the ANSI specification:
		- start < 1 or start > receiver's size + 1 (error)
		- stop < start - 1 or stop > receiver's size (error)
		- stop = start - 1 and start <= receiver's size (insert between stop and start)
		- stop = start - 1 and start = receiver's size + 1 (append at the end of the new collection)
		- otherwise, replace from start through stop with the elements from replacementElements.
	"

	^(self asArray
		copyReplaceFrom: startIndex
		to: stopIndex
		with: aSequenceableCollection)
			asSortedCollection: self sortBlock.
%

category: 'Copying'
method: SortedCollection
copyReplacing: oldObject withObject: newObject
	"Returns a copy of the receiver in which all occurrences of objects equal to
	 oldObject have been replaced by newObject."

	^(self asArray copyReplacing: oldObject withObject: newObject)
		asSortedCollection: self sortBlock
%

category: 'Copying'
method: SortedCollection
copyWith: anObject
	"Returns a copy of the receiver with the given object included in the correct sort order."
	| res |
	(res := self copy) add: anObject .
	^ res
%

category: 'Copying'
method: SortedCollection
copyWithout: anObject

"Returns a copy of the receiver that does not contain the given object.
 Comparisons are by equality (not identity)."

| copy element sz |

copy := self species sortBlock: sortBlock.
sz := 0.
1 to: self size do: [:i |
  element := self at: i.
  element = anObject ifFalse: [
    sz := sz + 1.
    copy _at: sz put: element
  ]
].
		"no need to resort "
^ copy
%

category: 'Searching'
method: SortedCollection
indexOf: anObject

"Returns the index of the first occurrence of an object equal to anObject
 in the receiver.  If the receiver does not contain such an object, this
 returns zero."

| idx |
" _findIndex: returns where anObject would be inserted, so look at
surrounding slots to see if the object is present "
idx := self _findIndex: anObject.

(sortBlock value: anObject value: anObject) ifTrue: [
  "Need to look at the slots less than idx"
  | foundIdx | "We need to find the first one that is equal"
  foundIdx := 0.
  (idx-1) _downTo: 1 do: [ :i |
    (anObject = (self at: i)) ifTrue: [foundIdx := i].
    (i > 1) ifTrue: [
      "Check to see if the next element might be equal to the current element
       according to the sort block."
      (sortBlock value: (self at: i) value: (self at: i-1)) ifFalse: [
        "We don't need to look at any more because i-1 can't be equal
         to i."
        ^foundIdx
      ].
    ].
  ].
  ^foundIdx
] ifFalse: [
  "Need to look at the slots greater than idx"
  | lastIdx |
  lastIdx := self size.
  idx to: lastIdx do: [ :i |
    (anObject = (self at: i)) ifTrue: [^i].
    (i < lastIdx) ifTrue: [
      "Check to see if the next element might be equal to the current element
       according to the sort block."
      (sortBlock value: (self at: i) value: (self at: i+1)) ifTrue: [
        "We don't need to look at any more because i+1 can't be equal
         to i."
        ^0
      ].
    ].
  ].
  ^0.
].
%

category: 'Searching'
method: SortedCollection
indexOfIdentical: anObject

"Private.  Returns the index of the first element in the receiver that is
 identical to the argument.  If the receiver does not have any elements that are
 identical to the argument, returns zero."

| idx |

(self size <= 2000) ifTrue: [
  "OrderedCollection's indexOfIdentical: uses a primitive and is
   faster for smaller SortedCollections"
  ^super indexOfIdentical: anObject.
].

" _findIndex: returns where anObject would be inserted, so look at
surrounding slots to see if the object is present "
idx := self _findIndex: anObject.

(sortBlock value: anObject value: anObject) ifTrue: [
  "Need to look at the slots less than idx"
  | foundIdx | "We need to find the first one that is equal"
  foundIdx := 0.
  (idx-1) _downTo: 1 do: [ :i |
    (anObject == (self at: i)) ifTrue: [foundIdx := i].
    (i > 1) ifTrue: [
      "Check to see if the next element might be equal to the current element
       according to the sort block."
      (sortBlock value: (self at: i) value: (self at: i-1)) ifFalse: [
        "We don't need to look at any more because i-1 can't be equal
         to i."
        ^foundIdx
      ].
    ].
  ].
  ^foundIdx
] ifFalse: [
  "Need to look at the slots greater than or equal to idx"
  | lastIdx |
  lastIdx := self size.
  idx to: lastIdx do: [ :i |
    (anObject == (self at: i)) ifTrue: [
      ^i
    ].
    (i < lastIdx) ifTrue: [
      "Check to see if the next element might be equal to the current element
       according to the sort block."
      (sortBlock value: (self at: i) value: (self at: i+1)) ifTrue: [
        "We don't need to look at any more because i+1 can't be equal
         to i."
        ^0
      ].
    ].
  ].
  ^0.
].
%

category: 'Searching'
method: SortedCollection
indexOfValue: anObject

self deprecated: 'SortedCollection>>indexOfValue: deprecated v3.2. Use indexOf: instead.'.
^ self indexOf: anObject
%

category: 'Copying'
method: SortedCollection
insertAll: aCollection at: anIndex

"Disallowed.  Reports an error since SortedCollections have a sorting order
 that prohibits outside interference."

self shouldNotImplement: #insertAll:at:
%

category: 'Copying'
method: SortedCollection
insertObject: anObject at: anIndex

"Disallowed.  Reports an error since SortedCollections have a sorting order
 that prohibits outside interference."

self shouldNotImplement: #insertObject:at:
%

category: 'Storing and Loading'
method: SortedCollection
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into the receiver."

super loadFrom: passiveObj.
sortBlock ifNil:[ sortBlock := self _defaultBlock ].
%

category: 'Searching'
method: SortedCollection
reject: aBlock

"Pass on the sort block."

|result mySize each |

result := self species sortBlock: sortBlock.
mySize := self size .
1 to: mySize do:[:j|
  each := self at: j .
  (aBlock value: each) ifFalse: [result _addLast: each]
  ].
^result
%

category: 'Removing'
method: SortedCollection
removeIdentical: anObject otherwise: notFoundValue

"Removes from the receiver an object that is identical to anObject and
 returns anObject.  If several elements of the receiver are identical to
 oldObject, only one instance is removed.  If anObject is not present in
 the receiver, returns notFoundValue."

| i |
i := self indexOfIdentical: anObject.
i ~~ 0 ifTrue: [
   self removeFrom: i to: i.
   ^ anObject
] ifFalse: [
  ^ notFoundValue
]
%

category: 'Copying'
method: SortedCollection
replaceFrom: startIndex to: stopIndex with: aSeqCollection

"Disallowed."
^ self shouldNotImplement: #replaceFrom:to:with:
%

category: 'Copying'
method: SortedCollection
replaceFrom: startIndex to: stopIndex with: aSeqCollection startingAt: repIndex

"Disallowed."
^ self shouldNotImplement: #replaceFrom:to:with:startingAt:
%

category: 'Copying'
method: SortedCollection
replaceFrom: startIndex to: stopIndex withObject: anObject

"Disallowed."
^ self shouldNotImplement: #replaceFrom:to:withObject:
%

category: 'Copying'
method: SortedCollection
reverse
  "Returns an OrderedCollection which is a copy of the receiver
   with element order reversed."

  | n result src |
  n := self size.
  "GemStone disallows at: put: on a SortedCollection
    ... reversing a SortedCollection without changing the sortBlock seems odd"
  result := OrderedCollection new: n.
  src := n + 1.
  1 to: n do: [:i | result at: i put: (self at: (src := src - 1))].
  ^ result
%

category: 'Searching'
method: SortedCollection
select: aBlock

"Returns an instance of the receiver's species that has the receiver's sort
 block."

| result mySize each |

result:= self species sortBlock: sortBlock.
mySize := self size .
1 to: mySize do:[:j|
  each := self at: j .
  (aBlock value: each) ifTrue: [result _addLast: each]
  ].
^result
%

category: 'Updating'
method: SortedCollection
size: anInteger

"If anInteger is less than the current size of the receiver, shrinks
 the receiver, otherwise has no effect."

"This implementation choice allows removal methods to be safely inherited
 from SequenceableCollection."

anInteger < self size ifTrue:[ super size: anInteger ].
%

category: 'Accessing'
method: SortedCollection
sortBlock

"Returns the value of the instance variable 'sortBlock'."

^sortBlock
%

category: 'Updating'
method: SortedCollection
sortBlock: newBlock

"Installs a new sort block in the receiver and forces a resort."

sortBlock := newBlock.
self size > 1 ifTrue: [
  self resort
]
%

category: 'Enumerating'
method: SortedCollection
speciesForCollect

"Returns a class, an instance of which should be used as the result of
 collect: or other projections applied to the receiver."

^ OrderedCollection
%

category: 'Private'
method: SortedCollection
_addLast: anObject

"Used when caller knows that appends to the receiver are being done in
 sorted order."

^ super add: anObject
%

category: 'Private'
method: SortedCollection
_at: anIndex put: aValue

"Puts the given element into the receiver at the given index, without
 sorting."

^super at: anIndex put: aValue
%

category: 'Instance Creation'
method: SortedCollection
_defaultBlock
  "a single ExecBlock used for the default sort block for both #new and #loadFrom"
^ [ :a :b | a <= b ]
%

category: 'Private'
method: SortedCollection
_findIndex: anObject

"Finds and returns the index for placing the given object into the receiver.
 A simple binary probe is used."

| obj lower upper half probe |

self size == 0 ifTrue: [ ^1 ].

lower := 1.
upper := self size.
[ half := upper - lower + 1 // 2.
  probe := lower + half.
  -1 < half and: [ lower <= upper ] ] whileTrue: [
  obj := self at: probe.
  (sortBlock value: obj value: anObject) ifTrue: [
    "after this index - go to the next higher cell"
    lower := probe + 1.
  ]
  ifFalse: [
    "before this index - go to the next lower cell"
    upper := probe - 1
  ]
].
^probe
%

category: 'Private'
method: SortedCollection
_insertAll: aCollection at: anIndex

"Inserts all the elements of aCollection into the receiver beginning at index
 anIndex.  Returns aCollection."

"The argument anIndex must be greater than or equal to one.  If anIndex is one
 greater than the size of the receiver, appends aCollection to the receiver.  If
 anIndex is more than one greater than the size of the receiver, generates an
 error."

<primitive: 606 >
( self _hasModificationTracking and:
[ aCollection isKindOf: CharacterCollection ] )
  ifTrue: [ " primitive failed because aCollection is not an
             Array or OrderedCollection"
    ^ self insertAll: (Array withAll: aCollection) at: anIndex
  ].
self _primitiveFailed: #_insertAll:at: args: { aCollection . anIndex }
%

category: 'Adding'
method: SortedCollection
_insertObject: anObject at: anIndex

"Inserts anObject into the receiver at index anIndex and returns anObject."

<primitive: 398>
"primitive fails if receiver is byte format"
(anIndex _isInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
(anIndex < 1 or: [ anIndex > (self size + 1)]) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].

^ self _primitiveFailed: #_insertAll:at: args: { anObject . anIndex }
%

category: 'Copying'
method: SortedCollection
_mergeSortAddAll: aCollection

"The receiver must be empty. Adds aCollection to the receiver
 using the merge-sort implementation provided by BlockSorter.
 Returns the receiver."

| arr |
self size == 0 ifFalse:[ self error:'not empty' ].
arr := aCollection sortWithBlock: sortBlock . "merge sort"
self _insertAll: arr at: 1 .
^ self
%

category: 'Repository Conversion'
method: SortedCollection
_sortBlock: newBlock

"Installs a new sort block in the receiver"

sortBlock := newBlock.
%

! Class extensions for 'String'

!		Class methods for 'String'

removeallmethods String
removeallclassmethods String

category: 'Formatting'
classmethod: String
charSize
"Returns number of bytes that make up a character for this string class."

^ 1
%

category: 'Deprecated'
classmethod: String
fromClientTextFile: fileName

| file instance |
self deprecated: 'String class>>fromClientTextFile: deprecated long before v3.0. Use an instance of GsFile to
 access the file system.'.
file := GsFile openRead: fileName.
file == nil ifTrue: [
  self _halt: 'Unable to open file: ' , fileName.
].
instance := self new.
file next: file fileSize into: instance.
file close.
^instance
%

category: 'Storing and Loading'
classmethod: String
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into a new instance of the receiver.
 Returns the new instance."

| inst |

inst := self new: passiveObj readSize.
inst loadFrom: passiveObj.
^inst
%

category: 'Instance Creation'
classmethod: String
new

"Returns an instance of the receiver of size zero"
<primitive: 51>
self _primitiveFailed: #new .
self _uncontinuableError
%

category: 'Instance Creation'
classmethod: String
new: anInteger

"Returns an instance of the receiver with the specified size,
 with all codepoints set to zero.
 Generates an error if anInteger is not a positive SmallInteger."

<primitive: 53>
(anInteger _isInteger)
  ifFalse:[ anInteger _validateClass: Integer . ^ self new ]
  ifTrue:[
    (anInteger < 0) ifTrue:[ anInteger _error: #rtErrArgNotPositive .
                            ^ self new].
    anInteger _error: #rtErrArgOutOfRange .
    ^ self new
  ].
self _primitiveFailed: #new: args: { anInteger }.
self _uncontinuableError
%

category: 'Instance Creation'
classmethod: String
withAll: aString

"Returns an instance of the receiver, DoubleByteString, or QuadByteString
 using the minimum bytes per character required to represent the argument."

<primitive: 294>
^ super withAll: aString
%

category: 'Instance Creation'
classmethod: String
withBytes: aCollection

"Returns an instance of the receiver. aCollection must be a kind of
 String or ByteArray"

<primitive: 1021>
ArgumentTypeError new name: 'aCollection' expectedClass: { String . ByteArray . Utf8 }
		         actualArg: aCollection ; signal
%

category: 'Class Membership'
classmethod: String
_classHistoryIncludesIdentical: aClass
  aClass == MultiByteString ifTrue:[ ^ true ] .
  ^ super _classHistoryIncludesIdentical: aClass
%

category: 'Documentation'
classmethod: String
_javascriptCharacters
  "Documentation/testing for String>>javaScriptEncode primitive.
   Returns a Translation Array."
  | jsc escapes |
  jsc := Array new: 128.
  0 to: 31 do: [ :index |
    jsc
      at: index + 1
      put: '\x' , (index asHexStringWithLength: 2)
  ].
  escapes := #( 0 '\0' 7 '\a' 8 '\b' 9 '\t' 10 '\n' 12 '\f' 13 '\r' 34 '\"' 92 '\\' ).
  1 to: escapes size - 1 by: 2 do: [ :index |
    jsc at: (escapes at: index) + 1  put: (escapes at: index + 1)
  ].
  ^ jsc
%

category: 'Documentation'
classmethod: String
_javaScriptEncode: aString
 "Documentation/testing for String>>javaScriptEncode primitive.
 The smalltalk algorithm which is the specification for javaScriptEncode primitive."
 | estr jscChars jscCharsSize |
  estr := aString class new .
  estr add: $" .
  jscChars := self _javascriptCharacters . "method temp access faster than classVar"
  jscCharsSize := jscChars size . "avoid multiple sends of size "
  1 to: aString size do: [ :index |
    | char value encoded |
    char := aString at: index.
    value := char codePoint"was greaseInteger" .
    value < jscCharsSize
      ifFalse: [
        "U+2028 and U+2029 have to be treated as new lines"
        value == 16r2028 "Line separator"
          ifTrue: [ estr addAll: '\u2028' ]
          ifFalse: [
            value == 16r2029 "Paragraph separator"
              ifTrue: [ estr addAll: '\u2029' ]
              ifFalse: [ estr add: char ] ] ]
      ifTrue: [
        encoded := jscChars at: value + 1.
        "we use nil markers because #ifNil is faster than #isString because it's not
        actually sent"
        encoded
          ifNil: [ estr add: char ]
          ifNotNil: [ estr addAll: encoded ] ] ].
  estr add: $" .
  ^ estr
%

!		Instance methods for 'String'

category: 'Concatenating'
method: String
, aCharOrCharCollection

"Returns a new instance of the receiver's class that contains the elements of
 the receiver followed by the elements of aCharOrCharCollection.  The argument
 must be a String or an AbstractCharacter.

 The result may not be an instance of the class of the receiver if one of the
 following rules applies:

 1) If the receiver is a Symbol or InvariantString, the result is a String.

 2) If the class of the argument is not String, and is ISOLatin or some other
    user-defined subclass of String, and is not a subclass of
    ObsoleteInvariantString, then the result is an instance of the class of the
    argument.

 3) If the argument is a kind of DoubleByteString, the result is a
    DoubleByteString.

 4) If the argument is a kind of QuadByteString, the result is a
    QuadByteString.

 Warning: Creating a new instance and copying the receiver take time.  If you
 can safely modify the receiver, it can be much faster to use the addAll:
 method.  See the documentation of the Concatenating category of class
 SequenceableCollection for more details."

<primitive: 615 >

"If the primitive failed, the argument is bad, or the argument is
 a JapaneseString or JISCharacter, or Character with value > 255."

| cSize result |
(cSize := aCharOrCharCollection stringCharSize) >= 2 ifTrue:[
  (cSize == 2) ifTrue:[
    result := DoubleByteString withAll: self.
    result addAll: aCharOrCharCollection.
    ^ result .
  ] ifFalse:[
    result:= QuadByteString withAll: self.
    result addAll: aCharOrCharCollection.
    ^ result .
  ].
 ].
(aCharOrCharCollection class == Character) ifTrue:[
  "single byte char would have been handled by prim"
  aCharOrCharCollection codePoint <= 16rFFFF ifTrue:[
    result := DoubleByteString withAll: self.
  ] ifFalse:[
    result:= QuadByteString withAll: self.
  ].
  result at: (result size + 1) put: aCharOrCharCollection .
  ^ result
 ].
(aCharOrCharCollection isKindOf: JapaneseString) ifTrue:[
  ^ ArgumentError signal:'EUCString and JISString no longer supported'
  ].
(aCharOrCharCollection isKindOf: JISCharacter) ifTrue:[
  ^ ArgumentError signal:'JISCharacter no longer supported'
  ].
aCharOrCharCollection _validateClasses:{ AbstractCharacter . CharacterCollection }.
^ self _primitiveFailed: #,  args: { aCharOrCharCollection }
%

category: 'Case-Insensitive Comparisons'
method: String
< aCharCollection

"Returns true if the receiver collates before the argument.
 Returns false otherwise.

 The comparison is case-insensitive unless the receiver and argument
 are equal ignoring case, in which case upper-case letters collate before
 lower-case letters.  The default behavior for SortedCollections and for
 the sortAscending method in UnorderedCollection is consistent with this
 method, and collates as follows:

     #( 'c' 'MM' 'Mm' 'mb' 'mM' 'mm' 'x' ) asSortedCollection

   yields the following sort order:

        'c' 'mb' 'MM' 'Mm' 'mM' 'mm' 'x'

 Signals an Error if argument is a kind of Unicode7, Unicode16 or Unicode32."

<primitive: 28>
(aCharCollection _stringCharSize bitAnd: 16r7) ~~ 0 ifTrue:[
  ^ (DoubleByteString withAll: self) < aCharCollection .
].
aCharCollection _validateClass: CharacterCollection .
^ aCharCollection > self
%

category: 'Case-Insensitive Comparisons'
method: String
<= aCharCollection

"Returns true if the receiver collates before the argument or if all of the
 corresponding Characters in the receiver and argument are equal.
 Returns false otherwise.

 The comparison is consistent with that defined for the < method."

^(self > aCharCollection) == false
%

category: 'Case-Sensitive Comparisons'
method: String
= aCharCollection

"Returns true if corresponding Characters in the receiver and argument are equal
 and aCharCollection is comparable with the receiver, and aCharCollection is
 not a kind of Symbol.  Returns false otherwise.

 The comparison for equal is case-sensitive.
 Signals an Error if argument is a kind of Unicode7, Unicode16 or Unicode32 .

 Note that 'kind of Symbol' means either an instance of Symbol or instance of
 DoubleByteSymbol."

<primitive: 27>
| argInfo |
argInfo := aCharCollection _stringCharSize .
(argInfo bitAnd: 16r8) ~~ 0 ifTrue:[  "arg is Unicode"
  ^ (self compareTo: aCharCollection collator: nil useMinSize: false) == 0
].
(aCharCollection isKindOf: Utf8) ifTrue:[
  ^ (self compareTo: aCharCollection collator: nil useMinSize: false) == 0
].
(aCharCollection isKindOf: CharacterCollection) ifTrue:[ ^ aCharCollection = self].
^ false .
%

category: 'Case-Insensitive Comparisons'
method: String
> aCharCollection

"Returns true if the receiver collates after the argument.  Returns false
 otherwise.

 The comparison is consistent with that defined for the < method.

 Signals an Error if argument is a kind of Unicode7, Unicode16 or Unicode32."

<primitive: 26>
(aCharCollection _stringCharSize bitAnd: 16r7) ~~ 0 ifTrue:[
  ^ (DoubleByteString withAll: self) > aCharCollection
].
aCharCollection _validateClass: CharacterCollection.
^ aCharCollection < self
%

category: 'Case-Insensitive Comparisons'
method: String
>= aCharCollection

"Returns true if the receiver collates after the argument or if all of the
 corresponding Characters in the receiver and argument are equal.
 Returns false otherwise.

 The comparison is consistent with that defined for the < method."

^ (self < aCharCollection) == false
%

category: 'Adding'
method: String
add: aCharOrCharColl

"Appends all of the elements of aCharOrCharColl to the receiver and returns
 aCharOrCharColl."

<primitive: 296>
| cSize aString |
aCharOrCharColl class == Character ifTrue:[ | av |
  (av := aCharOrCharColl codePoint) > 16rFFFF ifTrue:[
    ^ self _convertToQuadByte add: aCharOrCharColl
  ].
  av > 16rFF ifTrue:[
    ^ self _convertToDoubleByte add: aCharOrCharColl
  ].
  ^ self _primitiveFailed: #add: args: { aCharOrCharColl }
].
((cSize := aCharOrCharColl stringCharSize) >= 2) ifTrue:[
  cSize == 2 ifTrue:[
    (aString := aCharOrCharColl _asString)
      ifNil:[ ^ self _convertToDoubleByte addAll: aCharOrCharColl ]
      ifNotNil:[ ^ self addAll: aString ].
  ].
  (aString := aCharOrCharColl _asString)
    ifNil:[ ^ self _convertToQuadByte addAll: aCharOrCharColl ]
    ifNotNil:[ ^ self addAll: aString ].
].
(aCharOrCharColl isKindOf: AbstractCharacter) ifTrue:[
  ^ self add: aCharOrCharColl asCharacter .
].
aCharOrCharColl _validateClass: CharacterCollection .
^ self add: aCharOrCharColl asString.
%

category: 'Adding'
method: String
addAll: aCharOrCharColl

"Equivalent to add: aCharOrCharColl."

<primitive: 296>
| cSize aString |
aCharOrCharColl class == Character ifTrue:[ | av |
  (av := aCharOrCharColl codePoint) > 16rFFFF ifTrue:[
    ^ self _convertToQuadByte add: aCharOrCharColl
  ].
  av > 16rFF ifTrue:[
    ^ self _convertToDoubleByte add: aCharOrCharColl
  ].
  ^ self _primitiveFailed: #add: args: { aCharOrCharColl }
].
((cSize := aCharOrCharColl stringCharSize) >= 2) ifTrue:[
  cSize == 2 ifTrue:[
    (aString := aCharOrCharColl _asString)
      ifNil:[ ^ self _convertToDoubleByte addAll: aCharOrCharColl ]
      ifNotNil:[ ^ self addAll: aString ].
  ].
  (aString := aCharOrCharColl _asString)
    ifNil:[ ^ self _convertToQuadByte addAll: aCharOrCharColl ]
    ifNotNil:[ ^ self addAll: aString ].
].
(aCharOrCharColl isKindOf: AbstractCharacter) ifTrue:[
  ^ self add: aCharOrCharColl asCharacter .
].
(aCharOrCharColl isKindOf: CharacterCollection) ifTrue:[
  ^ self add: aCharOrCharColl asString.
].
aCharOrCharColl do: [:each | self add: each].
^aCharOrCharColl.
%

category: 'Adding'
method: String
addAllBytes: aCharacterCollection

"Adds the byte contents of aCharacterCollection to the receiver ,
 using big-endian byte ordering of aCharacterCollection .
 Returns aCharacterCollection.

 The aCharacterCollection argument must be a kind of String or
 DoubleByteString."

"Used in implementation of methods to support PassiveObject."

<primitive: 470>
aCharacterCollection _validateClasses:{ String }.
^ self _primitiveFailed: #addAllBytes: args: { aCharacterCollection }
%

category: 'Encoding'
method: String
addAllUtf8: aCharacterOrString
  "appends the UTF8 encoding of the argument to the receiver."
  <primitive: 1045>
  aCharacterOrString _validateKindOfClasses: { Character . String } .
  ^ self _primitiveFailed: #nextPutAllUtf8: args: { aCharacterOrString }
%

category: 'Adding'
method: String
addCodePoint: aSmallInteger

<primitive: 1047>
^ self add: (Character codePoint: aSmallInteger).
%

category: 'Adding'
method: String
addLast: aCharOrCharColl

"Equivalent to add: aCharOrCharColl."

<primitive: 296>
| cSize aString |
aCharOrCharColl class == Character ifTrue:[ | av |
  (av := aCharOrCharColl codePoint) > 16rFFFF ifTrue:[
    ^ self _convertToQuadByte add: aCharOrCharColl
  ].
  av > 16rFF ifTrue:[
    ^ self _convertToDoubleByte add: aCharOrCharColl
  ].
  ^ self _primitiveFailed: #add: args: { aCharOrCharColl }
].
((cSize := aCharOrCharColl stringCharSize) >= 2) ifTrue:[
  cSize == 2 ifTrue:[
    (aString := aCharOrCharColl _asString)
      ifNil:[ ^ self _convertToDoubleByte addAll: aCharOrCharColl ]
      ifNotNil:[ ^ self addAll: aString ].
  ].
  (aString := aCharOrCharColl _asString)
    ifNil:[ ^ self _convertToQuadByte addAll: aCharOrCharColl ]
    ifNotNil:[ ^ self addAll: aString ].
].
(aCharOrCharColl isKindOf: AbstractCharacter) ifTrue:[
  ^ self add: aCharOrCharColl asCharacter .
].
aCharOrCharColl _validateClass: CharacterCollection .
^ self add: aCharOrCharColl asString.
%

category: 'Converting'
method: String
asArrayOfKeywords

"Returns an Array of keyword substrings held by the receiver.	 The receiver
 is assumed to be a colon-separated list of substrings.  These substrings
 are extracted and collected in an Array.  If the receiver contains no
 colons, the Array will hold a copy of the receiver."

| c nextName result |

result := { }.
nextName := self class _newString .
1 to: self size do: [ :i |
  c := self at: i.
  nextName add: c.
  c == $: ifTrue: [
    result add: nextName.
    nextName := self class _newString .
  ].
].
nextName size ~~ 0 ifTrue: [
  result add: nextName
]
ifFalse: [
  result size == 0 ifTrue: [result add: nextName]
].
^result
%

category: 'Converting'
method: String
asByteArray

	^ByteArray fromString: self
%

category: 'Deprecated'
method: String
asciiLessThan: aString

"Returns true if the receiver collates before the argument using the
 ASCII collating table, which collates AB...Z...ab..z."

self deprecated: 'String>>asciiLessThan: deprecated v3.2.'.
^ self lessThan: aString collatingTable: AsciiCollatingTable
%

category: 'Converting'
method: String
asLowercase

"Returns a new instance of the receiver's class, with all lower-case Characters
 in the receiver changed to upper-case.

 Lower-case Characters are defined by the Unicode standard and
 u_tolower from libicu is called for each character in the receiver.

 If the receiver is a Symbol, returns an instance of String."
<primitive: 868>
self _primitiveFailed: #asLowercase .
self _uncontinuableError
%

category: 'Converting'
method: String
asLowercaseOld

"Returns a new instance of the receiver's class, with all lower-case Characters
 in the receiver changed to upper-case.

 Lower-case Characters are the a to z,
 including letters with diacriticals, up to codePoint 255.
 (deprecated)character data tables are installed are used if installed.

 If the receiver is a Symbol, returns an instance of String."
<primitive: 986>
self _primitiveFailed: #asLowercaseOld .
self _uncontinuableError
%

category: 'Formatting'
method: String
asString

"Returns the receiver."

^self
%

category: 'Converting'
method: String
asSymbol

"Returns a Symbol that corresponds to the receiver."

<primitive: 313>
self _primitiveFailed: #asSymbol
%

category: 'Converting'
method: String
asSymbolKind

"Equivalent to asSymbol."

<primitive: 313>
self _primitiveFailed: #asSymbolKind
%

category: 'Converting'
method: String
asUnicodeString

"If receiver is an Unicode7 , return the receiver,
 else return an instance of Unicode7 or Unicode16
 using the class with smallest character size needed to
 represent a copy of the receiver."

<primitive: 927>

self _primitiveFailed:#asUnicodeString
%

category: 'Converting'
method: String
asUppercase

"Returns a new instance of the receiver's class, with all lower-case Characters
 in the receiver changed to upper-case.

 Upper-case Characters are defined by the Unicode standard and
 u_toupper from libicu is called for each character in the receiver.

 If the receiver is a Symbol, returns an instance of String."
<primitive: 298>
self _primitiveFailed: #asUppercase .
self _uncontinuableError
%

category: 'Converting'
method: String
asUppercaseOld

"Returns a new instance of the receiver's class, with all lower-case Characters
 in the receiver changed to upper-case.

 Upper-case Characters are the A to Z,
 including letters with diacriticals, up to codePoint 255.
 (deprecated)character data tables are installed are used if installed.

 If the receiver is a Symbol, returns an instance of String."
<primitive: 985>
self _primitiveFailed: #asUppercaseOld .
self _uncontinuableError
%

category: 'Accessing'
method: String
at: anIndex

"Returns the Character at anIndex."

<primitive: 69>
(anIndex _isSmallInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #at: args: { anIndex } .
self _uncontinuableError
%

category: 'Case-Sensitive Comparisons'
method: String
at: anIndex equals: aCharCollection

"Returns true if aCharCollection is contained in the receiver, starting at
 anIndex.  Returns false otherwise.  The comparison is case-sensitive."

<primitive: 778>
anIndex _validateClass: SmallInteger .
((anIndex <= 0) or: [(anIndex > self size)])
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex ].
aCharCollection _validateClass: CharacterCollection.
^ self _primitiveFailed: #at:equals: args: { anIndex . aCharCollection }
%

category: 'Case-Insensitive Comparisons'
method: String
at: anIndex equalsNoCase: aCharCollection

"Returns true if aCharCollection is contained in the receiver, starting at
 anIndex.  Returns false otherwise.  The comparison is case-insensitive."

<primitive: 29>
anIndex _validateClass: SmallInteger.
((anIndex <= 0) or: [(anIndex > self size)])
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex ].
aCharCollection _validateClass: CharacterCollection.
^ self _primitiveFailed: #at:equalsNoCase: args: { anIndex . aCharCollection }
%

category: 'Updating'
method: String
at: anIndex put: aChar

"Stores aChar at the specified location."

<primitive: 293>
(aChar class == Character) ifTrue:[  | av |
  (av := aChar codePoint) > 16rFFFF ifTrue:[
    ^ self _convertToQuadByte at:anIndex put:aChar
  ].
  av > 16rFF ifTrue:[
    ^ self _convertToDoubleByte at:anIndex put:aChar
  ].
] ifFalse:[
 aChar _validateClass: AbstractCharacter .
 ^ self at: anIndex put: aChar asCharacter
].
(anIndex _isSmallInteger) ifTrue: [
  ((anIndex > (self size + 1)) or: [anIndex <= 0]) ifTrue: [
    ^ self _errorIndexOutOfRange: anIndex
  ]
] ifFalse: [
  ^ self _errorNonIntegerIndex: anIndex
] .
self _primitiveFailed: #at:put: args: { anIndex . aChar }
%

category: 'Formatting'
method: String
charSize
"Returns number of bytes that make up a character for this string class."

^ 1
%

category: 'Accessing'
method: String
codePointAt: anIndex

"Returns the ascii value of the Character at anIndex."

<primitive: 756>

(anIndex _isSmallInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #codePointAt: args: { anIndex } .
self _uncontinuableError
%

category: 'Updating'
method: String
codePointAt: anIndex put: aValue

"Stores aValue at the specified location, with autogrow, but
 without auto-conversion of receiver to larger character size.
 Returns aValue.
"
<primitive: 784>
aValue _validateClass: SmallInteger .
(aValue < 0 or:[ aValue > 255]) ifTrue:[
  aValue _error: #rtErrArgOutOfRange args:{ 0 . 255 }
].
(anIndex _isSmallInteger) ifTrue: [
  ((anIndex > (self size + 1)) or: [anIndex < 1]) ifTrue: [
    ^ self _errorIndexOutOfRange: anIndex
  ]
] ifFalse: [
  ^ self _errorNonIntegerIndex: anIndex
] .
self _primitiveFailed: #codePointAt:put: args: { anIndex . aValue }
%

category: 'Comparing'
method: String
codePointCompareTo: aString

"aString must be a CharacterCollection or ByteArray,
 and must not be a Utf8 or Utf16 .
 Return -1 , 0, or 1  based on how self compares to aString
 using codePoint comparison of each character. "

<primitive: 844>
^ self _primitiveFailed: #codePointCompareTo: args: { aString }
%

category: 'Testing'
method: String
containsSeparator

"Returns true if the receiver contains a separator Character."

1 to: self size do: [:i |
  (self at: i) isSeparator ifTrue: [^true].
].
^false
%

category: 'Copying'
method: String
copyFrom: startIndex to: stopIndex

"Returns a new Array containing the elements of the receiver
 between startIndex and stopIndex, inclusive.  The result is of the same class
 as the receiver.

 If startIndex > stopIndex then an empty collection is returned.
 Otherwise both startIndex and stopIndex must be positive integers not larger than the
 size of the receiver, with startIndex <= stopIndex.
"

<primitive: 818>
(startIndex < 1) ifTrue: [ ^ self _errorIndexOutOfRange: startIndex].

((stopIndex > self size) or: [(stopIndex < 0)])
   ifTrue: [ ^ self _errorIndexOutOfRange: stopIndex].

self _primitiveFailed: #copyFrom:to: args: { startIndex . stopIndex }
%

category: 'Encoding'
method: String
decodeFromUTF8

"Decode receiver from UTF8 format.
 Returns either a Unicode7 , Unicode16 or Unicode32 ,
 using the minimum character size needed to represent decoded result."

^ self _decodeUtf8StartingAt: 1 unicodeResult: true maxSize: nil bytesConsumed: nil
%

category: 'Encoding'
method: String
decodeFromUTF8ToString

"Decode UTF8 contents of the receiver returning a String, DoubleByteString or
  QuadByteString."

^ self _decodeUtf8StartingAt: 1 unicodeResult: false maxSize: nil bytesConsumed: nil
%

category: 'Encoding'
method: String
decodeFromUTF8ToUnicode

"Decode UTF8 contents of the receiver returning a Unicode7 , Unicode16 or Unicode32"

^ self _decodeUtf8StartingAt: 1 unicodeResult: true maxSize: nil bytesConsumed: nil
%

category: 'Encoding'
method: String
encodeAsUTF16

"Encode receiver in UTF16 format.  Returns a Utf16 ."

<primitive: 1084>
self _primitiveFailed: #encodeAsUTF16
%

category: 'Encoding'
method: String
encodeAsUTF8

"Encode receiver in UTF8 format.  Returns a Utf8 ."

<primitive: 468>
self _primitiveFailed: #encodeAsUTF8
%

category: 'Encoding'
method: String
encodeAsUTF8IntoString
"Encode receiver in UTF8 format.  Returns a String. For Seaside"

<primitive: 994>
self _primitiveFailed: #encodeAsUTF8IntoString
%

category: 'Encoding'
method: String
encodeUsing: encodingArray

  "A new instance of the class of the receiver will be returned where
   each character in the receiver is replaced by the character or string
   from encodingArray that is located at the position in encodingArray for
   the character.  The codePoint of the character is used as an index
   into the encodingArray,  value 16r0 accesses   encodingArray at:1 .
   An Error is signaled if any of the following are true
    the codePoint is >= size of the encodingArray

    the value from the encodingArray is a Character with codePoint too large
    to fit in an instance of the receiver's class.

    the value from the encodingArray is a CharacterCollection with
    bytes-per-character bigger than bytes-per-character of the receiver

    the value from the encodingArray is neither a String, MultiByteString,
    nor Character . "
  <primitive: 654>
  encodingArray _validateInstanceOf: Array  .
  self _primitiveFailed: #encodeUsing: args: { encodingArray }
%

category: 'Deprecated'
method: String
equals: aString collatingTable: aTable

"Returns true if the receiver collates the same as the argument.
 The collating sequence is defined by aTable,
 See also lessThan:collatingTable:  for description of aTable.

 If aString is a Symbol, there is no difference in behavior
 ( contrast with String>>= )."

self deprecated: 'String>>equals:collatingTable: deprecated v3.2'.
^ self _primEquals: aString collatingTable: aTable
%

category: 'Case-Insensitive Comparisons'
method: String
equalsNoCase: aCharCollection

"Returns true if corresponding Characters in the receiver and argument are equal
 and aCharCollection is comparable with the receiver.  Returns false otherwise.

 The comparison for equal is case-insensitive.

 If aString is a Symbol, there is no difference in behavior
 (contrast with String | =)."

<primitive: 95>
aCharCollection _stringCharSize ~~ 0 ifTrue:[
  self _primitiveFailed: #equalsNoCase:  args: { aCharCollection } .
  ^ self _uncontinuableError
].
(aCharCollection isKindOf: CharacterCollection) ifTrue: [
  ^ self equalsNoCase: (String withAll: aCharCollection)
] ifFalse:[
  ^ false
].
%

category: 'Execution'
method: String
evaluate

"Compiles the receiver as an unbound method and executes it using the current
 default symbol list."

^ (self _compileInContext: nil
       symbolList: GsSession currentSession symbolList
       oldLitVars: nil environmentId: 0 flags: 0)
  _executeInContext: nil
%

category: 'Execution'
method: String
evaluateInContext: anObject symbolList: aSymbolList

"Compiles the receiver as an instance method for the class of anObject, using
 aSymbolList as the symbol list.  Executes the resulting GsMethod using anObject
 as self and returns the result of the execution.  Generates an error if
 compilation errors occur."

^ (self _compileInContext: anObject symbolList: aSymbolList oldLitVars: nil
	environmentId: 0 flags: 0)
    _executeInContext: anObject
%

category: 'Searching'
method: String
findPattern: aPattern startingAt:  anIndex

"This method searches the receiver, beginning at anIndex, for a substring that
 matches aPattern.  If a matching substring is found, this method returns the
 index of the first Character of the substring.  Otherwise, this returns 0.

 The argument aPattern is an Array containing zero or more CharacterCollections
 plus zero or more occurrences of the special Characters asterisk or
 question-mark.  See the description of the matchPattern: method for more
 information about this argument.

 Performs a case-sensitive search."

^self
    indexOf: aPattern
    matchCase: true
    startingAt: anIndex
%

category: 'Searching'
method: String
findPatternNoCase: aPattern startingAt:  anIndex

"This method searches the receiver, beginning at anIndex, for a substring that
 matches aPattern.  If a matching substring is found, this method returns the
 index of the first Character of the substring.  Otherwise, this returns 0.

 The argument aPattern is an Array containing zero or more CharacterCollections
 plus zero or more occurrences of the special Characters asterisk or
 question-mark.  See the description of the matchPattern: method for more
 information about this argument.

 Performs a case-insensitive search."

^self
    indexOf: aPattern
    matchCase: false
    startingAt: anIndex
%

category: 'Deprecated'
method: String
greaterThan: aString collatingTable: aTable

"Returns true if the receiver collates after the argument.
 See also lessThan:collatingTable:  for description of aTable."

self deprecated: 'String>>greaterThan:collatingTable: deprecated v3.2'.
^self _primGreaterThan: aString collatingTable: aTable
%

category: 'Deprecated'
method: String
greaterThanOrEqual: aString collatingTable: aTable

"Returns true if the receiver collates after or the same as the argument.
 The collating sequence is defined by aTable."

self deprecated: 'String>>greaterThanOrEqual:collatingTable: deprecated v3.2'.
^ (self lessThan: aString collatingTable: aTable) == false
%

category: 'Hashing'
method: String
hash

"Returns a positive Integer based on a case-sensitive hash of the contents
 of the receiver.  The algorithm implemented is described in:

 [Pearson 90]
 Pearson, Peter K., Fast Hashing of Variable-Length Text Strings,
 Communications of the ACM 33, 6, (June 1990), 677-680."

<primitive: 31>
self _primitiveFailed: #hash .
self _uncontinuableError
%

category: 'Case-Sensitive Searching'
method: String
includes: aCharacter

"Returns true if the receiver contains aCharacter, false otherwise.
 The search is case-sensitive."

^(aCharacter isKindOf: AbstractCharacter) and:
	[self includesValue: aCharacter].
%

category: 'Case-Sensitive Searching'
method: String
includesValue: aCharacter

"Returns true if the receiver contains aCharacter, false otherwise.
 The search is case-sensitive."

<primitive: 94>
aCharacter _validateClass: AbstractCharacter .
^ self includesValue: aCharacter asCharacter .
%

category: 'Searching'
method: String
indexOf: patternArray matchCase: caseBoolean startingAt:  anInteger

"Searches the receiver, starting at the specified index,
 for a substring that matches the pattern.  If a matching substring
 is found, returns the index of the first Character of the substring.
 Otherwise, returns 0.

 The pattern is an Array containing zero or more CharacterCollections
 plus zero or more occurrences of the special Characters asterisk or
 question-mark.  See the description of the matchPattern: method for more
 information about this argument.

 If matchCase is true, a case-sensitive search is performed.  Otherwise,
 a case-insensitive search is performed."

  | result |
  result := self
      _primMatchPattern: patternArray
      ignoreCase: (caseBoolean not)
      findSubString: true
      startingAt: anInteger.
  result == -1 ifTrue:[ ^ super indexOf: patternArray matchCase: caseBoolean startingAt:  anInteger].
  ^ result ifNil:[ 0 ] ifNotNil:[ result at: 1 ]
%

category: 'Case-Sensitive Searching'
method: String
indexOf: aCharacter startingAt: startIndex

"Returns the index of the first occurrence of aCharacter in the receiver,
 not preceding startIndex.  If the receiver does not contain aCharacter,
 returns zero.

 The search is case-sensitive."

<primitive: 93>
startIndex _validateClass: Integer .
startIndex < 1 ifTrue:[ ^ self _errorIndexOutOfRange: startIndex].

^ self indexOf: aCharacter asCharacter startingAt: startIndex
%

category: 'Case-Sensitive Searching'
method: String
indexOfByte: anInt startingAt: startIndex

"Returns the index of the first occurrence of (Character withValue: anInt)
 in the receiver, not preceding startIndex.
 If the receiver does not contain the specified Character, returns zero.

 The search is case-sensitive."

<primitive: 704>
startIndex _validateClass: SmallInteger .
startIndex < 1 ifTrue:[ ^ self _errorIndexOutOfRange: startIndex].
anInt _validateClass: SmallInteger  .
(anInt < 0 or:[ anInt > 16rFFFFFFFF]) ifTrue:[ ^ self _errorIndexOutOfRange: anInt ] .
^ self _primitiveFailed: #indexOfByte:startingAt: args: { anInt . startIndex }
%

category: 'Case-Sensitive Searching'
method: String
indexOfLastByte: anInt startingAt: startIndex

"Returns the index of the last occurrence of (Character withValue: anInt)
 in the receiver, starting from startIndex and searching backwards.
 If the receiver does not contain the specified Character, returns zero.

 The search is case-sensitive."

<primitive: 780>
startIndex _validateClass: SmallInteger .
startIndex < 1 ifTrue:[ ^ self _errorIndexOutOfRange: startIndex].
anInt _validateClass: SmallInteger  .
(anInt < 0 or:[ anInt > 16rFFFFFFFF]) ifTrue:[ ^ self _errorIndexOutOfRange: anInt ] .
^ self _primitiveFailed: #indexOfLastByte:startingAt:
       args: { anInt . startIndex }
%

category: 'Adding'
method: String
insertAll: aCharOrCharColl at: anIndex

"Inserts aCharOrCharColl at the specified index. Returns aCharOrCharColl"

<primitive: 299>
| aString cSize |
anIndex _isSmallInteger ifFalse:[ anIndex _validateClass: SmallInteger ].
((anIndex <= 0) or: [anIndex > (self size + 1)])
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].

(cSize := aCharOrCharColl stringCharSize) == 2 ifTrue:[
  (aString := aCharOrCharColl _asString)
    ifNil:[ ^ self _convertToDoubleByte insertAll: aCharOrCharColl at: anIndex]
    ifNotNil:[ ^ self insertAll: aString at: anIndex ].
].
cSize == 4 ifTrue:[
  (aString := aCharOrCharColl _asString)
    ifNil:[ ^ self _convertToQuadByte insertAll: aCharOrCharColl at: anIndex]
    ifNotNil:[ ^ self insertAll: aString at: anIndex ].
].
aCharOrCharColl _validateClasses: { AbstractCharacter . CharacterCollection }.
^self insertAll: aCharOrCharColl asString at: anIndex.
%

category: 'Case-Insensitive Comparisons'
method: String
isEquivalent: aCharCollection

"Returns true if the receiver is equivalent to aCharCollection.  The receiver
 and aCharCollection are compared without regard to case or internal
 representation of Characters."

<primitive: 95>
aCharCollection _stringCharSize ~~ 0 ifTrue:[
  self _primitiveFailed: #isEquivalent:  args: { aCharCollection } .
  ^ self _uncontinuableError
].
(aCharCollection isKindOf: CharacterCollection) ifTrue: [
  ^ self isEquivalent: (String withAll: aCharCollection)
] ifFalse:[
  ^ false
].
%

category: 'Encoding'
method: String
javaScriptEncode
  "Returns a String.
   The primitive encapulates all of the encoding logic
   from Seaside JSStream>>encodeString:on: .
   See   String class>>_javaScriptEncode: for the algorithm in Smalltalk.
   Assumes Character>>greaseInteger always equivalent to Character>>codePoint
  "
<primitive: 1046>
^ self _primitiveFailed: #javaScriptEncode
%

category: 'Deprecated'
method: String
lessThan: aString collatingTable: aTable

"Returns true if the receiver collates before the argument.
 The collating sequence is defined by aTable which must be
 a string with the same character size as the receiver."

self deprecated: 'String>>lessThan:collatingTable: deprecated v3.2'.
^ self _primLessThan: aString collatingTable: aTable
%

category: 'Deprecated'
method: String
lessThanOrEqual: aString collatingTable: anArray

"Returns true if the receiver collates before or the same as the argument
 as defined by the collating table anArray. "

self deprecated: 'String>>lessThanOrEqual:collatingTable: deprecated v3.2'.
^ ( self greaterThan: aString collatingTable: anArray ) == false
%

category: 'Storing and Loading'
method: String
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into the receiver."

passiveObj hasRead: self.
passiveObj next: self _basicSize bytesTo: self.
passiveObj next.  "consume the space put after all strings"
^self
%

category: 'Comparing'
method: String
matchPattern: patternArray

"Returns true if the receiver matches the pattern, false if it doesn't.
 An exact match is required.  For partial matching, use the 'Searching' method
 findPattern:startingAt: instead.

 The pattern is a kind of Array containing zero or more
 CharacterCollections, plus zero or more occurrences of the special Characters
 $* or $?.  If either $* or $? occurs in the pattern, it acts as a wild card.
 The Character $? matches any single Character in the receiver, and $* matches
 any sequence of zero or more Characters in the receiver.  For example,

 'weimaraner' matchPattern: #('w' $* 'r')

 returns true, because the Character $* is interpreted as a wild card.

 If either of these special Characters occurs in the receiver, it is
 interpreted literally.  For example,

 'w*r' matchPattern: #('weimaraner')

 returns false - because the Character $* occurs in the receiver, it is
 interpreted as a literal asterisk (not as a wild card)."
 | res |
 res := self
    _primMatchPattern: patternArray
    ignoreCase: false
    findSubString: false
    startingAt: 1 .
 res == -1 ifTrue:[ ^ super matchPattern: patternArray "handle Unicode in arg" ].
 ^ res
%

category: 'Formatting'
method: String
printOn: aStream

"Puts a displayable representation of the receiver on the given stream."

"For classes whose instances can be literals, the result should contain
 formatting information.  For example, the following expression should
 evaluate to true:

 #abc asString = String withAll: '#abc'

 The formatting information is may not be included if the String exceeds
 1000000 (1 MByte) in size."

| idx lastIdx sz maxSz|

aStream _nextPut: $' .

idx := 0.
self size < (maxSz := aStream maxSize) ifTrue:[
  idx := self indexOf: $' startingAt: 1 .
].
idx == 0 ifTrue:[
  aStream nextPutAll: self .
  ]
ifFalse:[
  lastIdx := 1.
  [ (idx == 0) or: [aStream isFull]] whileFalse: [
    aStream nextPutAll: (self copyFrom: lastIdx to: idx) .
    aStream nextPut: $' .
    lastIdx := idx + 1 .
    idx := self indexOf: $' startingAt: lastIdx .
    ].
  sz := self size .
  ((lastIdx <= sz) and: [aStream isFull == false ]) ifTrue: [
    aStream nextPutAll: (self copyFrom: lastIdx to: sz)  .
    ].
  ].
aStream _nextPut: $'
%

category: 'Removing'
method: String
removeFrom: startIndex to: stopIndex

"Deletes the elements of the receiver from startIndex to stopIndex.  The
 size of the receiver is decreased by stopIndex - startIndex + 1.

 Both startIndex and stopIndex must be positive integers not larger than the
 size of the receiver, with startIndex <= stopIndex."

<primitive: 386>
(stopIndex < startIndex)
ifTrue:
   [ ^ startIndex _error: #rtErrBadCopyFromTo args: { stopIndex }].
((stopIndex > self size) or: [(stopIndex < 1)])
   ifTrue: [ ^ self _errorIndexOutOfRange: stopIndex].
(startIndex < 1)
   ifTrue: [ ^ self _errorIndexOutOfRange: startIndex].
^ self _primitiveFailed: #removeFrom:to: args: { startIndex . stopIndex }
%

category: 'Copying'
method: String
replaceFrom: startIndex to: stopIndex with: charCollection startingAt: repIndex

"Replaces the elements of the receiver between the indexes startIndex and
 stopIndex inclusive with the elements of aSeqCollection starting at repIndex.
 If aSeqCollection is identical to the receiver, the source and
 destination blocks may overlap.

 The primitive supports directly the case where
   (charCollection isKindOfClass: CByteArray) == true
 with repIndex being one-based .

 Returns the receiver."

<primitive: 297>
| cSize argInfo recInfo src7 |
startIndex _isSmallInteger ifFalse:[ startIndex _validateClass: SmallInteger ].
stopIndex _isSmallInteger ifFalse:[ stopIndex _validateClass: SmallInteger ].
repIndex _isSmallInteger ifFalse:[ repIndex _validateClass: SmallInteger ].
argInfo := charCollection _stringCharSize .
recInfo := self _stringCharSize .
(recInfo bitAnd: 16r8) ~~ 0 ifTrue:[  "receivier is a Utf string"
  (cSize := argInfo bitAnd: 16r7) ~~ 0 ifTrue:[
    src7 := charCollection _asUnicode7 .
    (src7 ~~ nil and:[ src7 ~~ charCollection]) ifTrue:[
      ^ self replaceFrom: startIndex to: stopIndex with: src7 startingAt: repIndex
    ].
  ].
  cSize == 2 ifTrue:[
    ^ self _convertToDoubleByte
      replaceFrom: startIndex to: stopIndex with: charCollection startingAt: repIndex
  ].
  ^ self _convertToQuadByte
      replaceFrom: startIndex to: stopIndex with: charCollection startingAt: repIndex
].
(cSize := argInfo bitAnd: 16r7) >= 2 ifTrue:[
   src7 := charCollection _asString .
   (src7 ~~ nil and:[ src7 ~~ charCollection]) ifTrue:[
      ^ self replaceFrom: startIndex to: stopIndex with: src7 startingAt: repIndex
   ].
   cSize == 2 ifTrue:[ ^ self _convertToDoubleByte replaceFrom: startIndex to: stopIndex
			     with: charCollection startingAt: repIndex
   ].
   ^ self _convertToQuadByte replaceFrom: startIndex to: stopIndex
                        with: charCollection startingAt: repIndex
].
^ super replaceFrom: startIndex to: stopIndex with: charCollection startingAt: repIndex
%

category: 'Copying'
method: String
reverse

"Returns a copy of the receiver with its elements in reverse order."
| copy |
copy := self species new: self size .
^ copy _reverseFrom: self .
%

category: 'Accessing'
method: String
size

"Returns the size of the receiver, in Characters."

<primitive: 0>
self _primitiveFailed: #size .
self _uncontinuableError
%

category: 'Updating'
method: String
size: anInteger

"Changes the size of the receiver to anInteger.
 Returns the receiver.

 If anInteger is less than the current size of the receiver, the receiver is
 shrunk accordingly.  If anInteger is greater than the current size of the
 receiver, the receiver is extended and new elements are initialized to nil."

<primitive: 603 >
(anInteger _isSmallInteger)
   ifTrue: [^ self _errorIndexOutOfRange: anInteger]
   ifFalse: [^ self _errorNonIntegerIndex: anInteger].
self _primitiveFailed: #size: args: { anInteger } .
self _uncontinuableError
%

category: 'Encoding'
method: String
sizeForEncodeAsUTF8

"Returns a SmallInteger, the size of the result of encodeAsUTF8."

<primitive: 520>
self _primitiveFailed: #sizeForEncodeAsUTF8
%

category: 'Deprecated'
method: String
toClientTextFile: fileName

| file |
self deprecated: 'String>>toClientTextFile: deprecated long before v3.0.  Use an instance of GsFile to access the file system.'.

file := GsFile openWrite: fileName.
[file == nil] whileTrue: [
  self _halt: 'Unable to open file: ' , fileName.
  file := GsFile openWrite: fileName
].
file nextPutAll: self.
file close
%

category: 'Repository Conversion'
method: String
transformIntoString

"Private. Transforms the receiver to an instance of String."

^ self.
%

category: 'Copying'
method: String
withLFs

"Returns a copy of the receiver with all back-slashes replaced by line-feeds."

^self copyReplaceAll: '\' with: Character lf
%

category: 'Storing and Loading'
method: String
writeTo: passiveObj
"Converts the receiver to its passive form and writes that information on
 passiveObj."

passiveObj writeClass: self class;
  writeSize: self size;
    nextPutAll: self; space
%

category: 'Decompiling without Sources'
method: String
_asSource

^ self quoted
%

category: 'Converting'
method: String
_asUnicode7

<primitive: 925>
"Return an instance of Unicode7 if receiver can be
 represented as an Unicode7 string, else return nil."

self _primitiveFailed:#_asUnicode7
%

category: 'Private'
method: String
_at: anIndex equals: aCharCollection ignoreCase: aBoolean

"Returns true if aCharCollection is contained in the receiver, starting at
 anIndex.  Returns false otherwise.  The comparison is case-insensitive."

^ aBoolean ifTrue:[ self at: anIndex equalsNoCase: aCharCollection]
	ifFalse:[ self at: anIndex equals: aCharCollection]
%

category: 'Private'
method: String
_changeClassToMultiByte: aClass

"Private.   Used when loading strings from a PassiveObject.
 Change the class of the receiver to aClass, which must be
 a subclass of MultiByteString, and convert bytes of receiver
 from  big-endian to native byte order.
 structure of the receiver.  Returns the receiver."

<primitive: 466>
self _primitiveFailed: #_changeClassToMultiByte: args: { aClass }
%

category: 'Execution'
method: String
_compileInContext: anObject symbolList: aSymbolList

"Compiles the receiver as an instance method for the class of
 anObject, using aSymbolList as the symbol list, and using the
 default compilation environment.
 The receiver is parsed as a doit, i.e. it does not have a selector.

 Returns a GsMethod that is not in the method dictionary of any class, or
 else generates an error if there are compilation errors."

^ self _compileInContext: anObject symbolList: aSymbolList oldLitVars: nil
      environmentId: 0 flags: 0
%

category: 'Execution'
method: String
_compileInContext: anObject symbolList: aSymbolList
oldLitVars: litVarArray  environmentId: anEnvironmentId flags: flagsInt

"Compiles the receiver as an instance method for the class of
 anObject, using aSymbolList as the symbol list.
 The receiver is parsed as a doit, i.e. it does not have a selector.

 If litVarArray is not nil, it must be an Array of Symbol,SymbolAssociation pairs
 and this Array will be searched prior to searching aSymbolList to
 resolve literal variables within the method.

 anEnvironmentId must be a SmallInteger >= 0 and <= 255
 specifying a compilation environment.

 flagsInt must be a SmallInteger >= 0 and <= 255 and contains the bits
 specified by GCI_COMPILE_* in $GEMSTONE/include/gcicmn.ht .

 Returns a GsMethod that is not in the method dictionary of any class, or
 else generates an error if there are compilation errors."

<primitive: 150>
aSymbolList _validateClass: SymbolList .
^ self _primitiveFailed:
       #_compileInContext:symbolList:oldLitVars:environmentId:flags:
       args: { anObject . aSymbolList . litVarArray . anEnvironmentId . flagsInt}
%

category: 'Converting'
method: String
_convertToDoubleByte

"Change the receiver from a String to a DoubleByteString,
 or from an Unicode7 to Unicode16.  Changes the
 class of the receiver and the receiver's storage structure.
 Returns the receiver."

<primitive: 196>
%

category: 'Converting'
method: String
_convertToQuadByte

"Change the receiver from a DoubleByteString to a QuadByteString,
 or from a Unicode16 to Unicode32.  Changes the
 class of the receiver and the receiver's storage structure.
 Returns the receiver."

<primitive: 445>
%

category: 'Private'
method: String
_decodeFromUtf8: unicodeResultBool

"Decode UTF8 contents of the receiver.
 If unicodeResultBool == true, result is a Unicode7 , Unicode16 or Unicode32.
 If unicodeResultBool == false, result is a String, DoubleByteString or
  QuadByteString."

^ self _decodeUtf8StartingAt: 1 unicodeResult: unicodeResultBool maxSize: nil bytesConsumed: nil
%

category: 'Private'
method: String
_decodeFromUtf8: unicodeResultBool maxSize: aSize

"Decode UTF8 contents of the receiver.
 If unicodeResultBool == true, result is a Unicode7 , Unicode16 or Unicode32.
 If unicodeResultBool == false, result is a String, DoubleByteString or
  QuadByteString."

^ self _decodeUtf8StartingAt: 1 unicodeResult: unicodeResultBool maxSize: aSize bytesConsumed: nil
%

category: 'Private'
method: String
_decodeUtf8StartingAt: anOffset unicodeResult: unicodeResultBool
                 maxSize: aSize bytesConsumed: anArray

"Attempt to decode Utf8 data starting at anOffset in the receiver.
 If unicodeResultBool == true, result is a Unicode7 , Unicode16 or Unicode32.
 If unicodeResultBool == false, result is a String, DoubleByteString or
  QuadByteString.
 If there are insufficient bytes in the receiver and anArray ~~ nil,
 returns nil .
  If there are insufficient bytes in the receiver and anArray == nil, signals an Error.
 Signals an Error if the bytes starting at anOffset are not legal UTF8 data .
 If anArray ~~ nil, returns in (anArray at: 1) the number of bytes consumed by the decode."

<primitive: 493>
unicodeResultBool _validateClass: Boolean .
aSize _validateClass: SmallInteger .
anArray ifNotNil:[ anArray _validateClass: Array  ].
anOffset _validateClass: SmallInteger .

^ self _primitiveFailed: #_decodeUtf8StartingAt:unicodeResult:maxSize:bytesConsumed:
          args: { anOffset . unicodeResultBool . aSize . anArray }
%

category: 'Deprecated'
method: String
_executeAsSource
	"Compiles the receiver as an unbound method and executes it using the current
	default symbolList."

self deprecated: 'String>>_executeAsSource: deprecated long before v3.0. Use #evaluate'.
^ self evaluate
%

category: 'Private'
method: String
_findLastString: subString startingAt: startIndex ignoreCase: aBoolean

"Searches backwards through receiver    starting at startIndex  .
 Returns the index at which subString begins.  If the
 receiver does not contain subString, this returns 0."

<primitive: 451>

startIndex _validateClass: SmallInteger.
(startIndex < 1) | (startIndex > self size)
  ifTrue: [ ^ self _error: #objErrBadOffsetIncomplete args: { startIndex } ].
subString _validateClasses:{ String }.
aBoolean _validateClass: Boolean .
^ self _primitiveFailed: #_findLastString:startingAt:ignoreCase:
       args: { subString . startIndex . aBoolean }
%

category: 'Private'
method: String
_findString: subString startingAt: startIndex ignoreCase: aBoolean

"If a receiver contains subString beginning at some point at or after
 startIndex, this returns the index at which subString begins.  If the
 receiver does not contain subString, this returns 0."

<primitive: 30>

startIndex _validateClass: SmallInteger.
"startIndex > self size returns 0"
(startIndex < 1)
  ifTrue: [ ^ self _error: #objErrBadOffsetIncomplete args: { startIndex } ].
subString _validateClasses:{ String }.
aBoolean _validateClass: Boolean .
^ self _primitiveFailed: #_findString:startingAt:ignoreCase:
       args: { subString . startIndex . aBoolean }
%

category: 'New Indexing Comparison - prims'
method: String
_idxPrimCompareEqualTo: aCharCollection
  "This comparison operation is used for the indexing subsystem to
 determine an ordering for insertion into indexing objects.

 This method collates letters AaBb..Zz."

  "The comparison should be compatible with the case-insensitive semantics
 of the String method with selector #= .
 Same primitive as String>>lessThan: "

  <primitive: 981>
  aCharCollection isUnicodeString
    ifTrue: [ ArgumentError signal:'Unicode argument disallowed in String comparison' ]
    ifFalse: [
      aCharCollection ifNil:[ ^ false ].
      aCharCollection _validateClass: CharacterCollection.
      "JapaneseString or MixedString"
      ^ self = aCharCollection ]
%

category: 'New Indexing Comparison - prims'
method: String
_idxPrimCompareGreaterThan: aCharCollection

"This comparison operation is used for the indexing subsystem to
 determine an ordering for insertion into indexing objects and for
 doing indexing subsystem comparisons.

 This method collates the same as the > method, except that
 it returns true if the argument is nil."

  <primitive: 184>
  (aCharCollection _stringCharSize bitAnd: 16r8) ~~ 0
    ifTrue: [
      "Unicode"
      ArgumentError signal:'Unicode argument disallowed in String comparison' ]
    ifFalse: [
      aCharCollection _validateClass: CharacterCollection.
      "JapaneseString or MixedString"
      ^ self > aCharCollection ]
%

category: 'New Indexing Comparison - prims'
method: String
_idxPrimCompareLessThan: aCharCollection

"This comparison operation is used for the indexing subsystem to
 determine an ordering for insertion into indexing objects and for
 doing indexing subsystem comparisons.

 This method collates the same as the < method, except that
 it returns false if the argument is nil."

  <primitive: 434>
  aCharCollection isUnicodeString
    ifTrue: [ ArgumentError signal:'Unicode argument disallowed in String comparison' ]
    ifFalse: [
      aCharCollection _validateClass: CharacterCollection.
      "JapaneseString or MixedString"
      ^ self < aCharCollection ]
%

category: 'Storing and Loading'
method: String
_loadFrom: passiveObj
   passiveObj next: self _basicSize bytesTo: self.
   passiveObj next.  "consume the space put after all strings"
  "caller sends hasRead: "
   ^self
%

category: 'Comparing'
method: String
_matchPatternNoCase: patternArray

"Just like matchPattern: except the match is case-insensitive."

| res |
res := self
    _primMatchPattern: patternArray
    ignoreCase: true
    findSubString: false
    startingAt: 1 .
res == -1 ifTrue:[ ArgumentError signal:'invalid element of patternArray'].
^ res
%

category: 'Deprecated'
method: String
_primEquals: aString collatingTable: aTable

"Returns true if the receiver collates the same as the argument.
 The collating sequence is defined by aTable,
 See also lessThan:collatingTable:  for description of aTable.

 If aString is a Symbol, there is no difference in behavior
 ( contrast with String>>= )."

<primitive: 446>
aTable _validateInstanceOf: String .
aTable size == 256 ifFalse:[
  aTable _error: #rtErrBadSize args:{ 256 . aTable size }.
  ].
aString _validateClass: String .
self _primitiveFailed: #equals:collatingTable: args: { aString . aTable }
%

category: 'Deprecated'
method: String
_primGreaterThan: aString collatingTable: aTable

"Returns true if the receiver collates after the argument.
 See also lessThan:collatingTable:  for description of aTable."

<primitive: 433>
aTable _validateInstanceOf: String .
aTable size == 256 ifFalse:[
  aTable _error: #rtErrBadSize args:{ 256 . aTable size }.
  ].
aString _validateClass: String .
self _primitiveFailed: #greaterThan:collatingTable: args: { aString . aTable }
%

category: 'Deprecated'
method: String
_primLessThan: aString collatingTable: aTable

"Returns true if the receiver collates before the argument.
 The collating sequence is defined by aTable which must be
 a string with the same character size as the receiver."

<primitive: 359>
aTable _validateClass: String .
aTable size == 256 ifFalse:[
  aTable _error: #rtErrBadSize args:{ 256 . aTable size }.
  ].
aString _validateClass: String .
self _primitiveFailed: #lessThan:collatingTable: args: { aString . aTable }
%

category: 'Comparing'
method: String
_primMatchPattern: anArray ignoreCase: caseBoolean findSubString: subStringBoolean startingAt:  anInteger

"If findSubString is false then the pattern array must match the entire
 receiver starting at the specified index and the result is true or false.
 If findSubString is true then the pattern array must match a substring
 starting at the specified index and the result is nil if no substring is
 found and a two element Array containing the start and end index if a match
 is found.
 The Strings in the pattern array must be Strings or ByteArrays .
 multi-byte strings are not supported.

 Returns -1 if an element of anArray is a kind of MultiByteString .
"

<primitive: 514>
^ self _primitiveFailed:
       #_primMatchPattern:ignoreCase:findSubString:startingAt:
       args: { anArray . caseBoolean . subStringBoolean . anInteger }
%

category: 'Copying'
method: String
_reverseFrom: aString

"copies elements of aString into receiver in reverse order.
 Size of aString must be same as size of receiver.
 Returns receiver.
 aString==self is allowed. "

<primitive: 793>
aString stringCharSize == 1 ifFalse:[
  aString _error: #rtErrBadArgKind args:{ String }.
].
aString size == self size ifFalse:[
  aString _error: #rtErrBadSize args:{ self size . aString size }.
].
self _primitiveFailed: #_reverseFrom: args: { aString } .
%

! Class extensions for 'StringKeyValueDictionary'

!		Instance methods for 'StringKeyValueDictionary'

removeallmethods StringKeyValueDictionary
removeallclassmethods StringKeyValueDictionary

category: 'Accessing'
method: StringKeyValueDictionary
at: aKey ifAbsent: aBlock

"Returns the value that corresponds to aKey.  If no such key/value pair
 exists, returns the result of evaluating the zero-argument block aBlock."
| x keyOfs buckOfs |
x := self _lookup: aKey .
x _isSmallInteger
  ifTrue:[ keyOfs := x bitAnd: 16r3FFFFFFF . buckOfs := x bitShift: -30]
  ifFalse:[ x ifNil:[ aBlock _isExecBlock ifFalse:[ aBlock _validateClass: ExecBlock ].
                     ^ aBlock value ].
            keyOfs := x at: 1 . buckOfs := x at: 2 ].
buckOfs ~~ 0 ifTrue:[ | collisionBkt |
  collisionBkt := self _at: keyOfs + 1 .
  ^ collisionBkt _at: buckOfs + 1 .
].
^ self _at: keyOfs + 1.
%

category: 'Accessing'
method: StringKeyValueDictionary
at: aKey otherwise: defaultValue

"Returns the value that corresponds to aKey.  If no such key/value pair
 exists, returns the given defaultValue ."

| x keyOfs buckOfs |
x := self _lookup: aKey .
x _isSmallInteger
  ifTrue:[ keyOfs := x bitAnd: 16r3FFFFFFF . buckOfs := x bitShift: -30]
  ifFalse:[ x ifNil:[ ^ defaultValue ].
            keyOfs := x at: 1 . buckOfs := x at: 2 ].
buckOfs ~~ 0 ifTrue:[ | collisionBkt |
  collisionBkt := self _at: keyOfs + 1 .
  ^ collisionBkt _at: buckOfs + 1 .
].
^ self _at: keyOfs + 1.
%

category: 'Updating'
method: StringKeyValueDictionary
at: aKey put: aValue

"Stores the aKey/aValue pair in the receiver.  Rebuilds the hash table if the
 addition caused the number of collisions to exceed the limit allowed.
 Returns aValue.  Uses code point comparison.

 If aKey is being added for the first time, an invariant copy of it is stored
 as the key.

 Gs64 v3.0 - behavior changed to match method comments.
 Previous versions made aKey invariant without making a copy of it, which
 could signal authorization errors or create unintended commit conflicts."

self _stringAt: aKey put: aValue .
(numCollisions > collisionLimit)
  ifTrue: [self rebuildTable: tableSize * 2].
^aValue
%

category: 'Private'
method: StringKeyValueDictionary
collisionBucketClass

"Returns the class of object to create when keys collide."

^ EqualityCollisionBucket
%

category: 'Hashing'
method: StringKeyValueDictionary
compareKey: key1 with: key2

"No longer used by implementation of StringKeyValueDictionary"

self shouldNotImplement: #compareKey:with:
%

category: 'Hashing'
method: StringKeyValueDictionary
hashFunction: aKey

"No longer used by implementation of StringKeyValueDictionary"

self shouldNotImplement: #hashFunction:
%

category: 'Accessing'
method: StringKeyValueDictionary
keyAt: aKey otherwise: defaultValue

"Returns the key that corresponds to aKey.  If no such key/value pair
 exists, returns the given alternate value. "

| x keyOfs buckOfs |
x := self _lookup: aKey .
x _isSmallInteger
  ifTrue:[ keyOfs := x bitAnd: 16r3FFFFFFF . buckOfs := x bitShift: -30 ]
  ifFalse:[ x ifNil:[ ^ defaultValue ].
            keyOfs := x at: 1 . buckOfs := x at: 2 ].
buckOfs ~~ 0 ifTrue:[ | collisionBkt |
  collisionBkt := self _at: keyOfs + 1 .
  ^ collisionBkt _at: buckOfs
].
^ self _at: keyOfs .
%

category: 'Accessing'
method: StringKeyValueDictionary
name
"Returns the key of a key/value pair whose value is the receiver.
 If the receiver contains no such Association, returns nil."

| hashKey hashValue collisionBkt |
1 to: tableSize do: [ :tableIndex |
  hashKey := self keyAt: tableIndex.
  hashValue := self valueAt: tableIndex.
  (hashKey == nil)
    ifTrue: [ collisionBkt := hashValue.
              (collisionBkt == nil)
                 ifFalse: [ 1 to: collisionBkt numElements do: [ :i |
                            ((collisionBkt valueAt: i) == self)
                              ifTrue: [ ^collisionBkt keyAt: i ] ] ] ]
    ifFalse: [ (hashValue == self)
                  ifTrue: [ ^hashKey ] ]
  ].
^nil
%

category: 'Formatting'
method: StringKeyValueDictionary
printNonRecursiveRepresentationOn: aStream recursionSet: anIdentitySet

  self name ifNotNil:[:nam |
    aStream nextPutAll: self class name describeClassName .
    aStream nextPutAll: '( name: ';
            nextPutAll: nam printString ;
            nextPutAll: ' )' .
 ] ifNil:[
   super printNonRecursiveRepresentationOn: aStream recursionSet: anIdentitySet
 ]
%

category: 'Removing'
method: StringKeyValueDictionary
removeKey: aKey ifAbsent: aBlock

"Removes the key/value pair with key aKey from the receiver and returns
 the value.  If no key/value pair is present with key aKey, evaluates
 the zero-argument block aBlock and returns the result of that evaluation."

| x keyOfs buckOfs oldValue |
x := self _lookup: aKey .
x _isSmallInteger
  ifTrue:[ keyOfs := x bitAnd: 16r3FFFFFFF . buckOfs := x bitShift: -30]
  ifFalse:[ aKey ifNil:[ self _error: #rtErrNilKey].
            x ifNil:[ ^ aBlock value ].
            keyOfs := x at: 1 . buckOfs := x at: 2 ].

buckOfs ~~ 0 ifTrue:[ | collisionBkt |
  collisionBkt := self _at: keyOfs + 1 .
  oldValue := collisionBkt _at: buckOfs + 1 .
  collisionBkt _removePairAt: buckOfs  .
] ifFalse:[ | vOfs |
  oldValue := self _at: (vOfs := keyOfs + 1) .
  self _at: keyOfs put: nil ; _at: vOfs put: nil
].
numElements := numElements - 1.
^ oldValue
%

category: 'Removing'
method: StringKeyValueDictionary
removeKey: aKey otherwise: defaultValue

  ^ self removeKey: aKey ifAbsent:[ defaultValue ]
%

category: 'Private'
method: StringKeyValueDictionary
_hashOfKey: aKey

"computes the hash of the given key, equivalent to String>>hash"
<primitive: 993>
^ self _primitiveFailed: #_hashOfKey: args: { aKey }
%

category: 'Accessing'
method: StringKeyValueDictionary
_lookup: aKey

"Performs a search of the receiver for the key aKey, using code point comparison.
 Returns a SmallInteger,  least significant 30 bits are unsigned key offset
                          higher 30 bits of are unsigned bucket offset,
 or an Array of two SmallIntegers { keyOffset . bucketOffset }.
 The offsets are 1 based, a zero offset means not found, or no bucket.
 A result of nil means the key was not found."
<primitive: 83>
aKey _validateKindOfClass: String .
self _primitiveFailed: #_lookup: args: { aKey }.
%

category: 'Removing'
method: StringKeyValueDictionary
_removeKey: aKey

"Removes the key/value pair with key aKey from the receiver ,
 without accessing the value of the key/value pair.
 If no key/value pair is present with key aKey, generates an error.
 Does not remove empty collision buckets."
| x keyOfs buckOfs |
x := self _lookup: aKey .
x _isSmallInteger
  ifTrue:[ keyOfs := x bitAnd: 16r3FFFFFFF . buckOfs := x bitShift: -30]
  ifFalse:[ x ifNil:[ ^ self _errorKeyNotFound: aKey ].
            keyOfs := x at: 1 . buckOfs := x at: 2 ].
buckOfs ~~ 0 ifTrue:[ | collisionBkt |
  collisionBkt := self _at: keyOfs + 1 .
  collisionBkt _removePairAt: buckOfs  .
] ifFalse:[
  self _at: keyOfs put: nil ; _at: keyOfs + 1 put: nil
].
numElements := numElements - 1.
^ self
%

category: 'Private'
method: StringKeyValueDictionary
_reportString
  | str |
  str := String new .
  self keysAndValuesDo:[:k :v |
   str add: k ; add: '  ' ; add: v asString ; lf .
  ].
  ^ str
%

category: 'Updating'
method: StringKeyValueDictionary
_stringAt: aKey put: aValue

"Stores the aKey/aValue pair in the hash dictionary, using code point comparison."

<primitive: 256>
self _primitiveFailed: #_stringAt:put: args: { aKey . aValue } .
self _uncontinuableError
%

! Class extensions for 'Symbol'

!		Class methods for 'Symbol'

removeallmethods Symbol
removeallclassmethods Symbol

category: 'Class Membership'
classmethod: Symbol
isSubclassOf: aClassHistoryOrClass

  (self _classHistoryIncludesIdentical: aClassHistoryOrClass) ifTrue:[ ^ true].
  ^ super isSubclassOf: aClassHistoryOrClass
%

category: 'Storing and Loading'
classmethod: Symbol
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into a new instance of the receiver.
 Returns the new instance."

| result inst theSize marker |

"since Symbols can't refer to other objects, the 'hasRead:' message
 may be sent after values have been filled in."

theSize := passiveObj readSize .
inst := String new: theSize .
marker := passiveObj objectPositionMarker .
passiveObj next: theSize bytesTo: inst .
passiveObj next .
result := inst asSymbol .
passiveObj hasRead: result marker: marker .
^ result
%

category: 'Instance Creation'
classmethod: Symbol
migrateNew

"Override default migrateNew behavior with #_basicNew."

^ self _basicNew
%

category: 'Instance Creation'
classmethod: Symbol
new

"Disallowed.  To create a new Symbol, use the class method withAll: instead."

self shouldNotImplement: #new
%

category: 'Instance Creation'
classmethod: Symbol
new: size

"Disallowed.  To create a new Symbol, use the class method withAll: instead."

self shouldNotImplement: #new:
%

category: 'Instance Creation'
classmethod: Symbol
withAll: aString

"Returns a canonical symbol that has the same Characters as aString.  Returns
 an existing canonical symbol if it is already in AllSymbols, or if it was
 created earlier in the current session.  Otherwise, creates and returns a new
 canonical symbol.

 The result is a Symbol, DoubleByteSymbol, or QuadByteSymbol as required
 to hold the largest code point in aString ."

<primitive: 300>
aString _isOneByteString ifTrue:[  "charSize 1 and not a ByteArray"
  ^ self _primitiveFailed: #withAll: args: { aString } .
].
aString stringCharSize == 0 ifTrue:[ "not charSize 2 or 4"
  aString _validateClass: CharacterCollection .
  ^ self withAll: aString asString
].
aString _error: #rtErrBadArgKind2 args: { String . DoubleByteString . QuadByteString }.
%

category: 'Instance Creation'
classmethod: Symbol
_basicNew
^ self shouldNotImplement: #_basicNew
%

category: 'Instance Creation'
classmethod: Symbol
_basicNew: aSize
^ self shouldNotImplement: #_basicNew:
%

category: 'Class Membership'
classmethod: Symbol
_classHistoryIncludesIdentical: aClass
  ^ aClass == Symbol or:[ aClass == DoubleByteSymbol or:[ aClass == QuadByteSymbol]]
%

category: 'Instance Creation'
classmethod: Symbol
_existingWithAll: aString

"Returns an existing canonical symbol that has the same value as 'aString'.
 If no such Symbol, DoubleByteSymbol or QuadByteSymbol already exists, returns nil."

"This method is useful in processing Strings that are supposed to be names
 of already existing instance variables, such as in indexing and sorting."

<primitive: 310>
aString _validateClass: CharacterCollection .
^ self _existingWithAll: aString asString .
%

category: 'Converting'
classmethod: Symbol
_keywords: aSymbol
| result str |
result := { } .
str := String new .
1 to: aSymbol size do:[:n | | ch |
  ch := aSymbol at: n .
  ch == $:  ifTrue:[
    str add: $: .  result add: str . str := String new .
  ] ifFalse:[
    str add: ch .
  ].
].
str size ~~ 0 ifTrue:[ result add: str ].
^ result
%

category: 'Instance Creation'
classmethod: Symbol
_newString

"Returns a new instance of String."

^ String new
%

category: 'Instance Creation'
classmethod: Symbol
_newString: aSize

"Returns a new instance of String of the specified size."

^ String new: aSize
%

category: 'Private'
classmethod: Symbol
_validateNewSymbols: newSymbols  allSymbols: allSymbols
newSymbols keysDo:[ :aSym | | val |
  val := allSymbols at: aSym ifAbsent:[
    (Globals at: #GsFile) gciLogServer:'symbol ', aSym ,
          ', oop ' , aSym asOop asString , ' not in AllSymbols' .
    nil pause .
  ].
  val == aSym ifFalse:[
    (Globals at: #GsFile) gciLogServer:'bad symbol identity', aSym ,
		', oop ' , aSym asOop  asString.
    nil pause .
  ].
].
^ true
%

!		Instance methods for 'Symbol'

category: 'Concatenating'
method: Symbol
, aCharOrCharCollection

"Returns a new instance of String that contains the elements of the receiver
 followed by the elements of aCharOrCharCollection.  A String is returned rather
 than a Symbol to avoid the expense of unnecessary creation and canonicalization
 of Symbols."

| result |

result := String withAll: self .
result  addAll: aCharOrCharCollection .
^ result
%

category: 'Converting'
method: Symbol
-> anObject

"Returns a SymbolAssociation with the receiver as the key and the given object
 as the value."

^ SymbolAssociation newWithKey: self value: anObject
%

category: 'Comparing'
method: Symbol
= aCharCollection

"Returns true if and only if the receiver and aCharCollection are identical,
 unless aCharCollection is an instance of ObsoleteSymbol.  In that case, this
 method returns true if the receiver and aCharCollection are equivalent (in the
 sense of the String | = method).  Returns false otherwise."

<primitive: 295>
self _primitiveFailed: #= .
self _uncontinuableError
%

category: 'Compatibility'
method: Symbol
argumentCount

 ^ self numArgs
%

category: 'Formatting'
method: Symbol
asString

"Returns a copy of the receiver as an instance of class String."

^ String withAll: self
%

category: 'Converting'
method: Symbol
asSymbol

"Returns the receiver."

^ self
%

category: 'Converting'
method: Symbol
asSymbolKind

"Equivalent to asSymbol."

 ^self
%

category: 'Clustering'
method: Symbol
cluster

"Has no effect.  Clustering of Symbols is only performed by
 the clusterAllSymbols method in class System . "

^ true
%

category: 'Clustering'
method: Symbol
clusterInBucket: aClusterBucketOrId

"Has no effect.  Clustering of Symbols is only performed by
 the clusterAllSymbols method in class System . "

^ true
%

category: 'Copying'
method: Symbol
copy

"Returns self, copies of canonical symbols are not allowed."

^ self
%

category: 'Copying'
method: Symbol
copyReplacing: oldObject withObject: newObject
	"Returns a String comprising a copy of the receiver in which all occurrences
	 of objects equal to oldObject have been replaced by newObject."

	^self asString copyReplacing: oldObject withObject: newObject
%

category: 'Comparing'
method: Symbol
hash

"Returns a numeric hash key for the receiver.
 This implementation for new databases only and is
 replaced in the upgrade scripts."

^ self identityHash
%

category: 'Converting'
method: Symbol
keywords

"Returns an Array of the keywords in the receiver, treating
 any colon-delimited segment as if it was a legal keyword."

^ Symbol _keywords: self
%

category: 'Testing'
method: Symbol
precedence

"Returns the precedence of the receiver, were it a message selector, with
 1=unary, 2=binary and 3=keyword."

^self isInfix
    ifTrue: [ 2 ]
    ifFalse: [ self isKeyword ifTrue: [ 3 ]
                              ifFalse: [ 1 ] ]
%

category: 'Formatting'
method: Symbol
printOn: aStream

"Puts a displayable representation of the receiver on the given stream.
 That representation conforms to GemStone Smalltalk parsing rules."

aStream nextPut: $# .
super printOn: aStream
%

category: 'Copying'
method: Symbol
shallowCopy

"Returns self, copies of canonical symbols are not allowed."

^ self
%

category: 'Class Membership'
method: Symbol
species

"Returns a class similar to, or the same as, the receiver's class which
 can be used for containing derived copies of the receiver."

^ String
%

category: 'Private'
method: Symbol
speciesForConversion

"Returns String."

^ String.
%

category: 'Formatting'
method: Symbol
withNoColons

"Returns a String containing the value of the receiver with all colons removed.

 A String is returned rather than a Symbol to avoid the expense of unnecessary
 creation and canonicalization of Symbols."

^ self copyWithout: $:   .
%

category: 'Decompiling without Sources'
method: Symbol
_asSource

| result |
result := String new .
result add: $#  .
result addAll: super _asSource .
^ result
%

category: 'New Indexing Comparison'
method: Symbol
_idxForCompareEqualTo: arg

  ^arg _idxForCompareEqualToSymbol: self
%

category: 'New Indexing Comparison'
method: Symbol
_idxForCompareEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ aCharacterCollection _idxForCompareEqualToSymbol: self
%

category: 'New Indexing Comparison - for Compare'
method: Symbol
_idxForCompareEqualToCharacterCollection: aCharacterCollection
  "second half of a double dispatch call from CharacterCollection>>_idxForCompareEqualTo:."

  ^ false
%

category: 'New Indexing Comparison - for Compare'
method: Symbol
_idxForCompareEqualToUnicode: aUnicodeString collator: aCollator
"second half of a double dispatch call from CharacterCollection>>_idxForCompareEqualTo:collator:."

  ^ false
%

category: 'New Indexing Comparison'
method: Symbol
_idxForSortEqualTo: arg
  ""

  ^ arg _idxForSortEqualToSymbol: self
%

category: 'New Indexing Comparison'
method: Symbol
_idxForSortEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ self asString
    _idxForSortEqualTo: aCharacterCollection
    collator: anIcuCollator
%

category: 'New Indexing Comparison'
method: Symbol
_idxForSortEqualToSymbol: aSymbol
  "second half of a double dispatch call from Symbol>>_idxForSortEqualTo:."

  ^ self asString _idxPrimCompareEqualTo: aSymbol asString
%

category: 'New Indexing Comparison'
method: Symbol
_idxForSortNotEqualTo: aCharacterCollection collator: anIcuCollator
  ""

  ^ (self _idxForSortEqualTo: aCharacterCollection collator: anIcuCollator) not
%

category: 'Comparing'
method: Symbol
~= aCharCollection

"This method can be optimized for Symbols since they are canonical."

^ self ~~ aCharCollection
%

! Class extensions for 'SymbolAssociation'

!		Instance methods for 'SymbolAssociation'

removeallmethods SymbolAssociation
removeallclassmethods SymbolAssociation

category: 'Accessing'
method: SymbolAssociation
key: aKey

"Sets the object aKey as the key of the receiver, after checking
 constraints."

 (aKey _isSymbol ) ifTrue:[
   key := aKey .
 ] ifFalse:[
   ArgumentTypeError new constrainedIv: 'SymbolAssociation.key'
	  expectedClass: Symbol actualArg: aKey ;
     signal
 ]
%

category: 'Updating'
method: SymbolAssociation
key: aKey value: aValue


"Sets the object aKey as the key of the receiver, and the object aValue as the
 value of the receiver."

 self key: aKey .  "message send to enforce constraints"
 self value: aValue .
%

category: 'Private'
method: SymbolAssociation
_resolveRcConflictsWith: conflictObjects

"A logical write-write conflict has occurred on the receiver.  This
 may have occurred due to artificially placing the receiver in the write
 set when a subclass re-implemented this method (this is done to maintain
 cache coherency. See if the state has changed in this transaction.  If
 so, then it is a valid conflict (and returns false); otherwise returns true."

| preAbortValue |
preAbortValue := value.
self _selectiveAbort.
^ value == preAbortValue
%

! Class extensions for 'SymbolDictionary'

!		Class methods for 'SymbolDictionary'

removeallmethods SymbolDictionary
removeallclassmethods SymbolDictionary

category: 'Browser Methods'
classmethod: SymbolDictionary
_listClassesIn: dictArg matching: aString

"Returns a String containing names of classes, one per line; or false.
 If dictArg == nil, list the whole symbolList .
 If aString ~~ nil, limit result to those classes for which
    (aClass name includesString: aString)==true .
 If aDict is not a kind of SymbolDictionary, returns false.
 Used by Topaz."
| list result |
dictArg  ifNil:[ list := GsSession currentSession symbolList ]
      ifNotNil:[ (dictArg isKindOf: self ) ifFalse:[ ^ false ].
                 list := { dictArg } ].
result := String new .
list do:[:aDict | | aSet sortedNames |
  aSet := SymbolSet new .
  aDict associationsDo:[:assoc |
    (assoc _value isKindOf:Class) ifTrue:[ aSet add: assoc key ].
  ].
  aString ifNil:[
    result size > 0 ifTrue:[ result lf ].
    result addAll: 'dictionary ' ;
           addAll: (aDict name ifNil:[ '(unnamed)' ]) .
    aSet size == 0 ifTrue:[ result addAll:' is empty' ].
    result lf .
  ] ifNotNil:[
    aSet := aSet select:[:n | n includesString: aString]
  ].
  sortedNames := SortedCollection withAll: aSet .
  1 to: sortedNames size do:[:j |
    result addAll: (sortedNames at: j); lf .
  ].
].
^ result
%

!		Instance methods for 'SymbolDictionary'

category: 'CodeModification Override'
method: SymbolDictionary
addAll:  aCollection

(aCollection isKindOf: AbstractDictionary)
ifTrue: [
    aCollection associationsDo: [:x |
        (self _validatePrivilegeOld: (self at: x key otherwise: nil)
                               new: x _value) ifFalse:[ ^ nil ] ]
] ifFalse: [
    aCollection do: [:x |
        (x _validateClass: Association) ifFalse:[ ^ nil ].
        (self _validatePrivilegeOld: (self at: x key otherwise: nil)
                               new: x _value) ifFalse:[ ^ nil ] ]
].
^ super addAll: aCollection
%

category: 'Updating'
method: SymbolDictionary
addAssociation: aSymbolAssociation

"Add the argument to the receiver."

(aSymbolAssociation isKindOf: SymbolAssociation) ifFalse:[
  aSymbolAssociation _validateClass: SymbolAssociation
].
(self _validatePrivilegeOld: (self at: aSymbolAssociation key otherwise: nil)
                       new: aSymbolAssociation _value) ifTrue:[
  ^ self _at: aSymbolAssociation key put: aSymbolAssociation
].
^ nil
%

category: 'Accessing'
method: SymbolDictionary
associationAt: aKey

"Returns the SymbolAssociation with key aKey.  Generates an error if
 no such SymbolAssociation exists."

| anAssoc |

anAssoc :=  self associationAt: aKey otherwise: nil.
anAssoc == nil ifTrue:[ anAssoc := self _errorKeyNotFound: aKey] .
^ anAssoc
%

category: 'Accessing'
method: SymbolDictionary
associationAt: aKey ifAbsent: aBlock

"Returns the SymbolAssociation with key aKey.  If no such SymbolAssociation
 exists, returns the result of evaluating the zero-argument block aBlock."

<primitive: 57>
| aSym |
aKey _isSymbol ifFalse:[
  (aSym := Symbol _existingWithAll: aKey ) ifNotNil:[
    ^ self associationAt: aSym ifAbsent: aBlock .
  ].
].
^ aBlock ifNotNil:[ aBlock value ].
%

category: 'Accessing'
method: SymbolDictionary
associationAt: aKey otherwise: defaultValue

"Returns the SymbolAssociation with key aKey.  If no such SymbolAssociation
 exists, returns the given default value."

<primitive: 57>

| aSym |
aKey _isSymbol ifFalse:[
  (aSym := Symbol _existingWithAll: aKey ) ifNotNil:[
    ^ self associationAt: aSym otherwise: defaultValue
  ].
].
^ defaultValue
%

category: 'Accessing'
method: SymbolDictionary
at: aKey

"Returns the value at the given key.  Generates an error if aKey not found."

| assoc |
assoc := self associationAt: aKey otherwise: nil .
assoc == nil ifTrue:[ ^ self _errorKeyNotFound: aKey ].
^ assoc _value
%

category: 'Accessing'
method: SymbolDictionary
at: aKey ifAbsent: aBlock

"Returns the value of the SymbolAssociation with key aKey.  If no such
 SymbolAssociation exists, returns the result of evaluating the
 zero-argument block aBlock."

<primitive: 240>
| aSym |
aKey _isSymbol ifFalse:[
  aKey ifNotNil:[
    aSym := Symbol _existingWithAll: aKey .
    aSym ifNotNil:[
      ^ self at: aSym ifAbsent: aBlock .
    ]
  ].
].
aBlock ifNil:[ ^ nil ] .
^ aBlock value
%

category: 'Accessing'
method: SymbolDictionary
at: aKey otherwise: defaultValue

"Returns the value at the given key.  If aKey is not found, returns
 defaultValue."

| assoc |
assoc := self associationAt: aKey otherwise: nil .
assoc == nil ifTrue:[ ^ defaultValue ].
^ assoc _value
%

category: 'Updating'
method: SymbolDictionary
at: aKey put: aValue

"If the receiver already contains a SymbolAssociation with the given key, this
 makes aValue the value of that SymbolAssociation.  Otherwise, this creates a
 new SymbolAssociation with the given key and value and adds it to the
 receiver.  aKey must be a Symbol.   Returns aValue."

| anAssoc |
(self _validatePrivilegeOld: (self at: aKey otherwise: nil) new: aValue) ifTrue:[
  anAssoc:= self associationAt: aKey otherwise: nil .
  anAssoc == nil ifTrue:[
       self _at: aKey put:
       (SymbolAssociation newWithKey: aKey value: aValue).
       ^aValue
  ].

  tableSize := tableSize.  "make sure SymbolDictionary is dirty, not just Association (#42383)"
  anAssoc value: aValue.
  ^aValue
].
%

category: 'Updating'
method: SymbolDictionary
atHash: hashIndex putKey: aKey

"Updates the hash table by storing aKey at the specified hashIndex."

aKey _isSymbol ifFalse:[
  aKey ifNotNil:[ aKey _errorExpectedClass: Symbol ].
].
^super atHash: hashIndex putKey: aKey.
%

category: 'Updating'
method: SymbolDictionary
atHash: hashIndex putKey: aKey value: aValue

aKey _isSymbol ifFalse:[
  aKey ifNotNil:[ aKey _errorExpectedClass: Symbol ].
].
self _validateAtHash: hashIndex putValue: aValue .

^ super atHash: hashIndex putKey: aKey value: aValue
%

category: 'Updating'
method: SymbolDictionary
atHash: hashIndex putValue: aValue

"Updates the hash table by storing aValue at the specified hashIndex."

self _validateAtHash: hashIndex putValue: aValue .
^ super atHash: hashIndex putValue: aValue.
%

category: 'Enumerating'
method: SymbolDictionary
classesDo: aBlock

"Execute 3-argument block over all classes (plus those referenced by
 its class history) that are contained by this SymbolDictionary.

 Arguments to block are:
    [:aUserProfile :aSymbolDictionary :aClass | ... ]

"

  | coveredClasses |

  coveredClasses := IdentitySet new.
  self valuesDo: [ :namedClass |
    (namedClass isKindOf: Class) ifTrue: [
      namedClass classHistory do: [ :aClass |
        (coveredClasses includes: aClass) ifFalse: [
          coveredClasses add: aClass.
          aBlock value: (System myUserProfile)
                 value: self
                 value: aClass ]]]]
%

category: 'Deprecated'
method: SymbolDictionary
detectValues: aBlock ifNone: exceptionBlock

self deprecated: 'SymbolDictionary>>detectValues:ifNone: deprecated long before v3.0. Use keysAndValuesDo: instead.'.

"Evaluates aBlock repeatedly, with values of the receiver as the argument.
 Returns the first key for which aBlock evaluates to true.  If none of the
 receiver's values evaluates to true, evaluates the argument exceptionBlock and
 returns its value.  The argument aBlock must be a one-argument block, and
 exceptionBlock must be a zero-argument block."

self keysAndValuesDo: [ :aKey :aValue |
  (aBlock value: aValue) ifTrue: [ ^ aKey ].
  ].
^ exceptionBlock value.
%

category: 'Searching'
method: SymbolDictionary
includesKey: aKey

"Reimplemented from KeyValueDictionary for efficiency."

^ (self associationAt: aKey otherwise: nil ) ~~ nil
%

category: 'Accessing'
method: SymbolDictionary
keys

"Returns a SymbolSet containing the receiver's keys."

| result |
result := SymbolSet new .
self keysDo:[ :aKey | result add: aKey ].
^ result
%

category: 'Accessing'
method: SymbolDictionary
name

"Returns the key of a SymbolAssociation whose value is the receiver.  If the
 receiver contains no such SymbolAssociation, returns nil."

^ self keyAtValue: self ifAbsent:[ nil ]
%

category: 'Accessing'
method: SymbolDictionary
name: aSymbol

"Equivalent to self at: aSymbol put: self."

^ self at: aSymbol put: self
%

category: 'Accessing'
method: SymbolDictionary
names

"Returns an Array that contains all the keys of entries in the receiver whose
 value is the receiver itself.  The order of the elements in the result is
 arbitrary.  If no such keys are found, returns an empty Array."

| result |
result := { }  .
self keysAndValuesDo:[:aKey :aValue |
  aValue == self ifTrue:[ result add: aKey ].
  ].
^ result
%

category: 'Formatting'
method: SymbolDictionary
printNonRecursiveRepresentationOn: aStream recursionSet: anIdentitySet
  
  self name ifNotNil:[:nam |
    aStream nextPutAll: self class name describeClassName .
    aStream nextPutAll: '( name: ';
            nextPutAll: nam printString ; 
            nextPutAll: ' )' .
 ] ifNil:[
   super printNonRecursiveRepresentationOn: aStream recursionSet: anIdentitySet
 ]
%

category: 'CodeModification Override'
method: SymbolDictionary
rebuildTable: newSize

<primitive: 2001>  "enter protected mode, to disable CodeModification checks"
| prot |
prot := System _protectedMode .
[
  super rebuildTable: newSize.
] ensure:[
  prot _leaveProtectedMode
]
%

category: 'CodeModification Override'
method: SymbolDictionary
removeAssociation: anAssociation

(self _validatePrivilegeOld: (self at: anAssociation key otherwise: nil)
                       new: anAssociation _value ) ifTrue:[
  ^ super removeAssociation: anAssociation
]
%

category: 'CodeModification Override'
method: SymbolDictionary
removeAssociation: anAssociation ifAbsent: aBlock

(self _validatePrivilegeOld: (self at: anAssociation key otherwise: nil)
                       new: anAssociation _value) ifTrue:[
  ^ super removeAssociation: anAssociation ifAbsent: aBlock
]
%

category: 'CodeModification Override'
method: SymbolDictionary
removeKey: aKey

(self _validatePrivilegeOld: ( self at: aKey otherwise: nil )
                       new: ( self at: aKey otherwise: nil )) ifTrue:[
  ^ super removeKey: aKey
]
%

category: 'CodeModification Override'
method: SymbolDictionary
removeKey: aKey ifAbsent: aBlock

(self _validatePrivilegeOld: ( self at: aKey otherwise: nil )
                       new: ( self at: aKey otherwise: nil )) ifTrue:[
  ^ super removeKey: aKey ifAbsent: aBlock
]
%

category: 'Updating'
method: SymbolDictionary
renameAssociationFrom: key1 to: key2

"Look up the Association in the receiver that has key1, and change its
 key to key2.  Raises an error if key1 is not found, or if key2 already
 exists.  key1 and key2 must be Symbols."

| assoc |
key1 _validateClass: Symbol.
key2 _validateClass: Symbol.
assoc := self associationAt: key1 .
(self _validatePrivilegeOld: assoc _value new: assoc _value ) ifTrue:[
  (self associationAt: key2 otherwise: nil) ~~ nil ifTrue:[
    ^ self _error: #rtErrDuplicateKey args: { key2 }.
  ].
  self removeKey: key1 .
  assoc key: key2 .
  self addAssociation: assoc .
]
%

category: 'CodeModification Override'
method: SymbolDictionary
squeakBasicAt: anIndex put: aValue

^ self _basicAt: anIndex put: aValue
%

category: 'Updating'
method: SymbolDictionary
swapKey: key1 with: key2

"In the receiver, look up the Associations for key1 and key2 and
 swap the keys of the two Associations.  Returns the receiver.
 If either key1 or key2 is not found in the receiver, raises an error."

| assoc1 assoc2 |

key1 _validateClass: Symbol.
key2 _validateClass: Symbol.
assoc1 := self associationAt: key1 .
assoc2 := self associationAt: key2 .
(self _validatePrivilegeOld: assoc1 _value new: assoc1 _value) ifTrue:[
  (self _validatePrivilegeOld: assoc2 _value new: assoc2 _value) ifTrue:[
    self removeKey: key1 .
    self removeKey: key2 .
    assoc1 key: key2 .
    assoc2 key: key1 .
    self addAssociation: assoc1 .
    self addAssociation: assoc2
  ]
]
%

category: 'Evaluation'
method: SymbolDictionary
textForError: aNumber args: anArray

self deprecated: 'SymbolDictionary>>textForError:args: deprecated v3.3. Use ANSI API in
AbstractException'.

"This code no longer functions correctly and is not used in the server image."

^ [
    | descrsArray errDescriptor myLanguage msg arrSize bad |
    myLanguage := System myUserProfile nativeLanguage.
    descrsArray := self at: myLanguage otherwise: nil .
    descrsArray ifNil:[ ^ self _errorKeyNotFound: myLanguage].
    errDescriptor := descrsArray atOrNil: aNumber .
    errDescriptor ifNil:[  | exCls |
      exCls := AbstractException legacyNumberToClass: aNumber .
      (msg := String withAll:'a ') add: exCls name ; add: ' occurred'.
    ] ifNotNil:[
      msg := String new.
      arrSize := anArray size .
      errDescriptor do: [ :arg |
	(arg isKindOf: String) ifTrue: [
	    msg addAll: arg
	  ] ifFalse: [
	    (arg _isSmallInteger and:[ arg >= 1 and:[ arg <= arrSize]]) ifTrue: [
		msg addAll:  (anArray at: arg) asString
	    ] ifFalse: [
		msg addAll: '???'.  "peculiar error list"
		bad := true
	    ]
	  ]
	].
      bad ifNotNil:[
	msg addAll: ' , for error number ' , aNumber asString
      ].
    ].
    msg
  ] onException: Error do:[:ex |
    ^ [
         ex return: 'Error building error string for ' , ex category asString , ':' ,
                     ex number asString
      ] onSynchronous: AbstractException do:[:exb |
         exb return: 'Error building error for uninitialized Exception'
      ]
  ].
%

category: 'CodeModification Override'
method: SymbolDictionary
_at: aKey put: anObject

(self _validatePrivilegeGeneric: anObject) ifTrue:[
  ^ super _at: aKey put: anObject
]
%

category: 'CodeModification Override'
method: SymbolDictionary
_basicAt: anInteger put: anObject

(self _validatePrivilegeGeneric: anObject) ifTrue:[
  ^ super _basicAt: anInteger put: anObject
]
%

category: 'Accessing'
method: SymbolDictionary
_behaviorKeys

"Returns a SymbolSet containing keys in the receiver whose values are
 Behaviors."

"This is used as an optimization by the GemBuilder for Smalltalk browser."

| result |
result := SymbolSet new .
self keysAndValuesDo:[ :aKey :aValue |
  aValue isBehavior ifTrue: [ result add: aKey ]].
^ result
%

category: 'Browser Methods'
method: SymbolDictionary
_classAndVersionStrings

"For all Behaviors in the receiver, returns an OrderedCollection of Strings
 showing the class name and version.  This method is used as an optimization by
 the GemBuilder for Smalltalk browser."

| result |
result := OrderedCollection new .
self associationsDo: [ :anAssoc | | each |
  each := anAssoc _value.
  each isBehavior
  ifTrue: [ result add:
              ( each classHistory size == 1
                 ifTrue: [ each name asString ]
                 ifFalse: [ each name , ' [ ' ,
                      ( each classHistory indexOf: each ) printString , ' ]' ]
               )
    ]
  ].
^result
%

category: 'CodeModification Override'
method: SymbolDictionary
_primitiveAt: anInteger put: anObject

(self _validatePrivilegeGeneric: anObject) ifTrue:[
  ^ super _primitiveAt: anInteger put: anObject
]
%

category: 'CodeModification Override'
method: SymbolDictionary
_unsafeAt: anInteger put: anObject

(self _validatePrivilegeGeneric: anObject) ifTrue:[
  ^ super _unsafeAt: anInteger put: anObject
]
%

category: 'Updating'
method: SymbolDictionary
_validateAtHash: hashIndex putValue: aValue

| valueCls |
(self _validatePrivilegeOld: (self valueAtHash: hashIndex) new: aValue) ifTrue:[
 valueCls := aValue class .
 (valueCls == SymbolAssociation) ifFalse:[
  (valueCls == CollisionBucket) ifFalse:[
    aValue ifNotNil:[
      (aValue isKindOf: SymbolAssociation) ifFalse:[
        (aValue isKindOf: CollisionBucket) ifFalse:[
          aValue _error: #rtErrInvalidArgClass args:
              { SymbolAssociation . CollisionBucket . UndefinedObject }
          ]
        ]
      ].
    ].
  ].
]
%

category: 'CodeModification Override'
method: SymbolDictionary
_validatePrivilegeGeneric: newValue

" This version covers any weird arguments being passed
  in primitive _at:put: methods and variations "

System _inProtectedMode
    ifTrue: [
      "We go into protected mode when we rebuildTable:, do we don't need to worry about privileges"
      ^ true
    ].

( newValue isKindOf: Behavior ) ifTrue: [
   ^ System myUserProfile _validateCodeModificationPrivilege
   ].

( newValue isKindOf: Association ) ifTrue: [
    ( newValue _value isKindOf: Behavior ) ifTrue: [
        ^ System myUserProfile _validateCodeModificationPrivilege
       ]
    ].

( newValue isKindOf: AbstractCollisionBucket ) ifTrue: [
    1 to: newValue size do: [ :i |
      (( newValue valueAt: i ) isKindOf: Behavior ) ifTrue: [
	^ System myUserProfile _validateCodeModificationPrivilege
	].
      ].
    ].
^ true
%

category: 'CodeModification Override'
method: SymbolDictionary
_validatePrivilegeOld: oldValue new: newValue

( oldValue isKindOf: Behavior ) ifTrue: [
  ^ System myUserProfile _validateCodeModificationPrivilege
  ].
( newValue isKindOf: Behavior ) ifTrue: [
  ^ System myUserProfile _validateCodeModificationPrivilege
  ].
^ true
%

! Class extensions for 'SymbolKeyValueDictionary'

!		Instance methods for 'SymbolKeyValueDictionary'

removeallmethods SymbolKeyValueDictionary
removeallclassmethods SymbolKeyValueDictionary

category: 'Accessing'
method: SymbolKeyValueDictionary
at: aKey ifAbsent: aBlock
  | sym |
  aKey ifNotNil:[ sym := Symbol _existingWithAll: aKey ].
  sym ifNil:[
    aBlock ifNil:[^ self _errorKeyNotFound: aKey ] .
    ^ aBlock value
  ] ifNotNil:[
    ^ super at: sym ifAbsent: aBlock
  ]
%

category: 'Accessing'
method: SymbolKeyValueDictionary
at: aKey otherwise: aValue

  (Symbol _existingWithAll: aKey)
     ifNil:[ ^ aValue ]
     ifNotNil:[ :sym | ^ super at: sym otherwise: aValue ]
%

category: 'Updating'
method: SymbolKeyValueDictionary
at: aKey put: aValue

"Stores the aKey/aValue pair in the hash dictionary.
 aKey must be convertible to a Symbol.

 Rebuilds the hash table if the addition caused the number of collisions
 to exceed the limit allowed."

^ super at: (aKey asSymbol) put: aValue
%

category: 'Accessing'
method: SymbolKeyValueDictionary
keys

"Returns a SymbolSet containing the receiver's keys."

| result |
result := SymbolSet new .
self keysAndValuesDo:[ :aKey :aValue | result add: aKey ].
^ result
%

category: 'Removing'
method: SymbolKeyValueDictionary
removeKey: aKey ifAbsent: aBlock

  (Symbol _existingWithAll: aKey) ifNil:[
    aBlock ifNil:[^ self _errorKeyNotFound: aKey ] .
    ^ aBlock value
  ] ifNotNil:[ :sym |
    ^ super removeKey: sym ifAbsent: aBlock
  ]
%

category: 'Removing'
method: SymbolKeyValueDictionary
removeKey: aKey otherwise: aValue

  (Symbol _existingWithAll: aKey)
     ifNil:[ ^ aValue ]
     ifNotNil:[ :sym | ^ super removeKey: sym otherwise: aValue ]
%

! Class extensions for 'SymbolList'

!		Instance methods for 'SymbolList'

removeallmethods SymbolList
removeallclassmethods SymbolList

category: 'CodeModification Override'
method: SymbolList
, aSequenceableCollection

self _validatePrivilegeAdd: aSequenceableCollection .
^ super , aSequenceableCollection
%

category: 'CodeModification Override'
method: SymbolList
add: newObject

self _validatePrivilegeAdd: newObject .
self _validateElement: newObject .
^ super add: newObject
%

category: 'CodeModification Override'
method: SymbolList
add: newObject after: target

self _validatePrivilegeAdd: newObject .
self _validateElement: newObject .
^ super add: newObject after: target
%

category: 'CodeModification Override'
method: SymbolList
add: newObject before: target

self _validatePrivilegeAdd: newObject .
self _validateElement: newObject .
^ super add: newObject  before: target
%

category: 'CodeModification Override'
method: SymbolList
addAll: aCollection

self _validatePrivilegeAdd: aCollection .
aCollection accompaniedBy: self do:[:me :anObj |
	me _validateElement: anObj .
].
^ super addAll: aCollection
%

category: 'CodeModification Override'
method: SymbolList
addAll: aCollection afterIndex: index

self _validatePrivilegeAdd: aCollection .
aCollection accompaniedBy: self do:[:me :anObj |
	me _validateElement: anObj .
].
^ super addAll: aCollection afterIndex: index
%

category: 'CodeModification Override'
method: SymbolList
addAll: aCollection before: target

self _validatePrivilegeAdd: aCollection .
aCollection accompaniedBy: self do:[:me :anObj |
	me _validateElement: anObj .
].
^ super addAll: aCollection before: target .
%

category: 'CodeModification Override'
method: SymbolList
addAll: aCollection beforeIndex: index

self _validatePrivilegeAdd: aCollection .
aCollection accompaniedBy: self do:[:me :anObj |
	me _validateElement: anObj .
].
^ super addAll: aCollection beforeIndex: index
%

category: 'CodeModification Override'
method: SymbolList
addLast: newObject

self _validatePrivilegeAdd: newObject .
self _validateElement: newObject .
^ super addLast: newObject
%

category: 'CodeModification Override'
method: SymbolList
at: anIndex put: aValue

self _validatePrivilegeAdd: aValue .
self _validateElement: aValue .
^ super at: anIndex put: aValue
%

category: 'CodeModification Override'
method: SymbolList
atAllPut: anObject

self _validatePrivilegeAdd: anObject .
self _validateElement: anObject .
^ super atAllPut: anObject
%

category: 'Updating'
method: SymbolList
createDictionaryNamed: dictName at: anIndex

"Creates a new SymbolDictionary in the GsObjectSecurityPolicy of the receiver.  Adds to the
 dictionary an Association whose key is dictName and whose value is the
 dictionary.  Inserts the dictionary in the receiver at anIndex.

 If anIndex is less than 1, the dictionary is inserted at the front of the list.
 If anIndex is greater than the size of the list, it is inserted at the end of
 the list.  If the receiver already contains a dictionary named dictName, raises
 an error."

| newDict nameSym offset |

self _validatePrivilege .
nameSym := dictName asSymbol .
self do:[ :aDict |
(aDict names includesIdentical: nameSym) ifTrue:[
  aDict _error: #rtErrDuplicateKey args: { nameSym }.
  ].
].
(newDict := SymbolDictionary new) name: nameSym .
newDict objectSecurityPolicy: self objectSecurityPolicy .
offset := anIndex .
offset < 1 ifTrue:[ offset := 1 ].
offset > self size
ifTrue:[ self addLast: newDict ]
ifFalse:[ self insertObject: newDict at: offset ].
%

category: 'Searching'
method: SymbolList
dictionariesAndSymbolsOf: anObject

"Returns an Array, possibly empty, of the form { { aDictionary . aSymbol } ... }.
 Searches the receiver for Associations whose value is identical to
 anObject and returns information for all matching Associations. "

| res aDict |
res := { } .
1 to: self size do:[ :j |
  (aDict := self at: j) ifNotNil:[
    aDict associationsDo:[ :assoc |
      assoc _value == anObject ifTrue:[ res add: { aDict . assoc key }]
    ].
  ].
].
^ res
%

category: 'Searching'
method: SymbolList
dictionaryAndSymbolOf: anObject

"Returns an Array containing two elements for the first Association
 in the enumeration of the receiver for which value == anObject .

 * The Dictionary in the receiver that contains an Association whose
   value is anObject.
 * The Symbol which is that Association's key.

 The receiver is searched in the same order that the compiler searches it.
 (For more information about symbol resolution, see the GemStone Programming
 Guide.)  If anObject is not found in the receiver, returns nil."

| aDict |
self deprecated: 'SymbolList>>dictionaryAndSymbolOf: deprecated v3.1, use dictionariesAndSymbolsOf:' .
1 to: self size do:[ :j |
  (aDict := self at: j) ifNotNil:[
    aDict associationsDo:[ :assoc |
      assoc value == anObject ifTrue:[ ^ { aDict . assoc key } ].
    ].
  ].
].
^ nil
%

category: 'CodeModification Override'
method: SymbolList
first: anObject

self _validatePrivilegeAdd: anObject .
self _validateElement: anObject .
^ super first: anObject
%

category: 'CodeModification Override'
method: SymbolList
insertAll: aCollection at: anIndex

self _validatePrivilegeAdd: aCollection .
aCollection accompaniedBy: self do:[:me :anObj |
	me _validateElement: anObj .
].
^ super insertAll: aCollection at: anIndex
%

category: 'CodeModification Override'
method: SymbolList
insertObject: anObject at: anIndex

self _validatePrivilegeAdd: anObject .
self _validateElement: anObject .
^ super insertObject: anObject   at: anIndex
%

category: 'CodeModification Override'
method: SymbolList
last: anObject

self _validatePrivilegeAdd: anObject .
self _validateElement: anObject .
^ super last: anObject
%

category: 'Accessing'
method: SymbolList
names

"Returns an Array of Strings containing the names of the receiver's
 SymbolDictionaries.

 This method assumes that each SymbolDictionary in the receiver contains a
 SymbolAssociation whose value is that SymbolDictionary.  If any
 SymbolDictionary does not contain such a SymbolAssociation, it is
 represented in the result Array as '(unnamed Dictionary)'."

| result aDict aSymAssoc|
result:= { }  .
1 to: self size do:[:j|
  aDict := self at: j .
  aDict == nil
    ifTrue:[ result add: '(nil)' ]
    ifFalse:[
      aSymAssoc:= aDict associationsDetect:[:anAssoc | anAssoc value == aDict]
			ifNone:[ nil ].
      aSymAssoc == nil
        ifTrue:[ result add: '(unnamed Dictionary)' ]
        ifFalse:[ result add: (aSymAssoc key)]
      ].
  ].
^ result
%

category: 'Formatting'
method: SymbolList
namesReport

"Returns a formatted String describing the position and name of each Dictionary
 in the receiver's symbol list.

 This method assumes that each Dictionary in the symbol list contains an
 Association whose value is that Dictionary.  If any Dictionary does not
 contain such an Association, it is represented in the result String as
 '(unnamed Dictionary)'."

"used by the topaz list dictionaries command"

| arrayOfNames result lf |

arrayOfNames := self names .
result := String new .
lf := Character lf .
1 to: arrayOfNames size do:[:j|
  result add: (j asString).
  result addAll: '  '.
  result addAll: (arrayOfNames at: j) .
  result add: lf.
  ].
^result
%

category: 'Searching'
method: SymbolList
objectNamed: aSymbol

"Returns the first object in the receiver that has the given name.  If no object
 is found with the given name, returns nil."

| assn |

assn := self resolveSymbol: aSymbol.
assn == nil ifTrue: [ ^nil ].
^assn value
%

category: 'CodeModification Override'
method: SymbolList
protectedRemoveFrom: startIndex to: stopIndex

<protected>
^ super removeFrom: startIndex to: stopIndex
%

category: 'CodeModification Override'
method: SymbolList
remove: oldObject

self _validatePrivilege: oldObject .
^ super remove: oldObject
%

category: 'CodeModification Override'
method: SymbolList
remove: oldObject ifAbsent: anExceptionBlock

self _validatePrivilege: oldObject .
^ super remove: oldObject ifAbsent: anExceptionBlock
%

category: 'CodeModification Override'
method: SymbolList
removeAll: aCollection

self _validatePrivilege: aCollection .
^ super removeAll: aCollection
%

category: 'CodeModification Override'
method: SymbolList
removeAllIdentical: aCollection

self _validatePrivilege: aCollection .
^ super removeAllIdentical: aCollection
%

category: 'CodeModification Override'
method: SymbolList
removeAllSuchThat: aBlock

self _validatePrivilege: self .
^ super removeAllSuchThat: aBlock
%

category: 'CodeModification Override'
method: SymbolList
removeAtIndex: anIndex

self _validatePrivilege: self .
^ super removeAtIndex: anIndex
%

category: 'Updating'
method: SymbolList
removeDictionaryNamed: aSymbol

"Removes the first dictionary found in the receiver that contains an Association
 whose key is aSymbol and whose value is the dictionary.  Returns the removed
 dictionary.

 If no such dictionary is found, raises a KeyNotFound error."

self _validatePrivilege.
^ self removeDictionaryNamed: aSymbol ifAbsent:[
  self _error: #rtErrKeyNotFound args: { aSymbol }.
  nil
]
%

category: 'Updating'
method: SymbolList
removeDictionaryNamed: aString ifAbsent: aBlock

"Removes the first dictionary found in the receiver that contains an Association
 whose key is aSymbol and whose value is the dictionary.  Returns the removed
 dictionary.

 If no such dictionary is found, returns the result of evaluating the
 zero-argument Block aBlock."

self _validatePrivilege .
(Symbol _existingWithAll: aString) ifNotNil:[ :nameSym |
	1 to: self size do:[ :j | | aDict |
		aDict := self at: j .
		(aDict names includesIdentical: nameSym) ifTrue:[
		self removeAtIndex: j .
		^ aDict .
		].
	].
].
^ aBlock value
%

category: 'CodeModification Override'
method: SymbolList
removeFirst

self _validatePrivilege: self .
^ super removeFirst
%

category: 'CodeModification Override'
method: SymbolList
removeFrom: startIndex to: stopIndex

self _validatePrivilege: self .
^ super removeFrom: startIndex to: stopIndex
%

category: 'CodeModification Override'
method: SymbolList
removeIdentical: oldObject

self _validatePrivilege: oldObject .
^ super removeIdentical: oldObject
%

category: 'CodeModification Override'
method: SymbolList
removeIdentical: oldObject ifAbsent: anExceptionBlock

self _validatePrivilege: oldObject .
^ super removeIdentical: oldObject ifAbsent: anExceptionBlock
%

category: 'CodeModification Override'
method: SymbolList
removeLast

self _validatePrivilege: self .
^ super removeLast
%

category: 'Updating'
method: SymbolList
replaceElementsFrom: aSymbolList

"Removes all elements in the receiver and inserts all elements of aSymbolList in
 it, in the same order as in aSymbolList.

 If the argument is not a SymbolList, raises an error."

| newSize |
self _validatePrivilege: aSymbolList .
aSymbolList == self ifTrue:[ ^ self ].
(aSymbolList isKindOf: SymbolList) ifFalse:[
aSymbolList _validateClass: SymbolList .
^ self
].
self size: ( newSize := aSymbolList size) .
self replaceFrom: 1 to: newSize with: aSymbolList startingAt: 1
%

category: 'CodeModification Override'
method: SymbolList
replaceFrom: startIndex to: stopIndex with: aCollection

self _validatePrivilege: self .
self _validatePrivilegeAdd: aCollection .
aCollection accompaniedBy: self do:[:me :anObj |
	me _validateElement: anObj.
].
^ super replaceFrom: startIndex to: stopIndex with: aCollection
%

category: 'CodeModification Override'
method: SymbolList
replaceFrom: startIndex to: stopIndex with: aSeqCollection startingAt: repIndex

| count |
self _validatePrivilege: self .
self _validatePrivilegeAdd: aSeqCollection .
count := stopIndex - startIndex + 1 .
0 to: count -1 do:[:j |
	self _validateElement:( aSeqCollection at: repIndex + j) .
].
^ super replaceFrom: startIndex to: stopIndex with: aSeqCollection startingAt: repIndex
%

category: 'CodeModification Override'
method: SymbolList
replaceFrom: startIndex to: stopIndex withObject: anObject

self _validatePrivilege: self .
self _validatePrivilegeAdd: anObject .
self _validateElement: anObject .
^ super replaceFrom: startIndex to: stopIndex withObject: anObject
%

category: 'Searching'
method: SymbolList
resolveSymbol: aString

"Searches the receiver for an Association whose key is equal to aString, and
 returns that Association.  If no such Association is found, returns nil."

<primitive: 49>

(aString _isSymbol) ifFalse:[
  ^ (Symbol _existingWithAll: aString) ifNotNil:[ :sym | self resolveSymbol: sym]
].
aString _validateClasses: { String }.
self _primitiveFailed: #resolveSymbol: args: { aString } .
self _uncontinuableError
%

category: 'CodeModification Override'
method: SymbolList
size: anInteger

self _validatePrivilege: self .
^ super size: anInteger
%

category: 'Class Membership'
method: SymbolList
species

"Returns a class, an instance of which should be used as the result of
 select: or other projections applied to the receiver."

^ Array
%

category: 'CodeModification Override'
method: SymbolList
squeakBasicAt: anIndex put: aValue

^ self _basicAt: anIndex put: aValue
%

category: 'Searching'
method: SymbolList
symbolResolutionOf: aString

"Searches the receiver for aString.  If aString is found, returns a formatted
 String that describes the position in the receiver of the Dictionary defining
 aString, the name of that Dictionary, and aString.

 Generates an error if aString is not defined in the receiver."

"Example: GsSession currentSession symbolList symbolResolutionOf: #Integer"

| lf result theAssoc aDict aSymbol |

aSymbol := Symbol _existingWithAll: aString.
aSymbol == nil ifTrue:[ self _error: #rtErrSymNotFound args: { aString } ].

result := String new.
lf := Character lf.
1 to: self size do:[ :j|
  aDict := self at: j .
  aDict == nil ifFalse:[
    (aDict at: aSymbol otherwise: nil) ifNotNil:[
      " found aSymbol in this dict "
      result add: lf; add: (j asString); addAll: '  '.
      theAssoc:= aDict associationsDetect: [:anAssoc | anAssoc value == aDict]
                               ifNone: [ nil ] .
      theAssoc == nil
        ifTrue:[ result add: '(Unnamed Dictionary)']
        ifFalse: [ result add: theAssoc key ].
      result add: lf; addAll: '      '; addAll: aSymbol;
                 addAll: '  '; add: ((aDict at: aSymbol) asString).
      ^ result
      ].
    ].
  ].
^ self _error: #rtErrSymNotFound args: { aString }
%

category: 'CodeModification Override'
method: SymbolList
_addAllFromNsc: aBag

self _validatePrivilegeAdd: aBag .
aBag accompaniedBy: self do:[:me :anObj |
	me _validateElement: anObj .
].
^ super _addAllFromNsc: aBag
%

category: 'CodeModification Override'
method: SymbolList
_at: anIndex put: aValue

self _validatePrivilegeAdd: aValue .
self _validateElement: aValue .
^ super at: anIndex put: aValue
%

category: 'CodeModification Override'
method: SymbolList
_basicAt: anIndex put: aValue

self _validatePrivilegeAdd: aValue .
self _validateElement: aValue .
^ super _basicAt: anIndex put: aValue
%

category: 'Private'
method: SymbolList
_deferredGciUpdateWith: valueArray

self size == 0 ifTrue:[
  self addAll: valueArray .
] ifFalse: [
  " semantics of the deferred update not defined"
  self _error: #errNoStructuralUpdate
]
%

category: 'CodeModification Override'
method: SymbolList
_deleteNoShrinkFrom: startIndex to: stopIndex

self _validatePrivilege: self .
^ super _deleteNoShrinkFrom: startIndex to: stopIndex
%

category: 'Instance Initialization'
method: SymbolList
_gciInitialize

self size: 0 .
^ true
%

category: 'CodeModification Override'
method: SymbolList
_insertAt: destOffset
from: anArray
fromStart: startOffset
fromEnd: endOffset
numToMoveDown: numToMove

self _validatePrivilegeAdd: anArray .
startOffset to: endOffset do:[ :j |
	self _validateElement:(anArray at: j) .
].
^ super
	_insertAt: destOffset
	from: anArray
	fromStart: startOffset
	fromEnd: endOffset
	numToMoveDown: numToMove
%

category: 'CodeModification Override'
method: SymbolList
_primitiveAt: anIndex put: aValue

self _validatePrivilegeAdd: aValue .
self _validateElement: aValue .
^ super _primitiveAt: anIndex put: aValue
%

category: 'CodeModification Override'
method: SymbolList
_unsafeAt: anIndex put: aValue

self _validatePrivilegeAdd: aValue .
self _validateElement: aValue .
^ super _unsafeAt: anIndex put: aValue
%

category: 'Private'
method: SymbolList
_validateElement: anObj

"if anObj is a kind of SymbolDictionary, return true, otherwise return false"

(anObj isKindOf: SymbolDictionary) ifTrue: [ ^ true ].
ArgumentTypeError new constrainedIv: 'SymbolList.element'
expectedClass: SymbolDictionary actualArg: anObj ;
 signal .
self _uncontinuableError
%

category: 'Updating'
method: SymbolList
_validatePrivilege

System myUserProfile _validateCodeModificationPrivilege
%

category: 'Updating'
method: SymbolList
_validatePrivilege: item

"Only need to #_validatePrivilege if adding/removing a SymbolDictionary"

(item isKindOf: SymbolDictionary) ifTrue: [
   self _validatePrivilege .
   ^self
].
(item isKindOf: Collection) ifTrue: [
  item do: [:x |
    (x isKindOf: SymbolDictionary) ifTrue: [
      ^ self _validatePrivilege .
    ]
  ].
].
^ true
%

category: 'Updating'
method: SymbolList
_validatePrivilegeAdd: item

"Check that item satisfies constraints and then check privilege."

(item isKindOf: SymbolDictionary) ifTrue: [
  self _validatePrivilege: item .
  ^ self
].
(item isKindOf: Collection) ifTrue: [
     item do: [:x | x _validateClass: SymbolDictionary ].
] ifFalse:[
    item _validateClass: SymbolDictionary .
].
self _validatePrivilege: item
%

! Class extensions for 'SymbolSet'

!		Instance methods for 'SymbolSet'

removeallmethods SymbolSet
removeallclassmethods SymbolSet

category: 'Deprecated'
method: SymbolSet
_addSymbol: aSymbol

self deprecated: '_addSymbol: Obsolete, provided for compatibility.
 Same functionality as IdentitySet | add: '.

^ self add: aSymbol
%

category: 'Accessing'
method: SymbolSet
_returnValueOf: anObject

"Returns the element of the receiver whose value is the same as the argument.
 If the argument is not equal to any value in the receiver, then add the
 argument to the receiver."

|symbolizedObject|

symbolizedObject := anObject asSymbol.
self add: symbolizedObject.
^ symbolizedObject
%

! Class extensions for 'System'

!		Class methods for 'System'

removeallmethods System
removeallclassmethods System

category: 'Transaction Control'
classmethod: System
abort

^ System abortTransaction
%

category: 'Transaction Control'
classmethod: System
abortAll
"Abort all levels of nested transactions and then
 do an outer level abortTransaction."

 [ (self _zeroArgPrim: 5) > 1 ] whileTrue:[
   self _abortNestedTransaction.
 ].
 ^ self abortTransaction
%

category: 'Transaction Control'
classmethod: System
abortTransaction

"If in a nested transaction (System transactionLevel > 1) ,
 aborts the current nested transaction.
 and leaves the session in the parent level of transaction.
 The in-memory state of persistent objects modified during the nested transaction
 is rolled back to the in-memory state as of the start of the nested transaction.
 Temporary objects which were part of the closure of the outer level transaction
 at the start of this nested transaction, and were modified during this
 nested transaction, will be rolled back to their state as of the start of
 this nested transaction.

 If in an outer level transaction, (System transactionLevel <= 1) ,
 rolls back all modifications made to committed GemStone objects and provides
 the session with a new view of the most recently committed GemStone state.
 The roll back and new view happen whether or not the session was previously in a
 transaction.  If the transaction mode is set to #autoBegin, then a new
 transaction is started.  If the transaction mode is set to #manualBegin, then
 a new transaction is not started."

 (self _zeroArgPrim: 5) > 1 ifTrue:[
   self _abortNestedTransaction.
   ^ self
 ] ifFalse:[ | coordinator |
  ^ (coordinator := self _commitCoordinator) == nil
    ifTrue: [ self _localAbort ]
    ifFalse: [ coordinator abort ].
 ].
%

category: 'System Control'
classmethod: System
activeRepositories

"Returns an Array containing references to the repositories that are
 attached at the time the message is sent."

^ self _zeroArgCateg2Prim: 0
%

category: 'System Control'
classmethod: System
addAllToStoneLog: srcString

"Appends text to the Stone's informational log file.  First, this method writes
 a banner that identifies the session from which aString came.  It then appends
 aString itself.  The argument must be convertable to a Utf8, and the total
 number of bytes of the converted string be 16270 or less."

| aString  |


aString := srcString encodeAsUTF8.
aString basicSize < 16271  "truncate message if too long"
	ifFalse: [(aString := aString size: 16200) addAll: '..truncated..' encodeAsUTF8].

^ self _addAllToStoneLog: aString

%

category: 'Deprecated'
classmethod: System
adminGcGemSessionId

"Return the session ID of the Admin GC session.  Returns
 zero if the Admin GC session is not running."

self deprecated: 'System class >> adminGcGemSessionId deprecated v3.2.  Use adminGemSessionId instead'.
^self adminGemSessionId
%

category: 'Garbage Collection Management'
classmethod: System
adminGemConfigs
"Returns a String that documents the admin gem configuration options"

^'#adminVerboseLogging
   Controls the amount of logging information written to the log file.
   If set to false, normal logging information will be written.  If set to
   true, extra logging information is written to the log file.
     (default: false)

 #epochGcMaxThreads
   MaxThreads used for next epochGc
     (default:1, min:1, max: 32)

 #epochGcPageBufferSize
   Size in pages of buffer used for epoch GC (must be power of 2)
     (default: 64, min:8, max 1024)

 #epochGcPercentCpuActiveLimit
   "Limit active epoch threads when system percentCpuActive is above this limit
     (default: 90, min: 0, max: 100)

 #epochGcTimeLimit
   Controls the maximum frequency of epochs, in seconds. It is recommended that
   this value not be less than 1800 (i.e. 30 minutes), since the aging of objects
   faulted into gem memory uses 5 minute aging for each of 10 subspaces of the pom
   generation.  Ideally epochGcTimeLimit should be several hours.
     (default: 3600, min: 5, max: maxInt32)

 #epochGcTransLimit
   Minimum number of commits that must have occurred in order to start an
   epoch garbage collection.
     (default: 5000, min: 0, max: maxInt32)

 #saveWriteSetUnionToFile
   If true, causes the Admin GC gem to write out the write set union bitmap to
   a binary file before sweeping the union.  The file will be placed in the
   same directory as the Admin GC gem log file.
     (default: false)

 #sweepWsUnionMaxThreads
   MaxThreads used for next wsUnion sweep
     (default: 1 min: 1, max: 32)

 #sweepWsUnionPageBufferSize
   Size in pages of buffer used for wsUnion sweep.
     (default: 64, min: 8, max: 1024)

 #sweepWsUnionPercentCpuActiveLimit
   Limit active wsUnion threads when system percentCpuActive is above this limit.
     (default: 90, min: 0, max: 100);
'
%

category: 'Garbage Collection Management'
classmethod: System
adminGemSessionId

"Return the session ID of the Admin GC session.  Returns
 zero if the Admin GC session is not running."

^self _zeroArgPrim: 64
%

category: 'Session Control'
classmethod: System
allZombieSessions

"Returns a list of session IDs that are partially logged out and are in the process of being
 disposed by the stone.  These zombie sessions may or may not reference a commit record."

^ self _zeroArgPrim: 165
%

category: 'Session Control'
classmethod: System
allZombieSessionsRefOldestCr

"Returns a list of session IDs are partially logged out and are in the process of being
 disposed by the stone which also reference the oldest commit record."

^ self _zeroArgPrim: 166
%

category: 'Signals'
classmethod: System
almostOutOfMemoryErrorThreshold

"Returns a positive percentage of memory full at which the AlmostOutOfMemory
 error will be raised, or -1  if the error is not enabled. "

^self _signalErrorStatus: 5
%

category: 'System Control'
classmethod: System
applyKeyfileNamed: arg

"Causes the stone to read the keyfile with file name arg.  If arg is nil, the
 stone will use the same file name used when the stone was started.

 Requires the SystemControl privilege.  The stone must have permission to
 open the given keyfile for reading.

 The new keyfile permissions apply only as long as the stone is running.
 To permanently use the new keyfile, modify the KEYFILE option in the
 stone configuration file.

 Returns true upon success.  In this case, the keyfile was read and one or more
 changes were made to the system.  The system is now running as if it was
 restarted with the new keyfile.

 Returns false if the keyfile was successfully read but contained no
 meaningful changes from the current configuration.  For example,
 if the only change from the previous keyfile is a new license number.

 Raises an exception if the keyfile could not be read, has expired,
 or if applying it would modify parameters which may not be changed
 at runtime."

<primitive: 1031>
arg ifNotNil:[
  arg _validateClass: String
].
^ self _primitiveFailed: #applyKeyfileNamed:
%

category: 'Transaction Control'
classmethod: System
beginNestedTransaction
"Enter a new nested transaction.
 If session is outside of a transaction, equivalent to beginTransaction.
 Signals a ImproperOperation exception if the begin would exceed
 16 levels of nested transactions."

 (self _zeroArgPrim: 5) == 0 ifTrue:[
    ^ self beginTransaction
 ] ifFalse:[
   (self _zeroArgPrim: 146 "_beginNested") ifFalse:[
      ImproperOperation signal:'attempt to exceed 16 levels of nested transactions'
   ].
   ^ self
 ]
%

category: 'Transaction Control'
classmethod: System
beginTransaction

"Starts a new transaction for the session.  An abort is done before the new
 transaction is started - giving the session a new snapshot of the repository.

 If any permanent objects had been written by the session, their
 state is aborted.  This method returns the receiver (System).

 If in a nested transaction (System transactionLevel > 1) ,
 an error is signaled. and no change is made to the transaction state."

| coordinator |
^ (coordinator := self _commitCoordinator) == nil
  ifTrue: [ self _localBeginTransaction ]
  ifFalse: [ coordinator beginTransaction ].
%

category: 'Cache Statistics'
classmethod: System
cacheName: aString

"Sets the name the current session is known by in the shared cache
 to 'aString'. Does nothing if the current session is not attached
 to a shared cache. Raises an error if 'aString' is too long.
 Returns self."

^ self _cacheName: aString
%

category: 'Cache Statistics'
classmethod: System
cacheSlotForProcessId: anInt

"Return the cache slot at being used by the given process ID.
 Return -1 if the slot was not found."

^ self _primCacheStatsForSlotOrSession: anInt opCode: 5
%

category: 'Cache Statistics'
classmethod: System
cacheSlotForSessionId: aSessionId

"Return the cache process slot number (a SmallInteger) for the given session
 ID.  The session must be connected to the same shared page cache as the
 session invoking this method.

 A return of nil indicates the session could not be located."

<primitive: 567>
aSessionId _validateClass: SmallInteger.
aSessionId _error: #rtErrArgOutOfRange .
^ self _primitiveFailed: #cacheSlotForSessionId: args: { aSessionId }
%

category: 'Cache Statistics - Deprecated'
classmethod: System
cacheStatistics: aProcessSlot

"Returns an Array whose contents are described by the result of the
 cacheStatisticsDescription method.  The Array contains statistics for the
 specified slot in the GemStone shared memory cache to which this session is
 attached.

 The argument aProcessSlot should be a SmallInteger between 0 and the number of
 process slots in the shared cache minus 1, inclusive.  If aProcessSlot is
 outside the range of valid process slots, or the session executing this
 method is not using a shared cache, generate an error.  If the slot specified
 by aProcessSlot is an inactive slot, returns nil.  The method
 cacheSlotForSessionId: may be used to determine the process slot of a session
 on the same shared cache.

 The process slots that are predefined are:

    slot 0: The shared page cache monitor.

    slot 1: The Stone if the cache is on the same machine as the Stone.
            Otherwise, a page server that is used to monitor the cache for
            the Stone.

 No other slots are guaranteed.  However, slot 2 is the often the page server
 and slot 3 is often the GcGem.  These depend to some extent on the relative
 speed of the processes during startup.  In addition, the GcGem can be
 shut down, and when it is restarted, it is unlikely to end up at the same
 position.

 This method is deprecated and may be removed in a future release.
 It is replaced by the following classmethod in class System:

  cacheStatisticsAt:
"

self deprecated: 'System class>>cacheStatistics: deprecated v3.0.
Use #cacheStatisticsAt: instead'.
 ^self _primCacheStatsForSlotOrSession: aProcessSlot opCode: 0
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsAt: aProcessSlot

"Returns an array of statistics appropriate to the specific executable (common + unique).
 The argument aProcessSlot is the same as described in #'cacheStatistics:' method.
 The contents are described by the result of sending element 4 of the result to the
 #cacheStatisticsDescriptionFor: method."

^ self _primCacheStatsForSlotOrSession: aProcessSlot opCode: 1
%

category: 'Cache Statistics - Deprecated'
classmethod: System
cacheStatisticsDescription

"Returns an Array of Strings describing the result of the method
 cacheStatistics: .

 A new Array of new Strings is created every  time this primitive
 is called, so the application may wish to cache the result.

 This method is deprecated and may be removed in a future release.
 It is replaced by the following classmethods in class System:

  cacheStatisticsDescriptionForMonitor
  cacheStatisticsDescriptionForStone
  cacheStatisticsDescriptionForPageServer
  cacheStatisticsDescriptionForGem
  cacheStatisticsDescriptionAt:
  cacheStatisticsDescriptionForType:
"

self deprecated: 'System class>>cacheStatisticsDescription: deprecated v3.0.
Use #cacheStatisticsDescriptionForType: instead'.

^ self _deprecatedCacheStatisticsDescription
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsDescriptionAt: aSlot

"Returns an array of strings which describe the cache statistics for the
 kind using the cache slot with index anInt.  Returns an empty array if
 the slot is not in use."

| kind |
kind := self processKindForSlot: aSlot .
^ kind == -1
    ifTrue:[ Array new ]
    ifFalse:[ self cacheStatisticsDescriptionForType: kind ]
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsDescriptionForGem

"Returns an Array of Strings describing cache statistics applicable to a
 gem process."

^ self _zeroArgPrim: 4008
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsDescriptionForMonitor

"Returns an Array of Strings describing cache statistics applicable to the shared
 page cache monitor."

^ self _zeroArgPrim: 4001
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsDescriptionForPageServer

"Returns an Array of Strings describing cache statistics applicable to a
 page server process."

^ self _zeroArgPrim: 4004
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsDescriptionForStone

"Returns an Array of Strings describing cache statistics applicable to the
 stone."

^ self _zeroArgPrim: 4002
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsDescriptionForType: aProcessKind

"Returns an Array of Strings describing the result of the method #'cacheStatisticsAt:'. The
 argument <aProcessKind> is the StatType (as returned as element #4 by #'cacheStatisticsAt:')
 and must be one of the following:

   1 = Shared Page Cache Monitor
   2 = Stone
   4 = Page Server
   8 = Gem (including Topaz, GBS, and other GCI applications).

 The result includes only those appropriate to the specific executable (common and unique).
 Note that Statmonitor will capture additional statistics (e.g., on itself and the system)
 that are not actually stored in shared memory."

^ self _zeroArgPrim: aProcessKind + 4000
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsForAllSlots

"Return an Array with one entry for each process attached to the shared
 cache.  Each element in the result contains the cache statistis for that
 slot.  The last element in each array is the slot ID of the slot.
 The subarrays will be different sizes depending on what type of process
 is using the slot."

^ self _zeroArgPrim: 139
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsForAllSlotsShort

"Return an Array with one entry for each process attached to the shared
 cache.  Each element in the result is an array containing 5 elements.
 The first 4 elements are the same first 4 elements returned when fetching
 cache statistics for a session, slot, or process.  The fifth element is the
 process slot for the process.  Each sub-array of 5 elements contain the
 following elements:

  1 - ProcessName (String)
  2 - ProcessId   (SmallInteger)
  3 - SessionId   (SmallInteger)
  4 - StatType    (SmallInteger)
  5 - Cache Slot  (SmallInteger)
"

^ self _zeroArgPrim: 138
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsForProcessWithCacheName: aString

"Search the cache for the first process with cache name matcing aString.
 If found, return the statistics appropriate for that process.

 Returns nil if the process with the given cache name was not found."

^ self _primCacheStatsForSlotOrSession: aString opCode: 3
%

category: 'Cache Statistics - Deprecated'
classmethod: System
cacheStatisticsForSessionId: aSessionId

"Same as the cacheStatistics: method except the argument is the session ID
 of a session currently connected to the shared memory cache.  In systems
 that use multiple shared memory caches, the session must exist on the same
 cache as the session invoking this method.

 nil is returned if the session cannot be located.

 This method is deprecated and may be removed in a future release.
 It is replaced by the following classmethod in class System:

  gemCacheStatisticsForSessionId:
"

self deprecated: 'System class>>cacheStatisticsForSessionId: deprecated v3.0.
Use #gemCacheStatisticsForSessionId: instead'.
 ^self _primCacheStatsForSlotOrSession: aSessionId negated opCode: 0
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsProcessId: aPid

"Search the cache for a process id matching aPid.  If found, return the
 statistics appropriate for that process.  Returns nil if the process
 was not found."

^ self _primCacheStatsForSlotOrSession: aPid opCode: 2
%

category: 'Cache Statistics'
classmethod: System
cacheStatsForGemWithName: aStringOrSymbol

"This method does a case-sensitive search for the first gem with the given
 cache name and returns an array of cache statistics for the gem if found.

 Returns nil if the gem could not be located."

<primitive: 565>
aStringOrSymbol _validateClass: String.
^ self _primitiveFailed: #cacheStatsForGemWithName: args: { aStringOrSymbol }
%

category: 'Cache Statistics'
classmethod: System
cacheStatsForPageServerWithSessionId: anInt

"Return the cache statistics for the page server with the given session
 ID.  Page servers assume the same session ID as their client gem.

 Return -1 if the page server with the given session ID was not found."

^ self _primCacheStatsForSlotOrSession: anInt opCode: 8
%

category: 'Shared Cache Management'
classmethod: System
cacheStatusCount
  "Returns a positive SmallInteger value.
   The value maintained in the stone process increments
   whenever a remote cache starts up , shuts down, or becomes a midCache"

^ self _zeroArgPrim: 73
%

category: 'Session Control'
classmethod: System
cacheWarmerSessions
  "Returns an Array of sessionIds of cache warmer sessions.
   Requires SessionAccess privilege."
  ^ self currentSessions select:[:sessId |
      ((self descriptionOfSession: sessId) at: 17) = 'cachewarmer'
    ].
%

category: 'Session Control'
classmethod: System
cacheWarmerSessionsReport
  "Returns a String describing cache warmer sessions.
   Requires SessionAccess privilege."
  ^ self _sessionsReport: self cacheWarmerSessions .
%

category: 'Authorization'
classmethod: System
canRead: anObject

"In Gemstone64 , an object must be faulted into memory before
 it can be passed as an argument to a method.  Thus by definition
 if you enter this method you can read the argument. "

^ true
%

category: 'Authorization'
classmethod: System
canWrite: anObject

"This method tests whether the user has authorization to write anObject without
 adding it to write set and returns a Boolean result."

<primitive: 103>
self _primitiveFailed: #canWrite: args: { anObject } .
self _uncontinuableError
%

category: 'Private'
classmethod: System
changeCacheSlotFreeFrameLimit: aSlot to: aValue

"Changes the freeFrameLimit for the process associated with a cache slot to
 aValue, which must be a positive SmallInteger.  The argument aSlot
 should be a SmallInteger between 0 and the number of process slots in the
 shared cache minus 1, inclusive.

 To execute this method, you must have the SystemControl privilege."

<primitive: 349>

aSlot _validateClasses:{ SmallInteger }.
aValue _validateClasses:{ SmallInteger }.
self _primitiveFailed: #changeCacheSlotFreeFrameLimit:to:
     args: { aSlot . aValue }
%

category: 'Garbage Collection Management'
classmethod: System
changeNumberOfReclaimGemSessions: num

"This method attempts to reconfigure the system to have num reclaim
 threads running and returns the actual number that it is able to
 accommodate. If num is smaller than the current number of reclaim
 threads, then threads are stopped and num is returned.  If the value
 of num is 0 and the reclaim gem is running then it is stopped.
 If num is larger than the current number of reclaim threads and
 if the number is less than STN_MAX_GC_RECLAIM_SESSIONS, then threads
 are started until that number are running and the value of num
 is returned.  If num is larger than STN_MAX_GC_RECLAIM_SESSIONS then the
 system may be able to accommodate the request by using some of the
 cache slots allocated for user sessions (from STN_MAX_SESSIONS).
 If this occurs then this method does not limit to STN_MAX_GC_RECLAIM_SESSIONS,
 and the value returned may be less than or equal to num depending on
 whether cache slots are currently considered available.

 A new value > 0 will be reflected in result of any subsequent
    System stoneConfigurationAt: #StnNumGcReclaimSessions .

 If the result of this method is more than STN_MAX_GC_RECLAIM_SESSIONS, then
 a warning is posted in the stone log indicating that user cache slots
 were allocated for reclaim threads.

 It is the callers responsibility to reset the number of reclaim threads to
 a value less than or equal to STN_MAX_GC_RECLAIM_SESSIONS to prevent users from
 getting login errors after the reclaim that needed more resources has
 completed.
"

<primitive: 287>
num _validateClass: SmallInteger.
%

category: 'Online Backup Support'
classmethod: System
checkpointStatus
"Returns an array of 2 elements.  The first element is a boolean indicating
 if checkpoints are currently suspended.  The second element is an
 Integer indicating the number of seconds before checkpoints will be
 resumed by the stone."


 ^self _zeroArgPrim: 61
%

category: 'Garbage Collection Management'
classmethod: System
clearEpochGcState

"Resets the epoch GC state which keeps track of objects eligible to be
 marked as possible dead during the next epoch GC operation.  Has no effect
 on objects already marked possible dead by previously completed epochs."

^self _zeroArgPrim: 38
%

category: 'Reduced Conflict Support'
classmethod: System
clearRcValueCache

"Clears the cache of calculated values for reduced conflict classes by setting
 the temporary session state slot to nil."

self __sessionStateAt: 5 put: nil
%

category: 'Reduced Conflict Support'
classmethod: System
clearRedoLog

"Clear the redo log by setting the temporary session state slot to nil.

                            Warning:
 Clearing the redo log will probably prevent Reduced Conflict
 classes from resolving conflicts.  Sending this message negates this
 capability for the current transaction.

 This is a protected method."

< protected >

self __sessionStateAt: 4 put: nil
%

category: 'Environment Access'
classmethod: System
clientEnvironmentVariable: varName

"Expands the environment variable named varName in the GemBuilder for C client
 process, returning a String.  The varName argument should be a kind of String.

 Returns nil if any of the following are true:

 * varName is not a byte format object.
 * There is no environment variable defined with name varName.
 * The value of the environment variable is more than approximately 8000 bytes.
 * The size of varName exceeds approximately 8000 bytes.

 Signals an error if NoGsFileOnServer privilege is set in the UserProfile.
 "

^ GsFile _expandEnvVariable: varName isClient: true
%

category: 'Environment Access'
classmethod: System
clientEnvironmentVariable: varName put: valueStr

"Sets the environment variable named varName in the GCI client process
 to value valueStr .
 Both arguments must be either a String or a Unicode7 not larger than 8000 bytes.
 The NoGsFileOnClient privilege must not be set in the session's UserProfile.

 Returns self or signals an Error if the operation failed.
"
^ GsFile _setEnvVariable: varName value: valueStr isClient: true
%

category: 'Session Control'
classmethod: System
clientIsRemote

"Returns true if the GemBuilder for C client for this session is in a different
 process than the Gem, otherwise returns false."

^ self _zeroArgPrim: 20
%

category: 'Version Management'
classmethod: System
clientVersionAt: aString

"Returns version information about the client GemBuilder for C process.  If the
 client is a session using the linkable GemBuilder for C login, this method is
 equivalent to gemVersionAt:.

 See System(C) | gemVersionAt: for further details."

| verId |
verId := VersionParameterDict at: aString otherwise: nil .
verId ifNil:[ ^ nil ].
^ GsFile _clientVersionAt: verId
%

category: 'Version Management'
classmethod: System
clientVersionReport

"Return a StringKeyValueDictionary whose keys are the names of operating system,
 hardware, or GemStone version attributes, and whose values are the
 current values of those attributes in the client GemBuilder for C process."

| result |
result := StringKeyValueDictionary new .
VersionParameterDict keysAndValuesDo:[ :aName :anId | | aVal |
  aVal := GsFile _clientVersionAt: anId .
  aVal ~~ nil ifTrue:[
    result at: aName put: aVal
    ].
  ].
^ result
%

category: 'Clustering'
classmethod: System
clusterAllSymbols

"This method clusters the AllSymbols hash dictionary and all of the symbols to
 which it refers.
 The clustering is performed by the Symbol Creation Gem, and other
 sessions may see a long latency on symbol creation requests while
 the clustering is in progress."

| result |
GsFile gciLogServer:'-- sending clusterAllSymbols to Symbol Creation gem'.
result := self _zeroArgCateg2Prim: 6 .
GsFile gciLogServer:'-- waiting 10 seconds for Symbol Creation gem commit'.
20 timesRepeat:[
  self _sleepMs: 500 .
  System inTransaction ifFalse:[ System abortTransaction ].
].
^ result
%

category: 'Clustering'
classmethod: System
clusterBucket: aClusterBucketOrId

"This method sets the current default ClusterBucket to the ClusterBucket with
 the specified clusterId.  The argument may be an instance of ClusterBucket, or
 a positive SmallInteger which specifies an instance of ClusterBucket."

<primitive: 348>
self _primitiveFailed: #clusterBucket: args: { aClusterBucketOrId } .
self _uncontinuableError
%

category: 'Host System Access'
classmethod: System
commandLineArguments
  "Returns an Array of Strings, the command line arguments to
   the gem or topaz -l  process for this session.
   Each invocation of this method returns a new instance of the result."

  ^ self _zeroArgPrim: 160
%

category: 'Transaction Control'
classmethod: System
commit

"
 If session is not in transaction, signals an ImproperOperation exception.

 If in a nested transaction (System transactionLevel > 1) ,
 commits the current level of transaction
 and leaves the session in the parent level of transaction.

 The following applies if the current level is 1 (i.e. outer level):
     Attempts to update the persistent state of the Repository to include changes
     made by this transaction.  If the outer level commit succeeds,
     then this method returns true, and the current transaction's changes, if any,
     become a part of the persistent Repository.
     After the repository update, the session exits the current
     transaction.  If the transaction mode is #autoBegin, then the session enters
     a new transaction.  If the transaction mode is #manualBegin, then the session
     remains outside of a transaction.

     If conflicts prevent the repository update, then a TransactionError is signalled.
     Call the transactionConflicts method to determine the nature of the
     conflicts.  If the session is outside of a transaction, then this method
     signals the Error #rtErrPrimOutsideTrans.

     This method also updates the session's view of GemStone.  If the commit
     operation succeeds, then all objects in the session's view are consistent with
     the current state of GemStone.  If the commit fails, then this method retains
     all the changes that were made to objects within the current transaction.
     However, commits made by other sessions are visible to the extent that changes
     in this transaction do not conflict with them.

 Returns true if commit was read-only or succeeded .
 Signals an Error if there was a failure.  "

  (self _zeroArgPrim: 5) <= 1 ifTrue:[ "level 0 or 1"
    (self _commit: 0 "don't release locks") ifFalse:[
      TransactionError new reason: 'commitConflicts' ; signal:'commit conflicts'
    ].
  ] ifFalse:[
    self _commitNestedTransaction.
  ] .
  ^ true
%

category: 'Transaction Control'
classmethod: System
commitAll
"Commit all levels of nested transactions, if any, and then attempt
 to commit the outer level transaction.

 If conflicts prevent the outer level commit, a TransactionError is signalled.
 If session is outside of a transaction, an Error is signalled."

 [ (self _zeroArgPrim: 5) > 1 ] whileTrue:[
   self _commitNestedTransaction.
 ].
 ^ self commit
%

category: 'Transaction Control'
classmethod: System
commitAndReleaseLocks

"Attempt to commit the transaction for the current session.

 If in a nested transaction (System transactionLevel > 1) ,
 an error is signalled and no change is made to the transaction state.

 If in an outer level transaction, this method is the same
 as 'commitTransaction' except for the handling of locks.
 If the commit succeeds, this method releases all locks for the session
 and returns true.  Otherwise, it returns false and does not release locks.

 This method also clears the commit release locks and commit-or-abort release
 locks sets.  See the 'Releasing Locks' method category for more
 information.

 Returns true if commit was read-only or succeeded ,
 false if there was a failure.  "

 (self _zeroArgPrim: 5) <= 1 ifTrue:[ "level 0 or 1"
   ^ self _commit: 1 "release locks" .
 ].
 ImproperOperation signal:'operation illegal while in a nested transaction'
%

category: 'System Control'
classmethod: System
commitRecordBacklog

"Return the number of commit records in the system."
^self stoneCacheStatisticWithName: 'CommitRecordCount'
%

category: 'Session Control'
classmethod: System
commitRecordPageForSessionId: aSessionId

"Return the page ID of the commit record referenced by the given session ID.
 The result will usually be a SmallInteger but could also be a
 LargeInteger.

 Returns -1 if the session does not exist or if it does not currently reference a
 commit record (such as after a Lost OT root error).

 To execute this method for any session other than your current session, you
 must have the SessionAccess privilege.
 Gs64 v3.5+  returns view.viewCommitRecord ; previously returned view.endingCr "

<primitive: 663>
aSessionId _validateClass: SmallInteger .
^ self _primitiveFailed: #commitRecordPageForSessionId: args: { aSessionId } .
%

category: 'Transaction Control'
classmethod: System
commitsDisabledUntilAbort

"Answers true if further commits have been disabled until the session aborts.
 Otherwise answers false.

 It is possible the session may still not be able to commit when this
 method returns true.  Possbile reasons are the session is not in transaction,
 commits are disabled at the UserProfile level, or commits have been disabled
 until the session logs out.

 Use the #sessionCanCommit method to definitively determine if the current session
 can commit."

^self _zeroArgPrim: 113.
%

category: 'Transaction Control'
classmethod: System
commitsDisabledUntilLogout

"Answers true if further commits have been disabled for the session.
 Otherwise answers false.

 It is possible the session may still not be able to commit when this
 method returns true.  Possbile reasons are the session is not in transaction,
 commits are disabled at the UserProfile level, or commits have been disabled
 only until the next abort.

 Use the #sessionCanCommit method to definitively determine if the current session
 can commit."

^self _zeroArgPrim: 112.
%

category: 'Transaction Control'
classmethod: System
commitsSinceCurrentView

"Returns a SmallInteger which is the number of commits of
 transaction level 1  which have occurred since
 the session obtained its current view.  If the session has no current view due to
 a lost OT root error, then -1 is returned.  A result of 0 indicates the session
 has the most current view of the repository."

^ self _zeroArgPrim: 134
%

category: 'Transaction Control'
classmethod: System
commitTransaction

"Same as System(C)>>commit, except that if transaction conflicts are
  detected, returns false instead of signalling a TransactionError."

  (self _zeroArgPrim: 5) <= 1 ifTrue:[ "level 0 or 1"
    (self _commit: 0 "don't release locks") ifFalse:[
      ^ false "commit conflicts"
    ].
  ] ifFalse:[
    self _commitNestedTransaction.
  ] .
  ^ true
%

category: 'Deprecated'
classmethod: System
concurrencyMode

"This method is obsolete, concurrency mode is always #NO_RW_CHECKS"

self deprecated: 'System class>>concurrencyMode is obsolete in GemStone/S 64 Bit.'.

^ #NO_RW_CHECKS
%

category: 'Configuration File Access'
classmethod: System
configurationAt: aName

"Returns the value of the specified configuration file parameter, giving
 preference to the Gem process if the parameter applies to the Gem."

| result |
result := self gemConfigurationAt: aName .
result == nil ifTrue:[
  result := self stoneConfigurationAt: aName
  ].
^ result
%

category: 'Runtime Configuration Access'
classmethod: System
configurationAt: aName put: aValue

"Change the value of the specified configuration parameter.

 The changeable parameters all require aValue to be a SmallInteger.

 Configuration parameters should not be changed unless there is a clear
 reason for doing so, since incorrect settings of parameters can
 have serious adverse effects on GemStone performance.

 Configuration parameters for Stone that are transferred to Gem processes
 are only read by the Gem at login, so changes using this method to
 Stone parameters may have no effect on existing sessions.

 Parameters in the Gem with the following names may be changed by any user at
 any time:
   * #GemAbortMaxCrs
   * #GemCommitConflictDetails
   * #GemCommitStubsForNpObjects
   * #GemConvertArrayBuilder (takes effect at start of next method compile)
   * #GemDropCommittedExportedObjs
   * #GemExceptionSignalCapturesStack
   * #GemFreeFrameLimit
   * #GemFreePageIdsCache
   * #GemHaltOnError
   * #GemKeepMinSoftRefs
   * #GemKeyRingDirs
   * #GemNativeCodeEnabled (only can disable at runtime)
   * #GemPgsvrCompressPageTransfers
   * #GemPgsvrUpdateCacheOnRead
   * #GemPomGenPruneOnVote
   * #GemReadAuthErrStubs
   * #GemRepositoryInMemory
   * #GemSoftRefCleanupPercentMem
   * #GemTempObjOomstatsCsv
   * #GemTempObjConsecutiveMarksweepLimit
   * #GemTempObjPomgenScavengeInterval

 Parameters with the following names may be changed only by users who have
 the correct privilege:

   * #StnAdminGcSessionEnabled - Requires GarbageCollection privilege.
   * #StnDisableLoginFailureLimit - Requires OtherPassword privilege.
   * #StnDisableLoginFailureTimeLimit - Requires OtherPassword privilege.
   * #StnEpochGcEnabled - Requires GarbageCollection privilege.
   * #StnLoginsSuspended - Requires SystemControl privilege.
   * #StnLogLoginFailureLimit - Requires OtherPassword privilege.
   * #StnLogLoginFailureTimeLimit - Requires OtherPassword privilege.
   * #StnMaxLoginLockSpinCount - Requires SystemControl privilege.
   * #StnNumGcReclaimSessions - Requires GarbageCollection privilege.
   * #StnObjLockTimeout - Requires SystemControl privilege.
   * #StnRemoteCachePgsvrTimeout - Requires SystemControl privilege.
   * #StnSignalAbortCrBacklog - Requires GarbageCollection privilege.
   * #StnSignalAbortAggressive - Requires GarbageCollection privilege.
   * #StnSmcSpinLockCount - Requires SystemControl privilege.
   * #StnSymbolGcEnabled - Requires GarbageCollection privilege.
   * #StnTranLogDebugLevel - Requires SystemControl privilege.

 All other parameters that are changeable at run time may be changed
 only by SystemUser, and should not normally need to be changed in the
 course of GemStone operation."

| cfgId |

cfgId := ConfigurationParameterDict at: aName .
self _atConfigId: cfgId put: aValue .
^ self configurationAt: aName
%

category: 'Transaction Control'
classmethod: System
conflictReportString
"Return a String describing the result of  System(C)>>transactionConflicts"

^ self conflictReportString: self transactionConflicts
%

category: 'Transaction Control'
classmethod: System
conflictReportString: conflicts
"Return a String describing the result of  System(C)>>transactionConflicts"

^ self conflictReportString: conflicts maxOops: 100
%

category: 'Transaction Control'
classmethod: System
conflictReportString: conflicts maxOops: maxOops
 "The conflicts argument is Array returned by System(C)>>transactionConflicts .
  The report contains oops and class names for each object in conflict,
  limited to the first maxOops oops in each category of conflict.
 "
  | result |
  result := String new .
  #( #'Read-Write' #'Write-Write' #'Write-Dependency' #'Write-ReadLock' 
	 #'Write-WriteLock' 'Rc-Write-Write' #'Synchronized-Commit'
   ) do:[:aSym | | v sz |
      v := conflicts at: aSym otherwise: nil .
      sz := v size .
      sz ~~ 0 ifTrue:[  | limit |
        result add: sz asString; add: $  ; add: aSym  ; add: ' Conflicts(' ; lf.
        limit := sz min: maxOops .
        1 to: limit do:[ :n | | each | 
          each := v at: n .
          result add:' ' ; add: each asOop asString; add: '(a ';
                 add: each class name ; add: $) .
	  (0 == (n \\ 8)) ifTrue:[ result lf ].	 
        ] .
        sz > maxOops ifTrue:[ result lf ; addAll:' ('; addAll: (sz - maxOops)  asString;
                            addAll:' more oops)' ].
        result add: $) ;  lf .
      ]
  ].
  ^ result
%

category: 'Transaction Control'
classmethod: System
conflictsReport

"Returns an Array describing conflicts or success of the last attempted commit.
 First element of result is a symbol describing the status.
 If status was not #readOnly or #success,
 the following elements are pairs of aKey,anArray  with each aKey
 describing the kind of conflict.  Empty sets are omitted from the result.

     Key                Conflicts
 Read-Write          StrongReadSet and WriteSetUnion conflicts.
 Write-Write         WriteSet and WriteSetUnion conflicts.
 Write-Dependency    WriteSet and DependencyChangeSetUnion conflicts.
 Write-WriteLock     WriteSet and WriteLockSet conflicts.
 Write-ReadLock      WriteSet and ReadLockSet conflicts.
 Rc-Write-Write      Logical write-write conflict on reduced conflict object.
 WriteWrite_minusRcReadSet  ((WriteSet and WriteSetUnion conflicts) - RcReadSet)

 see also System>>transactionConflicts for more documentation.
"

^ self _conflictsReport: false withDetails: false
%

category: 'Deprecated'
classmethod: System
contentsOfServerDirectory: aSpecString

"Obsolete: Use the GsFile>>contentsOfDirectory: onClient:  method instead."

| result |
self deprecated: 'System class>>contentsOfServerDirectory: deprecated v3.0.
Replace with GsFile class >> contentsOfDirectory:onClient:'.
result := GsFile contentsOfDirectory: aSpecString onClient: false.
result ifNil: [ ^ { }  ].
^ result
%

category: 'Session Control'
classmethod: System
continuationsEnabled

"Returns true if Smalltalk (i.e. Seaside style) continuations are enabled
 for this session.
 Continuations are enabled by default at the start of a new session."

^ self _zeroArgPrim: 114
%

category: 'Transaction Control'
classmethod: System
continueTransaction

"Updates the session's view to the most recently committed GemStone state.

 If in a nested transaction (System transactionLevel > 1) ,
 an error is signalled and no change is made to the transaction state.

 If the session is in an outer level transaction (System transactionLevel == 1)
 this method preserves modifications made to committed GemStone objects.
 The read and write sets of the session are carried forward and continue
 to accumulate until the session either commits or aborts.

 If the session is not in a transaction, this method performs an abort,
 discarding any accumulated changes to committed objects.

 If in an outer level transaction and a
 previous attempt to commit the transaction
 failed due to conflicts, then continueTransaction will generate
 error 2409.  After a failed attempt to commit, you must abort before
 continueTransaction can be used again.

 Returns true if accumulated modifications to the committed objects would not
 cause concurrency conflicts; otherwise returns false.  If the method
 returns false, you can call the transactionConflicts method to determine
 the nature of the conflicts.

 This method can be used whether or not the session is outside of a transaction.
 Of course, the session cannot commit accumulated changes unless it is
 inside a transaction.

 If transaction mode is #manualBegin, then continueTransaction does not alter
 the inside/outside of transaction state of the session.

 Modifications made by other committed transactions are accumulated for
 retrieval by GciDirtyObjs() and GciDirtySavedObjs() just as they are
 accumulated for commitTransaction or abortTransaction.

 This method has no effect on object locks.  Locks in the release locks sets
 are not released."

| continueResult |

(self _zeroArgPrim: 5) > 1 ifTrue:[
  ImproperOperation signal:'operation illegal while in a nested transaction'
].
continueResult := self _zeroArgPrim: 9.

continueResult <= 0 ifTrue: [
  continueResult >= -1 ifTrue:[
    ^ true  "read only or success"
  ].
] ifFalse: [
  continueResult == 1 ifTrue: [
    ^ self _resolveRcConflicts .
  ] ifFalse: [
    ^ false "validation failure"
    "3 means validation failure,
    4 means maximum number of commit retries was reached,
    5 means commit disallowed (usually because of indexes)"
  ]
].
self _uncontinuableError "logic error if we get here"
%

category: 'Clustering'
classmethod: System
currentClusterBucket

"Returns the instance of ClusterBucket that is the current default."

^ AllClusterBuckets at: self currentClusterId
%

category: 'Clustering'
classmethod: System
currentClusterId

"This method returns a SmallInteger that is the ID of the ClusterBucket that
 is the current default bucket."

<primitive: 96>
self _primitiveFailed: #currentClusterId .
self _uncontinuableError
%

category: 'Deprecated'
classmethod: System
currentGcReclaimSessionsByExtent

"Deprecated - all extents are covered by a single reclaim gem.
Return an array where the size is the number of extents, and in which
each element of the array is the session ID for the reclaim gem for that
extents. A 0 means no reclaim gcgems are running; 1 is used in v3.2 to
indicate that the extent has reclaim coverage."

self deprecated: 'System class>>currentGcReclaimSessionsByExtent deprecated in v3.2.'.
^(Array new: SystemRepository numberOfExtents)
     atAllPut: (self reclaimGemSessionCount max: 1);
     yourself.
%

category: 'Runtime Configuration Access'
classmethod: System
currentObjectReadLogFile
 "Answer a string containing the current object read log file which the stone
 is writing to, or nil if the object read logging feature is disabled."
   self _objectReadLogInfo ifNotNil:[:arr | ^ arr at: 1 ].
   ^ nil
%

category: 'Session Control'
classmethod: System
currentObjectSecurityPolicy

"Returns the GsObjectSecurityPolicy in which objects created in the current session
 are stored.  The result can be nil, in which case objects are
 being created with World write  permission.

 At login, the current security policy is the default security policy of the
 UserProfile for the session of the sender."

<primitive: 341>
self _primitiveFailed: #currentObjectSecurityPolicy .
^ nil
%

category: 'Session Control'
classmethod: System
currentObjectSecurityPolicy: anObjectSecurityPolicy

"Redefines the GsObjectSecurityPolicy in which subsequent objects created in the
 current session will be stored.  Returns the receiver.

 If the argument is nil , subsequent objects are created
 with World write permission.

 The argument must be a committed GsObjectSecurityPolicy and you must have
 writeAuthorization to that objectSecurityPolicy, otherwise an error is generated.
"

<primitive: 339>
anObjectSecurityPolicy _validateClass: GsObjectSecurityPolicy.
self _primitiveFailed: #currentObjectSecurityPolicy:
     args: { anObjectSecurityPolicy } .
self _uncontinuableError
%

category: 'Deprecated'
classmethod: System
currentSegment

self deprecated: 'System class>>currentSegment deprecated v3.0.
Use #currentObjectSecurityPolicy instead'.
^self currentObjectSecurityPolicy.
%

category: 'Deprecated'
classmethod: System
currentSegment: anObjectSecurityPolicy

self deprecated: 'System class>>currentSegment: deprecated v3.0.
Use #currentObjectSecurityPolicy: instead'.
^self currentObjectSecurityPolicy: anObjectSecurityPolicy
%

category: 'Session Control'
classmethod: System
currentSessionCount

"Return a SmallInteger which is the number of sessions present in the system,
 including the Symbol Gem, garbage collection sessions, but not the page manager
 session.  Gets the result from the stone process."

^self _zeroArgPrim: 49
%

category: 'Session Control'
classmethod: System
currentSessionNames

"Returns a formatted String containing, for each current GemStone session, the
 session number and userId.

 This method requires SessionAccess privilege if there is more
 than one session logged in."

| lf result profiles |
result := String new.
profiles := self _currentSessionProfiles.  "sessId, userProfile pairs"
lf := Character lf.

1 to: profiles size by: 2 do:[:j| | aSessionNumber aUserProfile |
   aSessionNumber := profiles at: j .
   aUserProfile := profiles at: j + 1 .
   j > 1 ifTrue:[ result add: lf ].
   result
     addAll: 'session number: ';
     addAll: (aSessionNumber asString) .
   aUserProfile ifNotNil:[ result addAll: (self _sessionUproString: aUserProfile) ]
                ifNil:[    result addAll: '    in login' ].
].
^ result
%

category: 'Session Control'
classmethod: System
currentSessions

"Returns an Array of SmallIntegers corresponding to all of the sessions
 currently running on the GemStone system.  Excludes sessions in before-login
 or after-logout states."

^self _zeroArgPrim: 85
%

category: 'Session Control'
classmethod: System
currentSessionsReport
  ^ self _sessionsReport: self currentSessions .
%

category: 'Transaction Control'
classmethod: System
currentTransactionHasWDConflicts

"Returns a Boolean indicating if the current transaction has one or
 more write-dependency conflicts.  A result of true indicates a commit
 will most likely fail.  A result of false indicates no write-dependency
 conflicts exist.

 It is inefficient to invoke this method and then invoke
 currentTransactionWDConflicts. If you want conflict details, invoke
 currentTransactionWDConflicts directly.
 "

 ^ self currentTransactionWDConflicts size ~~ 0
%

category: 'Transaction Control'
classmethod: System
currentTransactionHasWWConflicts

"Returns a Boolean indicating if the current transaction has one or
 more write-write conflicts.  A result of true indicates a commit
 will most likely fail.  However some RC objects have conflict
 resolution mechanisms which could allow a commit to suceed.
 A result of false indicates no write-write conflicts exist.

 It is inefficient to invoke this method and then invoke
 currentTransactionWWConflicts. If you want conflict details, invoke
 currentTransactionWWConflicts directly.
 "

 ^ self currentTransactionWWConflicts size > 0
%

category: 'Transaction Control'
classmethod: System
currentTransactionWDConflicts

"Returns an Array of objects which have write-dependency conflicts.
 The array is created by building the union of all dependency
 change sets from all commit records created since the session's
 current transaction and intersecting it with the session's write
 set."

 | status conflictDict wdConflicts |
 status := self _validateTransaction .
 status <= 0 ifTrue:[
   ^ #()  "read only or empty conflicts"
 ].
 conflictDict := self transactionConflicts .
 wdConflicts := conflictDict at: #'Write-Dependency' otherwise: #() .
 ^ wdConflicts .
%

category: 'Transaction Control'
classmethod: System
currentTransactionWWConflicts

"Returns an Array of objects which have write-write conflicts.
 The array is created by building the write set union of all
 commit records created since the session's current transaction
 and intersecting it with the session's write set."

 | status conflictDict wwConflicts |
 status := self _validateTransaction .
 status <= 0 ifTrue:[
   ^ #()  "read only or empty conflicts"
 ].
 conflictDict := self transactionConflicts .
 wwConflicts := conflictDict at: #'Write-Write' otherwise: #() .
 ^ wwConflicts .
%

category: 'Session Control'
classmethod: System
currentUserSessionCount

"Return a SmallInteger which is the number of user sessions present in the system.
 Garbage collection sessions, the Symbol Gem, and the Page Manager Gem are not
 included in the count.  Gets the result from the stone process."

^self _zeroArgPrim: 48
%

category: 'System Control'
classmethod: System
deadNotReclaimedCount

"Return the number of dead not reclaimed objects in the system."
^self stoneCacheStatisticWithName: 'DeadNotReclaimedObjs'
%

category: 'Transient Session State'
classmethod: System
decrementIndexProgressCountBy: aSmallInt

"Decrements the value of the indexProgressCount statistic by aSmallInt and
 returns the new value."

^ self _sessionCacheStatAt: -2 incrementBy: aSmallInt negated
%

category: 'Transient Session State'
classmethod: System
decrementProgressCountBy: aSmallInt

"Decrements the value of the progressCount statistic by aSmallInt and
 returns the new value."

^ self _sessionCacheStatAt: -1 incrementBy: aSmallInt negated
%

category: 'Deprecated'
classmethod: System
deleteServerFile: aFileSpec

self deprecated: 'System class>>deleteServerFile: deprecated v3.0. Use GsFile class>>removeServerFile: instead.'.
^self _deleteServerFile: aFileSpec.
%

category: 'Session Control'
classmethod: System
descriptionOfSession: aSessionId

"Returns an Array describing the session as follows:
 Most elements are zero or a String of size zero if no session exists for aSessionId.

 1.  The UserProfile of the session; nil if the UserProfile is recently
     created and not visible from this session's transactional view
     or the session is in login or processing, or has logged out.
 2.  A SmallInteger, the process ID of the Gem or topaz -l process, zero if no
     process registered with stone.
 3.  The hostname of the machine running the Gem process. Specifically,
     the peer's hostname as seen by stone, for the gem to stone network
     connection used for login.
     (a String, limited to 127 bytes).
 4.  Primitive number in which the Gem is executing, or 0 if it is not executing
     in a long primitive.
 5.  Time of the session's most recent beginTransaction, commitTransaction, or
     abortTransaction (from System timeGmt).
 6.  The session state (a SmallInteger).
 7.  A SmallInteger whose value is -1 if the session is in transactionless mode,
     0 if it is not in a transaction and 1 if it is in a transaction.
 8.  A Boolean whose value is true if the session is currently referencing the
     oldest commit record, and false if it is not.
 9.  The session's serial number (a SmallInteger).
 10. The session's sessionId (a SmallInteger), zero if no session exists.
 11. A String containing the ip address of host running the GCI process.
     If the GCI application is remote,  the peer address as seen by
     the gem of the GCI app to gem network connection.
     For a hostagent, this is the ip address of the remote host being serviced,
     otherwise if the GCI application is linked (using libgcilnk*.so or gcilnk*.dll)
     this is the peer's ip address as seen by stone, for the gem to
     stone network connection used for login.
 12. The priority of the session (a SmallInteger).
 13. Unique host ID of the host where the session is running (an Integer)
 14. Time of the session's most recent request to stone (from System timeGmt)
 15. Time the session logged in (from System timeGmt)
 16. Number of commits which have occurred since the session obtained its view.
 17. Nil or a String describing a system or gc gem .
 18. Number of temporary (uncommitted) object IDs allocated to the session.
 19. Number of temporary (non-persistent) page IDs allocated to the session.
 20. A SmallInteger, 0 session has not voted, 1 session voting in progress,
     2 session has voted, or voting not active.
 21. A SmallInteger, processId of the remote GCI client process,
     or -1 if the session has no remote GCI client .
 22. The KerberosPrincipal object used for passwordless login to the session,
     or nil if passwordless login was not used.
 23. The sessionId of the hostagent session through which this session is
     communicating to stone, or -1 if session is not using a hostagent .
 24. SmallInteger listening port if this session is a hostagent, or -1 .

 Because a session can update its commit record without committing a
 transaction, it is possible that no session actually references the oldest
 commit record.  Therefore, the eighth element may be false for all current
 sessions.

 To execute this method for any session other than your current session, you
 must have the SessionAccess privilege."

^ self _descriptionOfSessionSerialNum: 0 sessionId: aSessionId
%

category: 'Session Control'
classmethod: System
descriptionOfSessionSerialNum: aSerialNumber

"Returns an Array describing the session identified by aSerialNumber.

 See System (C) | descriptionOfSession: for documentation on the contents of the
 result Array.

 Requires SessionAccess privilege if aSerialNumber is not the current session."

^ self _descriptionOfSessionSerialNum: aSerialNumber sessionId: -1
%

category: 'Transaction Control'
classmethod: System
detailedConflictReportString
  "Return a String describing which commits by other sessions caused this
   sessions most recent commit attempt to fail."
| arr result printObjsBlk |
self _gemCommitConflictDetails == true ifFalse:[
  ^ 'No details.  You must execute
      System gemConfigurationAt: #GemCommitConflictDetails put: true
   prior to attempting commit in order to get per-session conflict details.'
].
result := String new .
arr := self _conflictsReport: false withDetails: true .
(arr at: 1) == #success ifTrue:[
  ^ 'Commit succeeded' copy lf .
].
(arr at: 1) == #readOnly ifTrue:[
  ^ 'Commit was readOnly' copy lf .
].
printObjsBlk := [:objs :isRcDetail | | nO limit str maxO |
  str := '  ('  copy .
  limit := (nO := objs size) min: (maxO := 50) .
  1 to: limit do:[ :n | | each didObj  |
    each := objs at: n .
    str add:' ' ; add: each asOop asString; add: $( .
    isRcDetail ifTrue:[
      each isSymbol ifTrue:[ str add: each printString . didObj := true ] ifFalse:[
      each _isOneByteString ifTrue:[ str add: (each copyFrom: 1 to: (each size min:100)).
                                    didObj := true ]].
    ].
    didObj ifNil:[ str add: 'a '; add: each class name ].
    str add: $) .
  ].
  nO > maxO ifTrue:[ str addAll:' ('; addAll: (nO - maxO)  asString;
                         addAll:' more oops)' ].
  str add: $) ;  lf .
  str
  ].

result add: 'Commit failed , ' ; add: (arr at: 1) asString ; lf .
result add: ((arr at: 2) ifNil:['(Time of attempted commit not available)']);
       lf .
3 to: arr size by: 3 do:[:j | | aSym objArr sessArr nObj |
  aSym :=   arr at: j .
  objArr := arr at: j + 1 .
  sessArr := arr at: j + 2 .
  (nObj := objArr size) > 0 ifTrue:[ | nOtherSess |
     result add: nObj asString; add: $  ; add: aSym ; add:' Conflicts' ; lf .
     result add: ( printObjsBlk value: objArr
                            value: (aSym at: 1 equals: 'Rc-Retry-Failure') ).
     nOtherSess := sessArr size // 2 .
     nOtherSess > 0 ifTrue:[
       result add:'  '; add: nOtherSess asString ; add: ' commits by other sessions'; lf .
       1 to: sessArr size by: 2 do:[:k |
         result add:'  '; add: (sessArr at: k) ; lf .
         result add:'  '; add: ( printObjsBlk value: (sessArr at: k + 1) value: false).
       ].
     ].
  ].
].
^ result
%

category: 'Transaction Control'
classmethod: System
dirtyListId

"Return a SmallInteger identifying the currently active dirtyList.
 Result is always zero."

^ self _zeroArgPrim:28
%

category: 'Signals'
classmethod: System
disableAlmostOutOfMemoryError

"Disables raising an error error when session's temporary object memory
 is almost full.

 See signalAlmostOutOfMemoryThreshold:  for more details. "

self _updateSignalErrorStatus: 5 toState: -1
%

category: 'Session Control'
classmethod: System
disableCommits

"Disables further commits by the session.  See the method #disableCommitsWithReason:
 for more information."

^ self disableCommitsWithReason: nil
%

category: 'Session Control'
classmethod: System
disableCommitsUntilAbortWithReason: aString

"Prevents this session from performing any further commits until the session
 performs an abort.  Attempting to commit a transaction after this method is
 successfully executed before aborting will raise an #rtErrCommitDisallowed
 error and will include 'aString' as the reason.

 aString must be an instance or subclass of String, or nil.  If aString is nil,
 a default reason of 'Commits disabled by user' will be used.

 Care should be taken when using this method.  If the session has any uncommitted
 changes, those changes will be lost after this method is executed.  Use the
 #needsCommit method to determine if uncommitted changes exist.

 This method returns true on success.  A return of false indicates commits were
 previously disabled by the user or by the system."

^ self _disableCommitsWithReason: aString untilAbort: true
%

category: 'Session Control'
classmethod: System
disableCommitsWithReason: aString

"Prevents this session from performing any further commits.  Attempting to
 commit a transaction after this method is successfully executed will raise
 an #rtErrCommitDisallowed error and will include 'aString' as the reason.

 This method disables commits only for the current session, not permanently.
 Commits may be performed if the session logs out and logs in again.  Use the
 UserProfile>>disableCommits method to permanently disable commits for a
 UserProfile.

 aString must be an instance or subclass of String, or nil.  If aString is nil,
 a default reason of 'Commits disabled by user' will be used.

 Care should be taken when using this method.  If the session has any uncommitted
 changes, those changes will be lost after this method is executed.  Use the
 #needsCommit method to determine if uncommitted changes exist.

 This method returns true on success.  A return of false indicates commits were
 previously disabled by the user or by the system."

^ self _disableCommitsWithReason: aString untilAbort: false
%

category: 'Garbage Collection Management'
classmethod: System
disableEpochGc
"Disables epoch garbage collection from running and resets
 the epoch GC state.  Appends the new configuration state
 to the stones configuration file.

 No further epoch GC operations will be run after this method is
 successfully executed.  An epoch GC operation already in progress
 when this method is executed will not be interrupted.  Has no
 effect if epoch GC is already disabled.

 Requires the GarbageCollection privilege."

^self stoneConfigurationAt: #StnEpochGcEnabled put: 0.
%

category: 'Transaction Control'
classmethod: System
disableSignaledAbortError

"Disables the generation of an error when Stone signals the Gem session that it
 should abort when running outside of a transaction."

self _updateSignalErrorStatus: 2 toState: false
%

category: 'Transaction Control'
classmethod: System
disableSignaledFinishTransactionError

"Disables the generation of the TransactionBacklog notification ."

self _updateSignalErrorStatus: 4 toState: false
%

category: 'Notification'
classmethod: System
disableSignaledGemStoneSessionError

"Set the current GemStone session so that it cannot receive signals from
 other GemStone sessions."

self _updateSignalErrorStatus: 3 toState: false
%

category: 'Notification'
classmethod: System
disableSignalTranlogsFull

"Disables generation of error 2339 to this session when stone detects
 a tranlogs full condition."

self _updateSignalErrorStatus: 6 toState: false
%

category: 'Session Control'
classmethod: System
disableStoneGemTimeout

"Prevents the current session from being killed due to inactivity
 (lack of communication with the stone).

 Use the #enableStoneGemTimeout method to re-enable the stone gem timeout.

 Returns true if the action was successful or false if the stone gem timeout
 was previously disabled for this session.

 Refer to the STN_GEM_TIMEOUT configuration parameter for more information.

 This sets the PrimitiveNumber statistic for this session to 9999 .
"

^ self _zeroArgPrim: 120
%

category: 'Session Control'
classmethod: System
effectiveUserId

"Returns a SmallInteger indicating the effective Unix user ID of the gem process."
^ self _zeroArgPrim: 169
%

category: 'Session Control'
classmethod: System
effectiveUserIdName

"Returns a String indicating the effective Unix user ID of the gem process."
^ self _zeroArgPrim: 170
%

category: 'Signals'
classmethod: System
enableAlmostOutOfMemoryError

"Enables or reenables error when session's temporary object memory
 is almost full , with previous threshold .

 See also AlmostOutOfMemory(C)>>enable: .
 You must install a handler for AlmostOutOfMemory to take action
 or execute
   Notification enableSignalling
 to have AlmostOutOfMemory and other Notifications signalled to the
 application to avoid having AlmostOutOfMemory silently ignored.

 See signalAlmostOutOfMemoryThreshold:  for more details.

 This method or  signalAlmostOutOfMemoryThreshold:  must be
 invoked after each delivery of the
 AlmostOutOfMemory error to reenable generation of the error.
 "

self _updateSignalErrorStatus: 5 toState: 0
%

category: 'Garbage Collection Management'
classmethod: System
enableEpochGc
"Enables epoch garbage collection to run and resets
 the epoch GC state.  Appends the new configuration state
 to the stones configuration file.

 Has no effect if epoch GC is already enabled.

 Requires the GarbageCollection privilege."

^self stoneConfigurationAt: #StnEpochGcEnabled put: 1.
%

category: 'Transaction Control'
classmethod: System
enableSignaledAbortError

"Enables the generation of an error when the Stone has signaled that the Gem
 process should abort to connect to a more current GemStone root.

 This method must be invoked after each delivery of the signal-abort error, to
 reenable generation of the error.

 If invoked when in a transaction, the new state will take effect
 after the next commit or abort which exits the transaction.
 "

self _updateSignalErrorStatus: 2 toState: true
%

category: 'Transaction Control'
classmethod: System
enableSignaledFinishTransactionError

"Enables the generation of an error when the Stone has signaled that a Gem
 process which is in-transaction should abort, commit, or continueTransaction
 to move to a newer transactional view.

 This method must be invoked after each delivery of the TransactionBacklog
 notification to reenable generation of the error."

self _updateSignalErrorStatus: 4 toState: true
%

category: 'Notification'
classmethod: System
enableSignaledGemStoneSessionError

"Enable the current GemStone session to receive signals from other GemStone
 sessions.  One GemStone session receives a signal from another session when a
 InterSessionSignal exception is signalled.

 The receiving session processes the signal with an exception handler.  When
 GemStone raises one signal exception, it also disables further signal
 exceptions, to allow the exception handler to run without receiving another
 interrupt.  The exception handler should therefore re-enable signal exceptions
 when it is done with its other processing.

 A signal is not exactly an interrupt, and it does not automatically awaken an
 idle session.  Both the GemStone Smalltalk virtual machine and GemBuilder for C
 can raise the signal exception.  But the process of the session must activate
 the virtual machine or interface before the signal can be received."

self _updateSignalErrorStatus: 3 toState: true
%

category: 'Notification'
classmethod: System
enableSignalTranlogsFull

"Enables generation of error 2339 to this session when stone detects
 a tranlogs full condition."

self _updateSignalErrorStatus: 6 toState: true
%

category: 'Session Control'
classmethod: System
enableStoneGemTimeout

"Re-enables the stone gem timeout for the current session after it was previously
 disabled by the #disableStoneGemTimeout method.

 Returns true if the action was successful.  Returns false if the stone gem timeout
 was already re-enabled or if it was not disabled for this session.

 Refer to the STN_GEM_TIMEOUT configuration parameter for more information."

^ self _zeroArgPrim: 121
%

category: 'Host System Access'
classmethod: System
encryptPassword: pw withSalt: salt

"Calls the C function crypt() to encrypt pw, which is assumed to be the
 clear text password to be encrypted.  Returns a new String which contains
 the encrypted password.

 Both the pw and salt arguments must be instances of String."

<primitive: 865>
pw _validateClass: String .
salt _validateClass: String .
^self _primitiveFailed: #encryptPassword:withSalt: args: { pw . salt }
%

category: 'Garbage Collection Management'
classmethod: System
ensureGcRunning
 "Makes sure that the reclaimGem and adminGem are running."

(self reclaimGemSessionCount == 0) ifTrue:[
  self startReclaimGem.
  ].
((SystemRepository restoreStatusInfo at: 2) == 0 and: [self adminGemSessionId == 0])
 ifTrue:[
    System startAdminGem .
  ].
^System waitForAllGcGemsToStartForUpToSeconds: 60 .
%

category: 'Transaction Control'
classmethod: System
enumerateDirtyList: listId

"Returns an Array which is the result of enumerating
 the dirtyList(s) specified by listId.
 listId = -1  specifies the currently active dirtyList .

 For listId >= 0,   listId + 1 is the  transaction level being enumerated,
 listId == 0 is the outer level transaction, listId == 1 is first level
 of nested transactions.
 When entering a nested transaction, the dirtyList of that transaction
 is initialized to be the closure of the dirtyList of the parent level.

 Example: to enumerate the closure of System _writtenObjects, 
   System beginNestedTransaction .
   closureList := System enumerateDirtyList: -1 .
   System commit .
"

^ self _dirtyListOp: 2 id: listId
%

category: 'Host System Access'
classmethod: System
fetchLinkedGciLibraryName

"Returns a string containing the name of the linked 64-bit GCI shared library
 file for this version of the product.
 The string does not include a path to the file, only the file name."

^ self _zeroArgPrim: 152
%

category: 'Host System Access'
classmethod: System
fetchRpcGciLibraryName

"Returns a string containing the name of the RPC 64-bit GCI shared library
 file for this version of the product.
 The string does not include a path to the file, only the file name."

^ self _zeroArgPrim: 153
%

category: 'Host System Access'
classmethod: System
fetchSystemStatNames

"On systems that support it, returns an array of Strings which describe
 the statistics returned by the #fetchSystemStats method.  The length
 of the result array is host technology dependent.

 Returns nil if the host system does not support system statistics."

^ self _zeroArgPrim: 117
%

category: 'Host System Access'
classmethod: System
fetchSystemStats

"On systems that support it, returns an array of Numbers which describe
 the statistics returned by the #fetchSystemStats method.  The length
 of the result array is host technology dependent.  A description of
 each element in the array can be obtained using the #fetchSystemStatNames
 method.

 While most elements in the result array will be SmallIntegers, the
 result may also contain other types of Numbers such as SmallDoubles,
 Floats, LargeInteger, etc.

 Returns nil if the host system does not support system statistics."

^ self _zeroArgPrim: 118
%

category: 'Private'
classmethod: System
flushAllExtents

"Deprecated, has no effect, extents are only flushed to file system
 by the stone process at checkpoints."

self deprecated: 'System class>>flushAllExtents deprecated in v3.2. flush is done at checkpoints'.
^ self
%

category: 'System Control'
classmethod: System
flushObjectReadBuffer
  "Forces the gem to immediately send any buffered object read records to the stone.
   Has no effect if object read logging is not enabled.
   Returns the receiver."

  ^ self _zeroArgPrim: 192
%

category: 'Garbage Collection Management'
classmethod: System
forceEpochGc
"Force an Epoch GC to run as soon as possible, regardless of the setting
of the GcUser configuration parameters epochGc[Time|Trans]Limit.
Note that the stone configuration parameter STN_EPOCH_GC_ENABLED must be
TRUE for this method to have any effect.

This method will fail and return false under the following conditions:
  *Checkpoints are suspended
  *Another garbage collection operation is in progress
  *Unfinalized possible dead objects exist (i.e., System>>voteState
    returns any value except 0).
  *The system is in restore mode.
  *The Admin GC session is not running.
  *Epoch GC is not enabled (STN_EPOCH_GC_ENABLED is set to FALSE)
  *The system is performing a reclaimAll.
  *A previous forceEpochGc operation was performed and the epoch has
   not yet started or completed.

If successful, this method sets the stone cache statistic EpochForceGc
to 1.  Once the Admin GcGem has started the epoch GC, EpochForceGc
will return back to 0.

This method returns true if the epoch GC was started, false if not.

You must have GarbageCollection privilege to be able to run this method."

^self _zeroArgPrim: 41
%

category: 'Host System Access'
classmethod: System
fullyQualifiedDomainName

"Returns a string representing the fully qualified domain name (FQDN) of the
 host system for this session.  Raises an exception if the FQDN could not
 be determined."

^ self _zeroArgPrim: 182
%

category: 'Garbage Collection Management'
classmethod: System
gcConfigurationReport

"Returns a string describing the current garbage collection settings.
 Sender must have permission to read GcUsers UserGlobals."

| gcug keys result lf |

result := String new.
lf := Character lf.
result addAll: 'Reclaim and Admin GC gem settings:' ; add: lf.
gcug := ((AllUsers userWithId: 'GcUser') resolveSymbol: #UserGlobals) value.
(keys := gcug keys)
	remove: #NativeLanguage ifAbsent: [] ;
	remove: #UserGlobals ;
	remove: #GcUser ;
	remove: #defaultAdminConfigValues ;
	remove: #defaultReclaimConfigValues.

keys do:[:eachSymbol| | val |
  val := gcug at: eachSymbol.
  result add: $#; addAll: eachSymbol asString; addAll: ' -> '; addAll: val asString; add: lf.
].
result add: lf; addAll: 'Stone GC settings:' ; add: lf.
result add: $# ; add: 'StnEpochGcEnabled'; addAll: ' -> ' .
(System stoneConfigurationAt: #StnEpochGcEnabled) == 1
  ifTrue:[result addAll: 'true']
  ifFalse:[result addAll: 'false'].
result add: lf.

^result
%

category: 'Runtime Configuration Access'
classmethod: System
gciClientConnectionIsEncrypted

"Answer a Boolean indicating whether the network connection to the gem's GCI client
 encrypted (using TLS).  Always answers false for gems linked gems and for gems running
 on the same host as the GCI client."

^ self _zeroArgPrim: 177
%

category: 'Gci Set Support'
classmethod: System
gciDirtyObjsInit

"Enable GCI tracking of dirty objects, equivalent
 to GciDirtyObjsInit()."

^ self _gciDirtyInit: 22
%

category: 'Gci Set Support'
classmethod: System
gciTrackedObjsInit

"Enable GCI tracking of dirty objects, equivalent to
 GciTrackedObjsInit(). "

^ self _gciDirtyInit: 23
%

category: 'Cache Statistics'
classmethod: System
gemCacheStatisticsForSessionId: aSessionId

"Same as the cacheStatisticsAt: method except the argument is the session ID
 of a session currently connected to the shared memory cache.  In systems
 that use multiple shared memory caches, the session must exist on the same
 cache as the session invoking this method.

 Only cache statistics applicable to a Gem process are returned.
 The description of these statistics can be determined by evaluating:
   'System cacheStatisticsDescriptionForGem'

 nil is returned if the session cannot be located."

(aSessionId == 1 and:[ GsSession currentSession isSolo ]) ifTrue:[
  ^ self _primCacheStatsForSlotOrSession: nil opCode: 11 .
].
^ self _primCacheStatsForSlotOrSession: aSessionId negated opCode: 1
%

category: 'Configuration File Access'
classmethod: System
gemConfigurationAt: aName

"Returns the value of the specified configuration file parameter from the
 current session.  Returns nil if that parameter is not applicable to a Gem."

| cfgId |
cfgId := ConfigurationParameterDict at: aName otherwise: nil .
cfgId == nil ifTrue:[ ^ nil ].
^ self _configurationAt: cfgId isStone: false kind: $C
%

category: 'Runtime Configuration Access'
classmethod: System
gemConfigurationAt: aName put: aValue

"Changes the value of the specified Gem configuration parameter.

 See comments in the method configurationAt:put: for complete documentation."

^ self configurationAt: aName put: aValue
%

category: 'Configuration File Access'
classmethod: System
gemConfigurationFileNames
"Answer a SymbolDictionary containing two keys-value pairs:
  #GEMSTONE_SYS_CONF -> (String) Filename of the gem's system configuration file.
  #GEMSTONE_EXE_CONF -> (String) Filename of the gem's executable configuration file.

 If either file does not exist, the value will be an empty String."

|array result|
array := self gemConfigurationAt: #GemConfigFileNames .
result := SymbolDictionary new.
result at: #GEMSTONE_SYS_CONF put: (array at: 1) ;
       at: #GEMSTONE_EXE_CONF put: (array at: 2) .
^ result
%

category: 'Configuration File Access'
classmethod: System
gemConfigurationReport

"Returns a SymbolDictionary whose keys are the names of configuration file
 parameters, and whose values are the current settings of those parameters in
 the current session's Gem process.  Parameters that are not applicable to Gem
 and those that are undefined are not included in the result."

^ self _configurationReport: false
%

category: 'Environment Access'
classmethod: System
gemEnvironmentVariable: varName

"Expands the environment variable named varName in the Gem
 process, returning a String. varName should be a kind of String.

 Returns nil if any of the following are true:

 * varName is not a byte format object.
 * There is no environment variable defined with name varName.
 * The value of the environment variable is more than approximately 8000 bytes.
 * The size of varName exceeds approximately 8000 bytes.

 Signals an error if NoGsFileOnServer privilege is set in the UserProfile.
"

^ GsFile _expandEnvVariable: varName isClient: false
%

category: 'Environment Access'
classmethod: System
gemEnvironmentVariable: varName put: valueStr

"Sets the environment variable named varName in the Gem
 to value valueStr .
 Both arguments must be either a String or a Unicode7 not larger than 8000 bytes.
 The NoGsFileOnServer privilege must not be set in the session's UserProfile.

 Returns self or signals an Error if the operation failed.
"
GsFile _setEnvVariable: varName value: valueStr isClient: false.
^ self
%

category: 'Runtime Configuration Access'
classmethod: System
gemIsBigEndian

" Returns true if the gem process is running
  on a machine using big endian byte ordering "

^ self _zeroArgPrim: 106

%

category: 'Host System Access'
classmethod: System
gemLogFileName
  "Returns a String, the full path of the file to which the session's
   gem log file is being written.
   Returns an empty String in a topaz -l process.
   Each invocation of this method returns a new instance of String."

  ^ self _zeroArgPrim: 150
%

category: 'Host System Access'
classmethod: System
gemLogPath
  "Returns a String, the path of the directory in which the session's
   gem log file is being written.
   Returns an empty String in a topaz -l process.
   Each invocation of this method returns a new instance of String."

  ^ self _zeroArgPrim: 159
%

category: 'Host System Access'
classmethod: System
gemProcessId

  "Returns a SmallInteger, the processId from Unix getpid() of this session's
  gem or topaz -l  process ."

  ^ self _zeroArgPrim: 158
%

category: 'Version Management'
classmethod: System
gemVersionAt: aString

"Returns information about the Gem process of the current session.
 aString must be equal to a key in VersionParameterDict, otherwise nil
 is returned.  The semantics of these keys are:

 aString              meaning
 -------              -------
 cpuKind             detailed CPU type obtained at runtime: 'sun4u', 'x86_64'.
 cpuArchitecture     target CPU for which GemStone was compiled:
                       'SPARC', 'x86-64'.
 gsBuildArchitecture operating system name and CPU for which GemStone
                       was compiled: 'SPARC (Solaris)', 'x86-64 (Linux)'.
 gsBuildDate         time at which the Gem executable was compiled (a String).
 gsBuildType         usually FAST, but could be SLOW, PROFILE or NOOP
 gsRelease           major and minor version of GemStone, such as '3.0.0'.
 gsVersion           major version of GemStone, such as '3.0'.
 imageKind           a Symbol: #server.
 nodeName            network node name: 'speedy'.
 osName              operating system name: 'SunOS', 'Linux'.
 osRelease           release number of the operating system:
                       '5.10', '2.6.16.46-0.12-smp'.
 osVersion           vendor defined major version of the OS:
                       'Generic_141444-09', '#1 SMP Thu May 17 14:00:09 UTC 2007'.
 processId           operating system process identifier (an Integer): 13529.
 processorCount      number of processors on the machine running the process."

| verId |
aString = 'imageKind' ifTrue:[ ^ self imageVersionAt: aString ] .
verId := VersionParameterDict at: aString otherwise: nil .
verId ifNil:[ ^ nil ].

^ self _configurationAt: verId isStone: false kind: $V
%

category: 'Version Management'
classmethod: System
gemVersionReport

"Return a SymbolDictionary whose keys are the names of operating system,
 hardware, or GemStone version attributes, and whose values are the
 current values of those attributes in the Gem process."

^ self _serverVersionReport: false
%

category: 'Version Management'
classmethod: System
gemVersionReportString
  "Returns a String with keys and values of gemVersionReport one pair per line."
  ^ self gemVersionReport _reportString
%

category: 'Error Handling'
classmethod: System
genericSignal: errIdentifier text: aString

"Will be deprecated.
 Raise a user-defined signal with no arguments.

 The argument errIdentifier is a user-defined object, to distinguish user
 errors, and may be nil.  The argument aString appears in GemStone's error
 message for this error, and may be nil."

^ UserDefinedError new _number: 2318;
         reason: errIdentifier ; details: aString ; signal
%

category: 'Error Handling'
classmethod: System
genericSignal: errIdentifier text: aString arg: anArg

"Will be deprecated.
 Raise a user-defined signal with one argument.

 The argument errIdentifier is a user-defined object, to distinguish user
 errors, and may be nil.  The argument aString appears in GemStone's error
 message for this error, and may be nil. "

^ UserDefinedError new _number: 2318;
         reason: errIdentifier ; details: aString;
	args: { anArg } ; signal .
%

category: 'Error Handling'
classmethod: System
genericSignal: errIdentifier text: aString args: anArray

"Will be deprecated.
 Raise a user-defined signal.

 The argument errIdentifier is a user-defined object, to distinguish user
 errors, and may be nil.  The argument aString appears in GemStone's error
 message for this error, and may be nil.  "

^ UserDefinedError new _number: 2318;
     reason: errIdentifier ; details: aString ;
     args: (anArray  _isArray ifTrue:[ anArray ] ifFalse:[ { anArray } ]) ;
     signal .
%

category: 'Garbage Collection Management'
classmethod: System
getAdminConfig: configSymbol

"Returns the current runtime
 Valid configSymbols are:
    #adminVerboseLogging
    #epochGcMaxThreads
    #epochGcPageBufferSize
    #epochGcPercentCpuActiveLimit
    #epochGcTimeLimit
    #epochGcTransLimit
    #saveWriteSetUnionToFile
    #sweepWsUnionMaxThreads
    #sweepWsUnionPageBufferSize
    #sweepWsUnionPercentCpuActiveLimit
"

^self _gcGemConfig: false symb: configSymbol toValue: nil
%

category: 'Configuration File Access'
classmethod: System
getKeyfileAttributes

"Returns a SymbolDictionary which contains attributes of the current keyfile as follows:

   #allowsGbj -> (Boolean) true if GemBuilder for Java is allowed.
   #allowsGciTraversal -> (Boolean) true if GBS applications are allowed.
   #allowsGemConnect -> (Boolean) true if GemConnect is allowed.
   #allowsRemoteSpc -> (Boolean) true if remote shared page caches are allowed.
   #creationDate -> (DateTime) representing when the key was created.
   #customerName -> (String) name of the customer for which the keyfile was created.
   #fileName -> (String) full path to the current keyfile on the stone's host.
   #license ->  (String) license number of the keyfile.
   #machineKind -> (String) machine kind for the keyfile.
   #maxCpus -> (SmallInteger) maximum number of CPUs, or 0 for no limit.
   #maxDbfSizeMb -> (SmallInteger) maximum database size in megabytes, or 0 if unlimited.
   #maxOops -> (SmallInteger) maximum number of objects.
   #maxSpcSizeMb -> (SmallInteger) maximum shared page cache size in megabytes.
   #sessionLimit -> (SmallInteger) maximum number of sessions allowed.
   #sunsetDate -> (DateTime) when the keyfile expires, or nil if no expiry date is set."

^ self _zeroArgPrim: 181
%

category: 'Garbage Collection Management'
classmethod: System
getReclaimConfig: configSymbol

"Returns the current runtime value of the specified reclaim gem configuration option.
 Valid configSymbols are:
   #deferReclaimCacheDirtyThreshold
   #deadObjsReclaimedCommitThreshold
   #maxTransactionDuration
   #objsMovedPerCommitThreshold
   #reclaimDeadEnabled
   #reclaimMinFreeSpaceMb
   #reclaimMinPages
   #reclaimVerboseLogging
   #sleepTimeBetweenReclaimMs
   #sleepTimeWithCrBacklogMs
"
^self _gcGemConfig: true symb: configSymbol toValue: nil
%

category: 'Configuration File Access'
classmethod: System
getSharedCacheAttributes
"Returns a SymbolDictionary which contains attributes for the shared page cache to 
 which the current session is attached as follows:

   #cacheName ->              (String) Name of the cache
   #numFrames ->              (SmallInteger) Total number of 16Kb frames in the cache.
   #physicalSizeInBytes ->    (Integer) Physical size of the shared memory 
                                segment in bytes.
   #numSharedCounters ->      (SmallInteger) Number of shared counters in the cache 
                                (see config option SHR_PAGE_CACHE_NUM_SHARED_COUNTERS).
   #numProcesses ->           (SmallInteger) Maximum number of processes that can attach
                                the cache (see config option SHR_PAGE_CACHE_NUM_PROCS).
   #spinLockCount ->          (SmallInteger) Spin lock count 
                                (see config option SHR_SPIN_LOCK_COUNT).
   #largeMemoryPagePolicy ->  (SmallInteger) Large memory page policy 
                                (see config option SHR_PAGE_CACHE_LARGE_MEMORY_PAGE_POLICY).
   #largeMemoryPageSizeMb ->  (SmallInteger) Large memory page size in megabytes 
                                (see config option SHR_PAGE_CACHE_LARGE_MEMORY_PAGE_SIZE_MB).
   #largeMemoryPagesUsed ->   (SmallInteger) Number of large memory pages used 
                                to create the cache.
   #targetFreeFrameCount ->   (SmallInteger) Target free frame count 
                                (see config option SHR_TARGET_FREE_FRAME_COUNT).
   #cacheLocked ->            (Boolean) Cache locked in memory 
                                (see config option SHR_PAGE_CACHE_LOCKED).
   #isRemoteCache ->          (Boolean) true if the cache is remote from the stone.
   #isRemoteMidCache ->       (Boolean) true if the cache is a mid-level remote cache.

Returns nil if the current session is a solo session.
"

| list result |
list := self _sharedCacheAttributes .
list ifNil:[ ^ nil ].
result := SymbolDictionary new: 11 .
^ result at: #cacheName put: (list at: 1) ;
         at: #numFrames put: (list at: 2) ;
         at: #physicalSizeInBytes put: (list at: 3) ;
         at: #numSharedCounters put: (list at: 4) ;
         at: #numProcesses put: (list at: 5) ;
         at: #spinLockCount put: (list at: 6) ;
         at: #largeMemoryPagePolicy put: (list at: 7) ;
         at: #largeMemoryPageSizeMb put: (list at: 8) ;
         at: #largeMemoryPagesUsed put: (list at: 9) ;
         at: #targetFreeFrameCount put: (list at: 10) ;
         at: #cacheLocked put: (list at: 11) ;
	 at: #isRemoteCache put: (list at: 12) ;
	 at: #isRemoteMidCache put: (list at: 13) ;
         yourself
%

category: 'Global Session Statistics'
classmethod: System
globalSessionStatAt: index
"Returns the current value of a global session statistic.
 See the method System>>_updateGlobalSessionStat: index by: i overwrite: aBool
 for more information on global session statistics."

^self _updateGlobalSessionStat: index by: 0 overwrite: false
%

category: 'Global Session Statistics'
classmethod: System
globalSessionStatAt: index put: newVal
"Overwrites the value of the given global session statistic with newVal
 and returns the new value.  See the method
 System>>_updateGlobalSessionStat: index by: i overwrite: aBool
 for more information on global session statistics."

^self _updateGlobalSessionStat: index by: newVal overwrite: true
%

category: 'Garbage Collection Management'
classmethod: System
hasMissingGcGems
"Return a Boolean indicating if any garbage collection systems are
 missing from the system.  Answers true if either admin gem or
 reclaim gem is not running."

((SystemRepository restoreStatusInfo at: 2) == 0 and: [self adminGemSessionId == 0])
  ifTrue:[^true]. "missing the Admin gem"

(self reclaimGemSessionCount == 0)
  ifTrue:[^true]. "missing reclaim gem"

^false "all GC gems accounted for"
%

category: 'User-Defined Actions'
classmethod: System
hasUserAction: aSymbol

"Returns true if the user action named aSymbol is installed in this GemStone
 session.  Returns false otherwise."

<primitive: 105>
aSymbol _validateClass: String.
^ self _primitiveFailed: #hasUserAction: args: { aSymbol }
%

category: 'Session Control'
classmethod: System
hostAgentSessions

"Returns an Array of SmallIntegers which are stone sessionIds
 corresponding to all of the hostagent sessions currently running."

| haUp pairs res |
pairs := self _currentSessionProfiles.
haUp := AllUsers userWithId:'HostAgentUser'.
res := { } .
1 to: pairs size by: 2 do:[:j |
  (pairs at: j+1 ) == haUp ifTrue:[ | desc sesId |
    sesId := pairs at: j .
    desc := System descriptionOfSession: sesId .
    (desc at: 17) ifNotNil:[ "a hostagent process"
      res add: sesId
    ].
  ].
].
^ res
%

category: 'Session Control'
classmethod: System
hostAgentSessionsReport
| haUp pairs sess rpt |
pairs := self _currentSessionProfiles.
haUp := AllUsers userWithId:'HostAgentUser'.
sess := { } .
1 to: pairs size by: 2 do:[:j |
  (pairs at: j+1 ) == haUp ifTrue:[ sess add: (pairs at: j)].
].
rpt := String new .
sess do:[:sesId | | desc port leafIp |
  rpt add: 'session '; add: (sesId asString width: -3) ; add:'  '.
  desc := System descriptionOfSession: sesId .
  rpt add: ' gemPid: ', (desc at:2) asString ; add: '  '  .
  (desc at: 17) ifNotNil:[:elem |
    rpt add: elem .
    rpt add: (elem = 'midcache_hostagent' ifTrue:[ ' on: '] ifFalse:[ ' servicing: ' ]).
    rpt add: (leafIp := desc at: 11) .   "host"
    rpt add: $( ; add: (GsSocket getHostNameByAddress:leafIp); add: $) .
  ] ifNil:[ rpt add: '(normal login by HostAgentUser)' ].
  port := desc at: 24.
  port > 0 ifTrue:[ rpt add: ' listeningPort: '; add: port asString ].
  rpt lf .
].
^ rpt
%

category: 'Host System Access'
classmethod: System
hostCpuCount

"Returns a SmallInteger which is the number of CPUs on the host system.  In most cases,
 this number represents the total number of CPU cores on the host system."

^ self _zeroArgPrim: 135
%

category: 'Cache Statistics'
classmethod: System
hostCpuStatsForProcessId: anInt

"Return an Array of 2 integers as follows:
  array[1] - user mode CPU milliseconds
  array[2] - system mode CPU milliseconds

 Both array elements will be -1 if the process slot is out of range
 or not in use or if this method is not supported for the host
 architecture.

 Is not required to be attached to the shared page cache or even be
 a GemStone process.  The method will success for any process for which
 the gem session has permission to view its CPU usage statistics."

^ self _primCacheStatsForSlotOrSession: anInt opCode: 7
%

category: 'Cache Statistics'
classmethod: System
hostCpuStatsForProcessSlot: anInt

"For the process using the cache process slot anInt, return an Array of
 2 integers as follows:
   array[1] - user mode CPU milliseconds used
   array[2] - system mode CPU milliseconds used

 Both array elements are set to -1 if the process slot is out of range
 or not in use, or if this method is not supported for the host
 architecture."

^ self _primCacheStatsForSlotOrSession: anInt opCode: 6
%

category: 'Host System Access'
classmethod: System
hostCpuUsage

"Returns an Array of 5 SmallIntegers with values between 0 and 100 which
 have the following meanings:

 1 - Percent CPU active (user + system)
 2 - Percent CPU idle
 3 - Percent CPU user
 4 - Percent CPU system (kernel)
 5 - Percent CPU I/O wait

 On hosts with multiple CPUs, these figure represent the average across all
 processors.

 The values on first invocation or with invocations very close together may show 
 100% idle; these results are unreliable. 

 Returns nil if the host system does not support collecting CPU statistics or an
 error occurs."

^ self _zeroArgPrim: 130
%

category: 'Host System Access'
classmethod: System
hostId

"Return an Integer which represents a unique 64 bit identifier for the host
 where the session is running.  This identifier is created and managed by
 GemStone and is not related in anyway to the hostname or IP address of the
 host."

^ self _zeroArgPrim: 131
%

category: 'Host System Access'
classmethod: System
hostname

"Return a String which is the name of the host on which the session is running."
^ self _zeroArgPrim: 163
%

category: 'Host Process Statistics'
classmethod: System
hostProcessStatisticsNames

"Returns an array of Strings which are the names of the per-process statistics
 provided by this host."

^ self _primCacheStatsForSlotOrSession: nil opCode: 10

%

category: 'Host Process Statistics'
classmethod: System
hostStatisticsForMyProcess

"Returns an array of SmallIntegers which represent the host statistics for this process.
 The names of each statistic are returned by the #hostProcessStatisticsNames
 method."

^ self _primCacheStatsForSlotOrSession: -1 opCode: 10

%

category: 'Host Process Statistics'
classmethod: System
hostStatisticsForProcess: anInteger

"Returns an array of SmallIntegers which represent the host statistics for the given
 process ID.  The names of each statistic are returned by the #hostProcessStatisticsNames
 method.

 Returns nil if the process could not be found or if this process does not have
 permission to fetch the statistics."

^ self _primCacheStatsForSlotOrSession: anInteger opCode: 10

%

category: 'Session Control'
classmethod: System
iAmSystemUser

"Returns the true if the current session is logged in as SystemUser, otherwise false"

^ self myUserProfile isSystemUserProfile
%

category: 'Version Management'
classmethod: System
imageVersionAt: aSymbol

"Returns information about the GemStone kernel class image where aSymbol is one
 of the following:

 #gsBuildDate  DateTime of last kernel class filein or upgrade.
 #gsRelease    Version String of last kernel class filein or upgrade: '3.0.0'.
 #gsVersion    Major version of image, such as '3.0' .
 #imageKind    A Symbol: #server.

 If aSymbol is not one of the above, returns nil."

^ ImageVersion at: aSymbol otherwise: nil
%

category: 'Transaction Control'
classmethod: System
inContinueTransaction

"Returns a boolean indicating if the current transaction has been
 continued.  Answers true if System>>continueTransaction has been
 executed and no abort or successful commit has been performed.
 Also answer true if a commit was attempted and failed and no
 abort has been executed.  Otherwise returns false."

^ self _zeroArgPrim: 105
%

category: 'Global Session Statistics'
classmethod: System
incrementGlobalSessionStatAt: index by: value
"Increments the value of the given global session statistic by newVal
 and returns the new value.  See the method
 System>>_updateGlobalSessionStat: index by: i overwrite: aBool
 for more information on global session statistics."

^self _updateGlobalSessionStat: index by: value overwrite: false
%

category: 'Transient Session State'
classmethod: System
incrementIndexProgressCountBy: aSmallInt

"Increments the value of the indexProgressCount statistic by aSmallInt and
 returns the new value."

^ self _sessionCacheStatAt: -2 incrementBy: aSmallInt
%

category: 'Transient Session State'
classmethod: System
incrementProgressCountBy: aSmallInt

"Increments the value of the progressCount statistic by aSmallInt and
 returns the new value."

^ self _sessionCacheStatAt: -1 incrementBy: aSmallInt
%

category: 'Transaction Control'
classmethod: System
inTransaction

"Returns true to indicate that the session is in a transaction, false
 otherwise."

^ (self _zeroArgPrim: 5) ~~ 0
%

category: 'Session Control'
classmethod: System
killHostAgents

"Stops all HostAgent processes that are logged in to this stone.
 Each hostagent receives a fatal stop session Error.
 Signals an Error if unable to stop all hostagents."

^ self _stopHostAgents: 16r8"Fatal stop session" timeOut: 10
%

category: 'Session Control'
classmethod: System
killHostAgentSession: sessionId

 "Shutdown of the HostAgent with the specified sessionId with fatal Error."

^ self _stopHostAgentSession: sessionId kind: 16r8 "Fatal stop session"
%

category: 'Session Control'
classmethod: System
lastTransactionTimeForSessionWithId: aSessionId

"Returns a SmallInteger representing the time of the session's most recent
 beginTransaction, commitTransaction, or abortTransaction (from System timeGmt).

 Equivalent to executing:
   (System descriptionOfSession: aSessionId) at: 5

 Returns -1 if the session with aSessionId could not be found."

<primitive: 834>
aSessionId _validateClass: SmallInteger .
^self _primitiveFailed: #lastTransactionTimeForSessionWithId:
      args: { aSessionId }
%

category: 'Session Control'
classmethod: System
latestCommitRecordPageId

"Return the page ID of the most recent (newest) commit record on the system.
 Returns -1 if an error occurs."

^ self _zeroArgPrim: 108
%

category: 'Debugging Support'
classmethod: System
listenForDebugConnection

"Requires CodeModification privilege.
 Returns an Integer (the debug token) or signals an Error .
 If this session was configured with GEM_LISTEN_FOR_DEBUG=false ,
 starts a thread in the vm which listens on localhost for a process
 to connect with GciDebugConnectToGem, GciDebugStartDebugService,
 or for a topaz -r to connect with DEBUGGEM command.  Prints the
 required DEBUGGEM command including token to gem log file or stdout of a topaz -l
 process.  The token is a random 64 bit integer .

 If the debug thread is already running or GEM_LISTEN_FOR_DEBUG=true,
 returns the Integer debug token."
 | token |
 token := self _zeroArgPrim: 116.
 token == 0 ifTrue:[
    Error signal:'listenForDebugConnection failed, check gem log or topaz -l sterr'.
 ].
 ^ token
%

category: 'User-Defined Actions'
classmethod: System
loadUserActionLibrary: aString

"Loads the session user action library specified by aString.
 This method always returns the receiver (System).

 Searches for the shared library in current directory,
 $GEMSTONE/ualib ,
 directory containing the executable that loaded the libgci library,
   $GEMSTONE/bin for a topaz or $GEMSTONE/sys for a gem .
 and on Linux the directories specified by LD_LIBRARY_PATH environment variable."

<primitive: 426>
^ self _primitiveFailed: #loadUserActionLibrary: args: { aString }
%

category: 'Lock Status'
classmethod: System
lockKind: anObject

"Returns a Symbol (#none, #deferredUnlock, #read, or #write) representing the kind
 of lock held on anObject by any session in the system."

| anArray |

anArray := self lockStatus: anObject.
^anArray at: 1.
%

category: 'Lock Status'
classmethod: System
lockOwners: anObject

"Returns an Array of session numbers (SmallIntegers) representing the sessions
 that hold a lock on anObject.  If the object is not locked by any session, the
 result Array is empty.  Note that a write lock can have only one owner."

| anArray |

anArray := self lockStatus: anObject.
^anArray at: 2.
%

category: 'Lock Status'
classmethod: System
lockStatus: anObject

"Returns a two-element Array, where the first element is a Symbol representing
 the kind of lock held on anObject (#none, #read or #write) and
 the second element is an Array of session numbers (SmallIntegers) representing
 the sessions that hold the lock.

 If there are no locks on anObject, the first element is the Symbol #none and
 the second element is an empty Array.

 Only locks on permanent objects are reported."

<primitive: 351>
self _primitiveFailed: #lockStatus: args: { anObject } .
self _uncontinuableError
%

category: 'System Control'
classmethod: System
loginLogFileName

"Returns a String containing the name of the current login log file,
 or nil if the login log file is not enabled."

^ self _zeroArgPrim: 174
%

category: 'Session Control'
classmethod: System
loginUserId

"Returns a SmallInteger which is the numeric value of the Unix userId of
 the gem or topaz -l process of the current session.

 Returns -1 if the Unix user ID could not be determined or if the host
 system does not support this feature."

^ self _zeroArgPrim: 171
%

category: 'Session Control'
classmethod: System
loginUserIdName

"Returns a String containing the Unix login user ID of the gem or topaz -l process
 for the current session.

 Returns 'unknown user' if the Unix user ID could not be determined or if the host
 system does not support this feature."

^ self _zeroArgPrim: 172
%

category: 'Session Control'
classmethod: System
logout

"Immediately logs out the current session and generates a fatal error
 (GS_ERR_GEM_NORMAL_SHUTDOWN).  This method does not return."

self _zeroArgPrim: 104
%

category: 'Session Control'
classmethod: System
logreceiverSessionId

"Return the session ID of the logreceiver process.
 Returns zero if no logreceiver is connected to the stone process.
 Note a logreceiver can exist without a connection to the stone."

| id |
id := self _zeroArgPrim: 151 .
id > 0 ifTrue:[ ^ 0 "a logsender is connected"].
^ 0 - id
%

category: 'Session Control'
classmethod: System
logsenderSessionId

"Return the session ID of the logsender process.
 Returns zero if no logsender is connected to the stone process.
 Note a logsender can exist without a connection to the stone."

| id |
id := self _zeroArgPrim: 151 .
id < 0 ifTrue:[ ^ 0 "a logreceiver is connected"].
^ id
%

category: 'Clustering'
classmethod: System
maxClusterBucket

"Returns the maximum legal clusterId as a SmallInteger."

^ AllClusterBuckets size
%

category: 'Session Control'
classmethod: System
maxSessionId

"Returns a SmallInteger representing the maximum number of sessions
 allowed on the system based upon the Stone configuration parameter."

^ self _zeroArgPrim: 6
%

category: 'Performance Monitoring'
classmethod: System
microsecondsToRun: aBlock

"Returns the number of CPU microseconds used while evaluating aBlock.
 The argument aBlock must be a zero-argument block."

| startTime endTime |

startTime := self readClockNano.
aBlock value.
endTime := self readClockNano.
^ (endTime - startTime) // 1000
%

category: 'Shared Cache Management'
classmethod: System
midLevelCacheAddress

"If connected to a mid level cache, return a String , the IP
 address of the mid cache host.  Otherwise return nil."

^ self _midLevelCacheConnect: nil options: 2
%

category: 'Shared Cache Management'
classmethod: System
midLevelCacheConnect: hostName

"Attempt to connect to mid-level cache on the
 specified host, if cache already exists, using
 the same netldi name as was used to login to the stone.

 Returns true if connection succeeded, or a String
 describing the failure.

 The session's gem process must be on a machine different from
 the machine running the stone process in order for this to succeed."

^ self midLevelCacheConnect: hostName options: nil .
%

category: 'Shared Cache Management'
classmethod: System
midLevelCacheConnect: hostName cacheSizeKB: aSize maxSessions: nSess

"If a cache already exists on host specified by hostName,
 connects to that cache.
 If current session was created by GciX509Login, and the cache does
 not exist, returns a String describing the connection failure.
 Arguments aSize and nSess are ignored in X509 sessions.

 Otherwise if a mid-level cache does not exist on a specified host,
 and aSize > 0, attempt to start one.
 Then attempt to connect to the mid-level cache.
 Connection may fail if cache could not be started or
 if the cache could not accept another session .
 Uses the same netldi name as was used to login to the stone.

 The session's gem process must be on a machine different from
 the machine running the stone process in order for this to succeed.

 If session A's gem process is running on a machine where the
 shared cache is in use as a mid-level cache by gem processes
 on other machines, then session A is cannot use this method;
 i.e. cross-connected mid-level caches are not allowed .

 hostName must be a String, else an error is generated.

 If aSize <= 0, nSess is ignored ,
 otherwise nSess must be >= 11 and aSize must be >= 16000 .
 The nSess value may be reduced by stone to the
 number of sessions stone is configured and licensed for.

 When creating a cache
   aSize is cache size in K bytes (minumum 16000) ,
   nSess is maximum number of sessions (minimum 11) .

 The spinLockCount, targetFreeFrameCount, numFreeFrameServers,
 and numSharedCounters for creating a cache come from gem's config file
 just as for a cache the gem would have created as a side effect
 of login .

 Returns true if connection succeeded, or a String
 describing the connection failure."

aSize <= 0 ifTrue:[
  "attempt to connect to an existing cache"
  ^ self midLevelCacheConnect: hostName options: nil
].
nSess < 1 ifTrue:[ Error signal:'nSess arg must be >= 1' ].
^ self midLevelCacheConnect: hostName
   options: { 'SHR_PAGE_CACHE_SIZE_KB' . aSize  .
	      'SHR_PAGE_CACHE_NUM_PROCS' . nSess + 4 "pcmon, cachePgsvr, 2 FF slots"}
%

category: 'Shared Cache Management'
classmethod: System
midLevelCacheConnect: hostName options: anArray

"Attempts to connects to a mid-level cache on the host specified by hostName.
 If anArray is nil, or if current session is from GciX509Login,
 the cache must exist.  Otherwise if cache
 does not exist an attempt is made to start the cache.
 Argument anArray is ignored in X509 sessions.

 Connection may fail if cache could not be started or
 if the cache could not accept another session .
 Uses the same netldi name as was used to login to the stone.

 The session's gem process must be on a machine different from
 the machine running the stone process in order for this to succeed.

 If session A's gem process is running on a machine where the
 shared cache is in use as a mid-level cache by gem processes
 on other machines, then session A is cannot use this method;
 i.e. cross-connected mid-level caches are not allowed .

 hostName must be a String, else an error is generated.

 If not nil, anArray must be an Array of pairs specifing config item and value
 for a mid-level cache.
 The first element of each pair must be one of the
 following case-insensitive strings , with second element of a pair
 being a value as follows

  'SHR_PAGE_CACHE_SIZE_KB'  anInteger representing KB,
                            or a String with a units suffix such as '3GB' or '3000MB'
  'SHR_PAGE_CACHE_NUM_PROCS'  anInteger
  'SHR_TARGET_FREE_FRAME_COUNT' anInteger
  'SHR_NUM_FREE_FRAME_SERVERS' anInteger
  'SHR_PAGE_CACHE_NUM_SHARED_COUNTERS' anInteger
  'SHR_PAGE_CACHE_LOCKED'   0, 1 or a Boolean
  'GEM_PGSVR_COMPRESS_PAGE_TRANSFERS'  0, 1 or a Boolean
  'SHR_PAGE_CACHE_LARGE_MEMORY_PAGE_POLICY'   0, 1 or 2
  'SHR_PAGE_CACHE_LARGE_MEMORY_PAGE_SIZE_MB' anInteger: Linux: 0, 2, 1024; Others: 0
  'SHR_PAGE_CACHE_PERMISSIONS'   anInteger
  'SHR_WELL_KNOWN_PORT_NUMBER' anInteger
  'MidHostRoute'  a String , a hostname or ipAddress .

 NOTE: These are expected to match the names and behavior of configuration
   parameters, however 'MidHostRoute' is not a existing configuration parameter
   and is specific to this method.

 MidHostRoute if specified is used to access an already running cache;
 it specifies the host used by the gem to contact the mid cache.
 hostName is always used to ask stone for the existance of the cache regardless
 of whether MidHostRoute is defined.  hostName must be the route to the
 mid cache as seen by stone ; MidHostRoute allows for a leaf node to
 have a different route to a multi homed mid cache host.  All other
 config items are used only to start a cache .

 Any other item which is not specified will have a value
 taken from this session's gem config file.
 The SHR_TARGET_FREE_FRAME_COUNT is normally computed automatically
 based on the cache size .
 The default for GEM_PGSVR_COMPRESS_PAGE_TRANSFERS is taken from
 the current runtime value in the gem process.

 See the comments in the file $GEMSTONE/data/system.conf for
 more details on each item.

 Signals an error if a config item is invalid or out of range.
 Returns true if connection succeeded, or a String
 describing a connection failure."

hostName  _validateClass: String .
anArray ifNotNil:[ anArray _validateClass: Array ].
^ self _midLevelCacheConnect: hostName options: anArray
%

category: 'Shared Cache Management'
classmethod: System
midLevelCacheReconnect: hostName

"If already connected to a mid level cache, disconnect from that cache.
 Then attempt to connect to mid-level cache on the
 specified host, if cache already exists, using
 the same netldi name as was used to login to the stone.

 Returns true if connection succeeded, or a String
 describing the failure.

 The session's gem process must be on a machine different from
 the machine running the stone process in order for this to succeed."

 hostName  _validateClass: String .
 ^ self _midLevelCacheConnect: hostName options: 1 .
%

category: 'Shared Cache Management'
classmethod: System
midLevelCachesList

"Return an Array as described for remoteCachesList,
 filtered to include only the mid-level caches, plus the stone cache. "

^ self _remoteCachesList: true
%

category: 'Shared Cache Management'
classmethod: System
midLevelCachesReport

"Return the result of midLevelCachesList formatted as a string,
 one line per cache."

^ self _formatCacheReport: (self _remoteCachesList: true)
%

category: 'Transaction Control'
classmethod: System
millisecondsSinceLogin

"Answer a SmallInteger representing the number of milliseconds since this
 session logged in to the repository."

^ self _zeroArgPrim: 123
%

category: 'Transaction Control'
classmethod: System
millisecondsSinceTransactionBoundary

"Answer a SmallInteger representing the number of milliseconds since this
 session updated its view of the repository by executing #abortTransaction,
 #beginTransaction, #continueTransaction, or #commitTransaction.  Note that
 an attempt to commit which fails (due to transaction conflicts) is considered
 a transaction boundary since the session is shifted to the most recent view
 by the failed commit attempt."

^ self _zeroArgPrim: 122
%

category: 'Performance Monitoring'
classmethod: System
millisecondsToRun: aBlock

"Returns the number of CPU milliseconds used while evaluating aBlock.
 The argument aBlock must be a zero-argument block.

 The resolution of the result is operating system dependent."

| startTime endTime |
aBlock _isExecBlock ifFalse:[ ArgumentError signal:'expected a block'].
startTime := self readClockNano .
aBlock value.
endTime := self readClockNano .
^ (endTime - startTime) // 1000000
%

category: 'Garbage Collection Management'
classmethod: System
missingGcGem
"Return a string indicating which gc gem is missing"
((SystemRepository restoreStatusInfo at: 2) == 0 and: [self adminGemSessionId == 0])
  ifTrue:[^ 'missing the Admin gem' ].

(self reclaimGemSessionCount == 0)
  ifTrue:[^ 'missing reclaim gem' ].

^ 'all GC gems accounted for'
%

category: 'Cache Statistics'
classmethod: System
myCacheProcessSlot

"Returns the process slot in the SharedPageCache that corresponds
 to my process.  If the SharedPageCache is not in use, returns -1."

^ self _zeroArgPrim: 7
%

category: 'Cache Statistics'
classmethod: System
myCacheStatistics

"Returns the cache statistics for the current session."

GsSession currentSession isSolo ifTrue:[
  ^ self _primCacheStatsForSlotOrSession: nil opCode: 11 .
].
^ self cacheStatisticsAt: self myCacheProcessSlot
%

category: 'Cache Statistics'
classmethod: System
myCacheStatisticWithName: aString

"Return the value of the cache stat with the given name for the current session.
 The name must match an element of the Array returned by the
 #cacheStatisticsDescription method applicable to a gem process.  The UserTime
 and SysTime statistics cannot be accessed using this method.

 Returns nil if a statistic matching aString was not found for a gem.

 This method may be used on hosts remote from the stone process."

^ self _cacheStatWithName: aString opCode: 8
%

category: 'Session Control'
classmethod: System
myKerberosPrincipal

"Returns the KerberosPrincipal of the current session, or nil
 if Kerberos was not used by the session during login or the
 session does not have read access to the KerberosPrincipal
 object."

^ self _zeroArgPrim: 180
%

category: 'Lock Status'
classmethod: System
myLockKind: anObject

"Returns a Symbol that indicates what kind of lock the current session has on
 anObject: one of #none, #read or #write."

| status |
status := self lockStatus: anObject.
( (status at: 1) ~~ #none and:
[ (status at: 2) includesIdentical: self session] )
  ifTrue: [ ^status at: 1 ].

^#none
%

category: 'Private'
classmethod: System
myPageServerProcessId
"Answer an Integer which is the process ID of the page server on stone's
 machine for this session.
 Returns 0 if the session does not have a page server."

^ self _zeroArgPrim: 59
%

category: 'Private'
classmethod: System
myReadPageServerProcessId
"Answer an Integer which is the process ID of the page server
being used for disk reads.
Returns 0 if the session does not have a page server.
Result may be different from the result of myPageServerProcessId if
the session is using a mid-level cache."

^ self _zeroArgPrim: 108
%

category: 'Deprecated'
classmethod: System
myUserGlobals

"Obsolete in GemStone/64."

| assn |

self deprecated: 'System class>>myUserGlobals is deprecated in GemStone/S 64 Bit.
Replace with (System myUserProfile resolveSymbol: #UserGlobals) value'.

(assn := self myUserProfile resolveSymbol: #UserGlobals) == nil ifTrue: [
  ^self _error: #rtErrKeyNotFound args: #(#UserGlobals)
].
^assn value.
%

category: 'Session Control'
classmethod: System
myUserProfile

"Returns the UserProfile of the current session."

"Implemented as a primitive to handle image bootstrap problems."

<primitive: 318>
^ self _primitiveFailed: #myUserProfile
%

category: 'Transaction Control'
classmethod: System
needsCommit

"Returns true if the session has made changes to the repository.  Returns false
 otherwise."

^ self _numPersistentObjsModified > 0
%

category: 'Instance Creation'
classmethod: System
new

"Disallowed.  You may not create new instances of System."

self shouldNotImplement: #new
%

category: 'Session Control'
classmethod: System
notVotedSessionNames
  "Return a string formatted as for System(C)>>currentSessionNames
   for each session that has not voted during the last epochGc or
   markForCollection .
   Requires SessionAccess privilege if there is more
   than one session logged in."
  | resBlock |
  resBlock := [ :res :num :userPro |
              res lf ; add: 'session number: '; add: num asString ;
                  add: '    UserId: ' ; add: userPro userId ].
  ^ self _notVotedSessions: resBlock into: String new
%

category: 'Session Control'
classmethod: System
notVotedSessions
  "Return an array of session numbers as for System(C)>>currentSessions
   for each session that has not voted during the last epochGc or
   markForCollection .
   Requires SessionAccess privilege if there is more
   than one session logged in."
  | resBlock |
  resBlock := [ :res :num :userPro | res add: num ].
  ^ self _notVotedSessions: resBlock into: { } .
%

category: 'Deprecated'
classmethod: System
numberOfExtentRangesWithoutGC
"Deprecated - all extents are covered by a single reclaim gem.
 Return a SmallInteger indicating the minimum number of additional
 reclaim gems that would need to be started to have all extents
 covered by a reclaim gem."

self deprecated: 'System class>>numberOfExtentRangesWithoutGC deprecated in v3.2.'.
^self reclaimGemSessionCount > 0
  ifTrue: [ 0 ]
  ifFalse: [ 1 ]
%

category: 'Deprecated'
classmethod: System
numberOfExtentsWithoutGC

"Deprecated - all extents are covered by a single reclaim gem.
 Return a SmallInteger indicating the number of extents not subject to
 garbage collection because they have no reclaim gem assigned."

self deprecated: 'System class>>numberOfExtentsWithoutGC deprecated in v3.2. '.
^self reclaimGemSessionCount > 0
  ifTrue: [ 0 ]
  ifFalse: [ SystemRepository numberOfExtents ]
%

category: 'Persistent Counters'
classmethod: System
numberOfPersistentSharedCounters

"Returns a SmallInteger which is the number of persistent shared counters
 the system supports."

^ self _zeroArgPrim: 154
%

category: 'Shared Counters'
classmethod: System
numSharedCounters

"Returns the number of shared counters supported by this session's shared
 page cache. This is set by the configuration file parameter
 SHR_PAGE_CACHE_NUM_SHARED_COUNTERS, which has a default of 1900."

  ^self _numSharedCounters
%

category: 'Configuration File Access'
classmethod: System
objectReadLogEnabled
 "Answer a Boolean indicating if the object read logging feature is currently enabled."
 ^ System stoneConfigurationAt: #STN_OBJECT_READ_LOG_ENABLED .
%

category: 'System Control'
classmethod: System
objectReadTrackingEnabled
"Returns true if read tracking is currently enabled for this session.
 The result will be true if (System objectReadLogEnabled == true) and
 ((System myUserProfile _hasPrivilegeName: #DisableObjectReadLogging) == false
 and (System setObjectReadTracking: false) has not been successfully executed as the most recent
 invocation of System class >> setObjectReadTracking: ."

^ self _zeroArgPrim: 193
%

category: 'Session Control'
classmethod: System
oldestCommitRecordAge
"Return a SmallInteger representing the age of the oldest commit record in seconds.
 This value represents the elapsed time since the commit that created the oldest
 commit record."

  ^ (self _zeroArgPrim: 164) // 1000
%

category: 'Session Control'
classmethod: System
oldestCommitRecordAgeMilliseconds

"Return a SmallInteger representing the age of the oldest commit record in milliseconds.
 This value represents the elapsed time since the commit that created the oldest
 commit record."

^ self _zeroArgPrim: 164
%

category: 'Session Control'
classmethod: System
otherSessionNames

"Returns a formatted String containing, for each current GemStone session
  other than the session executing this method, the
 session number and userId.

 This method requires SessionAccess privilege."

| lf result mySessNum profiles |

result := String new.  " string to return "
lf := Character lf.
mySessNum := self session .

profiles := self _currentSessionProfiles.  "sessId, userProfile pairs"
1 to: profiles size by: 2 do:[:j| | aSessionNumber aUserProfile |
   aSessionNumber := profiles at: j .
   aSessionNumber == mySessNum ifFalse:[
     aUserProfile := profiles at: j+ 1 .
     aUserProfile ifNotNil: [
       result add: lf;
	  addAll: 'session number: ';
	  addAll: (aSessionNumber asString);
	  addAll: (self _sessionUproString: aUserProfile) .
     ].
   ].
].
^result
%

category: 'Session Control'
classmethod: System
otherSessionsReport
  | mySession |
  mySession := System session .
  ^ self _sessionsReport: (self currentSessions reject:[:n | n == mySession ])
%

category: 'Performance Monitoring'
classmethod: System
pageReads

"Returns the number of Repository page read operations performed since the start
 of the Gem process.  If the Gem is remote, this corresponds to reads performed
 in the current session.  If the Gem is linked, it corresponds to reads
 performed since the application was invoked."

^ self _zeroArgPrim: 2
%

category: 'Cache Statistics'
classmethod: System
pageServerCacheName: aString

"Sets the cache name for the page server thread serving the current session.
 Also sets the cache name for the page server on the mid-level cache, if it
 exists.

 Raises an error if aString is too long (exceeds 31 characters) or is empty.

 Has no effect and returns false if the current session exists on the same host
 as the stone process.  Otherwise returns true."

<primitive: 1019>
self _primitiveFailed: #pageServerCacheName: args: { aString }
%

category: 'Runtime Configuration Access'
classmethod: System
pageServerConnectionIsEncrypted

"Answer a Boolean indicating whether the network connection to the gem's page server is
 encrypted (using TLS).  Always answers false for gems without a page server, i.e., gems
 that run on the same host as the stone process."

^ self _zeroArgPrim: 176
%

category: 'Performance Monitoring'
classmethod: System
pageWrites

"Returns the number of Repository page write operations performed since the
 start of the Gem process.  If the Gem is remote, this corresponds to writes
 performed in the current session.  If the Gem is linked, it corresponds to
 writes performed since the application was invoked."

^ self _zeroArgPrim: 3
%

category: 'Private Cache Warming'
classmethod: System
parseWarmerConfig: aString
"Returns a SymbolKeyValueDictionary with zero or more keys
   #midHost #d #n
"
 | res |
 res := SymbolKeyValueDictionary new .
 aString ifNotNil:[ | arr n |
   arr := aString subStringsDelimitedByAny: { Character space . Character tab }.
   n := 1 .
   [ n <= arr size ] whileTrue:[ | s val |
     s := arr at: n . n := n + 1 .
     s = '-d' ifTrue:[ res at: #d put: true ] ifFalse:[
     s = '-n' ifTrue:[
        val := arr atOrNil: n. n := n + 1.
        val ifNotNil:[ val := Integer fromString: val ].
        val ifNil:[ Error signal:'missing value for -n'].
        (val < 1 or:[ val > 20]) ifTrue:[
          Error signal:'''-n ', val asString, ''' , number of threads not within 1..20'].
        res at: #n put: val .
     ] ifFalse:[
     s = '-M' ifTrue:[
       val := arr atOrNil: n. n := n + 1.
       val ifNil:[ Error signal:'missing value for -M'].
       res at: #midHost put: val .
     ]]].
   ]
 ].
 ^ res
%

category: 'Host System Access'
classmethod: System
performOnServer: aString
"This method causes the operating system commands in aString to be executed as
 a spawned subprocess using the default shell, /bin/sh.  Generates an error
 if aString cannot be executed by the operating system.

 Under Unix, commands in aString can have exactly the same form as a shell
 script.  For example, newlines or semicolons can separate commands, and a
 backslash can be used as an escape Character.

 Returns the stdout result of the spawned process as a String .
 If the result is actually UTF8 data, the application must determine
 when to send decodeFromUTF8 to the result.

 Signals an error if the privilege NoPerformOnServer is true and aString is
 not present in the whitelist of allowed commands for the session's UserProfile.
 See the methods in UserProfile under category PerformOnServer for more
 information."

^ self performOnServer: aString withShell: nil
%

category: 'Host System Access'
classmethod: System
performOnServer: aString withShell: aShellOrNil

"This method causes the operating system commands in aString to be executed as
 a spawned subprocess.  Generates an error if aString cannot be executed by the
 operating system.

 Under Unix, commands in aString can have exactly the same form as a shell
 script.  For example, newlines or semicolons can separate commands, and a
 backslash can be used as an escape Character.

 aShellOrNil may be used to specify the shell to use to process the command.
 If aShellOrNil is nil, the default shell (/bin/sh) is used. Otherwise the
 specified shell is used. The specified shell must accept the -c command line
 argument. Note that C SHell (/bin/csh) and its variants (/bin/tcsh) are known
 to not work with this method. However Korn SHell (/bin/ksh), Z SHell
 (/bin/zsh) and Bourne-Again SHell (/bin/bash) are known to work.

 Returns the stdout result of the spawned process as a String .
 If the result is actually UTF8 data, the application must determine
 when to send decodeFromUTF8 to the result.

 Signals an error if the privilege NoPerformOnServer is true and aString is
 not present in the whitelist of allowed commands for the session's UserProfile.
 See the methods in UserProfile under category PerformOnServer for more
 information."

| resultStr arr |
arr := self _performOnServer: aString withShell: aShellOrNil.
resultStr := arr at: 3 .
resultStr ifNil:[  | rawStatus childStatus errno errMsg |
  rawStatus := arr at: 1 .
  childStatus := arr at: 2 .
  errno := arr at: 5 .
  errMsg := arr at: 4 .
  aString _error: #hostErrPerform args:{ errMsg . errno . rawStatus . childStatus }.
  ^ nil
].
resultStr _stringCharSize == 0 ifTrue:[
  ^ resultStr decodeToString  "decode a Utf8"
].
^ resultStr
%

category: 'Persistent Counters'
classmethod: System
persistentCounterAt: index

"Returns the value of the persistent shared counter at index, which
 may be a SmallInteger or LargeInteger.

 See the method:
   System>>_updateSharedCounterAt: index by: amount withOpCode: code
 for more information on persistent shared counters."

^self _updateSharedCounterAt: index by: 0 withOpCode: 0
%

category: 'Persistent Counters'
classmethod: System
persistentCounterAt: index decrementBy: amount

"Decrements the persistent shared counter at index by the specified
 amount. Amount must be a SmallInteger or LargeInteger.
 For a LargeInteger,  amount must be representable as a signed
 64-bit integer (between -9223372036854775808 and 9223372036854775807).

 Returns the new value of the counter after the decrement.

 See the method:
   System>>_updateSharedCounterAt: index by: amount withOpCode: code
 for more information on persistent shared counters."

^self _updateSharedCounterAt: index by: amount negated withOpCode: 0
%

category: 'Persistent Counters'
classmethod: System
persistentCounterAt: index incrementBy: amount

"Increments the persistent shared counter at index by the specified
 amount.  Amount must be a SmallInteger or LargeInteger.
 For a LargeInteger,  amount must be representable as a signed
 64-bit integer (between -9223372036854775808 and 9223372036854775807).

 Returns the new value of the counter after the increment.

 See the method:
   System>>_updateSharedCounterAt: index by: amount withOpCode: code
 for more information on persistent shared counters."

^self _updateSharedCounterAt: index by: amount withOpCode: 0
%

category: 'Persistent Counters'
classmethod: System
persistentCounterAt: index put: value

"Sets the persistent shared counter at index to the specified
 value.  value must be a SmallInteger or LargeInteger.
 For a LargeInteger,  amount must be representable as a signed
 64-bit integer (between -9223372036854775808 and 9223372036854775807).

 Returns value, the new value of the counter.

 See the method:
   System>>_updateSharedCounterAt: index by: amount withOpCode: code
 for more information on persistent shared counters."

^self _updateSharedCounterAt: index by: value withOpCode: 1
%

category: 'System Control'
classmethod: System
possibleDeadSize

"Return the number of possible dead objects in the system."
^self stoneCacheStatisticWithName: 'PossibleDeadObjs'
%

category: 'System Control'
classmethod: System
possibleDeadSymbols

"Return the number of possible dead Symbols in the system."
^self stoneCacheStatisticWithName: 'PossibleDeadSymbols'
%

category: 'Cache Statistics'
classmethod: System
primaryCacheMonitorCacheStatisticWithName: aString

"Return the value of the cache stat with the given name for the cache monitor
 process running on the stone's host.  The name must match an element of the
 Array returned by the #cacheStatisticsDescription method  applicable to the
 cache monitor process.  The UserTime and SysTime statistics cannot be accessed
 using this method.

 Returns nil if a statistic matching aString was not found for the cache monitor.

 This method may be used on hosts remote from the stone process."

^ self _cacheStatWithName: aString opCode: 1
%

category: 'Session Control'
classmethod: System
priorityForSessionId: aSessionId

"Returns an integer between 0 and 4 indicating the current priority
 of the session with the given session ID.  Returns -1 if aSession ID
 does not refer to a valid session.

 Requires the SessionAccess privilege to query the priority of another
 session.  No privilege is required to query the priority of the current
 session."

^self _sessionPriorityPrim: -1 forSessionId: aSessionId
%

category: 'Cache Statistics'
classmethod: System
processIdForSlot: anInt

"Return the process ID of the process using the ncache slot at
 index anInt.  Return -1 if the slot is not in use or out of
 range."

^ self _primCacheStatsForSlotOrSession: anInt opCode: 4
%

category: 'Cache Statistics'
classmethod: System
processKindForSlot: anInt

"Return the process kind occupying the given cache slot as follows:

   1 = Shared Page Cache Monitor
   2 = Stone
   4 = Page Server
   8 = Gem (including Topaz, GBS, and other GCI applications).

 A return value of -1 means the slot index was out of range or the
 slot is not in use."

^ self _primCacheStatsForSlotOrSession: anInt opCode: 9
%

category: 'Performance Monitoring'
classmethod: System
profMonSample

"Take a snapshot of the current execution stack for a ProfMonitor report.
 Only applies if a ProfMonitor instance is actively monitoring and its
 timing interval has been set to zero.  See documention for ProfMonitor
 for more details."

^ self _zeroArgPrim: 157
%

category: 'Reduced Conflict Support'
classmethod: System
rcValueCache

"Returns the cache dictionary that is stored in temporary session state
 used to hold calculated values for reduced conflict classes.  If it does
 not exist, create it."

| dict |
dict := self __sessionStateAt: 5.
dict == nil
    ifTrue: [
        dict := IdentityKeyValueDictionary new: 47.
        self __sessionStateAt: 5 put: dict
    ].
^ dict
%

category: 'Reduced Conflict Support'
classmethod: System
rcValueCacheAt: aKey for: anObject ifAbsent: aBlock

"Returns the associated value at the given key for anObject.
 If the key is not present, execute the zero-argument block."

| valueArray cache |
cache := self _rcValueCache.
" if the cache does not exist, then the key is not present "
cache ifNil: [ ^ aBlock value ].

valueArray := cache at: anObject otherwise: nil .
valueArray ifNil:[ ^ aBlock value ].

1 to: valueArray size by: 2 do: [ :i |
    aKey == (valueArray at: i)
        ifTrue: [ ^ valueArray at: (i + 1) ]
].

^ aBlock value
%

category: 'Reduced Conflict Support'
classmethod: System
rcValueCacheAt: aKey for: anObject otherwise: aValue

"Returns the associated value at the given key for anObject.
 If the key is not present, returns aValue."

| valueArray cache |
cache := self _rcValueCache.
" if the cache does not exist, then the key is not present "
cache == nil
  ifTrue: [ ^ aValue ].

valueArray := cache at: anObject otherwise: nil.

valueArray == nil
    ifTrue: [ ^ aValue ].

1 to: valueArray size by: 2 do: [ :i |
    aKey == (valueArray at: i)
        ifTrue: [ ^ valueArray at: (i + 1) ]
].

^ aValue
%

category: 'Reduced Conflict Support'
classmethod: System
rcValueCacheAt: aKey otherwise: aValue

"Returns the associated value at the given key for anObject.
 If the key is not present, returns aValue."

| cache |
cache := self _rcValueCache.
" if the cache does not exist, then the key is not present "
cache == nil
  ifTrue: [ ^ aValue ].

^ cache at: aKey otherwise: aValue.
%

category: 'Reduced Conflict Support'
classmethod: System
rcValueCacheAt: aKey put: aValue for: anObject

"Adds the given key/value pair for anObject.  Returns the receiver."

| valueArray |
valueArray := self rcValueCache at: anObject otherwise: nil.
valueArray == nil
  ifTrue: [
    valueArray := { } .
    self rcValueCache at: anObject put: valueArray.
  ].

1 to: valueArray size by: 2 do: [ :i |
    aKey == (valueArray at: i)
        ifTrue: [
            valueArray at: (i + 1) put: aValue.
            ^ self
        ]
].

"add a new entry"
valueArray addLast: aKey; addLast: aValue
%

category: 'Performance Monitoring'
classmethod: System
readClockNano

"Returns an Integer indicating the amount of CPU time used by the current
 process, in units of nanoseconds.  The resolution of the result is operating
 system dependent."

^ self _zeroArgPrim: 178
%

category: 'Setting Locks'
classmethod: System
readLock: anObject

"Requests a read lock on anObject. This method denies a read lock
 on anObject under any one of the following circumstances:

 * Another session has a write lock on the object.

 * The object is special.

 Note that once a readLock is obtained, any write of the object will
 cause a commit failure, even by the session holding the read lock."

| result |
result := self _lock: anObject kind: 2 autoRelease: false .
(result == 1) ifTrue:[ ^ self ].
self _lockError: result obj: anObject details: 'read lock'
%

category: 'Setting Locks'
classmethod: System
readLock: anObject ifDenied: denyBlock ifChanged: changeBlock

"Requests a read lock on anObject.  This method denies a read lock
 on anObject under any one of the following circumstances:

 * Another session has a write lock on the object.

 * The object is special.

 Returns the receiver if the requested lock was granted and was not dirty.

 Note that once a readLock is obtained, any write of the object will
 cause a commit failure, even by the session holding the read lock.

 If the requested lock is otherwise denied, it returns the value of
 the zero-argument block denyBlock.  If it grants a dirty lock, then it
 returns the value of the zero-argument block changeBlock.  In that case the
 lock remains, even after the transaction is aborted."

| result |

result := self _lock: anObject kind: 2 autoRelease: false .
(result == 1) ifTrue:[ ^ self ].
^ self _lockEvaluateErr: result obj: anObject denied: denyBlock changed: changeBlock
%

category: 'Setting Locks'
classmethod: System
readLockAll: aCollection

"Requests a read lock on each object in aCollection.  This method denies
 a read lock on an object under any one of the following circumstances:

 * Another session already holds a write lock on the object.

 * The object is special.

 This method grants a read lock on an object whenever it finds no reason
 to deny it.  However, a lock that it grants may be dirty.  One session's
 lock is dirty if another session has committed a change to the locked object
 since the beginning of the first session's current transaction.  A session
 that holds a dirty lock cannot commit its transaction.  To clean its locks,
 it must abort the transaction and obtain updated values for each object whose
 lock is dirty.

 Note that once a readLock is obtained, any write of the object will
 cause a commit failure, even by the session holding the read lock.

 If a lock was acquired for every element of aCollection, and no locks are
 dirty, returns the receiver.

 This method generates an error if it is unable to acquire a lock for every
 element of aCollection, or if any lock that it acquires is dirty.  However,
 all the locks that it acquires remain in place, even after the current
 transaction is aborted."


| result |
result := self _lockAll: aCollection kind: 2 .
(result == self )
  ifTrue: [^self]
  ifFalse: " else we generate the error lock incomplete "
           [^ (result at: 1) _error: #lockErrIncomplete
                             args: { (result at:2) . (result at:3) }].
%

category: 'Setting Locks'
classmethod: System
readLockAll: aCollection ifIncomplete: incompleteBlock

"Requests a read lock on each object in aCollection.  This method denies
 a read lock on an object in the collection under any one of the following
 circumstances:

 * Another session already holds a write lock an object.

 * The object is special.

 If all requested locks were granted and none of the locks are 'dirty',
 returns the receiver.

 Note that once a readLock is obtained, any write of the object will
 cause a commit failure, even by the session holding the read lock.

 This method grants a read lock on an object whenever it finds no reason
 to deny it.  However, a lock that it grants may be dirty.  One session's
 lock is dirty if another session has committed a change to the locked object
 since the beginning of the first session's current transaction.  A session
 that holds a dirty lock cannot commit its transaction.  To clean its locks,
 it must abort the transaction and obtain updated values for each object whose
 lock is dirty.

 If this method is unable to acquire a lock for every element of aCollection,
 or if any lock that it acquires is dirty, then it returns the value of the
 three-argument block incompleteBlock.  The arguments to the block are:

 1.  An Array of objects that could not be locked.

 2.  An Array of objects that were locked but whose locks are dirty.

 3.  An empty Array, retained for backward compatibility with old GemStone versions.

 All the locks that it acquires remain in place, even after the current
 transaction is aborted."

| result |
result := self _lockAll: aCollection kind: 2 .
(result == System)
  ifTrue: [^self]
  ifFalse: " else we execute the incomplete block with 3 arguments "
          [^incompleteBlock value: (result at: 1)
                            value: (result at:2)
                            value: (result at:3)].
%

category: 'Session Control'
classmethod: System
realUserId

"Returns a SmallInteger indicating the numeric value of the real Unix user ID of the gem process."
^ self _zeroArgPrim: 167
%

category: 'Session Control'
classmethod: System
realUserIdName

"Returns a String indicating the real Unix user ID of the gem process."
^ self _zeroArgPrim: 168
%

category: 'System Control'
classmethod: System
reclaimedSymbols

"Return the number of Symbols reclaimed since stone was started."
^self stoneCacheStatisticWithName: 'ReclaimedSymbols'
%

category: 'Deprecated'
classmethod: System
reclaimGcSessionCount

"Return the number of reclaim GC sessions currently running."

self deprecated: 'System class >> reclaimGcSessionCount deprecated v3.2.  Use reclaimGemSessionCount instead'.
^self reclaimGemSessionCount
%

category: 'Garbage Collection Management'
classmethod: System
reclaimGemConfigs
"Returns a String that documents the reclaim gem configuration options"

^'
#deadObjsReclaimedCommitThreshold
   The maximum number of dead objects to reclaim in a single transaction.
     (default:20000, min:32, max: maxInt32)

#deferReclaimCacheDirtyThreshold
   Specifies a threshold percentage of dirty pages in the stones shared
   page cache.  If more than this percentage of pages in the stone
   shared cache are dirty pages, page reclaims will be deferred until
   the dirty percentage drops below the threshold minus 5%.
   Setting the value to 100 disables this feature.
     (default:75, min:10, max: 100)

#maxTransactionDuration
   Controls the approximate maximum length of a reclaim transaction in
   seconds.  If the Reclaim gem has been in transaction longer than
   this duration, it will commit even if the #objsMovedPerCommitThreshold
   condition has not been satisfied.
     (default:300, min:1, max: maxInt32)

#objsMovedPerCommitThreshold
   Controls the approximate number of object table updates to be performed
   per transaction.  Reclaim gems will commit when at least this many
   live objects have been moved to new data pages.
     (default:20000, min:100, max: maxInt32)

#reclaimDeadEnabled
   Controls whether the reclaim will try to clean up dead objects.
     (default:true, min:false, max:true)

#reclaimMinFreeSpaceMb
   Minimum repository free space, expressed in megabytes, which must be
   available in order for reclaims to proceed.  Reclaims are temporarily
   suspended if the repository free space drops below this threshold.
   The default value of 0 specifies a varying reclaimMinFreeSpaceMb that
   is computed as the current size of the repository divided by 1000, with
   a minimum value of 5 mega bytes.  The default calculation is the same as
   that used by the stone for the STN_FREE_SPACE_THRESHOLD configuration paramenter.
     (default:0, min:0, max: 65536)

#reclaimMinPages
   The minimum number of pages to process in a single reclaim commit.  If there are not this
   many pages to reclaim, the reclaim gem sleeps until there are at least this number available.
     (default:40, min:1, max: maxInt32)

#reclaimVerboseLogging
   Controls the amount of logging information written to the log file.
   If set to false, normal logging information will be written.  If set to
   true, extra logging information is written to the log file."
     (default:false, min:false, max:true)

#sleepTimeBetweenReclaimMs
   Amount of time (in milliseconds) to sleep after a reclaim that did work.
     (default:0, min:0, max:300000)

#sleepTimeWithCrBacklogMs
   Amount of time (in milliseconds) to sleep after a commit when the
   commit record backlog is larger than 1.25 * the current setting for
   STN_CR_BACKLOG_THRESHOLD.  For each 25 percent above the threshold
   the sleep time is increased so that the reclaimGem does fewer commits
   the higher the number of commit records is above the threshold.
     (default:0, min:0, max:300000)
'
%

category: 'Garbage Collection Management'
classmethod: System
reclaimGemSessionCount

"Return the number of reclaim GC sessions (threads) currently running."

^ self _zeroArgPrim: 65
%

category: 'Garbage Collection Management'
classmethod: System
reclaimGemSessionId

"Returns the reclaim sessionId, zero if the reclaimGem is not running."

^ self _zeroArgPrim: 66
%

category: 'Garbage Collection Management'
classmethod: System
reclaimGemSessionIds

"Return an array with the reclaim sessionIds."
self deprecated: 'System class>>reclaimGemSessionIds deprecated v3.6. Use reclaimGemSessionId '.

^ { self reclaimGemSessionId }
%

category: 'Reduced Conflict Support'
classmethod: System
redoLog

  "Returns the redo log for the current transaction level 
   Creates it if it does not exist.
   The redo log is used by Reduced Conflict classes."
  
  | redoLog arr lev |
  (lev := self _zeroArgPrim: 5"transactionLevel") == 0 ifTrue:[ lev := 1 ].
  arr := self __sessionStateAt: 4.
  arr ifNotNil:[
    (redoLog := arr atOrNil: lev) ifNotNil:[ ^ redoLog]
  ] ifNil:[
    arr := Array new: lev .
    self __sessionStateAt: 4 put: arr .
  ].
  arr size < lev ifTrue:[
    arr size: lev .
  ].
  (redoLog := arr at: lev) ifNil:[
    redoLog := RedoLog new.
    arr at: lev put: redoLog .
  ].
  ^ redoLog
%

category: 'Transaction Control'
classmethod: System
refreshTransientSymbolList

"Installs a transient symbol list in the current session by
 copying the symbolList of the current UserProfile.
 Does not require CodeModification privilege .

 GsCurrentSession currentSession transientSymbolList: (System myUserProfile symbolList copy)

 Returns the receiver."

^ self _zeroArgPrim: 129
%

category: 'Shared Cache Management'
classmethod: System
remoteCachesList

"Return an Array describing all shared caches that the
 stone process is managing, including the cache on the stone machine.

 Result contains one element per cache. Each
 element of result is a 8 element Array containing
   hostName, a String
   a Boolean - true if cache was created as a mid-level cache
   a SmallInteger - total number of sessions connected to the cache
   a SmallInteger - max number of sessions for the cache
   a SmallInteger - size of cache in KB
   a SmallInteger - number of sessions using cache as a mid-level cache
   a SmallInteger - zero or sessionId of hostagent on stone host servicing the cache
   a String       - ipAddress of the host of the cache
   a SmallInteger - zero or sessionId hostagent running on a mid-level cache host"

^ self _remoteCachesList: false
%

category: 'Shared Cache Management'
classmethod: System
remoteCachesReport

"Return the result of remoteCachesList formatted as a string,
 one line per cache."

^ self _formatCacheReport: (self _remoteCachesList: false)

%

category: 'System Control'
classmethod: System
remoteSharedCacheCount

"Return the number of remote shared page caches on the system."
^self stoneCacheStatisticWithName: 'RemoteSharedPageCacheCount'
%

category: 'Runtime Configuration Access'
classmethod: System
removeGemLogOnExit: aBoolean

"Set state in a gem process that overrides the state of the
 GEMSTONE_KEEP_LOG environment variable. See $GEMSTONE/sys/gemnetdebug
 for documentation on GEMSTONE_KEEP_LOG .

 If aBoolean is true, the gem log file will be deleted if the
 gem process exits normally. If aBoolean is false, the
 the gem log file will not be deleted .

 Has no effect in a linked session"

<primitive: 554>
aBoolean _validateClass: Boolean .
^self _primitiveFailed: #removeGemLogOnExit: args: { aBoolean }
%

category: 'Removing Locks'
classmethod: System
removeLock: anObject

"Removes the lock held by the current session on anObject.
 Returns the receiver."

self removeLockAll: { anObject }.
^self
%

category: 'Removing Locks'
classmethod: System
removeLockAll: aCollection

"Removes all locks held by the current session on the objects in aCollection.
 If an object in aCollection is not locked by the current session, that
 object is ignored. Returns the receiver."

<primitive: 99>
| anArray |
anArray := aCollection asArray.
anArray == aCollection ifFalse:[
  ^ self removeLockAll: anArray .
  ] .
aCollection _validateClass: Collection.
self _primitiveFailed: #removeLockAll: args: { aCollection } .
self _uncontinuableError
%

category: 'Removing Locks'
classmethod: System
removeLocksForSession

"Removes all locks held by this session.  Returns the receiver.
 This method succeeds even if the session no longer has
 read authorization for one or more of its locked objects."

^ self _zeroArgPrim: 0
%

category: 'Runtime Configuration Access'
classmethod: System
repositoryEncryptionScheme

"Answers a symbol describing the algorithm used to encrypt the repository (extents and tranlogs) or
 #none if the repository is not encrypted."

^ self _zeroArgPrim: 1039
%

category: 'Runtime Configuration Access'
classmethod: System
repositoryIsEncrypted

"Answers a boolean indicating if the repository is encrypted."
^self repositoryEncryptionScheme ~~ #none
%

category: 'Runtime Configuration Access'
classmethod: System
repositoryPrivateKey

"If the repository is encrypted and the current user is SystemUser, returns an
 instance of GsTlsPrivateKey which represents the private key used to decrypt
 the repository. Returns nil if the repository is not encrypted. Raises an
 exception if this method is invoked by any user except SystemUser."

^self _zeroArgPrim: 186
%

category: 'Runtime Configuration Access'
classmethod: System
repositoryPublicKey

"If the repository is encrypted, returns an instance of GsTlsPublicKey or 
 GsX509Certificate which represents the public key used to encrypt the repository.
 Returns nil if the repository is not encrypted."

^self _zeroArgPrim: 185
%

category: 'Online Backup Support'
classmethod: System
resumeCheckpoints
"Resumes regular checkpoints if they were previously suspended by
 the System suspendCheckpointsForMinutes: method.

 Requires the SystemControl privilege.

 Returns the previous checkpoint state.  Returns true if checkpoints
 were suspended or false if checkpoints were not suspended."

 ^self _zeroArgPrim: 60
%

category: 'System Control'
classmethod: System
resumeLogins

"Allows new sessions to be initiated.  (Enables users to login.)  Logins are
 enabled when the GemStone system is started.  This message reverses the effect
 of System | suspendLogins.  Requires the SystemControl privilege."

self configurationAt: #StnLoginsSuspended put: 0
%

category: 'System Control'
classmethod: System
scavengablePagesCount

"Return the number of scavengable pages in the system."
^self stoneCacheStatisticWithName: 'PagesNeedReclaimSize'
%

category: 'Environment Access'
classmethod: System
secondsSinceStoneStarted

"Answer a SmallInteger which is the number of seconds since the stone started."

^ self _zeroArgPrim: 124
%

category: 'System Control'
classmethod: System
sendLostOtToSession: sessionIdOrSerial

"Sends a lostOt signal to the specified session.
 To execute this method, you must have the SystemControl privilege.

 The sessionIdOrSerial argument is a SmallInteger specifying a session.  A
 positive value is interpreted as a serialNumber.  A negative value is
 interpreted as a negated sessionId.

 Returns a Boolean , true if stone sent the OOB byte to the
 specified session, false of the OOB socket was blocked."

^ self _sendTransactionSignal: sessionIdOrSerial kind: 2
%

category: 'System Control'
classmethod: System
sendSigAbortToSession: sessionIdOrSerial

"Sends a sigAbort signal to the specified session.
 To execute this method, you must have the SystemControl privilege.

 The sessionIdOrSerial argument is a SmallInteger specifying a session.  A
 positive value is interpreted as a serialNumber.  A negative value is
 interpreted as a negated sessionId.

 SigAborts sent to any session more frequently than
 every 2 seconds are ignored.
 Returns a Boolean , true if stone sent the OOB byte to the
 specified session, false of the OOB socket was blocked."

^ self _sendTransactionSignal: sessionIdOrSerial kind: 0
%

category: 'Signals'
classmethod: System
sendSignal: aSignal to: aSessionId withMessage: aString

"Sends a signal (a SmallInteger) to the specified session (a SmallInteger) with
 aString as a message.  The aString argument is currently limited to 1023
 bytes."

aSessionId _validateClass: SmallInteger.
aSessionId < 1 ifTrue:[ aSessionId _error: #rtErrArgNotPositive  ].
self _sendSignal: aSignal toSess: aSessionId negated withMessage: aString oob: nil
%

category: 'System Control'
classmethod: System
sendSignalFinishTransactionToSession: sessionIdOrSerial

"Sends a FinishTransaction signal to the specified session.
 To execute this method, you must have the SystemControl privilege.

 The sessionIdOrSerial argument is a SmallInteger specifying a session.
 A positive value is interpreted as a serialNumber.  A negative value is
 interpreted as a negated sessionId.

 Returns a Boolean , true if stone sent the OOB byte to the
 specified session, false of the OOB socket was blocked."

^ self _sendTransactionSignal: sessionIdOrSerial kind: 1
%

category: 'Session Control'
classmethod: System
session

"Returns a SmallInteger representing the stone sessionId of the sender."

^self _zeroArgPrim: 33
%

category: 'Transient Session State'
classmethod: System
sessionCacheStatAt: anIndex

"Returns the value of the session statistic at the specified index, which
 must be in the range -2 to 47.  0 to 47 are for application use.  -1 means
 return the value of the progressCount statistic.  -2 means return the value
 of the indexProgressCount statistic."

^ self _sessionCacheStatAt: anIndex incrementBy: 0
%

category: 'Transient Session State'
classmethod: System
sessionCacheStatAt: anIndex decrementBy: aSmallInt

"Decrements the value of the session statistic at the specified index
 by aSmallInt and returns the new value.  anIndex must be in the range
 of -2 to 47. 0 to 47 are for application use.  -1 means decrement the
 progressCount statistic.  -2 means decrement the indexProgressCount
 statistic."

^ self _sessionCacheStatAt: anIndex incrementBy: aSmallInt negated
%

category: 'Transient Session State'
classmethod: System
sessionCacheStatAt: anIndex incrementBy: aSmallInt

"Increments the value of the session statistic at the specified index
 by aSmallInt and returns the new value.  anIndex must be in the range
 of -2 to 47.  0 to 47 are for application use.
 -1 means increment the progressCount statistic.  -2 means increment the
 indexProgressCount statistic."

 ^self _sessionCacheStatAt: anIndex incrementBy: aSmallInt
%

category: 'Transient Session State'
classmethod: System
sessionCacheStatAt: anIndex put: i

"This method sets the session statistic at the specified index (which should be
 in the range -2 to 47) to the specified value i, which must be a SmallInteger.
 0 to 47 are for application use.  -1 means set the progessCount statistic to
 the value i.  -2 means set the indexProgessCount statistic to the value i."

 ^ self _sessionCacheStatAt: anIndex put: i
%

category: 'Transient Session State'
classmethod: System
sessionCacheStatsForProcessSlot: aProcessSlot

"Return an array containing the 48 SmallIntegers representing the
 session statistics for the given process slot; the stats
 corresponding to indexes 0 to 47, not including the stats for -2
 and -1.

 Returns nil if the given process slot is not found or if the slot
 is not in use by a gem process.

 See additional information in the method #_sessionCacheStatAt: put:"

^self _sessionCacheStatsForSlotOrSessionId: aProcessSlot
%

category: 'Transient Session State'
classmethod: System
sessionCacheStatsForSessionId: aSessionId

"Return an array containing the 48 session statistics for the given
 session ID; the stats corresponding to indexes 0 to 47, not including
 the stats for -2 and -1.

 Returns nil if the given session is not found or if the slot is not
 in use by a gem process.

 See additional information in the method #_sessionCacheStatAt: put:"

^self _sessionCacheStatsForSlotOrSessionId: aSessionId negated.
%

category: 'Transaction Control'
classmethod: System
sessionCanCommit

"Answers true if the session may commit the current transaction (if any).
 Otherwise answers false."

"Must be in a transaction to commit."
self inTransaction
  ifFalse:[ ^ false].
self commitsDisabledUntilLogout
  ifTrue:[ ^ false].
self commitsDisabledUntilAbort
  ifTrue:[^ false].
self myUserProfile isReadOnly
  ifTrue:[^ false]. "This user can never commit"
(SystemRepository restoreActive)
  ifTrue:[^ false]. "restore from log or backup is active"

^true
%

category: 'Session Control'
classmethod: System
sessionIdHoldingGcLock

"Returns a SmallInteger which is the session ID of the session currently holding
 the garbage collection lock.  Returns 0 if the GC lock is free.

 Requires the SessionAccess privilege."

^ self _zeroArgPrim: 136
%

category: 'System Control'
classmethod: System
sessionIsLinked

"Answer true if the session and its client run in the same operating system
 process.  Otherwise answer false."

^ self sessionIsRpc not
%

category: 'Session Control'
classmethod: System
sessionIsOnStoneHost

"Answer true if this session is running on the same host computer as the stone
 process.  Otherwise answer false."

^ self _zeroArgPrim: 137
%

category: 'System Control'
classmethod: System
sessionIsRpc

"Answer true if the session and its client run in different operating system
 processes.  Otherwise answer false."

^ self _zeroArgPrim: 175
%

category: 'Session Control'
classmethod: System
sessionIsUsingHostagent
"Answer true if this session is connected to stone via a hostagent process,
 i.e. true if this session used GciX509Login .
 Otherwise answer false."

^ ((System descriptionOfSession: System session) at: 23) > 0
%

category: 'Lock Status'
classmethod: System
sessionLocks

"Returns a three-element Array describing locks held by current session.
 The elements are
   an Array of read-locked objects
   an Array of write-locked objects
   an Array of objects with deferred unlocks
 If the current session holds no locks of a particular kind (read, write),
 then the corresponding Array is empty.
 Deferred unlocks are objects for which the unlock request was received by
 stone while another session was holding the commit token; they will
 be unlocked as soon as the commit token is released.
 "

<primitive: 352>
self _primitiveFailed: #sessionLocks .
self _uncontinuableError
%

category: 'Environment Access'
classmethod: System
sessionPerformingBackup

"Returns the session ID of the session that is performing a backup.
 If there is no such session, returns -1."

^ self configurationAt: #SessionInBackup
%

category: 'Session Control'
classmethod: System
sessionsReferencingOldestCr

"Returns an Array containing the sessionIds of the sessions that are
 currently referencing the oldest commit record.  Sessions both inside and outside
 of a transaction will be returned.  Because a session can update its commit
 record without committing a transaction, it is possible that no session
 actually references the oldest commit record.  Therefore, this method
 may return an empty Array."

^self _zeroArgPrim: 21
%

category: 'Session Control'
classmethod: System
sessionsReferencingOldestCrInTransaction

"Returns an Array containing the sessionIds of the sessions that are in
 transaction and currently reference the oldest commit record."
^self _zeroArgPrim: 109
%

category: 'Session Control'
classmethod: System
sessionsReferencingOldestCrNotInTransaction

"Returns an Array containing the sessionIds of the sessions that are not in
 transaction and currently reference the oldest commit record."
^self _zeroArgPrim: 110
%

category: 'Transient Session State'
classmethod: System
sessionStateAt: anIndex

"Access to customer-available session states."

anIndex < 1 ifTrue: [  ArgumentError new
     object: anIndex ;
     signal: 'argument out of range in sessionStateAt:'].
^ self __sessionStateAt: anIndex + 39
%

category: 'Transient Session State'
classmethod: System
sessionStateAt: anIndex put: aValue

"Access to customer-available session states."
(anIndex < 1 or: [anIndex > 1994]) ifTrue: [  ArgumentError new
     object: anIndex ;
     signal: 'argument out of range in sessionStateAt:put:'].
^ self __sessionStateAt: anIndex + 39 put: aValue
%

category: 'Transient Session State'
classmethod: System
sessionStateSize

"Return the current size of the public part of the Session State Array."

^self _sessionStateSize - 39.
%

category: 'Garbage Collection Management'
classmethod: System
setAdminConfig: configSymbol toValue: aVal

"Sets the specified Admin gem runtime configuration option to aVal.  This change
 remains in effect until it is changed with this method or setPersistentAdminConfig:.

 Boolean values can be set using either true/false or 1/0.
 See getAdminConfig documentation for the list of valid config symbols.
 Returns the new value.
 Requires garbage collection privilege.
 The new setting should take effect within a few seconds unless the admin process is in a long
 operation such as epochGc or a wsUnionSweep.
 The adminGem will print 'Parameter changes noticed:' message to its log file when it discovers the change.
"
^self _gcGemConfig: false symb: configSymbol toValue: aVal
%

category: 'Garbage Collection Management'
classmethod: System
setDefaultAdminConfigs
  "Sets all of the admin configuration parameters to their default values.

   ***** Note: changes to these values need to be made in the comments in the
         adminGemConfigs method and gcgemutl.c

   This method aborts the current transaction; if an abort would cause unsaved
   changes to be lost it signals an error, #rtErrAbortWouldLoseData.

   You must have the GarbageCollection privilege to execute this method.
  "

  self needsCommit ifTrue: [ self _error: #rtErrAbortWouldLoseData ] .

  self setPersistentAdminConfig: #adminVerboseLogging toValue: false .
  self setPersistentAdminConfig: #epochGcMaxThreads toValue: 1 .
  self setPersistentAdminConfig: #epochGcPageBufferSize toValue: 64 .
  self setPersistentAdminConfig: #epochGcPercentCpuActiveLimit toValue: 90.
  self setPersistentAdminConfig: #epochGcTimeLimit toValue: 3600.
  self setPersistentAdminConfig: #epochGcTransLimit toValue: 5000.
  self setPersistentAdminConfig: #saveWriteSetUnionToFile toValue: false .
  self setPersistentAdminConfig: #sweepWsUnionMaxThreads toValue: 1 .
  self setPersistentAdminConfig: #sweepWsUnionPageBufferSize toValue: 64 .
  self setPersistentAdminConfig: #sweepWsUnionPercentCpuActiveLimit toValue: 90 .
%

category: 'Garbage Collection Management'
classmethod: System
setDefaultReclaimConfigs
  "Sets all of the reclaim configuration options to their default values.

   ***** Note: changes to these values need to be made in the comments in the
         reclaimGemConfigs method and gcgemutl.c

   This method aborts the current transaction; if an abort would cause unsaved
   changes to be lost it signals an error, #rtErrAbortWouldLoseData.

  "

  self needsCommit ifTrue: [ self _error: #rtErrAbortWouldLoseData ] .

  self setPersistentReclaimConfig: #deadObjsReclaimedCommitThreshold toValue: 20000 .
  self setPersistentReclaimConfig: #deferReclaimCacheDirtyThreshold toValue: 75 .
  self setPersistentReclaimConfig: #maxTransactionDuration toValue: 300 .
  self setPersistentReclaimConfig: #objsMovedPerCommitThreshold toValue: 20000 .
  self setPersistentReclaimConfig: #reclaimDeadEnabled toValue: true.
  self setPersistentReclaimConfig: #reclaimMinFreeSpaceMb toValue: 0 .
  self setPersistentReclaimConfig: #reclaimMinPages toValue: 40 .
  self setPersistentReclaimConfig: #reclaimVerboseLogging toValue: false .
  self setPersistentReclaimConfig: #sleepTimeBetweenReclaimMs toValue: 0.
  self setPersistentReclaimConfig: #sleepTimeWithCrBacklogMs toValue: 0.
%

category: 'Cache Statistics - Session'
classmethod: System
setGemKind: aSmallInt

"Set the gemKind cache stat to be aSmallInt.
 The default gemKind is zero. Customers are free to use any positive value for
 this statistic. Negative values are reserved for internal use."

^ self _sessionCacheStatAt: -3 put: aSmallInt
%

category: 'Transient Session State'
classmethod: System
setIndexProgressCountTo: aSmallInt

"Sets the value of the indexProgressCount statistic to aSmallInt.
 Returns aSmallInt."

^ self _sessionCacheStatAt: -2 put: aSmallInt
%

category: 'System Control'
classmethod: System
setObjectReadTracking: aBoolean
  "If (System objectReadLogEnabled == false)
   or ((System myUserProfile _hasPrivilegeName: #DisableObjectReadLogging) == true)
   has no effect and always returns false .

   If (System objectReadLogEnabled == true)
   and ((System myUserProfile _hasPrivilegeName: #DisableObjectReadLogging) == false)
   and ((System myUserProfile _hasPrivilegeName: #DynamicDisableObjectReadLogging) == true)
   returns the previous state of readLoggingEnabled,
   and sets readLoggingEnabled for this session to aBoolean .
   aBoolean == true enables read tracking.
   aBoolean == false will  disable any read tracking implied by
   GsObjectSecurityPolicy >> trackReads .

   Signals an Exception if aBoolean == false and
   ((System myUserProfile _hasPrivilegeName: #DynamicDisableObjectReadLogging) == false) .
  "
<primitive: 824>
aBoolean _validateClass: Boolean .
self _primitiveFailed: #setObjectReadTracking: args: { aBoolean }
%

category: 'Garbage Collection Management'
classmethod: System
setPersistentAdminConfig: configSymbol toValue: aVal

"Updates the runtime admin configuration option, see setAdminConfig:toValue:.
 If that is successful, then it commits the change to the persistent configuration.
 If an abort would cause unsaved changes to be lost, it does not execute and
 signals an error, #rtErrAbortWouldLoseData.

 User must have 'Group System write' permissions for the GcUser's UserGlobals
 and garbage collection privilege.
"
^self _setPersistentConfig: false symbol: configSymbol toValue: aVal
%

category: 'Garbage Collection Management'
classmethod: System
setPersistentReclaimConfig: configSymbol toValue: aVal

"Updates the runtime reclaim configuration option, see setReclaimConfig:toValue:.
 If that is successful, then it commits the change to the persistent configuration.
 If an abort would cause unsaved changes to be lost, it does not execute and
 signals an error, #rtErrAbortWouldLoseData.

 User must have 'Group System write' permissions for the GcUser's UserGlobals
 and garbage collection privilege.
"
^self _setPersistentConfig: true symbol: configSymbol toValue: aVal
%

category: 'Transient Session State'
classmethod: System
setProgressCountTo: aSmallInt

"Sets the value of the progressCount statistic to aSmallInt.
 Returns aSmallInt."

^ self _sessionCacheStatAt: -1 put: aSmallInt
%

category: 'Garbage Collection Management'
classmethod: System
setReclaimConfig: configSymbol toValue: aVal

"Sets the specified runtime reclaim gem configuration option to aVal.  This change
 remains in effect until it is changed with this method or setPersistentReclaimConfig:.

 Boolean values can be set using either true/false or 1/0.
 See getReclaimConfig documentation for the list of valid config symbols.
 Returns the new value.
 Requires garbage collection privilege.
 The new setting should take effect within a few seconds unless the reclaim process
 is in a long commit.
 The reclaimGem will print 'Parameter changes noticed:' message to its log file when it discovers the change.
"

^self _gcGemConfig: true symb: configSymbol toValue: aVal
%

category: 'Session Control'
classmethod: System
setSessionPriority: anInt forSessionId: aSessionId

"Sets the priority of the given session to a new value.

 Session priority is used by the stone to order requests for
 service by sessions.  anInt must be one of the following valid
 priorities:

 0 - lowest priority
 1 - low priority
 2 - normal priority (default)
 3 - high priority
 4 - highest priority

 By default, user sessions have normal (medium) priority and reclaim and
 Admin GC gems have lowest priority.

 A session which holds or is about the receive the commit token is always
 given highest priority by the stone regardless of what priority is assigned
 to the session.

 Session priority is only pertinent when more than one session requests
 service from the stone during the same stone wakeup interval.  A single
 session requesting service is always processed immediately.

 Sessions with the same priority are serviced in ascending session ID order.

 Requires the SessionAccess and SessionPriority privileges.

 The priority of the PageManager session may not be changed.

 If successful, this method answers a SmallInteger which indicates the previous
 priority of the session.  -1 is returned if aSessionId does not refer to
 a valid session.  An exception is raised if anInt is not a valid priority or
 the session executing this method does not have the required privileges."

^self _sessionPriorityPrim: anInt forSessionId: aSessionId
%

category: 'System Control'
classmethod: System
shadowPagesCount

"Return the number of shadow pages in the system, same as scavengablePagesCount."
^self stoneCacheStatisticWithName: 'PagesNeedReclaimSize'
%

category: 'Configuration File Access'
classmethod: System
sharedCacheAttributesReport
"Returns a string describing attributes for the shared page cache to which the current 
 session is attached. Returns nil if the current session is a solo session."

 | dict cacheName keys ws |
 dict := self getSharedCacheAttributes .
 dict ifNil:[ ^ nil ].
 cacheName := dict at: #cacheName .
 keys := SortedCollection withAll: dict keys .
 ws := WriteStream on: String new.
 ws nextPutAll: 'Attributes Report For Shared Cache Named ''';
    nextPutAll: cacheName;
    nextPut: $' ;
    nextPut: $: .

 keys do:[:key|
   ws lf ; tab ;
      nextPutAll: key asString;
      nextPutAll: ' -> ';
      nextPutAll: (dict at: key) asString .
].
^ ws contents
%

category: 'Shared Counters'
classmethod: System
sharedCounter: index

"Return the value at the shared counter at the specified index.
 index must be in the range 0...(System numSharedCounters - 1) .
 See sharedCounter:setValue: for more information on shared counters."

  ^(self _sharedCounterFetchValuesFrom: index to: index) first
%

category: 'Shared Counters'
classmethod: System
sharedCounter: index decrementBy: amount

"Decrement the value of the shared counter at index by the amount amount.
 Return the new value of that shared counter.
 See sharedCounter:setValue: for more information on shared counters.

 index argument must a SmallInteger in the range 0 to (System _numSharedCounters - 1).

 amount argument must be in the range 0 to 2**63 - 1 (9223372036854775807).
 If decrementing by <amount> would decrease the counter value to be less than the
 minimum (-2**63 or -9223372036854775808), the value is set to the minmum instead."

  ^self _sharedCounter: index decrementBy: amount withFloor: nil
%

category: 'Shared Counters'
classmethod: System
sharedCounter: index decrementBy: amount withFloor: floorValue

"Decrement the value of the shared counter at <index> by the amount <amount>.
 Return the new value of that shared counter.
 See sharedCounter:setValue: for more information on shared counters.

 index argument must a SmallInteger in the range 0 to (System _numSharedCounters - 1).

 amount argument must be in the range 0 to 2**63 - 1 (9223372036854775807).

 floorValue specifies the lowest counter value. If decrementing by <amount> would
 decrease the counter value to be less than the floorValue, the value is set to
 floorValue instead."

  ^self _sharedCounter: index decrementBy: amount withFloor: floorValue
%

category: 'Shared Counters'
classmethod: System
sharedCounter: index incrementBy: amount

"Increment the value of the shared counter at the specified index by the specified
 amount. Return the new value of that shared counter.
 See sharedCounter:setValue: for more information on shared counters.

 index argument must a SmallInteger in the range 0 to (System _numSharedCounters - 1).

 amount argument must be in the range 0 to 2**63 - 1 (9223372036854775807).
 If incrementing by <amount> would increase the counter value to be greater than the
 maximum (2**63 - 1 or 9223372036854775807), the value is set to the maximum instead."

<primitive: 354>
| sz |
index _validateClass: SmallInteger.
sz := self _numSharedCounters .
(index < 0 or:[ index >= sz ])
  ifTrue:[ index _error: #rtErrArgOutOfRange args: { 0 . sz - 1 } ] .
amount _validateClass: Integer .
(amount < 0 or:[ amount > 9223372036854775807 ])
  ifTrue:[ amount _error: #rtErrArgOutOfRange args: { 0 . } ] .
self _primitiveFailed: #sharedCounter:incrementBy: args: { index . amount }
%

category: 'Shared Counters'
classmethod: System
sharedCounter: n setValue: i

"Set the shared counter at the specified index to ths specified value.
 Return the receiver.

 The argument n must a SmallInteger in the range 0 to
(System _numSharedCounters - 1) .

 The argument i must be a SmallInteger or LargeInteger in the range
 -2**63 (-9223372036854775808) to 2**63 - 1 (9223372036854775807).

 Shared counters allow multiple sessions on the same host to read and update
 a common counter value. Each counter is protected by a unique spinlock.
 Shared counters are transient, i.e. do not persist across cache restart.
 See persistentCounterAt:put:, etc. for accessing persistent shared counters,
 for which values do persist over shutdown and are recoverable from tranlogs.
 Shared counter values are recorded by statmonitor when using the -n option
 and recorded as AppStats.

 Shared counters are indexed from 0 to (System numSharedCounters - 1), which is
 set by the config parameter SHR_PAGE_CACHE_NUM_SHARED_COUNTERS, and defaults
 to 1900."

<primitive: 350>
| sz |
n _validateClass: SmallInteger.
i _validateClass: Integer .
sz := self _numSharedCounters .

(n < 0 or:[ n >= sz ])
  ifTrue:[n _error: #rtErrArgOutOfRange args:{ 0 . sz - 1 } ] .
(i < -9223372036854775808 or:[ i > 9223372036854775807 ])
   ifTrue:[i _error: #rtErrArgOutOfRange args:{ -9223372036854775808 . 9223372036854775807 } ] .
self _primitiveFailed: #sharedCounter:setValue: args: { n . i }
%

category: 'Shared Counters'
classmethod: System
sharedCounterFetchValuesFrom: firstCounter to: lastCounter

"Return an array containing the values from all shared counters starting
 with the counter at index <firstCounter>, up to and including the value
 from the counter <lastCounter>.  Note that shared counters are indexed
 starting at 0 and the result array indexes starting at 1.

 Both arguments must be in the range 0...(System numSharedCounters - 1), and
 lastCounter must be greater than or equal to firstCounter."

  ^self _sharedCounterFetchValuesFrom: firstCounter to: lastCounter
%

category: 'Cache Statistics'
classmethod: System
sharedPageCacheMonitorCacheStatistics

"Return the cache statistics for the SPC monitor process.

 Only cache statistics applicable to the SPC monitor process are returned.
 The description of these statistics can be determined by evaluating:
   'System cacheStatisticsDescriptionForMonitor'"

^ self cacheStatisticsAt: 0  "monitor is always slot 0"
%

category: 'System Control'
classmethod: System
shutDown

"Aborts all current sessions, then terminates them.  Finally, the GemStone
 system is shut down.  The session issuing this message terminates with a
 broken connection.  Requires the SystemControl privilege."

^ self _zeroArgPrim: 8
%

category: 'Error Handling'
classmethod: System
signal: anInteger args: anArray signalDictionary: anErrorDict

"Will be deprecated.
 This method generates the specified Exception, along with its
 associated arguments, and signals it.

 Gs64 v3.0  anErrorDict is ignored. "

^ (AbstractException _new: anInteger args: anArray ) signal
%

category: 'Signals'
classmethod: System
signalAlmostOutOfMemoryThreshold: anInteger

"Controls the generation of an error when session's temporary object memory
 is almost full .
 anInteger = -1, disable the generation of the error, resets threshold to 85% ,
		and suppress smalltalk stack printing before OOM.
 anInteger = 0,  enable generation of the error with previous threshold,
		default threshold after session login is 85% .
 0 < anInteger < 125 , enable generation of the error with specified threshold.

 The error is generated at the end of an in-memory markSweep,
 when the amount of memory used exceeds the specified threshold.
 If the session is executing a user action, or in index maintenance,
 the error is deferred and generated when execution returns to the
 bottom-level Smalltalk or GCI portion of the stack.

 This method or enableAlmostOutOfMemoryError must be invoked after
 each delivery of the AlmostOutOfMemory error to reenable generation
 of the error."

self _updateSignalErrorStatus: 5 toState: anInteger
%

category: 'Transaction Control'
classmethod: System
signaledAbortErrorStatus

"Returns true to indicate that the system generates an error when it receives
 the abort signal from Stone.  (In other words, verify that
 enableSignaledAbortError has been called to activate detection of the
 RT_ERR_SIGNAL_ABORT signal.)  Returns false otherwise.

 If in transaction, the result reflects what will happen to abort signals
 after the next commit or abort which exits the transaction.
 "

^self _signalErrorStatus: 2
%

category: 'Transaction Control'
classmethod: System
signaledFinishTransactionErrorStatus

"Returns true to indicate that the system generates an error when it receives
 the finishTranaction signal from Stone.  (In other words, verify that
 enableSignaledFinishTransactionError has been called ).  Returns false otherwise."

^self _signalErrorStatus: 4
%

category: 'Notification'
classmethod: System
signaledGemStoneSessionErrorStatus

"Returns true to indicate that the current GemStone session can receive signals
 from other GemStone sessions.  Returns false otherwise."

^self _signalErrorStatus: 3
%

category: 'Deprecated'
classmethod: System
signalFromGemStoneSession

"Returns a 4 element Array containing information about a signal from
 another GemStone session:

 1.  An instance of GsSession representing the session that sent the signal.
 2.  The signal value (a SmallInteger).
 3.  A signal message (a String).
 4.  A SmallInteger, number of additional signals pending in stone

 If there is no signal in the queue, returns an empty Array."

| result |
self deprecated: 'System class >> signalFromGemStoneSession: deprecated in v3.0; use InterSessionSignal'.
result := self _signalFromGemStoneSession .
result == nil ifTrue:[ ^ { } ].
result at: 1 put: (GsSession sessionWithSerialNumber: (result at: 1)).
^ result
%

category: 'Notification'
classmethod: System
signalTranlogsFullStatus

"Returns true to indicate that the session will get an error when
 stone detects a tranlogs full condition.  Returns false otherwise."

^self _signalErrorStatus: 6
%

category: 'Session Control'
classmethod: System
sleep: aTime

"Sleep for aTime seconds.  aTime must be a positive SmallInteger.
 If aTime is zero, this method has no effect.
 Returns the receiver.
 The ProcessorScheduler is not used, and other waiting GsProcesses
 will not execute during the sleep. Use a Delay or Semaphore to wait
 while allowing other GsProcesses to run."

| timeLeft |
aTime _isSmallInteger ifFalse:[ aTime _validateClass: SmallInteger ].
aTime < 0 ifTrue: [ aTime _error: #errArgTooSmall args:{ 0 } ] .
timeLeft := aTime .
[ true ] whileTrue:[
  timeLeft := self _sleep: timeLeft .
  timeLeft <= 0 ifTrue:[ ^ self ].
]
%

category: 'Debugging Support'
classmethod: System
stackDepth

"Returns current depth of the GemStone Smalltalk stack."

^ self _zeroArgPrim: 26
%

category: 'Debugging Support'
classmethod: System
stackDepthHighwater

"Returns largest depth of the GemStone Smalltalk stack since session login."

^ self _zeroArgPrim: 27
%

category: 'Debugging Support'
classmethod: System
stackLimit

"Returns the approximate limit on the depth of the GemStone Smalltalk stack.
 The stack size is determined by the configuration file parameter
 GEM_MAX_SMALLTALK_STACK_DEPTH."

^ self _zeroArgPrim: 1
%

category: 'Debugging Support'
classmethod: System
stackLimit: anInteger

"Has no effect in this release.  Provided for compatibility."

anInteger _validateClass: SmallInteger .
(anInteger < 0) ifTrue: [ anInteger _error: #rtErrArgNotPositive ].
^ self
%

category: 'Deprecated'
classmethod: System
startAdminGcSession

"Start the Admin GC session if it is not running.
 Returns true if the gem was started and false if
 it could not be started because it is already running
 or logins are suspended.
 Requires the GarbageCollection privilege."

self deprecated: 'System class >> startAdminGcSession deprecated v3.2.  Use startAdminGem instead'.
^self startAdminGem
%

category: 'Garbage Collection Management'
classmethod: System
startAdminGem

"Start the Admin GC session if it is not running.
 Returns true if the gem was started and false if
 it could not be started because it is already running.
 Requires the GarbageCollection privilege."

^self _zeroArgPrim: 67
%

category: 'Garbage Collection Management'
classmethod: System
startAllGcGems

"Start all enabled garbage collector gems that are not running.
 Does not start the Symbol Creation Gem.
 Requires the GarbageCollection privilege."

^ self _zeroArgPrim: 18
%

category: 'Deprecated'
classmethod: System
startAllGcSessions

"Start all enabled garbage collector gems that are not running.
 Does not start the Symbol Creation Gem.
 Requires the GarbageCollection privilege."

self deprecated: 'System class >> startAllGcSessions deprecated v3.2.  Used startAllGcGems instead'.
^ self startAllGcGems
%

category: 'Deprecated'
classmethod: System
startAllReclaimGcSessions

"Start all recalim GC sessions that are configured to run.
 Returns the number of reclaim sessions started.

 Requires the GarbageCollection privilege."

self deprecated: 'System class >> startAllReclaimGcSessions deprecated v3.2.  Use startReclaimGem instead'.
^self startReclaimGem
%

category: 'Transaction Control'
classmethod: System
startCheckpointAsync
"Starts an asynchronous checkpoint.  If a checkpoint is already
 in progress, this method will not start another.  Instead, it
 will return 'true', indicating a checkpoint is in progress.   If a
 checkpoint is not in progress, a new checkpoint is started and
 this method returns immediately.   To block until the checkpoint
 completes, use the startCheckpointSync method instead.

 Unlike the commitTransactionWithCheckpoint method, this method
 does not commit (or otherwise modify) the current transaction.

 Requires the SystemControl privilege.

 A result of true means a checkpoint was successfully started.
 A result of false means a checkpoint could not be started.  This
 usually happens because checkpoints are suspended or no tranlog
 space is available, or the repository is in restore-from-log state.
 It can also happen if more than one session
 calls this method at the same time."

 | status |
 status := self _zeroArgPrim: 63 .
 ^ status == 1"in progress" or:[ status == 2"started"].
%

category: 'Transaction Control'
classmethod: System
startCheckpointSync
"Starts a synchronous checkpoint.  If a checkpoint is already in progress, 
 this method waits for it to complete, and then starts another checkpoint. 
 The wait for the already running checkpoint to complete.  
 can be interrupted by GciSoftBreak  (ctl-C in topaz).  Once this
 method starts its own checkpoint, the session is suspended in stone
 (waiting in a primitive method)  until stone completes the checkpoint.
 
 If successful, this method returns after starting and completing a checkpoint.
 Otherwise it signals an Error.  
 To start a new checkpoint and return before it completes, use the 
 startCheckpointAsync method instead of this one.

 This method does not commit (or otherwise modify) the current transaction.

 Requires the SystemControl privilege.

 A result of true means a checkpoint was successfully finished, 
 otherwise an Error is signaled.  Failure to start a checkpoint can occur 
 because checkpoints are suspended or tranlog space is full, or
 the repository is in restore-from-log mode.
"
 | printed |
 [ true ] whileTrue:[ | status |
   "Session waits in prim for stone to reply, if checkpoint was not in progress
    and we started one."
   status := self _zeroArgPrim: 62 . 
   status == 0 ifTrue:[ 
     (self checkpointStatus at: 1) == true ifTrue:[
        Error signal:'startCheckpointSync failed, checkpoints are suspended'.
     ].
     Error signal:'startCheckpointSync failed'.
   ].
   status == 2 ifTrue:[ ^ true "we started and completed a checkpoint"].

   "status == 1 , a checkpoint already running but not one that we started"
   printed ifNil:[ 
     GsFile gciLogClient:'Waiting for preceeding checkpoint to finish'.
     printed := true .
   ].
   Delay waitForSeconds: 5
 ].
%

category: 'Garbage Collection Management'
classmethod: System
startMaxReclaimGemSessions

"Starts the configured maximum number of reclaim threads."

| maxReclaim |

maxReclaim := System stoneConfigurationAt: #STN_MAX_GC_RECLAIM_SESSIONS.
maxReclaim == 0 ifTrue: [ maxReclaim := SystemRepository numberOfExtents ].

^self changeNumberOfReclaimGemSessions: maxReclaim.
%

category: 'Host System Access'
classmethod: System
startNewGemLog: aFileName
 "aFileName must be a file name (no '/' allowed) which will be appended to
  the result of System class >> gemLogPath to open a new log file.
  The existing log, the path for which is System class >> gemLogFileName prior to sending
  startNewGemLog:,  will be closed,  then stdout and stderr of this sessions' gem process
  will be redirected to the new log file. If aFileName already exists, it will be appended 
  to. If aFileName cannot be created or written to, the existing log file will continue 
  to be used.

  Has no effect and signals a Warning in a topaz -l or other linked GCI application.
  Returns receiver if successful. "

 <primitive: 683>
 self clientIsRemote ifFalse:[
   ^ Warning signal:'System class>>startNewGemLog:  has no effect in a linked GCI application'.
 ].
 aFileName _validateClass: String.
 self _primitiveFailed: #startNewGemLog args: { aFileName }
%

category: 'System Control'
classmethod: System
startNewLoginLog

"Tells the stone to close the current login log file and open a new one.

 Requires the FileControl privilege.

 Returns true if the action was successful or false if the action could
 not be completed or if the login log file is not enabled."

^ self _zeroArgPrim: 173
%

category: 'System Control'
classmethod: System
startNewObjectReadLog
 "Requests stone close the current object read log file and start a new one.
 Blocks until the operation has completed. Returns true on success or false if
 the object read logging feature is disabled. Raises an exception on error.
 Requires the FileControl privilege."

  ^ self _zeroArgPrim: 191
%

category: 'Garbage Collection Management'
classmethod: System
startReclaimGem

"If a reclaimGem is running and the number of threads is greater than the value of 
 the runtime config #StnNumGcReclaimSessions, then simply return the current number of threads.
 If the number of threads running is less than the config value then set it to the config. 

 If a reclaimGem is not running it starts a reclaimGem using the value of the 
 #StnNumGcReclaimSessions runtime configuration to determine the number of sessions to start.

 Returns the number of reclaim threads that are running.
"
| numReclaim configReclaimSessions|

numReclaim := self reclaimGemSessionCount.
configReclaimSessions := System stoneConfigurationAt: #StnNumGcReclaimSessions.
(numReclaim > 0) ifTrue: [
     (numReclaim > configReclaimSessions) ifTrue: [ ^ numReclaim ].
   ].
^ self changeNumberOfReclaimGemSessions: configReclaimSessions.
%

category: 'Deprecated'
classmethod: System
startReclaimGemForExtentRange: startExtent to: endExtent

"Deprecated - all extents are covered by a single reclaim gem"

self deprecated: 'System class>>startReclaimGemForExtentRange:to: deprecated in v3.2.'.
^self startReclaimGem
%

category: 'Deprecated'
classmethod: System
startReclaimGemForExtentRange: startExtent to: endExtent onHost: aHostNameOrNil

"Deprecated - all extents are covered by a single reclaim gem"

self deprecated: 'System class>>startReclaimGemForExtentRange:to:onHost: deprecated in v3.2.'.
^self startReclaimGem
%

category: 'Deprecated'
classmethod: System
startReclaimGemForExtentRange: startExtent to: endExtent onHost: aHostNameOrNil stoneHost: stoneHostOrNil

"Deprecated - all extents are covered by a single reclaim gem"

self deprecated: 'System class>>startReclaimGemForExtentRange:to:onHost:stoneHost: deprecated in v3.2.'.
^self startReclaimGem
%

category: 'Deprecated'
classmethod: System
startSymbolCreationSession

"Start the Symbol Creation session if it is not running.
 Returns true if the gem was started and false if
 it could not be started because it is already running
 or logins are suspended.

 Requires the GarbageCollection privilege."

self deprecated: 'System class >> startSymbolCreationSession deprecated v3.2.  Use startSymbolGem instead'.
^self startSymbolGem
%

category: 'Garbage Collection Management'
classmethod: System
startSymbolGem

"Start the Symbol Creation session if it is not running.
 Returns true if the gem was started and false if
 it could not be started because it is already running.

 Requires the GarbageCollection privilege."

^self _zeroArgPrim: 45
%

category: 'Cache Statistics'
classmethod: System
stoneCacheStatistics

"Return the cache statistics for the stone if this session is located on
 the same host as the stone process.

 Only cache statistics applicable to the Stone process are returned.
 The description of these statistics can be determined by evaluating:
   'System cacheStatisticsDescriptionForStone'

 nil is returned if the stone is not running the session's host."

^ self sessionIsOnStoneHost
    ifTrue:[ self cacheStatisticsAt: 1 ] "stone is always slot 1, if it's here at all"
    ifFalse:[ nil ]
%

category: 'Cache Statistics'
classmethod: System
stoneCacheStatisticWithName: aString

"Return the value of the cache stat with the given name, which must match an
 element of the Array returned by the #cacheStatisticsDescription method
 applicable to the stone process.  The UserTime and SysTime statistics cannot
 be accessed using this method.

 Returns nil if a statistic matching aString was not found for the stone.

 This method may be used on hosts remote from the stone process."

^ self _cacheStatWithName: aString opCode: 2
%

category: 'Runtime Configuration Access'
classmethod: System
stoneCommandLineArgs

"Returns a String representing the command line arguments used to start the stone,
 or nil if an error occurs."

^ self _zeroArgPrim: 1038
%

category: 'Configuration File Access'
classmethod: System
stoneConfigurationAt: aName

"Returns the value of the specified configuration file parameter from the
 repository monitor process (Stone).  Returns nil if that parameter is not
 applicable to the Stone."

| cfgId |
cfgId := ConfigurationParameterDict at: aName otherwise: nil .
cfgId == nil ifTrue:[ ^ nil ].
^ self _configurationAt: cfgId isStone: true kind: $C
%

category: 'Configuration File Access'
classmethod: System
stoneConfigurationAt: aName put: aValue

"Changes the value of the specified Stone configuration parameter.

 See comments in the method configurationAt:put: for complete documentation."

^ self configurationAt: aName put: aValue
%

category: 'Configuration File Access'
classmethod: System
stoneConfigurationFileNames
"Answer a SymbolDictionary containing two keys-value pairs:
  #GEMSTONE_SYS_CONF -> (String) Filename of the stone's system configuration file.
  #GEMSTONE_EXE_CONF -> (String) Filename of the stone's executable configuration file.

 If either file does not exist, the value an empty String."

|array result|
array := self stoneConfigurationAt: #StnConfigFileNames .
result := SymbolDictionary new.
result at: #GEMSTONE_SYS_CONF put: (array at: 1) ;
       at: #GEMSTONE_EXE_CONF put: (array at: 2) .
^ result
%

category: 'Configuration File Access'
classmethod: System
stoneConfigurationReport

"Returns a SymbolDictionary whose keys are the names of configuration file
 parameters, and whose values are the current settings of those parameters in
 the repository monitor process (Stone).  Parameters that are not applicable to
 Stone and those that are undefined are not include in the result."

^ self _configurationReport: true
%

category: 'Runtime Configuration Access'
classmethod: System
stoneIsBigEndian

" Returns true if the stone process is running
  on a machine using big endian byte ordering "

^ self _zeroArgPrim: 107

%

category: 'Cache Statistics'
classmethod: System
stoneMessageKinds

"Returns an array of Strings that lists the request kinds which can be sent
 from a session to the stone.  The numeric index of a message may be obtained
 from the 'MessagesToStone' (gem and page servers) or 'GsMsgKind' (stone)
 cache statistics.

 The result of this method may change with every new release of GemStone/64."

^ self _zeroArgPrim: 133
%

category: 'Environment Access'
classmethod: System
stoneName

"Returns a String whose value is the full network name of the
 Stone to which this session is logged in."

^ self _zeroArgPrim: 11
%

category: 'Session Control'
classmethod: System
stoneStartupId
"Returns an Integer, a 64bit random number created in stone startup."

^ self _zeroArgPrim: 149
%

category: 'Version Management'
classmethod: System
stoneVersionAt: aString

"Returns version information about the Stone (repository monitor) process.

 See System (C) | gemVersionAt: for further details.
 aString = 'imageKind' returns nil for the Stone."

| verId |

aString = 'imageKind' ifTrue:[ ^ nil "image not applicable to Stone"].

verId := VersionParameterDict at: aString otherwise: nil .
verId ifNil:[ ^ nil ].

^ self _configurationAt: verId isStone: true kind: $V
%

category: 'Version Management'
classmethod: System
stoneVersionReport

"Return a StringKeyValueDictionary whose keys are the names of operating system,
 hardware, or GemStone version attributes, and whose values are the
 current values of those attributes in the Gem process."

^ self _serverVersionReport: true
%

category: 'Version Management'
classmethod: System
stoneVersionReportString
  "Returns a String with keys and values of stoneVersionReport one pair per line."
^ self stoneVersionReport _reportString
%

category: 'Deprecated'
classmethod: System
stopAdminGcSession

"Stop the Admin GC session if it is running.
 Requires the GarbageCollection privilege."

self deprecated: 'System class >> stopAdminGcSession deprecated v3.2.  Use stopAdminGem instead'.
^self stopAdminGem
%

category: 'Garbage Collection Management'
classmethod: System
stopAdminGem

"Stop the Admin GC session if it is running.
 Requires the GarbageCollection privilege."

^self _zeroArgPrim: 42
%

category: 'Garbage Collection Management'
classmethod: System
stopAllGcGems

"Stop any and all Garbage Collection sessions.
 Does not stop the Symbol Creation Gem .
 Requires the GarbageCollection privilege."

^self _zeroArgPrim: 40
%

category: 'Deprecated'
classmethod: System
stopAllGcSessions

"Stop any and all Garbage Collection sessions.
 Does not stop the Symbol Creation Gem .
 Requires the GarbageCollection privilege."

self deprecated: 'System class >> stopAllGcSessions deprecated v3.2.  Used stopAllGcGems instead'.
^self stopAllGcGems
%

category: 'Deprecated'
classmethod: System
stopAllReclaimGcSessions

"Stop any and all reclaim GC sessions.
 Requires the GarbageCollection privilege."

self deprecated: 'System class >> stopAllReclaimGcSession deprecated v3.2.  Use stopReclaimGem instead'.
^self stopReclaimGem
%

category: 'Session Control'
classmethod: System
stopAllSessionsReferencingOldestCr

"Causes the stone to take the following actions:
 1 - send a stopSession message to all sessions referencing the oldest
     commit record (see System(C)>>stopSession:)
 2 - forces all sessions referencing the oldest commit record to immediately
     log out.
 3 - disposes all commit records eligible for disposal.

 Does not stop the current session, even if it references the
 oldest commit record.  Has no effect if there is only one commit
 record present.

 Does not force the immediate disposal of the oldest commit record.

 Requires the SystemControl privilege.

 Returns a SmallInteger indicating the number of commit records
 disposed."

^ self _zeroArgPrim: 162
%

category: 'Deprecated'
classmethod: System
stopGcSession: aSessionId

"Aborts the transaction of the specified session (a SmallInteger), then
 terminates that session.  Returns the receiver.  If the indicated
 session is not active, no operation is performed.

 Will stop Garbage Collector sessions but will not stop the
 Symbol Creation Gem session .

 To execute this method, you must have SystemControl and SessionAccess
 privileges."

| serialNum |

self deprecated: 'System class>>stopGcSession: is deprecated v3.3. Use stopAdminGem or stopReclaimGem.'.

aSessionId _validateClass: SmallInteger.
serialNum := GsSession serialOfSession: aSessionId .
self _stopSession: serialNum kind: 16r1 timeout: -1
%

category: 'Session Control'
classmethod: System
stopHostAgentForHost: hostNameOrIp
  "hostNameOrIp is a String.
   Returns a string describing action taken."
  | targetIp pairs haUp sess |
  targetIp := GsSignalingSocket getHostAddressByName: hostNameOrIp .
  pairs := self _currentSessionProfiles.
  haUp := AllUsers userWithId:'HostAgentUser'.
  sess := { } .
  1 to: pairs size by: 2 do:[:j |
    (pairs at: j+1 ) == haUp ifTrue:[ sess add: (pairs at: j)].
  ].
  sess do:[:sesId | | desc leafIp |
    desc := System descriptionOfSession: sesId .
    leafIp := desc at: 11 .
    leafIp = targetIp ifTrue:[
      self stopHostAgentSession: sesId .
      ^ 'stopped session ' , sesId asString .
    ]
  ].
  ^ 'hostagent not found'
%

category: 'Session Control'
classmethod: System
stopHostAgents

"Stops all HostAgent processes that are logged in to this stone.
 Each hostagent executes graceful shutdown.
 Signals an Error if unable to stop all hostagents."

^ self _stopHostAgents: 16r4"Break" timeOut: 20
%

category: 'Session Control'
classmethod: System
stopHostAgentSession: sessionId
  "Graceful shutdown of the HostAgent with the specified sessionId"

^ self _stopHostAgentSession: sessionId kind: 16r4"Break"
%

category: 'Session Control'
classmethod: System
stopOtherSessions
  "Stops user sessions other than the current session and reenables logins.
   Does not stop reclaim, admin nor symbol creation Gems."

  self stopUserSessions ; resumeLogins
%

category: 'Garbage Collection Management'
classmethod: System
stopReclaimGem

"Stop all reclaim GC sessions.
 Requires the GarbageCollection privilege."

^ self changeNumberOfReclaimGemSessions: 0
%

category: 'Session Control'
classmethod: System
stopSession: aSessionId

"Aborts the transaction of the specified session (a SmallInteger), then
 terminates that session.  Returns the receiver.  If the indicated
 session is not active, no operation is performed.

 Implemented by sending a stopSession out-of-band byte to the session's
 gem process.   If the session is not responsive to this request,
 it will eventually be killed per STN_GEM_TIMEOUT in stone's config file.

 Does not stop Garbage Collector or Symbol Creation Gem sessions.

 To execute this method, you must have SystemControl and SessionAccess
 privileges."

| serialNum |

aSessionId _validateClass: SmallInteger.
serialNum := GsSession serialOfSession: aSessionId .
self _stopSession: serialNum kind: 16r0 timeout: -1
%

category: 'Deprecated'
classmethod: System
stopSymbolCreationSession

"Stop the Symbol Creation session if it is running.

 This is an extreme action that should never be done in normal
 operation of a system.  Stopping the Symbol Creation session
 can prevent other sessions from being able to commit until
 after the Symbol Creation session restarts and after
 the other sessions logout and login .

 Only SystemUser may execute this method, otherwise an error is generated."

self deprecated: 'System class >> stopSymbolCreationSession deprecated v3.2.  Use stopSymbolGem instead'.
^self stopSymbolGem
%

category: 'Session Control'
classmethod: System
stopSymbolCreationSession: aSessionId

"Aborts the transaction of the specified session (a SmallInteger), then
 terminates that session.  Returns the receiver.  If the indicated
 session is not active, no operation is performed.

 Will stop any session including garbage Collector and
 Symbol Creation Gem sessions.

 Only SystemUser may execute this method, otherwise an error is generated."

| serialNum |

aSessionId _validateClass: SmallInteger.
serialNum := GsSession serialOfSession: aSessionId .
self _stopSession: serialNum kind: 16r3  timeout: -1
%

category: 'Garbage Collection Management'
classmethod: System
stopSymbolGem

"Stop the Symbol Creation session if it is running.

 This is an extreme action that should never be done in normal
 operation of a system.  Stopping the Symbol Creation session
 can prevent other sessions from being able to commit until
 after the Symbol Creation session restarts and after
 the other sessions logout and login .

 Only SystemUser may execute this method, otherwise an error is generated."

^self _zeroArgPrim: 44
%

category: 'Session Control'
classmethod: System
stopUserSessions

"Prevents any new sessions from being initiated; then, for each
 active user session other than the session of the user executing this
 method, aborts the transaction and terminates that session.

 This method does not stop Gc Gem sessions nor does it stop
 the Symbol Creation Gem .

 To reenable logins, send the message System resumeLogins.  Otherwise,
 logins are automatically reenabled when this session logs out.

 To execute this method, you must have explicit privilege for
 SessionAccess and SystemControl in your UserProfile."


"Performs a System | suspendLogins, then
 performs a System | stopSession."

| anArrayOfSessions mySession myPrivs |

myPrivs := self myUserProfile privileges.
((myPrivs includesValue: #SystemControl) and:
   [myPrivs includesValue: #SessionAccess])
   ifFalse: [ self _error: #rtErrNoPriv .  ^ self "do nothing" ].
self suspendLogins.  " Prevent anyone from logging in "

anArrayOfSessions := self currentSessions.  " All Session running "
mySession := self session.  " My Session number "

anArrayOfSessions do: " Loop all session and log then off if not me "
   [:aSessionNumber | (aSessionNumber == mySession )
                         ifFalse:[self stopSession: aSessionNumber].
   ].
%

category: 'Session Control'
classmethod: System
stopZombieSession: aSessionId

"Forces the stone to finish logout processing for the given session ID.
 Takes no action if the given session has not begun the logout process or
 if the session no longer exists or if the session is the current session.
 When logout processing is completed, references to a commit record by
 that session are released and resources consumed by the session are recycled.

 To execute this method, you must have SessionAccess and SystemControl
 privileges.

 WARNING: This method bypasses certain internal mechanisms which
          prevent the immediate termination of active sessions,
          and therefore should only be used if the session
          could not be stopped by any other mechanisms.

 Returns true if the action was successful and the stone acted on the
 specified session.  A message will be written to the stone log in this
 case.  Returns false if no action was taken.  No message is added to the
 stone log when false is returned."

^ self _cleanupZombieSession: aSessionId
%

category: 'Online Backup Support'
classmethod: System
suspendCheckpointsForMinutes: anInt
"Suspend all checkpoints for the given number of minutes or until the
 resumeCheckpoints method is executed, which ever occurs first.  anInt
 must be a positive SmallInteger.

 Requires the SystemControl privilege.

 It is safe to copy the repository extents for backup purposes while
 checkpoints are suspended.

 Checkpoint suspension is not supported in partial tranlog mode.  This
 method will always return false STN_TRAN_FULL_LOGGING is set to FALSE.

 Certain operations which require checkpoints are not permitted while
 checkpoints are suspended, such as full backups.

 Calling this method while checkpoints are already suspended has
 the effect of changing the duration of the suspension.

 If a checkpoint is in progress when this method is called, the call
 will block until the current checkpoint completes, at which time
 checkpoints will be suspended.  If any session has made this
 call and is waiting for the current checkpoint to complete, calls
 to this method by other sessions will fail.

 Returns true if checkpoints were successfully suspended.  Returns
 false if checkpoints could not be suspended because the repository
 is in partial log mode or is in restore from backup or log mode."

<primitive: 285>
anInt  _validateClass: SmallInteger.
(anInt < 0)
  ifTrue:[ anInt _error: #errArgTooSmall args:{ 0 } ] .
^self _primitiveFailed: #suspendCheckpointsForMinutes: args: { anInt }
%

category: 'System Control'
classmethod: System
suspendLogins

"Prevents any new sessions from being initiated.  That is, no new user is
 allowed to login.  However, users already active will be allowed to continue
 processing.

 To reenable logins, send the message System | resumeLogins.  (If you fail to do
 so, GemStone automatically reenables logins when the last user logs out.)

 Requires the SystemControl privilege."

self configurationAt: #StnLoginsSuspended put: 1
%

category: 'Deprecated'
classmethod: System
symbolCreationSessionId

"Return the session ID of the SymbolCreation session.  Returns
 zero if the SymbolCreation session is not running."

self deprecated: 'System class >> symbolCreationSessionId deprecated v3.2.  Use symbolGemSessionId instead'.
^self symbolGemSessionId
%

category: 'Garbage Collection Management'
classmethod: System
symbolGemSessionId

"Return the session ID of the SymbolCreation session.  Returns
 zero if the SymbolCreation session is not running."

^self _zeroArgPrim: 46
%

category: 'Lock Status'
classmethod: System
systemLocks
"
 This is an expensive method, it makes a call to stone for each element
 of the result of
    System currentSessions

 Returns a Dictionary describing all objects that are currently locked.  For
 each Association in the result Dictionary, the key is a SmallInteger (the
 session number of a GemStone session that holds locks) and the value is the
 three-element Array described in the sessionLocks method.  If no sessions hold
 any locks, the result Dictionary is empty.

 The Arrays in the result Dictionary contain by identity those objects that are
 visible to the current session.  For locks on objects that the
 current session cannot see , the Arrays contain uncommitted Strings
 using one of the forms
    uncommitted object with oop NNN
    object with oop NNN (no read auth)  ."

| anArray result |
anArray := self _systemLocksPrim  .
result := Dictionary new .
1 to: anArray size do:[ :j |
   result add: (anArray at:j)
].
^ result
%

category: 'Lock Status'
classmethod: System
systemLocksDetailedReport

"Returns a String describing all of the locked objects.
 This is an expensive method, it makes a call to stone for each element
 of the result of
    System currentSessions

 For each object the oop and the class are printed, example:
session 5(
  (1 writeLocks: 11074305(a SymbolDictionary)))
session 6(  (1 readLocks: 240641(a Repository))
  (2 writeLocks: 979969(a UserProfile) 11074561(a SymbolDictionary)))
"
| str kinds assocs |
kinds := #( 'readLocks' 'writeLocks' 'deferredUnlocks' ).
assocs := self _systemLocksPrim .
assocs size == 0 ifTrue:[ ^ 'no locks' ].
str := String new .
1 to: assocs size do:[:j | | assoc val |
  assoc := assocs at: j .
  str add: 'session '; add: assoc key asString ; add: $( .
  val := assoc value .
  1 to: val size do:[:n | | arr |
    arr := val at: n .
    arr size > 0 ifTrue:[
      n > 1 ifTrue:[ str lf ].
      str add: '  ('; add: arr size asString; add:' ';
          add: (kinds at: n); add: ':' .
      arr do:[:obj |
       str add: ' '; add: obj asOop asString ;
           add: '(a '; add: obj class name .
       obj isCommitted ifFalse:[ str add: ' notCommitted'].
       str add: $)  .
      ].
      str add: $).
    ].
  ].
  str add: $) .
  j < assocs size ifTrue:[ str lf ].
].
^ str
%

category: 'Lock Status'
classmethod: System
systemLocksQuick
 "Returns an Array of 3 Arrays. The elements of the result are
    readLocks
    writeLocks
    deferredUnlocks  (will be unlocked after commit token is released)
 Details about which sessions hold the locks are not included.
 readLocks and writeLocks include locks on not-committed objects from
 the current session.
 "
<primitive: 1083>
^ self _primitiveFailed: #systemLocksQuick .
%

category: 'Lock Status'
classmethod: System
systemLocksReport

"Returns a String that summarizes all of the locked objects.
 details about which session holds the locks are not included.
 Locks on not-committed objects held by the current session are included.

 Example report:
  (1 readLocks: 207361(a SymbolDictionary))
  (1 writeLocks: 11079425(a SymbolDictionary))
  (2 deferredUnlocks: 207361(a SymbolDictionary) 11079425(a SymbolDictionary))
"

 | val kinds str |
 val := self systemLocksQuick .
 str := String new .
 kinds := #( 'readLocks' 'writeLocks' 'deferredUnlocks' ).
 1 to: val size do:[:n | | arr |
    arr := val at: n .
    arr size > 0 ifTrue:[
      n > 1 ifTrue:[ str lf ].
      str add: '  ('; add: arr size asString; add:' ';
          add: (kinds at: n); add: ':' .
      arr do:[:obj |
       str add: ' '; add: obj asOop asString ;
           add: '(a '; add: obj class name .
       obj isCommitted ifFalse:[ str add: ' notCommitted'].
       str add: $)  .
      ].
      str add: $).
    ].
 ].
 ^ str
%

category: 'User-Defined Actions'
classmethod: System
systemUserActionReport

"Returns a SymbolDictionary that provides information about GemStone system user
 actions.  These are user actions that are automatically installed in every
 GemStone session to support classes such as GsFile and GsSocket.

 In the resulting SymbolDictionary, the keys are the symbolic names of the user
 actions, and the values are Booleans.  A values is true if the user action is
 linked with your application, and false if the user action is linked with the
 current GemStone session."

| anArray result |

anArray :=  self _zeroArgCateg2Prim: 9 . "get an Array of SymbolAssociations"
result := SymbolDictionary new .
1 to: anArray size do:[:j | result addAssociation: (anArray at: j ) ].
^ result
%

category: 'Extreme Measures'
classmethod: System
terminateAllSessionsReferencingOldestCr

"Causes the stone to take the following actions:
 1 - send a stopSession message to all sessions referencing the oldest
     commit record (see System(C)>>stopSession:)
 2 - forces all sessions referencing the oldest commit record to immediately
     log out.
 3 - forces all references to the oldest commit record to be released.
 4 - disposes all commit records eligible for disposal.

 Does not terminate the current session, even if it references the
 oldest commit record.  Has no effect if there is only one commit
 record present.

 WARNING: This method bypasses certain internal mechanims which
          prevent the immediate termination of active sessions,
          and therefore should only be used if the oldest CR session(s)
          could not be stopped by any other mechanisms.  The
          System(C)>>stopAllSessionsReferencingOldestCr method should be
          attempted before this one.  Usage of this method when one or more
          session referencing the oldest CR is active may cause cache coherency
          errors.

 Requires the SystemControl privilege.

 Returns a SmallInteger indicating the number of commit records disposed."

^ self _zeroArgPrim: 161
%

category: 'Session Control'
classmethod: System
terminateSession: aSessionId timeout: aSeconds

"Supports faster termination of the specified session than is
 possible with   stopSession:   .

 Sends a stopSession out-of-band byte to the specified session's gem
 process.
 The argument aSeconds is a SmallInteger. aSeconds >= 0 specifies number of
 seconds to wait after sending the stopSession out-of-band byte
 before killing specified session's gem process by sending
 it a SIGTERM.  aSeconds < 0 means move the session to after_logout state
 immediately, but don't kill .

 If the stone config file specifies non-default STN_GEM_TIMEOUT ,
 the actual timeout will be limited by    aSeconds min: STN_GEM_TIMEOUT*60  ,
 otherwise timeout is limited in stone by     aSeconds min: 300 .

 If the current session is not logged in as SystemUser,
 this method requires SystemControl and SessionAccess privileges .

 Will stop normal sessions and Garbage Collector Gem sessions.
 Does not prevent auto-restart of Garbage Collector Gem sessions
 that are stopped by this method.

 Will not stop the Symbol Creation Gem."

| serialNum |
aSessionId _validateClass: SmallInteger.
serialNum := GsSession serialOfSession: aSessionId .
self _stopSession: serialNum kind: 16r11 timeout: aSeconds.
%

category: 'Session Control'
classmethod: System
terminateSymbolCreationSession: aSessionId timeout: aSeconds

"Will stop normal sessions, Garbage Collector Gem sessions,
 and  Symbol Creation Gem .

 Only SystemUser may use this method.
 Does not prevent auto-restart of Symbol Creation Gem sessions
 stopped by this method.

 Otherwise same as terminateSession:timeout:"

| serialNum |
aSessionId _validateClass: SmallInteger.
serialNum := GsSession serialOfSession: aSessionId .
self _stopSession: serialNum kind: 16r13 timeout: aSeconds.
%

category: 'Time'
classmethod: System
timeGmt

"Returns a SmallInteger, the time since January 1, 1970, in seconds.
 The time is computed from the clock of the machine on which the session is
 running, using the offset from the clock on the Stone's (GemStone repository
 monitor process) machine which is cached in the session at login."

^ self _timeGmt: false
%

category: 'Time'
classmethod: System
timeGmt2005

"Returns a SmallInteger, the time since January 1, 2005, in seconds.
 The time is computed from the clock of the machine on which the session is
 running, using the offset from the clock on the Stone's (GemStone repository
 monitor process) machine which is cached in the session at login."

^ self _timeGmt2005: false
%

category: 'Deprecated'
classmethod: System
timeGmt95

"Obsolete - use timeGmt2005 instead of this method.

 Returns a SmallInteger, the time since January 1, 1995, in seconds.
 The time is computed from the clock of the machine on which the session is
 running, using the offset from the clock on the Stone's (GemStone repository
 monitor process) machine which is cached in the session at login."

self deprecated: 'System class>>timeGmt95 deprecated v3.0. Use #timeGmt2005 instead.'.

^ self _timeGmt95: false
%

category: 'Time'
classmethod: System
timeNs

"Returns a SmallInteger representing the current high-resolution
 real time in nanoseconds since some arbitrary time in the past.  The
 result is not correlated in any way with the time of day."

^ self _zeroArgPrim: 95
%

category: 'Session Control'
classmethod: System
totalSessionsCount

"Return the approximate number of sessions logged into the system.
 Avoids a call to stone."
^self stoneCacheStatisticWithName: 'TotalSessionsCount'
%

category: 'Transaction Control'
classmethod: System
transactionConflicts

"Returns a SymbolDictionary that contains an Association with key #commitResult
 and  value is one of the following Symbols:
   #readOnly #success #rcFailure #failure #dependencyFailure #retryFailure
   #commitDisallowed #retryLimitExceeded #symbolFailure #lockFailure  .

 The remaining Associations in the dictionary are used to report the conflicts
 found.  Each Association's key indicates the kind of conflict detected; its
 associated value is an Array of objects that are conflicting.
 If there are no conflicts for the transaction, the returned SymbolDictionary
 has no additional Associations.

 The conflict sets are cleared at the beginning of a commit or abort and
 therefore may be examined until the next commit, continue or abort.
 Conflict sets are empty if the value of commitResult was #readOnly or #success.

 The keys for the conflicts are as follows:

     Key                Conflicts
 Read-Write          StrongReadSet and WriteSetUnion conflicts.
 Write-Write         WriteSet and WriteSetUnion conflicts.
 Write-Dependency    WriteSet and DependencyChangeSetUnion conflicts.
 Write-WriteLock     WriteSet and WriteLockSet conflicts.
 Write-ReadLock      WriteSet and ReadLockSet conflicts.
 Rc-Write-Write      Logical write-write conflict on reduced conflict object.
 WriteWrite_minusRcReadSet  (WriteSet and WriteSetUnion conflicts) - RcReadSet)
 Synchronized-Commit
 RcReadSet

 The Read-Write conflict set has already had RcReadSet subtracted from it.
 The Write-Write conflict set does not have RcReadSet subtracted .

 The WriteSet does not includes objects newly committed by this transaction.
 So a conflict between a lock and a newly committed object will not be a conflict.

 The Write-Dependency conflict set contains objects modified (including DependencyMap
 operations) in the current transaction that were either added to, removed from,
 or changed in the DependencyMap by another transaction. Objects in the
 Write-Dependency conflict set may be in the Write-Write conflict set.

 Note: If you create a reference to a conflict set, be sure to disconnect it before
 committing, to avoid creating a persistent reference to the objects."

 | arr res |

 arr := self _conflictsReport: true withDetails: false .

 res := SymbolDictionary new.
 res at: #commitResult put: (arr at: 1) .

 3 to: arr size by: 3 do:[:j |
   res at: (arr at: j) put: ( arr at: j + 1 "conflict set") .
 ].
 ^ res
%

category: 'Transaction Control'
classmethod: System
transactionLevel
  "Returns 0 if not in a transaction, or a SmallInteger > 0
   indicating the transaction level. > 1 means a nested transaction."
  ^ self _zeroArgPrim: 5
%

category: 'Transaction Control'
classmethod: System
transactionMode

"Returns the current transaction mode for the current GemStone session, either
 #autoBegin, #manualBegin or #transactionless.

 The default is controlled by the stone configuration item
 STN_GEM_INITIAL_TRANSACTION_MODE ."

^ self _zeroArgPrim: 4
%

category: 'Transaction Control'
classmethod: System
transactionMode: newMode

"Sets a new transaction mode for the current GemStone session and exits the
 previous mode by aborting the current transaction.  Valid arguments are
 #autoBegin, #manualBegin and #transactionless.
 The mode transactionless is intended primarily for idle sessions. Users
 may scan database objects, but are at risk of obtaining inconsistent views.
"

| coordinator |
^ (coordinator := self _commitCoordinator) == nil
  ifTrue: [ self _localTransactionMode: newMode ]
  ifFalse: [ coordinator transactionMode: newMode ].
%

category: 'Transaction Control'
classmethod: System
trapAddToClosureList: anObject

"If anObject is a non-committed object, signal an error
 when a subsequent attempt to commit adds anObject to the closure list
 for the commit.  The error will be not-trappable within Gemstone Smalltalk.
 The error information will include the parent dirty or not-committed
 object which caused anObject to be added to the closure list.
 After the error is returned to topaz or GBS, you can then save the
 stack and use Object>>findReferencesInMemory:
 to determine what paths might be causing anObject to become reachable
 from persistent state.

 If anObject == nil, then cancel a previous trap object.
 Returns receiver."

<primitive: 850>
((anObject _status: false) bitAnd: 2) ~~ 0 ifTrue:[
  ArgumentError signal:
   'arg to trapAddToClosureList: must be nil or an object that is neither committed nor in closure of a failed commit'
].
^self _primitiveFailed: #trapAddToClosureList: args: { anObject }
%

category: 'Host System Access'
classmethod: System
unixUserIdExists: aString

"Returns true or false indicating whether or not the UNIX user
 ID representing by aString exists on the host system."

<primitive: 820>
aString _validateClass: String .
self _primitiveFailed: #unixUserIdExists: args: { aString }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol

"Invokes the user-defined action represented by aSymbol.  Generates an error if
 the user action is not installed in this session, or if it expects any
 arguments, or if the user has the NoUserAction privilege.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 200>
^ self _primitiveFailed: #userAction: args: { aSymbol }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: anArg

"Invokes the user-defined action represented by aSymbol, passing it the argument
 anArg.  Generates an error if the user action is not installed in this session,
 or if the number of arguments expected by the user action is not 1, or if the
 user has the NoUserAction privilege.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 200>
^ self _primitiveFailed: #userAction:with: args: { aSymbol . anArg }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg and secondArg.  Generates an error if the user action is not
 installed in this session, or if the number of arguments expected by the user
 action is not 2, or if the user has the NoUserAction privilege.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:
       args: { aSymbol . firstArg . secondArg }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, and thirdArg.  Generates an error if the user
 action is not installed in this session, or if the number of arguments expected
 by the user action is not 3, or if the user has the NoUserAction privilege.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:
       args: { aSymbol . firstArg . secondArg . thirdArg }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, and fourthArg.  Generates an error
 if the user action is not installed in this session, or if the number of
 arguments expected by the user action is not 4, or if the user has the
 NoUserAction privilege."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:
       args: { aSymbol . firstArg . secondArg . thirdArg . fourthArg }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg with: fifthArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, fourthArg, and fifthArg.  Generates
 an error if the user action is not installed in this session, or if the number
 of arguments expected by the user action is not 5, or if the user has the
 NoUserAction privilege."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:with:
       args: { aSymbol . firstArg . secondArg .
               thirdArg . fourthArg . fifthArg}
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg with: fifthArg with: sixthArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, fourthArg, fifthArg, and sixthArg.
 Generates an error if the user action is not installed in this session, or if
 the number of arguments expected by the user action is not 6, or if the user
 has the NoUserAction privilege."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:with:with:
       args: { aSymbol . firstArg . secondArg . thirdArg .
               fourthArg . fifthArg . sixthArg}
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg with: fifthArg with: sixthArg with: seventhArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, fourthArg, fifthArg, sixthArg, and
 seventhArg.  Generates an error if the user action is not installed in this
 session, or if the number of arguments expected by the user action is not 7,
 or if the user has the NoUserAction privilege."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:with:with:with:
       args: { aSymbol . firstArg . secondArg . thirdArg .
               fourthArg . fifthArg . sixthArg . seventhArg }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg with: fifthArg with: sixthArg with: seventhArg
  with: eighthArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, fourthArg, fifthArg, sixthArg,
 seventhArg, and eighthArg.  Generates an error if the user action is not
 installed in this session, or if the number of arguments expected by the
 user action is not 8, or if the user has the NoUserAction privilege."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:with:with:with:with:
       args: { aSymbol . firstArg . secondArg . thirdArg .
               fourthArg . fifthArg . sixthArg . seventhArg . eighthArg }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol withArgs: anArray

"Invokes the user-defined action represented by aSymbol, passing it the elements
 of anArray as arguments.  Generates an error if the user action is not
 installed in this session, or if the number of arguments expected by the user
 action is not the same as the number of elements in anArray, or if the user has
 the NoUserAction privilege.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 201>
^ self _primitiveFailed: #userAction:withArgs: args: { aSymbol . anArray }
%

category: 'User-Defined Actions'
classmethod: System
userActionReport

"Returns a SymbolDictionary that provides information about all user actions
 installed in this GemStone session.  In that SymbolDictionary, the keys are
 the symbolic names of the user actions, and the values are Booleans (true if
 the user action is linked with your application, false if the user action is
 linked with the current GemStone session)."

 | anArray result |
 anArray :=  self _zeroArgCateg2Prim: 7 . "get an Array of SymbolAssociations"
 result := SymbolDictionary new .
 1 to: anArray size do:[:j | result addAssociation: (anArray at: j ) ].
 ^ result

%

category: 'Session Control'
classmethod: System
userProfileForSession: aSessionId

"Returns the UserProfile attached to the specified session (a SmallInteger).
 If the indicated session is not active, returns nil.

 Requires SessionAccess privilege if aSessionId is not the current session."

^ ( self descriptionOfSession: aSessionId) at: 1
%

category: 'Session Control'
classmethod: System
users

"Returns a Set of UserProfiles for all users known to the system."

^ AllUsers
%

category: 'Session Control'
classmethod: System
userSessionsCount

"Return the approximate number of sessions logged into the system.
 Avoids a call to stone."
^ self stoneCacheStatisticWithName: 'UserSessionsCount'
%

category: 'Host System Access'
classmethod: System
validatePasswordForUser: uid password: pw

"Validate the password pw for the user id uid using the Pluggable Authentication
 Modules (PAM) interface.

 Both the uid and pw arguments must be instances of String.

 Returns true if the password was successfully authenticated or false if
 the password is incorrect. Raises an ArgumentTypeError error if the arguments
 are not correct.

 Requires the UserPassword privilege."

^ [ System _validatePasswordForUser: uid password: pw ]
  on: InternalError do: [:ex |
	(Delay forSeconds: 1) wait.
	ex return: false.
    ]
%

category: 'Deprecated'
classmethod: System
validatePasswordForUser: uid password: pw isEncrypted: encrypted

"Obsolete.  Use System class>>validatePasswordForUser:password: instead of this
 method."

self deprecated: 'System class>>validatePasswordForUser:password:isEncrypted
deprecated in v3.2.  Replace with:
System class>>validatePasswordForUser:password:'.

^ [
    System _validatePasswordForUser: uid password: pw
] on: InternalError do: [:ex |
	(Delay forSeconds: 1) wait.
	ex return: false.
].
%

category: 'LDAP Support'
classmethod: System
validatePasswordUsingLdapServers: aUriArray baseDn: aBaseDn filterDn: aFilterDn userId: aUserId password: pw

"Authenticates a password using one or more LDAP servers.
 See the comments in the method
   #validatePasswordUsingLdapServers:baseDn:filterDn:userId:password:bindDn:bindPassword:
 for more information."

^ self validatePasswordUsingLdapServers: aUriArray
       baseDn: aBaseDn
       filterDn: aFilterDn
       userId: aUserId
       password: pw
       bindDn: nil "anonymous bind for searches"
       bindPassword: nil "anonymous bind for searches"
%

category: 'LDAP Support'
classmethod: System
validatePasswordUsingLdapServers: aUriArray baseDn: aBaseDn filterDn: aFilterDn userId: aUserId password: pw bindDn: bindDn bindPassword: bindPw

"Use the LDAP server URI(s) specified in aUriArray to validate the password pw
 is valid for aUserId.

 aUriArray must be an array of strings containing LDAP URI addresses
 (for example: 'ldaps://foo.bar.com').

 In order to validate the password, the complete distinguish name (DN) for
 userId must be determined.  The DN can either be constructed from the baseDn
 pattern (explicit mode) or the DN may be resolved by searching the LDAP directory
 (search mode).  In explicit mode, baseDn must be a string that contains the string
 wildcard sequence '%s'.  GemStone will substitute '%s' with userId before
 doing the password validation.  aFilterDn must be nil in explicit mode.

 In search mode, baseDn is the search pattern used to resolve the DN and must
 NOT contain the string wildcard sequence '%s'.  In search mode, aFilterDn must
 be a string that contains the string wildcard sequence '%s'.  See the examples
 below.

 aUserId must be a string which represents the userId to be validated.
 pw must be a string which is the password for userId.

 bindDn and bindPw must both be either strings or nils.  If both are strings,
 they are used as credentials to bind to the LDAP server to perform the search in
 search mode.  If bindDn and bindPw are both nil, the search is attempted using an
 anonymous bind.  Not all LDAP servers are configured to support anonymous
 binds.  bindDn and bindPw are not used in explicit mode and should be set to nil.

 Returns true if pw is the correct password for userId. Otherwise returns
 false if the password is incorrect or an error occurred while communicating
 with the LDAP directory.

 Setting the variable GS_DEBUG_LDAP in the gem's environment will cause LDAP debugging
 information to be printed to stdout.

 Example 1: Explicit mode

 System validatePasswordUsingLdapServers: (Array with: 'ldaps://myldap.mydomain.com')
        baseDn: 'uid=%s,ou=Users,dc=mycompany,dc=com' filterDn: nil
        userId: 'MyUserId' password: 'swordfish' bindDn: nil bindPassword: nil


 Example 2: Search mode with anonymous bind

 System validatePasswordUsingLdapServers: (Array with: 'ldaps://myldap.mydomain.com')
        baseDn: 'ou=Users,dc=mycompany,dc=com' filterDn: '(uid=%s)'
        userId: 'MyUserId' password: 'swordfish' bindDn: nil bindPassword: nil


 Example 3: Search mode with authenticated bind

 System validatePasswordUsingLdapServers: (Array with: 'ldaps://myldap.mydomain.com')
        baseDn: 'ou=Users,dc=mycompany,dc=com' filterDn: '(uid=%s)'
        userId: 'MyUserId' password: 'swordfish' bindDn: 'LdapBindUser'
        bindPassword: 'LdapBindPassword'
"

<primitive: 859>
aUriArray == nil
  ifFalse:[ aUriArray _validateClass: Array ] .
aUriArray do:[:e| e _validateClass: String ].
aBaseDn _validateClass: String .
aFilterDn == nil
  ifFalse:[ aFilterDn _validateClass: String ] .
aUserId _validateClass: String .
pw _validateClass: String .

bindDn == nil
   ifTrue:[ bindPw _validateClass: UndefinedObject ]
  ifFalse:[ bindDn _validateClass: String ] .
bindPw == nil
   ifTrue:[ bindDn _validateClass: UndefinedObject ]
  ifFalse:[ bindPw _validateClass: String ] .

^ self _primitiveFailed:
       #validatePasswordUsingLdapServers:baseDn:filterDn:userId:password:bindDn:bindPassword
       args: { aUriArray . aBaseDn . aFilterDn . aUserId . pw . bindDn . bindPw }
%

category: 'Garbage Collection Management'
classmethod: System
voteState

"Returns an integer, the voteState of the Stone garbage
 collection voting state machine.
 The states are  0 IDLE, 1 HIDING_SYMBOLS, 2 VOTING,
	 3 DONE_VOTING, 4 IN_PDWSU_SWEEP, 5 PDWSU_DONE"

 ^ self _zeroArgPrim: 17
%

category: 'Garbage Collection Management'
classmethod: System
voteStateString

"Returns a String which is the name of the current value of
 the voteState of the Stone garbage collection voting state
 machine. "

 ^ self _zeroArgPrim: 19
%

category: 'Garbage Collection Management'
classmethod: System
waitForAllGcGemsToStartForUpToSeconds: anInt
"Wait for all GC gems on the system to start for up to anInt
 seconds.  Returns true if all GC gems started, false if they
 did not within the specified amount of time."

|count|

count := 0.
[self hasMissingGcGems] whileTrue:[
  self sleep: 1.
  count := count + 1.
  (count > anInt)
   ifTrue:[^false]. "timeout waiting for GC gems to start"
].
^true "success!"
%

category: 'Setting Locks'
classmethod: System
waitForApplicationWriteLock: lockObject queue: lockIdx autoRelease: aBoolean

"Waits for a write lock using the specified object and
 the Application lock queue specified by lockIdx .

 lockIdx must be a SmallInteger,  >= 1  and  <= 10

 Throws error LOCK_ERR_ALL_SYMBOLS (2331) if you attempt to lock AllSymbols.
 Throws error RT_ERR_OBJ_MUST_BE_COMMITTED (2405) if you attempt to lock a temporary object.

 returns a SmallInteger, one of
   1  granted
   2071  undefined lock (lockIdx out of range)
   2074  dirty (object written by other session since start of this transaction)
   2075  lockDenied (lock object is a special object)
   2418  deadlock
   2419  timeout  (per STN_OBJ_LOCK_TIMEOUT config parameter)

 For a given value of lockIdx, all calls must use a persistent lockObject
 which is identical during the life of a stone process .

 If result is 1 or 2074 and aBoolean == true,  the lockObject is added to both
 the CommitReleaseLocksSet and CommitOrAbortReleaseLocksSet
 hidden sets of this session.

 The result 2418 (deadlock)   is returned if
   the requesting session already holds a readLock on lockObject
 or if
  the session would have to wait for the lock on lockObject, but doing
  so would create a cycle in the graph of locks being waited for, i.e:

    the session already holds a read or write lock on some object X
        waitForApplicationWriteLock: X ...
    and this session's attempt at
       waitForApplicationWriteLock: lockObject
    would have to wait to get the lock on lockObject
"

lockIdx < 1 ifTrue:[ ^ 2071 "lockIdx out of range"].
^ self _lock: lockObject kind: lockIdx + 4 autoRelease: aBoolean
%

category: 'Garbage Collection Management'
classmethod: System
waitForVoteStateIdle

"Returns when the vote state is idle."

self waitForVoteStateIdleSecs: -1. "wait forever for vote state idle"
^ self
%

category: 'Garbage Collection Management'
classmethod: System
waitForVoteStateIdleSecs: waitTime
| timeLimit sleepCount voteState voteCount currSessVoting prevSessVoting |
  timeLimit := waitTime.
  timeLimit = -1 ifTrue: [ timeLimit := SmallInteger maximumValue ].

  self needsCommit
    ifTrue: [ self _error: #rtErrAbortWouldLoseData ] .

  self abortTransaction .

  voteState := System voteState.

  "Admin gem must be present if voteState is not 0"
  ((voteState ~~ 0) and:[System adminGemSessionId == 0])
    ifTrue:[self _error: #rtErrReclaimAllMissingGcGem args:
             {'missing admin or reclaim gem needed to get vote state idle'}].
  sleepCount := 0.
  [ voteState := System voteState .
    voteState ~~ 0
  ] whileTrue:[
    voteState == 2 ifTrue: [  "voting"
       currSessVoting := System stoneCacheStatisticWithName: 'WaitingForSessionToVote'.
       currSessVoting == prevSessVoting
       ifTrue: [
         self abortTransaction .
         sleepCount - voteCount > 20
	 ifTrue: [ GsFile gciLogServer:'WARNING -- session ', currSessVoting asString ,
		                     ' taking too long to vote']]
       ifFalse: [ prevSessVoting := currSessVoting. voteCount := sleepCount].
     ].
     System sleep: 1. System abortTransaction.
     sleepCount := sleepCount + 1.
     (sleepCount > timeLimit) ifTrue: [
       UserDefinedError new _number: 2318;
         reason: #fail ; details: 'Timeout expired while waiting for voteState idle' ; signal.
     ]
  ].
^ timeLimit - sleepCount
%

category: 'Private Cache Warming'
classmethod: System
warmRemoteCache: aString

"Used to warm an X509 remote cache.
Invoked by a topaz -r forked from a netldi started with -E and -S ,
see $GEMSTONE/sys/cachewarmertopaz.ini .
aString is the value of either GEM_CACHE_WARMER_ARGS or GEM_CACHE_WARMER_MID_CACHE_ARGS
from the config file specified by -E arg to startnetldi.

Reads pages into the local cache
"
<primitive: 1103>
aString _validateClass: String .
self _primitiveFailed: #warmRemoteCache: args: { aString }
%

category: 'Private Cache Warming'
classmethod: System
warmX509LeafCache
  "Executes in a topaz -r forked by a netldi -S -E with NETLDI_START_MIDCACHE=false."
  | cfg dict midHost cfgStr val |
  cfg := self clientEnvironmentVariable:'GS_NETLDI_WARMER_ARGS' .
  cfg ifNotNil:[
    GsFile gciLogClient:'Using GS_NETLDI_WARMER_ARGS=''', cfg asString, '''' .
    dict := self parseWarmerConfig: cfg
  ] ifNil:[ Error signal:'missing GS_NETLDI_WARMER_ARGS environment variable' ].
  midHost := dict at: #midHost otherwise: nil .
  midHost ifNotNil:[ | status |
    (GsSocket hostIsLocalhost: midHost) ifTrue:[
      GsFile gciLogClient: '''-M ', midHost, ''' specifies localhost, warming direct from stone cache'.
    ] ifFalse:[
      status := self midLevelCacheConnect: midHost .
      status ~~ true ifTrue:[
        ^ 'Failed to connect to mid level cache on ', midHost, '; ',  status asString.
      ].
      GsFile gciLogClient: 'Successful connection to mid cache on ' , midHost .
    ]
  ].
  cfgStr := String new .
  (dict at: #d otherwise: nil) ifNotNil:[ cfgStr add: '-d ' ].
  (val := dict at: #n otherwise: nil) ifNotNil:[
    val > 2 ifTrue:[ val := 2 ].
    cfgStr add: '-n ' , val asString
  ] ifNil:[ cfgStr add: '-n 2' ].
  GsFile gciLogClient:'Detailed warming status is in ' , System gemLogFileName .
  ^ self warmRemoteCache: cfgStr .
%

category: 'Setting Locks'
classmethod: System
writeLock: anObject

"Analogous to System | readLock:.  However, this method requests and
 grants write locks."

| result |

result := self _lock: anObject kind: 3 autoRelease: false .
(result == 1) ifTrue:[ ^ self ].
self _lockError: result obj: anObject details: 'write lock'
%

category: 'Setting Locks'
classmethod: System
writeLock: anObject ifDenied: denyBlock ifChanged: changeBlock

"Analogous to System | readLock:ifDenied:ifChanged:.  However,
 this method requests and grants write locks."

| result |
result := self _lock: anObject kind: 3 autoRelease: false .
(result == 1) ifTrue:[ ^ self ].
^ self _lockEvaluateErr: result obj: anObject denied: denyBlock changed: changeBlock
%

category: 'Setting Locks'
classmethod: System
writeLockAll: aCollection

"Analogous to System | readLockAll:.  However, this method requests and
 grants write locks."

| result |
result := self _lockAll: aCollection kind: 3 .
(result == System)
  ifTrue: [^self]
  ifFalse: " else we generate the error lock incomplete "
           [^ (result at: 1) _error: #lockErrIncomplete
                             args: { (result at:2) . (result at:3) }].
%

category: 'Setting Locks'
classmethod: System
writeLockAll: aCollection ifIncomplete: incompleteBlock

"Analogous to System | readLockAll:ifIncomplete:.  However, this method
 requests and grants write locks."

| result |
result := self _lockAll: aCollection kind: 3 .
(result == System)
  ifTrue: [^self]
  ifFalse: " else we execute the incomplete block with 3 arguments "
          [^incompleteBlock value: (result at: 1)
                            value: (result at:2)
                            value: (result at:3)].
%

category: 'Private'
classmethod: System
_abortNestedTransaction
  (self __sessionStateAt: 4) ifNotNil:[:rlArray |  | lev |
     (rlArray atOrNil: (lev := self transactionLevel)) ifNotNil:[
       "deref redo log entries for level being aborted"
       rlArray at: lev put: nil .
     ].
  ].
  self clearRcValueCache .
  (self _zeroArgPrim: 148 "_abortNested") ifFalse:[
    InternalError signal:'inconsistent transaction nesting'
  ].
%

category: 'System Control'
classmethod: System
_addAllToStoneLog: aString

"Appends text to the Stone's informational log file.  First, this method writes
 a banner that identifies the session from which aString came.  It then appends
 aString itself.  The argument must be a Utf8.

 Generates an error if aString is larger than approximately 16000 bytes."

<primitive: 463>
aString _validateClasses: { Utf8 }.
self _primitiveFailed: #addAllToStoneLog: args: { aString }
%

category: 'Reduced Conflict Support'
classmethod: System
_addEntireObjectToRcReadSet: anObject

"Adds anObject to the reduced-conflict read set (RcReadSet).
 If the object is a large object, all nodes of the object are added.

 See also documentation for _addToRcReadSet:includingAllNodes: "

self _addToRcReadSet: anObject includingAllNodes: true.
%

category: 'Reduced Conflict Support'
classmethod: System
_addRootObjectToRcReadSet: anObject

"Adds anObject to the reduced-conflict read set (RcReadSet).
 Only the root object is added (even if the object is a large object).

 See also documentation for _addToRcReadSet:includingAllNodes: "

self _addToRcReadSet: anObject includingAllNodes: false.
%

category: 'Reduced Conflict Support'
classmethod: System
_addToRcReadSet: anObject includingAllNodes: aBoolean

"Adds anObject to the reduced-conflict read set (RcReadSet).
 All objects that are part of composite used to represent a reduced
 conflict object should be added to this set to avoid unnecessary
 conflict handling.
 If aBoolean is true, the entire object (including all interior nodes
 of a large object) are added.  If aBoolean is false, only the root object
 is added.

 The RcReadSet is used as follows during a commit attempt .
   readWriteConflicts =   (strongReadSet * writeSetUnion) - RcReadSet .
   writeWriteConflicts = (writeSet * writeSetUnion) .
 If there are readWriteConflicts, the commit always fails.
 If writeWriteConflicts is not empty,
 but (writeWriteConflicts - RcReadSet ) is empty ,
 then RC replay is executed and the commit is reattempted once .
 "

<primitive: 116>
aBoolean _validateClass: Boolean.
self _primitiveFailed: #_addToRcReadSet:includingAllNodes:
     args: { anObject . aBoolean } .
self _uncontinuableError
%

category: 'Performance Monitoring'
classmethod: System
_approxOopHighWaterMark

"Returns an Integer that gives the approximate highest oop allocated in GemStone."

^self _zeroArgPrim: 23
%

category: 'Performance Monitoring'
classmethod: System
_approxOopNumberHighWaterMark

"Returns an Integer that gives the approximate highest oop number allocated in GemStone."

^self _zeroArgPrim: 36
%

category: 'Runtime Configuration Access'
classmethod: System
_atConfigId: rtConfigId put: aValue

"Change the specified Stone runtime configuration parameter.  The current user
 must be SystemUser to set any configuration parameter except the parameters

 Changing a runtime parameter does not alter the configuration file.
 The new value will show up in the result of configuration report methods,
 but will not be used when Stone next starts up."

<primitive: 355>
rtConfigId _validateClass: SmallInteger .
aValue _validateKindOfClasses: { SmallInteger . Array } .
(self _configParameterName: rtConfigId kind: $C ) == nil
   ifTrue:[ rtConfigId _error: #rtErrArgOutOfRange ].
rtConfigId >= 0 ifTrue:[
  (self _configParameterName: rtConfigId kind: $C ) _error: #rtErrConfigReadOnly
  ].

self _primitiveFailed: #_atConfigId:put: args: { rtConfigId . aValue } .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_bypassReadAuth

"For use by indexing and query subsystem only.

 Subsequent object read-faults will bypass
 read-authorization checks to allow indexed queries and index maintenance
 to execute.  This allows indexing code to execute comparisons on keys
 in btree nodes without regard to read-authorization of the keys.
 Query results will never contain objects for which the current user
 is not read-authorized.

 Has no effect if current user is SystemUser.

 Indexing code should use this method in combination with an ensure: block
 which contains   _exitBypassReadAuth

 The bypass is shutoff automatically (equivalent to _exitBypassReadAuth)
 upon return from a userAction, from a recurFromOm, , upon return to the GCI,
 or upon generation of an error which would unwind the VM C stack.
"

<protected primitive: 595>
self _primitiveFailed: #_bypassReadAuth
%

category: 'Shared Cache Management'
classmethod: System
_cacheAddresses: midLevelBool includeMyCache: includeMyCacheBool
| idx result batch |
idx := 0 .
result := { }  .
[ idx >= 0 ] whileTrue:[ | batSize |
  batch := self _remoteCachesListStartingAt: idx midLevelOnly: midLevelBool
					    includeMyCache: includeMyCacheBool .
  batSize := batch size .
  batSize > 0 ifTrue:[
    1 to: batSize - 1 do:[:n | result add: ((batch at:n) at: 8) ].
    idx := batch at: batSize .
  ] ifFalse:[
    idx := -1 . "done"
  ].
].
^ result
%

category: 'Private'
classmethod: System
_cacheName: aString

"Sets the name the current session is known by in the shared cache
 to 'aString'. Does nothing if the current session is not attached
 to a shared cache. Raises an error if 'aString' is too long.
 Returns self."

<primitive: 509>
self _primitiveFailed: #_cacheName: args: { aString }
%

category: 'Private'
classmethod: System
_cacheStatWithName: aString opCode: anInt

"Return the value of the cache stat with the given name, which must match an
 element of the Array returned by the #cacheStatisticsDescription method.

 opCode 1 means the statistic is for the primary shared page cache monitor.
 opCode 2 means the statistic is for the stone.
 opCode 8 means the statistic is for the current session.

 This method may be used to access primary shared page cache monitor and stone
 statistics from hosts other than that on which the stone is running.

 Returns nil if a statistic matching aString was not found for the process kind
 specified by the opCode."

<primitive: 430>
aString _validateClass: String .
anInt _validateClass: SmallInteger .
^ self _primitiveFailed: #_cacheStatWithName:opCode:
       args: { aString . anInt }
%

category: 'Private'
classmethod: System
_checkCacheAndClear: aBool

"Scans the cache and if aBool is false, returns an array of statistics
 for the current contents of the cache:
    total number of pages in the cache
    number of free pages
    number of pinned pages
    number of locked pages
    number of dirty pages
    number of clean pages
    number of pages written (only non zero if arg is true)
    number of pages preemted (only non zero if arg is true)

If the aBool argument is true, as it scans it also tries to write and
preempt pages from the cache and fills in the last two array elements with
the number of operations performed.

Requires the SystemControl privilege if aBool is true.

No guarantees are made that the values will be consistent from run to run
since the contents of the cache may change depending on other commits,
aborts and garbage collection (reclaims) that might occur. "

<primitive: 1001>
aBool _validateClass: Boolean .
^ self _primitiveFailed: #_checkCacheAndClear:
       args: { aBool }
%

category: 'Private'
classmethod: System
_cleanupZombieSession: sessionId

"Forces the stone to finish logout processing for the given session ID.
 Takes no action if the given session has not begun the logout process or
 if the session no longer exists or if the session is the current session.
 When logout processing is completed, references to a commit record by
 that session are released and resources consumed by the session are recycled.

 To execute this method, you must have SessionAccess and SystemControl
 privileges.

 WARNING: This method bypasses certain internal mechanisms which
          prevent the immediate termination of active sessions,
          and therefore should only be used if the session
          could not be stopped by any other mechanisms.

 Returns true if the action was successful and the stone acted on the
 specified session.  A message will be written to the stone log in this
 case.  Returns false if no action was taken.  No message is added to the
 stone log when false is returned."

<primitive: 774>

sessionId _validateClass: SmallInteger .
((sessionId < 1) or:[sessionId > 10000])
  ifTrue:[ sessionId _error: #rtErrArgOutOfRange args:{ 1 . 10000 } ].
self _primitiveFailed: #_cleanupZombieSession: args: { sessionId }
%

category: 'Private'
classmethod: System
_clearDeadObjects
  "If admin, reclaim, or symbol sessions are running, does nothing and returns false.
   Signals an error if the vote state is 'in wsUnion sweep'.
   Otherwise resets Vote state to idle and
   clears the deadObjs, possibleDeadObjs, possibleDeadWsUnion, and possibleDeadWithSymbols
   bitmaps in stone, and returns true ."

^ self _zeroArgPrim: 39
%

category: 'Clustering'
classmethod: System
_clusterImage

"For all classes in Globals, clusters behavior and descriptions "

| classes nonClasses behaviorBucket descriptionBucket otherBucket |
behaviorBucket := AllClusterBuckets at: 4 .
descriptionBucket := AllClusterBuckets at: 5 .
otherBucket := AllClusterBuckets at: 6  .

classes := Globals select:[:i| i isKindOf: Behavior ] .
nonClasses := Globals reject:[:i| i isKindOf: Behavior ] .
nonClasses removeAssociation: (Globals associationAt: #Globals ).

self clusterBucket: behaviorBucket.
classes associationsDo: [ :each |
    each value _alias . "for debugging"
    each value clusterBehavior.
    ].

self clusterBucket: descriptionBucket.
classes associationsDo: [ :each |
    each value clusterDescription.
    ] .
^ true
%

category: 'Clustering'
classmethod: System
_clusterUserClassesVisibleFromUserProfile: aUserProfile

"Cluster all classes contained in symbol list dictionaries for the
 given user profile.  Kernel classes in Globals are not clustered;
 the _clusterImage method should be used to cluster kernel classes."

|behaviorBucket descriptionBucket symbolList|

symbolList := aUserProfile symbolList copy.
symbolList remove: Globals.
symbolList removeDictionaryNamed: #Published ifAbsent: [ ].

"Identify cluster buckets for class and method objects"
behaviorBucket := AllClusterBuckets at: 4.
descriptionBucket := AllClusterBuckets at: 5.
self clusterBucket: behaviorBucket.
symbolList do:[:eachSymbolDictionary|
  eachSymbolDictionary rebuildIfNeeded.
  eachSymbolDictionary cluster.
  eachSymbolDictionary associationsDo:[ :eachAssociation |
    eachAssociation cluster.
    (eachAssociation value isKindOf: Behavior)
      ifTrue:[eachAssociation value clusterBehavior.].
  ].
].

self clusterBucket: descriptionBucket.
symbolList do:[:eachSymbolDictionary|
  eachSymbolDictionary associationsDo:[ :eachAssociation | |val|
  val := eachAssociation value.
  (val isKindOf: Behavior)
    ifTrue:[val clusterDescription.].
  ].
].
%

category: 'Private'
classmethod: System
_comClearOpcodesEmitted

^ self _zeroArgPrim: 92
%

category: 'Reduced Conflict Support'
classmethod: System
_commit: commitMode

"commitMode is a SmallInteger:

 0 for normal,
 1 for release locks,
 2 for checkpoint and release locks.

 Returns true if commit was read-only or succeeded ,
 false if there was a failure.  See also _localCommit: .  "

| coordinator |

^ (coordinator := self _commitCoordinator) == nil
  ifTrue: [ self _localCommit: commitMode ]
  ifFalse: [ coordinator commit: commitMode ].
%

category: 'Private'
classmethod: System
_commitCoordinator

"Returns the commit coordinator stored in temporary session state."

^ self __sessionStateAt: 14
%

category: 'Private'
classmethod: System
_commitCoordinator: aCoordinator

"Sets the commit coordinator stored in temporary session state."

^ self __sessionStateAt: 14 put: aCoordinator
%

category: 'Private'
classmethod: System
_commitNestedTransaction
  | lev log outerLog rlArray |
  lev := self transactionLevel.
  (self _zeroArgPrim: 147 "commitNestedTransaction" ) ifFalse:[
     InternalError signal:'inconsistent transaction nesting'
  ].
  (rlArray := self __sessionStateAt: 4) ifNil:[
    ^ true "no entries at any level"
  ].
  (log := rlArray atOrNil: lev) ifNil:[
     ^ true "no Rc operations in level we are committing"
  ].
  (outerLog := rlArray atOrNil: lev - 1) ifNil:[
    "no entries yet in parent level, just move nested entries to parent level."
    rlArray at: lev - 1 put: log .
    rlArray size: lev - 1.
    ^ true.
  ].
  outerLog _commitNested: log .  "merge redo log into parent level redo log"
  rlArray at: lev put: nil .
%

category: 'Private'
classmethod: System
_commitPrintingDiagnostics

"Commits the current transaction. Answers true if successful,
 otherwise false. If false, it also prints the OOPs for
 each kind of conflict, ordered by kind. This is used in the
 upgrade scripts."

self commitTransaction
  ifTrue: [^true]
  ifFalse: [
    | conflicts |
    conflicts := self transactionConflicts.
    (conflicts at: #commitResult) == #success ifFalse: [
      GsFile gciLogClient: (self conflictReportString: conflicts) .
    ].
    ^false
  ]
%

category: 'Private'
classmethod: System
_commitResult

"Private.  Returns a SmallInteger which represents the result of the last
 commitTransaction or continueTransaction operation.  If abortTransaction was
 issued since the last commit or continue then it returns SmallInteger 0.

 See   _primitiveCommit:  for documentation of the return values."

^ self _zeroArgPrim: 30
%

category: 'Private'
classmethod: System
_committedDataPages

"Returns an Array of the committed data pages referenced from the session's
 view of GemStone.  The contents of this Array are invalid after the next
 commit or abort."

^ self _zeroArgCateg2Prim: 10
%

category: 'Version Management'
classmethod: System
_compareGlobalsWith: passiveSourceFileName
writeReportTo: reportFileName
generateFileinTo: fileinFileName
"returns a string, a report on Smalltalk method differences between this
 version and a previous version.

 If reportFileName is nil, no report file is written. If reportFile is not
 nil, the report string will be written to a file.

 If fileinFileName is nil, no filein file will be generated. If fileinFileName
 is not nil, a topaz filein script will be generated.

 See also System(C)>>_generatePassiveSourceFileTo:
"
| prevClasses prevClassesFile report reportFile lf addedMeths changedMeths
  deletedMeths addedClasses deletedClasses fileinFile
  theCls prevDict clsMethStr |

lf := Character lf.
report := String new.
addedMeths := { } .
changedMeths := { } .
deletedMeths := { } .
addedClasses := { } .
deletedClasses := { } .

fileinFileName ifNotNil: [
  fileinFile := GsFile openWriteOnServer: fileinFileName.
  (fileinFile == nil) ifTrue: [
    ^'Can''t open ', fileinFileName, ' for writing.'
  ].
].

prevClassesFile := GsFile openReadOnServer: passiveSourceFileName.
prevClassesFile ifNil: [
  ^'Can''t open ', passiveSourceFileName, ' for reading.'
].

"read in the prevClasses from passivated object file"
prevClasses := (PassiveObject newOnStream: prevClassesFile) activate.
prevClasses ifNil: [
  ^'Failed to activate source dictionaries from ', passiveSourceFileName, '.'
].

"iterate classes in Globals and compare method source strings"
Globals do: [ :ea | | cls |
  (ea isBehavior) ifTrue: [ | className |
    cls := ea thisClass .
    className := cls name .
    (prevClasses includesKey: className) ifFalse: [
      addedClasses add: className.
      fileinFile ifNotNil: [
        cls fileOutClassOn: fileinFile.
      ].
    ] ifTrue: [ | methsBlk |
      methsBlk := [ :mDict |
        mDict keysAndValuesDo: [ :selector :aMeth | | envId aName |
          envId := aMeth environmentId .
          aName := envId == 0 ifTrue:[ selector ] ifFalse:[
             ':env_', envId asString , ':', selector asString
          ].
          " check to see if same method exists in prevClasses"
          (prevDict includesKey: aName) ifFalse: [
            addedMeths add: className, '>>', clsMethStr, aName .
            fileinFile ifNotNil: [
              fileinFile nextPutAll: (theCls _fileOutMethod: selector environmentId: envId ).
            ].
          ] ifTrue: [
            " method exists in both versions, do a diff."
            (aMeth sourceString = (prevDict at: aName)) ifFalse: [
              changedMeths add: className, '>>', clsMethStr, aName.
              fileinFile ifNotNil: [
                fileinFile nextPutAll: (theCls fileOutMethod: selector environmentId: envId).
              ].
            ].
          ].
        ].
      ].

      clsMethStr := '' .
      prevDict := (prevClasses at: className) at: 1 .
      theCls := ea .
      theCls persistentMethodDictsDo: methsBlk .

      clsMethStr := '(C)' .
      prevDict := (prevClasses at: className) at: 2 .
      theCls := ea class .
      theCls persistentMethodDictsDo: methsBlk .
    ]
  ]
]. "Globals do:"

"Now check to see if there are any deleted methods or classes"
prevClasses keysAndValuesDo: [ :clsNam :v |
  ((Globals includesKey: clsNam) and: [(Globals at: clsNam) isBehavior]) ifFalse: [
    deletedClasses add: clsNam.
  ] ifTrue: [ | meths envPrefix prefixSize |
    meths := v at: 1 .
    clsMethStr := '' .
    theCls := Globals at: clsNam .
    envPrefix := ':env_' .
    prefixSize := envPrefix size .
    2 timesRepeat:[
      meths keysDo:[ :aName | | envId selectr |
        envId := 0 .
        selectr := aName .
        (aName at: 1 equals: envPrefix) ifTrue:[ | endIdx |
          endIdx := aName indexOf: $: startingAt: 2 .
          envId := Integer fromString:( aName copyFrom: prefixSize + 1 to: endIdx - 1 ).
          selectr := aName copyFrom: endIdx + 1 to: aName size .
        ].
        (theCls includesSelector: selectr environmentId: envId) ifFalse:[
          deletedMeths add: clsNam, '>>', clsMethStr, aName.
          fileinFile ifNotNil: [
            fileinFile nextPutAll: 'doit', lf,
	      clsNam , ' removeSelector: #', selectr, ' environmentId:', envId asString ,
		lf, $%, lf.
          ].
        ].
      ].
      meths := v at: 2 .
      clsMethStr := '(C)' .
      theCls := theCls class .
    ].
  ].
].
fileinFile ifNotNil:[ fileinFile close ].

"put together the report"
report := '====', lf, 'Image change report', lf, '====', lf.
(addedClasses size ~~ 0) ifTrue: [
  report := report, '----', lf, 'Added Classes:', lf, '----', lf.
  (addedClasses sortAscending) do: [ :ea | report := report, ea, lf ].
].
(deletedClasses size ~~ 0) ifTrue: [
  report := report, '----', lf, 'Deleted Classes:', lf, '----', lf.
  (deletedClasses sortAscending) do: [ :ea | report := report, ea, lf ].
].
(addedMeths size ~~ 0) ifTrue: [
  report := report, '----', lf, 'Added Methods:', lf, '----', lf.
  (addedMeths sortAscending) do: [ :ea | report := report, ea, lf ].
].
(deletedMeths size ~~ 0) ifTrue: [
  report := report, '----', lf, 'Deleted Methods', lf, '----', lf.
  (deletedMeths sortAscending) do: [ :ea | report := report, ea, lf ].
].
(changedMeths size ~~ 0) ifTrue: [
  report := report, '----', lf, 'Changed Methods', lf, '----', lf.
  (changedMeths sortAscending) do: [ :ea | report := report, ea, lf ].
].

reportFileName ifNotNil:[
  reportFile := GsFile openWriteOnServer: reportFileName.
  reportFile ifNotNil:[
    reportFile nextPutAll: report; close.
  ].
].
^report
%

category: 'Private'
classmethod: System
_comPrintOpcodesEmitted

^ self _zeroArgPrim: 87
%

category: 'Private'
classmethod: System
_comPrintOpcodesNotEmitted

^ self _zeroArgPrim: 88
%

category: 'Configuration File Access'
classmethod: System
_configConstantsDict

"Returns a Dictionary of internal configuration constants that
 are fixed at compilation of the gem and stone executables. The dictionary
 keys are Strings.  Its values are SmallInteger configuration constants."


| anArray result |
anArray := self _zeroArgCateg2Prim: 8 .
result := StringKeyValueDictionary new .
1 to: anArray size do:[ :j | | anAssoc |
  anAssoc := anArray at: j .
  result add: anAssoc .
  ].
^ result
%

category: 'Configuration File Access'
classmethod: System
_configFileParameterDict

"Returns a Dictionary of names for configuration file parameters.  The
 dictionary keys are Symbols.  Its values are SmallInteger configuration IDs."

| result cfgId cfgName |
result := SymbolKeyValueDictionary new .
cfgId := 0 .
[
  cfgName := self _configParameterName: cfgId kind: $C .
  cfgName == nil ifFalse:[
    result at: cfgName put: cfgId .
    cfgId := cfgId + 1 .
    ] .
  cfgName == nil
  ] untilTrue .

cfgId := -1 .
[
  cfgName := self _configParameterName: cfgId kind: $C .
  cfgName == nil ifFalse:[
    result at: cfgName put: cfgId .
    cfgId := cfgId - 1 .
    ] .
  cfgName == nil
  ] untilTrue .

^ result
%

category: 'Configuration File Access'
classmethod: System
_configParameterName: anInt kind: aChar

"Returns a String, which is the name of the configuration parameter whose
 internal identifier is anInt.  Returns nil if anInt is out of range.

 aChar must be one of
   $C - anInt is identifier of a configuration parameter
   $V - anInt is identifier of a version information item
"

<primitive: 342>
anInt _validateClass: SmallInteger .
^ self _primitiveFailed: #_configParameterName:kind: args: { anInt . aChar }
%

category: 'Configuration File Access'
classmethod: System
_configurationAt: cfgId isStone: aBool kind: aKind

"Returns the configuration value specified. If aBool is true, get the
 configuration value from the Stone process, otherwise from the current
 session.

 cfgId must be a SmallInteger; if >= 0, returns the value of the specified
 configuration parameter;  if < 0, returns the value of the specified
 runtime control parameter.

 aKind must be either $C (for configuration info) or $V (for version info)"

<primitive: 338>
aBool _validateClass: Boolean .
cfgId _validateClass: SmallInteger .
aKind == $C ifTrue:[
  ConfigurationParameterDict keyAtValue: cfgId ifAbsent:[
     cfgId _error: #rtErrArgOutOfRange
     ].
  ]
  ifFalse:[
  VersionParameterDict keyAtValue: cfgId ifAbsent:[
     cfgId _error: #rtErrArgOutOfRange
     ].
  ].
self _primitiveFailed: #_configurationAt:isStone:kind:
     args: { cfgId . aBool . aKind } .
self _uncontinuableError
%

category: 'Configuration File Access'
classmethod: System
_configurationReport: isStone

"Private."

| result |
result := SymbolDictionary new .
ConfigurationParameterDict keysAndValuesDo:[ :aName :anId |
  (self _configurationAt: anId isStone: isStone kind: $C) ifNotNil:[:val |
    result at: aName put: val
    ].
  ].
^ result
%

category: 'Private'
classmethod: System
_cpuClockTicksPerSecond

"Calls the UNIX function sysconf(_SC_CLK_TCK) to determine the number
 of clock ticks in a second.  Returns zero if the call is not supported."

^self _zeroArgPrim: 57
%

category: 'Private'
classmethod: System
_currentSessionProfiles

"Returns an Array containing pairs of sessionId, UserProfile
 for all sessions currently logged in.
 Some sessionIds may have a userProfile of nil, including those which
 are part way through login.
 logsender processes have userProfile==1 ,
 logreceiver processes have userProfile==2.

 This method requires SessionAccess privilege if there is more
 than one session logged in."

| sessId result batch |
GsSession currentSession isSolo ifTrue:[
  ^ { 1 . System myUserProfile }
].
sessId := 1 .
result := { }  .
[ sessId > 0 ] whileTrue:[
  batch := self _sessionProfilesStartingAt: sessId .
  batch size > 0 ifTrue:[
    result addAll: batch .
    sessId := (batch at: (batch size - 1)) + 1 .
  ] ifFalse:[
    sessId := -1 . "done"
  ].
].
^ result
%

category: 'Session Control'
classmethod: System
_currentSessionsIncludingLogout

"Returns an Array of SmallIntegers corresponding to all of the sessions
 currently running on the GemStone system.  Includes sessions in before-login
 or after-logout states.
 Gets the result from the stone process."

^self _zeroArgPrim: 83
%

category: 'Session Control'
classmethod: System
_currentUserSessionCountIncludingLogout
"Return a SmallInteger which is the number of user sessions present in the system.
 Garbage collection sessions, the Symbol Gem, and the Page Manager Gem are not
 included in the count.
 Includes sessions in before-login or after logout state.
 Gets the result from the stone process."

^self _zeroArgPrim: 70
%

category: 'Private'
classmethod: System
_deadNotReclaimed

"Private, deprecated.  Returns a GsBitmap that contains the current contents of the
 global set of objects which have been identified as dead but have not been reclaimed."

  self deprecated: 'System class>>_deadNotReclaimed: deprecated v3.6. Use GsBitmap class>>deadNotReclaimed'.

^ GsBitmap _deadNotReclaimed
%

category: 'Private'
classmethod: System
_debugInfoForPage: anInt

"If anInt is the page ID of a data page, return an Array of 12 elements:

  1 - Time page was written in seconds since January 1, 1970 (SmallInteger)
  2 - Session ID that wrote the page. (SmallInteger)
  3 - Number of valid objects in the page. (SmallInteger)
  4 - Number of invalid objects in the page. (SmallInteger)
  5 - Array of valid object IDs (Array of SmallIntegers)
  6 - Array of classes of valid objects.
  7 - Array of invalid object IDs (Array of SmallIntegers)
  8 - Array of classes of invalid objects.
  9 - Array of correct page IDs for invalid objects.
 10 - Array of correct classes for invalid objects.
 11 - Array of session IDs that wrote the correct pages in element 9.
 12 - Array of timestamps that the correct pages in element 9 were written.

 This method is intended to be used only by GemStone staff to aid in the debugging of
 repository problems.

 This method may only be executed by SystemUser.

 Returns nil if the page is not a data page."

<primitive: 870>

anInt _validateClass: SmallInteger .
self _primitiveFailed: #_debugInfoForPage: args: { anInt }
%

category: 'Deprecated'
classmethod: System
_deleteServerFile: aFileSpec

<primitive: 346>
aFileSpec _validateClass: String.
self _primitiveFailed: #_deleteServerFile: args: { aFileSpec }
%

category: 'Cache Statistics - Deprecated'
classmethod: System
_deprecatedCacheStatisticsDescription

<primitive: 580>
^ self _primitiveFailed: #_deprecatedCacheStatisticsDescription
%

category: 'Session Control'
classmethod: System
_descriptionOfSessionSerialNum: serialNum sessionId: aSessionId

"Returns an Array describing the session identified by the arguments.
 The session is looked up by serial number if serialNum is > 0.  Otherwise
 it is looked up by sessionId.

 See System (C) | descriptionOfSession: for documentation on the contents of the
 result Array.
 Requires SessionAccess privilege if the session being looked up is not
 the current session."

<primitive: 334>
aSessionId _validateClass: SmallInteger.
^ self _primitiveFailed: #_descriptionOfSessionSerialNum:sessionId:
       args: { serialNum . aSessionId }
%

category: 'Private'
classmethod: System
_detailedConflicts
  "Returns nil or an Array information about which commits by other sessions
   caused commit conflicts.  They are ordered by youngest commit first.

    { attemptedCommitTimeMs .
      { sessionIdInteger . commitTimeMsInteger . aUserProfile . wwArray . depArray }
      ...
    }
    wwArray is nil or an Array of objects with write-write conflicts.
    depArray is nil or an Array of objects with dependency conflicts.

    wwArray and depArray are limited to the first 100 objects

    The time values are Integers, milliseconds since 1 Jan 1970 UTC .
  "

^  self __sessionStateAt: 24
%

category: 'Private'
classmethod: System
_dirtyListOp: opcode id: listId

"execute an operation on one of the object manager dirty lists
 opcode 1  not supported (was rollBackDirtyList in v3.0 )
 opcode 2  enumerateDirtyList.

   listId may be -1 (current),
    or a specific dirty list where listId is  (System transactionLevel - 1)

 Not-tranlogged objects are no longer segregated in a separate list
"

<primitive: 276>
self _primitiveFailed: #_dirtyListOp:id: args: { opcode . listId } .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_disableAddToDirtySets: aBoolean

"Disable/reenable additions to the GCI export and tracked dirty lists .
 Returns previous state of the flag"

<primitive: 569>
aBoolean _validateClass: Boolean .
self _primitiveFailed: #_disableAddToDirtySets: args: { aBoolean }
%

category: 'Private'
classmethod: System
_disableCommitsWithReason: aString untilAbort: aBool

"Private - this method should not be called directly.  Call the #disableCommitsWithReason:
 method instead of this one."

<primitive: 701>
aBool _validateClass: Boolean .
aString == nil
  ifFalse:[ aString _validateClass: String].
self _primitiveFailed: #_disableCommitsWithReason:untilAbort:
     args: { aString . aBool }
%

category: 'Modifying Classes'
classmethod: System
_disallowCommitClassModFailure

"Disallows subsequent attempts due to failure during class modification."
^ self _zeroArgPrim: 37
%

category: 'Reduced Conflict Support'
classmethod: System
_disallowSubsequentCommits: traceInt
  "GsFile gciLogServer:'_disallowSubsequentCommits: ' , traceInt asString .""uncomment to debug"
  ^ self _zeroArgPrim: 22
%

category: 'Private'
classmethod: System
_dumpFrameData

"Private.  Prints out shared page cache frame data to the session log file."

^ self _zeroArgPrim: 58
%

category: 'Private'
classmethod: System
_exitBypassReadAuth

"For use by indexing and query subsystem only.

 Any in-memory objects which were faulted in while bypass was enabled by
 a preceeding _bypassReadAuth, and for which a read-authorization error would
 have otherwise occurred, will be marked not-valid, so that subsequent
 access will get a read-authorization error.

 Has no effect if current user is SystemUser."

<primitive: 692>
self _primitiveFailed: #_exitBypassReadAuth
%

category: 'Reduced Conflict Support'
classmethod: System
_failedCommitWithResult: commitResult

"Increments the failedCommitCount stat for the session and sets
 the lastFailedCommitReasonCode stat to commitResult. 
 Returns true."
 
  ^ self _primFailedCommitWithResult: commitResult
%

category: 'Private'
classmethod: System
_findAllPublishedOn: machine
  "Description:
     Find all published servers on the given machine.
   Input:
     machine <String or nil>: name of machine to find on or nil if local.
   Result <Array or nil>:
     nil if no servers were found.
     instance of Array that contains one element for each server found.
     Each element will itself be an Array with the contents described
     for the _findPublished:on: method.
   Exceptions:
     Illegal input raises various errors.
     Error 2367 raised if low level C code fails with system error.
  "

<primitive: 499>
self _primitiveFailed: #_findAllPublishedOn: args: { machine }
%

category: 'Private'
classmethod: System
_findPublished: server on: machine
  "Description:
     Find a published server on the given machine.
   Input:
     server <String>: name of server to findunpublish.
     machine <String or nil>: name of machine to find on or nil if local.
   Result <Array or nil>:
     nil if server was not found.
     instance of Array if server was found. The array contents are:
       1. <String>: the name of the server
       2. <SmallInteger>: the port number of ther servers listening socket.
       3. <Integer>: the process id of the server's main process.
       4. <Boolean>: true if process exists, false if not.
       5. <String>: type of server: 'Stone', 'NetLDI', 'Cache', 'Broker',
                                    'Agent', 'RCPD', 'Service-Stone',
                                    'Service-NetLDI', or '?'.
       6. <String>: name of user that created the server.
       7. <String>: time server was created.
       8. <String>: name of machine server is running on.
       9. <String or nil>: version of GemStone used by server.
       10.<String or nil>: GemStone product directory.
       11.<String or nil>: log file name of server.
       12.<String or nil>: startup options of server.
       13.<String or nil>: GemStone system configuration file used by server.
       14.<String or nil>: GemStone executable configuration file used by server.
       15.<String or nil>: linux executable path, value of /proc/<processId>/exe
   Exceptions:
     Illegal input raises various errors.
     Error 2367 raised if low level C code fails with system error.
  "

<primitive: 498>
self _primitiveFailed: #_findPublished:on: args: { server . machine }
%

category: 'Shared Cache Management'
classmethod: System
_formatCacheReport: anArray
"given an Array produced by remoteCachesList or midLevelCachesList
 return a String containing one line per element of the list."

| rpt body formatBlk header anyX509 stnLine |
formatBlk := [:aCache :aRpt | |kind kindStr hostAgentSessionId haStr aSize |
  aRpt addAll: ((aCache at: 1) width: 30) . "hostname"
  hostAgentSessionId := aCache atOrNil: 7 .
  kind := aCache at: 2 .
  kind == nil ifTrue:[ kindStr := 'stone' ] ifFalse:[
  kind == true ifTrue:[ 
    kindStr := hostAgentSessionId > 0 ifTrue:[ 'x509mid'] ifFalse:[ 'mid'].  ] ifFalse:[
  kind == false 
     ifTrue:[ kindStr := hostAgentSessionId > 0 ifTrue:[ 'x509remote'] ifFalse:['remote']]
    ifFalse:[ kindStr := 'kind' . haStr := hostAgentSessionId.  "header line"].
  ]].
  kindStr := String withAll: kindStr .
  aRpt addAll: (kindStr width: 10) .
  aRpt addAll: ((aCache at: 3) asString width: -11 ). "sessions"
  aRpt addAll: ((aCache at: 6) asString width: -13 ). "sessions using as midCache"
  aRpt addAll: ((aCache at: 4) asString width: -10 ). "maxsessions"
  aSize := aCache at: 5 .
  aSize _isSmallInteger ifTrue:[ aSize := aSize // 1024 ].
  aRpt addAll: (aSize asString width: -10 ). "sizeMB"
  haStr ifNil:[ 
    hostAgentSessionId > 0 ifTrue:[ 
      haStr := '    ', hostAgentSessionId asString . anyX509 := true .
      (aCache atOrNil: 9 ) ifNotNil:[ :midHaSess |
         midHaSess > 0 ifTrue:[
           haStr addAll: '  midHa=' ; addAll: midHaSess asString .
         ].
      ].
    ] ifFalse:[ haStr := String new ] 
  ].
  aRpt addAll: haStr ; lf .
].  
body := String new .
1 to: anArray size do:[:k || aCache |
  aCache := anArray at: k .
  formatBlk value: aCache value: body .
].
header := { 'hostname' . 'kind' . 'sessions' . 'maxSess' . 'sizeMB' . 'mid-sess''s' }.
anyX509 ifNil:[ header add:''] ifNotNil:[ header add: '  hostAgentSession' ].
header := header collect:[ :s | s copy ]. "make strings variant"
rpt := String new .
formatBlk value: header value: rpt . "header line"
rpt addAll: body .
header := { System stoneVersionReport at: #nodeName .
            nil . System currentSessionCount .
            System stoneConfigurationAt: 'STN_MAX_SESSIONS' .
            (System stoneConfigurationAt: 'SHR_PAGE_CACHE_SIZE_KB'). 0 . 0 } .
stnLine := String new .  
formatBlk value: header value: stnLine . "stone cache line"
rpt addAll: stnLine .
^ rpt
%

category: 'Private'
classmethod: System
_gcGemConfig: isReclaim symb: configSymbol toValue: aVal

"Primitive implementation to fetch or set gcGem configs at runtime.
 If the gcUserGlobals argument is passed, it also updates the corresponding persistent
 entry in the GcUser's UserGlobals and commits.

 Use nil for the aVal argument to fetch the current value."

<primitive: 178>
isReclaim _validateClass: Boolean.
configSymbol _validateClass: Symbol.
aVal _validateClass: SmallInteger.
^ self _primitiveFailed: #_gcGemConfig:symb:toValue:
       args: { isReclaim . configSymbol . aVal }
%

category: 'Transaction Control'
classmethod: System
_gciCommit
  "used by GciCommit"

  (self _zeroArgPrim: 5) <= 1 ifTrue:[ "level 0 or 1"
    (self _commit: 3"from GCI") ifFalse:[
      ^ false "commit conflicts"
    ].
  ] ifFalse:[
    self _commitNestedTransaction.
  ] .
  ^ true
%

category: 'Version Management'
classmethod: System
_gemBuildShaString

 "Private. result is first 64 bits of the git SHA of the build as a Hex string"
 | str count | 
 str :=  (self _zeroArgCateg2Prim: 1) asHexString .
 count := 16 - str size .
 count > 0 ifTrue:[ | pad |
   pad := String new .
   count timesRepeat:[ pad add: $0 ].
   ^ pad , str
 ].
 ^ str
%

category: 'Private'
classmethod: System
_gemCommitConflictDetails
  "Optimized implementation of  (self gemConfigurationAt: #GemCommitConflictDetails) ,
   returns a Boolean. "
^ self _zeroArgPrim: 72
%

category: 'Performance Monitoring'
classmethod: System
_gemCpuTimeMs

"Returns an Integer indicating the amount of CPU time used by the current
 process, in units of milliseconds.  The resolution of the result is operating
 system dependent; typical resolution is 10 to 50 milliseconds."

^ self _zeroArgPrim: 10
%

category: 'Version Management'
classmethod: System
_gemVersion

"Private."

^ self _zeroArgCateg2Prim: 3
%

category: 'Version Management'
classmethod: System
_gemVersionNum

 "Private. result is of form 30501 for 3.5.1"
 
 ^ self _zeroArgCateg2Prim: 2 
%

category: 'Version Management'
classmethod: System
_generatePassiveSourceFileTo: fileName
"generate a passivated data structure containing all source code for classes
 in Globals. The file generated by this method can be used by
 System(C)>>_compareGlobalsWith:writeReportTo:generateFileinTo: to generate
 a report and/or filein script to reconcile kernel class differences between
 two versions of the image. If fileName is nil, file will be given a useful
 name like '1.0.0-sourceStrings.obj'.
"
|ver classes outfile outfilename processedClasses |
fileName ifNil: [ ver := self gemVersionReport at: #gsRelease.
                  ver := (ver copyReplaceAll: ' ' with: '').
                  outfilename := ver, '-sourceStrings.obj' ]
       ifNotNil: [outfilename := fileName].
classes := Dictionary new.
processedClasses := IdentitySet new .
Globals do: [ :ea | | cls methSrcDict classMethSrcDict |
  (ea isKindOf: Behavior) ifTrue: [  | methsBlk |
    cls := ea thisClass .
    (processedClasses includes: cls) ifFalse:[
      processedClasses add: cls .
      methSrcDict := Dictionary new.
      classMethSrcDict := Dictionary new.
      classes at: ea name put: { methSrcDict . classMethSrcDict } .
      methsBlk := [ :mDict :targetDict |
        mDict keysAndValuesDo: [ :k :aMeth | | envId aName |
          envId := aMeth environmentId .
          aName := envId == 0 ifTrue:[ k ] ifFalse:[ k asString ,':env_', envId asString].
          targetDict at: aName put: aMeth sourceString.
        ].
      ].
      ea _persistentMethodDicts do:[:aDict | methsBlk value: aDict value: methSrcDict].
      ea class _persistentMethodDicts do:[:aDict | methsBlk value: aDict value: classMethSrcDict].
    ].
  ].
].
outfile := GsFile openWriteOnServer: outfilename.
outfile ifNil: [^'error opening outfile', outfilename].
(PassiveObject passivate: classes toStream: outfile)
     ifNil:[ 'error writing outfile', outfilename].
outfile close.
^true
%

category: 'Private GC'
classmethod: System
_generationScavenge

"Returns true if the scavenge succeeded, false if it got promoted
 to a markSweep

 Explicitly triggers in-memory scavenge collection of temporary objects.
 Should only be used when debugging internal problems. Frequent use
 can degrade performance."

^ self _zeroArgPrim: 14
%

category: 'Private GC'
classmethod: System
_generationScavenge_vmMarkSweep

"Execute an in-memory scavenge and an in-memory markSweep.
 Should only be used when debugging internal problems. Frequent use
 can degrade performance."

self _generationScavenge ifTrue:[
  self _vmMarkSweep
  ].
%

category: 'Private'
classmethod: System
_getMidCachePort: aHostName

"Get the mid cache IP and port from stone
 for an already running mid cache on the
 specified host, without actually connecting to the mid cache.

 returns an Array  { sourceMidCacheIp . sourceMidCachePort }
 or a String with error details.
"

<primitive: 1104>
aHostName _validateInstanceOf: String .
self _primitiveFailed: #_getMidCachePort: args: { aHostName }
%

category: 'Private'
classmethod: System
_hostAgentVersionString
  "Returns a version and build string used by the hostagent."
  ^ self _zeroArgCateg2Prim: 4
%

category: 'Private'
classmethod: System
_hostCallDebugger

"Private, for debugging.
 calls HostCallDebugger in the VM which causes a core dump in customer
 executables and sleeps in non-customer executables.  Depending on the
 value of STN_HALT_ON_FATAL_ERR in stone's config file, this method may also
 cause the stone process to halt."

^ self _zeroArgPrim: 25
%

category: 'Host System Access'
classmethod: System
_hostProcessExists: aProcessId
  "Returns true if aProcessId exists as a process on this sessions' machine,
   false otherwise."

<primitive: 1023>
aProcessId _validateClass: SmallInteger .
^ self _primitiveFailed: #_hostProcessExists:
%

category: 'Host System Access'
classmethod: System
_hostTimes
" return a 4 element Array holding the elements of the struct tms result
  from
      clock_t times(struct tms *buffer)

  The Array elements are tms_utime, tms_stime, tms_cutime, tms_cstime ,
  and each element ia s Float in units of seconds.

  tms_utime is the CPU time  used  while  executing
  instructions in the user space of the calling process.

  tms_stime is the CPU time used by the  system  on
  behalf of the calling process.

  tms_cutime is the sum of the  tms_utime  and  the
  tms_cutime of the child processes.

  tms_cstime is the sum of the  tms_stime  and  the
  tms_cstime of the child processes.
"
^ self _zeroArgPrim: 119
%

category: 'Private'
classmethod: System
_hostWaitForDebugger

"Private, for debugging,
 calls HostExplicitWaitForDebugger, in the VM which puts this session's
 VM into an infinite wait for gdb or dbx to be attached.  Does not
 notify stone, nor trigger core dump."

^ self _zeroArgPrim: 16
%

category: 'Private'
classmethod: System
_ignoreModTracking

"Private. Disables Modification Tracking."

<primitive: 516>
self _primitiveFailed: #_ignoreModTracking
%

category: 'Version Management'
classmethod: System
_imageVersion

"Returns a String, which contains the history of the image."

^Globals at: #DbfHistory.
%

category: 'Private'
classmethod: System
_initMap: aMap

"Initializes the underlying data structures used to perform the mapping
 of passivated objects.
 aMap must be a SmallInteger  1 or 2 .
 Returns a SmallInteger serial number to be used for calls
 to _inMap:at:putIfAbsent:serialNum:
"

<primitive: 319>
aMap _validateClass: SmallInteger .
self _primitiveFailed: #_initMap: args: { aMap }
%

category: 'Private'
classmethod: System
_inMap: aMap at: anOop putIfAbsent: newValue serialNum: serialNum

"If there is no current value at anOop in the passive object map aMap,
 (that is, it is absent), then this method stores newValue and returns newValue.
 If the current value at anOop is valid however, it returns the value and does
 not store the newValue in the map.

 The argument aMap must be in the range 1 - 4
 and newValue must be a positive SmallInteger.

 anOop must not be a special object.
 serialNum must be a SmallInteger
"

<primitive: 320>
aMap _validateClass: SmallInteger .
(aMap < 1 or:[ aMap > 2]) ifTrue:[
  OutOfRange new name: #aMap min: 1 max: 2 actual: aMap ; signal
].
anOop isSpecial ifTrue:[ ArgumentError signal:'special object not allowed' ].
newValue _validateClass: SmallInteger .
serialNum _validateClass: SmallInteger .
newValue < 1 ifTrue:[ newValue _error: #rtErrArgNotPositive ].
self _primitiveFailed: #_inMap:at:putIfAbsent:serialNum:
     args: { aMap . anOop . newValue . serialNum }
%

category: 'Private'
classmethod: System
_inProtectedMode

 "Return true if in protected mode"

  ^self _zeroArgPrim: 84
%

category: 'Private'
classmethod: System
_internalMessageForErrorNum: aNumber

"Private.  Returns a String describing the error message generated from the
 errmsg.c module for GemStone error numbered aNumber.  Returns nil if no such
 error number or message exists in errmsg.c."

<primitive: 384>
aNumber _validateClass: SmallInteger .
self _primitiveFailed: #_internalMessageForErrorNum: args: { aNumber }
%

category: 'Error Handling'
classmethod: System
_lastGsErrorNumber

"Return the value of the last error number.  Also clears the error number back to zero."
^self _zeroArgPrim: 47
%

category: 'Private'
classmethod: System
_lastReservedOopNumber
  "Used in image upgrade, result is a constant from the VM."

^ self _zeroArgPrim: 115
%

category: 'Transaction Control'
classmethod: System
_localAbort

"Rolls back all modifications made to committed GemStone objects and provides
 the session with a new view of the most recently committed GemStone state.

 These operations are performed whether or not the session was previously in a
 transaction.  If the transaction mode is set to #autoBegin, then a new
 transaction is started.  If the transaction mode is set to #manualBegin, then
 a new transaction is not started."
 | res |
 self _pendingCommitAbort .  "Signal error to application if appropriate"
 res := self _primitiveAbort . "Abort the current transaction"
 (self _zeroArgPrim: 82 "_sessionMethodsChangedForAbort") ifTrue:[ | policy|
   (policy := GsPackagePolicy current) enabled ifTrue:[
     policy refreshSessionMethodDictionary
  ]. 
 ].
 ^ res
%

category: 'Transaction Control'
classmethod: System
_localBeginTransaction

"Starts a new transaction for the session."

self _pendingCommitAbort .  "Signal error to application if appropriate"
self _primitiveBegin
%

category: 'Reduced Conflict Support'
classmethod: System
_localCommit: commitMode

  "commitMode is a SmallInteger:

   0 for normal,
   1 for release locks,
   2 for checkpoint and release locks,
   3 for normal commit called from GciCommit .

   Returns true if commit was read-only or succeeded ,
   false if there was a failure."

  | commitResult actualMode |
  self _processDeferredGciUpdates .
  commitMode == 3 ifTrue:[
    "called from GciCommit, no pending commit action."
    actualMode := 0
  ] ifFalse:[
    self _pendingCommitAbort .
    actualMode := commitMode
  ] .
  commitResult := [ self _primitiveCommit: actualMode
		  ] onException: Error do:[:ex |
		    "ensure primitive commit is not on stack, and won't be retried
		     on continuation of execution."
		    ex pass .
		    5
		  ].
  commitResult <= 0 ifTrue: [ 
    (self _zeroArgPrim: 81 "_sessionMethodsChangedForCommit") ifTrue:[ | policy |
      (policy := GsPackagePolicy current) enabled ifTrue:[
        policy refreshSessionMethodDictionary
      ].
    ].
    ^ true
  ] ifFalse: [
    commitResult == 1 ifTrue: [
      "Execute RC replay and then retry the commit."
      ^ self _resolveRcConflictsForCommit: actualMode
    ].
    self _failedCommitWithResult: commitResult . "48387"
  ].
  self _disallowSubsequentCommits: 1 .
  ^ false
%

category: 'Private'
classmethod: System
_locale
"Returns a Locale object setup according to the session's Locale configuration."

^ self _zeroArgPrim: 90.
%

category: 'Transaction Control'
classmethod: System
_localTransactionMode: newMode

"Sets a new transaction mode for the current GemStone session and exits the
 previous mode by aborting the current transaction.  Valid arguments are
 #autoBegin, #manualBegin and #transactionless."

self _pendingCommitAbort .
^ self _primTransactionMode: newMode
%

category: 'Setting Locks'
classmethod: System
_lock: anObject kind: lockKind  autoRelease: autoReleaseBoolean

"Attempt to lock a single object.
 lockKind argument must be a SmallInteger , one of
    2  read
    3  write
    4  (not used)
    5  wait for Application write lock 1
    ...
   14  wait for Application write lock 10

 lockKinds 5..14  may be used with only one unique persistent object
 during the life of a stone process .  The first use of that lockKind
 registers anObject with that transient queue in the stone process, and
 subsequent uses of that lockKind with an object not identical to anObject
 will generate an error.  An error will be generated if anObject is
 not committed.  The queues are reinitialized each time stone process restarts.

 Any attempt to lock AllSymbols generates an error.

 This method returns a SmallInteger.
 Attempt to lock a special object always returns  2075  denied .
 If lockKind is < 2 or > 14 ,   returns 2071 .

 read and write requests return one of
   1  granted
   2074  dirty (object written by other session since start of this transaction)
   2075  denied (locked by another session)

 'wait for' requests return one of
   2074  dirty (object written by other session since start of this transaction)
   2075  denied (locked by another session)
   2418  deadlock
   2419  timeout  (per STN_OBJ_LOCK_TIMEOUT config parameter)
   2420  a different object is already registered with the specified
         Application write lock queue

 If autoReleaseBoolean == true and result is either 1 or 2074 ,
 then the object is added to  both the CommitReleaseLocksSet
 and CommitOrAbortReleaseLocksSet hidden sets.
"

<primitive: 97>
self _primitiveFailed: #_lock:kind:autoRelease:
     args: { anObject . lockKind . autoReleaseBoolean } .
self _uncontinuableError
%

category: 'Setting Locks'
classmethod: System
_lockAll: aCollection kind: lockKind

"Attempts to lock all the objects in aCollection.

 The lockKind argument specified the kind of lock and
 must be a SmallInteger, one of
    2  read
    3  write

 An error is generated and no locks are obtained
 if aCollection is a byte format object.

 An error is generated and no locks are obtained if any element
 of aCollection is identical to AllSymbols .

 Returns the receiver if all locks were granted and not dirty .
 Otherwise returns an Array of size 3 with elements :
   1.  An Array of objects that could not be locked.
   2.  An Array of objects that were locked but whose locks are dirty.
   3.  An empty Array, retained for backward compatibility with old GemStone versions.
"

<primitive: 100>
"primitive fails if aCollection is not a kind of SequenceableCollection
 or IdentityBag "
| anArray |
anArray := aCollection asArray.
anArray == aCollection ifFalse:[
  ^ self _lockAll: anArray kind: lockKind
  ] .
aCollection _validateClass: Collection.
lockKind _validateClass: SmallInteger .
self _primitiveFailed: #_lockAll:kind: args: { aCollection . lockKind } .
self _uncontinuableError
%

category: 'Setting Locks'
classmethod: System
_lockError: errNum obj: anObj details: details

"Translate from SmallInteger to Symbol, for error reporting."
"Ugh."

| ex sym |
ex := LockError new _number: errNum ; object: anObj ; details: details .

(errNum == 2075) ifTrue: [  sym := #lockErrDenied ] ifFalse:[
(errNum == 2074) ifTrue: [  sym := #lockErrObjHasChanged] ifFalse:[
(errNum == 2071) ifTrue: [  sym := #lockErrUndefinedLock ] ifFalse:[
(errNum == 2073) ifTrue: [  sym := #lockErrIncomplete ] ifFalse:[
(errNum == 2418) ifTrue: [  sym := #lockErrDeadlock ] ifFalse:[
(errNum == 2419) ifTrue: [  sym := #lockErrTimeout ] ifFalse:[
]]]]]].

^ ex reason: sym ; signal
%

category: 'Setting Locks'
classmethod: System
_lockEvaluateErr: errNum obj: anObject denied: denyBlock changed: changeBlock

"Private."

(errNum == 2074) ifTrue:[ ^ changeBlock value ]. "object  written by other session"
(errNum == 2075) ifTrue:[ ^ denyBlock value ]. " lock denied "

^ self _lockError: errNum obj: anObject details: 'unexpected lock error'
%

category: 'Private'
classmethod: System
_markTranlogWith: aString

"Writes a debug record to the tranlog with the specified string.
 String should be <= 16 characters, the rest are truncated."

<primitive: 1015>
self _primitiveFailed: #_markTranlogWith: args: { aString }
%

category: 'Private'
classmethod: System
_maxClusterId

" return the maximum clusterId supported vy the VM."
^ self _zeroArgPrim: 91
%

category: 'Private'
classmethod: System
_maxPrimitiveNumber

"Return a SmallInteger, the maximum primitive number for this VM."

^ self _zeroArgPrim: 184
%

category: 'Private'
classmethod: System
_messageForErrorNum: aNumber

"Private.  Returns a String describing the error message produced for GemStone
 error numbered aNumber in native language English, or nil if no such error
 number or language exists."

^ self _messageForErrorNum: aNumber inLanguage: #English
%

category: 'Private'
classmethod: System
_messageForErrorNum: aNumber inLanguage: aSymbol

"Private.  Returns a String describing the error message produced for GemStone
 error numbered aNumber in native language aSymbol, or nil if no such error
 number or language exists."

 | errArray msgParts result |

 errArray := GemStoneError at: aSymbol otherwise: nil .
 errArray == nil ifTrue:[ ^ nil ].
 (aNumber < 1 or:[ aNumber > errArray size]) ifTrue:[ ^ nil ].
 msgParts := errArray at: aNumber .
 msgParts == nil ifTrue:[ ^ nil ].
 result := String new .
 1 to: msgParts size do:[:j | | element |
   element := msgParts at: j .
   element _isSmallInteger
     ifTrue:[ result addAll: '<arg'; addAll: element asString; add: $> ]
    ifFalse:[ result addAll: element ].
   ].
 ^ result
%

category: 'Private'
classmethod: System
_midCachePgsvrListeningPort

 "Returns nil or the listening port of the multithreaded pgsvr
  on the mid cache for this gem."

^ self _zeroArgPrim: 69
%

category: 'Private'
classmethod: System
_midLevelCacheConnect: hostName options: anArg

"If (hostName == nil) and (anArg class == Array) returns a String printout of the
 results of parsing the options array.

 If (anArg == 1) then semantics are disconnect any existing mid level cache
 and connect to an already running mid level cache on host  hostName .

 If (anArg == 2) then hostNames is ignored;
 return a String, the IP address of an existing mid cache
 connection, or nil if no connection to a mid cache.

 Otherwise behaves per  System(C)>>midLevelCacheConnect:options:  "

<primitive: 664 >
hostName ifNotNil:[ hostName _validateInstanceOf: String ] .
anArg ifNotNil:[
  (anArg ~~ 1 and:[ anArg ~~ 2]) ifTrue:[ anArg _validateInstanceOf: Array ] .
].
hostName size > 2000 ifTrue:[ hostName _error: #errArgTooLarge args:{ 2000 } ].
^ self _primitiveFailed: #_midLevelCacheConnect:options:
       args: { hostName . anArg }
%

category: 'Private'
classmethod: System
_notVotedSessions: resultBlock into: res
  | list userPro num desc |
  list := self _currentSessionProfiles .
  1 to: list size by: 2 do:[:j |
    (userPro := list at: j + 1) ifNotNil:[
      userPro _isInteger ifFalse:[  "ignore logsender, logreceiver"
        desc := self _descriptionOfSessionSerialNum: 0 sessionId: (num := list at: j).
        (desc at: 20) < 2 ifTrue:[  resultBlock value: res value: num value: userPro ].
      ].
    ].
  ].
  ^ res
%

category: 'Private'
classmethod: System
_numPagesInSharedOt

"Returns the number of pages in the shared object table.
 This method aborts the current transaction."

^self _zeroArgPrim: 55
%

category: 'Transaction Control'
classmethod: System
_numPersistentObjsModified

"Returns number of persistent objects modified during current transaction.
 Includes number of objects added/removed to/from depMap."

^ self _zeroArgPrim: 54
%

category: 'Shared Counters'
classmethod: System
_numSharedCounters

"Returns the number of shared counters supported by this session's shared
 page cache. This is set by the value for SHR_PAGE_CACHE_NUM_SHARED_COUNTERS
 that is specified in the configuration file parameter used by this shared cache;
 this parameter has a default of 1900."

  ^self _zeroArgPrim: 100
%

category: 'Configuration File Access'
classmethod: System
_objectReadLogInfo
 "Returns nil if object read logging is not enabled.
  Otherwise returns an Array containing
    a String , path to object read log file that stone is writing to
    an Integer, the zero based offset of element of STN_OBJECT_READ_LOG_DIRECTORIES in use,
    an Integer,  size in bytes of the current object read log file"
  ^ self _zeroArgPrim: 190
%

category: 'Private'
classmethod: System
_objectsInNscEnumCache

"Returns an array with the objects in the NscEnumerationCache"

^self _zeroArgPrim: 56
%

category: 'Private'
classmethod: System
_objectTableCleanup

"Enters a transaction critical region and writes all deltas in the
 object table to the leaves using a multithreaded algorithm.
 Requires SystemControl and GarbageCollection privilege."

^self _zeroArgPrim: 15
%

category: 'Performance Monitoring'
classmethod: System
_objectTableSizeInMegabytes

"Returns an Integer that gives the size of the object table in megabytes"

^ self _zeroArgPrim: 101
%

category: 'Performance Monitoring'
classmethod: System
_oopHighWaterMark

"Returns an Integer that gives the highest oop allocated in GemStone."

^self _zeroArgPrim: 93
%

category: 'Performance Monitoring'
classmethod: System
_oopNumberHighWaterMark

"Returns an Integer that gives the highest oop number allocated in GemStone."

^self _zeroArgPrim: 94
%

category: 'Shared Cache Management'
classmethod: System
_otherCacheAddresses

"Return an Array of the ipAddresses of the currently active leaf caches,
 excluding the shared cache on which this session is running."

^ self _cacheAddresses: false includeMyCache: false
%

category: 'Shared Cache Management'
classmethod: System
_otherMidCacheAddresses

"Return an Array of the ipAddresses of the currently active midLevel caches,
 excluding the shared cache on which this session is running."

^ self _cacheAddresses: true includeMyCache: false
%

category: 'Private'
classmethod: System
_parseClientNrs: aString

"aString should be the result of reading a request sent to a netldi
 on a socket. A session reading nrs requests from clients can tell
 that a full request has come in by reading until it gets a zero byte.
 This method raises the error #genericKernelError if something is
 wrong with the input.
 Otherwise it returns a 8 element Array with the following contents:
 1  Symbol describing type of client. It will be one of the following:
      #RpcApplication, #Gem, #PageServer, #Stone, #Unknown
 2  Boolean: true if client is trusted.
 3  String: name of user.
 4  Boolean: (obsolete)
 5  Symbol describing type of request. It will be one of the following:
      #invalid, #file, #monitor, #server, #task, #spawn, #cmd, #dbf #fork
 6  String: working directory
 7  String: logfile name
 8  String: body of request
"

<primitive: 515>
| syms |
" make sure we don't have to create new symbols at runtime"
syms := { #RpcApplication . #Gem . #PageServer . #Stone . #Unknown .
	#invalid . #file . #monitor . #server . #task . #spawn . #cmd .
        #dbf . #fork } .
self _primitiveFailed: #_parseClientNrs: args: { aString }
%

category: 'Transaction Control'
classmethod: System
_pendingCommitAbort

""

| commitAction |
commitAction := self __sessionStateAt: 7 .
commitAction ifNotNil:[
  commitAction _isSmallInteger ifTrue:[
      "Signal commit-abort pending error"
      TransactionError new reason:#rtErrCommitAbortPending; _number: 6007 ; signal
      "expect the application to continue with GciContinue"
    ]
    ifFalse: [ self userAction: commitAction ]
].
%

category: 'Private'
classmethod: System
_performOnServer: aString

"For compatibility, used by seaside"

^ self _performOnServer: aString withShell: nil
%

category: 'Private'
classmethod: System
_performOnServer: aString withShell: aShellOrNil

"Attempts to execute aString using the shell aShellOrNil.
 Uses the default shell /bin/sh if aShellOrNil is nil.
 Otherwise uses aShellOrNil, which must accept the -c argument.

 Returns an Array of 5 elements:
  raw status Integer,
  child process status Integer (after WEXITSTATUS macro applied),
  result String ,
  error string from script file write, fork, or result file read ,
  errno value, a SmallInteger from file write, fork, or file read
"
<primitive: 347>
aString _validateClasses: { String . Utf8 }.
aShellOrNil ifNotNil:[ aShellOrNil _validateClass: String ] .
^ aString _error: #hostErrPerform args:  { aShellOrNil }
%

category: 'Private'
classmethod: System
_prepareToCommit

"This is a placeholder for the implementation of voting in a two-phase commit
 protocol.  The future implementation should use #hasConflicts to return
 whether the current transaction could commit at this exact point in time.

 The current design of #hasConflicts however will issue false positives,
 indicating that there are conflicts even when there are not (bug 40094).
 Because of this, certain types of operations would always cause
 #_prepareToCommit to fail.  Until this is resolved, we'll default to
 always returning true -- later steps in the psuedo two-phase commit will
 still catch any problems.

 Note: See the comment for GsCurrentSession | hasConflicts to get a
 description of the side-effects of hasConflicts."

 " ^ GsSession currentSession hasConflicts not "
 ^ true
%

category: 'Private'
classmethod: System
_primCacheStatsForSlotOrSession: arg opCode: code

"This primitive performs various functions relating to cache statistics per
 the following table:

arg  code    meaning
=======================================================================
 +     0     arg is slot.  Return all stats for all kinds*
 +     1     arg is slot.  Return only stats for the process.
 -     0     arg is a gem -session ID.  Return all stats for all kinds*
 -     1     arg is a gem -session ID.  Return only stats for the gem.
 +     2     arg is a PID.  Return only stats for the process.

 str   3     arg is a cache name string.  Return only stats for the
             process matching the name.
 +     4     arg is a slot.  Return  the Pid for slot.
 +     5     arg is a PID.  Return the slot for the PID.
 +     6     arg is a slot.  Return host cpu stats for slot.
 +     7     arg is a PID.  Host cpu stats for pid.
 +     8     arg is a pgsvr session ID.  Return only stats for the
             pgsvr.
 +     9     arg is a slot.  Return the process kind for the slot
             (1 = monitor, 2 = stone, 3 = pgsvr, 4 = gem) or -1 if
             the slot is not in use.
 nil  10     arg is nil.  Return an array of strings which are the
             system stats available for each process.
  +   10     arg is a process ID.  Return an array of Integers which
             are the system statistics for the given process. arg == -1
             indicates the current gem process.
 nil  11    Return a Solo session's own stats .
=======================================================================

Legend:

+     a postive SmallInteger
-     a negative SmallInteger
str   aString
*     deprecated functionality which may be removed in a future release.
"

<primitive: 220>
code _validateClass: SmallInteger .
code == 3
  ifTrue:[ arg _validateClass: String ]
  ifFalse:[ arg _validateClass: SmallInteger] .
(code < 0 or:[ code > 11])
  ifTrue:[ code _error: #rtErrArgOutOfRange args:{ 0 . 7 } ] .
arg _error: #rtErrArgOutOfRange .
^ self _primitiveFailed: #_primCacheStatsForSlotOrSession:opCode:
       args: { arg . code }
%

category: 'Reduced Conflict Support'
classmethod: System
_primFailedCommitWithResult: commitResult

"Increments the failedCommitCount stat for the session and sets
 the lastFailedCommitReasonCode stat to commitResult. 
 Returns true."
 
  <primitive: 1108>
  self _primitiveFailed: #_primFailedCommitWithResult: args: { commitResult } .
%

category: 'Private'
classmethod: System
_primitiveAbort

"Do not call this method directly; it is private to GemStone.  You must use
 abortTransaction.  Otherwise the transparent GemStone Smalltalk interface will
 break.

 This method rolls back all modifications made to committed GemStone objects
 (connected to the root) and provides the session with a new view of the most
 current committed state.  These operations are performed whether or not the
 session was previously in a transaction.  If the transactionMode is set to
 #autoBegin, then a new transaction is started.  If the transactionMode is set
 to #manualBegin or #transactionless, then a new transaction is not started,
 but the session's view of the database is updated.
 This method always returns the receiver (System)."

<primitive: 335>
self _primitiveFailed: #_primitiveAbort .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_primitiveBegin

"Starts a new transaction for the session.  If the session is already
 in a transaction, aborts the transaction and starts a new transaction.

 If any permanent objects had been written by the session, their
 state is aborted.  This method returns the receiver (System)."

<primitive: 362>
self _primitiveFailed: #primitiveBegin.
self _uncontinuableError
%

category: 'Private'
classmethod: System
_primitiveCommit: commitMode

"Do not call this method directly; it is private to GemStone.  You must use
 commitTransaction or commitAndReleaseLocks.  Otherwise Gem Builder breaks.

 The argument commitMode is a SmallInteger:
    0 for normal,
    1 for release locks,
    2 for checkpoint and release locks.
    3  value not used , primitive will fail
    4 checkpoint and start dpNsUnion (used by full backup)

 The result of this primitive is a SmallInteger:
      -1 read-only  (there were no modified objects to commit),
       0 success,
       1 rcFailure  (replay of changes to instances of Rc classes failed),
       2 dependencyMap failure  (concurrency conflict on dependencyMap),
       3 validationFailure  (concurrency conflict),
       4 retryFailure,   (previous commit attempt failed with an rcFailure)
       5 commitDisallowed (disallowed due to other error),
       6 retry limit exceeded 
       7 symbol failure (symbol VM missing or restarted)
       8 lock failure "

<primitive: 336>
commitMode _validateClass: SmallInteger .

(commitMode < 0 or:[ commitMode > 2 ])
  ifTrue:[ commitMode _error: #rtErrArgOutOfRange args:{ 0 . 2 } .  ^ 1 "failure" ] .

self _primitiveFailed: #_primitiveCommit: args: { commitMode } .
self _uncontinuableError
%

category: 'Transaction Control'
classmethod: System
_primTransactionMode: newMode

""

<primitive: 368>
self _primitiveFailed: #_primTransactionMode: args: { newMode } .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_printCHeap

"in a slow or fastdebug VM , calls UtlHeapPrint()"
^ self _zeroArgPrim: 89
%

category: 'Private'
classmethod: System
_printSocketTrace
  ^ self _zeroArgPrim: 605 "has effect only in debug build of VM"
%

category: 'Private'
classmethod: System
_processDeferredGciUpdates

"Process deferred GemBuilder for C updates and return the number of objects that
 had deferred updates."

"The deferred updates are stored in __sessionStateAt: 12.  These
 are updates to objects that do not allow direct structural update such
 as AbstractDictionary, Bag, and Set ."

| res |
self _disableAddToDirtySets: true .
[ | arr count |
  arr := self __sessionStateAt: 13 "virtual machine constant".
  (count := arr size) ~~ 0 ifTrue:[
    1 to: count by: 2 do:[ :j |
      (arr at: j) _deferredGciUpdateWith: (arr at: j + 1).
      ].
    arr size: 0 .
    "reinitialize the identity dictionary"
    (self __sessionStateAt: 12 "virtual machine constant")
	initialize: 751"a prime number for a large small object"
    ].
  res := count // 2
] ensure:[
  self _disableAddToDirtySets: false .
].
^ res
%

category: 'Private'
classmethod: System
_protectedMode

"Returns a SmallInteger, the value of protected mode .

 The usage pattern for writing a protected method is

     <primitive: 2001>
     | prot |
     prot := System _protectedMode .
     [
       ""method body""
     ] ensure:[
       prot _leaveProtectedMode
     ]

    primitive 2001 enters protected mode.
    The _protectedMode method returns the current nesting level of
    protected methods (a SmallInteger).
    _leaveProtectedMode is a special selector which compiles to a
     bytecode ."

<protected primitive: 691>
self _primitiveFailed: #_protectedMode .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_publish: server port: port log: logname options: optstr
  "Description:
     Publishes information about a server so that the gslist utility
     can display it. The information will be published on the local machine.
   Input:
     server <String>: name of server to publish.
     port <SmallInteger>: port number of server listening socket.
     logname <String or nil>: name of log file or nil if none.
     optstr <String or nil>: startup options or nil if none.
   Result <Boolean>:
     true if publish was successful.
     false if server was already published.
   Exceptions:
     Illegal input raises various errors.
     Error 2367 raised if low level C code fails with system error.
  "

<primitive: 496>
self _primitiveFailed: #_publish:port:log:options:
     args: { server . port . logname . optstr }
%

category: 'Reduced Conflict Support'
classmethod: System
_rcValueCache

"Returns the cache dictionary that is stored in temporary session state
 used to hold calculated values for reduced conflict classes. "

^ self __sessionStateAt: 5
%

category: 'Performance Monitoring'
classmethod: System
_readClock

"Returns an Integer indicating the amount of CPU time used by the current
 process, in units of milliseconds.  The resolution of the result is operating
 system dependent; typical resolution is 10 to 50 milliseconds."

^ self _zeroArgPrim: 10
%

category: 'Private'
classmethod: System
_redoLog

"Returns the redo log for transaction level 1.
 For use by Rc replay during commit."

| lev | 
(lev := self _zeroArgPrim: 5"transactionLevel") == 1 ifFalse:[
  Error signal:'Replay not allowed at transactionLevel ' , lev asString .
].
(self __sessionStateAt: 4) ifNotNil:[:a | ^ a atOrNil: 1 ].
^ nil
%

category: 'Private'
classmethod: System
_reinitializeSharedCache

"This method shuts down the GcGem process and verifies that this session
 is the only one logged in.  If not the only user, then the primitive fails.
 If it is the only user, all attached pages for this session are detached
 and the cache is swept.  All dirty pages are written.  All remaining
 unattached pages are removed from the cache and their frames are put back
 in the free list.

 Returns self."

^self _zeroArgPrim: 36
%

category: 'Private'
classmethod: System
_reloginAsUser: aUserId password: aPassword encrypted: aBoolean

"Relogin this session with the specified user identification."

<primitive: 495>
aUserId _validateClass: String.
aPassword _validateClass: String.
aBoolean _validateClass: Boolean.
self _primitiveFailed: #_reloginAsUser:password:encrypted:
     args: { aUserId . aPassword . aBoolean }
%

category: 'Shared Cache Management'
classmethod: System
_remoteCachesList: forMidLevel

"Return an Array describing shared caches, per remoteCachesList."

| idx result batch |
idx := 0 .
result := { }  .
[ idx >= 0 ] whileTrue:[ | nextIdx batSize |
  batch := self _remoteCachesListStartingAt: idx midLevelOnly: forMidLevel
                       includeMyCache: true .
  batSize := batch size .
  batSize > 0 ifTrue:[
    nextIdx := batch at: batSize .
    batch size: batSize - 1.
    result addAll: batch .
    idx := nextIdx .
  ] ifFalse:[
    idx := -1 . "done"
  ].
].
^ result
%

category: 'Shared Cache Management'
classmethod: System
_remoteCachesListStartingAt: anOffset midLevelOnly: midBoolean
includeMyCache: includeMyCacheBoolean

"Return a portion of the result for remoteCachesList,
 starting at anOffset in stone's list.    anOffset is zero-based.

 Result size is zero if anOffset is beyond end of stone's list,
 otherwise last element of result is a SmallInteger, the nextEnumIdx.
 If nextEnumIdx is < 0 there are no more elements in stone's list,
 otherwise pass nextEnumIdx as anOffset argument in next call to this
 primitive to continue the enumeration.

 Result size is limited by size of gem to stone communication buffers."

<primitive: 665>
anOffset _validateClass: SmallInteger .
anOffset < 0 ifTrue:[ anOffset _error: #errArgTooSmall args:{ 0 } ].
midBoolean _validateClass: Boolean .
includeMyCacheBoolean _validateClass: Boolean .
^ self _primitiveFailed: #_remoteCachesListStartingAt:midLevelOnly:includeMyCache:
       args: { anOffset . midBoolean . includeMyCacheBoolean  }
%

category: 'Private'
classmethod: System
_removeFromRcQueue

"Release commit token if it is held by this session.
 Remove this session from the RcQueue in stone."

^ self _zeroArgPrim: 71
%

category: 'Private'
classmethod: System
_removeLockWriteSubset

"Removes the locks acquired on any objects that were locked using
 _writeLockWriteSubset and empties the hidden set."

^self _zeroArgPrim: 35

%

category: 'Private'
classmethod: System
_removePageFromCache: aPageId

"aPageId must be an Integer >= 0 and <= 549755813887  .
 If the specified page is in gem's shared cache,
 prints the page header and attempts to remove the page
 from the shared cache.   If the gem is running on stone's machine
 will also print the page header from the state of the page on disk.

 Returns true if successfully removed, false otherwise.
 The result will be false if the page was dirty, locked or pinned."

<primitive: 590>
aPageId  _validateClass: Integer.
(aPageId < 0 or:[aPageId > 549755813887 ]) ifTrue:[
  aPageId _error: #rtErrArgOutOfRange args:{ 0 . 549755813887 }
] .
^self _primitiveFailed: #_removePageFromCache: args: { aPageId }
%

category: 'Reduced Conflict Support'
classmethod: System
_resolveRcConflicts

  "Checks for selected conflicts.  If some are found, attempt to resolve those
   conflicts.  If any conflicts could not be resolved, returns false."
 [
  | scanArray redoObject replayFailed conflicts idx limit |
  scanArray := self _getRedoAndConflictObjects .
  scanArray ifNil: [
    "no redo objects were found, cannot resolve conflicts"
    self _removeFromRcQueue; _disallowSubsequentCommits: 3 .
    ^ false
  ].
  replayFailed := false.
  limit := scanArray size .
  idx := 1 .
  [ idx <= limit ] whileTrue:[
		redoObject := scanArray at: idx .
		conflicts := scanArray at: idx + 1 .
		idx := idx + 2 .
		(redoObject _resolveRcConflictsWith: conflicts) ifFalse: [
	    self rcValueCacheAt: #'Rc-Write-Write' put: redoObject for: self.
	    replayFailed := true.
      limit := 0 . "exit loop"
    ]
  ].
  replayFailed ifTrue: [
    " force subsequent attempts to commit to fail with retry limit error"
    self _removeFromRcQueue; _disallowSubsequentCommits: 4 .
    ^ false
  ].
  ^ true
 ] onException: Error do:[:ex |
   self _removeFromRcQueue; _disallowSubsequentCommits: 5 .
   ex pass .
   ^ false
 ]
%

category: 'Reduced Conflict Support'
classmethod: System
_resolveRcConflictsForCommit: commitMode

| result |

self _processDeferredGciUpdates .

"GsFile gciLogServer:'---- RedoLog: ' , self _redoLog printString ."
self _resolveRcConflicts ifFalse: [
  ^ false
].

result := self _primitiveCommit: commitMode.
(result == 0 or: [result == -1]) ifTrue: [
  ^ true.
].
self _failedCommitWithResult: result ; "48387"
     _disallowSubsequentCommits: 2 .
^ false
%

category: 'Private'
classmethod: System
_restoreModTracking

"Private. Enables Modification Tracking.

 Has no effect in this release"

<primitive: 517>
self _primitiveFailed: #_RestoreModTracking
%

category: 'Signals'
classmethod: System
_sendSignal: aSignal toSess: sessionIdOrSerial withMessage: aString

"Sends a signal (a SmallInteger) to the specified session with aString as a
 message.  The aString argument is currently limited to 1023 bytes.

 The sessionIdOrSerial argument is a SmallInteger specifying a session.  A
 positive value is interpreted as a serialNumber.  A negative value is
 interpreted as a negated sessionId."

^ self _sendSignal: aSignal toSess: sessionIdOrSerial withMessage: aString oob: nil
%

category: 'Private'
classmethod: System
_sendSignal: aSignal toSess: sessionIdOrSerial withMessage: aString oob: oobInt
"If oobInt nil or zero,  Send aSignal and aString to specified session .
 aString is limited to 1023 bytes.
 The sessionIdOrSerial argument is a SmallInteger specifying a session.  A
 positive value is interpreted as a serialNumber.  A negative value is
 interpreted as a negated sessionId.

 If oobInt > 0, send only specified GscOobMsgEType value to the session
 for debugging purposes only.  Requires SystemControl privilege.
 aSignal and aString are ignored, but must be a SmallInteger and a String.

 Signals an error if the specified session cannot be found, or if the session
 is the pagemanager thread, a logreceiver or a logsender .

 Warning, some values sent to some sessions might cause a session logout."

<primitive: 330>
aSignal _validateClass: SmallInteger.
sessionIdOrSerial _validateClass: SmallInteger.
aString _validateClass: String.
oobInt ifNotNil:[ oobInt _validateClass: SmallInteger ].
self _primitiveFailed: #_sendSignal:toSerialNum:withMessage:oob:
     args: { aSignal . sessionIdOrSerial . aString . oobInt } .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_sendTransactionSignal: sessionIdOrSerial kind: aKind

"send a sigAbort or a finishTransaction signal to the specified
 session.  The signal will be ignored in the session
 if the session has not enabled that specific signal.
 Also supports sendLostOt to a session.

 Returns a Boolean , true if stone sent the OOB byte to the
 specified session, false of the OOB socket was blocked.

 Generates an Error if no session exists for sessionIdOrSerial .

 You must have SystemControl privilege .

 aKind == 0 : sigAbort
 aKind == 1 : finishTransaction
 aKind == 2 : lostOt "

<primitive: 327>
sessionIdOrSerial _validateClasses:{ SmallInteger }.
aKind _validateClasses:{ SmallInteger }.
self _primitiveFailed: #sendSigAbortToSession:
     args: { sessionIdOrSerial . aKind }
%

category: 'Version Management'
classmethod: System
_serverVersionReport: isStone

"Private, returns a StringKeyValueDictionary."

| result |
result := StringKeyValueDictionary new .
VersionParameterDict keysAndValuesDo:[ :aName :anId | | aVal |
  aVal := self _configurationAt: anId isStone: isStone kind: $V .
  aVal ~~ nil ifTrue:[
    result at: aName put: aVal
    ].
  ].
^ result
%

category: 'Private'
classmethod: System
_sessionCacheStatAt: anIndex

"Returns the value of the session statistic at the specified index, which
 must be in the range -2 to 47.  -1 means return the value of the
 progressCount statistic.  -2 means return the value of the indexProgressCount
 statistic. -3 means set the GemKind statistic to value i."

^ self _sessionCacheStatAt: anIndex incrementBy: 0
%

category: 'Private'
classmethod: System
_sessionCacheStatAt: anIndex decrementBy: aSmallInt

"Decrements the value of the session statistic at the specified index
 by aSmallInt and returns the new value.  anIndex must be in the range
 of -2 to 47.  -1 means decrement the progressCount statistic.  -2 means
 decrement the indexProgressCount statistic."

^ self _sessionCacheStatAt: anIndex incrementBy: aSmallInt negated
%

category: 'Private'
classmethod: System
_sessionCacheStatAt: anIndex incrementBy: aSmallInt

"Increments the value of the session statistic at the specified index
 by aSmallInt and returns the new value.  anIndex must be in the range
 of -2 to 47.  -1 means increment the progressCount statistic.  -2 means
 increment the indexProgressCount statistic."

<primitive: 477>
anIndex _validateClass: SmallInteger.
aSmallInt _validateClass: SmallInteger.
((anIndex < -2) or: [ anIndex > 47]) ifTrue: [
   anIndex _error: #rtErrArgOutOfRange args: { -2 . 47 } ].
self _primitiveFailed: #_sessionCacheStatAt:incrementBy:
     args: { anIndex . aSmallInt } .
%

category: 'Private'
classmethod: System
_sessionCacheStatAt: anIndex put: i

"This method sets the session statistic at the specified index (which should be
 in the range -2 to 47) to the specified value i, which must be a SmallInteger.
 -1 means set the progessCount statistic to the value i.  -2 means set the
 indexProgessCount statistic to the value i."

<primitive: 476>
anIndex _validateClass: SmallInteger.
i _validateClass: SmallInteger.
((anIndex < -2) or: [ anIndex > 47]) ifTrue: [
   anIndex _error: #rtErrArgOutOfRange args: { -2 . 47 } ].
self _primitiveFailed: #_sessionCacheStatAt:put: args: { anIndex . i } .
%

category: 'Private'
classmethod: System
_sessionCacheStatsForProcessSlot: aProcessSlot

"Return an array containing the 48 SmallIntegers representing the
 session statistics for the given process slot.

 Returns nil if the given process slot is not found or if the slot
 is not in use by a gem process.

 See additional information in the method #_sessionCacheStatAt: put:"

^self _sessionCacheStatsForSlotOrSessionId: aProcessSlot
%

category: 'Private'
classmethod: System
_sessionCacheStatsForSessionId: aSessionId

"Return an array containing the 48 session statistics for the given
 session ID.

 Returns nil if the given session is not found or if the slot is not
 in use by a gem process.

 See additional information in the method #_sessionCacheStatAt: put:"

^self _sessionCacheStatsForSlotOrSessionId: aSessionId negated.
%

category: 'Private'
classmethod: System
_sessionCacheStatsForSlotOrSessionId: aProcessSlotOrSessionId

"Return an array containing the 48 session statistics for the given
 session ID or process slot number.  If the argument is positive
 it is assumed to be a process slot.  If it is negative it is assumed
 to be a negated session ID.

 Returns nil if the given session or cache slot is not found or
 if the slot is not in use by a gem process.

 See additional information in the method #_sessionCacheStatAt: put:"

<primitive: 566>
aProcessSlotOrSessionId _validateClass: SmallInteger.
aProcessSlotOrSessionId _error: #rtErrArgOutOfRange .
^self _primitiveFailed: #_sessionCacheStatsForSlotOrSessionId:
      args: { aProcessSlotOrSessionId } .
%

category: 'Updating the Method Dictionary'
classmethod: System
_sessionMethodsChangedForAbort
  "Returns a Boolean, the union of other sessions commits of
   Behavior >> _sessionMethodsChanged , unioned with our own
    Behavior >> _sessionMethodsChanged bit "
  ^ self _zeroArgPrim: 82 
%

category: 'Updating the Method Dictionary'
classmethod: System
_sessionMethodsChangedForCommit
  "Returns a Boolean, the union of other sessions commits of
   Behavior >> _sessionMethodsChanged ."
  ^ self _zeroArgPrim: 81 
%

category: 'Session Control'
classmethod: System
_sessionPriorityPrim: anInt forSessionId: aSessionId

"Fetches or sets the priority for the given session ID.
 If anInt is -1, then the current priority for the given
 session ID is returned.  Otherwise anInt must be a valid
 priority between 0 and 4.  See the method:
   System(C)>>setSessionPriority:forSessionId:
 for more information on session priority."

<primitive: 898>
anInt _validateClass: SmallInteger .
aSessionId _validateClass: SmallInteger .
((anInt < -1) or:[ anInt > 4])
  ifTrue:[anInt _error: #rtErrArgOutOfRange args:{ -1 . 4 } .].
aSessionId < 1
  ifTrue:[aSessionId _error: #rtErrArgOutOfRange ].
^self _primitiveFailed: #_setSessionPriorityPrim:forSessionId:
      args: { anInt . aSessionId }
%

category: 'Private'
classmethod: System
_sessionProfilesStartingAt: startSessionId

"Returns an Array containing pairs of sessionId, UserProfile ,
 for some sessions currently logged in, beginning at the specified sessionId.
 Some sessionIds may have a userProfile of nil
 if that UserProfile is not visible in the transactional view of
 the current session.
 The size of the Array is limited by number of sessions logged in
 and/or gem to stone communication buffer sizes.  If there are
 no more valid sessionIds beginning with startSessionId, an
 Array of size 0 is returned.

 This method requires SessionAccess privilege if there is more
 than one session logged in."

<primitive: 329>
startSessionId _validateClass: SmallInteger .
startSessionId _error: #rtErrArgOutOfRange .
^ self _primitiveFailed: #_sessionProfilesStartingAt: args: { startSessionId }
%

category: 'Private'
classmethod: System
_sessionsReport: arrayOfSessionIds
  | str |
  str := String new .
  arrayOfSessionIds do:[:sessId | | desc uId |
    sessId ifNotNil:[
      desc := self descriptionOfSession: sessId .
      (desc at: 1) ifNotNil:[:up|  uId := up userId ]
                   ifNil:[ (desc at: 17) ifNotNil:[:x | uId := '    ' "logsender, etc"]].
      uId ifNotNil:[
        str add: sessId asString; add:' '; add: uId; add: ' '.
        (desc at: 17) ifNotNil:[ :sysgem | str add: sysgem ; add: ' ' ; add:(desc at:2) asString ]
          ifNil:[ str add:'gem '; add:(desc at:2) asString; add:' on ' ;
                   add: (desc at: 3) ].
        str lf.
      ].
    ].
  ].
  ^ str
%

category: 'Transient Session State'
classmethod: System
_sessionStateAt: anIndex

"Deprecated, use sessionStateAt:, and note changes in allowed range"

self deprecated: 'System class>>_sessionStateAt: deprecated v3.2.
Use sessionStateAt:, with available range starting at 1.'.

^ self __sessionStateAt: anIndex + 20
%

category: 'Transient Session State'
classmethod: System
_sessionStateAt: anIndex put: aValue

"Deprecated, use sessionStateAt:put:, and note changes in allowed range"

self deprecated: 'System class>>_sessionStateAt:put: deprecated v3.2.
Use sessionStateAt:put:, with available range starting at 1.'.
^ self __sessionStateAt: anIndex + 20 put: aValue
%

category: 'Transient Session State'
classmethod: System
_sessionStateSize

"Return the current size of the Session State Array."

<primitive: 214>
self _primitiveFailed: #_sessionStateSize .
self _uncontinuableError
%

category: 'System Control'
classmethod: System
_sessionTrackReads
"Returns true if (System objectReadLogEnabled == true)
 and ((System myUserProfile _hasPrivilegeName: #DisableObjectReadLogging) == false)"

^ self _zeroArgPrim: 194
%

category: 'Private'
classmethod: System
_sessionUproString: rawUp
  ^  rawUp _isInteger ifTrue:[
       rawUp == 1 ifTrue:[ '    logsender'] ifFalse:['    logreceiver']
     ] ifFalse:[
       '    UserId: ' , rawUp userId .
     ]
%

category: 'Private'
classmethod: System
_setCategory: categorySym locale: localeStr

" Set Locale information.  See man page for setlocale( ) for details.

Valid category types include:

   LC_CTYPE        /* locale's ctype handline */
   LC_NUMERIC      /* locale's decimal handling */
   LC_TIME         /* locale's time handling */
   LC_COLLATE      /* locale's collation data */
   LC_MONETARY     /* locale's monetary handling */
   LC_MESSAGES     /* locale's messages handling */
   LC_ALL          /* name of locale's category name */

Applications should not call this directly -- instead use:

  Locale setCategory: <sym> locale: <str>'

"

<primitive: 661>
categorySym _validateClass: Symbol.
localeStr _validateClass: String.
self _primitiveFailed: #_setCategory:locale: args: { categorySym . localeStr }
%

category: 'Private'
classmethod: System
_setPersistentConfig: isReclaim symbol: configSymbol toValue: aVal

| gcUg result|
self needsCommit ifTrue: [ self _error: #rtErrAbortWouldLoseData ] .

gcUg := ((AllUsers userWithId:'GcUser') resolveSymbol: 'UserGlobals') value .
[
  self beginTransaction.
  gcUg at: configSymbol put: aVal.
  result := self _gcGemConfig: isReclaim symb: configSymbol toValue: aVal.
  self commitTransaction.
] on: Error do:[:ex |
  System abortTransaction .
  ex pass
].
^result
%

category: 'Debugging'
classmethod: System
_setPrintStackAtError
  "Set a flag in the VM so that a gem process will print error information
   and a Smalltalk stack to the gem log file when any error is generated."

  self _zeroArgPrim:102
%

category: 'Configuration File Access'
classmethod: System
_sharedCacheAttributes
"Returns an Array of attributes for the shared page cache to which the current 
 session is attached as follows:

 1.  (String)       Name of the cache
 2.  (SmallInteger) Total number of 16Kb frames in the cache.
 3.  (Integer)      Physical size of the shared memory segment in bytes.
 4.  (SmallInteger) Number of shared counters in the cache 
                    (see config option SHR_PAGE_CACHE_NUM_SHARED_COUNTERS).
 5.  (SmallInteger) Maximum number of processes that can attach the cache 
                    (see config option SHR_PAGE_CACHE_NUM_PROCS).
 6.  (SmallInteger) Spin lock count (see config option SHR_SPIN_LOCK_COUNT).
 7.  (SmallInteger) Large memory page policy 
                    (see config option SHR_PAGE_CACHE_LARGE_MEMORY_PAGE_POLICY).
 8.  (SmallInteger) Large memory page size in megabytes 
                    (see config option SHR_PAGE_CACHE_LARGE_MEMORY_PAGE_SIZE_MB).
 9.  (SmallInteger) Number of large memory pages used to create the cache.
10.  (SmallInteger) Target free frame count 
                    (see config option SHR_TARGET_FREE_FRAME_COUNT).
11.  (Boolean)      Cache locked in memory 
                    (see config option SHR_PAGE_CACHE_LOCKED).
12.  (Boolean)      True if the shared cache is remote from the stone, false otherwise.
13.  (Boolean)      True if the shared cache is a mid-level shared cache, false otherwise.
Returns nil if the current session is a solo session.
"
^ self _zeroArgPrim: 187
%

category: 'Shared Counters'
classmethod: System
_sharedCounter: n decrementBy: i withFloor: f

"The shared counter specified by n is decremented by i.
The new value of the counter will not be less than f .
Returns the new value of the counter.

The argument n must a SmallInteger in the range 0 to
(System _numSharedCounters - 1) .

The argument i must be a SmallInteger or LargeInteger in the range
0 to 2**63 - 1 (9223372036854775807).

f specifies the minimum final value of the shared counter.  Specifying a floor
of nil means the minimum value is INT64_MIN (-9223372036854775808, or -2**63),
which is the lowest possible value for any shared counter.  Attempting to
decrement any counter to a final value less than f will set the counter
to floor value f."

<primitive: 551>
| sz |
n _validateClass: SmallInteger.
i _validateClass: Integer .
(f == nil)
  ifFalse:[
     f _validateClass: Integer .
     (f < -9223372036854775808 or:[ f > 9223372036854775807 ])
   ifTrue:[f _error: #rtErrArgOutOfRange args: { -9223372036854775808 . 9223372036854775807 } ] .
  ].
sz := self _numSharedCounters .
(n < 0 or:[ n >= sz ])
  ifTrue:[ n _error: #rtErrArgOutOfRange args: { 0 . sz - 1 } ] .
(i < 0 or:[ i > 9223372036854775807 ])
  ifTrue:[i _error: #rtErrArgOutOfRange args: { 0 . 9223372036854775807 } ] .
self _primitiveFailed: #_sharedCounter:decrementBy:withFloor:
     args: { n . i . f }
%

category: 'Shared Counters'
classmethod: System
_sharedCounterFetchValuesFrom: firstCounter to: lastCounter

 "Returns an array containing the values from all shared counters starting
 with the counter at index firstCounter, up to and including the value
 from the counter lastCounter.

 Both arguments must be in the range 0...(System numSharedCounters - 1), and
 lastCounter must be greater than or equal to firstCounter."

<primitive: 589>
| sz |
firstCounter _validateClass: SmallInteger.
lastCounter _validateClass: SmallInteger.
(firstCounter < 0 or:[ firstCounter > lastCounter])
  ifTrue:[firstCounter _error: #rtErrArgOutOfRange args:{ 0 . lastCounter } ] .
sz := self _numSharedCounters .
(lastCounter < 0 or:[ lastCounter >= sz ])
  ifTrue:[lastCounter _error: #rtErrArgOutOfRange args:{ 0 . sz - 1 }].
self _primitiveFailed: #_sharedCounterFetchValuesFrom:to:
     args: { firstCounter . lastCounter }
%

category: 'Private'
classmethod: System
_signalErrorStatus: aSignal

"Returns status of the specified signal .
 Signals are specified by SmallIntegers, the following are defined:

 1   SignaledObjects
 2   SignaledAbort (if in transaction, reflects status after transation finishes)
 3   SignaledGemStoneSession
 4   SignaledFinishTransaction
 5   AlmostOutOfMemory (a Notification)
 6   SignalTranlogsFull
 9   AlmostOutOfMemoryError
 10  AlmostOutOfStackError
"
<primitive: 366>
self _primitiveFailed: #_signalErrorStatus: args: { aSignal } .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_signalFromGemStoneSession

"Returns a 4 element Array containing information about a signal from
 another GemStone session:

 1.  The sessionSerialNumber of the session that sent the signal.
 2.  The signal value (a SmallInteger).
 3.  A signal message (a String).
 4.  A SmallInteger, number of additional signals pending in stone

 If there is no signal in the queue, returns nil."

<primitive: 369>
self _primitiveFailed: #signalFromGemStoneSession .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_simpleCommitForReclaim: includeDeadBool

  "Calls stone to request a simple commit be performed.
   if includeDeadBool==true, returns true if both the
   number of dead objects and number of shadow pages needing reclaiming
   are zero .
   if includeDeadBool==false, returns true if
   the number of shadow pages needing reclaiming is zero .

   Has no effect on this session's transactional view."

<primitive: 290>

self _primitiveFailed: #_simpleCommitForReclaim: args: { includeDeadBool } .
self _uncontinuableError
%

category: 'Session Control'
classmethod: System
_sleep: aTime

"Sleep for aTime seconds.  aTime must be a positive SmallInteger.
 If aTime is zero, this method has no effect.

 Returns time left to sleep , which will be > 0 if interrupted.
 The ProcessorScheduler is not used, and other waiting GsProcesses
 will not execute during the sleep. Use a Delay or Semaphore to wait
 while allowing other GsProcesses to run."

<primitive: 370>
aTime _validateClass: SmallInteger .
aTime < 0 ifTrue: [ aTime _error: #errArgTooSmall args:{ 0 } ] .
^ self _primitiveFailed: #_sleep: args: { aTime }
%

category: 'Session Control'
classmethod: System
_sleepMs: milliSecondsToSleep

"Sleep for the number of milliseconds specified by milliSecondsToSleep.
 The argument milliSecondsToSleep must be a positive SmallInteger.
 If milliSecondsToSleep is zero, this method has no effect.
 Returns the receiver."

| timeLeft |
milliSecondsToSleep _isSmallInteger ifFalse:[
  milliSecondsToSleep _validateClass: SmallInteger
].
milliSecondsToSleep < 0 ifTrue:[
  milliSecondsToSleep _error: #errArgTooSmall args:{ 0 } ].
timeLeft := milliSecondsToSleep .
[ true ] whileTrue:[
  timeLeft := self __sleepMs: timeLeft .
  timeLeft <= 0 ifTrue:[ ^ self ].
]
%

category: 'Private'
classmethod: System
_stoneClearCommitQueues

"Cause all sessions waiting in stone's TransQueue or stone's RcTransQueue
 to have the session's call  System(C>>_primitiveCommit return with an error.
 Returns a SmallInteger, the number of sessions that were sent the error.

 For use in debugging commit deadlocks.

 Only SystemUser may execute this method, otherwise an error is generated.
"

^ self _zeroArgPrim: 43
%

category: 'Private'
classmethod: System
_stoneCommandsReport

"Returns a String describing heavily used commands to stone
 since the last report.  Result is non-empty only from slow stoned executable."

^ self _zeroArgPrim: 183
%

category: 'Performance Monitoring'
classmethod: System
_stoneCpuTimeMs

"Returns an Integer indicating the amount of CPU time used by the stone
 process, in units of milliseconds.  The resolution of the result is operating
 system dependent; typical resolution is 10 to 50 milliseconds."

^ self _zeroArgPrim: 197
%

category: 'Private'
classmethod: System
_stonePgsvrListeningPort

 "Returns nil or the listening port of the multithreaded pgsvr
  on stone's host for this gem's pgsvr."

^ self _zeroArgPrim: 68
%

category: 'Session Control'
classmethod: System
_stopHostAgents: aKind timeOut: waitSeconds
| list midList endMs waitMs |
endMs := DateTime now millisecondsGmt + (waitMs := waitSeconds * 1000) .
list := self hostAgentSessions .
midList := { } .
1 to: list size do:[:j | | sessionId descr |
  "first stop the midcache hostagents on mid cache hosts"
  sessionId := list at: j .
  descr := self descriptionOfSession: sessionId.
  (descr at: 17) ifNotNil:[:str |
     (str at: 1 equals: 'midcache') ifTrue:[
       list at: j put: nil .
       midList add: sessionId ; add: (descr at: 9) .
       self _stopHostAgentSession: sessionId kind: aKind .
     ].
  ].
].
aKind == 16r4"Break" ifTrue:[ | midEndMs waitingFor  |
  "Wait for a clean logout from the mid hostagents"
  midEndMs := DateTime now millisecondsGmt + (10000 max:( waitMs // 2)) .
  waitingFor := midList size // 2 .
  [ waitingFor > 0 ] whileTrue:[
    Delay waitForMilliseconds: 100 .
    1 to: midList size by: 2 do:[:n |
      (midList at: n) ifNotNil:[ :sessId || serialNum descr |
        serialNum := midList at: n + 1 .
        descr := self descriptionOfSession: sessId.
        ((descr at: 1"userPro") == nil or:[ (descr at: 9) ~~ serialNum]) ifTrue:[
           midList at: n put: nil .
           waitingFor := waitingFor - 1.
         ].
      ]
    ].
    DateTime now millisecondsGmt > midEndMs ifTrue:[ waitingFor := 0 ].
  ].
].
list do:[:sessionId |
   sessionId ifNotNil:[ self _stopHostAgentSession: sessionId kind: aKind ]
].
[ list size > 0 ] whileTrue:[
  (DateTime now millisecondsGmt > endMs )
    ifTrue:[ Error signal: list size asString , ' hostagents could not be stopped'].
  Delay waitForMilliseconds: 100 .
  list := self hostAgentSessions
]
%

category: 'Session Control'
classmethod: System
_stopHostAgentSession: sessionId  kind: aKind

| serialNum |
sessionId _validateClass: SmallInteger.
serialNum := GsSession serialOfSession: sessionId .
self _stopSession: serialNum kind: aKind timeout: -1
%

category: 'Session Control'
classmethod: System
_stopSession: aSerialNumber  kind: aKind timeout: aSeconds

"Aborts the transaction of the specified session (a SmallInteger), then
 terminates that session.  Returns the receiver.  If the indicated session is
 not active, no operation is performed.

 aKind has the following bits
   16r1  include GcGem sessions
   16r2  include SymbolCreation session
   16r4  include Hostagent sessions

   16r10  timeout aSeconds per terminateSession:timeout: ,
	  and do not inhibit   restart of gc gems or symbol gem

 To execute this method, you must have explicit privilege from your system
 data curator."

<primitive: 332>
aSerialNumber _validateClass: SmallInteger.
aSeconds _validateClass: SmallInteger.
aKind _validateClass: SmallInteger.
aKind _error: #rtErrArgOutOfRange .
^ self _primitiveFailed: #_stopSession:kind:timeout:
       args: { aSerialNumber . aKind . aSeconds }
%

category: 'Private'
classmethod: System
_systemLocksPrim

"For use in implementation of System(C) | systemLocks.
 This is an expensive method, it makes a call to stone for each element
 of the result of
    System currentSessions

 Returns an Array of Associations.
 Each Association has key sessionId,
  and value an Array of size 3.
  The elements of the Array are each Arrays ,
    first element  -  objects that are read locked
    second element -  objects that are write locked
    third  element -  objects for which unlock is deferred while a commit is in progress
 "
<primitive: 353>

self _primitiveFailed: #_systemLocksPrim .
self _uncontinuableError
%

category: 'Performance Monitoring'
classmethod: System
_tempObjSpaceMax

"Returns the approximate maximum number of bytes of temporary object memory which is
 usable for storing objects."

^self _zeroArgPrim: 78
%

category: 'Performance Monitoring'
classmethod: System
_tempObjSpacePercentUsed

"Returns the approximate percentage of temporary object memory which is
 in use to store temporary objects.  This is equivalent to the
 expression:

 (self _tempObjSpaceUsed * 100) // self _tempObjSpaceMax.

 Note that it is possible for the result to be slightly greater than 100%.
 This result indicates temporary memory is almost completely full."

^self _zeroArgPrim: 79
%

category: 'Performance Monitoring'
classmethod: System
_tempObjSpacePercentUsedLastMark

"Returns the approximate percentage of temporary object memory which is
 in use to store temporary objects, as of the last MarkSweep."

^self _zeroArgPrim: 86
%

category: 'Performance Monitoring'
classmethod: System
_tempObjSpaceUsed

"Returns the approximate number of bytes of temporary object memory being used
 to store objects."

^self _zeroArgPrim: 77
%

category: 'Time'
classmethod: System
_timeGmt2005: aBoolean

"Returns a SmallInteger, the time since January 1, 2005, in seconds.
 aBoolean is true, call Stone and update our offset from Stone's clock,
 otherwise compute the current time locally using the offset from Stone's time
 that was cached in the session at login."

<primitive: 598>
^ self _primitiveFailed: #_timeGmt2005: args: { aBoolean }
%

category: 'Time'
classmethod: System
_timeGmt95: aBoolean

"Returns a SmallInteger, the time since January 1, 1995, in seconds.
 aBoolean is true, call Stone and update our offset from Stone's clock,
 otherwise compute the current time locally using the offset from Stone's time
 that was cached in the session at login."

self deprecated: 'System class>>_timeGmt95 is deprecated v3.0. Use timeGmt2005'.
^self _timeGmt95Prim: aBoolean.
%

category: 'Time'
classmethod: System
_timeGmt95Prim: aBoolean

"Returns a SmallInteger, the time since January 1, 1995, in seconds.
 If aBoolean is true, call Stone and update our offset from Stone's clock,
 otherwise compute the current time locally using the offset from Stone's time
 that was cached in the session at login.

 Deprecated, use   System (C) >> _timeGmt2005:   .
"
<primitive: 381>
^ self _primitiveFailed: #_timeGmt95: args: { aBoolean }
%

category: 'Time'
classmethod: System
_timeGmt: aBoolean

"Returns a SmallInteger, the time since January 1, 1970, in seconds.
 aBoolean is true, call Stone and update our offset from Stone's clock,
 otherwise compute the current time locally using the offset from Stone's time
 that was cached in the session at login.

 The result is a posixSeconds value as used in class DateAndTimeANSI ."

<primitive: 371>
^ self _primitiveFailed: #_timeGmt: args: { aBoolean }
%

category: 'Time'
classmethod: System
_timeGmtFloat

"Returns a Float representing the time since January 1, 1970 in units
 of seconds, to microsecond resolution, or such resolution as provided
 by the operating system's gettimeofday() call.  This time is computed locally
 in the session process, using the offset from Stone's time that was
 cached in the session at login.

 The result is a posixSeconds value as used in class DateAndTimeANSI ."

^ self _zeroArgPrim: 13
%

category: 'Time'
classmethod: System
_timeGmtMicroSeconds
  "Returns a SmallInteger representing time since January 1, 1970 in microseconds,
  or such resolution as provided
  by the operating system's gettimeofday() call.  This time is computed locally
  in the session process, using the offset from Stone's time that was
  cached in the session at login."

^ self _zeroArgPrim: 188
%

category: 'Time'
classmethod: System
_timeMs

"Returns a SmallInteger representing the current relative time in milliseconds.
 The result is a SmallInteger equivalent to
    (System _timeGmtFloat * 1000) asInteger

 The result is computed locally in the session process, using the offset
 from Stone's time that was cached in the session at login.

 Gs64 v2.2, changed to no longer rollover to zero after 524287999 "

<primitive: 651>
^ self _primitiveFailed: #_timeMs
%

category: 'Time'
classmethod: System
_timeMsLegacy

"Returns a SmallInteger representing the current relative time in milliseconds.
 The result is a SmallInteger equivalent to
    (System _timeGmtFloat * 1000) asInteger \\ 524288000

 The result is computed locally in the session process, using the offset
 from Stone's time that was cached in the session at login."

^ self _zeroArgPrim: 12
%

category: 'Time'
classmethod: System
_timeUs

"Returns a SmallInteger between 0 and 999,999 representing the current number
 of microseconds that have passed within the current second."

^ self _zeroArgPrim: 103 .
%

category: 'Private'
classmethod: System
_trapMarkDirty: anObject

 "Returns the previous value of the mark dirty trap, or nil.
  anObject may be one of
    any committed object that does not have special format,
    nil  to disable the printing,
    the SmallInteger 1 to print any invariant object that is not a class.
  When the specified object is added to the dirtyList of the
  outer level transaction, C and Smalltalk stacks will be printed to
  topaz -l stdout, or to the gem log file."

<primitive: 1087>
%

category: 'Private'
classmethod: System
_uncommittedDataPages

"Returns an Array of the data pages referenced from the session's shadowed
 view of GemStone.  After the next commit or abort, the contents of this
 Array are invalid and may be incomplete.  That is, very shortly after the
 commit or abort operation, they may not show all data pages, because more pages
 may be added by the currently running methods.  However, it might be useful to
 check on modified objects (in particular, reclustering) before the transaction
 commits."

^ self _zeroArgCateg2Prim: 11
%

category: 'Private'
classmethod: System
_unpublish: server on: machine
  "Description:
     Unpublishes information about a server on the given machine.
   Input:
     server <String>: name of server to unpublish.
     machine <String or nil>: name of machine to unplubish on or nil if local.
   Result <Boolean>:
     true if unpublish was successful.
     false if server was not published.
   Exceptions:
     Illegal input raises various errors.
     Error 2367 raised if low level C code fails with system error.
  "

<primitive: 497>
self _primitiveFailed: #_unpublish:on: args: { server . machine }
%

category: 'Global Session Statistics'
classmethod: System
_updateGlobalSessionStat: index by: i overwrite: aBool

"Global session statistics are similar to session statistics except that global
 statistics are accessible by every logged in session on every host.  Global
 sessions statistics are not transactional.  Therefore every session will see the
 same value in a given statistic regardless of its transactional view.  Reads and
 updates to these statistics are guaranteed to be atomic.

 This method updates the global session statistic at the specified index (which must be
 in the range 0 to 47) by the specified value i, which must be a SmallInteger.
 If overwrite is true, then the current value of the  statistic is replaced by i.
 If overwrite is false, then the current value of the  statistic is incremented by i.

 Returns the new value of the statistic."

<primitive: 518>
index _validateClass: SmallInteger.
i _validateClass: SmallInteger.
aBool _validateClass: Boolean.
self _primitiveFailed: #_updateGlobalSessionStat:by:overwrite:
     args: { index . i . aBool } .
%

category: 'Persistent Counters'
classmethod: System
_updateSharedCounterAt: index by: amount withOpCode: code

"Persistent shared counters provide a means for multiple sessions
 to share common integer values.  There are 1536 persistent shared
 counters, numbered from 1 to 1536 (the index of the first counter is 1).

 Each update to a persistent shared counter causes a roundtrip
 to the stone process.  However reading the value of a counter is
 handled by the gem (and its page server, if any) and does not
 cause a roundtrip to the stone.

 Persistent shared counters are globally visible to all sessions on all
 shared page caches.

 Persistent shared counters hold 64 bit values and may be set to any
 signed 64 bit integer value.  No limit checks are done when
 incrementing or decrementing a counter.  Attempts to increment/decrement
 the counter above/below the minimum/maximum value of a signed
 64-bit integer will cause the counter to 'roll over'.

 Persistent shared counters are independent of database transactions.
 Updates to counters are visible immediately and aborts have no
 effect on them.

 The values of all persistent shared counters are written to the
 primary database extent at checkpoint time.  Updates between
 checkpoints are written to the transaction log by the stone.
 Therefore the state of the persistent shared counters is
 recoverable after a crash, restore from backup, and restore from
 transaction logs.

 Persistent shared counter performance is affected by the stone
 configuration option STN_COMMITS_ASYNC.  Setting this option
 to TRUE will result in faster update performance because
 the stone will respond to update requests after the tranlog
 write has been queued but before it completes.  Operating
 in this mode leaves a small chance of losing data should the
 stone or host machine crash after the tranlog write was queued
 but before it completes.  If this value is set to FALSE, the
 stone will only respond to update requests after the tranlog
 write has completed."

<primitive: 669>
| sz |
index _validateClass: SmallInteger .
amount _validateClass: Integer .
code _validateClass: SmallInteger .
sz := self numberOfPersistentSharedCounters .
(index < 1 or:[ index > sz ])
  ifTrue:[ ^index _error: #rtErrArgOutOfRange args:{ 1 . sz }] .
(code < 0 or:[ code > 1])
  ifTrue:[ ^code _error: #rtErrArgOutOfRange args: { 0 . 1 }] .
(amount < -9223372036854775808 or:[amount > 9223372036854775807])
  ifTrue:[ ^amount _error: #rtErrArgOutOfRange args:{ -9223372036854775808 . 9223372036854775807 }] .
^self _primitiveFailed: #_updateSharedCounterAt:by:withOpCode:
      args: { index . amount . code }
%

category: 'Private'
classmethod: System
_updateSignalErrorStatus: aSignal toState: newState

"This method changes the generation of the error for the specified signal
 generation is disabled.  Signals are specified by SmallIntegers, and the
 following are defined:

 1.  SignaledObjects
 2.  SignaledAbort       (delayed effect if currently in transaction)
 3.  SignaledGemStoneSession
 4.  SignaledFinishTransaction
 5.  AlmostOutOfMemory (a Notification) (disable/enable/enable and set threshold)
 6   SignalTranlogsFull
 7.  AlmostOutOfMemory threshold (read/set independent of enabling)
 8   both SignaledAbort and SignaledFinishTransaction 
 9   AlmostOutOfMemoryError  ((disable/enable/enable and set threshold)
 10  AlmostOutOfStackError
"
<primitive: 367>
self _primitiveFailed: #_updateSignalErrorStatus:toState:
     args: { aSignal . newState } .
self _uncontinuableError
%

category: 'Host System Access'
classmethod: System
_validatePasswordForUser: uid password: pw

<primitive: 864>
uid _validateClass: String .
pw _validateClass: String .
^ self _primitiveFailed: #validatePasswordForUser:password: args: { uid . pw }
%

category: 'Private'
classmethod: System
_validateTransaction

" Determine whether the current transaction could commit.

  See   _primitiveCommit:  for documentation of the return values.

  If the result is > 0 , the current transaction cannot commit and
  the transaction conflict hidden sets have
  been updated to reflect the conflicts preventing commit .  "


^ self _zeroArgPrim: 32
%

category: 'Version Management'
classmethod: System
_version

"Returns a String describing the versions of the pieces of the running GemStone
 system."

| result |
result := String new .
result addAll: 'GEMSTONE: ' ; addAll: self _gemVersion ;
	addAll: '; IMAGE: ' ; addAll: self _imageVersion .
^ result
%

category: 'Private'
classmethod: System
_versionParameterDict

"Returns a Dictionary of names for version information parameters.  The
 dictionary keys are Strings.  Its values are SmallInteger parameter IDs."

"gemstone64 changed  result keys from Symbols to Strings, to reduce creation
  of new Symbols at runtime. "

| result cfgId cfgName |
result := StringKeyValueDictionary new .
cfgId := 0 .
[
  cfgName := self _configParameterName: cfgId kind: $V .
  cfgName == nil ifFalse:[
    result at: cfgName put: cfgId .
    cfgId := cfgId + 1 .
    ] .
  cfgName == nil
  ] untilTrue .

^ result
%

category: 'Private GC'
classmethod: System
_vmInstanceCounts: anInt
"
 Report on object memory usage by instances of each in-memory class.
 (anInt bitAnd: 16r10)  if 1 returns a String,
                        if 0 prints to gem stdout or topaz -l  output file
 (anInt bitAnd: 16r20)  if 1 report is in CSV format, 0 is legacy format.

 (anInt bitAnd:7) = 0   all of object memory
           1   new generation (young temporary objects)
           2   pom generation (copies of committed objects)
           3   old generation (old temporary objects)
           4   perm generation (loaded classes)
           5   code generation (loaded methods)
"
<primitive: 153>
| arg |
anInt _validateClass: SmallInteger.
arg := anInt bitAnd: 16rF .
(arg < 0 or:[ arg > 5 ])
  ifTrue:[ anInt _error: #rtErrArgOutOfRange args:{ 0 . 5 } ].
self _primitiveFailed: #_vmInstanceCounts: args: { anInt }
%

category: 'Private GC'
classmethod: System
_vmInstanceCountsArray: anInt

"Returns an Array of Associations. Each Association
 is of the form  Class->{ instanceCount . byteSize } .
 Returns nil if argument is invalid.
 The argument specifies a portion of object memory to
 include in the results
   anInt = 0   all of object memory
           1   new generation (young temporary objects)
           2   pom generation (copies of committed objects)
           3   old generation (old temporary objects)
           4   perm generation (loaded classes)
           5   code generation (loaded methods)"
<primitive: 652>
anInt _validateClass: SmallInteger.
(anInt < 0 or:[ anInt > 5 ])
  ifTrue:[ anInt _error: #rtErrArgOutOfRange args:{ 1 . 4 } ].
self _primitiveFailed: #_vmInstanceCountsArray: args: { anInt }
%

category: 'Private GC'
classmethod: System
_vmInstanceCountsReport: anInt

"Returns a String which is
 a report about how many instances of each in-memory class
 are present in the specified portion of temporary object memory.

   (anInt bitAnd: 16rF)
           0   all of object memory
           1   new generation (young temporary objects)
           2   pom generation (copies of committed objects)
           3   old generation (old temporary objects)
           4   perm generation (loaded classes)
           5   code generation (loaded methods)

  (anInt bitAnd: 16r20)  if 1 report is in CSV format, 0 is legacy format.

   If covering all of object memory , also reports those classes
   which have zero instances in memory , otherwise classes with
   zero instances are not printed.  The report contains one
   line per class, with className, number of instances, and total
   bytes occupied by the instances.
"
^ self _vmInstanceCounts: (anInt bitOr: 16r10)
%

category: 'Private GC'
classmethod: System
_vmMarkSweep

"Explicitly triggers in-memory markSweep collection of temporary objects.
 Should only be used when debugging internal problems. Frequent use
 can degrade performance."

^ self _zeroArgPrim: 29
%

category: 'Private GC'
classmethod: System
_vmPrintInstanceCounts: anInt

"Print to VM's stdout or  topaz -l  output file
 a report about how many instances of each in-memory class
 are present in the specified portion of temporary object memory.

 The argument specifies the portion of temporary object memory
 to include in the report.
   (anInt bitAnd 16rF)
           0   all of object memory
           1   new generation (young temporary objects)
           2   pom generation (copies of committed objects)
           3   old generation (old temporary objects)
           4   perm generation (loaded classes)
           5   code generation (loaded methods)

   (anInt bitAnd: 16r20)  if 1 report is in CSV format, 0 is legacy format.

   If printing all of object memory , also prints those classes
   which have zero instances in memory , otherwise classes with
   zero instances are not printed.  The printout contains one
   line per class, with className, number of instances, and total
   bytes occupied by the instances.
"

^ self _vmInstanceCounts: (anInt bitAnd: 16r2F)
%

category: 'Private'
classmethod: System
_vmStEntryCount
  "returns a SmallInteger,  omPtr->currStack->stEntryCount_() "
  ^ self _zeroArgPrim: 196
%

category: 'Private'
classmethod: System
_writeLockWriteSubset

"Start with the object manager dirty list ,
 and perform filtering of objects already locked, putting the results
 in a new hidden set (writeLockWriteSubset).  Next, attempt to acquire a
 write lock on the objects in the hidden set.  If all locks are acquired
 cleanly, return true.  If all locks acquired but some are dirty, return
 an array of dirty locked objects.  If not all locks could be acquired,
 return false (locks that were acquired, including dirty locks are available
 in the writeLockWriteSubset hidden set.  Any locks that are acquired are
 also put in the CommitOrAbortReleaseLocksSet."

^self _zeroArgPrim: 34
%

category: 'Private'
classmethod: System
_writtenObjects

"Returns an Array which is the result of enumerating
 the dirty list of the outer level transaction."

^ self _dirtyListOp: 2 id: 0
%

category: 'Private'
classmethod: System
_zeroArgCateg2Prim: opcode

"Private."

"opcode 0 = activeRepositories
        1 = _gemBuildSha
        2 = _gemVersionNum
        3 = _gemVersion
        4 = _hostAgentVersionString
        5 = (not used)
        6 = clusterAllSymbols
        7 = userActionReport - returns an Array of SymbolAssociations
        8 = configConstantsArray - returns an Array of Assocations
        9 = systemUserActionReport
       10 = _committedDataPages
       11 = _uncommittedDataPages"

<primitive: 325>

self _primitiveFailed: #_zeroArgCateg2Prim: args: { opcode } .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_zeroArgPrim: opcode

"Private."

"opcode 0 = removeLocksForSession
        1 = stackLimit
        2 = pageReads
        3 = pageWrites
        4 = transactionMode   (also sent by GsCurrentSession)
        5 = transactionLevel   (also sent by GsCurrentSession)
        6 = maxSessionId
        7 = myCacheProcessSlot
        8 = shutDown
        9 = continueTransaction
       10 = _readClock   (get CPU time used)
       11 = stoneName
       12 = _timeMsLegacy
       13 = _timeGmtFloat
       14 = _generationScavenge
       15 = _objectTableCleanup
       16 = _hostWaitForDebugger  HostExplicitWaitForDebugger
       17 = voteState  (gets GC voteState from stone)
       18 = startAllGcSessions
       19 = voteStateString
       20 = clientIsRemote
       21 = sessionsReferencingOldestCr
       22 = _disallowSubsequentCommits  (Rc failure)
       23 = _approxOopHighWaterMark
       24 = _sharedAttached (result always zero)
       25 = _hostCallDebugger   HostCallDebugger
       26 = stackDepth
       27 = stackDepthHighwater
       28 = writtenObjects
       29 = _vmMarkSweep
       30 = _commitResult
       31 = _deadNotReclaimed
       32 = _validateTransaction
       33 = session
       34 = _writeLockWriteSubset
       35 = _removeLockWriteSubset
       36 = _approxOopNumberHighWaterMark
       37 = _disallowCommitClassModFailure
       38 = clearEpochGcState
       39 = _clearDeadObjects
       40 = stopAllGcGems
       41 = forceEpochGc
       42 = stopAdminGcGem
       43 = _stoneClearCommitQueue
       44 = stopSymbolCreationSession,
       45 = startSymbolCreationSession,
       46 = symbolCreationSessionId,
       47 = _lastGsErrorNumber
       48 = currentUserSessionCount
       49 = currentSessionCount
       50 = Repository pageSize
       51 = _enteringIncompleteIndexRemoval (controls assertions in C, returns previous state)
       52 = _leavingIncompleteIndexRemoval (controls assertions in C, returns previous state)
       53 = getHighAllocatedOopNum
       54 = _numPersistentObjsModified
       55 = _numPagesInSharedOt
       56 = _objectsInNscEnumCache
       57 = _cpuClockTicksPerSecond
       58 = _dumpFrameData
       59 = myPageServerProcessId
       60 = resumeCheckpoints
       61 = checkpointStatus
       62 = startCheckpointSync
       63 = startCheckpointAsync
       64 = adminGemSessionId
       65 = reclaimGemSessionCount
       66 = reclaimGemSessionId
       67 = startAdminGem
       68 = _stonePgsvrListeningPort
       69 = _midCachePgsvrListeningPort
       70 = _currentUserSessionCountIncludingLogout
       71 = _breakSerialization
       72 = _gemCommitConflictDetails
       73 = cacheStatusCount from stone
       74 = _remoteSharedCacheCount
       75 = _enableTraceNewPomObjs
       76 = _disableTraceNewPomObjs
       77 = _tempObjSpaceUsed
       78 = _tempObjSpaceMax
       79 = _tempObjSpacePercentUsed
       80 = failOverStatus
       81 = _getSessionMethodsChangedForCommit
       82 = _getSessionMethodsChangedForAbort
       83 = _currentSessionsIncludingLogout
       84 = _inProtectedMode
       85 = currentSessions
       86 = _tempObjSpacePercentUsedLastMark
       87 = _comPrintOpcodesEmitted
       88 = _comPrintOpcodesNotEmitted
       89 = _printCHeap
       90 = _locale
       91 = _maxClusterId
       92 = _comClearOpcodesEmitted
       93 = _oopHighWaterMark
       94 = _oopNumberHighWaterMark
       95 = timeNs
       96 = _updateObjectsRead
       97 = _disableTraceObjectsRead
       98 = _resumeTraceObjectsRead
       99 = _enableTraceObjectsRead
      100 = _numSharedCounters
      101 = _objectTableSizeInMegabytes
      102 = _setPrintStackAtError
      103 = _timeUs
      104 = logout
      105 = inContinueTransaction
      106 = gemIsBigEndian
      107 = stoneIsBigEndian
      108 = latestCommitRecordPageId
      109 = sessionsReferencingOldestCrInTransaction
      110 = sessionsReferencingOldestCrNotInTransaction
      111 = myReadPageServerProcessId
      112 = commitsDisabledUntilLogout
      113 = commitsDisabledUntilAbort
      114 = continuationsEnabled
      115 = _lastReservedOopNumber
      116 = listenForDebugConnection
      117 = fetchSystemStatNames
      118 = fetchSystemStats
      119 = _hostTimes
      120 = disableStoneGemTimeout
      121 = enableStoneGemTimeout
      122 = millisecondsSinceTransactionBoundary
      123 = millisecondsSinceLogin
      124 = secondsSinceStoneStarted
      125 = _enableTracePagesAccessed
      126 = _resumeTracePagesAccessed
      127 = _updatePagesAccessed
      128 = _disableTracePagesAccessed
      129 = refreshTransientSymbolList
      130 = hostCpuUsage
      131 = hostId
      132 = fast   System gemConfigurationAt:#GemConvertArrayBuilder
      133 = stoneMessageKinds
      134 = commitsSinceCurrentView
      135 = hostCpuCount
      136 = sessionIdHoldingGcLock
      137 = sessionIsOnStoneHost
      138 = cacheStatisticsForAllShort
      139 = cacheStatisticsForAll
      140 = failOverToSlave
      141 = commitRestore
      142 = restoreStatusInfo
      143 = _restoreDestroyOt
      144 = commitRestoreWithLostData
      145 = reclaimedSymbols
      146 = begin nested transaction
      147 = commit nested transaction
      148 = abort nested transaction
      149 = stoneStartupId
      150 = gemLogFileName
      151 = get  logsender sessionId or logreceiver negated sessionId
      152 = linkedGciLibraryName64
      153 = rpcGciLibraryName64
      154 = numberOfPersistentSharedCounters
      155 = suspendCommits
      156 = resumeCommits
      157 = profMonSample
      158 = gemProcessId
      159 = gemLogPath
      160 = commandLineArguments
      161 = terminateAllSessionsReferencingOldestCr
      162 = stopAllSessionsReferencingOldestCr
      163 = hostname
      164 = oldestCommitRecordAgeMs
      165 = allZombieSessions
      166 = allZombieSessionsRefOldestCr
      167 = realUserId
      168 = realUserIdName
      169 = effectiveUserId
      170 = effectiveUserIdName
      171 = loginUserId
      172 = loginUserIdName
      173 = startNewLoginLog
      174 = loginLogFileName
      175 = sessionIsRpc
      176 = pageServerConnectionIsEncrypted
      177 = gciClientConnectionIsEncrypted
      178 = readClockNano
      179 = refPathCleanup
      180 = myKerberosPrincipal
      181 = getKeyfileAttributes
      182 = fullyQualifiedDomainName
      183 = _stoneCommandsReport
      184 = _maxPrimitiveNumber
      185 = repositoryPublicKey
      186 = repositoryPrivateKey
      187 = _sharedCacheAttributes
      188 = _timeGmtMicroSeconds
      190 = _objectReadLogInfo
      191 = startNewObjectReadLog
      192 = flushObjectReadBuffer
      193 = objectReadTrackingEnabled
      194 = _sessionTrackReads
      196 = _vmStEntryCount

     1020 is  HostExplicitWaitForDebugger
     1036 is  freezeCache, must be SystemUser
     1037 is  RDbfTestPgsvrSegv , must be SystemUser
     1038 = stoneCommandLineArgs
     1039 = extentEncrpytionScheme
     4001 = cacheStatisticsDescriptionForMonitor
     4002 = cacheStatisticsDescriptionForStone
     4004 = cacheStatisticsDescriptionForPageServer
     4008 = cacheStatisticsDescriptionForGem
"

<primitive: 98>

self _primitiveFailed: #_zeroArgPrim: args: { opcode } .
self _uncontinuableError
%

category: 'Transient Session State'
classmethod: System
__sessionStateAt: anIndex
"
 Returns an indexed instance variable of the Transient Session State object.
 This object is a temporary instance of Array created after each GciLogin.

 If the specified instance variable of the Session State has not been stored
 into during this session, the result is nil.  This is different from
 Array>>at: semantics, where accessing beyond the end of an Array
 generates an error.

 Users of this method must observe the following instance variable allocations
 for this Array.

    anIndex      assignedUse
    -------      -----------
    1            Reserved for SymbolDictionary of Temporaries
    2            Reserved for ephemerons finalization queue .
    3            Topaz session state
    4            Reduced Conflict classes logging info
    5            Cache for Reduced Conflict values
    6            The one instance of GsCurrentSession
    7            Pending commit/abort errorNumber or userAction
    8            Reserved for GemStone GBJ support
    9            Other known sessions (a dictionary keyed on sessionSerialNum)
    10           Reserved for GemStone relational support
    11           Reserved for GemStone IndexManager support
    12           GemBuilder for C (GCI) deferred update Dictionary.
    13           GemBuilder for C (GCI) deferred update Array.
    14           Commit coordinator.
    15           Reserved for GemStone ProcessorScheduler support and prim 457
    16           Reserved for GemStone Locale support.
    17           Reserved for timezone support.
    18           Reserved for GemStone GsFile support, caching of std files.
    19           Last GsComMethNode produced by primitive 228.
    20           IcuCollator for the current default IcuLocale.
    21           a Boolean, IcuCollator.caseInsensitive
    22           Boolean controlling Notification >> defaultAction .
    23           nil or a Boolean,  usingUnicodeCompares .
    24           nil or an Array, detailed transaction conflicts of last failed commit
    25           Reserved for GemConnect for Postgres
    26..39       reserved for GemStone internal use.
    40..         Available for customer use . Customer range changed in v3.1"

<primitive: 111>

(anIndex _isSmallInteger)
  ifTrue: [self _errorIndexOutOfRange: anIndex .  ^ nil ]
  ifFalse: [self _errorNonIntegerIndex: anIndex .  ^ nil ].
self _primitiveFailed: #__sessionStateAt: args: { anIndex } .
self _uncontinuableError
%

category: 'Transient Session State'
classmethod: System
__sessionStateAt: anIndex put: aValue

"Modifies an indexed instance variable of the Transient Session State Array.
 The Session State Array will be grown as required to accommodate a store
 into the specified instance variable.

 anIndex must be < approx 2034 (Session State Array may not be a large object).

 See additional important documentation in the method #__sessionStateAt: .
 Returns receiver."

<primitive: 372>
(anIndex _isSmallInteger)
  ifTrue: [self _errorIndexOutOfRange: anIndex]
  ifFalse: [self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #__sessionStateAt:put: args: { anIndex . aValue } .
self _uncontinuableError
%

category: 'Session Control'
classmethod: System
__sleepMs: milliSecondsToSleep

"Sleep for the number of milliseconds specified by milliSecondsToSleep.
 The argument milliSecondsToSleep must be a positive SmallInteger.
 If milliSecondsToSleep is zero, this method has no effect.
 Returns time left to sleep , which will be > 0 if interrupted.
 The ProcessorScheduler is not used, and other waiting GsProcesses
 will not execute during the sleep. Use a Delay or Semaphore to wait
 while allowing other GsProcesses to run."

<primitive: 344>
milliSecondsToSleep _validateClass: SmallInteger.
milliSecondsToSleep < 0
   ifTrue: [ milliSecondsToSleep _error: #errArgTooSmall args:{ 0 } ] .
^ self _primitiveFailed: #_sleepMs: args: { milliSecondsToSleep }
%

! Class extensions for 'SystemCallError'

!		Instance methods for 'SystemCallError'

removeallmethods SystemCallError
removeallclassmethods SystemCallError

category: 'Instance initialization'
method: SystemCallError
args: anArray
  anArray _isArray ifTrue:[ | sz |
    errno := anArray atOrNil: 1 .
    (sz := anArray size) > 1 ifTrue:[
      gsArgs := anArray copyFrom:2 to: sz
    ].
  ]
%

category: 'Formatting'
method: SystemCallError
buildMessageText
^ self buildMessageText:(
  [ | str |
    errno ifNotNil:[:e| 
      (str := String withAll:'errno=') add: errno asString;
         add:' '; add: (AbstractException errnoToName: e) .
    ].
    gsDetails ifNotNil:[:d| str add:', '; add: d  asString ].
    GsFile gciLogServer:' SystemCallError >> asString : ', str asString .
    str
  ] onException: Error do:[:ex |
    ex return: nil
  ]
)
%

category: 'Instance initialization'
method: SystemCallError
errno: anInt
  errno := anInt
%

category: 'Instance initialization'
method: SystemCallError
initialize
  gsNumber := ERR_SystemCallError.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Instance initialization'
method: SystemCallError
_legacyHandlerArgs
  | arr |
  (arr := { errno } ) addAll: super _legacyHandlerArgs .
  ^ arr
%

! Class extensions for 'TerminateProcess'

!		Instance methods for 'TerminateProcess'

removeallmethods TerminateProcess
removeallclassmethods TerminateProcess

category: 'Handling'
method: TerminateProcess
defaultAction

"Return an error to the controlling GCI client .
 See AbstractException>>defaultAction for documentation."

<primitive: 33>
self _uncontinuableError
%

category: 'Instance initialization'
method: TerminateProcess
initialize
  gsNumber := ERR_TerminateProcess.
  gsResumable := false .
  gsTrappable := false .
%

! Class extensions for 'TestFailure'

!		Instance methods for 'TestFailure'

removeallmethods TestFailure
removeallclassmethods TestFailure

category: 'Instance initialization'
method: TestFailure
initialize
  gsNumber := ERR_TestFailure .
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'ThreadError'

!		Instance methods for 'ThreadError'

removeallmethods ThreadError
removeallclassmethods ThreadError

category: 'Instance initialization'
method: ThreadError
initialize
  gsNumber := ERR_ThreadError.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Instance initialization'
method: ThreadError
object: anObject
  gsArgs := { anObject }
%

! Class extensions for 'TransactionBacklog'

!		Class methods for 'TransactionBacklog'

removeallmethods TransactionBacklog
removeallclassmethods TransactionBacklog

category: 'Signalling'
classmethod: TransactionBacklog
disableSignalling

"disable both SignaledAbort and SignaledFinishTransaction"
System _updateSignalErrorStatus: 8 toState: false .

%

category: 'Signalling'
classmethod: TransactionBacklog
enableSignalling

"Enables a TransactionBacklog Notification if this session
 should commit or abort.  If you want to only get the should commit or
 should abort Notification ,
 use System class>>enableSignaledFinishTransactionError
 or System class>>enableSignaledAbortError.

 If sent within a handlerBlock which is handling an instance of
 TransactionBacklog, the enable takes effect when execution
 leaves the handlerBlock via a return or resume .
"

"enable both SignaledAbort and SignaledFinishTransaction"
System _updateSignalErrorStatus: 8 toState: true .
%

category: 'Signalling'
classmethod: TransactionBacklog
signallingEnabled

"Return true if signaling enabled, return false otherwise.
 See TransactionBacklog(C)>>enableSignalling "

^ System _signalErrorStatus: 4
%

!		Instance methods for 'TransactionBacklog'

category: 'Instance initialization'
method: TransactionBacklog
args: anArray
  anArray _isArray ifTrue:[ | sz |
    inTransaction := anArray atOrNil: 1 .
    (sz := anArray size) > 1 ifTrue:[
      gsArgs := anArray copyFrom: 2 to: sz
    ].
  ]
%

category: 'Formatting'
method: TransactionBacklog
buildMessageText
^ self buildMessageText:(
  [ | str |
    str := 'inTransaction=', inTransaction asString .
    gsDetails ifNotNil:[:d| str add:', '; add: d  asString ].
    str
  ] onException: Error do:[:ex |
    ex return: nil
  ]
)
%

category: 'Handling'
method: TransactionBacklog
defaultAction

"Return an error to the controlling GCI client .
 See AbstractException>>defaultAction for documentation."

<primitive: 33>
self _uncontinuableError
%

category: 'Instance initialization'
method: TransactionBacklog
initialize
  gsNumber := ERR_TransactionBacklog.
  gsResumable := true .
  gsTrappable := true .
%

category: 'Accessing'
method: TransactionBacklog
inTransaction
  ^ inTransaction  "aBoolean"
%

category: 'Instance initialization'
method: TransactionBacklog
_legacyHandlerArgs
  | arr |
  (arr := { inTransaction } ) addAll: super _legacyHandlerArgs .
  ^ arr
%

! Class extensions for 'TransactionError'

!		Instance methods for 'TransactionError'

removeallmethods TransactionError
removeallclassmethods TransactionError

category: 'Formatting'
method: TransactionError
asString
  gsNumber == 2407 ifTrue:[
    gsArgs ifNotNil:[ | obj txt str |
      obj := gsArgs atOrNil: 1 .
      txt := (gsArgs atOrNil: 2) ifNil:[ ' cannot be committed'].
      (str := 'a' copy) add: (obj class name);
          add: ' (oop ' ; add: obj asOop asString; add: ') ' ;
          add: txt .
      ^ str
    ].
  ].
  ^ super asString "default computed in superclass"
%

category: 'Instance initialization'
method: TransactionError
initialize
  gsNumber := ERR_TransactionError.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'UncontinuableError'

!		Instance methods for 'UncontinuableError'

removeallmethods UncontinuableError
removeallclassmethods UncontinuableError

category: 'Instance initialization'
method: UncontinuableError
initialize
  gsNumber := ERR_UncontinuableError.
  gsResumable := false .
  gsTrappable := true .
%

! Class extensions for 'UndefinedObject'

!		Class methods for 'UndefinedObject'

removeallmethods UndefinedObject
removeallclassmethods UndefinedObject

category: 'Instance Creation'
classmethod: UndefinedObject
fromStream: aStream

"If the next Characters in aStream are nil (case-insensitive), returns nil.
 Otherwise, generates an error."

| ch |

self _checkReadStream: aStream forClass: CharacterCollection.

ch := aStream next.
[ ch == $  ]
whileTrue:
  [ ch := aStream next ].
aStream skip: -1.

(((aStream next isEquivalent: $N)
 and: [aStream next isEquivalent: $I ])
 and: [aStream next isEquivalent: $L ])
ifTrue:
  [ ^ nil ]
ifFalse:
  [ self _errIncorrectFormat: aStream ]
%

category: 'Instance Creation'
classmethod: UndefinedObject
fromString: aString

"If the first three Characters in aString are nil (case-insensitive), returns
 nil.  Otherwise, generates an error."

| s result |

s := ReadStream on: aString.
result := self fromStream: s.
[ s atEnd ]
whileFalse:
  [ (s next == $  )
    ifFalse:
      [ self _errIncorrectFormat: aString ]
  ].
^ result
%

category: 'Storing and Loading'
classmethod: UndefinedObject
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into a new instance of the receiver.
 Returns the new instance."

"This method supports activation of passive objects written by a GeODE
 version earlier than 2.0.1."

passiveObj hasRead: nil.
^nil
%

category: 'Instance Creation'
classmethod: UndefinedObject
new

"Disallowed.  You cannot create new instances of UndefinedObject."

self shouldNotImplement: #new
%

!		Instance methods for 'UndefinedObject'

category: 'Formatting'
method: UndefinedObject
asString

"Returns the string 'nil'."

self == nil ifTrue:[ ^ String withAll:'nil' ].
"#_remoteNil  is a key in Globals"
self == _remoteNil ifTrue:[ ^ String withAll:'remoteNil' ].

"following are normally only seen by stack printing code in GsProcess.
 and are defined as class variables in UndefinedObject."
self == _exceptionMarkNil ifTrue:[ ^ String withAll:'_exceptionMarkNil' ].
self == _gsExcHandlerMarkNil ifTrue:[ ^ String withAll:'_gsExcHandlerMarkNil' ].
self == _ensureMarkNil ifTrue:[ ^ String withAll:'_ensureMarkNil' ].
self == _rfhMarkNil ifTrue:[ ^ String withAll:'_rfhMarkNil' ].
self == _rfhGsExcHandlerMarkNil ifTrue:[ ^ String withAll:'_rfhGsExcHandlerMarkNil' ].
self == _tosIsIpNil ifTrue:[ ^ String withAll:'_tosIsIpNil' ].
self asOop == 2324 ifTrue:[ ^ String withAll:'_noResultNil' ].
self == _executeHandlerMarkNil ifTrue:[ ^ String withAll:'_executeHandlerMarkNil' ].
self == _reenterMarkNil ifTrue:[ ^ String withAll:'_reenterMarkNil' ].
self == _rubyNoArgNil ifTrue:[ ^ String withAll:'_rubyNoArgNil' ].
self == _syncExceptionMarkNil ifTrue:[ ^ String withAll:'_syncExceptionMarkNil'].

  "following instances should not be visible to application code
   unless an internal error occurs during VM execution."
self == _rubyEvalRfhMarkNil ifTrue:[ ^ String withAll:'_rubyEvalRfhMarkNil'].
self == _rubyRExecHandlerMarkNil ifTrue:[ ^ String withAll:'_rubyRExecHandlerMarkNil'].
self == _rubyFrameLocalsNil ifTrue:[ ^ String withAll:'_rubyFrameLocalsNil' ].
self == _partialContinuaLoadMarkNil ifTrue:[ ^ String withAll:'_partialContinuaLoadMarkNil' ].
^ 'unrecognized instance of UndefinedObject, oop ' , self asOop asString .
%

category: 'Clustering'
method: UndefinedObject
clusterDepthFirst

"Returns true.  (Because nil is a self-defining object, this method has no
 effect.)"

^ true
%

category: 'Storing and Loading'
method: UndefinedObject
containsIdentity

"Private."

^true
%

category: 'Copying'
method: UndefinedObject
copy

"Returns the receiver.  The pseudovariable nil is the only instance of
 UndefinedObject, and must preserve identity."

^self
%

category: 'Formatting'
method: UndefinedObject
describe
 "when printing stack reports avoid putting an exception handler on the
  stack for printing a nil."
^ self asString
%

category: 'Formatting'
method: UndefinedObject
describeClassName

"Returns a String to describe classes with nil names."

^ 'unnamed'
%

category: 'Indexing Support'
method: UndefinedObject
getDepListAndAddPathTerm: pathTerm withIVOffset: ivOffset logging: aBoolean

"Private."

^ self
%

category: 'Testing'
method: UndefinedObject
isSpecial

"Returns true if the receiver is a special object."

^ true
%

category: 'Json'
method: UndefinedObject
printJsonOn: aStream

	aStream nextPutAll: 'null'.
%

category: 'Formatting'
method: UndefinedObject
printString
 "when printing stack reports avoid putting an exception handler on the
  stack for printing a nil."
^ self asString
%

category: 'Deprecated'
method: UndefinedObject
subclass: aString
instVarNames: anArrayOfStrings
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
constraints: aConstraint
instancesInvariant: invarBoolean
isModifiable: modifyBoolean

	"Private.  Creates and returns a new subclass of the receiver.

 For further documentation see the same selector in Class."

	| theClass superClassOffset oldClass opts |
	self
		deprecated: 'UndefinedObject>>subclass:instVarNames:classVars:...constraints:... deprecated before v3.0; constraints no longer supported'.
	oldClass := Object _classNamed: aString inDictionary: aDictionary.
	opts := {}.
	invarBoolean
		ifTrue: [opts add: #instancesInvariant]
		ifFalse:
			[Object instancesInvariant
				ifTrue: [^self _error: #classErrInvariantSuperClass]].
	modifyBoolean ifTrue: [opts add: #modifiable].
	oldClass
		ifNotNil:
			[(Object
				_equivalentSubclass: oldClass
				superCls: self
				name: aString
				newOpts: opts
				newFormat: oldClass format
				newInstVars: anArrayOfStrings
				newClassInstVars: anArrayOfClassInstVars
				newPools: anArrayOfPoolDicts
				newClassVars: anArrayOfClassVars
				inDict: aDictionary
				isKernel: false) ifTrue: [^oldClass	"avoid creation of a new version"]].
	theClass := Object
				subclass: aString
				instVarNames: anArrayOfStrings
				classVars: anArrayOfClassVars
				classInstVars: anArrayOfClassInstVars
				poolDictionaries: anArrayOfPoolDicts
				inDictionary: aDictionary
				constraints: aConstraint
				instancesInvariant: invarBoolean
				isModifiable: modifyBoolean.
	superClassOffset := Behavior _ivOffsetOf: #superClass.
  (theClass instVarAt: superClassOffset) ifNotNil:[
	  theClass _unsafeAt: superClassOffset put: nil.
	  theClass class _unsafeAt: superClassOffset put: Object class superClass.
  ].
	^theClass
%

category: 'Subclass Creation'
method: UndefinedObject
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
newVersionOf: oldClass
description: aDescription
options: optionsArray

  | descr theClass superClassOffset |
  descr := aDescription.
  oldClass ifNotNil: [
     (Object _equivalentSubclass: oldClass
        superCls: self
        name: aString
        newOpts: optionsArray
        newFormat: oldClass format
        newInstVars: anArrayOfInstvarNames
        newClassInstVars: anArrayOfClassInstVars
        newPools: anArrayOfPoolDicts
        newClassVars: anArrayOfClassVars
        inDict: aDictionary
        isKernel: false ) ifTrue: [
            oldClass _commentOrDescription: aDescription.
            ^oldClass "avoid creation of a new version"
        ].
      descr ifNil:[ descr := [ oldClass commentForFileout ] on: Error do:[:ex | 'old comment not available']]
  ].
  theClass := Object
        subclass: aString
        instVarNames: anArrayOfInstvarNames
        classVars: anArrayOfClassVars
        classInstVars: anArrayOfClassInstVars
        poolDictionaries: anArrayOfPoolDicts
        inDictionary: aDictionary
        newVersionOf: oldClass
        description: descr
        options: optionsArray.
  theClass == oldClass ifFalse:[
    superClassOffset := Behavior _ivOffsetOf: #superClass.
    (theClass instVarAt: superClassOffset) ifNotNil:[
      theClass _unsafeAt: superClassOffset put: nil.
      theClass class _unsafeAt: superClassOffset put: Object class superClass
    ].
  ].
  ^theClass
%

category: 'Subclass Creation'
method: UndefinedObject
subclass: aString
instVarNames: anArrayOfInstvarNames
classVars: anArrayOfClassVars
classInstVars: anArrayOfClassInstVars
poolDictionaries: anArrayOfPoolDicts
inDictionary: aDictionary
options: optionsArray

^ self
  subclass: aString
  instVarNames: anArrayOfInstvarNames
  classVars: anArrayOfClassVars
  classInstVars: anArrayOfClassInstVars
  poolDictionaries: anArrayOfPoolDicts
  inDictionary: aDictionary
  newVersionOf: (self class _classNamed: aString inDictionary: aDictionary)
  description: nil
  options: optionsArray
%

category: 'Deprecated'
method: UndefinedObject
subclass: aString
instVarNames: anArrayOfStrings
inDictionary: aDictionary
constraints: constraintSpec

"Private.  Creates and returns a new subclass of the receiver.
 For further documentation see the same selector in Class."

self deprecated: 'UndefinedObject>>subclass:instVarNames:inDictionary:constraints: deprecated before v3.0; constraints no longer supported'.
^ self subclass: aString
    instVarNames: anArrayOfStrings
    classVars: #()
    classInstVars: #()
    poolDictionaries: #()
    inDictionary: aDictionary
    constraints: constraintSpec
    instancesInvariant: false
    isModifiable: false
%

category: 'Storing and Loading'
method: UndefinedObject
writeTo: passiveObj

"Converts the receiver to its passive form and writes that information on
 passiveObj."

| c |
self == _remoteNil ifTrue:[ c := $# ] ifFalse:[ c := $. ].
passiveObj nextPut: c
%

category: 'Decompiling without Sources'
method: UndefinedObject
_asSource

""

^ self asString
%

category: 'New Indexing Comparison'
method: UndefinedObject
_classSortOrdinal

^ 0
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForCompareEqualTo: aCharacterCollection collator: collator
  ""

  ^ self _idxForCompareEqualTo: aCharacterCollection
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForCompareGreaterThan: arg

""

^arg _idxForCompareUndefinedObjectGreaterThanSelf: self
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForCompareGreaterThan: aCharacterCollection collator: collator
  ""

  ^ self _idxForCompareGreaterThan: aCharacterCollection
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForCompareGreaterThanOrEqualTo: arg

""

^arg _idxForCompareUndefinedObjectGreaterThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForCompareGreaterThanOrEqualToUnicode: aUnicodeString collator: anIcuCollator

"second half of a double dispatch call from CharacterCollection>>_idxForCompareGreaterThanOrEqualTo:collator:. Note that aUnicodeString should be the receiver in any >= comparison"

  ^ self _idxForCompareUndefinedObjectGreaterThanOrEqualToSelf: aUnicodeString
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForCompareGreaterThanUnicode: aUnicodeString collator: anIcuCollator

"second half of a double dispatch call from CharacterCollection>>_idxForCompareGreaterThan:collator:. Note that aUnicodeString should be the receiver in any > comparison"

  ^ self _idxForCompareGreaterThan: aUnicodeString
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForCompareLessThan: arg

""

^arg _idxForCompareUndefinedObjectLessThanSelf: self
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForCompareLessThanOrEqualTo: arg

""

^arg _idxForCompareUndefinedObjectLessThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForCompareLessThanOrEqualToUnicode: aUnicodeString collator: anIcuCollator

"second half of a double dispatch call from CharacterCollection>>_idxForCompareLessThanOrEqualTo:collator:. Note that aUnicodeString should be the receiver in any <= comparison"

  ^ self _idxForCompareLessThanOrEqualTo: aUnicodeString
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForCompareLessThanUnicode: aUnicodeString collator: anIcuCollator

"second half of a double dispatch call from CharacterCollection>>_idxForCompareLessThan:collator:. Note that aUnicodeString should be the receiver in any < comparison"

  ^ self _idxForCompareLessThan: aUnicodeString
%

category: 'New Indexing Comparison - for Compare'
method: UndefinedObject
_idxForCompareUndefinedObjectGreaterThanOrEqualToSelf: anUndefinedObject

"second half of a double dispatch call from UndefinedObject>>_idxForCompareGreaterThanOrEqualTo:."

^true
%

category: 'New Indexing Comparison - for Compare'
method: UndefinedObject
_idxForCompareUndefinedObjectGreaterThanSelf: anUndefinedObject

"second half of a double dispatch call from UndefinedObject>>_idxForCompareGreaterThan:."

^false
%

category: 'New Indexing Comparison - for Compare'
method: UndefinedObject
_idxForCompareUndefinedObjectLessThanOrEqualToSelf: anUndefinedObject

"second half of a double dispatch call from UndefinedObject>>_idxForCompareLessThanOrEqualTo:."

^true
%

category: 'New Indexing Comparison - for Compare'
method: UndefinedObject
_idxForCompareUndefinedObjectLessThanSelf: anUndefinedObject

"second half of a double dispatch call from UndefinedObject>>_idxForCompareLessThan:."

^false
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForSortEqualTo: aCharacterCollection collator: collator
  ""

  ^ self _idxForSortEqualTo: aCharacterCollection
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForSortGreaterThan: arg

""

^arg _idxForSortUndefinedObjectGreaterThanSelf: self
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForSortGreaterThanOrEqualTo: arg

""

^arg _idxForSortUndefinedObjectGreaterThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForSortLessThan: arg

""

^arg _idxForSortUndefinedObjectLessThanSelf: self
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForSortLessThan: aCharacterCollection collator: collator
  ""

  ^ self _idxForSortLessThan: aCharacterCollection
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForSortLessThanOrEqualTo: arg

""

^arg _idxForSortUndefinedObjectLessThanOrEqualToSelf: self
%

category: 'New Indexing Comparison'
method: UndefinedObject
_idxForSortLessThanOrEqualTo: aCharacterCollection collator: collator
  ""

  ^ self _idxForSortLessThanOrEqualTo: aCharacterCollection
%

category: 'New Indexing Comparison - for Sort'
method: UndefinedObject
_idxForSortUndefinedObjectGreaterThanOrEqualToSelf: anObject

"second half of a double dispatch call from UndefinedObject>>_idxForSortGreaterThanOrEqualTo:. Note that anUndefinedObject should be the receiver in any >= comparison"

^true
%

category: 'New Indexing Comparison - for Sort'
method: UndefinedObject
_idxForSortUndefinedObjectGreaterThanSelf: anUndefinedObject

"second half of a double dispatch call from UndefinedObject>>_idxForSortGreaterThan:. Note that anUndefinedObject should be the receiver in any > comparison"

^false
%

category: 'New Indexing Comparison - for Sort'
method: UndefinedObject
_idxForSortUndefinedObjectLessThanOrEqualToSelf: anUndefinedObject

"second half of a double dispatch call from UndefinedObject>>_idxForSortLessThanOrEqualTo:. Note that anUndefinedObject should be the receiver in any <= comparison"

^true
%

category: 'New Indexing Comparison - for Sort'
method: UndefinedObject
_idxForSortUndefinedObjectLessThanSelf: anObject

"second half of a double dispatch call from UndefinedObject>>_idxForSortLessThan:. Note that anUndefinedObject should be the receiver in any < comparison"

^false
%

category: 'Filein Support'
method: UndefinedObject
_newKernelSubclass: clsName
instVarNames: ivArg
classVars: classVarArg
classInstVars: classInstVarArg
poolDictionaries: poolDicts
inDictionary: aDict
options: options
reservedOop: reservedOopNum

"Creates a new class only if a class with the given name does not already exist
 in Globals.  Returns the new class, old class or nil."

| result theClass superClassOffset |

result := Object
  _newKernelSubclass: clsName
  subclassOf: self
  instVarNames: ivArg
  classVars: classVarArg
  classInstVars: classInstVarArg
  poolDictionaries: poolDicts
  inDictionary: aDict
  options: options
  reservedOop: reservedOopNum .

result ifNotNil:[ "change superclass to nil"
  theClass := result .
  superClassOffset := (Behavior _ivOffsetOf: #superClass) .
  (theClass instVarAt: superClassOffset) ifNotNil:[
    theClass _unsafeAt: superClassOffset put: nil .
    theClass class _unsafeAt: superClassOffset put: Object class superClass  .
  ].
].
^ result
%

! Class extensions for 'UnorderedCollection'

!		Class methods for 'UnorderedCollection'

removeallmethods UnorderedCollection
removeallclassmethods UnorderedCollection

category: 'Storing and Loading'
classmethod: UnorderedCollection
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into a new instance of the receiver.
 Returns the new instance."

| size inst |
size := passiveObj readSize.
inst := self new.
inst loadFrom: passiveObj size: size.
^inst
%

category: 'Index Cache Statistics'
classmethod: UnorderedCollection
statValueForAuditingBtreeCounts
"Answer the value of the IndexProgressCount cache statistic when the
 btree counts audit phase of the index audit is in progress."
^ 4
%

category: 'Index Cache Statistics'
classmethod: UnorderedCollection
statValueForAuditingIdentityIndexes
"Answer the value of the IndexProgressCount cache statistic when the
 identity index audit phase of the index audit is in progress."
^ 1
%

category: 'Index Cache Statistics'
classmethod: UnorderedCollection
statValueForAuditingNscCounts
"Answer the value of the IndexProgressCount cache statistic when the
 NSC counts audit phase of the index audit is in progress."
^ 3
%

category: 'Index Cache Statistics'
classmethod: UnorderedCollection
statValueForAuditingRootTerms
"Answer the value of the IndexProgressCount cache statistic when the
 root term audit phase of the index audit is in progress."
^ 2
%

category: 'Index Cache Statistics'
classmethod: UnorderedCollection
statValueForRemovingAllIndexes
"Answer the value of the IndexProgressCount cache statistic when the
 IndexManager>>removeAllIndexes method is in progress."
^ 5
%

category: 'Indexing Support'
classmethod: UnorderedCollection
_canCreateQueryOnInstances
  "GsQuery may be created on most subclasses of Collection. Answer false if a GsQuery is not appropriate
   for the receiver."

  ^ true
%

category: 'Private'
classmethod: UnorderedCollection
_disableRcIndexLogging

"System-wide disabling of logging for RC indexes
 (only for the life of the transaction)."

System rcValueCache at: #rcIndexLogging put: false
%

category: 'Private'
classmethod: UnorderedCollection
_enableRcIndexLogging

"System-wide enabling of logging for RC indexes."

System rcValueCache at: #rcIndexLogging put: true
%

category: 'Private'
classmethod: UnorderedCollection
_isRcIndexLoggingEnabled

""

^ System rcValueCacheAt: #rcIndexLogging otherwise: true.
%

!		Instance methods for 'UnorderedCollection'

category: 'Adding'
method: UnorderedCollection
add: anObject withOccurrences: anInteger

"Includes anObject as an element of the receiver anInteger number of times.
 Generates an error if anObject is not a kind of the Bag's element kind."

self subclassResponsibility: #add:withOccurrences:
%

category: 'Indexing Audit'
method: UnorderedCollection
auditIndexes

"Verifies that the index objects are consistent.
 Returns a string that describes any inconsistencies found.

 Since #_fastAuditIndexes provides a significant improvement
 in speed over tthe older (pre 2.1) algorithms, it is recommended
 that this method be used on a regular basis to ensure index structure
 integrity.

 If the audit returns errors, the indexes should be dropped and rebuilt
 and the incident should be reported to Gemstone support for analysis.
"

^self _fastAuditIndexes
%

category: 'Auditing'
method: UnorderedCollection
auditInternalStructures
"See the method auditInternalStructuresWithRepair: aBoolean for
 more information on this method."

^self auditInternalStructuresWithRepair: false
%

category: 'Auditing'
method: UnorderedCollection
auditInternalStructuresWithRepair: aBoolean
"Audit and optionally repair the internal structures of the receiver.
 If aBoolean is true, errors will be fixed as they are detected, but
 the transaction is not automatically committed and a commit after this
 method returns is required.  If aBoolean is false, the audit is
 read-only operation and no objects are modified.

 Answers a Boolean indicating if the collection is free of errors.
 true means the collection passed the audit, false means the audit
 detected one or more errors.

 This primitive prints many messages to stdout and is intended to
 be run from a linked topaz session."

<primitive: 574>
^self _primitiveFailed: #auditInternalStructuresWithRepair: args: { aBoolean }
%

category: 'Storing and Loading'
method: UnorderedCollection
basicLoadFrom: passiveObj size: varyingSize

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into the receiver."

"This method is similar to basicLoadFrom:, but is used for objects whose size
 is not known when they are first instantiated (such as an IdentitySet)."

passiveObj hasRead: self.
^ self basicLoadFromNoRead: passiveObj size: varyingSize .
%

category: 'Storing and Loading'
method: UnorderedCollection
basicLoadFromNoRead: passiveObj size: varyingSize

"Private."

varyingSize == 0 ifTrue: [
  "Old NSC format had no named instance variable section.  A zero-length NSC in
   the old format might read instance variables from an enclosing object if
   there were not a special delimiter that could be reliably found."
  passiveObj checkForBagMark ifFalse: [
    passiveObj checkForInstVarMark ifFalse: [
      ^self
    ].
  ].
].
(passiveObj readNamedIV) ifFalse: [
  "old NSC format with no named instance variables"
  ^self loadVaryingFrom: passiveObj size: varyingSize
].
self loadNamedIVsFrom: passiveObj.
self loadVaryingFrom: passiveObj size: varyingSize.
%

category: 'Storing and Loading'
method: UnorderedCollection
basicWriteTo: passiveObj

"Converts the receiver to its passive form and writes that information on
 passiveObj."

| s cls c |
  "store my structure on the given passiveObj"
  cls := self class.
  passiveObj writeClass: cls.

  passiveObj writeSize: (s := self size) .

  passiveObj writeNamedIvsFrom: self class: cls .
  passiveObj writeBagMark.
  passiveObj endNamedInstVars.

  c := 0.
  self do: [ :x |
    passiveObj writeObject: x.
    c := c + 1.
    c > 99 ifTrue: [
      passiveObj lf.
      c := 0.
    ].
  ].
  passiveObj cr
%

category: 'Clustering'
method: UnorderedCollection
clusterDepthFirst

"Clusters the receiver and its named and unnamed instance variables in
 depth-first order. Returns true if the receiver has already been clustered
 during the current transaction; returns false otherwise."

self clusterIndexes.
^ super clusterDepthFirst
%

category: 'Clustering'
method: UnorderedCollection
clusterIndexes

"Clusters internal indexing objects using the current default ClusterBucket."

self _clusterIndexes
%

category: 'Repository Conversion'
method: UnorderedCollection
convRemoveIndexes

"Private. Removes the indexes for the receiver. This method is to be used
 only during Repository Conversion."

self _clearIndexList.
%

category: 'Parallel Index Creation Support'
method: UnorderedCollection
createParallelIndexes: anArrayOfIndexSpecifications

| indexes |
self _hasIncompleteIndexes ifTrue: [ ^self _error: #rtErrCollectionWithIncompleteIndex ].
self _hasIndexes ifTrue: [ ImproperOperation signal: 'createParallelIndexes: may only be used on a collection with no pre-existing indexes.' ].
[
  indexes := OrderedCollection new.
  IndexManager current executeStartingIndexMaintenance: [
	  anArrayOfIndexSpecifications do: [:spec | | index |
		  (index := self _createParallelIndexOn: spec)
			  ifNotNil: [ indexes add: (index -> index preIndexCreation) ]
	  ].
	  self _putInWriteSet.
	  self do: [:element | | indexResult |
		  indexResult := self _updateIndexesForAdditionOf: element logging: false.
		  indexResult == true ifFalse: [ self _raiseIndexingError: indexResult ]
	  ].
	  indexes do: [:assoc | assoc key postIndexCreation: assoc value ]
  ].
] onSynchronous: Error do:[:ex |
	IndexingErrorPreventingCommit resignal: ex objWithIndex: self .
].
%

category: 'Searching'
method: UnorderedCollection
detect: aBlock

"Evaluates aBlock repeatedly, with elements of the receiver as the argument.
 Returns the first element for which aBlock evaluates to true.  If none of the
 receiver's elements evaluates to true, generates an error.  The argument
 aBlock must be a one-argument block.  Uses associative access when the
 argument is a SelectionBlock."

| result |
(aBlock class == SelectBlock)
    ifTrue:[
        "send value: to the block to create the 4 element
        associative access Array; the argument nil is ignored to
        create the Array, but all SelectBlocks have one argument"
        result := (QueryExecuter on: self)
            _boundQueryDetect: (aBlock queryBlock value: nil).
        result == #_incompletePathTraversal
            ifTrue: [
                ^ self _error: #assocErrNoElementsDetected args: { aBlock }
            ].
        ^ result
       ].
^ self _detect: aBlock
%

category: 'Searching'
method: UnorderedCollection
detect: aBlock ifNone: exceptionBlock

"Evaluates aBlock repeatedly, with elements of the receiver as the argument.
 Returns the first element for which aBlock has the value true.  If none of the
 receiver's elements has the value true, this evaluates the argument
 exceptionBlock and returns its value.  The argument aBlock must be a
 one-argument block, and exceptionBlock must be a zero-argument block.  Uses
 associative access when the argument is a SelectionBlock."

| each bag result |
(aBlock class == SelectBlock)
    ifTrue:[
        "send value: to the block to create the 4 element
        associative access Array; the argument nil is ignored to
        create the Array, but all SelectBlocks have one argument"
        result := (QueryExecuter on: self)
            _boundQueryDetect: (aBlock queryBlock value: nil).
        result == #_incompletePathTraversal
            ifTrue: [ ^ exceptionBlock value ].
        ^ result
    ].
bag := self _asIdentityBag.
1 to: bag size do:[:i |
    each := (bag _at: i).
    (aBlock value: each)
        ifTrue:[ ^ each ].
].
^ exceptionBlock value
%

category: 'Accessing Indexes'
method: UnorderedCollection
equalityIndexedPaths

"Returns an Array of Strings, each of which represents a path for which an
 equality index exists in the receiver.  Each path originates with the elements
 of the receiver."

| anArray |
anArray := { } .
self _indexedPaths ifNil: [ ^ anArray ].

self _indexedPaths indexObjectsAndOffsetsDo: [ :indexObj :offset | | str |
  indexObj isRangeEqualityIndex
    ifTrue: [
      str := indexObj pathComponentsStringStartingAt: offset.
      indexObj isComplete
        ifFalse: [ str add: ' (incomplete)' ].
      (anArray includesValue: str)
        ifFalse: [ anArray addLast: str ]
    ]
].
^ anArray
%

category: 'Accessing Indexes'
method: UnorderedCollection
equalityIndexedPathsAndConstraints

"Returns an Array containing info about equality indexes.  The Array consists of
 String/Class pairs.  The string represents a path of the receiver's element
 kind for which an equality index exists in the receiver.  The class is the
 constraint on the last element in the path."

| anArray |
anArray := { } .
self _indexedPaths ifNil: [ ^ anArray ].

self _indexedPaths indexObjectsAndOffsetsDo: [ :indexObj :offset |
    indexObj isRangeEqualityIndex
        ifTrue: [
            anArray addLast: (indexObj pathComponentsStringStartingAt: offset).
            anArray addLast: indexObj lastElementClass
        ]
].
^ anArray
%

category: 'Repository Conversion'
method: UnorderedCollection
getIndexInfo

"Returns indexing information for the receiver."

| iList indexObj path kind lastConstraint result |
  result := { } .
  iList := self _indexedPaths.
  1 to: iList size by: 2 do: [ :i |
    indexObj := iList at: i.
    (iList at: i + 1) == 1 ifTrue: [
       " path is first arg "
       path := indexObj pathComponentsString.

       " kind is true if identity index, false if equality index "
       kind := indexObj isIdentityIndex.

       " for equality indexes, lastConstraint is second argument "
       kind ifFalse: [
          lastConstraint := indexObj lastElementClass.
          (lastConstraint == SmallInteger) ifTrue: [ lastConstraint := Integer ].
       ].
       result add: { self . path . kind . lastConstraint } .
    ].
  ].
  ^ result.
%

category: 'Accessing Indexes'
method: UnorderedCollection
identityIndexedPaths
  "Returns an Array of Strings, each of which represents a path for which an
 identity index exists in the receiver.  Each path originates with the elements
 of the receiver."

  | anArray |
  anArray := {}.
  self _indexedPaths == nil
    ifTrue: [ ^ anArray ].
  self _indexedPaths
    indexObjectsAndOffsetsDo: [ :indexObj :offset |
      | pathString |
      indexObj isIdentityIndex
        ifTrue: [
          pathString := indexObj pathComponentsStringStartingAt: offset.
          (anArray includesValue: pathString)
            ifFalse: [ anArray addLast: pathString ] ]
        ifFalse: [
          (self _isRangeable: indexObj lastElementClass)
            ifTrue: [
              pathString := indexObj pathComponentsStringStartingAt: offset.
              indexObj isComplete
                ifFalse: [ pathString add: ' (incomplete)' ].
              (anArray includesValue: pathString)
                ifFalse: [ anArray addLast: pathString ] ] ] ].
  ^ anArray
%

category: 'Searching'
method: UnorderedCollection
includes: anObject

"Returns true if anObject is equal to one of the elements of the receiver.
 Returns false otherwise."

self subclassResponsibility: #includes:
%

category: 'Searching'
method: UnorderedCollection
includesIdentical: anObject

"Returns true if anObject is identical to one of the elements of the receiver.
 Returns false otherwise."

self subclassResponsibility: #includesIdentical:
%

category: 'Searching'
method: UnorderedCollection
includesValue: anObject

"Returns true if the receiver contains an object of the same value as the
 argument, anObject.  Returns false otherwise.  (Compare with includes:,
 which is based on identity.)"

self subclassResponsibility: #includesValue:
%

category: 'Accessing'
method: UnorderedCollection
instVarAt: aSmallInteger

"If the receiver has a publicly accessible named instance variable at
 index aSmallInteger, this returns its value.  Generates an error if
 aSmallInteger is not a SmallInteger or is out of bounds, or if the
 receiver has no publicly accessible named instance variables."

(UnorderedCollection instSize >= aSmallInteger and:[ 1 <= aSmallInteger ])
  ifTrue:[ ^ nil ].

^ super instVarAt: aSmallInteger
%

category: 'Updating'
method: UnorderedCollection
instVarAt: anIndex put: aValue

"Stores the argument aValue in the instance variable indicated by anIndex and
 returns aValue. Generates an error if (1) anIndex is not a SmallInteger,
 (2) anIndex is out of bounds or (3) if the receiver has no publicly accessible
 named instance variables."

(UnorderedCollection instSize < anIndex) ifTrue:[
  ^ super instVarAt: anIndex put: aValue	"skip structural access check"
].
^ self _errorIndexOutOfRange: anIndex
%

category: 'Accessing Indexes'
method: UnorderedCollection
kindsOfIndexOn: aPathString
  "Returns a Symbol that indicates the kinds of indexes into the receiver that
 exist on aPathString: #identity, #equality, #equalityAndIdentity, or #none
 (either aPathString is not a path for the element kind of the receiver, or no
 indexes into the receiver exist on aPathString)."

  | pathList pathArray |
  self _indexedPaths ifNil: [ ^ #'none' ].
  pathList := {}.
  pathArray := aPathString asArrayOfPathTerms.
  self _indexedPaths
    indexObjectsAndOffsetsDo: [ :indexObj :offset |
      (offset <= indexObj size
        and: [
          pathArray size > (indexObj size - offset)
            and: [ indexObj hasIndexOnPath: pathArray startingAt: offset ] ])
        ifTrue: [ pathList add: indexObj ] ].
  pathList size == 2
    ifTrue: [ ^ #'equalityAndIdentity' ].
  pathList size == 1
    ifTrue: [
      pathList first isRangeEqualityIndex
        ifTrue: [
          (self _isRangeable: pathList first lastElementClass)
            ifTrue: [ ^ #'equalityAndIdentity' ]
            ifFalse: [ ^ #'equality' ] ]
        ifFalse: [ ^ #'identity' ] ].
  self _indexedPaths
    ifNotNil: [
      (self _indexedPaths _numberOfCommonPathTermsForPathArray: pathArray)
        == pathArray size
        ifTrue: [ ^ #'identity' ]
        ifFalse: [ ^ #'none' ] ].
  ^ #'none'
%

category: 'Storing and Loading'
method: UnorderedCollection
loadFrom: passiveObj size: varyingSize

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into the receiver."

"This method is similar to loadFrom:, but is used for objects whose size
 is not known when they are first instantiated (such as an IdentitySet)."

^self basicLoadFrom: passiveObj size: varyingSize
%

category: 'Storing and Loading'
method: UnorderedCollection
loadNamedIVsFrom: passiveObj

"Reads named instance variables from the given passive object.  The first
 instance variable should already have been parsed and be available in the
 passiveObj argument."

| name offset nameSym |

[ name := passiveObj ivName.
  name ifNotNil: [
    nameSym := Symbol _existingWithAll: name .
    nameSym ifNotNil:[
      offset := self class _ivOffsetOf: nameSym.
      offset ifNotNil:[ self instVarAt: offset put: passiveObj ivValue ]
             ifNil:[ self dynamicInstVarAt: nameSym put: passiveObj ivValue].
    ] ifNil:[
      self dynamicInstVarAt: name asSymbol put: passiveObj ivValue
    ].
    passiveObj readNamedIV
  ] ifNil: [
    false
  ]
] untilFalse.

passiveObj checkForBagMark.
passiveObj skipNamedInstVars.
%

category: 'Storing and Loading'
method: UnorderedCollection
loadVaryingFrom: passiveObj size: varyingSize

"Reads the varying part of the receiver from the given passive object.
 Does not record the receiver as having been read.  Does not read the
 receiver's named instance variables, if any."

1 to: varyingSize do: [:i |
  self add: (passiveObj readObject)
].
%

category: 'Instance Migration'
method: UnorderedCollection
migrateFrom: anotherObject instVarMap: otherivi

"Takes information from the given object and puts it in the receiver.  This
 message is sent to an object when its class is being migrated to another class
 to account for changes in a schema.  The otherivi argument is a precalculated
 indirection table associating the receiver's instance variables with instance
 variables in the other object.  If a table entry is 0, the other object is
 assumed not to have that instance variable.

 This method should be augmented to perform other necessary initializations in
 the receiver."

| otherClass |

super migrateFrom: anotherObject instVarMap: otherivi.

"if no elements in anotherObject, no need to do anything "
anotherObject size == 0
  ifTrue: [ ^ self ].

otherClass := anotherObject class.

otherClass isNonByteVarying
  ifTrue: [
    anotherObject accompaniedBy: self do: [ :me :anElement | me add: anElement ].
  ].

^ self.
%

category: 'Searching'
method: UnorderedCollection
occurrencesOf: anObject

"Returns the number of the receiver's elements that are identical (==) to
 anObject."

self subclassResponsibility: #occurrencesOf:
%

category: 'Copying'
method: UnorderedCollection
postCopy

"Cleanup new copy."

super postCopy.
_indexedPaths := nil
%

category: 'Updating Indexes'
method: UnorderedCollection
progressOfIndexCreation

"Returns a String that describes the progress of an index creation that is
 underway."

| indexObj cnt str lf |

indexObj := self _findIncompleteIndex.
( indexObj == nil or:
" get the current offset of enumeration through the NSC "
  [ (cnt := indexObj progress) == nil ] )
     ifTrue: [
    ^ 'No index creation is in progress.'
].

str := String new.
lf := Character lf.
str add: lf; add: 'Creating index on <';
  add: indexObj pathComponentsString; add: $>; add: lf;
  add: 'During enumeration of collection, finished ';
  add: cnt asString; add: ' out of ';
  add: self size asString; add: ' total elements.'.

indexObj isRangeEqualityIndex ifTrue: [
    " get the current offset of enumeration through the n-way merge "
    cnt := indexObj nwayMergeProgress.
    cnt ifNotNil: [
        str add: lf; add: 'During n-way merge, finished ';
          add: cnt asString; add: ' elements'; add: lf.
	]
].

^ str
%

category: 'Repository Conversion'
method: UnorderedCollection
rebuildIndexes: indexInfo

"Rebuilds the indexes."

" Interim conversion: need method declaration, but commented out..

| path kind lastConstraint newLastConstraint |

  path := indexInfo at: 2.
  kind := indexInfo at: 3.
  lastConstraint := indexInfo at: 4.

  kind
    ifTrue: [
      self createIdentityIndexOn: path
      ]
    ifFalse: [
      newLastConstraint := lastConstraint _correspondingNewClass.
      self createEqualityIndexOn: path withLastElementClass: newLastConstraint
      ].
"
%

category: 'Searching'
method: UnorderedCollection
reject: aBlock

"Evaluates aBlock with each of the receiver's elements as the argument.
 Stores the values for which aBlock is false into a collection of the same
 class as the receiver, and returns the new collection.  The argument aBlock
 must be a one-argument block.  Uses associative access when the argument is
 SelectionBlock."

| result |
(aBlock class == SelectBlock)
    ifTrue:[
       "send value: to the block to create the 4 element
        associative access Array; the argument nil is ignored to
        create the Array, but all SelectBlocks have one argument"
        result := (QueryExecuter on: self)
            _boundQueryReject: (aBlock queryBlock value: nil).
        ^ result
    ].
^ self _reject: aBlock
%

category: 'Removing'
method: UnorderedCollection
removeIfPresent: anObject

"Removes anObject from the receiver and returns anObject.  If anObject is
 present several times in the receiver, only one occurrence is removed.
 Returns nil if anObject is missing from the receiver."

self remove: anObject ifAbsent: [ ^ nil ].
^ anObject.
%

category: 'Updating Indexes'
method: UnorderedCollection
removeIncompleteIndex
  "If there is an incomplete index, clean it up. In general this method should not be used, a better way to remove
 incomplete indexes is to use IndexManager>>removeAllIncompleteIndexesOn:"

  [
    System _zeroArgPrim: 51 . "_enteringIncompleteIndexRemoval"
    [ | systm indexObj |
      systm := System.
      systm abortTransaction.
      indexObj := self _findIncompleteIndex.
      indexObj ifNil: [ ^ self ].
      self
        _undoIndexCreation: indexObj
        pathTerm: indexObj _findFirstUnsharedPathTerm.
      systm _commitPrintingDiagnostics
    ]
    untilTrue
  ] ensure:[
    System _zeroArgPrim: 52 . "_leavingIncompleteIndexRemoval"
  ]
%

category: 'Auditing'
method: UnorderedCollection
repairInternalStructures
"For an explanation of this method, refer to the comments in the method
 UnorderedCollection>>auditInternalStructuresWithRepair:"

^self auditInternalStructuresWithRepair: true
%

category: 'Searching'
method: UnorderedCollection
select: aBlock

"Evaluates aBlock with each of the receiver's elements as the argument.
 Stores the values for which aBlock is true into a collection of the same
 class as the receiver, and returns the new collection.  The argument aBlock
 must be a one-argument block.  Uses associative access when the argument is a
 SelectionBlock.

 The new collection returned by this method will not retain any indexes of
 the receiver.  If you want to perform indexed selections on the new
 collection, you must build all of the necessary indexes.  The discussion of
 'Transferring Indexes' in the 'Indexed Associative Access' chapter of the
 GemStone Programming Guide describes a technique for doing this."

| result |
(aBlock class == SelectBlock)
    ifTrue:[
       "send value: to the block to create the 4 element
        associative access Array; the argument nil is ignored to
        create the Array, but all SelectBlocks have one argument"
        result := (QueryExecuter on: self)
            _boundQuerySelect: (aBlock queryBlock value: nil).
        ^ result
    ].
^ self _select: aBlock
%

category: 'Searching'
method: UnorderedCollection
selectAsStream: aBlock

"Same functionality as select: except that the result is returned as a
 RangeIndexReadStream rather than an IdentitySet. The stream will contain
 the results sorted in the order specified by the equality index.

 The select block is limited in the following ways:

 * The select block may only contain a single predicate.
 * The predicate must contain one path expression.
 * An equality index must exist for the path expression.

 To use the stream that this method returns most effectively, avoid modifying
 both the receiver of this message and the selected objects returned by the
 stream as long as the stream is being accessed.  Changes that alter the
 equality index can cause stream access failures."

| result |
(aBlock class == SelectBlock)
    ifTrue:[
       "send value: to the block to create the 4 element
        associative access Array; the argument nil is ignored to
        create the Array, but all SelectBlocks have one argument"
        result := (QueryExecuter on: self)
            _boundQuerySelectAsStream: (aBlock queryBlock value: nil).
        ^ result
    ].
^ self _error: #rtErrBagOnlySelectBlockAllowed
%

category: 'Copying'
method: UnorderedCollection
shallowCopy

"Returns a copy of the receiver which shares the receiver's instance
 variables."

<primitive: 746>
self _primitiveFailed: #shallowCopy .
self _uncontinuableError
%

category: 'Private'
method: UnorderedCollection
_asCollectionForSorting

"Used by the sorting subsystem."

^ self _asIdentityBag.
%

category: 'Private'
method: UnorderedCollection
_asIdentityBag

"Returns the receiver."

^ self
%

category: 'Deprecated'
method: UnorderedCollection
_auditAndRepairDepListsLogging: aString
  "Get any dependency lists we can find and repair them if they need it.
Report any repairs on the given string."

  | depLists sharedDepLists depList cnt |
  self deprecated: 'UnorderedCollection>>_auditAndRepairDepListsLogging: deprecated v3.3.
     Use DepListTable>>_auditAndRepair: to find and repair dep lists'.	"bug 43886"
  sharedDepLists := SharedDependencyLists.
  depLists := self _getAllDependencyLists.
  cnt := 0.
  1 to: depLists size do: [ :i |
    (depList := depLists at: i) _needsSorting
      ifTrue: [
        cnt := cnt + 1.
        depList _sortEntries: (sharedDepLists depListBucketFor: depList) ] ].
  cnt ~~ 0
    ifTrue: [
      aString
        add: 'For nsc [';
        add: self identityHash asString;
        add: '] repaired ';
        add: cnt asString;
        add: ' dependency list(s)';
        add: Character lf ].
  ^ aString
%

category: 'Indexing Audit'
method: UnorderedCollection
_auditBtreeCounts: btreeCounts on: aString

"Private.  Unsupported method for GemStone Technical Support."

btreeCounts do: [:each |
  (each class == Array) ifTrue: [
      self _auditBtreeCounts: each on: aString.
    ] ifFalse: [
      "must be a 0, or there is a problem with index structure"
      each ~~ 0 ifTrue: [
              aString add: Character lf;
                  add: ' -- The number of entries in a Btree does not match the number of entries in the base collection (extra elements in either the base collection or btree).';
                  add: Character lf
        ].
    ].
].
%

category: 'Indexing Audit'
method: UnorderedCollection
_auditIndexBtree

"Verifies that the B-tree component of the indexing structure for
 this collection is consistent.  Returns a string that describes any
 inconsistencies found.  This method is intended to be used for audit
 when problems specific to the btrees are suspected; for general audit
 on the indexes use _auditIndexes"

^ self _fastAuditEqualityIndexes
%

category: 'Indexing Audit'
method: UnorderedCollection
_auditIndexDictionary

"Verifies that the index dictionary component of the indexing structure
 for this collection is consistent.  Returns a string that describes any
 inconsistencies found.  This method is intended to be used for audit when
 problems specific to the index dictionary are suspected; for general
 audit on the indexes use _auditIndexes"

^ self _fastAuditIdentityIndexes
%

category: 'Indexing Audit'
method: UnorderedCollection
_auditIndexes

"Verifies that the index objects are consistent.
 Returns a string that describes any inconsistencies found."

^self _fastAuditIndexes
%

category: 'Updating Indexes - Private'
method: UnorderedCollection
_calculateIndexDictionarySize: indexPathLength

"Heuristic for determining size of index dictionary root."

| sz  |
"One entry in dictionary for each element/pathTerm key and a bucket load factor of 2/3."
sz := (self size * indexPathLength) //
       ((RcIndexDictionary defaultBasicSize * 2) //
        (RcIndexBucket entrySize * 3)).
sz := sz max: RcIndexDictionary defaultBasicSize.
self _setIndexDictionaryCreationSize: (Integer _selectedPrimeGreaterThan: sz).
%

category: 'Indexing Support'
method: UnorderedCollection
_checkIndexPathExpression: aPathString
  "Raises an error if the given path expression (an Array of Strings), is
 not valid; otherwise returns the receiver."

  ^ aPathString asArrayOfPathTerms
%

category: 'Updating Indexes - Private'
method: UnorderedCollection
_cleanUpDependencies

"The receiver is having all of its indexes removed.  Remove dependency
 list entries for objects along the path of the indexes."

| rootTerms bag prevObj obj iList |
iList := self _indexedPaths.
rootTerms := iList rootTerms.

prevObj := #_incompletePathTraversal.
bag := self _asIdentityBag.
1 to: bag size do: [ :i |
  obj := bag _at: i.
  (obj == prevObj) ifFalse: [
      1 to: rootTerms size do: [ :j |
        (rootTerms at: j) cleanupDependencyListFor: obj
      ]
    ].
  prevObj := obj
].

iList hasTrackingObjects
  ifTrue: [ iList removeAllIndexesFor: self ]
  ifFalse: [ self _indexedPaths: nil ].

" resize the Array of path terms to avoid object audit errors "
rootTerms size: 0.
%

category: 'Indexing Support'
method: UnorderedCollection
_clearIndexList

"Private.  Sets the indexed paths for the receiver to nil."

self _indexedPaths: nil
%

category: 'Modification Tracking'
method: UnorderedCollection
_clearModificationTrackingTo: tracker

"Remove the given tracker from the receiver's indexedPaths and dependency lists."

"Must clear depmap entry first, so we don't get notified about IV changes"
super _clearModificationTrackingTo: tracker.

self _getIndexList removeTracker: tracker for: self.

%

category: 'Clustering'
method: UnorderedCollection
_clusterIndexes

"Clusters indexing objects.  This may cause concurrency conflicts on
 internal indexing objects.  Returns the receiver."

| iList |
(iList := self _indexedPaths) == nil
  ifTrue: [
    ^ self
  ].

" cluster the index dictionary "
self _indexDictionary ~~ nil
  ifTrue: [ self _indexDictionary clusterDepthFirst ].

" cluster each index object "
1 to: iList size by: 2 do: [ :j |
  (iList at: j) _isIndexObject
    ifTrue: [ (iList at: j) clusterDepthFirst ]
].
%

category: 'Parallel Index Creation Support'
method: UnorderedCollection
_createParallelIndexOn: anIndexSpecification
  | indexManager pathArray index indexes |
  indexManager := IndexManager current.
  pathArray := anIndexSpecification path asArrayOfPathTerms.
  indexes := self _findIndexesWithPath: pathArray.
  indexes
    do: [ :idxObj |
      " check if an index already exists for the path "
      anIndexSpecification indexType == indexManager _identityIndex
        ifTrue: [
          idxObj isIdentityIndex
            ifTrue: [ ^ nil ] ]
        ifFalse: [
          idxObj isRangeEqualityIndex
            ifTrue: [ ^ nil ] ] ].
  index := anIndexSpecification _createIndex.
  index _preIndexCreationBuildIndexDictionaryFor: pathArray for: self.
  index nscRoot: self.
  self _getIndexList buildPathTermsFor: index with: pathArray fromSpec: anIndexSpecification.
  indexManager _addIndex: index.
  ^ index
%

category: 'Searching'
method: UnorderedCollection
_detect: aBlock

| each bag |
bag := self _asIdentityBag.
1 to: bag size do:[:i |
    each := (bag _at: i) .
    (aBlock value: each) ifTrue:[ ^ each ].
].
^ self _error: #assocErrNoElementsDetected args: { aBlock } .
%

category: 'Updating Indexes - Private'
method: UnorderedCollection
_doUpdateIndexesForAdditionOf: anObject iList: iList logging: doLogging
  "anObject is being added to the receiver.  Update any indexes if necessary.
 Returns true if the index objects were modified correctly; otherwise returns
 an Array containing error information."

  | mapInfo rootTerms i sz num val prevTerm aSet rootObjects |
  rootTerms := iList rootTerms.
  sz := rootTerms size.
  1 to: iList size by: 2 do: [ :j |
    | indexObj offset indexDict pathTerm |
    "Make a pass through the indexedPaths to see if the receiver participates as a
 set-valued instance variable.  If so, then must update the index dictionary."
    indexObj := iList at: j.
    offset := iList at: j + 1.
    (offset > 1
      _and: [
        indexObj isComplete
          _and: [
            " get the previous path term (the one with an asterisk) "
            pathTerm := indexObj at: offset - 1.
            pathTerm hasIndexDictionary ] ])
      ifTrue: [
        " lazy initialization of aSet (in case we don't ever need it) "
        aSet == nil
          ifTrue: [ aSet := IdentitySet new ].	" only update the dictionary if we haven't already done it "
        (aSet includesIdentical: pathTerm)
          ifFalse: [
            aSet add: pathTerm.
            indexDict := indexObj indexDictionary.	" see how many entries exist "
            val := indexDict
              at: self
              term: pathTerm getParentTerm
              otherwise: nil.
            nil == val
              ifTrue: [ indexDict _errorNoEntryForKey: self term: pathTerm getParentTerm value: nil ]
              ifFalse: [
                (BucketValueBag _hasInstance: val)
                  ifTrue: [ num := val size ]
                  ifFalse: [ num := 1 ] ].
            num
              timesRepeat: [
                " add a dictionary entry for anObject -> NSC "
                indexDict
                  _at: anObject
                  put: self
                  term: pathTerm
                  logging: doLogging ] ] ] ].
  i := 1.
  [ i <= sz and: [ i <= rootTerms size ] ]
    whileTrue: [
      | pathTerm |
      " now make a pass through the root path terms "
      (pathTerm := rootTerms at: i) _isObsoletePathTerm
        ifFalse: [
          " see if the receiver is participating as a set-valued instance variable "
          pathTerm offset == 1
            ifTrue: [ num := 1 ]
            ifFalse: [
              pathTerm hasIndexDictionary
                ifTrue: [
                  " get the path term before the set-valued path term "
                  prevTerm := pathTerm getParentTerm.
                  pathTerm indicatesIndexOnNscElements
                    ifFalse: [ prevTerm := prevTerm getParentTerm ].
                  val := pathTerm getIndexDictionary
                    at: self
                    term: prevTerm
                    otherwise: nil.
                  nil == val
                    ifTrue: [
                      " if not found, then mappings have already been added "
                      num := 0 ]
                    ifFalse: [
                      (BucketValueBag _hasInstance: val)
                        ifTrue: [ num := val size ]
                        ifFalse: [ num := 1 ] ] ]
                ifFalse: [
                  "receiver is the '*' for a set valued term"
                  prevTerm := pathTerm indicatesIndexOnNscElements
                    ifTrue: [ pathTerm ]
                    ifFalse: [ pathTerm getParentTerm ].
                  rootObjects := prevTerm findReachableRootsFor: self.
                  pathTerm indicatesIndexOnNscElements
                    ifFalse: [ rootObjects := rootObjects asIdentitySet ] ] ].
          pathTerm indicatesIndexOnNscElements
            ifTrue: [
              " if the index is on elements of the NSC, go ahead and update the indexes "
              pathTerm updateBtree
                ifNotNil: [
                  (pathTerm _checkBtreeComparisonWith: anObject)
                    ifFalse: [
                      | exa |
                      (exa := ImproperOperation new)
                        _number:
                            (ErrorSymbols at: #'rtErrRangeEqualityIndexInvalidClassKindForBtree');
                        args: { anObject class. pathTerm name. pathTerm updateBtree lastElementClassDescription}.
                      i > 1
                        ifTrue: [
                          " indexing objects have been modified, prevent commits "
                          exa signal ]
                        ifFalse: [
                          ^ {false.	" indicating no indexing objects were modified "
                          exa} ] ] ].
              pathTerm hasIndexDictionary
                ifTrue: [ num timesRepeat: [ pathTerm addDirectMappingFor: anObject logging: doLogging ] ]
                ifFalse: [
                  rootObjects == nil
                    ifTrue: [
                      pathTerm
                        addDirectMappingFor: anObject
                        root: anObject
                        nsc: self
                        logging: doLogging ]
                    ifFalse: [
                      rootObjects
                        do: [ :root |
                          pathTerm
                            addDirectMappingFor: anObject
                            root: root
                            nsc: self
                            logging: doLogging ] ] ] ]
            ifFalse: [
              (pathTerm _doNotPerformPretraversalFor: anObject)
                ifTrue: [
                  " if the path has too many set-valued terms or we're using GsPathTerms, go ahead and update the indexes ... potential for too much or incorrect MappingInfo."
                  rootObjects
                    ifNil: [ num timesRepeat: [ pathTerm addMappingsForObject: anObject logging: doLogging ] ]
                    ifNotNil: [
                      rootObjects
                        do: [ :root | pathTerm addMappingsForObject: anObject root: root logging: doLogging ] ] ]
                ifFalse: [
                  " get all the mapping info first (this detects any errors
                      along the path before any changes are made to the indexes) "
                  mapInfo := pathTerm
                    getMappingInfoFor: anObject
                    ifObject: nil
                    atOffset: 0
                    replaceWith: nil.	" if the result is not a map info object, it is an Array
                      used for error information "
                  mapInfo class == Array
                    ifTrue: [
                      i > 1
                        ifTrue: [
                          " indexing objects have been modified, prevent commits "
                          (mapInfo at: 2) signal ].
                      ^ mapInfo ].
                  " now update dependency lists, index dictionary, B-trees "
                  num
                    timesRepeat: [ mapInfo pathTerm addMappingsUsing: mapInfo logging: doLogging ] ] ] ].
      i := i + 1 ].
  ^ true
%

category: 'Updating Indexes - Private'
method: UnorderedCollection
_doUpdateIndexesForRemovalOf: anObject iList: iList

"anObject is being removed from the receiver.  Update the indexes if
 necessary.  This method is invoked prior to removal of anObject
 from the receiver. "

  | depList lastOne doLogging origSize rootTerms sz i pathTerm num prevTerm val rootObjects currSize indexObj offset aSet  |
 	  " get the removed object's dependency list "
	  depList := DependencyList for: anObject.

	  " see if another occurrence of anObject is in the index objects "
	  lastOne := self _isLastOccurrenceInIndexObjects: anObject.

	  doLogging := UnorderedCollection _isRcIndexLoggingEnabled.
	  origSize := iList size.

	  rootTerms := iList rootTerms.
	  sz := rootTerms size.
	  i := 1.
	  " for each unique path on which there is an index "
	  [ i <= sz and: [ i <= rootTerms size ] ] whileTrue: [
      num := nil.
		pathTerm := rootTerms at: i.
		" see if the path term is still a root term and is not obsolete "
		((iList rootTerms includesIdentical: pathTerm) and:
		       [ pathTerm _isObsoletePathTerm not ]) ifTrue: [
			" see if the receiver is participating as a set-valued instance variable "
			pathTerm offset == 1
			  ifTrue: [ num := 1 ]
			  ifFalse: [
				pathTerm hasIndexDictionary
               ifTrue: [" get the path term before the set-valued path term "
				    prevTerm := pathTerm getParentTerm.
				    pathTerm indicatesIndexOnNscElements ifFalse: [ prevTerm := prevTerm getParentTerm ].

				    val := pathTerm getIndexDictionary at: self term: prevTerm otherwise: nil.
				    " if not found, then mappings have already been removed "
				    val ifNil: [ num := 0 ]
					   ifNotNil: [
					     (BucketValueBag _hasInstance: val)
						    ifTrue: [ num := val size ]
						    ifFalse: [ num := 1 ]
					   ] ]
                ifFalse: [
                  "receiver is the '*' for a set valued term"
                  prevTerm := pathTerm indicatesIndexOnNscElements
                    ifTrue: [ pathTerm ]
                    ifFalse: [ pathTerm getParentTerm ].
                  rootObjects := (prevTerm findReachableRootsFor: self) asIdentitySet ] ].

			" see if this index is on the elements of the NSC "
			pathTerm indicatesIndexOnNscElements ifTrue: [
              (pathTerm hasIndexDictionary or: [ rootObjects == nil  ])
                ifTrue: [ num timesRepeat: [ pathTerm removeDirectMappingFor: anObject logging: doLogging ] ]
                ifFalse: [
                  rootObjects do: [:root |
                    pathTerm removeDirectMappingFor: anObject root: root nsc: self logging: doLogging ] ].

				 ( depList ~~ nil and: [ pathTerm needsDepList ] )
				   ifTrue: [ DependencyList removePathTerm: pathTerm for: anObject ]
			  ] ifFalse: [
				anObject ifNotNil: [
                (pathTerm hasIndexDictionary or: [ num ~~ nil ])
                  ifTrue: [
					  num - 1 timesRepeat: [
					    pathTerm removeMappingsFor: anObject
					      lastOne: false
					     logging: doLogging ].
					  pathTerm removeMappingsFor: anObject
					    lastOne: lastOne
					    logging: doLogging ]
                 ifFalse: [
                   rootObjects do: [:root |
                     pathTerm removeMappingsFor: anObject
                        root: root
					      lastOne: lastOne
					      logging: doLogging ] ] .
				( depList ~~ nil and: [ lastOne ] ) ifTrue: [
					DependencyList removePathTerm: pathTerm for: anObject
				]
			  ] ].
		  ].
		i := i + 1
	  ].

     pathTerm hasIndexDictionary
       ifTrue: [
	      currSize := iList size.
	      i := 1.
	      [ i <= origSize and: [ i <= currSize ] ] whileTrue: [
		    indexObj := iList at: i.
		    offset := iList at: i + 1.
		    " see if receiver participates as a set-valued instance variable "
		    (offset > 1 and: [ indexObj isComplete ]) ifTrue: [
			   " lazy initialization of aSet (in case we don't ever need it) "
			   aSet ifNil: [ aSet := IdentitySet new ].
			   " get the previous path term (the one with an asterisk) "
			   pathTerm := indexObj at: offset - 1.
			   " only update the dictionary if we haven't already done it "
			   (aSet includesIdentical: pathTerm) ifFalse: [
				   aSet add: pathTerm.
				   " see how many entries exist "
				   val := indexObj indexDictionary at: self term: pathTerm getParentTerm otherwise: nil.

				   " if not found, then mappings have already been removed "
				   val ifNil: [ num := 0 ]
				     ifNotNil: [
					    (BucketValueBag _hasInstance: val)
				         ifTrue: [ num := val size ]
					  	  ifFalse: [ num := 1 ]
				  	   ].

				   num timesRepeat: [
				     " remove dictionary entry for anObject -> NSC "
				     indexObj indexDictionary removeKey: anObject value: self
									term: pathTerm logging: doLogging
				   ] ] ].
		    i := i + 2
	      ] ].
%

category: 'Indexing Audit'
method: UnorderedCollection
_fastAuditEqualityIndexes
  "Verifies that the equality index objects are consistent. When run in
 conjunction with _fastAuditIdentityIndexes a complete audit is performed.
 Returns a string that describes any inconsistencies found."

  | aString sys participatesInCollectionValuedIndex participatesInIndex |
  self _indexedPaths ifNil: [ ^ 'No indexes are present.' ].
  aString := String new.
  sys := System.
  [
  | roots sz btreeCounts rootTerm countArg |
  roots := self _indexedPaths rootTerms.
  sz := roots size.
  btreeCounts := Array new.
  sys setIndexProgressCountTo: self class statValueForAuditingRootTerms.	"2"
  sys setProgressCountTo: sz.
  participatesInIndex := participatesInCollectionValuedIndex := false.
  1 to: sz do: [ :j |
    | resultArray |
    rootTerm := roots at: j.
    rootTerm offset == 1
      ifTrue: [
        "index on receiver"
        participatesInIndex := true.
        resultArray := rootTerm auditNsc: self on: aString level: 1.
        btreeCounts add: resultArray ]
      ifFalse: [
        "participates in index with collection-valued path term"
        participatesInCollectionValuedIndex := true.
        btreeCounts add: {} ].
    sys decrementProgressCountBy: 1	"dec progress count after each term" ].
  sys setIndexProgressCountTo: self class statValueForAuditingNscCounts.	"3"
  sys setProgressCountTo: sz.	"Reset progressCount"
  1 to: sz do: [ :j |
    rootTerm := roots at: j.
    rootTerm offset == 1
      ifTrue: [
        countArg := btreeCounts at: j.
        self
          do: [ :obj | rootTerm auditNscCountsFor: obj on: aString count: countArg ] ].
    sys decrementProgressCountBy: 1	"dec progress count after each pass through the NSC" ].
  sys
    setIndexProgressCountTo: self class statValueForAuditingBtreeCounts;
    setProgressCountTo: 0.	"4"
  true ifTrue: [
    "bypass this audit step if using GsPathTerm>>auditNscForRootObj:rootObj:using: to get
     better information about
     BtreePlusNodeAuditor>>pathTermIncorrectNumberOfBtreeEntries:index:offset: failures."
    self _auditBtreeCounts: btreeCounts on: aString]  ]
    ensure: [
      sys
        setIndexProgressCountTo: 0;
        setProgressCountTo: 0 ].
  aString isEmpty
    ifTrue: [
      participatesInIndex
        ifTrue: [
          aString := 'Indexes are OK'.
          participatesInCollectionValuedIndex
            ifTrue: [
              aString := aString
                ,
                  ' and the receiver participates in one or more indexes with collection-valued path terms' ] ]
        ifFalse: [
          aString := 'No indexes are present'.
          participatesInCollectionValuedIndex
            ifTrue: [
              aString := aString
                ,
                  ', but the receiver participates in one or more indexes with collection-valued path terms' ] ] ].
  ^ aString
%

category: 'Indexing Audit'
method: UnorderedCollection
_fastAuditIdentityIndexes

"Verifies that the identity index objects are consistent. When run in
 conjunction with _fastAuditEqualityIndexes a complete audit is performed.
 Returns a string that describes any inconsistencies found."

| aString sys |

self _indexedPaths ifNil: [ ^ 'No indexes are present.' ].

sys := System .
aString := String new .

[ | roots sz |
  roots := self _indexedPaths rootTerms.
  sz := roots size.
  sys setIndexProgressCountTo: self class statValueForAuditingIdentityIndexes ; "1"
      setProgressCountTo: sz .

  1 to: sz do: [ :j |
    | rootTerm indexDictionary |
    rootTerm := roots at: j.
    indexDictionary := rootTerm getIndexDictionary.
    indexDictionary ~~ nil
      ifTrue: [
        indexDictionary auditNsc: self for: rootTerm on: aString.
        indexDictionary auditEntriesForNsc: self for: rootTerm on: aString.
      ].
      sys decrementProgressCountBy: 1 . "dec the progress count after each term"
  ].
  aString isEmpty
    ifTrue: [ aString := 'Indexes are OK' ].
] ensure:[ sys setIndexProgressCountTo: 0; setProgressCountTo: 0 ] .

^ aString
%

category: 'Indexing Audit'
method: UnorderedCollection
_fastAuditIndexes
  "Verifies that the index objects are consistent.
 Returns a string that describes any inconsistencies found."

  | str1 str2 success |
  self _indexedPaths ifNil: [ ^ 'No indexes are present.' ].	"IndexProgressCount stage 1: audit identity indexes"
  System setProgressCountTo: 0.
  self _hasIncompleteIndexes
    ifTrue: [
      | str0 |
      str0 := 'Incomplete indexes are present. Use IndexManager>>removeAllIncompleteIndexesOn:
to remove the incomplete indexes. After incomplete indexes have been removed, rerun the audit.
'
        copy.
      self _indexedPaths
        indexObjectsDo: [ :index |
          index isComplete
            ifFalse: [
              str0
                add:
                  '  -- The index ' , index printString , ' [' , index asOop printString
                    ,
                      '] is incomplete.
' ] ].
      ^ str0 ].
  str1 := self _fastAuditIdentityIndexes.
  System setProgressCountTo: 0.
  str2 := self _fastAuditEqualityIndexes.
  System
    setProgressCountTo: 0;
    setIndexProgressCountTo: 0.
  success := 'Indexes are OK'.
  str1 = success
    ifTrue: [
      str2 = success
        ifTrue: [ ^ success ].
      ^ str2 ].
  str2 = success
    ifTrue: [ ^ str1 ].
  ^ str1 , str2
%

category: 'Updating Indexes - Private'
method: UnorderedCollection
_findIncompleteIndex

"Returns an index that is incomplete, or nil if not found."

| iList |
iList := self _indexedPaths.
iList == nil
  ifTrue: [ ^ nil ].

" for each index on the NSC ... "
1 to: iList size by: 2 do: [ :i |
  " only look at the index if the receiver is the root "
  (iList at: i + 1) == 1
    ifTrue: [
      " see if the index is complete "
      (iList at: i) isComplete
        ifFalse: [ ^ iList at: i ]
    ]
].
^ nil
%

category: 'Indexing Support'
method: UnorderedCollection
_findIndexesWithPath: pathArray

"Returns an Array of index objects whose path components are the same as
 represented by the path string.  This method only considers those indexes that
 are defined with the receiver as the root NSC."

| anArray iList |
iList := self _indexedPaths.
iList == nil
  ifTrue: [ ^ { }  ].

anArray := { } .
" for each index on the NSC ... "
1 to: iList size by: 2 do: [ :i |
  " only look at the index if the receiver is the root "
  (iList at: i + 1) == 1
    ifTrue: [
      " if the index is on the same path, add it to the Array "
      ((iList at: i) hasIndexOnPath: pathArray)
        ifTrue: [ anArray addLast: (iList at: i) ]
    ]
].
^ anArray
%

category: 'Indexing Support'
method: UnorderedCollection
_findRangeIndexWithPath: pathArray

"Returns a range equality index whose path components are the same as
 represented by the pathArray (Array of strings)."

| index iList |
iList := self _indexedPaths.
iList ifNil: [ ^ nil ].

" for each index on the NSC ... "
1 to: iList size by: 2 do: [ :i |
  index := iList at: i.
  ( index _isIndexObject and:
    [ index isRangeEqualityIndex and:
     [ index hasIndexOnPath: pathArray ] ] )
       ifTrue: [ ^ index ]
].
^ nil
%

category: 'Deprecated'
method: UnorderedCollection
_getAllDependencyLists
  "Return a set of all dependency lists that we can find from the receiver."

  | all iList rootTerms obj sz prevObj bag |
  self deprecated: 'UnorderedCollection>>_getAllDependencyLists deprecated v3.3.
     Use DependencyList class>>depMapValues to collect dependency lists. (3.3)'.  "bug 43886"

  all := IdentitySet new.
  iList := self _indexedPaths.
  bag := self _asIdentityBag.
  iList ~~ nil
    ifTrue: [
      prevObj := #'_incompletePathTraversal'.
      rootTerms := iList rootTerms.
      sz := rootTerms size.
      1 to: bag size do: [ :i |
        obj := bag _at: i.
        obj == prevObj
          ifFalse: [ 1 to: sz do: [ :j | (rootTerms at: j) allDependencyListsFor: obj into: all ] ].
        prevObj := obj ] ].
  ^ all
%

category: 'Indexing Support'
method: UnorderedCollection
_getIndexDictionaryCreationSize

"Returns the basic size of an index dictionary when a new one is
 being created."

^ System
  rcValueCacheAt: #indexDictionarySize
  for: self
  otherwise: RcIndexDictionary defaultBasicSize.
%

category: 'Indexing Support'
method: UnorderedCollection
_getIndexReferencesInto: refsToRcvr
  "Returns an Array of information about references to the receiver due to the
 receiver's participation in an index as a set-valued instance variable.
 The Array consists of pairs:

 1) An object that references the receiver.
 2) The offset of the instance variable in that object that references the
  receiver.

 This is used only in support of the 'become:' method; therefore if
 the receiver is a root NSC with an index, an error will be raised."

  | indexList pathTerm index vals obj pathTerms |
  indexList := self _indexedPaths.
  pathTerms := IdentitySet new.
  2 to: indexList size by: 2 do: [ :i |
    " for each index that the receiver participates in ... "
    (indexList at: i) == 1
      ifTrue: [
        " check if it is the root NSC or a set-valued instance variable "
        " it is the root, 'become:' not allowed "
        self _error: #'rtErrCantBecomeIndexedNsc' ]
      ifFalse: [
        (indexList at: i) > 1
          ifTrue: [
            index := indexList at: i - 1.
            pathTerm := index at: (indexList at: i) - 2.
            (pathTerms includesIdentical: pathTerm)
              ifFalse: [
                " only want to check each unique path term "
                " get the objects that refer to the receiver "
                vals := index indexDictionary
                  at: self
                  term: pathTerm
                  otherwise: nil.
                nil == vals
                  ifTrue: [ index indexDictionary _errorNoEntryForKey: self term: pathTerm value: nil ].
                (BucketValueBag _hasInstance: vals)
                  ifTrue: [
                    1 to: vals size do: [ :j |
                      obj := vals _at: j.
                      pathTerm indicatesMultiValue
                        ifTrue: [
                          self
                            _addToReferences: obj
                            offset: 0
                            occurrences: (obj occurrencesOf: self)
                            into: refsToRcvr ]
                        ifFalse: [
                          self
                            _addToReferences: obj
                            offset: (pathTerm _ivOffsetFor: obj)
                            occurrences: 1
                            into: refsToRcvr ] ] ]
                  ifFalse: [
                    pathTerm indicatesMultiValue
                      ifTrue: [
                        self
                          _addToReferences: vals
                          offset: 0
                          occurrences: (vals occurrencesOf: self)
                          into: refsToRcvr ]
                      ifFalse: [
                        self
                          _addToReferences: vals
                          offset: (pathTerm _ivOffsetFor: vals)
                          occurrences: 1
                          into: refsToRcvr ] ].
                pathTerms add: pathTerm ] ] ] ].
  ^ super _getIndexReferencesInto: refsToRcvr	" since NSCs may have named instance variables, invoke this on super "
%

category: 'Indexing Support'
method: UnorderedCollection
_hasIncompleteIndexes

"Returns whether the receiver has any incomplete indexes on it.
 If the receiver has incomplete indexes use IndexManager>>removeAllIncompleteIndexesOn:
 to remove the incomplete indexes."

 | iList |
iList := self _indexedPaths.
iList == nil ifTrue: [ ^ false ].
1 to: iList size by: 2 do: [ :i |
  " if it is an incomplete root index ... "
  (((iList at: i + 1) == 1) and: [ (iList at: i) isComplete not])
      ifTrue: [ ^ true ]
].

^ false
%

category: 'Indexing Support'
method: UnorderedCollection
_hasIndexes

"Returns whether the receiver has any indexes on it."

| result iList |
iList := self _indexedPaths.
result := false.
iList == nil
  ifFalse: [
    2 to: iList size by: 2 do: [ :i |
      (iList at: i) > 0 ifTrue: [ result := true ]
    ]
  ].
^ result
%

category: 'Modification Tracking'
method: UnorderedCollection
_hasTrackingObjects

"Returns true if the receiver is being tracked."

| iList |
iList := self _indexedPaths.
iList == nil
  ifFalse: [
    ^ iList hasTrackingObjects
  ].
^ false
%

category: 'Searching'
method: UnorderedCollection
_idxOccurrencesOf: aValue

"Returns the number of occurrences of the given value in the receiver."

^ self occurrencesOf: aValue
%

category: 'Deprecated'
method: UnorderedCollection
_incrementalAuditIndexes: start to: end
  "Verifies that the index objects are consistent. Scans only the objects in the range
 from start to end, inclusive. This may take a while. A better option is to use
 _fastAuditIndexes. This method is not supported. "

  | roots sz aString prevObj nscBuilder index |
  self deprecated: 'UnorderedCollection>>_incrementalAuditIndexes:to: deprecated v3.2, Constraints are no longer supported'.
  self _indexedPaths ifNil: [ ^ 'No indexes are present.' ].
  roots := self _indexedPaths rootTerms.
  sz := roots size.
  aString := String new.	" audit by traversing the tree of path terms for each element "
  prevObj := #'_incompletePathTraversal'.
  nscBuilder := NscBuilder for: IdentityBag new max: NscBuilder maxSize.
  index := 1.
  self
    do: [ :obj |
      (index >= start and: [ index <= end ])
        ifTrue: [
          obj ~~ prevObj
            ifTrue: [
              | oCount |
              oCount := self occurrencesOf: obj.
              1 to: sz do: [ :j |
                | rootTerm |
                rootTerm := roots at: j.
                rootTerm
                  auditObject: obj
                  occurrences: oCount
                  on: aString
                  builder: nscBuilder.
                nscBuilder _resetForAudit ] ].
          prevObj := obj ].
      index := index + 1.
      index > end
        ifTrue: [
          aString isEmpty
            ifTrue: [ aString := 'Indexes are OK' ].
          ^ aString ] ].	"0 element nsc case"
  aString isEmpty
    ifTrue: [ aString := 'Indexes are OK' ].
  ^ aString
%

category: 'Indexing Support'
method: UnorderedCollection
_indexDictionary

"Returns the index dictionary that is shared by all indexes.  If there are no
 indexes, returns nil."

| iList |
iList := self _indexedPaths.
iList
  ifNil: [ ^ nil ]
  ifNotNil: [
    1 to: iList size by: 2 do: [ :i |
      ( (iList at: i + 1) == 1 and:
        [ (iList at: i) indexDictionary ~~ nil ] )
        	ifTrue: [ ^ (iList at: i) indexDictionary ]
    ].
    ^ nil
  ]
%

category: 'Indexing Support'
method: UnorderedCollection
_indexedPaths

"Private.  For use only by the indexing subsystem.

 Returns the indexed paths for the receiver."

^ _indexedPaths
%

category: 'Indexing Support'
method: UnorderedCollection
_indexedPaths: anIndexListOrNil
"used along with SimpleDomainTest to be able to efficiently switch between indexed and non-indexed queries"

_indexedPaths := anIndexListOrNil
%

category: 'Indexing Support'
method: UnorderedCollection
_indexParticipationInfoInto: array

"Returns an Array of pairs:  the root NSC and the path string that describes
 the path traversed from the root to reach the receiver."

| iList indexObj pathString included |
iList := self _indexedPaths.
1 to: iList size by: 2 do: [ :i |
  (iList at: i + 1) > 1 ifTrue: [
      indexObj := iList at: i.
      pathString := indexObj _partialPathComponentsStringUpTo: (iList at: i + 1).
      included := false.
      1 to: array size by: 2 do: [ :j |
        ( indexObj nscRoot == (array at: j) and:
           [ (array at: j + 1) = pathString ] )
               ifTrue: [ included := true ]
      ].
      included ifFalse: [
          array add: indexObj nscRoot.
          array add: pathString
        ]
    ]
].
^ super _indexParticipationInfoInto: array
%

category: 'Indexing Support'
method: UnorderedCollection
_indexStatistics

"Return a dictionary containing statistics that can be useful in determining the
 performance of indexes.  This information is useful for system administrators
 and customer support in determining how to optimize index usage."

| dict |
dict := SymbolDictionary new.
self _indexedPaths == nil
  ifTrue: [
    dict at: #NoIndexesPresent put: true.
    ^ dict
  ].

self _indexedPaths _statisticsInto: dict.
self _indexDictionary ~~ nil
  ifTrue: [ self _indexDictionary _statisticsInto: dict ].

^ dict
%

category: 'Testing'
method: UnorderedCollection
_isIdentityBag

"Return whether the receiver is an identity-based collection."

^ false
%

category: 'Testing'
method: UnorderedCollection
_isLarge

"Returns true if the object is implemented as a tree of private smaller objects"

^ _levels ~~ 0
%

category: 'Indexing Support'
method: UnorderedCollection
_isLastOccurrenceInIndexObjects: anObject
  "Returns true if the given object is maintained in the indexing objects for one
 occurrence."

  | rootTerms pathTerm key val val2 num parentTerm |
  rootTerms := self _indexedPaths rootTerms.	" find a path term with a mapping in the index dictionary "
  1 to: rootTerms size do: [ :i |
    pathTerm := rootTerms at: i.
    (pathTerm _isObsoletePathTerm)
      ifFalse: [
        pathTerm isRangeEqualityIndexLastPathTerm
          ifTrue: [
            pathTerm offset == 1
              ifTrue: [ ^ pathTerm _findAllValuesForIdenticalRootObject: anObject ]
              ifFalse: [
                pathTerm hasIndexDictionary
                  ifTrue: [
                    pathTerm := pathTerm getParentTerm.
                    val := pathTerm updateDict
                      at: anObject
                      term: pathTerm
                      otherwise: nil.
                    (BucketValueBag _hasInstance: val)
                      ifTrue: [
                        " see if more than one mapping "
                        pathTerm indicatesNsc
                          ifTrue: [
                            num := val occurrencesOf: self.
                            num < val size
                              ifTrue: [
                                " if anObject is contained in other NSCs "
                                ^ false ].
                            pathTerm := pathTerm getParentTerm.	" get path term before this one "
                            val2 := pathTerm updateDict
                              at: self
                              term: pathTerm
                              otherwise: nil.
                            (BucketValueBag _hasInstance: val2)
                              ifTrue: [ ^ val2 size == num ]
                              ifFalse: [ ^ true ] ]
                          ifFalse: [ ^ (val occurrencesOf: self) <= 1 ] ]
                      ifFalse: [ ^ true ] ]
                  ifFalse: [ ^ (self occurrencesOf: anObject) <= 1] ] ]
          ifFalse: [
            pathTerm hasIndexDictionary
              ifTrue: [
                " get key to look up in index dictionary "
                (pathTerm indicatesIndexOnNscElements or: [ nil == anObject ])
                  ifTrue: [ key := anObject ]
                  ifFalse: [
                    " see if a path with '*.*' in it "
                    pathTerm indicatesNsc
                      ifTrue: [
                        val := pathTerm updateDict
                          at: anObject
                          term: pathTerm getParentTerm
                          otherwise: nil.
                        ^ (BucketValueBag _hasInstance: val) not ]
                      ifFalse: [ key := pathTerm _nextObjectFor: anObject ] ].
                val := pathTerm updateDict at: key term: pathTerm otherwise: nil.	" look up the mapping in the index dictionary "
                (BucketValueBag _hasInstance: val)
                  ifTrue: [
                    " see if more than one mapping "
                    (num := val occurrencesOf: anObject) <= 1
                      ifTrue: [ ^ true ]
                      ifFalse: [
                        " see if multiple occurrences are due to more than one object
                        referencing self "
                        " if there is a parent term, it is a SetValuedPathTerm "
                        parentTerm := pathTerm getParentTerm.
                        parentTerm == nil
                          ifTrue: [ ^ false ].
                        val2 := pathTerm updateDict
                          at: anObject
                          term: parentTerm
                          otherwise: nil.
                        (BucketValueBag _hasInstance: val2)
                          ifTrue: [ ^ (val2 occurrencesOf: self) == num ]
                          ifFalse: [ ^ false ] ] ]
                  ifFalse: [ ^ true ] ]
              ifFalse: [ ^ (self occurrencesOf: anObject) <= 1] ] ] ].
  ^ true	" if we get this far, there are only incomplete indexes "
%

category: 'Indexing Support'
method: UnorderedCollection
_isRangeable: aClass

"Returns whether the given class should have an equality index created when an
 identity index is requested."

^ aClass _isSpecial and: [ (aClass isSubclassOf: AbstractCharacter) not ]
%

category: 'Testing'
method: UnorderedCollection
_levels

"If the object is implemented as a tree of private smaller objects, returns
 the depth of the tree not including the leaf nodes, otherwise returns 0.

 The result will be a SmallInteger >= 0"

^ _levels
%

category: 'Updating Indexes - Private'
method: UnorderedCollection
_lockForIndexCreation

"Locks the receiver to prevent concurrent users from invalidating index
 creation."

| lockList |
lockList := { self } .
self _indexDictionary ifNotNil:[
    self _indexDictionary _doCollisionBuckets: [ :cb | lockList add: cb ]
  ].
System writeLockAll: lockList ;
   addAllToCommitOrAbortReleaseLocksSet: lockList.
%

category: 'Deprecated'
method: UnorderedCollection
_oldAuditIndexes
  "Verifies that the index objects are consistent. _fastAuditIndexes is
 recommended over _oldAuditIndexes, because it is much, much faster and
 just as accurate.  This method is not supported. "

  self deprecated: 'UnorderedCollection>>_oldAuditIndexes deprecated v3.2, Constraints are no longer supported'.
  ^ self _incrementalAuditIndexes: 1 to: self size
%

category: 'Indexing Support'
method: UnorderedCollection
_putInWriteSet

"In some cases, it is necessary to put an NSC with indexes into the write set
 explicitly.  For example, when NO_RW_CHECKS concurrency mode is enabled, index
 creation should put the NSC in the write set to ensure that other transactions
 do not commit additions to the NSC successfully (and thus corrupting the
 internal indexing objects).  This method puts the receiver in the write set by
 writing the _indexedPaths."

self _indexedPaths: self _indexedPaths
%

category: 'Indexing Audit'
method: UnorderedCollection
_quickCheckIndexes
  "Verifies that the index objects are present. This method is does
 not catch most types of index corruption. The return values are
 messages that no indexes are present, or that indexes are okay.
 Any errors found will result in an unhandled exception.

 This method is unsupported, but is provided for customer support."

  | prevObj |
  self _indexedPaths ifNil: [ ^ 'No indexes are present.' ].	"execute minor code over all the path terms"
  self _indexedPaths rootTerms
    do: [ :rootTerm |
      rootTerm _thisAndAllChildTerms
        do: [ :aTerm |
          aTerm indicatesNsc
            ifFalse: [ aTerm isRangeEqualityIndexLastPathTerm ] ] ].
  prevObj := #'_incompletePathTraversal'.
  self
    do: [ :obj |
      obj ~~ prevObj
        ifTrue: [ obj _indexParticipationInfo ].
      prevObj := obj ].
  ^ 'Indexes are OK'
%

category: 'Error Handling'
method: UnorderedCollection
_raiseIndexingError: errArray
  "Raises an error that was due to updating indexes.  Information
 about the error is contained in the given error Array."

  | anException |
  anException := errArray at: 2.
  (errArray at: 1)
    ifTrue: [
      "indexed were modified"
      ^ IndexingErrorPreventingCommit new
        args: {anException};
        details: anException description;
        signal ]
    ifFalse: [
      "can't resignal without copy ; may have already been handled"
      ^ anException copy signal ]
%

category: 'Indexing Support'
method: UnorderedCollection
_refreshIndexCaches

"This method is currently unsupported, but is provided for customer support.

 Verifies that the index dictionary bucket caches are correct.  If an invalid
 entry is found, fixes it and reports where the fix occurred."

| result |
self _indexDictionary ~~ nil
  ifTrue: [ result := self _indexDictionary _refreshBucketCaches ].
( result == nil or: [ result isEmpty ] )
  ifTrue: [ result := 'No problems found with index caches' ].
^ result
%

category: 'Searching'
method: UnorderedCollection
_reject: aBlock

| each bag result |
bag := self _asIdentityBag.
result:= NscBuilder for: self species new max: bag size.
1 to: bag size do:[:i |
    each := (bag _at: i).
    (aBlock value: each)
        ifFalse:[ result add: each ].
].
^ result completeBag
%

category: 'Updating Indexes - Private'
method: UnorderedCollection
_removeAllRootIndexes: rootIndexes hasNonRootIndex: hasNonRootIndex

"Removes all root indexes on the receiver."

| array roots |

hasNonRootIndex ifTrue: [
    " only remove root indexes "
    1 to: rootIndexes size do: [ :i |
      self _removeIndex: (rootIndexes at: i)
    ].
    " if the receiver still has implicit indexes ... "
    self _indexedPaths ifNotNil: [ | iList |
        iList := self _indexedPaths.
        1 to: iList size by:2 do: [:i | | iObj |
           iObj := iList at: i.
           (iObj isComplete and: [ (iList at: i + 1) > 0 ]) ifTrue: [
               " and they are complete...."
               ^ self _indexParticipationInfo
             ].
         ].
      ]
  ] ifFalse: [
    array := { } .
    roots := self _indexedPaths rootTerms.
    1 to: roots size do: [ :i |
      (roots at: i) _thisAndAllChildTermsInto: array
    ].

    " this will clean up dependency tags and nil out the index list "
    self _cleanUpDependencies.

    " remove dependency lists from global table that are no longer needed "
    SharedDependencyLists removeEntriesContaining: array.
  ].
%

category: 'Searching'
method: UnorderedCollection
_select: aBlock

| each bag result |
bag := self _asIdentityBag.
result:= NscBuilder for: self species new max: bag size.
1 to: bag size do:[:i |
    each := (bag _at: i) .
    (aBlock value: each) ifTrue:[ result add: each ].
].
^result completeBag
%

category: 'Indexing Support'
method: UnorderedCollection
_setIndexDictionaryCreationSize: aNumber

"Sets the basic size of an index dictionary when a new one is being created.
 This will only be in effect for the life of the transaction.
 Returns self.  "

aNumber _validateClass: Number.
System
  rcValueCacheAt: #indexDictionarySize
  put: aNumber
  for: self.

%

category: 'Indexing Support'
method: UnorderedCollection
_setIndexDictionaryWorkingSetInterval: aNumber

"Sets the working set interval for the index dictionary when index creation
 is in progress.  This will only be in effect for the life of the transaction."

aNumber _validateClass: Number.
System
  rcValueCacheAt: #indexDictionaryWorkingSetInterval
  put: aNumber
  for: self.
^ aNumber
%

category: 'Modification Tracking'
method: UnorderedCollection
_setModificationTrackingTo: tracker

"Adds the given tracker to the receiver's indexedPaths and dependency lists."

"Must set indexedPaths first so tracker is not invoked for instance variable
 modification."
self _getIndexList addTracker: tracker.

super _setModificationTrackingTo: tracker.
%

category: 'Adding'
method: UnorderedCollection
_updateIndexesForAdditionOf: anObject logging: aBoolean
  "anObject is being added to the receiver.  Update any indexes if necessary.
 Returns true if the index objects were modified correctly; otherwise returns
 an Array containing error information."

  <primitive: 2001>
  | doLogging iList hasIndex prot |
  prot := System _protectedMode.
  [
  iList := self _indexedPaths.
  hasIndex := false.
  1 to: iList size by: 2 do: [ :j |
    " first handle modification tracking "
    (iList at: j + 1) > 0
      ifTrue: [ hasIndex := true ]
      ifFalse: [
        (iList at: j + 1) == 0
          ifTrue: [ (iList at: j) adding: anObject to: self ] ] ].
  hasIndex
    ifFalse: [ ^ true ].
  System _bypassReadAuth.
  doLogging := iList isCommitted
    and: [ UnorderedCollection _isRcIndexLoggingEnabled ].	" if the indexes are not committed, do not need to log "
  [ ^ self _doUpdateIndexesForAdditionOf: anObject iList: iList logging: aBoolean ]
    onSynchronous: Error
    do: [ :ex |
      " returns Array containing err info "
      ^ {true.	" indicating indexing objects may have been modified "
      ex} ] ]
    ensure: [
      System _exitBypassReadAuth.
      prot _leaveProtectedMode ].
^ true
%

category: 'Removing'
method: UnorderedCollection
_updateIndexesForRemovalOf: anObject

"anObject is being removed from the receiver.  Update the indexes if
 necessary.  This method is invoked prior to removal of anObject
 from the receiver. "

<primitive: 2001>
| iList hasIndex prot |
prot := System _protectedMode .
[ 	" if it is not an RcIdentityBag, anObject may not be in the receiver "
	( self class isNsc and: [ (self includes: anObject) not ] ) ifTrue: [
		^ true
	].
	iList := self _indexedPaths.
	" first handle modification tracking "
	hasIndex := false.
	1 to: iList size by: 2 do: [ :j |
		(iList at: j + 1) > 0
			ifTrue: [ hasIndex := true ]
			ifFalse: [
				(iList at: j + 1) == 0 ifTrue: [ (iList at: j) removing: anObject from: self ]
			]
	].
	hasIndex ifFalse: [ ^ true ].

	System _bypassReadAuth .
    [self _doUpdateIndexesForRemovalOf: anObject iList: iList ] onSynchronous: Error do:[ :ex |
      " returns Array containing err info "
      ^ {true.	" indicating indexing objects may have been modified "
      ex}	]
] ensure: [
	System _exitBypassReadAuth .
	prot _leaveProtectedMode
].
  ^ true
%

category: 'Reduced Conflict Support'
method: UnorderedCollection
_validateLegacyRcIndexSupport
  "legacy indexes are not supported on RcIndentitySet or RcLowMaintenanceIdentityBag (bug47179)"
%

! Class extensions for 'UserDefinedError'

!		Instance methods for 'UserDefinedError'

removeallmethods UserDefinedError
removeallclassmethods UserDefinedError

category: 'Instance initialization'
method: UserDefinedError
initialize
  gsNumber := ERR_UserDefinedError.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'UserProfile'

!		Class methods for 'UserProfile'

removeallmethods UserProfile
removeallclassmethods UserProfile

category: 'External Authentication - Private'
classmethod: UserProfile
authenticationSchemeIdToSymbol: aSmallInt

aSmallInt == 0 ifTrue:[ ^ #GemStone ] .
aSmallInt == 1 ifTrue:[ ^ #UNIX ] .
aSmallInt == 2 ifTrue:[ ^ #LDAP ] .
aSmallInt == 3 ifTrue:[ ^ #SingleSignOn ] .
aSmallInt == 4 ifTrue:[ ^ #X509  ] .
aSmallInt _validateClass: SmallInteger .
aSmallInt _error: #rtErrInvalidAuthSchemeId .
^ nil
%

category: 'External Authentication - Private'
classmethod: UserProfile
authenticationSchemeSymbolToId: aSymbol

aSymbol == #GemStone  ifTrue:[ ^ 0 ] .
aSymbol == #UNIX      ifTrue:[ ^ 1 ] .
aSymbol == #LDAP      ifTrue:[ ^ 2 ] .
aSymbol == #SingleSignOn ifTrue:[ ^ 3 ] .
aSymbol == #X509      ifTrue:[ ^ 4 ] .
aSymbol _validateClass: Symbol .
aSymbol _error: #rtErrInvalidAuthSchemeSymbol .
^ nil
%

category: 'PerformOnServer'
classmethod: UserProfile
gsSysLogFileName
 ^ '/GsSysLog/writeLogMessage'
%

category: 'Private'
classmethod: UserProfile
isSpecialUserId: aUserId

 aUserId = 'SystemUser'  ifTrue:[ ^ true ].
 aUserId = 'SymbolUser'  ifTrue:[ ^ true ].
 aUserId = 'GcUser'      ifTrue:[ ^ true ].
 aUserId = 'DataCurator' ifTrue:[ ^ true ].
 aUserId = 'Nameless'    ifTrue:[ ^ true ].
 aUserId = 'HostAgentUser'    ifTrue:[ ^ true ].
 "For 3.6,  CodeLibrarianUser no longer special."
 ^ false
%

category: 'Instance Creation'
classmethod: UserProfile
new

"Disallowed.  To create a new UserProfile, use newWithUserId:... instead."

self shouldNotImplement: #new
%

category: 'Instance Creation'
classmethod: UserProfile
newWithUserId: userIdString password: passwordString

"Creates a new UserProfile with the given password and no privileges.
 In so doing, creates a symbol list with three dictionaries: UserGlobals,
 Globals, and Published.  The first Dictionary (UserGlobals) is created
 for the user's private symbols.  Returns the new UserProfile.

 Adds the new UserProfile to AllUsers.  Creates the new UserProfile as a member
 of the group Subscribers.

 Generates an error if passwordString is equivalent to userIdString ignoring
 case (equalsNoCase:).

 The new UserProfile will have the default object security policy, which is nil.
"

^ self newWithUserId: userIdString password: passwordString
  defaultObjectSecurityPolicy: nil privileges: #() inGroups: #()
%

category: 'Instance Creation'
classmethod: UserProfile
newWithUserId: userIdString password: passwordString defaultObjectSecurityPolicy: aPolicy

"Creates a new UserProfile with the given password and no privileges.
 In so doing, creates a symbol list with three dictionaries: UserGlobals,
 Globals, and Published.  The first Dictionary (UserGlobals) is created
 for the user's private symbols.  Returns the new UserProfile.

 Adds the new UserProfile to AllUsers.  Creates the new UserProfile as a member
 of the group Subscribers and of the groups in aCollectionOfGroupStrings.

 Generates an error if passwordString is equivalent to userIdString ignoring
 case (equalsNoCase:).

 aPolicy must be nil or a committed instance of GsObjectSecurityPolicy.
"

^ self newWithUserId: userIdString password: passwordString
  defaultObjectSecurityPolicy: aPolicy privileges: #() inGroups: #()
%

category: 'Instance Creation'
classmethod: UserProfile
newWithUserId: userIdString
password: passwordString
defaultObjectSecurityPolicy: anObjectSecurityPolicy
privileges: anArrayOfStrings
inGroups: groupStringsOrObjs

self _validatePrivilege ifFalse:[ ^ nil ].
self _validate__password: passwordString forUserId: userIdString .
^ self _newWithUserId: userIdString
  password: passwordString
  defaultObjectSecurityPolicy: anObjectSecurityPolicy
  privileges: anArrayOfStrings
  inGroups: groupStringsOrObjs
%

category: 'Deprecated'
classmethod: UserProfile
newWithUserId: userIdString
password: passwordString
defaultSegment: anObjectSecurityPolicy
privileges: anArrayOfStrings
inGroups: aCollectionOfGroupStrings

"Deprecated - use methods in the Instance Creation category instead"

self deprecated: 'UserProfile class>>newWithUserId:password:defaultSegment:privileges:inGroups: deprecated v3.0.
Use #newWithUserId:password:defaultObjectSecurityPolicy:privileges:inGroups: instead.'.

^self
	newWithUserId: userIdString
	password: passwordString
	defaultObjectSecurityPolicy: anObjectSecurityPolicy
	privileges: anArrayOfStrings
	inGroups: aCollectionOfGroupStrings
%

category: 'Instance Creation'
classmethod: UserProfile
newWithUserId: userIdString password: passwordString privileges: anArrayOfStrings inGroups: aCollectionOfGroupStrings

"Creates a new UserProfile with the given password and the given privileges.
 In so doing, creates a symbol list with three dictionaries: UserGlobals,
 Globals, and Published.  The first Dictionary (UserGlobals) is created
 for the user's private symbols.  Returns the new UserProfile.

 Adds the new UserProfile to AllUsers.  Creates the new UserProfile as a member
 of the group Subscribers and of the groups in aCollectionOfGroupStrings.

 Generates an error if passwordString is equivalent to userIdString ignoring
 case (equalsNoCase:).

 The new UserProfile will have the default object security policy, which is nil.
"

^ self newWithUserId: userIdString password: passwordString
  defaultObjectSecurityPolicy: nil privileges: anArrayOfStrings inGroups: aCollectionOfGroupStrings
%

category: 'Instance Creation'
classmethod: UserProfile
newX509WithUserId: userIdString
"Creates a new UserProfile for which authenticationSchemeIsX509 will return true.
 The new UserProfile has no privileges,
 and has a symbol list with three dictionaries: UserGlobals,
 Globals, and Published.  The first Dictionary (UserGlobals) is created
 for the user's private symbols.  Returns the new UserProfile.

 Adds the new UserProfile to AllUsers.  Creates the new UserProfile as a member
 of the group Subscribers.

 The new UserProfile will have the default object security policy, which is nil.
"

^ self _newWithUserId: userIdString password: 509
  defaultObjectSecurityPolicy: nil privileges: #() inGroups: #()
%

category: 'Accessing'
classmethod: UserProfile
privilegeNames
  ^ PrivilegeNames
%

category: 'External Authentication - Private'
classmethod: UserProfile
_initPrivilegeNames
  classVars
    at: #PrivilegeNames
        put:
           #(#SystemControl #ObsoleteStatistics #SessionAccess #UserPassword
             #DefaultObjectSecurityPolicy #PrivUnused5 #OtherPassword
             #ObjectSecurityPolicyCreation #ObjectSecurityPolicyProtection #FileControl
             #GarbageCollection #CodeModification
             #NoPerformOnServer #NoUserAction
             #NoGsFileOnServer #NoGsFileOnClient
             #SessionPriority #CompilePrimitives #ChangeUserId
             #MigrateObjects #DisableObjectReadLogging  
             #DynamicDisableObjectReadLogging #ReadOtherUserProfile) ;
    at: #InversePrivileges put:
        #( NoPerformOnServer NoUserAction NoGsFileOnServer NoGsFileOnClient ).
%

category: 'Private'
classmethod: UserProfile
_newWithUserId: userIdString
password: passwordArg
defaultObjectSecurityPolicy: anObjectSecurityPolicy
privileges: anArrayOfStrings
inGroups: groupStringsOrObjs

"Creates a new UserProfile with the associated characteristics.  In so doing,
 creates a symbol list with three dictionaries: UserGlobals, Globals, and
 Published.  The first Dictionary (UserGlobals) is created for the user's
 private symbols.  Returns the new UserProfile.

 Adds the new UserProfile to AllUsers.  Creates the new UserProfile as a member
 of the group Subscribers and of the groups in groupStringsOrObjs.

 groupStringsOrObjs must be a collection of either Strings that represent group names,
 or UserProfileGroup objects.

 Generates an error if passwordString is equivalent to userIdString ignoring
 case (equalsNoCase:).

 anObjectSecurityPolicy must be nil or a committed GsObjectSecurityPolicy.
"

| result newDict packagePolicy newSymList myAssociations mySymbols
  userGroups newPrivSet publ |

self _validatePrivilege ifFalse:[ ^ nil ].
(self _validateUserId: userIdString ) ifFalse:[ ^ nil ].
passwordArg ~~ 509 ifTrue:[
  self _validate__password: passwordArg forUserId: userIdString .
].
anObjectSecurityPolicy == nil ifFalse:[
  (anObjectSecurityPolicy _validateClass: GsObjectSecurityPolicy ) ifFalse:[ ^ nil ].
  anObjectSecurityPolicy isCommitted ifFalse:[
    anObjectSecurityPolicy _error: #rtErrObjMustBeCommitted .
    ^ nil
  ].
].
newPrivSet := UserProfile _privilegesSetFromArray: anArrayOfStrings .

(groupStringsOrObjs _validateClass: Collection) ifFalse:[ ^ nil ].
userGroups := { }  .
groupStringsOrObjs do: [ :aStrOrGroup | | groupObj |
  (aStrOrGroup isKindOf: CharacterCollection)
     ifTrue:[ groupObj := UserProfileGroup _validateGroupString: aStrOrGroup ]
    ifFalse:[ aStrOrGroup _validateClass: UserProfileGroup.
              groupObj := aStrOrGroup ] .
  userGroups add: groupObj .
  ].
"by default all users are a member of Subscribers."
userGroups add: (UserProfileGroup groupWithName: 'Subscribers').

    "Create the new UserProfile and populate it using the given arguments."

GsObjectSecurityPolicy setCurrent: (Globals at: #AllUsers) objectSecurityPolicy  while:[
  | theUserId |
  result := super new _initialize .
  theUserId := userIdString asString copy immediateInvariant .
  result _userId: theUserId .
  result privileges: anArrayOfStrings.
].
result defaultObjectSecurityPolicy: anObjectSecurityPolicy .

   "Create UserGlobals Dictionary "
newDict := SymbolDictionary new.
newDict objectSecurityPolicy: anObjectSecurityPolicy .

   "Create GsPackagePolicy current (fix 41433)"
packagePolicy := GsPackagePolicy new.
packagePolicy objectSecurityPolicy: anObjectSecurityPolicy.

   "Create all SymbolAssociations and Symbols and UserGlobals."

myAssociations := { } .
mySymbols := { } .
myAssociations add: (SymbolAssociation newWithKey: #UserGlobals
                                       value: newDict).
myAssociations add: (SymbolAssociation newWithKey: GsPackagePolicy globalName
                                       value: packagePolicy).

myAssociations do:[:assoc|
  assoc objectSecurityPolicy: anObjectSecurityPolicy .
  newDict add: assoc
].

"Create the new user's symbol list."

newSymList := SymbolList new .
newSymList add: newDict; add: Globals .
"fix 40939"
publ := (AllUsers userWithId: 'DataCurator') symbolList objectNamed: #Published.
publ ifNotNil: [newSymList add: publ].
result symbolList: newSymList.
newSymList objectSecurityPolicy: anObjectSecurityPolicy.

GsObjectSecurityPolicy setCurrent: (Globals at: #AllUsers) objectSecurityPolicy  while:[
  AllUsers add: result .  "add to AllUsers after all of above succeeded"
    " password must be assigned after adding to AllUsers"
  [
    passwordArg _isOneByteString ifTrue:[ result password: passwordArg ]
      ifFalse:[ passwordArg == 509 ifTrue:[ result enableX509Authentication]
                     ifFalse:[ ArgumentError signal:'invalid password argument']].

    userGroups do: [:aGrp | result addToUserProfileGroup: aGrp ]
  ] on: Error do:[:ex |
    AllUsers removeAndCleanup: result .
    ex pass .
    ^ nil
  ]
].
^ result
%

category: 'Private'
classmethod: UserProfile
_privilegeMask: aPrivilegeString

| index privSet privSymbol |
privSet := UserProfile _privilegesSetFromArray: { aPrivilegeString }.
privSymbol := privSet _at: 1 .
(index := PrivilegeNames indexOf: privSymbol ) == 0 ifTrue:[
  aPrivilegeString _error: #rtErrBadPriv .
  ^ self
].
^ 1 bitShift: index - 1
%

category: 'Private'
classmethod: UserProfile
_privilegesSetFromArray: anArrayOfStrings

"Convert an Array of Strings to an IdentitySet of legal Privilege names.
 Raises an error if anArrayOfStrings is not an Array, or if any String
 within anArrayOfStrings cannot be translated to a canonical Symbol
 of a legal privilege name, otherwise returns an IdentitySet of Symbols."

| result |
(anArrayOfStrings _validateClass: Array) ifFalse:[ ^ nil ].
result := IdentitySet new .
anArrayOfStrings do:[ :aString | | aSymbol |
  aSymbol :=  Symbol _existingWithAll: aString .
  aSymbol ifNil:[ aString _error: #rtErrBadPriv ].
  "translate Segment for compatibility."
  aSymbol == #DefaultSegment ifTrue:[ aSymbol := #DefaultObjectSecurityPolicy ] .
  aSymbol == #SegmentCreation ifTrue:[ aSymbol := #ObjectSecurityPolicyCreation ] .
  aSymbol == #SegmentProtection ifTrue:[ aSymbol := #ObjectSecurityPolicyProtection ] .
  (PrivilegeNames includesIdentical: aSymbol) ifFalse:[
    aString _error: #rtErrBadPriv .
    ^ self
  ].
  result add: aSymbol
].
^ result
%

category: 'External Authentication - Private'
classmethod: UserProfile
_validateAuthenticationScheme: aSymbol

^ (self authenticationSchemeSymbolToId: aSymbol ) ~~ nil
%

category: 'Private'
classmethod: UserProfile
_validatePrivilege

" You must have the #OtherPassword to modify UserProfiles "

  ^ System myUserProfile _validatePrivilegeName: #OtherPassword
%

category: 'Private'
classmethod: UserProfile
_validateUserId: aString
  | sz specialChars | 
  aString _validateKindOfClass: String .
  aString _isOneByteString ifFalse:[ 
     ArgumentError signal:'userId must be a kind of String with codePoints <= 255'.
     ^ false .
  ].
  (sz := aString size) == 0 ifTrue:[ 
    ArgumentError signal:'userId may not be an empty String' . ^ false 
  ].
  sz > 1024 ifTrue:[ ArgumentError signal:'userId exceeds 1024 characters'. ^ false ].
  specialChars := #( $_ $. $- $  ) .
  1 to: sz do:[:n | | ch | ch := aString at: n .
      (ch isAlphaNumeric or:[ specialChars includesIdentical: ch] ) ifFalse:[
      ArgumentError signal:'userId contains invalid Character(s)'.
      ^ false .
    ].   
  ].
  ^ true 
%

category: 'Private'
classmethod: UserProfile
_validate__password: aString forUserId: aUserIdString

  (aString _validateClass: String) ifFalse:[ ^ false ] .

  "disallow aUserIdString as a password"
  (aString equalsNoCase: aUserIdString) ifTrue:[
    aString _error: #rtDisallowedPassword . ^ false
  ].
  aString size > 1024 ifTrue:[
   aString _error: #rtMaxPasswordSize args: { 1024 } . ^ false
  ].
  "disallow empty string as a password"
  (aString equalsNoCase: '') ifTrue:[
    aString _error: #rtMinPasswordSize args: { 1 } . ^ false
  ].

  ^ true
%

!		Instance methods for 'UserProfile'

category: 'Accessing'
method: UserProfile
activeUserIdLimit

"Returns the maximum number of concurrent logins allowed for the receiver.

 Generates an error if the receiver is not your UserProfile and 
 you have neither the #OtherPassword nor #ReadOtherUserProfile privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileRead ifTrue:[
    result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_ACTIVE_USR_LIM  .
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Updating'
method: UserProfile
activeUserIdLimit: aPositiveInteger

"Sets the maximum number of concurrent logins for the receiver to be
 aPositiveInteger.  This change will take effect after this session commits.

 Generates an error if you do not have the #OtherPassword privilege."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
    aPositiveInteger ifNotNil:[
      aPositiveInteger _isSmallInteger ifFalse:[
        ^ aPositiveInteger _validateClass: SmallInteger .
      ].
      aPositiveInteger < 0 ifTrue:[
        ^ aPositiveInteger _error: #rtErrArgNotPositive
      ].
    ].
    self _securityDataInstVarAt: OC_USER_SECUR_DATA_ACTIVE_USR_LIM put: aPositiveInteger .
  ].
] ensure:[
  prot _leaveProtectedMode
]
%

category: 'Group Membership'
method: UserProfile
addGroup: aGroupString

"Adds the user to the group with the name aGroupString, and returns the
 receiver.  If the user already belongs to the group aGroupString, no action
 occurs.

 If the group with name aGroupString is not defined in UserProfileGroup,
 generates an error.

 Legacy method; use addToUserProfileGroup: or UserProfileGroup >> addUser: instead."

| theGroup |
(theGroup := UserProfileGroup _validateGroupString: aGroupString) ifNil:[ ^ nil ] .
^self addToUserProfileGroup: theGroup
%

category: 'PerformOnServer'
method: UserProfile
addPerformOnServerCommand: aString

"Adds the command in aString to the list of commands the receiver is permitted
 to execute when calling either the System(C)>>performOnServer: or
 GsHostProcess>>fork methods when the receiver has the NoPerformOnServer
 privilege. aString must be an instance of String which represents the full
 path  (the first character of the string must be a forward slash [/]) to a
 file with execute permission. Command paths may not contain whitespace
 characters.

 Returns the receiver on success. Raises an exception if the caller does not have
 the OtherPassword privilege, if aString contains whitespace characters, 
 or is not an instance of String. Raises an exception if the command string is
 already present in the list."

^ self addPerformOnServerCommands: { aString }
%

category: 'PerformOnServer'
method: UserProfile
addPerformOnServerCommands: anArray

"Adds the commands in anArray to the list of commands the receiver is permitted
 to execute when calling either the System(C)>>performOnServer: or
 GsHostProcess>>fork methods when the receiver has the NoPerformOnServer
 privilege. Each element of anArray must be a non-empty instance of String,
 which represents the full path  (the first character of the string must
 be a forward slash [/]) to a file with execute permission. Command paths may
 not contain whitespace characters.

 Returns the receiver on success. Raises an exception if the caller does not have
 the OtherPassword privilege, if any element of anArray is empty, contains
 whitespace characters, or is not an instance of String.
 Raises an exception if any command string in anArray is already in the list."
 
^ self _primUpdatePerformOnServerCommands: anArray opCode: 1
%

category: 'Updating Privileges'
method: UserProfile
addPrivilege: aPrivilegeString

"Adds aPrivilegeString to the receiver's privileges.  Generates an error if
 aPrivilegeString is not a valid privilege name (as defined in the class
 variable PrivilegeNames).

 Only SystemUser may add privileges to special UserProfiles. Attempts which 
 violate this restriction will raise an exception.

 Example: 'System myUserProfile addPrivilege: #GarbageCollection' "

<primitive: 2001> "enter protected mode"
| prot |
prot := System _protectedMode .
[ self _validatePrivilege ifTrue:[
    self _addPrivilege: aPrivilegeString.
  ].
] ensure:[
  prot _leaveProtectedMode
]
%

category: 'Group Membership'
method: UserProfile
addToUserProfileGroup: aUserProfileGroup

"Adds the user to the UserProfileGroup. If the user already belongs to
 the group, no action occurs."

self _validatePrivilege ifTrue:[
    | userGroups |
    userGroups := groups .
    userGroups == nil ifTrue:[
      userGroups := IdentitySet new .
      userGroups objectSecurityPolicy: self objectSecurityPolicy .
    ].
  (userGroups includesIdentical: aUserProfileGroup) ifFalse:[
    groups ifNil:[ groups := userGroups ].
    groups add: aUserProfileGroup.
    aUserProfileGroup add: self .
    ] .
].
%

category: 'Group Membership'
method: UserProfile
auditGroups
"Audits the receiver's groups collection to ensure:
  -every UserProfileGroup in the collection contains the receiver.
  -every UserProfileGroup in the collection is present in the AllGroups global.

Raises an error if the audit fails and returns true if successful."

groups do:[:aGroup| |name obj|
  name := aGroup groupName .
  obj := AllGroups at: name ifAbsent:[ aGroup _error: 'Group is missing from AllGroups'. ^ false ].
  obj == aGroup
    ifFalse:[ aGroup error: 'Non-canonical group found' . ^ false ] .
  (aGroup includes: self)
    ifFalse: [ aGroup error: 'Inconsistent group state' . ^ false ] .
].
^ true
%

category: 'External Authentication'
method: UserProfile
authenticationScheme

"Returns a Symbol representing the authentication scheme for the receiver,
 result is one of #GemStone, #UNIX, #LDAP, #SingleSignOn , #X509 .

 Generates an error if the receiver is not your UserProfile and 
 you have neither the #OtherPassword nor #ReadOtherUserProfile privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[ self _validateUserProfileRead ifTrue:[
    result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_AUTH_SCHEME.
  ]
] ensure:[
  prot _leaveProtectedMode  "exit protected mode"
].
^ self class authenticationSchemeIdToSymbol: result
%

category: 'External Authentication - Private'
method: UserProfile
authenticationScheme: aSymbol

"Private - customers should not call this method directly.

 Sets the athentication scheme for the receiver to match aSymbol, which
 must be one of  #GemStone, #UNIX, #LDAP, #SingleSignOn, or #X509.
 The system user profiles, SystemUser, DataCurator, GcUser, SymbolUser and Nameless,
 must always use 'GemStone' authentication.  Attempts to change the authentication
 scheme for these UserProfiles will raise an error.

 Requires the OtherPassword privilege."

self _securityDataInstVarAt: OC_USER_SECUR_DATA_AUTH_SCHEME
        put: (self class authenticationSchemeSymbolToId: aSymbol).
^ self
%

category: 'External Authentication'
method: UserProfile
authenticationSchemeIsGemStone

"Answer true if the receiver uses GemStone for password authentication.
 Otherwise answer false.

 Requires the #OtherPassword privilege for any UserProfile other than
 your own."

^ self authenticationScheme == #GemStone
%

category: 'External Authentication'
method: UserProfile
authenticationSchemeIsLDAP

"Answer true if the receiver uses LDAP for password authentication.
 Otherwise answer false.

 Requires the #OtherPassword privilege for any UserProfile other than
 your own."

^ self authenticationScheme == #LDAP
%

category: 'External Authentication'
method: UserProfile
authenticationSchemeIsSingleSignOn

"Answer true if the receiver uses Single Sign On (Kerberos) for password authentication.
 Otherwise answer false.

 Requires the #OtherPassword privilege for any UserProfile other than
 your own."

^ self authenticationScheme == #SingleSignOn
%

category: 'External Authentication'
method: UserProfile
authenticationSchemeIsUNIX

"Answer true if the receiver uses UNIX for password authentication.
 Otherwise answer false.

 Requires the #OtherPassword privilege for any UserProfile other than
 your own."

^ self authenticationScheme == #UNIX
%

category: 'External Authentication'
method: UserProfile
authenticationSchemeIsX509

"Answer true if the receiver uses X509 certificates for authentication.
 Otherwise answer false.

 Requires the #OtherPassword privilege for any UserProfile other than
 your own."

^ self authenticationScheme == #X509
%

category: 'Enumerating'
method: UserProfile
classesDo: aBlock

"Execute 3-argument block over all classes (plus those referenced by
 its class history) that are contained by all SymbolDictionaries in
 this user's symbol list.  Checks are made to avoid  redundant processing
 of classes/symbol dictionaries already scanned.

 Arguments to block are:
    [:aUserProfile :aSymbolDictionary :aClass | ... ]

"

  | coveredSymDics coveredClasses |

  coveredSymDics := IdentitySet new.
  coveredClasses := IdentitySet new.

  self symbolList do: [ :aSymbolDictionary |
    (coveredSymDics includes: aSymbolDictionary) ifFalse: [
      coveredSymDics add: aSymbolDictionary.
      aSymbolDictionary valuesDo: [ :namedClass |
        (namedClass isKindOf: Class) ifTrue: [
          namedClass classHistory do: [ :aClass |
            (coveredClasses includes: aClass) ifFalse: [
              coveredClasses add: aClass.
              aBlock value: self
                     value: aSymbolDictionary
                     value: aClass ]]]]]]
%

category: 'Updating'
method: UserProfile
clearOldPasswords

"Clears the set of old passwords for the receiver, thus permitting reuse of
 some passwords that had been previously disallowed.

 Generates an error if you do not have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
    self _securityDataInstVarAt: OC_USER_SECUR_DATA_USED_VERIFIERS put: { } . "old salts"
    self _securityDataInstVarAt: OC_USER_SECUR_DATA_USED_SALTS put: { } . "old verifiers"
    result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_OLD_PASSWDS_ARRAY put: { } .
  ]
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Clustering'
method: UserProfile
clusterDepthFirst

"This method clusters the receiver's user ID, password, and symbol list.
 Because the password and user ID are assumed to be byte objects, and because
 the symbol list may contain shared elements, no further traversal of the
 objects is done.  Returns true if the receiver has already been clustered
 during the current transaction; returns false otherwise."

self cluster
  ifTrue: [ ^ true ]
  ifFalse: [
    encryptedPassword cluster.
    userId cluster.
    symbolList cluster.
    ^ false
  ]
%

category: 'Updating'
method: UserProfile
createDictionary: aSymbol

"Creates a new SymbolDictionary. The new Dictionary is created with a single
 SymbolAssociation, whose key is aSymbol and whose value is the new
 SymbolDictionary itself. It is recommended that the symbol be a valid
 identifer, though it is not disallowed to use any kind of symbol.

 Also creates a SymbolAssociation in the receiver's UserGlobals dictionary, with
 aSymbol as the key and the new dictionary as its value.

 Returns the new SymbolDictionary. Generates an error if aSymbol is already
 defined in the receiver's symbol list, or if aSymbol is not a Symbol."

| newDictionary anAssoc userGlobalsOfMe |

self _validatePrivilege ifFalse:[ ^ nil ].
(aSymbol _validateClass: Symbol) ifFalse:[ ^ nil ].

anAssoc := self resolveSymbol: aSymbol.
 (anAssoc == nil)
    ifFalse:[ ^ self _error: #rtErrSymAlreadyDefined args: { aSymbol }].

newDictionary := (SymbolDictionary new).  "Create new dictionary"
"Name it by making a reference to itself"
newDictionary at: aSymbol put: newDictionary.

userGlobalsOfMe := (self resolveSymbol: #UserGlobals) value.
"Get my UserGlobals"
userGlobalsOfMe at: aSymbol put: newDictionary.

^newDictionary
%

category: 'Accessing'
method: UserProfile
defaultObjectSecurityPolicy

"Returns the default login GsObjectSecurityPolicy associated with the receiver."

^ defaultObjectSecurityPolicy
%

category: 'Updating'
method: UserProfile
defaultObjectSecurityPolicy: anObjectSecurityPolicy

"Redefines the default login GsObjectSecurityPolicy associated with the receiver,
 and returns the receiver.  The argument may be nil (which implies World write for
 all newly created objects).  The new value will take effect for subsequent
 logins using the receiver.

 If the receiver is the UserProfile under which this session is logged in,
 this method requires the DefaultObjectSecurityPolicy privilege.

 If the receiver is not the UserProfile under which this session is logged
 in, you must have write authorization for the GsObjectSecurityPolicy where the receiver
 resides and the #OtherPassword privilege.

 Exercise extreme caution when executing this method.  If, at the time you commit your
 transaction, the receiver no longer had write authorization for anObjectSecurityPolicy,
 that user's GemStone session will be terminated and the user will be unable to log back
 in to GemStone."

anObjectSecurityPolicy ifNotNil:[
  (anObjectSecurityPolicy _validateClass: GsObjectSecurityPolicy) ifFalse:[ ^ nil].
] .
(self == System myUserProfile) ifTrue: [
  ^ self _defaultObjectSecurityPolicy: anObjectSecurityPolicy
] ifFalse:[
  self _validatePrivilege ifFalse:[ ^ nil ].
].
defaultObjectSecurityPolicy := anObjectSecurityPolicy .
%

category: 'Deprecated'
method: UserProfile
defaultSegment

self deprecated: 'UserProfile>>defaultSegment deprecated v3.0.
Use #defaultObjectSecurityPolicy instead.'.
^self defaultObjectSecurityPolicy.
%

category: 'Deprecated'
method: UserProfile
defaultSegment: anObjectSecurityPolicy

 self deprecated: 'UserProfile>>defaultSegment: deprecated v3.0.
Use #defaultObjectSecurityPolicy: instead.'.
 ^ self defaultObjectSecurityPolicy: anObjectSecurityPolicy
%

category: 'Updating Privileges'
method: UserProfile
deletePrivilege: aPrivilegeString

"Removes aPrivilegeString from the receiver's privileges.  Generates an error
 if aPrivilegeString is not a valid privilege name (as defined in the class
 variable PrivilegeNames).

 Only SystemUser may delete privileges from special UserProfiles.
 Attempts which violate this restriction will raise an exception."

"Example: 'System myUserProfile deletePrivilege: #GarbageCollection'"

<primitive: 2001> "enter protected mode"
| prot |
prot := System _protectedMode .
[ self _validatePrivilege ifTrue:[
    self _deletePrivilege: aPrivilegeString.
  ].
] ensure:[
  prot _leaveProtectedMode
]
%

category: 'Accessing the Symbol List'
method: UserProfile
dictionariesAndSymbolsOf: anObject

"Returns a Array, possibly empty, of the form { { aDictionary . aSymbol } ... }.
 Searches the receiver for Associations whose value is identical to
 anObject and returns information on all matching Associations ."

  ^ symbolList dictionariesAndSymbolsOf: anObject
%

category: 'Accessing the Symbol List'
method: UserProfile
dictionaryAndSymbolOf: anObject

"Returns the symbol list Dictionary that names anObject, and also returns the
 name which that Dictionary associates with anObject.  More precisely, this
 returns an Array containing two elements:

 * The Dictionary in the user's symbol list that contains an Association whose
   value is anObject.
 * The Symbol which is that Association's key.

 The symbol list is searched in the same order that the compiler searches it.
 (For more information about symbol resolution, see the GemStone Programming
 Guide.)  If anObject is not found in the symbol list, returns nil."

self deprecated: 'UserProfile class>>dictionaryAndSymbolOf: deprecated v3.1, use dictionariesAndSymbolsOf:' .

^ symbolList dictionaryAndSymbolOf: anObject
%

category: 'Accessing the Symbol List'
method: UserProfile
dictionaryNames

"Returns a formatted String describing the position and name of each Dictionary
 in the receiver's symbol list.

 This method assumes that each Dictionary in the symbol list contains an
 Association whose value is that Dictionary.  If any Dictionary does not
 contain such an Association, it is represented in the result String as
 '(unnamed Dictionary)'."

^ symbolList namesReport
%

category: 'Updating'
method: UserProfile
disable

"Disables the receiver when the transaction is committed.
 Generates an error if you do not have OtherPassword privilege.
 Returns the receiver."

^ self disableWithReason: 'Disabled by administrator'
%

category: 'Updating Privileges'
method: UserProfile
disableCommits

"Disables commits for the receiver if they were previously disabled.
 The current transaction must be committed to apply this change.

 Requires the #OtherPassword privilege.

 Has no effect on existing sessions.

 Commits will be disabled for the receiver at the next login attempt
 which occurs AFTER the current transaction is committed.

 Disabling commits for the reserved system accounts
 (SystemUser, DataCurator, GcUser, SymbolUser and Nameless) is not allowed
 and will always fail.

 Returns true if the operation was successful or if commits were already
 disabled for the receiver.  Returns false if the operation failed."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[ result := true.
  self _validatePrivilege ifTrue:[
    (self class isSpecialUserId: userId)
      ifTrue:[ result := false] "illegal"
      ifFalse:[
        self isReadOnly
          ifFalse:[ result := (self _securityDataInstVarAt: OC_USER_SECUR_DATA_READ_ONLY put: true) == true]].
  ].
] ensure:[
    prot _leaveProtectedMode "exit protected mode"
].
^ result
%

category: 'Updating'
method: UserProfile
disableLoginLogging

"Disables the printing of login/logout events for the receiver to the
 stone's login log file. Has no effect (i.e. logins still not logged)
 if the stone does not have login logging enabled using
 STN_LOGIN_LOG_ENABLED.

 This method requires the #OtherPassword privilege.

 Returns the receiver."

^ self exemptFromLoginLogging: true
%

category: 'PerformOnServer'
method: UserProfile
disableSystemLogAccess

"Disables the receiver from writing messages to the system log via executing 
 privileged methods in class GsSysLog. Has no effect unless the receiver also has
 the NoPerformOnServer privilege.

 Returns the receiver on success. Raises an exception if the caller does not have
 the OtherPassword privilege or if the receiver does not have GsSysLog access."

^ self removePerformOnServerCommand: self class gsSysLogFileName
%

category: 'Updating'
method: UserProfile
disableWithReason: aString

"Disables the receiver when the transaction is committed and specifies aString
 as the reason.  Generates an error if you do not have OtherPassword privilege.
 Returns the receiver."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  self _validateUserProfileWrite ifTrue:[
    (aString _validateClasses: { String } ) ifFalse:[ ^ nil ].
    self _securityDataInstVarAt: OC_USER_SECUR_DATA_VERIFIER put: nil.
    self _securityDataInstVarAt: OC_USER_SECUR_DATA_REASON_DISABLED put: aString copy immediateInvariant .
  ]
] ensure:[
  prot _leaveProtectedMode
].
^ self
%

category: 'Updating Privileges'
method: UserProfile
enableCommits

"Enables commits for the receiver if they were previously disabled.
 The current transaction must be committed to apply this change.

 Requires the #OtherPassword privilege.

 Has no effect on existing sessions.

 Commits will be reenabled for the receiver at the next login attempt
 which occurs AFTER the current transaction is committed.

 Disabling commits for the reserved system accounts
 (SystemUser, DataCurator, GcUser, SymbolUser and Nameless) is not allowed.
 Therefore enabling commits for these user profiles will always succeed.

 Returns true if the operation was successful or if commits were already
 enabled for the receiver."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[ result := true.
  self _validatePrivilege ifTrue:[
     (self class isSpecialUserId: userId)
     ifFalse:[
      self isReadOnly
        ifTrue:[  result := (self _securityDataInstVarAt: OC_USER_SECUR_DATA_READ_ONLY put: false) == false]].
  ]
] ensure:[
  prot _leaveProtectedMode "exit protected mode"
].
^ result
%

category: 'External Authentication'
method: UserProfile
enableGemStoneAuthenticationWithPassword: aPassword

"Modify the receiver so that GemStone authentication is used to validate
 logins to GemStone by the receiver.  After this method has
 been run and successfully committed, all future logins of the receiver
 must specify a valid GemStone password.

 Requires the OtherPassword privilege."

<primitive: 2001>
| prot |
prot := System _protectedMode .

[ self _validatePrivilege ifTrue:[
    self _validate__password: aPassword . "fix 47327"
    self authenticationScheme: #GemStone  .
    self userIdAlias: nil.
    self kerberosPrincipal: nil.
    self _setPassword: aPassword .
  ]
] ensure:[
prot _leaveProtectedMode  "exit protected mode"].

^ self
%

category: 'External Authentication'
method: UserProfile
enableLDAPAuthenticationWithAlias: aString baseDn: baseDn filterDn: filterDn

"Modify the receiver so that LDAP authentication is used to validate
 all logins to GemStone by the receiver.  After this method has
 been run and successfully committed, all future logins of the receiver
 must specify the password for the given LDAP user ID.  GemStone will then
 validate the password using an LDAP bind.

 If aString is nil, then the UNIX user ID and the UserProfile
 ID of the receiver are identical.  At the next login, an LDAP bind
 will be performed using the user ID of the receiver and the password
 specified during the login.

 Most LDAP binds require a fully qualified DN (distinguished name) to be
 specified in order to get a successful bind.  There are 2 ways to get
 thee fully qualified DN: specify it as baseDn, or have GemStone perform
 a query to resovle the DN before attempting the bind.

 If filterDn is aString, this filter will be used to search for the fully
 qualified DN, and the string must include the character sequence '%s' in
 place of the user ID.  The baseDn argument must not contain the '%s'
 sequence in this case.

 If no DN search is to be performed, then nil should be passed in the filterDn
 argument and the baseDn string must contain the character sequencey '%s'
 where the userId is to be inserted in the string.

 Refer to the LDAP section of the System Administrators Guide for more
 information on configuring GemStone for LDAP authentication.

 Special UserProfiles may not use LDAP authentication.  Attempting
 to change the authentication scheme to LDAP for a special UserProfile will
 raise an error.  See the method #isSpecialUserId: in this class to see which
 UserProfiles are considered special.

 Requires the OtherPassword privilege."

<primitive: 2001>
| prot |
prot := System _protectedMode .

[ self _validatePrivilege ifFalse:[ ^ nil ].
  (self _validateBaseDn: baseDn searchDn: filterDn ) ifTrue:[
    aString ifNotNil:[
      (aString _validateClass: String) ifFalse:[ ^ nil ].
               self userIdAlias: aString
    ].
    self kerberosPrincipal: nil .
    self ldapBaseDn: baseDn .
    filterDn ifNotNil:[ self ldapFilterDn: filterDn ].
    self authenticationScheme: #LDAP .
    self _enableNonGemStoneAuthentication .
  ]
] ensure:[ prot _leaveProtectedMode  "exit protected mode"].

^ self
%

category: 'Updating'
method: UserProfile
enableLoginLogging

"Enables the printing of login/logout events for the receiver to the
 stone's login log file. Has no effect (i.e. logins still not logged)
 if the stone does not have login logging enabled using
 STN_LOGIN_LOG_ENABLED.

 This method requires the #OtherPassword privilege.

 Returns the receiver."

^ self exemptFromLoginLogging: false
%

category: 'External Authentication'
method: UserProfile
enableSingleSignOnAuthenticationWithPrincipal: aKerberosPrincipal

"Modify the receiver so that single sign on (SSO) authentication is
 required to validate all logins to GemStone by the receiver.  After this method
 has been run and successfully committed, all future logins of the receiver
 must be authenticated by the Kerberos single sign on mechanism. No password may
 be specified during the GemStone login.

 Refer to the SSO section of the System Administrators Guide for more
 information on configuring GemStone for SSO authentication.

 aKerberosPrincipal must be an instance of KerberosPrincipal or nil.

 SystemUser must always use GemStone authentication.  Attempting
 to change the authentication scheme for a SystemUser will raise
 an error.

 Requires the OtherPassword privilege."

<primitive: 2001>
| prot |
prot := System _protectedMode .

[ self _validatePrivilege ifFalse:[ ^ nil ].
  aKerberosPrincipal ifNotNil:[
     aKerberosPrincipal _validateClass: KerberosPrincipal.
     self kerberosPrincipal: aKerberosPrincipal .
  ].
  self userIdAlias: nil .
  self ldapBaseDn: nil .
  self ldapFilterDn: nil .
  self authenticationScheme: #SingleSignOn .
  self _enableNonGemStoneAuthentication .
] ensure:[ prot _leaveProtectedMode  "exit protected mode"].

^ self
%

category: 'PerformOnServer'
method: UserProfile
enableSystemLogAccess

"Allows the receiver to write messages to the system log by executing privileged methods
 in class GsSysLog. Has no effect unless the receiver also has the NoPerformOnServer
 privilege.

 Returns the receiver on success. Raises an exception if the caller does not have
 the OtherPassword privilege or if the receiver already has GsSysLog access."

^ self addPerformOnServerCommand: self class gsSysLogFileName
%

category: 'External Authentication'
method: UserProfile
enableUnixAuthenticationWithAlias: aUnixUserIdString

"Modify the receiver so that UNIX authentication is used to validate
 all logins to GemStone by the receiver.  After this method has
 been run and successfully committed, all future logins of the receiver
 must specify the password for the given UNIX user ID.  GemStone will then
 validate the password using UNIX password calls.

 If aUnixUserIdString is nil, then the UNIX user ID and the UserProfile
 ID of the receiver are assumed to be the same.  At the next login,
 UNIX password authentication will be performed using the user ID of the
 receiver and the password specified during the login.

 Special UserProfiles may not use UNIX authentication.  Attempting
 to change the authentication scheme to UNIX for a special UserProfile will raise
 an error.  See the method #isSpecialUserId: in this class to see which
 UserProfiles are considered special.

 Requires the OtherPassword privilege."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[ self _validatePrivilege ifTrue:[
    (self _validateUnixUserIdOrNil: aUnixUserIdString ) ifTrue:[
      self authenticationScheme: #UNIX .
      self userIdAlias: nil .
      self kerberosPrincipal: nil .
      self ldapBaseDn: nil .
      self ldapFilterDn: nil .
      self _enableNonGemStoneAuthentication .
      aUnixUserIdString ifNotNil:[ self userIdAlias: aUnixUserIdString ].
    ].
  ]
] ensure:[
  prot _leaveProtectedMode  "exit protected mode"
].
^ self
%

category: 'External Authentication'
method: UserProfile
enableX509Authentication

"Modify the receiver so that GciX509Login with certificates must be
 used to validate all logins to GemStone by the receiver.
 After this method has been run and successfully committed,
 all future logins of the receiver must use GciX509Login.

 Special UserProfiles may not use X509 authentication.  Attempting
 to change the authentication scheme to X509 for a special UserProfile will
 signal an error.  See the method #isSpecialUserId: in this class to see which
 UserProfiles are considered special.

 GciX509Login is permitted regardless of the authentication scheme of a UserProfile.
 This method restricts a UserProfile to allow only GciX509Login .

 Requires the OtherPassword privilege."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[ self _validatePrivilege ifTrue:[
    self userIdAlias: nil .
    self ldapBaseDn: nil .
    self ldapFilterDn: nil .
    self kerberosPrincipal: nil .
    self authenticationScheme: #X509  .
    self _enableNonGemStoneAuthentication .
  ]
] ensure:[
  prot _leaveProtectedMode  "exit protected mode"
].
^ self
%

category: 'Updating'
method: UserProfile
exemptFromLoginLogging: aBoolean

"Enables or disables the printing of login/logout events for the receiver to the
 stone's login log file. Has no effect (i.e. logins still not logged)
 if the stone does not have login logging enabled using
 STN_LOGIN_LOG_ENABLED.

 This method requires the #OtherPassword privilege.

 Returns the receiver."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
    | newBits oldBits |
    ((aBoolean ~~ true) and:[ aBoolean ~~ false ])
      ifTrue:[ ^ aBoolean _validateClass: Boolean ] .
    oldBits := self _securityDataInstVarAt: OC_USER_SECUR_DATA_BIT_FLAGS .
    oldBits == nil ifTrue:[ oldBits := 0 ].
    aBoolean
      ifTrue:[ newBits := oldBits bitOr: GSC_SET_USER_PROFILE_no_login_logging ]
     ifFalse:[ newBits := oldBits bitAnd: GSC_SET_USER_PROFILE_no_login_logging bitInvert ] .
    oldBits ~~ newBits
      ifTrue:[  self _securityDataInstVarAt: OC_USER_SECUR_DATA_BIT_FLAGS put: newBits ] .
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ self
%

category: 'Group Membership'
method: UserProfile
groupNames
"Answers an IdentitySet of symbols which represent the names of all UserProfileGroups of
 which the receiver is a member."

^ groups collect:[:eachGroup| eachGroup groupName ]
%

category: 'Group Membership'
method: UserProfile
groups

"Returns an IdentitySet, the set of groups to which the user belongs."

groups == nil ifTrue:[ ^ IdentitySet new ].
^ groups copy

%

category: 'Testing'
method: UserProfile
hasLoginLogging

"Answer true if the receiver is configured to write login and logout
 events to the stone's login log file, false otherwise.

 If the stone is not configured to log logins (STN_LOGIN_LOG_ENABLED is
 FALSE), then logins and logouts are not logged, regardless of this
 setting.

 Generates an error if the receiver is not your UserProfile and 
 you have neither the #OtherPassword nor #ReadOtherUserProfile privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileRead ifTrue:[
    result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_BIT_FLAGS .
    result == nil ifTrue:[ result := 0 ] .
    result := (result bitAnd: GSC_SET_USER_PROFILE_no_login_logging) == 0 .
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'PerformOnServer'
method: UserProfile
hasSystemLogAccess

"Returns a boolean indicating if the receiver is allowed to write to the system log
 via the methods in class GsSysLog."

 self isSystemUserProfile ifTrue:[ ^ true ] .
 (self privileges includesIdentical: #NoPerformOnServer) ifFalse:[ ^ true ] .
 ^ self performOnServerCommands includes: self class gsSysLogFileName
%

category: 'Updating the Symbol List'
method: UserProfile
insertDictionary: aSymbolDictionary at: anIndex

"Inserts aSymbolDictionary into the receiver's symbol list.  If the
 receiver is identical to 'GsSession currentSession userProfile', inserts
 aSymbolDictionary into the transient session symbol list as well.  The
 insertion into the receiver's symbol list occurs first.

 If anIndex is less than or equal to the size of the receiver's symbol
 list, inserts aSymbolDictionary into the symbol list at anIndex.

 If anIndex is 1 greater than the size of the receiver's symbol list,
 appends aSymbolDictionary to the receiver's symbol list.

 If anIndex is  more than 1 greater than the size of the receiver's
 symbol list, or if anIndex is less than 1, generates an error.

 If an error occurs as the result of the persistent insertion, no
 further action is taken and an error is generated.  If the insertion
 completes correctly, aSymbolDictionary is inserted into the transient
 session symbol list.

 If anIndex is 1, prepend aSymbolDictionary to the beginning of the
 transient symbol list.

 Otherwise, finds the dictionary at (anIndex - 1) in the persistent symbol
 list, and searches for it by identity in the transient symbol list.  If
 found, insert aSymbolDictionary just after it in the transient symbol
 list.  If not found, append aSymbolDictionary to the end of the transient
 symbol list.

 If an error occurs as a result of the insertion in the transient symbol
 list, the persistent symbol list is left in its new state, the
 transient symbol list is left in its old state and the error is
 silently ignored.

 Note if the transient and persistent lists have different contents when
 an abort transaction occurs they will not be automatically synchronized
 after the abort.  The persistent list will be rolled back to the
 committed state, but the transient list will not be rolled back."

"Example: System myUserProfile
                        insertDictionary: (SymbolDictionary new) at: 3'"
(self == System myUserProfile
  ifTrue: [ self _validatePrivilegeName: #CodeModification  ]
  ifFalse: [ self _validatePrivilege ]) ifTrue:[

  (aSymbolDictionary _validateClass: SymbolDictionary) ifTrue:[ | ses |
    self symbolList insertObject:  aSymbolDictionary at: anIndex.
    (self == (ses := GsSession currentSession) userProfile) ifTrue: [
      ses _transientSymbolList ifNotNil:[ :transient |
        anIndex == 1 ifTrue: [
	  transient insertObject: aSymbolDictionary at: 1.
	] ifFalse: [ | dict index |
	  dict := self symbolList at: (anIndex - 1).
	  index := transient indexOfIdentical: dict.
	  index ~~ 0 ifTrue: [
	    transient insertObject: aSymbolDictionary at: index + 1.
	  ] ifFalse: [
	    transient add: aSymbolDictionary.
	  ].
	].
      ].
    ].
  ]
].
^self.
%

category: 'Accessing'
method: UserProfile
isDisabled

"Returns true if logins for the receiver are disabled, false otherwise.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_VERIFIER . "isDisabled"
  result ifFalse: [ result := self _pendingReasonForDisabledAccount ~~ nil ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Accessing Privileges'
method: UserProfile
isReadOnly

"Returns true if commits have been disabled for this user profile by the
 #disableCommits method.  Otherwise returns false.

 Generates an error if the receiver is not your UserProfile and 
 you have neither the #OtherPassword nor #ReadOtherUserProfile privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .

[ self _validateUserProfileRead ifTrue:[
    result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_READ_ONLY.
  ]
] ensure:[
  prot _leaveProtectedMode  "exit protected mode"
].
^ result == true
%

category: 'Private'
method: UserProfile
isSystemProfile
 "Return true if the receiver is one used for 'system sessions'"
 self isSystemUserProfile  ifTrue:[ ^ true ].
 userId = 'SymbolUser'  ifTrue:[ ^ true ].
 userId = 'GcUser'      ifTrue:[ ^ true ].
 userId = 'HostAgentUser'    ifTrue:[ ^ true ].
 ^ false
%

category: 'Private'
method: UserProfile
isSystemUserProfile
 "Return true if the receiver is the profile for SystemUser"
 ^ userId = 'SystemUser'
%

category: 'Accessing'
method: UserProfile
kerberosPrincipal

^ kerberosPrincipal
%

category: 'Updating'
method: UserProfile
kerberosPrincipal: aKerberosPrincipal

kerberosPrincipal := aKerberosPrincipal
%

category: 'Accessing'
method: UserProfile
lastLoginTime

"Returns a DateTime of the last login time of the receiver, or nil if
 no login time has been recorded.  If no stale account limits are set
 on the receiver, then no login times are recorded.  (See updating methods
 in UserProfileSet.)

 Generates an error if the receiver is not your UserProfile and 
 you have neither the #OtherPassword nor #ReadOtherUserProfile privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileRead ifTrue:[
    result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_LAST_LOGIN_TIM .
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Updating'
method: UserProfile
lastLoginTime: aVal

"Updates the lastLoginTime for the receiver to be aVal, which must be an
 instance of DateTime or nil.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Requires the OtherPassword privilege.

 Returns the receiver."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  aVal ifNotNil:[ aVal _validateClass: DateTime ].
  self _validateUserProfileWrite ifTrue:[
    self _validateAuthenticationSchemeIsGemStone ifTrue:[
      self _securityDataInstVarAt: OC_USER_SECUR_DATA_LAST_LOGIN_TIM put: aVal .
    ].
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ self
%

category: 'Accessing'
method: UserProfile
lastPasswordChange

"Returns a DateTime that specifies when the password was last changed.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Generates an error if the receiver is not your UserProfile and 
 you have neither the #OtherPassword nor #ReadOtherUserProfile privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileRead ifTrue:[
    self _validateAuthenticationSchemeIsGemStone ifTrue:[
      result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_LAST_PWD_CHNG .
    ].
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Accessing'
method: UserProfile
ldapBaseDn

"If the receiver is using LDAP authentication, returns a copy of the base
 distinguished name (DN) string.  Otherwise returns nil.

 Generates an error if the receiver is not your UserProfile and 
 you have neither the #OtherPassword nor #ReadOtherUserProfile privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileRead ifTrue:[
    result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_LDAP_BASE_DN .
  ]
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'External Authentication - Private'
method: UserProfile
ldapBaseDn: aString

"Private - customers should not call this method directly."

self _securityDataInstVarAt: OC_USER_SECUR_DATA_LDAP_BASE_DN put: aString .
^ self
%

category: 'External Authentication - LDAP'
method: UserProfile
ldapBaseDn: aString searchFilterDn: stringOrNil

"Specify the base distinguished name (DN) and optionally the search filter
 DN for LDAP logins done by the receiver.  Has no meaningful effect unless
 LDAP authenticaion is enabled for the receiver.

 Requires the OtherPassword privilege."


<primitive: 2001>
| prot |
prot := System _protectedMode .

[ self _validatePrivilege ifTrue:[
    (self _validateBaseDn: aString searchDn: stringOrNil ) ifTrue:[
       self ldapBaseDn: aString ; ldapFilterDn: stringOrNil .
    ]
  ]
] ensure:[
prot _leaveProtectedMode  "exit protected mode"].

^ self
%

category: 'External Authentication - Private'
method: UserProfile
ldapFilterDn: aString

"Private - customers should not call this method directly."

self _securityDataInstVarAt: OC_USER_SECUR_DATA_LDAP_FILTER put: aString .
^ self
%

category: 'Accessing'
method: UserProfile
ldapSearchFilterDn

"If the receiver is using LDAP authentication, returns a copy of the search
 filter string used to resolve the receiver complete distinguished name
 during LDAP authentication.  Otherwise returns nil.

 Generates an error if the receiver is not your UserProfile and 
 you have neither the #OtherPassword nor #ReadOtherUserProfile privilege."

<primitive: 2001>
| result prot |

prot := System _protectedMode .
[
  self _validateUserProfileRead ifTrue:[
    result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_LDAP_FILTER .
  ]
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Session Initialization'
method: UserProfile
loginHook: blockOrSymbol
  "block must be one of:
     nil - removes any previously specified block or method.
     a zero argument ExecBlock
     the selector of a zero argument instance method in UserProfile .

   If not nil, then after authentication of the receiver,
   at the start of each session using the receiver,
   the specified Block or method is invoked.
   Invocation is from GsCurrentSession >> initialize,
   after the ProcessorScheduler is initalized, and
   after the GsPackagePolicy is initialized (in the absense
   of customer modifications to GsCurrentSession >> initialize )."

  | meth nArgs |
  blockOrSymbol
    ifNotNil: [
      blockOrSymbol _isSymbol
        ifTrue: [
          meth := self class
            compiledMethodAt: blockOrSymbol
            environmentId: 0
            otherwise: nil
            usePackages: false.
          meth
            ifNil: [ ArgumentError signal: 'In loginHook:, method ' , blockOrSymbol , ' not found' ]
            ifNotNil: [
              (nArgs := meth numArgs) == 0
                ifFalse: [
                  ArgumentError
                    signal:
                      'Symbol argument to loginHook: must specify a zero argument method' ] ] ]
        ifFalse: [
          blockOrSymbol _isExecBlock
            ifTrue: [
              (nArgs := blockOrSymbol numArgs) == 0
                ifFalse: [
                  ArgumentError
                    signal:
                      'block argument to loginHook: must be a zero argument ExecBlock' ] ]
            ifFalse: [
              blockOrSymbol
                _validateClasses:
                  {Symbol.
                  ExecBlock} ] ] ].
  loginHook := blockOrSymbol
%

category: 'Accessing'
method: UserProfile
loginsAllowedBeforeExpiration

"Returns the number of logins allowed before the receiver's password will
 expire.  Zero or nil means unlimited logins.  A positive number is the number
 remaining before the password will expire.  A negative number means the account
 has expired.  The internal value is reset to nil when the password is
 changed.

 This method will always return nil if the authentication scheme of the
 receiver is not #GemStone.

 Generates an error if the receiver is not your UserProfile and 
 you have neither the #OtherPassword nor #ReadOtherUserProfile privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileRead ifTrue:[
    self authenticationSchemeIsGemStone ifTrue:[
      result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_LOGINS_BEFORE_EXPIR .
    ].
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Updating'
method: UserProfile
loginsAllowedBeforeExpiration: aPositiveInteger

"Sets the number of logins allowed using the receiver before the receiver's
 password will expire.  Zero means unlimited logins.  A positive number is
 the number of logins to be allowed before the current password will expire.
 The internal value is reset to nil when the password is next changed.

 Has no effect if the receiver is one of the special UserProfiles
 (SystemUser, SymbolUser, DataCurator, GcUser, and Nameless), which are always
 immune to login expiration.

 Has no effect on UserProfiles which do not use GemStone authentication.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Generates an error if you do not have the #OtherPassword privilege."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  self _validateAuthenticationSchemeIsGemStone ifTrue:[
    self _validatePrivilege ifTrue:[
      aPositiveInteger ifNotNil:[
        aPositiveInteger _isSmallInteger ifFalse:[
          ^ aPositiveInteger _validateClass: SmallInteger .
        ].
        aPositiveInteger < 0 ifTrue:[
          ^ aPositiveInteger _error: #rtErrArgNotPositive
        ].
      ].
      self _securityDataInstVarAt: OC_USER_SECUR_DATA_LOGINS_BEFORE_EXPIR put: aPositiveInteger .
    ].
  ].
] ensure:[
  prot _leaveProtectedMode
]
%

category: 'Repository Conversion'
method: UserProfile
migrateSecurityData

"Migrate the UserSecurityData object to the correct size for this release
 of GemStone/64.  Returns true if the migration was performed, false if
 no migration is required.

 This method may only be executed by SystemUser."

<primitive: 2001>
| prot |
prot := System _protectedMode .
^ [
  self _securityDataInstVarAt: -1 put: nil .
] ensure:[
  prot _leaveProtectedMode
].
%

category: 'Accessing'
method: UserProfile
nativeLanguage

"Returns a Symbol specifying the user's native language.  This value
 is no longer used by the base system, but is left for use by applications."

^ (self resolveSymbol: #NativeLanguage) ifNotNil:[ :assoc | assoc _value ]
					 ifNil:[ #English ]
%

category: 'Updating'
method: UserProfile
nativeLanguage: aLanguageSymbol

"Redefines the user's native language to be aLanguageSymbol.
 aLanguageSymbol must be a Symbol or DoubleByteSymbol .

 If you use this method, you must commit, and the change will
 take effect in sessions which login using the receiver after the commit."

self _validatePrivilege ifTrue:[
  (aLanguageSymbol _validateClass: Symbol) ifTrue:[
    ((self resolveSymbol: #UserGlobals) value)
       at: #NativeLanguage
       put: aLanguageSymbol
  ].
].
%

category: 'Accessing the Symbol List'
method: UserProfile
objectNamed: aSymbol

"Returns the first object in the receiver's symbol list that has the given
 name.  If no object is found with the given name, this returns nil."

| assn |
assn := symbolList resolveSymbol: aSymbol.
assn == nil ifTrue: [ ^nil ].
^assn value
%

category: 'Updating'
method: UserProfile
oldPassword: firstString
newPassword: secondString

"Modifies the receiver's password to be secondString.  Generates an error if
 either argument is not a String, if firstString is not the receiver's
 password, or if the receiver is not the UserProfile of the current session.
 Generates an error if secondString is equivalent to the userId of the
 receiver.  The new password (secondString) may not be longer than
 1024 Characters and may not be an empty string.

 Raises an #rtErrAuthIllegalPasswordChange error if the authentication mode of
 the receiver is not set to #GemStone or the receiver is disabled.

 This method allows you to change your own password.  To change the password of
 a disabled account or any other user, use password: instead.

 This method requires the UserPassword privilege."


<primitive: 2001> "enter protected mode"
| prot |
prot := System _protectedMode .
[
  self == System myUserProfile ifFalse:[
    ^ self _error: #rtErrCantChangePassword
  ].
  (self _validatePrivilegeName: #UserPassword ) ifTrue:[
    "check old password"
    firstString _validateClass: String .
    secondString _validateClass: String .
    (self validatePassword: firstString) ifFalse:[
        ^ userId _error: #rtErrBadPassword.
    ].
    (self _validateNewPassword:  secondString) ifTrue:[
      (AllUsers disallowUsedPasswords and:[ firstString = secondString]) ifTrue:[
        SecurityError signal:'A password may not have the value ', secondString .
      ].
      self _changePasswordFrom: firstString to: secondString .
    ].
  ].
] ensure:[
  prot _leaveProtectedMode
]
%

category: 'Updating'
method: UserProfile
password: aString

"Modifies the receiver's password to be aString, and returns the receiver.  If
 the argument is not a String, generates an error.
 Generates an error if aString is equivalent to the userId of the
 receiver.  aString may not be longer than 1024 Characters and may not be an
 empty string. In order to login, the characters in aString should all be
 within the 8-bit range, but this is not enforced.

 This method allows you to change the password of another GemStone user.
 To change your own password, use oldPassword:newPassword: instead.

 Raises an #rtErrAuthIllegalPasswordChange error if the authentication mode of
 the receiver is not set to #GemStone.

 This method requires the #OtherPassword privilege.

 If the repository was restored using
    Repository>>restoreFromBackup:newSystemUserPassword: (or equivalent)
 with a non-nil SystemUser password, then Repository>>commitRestore 
 must occur and then a login by SystemUser must occur before this method 
 can be executed for SystemUser's UserProfile by a user other than SystemUser."

<primitive: 2001> "enter protected mode"
| prot |
prot := System _protectedMode .
[
  self _validatePrivilege
    ifTrue:[ self _setPassword: aString ].
] ensure:[
  prot _leaveProtectedMode
].
%

category: 'Accessing'
method: UserProfile
passwordAgeLimit

"Answers the maximum number of hours for which the receiver can retain a password.
 When a password is set, it expires this number of hours later.  The user must change
 the password before it expires, or else GemStone disables the account.  Once
 a password has expired, an administrator must reset the password from another
 account before the user can login again.

 Returns nil if the repository-wide passwordAgeLimit, if any, will be used to
 enforce the password age limit.

 Returns zero if password aging is disabled for the receiver.

 Sending this message to any of the special UserProfiles (SystemUser, SymbolUser,
 DataCurator, GcUser, and Nameless) always returns zero or an error.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Generates an error if the receiver is not your UserProfile and 
 you have neither the #OtherPassword nor #ReadOtherUserProfile privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileRead ifTrue:[
    self _validateAuthenticationSchemeIsGemStone ifTrue:[
      result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_PASSWD_AGE_LIMIT .
    ]
  ]
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Updating'
method: UserProfile
passwordAgeLimit: numberOfHours

"Sets the maximum number of hours for which the receiver can retain a password.
 When a password is set, it expires this number of hours later.  The user must change
 the password before it expires, or else GemStone disables the account.  Once
 a password has expired, an administrator must reset the password from another
 account before the user can login again.

 Setting this value to nil causes the receiver to use the repository-wide
 passwordAgeLimit setting in AllUsers, if enabled.

 Setting this value to zero disables password aging for the receiver.

 Requires the OtherPassword privilege.

 Has no effect if the receiver is one of the special UserProfiles
 (SystemUser, SymbolUser, DataCurator, GcUser, and Nameless), which are always
 immune to the password age limit.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Returns the receiver."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
  self _validateAuthenticationSchemeIsGemStone ifTrue:[
    (self class isSpecialUserId: userId) ifFalse:[
      numberOfHours == nil ifFalse: [
        (numberOfHours _validateClasses: { SmallInteger }) ifFalse: [^ nil ].
        numberOfHours < 0 ifTrue: [^numberOfHours _error: #rtErrArgNotPositive].
      ].
    self _securityDataInstVarAt: OC_USER_SECUR_DATA_PASSWD_AGE_LIMIT put: numberOfHours ] .
    ]
  ]
] ensure:[ prot _leaveProtectedMode "exit protected mode"].
^ self
%

category: 'Accessing'
method: UserProfile
passwordAgeWarning

"The maximum number of hours prior to a password expiration time for which
 a login as the receiver can succeed without a warning.  If the user logs in
 to GemStone within this number of hours before the password is due to expire,
 GemStone issues a warning about the impending expiration.  This feature grants
 a user the opportunity to change the password conveniently and to prevent the
 account from being disabled.

 Returns nil if the repository-wide passwordAgeWarning, if any, will be used to
 generate password age warnings.

 Returns zero if password age warning is disabled for the receiver.

 Sending this message to any of the special UserProfiles (SystemUser, SymbolUser,
 DataCurator, GcUser, and Nameless) always returns zero or an error.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Generates an error if the receiver is not your UserProfile and 
 you have neither the #OtherPassword nor #ReadOtherUserProfile privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileRead ifTrue:[
    self _validateAuthenticationSchemeIsGemStone ifTrue:[
      result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_PASSWD_AGE_WARNING .
    ]
  ]
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Updating'
method: UserProfile
passwordAgeWarning: numberOfHours

"Sets the maximum number of hours prior to a password expiration time for which
 a login as the receiver can succeed without a warning.  If the user logs in
 to GemStone within this number of hours before the password is due to expire,
 GemStone issues a warning about the impending expiration.  This feature grants
 a user the opportunity to change the password conveniently and to prevent the
 account from being disabled.

 Has no effect if the a password age limit is not set for the receiver.

 Setting this value to nil causes the receiver to use the repository-wide
 passwordAgeWarning setting, if is enabled.

 Setting this value to zero disables the password age warning for the receiver.

 Requires the OtherPassword privilege.

 Has no effect if the receiver is one of the special UserProfiles
 (SystemUser, SymbolUser, DataCurator, GcUser, and Nameless), which are always
 immune to the password age warning.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Returns the receiver."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
    self _validateAuthenticationSchemeIsGemStone ifTrue:[
      (self class isSpecialUserId: userId) ifFalse:[
        numberOfHours == nil ifFalse: [
          (numberOfHours _validateClasses: { SmallInteger }) ifFalse: [^ nil ].
          numberOfHours < 0 ifTrue: [^numberOfHours _error: #rtErrArgNotPositive].
        ].
        self _securityDataInstVarAt: OC_USER_SECUR_DATA_PASSWD_AGE_WARNING put: numberOfHours
      ] .
    ]
  ]
] ensure:[ prot _leaveProtectedMode "exit protected mode"].
^ self
%

category: 'Accessing'
method: UserProfile
passwordNeverExpires

"Returns true if the receiver is an account that is immune from password
 expiration, or if the receiver uses an authentication scheme other than
 #GemStone.

 The special UserProfiles (SystemUser, SymbolUser, DataCurator, GcUser,
 and Nameless) are immune to password expiration and always answer true.

 Generates an error if the receiver is not your UserProfile and you 
 have neither #OtherPassword nor #ReadOtherUserProfile  privilege."

self authenticationSchemeIsGemStone
  ifFalse:[ ^ true] .

(self class isSpecialUserId: userId)
  ifTrue:[ ^ true ].

^self _fetchPasswordNeverExpires
%

category: 'PerformOnServer'
method: UserProfile
performOnServerCommands

"Answer an Array of Strings which represents a list of commands the receiver
 is permitted to use when calling either the System(C)>>performOnServer: or
 GsHostProcess>>fork: methods.

 Raises an exception if the receiver is not the caller's UserProfile and the
 caller does not have the OtherPassword privilege."

^self _fetchPerformOnServerCommandList
%

category: 'Accessing Privileges'
method: UserProfile
privileges

"Returns an Array of Symbols describing the receiver's privileges.  Those
 privilege Symbols specify the user's level of access to certain privileged
 system functions ordinarily performed by the GemStone data curator.

 See individual methods for specific required privileges.  For more
 information about privileges, see the GemStone Programming Guide.

 The following privileges are granted to UserProfiles, to allow them to
 perform privileged methods:

 DefaultObjectSecurityPolicy
   DefaultSegment - translated to DefaultObjectSecurityPolicy
 FileControl
 GarbageCollection
 OtherPassword
 ObjectSecurityPolicyCreation
   SegmentCreation - translated to ObjectSecurityPolicyCreation
 ObjectSecurityPolicyProtection
   SegmentProtection - translated to ObjectSecurityPolicyProtection
 SessionAccess
 SystemControl
 UserPassword
 CodeModification
 SessionPriority
 CompilePrimitives
 ChangeUserId
 MigrateObjects
 DisableObjectReadLogging
 DynamicDisableObjectReadLogging
 ReadOtherUserProfile

 The following four privileges are inverse privilege; the operations can be
 performed unless the privilege is given to the user.  UserProfiles with
 these privileges cannot perform the function.

 NoPerformOnServer
    prevents perform of System class >> performOnServer: and
    GsHostProcess >> fork  unless the command argument appears on a
    whitelist of allowed server commands. Also affects methods in
    class GsSysLog.

 NoUserAction
    prevents loading of any user action library and prevents use of the FFI.

 NoGsFileOnServer
    prevents any GsFile operation which accesses a file on the server,
    i.e. access from the gem process to the file system.

 NoGsFileOnClient
    prevents any GsFile operation which accesses a file on the client,
    i.e. access from the remote GCI process to the file system.

 The state of NoUserAction, NoGsFileOnServer, and NoGsFileOnClient are
 cached in the Virtual Machine at session login, and changes to a
 UserProfile for these 3 privileges will only be seen in sessions that
 login after those changes have been committed.

 "

| result |

result := { } .

1 to: (PrivilegeNames size) do: [:counter |
    (privileges bitAt: counter) = 1 ifTrue: [
       result add: (PrivilegeNames at: counter).
    ].
].
^result
%

category: 'Updating Privileges'
method: UserProfile
privileges: anArrayOfStrings

"Redefines the receiver's privileges to be those specified by anArrayOfStrings.
 Any privileges not contained in anArrayOfStrings will be turned off.
 Generates an error if any element of anArrayOfStrings is not a valid privilege
 name (as defined in the class variable PrivilegeNames).

 Only SystemUser may send this message to special UserProfiles. Attempts which
 violate this restriction will raise an exception.

Example: System myUserProfile privileges: #('SessionAccess' 'DefaultObjectSecurityPolicy')"


| privSet myUserProfile |
self _validatePrivilege ifFalse:[ ^ nil ].
(self class isSpecialUserId: userId) ifTrue:[
  System iAmSystemUser
    ifFalse:[ ^ self _error: #rtErrMustBeSystemUser ] .
] .

myUserProfile := ( self == System myUserProfile ).
privSet := UserProfile _privilegesSetFromArray: anArrayOfStrings .

" clear all privileges -- unless this is me,
  in which case leave #OtherPassword on "

myUserProfile
    ifTrue: [ privileges :=
              ( 1 bitShift: ( PrivilegeNames indexOf: #OtherPassword ) - 1 ) ]
    ifFalse: [ privileges := 0 ].

privSet do: [ :aPrivSymbol |
   self addPrivilege: aPrivSymbol .
   ].

" if me, clean up the #OtherPassword flag "
myUserProfile ifTrue: [
    ( privSet includes: #OtherPassword ) ifFalse: [
        self deletePrivilege: #OtherPassword ] ].
%

category: 'Accessing'
method: UserProfile
reasonForDisabledAccount

"Returns the String that contains the reason why the receiver's password
 has been automatically disabled or expired. Some possible strings are:
  'LoginLimitReached'
     The account's password in no longer valid because its login limit
     was reached.  See UserProfile>>loginsAllowedBeforeExpiration.
  'LoginsWithInvalidPassword'
     The account is disabled due to an attempt to login to it too many
     times with an incorrect password.  See the stone configuration
     option STN_DISABLE_LOGIN_FAILURE_LIMIT.
  'PasswordAgeLimit'
     The account's password is no longer valid due to it being too
     old as determined by UserProfileSet>>passwordAgeLimit.
  'StaleAccount'
     The account is disabled due to it being too old
     as determined by UserProfileSet>>staleAccountAgeLimit.
  'Disabled by administrator'
     The account was disabled by an Administrator using the
     UserProfile>>disable method.

 Returns nil if the account has not been disabled.

 Raises an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_REASON_DISABLED .
  result ifNil:[ result := self _pendingReasonForDisabledAccount ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Updating'
method: UserProfile
reasonForDisabledAccount: aString

"Updates the String that contains the reason why the receiver's password
 has been automatically disabled or expired.

 Raises an error if you do not have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
   (aString isKindOf: String) ifFalse:[
     ^ aString _validateClass: String
    ].
    "Note: primitive now stores a copy of aString"
    result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_REASON_DISABLED put: aString .
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'External Authentication'
method: UserProfile
reenable
"Resumes previously disabled logins for the receiver which must be using
 external authentication.

 Raises an error if the account is using GemStone authentication. Use the
 #reenableWithPassword: method to reenable a UserProfile which uses GemStone
 authentication.

 Returns the receiver on success or raises an exception and returns nil
 on error.
 Requires the #OtherPassword privilege."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[ self _validatePrivilege ifFalse:[ ^ nil ].
  self _validateAuthenticationSchemeIsExternal ifFalse:[ ^ nil ] .
  self isDisabled ifFalse:[ ^ self ] . "No action needed"
  self _enableNonGemStoneAuthentication .
] ensure:[ prot _leaveProtectedMode  "exit protected mode"].

^ self
%

category: 'Updating'
method: UserProfile
reenableWithPassword: aString
"Resumes previously disabled logins for the receiver, which must be using
 GemStone authentication.

 Raises an error if the account is using external authentication. Use the
 #reenable method to re-enable a UserProfile which uses external
 authentication.

 Returns the receiver on success or raises an exception and returns nil
 on error.
 Requires the #OtherPassword privilege."

^ self _validateAuthenticationSchemeIsGemStone
    ifTrue:[ self password: aString. self ]
   ifFalse:[ nil ].
%

category: 'PerformOnServer'
method: UserProfile
removeAllPerformOnServerCommands

"Removes all commands from the list of commands the receiver is permitted to
 use when calling either the System(C)>>performOnServer: or GsHostProcess>>fork
 methods. This method effectively disables these methods when the receiver has the
 NoPerformOnServer privilege.
 
 Returns the receiver on success. Raises an exception if the caller does not have
 the OtherPassword privilege."

^ self removePerformOnServerCommands: self performOnServerCommands
%

category: 'Updating the Symbol List'
method: UserProfile
removeDictionaryAt: anIndex

"Removes the SymbolDictionary at position anIndex from the receiver's symbol
 list, thus decreasing the size of the receiver's symbol list by one,
 and, if the receiver is identical to 'GsSession currentSession
 userProfile', removes the SymbolDictionary at position anIndex from the
 transient symbol list's symbol list (subject to the constraints below).
 Returns the receiver's dictionaryNames String.

 Generates an error if anIndex is not both a positive integer and less than the
 size of the receiver's symbol list.  If an error occurs in removing
 from the receiver's symbol list, the transient symbol list is left
 alone, and the error is handled immediately.

 If the removal from the receiver's symbol list succeeds, search in the
 transient symbol list for a SymbolDictionary identical to the one
 removed from the transient symbol list, and if found, remove it from
 the list.  Removes only the first such element.  If no such dictionary
 is found, silently return.

 Note if the transient and persistent lists have different contents when
 an abort transaction occurs they will not be automatically synchronized
 after the abort.  The persistent list will be rolled back to the
 committed state, but the transient list will not be rolled back."

"Example: System myUserProfile insertDictionary: (SymbolDictionary new) at: 3.
                  System myUserProfile dictionaryNames.
                  System myUserProfile removeDictionaryAt: 3.
                  System myUserProfile dictionaryNames."

| dict |
(self == System myUserProfile
  ifTrue: [ self _validatePrivilegeName: #CodeModification  ]
  ifFalse: [ self _validatePrivilege]) ifTrue:[

   (anIndex _validateClass: Integer) ifTrue:[ | ses |
     dict := self symbolList removeAtIndex: anIndex.
     ses := GsSession currentSession .
     (self == ses userProfile) ifTrue: [
        ses _transientSymbolList ifNotNil:[ :list |
             list removeIdentical: dict ifAbsent: [ ]
        ]
     ].
  ].
].
^symbolList namesReport.
%

category: 'Group Membership'
method: UserProfile
removeFromAllGroups

"Removes the receiver from all groups of which it is a member.
 Answers the receiver."

self groups do: [:aGroup | self removeFromUserProfileGroup: aGroup].
^ self
%

category: 'Group Membership'
method: UserProfile
removeFromUserProfileGroup: aUserProfileGroup

"Removes the user from the group UserProfileGroup.
 If the user does not belong to the group, no action occurs."

self _validatePrivilege ifTrue:[
  groups size > 0 ifTrue:[
    groups remove: aUserProfileGroup ifAbsent:[] .
    aUserProfileGroup remove: self ifAbsent: [] .
  ].
].
%

category: 'Group Membership'
method: UserProfile
removeGroup: aGroupString

"Removes the user from the group represented by aGroupString.  If aGroupString
 is not a group defined in UserProfileGroup, generates an error.
 If the user does not belong to the group, no action occurs.

 Legacy  method; use removeFromUserProfilGroup: or
    UserProfileGroup >> removeUser: instead."

| theGroup |
(theGroup := UserProfileGroup _validateGroupString: aGroupString) ifNotNil:[
  self removeFromUserProfileGroup: theGroup
].
%

category: 'PerformOnServer'
method: UserProfile
removePerformOnServerCommand: aString

"Removes the command aString from the list of commands the receiver is
 permitted to execute when calling either the System(C)>>performOnServer: or
 GsHostProcess>>fork methods when the receiver has the NoPerformOnServer
 privilege.

 Returns the receiver on success. Raises an exception if the caller does not
 have the OtherPassword privilege or aString is not present in the receiver's
 list of allowed commands."

^ self removePerformOnServerCommands: { aString }
%

category: 'PerformOnServer'
method: UserProfile
removePerformOnServerCommands: anArray

"Removes the commands in anArray from the list of commands the receiver is
 permitted to execute when calling either the System(C)>>performOnServer: or
 GsHostProcess>>fork methods when the receiver has the NoPerformOnServer
 privilege. Each element of anArray must be a non-empty instance of String.

 Returns the receiver on success. Raises an exception if the caller does not
 have the OtherPassword privilege or any command in anArray is not present in
 the receiver's list of allowed commands."

^ self _primUpdatePerformOnServerCommands: anArray opCode: 2
%

category: 'Accessing the Symbol List'
method: UserProfile
resolveSymbol: aString

"Searches the receiver's symbol list for an Association whose key is equal to
 aString, and returns that Association.  If no such Association is found in the
 symbol list, this returns nil."

^ symbolList resolveSymbol: aString
%

category: 'Updating'
method: UserProfile
setPasswordNeverExpires: aBoolean

"Makes the receiver immune from all forms of password expiration.  Once
 set, the receiver is never required to change the password and will
 never be denied access because too much time has passed since the last
 successful login.

 This method does not modify any password validity restrictions for
 the receiver such as maximum repeating characters, disallowed passwords,
 etc.

 Password expiration is always disabled for the special UserProfiles
 (SystemUser, SymbolUser, DataCurator, GcUser, and Nameless).  Attempting
 to enabled it for any of these UserProfiles will raise an error.  An
 attempt to disable password expiration for a special user profile is
 ignored.

 Running this method raises an error if the receiver is not using the
 #GemStone authentication scheme.

 Requires the #OtherPassword privilege."

self authenticationSchemeIsGemStone
  ifFalse:[ ^ self _error: #rtErrAuthIllegalOperation ].

(self class isSpecialUserId: userId)
  ifTrue:[ aBoolean  ifTrue:[ ^ self ] "Do nothing"
                    ifFalse:[ ^ self _error: #rtErrIllegalPasswordExpiration ].].

^ self _setPasswordNeverExpires: aBoolean
%

category: 'Accessing'
method: UserProfile
staleAccountAgeLimit

"The maximum number of hours for which the receiver will remain enabled without
 a login.  Once the user logs in for the first time, he or she has up to this
 number of hours to login again, or else GemStone disables the account.  Once
 the account has been disabled, an administrator must reset the password from
 another account before the user can login again.

 Returns nil if the repository-wide staleAccountAgeLimit, if any, will be used to
 suspend stale accounts.

 Returns zero if the stale account age limit is disbled for the receiver.

 Sending this message to any of the special UserProfiles (SystemUser, SymbolUser,
 DataCurator, GcUser, and Nameless) always returns zero or an error.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Generates an error if the receiver is not your UserProfile and 
 you have neither the #OtherPassword nor #ReadOtherUserProfile privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileRead ifTrue:[
    self _validateAuthenticationSchemeIsGemStone ifTrue:[
      result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_STALE_ACCOUNT_AGE_LIMIT .
    ]
  ]
] ensure:[ prot _leaveProtectedMode ].
^ result
%

category: 'Updating'
method: UserProfile
staleAccountAgeLimit: numberOfHours

"Sets maximum number of hours for which the receiver will remain enabled without
 a login.  Once the user logs in, he or she has up to this number of hours to
 login again, or else GemStone disables the account.  Once the account has been
 disabled, an administrator must reset the password from another account before
 the user can login again.

 Setting this value to nil causes the receiver to use repository-wide
 staleAccountAgeLimit setting in AllUsers, if enabled.

 Setting this value to zero disables the stale account age limit for the
 receiver.

 Requires the OtherPassword privilege.

 Has no effect if the receiver is one of the special UserProfiles
 (SystemUser, SymbolUser, DataCurator, GcUser, and Nameless), which are always
 immune to the stale account limit.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Returns the receiver."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
    self _validateAuthenticationSchemeIsGemStone ifTrue:[
      (self class isSpecialUserId: userId) ifFalse:[
         numberOfHours == nil ifFalse: [
           (numberOfHours _validateClasses: { SmallInteger }) ifFalse: [^ nil ].
           numberOfHours < 0 ifTrue: [^numberOfHours _error: #rtErrArgNotPositive].
         ] .
         self _securityDataInstVarAt: OC_USER_SECUR_DATA_STALE_ACCOUNT_AGE_LIMIT put: numberOfHours
      ] .
    ]
  ]
] ensure:[ prot _leaveProtectedMode "exit protected mode"].
^ self
%

category: 'Accessing the Symbol List'
method: UserProfile
symbolList

"Whenever the compiler is invoked, tokens in the method's source are bound to
 either Symbols found in the source, or to SymbolAssociations found in one of a
 number of SymbolDictionary passed to the compiler.  This method returns
 the Array of such SymbolDictionary that are associated with the
 receiver."

^ symbolList
%

category: 'Updating the Symbol List'
method: UserProfile
symbolList: aSymbolList

"Modifies the list of SymbolDictionaries associated with the receiver.
 If the receiver is identical to 'GsSession currentSession userProfile',
 replaces the contents of the session transient symbol
 list with the contents of aSymbolList.  If an error occurs as a result
 of the modification to the persistent symbol list, the transient list
 is left unmodified and the error is handled immediately.  If an error
 occurs as a result of the modification of the transient symbol list,
 the persistent symbol list is left in its new state, the transient
 symbol list is left in its old state, and the error is silently ignored.

 Note that if the transient and persistent lists have different contents when
 an abort transaction occurs they will not be automatically synchronized
 after the abort.  The persistent list will be rolled back to the
 committed state, but the transient list will not be rolled back."

"relax privilege to not require #OtherPassword if doing this to ourself"
(self == System myUserProfile
  ifTrue: [ self _validatePrivilegeName: #CodeModification ]
  ifFalse: [ self _validatePrivilege ]) ifTrue:[

  (aSymbolList _validateClass: SymbolList ) ifTrue:[ | ses |
      symbolList ifNil:[
         symbolList := aSymbolList
      ] ifNotNil:[ "must preserve old identity to avoid authorization errors"
         symbolList ~~ aSymbolList ifTrue:[		"fix 41658"
           symbolList size: 0.
           symbolList addAll: aSymbolList .
         ]
      ] .
      ses := GsSession currentSession .
      (self == ses userProfile) ifTrue: [
         [
           ses _transientSymbolList ifNotNil:[ :list |
             list replaceElementsFrom: symbolList
           ]
         ] onException: Error do:[:ex | ^ nil ].
      ].
  ].
].
^ self
%

category: 'Accessing the Symbol List'
method: UserProfile
symbolResolutionOf: aString

"Searches the receiver's symbol list for aString.  If aString is found, returns
 a formatted String describing the position in the symbol list of the
 Dictionary defining aString, the name of that Dictionary, and aString.

 Generates an error if aString is not defined for the receiver."

"Example: System myUserProfile symbolResolutionOf: #Integer"

^ self symbolList symbolResolutionOf: aString
%

category: 'Accessing'
method: UserProfile
userId

"Answers a String which is the user name of the receiver."

^ userId
%

category: 'Updating'
method: UserProfile
userId: newUserIdString password: passwordString

"Modifies the userId associated with the receiver to be newUserIdString,
 and installs the password passwordString .
 newUserIdString must not be the user ID of some other UserProfile in
 AllUsers, or an error will be raised.  Has no effect if newUserIdString
 is equal to the current userId of the receiver.

 newUserIdString must be a kind of String , with size <= 1024 characters .

 WARNING: Be careful when using this method with UserProfiles which use an
          external authentication scheme.  Executing this method will
          NOT change the userId used with external authentication if a user ID
          alias has been set.  See the method #userIdAlias: to set a
          alias for a UserProfile.

 Requires OtherPassword and ChangeUserId privileges."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[ | oldUserPro newUserId oldUserId duplicateUser|

  self _validatePrivilege ifFalse:[ ^ nil ].
  System myUserProfile _validatePrivilegeName: #ChangeUserId .
  (UserProfile _validateUserId: newUserIdString ) ifFalse:[ ^ nil ].

  (self class _validate__password: passwordString forUserId: newUserIdString)
    ifFalse:[ ^ nil ] .

  (self class isSpecialUserId: userId)
    ifTrue:[ ArgumentError signal: 'illegal attempt to modify a special UserProfile' ].

  (self class isSpecialUserId: newUserIdString)
    ifTrue:[ ArgumentError signal: 'illegal attempt to assume the name of a special UserProfile' ].

  oldUserId := userId .
  newUserId := newUserIdString asString .
  oldUserId = newUserId ifTrue:[
    ^ self
    ].
  oldUserPro := AllUsers userWithId: oldUserId ifAbsent:[
    ArgumentError signal:'userId ' , oldUserId , ' not found in AllUsers'
  ].

  duplicateUser := AllUsers userWithId: newUserId ifAbsent:[ nil ].
  duplicateUser ifNotNil:[
    ^ self _error: #rtErrUserIdAlreadyExists args: { newUserIdString }
    ].

  passwordString ifNotNil:[
    (oldUserPro _validate__password: passwordString) ifFalse:[ ^ nil ].
  ].
  AllUsers _oldUserId: oldUserId newUserId: newUserId for: self .
  userId := newUserId .
  oldUserPro password: passwordString .
  AllUsers addMsgToSecurityLog: '      userId ', oldUserId , ' changed to ',
	newUserId , ' and new password set'.
] ensure:[
  prot _leaveProtectedMode
]
%

category: 'External Authentication'
method: UserProfile
userIdAlias

"Returns a String which is the alias used by the receiver
 for external authentication, or nil if an alias is not used.

 Generates an error if the receiver is not your UserProfile and 
 you have neither the #OtherPassword nor #ReadOtherUserProfile privilege."

<primitive: 2001>
| prot result |
prot := System _protectedMode .
[ self _validateUserProfileRead ifTrue:[
    result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_ALIAS_USERID.
  ]
] ensure:[
  prot _leaveProtectedMode  "exit protected mode"
].
^ result
%

category: 'External Authentication - Private'
method: UserProfile
userIdAlias: aString

"Private - customers should not call this method directly.

 Sets the alias user ID for the receiver to be aString.  When external
 authentication is performed, the alias user ID will be used instead of
 the userId instance variable.  If no alias is specified, then the userId
 instance variable is used.

 If UNIX authentication is enabled for the receiver, then aString must
 be a valid UNIX user ID, otherwise an error will be raised.

 Requires the OtherPassword privilege."

self _securityDataInstVarAt: OC_USER_SECUR_DATA_ALIAS_USERID put: aString .
^ self
%

category: 'Accessing'
method: UserProfile
usesOldPasswordEncryption

"Answer a Boolean indicating if the receiver uses the old
 password encryption scheme.  The old encryption scheme was
 used in versions prior to 3.1.0.

 For upgraded repositories, passwords are automatically
 encrypted using the new scheme during the first successful
 login after upgrading to version 3.1.0 or later.

 Generates an error if the receiver is not your UserProfile and 
 you do not have #OtherPassword privilege."

<primitive: 2001>
| result prot |
result := false .
prot := System _protectedMode .
[
  self _validateUserProfileWrite ifTrue:[
    self authenticationSchemeIsGemStone ifTrue:[
      result := (self _securityDataInstVarAt: OC_USER_SECUR_DATA_SALT) == nil .
    ].
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'User Authorization'
method: UserProfile
validatePassword: aString

"This method allows an application to validate an operation that requires
 authentication by multiple individuals.  Returns true if aString is the
 password of the receiver, returns false if the receiver is disabled or
 the password is incorrect.

 Generates an error if the argument is not a String.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 306>
aString _validateByteClass: CharacterCollection.
self _primitiveFailed: #validatePassword: args: { aString } .
self _uncontinuableError
%

category: 'Storing and Loading'
method: UserProfile
writeTo: aPassiveObject

"Instances of UserProfile cannot be converted to passive form.  This method
 writes nil to aPassiveObject and stops GemStone Smalltalk execution with a
 notifier."

  aPassiveObject writeObject: nil.
  self _error: #rtErrAttemptToPassivateInvalidObject.
%

category: 'Accessing'
method: UserProfile
x509loginStatus

  "Returns a SmallInteger with bits 16r1 isDisabled , 16r2 isReadOnly .
   The isDisabled bit does not include pending disable per
   UserProfile >> _pendingReasonForDisabledAccount.
   Requires SessionAccess privilege, does not require OtherPassword privilege."
<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  result := self _securityDataInstVarAt: OC_USER_SECUR_X509_STATUS.
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Updating Privileges'
method: UserProfile
_addPrivilege: aPrivilegeString

< protected >
| index privSet privSymbol |
privSet := UserProfile _privilegesSetFromArray: { aPrivilegeString }.
privSymbol := privSet _at: 1 .
(index := PrivilegeNames indexOf: privSymbol ) == 0 ifTrue:[
  aPrivilegeString _error: #rtErrBadPriv .
  ^ self
].
(self class isSpecialUserId: userId) ifTrue:[
  System iAmSystemUser
    ifFalse:[ ^ self _error: #rtErrMustBeSystemUser ] .
].
privileges := privileges bitOr:( 1 bitShift: index - 1) .
%

category: 'Private'
method: UserProfile
_changePasswordFrom: firstString to: secondString

"Private.  Change the given password of the receiver, which is the UserProfile
 of the current session (firstString), to the new given password
 (secondString)."

"Tests AllUsers.disallowUsedPasswords, and if ok, saves firstString
  in encrypted form in receiver's UserSecurityData.oldPasswords. "

<protected primitive: 308>
firstString _validateByteClass: CharacterCollection.
secondString _validateByteClass: CharacterCollection.
self _primitiveFailed: #_changePasswordFrom:to:
     args: { firstString . secondString } .
self _uncontinuableError
%

category: 'Updating'
method: UserProfile
_defaultObjectSecurityPolicy: anObjectSecurityPolicy

"Change the default login GsObjectSecurityPolicy of the receiver, which is the UserProfile of
 the current session.

 This method requires the #DefaultObjectSecurityPolicy privilege,
 and requires write authorization for the receiver.

 This method does not change the GsObjectSecurityPolicy into which newly created objects
 are placed, that will happen after the current session commits
 at the next login using this UserProfile .
 See also System (C) >> currentObjectSecurityPolicy:  .

 Gs64 v2.2 , the requirement for write authorization for the receiver is new;
 it did not exist in Gemstone/S 6.x "

<primitive: 307>
anObjectSecurityPolicy _validateClass: GsObjectSecurityPolicy.
self _primitiveFailed: #_defaultObjectSecurityPolicy:
     args: { anObjectSecurityPolicy } .
^ nil
%

category: 'Updating Privileges'
method: UserProfile
_deletePrivilege: aPrivilegeString

< protected >
| index privSet privSymbol |
privSet := UserProfile _privilegesSetFromArray: { aPrivilegeString }.
privSymbol := privSet _at: 1 .
(index := PrivilegeNames indexOf: privSymbol ) == 0 ifTrue:[
  aPrivilegeString _error: #rtErrBadPriv .
  ^ self
].
(self class isSpecialUserId: userId) ifTrue:[
  System iAmSystemUser
    ifFalse:[ ^ self _error: #rtErrMustBeSystemUser ] .
].
privileges := privileges bitAnd:( 1 bitShift: index - 1) bitInvert
%

category: 'Private'
method: UserProfile
_enableNonGemStoneAuthentication
"Private. Customers should not call this method directly."

self _securityDataInstVarAt: OC_USER_SECUR_DATA_VERIFIER put: 0 .
self _securityDataInstVarAt: OC_USER_SECUR_DATA_REASON_DISABLED put: nil .
^ self
%

category: 'Private'
method: UserProfile
_fetchPasswordNeverExpires

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileRead ifTrue:[
    result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_PASSWD_NEVER_EXPIRES .
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Private'
method: UserProfile
_fetchPerformOnServerCommandList

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileWrite ifTrue:[
    result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_POS_WHITELIST
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Accessing Privileges'
method: UserProfile
_hasPrivilegeName: aSymbol

 "Returns true if receiver has the privilege specified by aSymbol,
  false otherwise."

 | bitNumber |
 bitNumber := (PrivilegeNames indexOfIdentical: aSymbol) .
 bitNumber < 1 ifTrue:[ ^ false ].
 ^ (privileges bitAt: bitNumber ) == 1
%

category: 'Private'
method: UserProfile
_initialize

"Private.  Initialize private instance variables."

self _validatePrivilege.
%

category: 'External Authentication - Private'
method: UserProfile
_invalidDnError: aString

self _error: #rtErrIllegalDn args: { aString }  .
self _uncontinuableError
%

category: 'Repository Conversion'
method: UserProfile
_migrateGroups: migrateBlock

"Converts the groups collections from IdentitySets of Strings to
 IdentitySets of UserProfileGroups."

groups ~~ nil  ifTrue:[ migrateBlock value: groups ] .
groups do:[:eachGroup| eachGroup add: self ].
%

category: 'Private'
method: UserProfile
_newSecurityData

"Creates a new instance of UserSecurityData for the receiver."

" only called from UserProfileSet>>_add: "
<protected primitive: 462>

self _primitiveFailed: #_newSecurityData .
self _uncontinuableError
%

category: 'Private'
method: UserProfile
_oldPasswordsIncludes: aString

"Returns true if the receiver's old passwords include the argument."

<protected primitive: 459>

self _primitiveFailed: #_oldPasswordsIncludes: args: { aString } .
self _uncontinuableError
%

category: 'Private'
method: UserProfile
_password: aString

"Modifies the receiver's password to be aString, and returns the receiver.  If
 the argument is not a String, generates an error.  The new password (aString)
 may not be longer than 1024 Characters.

 Raises an #rtErrAuthIllegalPasswordChange error if the authentication mode of
 the receiver is not set to #GemStone.

 This method allows you to change the password of another GemStone user.
 To change your own password, use oldPassword:newPassword: instead.

 This method requires the #OtherPassword privilege."

<protected primitive: 309>
aString _validateKindOfClass:  String  . "MultiByteString not allowed"
aString class isBytes ifFalse:[ aString _error: #objErrNotByteKind ] .
self _primitiveFailed: #_password: args: { aString } .
self _uncontinuableError
%

category: 'Private'
method: UserProfile
_pendingReasonForDisabledAccount

" Returns either 'StaleAccount' or 'PasswordAgeLimit' if this
  UserProfile will be disabled for that reason the next time it logs in.
  Otherwise returns nil."

| staleLimit passwordLimit |
self passwordNeverExpires ifTrue: [ ^ nil ].

" Check and see if we're using staleAccountLimits
  and if this account has exceeded the limit "
staleLimit := AllUsers staleAccountAgeLimit.
(staleLimit ~~ nil and: [ staleLimit > 0 ]) ifTrue: [
   | lastLogin |
   (lastLogin := self lastLoginTime) ifNotNil:[
         ((DateTime now) > (lastLogin addHours: staleLimit))
         ifTrue: [ ^ 'StaleAccount' ]]].

" Check and see if we're using passwordAging
  and if this account has exceeded the limit"
passwordLimit := AllUsers passwordAgeLimit.
(passwordLimit ~~ nil and: [ passwordLimit > 0 ]) ifTrue: [
      | lastPasswordChange |
      (lastPasswordChange := self lastPasswordChange) ifNotNil:[
            ((DateTime now) > (lastPasswordChange addHours: passwordLimit))
            ifTrue: [ ^ 'PasswordAgeLimit' ]]].
^ nil
%

category: 'Private'
method: UserProfile
_postLoginInitialize

  "Invoked from GsCurrentSession >> initialize."

  loginHook ifNotNil:[ :v |
    v _isExecBlock ifTrue:[ ^ v value ] .
    v _isSymbol ifTrue:[ ^ self perform: v ]
  ].
%

category: 'Private'
method: UserProfile
_primUpdatePerformOnServerCommands: anArray opCode: anInt

"opcode 1  add elements of anArray to list of allowed commands
        2  remove elements of anArray from list of allowed commands
"
<primitive: 1124>
anArray _validateClass: Array .
anInt _validateClass: SmallInteger .
self _primitiveFailed: #_primUpdatePerformOnServerCommands:opCode:
     args: { anArray . anInt } .    
self _uncontinuableError
%

category: 'Accessing Privileges'
method: UserProfile
_privileges

"Returns a SmallInteger specifying the user's level of access to certain
 privileged system functions ordinarily performed by the GemStone data
 curator."

"SystemControl (bit 0); ObsoleteStatistics (1); SessionAccess (2);  UserPassword (3);
 DefaultObjectSecurityPolicy (4); ReservedForFutureUse (5);  OtherPassword (6);
 ObjectSecurityPolicyCreation (7); ObjectSecurityPolicyProtection (8); FileControl (9);
 GarbageCollection (10); CodeModification (11) ;
 NoPerformOnServer (12); NoUserAction (13); NoGsFileOnServer (14);
 NoGsFileOnClient (15); SessionPriority (16);  CompilePrimitives (17);
 ChangeUserId (18) MigrateObjects(19) DisableObjectReadLogging (20) 
 DynamicDisableObjectReadLogging (21) ReadOtherUserProfile(22) "

^ privileges
%

category: 'Updating Privileges'
method: UserProfile
_privileges: aSmallInteger

"Modifies the user's privileges (level of access to privileged system
 functions) to the level represented by the argument aSmallInteger."

self _validatePrivilege ifTrue:[
  (aSmallInteger _validateInstanceOf: SmallInteger ) ifTrue:[
    privileges := aSmallInteger
  ].
].
%

category: 'Private'
method: UserProfile
_removeSymbol: aSymbol

"Remove aSymbol from the Symbol List by searching the symbol list for an
 Association whose key is equal to aSymbol.  Returns true if successful.
 Otherwise no such Association was found, and it returns false."

| temp dict |
self _validatePrivilege ifTrue:[
  1 to: symbolList size do:[:j |
    dict := symbolList at: j .
    temp := dict at: aSymbol otherwise: nil .
    temp ifNotNil:[
      dict removeKey: aSymbol.
      ^ true
    ].
  ].
].
^ false
%

category: 'Private'
method: UserProfile
_securityDataInstVarAt: offset

"Returns the value of the specified instance variable in the UserSecurityData
 of the receiver.  If the value is not a special object, returns a copy of the
 value of that instance variable.

 If the receiver is not your UserProfile,
 offset=OC_USER_SECUR_X509_STATUS requires SessionAccess privilege,
 Most other values offset require either OtherPassword or ReadOtherUserProfile
 privilege, otherwise signals an error.

 offset is zero based.  0 implements 'isDisabled' and returns true or false.
 The primitive fails if you attempt to access the password or the
 oldPasswords instVars."

<protected primitive: 460>
self _primitiveFailed: #_securityDataInstVarAt: args: { offset } .
self _uncontinuableError
%

category: 'Private'
method: UserProfile
_securityDataInstVarAt: offset put: aValue

"Private.  Modifies the specified instance variable in the UserSecurityData of
 the receiver.  The sender is responsible for making a copy of aValue and
 making the copy invariant, if appropriate.  If aValue is not a special
 object, the primitive will assign aValue to the SecurityDataObjectSecurityPolicy.

 offset is zero based.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege.

 The primitive fails if you attempt to modify the password instance variable
 of the receiver's security data."

<protected primitive: 461>
self _primitiveFailed: #_securityDataInstVarAt:put: args: { offset . aValue } .
self _uncontinuableError
%

category: 'Private'
method: UserProfile
_setPassword: aString
"Private. Customers should not call this method directly."

"no checks for format of new password, omit _validateNewPassword: "
(self _validate__password: aString)
  ifFalse:[ ^ userId _error: #rtErrBadPassword ].
self _password: aString . "change the password"
AllUsers addMsgToSecurityLog: '      new password set for ' , self userId , ' .' .
^ self
%

category: 'Private'
method: UserProfile
_setPasswordNeverExpires: aBoolean

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
    result := self _securityDataInstVarAt: OC_USER_SECUR_DATA_PASSWD_NEVER_EXPIRES put: aBoolean .
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Private'
method: UserProfile
_symbolListNamesAndDictionaries

"Returns an Array of Associations which name the dictionaries in the
 receiver's SymbolList."

| result |

result := { }  .
symbolList do: [ :aDict |
  result add:
    ( aDict associationsDetect: [ :anAssoc | anAssoc value == aDict ]
             ifNone: [ SymbolAssociation newWithKey: #unnamed value: aDict ] ).
  ].
^ result.
%

category: 'Private'
method: UserProfile
_userId: newUserId

"Modifies the userId associated with the receiver to be aString."

self _validatePrivilege ifTrue:[
  userId ifNotNil:[ Error signal:'illegal reassignment of userId'].
  userId := newUserId .
].
%

category: 'External Authentication - Private'
method: UserProfile
_validateAuthenticationSchemeIsExternal

"Raises an error if the authentication scheme for the recevier is
 #GemStone, otherwise returns true. If execution continued from the
 error, returns false."

self authenticationSchemeIsGemStone ifTrue:[
   self _error: #rtErrAuthIllegalOperation
                 args: { self authenticationScheme } .
   ^ false
].
^ true
%

category: 'External Authentication - Private'
method: UserProfile
_validateAuthenticationSchemeIsGemStone

"Raises an error if the authentication scheme for the recevier is not
 #GemStone, otherwise returns true. If execution continued from the
 error, returns false."

self authenticationSchemeIsGemStone ifFalse:[
   self _error: #rtErrAuthIllegalOperation
                 args: { self authenticationScheme } .
   ^ false
].
^ true
%

category: 'External Authentication - Private'
method: UserProfile
_validateBaseDn: baseDn searchDn: filterDn

"The baseDn must be a String.  If the filterDn is nil,
 then the baseDn must contain the '%s' character sequence.
 If the filterDn is not nil, then it must be a string that
 contains the '%s' sequence, and baseDn must not contain
 the sequence.  If all of these requirements are met
 returns true, otherwise signals an error.  If execution is
 continued from the error, returns false.
"

(baseDn _validateClass: String ) ifFalse:[ ^ false ].
filterDn ifNil:[
  baseDn hasLdapWildcardPattern ifFalse:[
     self _invalidDnError: baseDn . ^ false
  ].
] ifNotNil:[
   (filterDn _validateClass: String ) ifFalse:[  ^ false ].
   filterDn hasLdapWildcardPattern
     ifFalse:[ self _invalidDnError: filterDn . ^ false ].
   baseDn hasLdapWildcardPattern
     ifTrue:[ self _invalidDnError: baseDn . ^ false ].
  ].
^ true
%

category: 'Accessing Privileges'
method: UserProfile
_validateCodeModificationPrivilege

"Generates an error if the receiver does not have
 either the CodeModification or the OtherPassword privilege.
 Uses hardcoded bit number to handle filein and upgrade cases."

self isSystemUserProfile ifTrue: [ ^ true ].
(System _inProtectedMode or: [(privileges bitAnd: 16r840) ~~ 0]) ifTrue: [ ^ true ].
self _error: #rtErrNoPriv .
self _uncontinuableError
%

category: 'Private'
method: UserProfile
_validateIsMyUserProfileOrHasPrivilege: aPrivSymbol

"Validate that the receiver is my own UserProfile or that I have the
 privilege aPrivSymbol."

| myUserProfile |

myUserProfile := System myUserProfile .
self == myUserProfile ifFalse:[
  ^ myUserProfile _validatePrivilegeName: aPrivSymbol
].
^ true
%

category: 'Private'
method: UserProfile
_validateNewPassword: aString

"Generates an error if aString is not a valid new password for the receiver.
 Otherwise, returns true .  Note that the userId of the receiver is
 always a disallowed password."

<protected>

(self _validate__password: aString)
  ifFalse:[ ^ false ] .

(AllUsers _validateNewPassword: aString ) ifFalse:[ ^ false ].

AllUsers disallowUsedPasswords ifTrue:[
  (self _oldPasswordsIncludes: aString) ifTrue:[
      aString _error: #rtDisallowedPassword .
      ^ false
  ].
].
^ true
%

category: 'Private'
method: UserProfile
_validateOneOfPrivileges: anArray 
 anArray do:[:sym | | bitNum |
   bitNum := PrivilegeNames indexOfIdentical: sym .
   bitNum <= 0 ifTrue:[ Error signal:'invalid privilege ', sym asString ].
   (privileges bitAt: bitNum ) == 1 ifTrue:[ ^ true ].
 ].
 self _error: #rtErrNoPriv .
 self _uncontinuableError
%

category: 'Updating Privileges'
method: UserProfile
_validatePrivilege

" You must have the #OtherPassword to modify UserProfiles "

^  System myUserProfile _validatePrivilegeName: #OtherPassword
%

category: 'Accessing Privileges'
method: UserProfile
_validatePrivilege: anInt

"Generates an error if the receiver does not have the privilege specified
 by anInt.  anInt is one based"

(self isSystemUserProfile or:[(privileges bitAt: anInt ) == 1]) ifTrue: [ ^ true ].
self _error: #rtErrNoPriv .
self _uncontinuableError
%

category: 'Accessing Privileges'
method: UserProfile
_validatePrivilegeName: aSymbol

"Generates an error if the receiver does not have the privilege specified
 by aSymbol, or if aSymbol is not the name of a privilege."

| bitNumber |
self isSystemUserProfile ifTrue:[ ^ true ] .
bitNumber := PrivilegeNames indexOfIdentical: aSymbol .
bitNumber > 0 ifTrue: [ ^ self _validatePrivilege: bitNumber ].
self _error: #rtErrNoPriv .
self _uncontinuableError
%

category: 'External Authentication - Private'
method: UserProfile
_validateUnixUserIdOrNil: aString

"Confirm that aString is nil or is a valid UNIX user ID on this host"

aString ifNil:[ ^ true ].
(System unixUserIdExists: aString) ifFalse:[
  aString _error: #rtErrInvalidUnixUserId .
  ^ false
].
^ true
%

category: 'Private'
method: UserProfile
_validateUserProfileRead

"Validate that the receiver is my own UserProfile or that I have the
 at least one of the #OtherPassword or #ReadOtherUserProfile privilege."

 | myPro |
 myPro := System myUserProfile .
 self == myPro  ifTrue:[ ^ true].
 myPro isSystemUserProfile ifTrue:[ ^ true ] .
 ^ myPro _validateOneOfPrivileges: #( OtherPassword ReadOtherUserProfile )
%

category: 'Private'
method: UserProfile
_validateUserProfileWrite

"Validate that the receiver is my own UserProfile or that I have the
 #OtherPassword privilege."

^ self _validateIsMyUserProfileOrHasPrivilege: #OtherPassword
%

category: 'Private'
method: UserProfile
_validate__password: aString

^ self class _validate__password: aString forUserId: userId
%

! Class extensions for 'UserProfileSet'

!		Class methods for 'UserProfileSet'

removeallmethods UserProfileSet
removeallclassmethods UserProfileSet

category: 'Creation'
classmethod: UserProfileSet
new

"Disallowed"

self shouldNotImplement: #new
%

!		Instance methods for 'UserProfileSet'

category: 'Adding'
method: UserProfileSet
add: aUserProfile

"Adds a new UserProfile to the receiver.  Generates an error if aUserProfile
 has a userId that duplicates an existing element of the receiver.
 Requires OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
    result := self _add: aUserProfile .
  ]
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Adding'
method: UserProfileSet
addAll: aCollection
    ^ self shouldNotImplement: #addAll:
%

category: 'Group Membership'
method: UserProfileSet
addGroup: aGroupString

"Adds all the users in the receiver to the group represented by aGroupString.
 If the current session does not have the authorizations required for this
 operation, raises an error.
 Requires OtherPassword privilege."

self _validatePrivilege ifTrue:[
  self accompaniedBy: aGroupString do:[ :grpStr :aUserProfile | aUserProfile addGroup: grpStr ]
]
%

category: 'Logging'
method: UserProfileSet
addMsgToSecurityLog: aString

"This method adds the date, time and userId prefix to the specified message
 string and includes the resulting string in the security log.  The Stone
log file is the security log.

 Requires OtherPassword privilege. "

| message |

self _validatePrivilege ifFalse:[ ^ nil ].
message := String withAll:'    SecurityLog: ' .
message addAll:  DateTime now asString ; addAll: ', ';
        addAll: System myUserProfile userId ;  lf ;
        addAll: aString .
System addAllToStoneLog: message .
%

category: 'Adding'
method: UserProfileSet
addNewUserWithId: userIdString
password: passwordString

"Creates a new UserProfile and adds it to the receiver.  The new UserProfile
 has no privileges and belongs to no groups and has the default
 GsObjectSecurityPolicy, which is nil.

 This default method can be used by the data curator in batch user installations.
 Returns the new UserProfile.

 The current session must be in a transaction with no uncommitted changes
 or an error will be generated.
 This method will writeLock AllUsers and SystemRepository for
 the duration of the method for the creation of the new UserProfile .
 If a writeLock cannot be obtained on AllUsers or SystemRepository ,
 an error will be generated.

 This method will commit its changes.

 Requires OtherPassword privilege and write access to DataCuratorObjectSecurityPolicy."

^ self addNewUserWithId: userIdString
       password: passwordString
       createNewSecurityPolicy: false
%

category: 'Adding'
method: UserProfileSet
addNewUserWithId: userIdString
password: passwordString
createNewSecurityPolicy: aBoolean

"Creates a new UserProfile and adds it to the receiver.  The new UserProfile
 has no privileges and belongs to no groups.  If aBoolean is true, a new
 GsObjectSecurityPolicy is created for the new UserProfile.  Otherwise the
 new UserProfile has the default GsObjectSecurityPolicy (nil).
 Returns the new UserProfile.

 The current session must be in a transaction with no uncommitted changes
 or an error will be generated.
 This method will writeLock AllUsers and SystemRepository for
 the duration of the method for the creation of the new UserProfile .
 If a writeLock cannot be obtained on AllUsers or SystemRepository ,
 an error will be generated.

 This method will commit its changes.

 Requires OtherPassword privilege and write access to DataCuratorObjectSecurityPolicy.
 If aBoolean is true, requires ObjectSecurityPolicyCreation.  "

| commitAttempted |
self _validatePrivilege ifFalse:[ ^ nil ].
^ [ | newSeg newUp oldUp |
  oldUp := self userWithId: userIdString ifAbsent:[ nil ].
  oldUp ifNotNil:[ self _error: #rtErrUserIdAlreadyExists args: { userIdString } ].
  System writeLock: self
    ifDenied:[ self error:'unable to writeLock AllUsers' ]
    ifChanged:[
       System needsCommit ifTrue:[
	 System removeLock: self .
	 self _error: #rtErrAbortWouldLoseData
       ].
       System beginTransaction
    ].
  System writeLock: SystemRepository
    ifDenied:[ System removeLocksForSession.
	       self error:'unable to writeLock SystemRepository' ]
    ifChanged:[
      System needsCommit ifTrue:[
	System removeLocksForSession .
	self _error: #rtErrAbortWouldLoseData
      ].
      System beginTransaction
    ].
  "New GsObjectSecurityPolicy must be committed first!"
  aBoolean ifTrue:[
    newSeg := GsObjectSecurityPolicy newInRepository: SystemRepository .
    newSeg name: userIdString, 'ObjectSecurityPolicy' .
    newSeg worldAuthorization: #read .
    System commitTransaction ifFalse:[
      System removeLocksForSession .
      System abortTransaction.
      self error:'commit of new GsObjectSecurityPolicy failed'
    ].
  ].
  newUp := self addNewUserWithId: userIdString password: passwordString
	    defaultObjectSecurityPolicy: newSeg privileges:  #() inGroups:  #() .
  newSeg ifNotNil:[ newSeg owner: newUp] .
  commitAttempted := true .
  System commitAndReleaseLocks ifFalse:[
    System removeLocksForSession .
    System abortTransaction.
    self error:'commit of new UserProfile failed'
  ].
  newUp
] ensure:[
  commitAttempted ifNil:[ System removeLocksForSession ].
].
%

category: 'Adding'
method: UserProfileSet
addNewUserWithId: userIdString
password: passwordString
defaultObjectSecurityPolicy: anObjectSecurityPolicy

"Creates a new UserProfile and adds it to AllUsers.  The new UserProfile
 has no privileges, and belongs to no groups.

 anObjectSecurityPolicy is used as the defaultObjectSecurityPolicy of the new UserProfile and
 must be a committed GsObjectSecurityPolicy or nil.
 Pass nil as the value of anObjectSecurityPolicy to have the default GsObjectSecurityPolicy be
 the world-write security policy for applications not using object-level security.
"

^ self addNewUserWithId: userIdString
          password: passwordString
          defaultObjectSecurityPolicy: anObjectSecurityPolicy
          privileges:  #()
          inGroups:  #()
%

category: 'Adding'
method: UserProfileSet
addNewUserWithId: userIdString
password: passwordString
defaultObjectSecurityPolicy: anObjectSecurityPolicy
privileges: anArrayOfStrings
inGroups: aCollectionOfGroupStrings

"Creates a new UserProfile with the associated characteristics and adds it to
 AllUsers.  Generates an error if the userIdString duplicates the userId
 of any existing element of the receiver.  Returns the new UserProfile.

 In order to log in to GemStone, the user must be
 authorized to read and write in the specified default GsObjectSecurityPolicy.
 anObjectSecurityPolicy must be a committed GsObjectSecurityPolicy or nil.
 Pass nil as the value of anObjectSecurityPolicy to have the default GsObjectSecurityPolicy be
 the world-write security policy for applications not using object-level security.

 Requires OtherPassword privilege and write access to the DataCurator objectSecurityPolicy.
"

self _validatePrivilege ifTrue:[
  ^ UserProfile newWithUserId: userIdString
                       password: passwordString
                       defaultObjectSecurityPolicy: anObjectSecurityPolicy
                       privileges: anArrayOfStrings
                       inGroups: aCollectionOfGroupStrings
].
^ nil
%

category: 'Deprecated'
method: UserProfileSet
addNewUserWithId: userIdString
password: passwordString
defaultSegment: anObjectSecurityPolicy
"Deprecated - use method with #defaultObjectSecurityPolicy: instead of #defaultSegment: keyword"

self deprecated: 'UserProfileSet>>addNewUserWithId:password:defaultSegment: deprecated v3.0.
Use #addNewUserWithId:password:defaultObjectSecurityPolicy: instead.'.

^ self addNewUserWithId: userIdString
          password: passwordString
          defaultObjectSecurityPolicy: anObjectSecurityPolicy
          privileges:  #()
          inGroups:  #()
%

category: 'Deprecated'
method: UserProfileSet
addNewUserWithId: userIdString
password: passwordString
defaultSegment: anObjectSecurityPolicy
privileges: anArrayOfStrings
inGroups: aCollectionOfGroupStrings
"Deprecated - use method with #defaultObjectSecurityPolicy: instead of #defaultSegment: keyword"

self deprecated: 'UserProfileSet>>addNewUserWithId:password:defaultSegment:privileges:inGroups: deprecated v3.0.
Use #addNewUserWithId:password:defaultObjectSecurityPolicy:privileges:inGroups: instead.'.

^self addNewUserWithId: userIdString
	password: passwordString
	defaultObjectSecurityPolicy: anObjectSecurityPolicy
	privileges: anArrayOfStrings
	inGroups: aCollectionOfGroupStrings
%

category: 'Group Membership'
method: UserProfileSet
auditGroups
"Audit each user and every group, to make sure each
 user's groups and each group's members match.
 Raise an error on the first mismatch and return false."

self do: [:aUser | (aUser auditGroups) ifFalse: [^false] ].
^UserProfileGroup auditGroups.
%

category: 'Enumerating'
method: UserProfileSet
classesDo: aBlock

"Execute 3-argument block over all classes (plus those referenced by
 its class history) that are contained by all SymbolDictionaries held
 by all users held in this UserProfileSet.  Checks are made to avoid
 redundant processing of classes/symbol dictionaries already scanned.

 Arguments to block are:
    [:aUserProfile :aSymbolDictionary :aClass | ... ]

"

  | coveredSymDics coveredClasses |

  coveredSymDics := IdentitySet new.
  coveredClasses := IdentitySet new.

  self do: [ :aUserProfile |
    aUserProfile symbolList do: [ :aSymbolDictionary |
      (coveredSymDics includes: aSymbolDictionary) ifFalse: [
        coveredSymDics add: aSymbolDictionary.
        aSymbolDictionary valuesDo: [ :namedClass |
          (namedClass isKindOf: Class) ifTrue: [
            namedClass classHistory do: [ :aClass |
              (coveredClasses includes: aClass) ifFalse: [
                coveredClasses add: aClass.
                aBlock value: aUserProfile
                       value: aSymbolDictionary
                       value: aClass ]]]]]]]
%

category: 'Formatting'
method: UserProfileSet
dictionaryNames

"Returns a formatted String describing each user's symbol list.  For each user,
 the String contains the userId, and the position and name of each Dictionary
 in that user's symbol list.

 This method assumes that each Dictionary in the symbol list contains an
 Association whose value is that Dictionary.  If any Dictionary does not
 contain such an Association, it is represented in the result String as
 '(unnamed Dictionary)'."

"Example use: 'AllUsers dictionaryNames' "

| tempStr aUserPro |

tempStr := String new.

1 to: self size do:[:j|
  aUserPro := self _at: j .
  tempStr add: (Character lf).
  tempStr add: (aUserPro userId asString).
  tempStr add: (Character lf).
  tempStr add: (aUserPro dictionaryNames).
  ].

^tempStr
%

category: 'Querying'
method: UserProfileSet
disableUsersWithOldPasswordEncryption

"Disables logins for all the elements of the receiver that
 are using the old password encryption scheme.  The old encryption
 scheme was used in versions prior to 3.1.0.

 Generates an error if you do not have the #OtherPassword privilege.

 Answers the number of elements in the receiver for which logins were
 disabled."

| result |
result := 0 .
self do: [:each |
              each usesOldPasswordEncryption
                 ifTrue:[
                   result := result + 1 .
                   each disableWithReason: 'No logins since upgrade' ]
].
^ result
%

category: 'Accessing'
method: UserProfileSet
disallowedPasswords

"Returns the set of disallowed passwords defined for the receiver."

 ^ disallowedPasswords
%

category: 'Accessing'
method: UserProfileSet
disallowUsedPasswords

"Returns the value of the disallowUsedPasswords instance variable."

^ (securityBits bitAnd: 1) ~~ 0
%

category: 'Updating'
method: UserProfileSet
disallowUsedPasswords: aBoolean

"Determines whether the system prevents users from changing their password to
 a password previously used by the same user.  The number of previous passwords
 disallowed may be set by the #numberOfUsedPasswordsDisallowed: method.

 Requires the OtherPassword privilege. "

self _validatePrivilege ifFalse: [ ^ nil ].
aBoolean _validateClass: Boolean .
self disallowUsedPasswords == aBoolean
  ifFalse:[ self _updateSecurityBitsForMask: 1 newState: aBoolean ]
%

category: 'Querying'
method: UserProfileSet
findDisabledUsers

"Returns a SortedCollection of UserProfiles that are disabled.  The result
 includes users whose accounts have been disabled because their passwords have
 expired, or whose accounts were not used within the interval defined by the
 staleAccountAgeLimit, or who failed to login within the number of tries
 specified by the Stone configuration parameter.

 Generates an error if you do not have OtherPassword privilege."

| result |

result := SortedCollection sortBlock:[:a :b | a userId < b userId ].
self accompaniedBy: result do:[ :res :aUserProfile |
  aUserProfile isDisabled ifTrue:[ res add: aUserProfile ].
].
^ result .
%

category: 'Disk Space Management'
method: UserProfileSet
findObjectsLargerThan: aSize limit: aLimit

"Searches the symbol list of each user in the receiver for named objects larger
 than aSize.  Returns an Array of the form { { aUserId . aKey . anObject} }
 where aKey is the symbolic representation of anObject such that:

 ((AllUsers userWithId: aUserId)
   resolveSymbol: aKey ) value == anObject

 is true.  For each user in the receiver, the search continues until there are
 no more named objects in the user's symbol list, or until the size of the
 result reaches the specified maximum aLimit.

 Generates an error if an authorization violation occurs."

| test result optimizeSet theUserId aUser aSymList aDict |

aSize _validateClass: SmallInteger.
aLimit _validateClass: SmallInteger.

test := { } .
result := { } .
optimizeSet := IdentitySet new.

1 to: self size do:[:j|
    aUser := self _at: j .
    theUserId := aUser userId.
    aSymList := aUser symbolList .
    1 to: aSymList size do:[:k|
          aDict := aSymList at: k .
          (optimizeSet includesIdentical: aDict) "avoid repeated scans of
                                                  shared Dictionaries"
          ifFalse:
            [ optimizeSet add: aDict.
              aDict associationsDo: [:anAssoc | | theVal |
                theVal := anAssoc value.
                  (theVal size > aSize)
                  ifTrue:
                    [ (result size >= aLimit)
                      ifTrue:
                        [ ^result ].
                      (test includesIdentical: theVal)
                      ifFalse:
                        [ test add: theVal.
                          result add: { theUserId . anAssoc key . theVal }.
                        ].
                   ].  "ifTrue: "
                ].  "aDict do: "
            ].  "ifFalse: "
      ].  "aUser symbolList do:"
  ].  "AllUsers do: "

^result
%

category: 'Querying'
method: UserProfileSet
findProfilesWithAgingPassword

"Returns a SortedCollection of UserProfiles whose passwords will expire sooner
 than passwordAgeWarning hours from now.

 Generates an error if you do not have OtherPassword privilege."

| result nowDtime endWarningDtime |

self _validatePrivilege ifFalse:[ ^ nil ].
result := SortedCollection sortBlock:[:a :b | a userId < b userId ] .

(((passwordAgeWarning ~~ nil and:[ passwordAgeWarning > 0])
  and:[ passwordAgeLimit ~~ nil]) and:[ passwordAgeLimit > 0])
ifTrue:[
  nowDtime := DateTime now.
  endWarningDtime := nowDtime addHours: passwordAgeWarning .
  self do:[ :aUserProfile | | expireDtime |
    (aUserProfile passwordNeverExpires) ifFalse: [
      (aUserProfile lastPasswordChange ~~ nil) ifTrue:[
        expireDtime := aUserProfile lastPasswordChange addHours: passwordAgeLimit.
        expireDtime <= endWarningDtime ifTrue:[ result add: aUserProfile ].
      ].
    ].
  ].
].
^ result
%

category: 'Accessing'
method: UserProfileSet
maxCharsOfSameType

"Returns the value of the maxCharsOfSameType instance variable."

^ maxCharsOfSameType
%

category: 'Updating'
method: UserProfileSet
maxCharsOfSameType: aPositiveInteger

"Sets the value of the maxCharsOfSameType instance variable.
 Requires OtherPassword privilege."

self _validatePrivilege ifTrue:[
  (aPositiveInteger _validateClass: SmallInteger ) ifTrue:[
    aPositiveInteger < 0 ifTrue:[
      ^ aPositiveInteger _error: #rtErrArgNotPositive
    ].
    maxCharsOfSameType := aPositiveInteger
  ]
]
%

category: 'Accessing'
method: UserProfileSet
maxConsecutiveChars

"Returns the value of the maxConsecutiveChars instance variable."

^ maxConsecutiveChars
%

category: 'Updating'
method: UserProfileSet
maxConsecutiveChars: aPositiveInteger

"Sets the value of the maxConsecutiveChars instance variable.
 Requires OtherPassword privilege."

self _validatePrivilege ifTrue:[
  (aPositiveInteger _validateClass: SmallInteger ) ifTrue:[
    aPositiveInteger < 0 ifTrue:[
     ^  aPositiveInteger _error: #rtErrArgNotPositive
    ].
    maxConsecutiveChars := aPositiveInteger
  ]
]
%

category: 'Accessing'
method: UserProfileSet
maxPasswordSize

"Returns the value of the maxPasswordSize instance variable."

^ maxPasswordSize
%

category: 'Updating'
method: UserProfileSet
maxPasswordSize: aPositiveInteger

"Sets the value of the maxPasswordSize instance variable.
 Requires OtherPassword privilege."

self _validatePrivilege ifTrue:[
  (aPositiveInteger _validateClass: SmallInteger) ifTrue:[
    aPositiveInteger < 0 ifTrue:[
      ^ aPositiveInteger _error: #rtErrArgNotPositive
    ].
    aPositiveInteger > 1024 ifTrue:[
       aPositiveInteger _error: #errArgTooLarge args:{ 1024 }
    ].
    maxPasswordSize := aPositiveInteger
  ]
]
%

category: 'Accessing'
method: UserProfileSet
maxRepeatingChars

"Returns the value of the maxRepeatingChars instance variable."

^ maxRepeatingChars
%

category: 'Updating'
method: UserProfileSet
maxRepeatingChars: aPositiveInteger

"Sets the value of the maxRepeatingChars instance variable.
 Requires OtherPassword privilege."

self _validatePrivilege ifTrue:[
  (aPositiveInteger _validateClass: SmallInteger ) ifTrue:[
    aPositiveInteger < 0 ifTrue:[
      ^ aPositiveInteger _error: #rtErrArgNotPositive
    ].
    maxRepeatingChars := aPositiveInteger
  ]
]
%

category: 'Deprecated'
method: UserProfileSet
membersOfGroup: aString

self deprecated: 'UserProfileSet class >> membersOfGroup: deprecated v3.4.  Replace with
   (UserProfileGroup groupWithName aString) users, or userIdsInGroup:'.
^self userIdsInGroup: aString
%

category: 'Accessing'
method: UserProfileSet
minPasswordSize

"Returns the value of the minPasswordSize instance variable."

^ minPasswordSize
%

category: 'Updating'
method: UserProfileSet
minPasswordSize: aPositiveInteger

"Sets the value of the minPasswordSize instance variable.
 Requires OtherPassword privilege."

self _validatePrivilege ifTrue:[
  (aPositiveInteger _validateClass: SmallInteger) ifTrue:[
    aPositiveInteger < 0 ifTrue:[
      ^ aPositiveInteger _error: #rtErrArgNotPositive
    ].
    aPositiveInteger == 0 ifTrue:[
      ^ aPositiveInteger _error: #rtErrArgOutOfRange args:{ 1 . 1024 }
    ].
    minPasswordSize := aPositiveInteger
  ]
]
%

category: 'Accessing'
method: UserProfileSet
numberOfUsedPasswordsDisallowed

"Answers a SmallInteger between 0 and 65535 which is the number of previous passwords
 which may not be reused by users when changing their password.  A value of zero means
 users may not reuse any previous password.

 To disallow previous passwords, the following code must also be executed:
   AllUsers disallowUsedPasswords: true
"

^ (securityBits bitShift: -5) bitAnd: 65535
%

category: 'Updating'
method: UserProfileSet
numberOfUsedPasswordsDisallowed: aSmallInt

"Sets the number of previous passwords which may not be reused by users when
 changing their password.  aSmallInt must be a SmallInteger in the range of
 0 to 65535.  A value of zero means users may not reuse any previous password.

 Requires the OtherPassword privilege.

 To disallow previous passwords, the following code must also be executed:
   AllUsers disallowUsedPasswords: true
"

self _validatePrivilege ifFalse: [ ^ nil ].
(aSmallInt _validateClass: SmallInteger) ifFalse: [ ^ nil].
((aSmallInt < 0) or:[ aSmallInt > 65535 ])
     ifTrue:[ ^ aSmallInt  _error: #rtErrArgOutOfRange args:{ 0 . 65535 } ] .
self numberOfUsedPasswordsDisallowed == aSmallInt
  ifFalse:[securityBits := (securityBits bitAnd: 31) bitOr: (aSmallInt bitShift: 5) ]
%

category: 'Accessing'
method: UserProfileSet
passwordAgeLimit

"Returns the value of the passwordAgeLimit instance variable."

^ passwordAgeLimit
%

category: 'Updating'
method: UserProfileSet
passwordAgeLimit: numberOfHours

"Sets the global, default password age limit for UserProfiles.

 If numberOfHours is greater than zero, the password of a UserProfile
 will expire after the specified number of hours since the password was
 last changed if the following conditions are all true:

 1. The UserProfile is not a reserved, system UserProfile.  The special system
    UserProfiles are: SystemUser, SymbolUser, DataCurator, Nameless, and GcUser.

 2. The passwordAgeLimit has not been overridden or disabled in the UserProfile.
    See the method UserProfile>>passwordAgeLimit: for more information.

 The argument numberOfHours must be a SmallInteger or a Float and must be at
 least zero and at most 536870911.

 Applies only to UserProfiles which use GemStone authentication.

 Requires OtherPassword privilege."

 | message |

  self _validatePrivilege ifFalse:[ ^ nil ].
  (numberOfHours _validateClasses:{ SmallInteger . Float }) ifFalse:[ ^ nil ] .
  numberOfHours < 0 ifTrue:[
    ^ numberOfHours _error: #rtErrArgNotPositive
  ].
  numberOfHours > 536870911 ifTrue:[
    ^ numberOfHours _error: #errArgTooLarge args:{ 536870911 }
  ].

  passwordAgeLimit := numberOfHours .

  message :=
'      passwordAgeLimit for AllUsers changed to '
   , numberOfHours asString , ' hours.' .
  self addMsgToSecurityLog: message .
%

category: 'Accessing'
method: UserProfileSet
passwordAgeWarning

"Returns the value of the passwordAgeWarning instance variable."

^ passwordAgeWarning
%

category: 'Updating'
method: UserProfileSet
passwordAgeWarning: numberOfHours

"Sets the global, default password age warning for UserProfiles.

 If numberOfHours is greater than zero, a warning indicating the password for
 the UserProfile will be issued at login time if all of the following conditions
 are true:

 1. The UserProfile is not a reserved, system UserProfile.  The special system
    UserProfiles are: SystemUser, SymbolUser, DataCurator, Nameless, and GcUser.

 2. A non-zero passwordAgeLimit has been set which applies to the UserProfile.

 3. The remaining number of hours before the password for the UserProfile expirees is
    less than or equal to numberOfHours.

 The warning issued at login time is in the form of a non-fatal error raised immediately
 after the login succeeds.

 The argument numberOfHours must be a SmallInteger or a Float and must be at
 least zero and at most 536870911.

 Requires OtherPassword privilege."

self _validatePrivilege ifFalse:[ ^ nil ].
(numberOfHours _validateClasses:{ SmallInteger . Float }) ifFalse:[ ^ nil ].
numberOfHours < 0 ifTrue:[
    ^ numberOfHours _error: #rtErrArgNotPositive
] .
numberOfHours > 536870911 ifTrue:[
  ^ numberOfHours _error: #errArgTooLarge args:{ 536870911 }
] .

passwordAgeWarning := numberOfHours
%

category: 'Accessing'
method: UserProfileSet
passwordRequiresDigit

"Answers true if passwords changed by users are required to contain
 at least one digit"

^ (securityBits bitAnd: 8) ~~ 0
%

category: 'Updating'
method: UserProfileSet
passwordRequiresDigit: aBoolean

"Enables or disables the requirement that new passwords contain at least one
 digit.

 Requires the OtherPassword privilege."

self _validatePrivilege ifFalse: [ ^ nil ].
aBoolean _validateClass: Boolean .
self passwordRequiresDigit == aBoolean
  ifFalse:[ self _updateSecurityBitsForMask: 8 newState: aBoolean ].
%

category: 'Accessing'
method: UserProfileSet
passwordRequiresLowercase

"Answers true if passwords changed by users are required to contain
 at least one lower case character."

^ (securityBits bitAnd: 4) ~~ 0
%

category: 'Updating'
method: UserProfileSet
passwordRequiresLowercase: aBoolean

"Enables or disables the requirement that new passwords contain at least one
 lower case character.

 Requires the OtherPassword privilege."

self _validatePrivilege ifFalse: [ ^ nil ].
aBoolean _validateClass: Boolean .
self passwordRequiresLowercase == aBoolean
  ifFalse:[ self _updateSecurityBitsForMask: 4 newState: aBoolean ].
%

category: 'Accessing'
method: UserProfileSet
passwordRequiresSymbol

"Answers true if passwords changed by users are required to contain
 at least one Symbol character.  In the scope of passwords, a Symbol
 character is any character that answers false to the message
 #isAlphaNumeric ."

^ (securityBits bitAnd: 16) ~~ 0
%

category: 'Updating'
method: UserProfileSet
passwordRequiresSymbol: aBoolean

"Enables or disables the requirement that new passwords contain at least one
 symbol character.  In the scope of passwords, a Symbol character is any
 character that answers false to the message #isAlphaNumeric .

 Requires the OtherPassword privilege."

self _validatePrivilege ifFalse: [ ^ nil ].
aBoolean _validateClass: Boolean .
self passwordRequiresSymbol == aBoolean
  ifFalse:[ self _updateSecurityBitsForMask: 16 newState: aBoolean ].
%

category: 'Accessing'
method: UserProfileSet
passwordRequiresUppercase

"Answers true if passwords changed by users are required to contain
 at least one upper case character."

^ (securityBits bitAnd: 2) ~~ 0
%

category: 'Updating'
method: UserProfileSet
passwordRequiresUppercase: aBoolean

"Enables or disables the requirement that new passwords contain at least one
 upper case character.

 Requires the OtherPassword privilege."

self _validatePrivilege ifFalse: [ ^ nil ].
aBoolean _validateClass: Boolean .
self passwordRequiresUppercase == aBoolean
  ifFalse:[ self _updateSecurityBitsForMask: 2 newState: aBoolean ]
%

category: 'Illegal Operations'
method: UserProfileSet
remove: aUserProfile

"Disallowed."
self shouldNotImplement: #remove:
%

category: 'Illegal Operations'
method: UserProfileSet
remove: aUserProfile ifAbsent: aBlock

"Disallowed."
self shouldNotImplement: #remove:ifAbsent:
%

category: 'Illegal Operations'
method: UserProfileSet
remove: aUserProfile otherwise: something

"Disallowed."
self shouldNotImplement: #remove:otherwise:
%

category: 'Illegal Operations'
method: UserProfileSet
removeAll: aCollection

"Disallowed."
self shouldNotImplement: #removeAll:
%

category: 'Illegal Operations'
method: UserProfileSet
removeAllPresent: aCollection

"Disallowed."
self shouldNotImplement: #removeAllPresent:
%

category: 'Removing'
method: UserProfileSet
removeAndCleanup: aUserProfile

"Remove a UserProfile from the receiver.
 Also transfers ownership of any GsObjectSecurityPolicy objects owned
 by the UserProfile being removed to the UserProfile for the current session.
 Also adds a new DeletedUserProfile to the AllDeletedUsers collection
 in Globals.

 Requires write access to the GsObjectSecurityPolicy objects for both DataCurator
 and the UserProfile object to be removed.  Also requires the #OtherPassword
 privilege."

^ self removeAndCleanup: aUserProfile migrateSecurityPoliciesTo: System myUserProfile
%

category: 'Removing'
method: UserProfileSet
removeAndCleanup: aUserProfile migrateSecurityPoliciesTo: anotherUserProfile


"Remove a UserProfile from the receiver.
 Also transfers ownership of any GsObjectSecurityPolicy objects owned
 by the UserProfile being removed to anotherUserProfile.  Also adds a new
 DeletedUserProfile to the AllDeletedUsers collection in Globals.

 Requires write access to the GsObjectSecurityPolicy objects for both DataCurator
 and the UserProfile object to be removed.  Also requires the #OtherPassword
 privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[ | userId oldval |
  self _validatePrivilege ifFalse:[ ^ nil ].
  "protect against removal of special users"
  userId := aUserProfile userId.
  (UserProfile isSpecialUserId: userId) ifTrue:[ userId _error: #rtErrRemoveSpecialUser ].

  oldval := userIdDictionary at: userId otherwise: nil .
  (oldval ~~ aUserProfile) ifTrue:[
    ArgumentError signal:'(AllUsers at: aUserProfile userId) not identical to aUserProfile'
  ].
  oldval := userIdDictionary removeKey: userId otherwise: nil .
  (oldval ~~ aUserProfile) ifTrue:[
    ArgumentError signal:'(AllUsers at: aUserProfile userId) not identical to aUserProfile (2)'
  ].
  oldval ifNotNil:[
      aUserProfile removeFromAllGroups .
      self _removeSecurityDataFor: aUserProfile .
      self _migrateSecurityPoliciesFrom: aUserProfile to: anotherUserProfile .
      self _addToDeletedUsers: aUserProfile .
      ^ aUserProfile
  ].
  result := super remove: aUserProfile ifAbsent:[ nil ] .
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Removing'
method: UserProfileSet
removeAndCleanupUserWithId: aUserId ifAbsent: aBlock

"Remove a UserProfile whose userId is equal to aUserId from the receiver.
 Also transfers ownership of any GsObjectSecurityPolicy objects owned
 by the UserProfile being removed to the UserProfile for the current session.
 Also adds a new DeletedUserProfile to the AllDeletedUsers collection
 in Globals.

 Executes aBlock if the UserProfile could not be found.

 Requires write access to the GsObjectSecurityPolicy objects for both DataCurator
 and the UserProfile object to be removed.  Also requires the #OtherPassword
 privilege."

| aUserProfile |
aUserProfile := self userWithId: aUserId ifAbsent:[ nil ] .
aUserProfile ifNil:[ ^ aBlock value ] .

^ self removeAndCleanup: aUserProfile migrateSecurityPoliciesTo: System myUserProfile
%

category: 'Removing'
method: UserProfileSet
removeAndCleanupUserWithId: aUserId migrateSecurityPoliciesToUserWithId: anotherId ifAnyAbsent: aBlock

"Remove a UserProfile whose userId is equal to aUserId from the receiver.
 Also transfers ownership of any GsObjectSecurityPolicy objects owned
 by the UserProfile being removed to another UserProfile whose userId is equal
 to anotherId.  Also adds a new DeletedUserProfile to the AllDeletedUsers
 collection in Globals.

 Executes aBlock and returns if either UserProfile could not be found.

 Requires write access to the GsObjectSecurityPolicy objects for both DataCurator
 and the UserProfile object to be removed.  Also requires the #OtherPassword
 privilege."

| aUserProfile anotherUserProfile |
aUserProfile := self userWithId: aUserId ifAbsent:[ nil ] .
aUserProfile ifNil:[ ^ aBlock value ] .

anotherUserProfile := self userWithId: anotherId ifAbsent:[ nil ] .
anotherUserProfile ifNil:[ ^ aBlock value ] .

^ self removeAndCleanup: aUserProfile migrateSecurityPoliciesTo: anotherUserProfile
%

category: 'Group Membership'
method: UserProfileSet
removeGroup: aGroupString

"Removes all the users in the receiver from the group represented by
 aGroupString.  If the current session does not have the authorizations required
 for this operation, raises an error.
 Requires OtherPassword privilege."

self _validatePrivilege ifTrue:[
  self accompaniedBy: aGroupString do:[ :grpStr :aUserProfile | aUserProfile removeGroup: grpStr ].
]
%

category: 'Illegal Operations'
method: UserProfileSet
removeIdentical: aUserProfile

"Disallowed."
self shouldNotImplement: #removeIdentical:
%

category: 'Illegal Operations'
method: UserProfileSet
removeIdentical: aUserProfile ifAbsent: someBlock

"Disallowed."
self shouldNotImplement: #removeIdentical:ifAbsent:
%

category: 'Illegal Operations'
method: UserProfileSet
removeIdentical: aUserProfile otherwise: something

"Disallowed."
self shouldNotImplement: #removeIdentical:otherwise:
%

category: 'Illegal Operations'
method: UserProfileSet
removeIfPresent: aUserProfile

"Disallowed."
self shouldNotImplement: #removeIfPresent:
%

category: 'Class Membership'
method: UserProfileSet
species

"Returns a Class, an instance of which should be used as the result of
 collect: or other projections applied to the receiver."

^ IdentitySet
%

category: 'Class Membership'
method: UserProfileSet
speciesForCollect

"Returns a Class, an instance of which should be used as the result of
 collect: or other projections applied to the receiver."

^ Array
%

category: 'Accessing'
method: UserProfileSet
staleAccountAgeLimit

"Returns the value of the staleAccountAgeLimit instance variable."

^ staleAccountAgeLimit
%

category: 'Updating'
method: UserProfileSet
staleAccountAgeLimit: numberOfHours

"Sets the global, default stale account age limit for UserProfiles.

 If numberOfHours is greater than zero, the UserProfile will be disabled
 if all of the following conditions are true:

 1. The UserProfile is not a reserved, system UserProfile.  The special system
    UserProfiles are: SystemUser, SymbolUser, DataCurator, Nameless, and GcUser.

 2. The staleAccountAgeLimit has not been overridden or disabled in the UserProfile.
    See the method UserProfile>>staleAccountAgeLimit: for more information.

 3. The last successful login for the UserProfile occurred more than numberOfHours in the
    past.  Note that the staleAccountAgeLimit will never prevent the first login to a
    newly-created UserProfile.

 The argument numberOfHours must be a SmallInteger or a Float and must be at
 least zero and at most 536870911.

 Applies only to UserProfiles which use GemStone authentication.

 Requires OtherPassword privilege."

 | message |
  self _validatePrivilege ifFalse:[ ^ nil ].
  (numberOfHours _validateClasses:{ SmallInteger . Float })  ifFalse:[ ^ nil ].
  numberOfHours < 0 ifTrue:[
    ^ numberOfHours _error: #rtErrArgNotPositive
    ].
  numberOfHours > 536870911 ifTrue:[
    ^ numberOfHours _error: #errArgTooLarge args:{ 536870911 }
    ].

  staleAccountAgeLimit := numberOfHours .

  message :=
'      staleAccountAgeLimit for AllUsers changed to '
   , numberOfHours asString , ' hours.' .
  self addMsgToSecurityLog: message .
%

category: 'Group Membership'
method: UserProfileSet
userIdsInGroup: aString

"Returns an IdentitySet containing the userId for each member of the
 group with the sepcified name.
 If the group contains no members, returns an empty IdentitySet.
 Generates an error if aString is not a kind of String, or if aString is not
 defined in the global object AllGroups."

|  theGroup |

theGroup := UserProfileGroup _validateGroupString: aString .
^ theGroup userIds
%

category: 'Group Membership'
method: UserProfileSet
usersInGroup: aGroupString

"Returns all the elements of the receiver that are in the group represented by
 aGroupString.  If the current session does not have the authorizations required
 for this operation, raises an error."

| theGroup |
theGroup := UserProfileGroup _validateGroupString: aGroupString .
^ self * theGroup
%

category: 'Querying'
method: UserProfileSet
usersWithOldPasswordEncryption

"Returns all the elements of the receiver that are using the old
 password encryption scheme.  The old encryption scheme was
 used in versions prior to 3.1.0.

 Generates an error if you do not have the #OtherPassword privilege."


^ self select: [:each | each usesOldPasswordEncryption]
%

category: 'Querying'
method: UserProfileSet
userWithId: aString ifAbsent: aBlock

"Searches the receiver for a UserProfile whose userId is equal to aString, and
 returns that UserProfile.  Evaluates the argument aBlock if no userId
 is equal to aString."

"Example: (AllUsers userWithId: 'DataCurator') "

| aUserProfile aStr |
aStr := aString asString .
userIdDictionary ifNotNil:[
  ^ userIdDictionary at: aStr ifAbsent: aBlock
  ].
aUserProfile:=
   self detect: [:aUserPro | aUserPro.userId = aStr] ifNone: aBlock .

^aUserProfile "Should only be one as UserId are unique"
%

category: 'Private'
method: UserProfileSet
_add: aUserProfile

"Adds a new UserProfile to the receiver.  Generates an error if aUserProfile
 has a userId that duplicates an existing element of the receiver."

<protected>

| userId |

aUserProfile _validateClass: UserProfile.
userId := aUserProfile userId.
self userWithId: userId ifAbsent:[
  userIdDictionary ~~ nil ifTrue:[
     userIdDictionary at: userId put: aUserProfile .
     ].
  aUserProfile _newSecurityData .
  self __add: aUserProfile .
  ^ aUserProfile
  ].
^ self _error: #rtErrUserIdAlreadyExists args: { userId }
%

category: 'Illegal Operations'
method: UserProfileSet
_addIfAbsent: aUserProfile

"Disallowed."
self shouldNotImplement: #_addIfAbsent:
%

category: 'Private'
method: UserProfileSet
_addToDeletedUsers: aUserProfile

| cls |
cls := Globals at: #DeletedUserProfile .
(Globals at: #AllDeletedUsers) add: (cls createFrom: aUserProfile)
%

category: 'Private'
method: UserProfileSet
_initialize

"Private.  Only SystemUser has permission to execute this successfully."

| saveArr |

System myUserProfile userId = 'SystemUser' ifFalse:[
  self _halt:'Only SystemUser may execute this method.'
  ].

userSecurityData == nil ifTrue:[
  GsObjectSecurityPolicy setCurrent: self objectSecurityPolicy while:[
    self size > 0 ifTrue:[
      saveArr := { }  .
      saveArr addAll: self .
      saveArr do: [ :aProfile | super remove: aProfile ].
    ] .

    userSecurityData := StringKeyValueDictionary new .

    userIdDictionary == nil
       ifTrue:[ userIdDictionary := StringKeyValueDictionary new .  ]
      ifFalse:[  self _halt:'inconsistent state' ].

    maxPasswordSize := 0 .
    minPasswordSize := 1 .
    maxRepeatingChars := 0 .
    maxConsecutiveChars := 0 .
    maxCharsOfSameType := 0 .

    disallowedPasswords := Set new .
    securityBits := 0 .

    passwordAgeLimit := nil .
    passwordAgeWarning := nil .
    staleAccountAgeLimit := nil .

    saveArr ~~ nil ifTrue:[
      saveArr accompaniedBy: self do:[ :me :aProfile | me add: aProfile ].
    ].
  ].							"fix 12466"
].
%

category: 'Private'
method: UserProfileSet
_migrateSecurityPoliciesFrom: aUserProfile to: anotherUserProfile

| sr |
anotherUserProfile _validateClass: UserProfile .
anotherUserProfile isCommitted
  ifFalse:[ anotherUserProfile _error: #rtErrObjMustBeCommitted ] .
aUserProfile == anotherUserProfile
  ifTrue:[ aUserProfile halt: 'Cannot migrate GsSecurityPolicyObjects to this UserProfile' ] .
sr := SystemRepository .
1 to: sr size do:[:n|
  (sr at: n ) ifNotNil:[ :policy |
    policy owner == aUserProfile
      ifTrue:[ policy owner: anotherUserProfile ] .
  ].
].
%

category: 'Updating'
method: UserProfileSet
_oldUserId: oldUserId newUserId: newUserId for: aUserProfile

"Private."

<protected>

| oldUserPro duplicateUser |

self _validatePrivilege ifFalse:[ ^ nil ].
System myUserProfile _validatePrivilegeName: #ChangeUserId .

duplicateUser := self userWithId: newUserId ifAbsent:[ nil ].
duplicateUser ifNotNil:[
  ^ self _error: #rtErrUserIdAlreadyExists args: { newUserId }
  ].

oldUserPro := self userWithId: oldUserId .
oldUserPro == aUserProfile ifFalse:[
  ^ ArgumentError signal: 'oldUserId does not match the given UserProfile' .
  ].

userIdDictionary removeKey: oldUserId .
userIdDictionary at: newUserId put: aUserProfile .
self _userSecurityDataOldKey: oldUserId newKey: newUserId .
%

category: 'Illegal Operations'
method: UserProfileSet
_remove: aUserProfile

"Disallowed."
self shouldNotImplement: #_remove:
%

category: 'Illegal Operations'
method: UserProfileSet
_removeAll: aCollection errIfAbsent: aBlock

"Disallowed."
self shouldNotImplement: #_removeAll:errIfAbsent:
%

category: 'Private'
method: UserProfileSet
_removeSecurityDataFor: aUserProfile

""
<protected>

self _validatePrivilege ifTrue:[
  "use _removeKey: to avoid accessing value in userSecurityData, fix 36312"
  userSecurityData _removeKey: aUserProfile userId .
  super _remove: aUserProfile
]
%

category: 'Private'
method: UserProfileSet
_updateSecurityBitsForMask: bitMask newState: aBoolean

self _validatePrivilege ifTrue:[
  aBoolean
    ifTrue:[ securityBits := securityBits bitOr: bitMask ] "set"
    ifFalse:[ securityBits := securityBits bitAnd: bitMask bitInvert ]. "clear"
]
%

category: 'Private'
method: UserProfileSet
_userSecurityDataOldKey: oldUserId newKey: newUserId
  <protected primitive: 952>

  oldUserId _validateClass: String  .
  newUserId _validateClass: String  .
  self _primitiveFailed: #_userSecurityDataOldKey:newKey: args: { oldUserId . newUserId }
%

category: 'Private'
method: UserProfileSet
_validateNewPassword: aString

"Generates an error if aString is not a valid new password for the receiver;
 otherwise, returns true.
 If execution continued from the generated error, returns false."

<protected>
| subStr maxS |

maxS := maxPasswordSize > 0 ifTrue:[ maxPasswordSize ] ifFalse:[ 1024 ] .
aString size > maxS ifTrue:[
  aString _error: #rtMaxPasswordSize args:{ maxS } .
  ^ false .
].
(minPasswordSize > 0 and:[ aString size < minPasswordSize]) ifTrue:[
  aString _error: #rtMinPasswordSize args:{ minPasswordSize } .
  ^ false
].
(maxConsecutiveChars > 0) ifTrue:[
  subStr := aString maxConsecutiveSubstring .
  subStr size > maxConsecutiveChars ifTrue:[
    aString _error: #rtMaxConsecutiveChars args:{ maxConsecutiveChars . subStr }.
    ^ false
  ].
].
(maxRepeatingChars > 0) ifTrue:[
  subStr := aString maxRepeatingSubstring .
  subStr size > maxRepeatingChars ifTrue:[
    aString _error: #rtMaxRepeatingChars args:{ maxRepeatingChars . subStr }.
    ^ false
  ].
].
(maxCharsOfSameType > 0) ifTrue:[
  subStr := aString maxSameTypeSubstring .
  subStr size > maxCharsOfSameType ifTrue:[
    aString _error: #rtMaxCharsOfSameType
	    args:{ maxCharsOfSameType . (subStr at: 1) _typeAsSymbol asString .
                    subStr } .
    ^ false
  ].
].
(disallowedPasswords includes: aString) ifTrue:[
  aString _error: #rtDisallowedPassword  .
  ^ false
].

(self passwordRequiresUppercase and:[ aString containsUppercase not ]) ifTrue:[
  aString _error: #rtErrPasswordMissingChar args: { 'Uppercase' }  .
  ^ false
] .

(self passwordRequiresLowercase and:[ aString containsLowercase not ]) ifTrue:[
  aString _error: #rtErrPasswordMissingChar args: { 'Lowercase' }.
  ^ false
] .

(self passwordRequiresDigit and:[ aString containsDigit not ]) ifTrue:[
  aString _error: #rtErrPasswordMissingChar args: { 'Digit' } .
  ^ false
].

(self passwordRequiresSymbol and:[ aString containsPasswordSymbol not ]) ifTrue:[
  aString _error: #rtErrPasswordMissingChar args: { 'Symbol' }  .
  ^ false
] .
"disallowUsedPasswords tested by instance method in UserProfile."

^ true
%

category: 'Private'
method: UserProfileSet
_validatePrivilege

" You must have #OtherPassword privilege to modify UserProfileSets "

  ^ System myUserProfile _validatePrivilegeName: #OtherPassword
%

category: 'Private'
method: UserProfileSet
__add: aUserProfile

""
"Reimplementation from IdentityBag.
 Adds anObject to the receiver only if it is not already an element of the
 receiver.  Returns anObject.  Has no effect if anObject is nil."

<protected primitive: 208>
self _primitiveFailed: #add: args: { aUserProfile } .
self _uncontinuableError
%

! Class extensions for 'VariableContext'

!		Class methods for 'VariableContext'

removeallmethods VariableContext
removeallclassmethods VariableContext

category: 'Instance Creation'
classmethod: VariableContext
new

"Disallowed."

self shouldNotImplement: #new
%

category: 'Instance Creation'
classmethod: VariableContext
new: aSize

"Disallowed."

self shouldNotImplement: #new:
%

!		Instance methods for 'VariableContext'

category: 'Updating'
method: VariableContext
at: anIndex put: aValue

"Disallowed."

self shouldNotImplement: #at:put:
%

category: 'Copying'
method: VariableContext
copy

"Disallowed."

self shouldNotImplement: #copy
%

category: 'Accessing'
method: VariableContext
parent
  ^ parent  "may be nil"
%

category: 'Updating'
method: VariableContext
size: anInteger

"Disallowed."

self shouldNotImplement: #size:
%

! Class extensions for 'Warning'

!		Instance methods for 'Warning'

removeallmethods Warning
removeallclassmethods Warning

category: 'Instance initialization'
method: Warning
initialize
  gsNumber := ERR_Warning.
  gsResumable := true .
  gsTrappable := true .
%

! Class extensions for 'ZeroDivide'

!		Class methods for 'ZeroDivide'

removeallmethods ZeroDivide
removeallclassmethods ZeroDivide

category: 'Instance creation'
classmethod: ZeroDivide
dividend: argument
	"Signal the occurrence of a division by zero exception. Capture the number
	that was being divided such that it is available from the 'signaled exception.'

	If the message #dividend is subsequently sent to the <ZeroDivide> object
	that is the 'signaled exception' the value of 'argument' is returned."

	argument / 0.
%

category: 'Instance creation'
classmethod: ZeroDivide
signal

	0 / 0.
%

!		Instance methods for 'ZeroDivide'

category: 'Accessing'
method: ZeroDivide
buildMessageText
^ self buildMessageText:
   'attempt to divide ' , dividend asString , ' by zero'
%

category: 'Accessing'
method: ZeroDivide
dividend

	^ dividend
%

category: 'Accessing'
method: ZeroDivide
dividend: aNumber

  dividend := aNumber
%

category: 'Instance initialization'
method: ZeroDivide
initialize
  gsNumber := ERR_ZeroDivide.
  gsResumable := true .
  gsTrappable := true .
%

! Class Initialization Excluded by export visitor
!  GsCurrentSession initialize.
